<?php

/**
 * UTF8
 */

namespace Ron\RonPortal;

use Ron\Browser;
use Ron\BrowserField;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\StringUtils;

require_once('ron/browser.php');
require_once('ron/guiutils.php');

/**
 * Class ZruseniDokumentuField
 * Mazani dokumentu pomoci linku (GET)
 */
class ZruseniDokumentuField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        require_once('ron/guiutils.php');

        return GuiUtils::changeLocationQuest('<i class="inline-glyph glyph-button fa glyph-delete"></i>', \Ron\ronText('Opravdu_odstranit_dokument'),
            $this->link . '&record=' . $rowData[0]);
    }
}


/**
 * Class ZruseniDokumentuPostField
 * Mazani dokumentu pomoci POST nadrazeneho formulare. Formular musi obsahovat input s id="deleteDocumentId"
 */
class ZruseniDokumentuPostField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //$onClick = "deleteDokumentPost('" . \Ron\ronText('Opravdu_odstranit_dokument') . "', {$rowData[0]})";
        $text = \Ron\ronText('Opravdu_odstranit_dokument');
        return "<i class='inline-glyph glyph-button fa glyph-delete delete-document-post' data-ron-confirm-text='$text' data-document-id='{$rowData[0]}'></i>";
    }
}



class DokumentLinkField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '<a href="' . $this->link;

        if ($recordId != '') $result .= '&record=' . $recordId;
        $result .= '" target="_blank">' . StringUtils::htmlTextConvert($value) . '</a>';

        return $result;
    }
}




class DokumentyBrowser extends Browser
{
    var $downloadLink = '';

    var $deleteType = 0;
    var $deleteLink = null;

    var $datumField = null;

//    var $prvniDokument;  // pomocne promenne pro zobrazeni sekci
//    var $nadpisSekce;

    var $downloadField;

    // nactena data - jednotlive sekce
    //var $cursors = [];
    //var $sectionCaptions = [];


    /**
     * @param Database $database
     * @param int    $deleteType    Zpusob smazani (0=nic, 1=pres odkaz, 2=pres POST - nadrazeny formular musi obsahovat hidden pole deleteDocumentId)
     * @param string $downloadLink  Pouzije se pro stahnuti dokumentu
     * @param string $deleteLink    Pouzije se pro smazani, pokud je $deleteType=1
     */
    function __construct($database, $downloadLink = null, $deleteType = 0, $deleteLink = null)
    {
        parent::__construct($database);

        $this->downloadLink = $downloadLink;
        $this ->deleteType = $deleteType;
        $this ->deleteLink = $deleteLink;

        $this->title = \Ron\ronText('Dokumenty');
        $this->tableId = 'Dokumenty';
        $this->sqlFromPart = 'Dokument, TypDokumentu';
        $this->sqlWherePart = 'Dokument.Typ = TypDokumentu.Kod';
        $this->sqlSelectPart = // POZOR! V pripade zmeny se musi upravit showDokumentyDochazky!
            'Dokument.Id, Dokument.Datum, TypDokumentu.Popis, Dokument.Popis, Dokument.JmenoSouboru';

        $this ->primaryField = 0;
        $this->distinct = true;

        $this->cssClass = 'browser minwidth';

        //$this->prvniDokument = true;

        $this->defaultSort = 1; // podle datumu
        $this ->datumField = $this ->addField(
                ['sqlFieldName' => 'Dokument.Datum',     'sqlIndex' => 1, BROWSER_FIELD_HEADER_TEXT => 'Date', 'format' => 'date', 'ordering' => 1] );

        $this ->addFields( [
                ['sqlFieldName' => 'TypDokumentu.Popis', 'sqlIndex' => 2, BROWSER_FIELD_HEADER_TEXT => 'Typ_dokumentu',   'ordering' => 1],
                ['sqlFieldName' => 'Dokument.Popis',     'sqlIndex' => 3, BROWSER_FIELD_HEADER_TEXT => 'Popis', 'ordering' => 1]
        ]);

        // odkaz na stahnuti souboru
        $this->downloadField = new DokumentLinkField($this ->database,
                ['sqlFieldName' => 'Dokument.JmenoSouboru', 'sqlIndex' => 4, 'link' => $this ->downloadLink] );
        $this->addField($this->downloadField);

        switch ($this ->deleteType) {
            case 0 :
                break;

            case 1 :
                // odkaz na smazani souboru
                if ($this->deleteLink) {
                    $deleteField = new ZruseniDokumentuField();
                    $deleteField ->link = $this->deleteLink; // '?menuid=zrusitdokument&' . $this->downloadLink;
                    $this->addField($deleteField);
                }
                break;

            case 2 :
                $deleteField = new ZruseniDokumentuPostField();
                $this->addField($deleteField);
                break;
        }
    }

}

