<?php

/**
 * UTF8
 */

namespace Ron\RonPortal\XML;

use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\GuiUtils;
use Ron\Http\App;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PredavaciProcesyUtils;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\User;
use Ron\StringUtils;

require_once('ron/browser.php');
require_once('adsapp/ronportalfilter.php');



class XMLFilterForm extends RonPortalFilterForm
{
    /** @var ComboBox */
    var $comboPopis;

    private $xmlMenuID;

    public static function getFilterStav()
    {
        return $_SESSION['filtrstavxml'];
    }

    public static function setFilterStav($value)
    {
        $_SESSION['filtrstavxml'] = $value;
    }

    public static function getZobrazitVse()
    {
        return $_SESSION['xmlzobrazitvse'];
    }

    public static function setZobrazitVse($value)
    {
        $_SESSION['xmlzobrazitvse'] = $value;
    }

    public static function getJenPopisXML()
    {
        return $_SESSION['filtrjenpopisxml'];
    }

    public static function setJenPopisXML($value)
    {
        $_SESSION['filtrjenpopisxml'] = $value;
    }

    public static function getPopisXML()
    {
        return $_SESSION['filtrpopisxml'];
    }

    public static function setPopisXML($value)
    {
        $_SESSION['filtrpopisxml'] = $value;
    }

    public static function getJenOsobaXML()
    {
        return $_SESSION['filtrjenosobaxml'];
    }

    public static function setJenOsobaXML($value)
    {
        $_SESSION['filtrjenosobaxml'] = $value;
    }

    public function __construct($database, $xmlMenuId)
    {
        parent::__construct($database, '');

        $typPravaOsoby = RonPortalSettings::getValue('vyplnenexml_osoby_pravo','prohlizeni');
        if ($typPravaOsoby) {
            $prava = Prava::vratPrava($typPravaOsoby);
            if ($prava->jsouNejakePrava())
                $this->nastavOsoby(true, $prava, DateUtils::today(), DateUtils::today());
        }

        $this->nastavAkci('?menuid=seznamxmlformularu' . ($xmlMenuId ? '&xmlmenuid=' . $xmlMenuId : ''));
        $this->f_classname = 'filtercontainer';

        // datum od-do
        $stav = self::getFilterStav();
        //pri nastaveni stavu neukonceno, nebo vse, pro pripad, ze neni nastaveno obdobi, nastavi filtr na datum vytvoreni
        if ((isset($_GET['stavxml']) && !RonPortalSession::jenObdobi()) && (($stav == -2 || $stav == -3))) {
            RonPortalSession::setFiltrovatDatum(true);
        }
//        $filtrpovinny = ($stav == -2 || $stav == -3); // -2 = ukonceno, -3 = vse
        //30.9.2019 zrusena povinnost filtru na datum, nastaveno strankovani browseru
        $filtrpovinny = false;
        $this->nastavDatumOdDo(true, $filtrpovinny, \Ron\ronText('Datum_vytvoreni_xml') . ' ' . \Ron\ronText('Od'));

        //obdobi
        if (RonPortalUtils::jeNainstalovanaAplikace($this->database,'ADS')) {
            $this->nastavObdobi(true, 'ADS', 0, false);
        }

        // rozdeleni
        $this->nastavRozdeleni(true, RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI),
            RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_JEN_VLASTNI_ROZDELENI), '', true);

        // pouze vlastni/vse
        if (Prava::vratHodnotuPrava('xml') == 3) {
            // ma pravo na vsechny XML formulare - zobrazi se link vse/pouze vlastni
            if (isset($_GET['xmlzobrazitvse'])) XMLFilterForm::setZobrazitVse((int)$_GET['xmlzobrazitvse']);

            if (XMLFilterForm::getZobrazitVse()) {
                $this->nastavLink("<a href='?menuid=seznamxmlformularu&xmlzobrazitvse=0" . ($xmlMenuId ? '&xmlmenuid=' . $xmlMenuId : '') . "'>" . \Ron\ronText('Zobrazit_pouze_vlastni_xml') . "</a>");
            } else {
                $this->nastavLink("<a href='?menuid=seznamxmlformularu&xmlzobrazitvse=1" . ($xmlMenuId ? '&xmlmenuid=' . $xmlMenuId : '') . "'>" . \Ron\ronText('Zobrazit_vsechny_xml') . "</a>");
            }
        }

        $this->xmlMenuID = $xmlMenuId;
    }


    function loadData()
    {
        parent::loadData();

        if (isset($_POST['adsfilterform'])) {
            self::setJenPopisXML(isset($_POST['jenpopisxml']));
            self::setPopisXML($_POST['popisxml']);
            self::setJenOsobaXML(isset($_POST['jenosoba']));
        }

        $params = $this ->database ->createParameters();
        if (self::getFilterStav() == -1) {
            // filtr pro vsechny neukoncene bez ohledu na obdobi
            $sql =
                'SELECT DISTINCT VyplnenyXMLFormular.Popis, VyplnenyXMLFormular.Popis'.
                ' FROM VyplnenyXMLFormular LEFT OUTER JOIN SablonaXMLFormulare' .
                ' ON SablonaXMLFormulare.ID = VyplnenyXMLFormular.IdSablony'.
                ' WHERE Stav < 128 ';
        } else {
            // obdobi je povinne => filtr podle obdobi
            $sql =
                'SELECT DISTINCT VyplnenyXMLFormular.Popis, VyplnenyXMLFormular.Popis'.
                ' FROM VyplnenyXMLFormular LEFT OUTER JOIN SablonaXMLFormulare' .
                ' ON SablonaXMLFormulare.ID = VyplnenyXMLFormular.IdSablony'.
                ' WHERE VyplnenyXMLFormular.DatumVytvoreni >= ' . $params ->addParameter($this ->database ->dateToParam(RonPortalSession::getDatumOd())) .
                ' AND VyplnenyXMLFormular.DatumVytvoreni <= ' . $params ->addParameter($this ->database ->dateToParam(RonPortalSession::getDatumDo()));
        }

        if ($this->xmlMenuID) {
            $sql .=
                ' AND SablonaXMLFormulare.MenuId = ' . $params->addParameter($this->xmlMenuID);
        }

        $this ->comboPopis = new ComboBox('popisxml');
        $this ->comboPopis ->keyIndex = 0;
        $this ->comboPopis ->captionIndex = 1;
        $this ->comboPopis ->emptyRow = ' ';
        $this ->comboPopis ->dataTypeComparison = true;
        $this ->comboPopis ->className = 'form-input adsfilterforminput_full';
        $this ->comboPopis ->htmlParams = 'onChange="if (jenpopisxml.checked) submit();"';
        $this ->comboPopis ->loadData($this ->database, $sql, XMLFilterForm::getPopisXML(), $params);
    }


    function showFilterPopis()
    {
        echo '<DIV class="filterrow">';
        //echo '<TR><TD>';
        // checkbox "Jen typ"
        echo '<SPAN class="filtercaption">';
        GuiUtils::renderCheckbox('jenpopisxml',
            \Ron\ronText('Jen_typ_xml'), XMLFilterForm::getJenPopisXML(),
            'onClick="if (popisxml.value) submit()"', '', null, true);
        echo '</SPAN>';

        //echo '</TD><TD>';

        // combobox pro vyber
        $this ->comboPopis ->render();
        echo "</DIV>\n";
    }

    function renderOsoby()
    {
        echo '<DIV class="filterrow">';
        echo '<SPAN class="filtercaption">';
        GuiUtils::renderCheckbox('jenosoba', \Ron\ronText('Jen_osoba'), XMLFilterForm::getJenOsobaXML(), 'onClick=submit()', '', null, true);
        echo '</SPAN>';

        $this ->comboOsoba ->render();

        if ($this ->f_prava) {
            if ($this->comboOsoba->data && count($this->comboOsoba->data) > 1) {
                // tlacitka pro posun na dalsi a predchozi PP
                echo '<span id="adsfilterform_pp_fast" class="desktop-only">';
                echo '<i class="inline-glyph glyph-button fa glyph-left comboBoxMove" data-step="-1" data-submit="1" data-selector="#adsfilterform_osoba"></i>';
                echo '<i class="inline-glyph glyph-button fa glyph-right comboBoxMove" data-submit="1" data-selector="#adsfilterform_osoba"></i>';
                echo '</span>';
            }
        }

        echo "</DIV>\n";
    }


    function renderFilters()
    {
        $this->renderDatumOdDo();
        if ($this->showObdobi) $this->renderObdobi();
        $this->showFilterPopis();
        $this->renderRozdeleni();
        if ($this->f_showosoba) $this->renderOsoby();
        $this->renderLink();
    }
}




class EditaceXMLField extends BrowserField
{
    function __construct()
    {
        parent::__construct();

        $this->type = 'text';
        $this->align = 'left';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '<a href="?menuid=editovatdotaznik&record=' . $recordId . '">';

        global $STR_ZobrazitXML; // kompat.
        global $STR_EditovatXML; // kompat.

        if ($rowData[2] >= 128) $result .= $STR_ZobrazitXML ?: \Ron\ronText('Zobrazit_xml');
        else $result .= $STR_EditovatXML ?: \Ron\ronText('Editovat_xml');

        $result .= '</a>';

        return $result;
    }
}



class RozdeleniXMLField extends BrowserField
{
    function __construct()
    {
        parent::__construct();

        global $STR_Rozdeleni; // kompat.

        $this->header = $STR_Rozdeleni ?: \Ron\ronText('Rozdeleni');
        $this->type = 'normal';
        $this->align = 'left';
        $this->sqlFieldName = 'Rozdeleni.Kod';
        $this->ordering = 1;
        $this->sqlFieldIndex = 9;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = $value;
        if ($rowData[3]) $result .= ' (' . $rowData[3] . ')';

        return $result;
    }
}




class StavXMLField extends BrowserField
{
    function __construct()
    {
        parent::__construct();

        $this->header = \Ron\ronText('Stav');
        $this->type = 'normal';
        $this->align = 'left';
        $this->sqlFieldName = 'S_StavXMLFormulare.Popis';
        $this->ordering = 1;
        $this->sqlFieldIndex = 12;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = $value;

//        $backurl = 'seznamxmlformularu' . (isset($_GET['xmlmenuid']) ? "&xmlmenuid={$_GET['xmlmenuid']}" : '');
//        if ($rowData[4]) $result .= " <I>(<a href='?menuid=predavaciproces&xmlid={$rowData[0]}&backurl=$backurl'>"  .  $rowData[4] . '</a>)</I>';

        if ($rowData[4]) $result .= ' <I>(' . $rowData[4] . ')</I>';
        return $result;
    }
}




class VyplnenyXMLFormularBrowser extends Browser
{
    var $xmlMenuId;
    /** @var XMLFilterForm */
    var $filterForm;

    function __construct($database, $xmlMenuId)
    {
        parent::__construct($database);

        global $STR_VyplneneFormulare; // kompat.

        $this->xmlMenuId = $xmlMenuId;
        $this->browserUrl = '?menuid=seznamxmlformularu&' . ($xmlMenuId ? 'xmlmenuid=' . $xmlMenuId . '&' : '');

        $this->title = $STR_VyplneneFormulare ?: \Ron\ronText('Vyplnene_xml_formulare');

        $this->code = 'browservyplnenexml';

        $this->cssClass .= ' browservyplnenexml' . ($xmlMenuId ? '_' . $xmlMenuId : '') ;

        $this->sqlFromPart =
            '( ( ( ( VyplnenyXMLFormular LEFT OUTER JOIN Osoba ' .
            '          ON (VyplnenyXMLFormular.RC = Osoba.RC  OR   VyplnenyXMLFormular.PP_RC = Osoba.RC)' .
            '      ) LEFT OUTER JOIN PredavaciProces ' .
            '        ON PredavaciProces.Kod = VyplnenyXMLFormular.KodProcesu ' .
            '    ) LEFT OUTER JOIN PoradiPredavani ' .
            '      ON PoradiPredavani.KodProcesu = VyplnenyXMLFormular.KodProcesu ' .
            '         AND PoradiPredavani.PocatecniUroven = VyplnenyXMLFormular.PocatecniUroven ' .
            '         AND PoradiPredavani.Poradi = VyplnenyXMLFormular.PoradiPredavani ' .
            '  ) LEFT OUTER JOIN Rozdeleni ' .
            '    ON Rozdeleni.KodTypuRozdeleni = VyplnenyXMLFormular.KodTypuRozdeleni ' .
            '       AND Rozdeleni.Kod = VyplnenyXMLFormular.Rozdeleni ' .
            ') INNER JOIN S_StavXMLFormulare ' .
            '  ON S_StavXMLFormulare.Kod = VyplnenyXMLFormular.Stav ';

        // pokud je zadano $xmlmenuid, tak nacte jen formulare prirazene k sablonam se zadanym menuid
        if ($this->xmlMenuId)
            $this->sqlFromPart =
                '( ' . $this->sqlFromPart . ' ) ' .
                '  INNER JOIN SablonaXMLFormulare ON SablonaXMLFormulare.Id = VyplnenyXMLFormular.IdSablony' .
                "   AND SablonaXMLFormulare.MenuId = '" . $database ->addSlashes($this->xmlMenuId) . "'";

        //$this -> AddFieldDef( array('normal', 'RC', 'RC osoby ...', '', 'left', '', 0, 1) );
        $this ->sqlSelectPart =
            'VyplnenyXMLFormular.Id, PoradiPredavani.KodUrovne, VyplnenyXMLFormular.Stav, ' . // 0..2
            'Rozdeleni.Popis, PoradiPredavani.KodUrovne, VyplnenyXMLFormular.DodatecnyPopis, ' . // 3..5
            'VyplnenyXMLFormular.Popis, Osoba.Prijmeni, Osoba.Jmeno, Rozdeleni.Kod, ' . // 6..9
            'VyplnenyXMLFormular.DatumVytvoreni, VyplnenyXMLFormular.PosledniZmena, S_StavXMLFormulare.Popis, ' . // 10..12
            'VyplnenyXMLFormular.KodProcesu, VyplnenyXMLFormular.PocatecniUroven, VyplnenyXMLFormular.PoradiPredavani, ' . // 13..15
            'PoradiPredavani.Vlastni, PoradiPredavani.NekontrolovatHierarchii, VyplnenyXMLFormular.Obdobi, ' . // 16..18
            'VyplnenyXMLFormular.Rozliseni1, VyplnenyXMLFormular.Rozliseni2, VyplnenyXMLFormular.Rozliseni3, VyplnenyXMLFormular.Rozliseni4 , VyplnenyXMLFormular.Rozliseni5, ' . //19 .. 23
            'VyplnenyXMLFormular.Rozliseni6, VyplnenyXMLFormular.Rozliseni7, VyplnenyXMLFormular.Rozliseni8, VyplnenyXMLFormular.Rozliseni9, VyplnenyXMLFormular.Rozliseni10 '; //24 .. 28


        if (App::getDebugMode()) {
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.Id', BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_HEADER => 'ID', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1] );
        }

        $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.Popis', BROWSER_FIELD_SQL_INDEX => 6, BROWSER_FIELD_HEADER_TEXT => 'Typ', BROWSER_FIELD_ORDERING => 1] );

        $this->addField( [BROWSER_FIELD_SQL_NAME => 'Osoba.Prijmeni', BROWSER_FIELD_SQL_INDEX => 7, BROWSER_FIELD_HEADER_TEXT => 'Prijmeni', BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [BROWSER_FIELD_SQL_NAME => 'Osoba.Jmeno', BROWSER_FIELD_SQL_INDEX => 8, BROWSER_FIELD_HEADER_TEXT => 'Jmeno', BROWSER_FIELD_ORDERING => 1] );
        $this->addField( new RozdeleniXMLField() ); // Rozdeleni.Kod
        $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.Obdobi', BROWSER_FIELD_SQL_INDEX => 18, BROWSER_FIELD_HEADER_TEXT => 'Obdobi', BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.DatumVytvoreni', BROWSER_FIELD_SQL_INDEX => 10, BROWSER_FIELD_HEADER_TEXT => 'Cas_vytvoreni_xml', BROWSER_FIELD_FORMAT => 'datetime', BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.PosledniZmena', BROWSER_FIELD_SQL_INDEX => 11, BROWSER_FIELD_HEADER_TEXT => 'Posledni_zmena', BROWSER_FIELD_FORMAT => 'datetime', BROWSER_FIELD_ORDERING => 1] );
        $this ->defaultSort = count($this ->fields);
        $this->addField( new StavXMLField() ); // S_StavXMLFormulare.Popis

        if (RonPortalSettings::getValue('xml_rozliseni1_nadpis') != '')
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.Rozliseni1', BROWSER_FIELD_SQL_INDEX => 19, BROWSER_FIELD_HEADER => RonPortalSettings::getValue('xml_rozliseni1_nadpis'), BROWSER_FIELD_ORDERING => 1,
                BROWSER_FIELD_CLASS_NAME => 'rozliseni1'] );
        if (RonPortalSettings::getValue('xml_rozliseni2_nadpis') != '')
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.Rozliseni2', BROWSER_FIELD_SQL_INDEX => 20, BROWSER_FIELD_HEADER => RonPortalSettings::getValue('xml_rozliseni2_nadpis'), BROWSER_FIELD_ORDERING => 1,
                BROWSER_FIELD_CLASS_NAME => 'rozliseni2'] );
        if (RonPortalSettings::getValue('xml_rozliseni3_nadpis') != '')
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.Rozliseni3', BROWSER_FIELD_SQL_INDEX => 21, BROWSER_FIELD_HEADER => RonPortalSettings::getValue('xml_rozliseni3_nadpis'), BROWSER_FIELD_ORDERING => 1,
                BROWSER_FIELD_CLASS_NAME => 'rozliseni3'] );
        if (RonPortalSettings::getValue('xml_rozliseni4_nadpis') != '')
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.Rozliseni4', BROWSER_FIELD_SQL_INDEX => 22, BROWSER_FIELD_HEADER => RonPortalSettings::getValue('xml_rozliseni4_nadpis'), BROWSER_FIELD_ORDERING => 1,
                BROWSER_FIELD_CLASS_NAME => 'rozliseni4'] );
        if (RonPortalSettings::getValue('xml_rozliseni5_nadpis') != '')
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.Rozliseni5', BROWSER_FIELD_SQL_INDEX => 23, BROWSER_FIELD_HEADER => RonPortalSettings::getValue('xml_rozliseni5_nadpis'), BROWSER_FIELD_ORDERING => 1,
                BROWSER_FIELD_CLASS_NAME => 'rozliseni5'] );
        if (RonPortalSettings::getValue('xml_rozliseni6_nadpis') != '')
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.Rozliseni6', BROWSER_FIELD_SQL_INDEX => 24, BROWSER_FIELD_HEADER => RonPortalSettings::getValue('xml_rozliseni6_nadpis'), BROWSER_FIELD_ORDERING => 1,
                BROWSER_FIELD_CLASS_NAME => 'rozliseni6'] );
        if (RonPortalSettings::getValue('xml_rozliseni7_nadpis') != '')
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.Rozliseni7', BROWSER_FIELD_SQL_INDEX => 25, BROWSER_FIELD_HEADER => RonPortalSettings::getValue('xml_rozliseni7_nadpis'), BROWSER_FIELD_ORDERING => 1,
                BROWSER_FIELD_CLASS_NAME => 'rozliseni7'] );
        if (RonPortalSettings::getValue('xml_rozliseni8_nadpis') != '')
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.Rozliseni8', BROWSER_FIELD_SQL_INDEX => 26, BROWSER_FIELD_HEADER => RonPortalSettings::getValue('xml_rozliseni8_nadpis'), BROWSER_FIELD_ORDERING => 1,
                BROWSER_FIELD_CLASS_NAME => 'rozliseni8'] );
        if (RonPortalSettings::getValue('xml_rozliseni9_nadpis') != '')
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.Rozliseni9', BROWSER_FIELD_SQL_INDEX => 27, BROWSER_FIELD_HEADER => RonPortalSettings::getValue('xml_rozliseni9_nadpis'), BROWSER_FIELD_ORDERING => 1,
                BROWSER_FIELD_CLASS_NAME => 'rozliseni9'] );
        if (RonPortalSettings::getValue('xml_rozliseni10_nadpis') != '')
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.Rozliseni10', BROWSER_FIELD_SQL_INDEX => 28, BROWSER_FIELD_HEADER => RonPortalSettings::getValue('xml_rozliseni10_nadpis'), BROWSER_FIELD_ORDERING => 1,
                BROWSER_FIELD_CLASS_NAME => 'rozliseni10'] );

        if (App::getDebugMode()) {
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.PocatecniUroven', BROWSER_FIELD_SQL_INDEX => 14, BROWSER_FIELD_HEADER => 'Poc.uroven', BROWSER_FIELD_ORDERING => 1] );
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.PoradiPredavani', BROWSER_FIELD_SQL_INDEX => 15, BROWSER_FIELD_HEADER => 'Por.predavani', BROWSER_FIELD_ORDERING => 1] );
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.Vlastni', BROWSER_FIELD_SQL_INDEX => 16, BROWSER_FIELD_HEADER => 'Vlastni', BROWSER_FIELD_ORDERING => 1] );
            $this->addField( [BROWSER_FIELD_SQL_NAME => 'VyplnenyXMLFormular.NekontrolovatHierarchii', BROWSER_FIELD_SQL_INDEX => 17, BROWSER_FIELD_HEADER => 'NE hierariche', BROWSER_FIELD_ORDERING => 1] );
        }

        $this->addField(new EditaceXMLField());

        $this->primaryField = 0;

        $this->maxPageRows = 100;
    }


    function load($sqlFilter = '')
    {
        if (isset($_GET['stavxml'])) {
            XMLFilterForm::setFilterStav((int)$_GET['stavxml']);
        }

        $this ->filterForm = new XMLFilterForm($this ->database, $this->xmlMenuId);
        $this ->filterForm ->loadData();

        parent::load($sqlFilter);
    }


    function renderFilterStav()
    {
        $stav = XMLFilterForm::getFilterStav();
        if (!isset($stav)) {
            XMLFilterForm::setFilterStav(-1);
            $stav = -1; // vychozi hodnota - neukoncene
        }

        $data = [
            -1 => \Ron\ronText('Neukonceno'),
            -2 => \Ron\ronText('Ukonceno'),
            -3 => \Ron\ronText('Vse')
        ];

        $url = '?menuid=seznamxmlformularu' . ($this->xmlMenuId ? '&xmlmenuid=' . $this->xmlMenuId : '') . '&stavxml=';

        LinksList::renderLinks($data, $stav, $url);
    }

    function renderFilter()
    {
        $this->renderFilterStav();

        echo "<BR>\n";

        $this ->filterForm ->render();

        //parent::renderFilter();
    }



    /**
     * Vraci filtr pro vyber formularu podle procesu
     * @return string
     */
    function getSQLFilterXML()
    {
        // pokud ma prava na vse a ma zapnute vse, tak nic nefiltruje
        if (Prava::vratHodnotuPrava('xml') == 3  &&  XMLFilterForm::getZobrazitVse()) return '';

        //TODO: kontrola dle noveho nastaveni viditelnosti - provadet v enable line?

        // formulare ktere vlozil aktualni uzivatel - v puvodnich verzich se nepouzivala tabulka XMLFormularVyplnil
        $result =
            "( (RC_Vlozil = '" . $this ->database ->addSlashes(User::getRC()) . "') "; // prava zavorka bude dale !


        // formulare ktere vyplnil aktualni uzivatel (i kdyz jsou zrovna na jine urovni predavani)
        $result .=
            'OR ( EXISTS ( ' .
                ' SELECT * FROM XMLFormularVyplnil ' .
                " WHERE XMLFormularVyplnil.RC = '" . $this ->database ->addSlashes(User::getRC()) . "'" .
                '   AND XMLFormularVyplnil.IdFormulare = VyplnenyXMLFormular.Id' .
            ') ) ';

        // formulare na urovni oznacene "vlastni"
        $result .=
            'OR (PoradiPredavani.Vlastni = 1 ' .
                " AND (VyplnenyXMLFormular.RC = '" . $this ->database ->addSlashes(User::getRC()) . "'  OR  VyplnenyXMLFormular.PP_RC = '" . $this ->database ->addSlashes(User::getRC()) . "') " .
            ')';


        // zjisti do kterych urovni hodnoceni jsem zarazeny (podle mych pracovnich mist)
        $filtrurovne = PredavaciProcesyUtils::filtrUrovnePredavani($this ->database, 'PoradiPredavani.KodUrovne');
        $kodtypurozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI); // pouzije se, pokud neni typ rozdeleni u procesu zadan
        
        global $_XML_jenPodrizene; //pro zachovani funkcnosti v appdbset, pokud budou chtit zmenit, musi se zrusit v appdbset
        $XML_jenPodrizene = $_XML_jenPodrizene == 1 ? true : RonPortalSettings::getIntValue('jenvlastnirozdelenihodnoceni') == 2;
        
        if ($filtrurovne) // mam pravo na nejake urovne hodnoceni
        {
            // vybere XML zamestnancu, na ktere mam pravo (podle pracovniho mista a hierarchie)
            $result .=
                'OR ( PoradiPredavani.Vlastni = 0 ' . // urovne s volbou "vlastni" jsou resene samostatne
                    " AND ( $filtrurovne ) " .  // formular na "moji" urovni predavani
                    ' AND  (' .
                        // na aktualni urovni formulare se nekontroluje hierarchie
                        'PoradiPredavani.NekontrolovatHierarchii = 1 '.
                    '  OR ' .
                        // nebo jsem ve stejnem nebo nadrazenem rozdeleni
                        'EXISTS  ( SELECT * ' .
                        '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H, RozdeleniOsob R_Hodnoceny ' .
                        "  WHERE R_Ja.RC = '" . $this ->database ->addSlashes(User::getRC()) . "'" .
                        //"    AND R_JA.KodTypuRozdeleni = $kodtypurozdeleni". - puvodni verze - natvrdo jeden typ rozdeleni
                        '    AND ( R_JA.KodTypuRozdeleni = PredavaciProces.KodTypuRozdeleni ' . // proces ma vlastni typ rozdeleni
                        "          OR  (PredavaciProces.KodTypuRozdeleni IS NULL  AND  R_JA.KodTypuRozdeleni = $kodtypurozdeleni) " . // typ rozdeleni u procesu neni zadan
                        ' )' .
                        '    AND R_Ja.PlatnostOd <= ' . $this ->database ->timestampToString(time()) .
                        '    AND (R_Ja.PlatnostDo >= ' . $this ->database ->dateToString(time()) . '  OR  R_Ja.PlatnostDo IS NULL)' .
                        //"    AND R_Hodnoceny.KodTypuRozdeleni = $kodtypurozdeleni". - puvodni verze - natvrdo jeden typ rozdeleni
                        '    AND R_Hodnoceny.KodTypuRozdeleni = R_JA.KodTypuRozdeleni ' .
                        '    AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                        '    AND H.KodNadrazene = R_Ja.Rozdeleni ' .
                        '    AND H.TypPodrizene = R_Hodnoceny.KodTypuRozdeleni ' .
                        '    AND H.KodPodrizene = R_Hodnoceny.Rozdeleni ' .
                        ($XML_jenPodrizene ? ' AND H.KodNadrazene <> H.KodPodrizene ' : '') .
                        '    AND R_Hodnoceny.PlatnostOd <= ' . $this ->database ->timestampToString(time()) .
                        '    AND (R_Hodnoceny.PlatnostDo >= ' . $this ->database ->dateToString(DateUtils::startOfTheDay(time())) . '  OR  R_Hodnoceny.PlatnostDo IS NULL)' .

                        '    AND ( ' .
                        '      R_Hodnoceny.RC = VyplnenyXMLFormular.RC ' .
                        '      OR ( ' .
                        '        R_Hodnoceny.RC = VyplnenyXMLFormular.PP_RC ' .
                        '        AND R_Hodnoceny.OsobniCislo = VyplnenyXMLFormular.PP_OsobniCislo ' .
                        '        AND R_Hodnoceny.DatumNastupu = VyplnenyXMLFormular.PP_DatumNastupu ' .
                        '      ) ' .
                        '    ) ' .
                        ')' .
                    ')' .
                ')';

            // Vybere XML, ktere nejsou pro konkretni osobu, ale jsou pro moje a podrizene rozdeleni
            $result .=
                ' OR (' .
                    "( $filtrurovne ) " .  // dokument na "moji" urovni predavani
                    ' AND VyplnenyXMLFormular.RC IS NULL ' .    // neni pro konkretni osobu
                    ' AND VyplnenyXMLFormular.PP_RC IS NULL ' .
                    ' AND ('.
                    '    PoradiPredavani.NekontrolovatHierarchii = 1 '.
                    '   OR ' .
                    '    EXISTS ( SELECT * ' .        // jsem ve stejnem nebo nadrazenem rozdeleni
                        '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H ' .
                        "  WHERE R_Ja.RC = '" . $this ->database ->addSlashes(User::getRC()) . "'" .
                        //"    AND R_JA.KodTypuRozdeleni = $kodtypurozdeleni". - puvodni verze

                        '    AND ( R_JA.KodTypuRozdeleni = PredavaciProces.KodTypuRozdeleni ' . // proces ma vlastni typ rozdeleni
                        "          OR  (PredavaciProces.KodTypuRozdeleni IS NULL  AND  R_JA.KodTypuRozdeleni = $kodtypurozdeleni) " . // typ rozdeleni u procesu neni zadan
                        '    )' .

                        '    AND R_Ja.PlatnostOd <= ' . $this ->database ->timestampToString(time()) .
                        '    AND (R_Ja.PlatnostDo >= ' . $this ->database ->timestampToString(DateUtils::startOfTheDay(time())) . '  OR  R_Ja.PlatnostDo IS NULL)' .
                        '    AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                        '    AND H.KodNadrazene = R_Ja.Rozdeleni ' .
                        '    AND H.TypPodrizene = VyplnenyXMLFormular.KodTypuRozdeleni ' .
                        '    AND H.KodPodrizene = VyplnenyXMLFormular.Rozdeleni ' .
                        ($XML_jenPodrizene ? ' AND H.KodNadrazene <> H.KodPodrizene ' : '') .
                        ')' .
                    ')' .
                ')';
        }

        $result .= ')';

        return $result;
    }


    /**
     * @return string
     */
    function getSQLFilterRozdeleni()
    {
        if (!RonPortalSession::rozdeleni($this->filterForm->f_typrozdeleni)) return '';

        // filtr podle rozdeleni formulare
        $result =
            '( ' .
            ' EXISTS ( ' .
            '   SELECT * FROM HierarchieRozdeleni ' .
            '   WHERE TypNadrazene = ' . RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI) .
            "     AND KodNadrazene = '" . $this ->database ->addSlashes(RonPortalSession::rozdeleni($this->filterForm->f_typrozdeleni)) . "' " .
            '     AND TypPodrizene = VyplnenyXMLFormular.KodTypuRozdeleni ' .
            '     AND KodPodrizene = VyplnenyXMLFormular.Rozdeleni' .
            (RonPortalSession::rozdeleniVcetnePodrizenych($this->filterForm->f_typrozdeleni) ? '' : ' AND (HierarchieRozdeleni.kodNadrazene = HierarchieRozdeleni.kodPodrizene) ') .
            ' )';


        $result .= ') OR (';

        // filtr podle osoby formulare
        $result .=
            'EXISTS (' .
            '  SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
            '  WHERE HierarchieRozdeleni.TypNadrazene = ' . RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI) .
            "    AND HierarchieRozdeleni.KodNadrazene = '" . $this ->database ->addSlashes(RonPortalSession::rozdeleni($this->filterForm->f_typrozdeleni)) . "'" .
            '    AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.TypPodrizene' .
            '    AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.KodPodrizene' .
            (RonPortalSession::rozdeleniVcetnePodrizenych($this->filterForm->f_typrozdeleni) ? '' : ' AND (HierarchieRozdeleni.kodNadrazene = HierarchieRozdeleni.kodPodrizene) ') .
            '    AND ( RozdeleniOsob.RC = VyplnenyXMLFormular.RC ' .
            ' OR (' .
            '   RozdeleniOsob.RC = VyplnenyXMLFormular.PP_RC ' .
            '   AND RozdeleniOsob.OsobniCislo = VyplnenyXMLFormular.PP_OsobniCislo ' .
            '   AND RozdeleniOsob.DatumNastupu = VyplnenyXMLFormular.PP_DatumNastupu ' .
            ' )' .
            ' AND RozdeleniOsob.PlatnostOd <= ' . $this ->database ->timestampToString(time()) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $this ->database ->timestampToString(DateUtils::startOfTheDay(time())) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
            '    ) ' .
            ')';

        $result .= ')';

        return $result;
    }


    function getSQLFilter()
    {
        // ---------------------------------------------------------------------
        // filtr pro stav
        switch (XMLFilterForm::getFilterStav()) {
            case -3 : // vsechny
                $result = '';
                break;

            case -2 : // ukoncene
                $result = ' (VyplnenyXMLFormular.Stav >= 128) ';
                break;

            case -1 : // neukoncene
            default :
                $result = ' (VyplnenyXMLFormular.Stav < 128) ';
                break;
        }

        // filtr podle predavaciho procesu
        $pom = $this->getSQLFilterXML();
        if ($pom && $result) $result .= ' AND ';
        $result .= $pom;


        // filtr jen typ (popis)
        if (XMLFilterForm::getJenPopisXML() && XMLFilterForm::getPopisXML()) {
            if ($result) $result = $result . ' AND ';
            $result = $result . "VyplnenyXMLFormular.Popis = '" . $this ->database ->addSlashes(XMLFilterForm::getPopisXML()) . "'";
        }

        // filtr podle data vytvoreni
//        $stav = XMLFilterForm::getFilterStav();
//        $filtrpovinny = ($stav == -2 || $stav == -3); // -2 = ukonceno, -3 = vse
        $filtrpovinny = false;
        //30.9.2019 zrusena povinnost filtru na datum, nastaveno strankovani browseru
        if ($filtrpovinny || RonPortalSession::getFiltrovatDatum()) {
            if ($result) $result = $result . ' AND ';
            $result = $result .
                ' VyplnenyXMLFormular.DatumVytvoreni >= ' . $this ->database ->dateToString(RonPortalSession::getDatumOd()) .
                '   AND VyplnenyXMLFormular.DatumVytvoreni < ' . $this ->database ->dateToString( DateUtils::incDay(RonPortalSession::getDatumDo(), 1) );

        }

        if (RonPortalSession::jenObdobi()) {
            if ($result) {
                $result = $result . ' AND ';
            }
            $result .=
                ' (VyplnenyXMLFormular.Obdobi >= ' . $this->database->timestampToString(RonPortalSession::obdobi()) .
                ' AND VyplnenyXMLFormular.Obdobi <= ' . $this->database->timestampToString(RonPortalSession::konecObdobi()) . ')';
        }

        if (XMLFilterForm::getJenOsobaXML()) {
            if ($result) {
                $result = $result . ' AND ';
            }
            $result .=
                " ((VyplnenyXMLFormular.RC = '" . $this->database->addSlashes(RonPortalSession::osobaRC()) . "') OR " .
                " (VyplnenyXMLFormular.PP_RC = '" . $this->database->addSlashes(RonPortalSession::osobaRC()) . "' AND VyplnenyXMLFormular.PP_OsobniCislo = '" . $this->database->addSlashes(RonPortalSession::osobaOsobniCislo()) . "'" .
                "    AND VyplnenyXMLFormular.PP_DatumNastupu = " . $this->database->dateToString(RonPortalSession::osobaNastup()) . "))";
        }

        $filtrrozdeleni = $this->getSQLFilterRozdeleni();
        if ($filtrrozdeleni) {
            if ($result) $result .= ' AND ';
            $result .= '(' . $filtrrozdeleni . ')';
        }

        return $result;
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($values[5]) {
            // dodatecny popis
            echo "<tr class='$className'>";
            echo '<td></td><td colspan="' . ($this->columnCount - 1) . '"><span class="browserpoznamka">' . StringUtils::htmlTextConvert($values[5]) . '</span></td>';
            echo '</tr>';

        }
    }
}
    
