<?php

/**
 * UTF8
 */

namespace Ron\RonPortal;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\FileUtils;
use Ron\Http\HttpHeaders;
use Ron\Http\HttpResponse;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;


require_once('ron/sqlutils.php');
require_once('ron/mvc.php');
require_once('adsapp/ronportalutils.php');


class DokumentyUtils
{
    /**
     * Vlozi do databaze novy dokument. Vraci id dokumentu.
     * @param Database $database
     * @param string $data      Obsah dokumentu
     * @param string $fileName  Jmeno souboru
     * @param int    $typ       Typ dokumentu (odkaz do tabulky TypDokumentu)
     * @param string $popis
     * @param object|string $logSrcDescription
     * @return int
     * @throws DBException
     * @throws \Exception
     */
    public static function vytvorDokument($database, $data, $fileName, $typ, $popis, $logSrcDescription = 'Dokumenty')
    {
        // zkontroluje, jestli jsou zadana povinna pole
        if (!isset($typ) || $typ === '') {
            throw new \Exception( \Ron\ronText('Neni_zadan_typ_dokumentu') );
        } else if (!isset($popis) || $popis == '') {
            throw new \Exception( \Ron\ronText('Neni_zadan_popis_dokumentu') );
        }

        require_once('ron/fileutils.php');
        $fileName = FileUtils::truncateFileName($fileName, 160);

        require_once('adsapp/ronportallog.php');

        $data = gzcompress($data);

        require_once 'adsapp/ronportaldbrecord.php';
        $rec = new RonPortalDBRecord($database, 'Dokument', $logSrcDescription);
        //$rec ->addField('Id', 'int', false, 0, $newId);
        $f_serial = $rec ->addField('Id', 'serial');
        $rec ->addField('Popis', '', false, 160, $popis);
        $rec ->addField('Typ', 'int', false, 0, (int)$typ);
        $rec ->addField('Datum', 'timestamp', false, 0, DateUtils::now());
        $rec ->addField('TypKomprimace', 'int', false, 0, 1);
        $rec ->addField('JmenoSouboru', '', false, 160, $fileName);
        $rec ->addField('Insert_UserName', '', false, 25, RonPortalUtils::getLogUserName());
        $rec ->addField('Insert_TimeStamp', 'timestamp', false, 0, time());
        $rec ->addField('Data', 'binary', false, 0, $data);
        $rec ->insertRecord();

        //return $newId;
        return $f_serial ->value;
    }


    /**
     * Vlozi do databaze novy dokument. Vraci id dokumentu.
     * @param Database $database
     * @param string $htmlField
     * @param int $typ
     * @param string $popis
     * @param object|string $logSrcDescription
     * @param bool $utf8    pokud je true, tak se nazev souboru prevede z utf8 (je to potreba pri uploadu z javascriptu)
     * @return int
     * @throws DBException
     * @throws \Exception
     */
    public static function vlozDokument($database, $htmlField, $typ, $popis, $logSrcDescription = 'Dokumenty', $utf8 = false)
    {
        if ($_FILES[$htmlField]['error'] == UPLOAD_ERR_INI_SIZE ||
            $_FILES[$htmlField]['error'] == UPLOAD_ERR_FORM_SIZE
        ) {
            throw new \Exception( \Ron\ronText('Err_velky_soubor') );
        }

        if (!is_uploaded_file($_FILES[$htmlField]['tmp_name'])) {
            throw new \Exception('Chyba! Neni zadan soubor!');
        }

        $fileName = $_FILES[$htmlField]['name']; // jmeno puvodniho souboru
        $tmpfilename = $_FILES[$htmlField]['tmp_name']; // jmeno docasneho souboru (data pro upload do databaze)

        if ($utf8) {
            require_once('ron/stringutils.php');
            $fileName = StringUtils::fromUTF8($fileName);
        }

        $data = file_get_contents($tmpfilename);

        return self::vytvorDokument($database, $data, $fileName, $typ, $popis, $logSrcDescription);

//        require_once('adsapp/ronportallog.php');

//        $data = gzcompress($data);

//        if ($database ->getDbType() == 'mssql') {
//            // TODO: predelat na params! (upravit driver mssql, aby fungoval getBlobParam)
//            // mssql
//            $datastring = unpack("H*hex", $data);
//
//            $newId = SQLUtils::newSerialNumber($database, 'Dokument', 'Id');
//
//            $sql =
//                'INSERT INTO Dokument (Id, Popis, Typ, Datum, TypKomprimace, JmenoSouboru, Insert_UserName, Insert_TimeStamp, Data)' .
//                '  VALUES ( ' .
//                $newId . ", '" . $database ->addSlashes($popis) . "', " . (int)$typ . ', ' . $database ->timestampToString(time()) . ', 1, ' . // 1 = komprimovano
//                "'" . $database ->addSlashes($fileName) . "', '" . $database ->addSlashes(RonPortalUtils::getLogUserName()) . "', " . $database ->timestampToString(time()) . ', 0x';
//
//            $sqlLog = $sql . '...';
//            $sql .= $datastring['hex'];
//
//            $sql .= ')';
//            $sqlLog .= ')';
//
//            if ($database ->sql($sql)) {
//                Log::addRecordToDBLog($database,
//                    'Dokument', 'INSERT', $logSrcDescription, $sqlLog);
//
//            } else {
//                $errorMsg = $database ->lastErrorText();
//
//                Log::addRecordToDBLog($database,
//                    'Dokument', 'ERROR', $logSrcDescription,
//                    "Chyba pri vkladani dokumentu!\n$errorMsg\n\n$sqlLog");
//
//                throw new DBException($database, $sql, 'Chyba pri vkladani dokumentu', null, $errorMsg);
//            }
//        } else {
            //$newId = SQLUtils::newSerialNumber($database, 'Dokument', 'Id');

//            require_once 'adsapp/ronportaldbrecord.php';
//            $rec = new RonPortalDBRecord($database, 'Dokument', $logSrcDescription);
//            //$rec ->addField('Id', 'int', false, 0, $newId);
//            $f_serial = $rec ->addField('Id', 'serial');
//            $rec ->addField('Popis', '', false, 160, $popis);
//            $rec ->addField('Typ', 'int', false, 0, (int)$typ);
//            $rec ->addField('Datum', 'timestamp', false, 0, DateUtils::now());
//            $rec ->addField('TypKomprimace', 'int', false, 0, 1);
//            $rec ->addField('JmenoSouboru', '', false, 160, $fileName);
//            $rec ->addField('Insert_UserName', '', false, 25, RonPortalUtils::getLogUserName());
//            $rec ->addField('Insert_TimeStamp', 'timestamp', false, 0, time());
//            $rec ->addField('Data', 'binary', false, 0, $data);
//            $rec ->insertRecord();
//        }

        //return $newId;
//        return $f_serial ->value;
    }


    /**
     * Smaze z databaze dokument se zadanym Id. Pokud je zadana podrizenaTabulka, tak nejdrive smaze zaznam z ni.
     * @param Database $database
     * @param int $id
     * @param object|string $logDescription
     * @param string $podrizenaTabulka
     * @param string $filtr
     * @param Parameters $filtrParams
     * @param string $foreignKeyName    Jmeno pole s ID dokumentu v podrizene tabulce
     * @throws DBException
     * @throws \Exception
     */
    public static function smazDokument($database, $id, $logDescription, $podrizenaTabulka = '', $filtr = '', $filtrParams = null, $foreignKeyName = 'Id')
    {
        require_once('adsapp/ronportallog.php');

        // smazani zaznamu z podrizene tabulky
        if ($podrizenaTabulka) {
            $params = $database ->createParameters();
            $sql =
                'DELETE FROM ' . $podrizenaTabulka .
                " WHERE $foreignKeyName = " . $params ->addParameter((int)$id) .   // ? presunout tuto podminku do filtru?
                ' AND (' . $filtr . ')';
            if ($filtrParams) {
                $params ->appendParameters($filtrParams);
            }

            try {
                $database->sql($sql, $params);
            } catch (DBException $e) {
                Log::addRecordToDBLog($database,
                    $podrizenaTabulka, 'ERROR', $logDescription,
                    "Chyba pri mazani dokumentu!\n" . $e ->dbMessage . "\n\n$sql\n\nId = $id");

                throw $e;
            }

            Log::addRecordToDBLog($database, $podrizenaTabulka, 'DELETE', $logDescription, $sql . "\n\n$foreignKeyName = $id");
        }

        require_once('adsapp/ronportaldbrecord.php');
        $rec = new RonPortalDBRecord($database, 'Dokument', $logDescription);
        $rec ->addField('Id', 'int');
        $rec ->addField('Popis', '');
        $rec ->addField('Typ', 'int');
        $rec ->addField('Datum', 'timestamp');
        $rec ->addField('JmenoSouboru', '');
        $rec ->addField('Insert_UserName', '');
        $rec ->addField('Insert_Timestamp', 'timestamp');

        $params = $database ->createParameters();
        $rec ->deleteRecord('Id = ' . $params ->addParameter((int)$id), $params);
    }

    /**
     * @param Database $database
     * @return bool
     * @throws \Exception
     */
    public static function lzeZobrazitVyplatniListky($database)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT zobrazitNaIntrawebu FROM TypDokumentu WHERE kod = " . $params->addParameter(-1);

        $cursor = $database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        $cursor->close();

        return !!$row[0];
    }

}



/**
 * OutputObject pro stahnuti dokumentu z databaze
 * Class DokumentOutputObject
 */
class DokumentOutputObject implements OutputObject
{
    /** @var  Database */
    protected $database;
    /** @var  int */
    protected $idDokumentu;
    /** @var  bool */
    protected $verze;
    /** @var  int */
    protected  $platnostOd;


    /**
     * @param Database $database
     * @param int  $idDokumentu
     * @param bool $verze
     * @param int  $platnostOd
     */
    function __construct($database, $idDokumentu, $verze = false, $platnostOd = null)
    {
        $this ->database = $database;
        $this ->idDokumentu = $idDokumentu;
        $this ->verze = $verze;
        $this ->platnostOd = $platnostOd;
    }

    /**
     * @return void
     * @throws \Exception
     */
    public function output()
    {
        if ( ini_get('mssql.textlimit') < 8388608 ) ini_set('mssql.textlimit', '8388608');
        if ( ini_get('mssql.textsize') < 8388608 ) ini_set('mssql.textsize', '8388608');

        if ( ini_get('odbc.defaultlrl') < 8388608 ) ini_set('odbc.defaultlrl', '8388608');

        $params = $this ->database ->createParameters();
        $sql = $this ->verze
            ?
            'SELECT JmenoSouboru, Data, TypKomprimace, {fn_CASTWIDEMEMO(Link)}, Odkaz FROM VerzeDokumentu ' .
            ' WHERE Id = ' . $params ->addParameter((int)$this ->idDokumentu) .
            '   AND PlatnostOd = ' . $params ->addParameter($this ->database ->timestampToParam($this ->platnostOd))
            :
            'SELECT JmenoSouboru, Data, TypKomprimace, {fn_CASTWIDEMEMO(Link)}, Odkaz FROM Dokument WHERE Id = ' . $params ->addParameter((int)$this ->idDokumentu);

        $cursor = $this ->database ->getSqlResult($sql, $params);

        if (!$row = $cursor ->fetch()) {
            throw new \Exception('Chyba! Pozadovany dokument neexistuje!');
        }


        // headers

        if ($row[1]) {
            // dokument s daty
            HttpResponse::getInstance() ->getHeaders() ->addHeadersForContent($row[0]);
            HttpResponse::renderHeaders();

            switch ($row[2]) {
                case 0 :
                    print $cursor ->stringToBinary($row[1]);
                    exit;

                case 1 :
                    print gzuncompress($cursor ->stringToBinary($row[1]));
                    exit;
            }
        } else if ($row[3]) {
            // dokument ma definovany link - presmeruje se na zadany link
            $r = new Redirect($row[3]);
            $r ->render(); // nastavi headers

            HttpResponse::renderHeaders();

            exit;
            // TODO: tu by se mel doplnit exit, ne? Doplnit a odzkouset!
        } else if ($row[4]) {
            if (!file_exists($row[4])) {
                throw new \Exception('Pozadovany soubor nebyl nalezen!');
            }
            // dokument obsahuje jmeno souboru na disku
            HttpResponse::getInstance() ->getHeaders() ->addHeadersForContent($row[4]);
            HttpResponse::renderHeaders();

            readfile($row[4]);
        }

        // nic - tu by nemel dojit
    }
}



class DokumentUploadForm implements ViewObject
{
    /** @var Database */
    var $database;

    /** @var string */
    var $uploadUrl;

    /** @var string */
    var $redirectUrl;

    var $sqlPripojeniStarsiho = null;

    var $hiddenFields = null;
    
    /** @var int pokud je nastaven v potomku, ve formulari se nezobrazi vyber typu dokumentu */
    var $typDokumentu = null;

    /** @var int pokud je nastavena v potomku, ve formulari se zobrazi jen vyber typu dokumentu s touto kategorii */
    var $kategorieDokumentu = null;

    var $target = 'upload_target';

    /**
     * DokumentUploadForm constructor.
     * @param Database $database
     * @param string $uploadUrl
     */
    public function __construct(Database $database, $uploadUrl)
    {
        $this->database = $database;
        $this->uploadUrl = $uploadUrl;
    }


    /**
     * Potomek muze doplnit dalsi pole
     */
    public function dalsiRadkyKDokumentu()
    {

    }


    public function render()
    {
        require('view/dokument_novy.php');
    }
}


/**
 * Zajistuje funkcnost uploadu dokumentu pro view dokument_novy.php
 * Class DokumentUploadOutput
 */
class DokumentUploadOutput implements OutputObject
{
    var $result = 'OK';
    var $errorText = null;


    public function setError($errorText)
    {
        $this ->result = 'error';
        $this ->errorText = $errorText;
    }


    public function output()
    {
        $jsonResult = [
            'result' => $this ->result
        ];

        if ($this ->errorText) {
            $jsonResult['errors'] = [ ['errorText' => StringUtils::jsonText($this ->errorText)] ];
        }

        echo
            '<html>' .
            '<body>' .
            '<script type="text/javascript">' .
            'var result = ' . json_encode($jsonResult) . '; ' .
            ' window.parent.window.vlozitDokumentResult(result);' .
            '</script>' .
            '</body>' .
            '</html>';
    }
}

