<?php
/**
 * stanislav.elbl, 03.11.2019
 */

namespace Ron\RonPortal;

use Ron\DB\Database;

class Report
{
    const FORMATY = ['PDF', 'XLS', 'ODS', 'PDFA'];

    /** @var string */
    public $kod;
    /** @var string */
    public $nazev;
    /** @var string */
    public $prava;
    /** @var int        Bitova maska 1=PDF, 2=XLS, 4=ODS, 8=PDFA */
    public $formaty;
    /** @var string[]   Pole formatu - naplni se podle bitove masky */
    public $nazvyFormatu;
    /** @var string     Formular, ve kterem se ma sestava zobrazit */
    public $formular;

    /** @var string */
    public $applicationId;
    /** @var int */
    public $managerNumber;
    /** @var string     Jmeno sestavy z manazera sestav */
    public $appReportName;


    // zobrazene panely - doplni se dodatecne podle informaci z reportserveru
    /** @var bool */
    public $zobrazitPanelObdobi;
    /** @var bool */
    public $zobrazitPanelRozdeleni;
    /** @var bool */
    public $zobrazitPanelRozdeleni2;
    /** @var bool */
    public $zobrazitPanelOsoba;
    /** @var bool */
    public $zobrazitPanelKalendar;


    /**
     * @param Database $database
     * @param string $kod
     * @return Report
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public static function loadReport($database, $kod)
    {
        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult(
            'SELECT SestavyProIntraweb.Kod, SestavyProIntraweb.Nazev, SestavyProIntraweb.Prava,' . // 0..2
            ' SestavyProIntraweb.Formaty, SestavyProIntraweb.Formular,' . // 3,4
            ' SYS_RMRE.applicationId, SYS_RMRE.managerNumber, SYS_RMRE.ReportName' . // 5..7
            ' FROM SestavyProIntraweb LEFT OUTER JOIN SYS_RMRE' .
            ' ON SestavyProIntraweb.applicationId = SYS_RMRE.applicationId' .
            '  AND SestavyProIntraweb.managerNumber = SYS_RMRE.managerNumber' .
            '  AND SestavyProIntraweb.ReportName = SYS_RMRE.ReportName' .
            ' WHERE Kod = ' . $params ->addParameter($kod), $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new \Exception('Chyba pri nacitani reportu - sestava se zadanym kodem nebyla nalezena!');
        }

        $result = new Report();
        $result ->kod = $row[0];
        $result ->nazev = $row[1];
        $result ->prava = $row[2];
        $result ->formaty = $row[3];
        $result ->formular = $row[4];
        $result ->applicationId = $row[5];
        $result ->managerNumber = $row[6];
        $result ->appReportName = $row[7];

        $result ->nazvyFormatu = [];
        $bit = 0x01;
        foreach (self::FORMATY as $format) {
            if ($result ->formaty & $bit) {
                $result ->nazvyFormatu[] = $format;
            }
            $bit <<= 1;
        }

        return $result;
    }

}
