<?php
/**
 * stanislav.elbl, 18.11.2019
 */

namespace Ron\RonPortal;

use Ron\Controls\ComboBox;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\StringUtils;
use Ron\XmlLoader;

class ReportsFilterForm extends BaseViewObject
{
    /** @var Database */
    var $database;

    /** @var string */
    var $reportName;

    /** @var Report */
    var $report;

    /** @var ComboBoxObdobi */
    var $comboObdobi;
    /** @var ComboBox */
    var $comboCtvrtleti;
    /** @var ComboBox */
    var $comboRok;

    /** @var  RonPortalReport[] */
    var $reportList;

    /** @var ComboBox */
    var $comboTypRozdeleni;
    /** @var ComboBoxesRozdeleni */
    var $comboRozdeleni;

    /** @var ComboBox */
    var $comboTypRozdeleni2;
    /** @var ComboBoxesRozdeleni */
    var $comboRozdeleni2;

    /** @var ComboBoxOsoba */
    var $comboOsoba;
    /** @var ComboBox */
    var $comboTypOsoby;
    /** @var ComboBox */
    var $comboTypPP;
    /** @var bool   Urcuje, jestli ma uzivatel pravo sam na sebe */
    var $jePravoNaVlastni;

    /** @var ComboBox */
    var $comboKalendar;

    /**
     * ReportsFilterForm constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        parent::__construct('view/reportfilters.php');
        $this->database = $database;
    }


    /**
     * Nacte zadany report a doplni k nemu informace z ccf
     * @param Database  $database
     * @param string    $reportName
     * @return Report
     * @throws \Exception
     */
    public static function loadReport($database, $reportName)
    {
        require_once 'adsapp/report.php';
        $report = Report::loadReport($database, $reportName);

        if ($report ->appReportName) {
            require_once 'adsapp/reportserver.php';
            ReportServer::loadReportInfo($report);
        } else {
            // vychozi nastaveni pro sestavy, ktere nejsou z manazera
            $report ->zobrazitPanelObdobi = true;
            $report ->zobrazitPanelRozdeleni = true;
            $report ->zobrazitPanelRozdeleni2 = !!RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP);
            $report ->zobrazitPanelOsoba = true;
            $report ->zobrazitPanelKalendar = false;
        }

        if (!$report ->prava) {
            $report ->prava = 'prohlizeni';
        }
        return $report;
    }


    // panel obdobi
    public static function zvolenyTypObdobi()
    {
        return RonPortalSession::getValue('report_obdobi');
    }

    public static function setZvolenyTypObdobi($value)
    {
        RonPortalSession::setValue('report_obdobi', $value);
    }

    public static function zvoleneObdobiOd()
    {
        $value = RonPortalSession::getValue('report_obdobi_od');
        return $value ?: StringUtils::format('date', DateUtils::firstDayOfMonth(time()));
    }

    /**
     * @param string $value
     */
    public static function setZvoleneObdobiOd($value)
    {
        RonPortalSession::setValue('report_obdobi_od', $value);
    }

    public static function zvoleneObdobiDo()
    {
        $value = RonPortalSession::getValue('report_obdobi_do');
        return $value ?: StringUtils::format('date', DateUtils::lastDayOfMonth(time()));
    }

    public static function setZvoleneObdobiDo($value)
    {
        RonPortalSession::setValue('report_obdobi_do', $value);
    }


    // panel rozdeleni
    public static function getJenRozdeleni()
    {
        return (int)RonPortalSession::getValue('report_jen_rozdeleni');
    }

    public static function getTypRozdeleni()
    {
        return (int)RonPortalSession::getValue('report_typ_rozdeleni');
    }

    public static function getRozdeleni($database)
    {
        $result = RonPortalSession::getValue('report_rozdeleni');
        if (!isset($result)) {
            $result = RonPortalSession::rozdeleni();

            if (!isset($result))  {
                require_once('adsapp/usersettings.php');
                $result = NastaveniUzivatele::getUserSettingsItem($database, 'ads_rozdeleni');
            }
        }
        return $result;
    }

    public static function getRozdeleniVcetnePodrizenych()
    {
        return (int)RonPortalSession::getValue('report_rozdeleni_vcetne_podrizenych');
    }

    public static function setRozdeleni($jenZvoleneRozdeleni, $typRozdeleni, $kodRozdeleni, $vcetnePodrizenych)
    {
        RonPortalSession::setValue('report_jen_rozdeleni', $jenZvoleneRozdeleni ? 1 : 0);
        if ($jenZvoleneRozdeleni) {
            RonPortalSession::setValue('report_typ_rozdeleni', $typRozdeleni);
            RonPortalSession::setValue('report_rozdeleni', $kodRozdeleni);
            RonPortalSession::setValue('report_rozdeleni_vcetne_podrizenych', $vcetnePodrizenych ? 1 : 0);
        }
    }


    // panel osoba
    public static function getJenZvolenaOsoba()
    {
        return (int)RonPortalSession::getValue('report_jen_osoba');
    }

    public static function setJenZvolenaOsoba($value)
    {
        RonPortalSession::setValue('report_jen_osoba', $value ? 1 : 0);
    }


    public static function getJenTypOsoby()
    {
        return (int)RonPortalSession::getValue('report_jen_typ_osoby');
    }

    public static function setJenTypOsoby($value)
    {
        RonPortalSession::setValue('report_jen_typ_osoby', $value ? 1 : 0);
    }

    public static function getTypOsoby()
    {
        return (int)RonPortalSession::getValue('report_typ_osoby');
    }

    public static function setTypOsoby($value)
    {
        RonPortalSession::setValue('report_typ_osoby', (int)$value);
    }


    public static function getJenTypPP()
    {
        return (int)RonPortalSession::getValue('report_jen_typ_pp');
    }

    public static function setJenTypPP($value)
    {
        RonPortalSession::setValue('report_jen_typ_pp', $value ? 1 : 0);
    }

    public static function getTypPP()
    {
        return RonPortalSession::getValue('report_typ_pp');
    }

    public static function setTypPP($value)
    {
        RonPortalSession::setValue('report_typ_pp', $value);
    }


    // panel kalendar
    public static function getJenKalendar()
    {
        return (int)RonPortalSession::getValue('report_jen_kalendar');
    }

    public static function setJenKalendar($value)
    {
        RonPortalSession::setValue('report_jen_kalendar', $value ? 1 : 0);
    }

    public static function getKalendar()
    {
        return RonPortalSession::getValue('report_kalendar');
    }

    public static function setKalendar($value)
    {
        RonPortalSession::setValue('report_kalendar', $value);
    }



    private function createComboCtvrtletiRok()
    {
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT MIN (PocatekObdobi), MAX(PocatekObdobi) FROM Obdobi' .
            //' WHERE Aplikace = ' . $params ->addParameter($this->report->applicationId ?: 'ADS'),
            ' WHERE Aplikace = ' . $params ->addParameter('ADS'),
            $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        $minObdobi = DateUtils::firstDayOfQuarter($row ? $this ->database ->stringToTimestamp($row[0]) : time());
        $maxObdobi = DateUtils::firstDayOfQuarter($row ? $this ->database ->stringToTimestamp($row[1]) : time());

        $options = [];
        for ($obdobi = $maxObdobi;  $obdobi >= $minObdobi;  $obdobi = DateUtils::incMonth($obdobi, -3)) {
            $options[] = new Option(date('j.n.Y', $obdobi), DateUtils::quarterDescription($obdobi));
        }

        $this ->comboCtvrtleti = new ComboBox('obdobiCtvrtleti', $options);
        $this ->comboCtvrtleti ->setSelectedOption(self::zvoleneObdobiOd());
        $this ->comboCtvrtleti ->htmlParams['id'] = 'obdobiCtvrtleti';
        $this ->comboCtvrtleti ->className = 'form-input obdobiInput';

        $minRok = DateUtils::firstDayOfYear($minObdobi);
        $maxRok = DateUtils::firstDayOfYear($maxObdobi);

        $optionsRok = [];
        for ($rok = $maxRok;  $rok >= $minRok;  $rok = DateUtils::incMonth($rok, -12)) {
            $optionsRok[] = new Option(date('j.n.Y', $rok), date('Y', $rok));
        }

        $this ->comboRok = new ComboBox('obdobiRok', $optionsRok);
        $this ->comboRok ->setSelectedOption(self::zvoleneObdobiOd());
        $this ->comboRok ->htmlParams['id'] = 'obdobiRok';
        $this ->comboRok ->className = 'form-input obdobiInput';
    }


    /**
     * @param $reportName
     */
    public function load($reportName)
    {
        $this ->reportList = RonPortalReport::loadSQL($this ->database, " Povolit = 1  AND  (Formular IS NULL  OR  Formular = '')", null);

        if (count($this ->reportList) == 1) {
            $reportName = $this ->reportList[0] ->kod;
        }

        $this ->reportName = $reportName;

        if ($reportName) {
            $this->report = self::loadReport($this->database, $reportName);

            require_once 'ron/dateutils.php';
            $obdobiOd = DateUtils::startOfTheDay(time());
            $obdobiDo = DateUtils::endOfTheDay($obdobiOd);

            if ($this->report->zobrazitPanelObdobi) {
                require_once 'adsapp/ronportalcontrols.php';
                $this->comboObdobi = new ComboBoxObdobi('obdobiMesic');
                $this->comboObdobi->htmlParams = ['id' => 'obdobiMesic'];
                $this->comboObdobi->className = 'form-input obdobiInput';
                $this->comboObdobi->loadObdobi($this->database, $this->report->applicationId ?: 'ADS', false, false, false, RonPortalSession::obdobi());

                $this ->createComboCtvrtletiRok();

                switch ($this->zvolenyTypObdobi()) {
                    case 1 :    // ctvrtleti
                    case 2 :    // rok
                    case 3 :    // datum od-do
                        try {
                            $obdobiOd = self::zvoleneObdobiOd() ? DateUtils::dateStringToTimestamp(self::zvoleneObdobiOd()) : $obdobiOd;
                            $obdobiDo = self::zvoleneObdobiDo() ? DateUtils::dateStringToTimestamp(self::zvoleneObdobiDo()) : $obdobiDo;
                        } catch (\Exception $e) {
                            MessageManager::addErrorMessage($e);
                        }
                        break;

//                    case 1 :
//                        $obdobiOd = DateUtils::firstDayOfQuarter(self::zvoleneObdobiOd());
//                        $obdobiDo = DateUtils::incDay(DateUtils::incMonth(DateUtils::firstDayOfQuarter(self::zvoleneObdobiOd()), 3), -1);
//                        break;

                    case 0 :    // obdobi aplikace
                    default :
                        require_once 'adsapp/obdobi.php';
                        try {
                            $obdobi = Obdobi::vratObdobi($this->database, $this->report->applicationId ?: 'ADS',
                                $this ->comboObdobi ->pocatekObdobi ?: RonPortalSession::obdobi()
                            );
                            $obdobiOd = $obdobi->zacatek;
                            $obdobiDo = $obdobi->konec;
                        } catch (\Exception $e) {
                            // ignoruje
                        }
                }
            }

            // TODO: zmenit obdobi pro zobrazeni osob
            Prava::nastavObdobiProPrava($obdobiOd, $obdobiDo);

            if ($this->report->zobrazitPanelRozdeleni && RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) {  // prozatim jen typ, ktery je nastaveny
                //            $this ->comboTypRozdeleni = new ComboBox('typRozdeleni');
                //            $this ->comboTypRozdeleni ->htmlParams['id'] = 'typRozdeleni';
                //            $this ->comboTypRozdeleni ->dataTypeComparison = false;
                //            $this ->comboTypRozdeleni ->loadSQL($this ->database, 'SELECT Kod, Popis FROM TypRozdeleni ORDER BY Kod', null);
                //            $this ->comboTypRozdeleni ->setSelectedOption(self::getTypRozdeleni());

                require_once 'adsapp/ronportalcontrols.php';
                $this->comboRozdeleni = new ComboBoxesRozdeleni($this->database, 'rozdeleni', 'rozdeleni');
                $this->comboRozdeleni->loadRozdeleni(false, null, RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), self::getRozdeleni($this ->database), true);

                //            $combo1->htmlParams =
                //                "data-ron-select2='#ronportalfilter_{$htmlKod}_kod'" .
                //                "data-ronselect-class='ronselect adsfilterforminput_popis'" .
                //                ($this ->f_autosubmit ? ' data-ron-autosubmit="1"' : '');
            }

            $prava = Prava::vratPrava($this->report->prava);
            if ($this->report->zobrazitPanelOsoba) {
                require_once 'adsapp/ronportalcontrols.php';
                $this->comboOsoba = new ComboBoxZamestnanec('osoba');
                $this->comboOsoba->htmlParams = [
                    'id' => 'osoba',
                    'data-ronselect-class' => 'ronselect form-input',
                    'data-columns' => 'jmeno;osobni-cislo;datum-nastupu',
                    'data-captions' => \Ron\ronText('Jmeno') . ';' . \Ron\ronText('Osobni_cislo') . ';' . \Ron\ronText('Datum_nastupu')];
                $this->comboOsoba->className = 'form-input ronselect';
                if ($this->report->zobrazitPanelRozdeleni && RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) && self::getJenRozdeleni()) {
                    $this->comboOsoba->nastavFiltrRozdeleni(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP),
                        self::getRozdeleni($this ->database), self::getRozdeleniVcetnePodrizenych());
                }
                $this->comboOsoba->loadDataPP($this->database, $obdobiOd, $obdobiDo, $prava, false, RonPortalSession::vybranaOsoba());

                $this->comboTypOsoby = new ComboBox('typOsoby');
                $this->comboTypOsoby->htmlParams['id'] = 'typOsoby';
                $this->comboTypOsoby->dataTypeComparison = false;
                $this->comboTypOsoby->loadSQL($this->database, 'SELECT Kod, Popis FROM S_TypOsoby ORDER BY Kod', null);
                $this->comboTypOsoby->setSelectedOption(self::getTypOsoby());

                $this->comboTypPP = new ComboBox('typPP');
                $this->comboTypPP->htmlParams['id'] = 'typPP';
                $this->comboTypPP->loadSQL($this->database, 'SELECT Kod, Popis FROM TypPracovnihoPomeru ORDER BY Kod', null);
                $this->comboTypPP->setSelectedOption(self::getTypPP());
            }

            $this->jePravoNaVlastni = $prava->lzeZobrazitRC(User::getRC());

            if ($this->report->zobrazitPanelKalendar) {
                $this->comboKalendar = new ComboBox('kalendar');
                $params = $this->database->createParameters();
                $this->comboKalendar->loadSQL($this->database, 'SELECT Kod, Popis FROM Kalendar WHERE Aktivni = 1 ORDER BY Popis', $params);
                $this->comboKalendar->showCode = true;
                $this->comboKalendar->setSelectedOption(self::getKalendar());
            }
        }
    }


    public function renderReportList()
    {
        if (!$this ->reportList) {
            return;
        }

        $kategorie = null;

        echo "<ul class='list-unstyled list-padding reportlist'>\n";
        foreach ($this ->reportList as $report) {
            if (!$report ->prava  ||  Prava::jsouNejakePrava($report ->prava)) {
                if ($report->kategorie && $report->kategorie != $kategorie) {
                    $kategorie = $report->kategorie;
                    echo '<li class="form-section margin-top padding-left">' . StringUtils::htmlTextConvert($kategorie) . '</li>';
                }

                $classStr = $report->appReportName ? 'managerReport' : 'IWReport';

                if ($this->report && $this->report->kod == $report->kod) {
                    $classStr = ' active font-weight-bold';
                }

                echo
                    "<li class='$classStr'>" .
                    '<a href="?menuid=reports&reportname=' . StringUtils::htmlTextConvert($report->kod) . '">' .
                    StringUtils::htmlTextConvert($report->nazev) .
                    '</a>' .
                    '</li>';
            }
        }
        echo "</ul>\n";

    }


    public static function getPostPP($database, $report)
    {
        $pp = null;
        if ($report ->zobrazitPanelOsoba) {
            self::setJenZvolenaOsoba(isset($_POST['jenOsoba'])  &&  $_POST['jenOsoba']);

            if (self::getJenZvolenaOsoba()) {
                require_once 'adsapp/ppomer.php';
                $pp = PracovniPomer::nactiPP($database, $_POST['osoba']);
                $pp ->vyberNactenyPP();
            }

            self::setJenTypOsoby(isset($_POST['jenTypOsoby'])  &&  $_POST['jenTypOsoby']);
            self::setTypOsoby((int)$_POST['typOsoby']);

            self::setJenTypPP(isset($_POST['jenTypPP'])  &&  $_POST['jenTypPP']);
            self::setTypPP($_POST['typPP']);
        }

        return $pp;
    }


    public static function getPostRozdeleni($database, $report)
    {
        if (!$report ->zobrazitPanelRozdeleni) {
            return null;
        }

        $rozdeleni = null;

        if ($_POST['jenRozdeleni']  &&  RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)  &&  $_POST['rozdeleni'] != '') {
            require_once 'adsapp/rozdeleni.php';
            $rozdeleni = Rozdeleni::loadRozdeleni($database, RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $_POST['rozdeleni']);
        }

        self::setRozdeleni($_POST['jenRozdeleni'], $rozdeleni ? $rozdeleni ->typ : null, $rozdeleni ? $rozdeleni ->kod : null, $_POST['vcetnePodrizenych']);

        return $rozdeleni;
    }


    /**
     * @param Database $database
     * @param Report $report
     * @param string $format
     * @return ReportOutputObject
     * @throws \Exception
     */
    public static function getXmlReportOutputObject($database, $report, $format)
    {
        $result = new ReportOutputObject($report, $format);

        if ($report ->zobrazitPanelObdobi) {
            $typObdobi = (int)$_POST['typObdobi'];
            self::setZvolenyTypObdobi($typObdobi);

            switch ($typObdobi) {
                case 1 :    // ctvrtleti
                    $obdobiOd = DateUtils::firstDayOfQuarter(DateUtils::dateStringToTimestamp($_POST['obdobiCtvrtleti']));
                    $obdobiDo = DateUtils::incDay(DateUtils::incMonth($obdobiOd, 3), -1);
                    self::setZvoleneObdobiOd(date('j.n.Y', $obdobiOd));
                    self::setZvoleneObdobiDo(date('j.n.Y', $obdobiDo));

                    $result ->nastavObdobi($typObdobi, $obdobiOd, $obdobiDo);
                    break;

                case 2 :    // rok
                    $obdobiOd = DateUtils::firstDayOfYear(DateUtils::dateStringToTimestamp($_POST['obdobiRok']));
                    $obdobiDo = DateUtils::incDay(DateUtils::incMonth($obdobiOd, 12), -1);
                    self::setZvoleneObdobiOd(date('j.n.Y', $obdobiOd));
                    self::setZvoleneObdobiDo(date('j.n.Y', $obdobiDo));

                    $result ->nastavObdobi($typObdobi, $obdobiOd, $obdobiDo);
                    break;

                case 3 :    // datum od-do
                    $obdobiOd = DateUtils::dateStringToTimestamp($_POST['obdobiOd']);
                    self::setZvoleneObdobiOd($_POST['obdobiOd']);

                    $obdobiDo = DateUtils::dateStringToTimestamp($_POST['obdobiDo']);
                    self::setZvoleneObdobiDo($_POST['obdobiDo']);

                    $result ->nastavObdobi($typObdobi, $obdobiOd, $obdobiDo);
                    break;

                case 0 :
                default :
                    require_once 'adsapp/obdobi.php';
                    $obdobi = Obdobi::vratObdobi($database, 'ADS', DateUtils::dateStringToTimestamp($_POST['obdobiMesic']));
                    RonPortalSession::setObdobi($obdobi->zacatek, $obdobi->konec);
                    $result ->nastavObdobi($typObdobi, $obdobi ->zacatek, $obdobi ->konec, $obdobi ->popis);
                    break;
            }
        }

        if ($report ->zobrazitPanelOsoba) {
            $pp = self::getPostPP($database, $report);
            $result ->nastavPP($pp, false); // prava se kontroluji az v sestave

            $result ->nastavTypOsoby(self::getJenTypOsoby(), self::getTypOsoby());
            $result ->nastavTypPP(self::getJenTypPP(), self::getTypPP());
        }

        if ($report ->zobrazitPanelKalendar) {
            $jenKalendar = isset($_POST['jenKalendar']) && $_POST['jenKalendar'];
            self::setJenKalendar($jenKalendar);

            if ($jenKalendar) {
                require_once 'dochazka/kalendar.php';
                $kalendar = Kalendar::nactiKalendar($database, $_POST['kalendar']);
                self::setKalendar($kalendar->kod);
            } else {
                $kalendar = null;
            }

            $result ->nastavKalendar($jenKalendar, $kalendar);
        }

        if ($report ->zobrazitPanelRozdeleni) {
            $rozdeleni = self::getPostRozdeleni($database, $report);

            $typRozdeleni = $rozdeleni ? TypRozdeleni::getTypRozdeleni($database, $rozdeleni->typ) : null;

            $result ->nastavRozdeleni(self::getJenRozdeleni(), $rozdeleni, self::getRozdeleniVcetnePodrizenych(), $typRozdeleni);
        }

        return $result;
    }

    /*
    public static function getFiltersXml($database, $report)
    {
        $reportNameXml = StringUtils::xmlTextConvert($report ->kod);

        // sestavi xml
        $xmlContent =
            "  <printreport kod='$reportNameXml'>\n";

        // ... doplni promenne podle obdobi
        $nazevTypuObdobiDo = '';
        $nazevObdobiDo = '';
        $pocatePredchozihoObdobi = 0;

        if ($report ->zobrazitPanelObdobi) {
            self::setZvolenyTypObdobi((int)$_POST['typObdobi']);

            switch (self::zvolenyTypObdobi()) {
                case 3 :
                    $nazevTypuObdobi = \Ron\ronText('Obdobi_od');
                    $nazevTypuObdobiDo = \Ron\ronText('Do');

                    $obdobiOd = DateUtils::dateStringToTimestamp($_POST['obdobiOd']);
                    self::setZvoleneObdobiOd($_POST['obdobiOd']);

                    $obdobiDo = DateUtils::dateStringToTimestamp($_POST['obdobiDo']);
                    self::setZvoleneObdobiDo($_POST['obdobiDo']);

                    $nazevObdobi = StringUtils::format('date', $obdobiOd);
                    $nazevObdobiDo = StringUtils::format('date', $obdobiDo);
                    break;

                case 0 :
                default :
                    require_once 'adsapp/obdobi.php';
                    $obdobi = Obdobi::vratObdobi($database, 'ADS', DateUtils::dateStringToTimestamp($_POST['obdobiMesic']));
                    RonPortalSession::setObdobi($obdobi ->zacatek, $obdobi ->konec);
                    $nazevTypuObdobi = \Ron\ronText('Obdobi');
                    $obdobiOd = $obdobi ->zacatek;
                    $obdobiDo = $obdobi ->konec;
                    $nazevObdobi = $obdobi ->popis;
                    $pocatePredchozihoObdobi = DateUtils::incMonth($obdobiOd, -1);
                    break;
            }

            $xmlContent .=
                "    <reportparam name='TypObdobi' type='INT' value='" . (int)$_POST['typObdobi'] . "'/>\n" .
                "    <reportparam name='NazevTypuObdobi' value='" . StringUtils::xmlTextConvert($nazevTypuObdobi) . "'/>\n" .
                "    <reportparam name='NazevTypuObdobiDo' value='" . StringUtils::xmlTextConvert($nazevTypuObdobiDo) . "'/>\n" .
                "    <reportparam name='PocatekZvolenehoObdobi' type='DATE' value='" . date('j.n.Y', $obdobiOd) . "'/>\n" .
                "    <reportparam name='PocatekZvolenehoObdobi_SQL_Const' type='SQLDATE' value='" . date('j.n.Y', $obdobiOd) . "'/>\n" .
                "    <reportparam name='KonecZvolenehoObdobi' type='DATE' value='" . date('j.n.Y', $obdobiDo) . "'/>\n" .
                "    <reportparam name='KonecZvolenehoObdobi_SQL_Const' type='SQLDATE' value='" . date('j.n.Y', $obdobiDo) . "'/>\n" .
                "    <reportparam name='NazevZvolenehoObdobi' value='" . StringUtils::xmlTextConvert($nazevObdobi) . "'/>\n" .
                "    <reportparam name='NazevZvolenehoObdobiDo' value='" . StringUtils::xmlTextConvert($nazevObdobiDo) . "'/>\n" .
                "    <reportparam name='PocatekPredchozihoObdobi' type='DATE' value='" . date('j.n.Y', $pocatePredchozihoObdobi) . "'/>\n" .
                "    <reportparam name='PocatekZvolenehoMesice' type='DATE' value='" . date('j.n.Y', DateUtils::firstDayOfMonth($obdobiOd)) . "'/>\n" .
                "    <reportparam name='PocatekZvolenehoCtvrtleti' type='DATE' value='" . date('j.n.Y', DateUtils::firstDayOfQuarter($obdobiOd)) . "'/>\n" .
                "    <reportparam name='PocatekZvolenehoPololeti' type='DATE' value='" . date('j.n.Y', DateUtils::firstDayOfHalfYear($obdobiOd)) . "'/>\n" .
                "    <reportparam name='PocatekZvolenehoRoku' type='DATE' value='" . date('j.n.Y', DateUtils::firstDayOfYear($obdobiOd)) . "'/>\n" .
                '';
        }

        // ... doplni promenne podle osoby
        $pp = self::getPostPP($database, $report);

        $xmlContent .=
            "    <reportparam name='Jen_Zvolena_Osoba' type='INT' value='" . ($pp ? '1' : '0') . "'/>\n" .
            "    <reportparam name='Osoba_RC' value='" . StringUtils::xmlTextConvert($pp ? $pp ->rc : '') . "'/>\n" .
            "    <reportparam name='Osoba_OsobniCislo' value='" . StringUtils::xmlTextConvert($pp ? $pp ->osobniCislo : '') . "'/>\n" .
            "    <reportparam name='Osoba_DatumNastupu' type='DATE' value='" . ($pp ? date('j.n.Y', $pp ->datumNastupu) : '1.1.1970') . "'/>\n" . // TODO: jake datum?
            "    <reportparam name='Osoba_DatumNastupu_SQL_Const' type='SQLDATE' value='" . ($pp ? date('j.n.Y', $pp ->datumNastupu) : '1.1.1970') . "'/>\n" .
            "    <reportparam name='Jen_Typ_Osoby' type='INT' value='" . ($report ->zobrazitPanelOsoba ? self::getJenTypOsoby() : 0) . "'/>\n" .
            "    <reportparam name='Typ_Osoby' type='INT' value='" . ($report ->zobrazitPanelOsoba ? self::getTypOsoby() : 0) . "'/>\n" .
            "    <reportparam name='Jen_Typ_PP' type='INT' value='" . ($report ->zobrazitPanelOsoba ? self::getJenTypPP() : 0) . "'/>\n" .
            "    <reportparam name='Typ_PP' value='" . ($report ->zobrazitPanelOsoba ? StringUtils::xmlTextConvert(self::getTypPP()) : '') . "'/>\n" .
            '';

        // ... doplni filtry podle rozdeleni
        $rozdeleni = self::getPostRozdeleni($database, $report);

//        var_dump($rozdeleni);
//        throw new \Exception('aa');

        $xmlContent .=
            "    <reportparam name='Jen_Zvolene_Rozdeleni' type='INT' value='" .  (self::getJenRozdeleni() ? '1' : '0') ."'/>\n" .
            "    <reportparam name='Vcetne_Podrizenych_Rozdeleni' type='INT' value='" . (self::getRozdeleniVcetnePodrizenych() ? '1' : '0') . "'/>\n" .
            "    <reportparam name='TypRozdeleni_Kod' type='INT' value='" . ($rozdeleni ? $rozdeleni ->typ : 0) . "'/>\n" .
            "    <reportparam name='TypRozdeleni_Popis' value='" . ($rozdeleni ? $rozdeleni ->typ : 0) . "'/>\n" .
            "    <reportparam name='Rozdeleni_Kod' value='" . ($rozdeleni ? StringUtils::xmlTextConvert($rozdeleni ->kod) : '') . "'/>\n" .
            "    <reportparam name='Rozdeleni_Popis' value='" . ($rozdeleni ? StringUtils::xmlTextConvert($rozdeleni ->popis) : '') . "'/>\n" .
            "    <reportparam name='Jen_Zvolene_RozdeleniDo' type='INT' value='0'/>\n" .
            "    <reportparam name='RozdeleniDo_Kod' value=''/>\n" .
            "    <reportparam name='RozdeleniDo_Popis' value=''/>\n" .
            '';

        // ... doplni filtry podle rozdeleni 2
        $xmlContent .=
            "    <reportparam name='Jen_Zvolene_Rozdeleni2' type='INT' value='0'/>\n" .
            "    <reportparam name='Vcetne_Podrizenych_Rozdeleni2' type='INT' value='0'/>\n" .
            "    <reportparam name='TypRozdeleni_Kod2' type='INT' value='" . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP) . "'/>\n" .
            "    <reportparam name='TypRozdeleni_Popis2' value='" . StringUtils::xmlTextConvert(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_NADPIS)) . "'/>\n" .
            "    <reportparam name='Rozdeleni_Kod2' value=''/>\n" .
            "    <reportparam name='Rozdeleni_Popis2' value=''/>\n" .
            "    <reportparam name='Jen_Zvolene_RozdeleniDo2' type='INT' value='0'/>\n" .
            "    <reportparam name='RozdeleniDo_Kod2' value=''/>\n" .
            "    <reportparam name='RozdeleniDo_Popis2' value=''/>\n" .
            '';

        // ... panel Kalendar
        if ($report ->zobrazitPanelKalendar  && isset($_POST['jenKalendar'])  &&  $_POST['jenKalendar']) {
            require_once 'dochazka/kalendar.php';
            $kalendar = Kalendar::nactiKalendar($database, $_POST['kalendar']);

            self::setJenKalendar(1);
            self::setKalendar($_POST['kalendar']);
        } else {
            $kalendar = null;
            self::setJenKalendar(0);
        }

        $xmlContent .=
            "    <reportparam name='Jen_Zvoleny_Kalendar' type='INT' value='" . ($kalendar ? '1' : '0') . "'/>\n" .
            "    <reportparam name='KalendarOd_Kod' value='" . ($kalendar ? StringUtils::xmlTextConvert($kalendar ->kod) : '') . "'/>\n" .
            "    <reportparam name='KalendarOd_Popis' value='" . ($kalendar ? StringUtils::xmlTextConvert($kalendar ->popis) : '') . "'/>\n" .
            // TODO: dodelat nastaveni "kalendar do"
            "    <reportparam name='Jen_Zvoleny_KalendarDo' type='INT' value='0'/>\n" .
            "    <reportparam name='KalendarDo_Kod' value=''/>\n" .
            "    <reportparam name='KalendarDo_Popis' value=''/>\n";

        // neimplementovane panely
        $xmlContent .=
            // vycet osob
            "    <reportparam name='Jen_Zvoleny_Vycet_Osob' type='INT' value='0'/>\n" .
            //"    <reportparam name='_Zvoleny_Vycet_Osob_SQL_Filter' value='0 = 0'/>\n" . - z bezpecnostnich duvodu neprenaset
            // vycet rozdeleni
            "    <reportparam name='Jen_Zvolene_Rozdeleni_v1' type='INT' value='0'/>\n" .
            "    <reportparam name='TypRozdeleni_Kod_v1' type='INT' value='0'/>\n" .
            "    <reportparam name='TypRozdeleni_Popis_v1' value=''/>\n" .
            "    <reportparam name='Rozdeleni_v1' value=''/>\n" .
            //'    <reportparam name="_Rozdeleni_v1_sql" value="' . "''" . '"/>' . "\n" . - z bezpecnostnich duvodu neprenaset
            // vycet rozdeleni 2
            "    <reportparam name='Jen_Zvolene_Rozdeleni_v2' type='INT' value='0'/>\n" .
            "    <reportparam name='TypRozdeleni_Kod_v2' type='INT' value='0'/>\n" .
            "    <reportparam name='TypRozdeleni_Popis_v2' value=''/>\n" .
            "    <reportparam name='Rozdeleni_v2' value=''/>\n" .
            //'    <reportparam name="_Rozdeleni_v2_sql" value="' . "''" . '"/>' . "\n" . - z bezpecnostnich duvodu neprenaset
            '';


        $xmlContent .=
            "  </printreport>\n";

        return $xmlContent;
    }
    */


//    /**
//     * @param Database $database
//     * @param Report $report
//     * @param string $format
//     * @return ReportOutputObject
//     * @throws \Exception
//     */
//    public static function getReportOutputObject($database, $report, $format = 'pdf')
//    {
//        $result = new ReportOutputObject($database);
//
//        $result ->report = $report;
//        $result ->reportName = $report ->kod;
//        $result ->format = $format;
//
//        $obdobiOd = null;
//        $obdobiDo = null;
//
//        if ($report ->zobrazitPanelObdobi) {
//            self::setZvolenyTypObdobi((int)$_POST['typObdobi']);
//
//            switch (self::zvolenyTypObdobi()) {
//                case 3 :
//                    $obdobiOd = DateUtils::dateStringToTimestamp($_POST['obdobiOd']);
//                    self::setZvoleneObdobiOd($_POST['obdobiOd']);
//
//                    $obdobiDo = DateUtils::dateStringToTimestamp($_POST['obdobiDo']);
//                    self::setZvoleneObdobiDo($_POST['obdobiDo']);
//
//                    break;
//
//                case 0 :
//                default :
//                    require_once 'adsapp/obdobi.php';
//                    $obdobi = Obdobi::vratObdobi($database, 'ADS', DateUtils::dateStringToTimestamp($_POST['obdobiMesic']));
//                    RonPortalSession::setObdobi($obdobi->zacatek, $obdobi->konec);
//                    $obdobiOd = $obdobi->zacatek;
//                    $obdobiDo = $obdobi->konec;
//                    break;
//            }
//
//            $result ->dateParams =
//                "pocatekZvolenehoObdobi=" . date('j.n.Y', $obdobiOd) . ';' .
//                "konecZvolenehoObdobi=" . date('j.n.Y', $obdobiDo);
//        }
//
//        $pp = self::getPostPP($database, $report);
//
//        if ($pp) {
//            // obdobi pro prava podle zvoleneho obdobi
//            $result ->nastavPP($pp, $obdobiOd ?: time(), $obdobiDo ?: null);
//        }
//
//        if ($report ->zobrazitPanelOsoba) {
//            $result ->stringParams .=
//                ($result->stringParams ? ';' : '') .
//                'TypOsoby=' . (self::getJenTypOsoby() ? self::getTypOsoby() : '') .
//                ';TypPP=' . (self::getJenTypPP() ? self::getTypPP() : '');
//        }
//
//        if ($report ->zobrazitPanelRozdeleni) {
//            $rozdeleni = self::getPostRozdeleni($database, $report);
//
//            $result->stringParams .=
//                ($result->stringParams ? ';' : '') .
//                'TypRozdeleni=' . ($rozdeleni ? $rozdeleni->typ : 0) .
//                ';Rozdeleni=' . ($rozdeleni ? $rozdeleni->kod : '');
//        }
//
//        //$this ->stringParams = isset($_POST['stringparams']) ? $_POST['stringparams'] : null;
//        //$this ->dateParams = isset($_POST['dateparams']) ? $_POST['dateparams'] : null;
//        //$result ->stringParams =
//
//        return $result;
//    }
}
