<?php
/**
 * UTF8
 * stanislav.elbl, 12.10.2015
 */

namespace Ron\RonPortal;

use Ron\DateUtils;
use Ron\Http\HttpHeaders;
use Ron\Http\HttpResponse;
use Ron\MessageManager;
use Ron\MVC\OutputObject;
use Ron\DB\Database;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;

/**
 * Zajistuje komunikaci s report serverem
 * Class ReportServer
 */
class ReportServer
{
    var $socket;

    var $errorReporting;
        // pri open si zapamatuje nastaveni error reporting a pri close ho vrati zpet

    public function __construct()
    {

    }


    public static function getReportSessionId()
    {
        return sha1(session_id());
    }

    /**
     * @param $server
     * @param $port
     * @throws \Exception
     */
    public function open($server, $port)
    {
        $this ->errorReporting = error_reporting(E_ERROR); // vypne warningy

        $errno = 0;
        $errstr = '';
        $this ->socket = fsockopen($server, $port, $errno, $errstr);

        if (!$this ->socket) {
            throw new \Exception("Nelze navazat spojeni s report serverem!\nChyba cislo $errno: $errstr");
        }
    }


    public function close()
    {
        fclose($this ->socket);
        $this ->socket = null;

        error_reporting($this ->errorReporting); // nastavi puvodni error reporting
    }


    /**
     * Odesle zpravu, na kterou je odpoved jen OK/Err. Pri chybe vyhodi vyjimku.
     * @param string $msg
     * @throws \Exception
     */
    function sendMessage($msg)
    {
        $msg = StringUtils::toUTF8($msg);

        fputs($this ->socket, $msg . "\n");
        $answer = trim(fgets($this ->socket));

        //stream_set_timeout()

        if ($answer == 'ERR') {
            $answer = StringUtils::fromUTF8(trim(fgets($this ->socket)));

            throw new \Exception($answer);
        } else if ($answer != 'OK') {
            throw new \Exception("Chyba pri odesilani prikazu: $msg!");
        }
    }


    /**
     * @param string $msg
     * @param bool $oneRowResult   Pokud je true, nacte se jen jeden radek odpovedi
     * @return string
     * @throws \Exception
     */
    function sendRequest($msg, $oneRowResult = false)
    {
        $msg = StringUtils::toUTF8($msg);
        fputs($this ->socket, $msg . "\n");

        $result = '';
        $xml = null;

        while (1) {
            $answer = fgets($this ->socket);

            if ($answer === false) {
                MessageManager::addDebugLog($answer);
                $info = stream_get_meta_data($this ->socket);
                if ($info &&  $info['timed_out']) {
                    throw new \Exception('Chyba pri komunikaci s printserverem - vyprsel casovy limit odpovedi printserveru!');
                }
                throw new \Exception('Chyba pri komunikaci s printserverem - prijata neuplna odpoved!');
            }

            $answer = trim($answer);

            if ($oneRowResult) {
                return StringUtils::fromUTF8($answer);
            }

            if (!isset($xml)  &&  $answer == 'XML') {
                $xml = true; // priznak, ze se jedna o XML
            } else if (!$xml  &&  $answer == 'ERR') {
                $answer = StringUtils::fromUTF8(trim(fgets($this ->socket)));

                throw new \Exception($answer);
            } else if (!$xml  &&  $answer == 'OK') {
                return StringUtils::fromUTF8($result);
            } else if ($xml  &&  $answer == 'XMLEND') {
                return $result; // ponecha v UTF8!
            } else {
                $result .= $answer . "\n";
            }

            if (!isset($xml)) $xml = false;
        }

        return ''; // tu se vubec nedostane
    }


    function sendXmlRequest($xmlContent)
    {
        $time = date('j.n.Y G:i:s', time());
        $pwd = 'i7xU-D91AhjM/prD6'; // TODO: umoznit uzivatelske nastaveni hesla
        $xml =
            "<?xml version='1.0'?>\n" .  //  encoding='utf-8'
            "<report userRC='" . StringUtils::xmlTextConvert(User::getRC()) . "' key='" . sha1($pwd . StringUtils::toUTF8(User::getRC()) . $time) . "' time='$time'>\n" .
            $xmlContent .
            "</report>\n";

        return $this ->sendRequest("XML\n" . $xml . "\nXMLEND", false);
    }


    /**
     * Zapise na vystup zbyvajici obsah streamu
     */
    function outputContent()
    {
        fpassthru($this ->socket);
    }


    function downloadContent()
    {
        return stream_get_contents($this ->socket);
    }

    /**
     * Spusteni skriptu zadaneho jmena
     * @param $category
     * @param $name
     * @param $scriptParam
     * @throws \Exception
     */
    function executeScript($category, $name, $scriptParam) {
        $this ->sendMessage('USER ' . User::getLoginName() . "\n");
        $this ->sendMessage('LONGVAR AgentParameter=' . $scriptParam . "\n");

        $this ->sendMessage("SCRIPT $category/$name\n");

        $this ->sendMessage('EXIT');
    }


    /**
     * Vygenerovani reportu pro stare sestavy (ne z manazera sestav)
     * @param string $reportName
     * @param string $format
     * @param PracovniPomer $pp
     * @param string[]  $stringParams
     * @param string[]  $dateParams
     * @param int[]     $integerParams
     * @throws \Exception
     */
    function createReport($reportName, $format, $pp, $stringParams, $dateParams, $integerParams)
    {
        $this ->sendMessage('USER ' . User::getLoginName() . "\n");

        $this ->sendMessage('VAR LoginName=' . User::getLoginName() . "\n");
        $this ->sendMessage('VAR RealLoginName=' . User::getRealLoginName() . "\n");
        if ($pp) {
            $this ->sendMessage(
                'VAR RC=' . $pp->rc .
                ';osobniCislo=' . $pp->osobniCislo .
                "\n");

            $this ->sendMessage('DATEVAR datumNastupu=' . date('j.n.Y', $pp->datumNastupu) . "\n");
        } else {
            $this->sendMessage("VAR RC=\n");
        }

        // odesle dalsi prikazy (napr. parametry - id sluzebni cesty a podobne)
        $paramsText = '';
        if ($stringParams) {
            foreach ($stringParams as $name => $value) {
                $paramsText .= ($paramsText ? ';' : '') . $name . '=' . $value;
            }
        }
        // stare sestavy (ne z manazera) pouzivaly misto integeru stringy - prida je ke stringum
        foreach ($integerParams as $name => $value) {
            $paramsText .= ($paramsText ? ';' : '') . $name . '=' . $value;
        }

        if ($paramsText) {
            $this ->sendMessage("VAR $paramsText\n");
        }

        if ($dateParams) {
            $paramsText = '';
            foreach ($dateParams as $name => $value) {
                $paramsText .= ($paramsText ? ';' : '') . $name . '=' . $value;
            }
            $this ->sendMessage("DATEVAR $paramsText\n");
        }

        if ($format) {
            $this ->sendMessage("FORMAT $format\n");
        }

        //$this ->sendMessage( iconv('windows-1250', 'utf-8', "PRINT $reportName\n") );  // 22.12.2016 - standa - patri tu iconv? pro sestavy s diakritikou to nefachci!
        $this ->sendMessage( "PRINT $reportName\n" );
    }


    public static function getFileExtension($format)
    {
        if (!$format) {
            return 'pdf';
        }

        switch (strtolower($format)) {
            case 'pdf' :
            case 'pdfa' :
                return 'pdf';

            default :
                return $format;
        }
    }

    /**
     * Vyexportovani reportu pro stare sestavy
     * @param string $reportName
     * @param string $format
     * @param PracovniPomer $pp
     * @param string[] $stringParams
     * @param string[] $dateParams
     * @param int[] $integerParams
     * @throws \Exception
     */
    function outputReport($reportName, $format, $pp, $stringParams, $dateParams, $integerParams)
    {
        $this ->createReport($reportName, $format, $pp, $stringParams, $dateParams, $integerParams);

        HttpResponse::getInstance() ->getHeaders() ->addHeadersForContent($reportName . '.' . self::getFileExtension($format), 0);
        HttpResponse::getInstance() ->renderHeaders();

        $this ->outputContent();
    }


    /**
     * Nacte informace o stavu reportServeru
     * @return string
     * @throws \Exception
     */
    function getInfo()
    {
        $info = $this ->sendRequest('DBINFO');
        $this ->sendMessage('EXIT');

        return $info;
    }


    /**
     * Nacte informace o dane sestave z ccf
     * @param Report $report
     * @return array
     * @throws \Exception
     */
    public static function getReportInfo($report)
    {
        require_once('adsapp/reportserver.php');

        $reportNameXml = StringUtils::xmlTextConvert($report->kod);

        $xmlContent =
            "  <reportinfo kod='$reportNameXml' />\n";

        $server = ReportServer::createInstance();
        try {
            $xml = $server->sendXmlRequest($xmlContent);

            require_once 'ron/xmlloader.php';
            $info = XmlLoader::loadXmlToArray($xml, [XmlLoader::PARAM_INCLUDE_ATTRIBUTES => true, XmlLoader::PARAM_CONVERT_FROM_UTF8 => true]);

            return $info;
        } catch (\Exception $e) {
            $server->close();
            throw $e;
        }
    }


    /**
     * Zpracuje informace o sestave nactene z ccf pomoci reportserveru
     * @param Report $report
     * @param array $info
     */
    public static function processReportInfo($report, $info)
    {
        // zjisti z printserveru aktivni panely ...
        $report->zobrazitPanelObdobi = true;
        $report->zobrazitPanelOsoba = true;
        $report->zobrazitPanelKalendar = true;
        $report->zobrazitPanelRozdeleni = true;
        $report->zobrazitPanelRozdeleni2 = true;

        if ( is_array($info['panels']) &&  is_array($info['panels'][0]['panel']) ) {
            //var_dump($info['panels'][0]['panel']);
            foreach ($info['panels'][0]['panel'] as $panelInfo) {
                switch (strtolower($panelInfo['name'])) {
                    case 'cp_obdobi' :
                        $report ->zobrazitPanelObdobi = $panelInfo['visible'] != '0';
                        break;
                    case 'cp_osoba' :
                        $report ->zobrazitPanelOsoba = $panelInfo['visible'] != '0';
                        break;
                    case 'cp_kalendar' :
                        $report ->zobrazitPanelKalendar = $panelInfo['visible'] != '0';
                        break;

                    case 'cp_rozdeleni' :
                        $report ->zobrazitPanelRozdeleni = $panelInfo['visible'] != '0';
                        break;
                    case 'cp_rozdeleni2' :
                        $report ->zobrazitPanelRozdeleni2 = $panelInfo['visible'] != '0';
                        break;

                    case 'cp_filter' :
                    case 'cp_grouping' :
                    case 'cp_ordering' :
                    case 'cp_osoba_vycet' :
                    case 'cp_rozdeleni_v1' :
                    case 'cp_rozdeleni_v2' :
                        break;
                }
            }
        }
    }


    public static function loadReportInfo($report)
    {
        $info = self::getReportInfo($report);
        self::processReportInfo($report, $info);
    }


    /**
     * @return ReportServer
     * @throws \Exception
     */
    static function createInstance()
    {
        global $reportserver;
        global $reportserverport;

        if (!$reportserver || !$reportserverport) {
            throw new \Exception('Chyba pri spousteni skriptu pomoci reportserveru: v _appdbset.php neni nastaveno spojeni na reportserver!');
        }

        $result = new ReportServer();
        $result ->open($reportserver, $reportserverport);

        return $result;
    }
}


/**
 * Output object pro generovani reportu.
 * Class ReportOutputObject
 * @package Ron\RonPortal
 */
class ReportOutputObject implements OutputObject
{
    /** @var Report */
    var $report;

    var $format;
    //var $xmlContent; // jen obsah xml bez korenoveho elementu (<report>) a hlavicky

    // hodnoty filtru
    var $zvolenyTypObdobi;
    /** @var int    Pocatek zvoleneho obdobi (mesic nebo datum od) */
    var $obdobiOd;
    /** @var int    Konec zvoleneho obdobi */
    var $obdobiDo;
    /** @var string Nazev obdobi */
    var $nazevObdobi = null;

    /** @var PracovniPomer */
    var $pp = null;

    /** @var bool   Jestli se ma zobrazit jen jeden typ osoby */
    var $jenTypOsoby;
    /** @var int    Vybrany typ osoby */
    var $typOsoby;

    /** @var bool   Jestli se ma zobrazit jen jeden typ PP */
    var $jenTypPP;
    /** @var string Vybrany typ PP */
    var $typPP;

    /** @var bool */
    var $jenRozdeleni;
    /** @var bool */
    var $rozdeleniVcetnePodrizenych;
    /** @var Rozdeleni      Vybrane rozdeleni */
    var $rozdeleni;
    /** @var TypRozdeleni   Vybrany typ rozdeleni */
    var $typRozdeleni;

    /** @var bool */
    var $jenRozdeleni2;
    /** @var bool */
    var $rozdeleni2VcetnePodrizenych;
    /** @var Rozdeleni  Vybrane rozdeleni 2 */
    var $rozdeleni2;

    /** @var bool */
    var $jenKalendar;
    /** @var Kalendar */
    var $kalendar;

    /** @var string[]   Asociativni pole datumovych parametru (format j.n.Y) */
    var $dateParams = [];
    /** @var string[]   Asociativni pole parametru typu string */
    var $stringParams = [];
    /** @var int[]      Asociativni pole parametru typu integer */
    var $integerParams = [];

    /**
     * ReportOutputObject constructor.
     * @param Report $report
     * @param string $format
     * @throws \Exception
     */
    public function __construct($report = null, $format = null)
    {
        global $reportserver;
        global $reportserverport;

        if (!$reportserver || !$reportserverport) {
            throw new \Exception('V _appdbset.php neni nastaveno spojeni na reportserver!');
        }

        $this->report = $report;
        $this->format = $format;
    }

    /**
     * @param Database  $database
     * @param string    $reportName
     * @throws \Exception
     */
    public function loadReport($database, $reportName)
    {
        if (!$reportName) {
            throw new \Exception('Neni zadan kod sestavy!');
        }

        require_once 'adsapp/report.php';
        $this ->report = Report::loadReport($database, $reportName);

        // pokud neni nastaven format, tak nastavi prvni povoleny
        $bitMask = 0x01;
        $formaty = Report::FORMATY; // jen kvuli warningu v jIDEA
        for ($i = 0;  !$this ->format  &&  $i < count($formaty);  $i++) {
            if ($this ->report ->formaty & $bitMask) {
                $this ->format = $formaty[$i];
            }

            $bitMask *= 2;
        }
    }


    /**
     * Nacte parametry z GET (reportname, format). Ostatni parametry musi nastavit volajici kod.
     * @param Database $database
     * @param string|string[] $formName Jmeno formulare (nebo pole jmen), ke kteremu sestava patri
     * @param bool $utf8
     * @param string $deprecated_reportPrefix Pokud je zadan, zkontroluje se, ze ma nazev reportu zadany prefix
     * @throws \Exception
     */
    public function loadReportFromGet($database, $formName, $utf8 = false, $deprecated_reportPrefix = null)
    {
        $reportName = $utf8 ? StringUtils::fromUTF8($_GET['reportname']) : $_GET['reportname'];
        $this ->loadReport($database, $reportName);

        $ok = false;
        // zkontroluje, jestli zadany report patri k zadanemu formulari
        if ($formName) {
            if ($this ->report ->formular) {
                if (is_array($formName)) {
                    if (in_array($this ->report ->formular, $formName)) {
                        $ok = true;
                    }
                } else if ($formName == $this ->report ->formular) {
                    $ok = true;
                }

                if (!$ok) {
                    throw new \Exception('Zadana sestava nepatri k aktualnimu formulari!');
                }
            }
        }

        // ve starsich verzich se zobrazovaly sestavy podle zacatku kodu sestavy (ne podle prirazeneho formulare) - pro kompatibilitu jeste zkontroluje zacatek jmena
        if (!$ok  &&  $deprecated_reportPrefix) {
            if ($this ->report ->appReportName) {
                // nove sestavy uz musi mit nastaveny formName
                throw new \Exception('Sestava nema prirazen Formular! Prefix nazvu neni u sestav z manazera podporovan!');
            }

            if (0 !== stripos($reportName, $deprecated_reportPrefix)) {
                throw new \Exception('Neplatny nazev sestavy! Je ocekavan prefix ' . $deprecated_reportPrefix . '.');
            }
        }

        if (isset($_GET['format'])) {
            $this ->format =  $_GET['format'];
        }

        if ($this ->report ->appReportName) {
            require_once 'adsapp/reportserver.php';
            ReportServer::loadReportInfo($this ->report);
        }
    }


    /**
     * @param int $typObdobi    3=datum od-do, 0=obdobi aplikace
     * @param $obdobiOd
     * @param null $obdobiDo
     * @param null $nazevObdobi
     */
    public function nastavObdobi($typObdobi, $obdobiOd, $obdobiDo, $nazevObdobi = null)
    {
        $this ->zvolenyTypObdobi = $typObdobi;
        $this ->obdobiOd = $obdobiOd;
        $this ->obdobiDo = $obdobiDo;
        $this ->nazevObdobi = $nazevObdobi;
    }

    /**
     * Prida datumove promenne
     * @param $datumOd
     * @param $datumDo
     */
    public function nastavObdobiOdDo($datumOd, $datumDo)
    {
        $this ->zvolenyTypObdobi = 3;
        $this ->obdobiOd = $datumOd;
        $this ->obdobiDo = $datumDo;
    }

    /**
     * @param PracovniPomer $pp
     * @param bool $zkontrolovatPrava
     * @throws NeniPravoException
     * @throws \Exception
     */
    public function nastavPP($pp, $zkontrolovatPrava = true)
    {
        $this ->pp = $pp;

        if ($pp  &&  $zkontrolovatPrava) {
            if ($this ->obdobiOd) {
                Prava::nastavObdobiProPrava($this ->obdobiOd, $this ->obdobiDo);
            }

            //$prava = Prava::nactiPravaUzivatele($this ->database, User::getRC(), $this ->report ->prava ?: 'prohlizeni');
            $prava = Prava::vratPrava($this ->report ->prava ?: 'prohlizeni');

            if (!$prava->lzeZobrazitPP($this ->pp ->idPP)) {
                throw new NeniPravoException("Pro zobrazeni sestavy nemate pravo!");
            }
        }
    }

    /**
     * @param bool $jenTypOsoby
     * @param int $typOsoby
     */
    public function nastavTypOsoby($jenTypOsoby, $typOsoby = null)
    {
        $this ->jenTypOsoby = $jenTypOsoby;
        $this ->typOsoby = $jenTypOsoby ? $typOsoby : null;
    }


    /**
     * @param bool $jenTypPP
     * @param string $typPP
     */
    public function nastavTypPP($jenTypPP, $typPP = null)
    {
        $this ->jenTypPP = $jenTypPP;
        $this ->typPP = $jenTypPP ? $typPP : null;
    }


    /**
     * @param bool $jenRozdeleni
     * @param Rozdeleni $rozdeleni
     * @param bool $vcetnePodrizenych
     * @param TypRozdeleni $typRozdeleni
     */
    public function nastavRozdeleni($jenRozdeleni, $rozdeleni, $vcetnePodrizenych, $typRozdeleni = null)
    {
        $this ->jenRozdeleni = $jenRozdeleni;
        $this ->rozdeleni = $rozdeleni;
        $this ->rozdeleniVcetnePodrizenych = $vcetnePodrizenych;
        $this ->typRozdeleni = $typRozdeleni;
    }

    /**
     * @param bool  $jenRozdeleni
     * @param Rozdeleni $rozdeleni
     * @param bool  $vcetnePodrizenych
     */
    public function nastavRozdeleni2($jenRozdeleni, $rozdeleni, $vcetnePodrizenych)
    {
        $this ->jenRozdeleni2 = $jenRozdeleni;
        $this ->rozdeleni2 = $rozdeleni;
        $this ->rozdeleni2VcetnePodrizenych = $vcetnePodrizenych;
    }

    /**
     * @param bool $jenKalendar
     * @param string $kalendar
     */
    public function nastavKalendar($jenKalendar, $kalendar = null)
    {
        $this ->jenKalendar = $jenKalendar;
        $this ->kalendar = $jenKalendar ? $kalendar : null;
    }


    /**
     * Pokud jsou parametry zadany jako string (pouzivalo se ve starem kodu), tak je prevede na asociativni pole
     */
    private function processParams()
    {
        if ($this ->stringParams  &&   is_string($this ->stringParams)) {
            $result = [];
            $params = explode(';', $this ->stringParams);
            foreach ($params as $paramStr) {
                $pom = explode('=', $paramStr, 2);
                $result[$pom[0]] = $pom[1];
            }
            $this ->stringParams = $result;
        }

        if ($this ->dateParams  &&   is_string($this ->dateParams)) {
            $result = [];
            $params = explode(';', $this ->dateParams);
            foreach ($params as $paramStr) {
                $pom = explode('=', $paramStr, 2);
                $result[$pom[0]] = $pom[1];
            }
            $this ->dateParams = $result;
        }
    }


    /**
     * @param int $typObdobi
     * @param int $datumOd
     * @param int $datumDo
     * @param string $nazevObdobi
     * @return string
     */
    public static function getXMLParamsObdobi($typObdobi, $datumOd, $datumDo, $nazevObdobi)
    {
        $nazevTypuObdobiDo = '';
        $nazevObdobiDo = '';
        $pocatekPredchozihoObdobi = 0;

        switch ($typObdobi) {
            case 1 :
                // ctvrtleti
                $nazevTypuObdobi = \Ron\ronText('Ctvrtleti');
                $nazevObdobi = DateUtils::quarterDescription($datumOd);
                $pocatekPredchozihoObdobi = DateUtils::incMonth($datumOd, -3);
                break;

            case 2 :
                // rok
                $nazevTypuObdobi = \Ron\ronText('Rok');
                $nazevObdobi = date('Y', $datumOd);
                $pocatekPredchozihoObdobi = DateUtils::incMonth($datumOd, -12);
                break;

            case 3 :
                // datum od - do
                $nazevTypuObdobi = \Ron\ronText('Obdobi_od');
                $nazevTypuObdobiDo = \Ron\ronText('Do');

                $nazevObdobi = StringUtils::format('date', $datumOd);
                $nazevObdobiDo = StringUtils::format('date', $datumDo);
                break;

            case 0 :
            default :
                require_once 'adsapp/obdobi.php';
                $nazevTypuObdobi = \Ron\ronText('Obdobi');
                $pocatekPredchozihoObdobi = DateUtils::incMonth($datumOd, -1);
                break;
        }

        return
            "    <reportparam name='TypObdobi' type='INT' value='" . $typObdobi . "'/>\n" .
            "    <reportparam name='NazevTypuObdobi' value='" . StringUtils::xmlTextConvert($nazevTypuObdobi) . "'/>\n" .
            "    <reportparam name='NazevTypuObdobiDo' value='" . StringUtils::xmlTextConvert($nazevTypuObdobiDo) . "'/>\n" .
            "    <reportparam name='PocatekZvolenehoObdobi' type='DATE' value='" . date('j.n.Y', $datumOd) . "'/>\n" .
            "    <reportparam name='PocatekZvolenehoObdobi_SQL_Const' type='SQLDATE' value='" . date('j.n.Y', $datumOd) . "'/>\n" .
            "    <reportparam name='KonecZvolenehoObdobi' type='DATE' value='" . date('j.n.Y', $datumDo) . "'/>\n" .
            "    <reportparam name='KonecZvolenehoObdobi_SQL_Const' type='SQLDATE' value='" . date('j.n.Y', $datumDo) . "'/>\n" .
            "    <reportparam name='NazevZvolenehoObdobi' value='" . StringUtils::xmlTextConvert($nazevObdobi) . "'/>\n" .
            "    <reportparam name='NazevZvolenehoObdobiDo' value='" . StringUtils::xmlTextConvert($nazevObdobiDo) . "'/>\n" .
            "    <reportparam name='PocatekPredchozihoObdobi' type='DATE' value='" . date('j.n.Y', $pocatekPredchozihoObdobi) . "'/>\n" .
            "    <reportparam name='PocatekZvolenehoMesice' type='DATE' value='" . date('j.n.Y', DateUtils::firstDayOfMonth($datumOd)) . "'/>\n" .
            "    <reportparam name='PocatekZvolenehoCtvrtleti' type='DATE' value='" . date('j.n.Y', DateUtils::firstDayOfQuarter($datumOd)) . "'/>\n" .
            "    <reportparam name='PocatekZvolenehoPololeti' type='DATE' value='" . date('j.n.Y', DateUtils::firstDayOfHalfYear($datumOd)) . "'/>\n" .
            "    <reportparam name='PocatekZvolenehoRoku' type='DATE' value='" . date('j.n.Y', DateUtils::firstDayOfYear($datumOd)) . "'/>\n" .
            '';
    }


    /**
     * @param PracovniPomer $pp
     * @return string
     */
    public static function getXMLParamsOsoba($pp)
    {
        return
            "    <reportparam name='Jen_Zvolena_Osoba' type='INT' value='" . ($pp ? '1' : '0') . "'/>\n" .
            "    <reportparam name='Osoba_RC' value='" . StringUtils::xmlTextConvert($pp ? $pp->rc : '') . "'/>\n" .
            "    <reportparam name='Osoba_OsobniCislo' value='" . StringUtils::xmlTextConvert($pp ? $pp->osobniCislo : '') . "'/>\n" .
            "    <reportparam name='Osoba_DatumNastupu' type='DATE' value='" . ($pp ? date('j.n.Y', $pp->datumNastupu) : '1.1.1970') . "'/>\n" . // TODO: jake datum?
            "    <reportparam name='Osoba_DatumNastupu_SQL_Const' type='SQLDATE' value='" . ($pp ? date('j.n.Y', $pp->datumNastupu) : '1.1.1970') . "'/>\n";
    }

    public function getXML()
    {
        $xmlContent =
            "  <printreport kod='" . StringUtils::xmlTextConvert($this ->report ->kod) ."' format='" . StringUtils::xmlTextConvert($this ->format) . "'>\n";

        if ($this ->report ->zobrazitPanelObdobi) {
            // ... doplni promenne podle obdobi
            $xmlContent .= self::getXMLParamsObdobi((int)$this ->zvolenyTypObdobi, $this ->obdobiOd, $this ->obdobiDo, $this ->nazevObdobi);
        }

        if ($this ->report ->zobrazitPanelOsoba) {
            $xmlContent .=
                self::getXMLParamsOsoba($this ->pp) .
                "    <reportparam name='Jen_Typ_Osoby' type='INT' value='" . (int)$this->jenTypOsoby . "'/>\n" .
                "    <reportparam name='Typ_Osoby' type='INT' value='" . ($this->typOsoby ?: 0) . "'/>\n" .
                // POZOR! Ve mzdach je samostatny panel pro Typ PP a v nem jsou trochu jine promenne!
                "    <reportparam name='Jen_Typ_PP' type='INT' value='" . (int)$this ->jenTypPP . "'/>\n" .
                "    <reportparam name='Typ_PP' value='" . ($this->jenTypPP ? StringUtils::xmlTextConvert($this ->typPP) : '') . "'/>\n" .
                '';
        }

        if ($this ->report ->zobrazitPanelRozdeleni) {
            $xmlContent .=
                "    <reportparam name='Jen_Zvolene_Rozdeleni' type='INT' value='" .  ($this ->jenRozdeleni ? '1' : '0') ."'/>\n" .
                "    <reportparam name='Vcetne_Podrizenych_Rozdeleni' type='INT' value='" . ($this ->rozdeleniVcetnePodrizenych ? '1' : '0') . "'/>\n" .
                "    <reportparam name='TypRozdeleni_Kod' type='INT' value='" . ($this ->rozdeleni ? $this ->rozdeleni ->typ : 0) . "'/>\n" .
                "    <reportparam name='TypRozdeleni_Popis' value='" .
                        ($this ->typRozdeleni
                            ? StringUtils::xmlTextConvert($this ->typRozdeleni ->popis)
                            : ($this ->rozdeleni ? $this ->rozdeleni ->typ : 0)) .
                "'/>\n" .
                "    <reportparam name='Rozdeleni_Kod' value='" . ($this ->rozdeleni ? StringUtils::xmlTextConvert($this ->rozdeleni ->kod) : '') . "'/>\n" .
                "    <reportparam name='Rozdeleni_Popis' value='" . ($this ->rozdeleni ? StringUtils::xmlTextConvert($this ->rozdeleni ->popis) : '') . "'/>\n" .
                "    <reportparam name='Jen_Zvolene_RozdeleniDo' type='INT' value='0'/>\n" .
                "    <reportparam name='RozdeleniDo_Kod' value=''/>\n" .
                "    <reportparam name='RozdeleniDo_Popis' value=''/>\n" .
                '';
        }

        if ($this ->report ->zobrazitPanelRozdeleni2) {
            $xmlContent .=
                "    <reportparam name='Jen_Zvolene_Rozdeleni2' type='INT' value='" .  ($this ->jenRozdeleni2 ? '1' : '0') ."'/>\n" .
                "    <reportparam name='Vcetne_Podrizenych_Rozdeleni2' type='INT' value='" . ($this ->rozdeleniVcetnePodrizenych ? '1' : '0') . "'/>\n" .
                "    <reportparam name='TypRozdeleni_Kod2' type='INT' value='" . ($this ->rozdeleni2 ? $this ->rozdeleni2 ->typ : 0) . "'/>\n" .
                "    <reportparam name='TypRozdeleni_Popis2' value='" . StringUtils::xmlTextConvert(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_NADPIS)) . "'/>\n" .
                "    <reportparam name='Rozdeleni_Kod2' value='" . ($this ->rozdeleni2 ? StringUtils::xmlTextConvert($this ->rozdeleni2 ->kod) : '') . "'/>\n" .
                "    <reportparam name='Rozdeleni_Popis2' value='" . ($this ->rozdeleni2 ? StringUtils::xmlTextConvert($this ->rozdeleni2 ->popis) : '') . "'/>\n" .
                "    <reportparam name='Jen_Zvolene_RozdeleniDo2' type='INT' value='0'/>\n" .
                "    <reportparam name='RozdeleniDo_Kod2' value=''/>\n" .
                "    <reportparam name='RozdeleniDo_Popis2' value=''/>\n" .
                '';
        }

        if ($this ->report ->zobrazitPanelKalendar) {
            $xmlContent .=
                "    <reportparam name='Jen_Zvoleny_Kalendar' type='INT' value='" . ($this ->kalendar ? '1' : '0') . "'/>\n" .
                "    <reportparam name='KalendarOd_Kod' value='" . ($this ->kalendar ? StringUtils::xmlTextConvert($this ->kalendar ->kod) : '') . "'/>\n" .
                "    <reportparam name='KalendarOd_Popis' value='" . ($this ->kalendar ? StringUtils::xmlTextConvert($this ->kalendar ->popis) : '') . "'/>\n" .
                // TODO: dodelat nastaveni "kalendar do"
                "    <reportparam name='Jen_Zvoleny_KalendarDo' type='INT' value='0'/>\n" .
                "    <reportparam name='KalendarDo_Kod' value=''/>\n" .
                "    <reportparam name='KalendarDo_Popis' value=''/>\n";
        }

        // neimplementovane panely
        $xmlContent .=
            // vycet osob
            "    <reportparam name='Jen_Zvoleny_Vycet_Osob' type='INT' value='0'/>\n" .
            //"    <reportparam name='_Zvoleny_Vycet_Osob_SQL_Filter' value='0 = 0'/>\n" . - z bezpecnostnich duvodu neprenaset
            // vycet rozdeleni
            "    <reportparam name='Jen_Zvolene_Rozdeleni_v1' type='INT' value='0'/>\n" .
            "    <reportparam name='TypRozdeleni_Kod_v1' type='INT' value='0'/>\n" .
            "    <reportparam name='TypRozdeleni_Popis_v1' value=''/>\n" .
            "    <reportparam name='Rozdeleni_v1' value=''/>\n" .
            //'    <reportparam name="_Rozdeleni_v1_sql" value="' . "''" . '"/>' . "\n" . - z bezpecnostnich duvodu neprenaset
            // vycet rozdeleni 2
            "    <reportparam name='Jen_Zvolene_Rozdeleni_v2' type='INT' value='0'/>\n" .
            "    <reportparam name='TypRozdeleni_Kod_v2' type='INT' value='0'/>\n" .
            "    <reportparam name='TypRozdeleni_Popis_v2' value=''/>\n" .
            "    <reportparam name='Rozdeleni_v2' value=''/>\n" .
            //'    <reportparam name="_Rozdeleni_v2_sql" value="' . "''" . '"/>' . "\n" . - z bezpecnostnich duvodu neprenaset
            '';

        // neimplementovane panely z mezd
        if (!strcasecmp($this ->report ->applicationId, 'MZDY') ||  !strcasecmp($this ->report ->applicationId, 'PERSON')) {

//            "    <reportparam name='Jen_Typ_PP' type='INT' value='" . (int)$this ->jenTypPP . "'/>\n" .
//            "    <reportparam name='Typ_PP' value='" . ($this->jenTypPP ? StringUtils::xmlTextConvert($this ->typPP) : '') . "'/>\n" .


            $xmlContent .=
                // typ PP
                "    <reportparam name='Jen_Zvoleny_TypPP' type='INT' value='" . (int)$this ->jenTypPP . "'/>\n" .
                "    <reportparam name='TypPPOd_Kod' value='" . ($this->jenTypPP ? StringUtils::xmlTextConvert($this ->typPP) : '') . "'/>\n" .
                "    <reportparam name='Jen_Zvoleny_TypPPDo' type='INT' value='0'/>\n" .
                "    <reportparam name='TypPPDo_Kod' value=''/>\n" .
                // evidencni stav
                "    <reportparam name='Jen_Zvoleny_EvidStav' type='INT' value='0'/>\n" .
                "    <reportparam name='EvidStavOd_Kod' value=''/>\n" .
                "    <reportparam name='Jen_Zvoleny_EvidStavDo' type='INT' value='0'/>\n" .
                "    <reportparam name='EvidStavDo_Kod' value=''/>\n" .
                '';
        }

        if (!$this ->report ->appReportName) {
            // u starych reportu posle jeste puvodni parametry (pro kompatibilitu)
            $xmlContent .=
                "    <reportparam name='RC' value='" . StringUtils::xmlTextConvert($this->pp ? $this->pp->rc : '') . "'/>\n" .
                "    <reportparam name='osobniCislo' value='" . StringUtils::xmlTextConvert($this->pp ? $this->pp->osobniCislo : '') . "'/>\n" .
                "    <reportparam name='datumNastupu' type='DATE' value='" . ($this->pp ? date('j.n.Y', $this->pp->datumNastupu) : '1.1.1970') . "'/>\n" . // TODO: jake datum?
                // u starych sestav se pouzivala promenna LoginName
                "    <reportparam name='LoginName' value='" . StringUtils::htmlTextConvert(User::getLoginName()) . "'/>\n" .
                "    <reportparam name='RealLoginName' value='" . StringUtils::htmlTextConvert(User::getRealLoginName()) . "'/>\n";
        }

        $xmlContent .=
            "    <reportparam name='RealLoginName' value='" . StringUtils::htmlTextConvert(User::getRealLoginName()) . "'/>\n";

        foreach ($this ->integerParams as $paramName => $value) {
            $value = (int)$value; // pro jistotu
            $xmlContent .=
                "    <reportparam name='$paramName' type='INT' value='$value'/>\n";
        }

        foreach ($this ->stringParams as $paramName => $value) {
            $xmlContent .=
                "    <reportparam name='$paramName' value='" . StringUtils::xmlTextConvert($value) . "'/>\n";
        }

        foreach ($this ->dateParams as $paramName => $value) {
            $xmlContent .=
                "    <reportparam name='$paramName' type='DATE' value='$value'/>\n";
        }


        $xmlContent .=
            "  </printreport>\n";

//        echo "<pre>" . StringUtils::htmlTextConvert($xmlContent) . "</pre>";
//        throw new \Exception("test");

        return $xmlContent;
    }


    /**
     * Nastavi parametry (dateParams a stringParams) podle zadanych hodnot - pouzije se pro stare sestavy
     */
    public function setParams()
    {
        // pokud jsou parametry zadany jako retezec, tak je prevede do pole
        $this ->processParams();

        if (!isset($this ->dateParams['pocatekZvolenehoObdobi'])) {
            $this->dateParams['pocatekZvolenehoObdobi'] = date('j.n.Y', $this->obdobiOd);
            $this->dateParams['konecZvolenehoObdobi'] = date('j.n.Y', $this->obdobiDo);
        }

        if (!isset($this ->stringParams['TypOsoby'])) {
            $this->stringParams['TypOsoby'] = $this->jenTypOsoby ? $this->typOsoby : '';
        }

        if (!isset($this ->stringParams['TypPP'])) {
            $this->stringParams['TypPP'] = $this->jenTypPP ? $this->typPP : '';
        }

        if (!isset($this->stringParams['Rozdeleni'])) {
            $this->stringParams['TypRozdeleni'] = $this->rozdeleni ? $this->rozdeleni->typ : 0;
            $this->stringParams['Rozdeleni'] = $this->rozdeleni ? $this->rozdeleni->kod : '';
        }

        if (!isset($this->stringParams['Rozdeleni2'])) {
            $this->stringParams['TypRozdeleni2'] = $this->rozdeleni2 ? $this->rozdeleni2->typ : 0;
            $this->stringParams['Rozdeleni2'] = $this->rozdeleni2 ? $this->rozdeleni2->kod : '';
        }
    }

    /**
     * @throws \Exception
     */
    public function output()
    {
        error_reporting( E_ERROR | E_PARSE );

        $server = ReportServer::createInstance();
        try {
            if (!$this ->report ->appReportName) {
                $this ->setParams();
            }
//            if ($this ->report ->appReportName) {
                $this ->processParams(); // upravi parametry nastavene starym kodem (jako string)

                $server->sendXmlRequest($this->getXML());

                if (isset($_GET['test'])  &&  $_GET['test'] == 'redirect') {
                    HttpResponse::setHeader('Content-Type', 'text/html; charset=' . TextManager::getInstance() ->getCharSet());
                    HttpResponse::setHeader(HttpHeaders::LOCATION, $_SERVER['REQUEST_URI']);
                    HttpResponse::getInstance()->renderHeaders();

                    $server ->downloadContent();
                } else {

                    HttpResponse::getInstance()->getHeaders()->addHeadersForContent($this->report->kod . '.' . ReportServer::getFileExtension($this->format), 0);
                    HttpResponse::getInstance()->renderHeaders();

                    $server->outputContent();
                }
//            } else {
//                // ve stare verzi printserveru se musely puvodni reporty generovat jinym prikazem ...
//                $this ->setParams();
//                $server ->outputReport($this ->report ->kod, $this ->format, $this ->pp, $this ->stringParams, $this ->dateParams, $this ->integerParams);
//            }
        } catch (\Exception $e) {
            $server ->close();
            throw $e;
        }
    }


    /**
     * Vyexportovani reportu
     * @param PracovniPomer $pp
     * @param string[]  $stringParams
     * @param string[]  $dateParams
     * @param int[]     $integerParams
     * @return string
     * @throws \Exception
     */
    function download($pp, $stringParams, $dateParams, $integerParams)
    {
        $server = ReportServer::createInstance();
        try {
//            if ($this ->report ->appReportName) {
                $this ->stringParams = $stringParams ?: [];
                $this ->dateParams = $dateParams ?: [];
                $this ->integerParams = $integerParams ?: [];

                $server->sendXmlRequest($this->getXML());
                return $server ->downloadContent();
//            } else {
//                $server->createReport($this->report->kod, $this->format, $pp, $stringParams, $dateParams, $integerParams);
//                return $server->downloadContent();
//            }
        } catch (\Exception $e) {
            $server ->close();
            throw $e;
        }
    }


    public static function downloadReport($report, $format, $pp, $stringParams, $dateParams, $integerParams)
    {
        $outputObject = new ReportOutputObject($report, $format);
        return $outputObject ->download($pp, $stringParams, $dateParams, $integerParams);
    }
}



class OLDReportOutputObject extends ReportOutputObject
{
    /** @var Database */
    var $database;

    /**
     * OLDReportOutputObject constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct($database = null)
    {
        parent::__construct(null, null);

        $this->database = $database;
    }


    /**
     * Nacte parametry z GET (reportname, format). Ostatni parametry musi nastavit volajici kod.
     * @param string|string[] $formName     Jmeno formulare (nebo pole jmen), ke kteremu sestava patri
     * @param bool $utf8
     * @param string $deprecated_reportPrefix  Pokud je zadan, zkontroluje se, ze ma nazev reportu zadany prefix
     * @throws \Exception
     */
    public function loadGet($formName, $utf8 = false, $deprecated_reportPrefix = null)
    {
        $this ->loadReportFromGet($this ->database, $formName, $utf8, $deprecated_reportPrefix);
        /*
        $reportName = $utf8 ? StringUtils::fromUTF8($_GET['reportname']) : $_GET['reportname'];
        $this ->loadReport($this ->database, $reportName);

        $ok = false;
        // zkontroluje, jestli zadany report patri k zadanemu formulari
        if ($formName) {
            if ($this ->report ->formular) {
                if (is_array($formName)) {
                    if (in_array($this ->report ->formular, $formName)) {
                        $ok = true;
                    }
                } else if ($formName == $this ->report ->formular) {
                    $ok = true;
                }

                if (!$ok) {
                    throw new \Exception('Zadana sestava nepatri k aktualnimu formulari!');
                }
            }
        }

        // ve starsich verzich se zobrazovaly sestavy podle zacatku kodu sestavy (ne podle prirazeneho formulare) - pro kompatibilitu jeste zkontroluje zacatek jmena
        if (!$ok  &&  $deprecated_reportPrefix) {
            if ($this ->report ->appReportName) { // nove sestavy uz musi mit nastaveny formName

            }

            if (0 !== stripos($reportName, $deprecated_reportPrefix)) {
                throw new \Exception('Neplatny nazev sestavy! Je ocekavan prefix ' . $deprecated_reportPrefix . '.');
            }
        }

        $this ->format = isset($_GET['format']) ? $_GET['format'] : null;

        if ($this ->report ->appReportName) {
            require_once 'adsapp/reportserver.php';
            ReportServer::loadReportInfo($this ->report);
        }
        */
    }
}




class PreparedReportOutputObject implements OutputObject
{
    var $reportKey;
    var $format;

    /**
     * PreparedReportOutputObject constructor.
     * @param $reportKey
     * @param $format
     */
    public function __construct($reportKey, $format)
    {
        $this->reportKey = $reportKey;
        $this->format = $format;
    }


    /**
     * @throws \Exception
     */
    public function output()
    {
        $server = ReportServer::createInstance();
        try {
            $xml =
                "  <printpreparedreport key='" . StringUtils::xmlTextConvert($this ->reportKey) ."' format='" . StringUtils::xmlTextConvert($this ->format ?: '') . "'/>\n";
            $server->sendXmlRequest($xml);

            HttpResponse::getInstance()->getHeaders()->addHeadersForContent('report.' . ReportServer::getFileExtension($this ->format), 0);
            HttpResponse::getInstance()->renderHeaders();

            $server->outputContent();

//            if ($this ->format) {
//                $server->sendMessage("FORMAT {$this ->format}\n");
//            }
//
//            $server ->sendMessage("PRINTPREPAREDREPORT {$this ->reportKey}\n");
//
//            HttpResponse::getInstance() ->getHeaders() ->addHeadersForContent('report.' . $this ->format, 0);
//            HttpResponse::getInstance() ->renderHeaders();
//
//            $server ->outputContent();
        } catch (\Exception $e) {
            $server ->close();
            throw $e;
        }
    }

}


