<?php
/**
 * UTF8
 * stanislav.elbl, 24.05.2016
 */

namespace Ron\RonPortal;

use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\DBException;
use Ron\MVC\BaseViewObject;
use Ron\MVC\ViewObject;
use Ron\StringUtils;


class RonPortalReport
{
    const FORMAT_PDF = 0x01;
    const FORMAT_XLS = 0x02;
    const FORMAT_ODS = 0x04;

    /** @var string */
    var $kod;
    /** @var string */
    var $nazev;
    /** @var int */
    var $formaty;
    /** @var string     Nazev pripojene sestavy z aplikace */
    var $appReportName;

    /** @var string */
    var $kategorie;

    /** @var string */
    var $prava;

    /**
     * @param Database $database
     * @param string $sqlFilter
     * @param Parameters $sqlParams
     * @return RonPortalReport[]
     * @throws DBException
     */
    public static function loadSQL($database, $sqlFilter, $sqlParams = null)
    {
        $sql =
            'SELECT Kod, Nazev, Formaty, ReportName, Kategorie, Prava FROM SestavyProIntraweb ' .
            ' WHERE Povolit = 1' .
            ($sqlFilter ? ' AND (' . $sqlFilter . ')' : '') .
            ' ORDER BY Kategorie, Kod';

        $cursor = $database->getSqlResult($sql, $sqlParams);

        $result = [];

        while ($row = $cursor->fetch()) {
            $report = new RonPortalReport();
            $report->kod = $row[0];
            $report->nazev = $row[1];
            $report->formaty = $row[2];
            $report->appReportName = $row[3];

            $report->kategorie = $row[4];
            $report->prava = $row[5];

            $result[] = $report;
        }

        return $result;
    }


    /**
     * Vraci seznam reportu pro zadany formular (tj. reporty, jejich kod zacina zadanym jmenem formulare)
     *
     * @param Database $database
     * @param string|string[] $formName
     * @param string $deprecated_NameStart   NEPOUZIVAT - JEN PRO KOMPATIBILITU!
     * @return RonPortalReport[]
     */
    public static function loadFormReports($database, $formName, $deprecated_NameStart = null)
    {
        $params = $database ->createParameters();

        $sqlFilter = '';

        if (is_array($formName)) {
            foreach ($formName as $name) {
                if ($sqlFilter) {
                    $sqlFilter .= ' OR ';
                }
                $sqlFilter .= 'formular = ' . $params ->addParameter($name);
            }
        } else {
            $sqlFilter = 'formular = ' . $params->addParameter($formName);
        }

        if ($deprecated_NameStart) {
            $sqlFilter .= ($sqlFilter ? ' OR ' : '') . 'Kod LIKE ' . $params->addParameter($deprecated_NameStart . '%');
        }

        return self::loadSQL($database, $sqlFilter, $params);
    }

    /**
     * Vraci seznam reportu pro zadany formular (tj. reporty, jejich kod zacina zadanym jmenem formulare)
     * @param Database $database
     * @param string $reportNameStart
     * @return RonPortalReport[]
     * @deprecated
     */
    public static function loadFormReportsByName($database, $reportNameStart)
    {
        $params = $database ->createParameters();
        return self::loadSQL($database, 'Kod LIKE ' . $params->addParameter($reportNameStart . '%'), $params);
    }
}



class FormReportsListView implements ViewObject
{
    /** @var RonPortalReport[] */
    public $reportList;

    /** @var  string */
    public $action;

    /** @var bool */
    //public $usePOST;

    /**
     * FormReportsListView constructor.
     * @param RonPortalReport[] $reportList
     * @param string $action
     //* @param bool $usePOST
     */
    public function __construct($reportList, $action) // , $usePOST = false)
    {
        $this->reportList = $reportList;
        $this->action = $action;
        //$this->usePOST = $usePOST;
    }


    public function render()
    {
        if ( !count($this ->reportList) ) {
            return;
        }

        echo '<div class="reportlist">';

        foreach ($this ->reportList as $report) {
            //$reportNameParam = urlencode($report ->kod);
            //$reportNameParam = urlencode(StringUtils::htmlTextConvert($report ->kod));
            $reportNameParam = StringUtils::htmlTextConvert(urlencode($report ->kod));

            if (!$report ->formaty) {
                // pouze defaultni format (PDF)
                echo '<div class="report text_link">';
                echo '<a target="_blank" href="' . $this ->action . '&reportname=' . $reportNameParam . '">' . StringUtils::htmlTextConvert($report ->nazev) . '</a>';
                echo "</div>\n";
            } else {
                echo '<div class="report">';
                $first = true;

                if ($report ->formaty & 0x01) {
                    // PDF
                    echo '<SPAN class="text_link">';
                    echo '<a target="_blank" href="' . $this ->action . '&format=pdf&reportname=' . $reportNameParam . '">' .
                        ($first ? StringUtils::htmlTextConvert($report ->nazev) : \Ron\ronText('PDF') ) .
                        '</a>';
                    echo "</SPAN>\n";
                    $first = false;
                }

                if ($report ->formaty & 0x02) {
                    // XLS
                    echo '<SPAN class="text_link">';
                    echo '<a target="_blank" href="' . $this ->action . '&format=xls&reportname=' . $reportNameParam . '">' .
                        ($first ? StringUtils::htmlTextConvert($report ->nazev) : \Ron\ronText('XLS') ) .
                        '</a>';
                    echo "</SPAN>\n";
                    $first = false;
                }
                if ($report ->formaty & 0x04) {
                    // ODS
                    echo '<SPAN class="text_link">';
                    echo '<a target="_blank" href="' . $this ->action . '&format=ods&reportname=' . $reportNameParam . '">' .
                        ($first ? StringUtils::htmlTextConvert($report ->nazev) : \Ron\ronText('ODS') ) .
                        '</a>';
                    echo "</SPAN>\n";
                    $first = false;
                }

                echo '</div>';
            }
        }

        echo '</div>';
    }


    /**
     * @param RonPortalReport[] $reportList
     * @param string $action
     */
    public static function renderReportList($reportList, $action)
    {
        $view = new FormReportsListView($reportList, $action);
        $view ->render();
    }
}


/**
 * Zobrazeni standardnich reportu (menu Tisk)
 * Class ReportsListView
 * @package Ron\RonPortal
 */
class ReportsListView extends BaseViewObject
{
//    /** @var Database */
//    var $database;
//    /** @var string */
//    var $sql;

    /** @var int */
    var $idPP;
    /** @var string */
    var $stringParams;
    /** @var string */
    var $dateParams;

    /** @var  RonPortalReport[] */
    var $reportList;

    /**
     * ReportsListView constructor.
//     * @param Database $database
//     * @param string $sql
//     * @param int $idPP
     * @param string $stringParams
     * @param string $dateParams
     */
    public function __construct(/*$database, $sql, $idPP, */$stringParams = '', $dateParams = '')
    {
        parent::__construct('view/reportlist.php');

        $this->stringParams = $stringParams;
        $this->dateParams = $dateParams;
    }


    /**
     * @param Database $database
     * @param string $sqlFilter
     * @param Parameters $sqlParams
     */
    public function load($database, $sqlFilter, $sqlParams)
    {
        $this ->reportList = RonPortalReport::loadSQL($database, $sqlFilter, $sqlParams);
    }

}
