<?php
/**
 * UTF8
 * stanislav.elbl, 24.08.2015
 */

namespace Ron\RonPortal;

use Ron\Browser;
use Ron\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MVC\BrowserController;
use Ron\StringUtils;
use Ron\XmlLoader;

require_once('ron/browser.php');
require_once('adsapp/ronportalfilter.php');


class JmenoPrijmeniBrowserField extends BrowserField
{
    /** @var int    Index pole se jmenem */
    public $indexJmeno;
    /** @var int    Index pole s prijmenim */
    public $indexPrijmeni;
    /** @var int    Index pole s titulem - zobrazi se pred jmenem */
    public $indexTitul;
    /** @var int    Index pole s titulem - zobrazi se az za jmenem */
    public $indexTitul2;

    /** @var int    Index pole, ktere se zobrazi, kdyz jsou ostatni pole prazdna */
    public $indexIfNull;

    /** @var bool   Urcuje, jestli se ma zobrazit nejdrive jmeno a pak prijmeni */
    public $nejdriveJmeno = false;


    /**
     * JmenoPrijmeniBrowserField constructor.
     * @param null|Database $database
     * @param int $indexJmeno
     * @param int $indexPrijmeni
     * @param int $indexTitul       Index pole s titulem - zobrazi se pred jmenem
     * @param int $indexTitul2      Index pole s titulem - zobrazi se az za jmenem
     */
    public function __construct($database, $indexJmeno, $indexPrijmeni = null, $indexTitul = null, $indexTitul2 = null)
    {
        parent::__construct($database);

        $this->indexJmeno = $indexJmeno;
        $this->indexPrijmeni = $indexPrijmeni;
        $this->indexTitul = $indexTitul;
        $this->indexTitul2 = $indexTitul2;

        $this ->header = \Ron\ronText('Jmeno');
    }


    /**
     * @param boolean $nejdriveJmeno
     * @return JmenoPrijmeniBrowserField
     */
    public function setNejdriveJmeno($nejdriveJmeno)
    {
        $this->nejdriveJmeno = $nejdriveJmeno;
        return $this;
    }

    /**
     * @param int $indexIfNull
     * @return JmenoPrijmeniBrowserField
     */
    public function setIndexIfNull($indexIfNull)
    {
        $this->indexIfNull = $indexIfNull;
        return $this;
    }



    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $text =
            ((isset($this ->indexTitul)  &&  $rowData[$this ->indexTitul])
                ? $rowData[$this ->indexTitul]
                : '');

        // prijmeni/jmeno
        $i = $this ->nejdriveJmeno ? $this ->indexJmeno : $this ->indexPrijmeni;
        if (isset($i)  &&  $rowData[$i]) {
            $text .= ($text ? ' ' : '') . $rowData[$i];
        }

        // jmeno/prijmeni
        $i = $this ->nejdriveJmeno ? $this ->indexPrijmeni : $this ->indexJmeno;
        if (isset($i)  &&  $rowData[$i]) {
            $text .= ($text ? ' ' : '') . $rowData[$i];
        }

        if (isset($this ->indexTitul2)  &&  $rowData[$this ->indexTitul2]) {
            $text .= ($text ? ', ' : '') . $rowData[$this ->indexTitul2];
        }

        if (!$text  &&  isset($this ->indexIfNull)) {
            $text = $rowData[$this ->indexIfNull];
        }

        return parent::getViewText($text, $rowNumber, $recordId, $rowData);
    }


    function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'indexJmeno' :
            case 'indexPrijmeni' :
            case 'indexTitul' :
            case 'indexTitul2' :
            case 'nejdriveJmeno' :
                $this->{$key} = (int)$value;

            default :
                parent::loadConfigurationItem($key, $value);
        }
    }
}



class PriznakSchvaleniField extends BrowserField
{
    public $urovenSchvaleni; // 1..3


    /**
     * PriznakSchvaleniField constructor.
     * @param int $urovenSchvaleni
     * @param mixed[] $options
     */
    public function __construct($urovenSchvaleni, $options)
    {
        parent::__construct(null, $options);

        $this->urovenSchvaleni = $urovenSchvaleni;

        $this ->type = 'priznakschvaleni';

        if (!$this ->className) {
            $this ->className = 'schvaleni';
        }
    }




    function loadConfigurationArray($confArray)
    {
        parent::loadConfigurationArray($confArray); // TODO: Change the autogenerated stub
        if (isset($confArray['urovenSchvaleni'])) {
            $this ->urovenSchvaleni = (int)$confArray['urovenSchvaleni'];
        }
    }


    function getDataCellClass($value, $rowNumber, $recordId, $rowData)
    {
        return
            'schvaleni ' .
            ($value ? 'schvaleni' . $this ->urovenSchvaleni . ' ' : '') .
            parent::getDataCellClass($value, $rowNumber, $recordId, $rowData);
    }


    function getFieldCellText($value, $rowNumber, $recordId, $rowData)
    {
        return ''; // zadny text, je to jen barevny priznak
    }
}


// Pole pro nacteni a zobrazeni rozdeleni osoby - vyhledava jen podle RC nebo podle prac. pomeru
class RozdeleniOsobyField extends BrowserField
{
    var $typRozdeleni;

    var $zobrazitKod;
    var $zobrazitPopis;

    var $indexRC;       // index pole s rodnym cislem osoby (povinne)
    var $indexOsobniCislo; // index pole s OsobnimCislem
    var $indexDatumNastupu; // a datem nastupu (nepovinne)

    var $indexDatumOd;  // index pole s datumem od (nepovinne)
    var $indexDatumDo;  // index pole s datumem do (nepovinne)

    var $datumOd;       // obdobi prirazeni na rozdeleni
    var $datumDo;


    /**
     * @param Database $database
     * @param int $typRozdeleni
     */
    function __construct($database, $typRozdeleni)
    {
        parent::__construct($database);

        $this->database = $database;
        $this->typRozdeleni = $typRozdeleni;

        $this->indexRC = -1;
        $this->indexOsobniCislo = -1;
        $this->indexDatumNastupu = -1;

        $this->indexDatumOd = -1;
        $this->indexDatumDo = -1;

        $this->zobrazitKod = 1;
        $this->zobrazitPopis = 0;

        $this->datumOd = time();
        $this->datumDo = time();

        global $STR_Rozdeleni; // kompat
        $this->header = $STR_Rozdeleni ?: \Ron\ronText('Rozdeleni');
    }


    /**
     * nastavni indexu poli v master tabulce (tj. v $rowdata predanem do GetViewText)
     * @param $indexRC
     * @param int $indexOsC
     * @param int $indexNastup
     */
    function nastavIndexPole($indexRC, $indexOsC = -1, $indexNastup = -1)
    {
        $this->indexRC = $indexRC;
        $this->indexOsobniCislo = $indexOsC;
        $this->indexDatumNastupu = $indexNastup;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$this->database) return '';
        if ($this->indexRC < 0) return '';

        if (!$rowData[$this->indexRC]) return ''; // zadna osoba

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Rozdeleni.Kod, Rozdeleni.Popis ' .
            ' FROM RozdeleniOsob, Rozdeleni' .
            ' WHERE RozdeleniOsob.KodTypuRozdeleni = ' . $params ->addParameter((int)$this->typRozdeleni) .
            '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
            '   AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni ' .
            '   AND RozdeleniOsob.RC = ' . $params->addParameter($rowData[$this->indexRC]);

        if ($this->indexOsobniCislo >= 0 && $this->indexDatumNastupu >= 0) {
            // vyber podle pracovniho pomeru (ne jen podle RC)
            $sql .=
                '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($rowData[$this->indexOsobniCislo]) .
                '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($rowData[$this->indexDatumNastupu]));
        }

        // urceni datumu, ke kteremu se rozdeleni nacita
        if ($this->indexDatumOd >= 0) {
            $datumod = $this ->database ->stringToTimestamp($rowData[$this->indexDatumOd]);
            $datumdo = 0;

            if ($this->indexDatumDo >= 0) $datumdo = $this ->database ->stringToTimestamp($rowData[$this->indexDatumDo]);
        } else {
            $datumod = $this->datumOd;
            $datumdo = $this->datumDo;
        }

        if (!$datumdo) $datumdo = $datumod;

        $sql .=
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($this ->database ->dateToParam($datumdo)) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($this ->database ->dateToParam($datumod)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)';

        $sql .= ' ORDER BY RozdeleniOsob.PlatnostOd ';

        $result = '';

        $cursor = $this ->database ->select($sql, $params);

        while ($row = $cursor ->fetch()) {
            // nacetl zaznam
            if ($result) $result .= ', ';

            if ($this->zobrazitKod) $result .= $row[0];
            if ($this->zobrazitKod && $this->zobrazitPopis) $result .= ' - ';
            if ($this->zobrazitPopis) $result .= $row[1];
        }
        $cursor ->close();

        return $result;
    }
}


/**
 * Pole pro zobrazeni kodu a/nebo popisu rozdeleni (podle nastaveni) - data musi byt v SQL dotazu browseru
 * Class RozdeleniField
 * @package Ron\RonPortal
 */
class RozdeleniField extends BrowserField
{
    var $indexKod;
    var $indexPopis;

    /**
     * @param Database $database
     * @param int $indexKod
     * @param int $indexPopis
     * @param string $fieldCode
     */
    function __construct($database, $indexKod, $indexPopis, $fieldCode = 'col-rozdeleni')
    {
        parent::__construct($database);

        $this->indexKod = $indexKod;
        $this->indexPopis = $indexPopis;
        $this->code = $fieldCode;

        $this->header = \Ron\ronText('Rozdeleni');
        $this->type = 'normal';
        $this->align = 'left';

        if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT) == 1) $this->sqlFieldName = 'Rozdeleni.Kod';
        else $this->sqlFieldName = 'Rozdeleni.Popis';

        $this->ordering = 1;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        switch ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT) ) {
            case 0 : // jen popis
                return $rowData[$this->indexPopis];

            case 1 : // jen kod
                return $rowData[$this->indexKod];

            case 2 : // kod i popis
                $result = $rowData[$this->indexPopis];
                if (isset($rowData[$this->indexKod])) $result .= ' (' . $rowData[$this->indexKod] . ')';

                return $result;
        }

        return '';
    }
}

/**
 * Obali BrowserField do spanu se tridou podle data ukonceni. Zadane datum ukonceni vypise do elementu title.
 * Rozlisuje 3 stavy:
 * <ul>
 * <li> Platny pracovni pomer
 * <li> Pracovni pomer ukonceny nebo s datem ukonceni v aktualnim mesici (podle data).
 * <li> Pracovni pomer s datem ukonceni v budoucnu.
 * </ul>
 * @package Ron\RonPortal
 */
class UkonceniPracPomeruInfoField extends BrowserField
{
    /** @var  int index pole s datem ukonceni prac. pomeru */
    private $indexDatumUkonceniPP;

    /**
     * Obali BrowserField do spanu se tridou podle data ukonceni. Vice info v popisu tridy.
     * @param null|Database $database
     * @param array|null $options
     * @param int $indexDatumUkonceni index pole s datem ukonceni prac. pomeru
     */
    public function __construct($database, $options, $indexDatumUkonceni)
    {
        parent::__construct($database, $options);

        $this->indexDatumUkonceniPP = $indexDatumUkonceni;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $rowClass = 'browser_platny_pp';
        $title = '';

        if ($rowData[$this->indexDatumUkonceniPP]) {
            $datumUkonceni = $this->database->stringToTimestamp($rowData[$this->indexDatumUkonceniPP]);

            if ($datumUkonceni <= DateUtils::lastDayOfMonth(time())) {
                $rowClass = 'browser_ukonceny_pp';
            }
            else {
                $rowClass = 'browser_ukonceni_pp_budoucnost';
            }

            $title = \Ron\ronText('Browser_datum_ukonceni_pp') . ': ' . StringUtils::format('date', $datumUkonceni);
        }

        return '<span class="' . $rowClass . '" title="' . $title . '">' . parent::getViewText($value, $rowNumber, $recordId, $rowData) . ($rowClass ? '</span>' : '');
    }


}


class RonPortalBrowser extends Browser
{
    var $action;

    /** @var RonPortalFilterForm  */
    var $filterForm;
    var $filterFormAppCode; // kod aplikace pro vyber obdobi
    var $filterFormRights;  // typ prav pro vyber osob
    var $filterFormVcetneExternich = false;

    /**
     * @param Database $database
     * @param string $action
     */
    function __construct($database, $action)
    {
        parent::__construct($database);

        $this ->action = $action;
        $this ->browserUrl = "?menuid=$action&";

        $this ->filterFormAppCode = 'ADS';
        $this ->filterFormRights = 'prohlizeni';
        $this ->filterFormVcetneExternich = false;
    }



    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action);

        if ($this ->filterFormAppCode) {
            $this->filterForm->nastavObdobi(true, $this->filterFormAppCode, false); // jenotevrene = false
        }

        $prava = Prava::vratPrava($this ->filterFormRights);

        if ($prava ->zobrazitVyberRozdeleni()) { // napr. pro "pouze vlastni" se vyber nezobrazuje
            $this ->filterForm ->nastavRozdeleni(true);
            $this ->filterForm ->f_pravarozdeleni = $prava;
            $this->filterForm->vcetnePodrizenychRozdeleni = true;

            if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
                $this ->filterForm ->nastavRozdeleni2(true);
                $this->filterForm->vcetnePodrizenychRozdeleni2 = true;
            }
        }

        $this ->filterForm ->nastavOsoby(true, $prava, 0, 0, $this ->filterFormVcetneExternich); // datum od-do podle obdobi
    }


    /**
     * Nacte XML konfiguraci browseru se zadanym kodem
     * @param Database $database
     * @param string $browserCode
     * @return string|null
     */
    public static function loadBrowserConfiguration($database, $browserCode)
    {
        if (!$browserCode) {
            return null;
        }

        $params = $database ->createParameters();
        $cursor = $database ->select(
            'SELECT ObsahXML FROM SablonaXMLFormulare WHERE Popis = ' . $params ->addParameter('config_' . $browserCode),
            $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : null;
    }


    /**
     * Nacte konfiguraci browseru z XML sablony
     * @throws \Exception
     */
    function loadConfiguration()
    {
        if (!$this ->code) {
            return;
        }

        $xml = self::loadBrowserConfiguration($this ->database, $this ->code);
        if ($xml) {
            $this ->loadConfigurationXml($xml);
        }
    }


    function load($sqlFilter = '')
    {
        $this ->loadConfiguration();

        $this ->setFilterForm();

        if ($this ->filterForm) {
            $this ->filterForm ->loadData();
        }

        parent::load($sqlFilter);
    }


    function renderFilter()
    {
        if ($this ->filterForm) {
            $this->filterForm->render();
        }
    }
}



