<?php
/**
 * UTF8
 * stanislav.elbl, 20.08.2015
 */

namespace Ron\RonPortal;

require_once('ron/controls.php');

use Ron\Controls\ComboBox;
use Ron\Controls\ComboBoxes;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\StringUtils;


class ComboBoxObdobi extends ComboBox
{
    const SETTINGS_MAXIMALNI_POCET_OBDOBI = 'maximalnipocetobdobi';

    var $pocatekObdobi;
    var $konecObdobi;
    var $popisObdobi;
    var $uzavreno;


    function __construct($varname = 'obdobi')
    {
        parent::__construct($varname);
    }


    function nastavSession()
    {
        RonPortalSession::setObdobi($this->pocatekObdobi, $this->konecObdobi);
    }


    function nastavSessionVariantni()
    {
        RonPortalSession::setVariantniObdobi($this ->pocatekObdobi, $this->konecObdobi);
    }


    /**
     * @param Database $database
     * @param string $sql
     * @param bool   $emptyRow
     * @param int    $selectedValue
     * @return bool
     * @throws /Exception
     */
    function loadSQLObdobi($database, $sql, $emptyRow, $selectedValue)
    {
        $this->data = [];

        $this->selectedOption = null;

        if ($emptyRow) {
            $this->data[] = new Option('', '');
        }

        $cursor = $database ->getSqlResult($sql, null, 'Chyba pri nacitani dat v ComboBoxObdobi::loadSQLObdobi!');

        $first = true;
        $pocet = 0;
        $jeVybrane = false;

        // prida jednotlive radky
        while ( ($row = $cursor ->fetch())  &&
                (RonPortalSettings::getIntValue(self::SETTINGS_MAXIMALNI_POCET_OBDOBI) <= 0  ||  $pocet < RonPortalSettings::getIntValue(self::SETTINGS_MAXIMALNI_POCET_OBDOBI)) ) {
            $pocatek = $database->stringToTimestamp($row[1]);
            $konec = $database->stringToTimestamp($row[2]);

            $jeAktualni = (time() >= $pocatek) && (time() < DateUtils::incDay($konec, 1));

            $nastavit = false;
            if ($selectedValue == $pocatek) {
                $nastavit = true;
                $jeVybrane = true;
            } else if ($first || ($jeAktualni && !$jeVybrane)) {
                $nastavit = true;
            }

            $this->data[] = $option = new Option( DateUtils::dateToUrlString($pocatek), $row[0]);

            if ($nastavit) {
                $this->pocatekObdobi = $pocatek;
                $this->konecObdobi = $konec;
                $this->popisObdobi = $row[0];
                $this->uzavreno = $row[3];

                $this->selectedOption = $option;
            }

            $first = false;

            if ($pocatek <= time()) { // do maximalniho poctu zapocita jen predchozi obdobi
                $pocet++;
            }
        }
        $cursor ->close();

        if ($first) {
            // V databazi neni zadne obdobi - to je mozne pri pouziti pristupaku bez dalsich aplikaci
            // ! ! ! Co kdyz je v databazi napr. nejake starsi testovaci obdobi jine aplikace ? ? ?
            //       Pak v pristupaku neni mozne vybrat aktualni obdobi
            $maxpocet = RonPortalSettings::getIntValue(self::SETTINGS_MAXIMALNI_POCET_OBDOBI);
            if ($maxpocet < 1) $maxpocet = 5;

            $date = DateUtils::firstDayOfMonth(time());
            for ($i = 0; $i < $maxpocet; $i++) {
                $popis = date('n/Y', $date);
                $pocatek = $date;

                $this->data[] = $option = new Option($pocatek, $popis);

                if ($first || ($selectedValue == $pocatek)) {
                    //$selecteddbrow = array($popis, $pocatek, $konec, $uzavreno);
                    $this->pocatekObdobi = $pocatek;
                    $this->konecObdobi = DateUtils::lastDayOfMonth($date);
                    $this->popisObdobi = $popis;
                    $this->uzavreno = 0;

                    $this ->selectedOption = $option;
                }

                $first = false;

                $date = DateUtils::firstDayOfMonth(DateUtils::incDay($date, -10)); // prejde na predchozi mesic
            }
        }

        return true;
    }


    /**
     * Nacte seznam obdobi
     * @param Database $database
     * @param string $appId
     * @param bool $variantni
     * @param bool $pouzeNeuzavrene
     * @param bool $emptyRow
     * @param int  $selectedValue
     * @throws \Exception
     */
    function loadObdobi($database, $appId, $variantni, $pouzeNeuzavrene, $emptyRow, $selectedValue)
    {
        $params = $database ->createParameters();
        if ($variantni) {
            $sql =
                'SELECT PopisObdobi, PocatekObdobi, KonecObdobi FROM VariantniObdobi' .
                ' ORDER BY PocatekObdobi DESC';
        } else {
            $sql = 'SELECT DISTINCT Popis, PocatekObdobi, KonecObdobi, Uzavreno FROM Obdobi ';

            if ($appId) {
                $sql .= ' WHERE Aplikace = ' . $params->addParameter($appId);
            } else {
                // vybere obdobi ze vsech aplikaci (krome skladu)
                // pozor - muze zpusobit, ze se obdobi opakuje!
                $sql .= " WHERE (Aplikace = 'ADS'  OR  Aplikace = 'APS'  OR  Aplikace = 'JIDELNA'  OR  Aplikace = 'MZDY') ";
            }

            if ($pouzeNeuzavrene) $sql = $sql . ' AND (Uzavreno = 0  OR  Uzavreno IS NULL)';

            $sql = $sql . ' ORDER BY PocatekObdobi DESC';
        }

        $this->loadSQLObdobi($database, new SqlStatement($sql, $params), $emptyRow, $selectedValue);
    }
}


/**
 * Class ComboBoxZamestnanec
 * @package Ron\RonPortal
 */
class ComboBoxZamestnanec extends ComboBox
{
    // udaje aktualne zvolene osoby - nastavi se pri nacitani dat
    var $idPP;
    var $rc;
    var $osobniCislo;
    var $datumNastupu;
    var $datumUkonceni;
    var $jmenoPrijmeni;
    var $idFotografie;

    /** @var bool       Priznak, jestli nactena data obsahuji vlastni PP uzivatele */
    var $obsahujeVlastniPP;
    /** @var string     Id vlastniho PP (pokud jej obsahuje) */
    var $idVlastnihoPP;

    //var $f_DalsiPole; // hodnota uzivatelskeho pole

    var $filtrTypRozdeleni;
    var $filtrRozdeleni;
    var $filtrVcetnePodrizenych;

    var $filtrRypRozdeleni2;
    var $filtrRozdeleni2;
    var $filtrVcetnePodrizenych2;

    var $vcetneExternich = false;

    var $filtr; // obecny sql filtr


    function __construct($varName = 'osoba')
    {
        parent::__construct($varName);

        $this ->noHtmlConvert = true; // htmlTextConvert popisu zajistuje primo tato trida
    }


    function nastavFiltrRozdeleni($typ, $kod, $vcetnePodrizenych)
    {
        $this->filtrTypRozdeleni = $typ;
        $this->filtrRozdeleni = (isset($kod) &&  $kod !== '') ? $kod : null;
        $this->filtrVcetnePodrizenych = $vcetnePodrizenych;
    }


    function nastavFiltrRozdeleni2($typ, $kod, $vcetnePodrizenych)
    {
        $this->filtrRypRozdeleni2 = $typ;
        $this->filtrRozdeleni2 = (isset($kod) &&  $kod !== '') ? $kod : null;;
        $this->filtrVcetnePodrizenych2 = $vcetnePodrizenych;
    }



    /**
     * @param Database $database
     * @param array $row
     * @param bool  $rozlisitPP     Urcuje, jestli se maji rozlisit jednotlive PP
     * @param bool  $nastavit       Pokud je true, tak se udaje osoby nastavi do vlastnosti idPP, RC atd.
     * @return void                 Prida zaznam pro osobu ze zadaneho DB radku
     */
    protected function pridejOsobu($database, $row, $rozlisitPP, $nastavit)
    {
        $kod = $row[7]; // id pracovniho pomeru

        // jmeno osoby a pokud se ma rozlisovat PP tak i Os.c. a nastup
        $jmeno = $row[1] . ' ' . $row[2]; // prijmeni a jmeno
        if ($row[3]) $jmeno = $jmeno . ', ' . $row[3]; // prida titul

        $popis = $jmeno;
        if ($rozlisitPP) {
            $popis = $popis . ' - ' .
                \Ron\ronText('Osobni_cislo') . ' ' . $row[5] . ', ' .
                \Ron\ronText('Datum_nastupu') . ' ' . DBUtils::getFormatedValue($database, 'date', $row[6]);
        }

        $popis = StringUtils::htmlTextConvert($popis);

        // pripadne jeste uzivatelsky vybrane pole - nepouzivat htmlTextConvert, muze obsahovat HTML znacky
        $pomval = 0;
        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1)) {
            $popis = $row[9] . ' ' . $popis;
            $pomval = 1;
        }
        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2)) {
            $popis .= ' ' . $row[9 + $pomval];
        }

        $this->data[] = $option = new Option($kod, $popis);
        $option ->params = [
            'data-jmeno' => $jmeno,
            'data-osobni-cislo' => $row[5],
            'data-datum-nastupu' => DBUtils::getFormatedValue($database, 'date', $row[6])
        ];

        if (!$this ->obsahujeVlastniPP  &&  $row[0] == User::getRC()) {
            $this ->obsahujeVlastniPP = true;
            $this ->idVlastnihoPP = $kod;
            $option ->cssClass = 'vlastni-pp';
        }

        if ($nastavit) {
            $this->selectedOption = $option;

            $this->idPP = $row[7];

            $this->rc = $row[0];
            $this->idFotografie = $row[4];
            $this->osobniCislo = $row[5];
            $this->datumNastupu = $database ->stringToTimestamp($row[6]);
            $this->datumUkonceni = $row[8] ? $database ->stringToTimestamp($row[8]) : null;

            $this->jmenoPrijmeni = $popis;

            //$this -> f_DalsiPole = ...;
        }
    }


    /**
     *  Nastavi v session promenne osoby, ktera je vybrana
     */
    function nastavOsobuDoSession()
    {
        RonPortalSession::setVybranaOsoba($this->idPP, $this->rc, $this->osobniCislo, $this->datumNastupu);
    }


    /**
     * Vraci hodnotu, ktera se ma pouzit pro zobrazeni
     * @param Database $database
     * @param string|int $defaultValue
     * @return string|int
     */
    function getPostValue($database, $defaultValue)
    {
        if (isset($_POST[$this->varName])) return $_POST[$this->varName];

        if (isset($_GET[$this->varName])) return $_GET[$this->varName];

        return $defaultValue;
    }


    /**
     * Pomocna funkce - vraci true, pokud je aktualni zaznam ($row) vybrany.
     * @param array  $row
     * @param string $selectedValue
     * @return bool
     */
    protected function compareSelectedValue($row, $selectedValue)
    {
        return $selectedValue == $row[7]; // zaznam ma stejne IdPP jako vybrana osoba
    }


    /**
     * Nacte seznam osob ze zadaneho dotazu
     * dale nacte promenne pro vybrany zaznam - tj. zaznam z $_POST nebo $selectedvalue (prednost ma hodnota v $_POST)
     * @param Database $database
     * @param string   $sqlText
     * @param PravaNaOsoby $prava
     * @param bool     $emptyRow
     * @param int      $selectedValue
     * @param int      $datumOd
     * @param int      $datumDo
     * @return bool
     * @throws DBException
     */
    function loadSQLPP($database, $sqlText, $prava, $emptyRow, $selectedValue, $datumOd, $datumDo)
    {
        MessageManager::addDebugLog('ComboBoxZamestnanec: ' . $sqlText);
        $this->data = [];

        $selectedValue = $this->getPostValue($database, $selectedValue);

        if ($prava) if (!Prava::jsouNejakePrava($prava)) return false; // uzivatel nema zadna prava

        if ($emptyRow) {
            // v seznamu ma byt i prazdny zaznam
            $emptyOption = new Option('', $emptyRow === true ? '' : $emptyRow);
            $this ->data[] = $emptyOption;

            if ($selectedValue === '') {
                $this ->selectedOption = $emptyOption;
            }

            $prvniOsoba = false;
        } else {
            $prvniOsoba = true;
        }

        $cursor = $database ->getSqlResult($sqlText, null, 'Spatny SQL vyraz pro vyber osob!');

        $rozlisovat = false;
        $jeOsobaVybrana = $emptyRow && $this ->selectedOption;
        // priznak, jestli uz je vybrana osoba (nestaci kontrolovat, jestli uz jsou nastavene promenne,
        //  protoze ty jsou nastavene hned pri prvni osobe, ale prednost ma "muj" vlastni zaznam)

        $row = $cursor ->fetch();

        while ($row) {
            $newRow = $cursor ->fetch(); // nacte i dalsi radek - aby mohl zjistit, jestli ma rozlisovat PP

            $zobrazitOsobu = true;
            if ($prava) {
                if (!Prava::lzeZobrazitPP($prava, $row[7], $datumOd, $datumDo)) $zobrazitOsobu = false;
            }

            if ($zobrazitOsobu) {
                if ($newRow  &&  $row[0] === $newRow[0]) {
                    // pokud je nasledujici osoba stejna, musim rozlisovat PP
                    $rozlisovat = true;
                }

                $nastavit = false;

                if ($prvniOsoba || $this ->compareSelectedValue($row, $selectedValue) || (!$jeOsobaVybrana && $row[0] == User::getRC())) {
                    $nastavit = true;
                    $prvniOsoba = false;

                    if ($this ->compareSelectedValue($row, $selectedValue) || $row[0] == User::getRC()) {
                        $jeOsobaVybrana = true;
                    }
                }

                $this->pridejOsobu($database, $row, $rozlisovat, $nastavit);
            }

            if ($newRow) if ($row[0] !== $newRow[0]) $rozlisovat = false; // pokud je nasledujici osoba jina, nemusim u ni rozlisovat PP

            $row = $newRow;
        }

        $cursor ->close();

        return true;
    }


    /**
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int $typRozdeleni
     * @param string $rozdeleni
     * @param int $vcetnePodrizenych    0=jen zadane, 1=vcetne podrizenych, 2=jen podrizene, 3=primo podrizene
     * @return string
     */
    function vratFiltrRozdeleni($database, $datumOd, $datumDo, $typRozdeleni, $rozdeleni, $vcetnePodrizenych)
    {
        if (!$typRozdeleni) return '';
        if (!isset($rozdeleni)) return '';

        if (!$datumOd) $datumOd = time();
        if (!$datumDo) $datumDo = $datumOd;

        if ($vcetnePodrizenych == 1  ||  $vcetnePodrizenych == 2) { // vcetne podrizenych, jen podrizene
            return
                'EXISTS (' .
                '  SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
                '  WHERE HierarchieRozdeleni.TypNadrazene = ' . (int)$typRozdeleni .
                "    AND HierarchieRozdeleni.KodNadrazene = '" . $database ->addSlashes($rozdeleni) . "'" .

                '    AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.typPodrizene' .
                '    AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.kodPodrizene' .

                '    AND RozdeleniOsob.RC = PracovniPomer.RC ' .
                '    AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                '    AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $database ->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $database ->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                (($vcetnePodrizenych == 2) ? 'AND HierarchieRozdeleni.KodNadrazene <> HierarchieRozdeleni.KodPodrizene' : '') .
                ')';
        } else if ($vcetnePodrizenych == 3) { // jen primo podrizene
            return
                'EXISTS (' .
                '  SELECT * FROM RozdeleniOsob, Rozdeleni ' .
                '  WHERE Rozdeleni.KodTypuRozdeleni = ' . (int)$typRozdeleni .
                "    AND Rozdeleni.Rodic = '" . $database ->addSlashes($rozdeleni) . "'" .
                '    AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                '    AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod' .
                '    AND RozdeleniOsob.RC = PracovniPomer.RC ' .
                '    AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                '    AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $database ->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $database ->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ')';
        } else {
            return
                'EXISTS (' .
                '  SELECT * FROM RozdeleniOsob ' .
                '  WHERE RozdeleniOsob.KodTypuRozdeleni = ' . (int)$typRozdeleni .
                "    AND RozdeleniOsob.Rozdeleni = '" . $database ->addSlashes($rozdeleni) . "'" .
                '    AND RozdeleniOsob.RC = PracovniPomer.RC ' .
                '    AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                '    AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $database ->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $database ->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ')';
        }
    }


    /**
     * @param Database $database
     * @param int  $datumOd
     * @param int  $datumDo
     * @return string
     */
    private function getBasicSQL($database, $datumOd, $datumDo) {
        // POZOR! V pripade zmeny poradi muze byt potreba upravit ORDER BY!
        $sqltext =
            'SELECT Osoba.RC, Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul, Osoba.IdFotografie, ' . // 0 - 4
            ' PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, PracovniPomer.ID, ' . // 5 - 7
            ' PracovniPomer.DatumUkonceni '; // 8

        // prida uzivatelske pole z nastaveni intrawebu
        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1)) $sqltext .= ', ' . RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1); // (9)
        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2)) $sqltext .= ', ' . RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2); // (9 nebo 10)

        $sqltext .=
            "\nFROM Osoba, PracovniPomer\n" .
            ' WHERE Osoba.RC = PracovniPomer.RC ';

        if (!$this->vcetneExternich)
            $sqltext .= ' AND (PracovniPomer.Zamestnanec = 1  OR  PracovniPomer.Zamestnanec = 2)';

        $sqltext = $sqltext .
            ' AND PracovniPomer.DatumNastupu <= ' . $database ->dateToString($datumDo) .
            ' AND (PracovniPomer.DatumUkonceni >= ' . $database ->dateToString($datumOd) . ' OR PracovniPomer.DatumUkonceni IS NULL)';

        $filtr = $this->vratFiltrRozdeleni($database, $datumOd, $datumDo,
            (int)$this->filtrTypRozdeleni, $this->filtrRozdeleni, $this->filtrVcetnePodrizenych);

        $filtr2 = $this->vratFiltrRozdeleni($database, $datumOd, $datumDo,
            (int)$this->filtrRypRozdeleni2, $this->filtrRozdeleni2, $this->filtrVcetnePodrizenych2);

        if ($filtr && $filtr2) $filtr = '(' . $filtr . ")\nAND  (" . $filtr2 . ')';
        else if ($filtr2) $filtr = $filtr2;

        if ($filtr) $sqltext .= "\nAND (" . $filtr . ')';

        if ($this->filtr) $sqltext .= "\nAND (" . $this->filtr . ')';

        return $sqltext;
    }


    /**
     * @param Database $database
     * @param int  $datumOd
     * @param int  $datumDo
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @return string
     */
    function getSQLPodlePrav($database, $datumOd, $datumDo, $prava)
    {
        // filtr podle zadanych datumu nebo podle zacatku a konce obdobi
        if (!$datumDo) $datumDo = RonPortalSession::konecObdobi();
        if (!$datumOd) $datumOd = RonPortalSession::obdobi();

        //16.10.2015 RON-PR datum nemusi byt nastavene v session, pouzije se aktualni datum
        if (!$datumDo) $datumDo = DateUtils::lastDayOfMonth(time());
        if (!$datumOd) $datumOd = DateUtils::firstDayOfMonth(time());

        $sqltext = $this ->getBasicSQL($database, $datumOd, $datumDo);

        $vynechatVcetneSebe = true;
        $filtrPrava = Prava::sqlFiltrPodlePrav($prava, 'PracovniPomer', $datumOd, $datumDo, $vynechatVcetneSebe); // vynechat "vcetne sebe"

        if ($filtrPrava) {
            $sqltext = $sqltext . ' AND (' . $filtrPrava . ')';
        }

        $jeUnion = false;
        if ($vynechatVcetneSebe  && Prava::pravaVcetneSebe($prava)) {
            $sqltext .=
                "\nUNION\n" .
                $this ->getBasicSQL($database, $datumOd, $datumDo) .
                " AND PracovniPomer.RC = '" . $database ->addSlashes(User::getRC()) . "'\n";

            $jeUnion = true;
        }

        if ($jeUnion  &&  $database ->getDbType() == 'interbase') {
            $sqltext .=
                '  ORDER BY 2, 3, 6, 8'; // na
        } else {
            $sqltext .=
                '  ORDER BY Osoba.Prijmeni, Osoba.Jmeno, PracovniPomer.OsobniCislo, PracovniPomer.Id';
        }

        MessageManager::addDebugLog("ComboBoxZamestnanec: $sqltext");
        return $sqltext;
    }


    /**
     * Nacte seznam pracovnich pomeru platnych v zadanem obdobi (kdyz neni obdobi zadano, pouzije aktualni zvolene obdobi)
     * Pokud jsou zadana prava, tak nacita jen osoby, na ktere ma uzivatel prava
     * @param Database $database
     * @param int    $datumOd
     * @param int    $datumDo
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @param string $emptyRow
     * @param mixed  $selectedValue
     * @param bool   $vcetneExternich
     * @return bool
     */
    function loadDataPP($database, $datumOd, $datumDo, $prava, $emptyRow, $selectedValue, $vcetneExternich = false)
    {
        $this->vcetneExternich = $vcetneExternich;
        $sqltext = $this->getSQLPodlePrav($database, $datumOd, $datumDo, $prava);
        return $this->loadSQLPP($database, $sqltext, $prava, $emptyRow, $selectedValue, $datumOd, $datumDo);
    }


    function loadDataPredavaciProces($database, $kodProcesu, $typRozdeleniProces, $emptyRow, $selectedValue)
    {
        require_once('adsapp/predavaciproces.php');
        $sqltext = PredavaciProcesyUtils::sqlVyberPPProPredavaciProces($database, $kodProcesu, $typRozdeleniProces);
        $this->loadSQLPP($database, $sqltext, null, $emptyRow, $selectedValue, 0, 0); // $prava = null
    }
}





/**
 * Class ComboBoxOsoba
 * Vyber osoby - jen RC, ne pracovni pomer.
 * Do HTML z bezpecnostnich duvodu nedava RC, ale prirazene (nektere) idPP
 * @package Ron\RonPortal
 */
class ComboBoxOsoba extends ComboBoxZamestnanec
{
    private $seznamOsob = [];

    function __construct($varName = 'osoba')
    {
        parent::__construct($varName);
    }


    /**
     * Prida zaznam pro osobu ze zadaneho DB radku
     * @param Database $database
     * @param array $row
     * @param bool  $rozlisitPP     Urcuje, jestli se maji rozlisit jednotlive PP
     * @param bool  $nastavit       Pokud je true, tak se udaje osoby nastavi do vlastnosti f_...
     */
    protected function pridejOsobu($database, $row, $rozlisitPP, $nastavit)
    {
        if ($this ->seznamOsob[$row[0]]) return; // osoba s RC=$row[0] uz byla pridana (jiny PP) => nedela nic

        $this ->seznamOsob[$row[0]] = 1;

        parent::pridejOsobu($database, $row, false, $nastavit); // nerozlisovat PP
    }


    /**
     * Vraci hodnotu, ktera se ma pouzit pro zobrazeni
     * @param Database $database
     * @param string|int $defaultValue
     * @return string|int
     */
    function getPostValue($database, $defaultValue)
    {
        if (isset($_POST[$this->varName])) {
            $idPP = $_POST[$this->varName];
        } else if (isset($_GET[$this->varName])) {
            $idPP = $_GET[$this->varName];
        } else {
            return $defaultValue;
        }

        if ($idPP === '') return '';

        // Combobox do HTML generuje IdPP, ale je potreba RC => najdu RC pro zadane IdPP
        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult('SELECT RC FROM PracovniPomer WHERE Id = ' . $params ->addParameter((int)$idPP), $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) return null;

        return $row[0];
    }


    /**
     * Pomocna funkce - vraci true, pokud je aktualni zaznam ($row) vybrany.
     * @param array  $row
     * @param string $selectedValue
     * @return bool
     */
    protected function compareSelectedValue($row, $selectedValue)
    {
        return $selectedValue == $row[0]; // zaznam ma stejne rc jako vybrana osoba
    }


    /**
     * Nacte seznam osob ze zadaneho dotazu
     * dale nacte promenne pro vybrany zaznam - tj. zaznam z $_POST nebo $selectedvalue (prednost ma hodnota v $_POST)
     * @param Database $database
     * @param string   $sqlText
     * @param PravaNaOsoby $prava
     * @param bool     $emptyRow
     * @param int      $selectedValue
     * @param int      $datumOd
     * @param int      $datumDo
     * @return bool
     * @throws DBException
     */
    function loadSQLPP($database, $sqlText, $prava, $emptyRow, $selectedValue, $datumOd, $datumDo)
    {
        $this ->seznamOsob = [];
        return parent::loadSQLPP($database, $sqlText, $prava, $emptyRow, $selectedValue, $datumOd, $datumDo);
    }

}


class ComboBoxesRozdeleni extends ComboBoxes
{
    /** @var Database */
    protected $database;

    /** @var string */
    public $name;
    /** @var string */
    public $id;

    /** @var TypRozdeleni */
    public $typRozdeleni;


    /**
     * ComboBoxesRozdeleni constructor.
     * @param Database $database
     * @param string $name
     * @param string $id
     */
    public function __construct(Database $database, $name, $id)
    {
        $this->database = $database;

        $isError = MessageManager::getInstance() ->isError($name);

        $combo1 = new ComboBox($name);
        if ($id) {
            $combo1 ->htmlParams['id'] = "{$id}_popis";
        }
        $combo1 ->htmlParams['data-ron-select2'] = $name . '_kod';
        //$combo1 ->htmlParams = ($id ? "id='{$id}_popis' " : '') . "data-ron-select2='{$name}_kod'";
        $combo1 ->className .= ' form-input ronselect' . ($isError ? ' error' : '');


        $combo2 = new ComboBox($name.'_kod');
        //$combo2 ->htmlParams = ($id ? "id='{$id}_kod' " : '') . "data-ron-select2='$name'";
        if ($id) {
            $combo2 ->htmlParams['id'] = "{$id}_kod";
        }
        $combo2 ->htmlParams['data-ron-select2'] = $name;
        $combo2 ->htmlParams['data-drop-down-align'] = 'right';
        $combo2 ->className .= ' form-input ronselect' . ($isError ? ' error' : '');

        parent::__construct($combo1, $combo2);
    }


    private function loadData($sql, $params, $aktualniHodnota, $povinne)
    {
        $this ->combo1 ->alwaysShowSelected = isset($aktualniHodnota) && $aktualniHodnota != '';
        //$this ->combo2 ->cssClassIndex = 2; -
        $this ->combo1 ->htmlParams['data-columns'] = 'text;value';
        $this ->combo1 ->attrIndexes = ['title' => 1, 'class' => 2];
        $this ->combo1 ->emptyRow = (isset($aktualniHodnota) && $aktualniHodnota != ''  &&  $povinne) ? null : ' ';
        $this ->combo1 ->loadData($this ->database, $sql.' ORDER BY Popis', $aktualniHodnota, $params);

        $this ->combo2 ->alwaysShowSelected = isset($aktualniHodnota)  &&  $aktualniHodnota != '';
        $this ->combo2 ->cssClassIndex = 2;
        $this ->combo2 ->captionIndex = 0;
        $this ->combo2 ->titleIndex = 1;
        $this ->combo2 ->htmlParams['data-columns'] = 'value;popis';
        $this ->combo2 ->attrIndexes = ['data-popis' => 1];
        $this ->combo2 ->emptyRow = (isset($aktualniHodnota) && $aktualniHodnota != ''  &&  $povinne) ? null : ' ';
        $this ->combo2 ->loadData($this ->database, $sql.' ORDER BY Kod', $aktualniHodnota, $params);
    }


    /**
     * @param PracovniPomer $pp
     * @param int    $typRozdeleni
     * @param string $aktualniHodnota
     * @param bool   $povinne
     */
    function loadVlastniRozdeleni($pp, $typRozdeleni, $aktualniHodnota, $povinne)
    {
        $params = $this ->database ->createParameters();

        $sql =
            'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass ' .
            '  FROM RozdeleniOsob, Rozdeleni'.
            ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
            '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addParameter($this ->database ->timestampToParam(time())) . ' OR Rozdeleni.DatumZruseni IS NULL)'.
            '   AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
            '   AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod ' .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($this ->database ->timestampToParam(time())) . ' OR RozdeleniOsob.PlatnostDo IS NULL)'.
            '   AND RozdeleniOsob.RC = ' . $params->addParameter($pp ? $pp->rc : '') .
            '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($pp ? $pp->osobniCislo : '') .
            '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pp ? $pp->datumNastupu : 0)) .
            // ... OR ( Rozdeleni.Kod = '".db_addslashes($hodnota)."'" ... - nepouzivat - je moc pomale !
            "   AND (cssClass IS NULL  OR  cssClass <> 'hidden')";

        $this ->loadData($sql, $params, $aktualniHodnota, $povinne);
    }


    function loadVsechnaRozdeleni($typRozdeleni, $aktualniHodnota, $povinne)
    {
        $params = $this ->database ->createParameters();

        $sql =
            'SELECT Kod, Popis, cssClass FROM Rozdeleni' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
            '   AND ( (' .
            ($aktualniHodnota != '' ? 'Rozdeleni.Kod = ' . $params->addParameter($aktualniHodnota)  : '0 = 1')   // zvolenou hodnotu zobrazi vzdy - pozor pri volbe "pouze vlastni" je to moc pomale
            . ')' .
            ' OR (' .
            '   (DatumZruseni >= ' . $params ->addParameter($this ->database ->timestampToParam(time())) . ' OR DatumZruseni IS NULL)' .
            ' )' .
            ')' .
            " AND (cssClass IS NULL  OR  cssClass <> 'hidden') ";

        $this ->loadData($sql, $params, $aktualniHodnota, $povinne);
    }


    function loadRozdeleni($pouzeVlastni, $pp, $typRozdeleni, $aktualniHodnota, $povinne)
    {
        require_once('adsapp/rozdeleni.php');
        $this ->typRozdeleni = TypRozdeleni::getTypRozdeleni($this ->database, $typRozdeleni);

        if ($pouzeVlastni) {
            $this ->loadVlastniRozdeleni($pp, $typRozdeleni, $aktualniHodnota, $povinne);
        } else {
            $this ->loadVsechnaRozdeleni($typRozdeleni, $aktualniHodnota, $povinne);
        }
    }

}
