<?php

/**
 * UTF8
 */

namespace Ron\RonPortal;

use Ron\DBRecord;
use Ron\RonPortal\XML\RonPortalXMLForm;
use Ron\RonPortal\XML\XMLForm;

require_once('ron/dbrecord.php');
require_once('adsapp/ronportallog.php');


class RonPortalDBRecord extends DBRecord
{
    var $logDescription;
    var $logText;

    /** @var string */
    public $beforePostEvent;
    /** @var string */
    public $afterPostEvent;
    /** @var array */
    public $eventsParameters = null;

    /**
     * @param \Ron\DB\Database $database
     * @param string $tableName
     * @param string|object $logDescription
     */
    function __construct($database, $tableName, $logDescription = 'Intraweb')
    {
        parent::__construct($database, $tableName);
        $this->setLogDescription($logDescription);
    }


    /**
     * @param string $beforePostEvent
     * @param string $afterPostEvent
     * @param mixed[] $eventParameters
     * @return RonPortalDBRecord
     */
    function setXmlEvents($beforePostEvent, $afterPostEvent, $eventParameters = null)
    {
        $this ->beforePostEvent = $beforePostEvent;
        $this ->afterPostEvent = $afterPostEvent;

        $this ->eventsParameters = $eventParameters;

        return $this;
    }

    /**
     * @param string|object $description
     * @return RonPortalDBRecord
     */
    function setLogDescription($description)
    {
        if (is_object($description)) {
            $this ->logDescription = substr(strrchr(get_class($description), "\\"), 1);
        } else {
            $this ->logDescription = $description;
        }

        return $this;
    }


    /**
     * @param string   $eventType
     * @param string   $logText
     */
    function log($eventType, $logText)
    {
        if ($this->logText) $logText = $this->logText . "\n" . $logText;
        Log::addRecordToDBLog($this ->database, $this->tableName, $eventType, $this->logDescription, $logText);
    }


    /**
     * @param string    $eventName
     * @param string    $context
     * @param array     $variables  Asociativni pole ve formatu ['nazevpromenne' => hodnota]. Automaticky se pridava promenna record
     * @param bool      $exceptionOnError  Pokud je true, tak v pripade chyby ve formulari vyhodi vyjimku. Jinak jen vraci false.
     * @return bool
     */
    function executeXmlEvents($eventName, $context = '', $variables = null, $exceptionOnError = true)
    {
        require_once('xml/xmlformportal.php');

        if (!$variables) $variables = [];
        $variables['record'] = $this;

        return RonPortalXMLForm::xmlEvents($this ->database, $eventName, $variables, $context, false, $exceptionOnError);

        //return xmlEvents($this ->database, $eventName, $variables, $context);
    }


    function updateRecord($filtr = '', $filtrParams = null)
    {
        $changed = $this ->isChanged();
        if ($changed  &&  $this->beforePostEvent) {
            $this->executeXmlEvents($this->beforePostEvent, 'update', $this ->eventsParameters);
        }

        $result = parent::updateRecord($filtr, $filtrParams);

        if ($changed  &&  $this->afterPostEvent) {
            $this->executeXmlEvents($this->afterPostEvent, 'update', $this ->eventsParameters);
        }

        return $result;
    }


    /**
     * @return void
     * @throws \Ron\DB\DBException
     */
    function insertRecord()
    {
        if ($this ->beforePostEvent) {
            $this ->executeXmlEvents($this ->beforePostEvent, 'insert', $this ->eventsParameters);
        }

        parent::insertRecord();

        if ($this->afterPostEvent) {
            $this->executeXmlEvents($this->afterPostEvent, 'insert', $this ->eventsParameters);
        }

        return true; // RETURN je deprecated! Nepouzivat navratovou hodnotu!
    }


    function doDelete($filtr, $filtrParams = null)
    {
        if ($this ->beforePostEvent) {
            $this ->executeXmlEvents($this ->beforePostEvent, 'delete', $this ->eventsParameters);
        }

        parent::doDelete($filtr, $filtrParams);

        if ($this->afterPostEvent) {
            $this->executeXmlEvents($this->afterPostEvent, 'delete', $this ->eventsParameters);
        }
    }


}
