<?php
/**
 * UTF8
 * stanislav.elbl, 31.07.2015
 */

namespace Ron\RonPortal;

require_once('ron/form.php');
require_once('adsapp/ronportaldbrecord.php');

use Ron\DB\Database;
use Ron\DBRecordField;
use Ron\FormField;
use Ron\InputForm;
use Ron\MessageManager;
use Ron\StringUtils;


/**
 * Pole pro vlozeni RC aktualniho uzivatele
 * Class UserRCField
 * @package Ron\RonPortal
 */
class UserRCField extends FormField
{
    function __construct($fieldname)
    {
        parent::__construct();

        $this->fieldType = "userRC";
        $this->fieldName = $fieldname;  // jmeno pole v databazi
        $this->sqlFieldName = $fieldname;
        $this->maxLength = 15;
    }


    function createDBRecordField($database, $tableName, $insert)
    {
        $field = new DBRecordField($this->sqlFieldName, '', !$this->editable, $this->maxLength);
        $field->setValue(User::getRC());

        return $field;
    }

}


/**
 * Pole pro vlozeni Loginu aktualniho uzivatele
 * Class UserLoginField
 * @package Ron\RonPortal
 */
class UserLoginField extends FormField
{
    function __construct($fieldname, $edit)
    {
        parent::__construct();

        $this->fieldType = "userLogin";
        $this->fieldName = $fieldname;  // jmeno pole v databazi
        $this->sqlFieldName = $fieldname;
        $this->maxLength = 25;
        $this->editable = $edit;
    }

    function createDBRecordField($database, $tableName, $insert)
    {
        $field = new DBRecordField($this->sqlFieldName, '', !$this->editable, $this->maxLength);
        $field->setValue(User::getLoginName());

        return $field;
    }
}


/**
 * Pole pro vlozeni Typu rozdeleni {zvoleneho, nebo podle prav)
 * Class TypRozdeleniField
 * @package Ron\RonPortal
 */
class TypRozdeleniField extends FormField
{
    public $typRozdeleni;

    function __construct($fieldname, $typRozdeleni, $edit)
    {
        parent::__construct();

        $this->fieldType = "hidden";
        $this->fieldName = $fieldname;  // jmeno pole v databazi
        $this->sqlFieldName = $fieldname;
        $this->typRozdeleni = $typRozdeleni;
        $this->editable = $edit;
    }


    function createDBRecordField($database, $tableName, $insert)
    {
        $field = new DBRecordField($this->sqlFieldName, '', !$this->editable, $this->maxLength);
        $field->setValue($this->typRozdeleni);

        return $field;
    }

    /**
     * Nastavi typ rozdeleni, ktery je pouzivany v zadanem typu prava
     * @param string $typPrava
     */
    function setTypRozdeleniPodlePrav($typPrava) {
        $prava = Prava::vratPrava($typPrava);
        $this->typRozdeleni = $prava->typRozdeleni;
    }

}


class OsobaField extends FormField
{
    var $prava;
    var $datumPlatnostiPP;

    /** @var bool Priznak, jestli se ma zobrazit button na vyber vlastniho PP */
    var $zobrazitButtonVlastni = true;

    var $filtr;

    /**
     * OsobaField constructor.
     * @param string $caption nadpis
     * @param string $prava kod prava
     * @param int $datumPlatnostiPP datum ke kteremu se bere platnost prac. pomeru
     * @param bool $edit
     */
    function __construct($caption, $prava, $datumPlatnostiPP, $edit)
    {
        parent::__construct();

        require_once('adsapp/ronportalcontrols.php');

        $this->fieldType = 'osoba';
        $this->fieldName = 'osoba';
        $this->caption = $caption;
        $this->prava = $prava;
        $this->required = true;

        $this->datumPlatnostiPP = $datumPlatnostiPP;
        if (!$this->datumPlatnostiPP) $this->datumPlatnostiPP = time();

        $this->editable = $edit;
    }


    /**
     * @param \Ron\DB\Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws \Ron\DB\DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($canEdit) {
            echo '<div class="row no-margin-bottom no-flex-wrap">';

            if ( !isset($value) ) $value = RonPortalSession::vybranaOsoba();

            $combo = new ComboBoxZamestnanec($this->fieldName); // jmeno promenne
            $combo->filtr = $this->filtr;
            $combo->htmlParams = $this->otherHtmlParams;
            $combo->loadDataPP($database,
                $this->datumPlatnostiPP, $this->datumPlatnostiPP,
                Prava::vratPrava($this->prava), ' ', $value, false); // $emptyRow = '', $selectedValue, $vcetneExternich = false
            $combo->nastavOsobuDoSession();
            $combo->render();

            if ($combo ->obsahujeVlastniPP) {
                echo "<i class='inline-glyph glyph-button fa glyph-own-data comboBoxMove' data-value='{$combo ->idVlastnihoPP}' data-selector='#absence_osoba' data-trigger-change='1'></i>";
            }

            echo '</div>';
        } else {
            if ($value) {
                require_once('adsapp/ppomer.php');
                $pp = PracovniPomer::nactiPP($database, $value);
                echo '<input type="hidden" name="' . $this->fieldName . '" value="' . $value . '" />';
                echo StringUtils::htmlTextConvert($pp ->jmenoPrijmeni) . "\n";
//            } else if ($this ->required  &&  !$insert) {
//                // Puvodni kod, ale je nesmyslny, protoze zobrazuje osobu z session. Nepouzivat! Pouze pro kompatibilitu
//                echo '<div class="warning">POZOR! Chyba pri pouziti OsobaField - neni zadana hodnota (idpp)!</div>';
//                echo '<input type="hidden" name="' . $this->fieldName . '" value="' . RonPortalSession::vybranaOsoba() . '" />';
//                echo 'OSOBA ID ' . RonPortalSession::vybranaOsoba() . "\n";
            }
        }
    }
}


/**
 * Class RCOsobyField
 * Pole pro vyber osoby (jen RC, ne pracovni pomer)
 * @package Ron\RonPortal
 */
class RCOsobyField extends FormField
{
    var $prava;
    var $datumPlatnostiPP;
    var $vcetneExternich;

    var $filtr;

    /**
     * OsobaField constructor.
     * @param string $caption nadpis
     * @param string $prava kod prava
     * @param int $datumPlatnostiPP datum ke kteremu se bere platnost prac. pomeru
     * @param bool $edit
     */
    function __construct($caption, $prava, $datumPlatnostiPP, $edit)
    {
        parent::__construct();

        require_once('adsapp/ronportalcontrols.php');

        $this->fieldType = 'osoba';
        $this->fieldName = 'osoba';
        $this->caption = $caption;
        $this->prava = $prava;
        $this->required = true;

        $this->datumPlatnostiPP = $datumPlatnostiPP;
        if (!$this->datumPlatnostiPP) $this->datumPlatnostiPP = time();

        $this->editable = $edit;

        $this ->vcetneExternich = false;
    }


    /**
     * @param \Ron\DB\Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($canEdit) {
            $combo = new ComboBoxOsoba($this->fieldName); // jmeno promenne
            $combo->filtr = $this->filtr;
            $combo->className = 'form-input ' . MessageManager::getClass($this ->fieldName);
            $combo->htmlParams = $this->otherHtmlParams;
            $combo->loadDataPP($database,
                $this->datumPlatnostiPP, $this->datumPlatnostiPP,
                Prava::vratPrava($this->prava), ' ', $value, $this ->vcetneExternich); // $emptyRow = ' ', $selectedValue, $vcetneExternich
            $combo->render();
        } else {
            //echo '<input type="hidden" name="' . $this->fieldName . '" value="' . RonPortalSession::vybranaOsoba() . '" />';
            //echo RonPortalSession::osobaJmenoPrijmeni() . "\n";
        }
    }
}


class RonPortalForm extends InputForm
{
    /** @var string     Nazev zdroje pro DB LOG (implicitne nazev tridy) */
    var $dbLogSrcDescription;

    /** @var string     Nazev XML udalosti spustene pred zapisem zaznamu */
    var $xmlEventBeforePost;

    /** @var string     Nazev XML udalosti spustene po zapisu zaznamu */
    var $xmlEventAfterPost;

    /**
     * @param \Ron\DB\Database $database
     * @param string $formAction
     * @param bool $dialogMode
     */
    function __construct($database, $formAction = 'new', $dialogMode = false)
    {
        parent::__construct($database, $formAction, $dialogMode);

        //$this ->dbLogSrcDescription = get_class($this);
        $this ->dbLogSrcDescription = substr(strrchr(get_class($this), "\\"), 1);
    }


    /**
     * Nastavi nazvy XML udalosti - pridava sufixy _BEFORE_POST a AFTER_POST
     * @param $name
     */
    function setXmlEventsName($name)
    {
        $this ->xmlEventBeforePost = $name . '_BEFORE_POST';
        $this ->xmlEventAfterPost = $name . '_AFTER_POST';
    }


    function getHtmlFormParams()
    {
        return
            parent::getHtmlFormParams() .
            ($this ->xmlEventBeforePost
                ? " data-xml-beforepost='{$this ->xmlEventBeforePost}'"
                : '') .
            ($this ->xmlEventAfterPost
                ? " data-xml-afterpost='{$this ->xmlEventAfterPost}'"
                : '')
            ;
    }


    function createDBRecord()
    {
        $rec = new RonPortalDBRecord($this ->database, $this ->sqlTableName);

        if ($this->dbLogSrcDescription) {
            $rec ->logDescription = $this->dbLogSrcDescription;
        }

        return $rec;
    }


    function beforeInsertRecord($rec)
    {
        if ($rec instanceof RonPortalDBRecord  &&  $this ->xmlEventBeforePost) {
            $rec ->executeXmlEvents($this ->xmlEventBeforePost, 'insert');
        }

        return parent::beforeInsertRecord($rec);
    }

    function afterInsertRecord($rec)
    {
        if ($rec instanceof RonPortalDBRecord  &&  $this ->xmlEventAfterPost) {
            $rec ->executeXmlEvents($this ->xmlEventAfterPost, 'insert');
        }

        return parent::afterInsertRecord($rec);
    }

    function beforeUpdateRecord($rec)
    {
        if ($rec instanceof RonPortalDBRecord  &&  $this ->xmlEventBeforePost) {
            $rec ->executeXmlEvents($this ->xmlEventBeforePost, 'update');
        }

        return parent::beforeUpdateRecord($rec);
    }

    function afterUpdateRecord($rec)
    {
        if ($rec instanceof RonPortalDBRecord  &&  $this ->xmlEventAfterPost) {
            $rec ->executeXmlEvents($this ->xmlEventAfterPost, 'update');
        }

        return parent::afterUpdateRecord($rec);
    }


    /**
     * Nacte XML konfiguraci browseru se zadanym kodem
     * @param Database $database
     * @param string $code
     * @return string|null
     */
    public static function loadFormConfiguration($database, $code)
    {
        if (!$code) {
            return null;
        }

        $params = $database ->createParameters();
        $cursor = $database ->select(
            'SELECT ObsahXML FROM SablonaXMLFormulare WHERE Popis = ' . $params ->addParameter('config_' . $code),
            $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : null;
    }


    /**
     * Nacte konfiguraci browseru z XML sablony
     * @throws \Exception
     */
    function loadConfiguration()
    {
        if (!$this ->code) {
            return;
        }

        $xml = self::loadFormConfiguration($this ->database, $this ->code);
        if ($xml) {
            $this ->loadConfigurationXml($xml);
        }
    }


    function handleAction()
    {
        $this ->loadConfiguration();
        return parent::handleAction();
    }


}
