<?php
/**
 * UTF8
 * stanislav.elbl, 10.03.2016
 */

namespace Ron\RonPortal;

class RonPortalSession
{
    const SESSION_DOCHAZKA = 'skryt_dochazku';
    const SESSION_JIDELNA = 'skryt_jidelnu';
    const SESSION_PERSON = 'skryt_personalistiku';

    /** Cas (timestamp), ve kterem bylo zadano heslo pro pristup k chranenym sekcim */
    const SESSION_PASSWORD_PROTECTION_TIMESTAMP = 'pwd_protection_time';

    public static function getValue($name)
    {
        return $_SESSION[$name];
    }


    public static function setValue($name, $value)
    {
        $_SESSION[$name] = $value;
    }


    public static function initSessionTimestamp()
    {
        $_SESSION['sessionTimestamp'] = time();
    }

    /**
     * SessionTimestamp se nastavuje pri prihlaseni
     * @return int
     */
    public static function sessionTimestamp()
    {
        return $_SESSION['sessionTimestamp'];
    }

    /**
     * Vraci posledni zaregistrovany cas pristupu. Pokud je $setNewValue = true, tak zaroven nastavi aktualni cas.
     * @param bool $setNewValue
     * @return mixed
     */
    public static function lastSessionAccess($setNewValue = false)
    {
        $result = $_SESSION['lastSessionAccess'];
        if($setNewValue) $_SESSION['lastSessionAccess'] = time();

        return $result;
    }

    /**
     * Vraci true, pokud je session jeste platna
     * @param int $logoutTimer  Urcuje maximalni dobu platnosti session v sekundach
     * @return bool
     */
    public static function isValid($logoutTimer)
    {
        if (!$logoutTimer) return true;

        return (self::lastSessionAccess() + $logoutTimer > time());
    }


    /**
     * Vraci true, pokud je umoznen pristup k chranenym sekcim (tj. uzivatel zadal heslo)
     * Zaroven prodlouzi platnost zadaneho hesla.
     */
    public static function enablePasswordProtected()
    {
        if (!self::getValue(self::SESSION_PASSWORD_PROTECTION_TIMESTAMP)) {
            // heslo jeste nebylo zadano
            return false;
        }

        if (!RonPortalSettings::getIntValue('protected_timeout')) {
            // heslo je platne stale (neni nastavena platnost)
            return true;
        }

        if (time() < self::getValue(self::SESSION_PASSWORD_PROTECTION_TIMESTAMP) + 60 * RonPortalSettings::getIntValue('protected_timeout')) {
            // heslo uz bylo zadano a je jeste platne
            self::setValue(self::SESSION_PASSWORD_PROTECTION_TIMESTAMP, time()); // prodlouzi platnost hesla
            return true;
        }

        // heslo uz vyprselo
        return false;
    }


    // menu
    /**
     * Vraci nebo nastavi vypnute sekce menu (seznam kodu oddelenych strednikem)
     * @param string $newValue
     * @return mixed
     */
    public static function hiddenMenuSections($newValue = null)
    {
        $result = $_SESSION['hidden_menu_sections'];

        if (isset($newValue)) {
            $_SESSION['hidden_menu_sections'] = $newValue;
        }

        return $result;
    }

    public static function menuSectionIsVisible($section)
    {
        $hiddenSections = self::hiddenMenuSections();
        if (!$hiddenSections) return true;

        $sections = explode(';', $hiddenSections);
        return !in_array($section, $sections);
    }

    /**
     * Zapne nebo vypne (podle parametru $on) zadanou sekci menu. Vraci novy seznam vypnutych sekci
     * @param $section
     * @param $on
     * @return string
     */
    public static function switchMenuSection($section, $on)
    {
        $hiddenSections = self::hiddenMenuSections();

        $sections = explode(';', $hiddenSections);
        if ($on) {
            // zapne sekci = odstrani ji ze seznamu vypnutych
            $index = array_search($section, $sections);
            if ($index !== false) {
                unset($sections[$index]);
            }
        } else {
            // vypne sekci = prida ji do seznamu vypnutych
            if (!in_array($section, $sections)) {
                $sections[] = $section;
            }
        }

        $result = implode(';', $sections);
        self::hiddenMenuSections($result);
        return $result;
    }


    // filtry
    public static function vybranaOsoba()
    {
        return $_SESSION['vybranaosoba'];
    }

    public static function setVybranaOsoba($idPP, $rc, $osobniCislo, $datumNastupu)
    {
        $_SESSION['vybranaosoba'] = $idPP;
//        $_SESSION['osobajmenoprijmeni'] = $jmenoPrijmeni;
        $_SESSION['osobarc'] = $rc;
        $_SESSION['osobaosobnicislo'] = $osobniCislo;
        $_SESSION['osobanastup'] = $datumNastupu;
//        $_SESSION['osobaukonceni'] = $datumUkonceni;
    }

    /**
     * smaze ze session udaje zamestnance
     */
    public static function smazudajezamestnance()
    {
        unset($_SESSION['vybranaosoba']);
//        unset($_SESSION['osobajmenoprijmeni']);
        unset($_SESSION['osobarc']);
        unset($_SESSION['osobaosobnicislo']); // osobni cislo zobrazene osoby
        unset($_SESSION['osobanastup']);      // datum nastupu zobrazene osoby
    }


    /**
     * @return mixed
     * @deprecated  Pouzit ze session jen idPP a zbytek nacist z databaze
     */
//    public static function osobaJmenoPrijmeni()
//    {
//        return $_SESSION['osobajmenoprijmeni'];
//    }

    public static function osobaRC()
    {
        return $_SESSION['osobarc'];
    }

    /**
     * @deprecated
     * @return mixed
     */
    public static function osobaOsobniCislo()
    {
        return $_SESSION['osobaosobnicislo'];
    }

    /**
     * @deprecated
     * @return mixed
     */
    public static function osobaNastup()
    {
        return $_SESSION['osobanastup'];
    }

//    public static function osobaUkonceni()
//    {
//        return $_SESSION['osobaukonceni'];
//    }


    /**
     * Priznak, jestli ma uzivatel ve filtru vybrany "vsichni"
     * @return bool
     */
    public static function vsechnyOsoby()
    {
        return !!$_SESSION['vsechnyOsoby'];
    }

    public static function setVsechnyOsoby($value)
    {
        $_SESSION['vsechnyOsoby'] = $value;
    }

    /**
     * @param int $typRozdeleni nepovinny parametr pro browsery, ktere ve filtru pouzivaji jiny typ rozdeleni
     * @return string
     */
    public static function rozdeleni($typRozdeleni = null)
    {
        $result = $_SESSION['ads_rozdeleni' . ($typRozdeleni ? '_' . $typRozdeleni : '')];
        return ($result === '') ? null : $result;
    }

    /**
     * @param $value
     * @param int $typRozdeleni nepovinny parametr pro browsery, ktere ve filtru pouzivaji jiny typ rozdeleni
     */
    public static function setRozdeleni($value, $typRozdeleni = null)
    {
        $_SESSION['ads_rozdeleni' . ($typRozdeleni ? '_' . $typRozdeleni : '')] = $value;
    }

    /**
     * @param int $typRozdeleni nepovinny parametr pro browsery, ktere ve filtru pouzivaji jiny typ rozdeleni
     * @return int  0=jen vlastni, 1=vcetne podrizenych, 2=jen podrizene, 3=jen primo podrizene
     */
    public static function rozdeleniVcetnePodrizenych($typRozdeleni = null)
    {
        return $_SESSION['ads_rozdeleni_vcetne_podrizenych' . ($typRozdeleni ? '_' . $typRozdeleni : '')];
    }

    /**
     * @param int $value
     * @param int $typRozdeleni nepovinny parametr pro browsery, ktere ve filtru pouzivaji jiny typ rozdeleni
     */
    public static function setRozdeleniVcetnePodrizenych($value, $typRozdeleni = null)
    {
        $_SESSION['ads_rozdeleni_vcetne_podrizenych' . ($typRozdeleni ? '_' . $typRozdeleni : '')] = $value;
    }

    /**
     * @param int $typRozdeleni nepovinny parametr pro browsery, ktere ve filtru pouzivaji jiny typ rozdeleni
     * @return string
     */
    public static function rozdeleni2($typRozdeleni = null)
    {
        return $_SESSION['ads_rozdeleni2' . ($typRozdeleni ? '_' . $typRozdeleni : '')];
    }

    /**
     * @param $value
     * @param int $typRozdeleni nepovinny parametr pro browsery, ktere ve filtru pouzivaji jiny typ rozdeleni
     */
    public static function setRozdeleni2($value, $typRozdeleni = null)
    {
        $_SESSION['ads_rozdeleni2' . ($typRozdeleni ? '_' . $typRozdeleni : '')] = $value;
    }

    public static function rozdeleni2VcetnePodrizenych($typRozdeleni = null)
    {
        return $_SESSION['ads_rozdeleni2_vcetne_podrizenych' . ($typRozdeleni ? '_' . $typRozdeleni : '')];
    }

    /**
     * @param int $value
     * @param int $typRozdeleni nepovinny parametr pro browsery, ktere ve filtru pouzivaji jiny typ rozdeleni
     */
    public static function setRozdeleni2VcetnePodrizenych($value, $typRozdeleni = null)
    {
        $_SESSION['ads_rozdeleni2_vcetne_podrizenych' . ($typRozdeleni ? '_' . $typRozdeleni : '')] = $value;
    }


    public static function vybranaJidelna()
    {
        return $_SESSION['jidelna_jidelna'];
    }

    public static function setVybranaJidelna($value)
    {
        $_SESSION['jidelna_jidelna'] = $value;
    }



    /**
     * @return int
     */
    public static function obdobi()
    {
        return $_SESSION['vybraneobdobi'];
    }

    public static function setObdobi($pocatek, $konec)
    {
        $_SESSION['vybraneobdobi'] = $pocatek;
        $_SESSION['konecvybranehoobdobi'] = $konec;
        //$_SESSION['popisvybranehoobdobi'] = $popis;
        //$_SESSION['vybraneobdobiuzavreno'] = $uzavreno;
    }

//    public static function popisObdobi()
//    {
//        return $_SESSION['popisvybranehoobdobi'];
//    }

    public static function jenObdobi()
    {
        // pouze pokud je vyber obdobi nepovinny
        return $_SESSION['filtrjenobdobi'];
    }

    public static function konecObdobi()
    {
        return $_SESSION['konecvybranehoobdobi'];
    }

//    public static function obdobiUzavreno()
//    {
//        return $_SESSION['vybraneobdobiuzavreno'];
//    }


    public static function setJenObdobi($value)
    {
        $_SESSION['filtrjenobdobi'] = $value;
    }


    public static function variantniObdobi()
    {
        return $_SESSION['ads_variantni'];
    }

    public static function variantniObdobiKonec()
    {
        return $_SESSION['ads_konecvariantnihoobdobi'];
    }

    public static function setVariantniObdobi($zacatek, $konec)
    {
        $_SESSION['ads_variantni'] = $zacatek;
        $_SESSION['ads_konecvariantnihoobdobi'] = $konec;
    }


    public static function getFiltrovatDatum()
    {
        return isset($_SESSION['filtrjendatumoddo']) ? $_SESSION['filtrjendatumoddo'] : 0;
    }

    public static function getDatumOd()
    {
        return $_SESSION['datumod'];
    }

    public static function getDatumDo()
    {
        return $_SESSION['datumdo'];
    }

    public static function setFiltrovatDatum($filtrovat, $datumOd = null, $datumDo = null)
    {
        $_SESSION['filtrjendatumoddo'] = $filtrovat;
        $_SESSION['datumod'] = $datumOd;
        $_SESSION['datumdo'] = $datumDo;
    }
}



require_once('adsapp/sessionvars.php');  // pro kompatibilitu stareho kodu
