<?php
/**
 * UTF8
 * stanislav.elbl, 14.07.2015
 */

namespace Ron\RonPortal;


use Ron\DB\Database;


class NeniPravoException extends \Exception
{

    /**
     * NeniPravoException constructor.
     * @param string|null $message
     */
    public function __construct($message = null)
    {
        parent::__construct( $message ?: \Ron\ronText('Nemate_pravo') );
    }
}


class RonPortalUtils
{
    /**
     * Zjisti, jestli je nainstalovana aplikace se zadanym kodem. Pokud ano, vrati timestamp nainstalovane verze SDF.
     * @param Database $database
     * @param string $kod
     * @return false|int timestamp verze SDF
     */
    public static function jeNainstalovanaAplikace($database, $kod)
    {
        $params = $database ->createParameters();
        $sql = 'SELECT applicationID, VersionTimeStamp FROM SYS_DDEF WHERE applicationID = ' . $params ->addParameter($kod);
        $cursor = $database->select($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();
        if ($row) {
            return $database->stringToTimestamp($row[1]);            
        }

        return false;
    }


    /**
     * Vraci jmeno aktualniho uzivatele pro ulozeni do "logovaciho" pole (Schvalil, Zpracoval, ...)
     * Pokud je orezat = true, vraci retezec o delce maximalne 25 znaku
     * @param bool $orezat
     * @return string
     */
    public static function getLogUserName($orezat = true)
    {
        switch (RonPortalSettings::getIntValue('logovane_jmeno')) {
            case 1 :
                // loguje se jen uzivatelske jmeno
                $result = User::getLoginName();
                break;

            case 2 :
                // loguje se jmeno a prijmeni uzivatele
                $result = User::getUserName();
                break;

            case 0 :
            default :
                $result = User::getLoginName();

                if ($orezat) {
                    if (strlen($result) < 12) {
                        // prida jeste cele jmeno v zavorce
                        $result .= ' (' . substr(User::getUserName(), 0, 25 - strlen($result) - 3) . ')';
                    }
                } else {
                    $result .= ' (' . User::getUserName() . ')';
                }
        }

        if ($orezat  &&  strlen($result) > 25) {
            return substr($result, 0, 25);
        }

        return $result;
    }


    /**
     * @throws \Exception
     */
    public static function neniPravoException()
    {
        throw new NeniPravoException();
    }

}
