<?php

/**
 * UTF8
 */

namespace Ron\RonPortal;


use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;


class TypRozdeleni
{
    /** @var int */
    var $kod;
    /** @var string */
    var $popis;

    /**
     * @param Database $database
     * @param int $typRozdeleni
     * @return bool
     * @throws DBException
     */
    function load($database, $typRozdeleni)
    {
        $params = $database ->createParameters();
        $sql = 'SELECT Kod, Popis FROM TypRozdeleni WHERE Kod = ' . $params ->addParameter((int)$typRozdeleni);
        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) return false;

        $this->kod = (int)$row[0];
        $this->popis = $row[1];

//        $this ->f_kod = $this ->kod;  // 12/2017 - pouze pro kompatibilitu pripadneho uzivatelskeho kodu
//        $this ->f_popis = $this ->popis;

        return true;
    }


    /**
     * @param Database $database
     * @param int $kodTypuRozdeleni
     * @return TypRozdeleni
     * @throws Exception
     */
    public static function getTypRozdeleni($database, $kodTypuRozdeleni)
    {
        $typRozdeleni = new TypRozdeleni();
        if (!$typRozdeleni ->load($database, $kodTypuRozdeleni)) {
            throw new Exception('Chyba - zadany typ rozdeleni neexistuje!');
        }

        return $typRozdeleni;
    }
}



class Rozdeleni
{
    /** @var int    Kod typu rozdeleni */
    var $typ;
    /** @var string Kod rozdeleni */
    var $kod;
    /** @var string Popis rozdeleni */
    var $popis;

    /** @var string Kod nadrazeneho rozdeleni */
    var $rodic;
    /**
     * @var int     Jen pro kompatibilitu! Nepouzivat
     * @deprecated
     */
    var $f_typ;
    /**
     * @var string  Jen pro kompatibilitu! Nepouzivat
     * @deprecated
     */
    var $f_kod;

    var $cssClass;

    var $datumVytvoreni;
    var $datumZruseni;
    var $stat;

    var $eMail;

    /** @var string Kod prirazeneho kalendare */
    var $kalendar;

    private static function getSelectPart()
    {
        return
            'Rozdeleni.KodTypuRozdeleni, Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass, ' . // 0..3
            'Rozdeleni.DatumVytvoreni, Rozdeleni.DatumZruseni, ' . // 4,5
            'Rozdeleni.Stat, Rozdeleni.eMail, Rozdeleni.Rodic, Rozdeleni.Kalendar'; // 6..9
    }

    /**
     * @param Database $database
     * @param array $row
     */
    private function loadRow($database, $row)
    {
        $this ->typ = (int)$row[0];
        $this ->kod = $row[1];
        $this ->popis = $row[2];
        $this ->cssClass = $row[3];
        $this ->datumVytvoreni = $database ->stringToTimestamp($row[4]);
        $this ->datumZruseni = $database ->stringToTimestamp($row[5]);
        $this ->stat = $row[6];
        $this ->eMail = $row[7];
        $this ->rodic = $row[8];
        $this ->kalendar = $row[9];

        $this ->f_typ = $this ->typ;    // 12/2017 - pouze pro kompatibilitu pripadneho uzivatelskeho kodu
        $this ->f_kod = $this ->kod;
        //$this ->f_popis = $this ->popis;
    }

    /**
     * @param Database $database
     * @param int $typRozdeleni
     * @param string $kod
     * @throws DBException
     * @throws Exception
     */
    function load($database, $typRozdeleni, $kod)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT ' . self::getSelectPart() .
            '  FROM Rozdeleni '.
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
            '   AND Kod = ' . $params ->addParameter($kod);

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();
        if (!$row) throw new Exception('Zadane rozdeleni nebylo nalezeno!');

        $this ->loadRow($database, $row);
    }


    /**
     * @param Database $database
     * @param int    $typRozdeleni
     * @param string $kod
     * @return Rozdeleni
     * @throws Exception
     */
    public static function loadRozdeleni($database, $typRozdeleni, $kod)
    {
        $result = new Rozdeleni();
        $result ->load($database, $typRozdeleni, $kod);
        return $result;
    }


    /**
     * Nacte rozdeleni zadaneho typu pro zadany pracovni pomer.
     * @param Database $database
     * @param int $idPP
     * @param int $typRozdeleni
     * @param int $datumOd Obdobi, ve kterem ma byt osoba na danem rozdeleni. Pokud neni zadano, pouzije se time()
     * @param int $datumDo ... pokud neni zadano, nastavi se na stejnou hodnotu jako $datumOd
     * @param int $vcetnePodrizenych  0=jen prirazene, 1=vcetne podrizenych, 2=jen podrizene
     * @return Rozdeleni[]
     * @throws DBException
     */
    public static function loadRozdeleniOsoby($database, $idPP, $typRozdeleni, $datumOd = null, $datumDo = null, $vcetnePodrizenych = 0)
    {
        if (!$datumOd) $datumOd = time();
        if (!$datumDo) $datumDo = $datumOd;
        $params = $database ->createParameters();

        $sql =
            'SELECT ' . self::getSelectPart();

        if ($vcetnePodrizenych) {
            $sql .=
                ' FROM RozdeleniOsob, Rozdeleni, PracovniPomer, HierarchieRozdeleni' .
                ' WHERE HierarchieRozdeleni.TypPodrizene = Rozdeleni.KodTypuRozdeleni' .
                ' AND HierarchieRozdeleni.KodPodrizene = Rozdeleni.Kod' .
                ' AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.TypNadrazene' .
                ' AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.KodNadrazene';

            if ($vcetnePodrizenych == 2) {
                // jen podrizena
                $sql .=
                    ' AND HierarchieRozdeleni.KodNadrazene <> HierarchieRozdeleni.KodPodrizene';
            }
        } else {
            $sql .=
                ' FROM RozdeleniOsob, Rozdeleni, PracovniPomer' .
                ' WHERE RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                ' AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod';
        }

        $sql .=
            // typ rozdeleni
            ' AND Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .

            // PP
            ' AND RozdeleniOsob.RC = PracovniPomer.RC' .
            ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
            ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
            ' AND PracovniPomer.Id = ' . $params ->addParameter((int)$idPP) .

            // platnost prirazeni a platnost rozdeleni
            ' AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
            ' AND Rozdeleni.DatumVytvoreni <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            ' AND (Rozdeleni.DatumZruseni >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR Rozdeleni.DatumZruseni IS NULL)';

        $sql .=
            ' ORDER BY RozdeleniOsob.PlatnostOd, Rozdeleni.Kod';

        $cursor = $database ->getSqlResult($sql, $params);

        $result = [];
        while ($row = $cursor ->fetch()) {
            $rozdeleni = new Rozdeleni();
            $rozdeleni ->loadRow($database, $row);
            $result[] = $rozdeleni;
        }

        $cursor ->close();

        return $result;
    }



    /**
     * Nacte rozdeleni zadaneho typu pro zadany pracovni pomer.
     * @param Database $database
     * @param int $idPP
     * @param int $typRozdeleni
     * @param int $datumOd Obdobi, ve kterem ma byt osoba na danem rozdeleni. Pokud neni zadano, pouzije se time()
     * @param int $datumDo ... pokud neni zadano, nastavi se na stejnou hodnotu jako $datumOd
     * @param int $vcetnePodrizenych  0=jen prirazene, 1=vcetne podrizenych, 2=jen podrizene
     * @return string|null
     * @throws DBException
     */
    public static function vratRozdeleniOsoby($database, $idPP, $typRozdeleni, $datumOd = null, $datumDo = null, $vcetnePodrizenych = 0)
    {
        $seznamRozdeleni = self::loadRozdeleniOsoby($database, $idPP, $typRozdeleni, $datumOd, $datumDo, $vcetnePodrizenych);
        if (!count($seznamRozdeleni)) {
            return null;
        }

        return $seznamRozdeleni[0] -> kod;
    }

    /**
     * @param Database $database
     * @param string $tabulka
     * @param int $typRozdeleni
     * @param string $kod
     * @param int $datumOd
     * @param int $datumDo
     * @param int $vcetnePodrizenych   0=jen vybrane, 1=vcetne podrizenych, 2=jen podrizne, 3=jen primo podrizene
     * @param null|Parameters $params
     * @return string|SqlStatement
     */
    static function sqlFilterRozdeleni($database, $tabulka, $typRozdeleni, $kod, $datumOd, $datumDo, $vcetnePodrizenych, $params = null)
    {
        if ($vcetnePodrizenych === true) {
            $vcetnePodrizenych = 1;
        } else if ($vcetnePodrizenych === false) {
            $vcetnePodrizenych = 0;
        }

        switch ($vcetnePodrizenych) {
            case 1 : // vcetne podrizenych
            case 2 : // jen podrizene
                return
                    'EXISTS (' .
                    '  SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
                    '  WHERE HierarchieRozdeleni.TypNadrazene = ' . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    "    AND HierarchieRozdeleni.KodNadrazene = " . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod)) .

                    (($vcetnePodrizenych == 2) ? ' AND HierarchieRozdeleni.KodNadrazene <> HierarchieRozdeleni.KodPodrizene' : '') . // filtr pro jen podrizene

                    '    AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.typPodrizene' .
                    '    AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.kodPodrizene' .

                    '    AND RozdeleniOsob.RC = ' . $tabulka . '.RC ' .
                    '    AND RozdeleniOsob.OsobniCislo = ' . $tabulka . '.OsobniCislo ' .
                    '    AND RozdeleniOsob.DatumNastupu = ' . $tabulka . '.DatumNastupu ' .
                    '    AND RozdeleniOsob.PlatnostOd <= ' . (is_null($params) ? $database ->timestampToString($datumDo) : $params->addParameter($database->timestampToParam($datumDo))) .
                    '    AND (RozdeleniOsob.PlatnostDo >= ' . (is_null($params) ? $database ->timestampToString($datumOd) : $params->addParameter($database->timestampToParam($datumOd))) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';
                break;

            case 3 :
                // jen primo podrizene
                return
                    'EXISTS (' .
                    '  SELECT * FROM RozdeleniOsob, Rozdeleni ' .
                    '  WHERE Rozdeleni.RozdeleniRodice = ' . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    "    AND Rozdeleni.Rodic = " . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod)) .
                    '    AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                    '    AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod' .
                    '    AND RozdeleniOsob.RC = ' . $tabulka . '.RC ' .
                    '    AND RozdeleniOsob.OsobniCislo = ' . $tabulka . '.OsobniCislo ' .
                    '    AND RozdeleniOsob.DatumNastupu = ' . $tabulka . '.DatumNastupu ' .
                    '    AND RozdeleniOsob.PlatnostOd <= ' . (is_null($params) ? $database ->timestampToString($datumDo) : $params->addParameter($database->timestampToParam($datumDo))) .
                    '    AND (RozdeleniOsob.PlatnostDo >= ' . (is_null($params) ? $database ->timestampToString($datumOd) : $params->addParameter($database->timestampToParam($datumOd))) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';
            default :
                // jen vybrane
                return
                    'EXISTS (' .
                    '  SELECT * FROM RozdeleniOsob ' .
                    '  WHERE RozdeleniOsob.KodTypuRozdeleni = ' . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    "    AND RozdeleniOsob.Rozdeleni = " . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod)) .
                    '    AND RozdeleniOsob.RC = ' . $tabulka . '.RC ' .
                    '    AND RozdeleniOsob.OsobniCislo = ' . $tabulka . '.OsobniCislo ' .
                    '    AND RozdeleniOsob.DatumNastupu = ' . $tabulka . '.DatumNastupu ' .
                    '    AND RozdeleniOsob.PlatnostOd <= ' . (is_null($params) ? $database ->timestampToString($datumDo) : $params->addParameter($database->timestampToParam($datumDo))) .
                    '    AND (RozdeleniOsob.PlatnostDo >= ' . (is_null($params) ? $database ->timestampToString($datumOd) : $params->addParameter($database->timestampToParam($datumOd))) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';
        }
    }


    /**
     * Vraci true, pokud je RC/idPP v danem obdobi v zadanem rozdeleni nebo v nadrazenem/podrizenem rozdeleni (viz priznak $vcetneNadrazenych)
     * @param Database $database
     * @param string $RC Pokud je nastaveno, hleda podle RC
     * @param int $idPP Pokud je nastaveno, hleda podle pracovniho pomeru
     * @param int $typRozdeleni
     * @param string|string[] $kodRozdeleni Kod rozdeleni nebo seznam kodu (POZOR! seznam nefunguje pro vcetneNadrazenych = 2)
     * @param int $vcetneNadrazenych 0 - pouze vlastni;  1/true - vcetne nadrazenych,  2 - jen nadrazene, 3 - vcetne podrizenych,
     *                  4 - vlastni a primo podrizene, 5 - primo podrizene
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws Exception
     */
    static function jeVRozdeleni($database, $RC, $idPP, $typRozdeleni, $kodRozdeleni, $vcetneNadrazenych, $obdobiOd, $obdobiDo)
    {
        if (!isset($kodRozdeleni)) {
            return true;
        }

        $typRozdeleni = (int)$typRozdeleni;

        require_once('ron/sqlutils.php');
        $params = $database ->createParameters();

        if ($vcetneNadrazenych === 4) {
            //vlastni a primo podrizene
            $sql =
                'SELECT DISTINCT R_Ja.RC ' .
                '  FROM RozdeleniOsob R_JA, Rozdeleni R_Nad, Rozdeleni R_Pod ' . ( isset($idPP) ? ', PracovniPomer PP' : '') .
                ' WHERE R_Nad.KodTypuRozdeleni = ' .$params ->addParameter($typRozdeleni) .
                '   AND (' . SQLUtils::getINCondition($database, 'R_Nad.Kod', $kodRozdeleni, $params) .
                '       OR ' . SQLUtils::getINCondition($database, 'R_Pod.Kod', $kodRozdeleni, $params) . ' ) ' .
                '   AND (R_Nad.KodTypuRozdeleni = R_Ja.KodTypuRozdeleni AND R_Nad.Kod = R_Ja.Rozdeleni) ' .
                '   AND (R_Pod.RozdeleniRodice = R_Nad.KodTypuRozdeleni AND R_Pod.Rodic = R_Nad.Kod) ';
        } else if ($vcetneNadrazenych === 5) {
            //primo podrizene
            $sql =
                'SELECT DISTINCT R_Ja.RC ' .
                '  FROM RozdeleniOsob R_JA, Rozdeleni R_Nad, Rozdeleni R_Pod ' . ( isset($idPP) ? ', PracovniPomer PP' : '') .
                ' WHERE R_Nad.KodTypuRozdeleni = ' . $params ->addParameter($typRozdeleni) .
                '   AND ' . SQLUtils::getINCondition($database, 'R_Pod.Kod', $kodRozdeleni, $params) .
                '   AND (R_Nad.KodTypuRozdeleni = R_Ja.KodTypuRozdeleni AND R_Nad.Kod = R_Ja.Rozdeleni) ' .
                '   AND (R_Pod.RozdeleniRodice = R_Nad.KodTypuRozdeleni AND R_Pod.Rodic = R_Nad.Kod) ';
        } else if ($vcetneNadrazenych === 3) {
            // vcetne podrizenych rozdeleni
            $sql =
                'SELECT DISTINCT R_Ja.RC ' .
                '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H ' . ( isset($idPP) ? ', PracovniPomer PP' : '') .
                ' WHERE H.TypNadrazene = ' . $params ->addParameter($typRozdeleni) .
                '   AND ' . SQLUtils::getINCondition($database, 'H.KodNadrazene', $kodRozdeleni, $params) .
                '   AND H.TypPodrizene = R_Ja.KodTypuRozdeleni ' .
                '   AND H.KodPodrizene = R_Ja.Rozdeleni ';
        } else if ($vcetneNadrazenych) {
            $sql =
                'SELECT DISTINCT R_Ja.RC ' .
                '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H ' . ( isset($idPP) ? ', PracovniPomer PP' : '') .
                ' WHERE H.TypPodrizene =  ' . $params ->addParameter($typRozdeleni) .
                //"   AND H.KodPodrizene = '" . $database ->addSlashes($kodRozdeleni) . "'" .
                '   AND ' . SQLUtils::getINCondition($database, 'H.KodPodrizene', $kodRozdeleni, $params) .
                '   AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                '   AND H.KodNadrazene = R_Ja.Rozdeleni ';

            if ($vcetneNadrazenych === 2) // hledam jen v nadrazenych
                $sql .=
                    ' AND H.KodNadrazene <> ' . $params->addParameter($kodRozdeleni);
        } else {
            $sql =
                'SELECT DISTINCT R_Ja.RC ' .
                ' FROM RozdeleniOsob R_JA ' . ( isset($idPP) ? ', PracovniPomer PP' : '') .
                ' WHERE R_Ja.KodTypuRozdeleni = ' . $params ->addParameter($typRozdeleni) .
                //"   AND R_Ja.Rozdeleni = '" . $database ->addSlashes($kodRozdeleni) . "'";
                '   AND ' . SQLUtils::getINCondition($database, 'R_Ja.Rozdeleni', $kodRozdeleni, $params);
        }

        $sql .=
            '   AND R_Ja.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '   AND (R_Ja.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . '  OR  R_Ja.PlatnostDo IS NULL)';

        if (isset($idPP)) {
            $sql .=
                ' AND PP.RC = R_Ja.RC ' .
                ' AND PP.OsobniCislo = R_Ja.OsobniCislo ' .
                ' AND PP.DatumNastupu = R_Ja.DatumNastupu ' .
                ' AND PP.ID = ' . $params ->addParameter((int)$idPP) .
                ' AND PP.DatumNastupu <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
                ' AND (PP.DatumUkonceni >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . '  OR  PP.DatumUkonceni IS NULL)';
        } else {
            $sql .=
                ' AND R_Ja.RC = ' . $params ->addParameter($RC);
        }

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return !!$row;
    }


    /**
     * Vraci stejnou hodnotu jako funkce jeVRozdeleni. Rozdil je ve zpusobu zadani rozdeleni
     * @param Database $database
     * @param string $RC
     * @param int $idPP
     * @param string $rozdeleniStr retezec ve formatu "typRozdeleni/kod1;kod2;kod3;..."
     * @param int $vcetneNadrazenych
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws Exception
     */
    static function jeVRozdeleniStr($database, $RC, $idPP, $rozdeleniStr, $vcetneNadrazenych, $obdobiOd, $obdobiDo)
    {
        $pom = explode('/', $rozdeleniStr, 2);
        $typRozdeleni = (int)$pom[0];

        $kody = explode(';', $pom[1]);
        return self::jeVRozdeleni($database, $RC, $idPP, $typRozdeleni, $kody, $vcetneNadrazenych, $obdobiOd, $obdobiDo);
    }


    /**
     * Vraci true, pokud je RCpodrizeny v danem obdobi v nejakem podrizenem rozdeleni daneho typu
     * Pokud je vcetneVlastnich true, tak muze byt i ve stejnem rozdeleni
     * Pokud neni nastaveno $OCpodrizeny, $Nastuppodrizeny, kontroluje se jen RC
     *
     * @param Database $database
     * @param string $RCnadrizeny
     * @param string $RCpodrizeny
     * @param string $OCpodrizeny
     * @param int $nastupPodrizeny
     * @param int $typRozdeleni
     * @param int $vcetneVlastnich
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws Exception
     */
    static function jePodrizeny($database, $RCnadrizeny, $RCpodrizeny, $OCpodrizeny, $nastupPodrizeny, $typRozdeleni, $vcetneVlastnich, $obdobiOd, $obdobiDo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT R_Ja.RC ' .
            '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H, RozdeleniOsob R_Hodnoceny ' .
            '  WHERE R_Ja.RC = ' . $params->addParameter($RCnadrizeny) .
            '    AND R_JA.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
            '    AND R_Ja.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '    AND (R_Ja.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . '  OR  R_Ja.PlatnostDo IS NULL)' .
            '    AND R_Hodnoceny.KodTypuRozdeleni = R_JA.KodTypuRozdeleni ' .
            '    AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
            '    AND H.KodNadrazene = R_Ja.Rozdeleni ' .
            '    AND H.TypPodrizene = R_Hodnoceny.KodTypuRozdeleni ' .
            '    AND H.KodPodrizene = R_Hodnoceny.Rozdeleni ' .
            '    AND R_Hodnoceny.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '    AND (R_Hodnoceny.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . '  OR  R_Hodnoceny.PlatnostDo IS NULL)' .
            (!$vcetneVlastnich ? ' AND H.KodNadrazene <> H.KodPodrizene ' : '') .
            '    AND R_Hodnoceny.RC = ' . $params->addParameter($RCpodrizeny);

        if ($OCpodrizeny && $nastupPodrizeny) {
            // kontroluje se i osobni cislo a datum nastupu
            $sql .=
                '  AND R_Hodnoceny.OsobniCislo = ' . $params ->addParameter($OCpodrizeny) .
                '  AND R_Hodnoceny.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($nastupPodrizeny));
        }

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return !!$row;
    }
}
