<?php
/**
 * UTF8
 * stanislav.elbl, 28.11.2015
 * obsluha formulare pro prirazovani rozdeleni osoby
 */

namespace Ron\RonPortal;

use Ron\DB\Database;
use Ron\ShiftForm;
use Ron\StringUtils;

require_once('ron/shiftform.php');


class RozdeleniShiftForm extends ShiftForm
{
    /** @var int */
    var $typRozdeleni;
    /** @var PravaNaOsoby|PravaNaOsoby[]  */
    var $prava;

    /**
     * RozdeleniShiftForm constructor.
     * @param Database $database
     * @param int    $typRozdeleni
     * @param string $prava
     */
    public function __construct($database, $typRozdeleni, $prava)
    {
        parent::__construct($database);

        $this ->typRozdeleni = $typRozdeleni;
        $this ->prava = $prava;
    }


    function loadData()
    {
        $result = [];

        require_once('adsapp/ppomer.php');
        require_once('ron/sqlutils.php');

        $pp = new PracovniPomer();
        if (!$pp->nactiPPPodleId($this ->database, $_GET['idpp'])) {
            throw new \Exception('Nelze nacist pracovni pomer!');
        }

        $forceLoad = isset($_GET['force'])  &&  (int) $_GET['force'];
        $loadAll = !(isset($_GET['jenvlastni']) && (int)$_GET['jenvlastni']); // jen vlastni - pouziva se pro aktualizaci comboboxu

        $filter = isset($_GET['filter'])
            ? urldecode($_GET['filter'])        // iconv('utf-8', 'windows-1250', $_GET['filter'])
            : null;

        $result['filter'] = StringUtils::jsonText($filter);

        $count = null;

        session_write_close(); // aby fungoval .abort() v javascriptu

        // podle poctu existujicich zaznamu automaticky urci, jestli se nactou rovnou vsechna data
        // (pri velkem poctu zaznamu je to pomale - proto se nenacitaji zaznamy ihned)
        if ($loadAll) {
            $params = $this ->database ->createParameters();

            $sql =
                'SELECT COUNT(*) FROM Rozdeleni ' .
                ' WHERE KodTypuRozdeleni = ' . $params ->addParameter((int)$this ->typRozdeleni) .
                ' AND (Rozdeleni.DatumZruseni > ' . $params ->addParameter($this ->database ->timestampToParam(time())) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
                " AND (cssClass IS NULL  OR  cssClass <> 'hidden') ";

            if (isset($filter)  &&  $filter != '')
                $sql .= '   AND (Popis LIKE ' . $params->addParameter('%' . $filter. '%') . '  OR  Kod LIKE ' . $params->addParameter('%' . $filter . '%') . ')';

            $cursor = $this ->database ->getSqlResult($sql, $params);
            $row = $cursor ->fetch();

            $count = $row ? $row[0] : 0;
            $loadAll = $count < 3000;
        }

        $params = $this ->database ->createParameters();
        if ($loadAll  ||  $forceLoad) {
            // POZOR! Nefunguje na ODBC s driverem "SQL Server" - je potreba prepnout na jiny driver!
            // nacita vsechna rozdeleni vcetne neprirazenych
            $sql =
                'SELECT Rozdeleni.Kod, Rozdeleni.Popis, RozdeleniOsob.PlatnostOd, Rozdeleni.cssClass ' .
                ' FROM Rozdeleni LEFT OUTER JOIN RozdeleniOsob ' .
                '  ON RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
                '  AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod ' .
                '  AND RozdeleniOsob.RC = ' . $params->addParameter($pp->rc) .
                '  AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
                '  AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pp->datumNastupu)) .
                '  AND (RozdeleniOsob.PlatnostDo > ' . $params ->addParameter($this ->database ->dateToParam(time())) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$this ->typRozdeleni) .
                ' AND (Rozdeleni.DatumZruseni > ' . $params ->addParameter($this ->database ->dateToParam(time())) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
                " AND (Rozdeleni.cssClass IS NULL  OR  Rozdeleni.cssClass <> 'hidden') " .

                (isset($filter)
                    ? ' AND (Rozdeleni.Popis LIKE ' . $params ->addParameter("%{$filter}%") . ' OR  Rozdeleni.Kod LIKE ' . $params ->addParameter("%{$filter}%") . ')'
                    : '') .

                ' ORDER BY Rozdeleni.Popis ';

            $result['nactenoVse'] = 1;
        } else {
            // nacita jen vlastni rozdeleni
            $sql =
                'SELECT Rozdeleni.Kod, Rozdeleni.Popis, RozdeleniOsob.PlatnostOd, Rozdeleni.cssClass ' .
                ' FROM RozdeleniOsob, Rozdeleni ' .
                ' WHERE RozdeleniOsob.KodTypuRozdeleni = ' . $params ->addParameter((int)$this ->typRozdeleni) .
                ' AND RozdeleniOsob.RC = ' . $params->addParameter($pp->rc) .
                ' AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
                ' AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pp->datumNastupu)) .
                ' AND (RozdeleniOsob.PlatnostDo > ' . $params ->addParameter($this ->database ->dateToParam(time())) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                ' AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni ' .
                ' AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
                ' AND (Rozdeleni.DatumZruseni > ' . $params ->addParameter($this ->database ->dateToParam(time())) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
                " AND (cssClass IS NULL  OR  cssClass <> 'hidden') " .

                (isset($filter)
                    ? ' AND (Rozdeleni.Popis LIKE ' . $params ->addParameter("%{$filter}%") . '  OR  Rozdeleni.Kod LIKE ' . $params ->addParameter("%{$filter}%") . ')'
                    : '') .

                ' ORDER BY Rozdeleni.Popis ';

            $result['nactenoVse'] = 0;
        }

        if (isset($count)) {
            // pocet zaznamu
            $result['count'] = (int)$count;
        }

        $result['leftItems'] = [];
        $result['rightItems'] = [];

        $cursor = $this ->database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            //$obj = ['kod' => jsonText($row[0]), 'popis' => jsonText($row[1])];
            $obj = [
                'kod' => StringUtils::jsonText($row[0], false),
                'popis' => StringUtils::jsonText($row[1], false)
            ];

            if ($row[3]) {
                $obj['cssClass'] = StringUtils::jsonText($row[3]);
            }

            if ($row[2]) {
                // prirazena
                $result['rightItems'][] = $obj;
            } else {
                $result['leftItems'][] = $obj;
            }
        }

        return $result;
    }


    /**
     * @param PracovniPomer $pp
     * @param string $kodRozdeleni
     */
    function odstranRozdeleni($pp, $kodRozdeleni)
    {
        require_once('adsapp/ronportaldbrecord.php');

        $rec = new RonPortalDBRecord($this ->database, 'RozdeleniOsob');
        $rec->logDescription = StringUtils::fromUTF8('Rozdělení osob');

        $rec->addField('RC', '');
        $rec->addField('OsobniCislo', ''); // readonly = true
        $rec->addField('DatumNastupu', 'date');

        $rec->addField('KodTypuRozdeleni', 'int');
        $rec->addField('Rozdeleni', '');

        $rec->addField('PlatnostOd', 'date');
        $rec->addField('PlatnostDo', 'date');

        $params = $this ->database ->createParameters();
        $rec->setWhere(
            ' RC = ' . $params->addParameter($pp->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pp->datumNastupu)) .
            ' AND KodTypuRozdeleni = ' . $params ->addParameter((int)$this ->typRozdeleni) .
            ' AND Rozdeleni = ' . $params->addParameter($kodRozdeleni),
            $params);

        $rec->checkMultiple = false;
        $rec->deleteRecords();
    }


    function handleMoveLeft()
    {
        require_once('adsapp/ppomer.php');

        $pp = new PracovniPomer();
        if (!$pp->nactiPPPodleId($this ->database, $_GET['idpp'])) {
            throw new \Exception('Nelze nacist pracovni pomer!');
        }

        // zkontrolovat prava
        if (!Prava::lzeZobrazitPP($this ->prava, $pp ->idPP, time(), time())) {
            throw new NeniPravoException();
        }

        for ($i = 0; $i < $_POST['count']; $i++) {
            $kod = StringUtils::fromUTF8($_POST['kod#' . $i]);
            $this ->odstranRozdeleni($pp, $kod);
        }

        return [ 'rightAction' => 'remove', 'leftAction' => 'add' ];
    }


    /**
     * @param PracovniPomer $pp
     * @param string $kodRozdeleni
     * @param int    $platnostOd
     * @throws \Exception
     */
    function priradRozdeleni($pp, $kodRozdeleni, $platnostOd)
    {
        require_once('ron/sqlutils.php');
        require_once('adsapp/ronportaldbrecord.php');

        $rec = new RonPortalDBRecord($this ->database, 'RozdeleniOsob');
        $rec->logDescription = StringUtils::fromUTF8('Rozdělení osob');

        $rec->addField('RC', '', true, 0, $pp->rc); // readonly = true
        $rec->addField('OsobniCislo', '', true, 0, $pp->osobniCislo); // readonly = true
        $rec->addField('DatumNastupu', 'date', true, 0, $pp->datumNastupu);

        $rec->addField('KodTypuRozdeleni', 'int', true, 0, (int)$this ->typRozdeleni);
        $rec->addField('Rozdeleni', '', true, 30, $kodRozdeleni);

        $rec->addField('PlatnostOd', 'date', true, 0, $platnostOd);

        $params = $this ->database ->createParameters();
        $rec->setWhere(
            ' RC = ' . $params->addParameter($pp->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pp->datumNastupu)) .
            ' AND PlatnostOd <= ' . $params ->addParameter($this ->database ->timestampToParam(time())) .
            ' AND (PlatnostDo >=  ' . $params ->addParameter($this ->database ->timestampToParam(time())) . '  OR  PlatnostDo IS NULL )' .
            ' AND KodTypuRozdeleni = ' . $params ->addParameter((int)$this ->typRozdeleni) .
            ' AND Rozdeleni = ' . $params->addParameter($kodRozdeleni),
            $params);

        $rec->checkMultiple = false;

        if (!$rec->loadRecord()) {
            $rec->insertRecord();
        }
    }


    function handleMoveRight()
    {
        require_once('adsapp/ppomer.php');
        require_once('adsapp/rozdeleni.php');

        // zkontrolovat prava!
        $pp = new PracovniPomer();
        if (!$pp->nactiPPPodleId($this ->database, $_GET['idpp'])) {
            throw new \Exception('Nelze nacist pracovni pomer!');
        }

        if (!Prava::lzeZobrazitPP($this ->prava, $pp ->idPP, time(), time())) {
            throw new NeniPravoException();
        }

        for ($i = 0; $i < $_POST['count']; $i++) {
            $kod = StringUtils::fromUTF8($_POST['kod#' . $i]);
            $this ->priradRozdeleni($pp, $kod, $pp->datumNastupu);
        }

        return [ 'leftAction' => 'remove', 'rightAction' => 'add' ];
    }

}
