<?php
/**
 * Obecny predek pro schvaleni (3 urovne, datumDo)
 * stanislav.elbl, 27.05.2019
 */

namespace Ron\RonPortal;

use Ron\DB\Database;
use Ron\DB\Parameters;

abstract class Schvaleni
{
    /** @var  int   Datum, do ktereho je schvaleno 1. urovni */
    public $schvaleni1;
    /** @var  int   Datum, do ktereho je schvaleno 2. urovni */
    public $schvaleni2;
    /** @var  int   Datum, do ktereho je schvaleno 3. urovni */
    public $schvaleni3;


    public function jeSchvaleni()
    {
        return $this ->schvaleni1  ||  $this ->schvaleni2  ||  $this ->schvaleni3;
    }


    /**
     * Vraci nejvyssi uroven, kterou je schvaleno zadane datum
     * @param int $datum
     * @return int
     */
    public function vratUrovenSchvaleni($datum)
    {
        if ($this ->schvaleni3 >= $datum) {
            return 3;
        }
        if ($this ->schvaleni2 >= $datum) {
            return 2;
        }
        if ($this ->schvaleni1 >= $datum) {
            return 1;
        }

        return 0;
    }


    /**
     * Vraci true, pokud je zadane datum schvaleno.
     * @param int $datum    Datum, ktere se kontroluje
     * @param int $uroven   Uroven, ktera se kontroluje. Pokud neni zadano, kontroluji se vsechny
     * @return bool
     */
    public function jeSchvalenoDatum($datum, $uroven = null)
    {
        if ($datum <= $this ->schvaleni3) {
            return true;
        }

        if ($uroven >= 3) {
            return false;
        }

        if ($datum <= $this ->schvaleni2) {
            return true;
        }

        if ($uroven == 2) {
            return false;
        }

        return $datum <= $this ->schvaleni1;
    }


    /**
     * Funkce v potomkovi vraci pravo pro schvaleni zadanou urovni
     * @param int $uroven
     * @return \Ron\RonPortal\PravaNaOsoby
     */
    public abstract function vratPravoProUroven($uroven);


    public abstract function lzeSchvalovatDen($uroven, $datum);


    /**
     * Vraci true, pokud zadany den neni schvalen, nebo ma uzivatel pravo schvalovat vyssi urovni
     * @param $datum
     * @return bool
     */
    public function lzeEditovatDen($datum)
    {
        $schvalenaUroven = $this ->vratUrovenSchvaleni($datum);

        if (!$schvalenaUroven) return true;
        if ($schvalenaUroven > 2) return false; // schvaleno nejvyssi urovni => nelze

        if ($this ->lzeSchvalovatDen(3, $datum)) {
            return true; // uzivatel ma pravo schvalovat uroven 3 => muze editovat
        }

        if ($schvalenaUroven > 1) return false;

        if ($this ->lzeSchvalovatDen(2, $datum)) {
            return true; // uzivatel ma pravo schvalovat uroven 2 => muze editovat
        }

        return false;
    }


//    /**
//     * Vraci true, pokud dany den neni schvaleny, nebo ma uzivatel pravo schvalovat osobu vyssi urovni
//     * @param int $datum
//     * @param int $idPP
//     * @return bool
//     * @throws \Exception
//     */
//    public function lzeEditovatOsobu($datum, $idPP)
//    {
//        if ($this ->schvaleni3 >= $datum) return false;
//
//        if ($this ->schvaleni2 >= $datum) {
//            // schvaleno urovni 2, muze editovat jen kdyz ma pravo schvalovat uroven 3
//            $pravo = $this ->vratPravoProUroven(3);
//            return $pravo  &&  $pravo ->lzeZobrazitPP($idPP, $datum, $datum);
//        }
//
//        if ($this ->schvaleni1 >= $datum) {
//            // schvaleno urovni 1, muze editovat jen kdyz ma pravo schvalovat uroven 2 nebo 3
//            $pravo = $this ->vratPravoProUroven(2);
//            if ($pravo  &&  $pravo ->lzeZobrazitPP($idPP, $datum, $datum)) {
//                return true;
//            };
//
//            $pravo = $this ->vratPravoProUroven(3);
//            return $pravo  &&  $pravo ->lzeZobrazitPP($idPP, $datum, $datum);
//        }
//
//        return true;
//    }
//
//
//    /**
//     * Vraci true, pokud dany den neni schvaleny, nebo ma uzivatel pravo schvalovat rozdeleni vyssi urovni
//     * @param int    $datum
//     * @param int    $kodTypuRozdeleni
//     * @param string $rozdeleni
//     * @return bool
//     */
//    public function lzeEditovatRozdeleni($datum, $kodTypuRozdeleni, $rozdeleni)
//    {
//        if ($this ->schvaleni3 >= $datum) return false;
//
//        if ($this ->schvaleni2 >= $datum) {
//            // schvaleno urovni 2, muze editovat jen kdyz ma pravo schvalovat uroven 3
//            $pravo = $this ->vratPravoProUroven(3);
//            return $pravo  &&  $pravo ->jePravoNaRozdeleni($kodTypuRozdeleni, $rozdeleni, $datum, $datum);
//        }
//
//        if ($this ->schvaleni1 >= $datum) {
//            // schvaleno urovni 1, muze editovat jen kdyz ma pravo schvalovat uroven 2 nebo 3
//            $pravo = $this ->vratPravoProUroven(2);
//            if ($pravo  &&  $pravo ->jePravoNaRozdeleni($kodTypuRozdeleni, $rozdeleni, $datum, $datum)) {
//                return true;
//            };
//
//            $pravo = $this ->vratPravoProUroven(3);
//            return $pravo  &&  $pravo ->jePravoNaRozdeleni($kodTypuRozdeleni, $rozdeleni, $datum, $datum);
//        }
//
//        return true;
//    }


    /**
     * Vraci SELECT cast dotazu pro nacteni dat
     * @return string
     */
    public static function getSqlSelectPart()
    {
        return
            'Schvaleni1, Schvalil1, Cas_Schvaleni1,'  .
            'Schvaleni2, Schvalil2, Cas_Schvaleni2, ' .
            'Schvaleni3, Schvalil3, Cas_Schvaleni3';
    }


    /**
     * @param Database $database
     * @param array $row
     */
    public function loadRow($database, $row)
    {
        $this ->schvaleni1 = $database ->stringToTimestamp($row[0]);
        $this ->schvaleni2 = $database ->stringToTimestamp($row[3]);
        $this ->schvaleni3 = $database ->stringToTimestamp($row[6]);
    }


    /**
     * @param Database $database
     * @param string $tableName
     * @param string $whereCondition
     * @param Parameters $whereParams
     * @return Schvaleni
     * @throws \Ron\DB\DBException
     */
    public static function nactiZaznamSchvaleni($database, $tableName, $whereCondition, $whereParams)
    {
        $sql =
            'SELECT ' . static::getSqlSelectPart() .
            ' FROM ' . $tableName;

        if ($whereCondition) {
            $sql .= ' WHERE ' . $whereCondition;
        }

        $cursor = $database ->getSqlResult($sql, $whereParams);
        $row = $cursor ->fetch();
        $cursor ->close();

        $result = new static();

        if ($row) {
            $result ->loadRow($database, $row);
        }

        return $result;
    }



    /**
     * Vraci HTML class pro zobrazeni priznaku schvaleni (podle nejvyssiho schvaleni v danem dni)
     * @param int $datum
     * @return string
     */
    public function getHtmlClass($datum)
    {
        $uroven = $this ->vratUrovenSchvaleni($datum);
        if (!$uroven) return '';

        return 'schvaleni' . $uroven;
    }

}
