<?php
/**
 * UTF8
 * stanislav.elbl, 07.11.2018
 */

namespace Ron\RonPortal;

/**
 * Definice probihajici ulohy
 * Class TaskManagerTask
 * @package Ron\RonPortal
 */
class TaskManagerTask
{
    /** @var string     Id prirazuje TaskManager */
    public $id;

    public $caption;

    public $isRunning;
    /** @var bool       Priznak, ze uloha podporuje Abort */
    public $abortAvailable;
    /** @var integer    Cas posledniho zjisteni stavu */
    public $lastUpdateTime;

    /** @var integer */
    public $progressMax;
    /** @var integer */
    public $progressCurrent;
    /** @var string */
    public $progressMessage;
    /** @var string */
    public $errorMessage;

    // Udaje z executoru, napr. z Agent managera
    public $executorParams = [];


    public function __construct()
    {

    }

    public function storeToArray()
    {
        return [
            //'handle' => $this ->handle,
            'isRunning' => $this ->isRunning,
            'abortAvailable' => $this ->abortAvailable,
            'progressMax' => $this ->progressMax,
            'progressCurrent' => $this ->progressCurrent,
            'progressMessage' => $this ->progressMessage,
            'errorMessage' => $this ->errorMessage,
            'executorParams' => $this ->executorParams,
            'caption' => $this ->caption
        ];
    }

    public static function createFromArray($array, $id)
    {
        $result = new static();
        $result ->id = $id;
        $result ->isRunning = $array['isRunning'];
        $result ->abortAvailable = $array['abortAvailable'];
        $result ->progressMax = $array['progressMax'];
        $result ->progressCurrent = $array['progressCurrent'];
        $result ->progressMessage = $array['progressMessage'];
        $result ->errorMessage = $array['errorMessage'];
        $result ->executorParams = $array['executorParams'] ?: [];
        $result ->caption = $array['caption'];

        return $result;
    }
}



interface TaskExecutor
{
    function getTaskStatus(TaskManagerTask $task);
}



class TaskManager
{
    private static $instance;
    private static $lastId = 0;

    /** @var TaskManagerTask[] */
    public $tasks;

    private function __construct()
    {
        $this ->tasks = [];
    }

    public static function getInststance()
    {
        if (!self::$instance) {
            self::$instance = new static();
            self::$instance ->loadFromSession();
        }

        return self::$instance;
    }


    private function storeToSession()
    {
        $storeArray = [
            'lastId' => self::$lastId
        ];

        foreach ($this ->tasks as $task) {
            $storeArray[$task ->id] = $task ->storeToArray();
        }

        $_SESSION['ronportal.taskManager'] = $storeArray;

        return $this;
    }


    private function loadFromSession()
    {
        $array = $_SESSION['ronportal.taskManager'];

        if ($array) {
            foreach ($array as $key => $taskInfo) {
                if ($key == 'lastId') {
                    self::$lastId = $taskInfo;
                } else {
                    $task = TaskManagerTask::createFromArray($taskInfo, $key);
                    $this->tasks[$key] = $task;
                }
            }
        }

        return $this;
    }


    /**
     * @param TaskManagerTask $task
     * @return $this
     */
    public function addTask($task)
    {
        self::$lastId++;
        $task ->id = sprintf('%x-%04x', time(), self::$lastId);

        $this ->tasks = [$task ->id => $task] + $this->tasks;
        $this ->storeToSession();

        return $this;
    }


    /**
     * @param TaskManagerTask $task
     */
    public function updateTask(/** @noinspection PhpUnusedParameterInspection */$task)
    {
        if ($task  &&  $task ->isRunning) {
            require_once 'adsapp/agentmanager.php';
            $manager = new AgentManager();
            $manager->getTaskStatus($task);

            $task ->executorParams['updated'] = time();
        }

        $this ->storeToSession();
    }


    public function findTask($taskId)
    {
        return $this ->tasks[$taskId];
    }
}
