<?php
/**
 * UTF8
 * stanislav.elbl, 29.09.2017
 */

namespace Ron\RonPortal;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\JsonController;
use Ron\MVC\ViewObject;
use Ron\RonPortal\XML\RonPortalXMLForm;
use Ron\RonPortal\XML\XMLFormElement;
use Ron\StringUtils;
use Ron\TextManager;

/*
 * Navrh XML:
 * <dashboard defaultFrames="1" loadXMLFrames="1">
 *   <dashboardframe id="..." caption="..." xmlid="..."  xml="...nazev xml...">
 *      <sql>... sql dotaz ...</sql>
 *      <content>... html, napr. frame ...</content>
 *   </dashboardframe>
 * </dashboard>
 *
 * id muze byt bud defaultni id, nebo uzivatelske pro SQL a XML frame
 */

class DashboardFrame
{
    public $id;
    public $url;
    public $caption;
    // size?

    public $width;
    public $height;
    public $cols;
    public $hidden;

    /** @var string  Seznam prav - oddeleno strednikem */
    public $prava;
    /** @var string  URL prokliku */
    public $linkUrl;

    /** @var string  SQL */
    public $sql;

    public $xmlId;
}


/**
 * Seznam panelu, ktere muze uzivatel videt
 * Class DashboardFrames
 * @package Ron\RonPortal
 */
class DashboardFrames
{
    /** @var DashboardFrame[] */
    public $frames = [];

    public function addFrame($id, $dataUrl, $caption)
    {
        $frame = new DashboardFrame();
        $frame ->id = $id;
        $frame ->url = $dataUrl;
        $frame ->caption = $caption;

        $this ->frames[] = $frame;
    }


    public function addSQLFrame($id, $sql, $prava, $caption)
    {
        if ($prava) {
            if (!Prava::jsouNejakePrava($prava)) return;
        }

        $frame = new DashboardFrame();
        $frame ->id = $id;
        $frame ->sql = $sql;
        $frame ->caption = $caption;
        $frame ->prava = $prava;
        $frame ->url = '?menuid=dashboard&action=loaddata&id=' . $id;

        $this ->frames[] = $frame;
    }


    public function addXmlFrame($xmlId, $caption, $prava)
    {
        if ($prava) {
            if (!Prava::jsouNejakePrava($prava)) return;
        }

        $frame = new DashboardFrame();
        $frame ->id = 'xml_' . $xmlId;
        $frame ->xmlId = $xmlId;
        $frame ->caption = $caption;
        $frame ->prava = $prava;
        $frame ->url = '?menuid=dashboard&action=loaddata&xmlid=' . $xmlId;

        $this ->frames[] = $frame;
    }



    public function findFrame($id)
    {
        foreach ($this ->frames as $frame) {
            if ($frame ->id == $id) return $frame;
        }

        return null;
    }


    public function findXmlFrame($xmlId)
    {
        if (!$xmlId) return null;

        foreach ($this ->frames as $frame) {
            if ($frame ->xmlId == $xmlId) return $frame;
        }

        return null;
    }


    public function sort($orderStr)
    {
        if (!$orderStr) return;

        // sestavi si asociativni pole  id=>poradi
        // nejdrive panely, u kterych ma explicitne nastavene poradi ...
        $orderArray = explode(';', $orderStr);
        $indexArray = [];
        $index = 1;
        foreach($orderArray as $frameId) {
            $indexArray[$frameId] = $index;
            $index++;
        }

        // ... a doplni ostatni panely (napr. nove pridane)
        foreach($this ->frames as $frame) {
            if (!isset($indexArray[$frame ->id])) {
                $indexArray[$frame ->id] = $index;
                $index ++;
            }
        }

        // a seradi panely
        usort($this ->frames, function($item1, $item2) use($indexArray) {
            if ($indexArray[$item1 ->id] == $indexArray[$item2 ->id]) return 0;
            if ($indexArray[$item1 ->id] < $indexArray[$item2 ->id]) return -1;
            return 1;
        });
    }


    public function loadSettings($settingsStr)
    {
        // zpracuje nastaveni panelu - v retezci oddelene strednikem, pricemz nastaveni kazdeho panelu je ve formatu
        // id:velikost    TODO:{color}
        // velikost muze byt def(=default), off(=vypnuty), nebo cols x height

        if (!$settingsStr) return;
        $pomA = explode(';', $settingsStr);

        foreach ($pomA as $frameSettingsStr) {
            $frameSettingsArray = explode(':', $frameSettingsStr);
            $frame = $this ->findFrame($frameSettingsArray[0]);
            if ($frame) {
                if (isset($frameSettingsArray[1])) {
                    // velikost/off
                    if ($frameSettingsArray[1] == 'off') {
                        $frame->hidden = true;
                    } else if ($frameSettingsArray[1] == 'def') {
                        // defaultni velikost - nedela nic
                    } else if ($frameSettingsArray[1]) {
                        list($cols, $height) = explode('x', $frameSettingsArray[1]);
                        if ((int)$cols && (int)$height) {
                            $frame->cols = (int)$cols;
                            $frame->height = (int)$height;

                            if ($frame ->height < 1) $frame ->height = 1;
                            if ($frame ->cols > 12) $frame ->cols =  (int)(12 * $frame ->cols / 1400);
                            if ($frame ->cols > 12) $frame ->cols = 12;
                            if ($frame ->cols < 1) $frame ->cols = 6;
                        }
                    }
                }
            }
        }
    }
}



class DashboardView implements ViewObject
{
    /** @var Database */
    public $database;

    /** @var DashboardFrames */
    public $frames;


    /**
     * DashboardView constructor.
     * @param Database $database
     * @param DashboardFrames $frames
     */
    public function __construct(Database $database, $frames)
    {
        $this->database = $database;
        $this ->frames = $frames;
    }


    public function render()
    {
        echo '<div class="dashboard" id="dashboard">';

        $hiddenFrameExists = false;

        foreach ($this->frames ->frames as $frame) {
            if (!Prava::existujePravo('dashboard_' . $frame ->id) || Prava::jsouNejakePrava('dashboard_' . $frame ->id)) {
                if (!$frame->hidden) {
                    $style = '';

                    if ($frame->height > 0) {
                        //$style = 'width:' . (int)$frame ->width . 'px; height:' . (int)$frame ->height . 'px';
                        $style = 'height:' . (int)$frame->height . 'px';
                    }

                    $styleStr = $style ? 'style="' . $style . '"' : '';
                    $cols = ($frame->cols <= 12 && (int)$frame->cols > 0) ? (int)$frame->cols : RonPortalSettings::getIntValue('default_dashboard_width', 6);

                    echo
                        "<div class='dashboard-item col{$cols}' $styleStr data-dashboard-id='{$frame->id}'>" .
                        "<div class='dashboard-frame'>" .
                        '  <div class="dashboard-content">' .
                        '  <div class="dashboard-caption">' .
                        '<span class="dashboardbuttons">' .
                          '<div class="button-loading loading-indicator margin-right fa fa-spin glyph-spinner"></div>' .
                          '<div class="dashboardbutton refresh fa glyph-refresh"></div>' .
                          '<div class="dashboardbutton close fa glyph-close"></div>' .
                        '</span>' .
                        "<span class='caption'>{$frame->caption}</span>" .
                        '  </div>' .
                        '  <div class="dashboard-data" data-data-url="' . StringUtils::htmlTextConvert($frame->url) . '"></div>' .
                        '  </div>' .
                        '</div>' .
                        '</div>';
                } else {
                    $hiddenFrameExists = true;
                }
            }
        }

        echo '</div>';

        if ($hiddenFrameExists) {
            echo '<div id="addDashboardFrame" class="js">' . \Ron\ronText('Pridat_panel') . '</div>';

            echo '<div id="addFrameDialog" style="display:none" title="' . \Ron\ronText('Pridat_panel') . '">';
            foreach ($this->frames ->frames as $frame) {
                if ($frame->hidden) {
                    if (!Prava::existujePravo('dashboard_' . $frame ->id) || Prava::jsouNejakePrava('dashboard_' . $frame ->id)) {
                        echo "<div><input type='checkbox' class='addFrameCheckbox' data-dashboard-id='{$frame ->id}'/>" . StringUtils::htmlTextConvert($frame->caption) . '</div>';
                    }
                }
            }

            echo '<div class="buttons align-center"><input type="button" class="button" value="' . \Ron\ronText('OK') . '" id="addFrameOK"/></div>';

            echo '</div>';
        }
    }
}


class DashboardController extends JsonController
{
    /** @var Database */
    var $database;


    /**
     * @param Database $database
     */
    public function __construct($database)
    {
        $this ->database = $database;
    }


    /**
     * Doplni preddefinovane panely podle z adresare dashboard
     * @param DashboardFrames $dashboardFrames
     * @throws \Exception
     */
    private function addDefaultFrames($dashboardFrames)
    {
        $dirName = 'dashboard';
        $dh = opendir($dirName);

        $fileList = [];
        while (false !== ($fileName = readdir($dh)))
            if (!is_dir($dirName . '/' . $fileName)) $fileList[] = $fileName;

        foreach ($fileList as $fileName) {
            /** @noinspection PhpIncludeInspection */
            $newFrame = require($dirName . '/' . $fileName);

            if ($newFrame) {
                if (!$newFrame instanceof DashboardFrame) {
                    throw new \Exception("Chyba - neplatny soubor ve slozce dashboard ($fileName)!");
                }

                $dashboardFrames->frames[] = $newFrame;
            }
        }
    }


    /**
     * Doplni XML panely DASHBOARD_....
     * @param DashboardFrames $dashboardFrames
     * @throws DBException
     */
    private function addXmlFrames($dashboardFrames)
    {
        $sql =
            'SELECT Id, Popis, TypPrava FROM SablonaXMLFormulare ' .
            " WHERE Popis LIKE 'DASHBOARD%'" .
            ' ORDER BY Popis';

        $cursor = $this->database->getSqlResult($sql, null, 'Chyba pri nacitani XML formularu!');

        while ($row = $cursor->fetch()) {
            $dashboardFrames ->addXmlFrame($row[0], trim(substr($row[1], 9)), $row[2]);
        }

        $cursor ->close();
    }


    /**
     * @param bool $sort    Priznak, jestli se maji panely seradit podle nastaveni uzivatele
     * @return DashboardFrames
     */
    function initDashboards($sort = false)
    {
        $result = new DashboardFrames();

        $this ->addDefaultFrames($result);

        try {
            $this->addXmlFrames($result);
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

//        $result  ->addSQLFrame('puvodni',
//            'SELECT DatumCasOperace, KodOperace, Terminal' .
//            ' FROM PuvodniDochazka, Obdobi' .
//            " WHERE Obdobi.Aplikace = 'ADS' AND Obdobi.PocatekObdobi <= GETDATE() AND Obdobi.KonecObdobi >= GETDATE()" .
//            ' AND DatumCasOperace >= Obdobi.PocatekObdobi  AND  DatumCasOperace <= Obdobi.KonecObdobi' .
//            ' AND RC = {$RC}' .
//            ' ORDER BY DatumCasOperace DESC',
//            'prohlizeni',
//            'TEST - moje dochazka'
//        );

        // seradi panely podle uzivatele
        if ($sort) {
            require_once('adsapp/usersettings.php');
            $userOrderStr = NastaveniUzivatele::getUserSettingsItem($this ->database, 'dashboardframes');
            if ($userOrderStr) $result ->sort($userOrderStr);

            $userSettingsStr = NastaveniUzivatele::getUserSettingsItem($this ->database, 'dashboardsettings');
            if ($userSettingsStr) $result ->loadSettings($userSettingsStr);
        }

        return $result;
    }


    /**
     * @param DashboardFrame $frame
     * @return string
     */
    function loadSql($frame)
    {
        if ($frame ->prava) {
            if (!Prava::jsouNejakePrava($frame ->prava)) return \Ron\ronText('Nemate_pravo');
        }

        $sql = $frame ->sql;
        $sql = str_replace('{$RC}', "'" . $this ->database ->addSlashes(User::getRC()) . "'", $sql);

        if (!$sql) return 'Chyba pri nacitani dat - neni zadan SQL dotaz'; // nemelo by nastat
        $cursor = $this ->database ->getSqlResult($sql);
        $class = 'rowOver';
        if ($frame ->linkUrl) $class .= ' clickHref clickLink';
        $result = "<table class='$class'>";
        while ($row = $cursor ->fetch()) {
            $result .= '<tr>';
            foreach ($row as $col) {
                $result .= '<td>' . StringUtils::htmlTextConvert($col) . '</td>';
            }
            $result .= '</tr>';
        }
        $cursor ->close();
        return $result;
    }


    function getXmlHtml($xmlId)
    {
        require_once('xml/xmlformportal.php');
        $form = new RonPortalXMLForm($this ->database);
        $form -> nactiXMLFormular($xmlId, true); // zeSablony = true

        // kontrola prav
        if ($form ->typPrava) {
            if (!Prava::jsouNejakePrava($form ->typPrava)) {
                return \Ron\ronText('Nemate_pravo');
            }
        }

        $form ->loadXML();

        if ($form ->mode != 'show') {
            throw new \Exception('Mod XML formulare musi byt "show"!');
        }

        if ($form ->passwordProtected  &&  !RonPortalSession::enablePasswordProtected()) {
            return 'Chraneno heslem ...';
        }

        return $form ->getHtmlOutput();
    }


    function getHtml()
    {
        $frames = $this ->initDashboards();
        if (isset($_GET['id'])) {
            // sql
            $frame = $frames->findFrame($_GET['id']);
            if (!$frame) return StringUtils::fromUTF8('<div>Chyba - neplatné ID</div>');

            if ($frame ->sql) return $this ->loadSql($frame);
        } else if (isset($_GET['xmlid'])) {
            // xml
            $frame = $frames->findXmlFrame($_GET['xmlid']);
            if (!$frame) return StringUtils::fromUTF8('<div>Chyba - neplatné xmlID</div>');

            return $this ->getXmlHtml($frame ->xmlId);
        } else {
            // chyba
            return '<div>Chyba - neni zadano id panelu</div>';
        }

        return 'Spatne nakonfigurovany frame - napriklad neni zadan SQL dotaz'; // spatne nakonfigurovany frame
    }


    /**
     * Uzivatel prehodil poradi panelu - ulozi
     * @param $newOrder
     * @throws \Exception
     */
    function sortFrames($newOrder)
    {
        require_once('adsapp/usersettings.php');
        NastaveniUzivatele::setUserSettingsItem($this ->database, 'dashboardframes', $newOrder);
    }


    private function setFrameSize($frameId, $newSizeStr)
    {
        require_once('adsapp/usersettings.php');
        $old = NastaveniUzivatele::getUserSettingsItem($this ->database, 'dashboardsettings');
        $setArray = explode(';', $old);

        $found = false;
        $new = '';
        foreach ($setArray as $frameSet) {
            list($id, $sizeStr, $color) = explode(':', $frameSet);

            if ($id == $frameId) {
                $found = true;
                $new .= ($new ? ';' : '') . $id . ':' . $newSizeStr . ($color ?: '');
            } else {
                $new .= ($new ? ';' : '') . $frameSet;
            }
        }

        if (!$found) {
            $new .= ($new ? ';' : '') . $frameId . ':' . $newSizeStr;
        }

        NastaveniUzivatele::setUserSettingsItem($this ->database, 'dashboardsettings', $new);
    }


    function resizeFrame()
    {
        //$width = (int)$_GET['width'];
        $height = (int)$_GET['height'];
        $cols = (int)$_GET['cols'];

        if ($cols > 0  &&  $height > 0) {
            $this->setFrameSize($_GET['id'], $cols . 'x' . $height);
        }
    }


    function hideFrame($frameId)
    {
        if (!$frameId) return;

        $this->setFrameSize($frameId, 'off');

        require_once('ron/stringutils.php');
        $visibleFrames = NastaveniUzivatele::getUserSettingsItem($this ->database, 'dashboardframes');
        NastaveniUzivatele::setUserSettingsItem($this ->database, 'dashboardframes', StringUtils::removeSubstring(';', $visibleFrames, $frameId));
    }


    function showFrame($frameId)
    {
        if (!$frameId) return;

        $this->setFrameSize($frameId, 'def');

//        require_once('ron/stringutils.php');
//        $visibleFrames = NastaveniUzivatele::getUserSettingsItem($this ->database, 'dashboardframes');
//        NastaveniUzivatele::setUserSettingsItem($this ->database, 'dashboardframes', StringUtils::removeSubstring(';', $visibleFrames, $frameId));
    }


    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'loaddata' :
                $output ->addText('html', $this ->getHtml());
                break;

            case 'sort' :
                $this ->sortFrames($_GET['order']);
                break;

            case 'resize' :
                $this ->resizeFrame();
                break;

            case 'hide' :
                $this ->hideFrame($_GET['id']);
                break;

            case 'show' :
                $this ->showFrame($_GET['id']);
                break;
        }
    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dashboard.ini');

        Prava::nastavObdobiProPrava(time());
        Prava::nactiPravaUzivatele($this ->database, User::getRC());

        if (!Prava::jsouNejakePrava('dashboard')) {
            throw new NeniPravoException("Dashboard neni povolen v pravech uzivatele!");
        }

        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'translations' :
                    return $this ->translations();

                default :
                    return parent::handleAction($action);
            }
        }

        HtmlHeaders::getInstance() ->addScript('script/dashboard.js');
        HtmlHeaders::getInstance() ->addScript('', '?menuid=dashboard&action=translations&ver=' . HtmlHeaders::getInstance() ->versionId, 'translations');

        // div pro kazdou agendu
        $result = new DashboardView($this ->database, $this ->initDashboards(true));

        return $result;
    }


}

return new DashboardController($this ->database);
