<?php
/**
 * UTF8
 * stanislav.elbl, 27.03.2016
 */

namespace Ron\RonPortal;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\HttpController;
use Ron\MVC\Redirect;
use Ron\RonPortal\XML\RonPortalXMLForm;
use Ron\TextManager;


class EditaceDochazkyController extends HttpController
{
    /** @var Database */
    public $database;

    /**
     * EditaceDochazkyController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    private function insert()
    {
        require_once('editace/f_editacedochazky.php');
        $f = new EditaceDochazkyFormular($this ->database, 'new');
        try {
            if ($f ->insert()) {
                if (RonPortalXMLForm::$eventRedirect) {
                    return RonPortalXMLForm::$eventRedirect;
                }
                return new Redirect(null, '?menuid=zpracovanadochazka&idpp=' . $f ->zaznam ->f_pp ->idPP . '&datum=' . DateUtils::dateToUrlString($f->zaznam->fDatumCasPocatku ->value));
            };
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
        }

        return $f;
    }


    private function novyZaznam()
    {
        if (isset($_POST['zapsatZaznam'])  &&  $_POST['zapsatZaznam']) {
            return $this ->insert();
        }

        require_once('editace/f_editacedochazky.php');
        $f = new EditaceDochazkyFormular($this ->database, 'new');
        $f->nactiDataProVlozeni();
        return $f;
    }


    private function save()
    {
        require_once('editace/f_editacedochazky.php');
        $f = new EditaceDochazkyFormular($this ->database);

        try {
            if ($f->save()) {
                require_once('editace/b_zpracovana.php');
                require_once('xml/xmlformportal.php');

                if (RonPortalXMLForm::$eventRedirect) {
                    return RonPortalXMLForm::$eventRedirect;
                }
                return new Redirect(null, '?menuid=zpracovanadochazka&idpp=' . $f->zaznam->f_pp->idPP . '&datum=' . DateUtils::dateToUrlString($f->zaznam->fDatumCasPocatku->value));
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
        }

        return $f;
    }


    private function zobrazEditaciZaznamu()
    {
        if (isset($_POST['zapsatZaznam'])  &&  $_POST['zapsatZaznam']) {
            return $this ->save();
        }

        require_once('editace/f_editacedochazky.php');
        $f = new EditaceDochazkyFormular($this ->database, 'ulozzaznam');
        $f ->nactiData((int)$_GET['idpp'], $_GET['datumcas']);
        return $f;
    }


    private function zrusitZaznam()
    {
        require_once('editace/f_editacedochazky.php');

        try {
            $f = new EditaceDochazkyFormular($this->database);
            $f->delete((int)$_GET['idpp'], $_GET['datumcas']);

            if (RonPortalXMLForm::$eventRedirect) {
                return RonPortalXMLForm::$eventRedirect;
            }

            return new Redirect(null, '?menuid=zpracovanadochazka&idpp=' . $f ->zaznam ->f_pp ->idPP . '&datum=' . DateUtils::startOfTheDay($_GET['datumcas']));
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());

            return new Redirect(null, '?menuid=editacedochazky&idpp=' . $_GET['idpp'] . '&datumcas=' . $_GET['datumcas']);
        }

    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini');

        HtmlHeaders::getInstance()
            ->addCss('css/dochazka.css')
            ->setTitle(\Ron\ronText('RON_Portal') . ' - ' . \Ron\ronText('Editace_dochazky'))
            ->addScript('', '?menuid=editace&action=translations&lang=' . TextManager::getInstance() ->translationLang . '&ver=' . HtmlHeaders::getInstance() ->versionId, 'translations')
            ->addScript('script/dochazka.js');

        $action = isset($_POST['action']) ? $_POST['action'] : (isset($_GET['action']) ? $_GET['action'] : '');


        switch ($action) {
            case 'novyzaznam' :
            case 'new' :
                return $this ->novyZaznam();

            case 'zrusitzaznam' :
                return $this ->zrusitZaznam();

            default :
                return $this ->zobrazEditaciZaznamu();
        }
    }

}

return new EditaceDochazkyController($this ->database);
