<?php
/** pavel.rudol 9.4.2020 */

namespace Ron\RonPortal\Sklad;


use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\RonPortal\DokumentyFirmyBrowser;
use Ron\RonPortal\ReportOutputObject;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\TypRozdeleni;
use Ron\TextManager;

class FirmaController extends HttpController
{
    /** @var Database */
    private $database;

    public function __construct($database)
    {
        $this->database = $database;
    }

    private function detailFirmy($kodFirmy)
    {
        require_once('sklad/firma.php');

        $firma = Firma::nactiFirmu($this->database, $kodFirmy);

        Model::addData('firma', $firma);
        Model::addData('database', $this->database);

        return new BaseViewObject('sklad/firma_detail.php');
    }

    private function browserFirmy($action)
    {
        $typRozdeleni = $this->vratTypRozdeleniProFiltr();
        require_once('adsapp/rozdeleni.php');
        if ($typRozdeleni)
            $pomTyp = TypRozdeleni::getTypRozdeleni($this->database, $typRozdeleni);

        $indexFiltr = RonPortalSettings::getIntValue('dodavatele_filtrovat_dle_uzivatelska');
        require_once('sklad/b_firma.php');
        $b = new FirmaBrowser($this->database, $action, $indexFiltr, $pomTyp);
        $b->load();
        return $b;
    }

    private function report()
    {
        require_once('adsapp/reportserver.php');

        $result = new ReportOutputObject();
        $result->loadReportFromGet($this->database, 'Firma', false);

        $result->integerParams['TypRozdeleniFirmy'] = $this->vratTypRozdeleniProFiltr();
        $result->integerParams['Aktivni'] = $_GET['aktivni'];
        $result->stringParams =
            ['RozdeleniFirmy' => $_GET['rozdeleni'],
                'Mesto' => $_GET['mesto'],
                'KlicovaSlova' => $_GET['klicova-slova']];

        return $result;
    }

    private function vratTypRozdeleniProFiltr()
    {
        $params = $this->database->createParameters();

        switch (RonPortalSettings::getIntValue('vydanaobjednavka_dodavatel_dle_rozdeleni')) {
            case 1 :
                $typRozdeleniName = 'ObjednavkaVydana_TypRozdeleni1';
                break;
            case 2 :
                $typRozdeleniName = 'ObjednavkaVydana_TypRozdeleni2';
                break;
            case 3 :
                $typRozdeleniName = 'ObjednavkaVydana_TypRozdeleni3';
                break;
        }

        if (isset($typRozdeleniName)) {
            $sql =
                "SELECT Name, Val FROM SYS_VAR_GLOBAL WHERE Name = " . $params->addParameter($typRozdeleniName);

            $cursor = $this->database->getSqlResult($sql, $params);

            if ($row = $cursor->fetch()) {
                return $row[1];
            }
        }
        return 0;
    }

    public function handleAction($action)
    {
        TextManager::getInstance()->loadStrings('sklad.ini');

        HtmlHeaders::getInstance()
            ->addCss('css/sklad.css');

        switch ($_GET['action']) {
            case 'download' : // download dokumentu
                require_once('sklad/firma_dokumenty.php');
                return DokumentyFirmyBrowser::downloadDokument($this ->database, $_GET['firma'], (int)$_GET['record']);

            case 'report' :
                return $this->report();

            case 'detailfirmy' :
                return $this->detailFirmy($_GET['record']);

            case 'seznamfirem' :
            default :
                return $this->browserFirmy($action);
        }
    }

}

return new FirmaController($this->database);
