<?php
/**
 * UTF8
 * stanislav.elbl, 31.07.2015
 */

namespace Ron\RonPortal;

/** @var \Ron\Http\App $this */

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MVC;
use Ron\MVC\OutputObject;
use Ron\MVC\ViewObject;
use Ron\TextManager;


class VzkazyController implements MVC\Controller
{

    /** @var Database */
    var $database;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        $this->database = $database;
    }


    function seznamVzkazu()
    {
        require_once('adsapp/b_vzkazy.php');
        $result = new VzkazBrowser($this ->database);
        $result ->load();
        return $result;
    }


    function jePravoNaVzkaz($id, $pouzeProMe)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT RCOdesilatele, RCPrijemce, KodTypuRozdeleni, Rozdeleni, KodPrijemce' .
            ' FROM Vzkaz' .
            ' WHERE Id = '.$params ->addParameter((int)$id);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) return false;

        if ( $row[1] == User::getRC() ) return true; // je to vzkaz pro me
        if ($pouzeProMe) return false;

        if ( $row[0] == User::getRC() ) return true; // je to muj vzkaz

        if ( $row[2]  &&  $row[3] ) {
            // kontrola, jestli je to pro "moje" rozdeleni
            return Rozdeleni::jeVRozdeleni($this ->database,
                User::getRC(), null,     // kontroluje jen RC
                $row[2], $row[3], false, // $vcetnenadrazenych = false ? ?
                DateUtils::startOfTheDay(time()), DateUtils::startOfTheDay(time()) );
        }

        if ( !isset($row[0])   &&   !isset($row[1])  &&  !isset($row[2])  &&  !isset($row[3])   &&  !isset($row[4]) ) return true; // vzkaz pro vsechny

        return false;
    }


    function vyjadreni() {
        $id = isset($_POST['Id']) ? (int)$_POST['Id'] : (int)$_GET['record'];

        if (!$this ->jePravoNaVzkaz($id, true)) {
            RonPortalUtils::neniPravoException();
            //return errormsgNeniPravo();
        }

        require_once('adsapp/f_vyjadreni.php');

        $f = new VyjadreniFormular($this ->database);
        //if (isset($_POST['formaction'])) {
            if ($f ->handleAction()) {
                return new MVC\Redirect(null, '?menuid=vzkazy');
            }
        //}

        return $f;
    }


    function downloadDokument()
    {
        $idVzkazu = (int)$_GET['idvzkazu'];
        $idDokumentu = (int)$_GET['iddokumentu'];

        if (!$this ->jePravoNaVzkaz($idVzkazu, false)) {
            RonPortalUtils::neniPravoException();
        }

        $platnost = null;
        if ($_GET['platnostodtimestamp']) $platnost = $_GET['platnostodtimestamp'];
        else if ($_GET['platnostod']) $platnost = DateUtils::dateStringToTimestamp($_GET['platnostod']);

        if (!$platnost) {
            throw new \Exception('Chyba - neni zadana platnost pozadovaneho dokumentu!');
        }

        // kontrola jestli je zadany dokument pripojeny k zadanemu vzkazu
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT IdDokumentu FROM DokumentVzkazu ' .
            ' WHERE IdVzkazu = ' . $params ->addParameter((int)$idVzkazu) .
            '   AND IdDokumentu = ' . $params ->addParameter((int)$idDokumentu) .
            '   AND PlatnostOd = ' . $params ->addParameter($this ->database ->dateToParam($platnost));

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new \Exception('Chyba - pozadovany dokument neni pripojen k zadanemu vzkazu!');
        }

        require_once('adsapp/dokumentyutils.php');
        return new DokumentOutputObject($this ->database, $idDokumentu, true, $platnost);
    }


    /**
     * @param string $action
     * @return ViewObject|OutputObject
     * @throws \Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('vzkazy.ini');

        $action = isset($_GET['action']) ? $_GET['action'] : '';

        if ($action == 'download') {
            // pri downloadu nekontroluje, jestli jsou vzkazy zapnute (pouziva se to i v xml)
            return $this ->downloadDokument();
        }

        if (!RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_VZKAZY)) {
            throw new \Exception('Tato funkce neni povolena!');
        }

        switch ($action) {
            case 'novyvzkaz' :
                require_once('adsapp/f_vzkazy.php');
                return new VzkazyFormular($this ->database);

            case 'odeslatvzkaz' :
                require_once('adsapp/f_vzkazy.php');
                $f = new VzkazyFormular($this ->database);
                if (!$f ->handleAction()) return $f;
                return new MVC\Redirect(null, '?menuid=vzkazy');

            case 'vyjadreni' :
                return $this ->vyjadreni();

            default :
                return $this ->seznamVzkazu();
        }
    }


}


return new VzkazyController($this ->database);
