<?php
/**
 * UTF8
 * stanislav.elbl, 02.11.2015
 */

namespace Ron\RonPortal;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\Http\HtmlHeaders;
use Ron\IcalOutputObject;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;


class AbsenceController extends JsonController
{
    const SETTINGS_CASOVE_SLOZKY = 'casovaslozkaabsence'; // ktere casove slozky se maji zobrazit v planovani absenci
    const SETTINGS_CASOVE_SLOZKY_NADPISY = 'nadpisslozkaabsence';
    const SETTINGS_DOPORUCOVANI = 'doporucovaniabsence';

    /** @var Database */
    var $database;


    /**
     * AbsenceController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    public static function jePravoNaAbsenci($idpp)
    {
        $prava = Prava::vratPrava('absence');
        if ($prava->lzeZobrazitPP($idpp)) {
            return true;
        }

        $prava = Prava::vratPrava('absence_schval');
        if ($prava->lzeZobrazitPP($idpp)) {
            return true;
        }

        if (RonPortalSettings::getIntValue(self::SETTINGS_DOPORUCOVANI)) {
            $prava = Prava::vratPrava('absence_doporuc');
            if ($prava->lzeZobrazitPP($idpp)) {
                return true;
            }
        }

        $prava = Prava::vratPrava('absence_prehled');
        if ($prava->lzeZobrazitPP($idpp)) {
            return true;
        }

        return false;
    }


    public static function schvalovaniZapnuto()
    {
        return RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI);
    }


    function seznamAbsenci()
    {
        require_once('dochazka/b_absence.php');
        $b = new AbsenceBrowser($this ->database);
        $b ->load();

        return $b;
    }


    /**
     * Nacte absenci podle GET nebo POST
     * @param int    $idPP
     * @param string $datum
     * @param int    $poradi
     * @return Absence
     * @throws \Exception
     */
    function nactiAbsenci($idPP = null, $datum = null, $poradi = null)
    {
        if ($idPP) {
            $datumOd = DateUtils::dateStringToTimestamp($datum);
        } else {
            $datumOd = isset($_GET['datum'])
                ? DateUtils::dateStringToTimestamp($_GET['datum'])
                : (int)$_GET['datumod']; // stara verze s timestampem

            if (isset($_POST['osoba'])) {
                $idPP = (int)$_POST['osoba'];
            } else {
                $idPP = (int)$_GET['idpp'];

                if (!$_GET['idpp'] || !$datumOd || !isset($_GET['poradi'])) {
                    throw new \Exception('Neni zadana absence pro editaci!');
                }
            }

            $poradi = (int)$_GET['poradi'];
        }

        require_once('dochazka/absence.php');
        $absence = new Absence();
        $absence ->nactiDataZDB($this ->database, $idPP, $datumOd, $poradi);

        return $absence;
    }


    /**
     * Nacte zadanou absenci
     * @param int    $idPP
     * @param string $datum
     * @param int    $poradi
     * @return AbsenceDBRecord
     * @throws \Exception
     */
    function nactiAbsenciRecord($idPP, $datum, $poradi)
    {
        $datumOd = DateUtils::dateStringToTimestamp($datum);

        require_once 'adsapp/ppomer.php';
        $pp = PracovniPomer::nactiPP($this ->database, $idPP);

        require_once('dochazka/absencerec.php');
        $absenceRec = new AbsenceDBRecord($this ->database, $pp, 'absence', $datumOd, $poradi, $this);
        $absenceRec ->loadRecord();

        return $absenceRec;
    }


    function zobrazAbsenci()
    {
        $absence = $this ->nactiAbsenci();

        Prava::nastavObdobiProPrava($absence ->datumOd, $absence ->datumDo);
        if (!self::jePravoNaAbsenci($absence ->f_PP ->idPP)) {
            // nema pravo editovat ani schvalovat tuto absenci
            // jeste zkontroluje prava na prohlizeni udaju
            $prava = Prava::vratPrava('prohlizeni');
            if ( !$prava ->lzeZobrazitPP($absence ->f_PP ->idPP) ) {
                RonPortalUtils::neniPravoException();
            }
        }

        $prekryvaneAbsence = Absence::nactiPrekryvaneAbsence($this ->database, $absence);

        $backAction = null;
        if (isset($_GET['backaction'])) $backAction = $_GET['backaction'];
        else if (isset($_POST['backaction'])) $backAction = $_POST['backaction'];

        if ($backAction) {
            $backMenuId = StringUtils::secureString($backAction);
        } else if (self::schvalovaniZapnuto() && Prava::vratPrava('absence_schval') ->jsouNejakePrava()) {
            $backMenuId = 'schvalovaniabsence';
        } else {
            $poma = [Prava::vratPrava('planovani'), Prava::vratPrava('prohlizeni')];

            if (Prava::pravaNaViceOsob($poma)) {
                $backMenuId = 'prehledabsenci';
            } else {
                $backMenuId = 'planovani';
            }
        }

        Model::addData('absence', $absence);
        Model::addData('backMenuId', $backMenuId);

        Model::addData('prekryvaneAbsence', $prekryvaneAbsence);

        $poznamky = null;
        $typPoznamky = RonPortalSettings::getValue('absence_typpoznamky');
        if ($typPoznamky) {
            require_once('dochazka/poznamky.php');
            $typy = $typPoznamky == '*' ? null : explode(';', $typPoznamky);
            $poznamky = PoznamkaKeDni::nactiPoznamkyProOsobu($this ->database, $absence ->f_PP ->idPP, $absence ->datumOd, $absence ->datumDo, $typy);
        }
        Model::addData('poznamky', $poznamky);

        require_once('adsapp/reportutils.php');
        Model::addData('reportList', RonPortalReport::loadFormReports($this ->database, 'Absence', 'Absence')); // treti parametr z duvodu kompatibility!

        $kody = explode(';', RonPortalSettings::getValue(self::SETTINGS_CASOVE_SLOZKY) );
        $nadpisy = explode(';', RonPortalSettings::getValue(self::SETTINGS_CASOVE_SLOZKY_NADPISY));

        $casoveSlozky = [];
        foreach ($kody as $index => $kod) {
            $casoveSlozky[] = [
                'kod' => $kod,
                'nadpis' => $nadpisy[$index],
                'hodnota' => self::vratMesicniVysledekProAbsenci($this ->database, $absence ->f_PP ->idPP, $kod)
            ];
        }
        Model::addData('casoveSlozky', $casoveSlozky);


        return new BaseViewObject('dochazka/absence_view.php');
    }


    function redirect($idPP, $datum, $poradi)
    {
        if (isset($_POST['backaction']) && ($_POST['backaction'] == 'monitorovani'))
            return new Redirect(null, '?menuid=monitorovani');

        return new Redirect(null,
            '?menuid=absence&action=zobrazabsenci&idpp=' . (int)$idPP .
            '&datum=' . date('j.n.Y', $datum) .
            '&poradi=' . $poradi .
            (isset($_POST['backaction']) ? '&backaction=' . StringUtils::secureString($_POST['backaction']) : '')
        );
    }


    function form()
    {
        require_once('dochazka/f_absence.php');
        $form = new AbsenceFormular($this ->database);
        //$form ->loadRecord();
        if ($form ->handleAction()) {
            // zapis OK, redirect na detail absence
            return $this ->redirect($form ->idPP, $form ->datumOd, $form ->poradi);
        } else {
            return $form;
        }
    }


    function planovaniAbsence()
    {
        $form = $this ->form();
        Model::addViewObject($form);

        $b = $this ->seznamAbsenci();
        Model::addViewObject($b);

        return 'view/appview.php';
    }


    function iCal()
    {
        require_once('dochazka/absence_ical.php');

        Prava::nastavObdobiProPrava(time(), time());

        $idPP = (int)$_GET['idpp'];
        $datum = DateUtils::dateStringToTimestamp($_GET['datum']);
        $poradi = (int)$_GET['poradi'];

        if (!self::jePravoNaAbsenci($idPP)) {
            RonPortalUtils::neniPravoException();
        }

        $cal = AbsenceCalendarItem::loadEvent($this ->database, $idPP, $datum, $poradi);
        return new IcalOutputObject($cal);
    }


    public static $dochazkaZpracovanaDo = null;

    /**
     * Vrati hodnotu mesicniho vysledku pro zadanou absenci - vybere hodnotu pro posledni zpracovane obdobi
     * @param Database $database
     * @param int $idPP
     * @param string $casovaSlozka
     * @return bool|null|string
     */
    public static function vratMesicniVysledekProAbsenci($database, $idPP, $casovaSlozka)
    {
        if (!$casovaSlozka) return '';

        if (!self::$dochazkaZpracovanaDo) {
            require_once 'adsapp/dochazkautils.php';
            self::$dochazkaZpracovanaDo = DochazkaUtils::datumCasZpracovaniDochazky($database);
        }

        // zjisti jednotku (H/D/K) - pokud neni zadana, pouzije D
        $poma = explode(':', $casovaSlozka);
        $casovaSlozka = $poma[0];

        if (!$casovaSlozka) return '';

        $jednotka = 'D';
        if (isset($poma[1])) {
            if ($poma[1] == 'K' || $poma[1] == 'H') $jednotka = $poma[1];
        }

        //$idPP = (int)$idPP; // pro jistotu

        $mesic = DateUtils::firstDayOfMonth(time());
        if (self::$dochazkaZpracovanaDo  &&  $mesic > self::$dochazkaZpracovanaDo) {
            // dochazka pro aktualni obdobi jeste neni zpracovana (napr. prvni den v mesici)
            // - nacte data z predchoziho obdobi
            $mesic = DateUtils::incMonth($mesic, -1);
        }

        require_once('dochazka/mesicnivysledek.php');
        $mv = MesicniVysledek::vratMesicniVysledek($database, (int)$idPP, $mesic, $casovaSlozka);

        if ($mv) {
            switch ($jednotka) {
                case 'K' :
                    return DBUtils::getFormatedValue($database, 'number_2', $mv ->kalendarniDny); // (double)number_format($row[1], 2);

                case 'H' :
                    return DBUtils::getFormatedValue($database, 'hodiny', $mv ->hodiny);

                default :
                    return DBUtils::getFormatedValue($database, 'number_2', $mv ->dny); // (double)number_format($row[0], 2);
            }
        }

        return '';
    }


    /**
     * Doporuci (hodnota = 1) nebo nedoporuci (hodnota = 2) absenci
     * @param int $hodnota
     * @return bool
     * @throws
     */
    function nastavDoporuceniAbsence($hodnota)
    {
        if (!$_GET['idpp'] || !$_GET['datum'] || !$_GET['poradi']) {
            throw new \Exception('Neni zadana absence pro nastaveni doporuceni!');
        }

        $absence = $this ->nactiAbsenci((int)$_GET['idpp'], $_GET['datum'], (int)$_GET['poradi']);

//        require_once('dochazka/absence.php');
//
//        $absence = new Absence();
//        $absence ->nactiDataZDB($database, $_GET['idpp'], $datumOd, $_GET['poradi']);

        Prava::nastavObdobiProPrava($absence ->datumOd, $absence ->datumDo);

        $prava = Prava::vratPrava('absence_doporuc');
        if (!$absence ->f_PP ->lzeZobrazitPP($prava)) {
            RonPortalUtils::neniPravoException();
        }

        $absence ->nastavDoporuceni($this ->database, $hodnota);

        if (isset($_GET['list'])  &&  (int)$_GET['list']) {
            return $this ->seznamAbsenci();
        } else {
            //return new Redirect('?menuid=absence');
            return $this->redirect($absence->f_PP->idPP, $absence->datumOd, $absence->poradi);
        }
    }


    /**
     * @param AbsenceDBRecord $absence
     * @param int  $hodnota
     * @throws \Exception
     */
    function nastavSchvaleniAbsence($absence, $hodnota)
    {
        if ($absence ->fSchvaleni ->value == $hodnota) {
            return;
        }

        Prava::nastavObdobiProPrava($absence ->fDatumOd ->value, $absence ->fDatumDo ->value);

        $prava = Prava::vratPrava('absence_schval', 'absence_schval_' . $absence ->fOperace ->value );

        // TODO: osetrit situaci, kdy neni schvalovani absenci zapnuto
        if (!$absence ->pp ->lzeZobrazitPP($prava)) {
            // nemam pravo schvalovat
            if (Prava::vratPrava('absence_delete') ->lzeZobrazitPP($absence ->pp ->idPP)) {
                // mam pravo rusit schvalene absence
            } else if ($hodnota == 4) {
                // ruseni absence - musim mit pravo ji editovat
                $prava2 = Prava::vratPrava('absence', 'absence_' . $absence ->fOperace ->value);
                if (!$absence ->pp->lzeZobrazitPP($prava2)) {
                    throw new NeniPravoException;
                }

                if ($absence ->fSchvaleni ->value == 0) {
                    // absence jeste neni schvalena => mam pravo editovat tak ji muzu zrusit
                } else {
                    // jeste zkontroluje, jestli je mozne tuto operaci smazat i bez prav
                    require_once 'dochazka/operacedochazky.php';
                    $operace = OperaceDochazky::nactiOperaci($this ->database, $absence ->fOperace ->value);
                    if (!$operace ->absence_LzeZrusit) {
                        throw new NeniPravoException;
                    }
                }
            } else {
                throw new NeniPravoException();
            }
        }

        // zkontroluje obdobi - pri schvalovani se nekontroluje podle nastaveni "Pocet dnu pro zpetne planovani"
        // (pokud by to bylo potreba, tak se musi udelat samostatne nastaveni pro schvalovani)
        AbsenceDBRecord::zkontrolujObdobiAbsence($this ->database, $absence ->fDatumOd ->value, $absence ->fDatumDo ->value, $absence ->pp, false);

        $absence->nastavSchvaleni($hodnota);

        $absence->schvaleni_Email($hodnota);
    }


    /**
     * Nastavi schvaleni absence na zadanou hodnotu.
     * @param $hodnota
     * @return BaseViewObject|Redirect|AbsenceBrowser
     * @throws \Exception
     */
    function schvaleniAbsence($hodnota)
    {
        if (!$_GET['idpp'] || !$_GET['datum'] || !$_GET['poradi']) {
            throw new \Exception('Neni zadana absence pro nastaveni stavu (schvaleni, zruseni)!');
        }

        $rec = $this ->nactiAbsenciRecord((int)$_GET['idpp'], $_GET['datum'], (int)$_GET['poradi']);
        //$absence = $this ->nactiAbsenci((int)$_GET['idpp'], $_GET['datum'], (int)$_GET['poradi']);

        try {
            $this->nastavSchvaleniAbsence($rec, $hodnota);
        } catch (RonException $e) {
            MessageManager::addErrorMessage($e ->getMessage(), $e ->getFieldName(), null, $e ->getErrorDescription());
            return $this ->zobrazAbsenci();
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            return $this ->zobrazAbsenci();
        }

        if (isset($_GET['list'])  &&  (int)$_GET['list']) {
            return $this ->seznamAbsenci();
        } else {
            return $this->redirect($rec->pp->idPP, $rec->fDatumOd ->value, $rec->fPoradi ->value);
        }
    }


    public function hromadneSchvaleni()
    {
        $seznamAbsenci = explode("\n", $_POST['vybraneZaznamy']);

        foreach($seznamAbsenci as $absenceStr) {
            if ( trim($absenceStr) ) {
                $poma = explode(';', $absenceStr);

                $absence = $this ->nactiAbsenciRecord((int)$poma[0], $poma[1], (int)$poma[2]);

                try {
                    $this->nastavSchvaleniAbsence($absence, (int)$_POST['akceHromadneSchvaleni']);
                } catch (RonException $e) {
                    MessageManager::addErrorMessage($e ->getMessage(), $e ->getFieldName(), null, $e ->getErrorDescription());
                } catch (\Exception $e) {
                    MessageManager::addErrorMessage($e ->getMessage());
                }
            }
        }

        return new Redirect(null, '?menuid=schvalovaniabsence');
    }


    function zadostOZruseni()
    {
        if (!$_GET['idpp'] || !$_GET['datum'] || !$_GET['poradi']) {
            throw new \Exception('Neni zadana absence pro nastaveni stavu (schvaleni, zruseni)!');
        }

        $rec = $this ->nactiAbsenciRecord((int)$_GET['idpp'], $_GET['datum'], (int)$_GET['poradi']);
        Prava::nastavObdobiProPrava($rec ->fDatumOd ->value, $rec->fDatumDo ->value);

        $prava = Prava::vratPrava('absence', 'absence_' . $rec ->fOperace ->value); // musim mit pravo planovat absence
        if (!$rec ->pp ->lzeZobrazitPP($prava)) {
            throw new NeniPravoException();
        }

        try {
            $rec ->nastavSchvaleni(3);

            $rec->planovani_Email($this->database, 3);
        } catch (RonException $e) {
            MessageManager::addErrorMessage($e ->getMessage(), $e ->getFieldName(), null, $e ->getErrorDescription());
            return $this ->zobrazAbsenci();
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            return $this ->zobrazAbsenci();
        }

        if (isset($_GET['list'])  &&  (int)$_GET['list']) {
            return $this ->seznamAbsenci();
        } else {
            return $this->redirect($rec->pp->idPP, $rec->fDatumOd ->value, $rec->fPoradi ->value);
        }
    }


    function report()
    {
        if (!$_GET['idpp'] || !$_GET['datum'] || !$_GET['poradi']) {
            throw new \Exception('Neni zadana absence pro tisk!');
        }

        Prava::nastavObdobiProPrava(time(), time());

        $idPP = (int)$_GET['idpp'];

        if (!self::jePravoNaAbsenci($idPP)) {
            RonPortalUtils::neniPravoException();
        }

        $absence = $this ->nactiAbsenci($idPP, $_GET['datum'], (int)$_GET['poradi']);
        Prava::nastavObdobiProPrava($absence ->datumOd, $absence ->datumDo);

        require_once('adsapp/reportserver.php');

        $result = new ReportOutputObject();
        $result->loadReportFromGet($this->database, 'Absence', false, 'Absence');
        $result->nastavPP($absence ->f_PP);

        //$result->stringParams = "poradi={$absence ->f_Poradi}";
        $result->integerParams['poradi'] = $absence ->poradi;
        //$result->dateParams = 'datumOd=' . date('j.n.Y', $absence ->f_DatumOd);
        $result->dateParams['datumOd'] = date('j.n.Y', $absence ->datumOd);

        return $result;
    }



    /**
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function casoveSlozky($output)
    {
        $idpp = $_GET['idpp'];
        if (!$idpp) throw new \Exception('Chyba! Neni zadan pracovni pomer!');

        $hodnoty = [];

        $slozky = explode(';', RonPortalSettings::getValue(self::SETTINGS_CASOVE_SLOZKY));
        foreach ($slozky as $slozka) {
            $hodnoty[] = self::vratMesicniVysledekProAbsenci($this ->database, $idpp, $slozka);
        }

        $output ->addData('hodnoty', $hodnoty);
    }


    /**
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function mojeUdalosti($output)
    {
        $html = '<table class="rowOver clickHref clickLink" data-ron-href="?menuid=planovani">';

        $params = $this ->database ->createParameters();

        $sql =
            'SELECT PracovniPomer.Id, Absence.Poradi, ' .
            ' Absence.DatumOd, Absence.DatumDo, Absence.Delka, Absence.CasOd, Absence.CasDo, Absence.OperaceDochazky, Absence.Schvaleni,' .
            ' OperaceDochazky.Popis, Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul ' .
            ' FROM Absence, PracovniPomer, Osoba, OperaceDochazky ' .
            ' WHERE PracovniPomer.RC = Absence.RC ' .
            ' AND PracovniPomer.OsobniCislo = Absence.OsobniCislo ' .
            ' AND PracovniPomer.DatumNastupu = Absence.DatumNastupu ' .
            ' AND Osoba.RC = Absence.RC ' .
            ' AND OperaceDochazky.Kod = Absence.OperaceDochazky ' .
            ' AND Absence.Schvaleni <> 2 AND Absence.Schvaleni <> 4' . // nezobrazuje zamitnute a zrusene
            ' AND PracovniPomer.RC = ' . $params ->addParameter( User::getRC() ) .
            ' AND Absence.DatumOd >= ' . $params ->addParameter( $this ->database ->dateToParam(time()) );

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $datumStr = DBUtils::getFormatedValue($this ->database, 'date', $row[2]);
            if ($row[3] != $row[2]) $datumStr .= ' - ' . DBUtils::getFormatedValue($this ->database, 'date', $row[3]);

            $html .=
                '<tr>' .
                "<td>$datumStr</td>" .
                '<td>' . StringUtils::htmlTextConvert($row[9]) . '</td>' .
                '</tr>';
        }

        $html .= '</table>';

        $output ->addText('html', $html);
    }


    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'casoveslozky' :
                $this->casoveSlozky($output);
                break;

            case 'mojeudalosti' :
                $this ->mojeUdalosti($output);
                break;
        }
    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini');
        HtmlHeaders::getInstance() ->addScript('script/dochazka.js');
        HtmlHeaders::getInstance() ->addCss('css/dochazka.css');
        HtmlHeaders::getInstance() ->setTitle( \Ron\ronText('RON_Portal') . ' - ' . \Ron\ronText('Planovani_absenci'));

        if (isset($_POST['hromadneSchvaleni']) && isset($_POST['akceHromadneSchvaleni']) && $_POST['akceHromadneSchvaleni']) {
            // hromadne schvaleni absenci
            return $this ->hromadneSchvaleni();
        }

        if ($_GET['formaction']  ||  $_POST['formaction']) {
            return $this ->form();
        }

        switch ($_GET['action']) {
            // json akce
            case 'casoveslozky' :
            case 'mojeudalosti' :
                return parent::handleAction($action);

            // ostatni
            case 'zobrazabsenci' :
            case 'editujabsenci' : // pouze pro kompatibilitu - pouzivalo se v emailech!
                return $this ->zobrazAbsenci();

            case 'doporucitabsenci' :
                return $this ->nastavDoporuceniAbsence(1);

            case 'nedoporucitabsenci' :
                return $this ->nastavDoporuceniAbsence(2);

            case 'schvalitabsenci' :
                return $this ->schvaleniAbsence(1); // schvaleno

            case 'zamitnoutabsenci' :
                return $this ->schvaleniAbsence(2); // zamitnuto

            case 'zrusitabsenci' :
                return $this ->schvaleniAbsence(4); // zruseno

            case 'zadostozruseni' :
                return $this ->zadostOZruseni();

            case 'zamitnoutzruseni' :
                return $this ->schvaleniAbsence(5); // zruseno

            case 'ical' :
                return $this ->iCal();

            case 'report' :
                return $this ->report();
        }

        if ($action == 'planovani') {
            return $this ->planovaniAbsence();
        }

        return $this ->seznamAbsenci();
    }

}
