<?php

/**
 * UTF8
 */

namespace Ron\RonPortal;

use Ron\DateUtils;
use Ron\DB\Cursor;
use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\Controller;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\StringUtils;
use Ron\TextManager;

require_once 'dochazka/b_vysledkybrowser.php';
require_once 'adsapp/ronportaldbrecord.php';
require_once 'dochazka/kalendar.php';
require_once 'dochazka/schvalenidochazky.php';


class SloupecDennichVysledku
{
    /** @var string  vyraz pro vypocet */
    var $expression;
    /** @var string  ktera hodnota z DV se pouzije (H/D/K) */
    var $jednotka;
    /** @var string  casova slozka - v podstate puvodni vyraz bez podtrzitek a jednotky */
    var $casovaSlozka;
    var $soucet;

    var $editovat;

    /** @var bool priznak, ze je potreba prepocitat mesicni vysledek */
    var $prepocitat;
}



class DenniVysledkyBrowser extends VysledkyBrowser
{
    const SETTINGS_SLOUPCE = 'sloupcedennichvysledku';
    const SETTINGS_ZOBRAZIT_PUVODNI = 'zobrazitpuvodnivdennich';
    const SETTINGS_ZOBRAZIT_NULOVE = 'zobrazovatprazdnedv';

    /** @var SloupecDennichVysledku[] */
    var $sloupce;

    /** @var SchvaleniDochazky */
    var $schvaleni;

    var $pocetEditu; // pocet editovatelnych poli
    var $pocetPuvodni; // nejvetsi pocet zaznamu puvodni dochazky ve dni
    var $kalendar;

    /** @var Cursor */
    var $cursorPuvodni;

    var $lzeEditovat; // nastavi se podle prav a podle toho, jestli je obdobi uzavreno

    var $variantni; // jestli se maji zobrazovat variantni obdobi (true) nebo mesice (false)
    /** @var integer  zobrazovane obdobi nastavi podle vybraneho obdobi/variantniho obdobi */
    var $datumOd;
    /** @var integer  nastavi se v ShowFilter */
    var $datumDo;


    function __construct($database, $variantni)
    {
        parent::__construct($database, 'dennivysledky');

        $this ->code = 'browserdenni';
        $this ->logDescription = StringUtils::fromUTF8('Denní výsledky');
        $this ->cssClass = 'browser grid dennivysledky keyInputMove';
        $this ->variantni = $variantni;

        $this ->title = $variantni
            ? \Ron\ronText('Prehled_variantnich_vysledku')
            : \Ron\ronText('Denni_vysledky');

        $this ->pocetPuvodni = 0;

        $this ->kalendar = ''; // kalendar, podle ktereho se nacitaji svatky

        $this ->parseFieldsString( RonPortalSettings::getValue(self::SETTINGS_SLOUPCE) );
    }


    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action);

        if ($this ->variantni) {
            $this ->filterForm ->nastavVariantni(true);
            $this ->filterForm ->nastavAkciGet('dennivysledkyvariantni');
            //$this ->filterForm ->nastavAkci('?menuid=dennivysledkyvariantni');
        } else {
            $this ->filterForm ->nastavObdobi(true, 'ADS', false); // jenotevrene = false
            $this ->filterForm ->nastavAkciGet('dennivysledky');
            //$this ->filterForm ->nastavAkci('?menuid=dennivysledky');
        }

        $prava = Prava::vratPrava('prohlizeni');

        if ($prava ->zobrazitVyberRozdeleni()) {
            $this ->filterForm ->nastavRozdeleni(true);
            $this ->filterForm ->f_pravarozdeleni = $prava;
            $this->filterForm->vcetnePodrizenychRozdeleni = true;

            if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
                $this ->filterForm ->nastavRozdeleni2(true);
                $this->filterForm->vcetnePodrizenychRozdeleni2 = true;
            }
        }

        $this ->filterForm ->nastavOsoby(true, $prava, 0, 0); // datum od-do podle obdobi
        $this ->filterForm ->osobyFiltr = 'PracovniPomer.ZpracovatDochazku = 1';
        $this ->filterForm ->downloadFotoOsobyUrl = '?menuid=editace&action=foto&idpp=';
    }


    function renderFilter()
    {
        if ($this ->variantni) $this ->odkazyVariabilniVysledky('dennivysledkyvariantni');

        parent::renderFilter();
    }


    /**
     * vytvori pole hlavicek sloupcu a pole vyrazu pro sloupce
     * @param string $str
     */
    function parseFieldsString($str)
    {
        $this ->addField( [BROWSER_FIELD_TYPE => 'hidden', BROWSER_FIELD_SQL_NAME => ''] ); // pole pro ulozeni data

        $this ->addField( new PriznakSchvaleniField(1, [BROWSER_FIELD_CODE => 'col-schvaleni1', BROWSER_FIELD_SQL_INDEX => 1]) );
        $this ->addField( new PriznakSchvaleniField(2, [BROWSER_FIELD_CODE => 'col-schvaleni2', BROWSER_FIELD_SQL_INDEX => 2]) );
        $this ->addField( new PriznakSchvaleniField(3, [BROWSER_FIELD_CODE => 'col-schvaleni3', BROWSER_FIELD_SQL_INDEX => 3]) );

//        $this ->addField( [BROWSER_FIELD_SQL_NAME => '', BROWSER_FIELD_TYPE => 'schvaleniDV',  BROWSER_FIELD_CLASS_NAME => 'schvaleni', BROWSER_FIELD_CODE => 'col-schvaleni1'] );
//        $this ->addField( [BROWSER_FIELD_SQL_NAME => '', BROWSER_FIELD_TYPE => 'schvaleniDV',  BROWSER_FIELD_CLASS_NAME => 'schvaleni', BROWSER_FIELD_CODE => 'col-schvaleni2'] );
//        $this ->addField( [BROWSER_FIELD_SQL_NAME => '', BROWSER_FIELD_TYPE => 'schvaleniDV',  BROWSER_FIELD_CLASS_NAME => 'schvaleni', BROWSER_FIELD_CODE => 'col-schvaleni3'] );

        $this ->addField( [
            BROWSER_FIELD_SQL_NAME => '1', BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_CODE => 'col-datum',
            BROWSER_FIELD_FORMAT => 'timestamp',
            BROWSER_FIELD_HEADER => \Ron\ronText('Date'), BROWSER_FIELD_CLASS_NAME => 'border-right', BROWSER_FIELD_ALIGN => 'center', 'footer' => \Ron\ronText('Celkem')] );

        $col_defs = explode(';', $str);
        $this ->sloupce = [];

        foreach ($col_defs as $index => $colstr) {
            $pomarray = explode(':', $colstr); // Nadpis:[*][#H|#D|#K]vyraz

            $sloupec = new SloupecDennichVysledku();

            $expression = $pomarray[1];
            $sloupec ->editovat = ($expression[0] == '*');
            if ($expression[0] == '*') $expression = substr($expression, 1);

            if (substr($expression, 0, 2) == '#H'
                || substr($expression, 0, 2) == '#D'
                || substr($expression, 0, 2) == '#K'
            ) {
                $sloupec ->jednotka = $expression[1];
                $expression = substr($expression, 2);
            } else {
                $sloupec ->jednotka = 'H';
            }

            $sloupec ->casovaSlozka = str_replace('_', '', $expression);
            switch ($sloupec ->jednotka) {
                case 'H' :
                    $sloupec ->expression = StringUtils::processIdentifiers($expression, '$H_'); // str_replace('_', '$H_', $expression);
                    break;
                case 'D' :
                    $sloupec ->expression = StringUtils::processIdentifiers($expression, '$D_'); // str_replace('_', '$D_', $expression);
                    break;
                case 'K' :
                    $sloupec ->expression = StringUtils::processIdentifiers($expression, '$K_'); // str_replace('_', '$K_', $expression);
                    break;
            }

            $this ->sloupce[] = $sloupec;

            $this ->addField( [
                BROWSER_FIELD_SQL_NAME => '1', BROWSER_FIELD_HEADER => $pomarray[0], BROWSER_FIELD_SQL_INDEX => $index + 5,
                BROWSER_FIELD_CODE => 'col-dv-' . ($index + 1),
                BROWSER_FIELD_ALIGN => strcasecmp($sloupec ->expression, 'Poznamka') ? 'right' : 'left'] );
                //array('normal', '1', $pomarray[0], '', strcasecmp($sloupec ->f_expression, 'Poznamka') ? 'right' : 'left', '', 0, 0));
        }
    }


    function nactiDenniVysledky()
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Den, TypCasoveSlozkyMzdy, Hodiny, Dny, KalendarniDny ' .
            ' FROM DenniVysledekVypoctu' .
            ' WHERE RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
            ' AND OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)) .
            ' AND Den >= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumOd)) .
            ' AND Den <= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumDo)) .
            ' ORDER BY Den';

        MessageManager::addDebugLog('Denni vysledky: ' . $sql);
        return $this ->database ->getSqlResult($sql, $params);
    }


    function nactiPuvodniDochazku()
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT PuvodniDochazka.DatumCasOperace, OperaceDochazky.Popis' .
            ' FROM PuvodniDochazka, OperaceDochazky' .
            ' WHERE PuvodniDochazka.KodOperace = OperaceDochazky.Kod' .
            ' AND PuvodniDochazka.RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
            ' AND PuvodniDochazka.OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
            ' AND PuvodniDochazka.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)) .
            ' AND PuvodniDochazka.DatumCasOperace >= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumOd)) .
            ' AND PuvodniDochazka.DatumCasOperace < ' . $params ->addParameter($this ->database ->dateToParam(DateUtils::incDay($this ->datumDo, 1))) .
            ' ORDER BY PuvodniDochazka.DatumCasOperace';

        MessageManager::addDebugLog('Puvodni dochazka: ' . $sql);
        return $this ->database ->getSqlResult($sql, $params, 'Chyba pri nacitani puvodni dochazky!');
    }


    function nactiSchvaleni()
    {
        $this ->schvaleni = new SchvaleniDochazky();
        $this ->schvaleni ->load($this ->database,
            $this ->filterForm ->vybranaOsobaRC, $this ->filterForm ->vybranaOsobaOsobniCislo, $this ->filterForm ->vybranaOsobaDatumNastupu,
            $this ->datumOd, $this ->datumDo);
    }


    /**
     * @param $datum
     * @param $valuesStr
     * @param $jeToSoucet
     * @return array
     */
    function vytvorRadek($datum, $valuesStr, $jeToSoucet)
    {
        $browserrow = [];

        if (!$jeToSoucet) {
            // V nulte polozce radku ulozi typ dne (vikend/svatek/normalni)
            $browserrow[] = $datum;
            // V dalsich trech polozkach ulozi hodnotu true, pokud je dochazka
            // pro dany den zpracovana/zkontrolovana/schvalena
            $browserrow[] = $datum <= $this ->schvaleni ->schvaleni1;
            $browserrow[] = $datum <= $this ->schvaleni ->schvaleni2;
            $browserrow[] = $datum <= $this ->schvaleni ->schvaleni3;
            // Vytvori hodnotu v prvnim sloupci - datum
            $browserrow[] =
                '<span class="zkratkadne">' . \Ron\ronText('Day_short_' . date('w', $datum)) . '</span>' .
                date('j.n.Y', $datum);
        }

        // Vytvori jednotlive promenne pro vypocty a doplni jejich hodnoty
        eval($valuesStr);

        // Provede jednotlive vypocty
        foreach ($this ->sloupce as $sloupec) {
            $pomstr = '';
            $minus = false;

            if (strcasecmp($sloupec ->expression, 'Poznamka') == 0) {
                // Poznamka ke dni
                if (!$jeToSoucet) {
                    require_once('dochazka/poznamky.php');
                    $pomstr = DenniPoznamkaDochazky::vratPoznamkuDochazky($this ->database, $datum, $this ->filterForm ->vybranaOsoba);
                }
            } else if ($sloupec ->expression) {
                // Vyhodnoceni vzorecku
                $vysledek = null;
                eval('$vysledek = ' . $sloupec ->expression . ';');

                if (!$jeToSoucet && !$vysledek && !RonPortalSettings::getIntValue(self::SETTINGS_ZOBRAZIT_NULOVE)) {
                    $pomstr = '';
                } else {
                    switch ($sloupec->jednotka) {
                        case 'H' :
                            $vysledek = round($vysledek * 60);
                            $pomstr = StringUtils::format('minuty', $vysledek);
                            break;

                        case 'D' :
                        case 'K' :
                            $pomstr = StringUtils::format('number_2', $vysledek);
                            break;
                    }
                }

                if ($vysledek < 0) $minus = true;

                if (!$jeToSoucet) $sloupec ->soucet += $vysledek;
            }

            $pomstr = str_replace("'", '&#39;', $pomstr);

            // Pokud se pole muze editovat, vlozi edit, jinak jenom text
            if ($sloupec ->editovat && !$jeToSoucet && $this ->lzeEditovat &&
                $this ->schvaleni ->lzeEditovatPP($this ->filterForm ->vybranaOsoba, $datum)
            ) {
                $inputname = 'hodnota_' . $datum . '_' . $sloupec ->jednotka . '_' . $sloupec ->casovaSlozka;

                if (strcasecmp($sloupec ->expression, 'Poznamka') == 0) {
                    $col_str =
                        "<input type='text' name='$inputname' class='poznamkaedit' size='30' value='$pomstr' />";
                    // pozor, musi byt nahrazene '
                } else {
                    $col_str = '';

                    if ($minus) $col_str .= '<span class="minus">';

                    $className =
                        'vysledekedit ' .
                        ( ($sloupec ->jednotka == 'D' || $sloupec ->jednotka == 'K')
                            ? 'inputFloat'
                            : 'inputTime'
                        );

                    $col_str .=
                        "<input type='text' name='$inputname' class='$className' maxlength='5' size='5' value='$pomstr' />";
                    // pozor, musi byt nahrazene '

                    if ($minus) $col_str .= '</span>';
                }

                $browserrow[] = $col_str;

                $this ->pocetEditu++;
            } else {
                if ($minus) $browserrow[] = '<span class="minus">' . $pomstr . '</span>';
                else $browserrow[] = $pomstr;
            }

        }

        return $browserrow;
    }


    /**
     * @param Cursor $cursor
     * @param Cursor $puvodni
     * @return array
     */
    function prevedDataDoPole($cursor, $puvodni)
    {
        $this ->pocetEditu = 0;
        //$this -> f_soucet = array();

        $datum = DateUtils::startOfTheDay($this ->datumOd);
        $poslednidatum = DateUtils::startOfTheDay($this ->datumDo);
        $result = [];

        $zaznam = $cursor ->fetch();
        $zaznamPuvodni = $puvodni ? $puvodni ->fetch() : null;

        // Pro kazdy den vytvori radek
        while ($datum <= $poslednidatum) {
            $dalsidatum = DateUtils::incDay($datum, 1);

            // Prida do vypoctu vsechny zaznamy z daneho dne
            $valuesStr = '';

            while ($zaznam && $this ->database ->stringToTimestamp($zaznam[0]) < $dalsidatum) {
                // Nekdy se cislo prevede na retezec s desetinnou carkou misto
                // tecky, takze potom nefunguje eval. Proto se tady prevede s teckou
                $pom = str_replace(',', '.', $zaznam[2]);
                $valuesStr = $valuesStr . '$H_' . $zaznam[1] . ' += ' . $pom . ';';

                $pom = str_replace(',', '.', $zaznam[3]);
                $valuesStr = $valuesStr . '$D_' . $zaznam[1] . ' += ' . $pom . ';';

                $pom = str_replace(',', '.', $zaznam[4]);
                $valuesStr = $valuesStr . '$K_' . $zaznam[1] . ' += ' . $pom . ';';

                $zaznam = $cursor ->fetch();
            }

            $radek = $this ->vytvorRadek($datum, $valuesStr, false);

            $pocetPuvodni = 0;
            // pokud se ma zobrazit puvodni dochazka, tak do radku doplni zaznamy puvodni dochazky
            while ($zaznamPuvodni  &&  $this ->database ->stringToTimestamp($zaznamPuvodni[0]) < $dalsidatum) {
                $radek[] = date("H:i", $this ->database ->stringToTimestamp($zaznamPuvodni[0])) . " &nbsp;" . StringUtils::htmlTextConvert($zaznamPuvodni[1]);
                $pocetPuvodni++;
                $zaznamPuvodni = $puvodni ->fetch();
            }

            if ($pocetPuvodni > $this ->pocetPuvodni) $this ->pocetPuvodni = $pocetPuvodni;

            $result[] = $radek;

            // while ($zaznam && $zaznam[0]
            $datum = $dalsidatum; // incday($datum, 1);
        }

        return $result;
    }


//    function vlozPoleSBarvou($vyplneno, $classname)
//    {
//        if ($vyplneno) echo "<td class='schvaleni $classname'>";
//        else echo '<td class="schvaleni">';
//    }



    function renderDenniVysledky($poleVysledku)
    {
        $rowNumber = 0;
        foreach ($poleVysledku as $index => $row) {
            $rowNumber++;

            // Podle toho jestli je to vikend nebo ne zvoli barvu
            $cssclass = Kalendar::getStyleForDate($this->database, $row[0], 1, $this->kalendar);
            echo $cssclass ? "<tr class='$cssclass'>" : '<tr>';

            // Vykresli barvicky podle schvalovani
//            $this->vlozPoleSBarvou($row[1], 'schvaleni1'); // zelena
//            $this->vlozPoleSBarvou($row[2], 'schvaleni2'); // zluta
//            $this->vlozPoleSBarvou($row[3], 'schvaleni3'); // cervena

            // Vypise obsah radku
            $prvniDatovySloupec = 4;

            //for ($i = $prvniDatovySloupec; $i < count($this ->fields); $i++) {
            //for ($i = 0; $i < count($this ->fields); $i++) {
            foreach ($this->fields as $i => $field) {
                if ($field ->visible  &&  $field ->type != 'hidden') {
                    $class = $field ->getDataCellClass($row[$field ->sqlFieldIndex], $rowNumber, null, $row);

//                    if ($i < 4) {
//
//                    } else if ($i == $prvniDatovySloupec) {
//                        // Prvni radek je datum
//                        echo '<td class="browser_rowheader align-center">';
//                    } else {
//                        echo '<td align="' . $this->fields[$i]->align . '">';
//                    }
                    echo "<td class='$class'>";

                    if ($field ->type != 'priznakschvaleni') {
                        echo $row[$field ->sqlFieldIndex];
                    }
                    echo "</td>\n";
                }
            }

            echo '</tr>';
        }
    }



    function nactiSoucet($variantni)
    {
        $params = $this ->database ->createParameters();

        if ($variantni) {
            $sql =
                'SELECT TypCasoveSlozkyMzdy, Hodiny, Dny, KalendarniDny ' .
                '  FROM VariantniVysledek ' .
                ' WHERE RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
                '   AND OsobniCislo = ' . $params ->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
                '   AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)) .
                '   AND VariantniObdobi = ' . $params ->addParameter($this ->database ->dateToParam($this ->datumOd));
        } else {
            $sql =
                'SELECT TypCasoveSlozkyMzdy, Hodiny, Dny, KalendarniDny ' .
                '  FROM MesicniVysledekVypoctu ' .
                ' WHERE RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
                '   AND OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
                '   AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)) .
                '   AND Mesic = ' . $params ->addParameter($this ->database ->dateToParam($this ->datumOd));
        }

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $valuesStr = '';
        while ($zaznam = $cursor ->fetch()) {
            $pom = str_replace(',', '.', $zaznam[1]);
            $valuesStr = $valuesStr . '$H_' . $zaznam[0] . ' += ' . $pom . ';';

            $pom = str_replace(',', '.', $zaznam[2]);
            $valuesStr = $valuesStr . '$D_' . $zaznam[0] . ' += ' . $pom . ';';

            $pom = str_replace(',', '.', $zaznam[3]);
            $valuesStr = $valuesStr . '$K_' . $zaznam[0] . ' += ' . $pom . ';';
        }

        return $this ->vytvorRadek(0, $valuesStr, true);
    }


    function renderSoucet()
    {
        $soucet = $this ->nactiSoucet($this ->variantni);
        if ($soucet) {
            echo '<tr class="browserfooter">';

            foreach ($this ->fields as $i => $field) {
                if ($field ->visible  &&  $field ->type != 'hidden') {
                    $class = $field->getFooterClass();
                    echo "<td class='$class'>";

                    if ($field ->footer) {
                        echo $field ->footer;
                    } else if ($field ->sqlFieldIndex > 4) {
                        echo ($soucet[$i - 5] ?: '');
                    }

                    echo '</td>';
                }
            }

//            echo '<td class="schvaleni"></td><td class="schvaleni"></td><td class="schvaleni"></td>'; // barvicky
//            echo '<td>' . \Ron\ronText('Celkem') . '</td>';
//
//            foreach ($soucet as $value) {
//                echo "<td class='align-right'> $value </td>";
//            }
//
//            for ($i = 0; $i < $this ->pocetPuvodni; $i++) {
//                echo '<td></td>';
//            }

            echo "</tr>\n";
        }
    }


    function tlacitkoSchvaleni()
    {
        require_once('dochazka/schvalenidochazky.php');

        if ( SchvaleniDochazky::lzeSchvalovatOsobu($this ->filterForm ->vybranaOsoba, $this ->filterForm ->vybraneObdobiOd, $this ->filterForm ->vybraneObdobiDo) ) {
            echo "<input class='submitbutton' type='submit' name='dialogschvaleni' value='" . \Ron\ronText('Schvalit_dochazku') . "' />";
        }
    }


    function load($sqlFilter = '')
    {
        $this ->loadConfiguration();

        $this ->setFilterForm();
        $this ->filterForm ->loadData();

        $this ->datumOd = $this ->filterForm ->vybraneObdobiOd;
        $this ->datumDo = $this ->filterForm ->vybraneObdobiDo;

        $prava = Prava::vratPrava('editace');

        require_once 'adsapp/obdobi.php';

        $this ->lzeEditovat = !$this ->variantni &&
            !Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $this ->datumOd, $this ->datumDo) &&
            $prava ->lzeZobrazitPP($this ->filterForm ->vybranaOsoba, $this ->datumOd, $this ->datumDo);

        $this ->kalendar = Kalendar::vratKalendarOsoby($this ->database,
            $this ->filterForm ->vybranaOsobaRC, $this ->filterForm ->vybranaOsobaOsobniCislo, $this ->filterForm ->vybranaOsobaDatumNastupu,
            $this ->datumOd, $this ->datumDo);

        $this ->nactiSchvaleni();
        $this ->cursor = $this ->nactiDenniVysledky();

        $this ->cursorPuvodni = RonPortalSettings::getIntValue(self::SETTINGS_ZOBRAZIT_PUVODNI)
            ? $this ->nactiPuvodniDochazku()
            : null;
    }


    function render()
    {
        echo '<div class="dennivysledkybrowser">';
        $this ->renderHeader();

        $pomarray = $this ->prevedDataDoPole($this ->cursor, $this ->cursorPuvodni);

        // doplni sloupce pro puvodni dochazku
        $pocetPoliPuvodni = count($this ->fields);
        for ($i = 0; $i < $this ->pocetPuvodni; $i++) {
            $this ->addField( [BROWSER_FIELD_SQL_NAME => '1', BROWSER_FIELD_SQL_INDEX => $pocetPoliPuvodni + $i, BROWSER_FIELD_WRAP => 1, BROWSER_FIELD_CLASS_NAME => 'zaznampuvodni'] );
        }

        //if ($this ->f_poceteditu) {
            //echo '<FORM method="post" action="?menuid=editacedennich">';
            echo '<form method="post" action="?menuid=dennivysledky">';
            //echo '<INPUT type="hidden" name="osobaeditace" value="'.S_GetVybranaOsoba().'">';
            //echo '<INPUT type="hidden" name="obdobieditace" value="'.$this -> f_datumod.'">';

            echo '<INPUT type="hidden" name="editacedennich" value="1" />';
            echo '<INPUT type="hidden" name="obdobi" value="' . $this ->filterForm ->vybraneObdobiOd . '" />';
            echo '<INPUT type="hidden" name="osoba" value="' . $this ->filterForm ->vybranaOsoba . '" />';
        //}

        $this ->renderTableStart();
        echo "<tbody class='border-bottom'>\n";
        $this ->renderDenniVysledky($pomarray);
        echo "</tbody>\n";
        echo "<tbody>\n";
        $this ->renderSoucet();
        echo "</tbody>\n";
        $this ->renderTableEnd();

        echo '<div class="buttons">';
        if ($this ->pocetEditu) {
            //echo '<INPUT type="hidden" name="poceteditu" value="'.$this -> f_poceteditu.'">';
            echo '<INPUT type="submit" class="submitbutton" name="ulozit" value="' . \Ron\ronText('form.Store') . '">';

            if ( RonPortalSettings::getSpousteniADS() ) {
                echo '<INPUT class="submitbutton" type="submit" name="prepocet" value="' . \Ron\ronText('Prepocet_DV') . '" >';
            }

        }

        if ( !$this ->variantni ) $this ->tlacitkoSchvaleni();

        echo '</div>';
        echo "</form>\n";

        echo "</div>\n"; // class=dennivysledkybrowser
    }



    // EDITACE DAT ------------------------------------------------------------
    /**
     * @param PracovniPomer $pp
     * @param int $datum
     * @param string $hodnota
     */
    function ulozPoznamku($pp, $datum, $hodnota)
    {
        require_once('editace/editace_records.php');

        $rec = new DenniPoznamkaDochazkyRecord($this ->database, $pp, $datum, $this);
        $rec ->zapisPoznamku($hodnota);
    }


    /**
     * @param PracovniPomer $pp
     * @param int $datum
     * @param SloupecDennichVysledku $sloupec
     * @param string $hodnotaStr
     * @return bool
     */
    function ulozDenniVysledek($pp, $datum, $sloupec, $hodnotaStr)
    {
        try {
            if (trim($hodnotaStr) != '') {
                switch ($sloupec->jednotka) {
                    case 'D' :
                    case 'K' :
                        $hodnota = StringUtils::stringToFloat($hodnotaStr);
                        break;

                    default :
                        $hodnota = DateUtils::timeStringToFloat($hodnotaStr, true); // povolitZkraceny = true
                }
            } else {
                $hodnota = 0;
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage(date('j.n.Y', $datum) . ': ' . \Ron\ronText('Spatna_hodnota'));
            //MessageManager::addErrorMessage($e ->getMessage());
            return false;
        }

        require_once('editace/editace_records.php');
        $rec = new DenniVysledekVypoctuRecord($this ->database, $pp, $datum, $sloupec ->casovaSlozka, $this);
        $rec ->zapisHodnotu($sloupec ->jednotka, $hodnota);

        if ($rec ->recordUpdated) $sloupec ->prepocitat = true;

        return true;
    }


    /**
     * @param PracovniPomer $pp
     * @param int $datum
     * @return bool
     */
    function ulozDen($pp, $datum)
    {
        $result = true;

        foreach ($this ->sloupce as $sloupec) {
            if ($sloupec ->editovat) {

                // hodnota_<den>_H/D/K_<slozka>
                $inputname = 'hodnota_' . $datum . '_' . $sloupec ->jednotka . '_' . $sloupec ->casovaSlozka;

                if (isset($_POST[$inputname])) {
                    $hodnota = $_POST[$inputname];

                    if (strcasecmp($sloupec ->expression, 'Poznamka') == 0) {
                        $this ->ulozPoznamku($pp, $datum, $hodnota);
                    } else {
                        $result = $result && $this ->ulozDenniVysledek($pp, $datum, $sloupec, $hodnota);
                    }
                }
            }
        }

        return $result;
    }


    /**
     * Prepocita mesicni vysledky sloupcu, ktere maji nastaven priznak prepocitat
     * @param PracovniPomer $pp
     * @param Obdobi $obdobi
     * @return bool
     * @throws \Ron\DB\DBException
     */
    function prepocitejMesicniVysledky($pp, $obdobi)
    {
        $result = true;

        foreach ($this ->sloupce as $sloupec) {
            if ($sloupec ->prepocitat) {
                // zjisti mesicni soucty pro casovou slozku
                $params = $this ->database ->createParameters();
                $sql =
                    'SELECT SUM(Hodiny), SUM(Dny), SUM(KalendarniDny) FROM DenniVysledekVypoctu' .
                    ' WHERE RC = ' . $params->addParameter($pp ->rc) .
                    ' AND OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
                    ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pp ->datumNastupu)) .
                    ' AND Den >= ' . $params ->addParameter($this ->database ->dateToParam($obdobi ->zacatek)) .
                    ' AND Den <= ' . $params ->addParameter($this ->database ->dateToParam($obdobi ->konec)) .
                    ' AND TypCasoveSlozkyMzdy = ' . $params->addParameter($sloupec ->casovaSlozka);

                $cursor = $this ->database ->getSqlResult($sql, $params);
                $soucty = $cursor ->fetch();
                $cursor ->close();

                // nastavi zaznam
                require_once('editace/editace_records.php');
                $rec = new MesicniVysledekVypoctuRecord($this ->database, $pp, $obdobi ->zacatek, $sloupec ->casovaSlozka, $this);

                $rec ->zapisHodnotu($soucty[0], $soucty[1], $soucty[2], null); // poznamku nenastavuje
            }
        }

        if (!$result) {
            MessageManager::addErrorMessage('Chyba pri prepoctu mesicnich vysledku!');
        }

        return $result;
    }


    function ulozDenniVysledky()
    {
        if (!isset($_POST['editacedennich'])) return true; // zadna data, nedela nic

        // probehla editace, je potreba nacist udaje editovane osoby
        require_once('adsapp/ppomer.php');
        $pp = PracovniPomer::nactiPP($this ->database, $_POST['osoba']);
        $pp ->vyberNactenyPP($this ->database, null);

        // Pozor! Nefunguje pro variabilni obdobi!
        require_once('adsapp/obdobi.php');
        $obdobi = new Obdobi();
        if (!$obdobi ->nactiObdobi($this ->database, 'ADS', $_POST['obdobi'])) {
            throw new \Exception('Chyba pri editaci dennich vysledku - zadane obdobi nebylo nalezeno!'); // nemelo by nastat
        }

        $obdobi ->nastavSession();

        $this ->datumOd = $obdobi ->zacatek;
        $this ->datumDo = $obdobi ->konec;

        Prava::nastavObdobiProPrava($obdobi ->zacatek, $obdobi ->konec);

        if ($obdobi ->uzavreno) {
            throw new \Exception( \Ron\ronText('Obdobi_uzavreno') );
        }

        $this ->nactiSchvaleni();

        $result = true;

        $prava = Prava::vratPrava('editace');
        if ($prava ->lzeZobrazitPP($pp ->idPP, $obdobi ->zacatek, $obdobi ->konec)) {
            // pokud nema prava, tak se tu nic nedela, ale vyjimku nevyhodi, protoze muze mit pravo na schvalovani
            $datum = $obdobi->zacatek;

            while ($datum <= $obdobi->konec) {
                if ($this->schvaleni->lzeEditovatPP($pp->idPP, $datum)) {
                    $result &= $this->ulozDen($pp, $datum);
                }

                $datum = DateUtils::incDay($datum, 1);
            }

            $result &= $this->prepocitejMesicniVysledky($pp, $obdobi);

            if (RonPortalSettings::getSpousteniADS() && isset($_POST['prepocet'])) if ($_POST['prepocet']) {
                require_once('adsapp/agentmanager.php');
                AgentManager::prepocetDochazky(
                        $pp->idPP,
                        $obdobi->zacatek, $obdobi->konec);
            }
        }

        return $result;
    }
}




class DenniVysledkyController implements Controller
{
    /** @var  Database */
    var $database;


    /**
     * DenniVysledkyController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini');
        HtmlHeaders::getInstance() ->addScript('script/dochazka.js');
        HtmlHeaders::getInstance() ->addCss('css/dochazka.css');
        HtmlHeaders::getInstance() ->setTitle( \Ron\ronText('RON_Portal') . ' - ' . \Ron\ronText('Dochazka_title'));

        if (!RonPortalSettings::getValue(DenniVysledkyBrowser::SETTINGS_SLOUPCE)) {
            throw new \Exception( \Ron\ronText('Neni_nastaveni_sloupcu_tabulky') );
        }

        $b = new DenniVysledkyBrowser($this ->database, $action == 'dennivysledkyvariantni');

        if (isset($_POST['editacedennich'])) {
            if ( $b ->ulozDenniVysledky() ) {
                $params = '?menuid=dennivysledky&idpp=' . RonPortalSession::vybranaOsoba() . '&obdobi=' . DateUtils::dateToUrlString(RonPortalSession::obdobi());
                if (isset($_POST['dialogschvaleni']) && $_POST['dialogschvaleni']) {
                    $params .= '&dialogschvaleni=1';
                }

                return new Redirect(null, $params );
            }
        }

        $b ->load();

        if (isset($_GET['dialogschvaleni'])  &&  $_GET['dialogschvaleni']) {
            require_once('dochazka/schvalovani.php');
            $schvaleniView = new SchvaleniView($this ->database, true);
            $schvaleniView ->redirectUrl = '?menuid=dennivysledky&idpp=' . RonPortalSession::vybranaOsoba() . '&obdobi=' . DateUtils::dateToUrlString(RonPortalSession::obdobi());
            Model::addViewObject($schvaleniView);
        }

        return $b;
    }

}
