<?php

/**
 * UTF8
 */

namespace Ron\RonPortal\Dochazka;

use Exception;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalBrowser;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\Rozdeleni;
use function Ron\ronText;

require_once('ron/sqlutils.php');
require_once('adsapp/ronportalbrowser.php');


class ENeschopenkaBrowser extends RonPortalBrowser
{
    /** @var PravaNaOsoby */
    private $prava;

    /**
     * ENeschopenkaBrowser constructor.
     * @param Database $database
     * @param PravaNaOsoby $prava
     * @throws Exception
     */
    function __construct($database, $prava)
    {
        parent::__construct($database, 'eneschopenka');

        $this->prava = $prava;

        $this->code = 'browsereneschopenka';

        $this->title = ronText('E_neschopenka');
        $this->cssClass = 'browser';

        $this->sqlSelectPart = "Osoba.Jmeno, Osoba.Prijmeni, DruhDavkyNZ.Popis, ENeschopenka.DatumPocatkuDavky, ENeschopenka.DatumUkonceniDavky, ENeschopenka.UkoncenaDavka";

        $this->sqlFromPart = "ENeschopenka JOIN Osoba ON ENeschopenka.RC = Osoba.RC JOIN DruhDavkyNZ ON DruhDavkyNZ.Kod = ENeschopenka.DruhDavky";

        $this ->filterFormAppCode = 'ADS';
        $this ->filterFormRights = 'eneschopenka_prohl';
        $this ->filterFormVcetneExternich = true;

        $this->setFilterForm();
        $this->filterForm->loadData();

        if ($this->filterForm->vsechnyOsoby) {
            $this->addFields([
                [BROWSER_FIELD_SQL_NAME => "Osoba.Prijmeni", BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_HEADER_TEXT => 'Prijmeni',
                    BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_CLASS_NAME => 'font-weight-bold', BROWSER_FIELD_WRAP => 1],
                [BROWSER_FIELD_SQL_NAME => "Osoba.Jmeno", BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_HEADER_TEXT => 'Jmeno',
                    BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_CLASS_NAME => 'font-weight-bold', BROWSER_FIELD_WRAP => 1]
            ]);
        }

        $this->addField([ BROWSER_FIELD_HEADER_TEXT => 'E_neschopenka_druh', BROWSER_FIELD_SQL_NAME => "DruhDavkyNZ.Popis", BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center' ]);
        $this->addField([ BROWSER_FIELD_HEADER_TEXT => 'E_neschopenka_datum_pocatku', BROWSER_FIELD_SQL_NAME => "ENeschopenka.DatumPocatkuDavky", BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_FORMAT => 'date' ]);

        if (!$this->filterForm->vsechnyOsoby) {
            $this->orderFieldIndex = count($this->fields);
        }
        $this->addField([ BROWSER_FIELD_HEADER_TEXT => 'E_neschopenka_datum_ukonceni', BROWSER_FIELD_SQL_NAME => "ENeschopenka.DatumUkonceniDavky", BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_FORMAT => 'date' ]);
        $this->addField([ BROWSER_FIELD_HEADER_TEXT => 'E_neschopenka_ukoncena_davka', BROWSER_FIELD_SQL_NAME => "ENeschopenka.UkoncenaDavka", BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_FORMAT => 'anone' ]);
    }


    function setFilterForm()
    {
        parent::setFilterForm();
        $this->filterForm ->nastavAkciGet('eneschopenka');
        $this->filterForm->povolitVsechyOsoby = true;
    }

    function getSQLFilter()
    {
        require_once('adsapp/rozdeleni.php');

        $params = $this->database->createParameters();

        $sql =
            'ENeschopenka.DatumPocatkuDavky <= ' . $params->addParameter($this->database->dateToParam($this->filterForm->vybraneObdobiDo)) . ' AND ' .
            '(ENeschopenka.DatumUkonceniDavky >= ' . $params->addParameter($this->database->dateToParam($this->filterForm->vybraneObdobiOd)) . ' OR ENeschopenka.DatumUkonceniDavky IS NULL)';

        if ($this->filterForm->vsechnyOsoby) {


            $sql .=
                ' AND EXISTS(SELECT ID FROM PracovniPomer WHERE PracovniPomer.RC = Osoba.RC AND PracovniPomer.DatumNastupu <= ' . $params->addParameter($this->database->dateToParam($this->filterForm->vybraneObdobiDo)) .
                '           AND (PracovniPomer.DatumUkonceni >= ' . $params->addParameter($this->database->dateToParam($this->filterForm->vybraneObdobiOd)) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
                ' AND ' . $this->prava->sqlFiltr('PracovniPomer', $this->filterForm->vybraneObdobiOd, $this->filterForm->vybraneObdobiDo, $params);

            if ($this->filterForm->kodRozdeleni) {
                $sql .= ' AND ' . Rozdeleni::sqlFilterRozdeleni($this->database, 'PracovniPomer', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $this->filterForm->kodRozdeleni,
                        $this->filterForm->vybraneObdobiOd, $this->filterForm->vybraneObdobiDo, $this->filterForm->vcetnePodrizenychRozdeleni, $params);
            }

            if ($this->filterForm->kodRozdeleni2) {
                $sql .= ' AND ' . Rozdeleni::sqlFilterRozdeleni($this->database, 'PracovniPomer', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), $this->filterForm->kodRozdeleni2,
                        $this->filterForm->vybraneObdobiOd, $this->filterForm->vybraneObdobiDo, $this->filterForm->vcetnePodrizenychRozdeleni2, $params);
            }

            $sql .=
                ')';
        } else {
                $sql .= ' AND ENeschopenka.RC = ' . $params->addParameter($this->filterForm->vybranaOsobaRC);

        }

        return new SqlStatement($sql, $params);
    }


}
