<?php

/**
 * UTF8
 * seznam mesicnich vysledku - zobrazuje nastavene mesicni vysledky
 */

namespace Ron\RonPortal;

use Ron\DB\Cursor;
use Ron\MessageManager;
use Ron\StringUtils;

require_once 'dochazka/b_vysledkybrowser.php';

class MesicniVysledkyBrowser extends VysledkyBrowser
{

    /** @var string[]   pole vyrazu pro vypocet jednotlivych polozek */
    var $expressions;
    /** @var string[]   pole souctu jednotlivych polozek */
    var $rowCaptions;
    /** @var bool       jestli se maji zobrazovat variantni obdobi (true) nebo mesice (false) */
    var $variantni;
    /** @var bool       ve zpracovane dochazce se filtr nezobrazi */
    var $showFilter;

    var $slozky;



    function __construct($database, $variantni)
    {
        $action = $variantni ? 'variantnivysledky' : 'mesicnivysledky';
        parent::__construct($database, $action);

        $this->showFilter = true;
        $this->variantni = $variantni;

        $this->cssClass .= ' ' . $action;

        $this ->title = \Ron\ronText($variantni ? 'Prehled_variantnich_vysledku' : 'Mesicni_vysledky');

        $this->addField( [
            BROWSER_FIELD_SQL_NAME => '1', BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_HEADER_TEXT => 'Casova_slozka'] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => '2', BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_HEADER_TEXT => 'Vysledek_Hodiny',
            BROWSER_FIELD_FORMAT => 'hodiny', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_CLASS_NAME => 'hodiny'] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => '3', BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_HEADER_TEXT => 'Vysledek_Pracovni_dny',
            BROWSER_FIELD_FORMAT => 'number_2', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_CLASS_NAME => 'dny'] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => '4', BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_HEADER_TEXT => 'Vysledek_Kalendarni_dny',
            BROWSER_FIELD_FORMAT => 'number_2', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_CLASS_NAME => 'kalendarnidny'] );
    }


    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action);
        $this ->filterForm ->downloadFotoOsobyUrl = '?menuid=editace&action=foto&idpp=';

        if ($this->variantni) $this->filterForm->nastavVariantni(true);
        else $this->filterForm->nastavObdobi(true, 'ADS', false);  // jenOtevrene = false

        $prava = Prava::vratPrava('prohlizeni');

        if ($prava->zobrazitVyberRozdeleni()) // napr. pro "pouze vlastni" se vyber nezobrazuje
        {
            $this->filterForm->nastavRozdeleni(true);
            $this->filterForm->f_pravarozdeleni = $prava;
            $this->filterForm->vcetnePodrizenychRozdeleni = true;

            if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
                $this->filterForm->nastavRozdeleni2(true);
                $this->filterForm->vcetnePodrizenychRozdeleni2 = true;
            }
        }

        $this->filterForm->nastavOsoby(true, $prava, 0, 0); // datum od-do podle obdobi
        $this->filterForm->osobyFiltr = 'PracovniPomer.ZpracovatDochazku = 1';
        $this ->filterForm ->downloadFotoOsobyUrl = '?menuid=editace&action=foto&idpp=';
        $this ->filterForm ->nastavAkciGet($this ->variantni ? 'variantnivysledky' : 'mesicnivysledky');
    }


    function renderFilter()
    {
        if ($this->showFilter) {
            if ($this->variantni) $this->odkazyVariabilniVysledky('variantnivysledky');
            else $this->odkazyMesicniVysledky('mesicnivysledky');

            parent::renderFilter();
        }
    }


    /**
     * Vytvori pole hodnot z databaze ve tvaru <kod_casove_slozky> => [ 'h' => hodiny, 'd' => dny, 'k' => kal.dny ]
     * @param Cursor $cursor
     * @return array
     */
    function nactiDataDoPole($cursor)
    {
        $result = [];

        while ($zaznam = $cursor ->fetch()) {
            $result['_' . $zaznam[0]] = [ 'h' => $zaznam[1], 'd' => $zaznam[2], 'k' => $zaznam[3] ]; // nastaveni pridava _!
        }

        return $result;
    }


    /**
     * Vytvori pole PHP promennych z hodnot v databazi (pouziva se, pokud jsou povoleny vzorce)
     * @param Cursor $cursor
     * @return string
     */
    function nactiDataDoPromennych($cursor)
    {
        $result = '';

        while ($zaznam = $cursor ->fetch()) {
            $result = $result .
                '$h_' . "$zaznam[0]=" . str_replace(',', '.', $zaznam[1]) . ';' .
                '$d_' . "$zaznam[0]=" . str_replace(',', '.', $zaznam[2]) . ';' .
                '$k_' . "$zaznam[0]=" . str_replace(',', '.', $zaznam[3]) . ';';
        }

        return $result;
    }



    function nactiMesicniVysledky()
    {
        $params = $this ->database ->createParameters();

        if ($this->variantni) {
            $sql =
                'SELECT TypCasoveSlozkyMzdy, Hodiny, Dny, KalendarniDny' .
                ' FROM VariantniVysledek' .
                ' WHERE RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
                ' AND OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
                ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)) .
                ' AND VariantniObdobi = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybraneObdobiOd)); // variantni!
        } else {
            $sql =
                'SELECT TypCasoveSlozkyMzdy, Hodiny, Dny, KalendarniDny' .
                ' FROM MesicniVysledekVypoctu' .
                ' WHERE RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
                ' AND OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
                ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)) .
                ' AND Mesic = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybraneObdobiOd));
        }

        MessageManager::addDebugLog($sql);

        $cursor = $this ->database -> select($sql, $params);

        return $cursor;
    }



    // --------------------------------------------------------------------
    // Vypocte hodnotu podle zadaneho vzorce. Slozky jsou nactene z databaze
    // a pismeno urcuje jestli se pocitaji hodiny (h), dny (d) nebo kalendarni dny (k).
    function vypoctiHodnotu($slozky, $vzorec, $pismeno)
    {
        $vzorec = StringUtils::processIdentifiers($vzorec, '$' . $pismeno . '_');
        //$vzorec = str_replace('_', '$' . $pismeno . '_', $vzorec); - puvodi verze nezvlada podtrzitka v nazvu casove slozky!

        eval($slozky);

        $result = 0;
        eval('$result = ' . $vzorec . ';');

        return $result;
    }


    function vypisMesicniVysledky($slozky)
    {
        foreach ($this->rowCaptions as $index => $rowname) {
            $emptyrow = false;

            $h = null;
            $d = null;
            $k = null;

            if ($this->expressions[$index]) {
                if ( RonPortalSettings::getIntValue(RonPortalSettings::MESICNI_VYSLEDKY_ZOBRAZIT) == 2 ) {
                    // vzorce povoleny
                    $h = $this->vypoctiHodnotu($slozky, $this->expressions[$index], 'h');
                    $d = $this->vypoctiHodnotu($slozky, $this->expressions[$index], 'd');
                    $k = $this->vypoctiHodnotu($slozky, $this->expressions[$index], 'k');
                } else {
                    // vzorce nepovoleny
                    $data = $slozky[$this->expressions[$index]];
                    if ($data) {
                        $h = $data['h'];
                        $d = $data['d'];
                        $k = $data['k'];
                    }
                }
            } else {
                $emptyrow = true;
            }

            if (!RonPortalSettings::getIntValue('skrytnulovemesicni') || $emptyrow || $h || $d || $k) {
                $classname =
                    ($index % 2) ? 'browsercolor1' : 'browsercolor2';

                $classname .=
                    ' mv' . StringUtils::htmlTextConvert(str_replace(' ', '_', $this->expressions[$index]));

                $rowdata = [$rowname, $h, $d, $k];

                $this->renderLine($rowdata, $classname, $index);
            }
        }
    }


    function load($sqlFilter = '')
    {
        if ($this->showFilter) {
            $this->setFilterForm();
            $this->filterForm->loadData();
        }

        StringUtils::parseStringToArrays(
            RonPortalSettings::getValue(RonPortalSettings::MESICNI_VYSLEDKY_RADKY),
            $this->rowCaptions, $this->expressions, true);

        $cursor = $this->nactiMesicniVysledky();

        if ( RonPortalSettings::getIntValue(RonPortalSettings::MESICNI_VYSLEDKY_ZOBRAZIT) == 2 ) {
            // povoleny vyrazy => nacita do retezce PHP promennych
            $this->slozky = $this->nactiDataDoPromennych($cursor);
        } else {
            // nejsou povoleny vyrazy => nacita do pole
            $this ->slozky = $this ->nactiDataDoPole($cursor);
        }
    }


    function render()
    {
        $this->renderHeader();

        $this->renderTableStart();
        echo '<tbody>';
        $this->vypisMesicniVysledky($this ->slozky);
        echo "</tbody>\n";
        $this->renderTableEnd();
    }

}
