<?php

/**
 * UTF8
 */

namespace Ron\RonPortal;

use Ron\DateUtils;
use Ron\DB\SqlStatement;
use Ron\StringUtils;

require_once 'dochazka/b_vysledkybrowser.php';


class MesicniVysledkyRBrowser extends VysledkyBrowser
{
    function __construct($database)
    {
        parent::__construct($database, 'mesicnivysledkyr');

        $this->title = \Ron\ronText('Mesicni_vysledky_podle_rozdeleni');

        $this->sqlFromPart = 'MesicniVysledek_R, TypCasoveSlozkyMzdy, Rozdeleni';
        $this->sqlWherePart =
            'TypCasoveSlozkyMzdy.Kod = MesicniVysledek_R.TypCasoveSlozkyMzdy ' .
            ' AND Rozdeleni.KodTypuRozdeleni = MesicniVysledek_R.KodTypuRozdeleni ' .
            ' AND Rozdeleni.Kod = MesicniVysledek_R.Rozdeleni ';

        $this->cssClass .= ' mesicnivysledkyr';

        global $STR_Rozdeleni; // kompat.

        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'MesicniVysledek_R.TypCasoveSlozkyMzdy', BROWSER_FIELD_HEADER_TEXT => 'Kod_casove_slozky', BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'TypCasoveSlozkyMzdy.Popis', BROWSER_FIELD_HEADER_TEXT => 'Casova_slozka', BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'MesicniVysledek_R.Hodiny', BROWSER_FIELD_HEADER_TEXT => 'Vysledek_Hodiny',
            BROWSER_FIELD_FORMAT => 'hodiny', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_CLASS_NAME => 'hodiny'] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'MesicniVysledek_R.Dny', BROWSER_FIELD_HEADER_TEXT => 'Vysledek_Pracovni_dny',
            BROWSER_FIELD_FORMAT => 'number_2', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_CLASS_NAME => 'dny'] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'MesicniVysledek_R.KalendarniDny', BROWSER_FIELD_HEADER_TEXT => 'Vysledek_Kalendarni_dny',
            BROWSER_FIELD_FORMAT => 'number_2', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_CLASS_NAME => 'kalendarnidny'] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'Rozdeleni.Popis', BROWSER_FIELD_HEADER => $STR_Rozdeleni ?: \Ron\ronText('Rozdeleni'), BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'MesicniVysledek_R.Rozdeleni', BROWSER_FIELD_ORDERING => 1] );

        $this->defaultSort = 1;
        $this->primaryField = 0;
    }


    function setFilterForm()
    {
        parent::setFilterForm();
        $this->filterForm->osobyFiltr = 'PracovniPomer.ZpracovatDochazku = 1';
        $this ->filterForm ->downloadFotoOsobyUrl = '?menuid=editace&action=foto&idpp=';
        $this ->filterForm ->nastavAkciGet('mesicnivysledkyr');
    }

    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);

        $this ->browserUrl = '?menuid=mesicnivysledkyr&idpp=' . $this ->filterForm ->vybranaOsoba .
            '&obdobi=' . DateUtils::dateToUrlString($this ->filterForm ->vybraneObdobiOd);
    }


    function renderFilter()
    {
        $this->odkazyMesicniVysledky('mesicnivysledkyr');
        parent::renderFilter();
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();

        return new SqlStatement(
            'MesicniVysledek_R.Mesic = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybraneObdobiOd)) .
            ' AND MesicniVysledek_R.RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
            ' AND MesicniVysledek_R.OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
            ' AND MesicniVysledek_R.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)),
            $params);
    }

    function getRowAttributes($values, $className, $rowNumber)
    {
        // prida class mv_<kod_casove_slozky>
        $mvClass = 'mv_' . StringUtils::htmlTextConvert(str_replace(' ', '_', $values[0]));
        $className = ($className ? $className . ' ' : '') . $mvClass;

        return parent::getRowAttributes($values, $className, $rowNumber);
    }

}

