<?php

/**
 * UTF8
 */

namespace Ron\RonPortal;

use Ron\Browser;
use Ron\DateUtils;
use Ron\DB\Cursor;
use Ron\StringUtils;

require_once('ron/browser.php');
require_once('adsapp/ronportalfilter.php');
require_once('dochazka/kalendar.php');


class PoctyLidiBrowser extends Browser
{
    const SETTINGS_ZOBRAZOVAT_NULOVY_POCET = 'zobrazovatnulovypocet';
    const SETTINGS_SLOUPCE = 'sloupcepoctulidi';

    var $expressions; // pole vyrazu pro vypocet jednotlivych polozek
    var $soucet;      // pole souctu jednotlivych polozek
    var $zpracovanodo;
    var $zkontrolovanodo;
    var $schvalenodo;

    var $data;
    /** @var  RonPortalFilterForm */
    var $filterForm;


    function __construct($database)
    {
        parent::__construct($database);

        $this->cssClass = 'browser grid';

        $this->title = \Ron\ronText('Pocty_zamestnancu');

        $this ->parseFieldsString(RonPortalSettings::getValue(self::SETTINGS_SLOUPCE));
    }


    /**
     * Vytvori pole hlavicek sloupcu a pole vyrazu pro sloupce
     * @param string $str
     */
    function parseFieldsString($str)
    {
        $this->expressions = [];

        $this ->addField( [BROWSER_FIELD_SQL_NAME => '1', BROWSER_FIELD_HEADER_TEXT => 'Date', BROWSER_FIELD_ALIGN => 'center'] );

        $captions = [];
        StringUtils::parseStringToArrays($str, $captions, $this->expressions, false);

        foreach ($captions as $caption) {
            $this ->addField( [BROWSER_FIELD_SQL_NAME => '1', BROWSER_FIELD_HEADER => $caption, BROWSER_FIELD_ALIGN => 'right'] );
        }

        foreach ($this->expressions as $i => $expression)
            $this->expressions[$i] = str_replace('_', '$_', $expression);
    }


    function nactiPoctyLidi()
    {
        $params = $this ->database ->createParameters();

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)  &&  RonPortalSession::rozdeleni() !== null) {
            $sql =
                'SELECT DenniVysledekVypoctu.Den, DenniVysledekVypoctu.TypCasoveSlozkyMzdy, SUM(DenniVysledekVypoctu.Dny)' .
                ' FROM DenniVysledekVypoctu, RozdeleniOsob' .
                ' WHERE DenniVysledekVypoctu.RC = RozdeleniOsob.RC' .
                ' AND DenniVysledekVypoctu.OsobniCislo = RozdeleniOsob.OsobniCislo' .
                ' AND DenniVysledekVypoctu.DatumNastupu = RozdeleniOsob.DatumNastupu' .
                ' AND RozdeleniOsob.KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) .
                ' AND RozdeleniOsob.Rozdeleni = ' . $params->addParameter(RonPortalSession::rozdeleni()) .
                ' AND RozdeleniOsob.PlatnostOd <= DenniVysledekVypoctu.Den' .
                ' AND (RozdeleniOsob.PlatnostDo >= DenniVysledekVypoctu.Den OR RozdeleniOsob.PlatnostDo IS NULL)' .
                ' AND DenniVysledekVypoctu.Den >= ' . $params ->addParameter($this ->database ->dateToParam(RonPortalSession::obdobi())) .
                ' AND DenniVysledekVypoctu.Den <= ' . $params ->addParameter($this ->database ->dateToParam(RonPortalSession::konecObdobi())) .
                ' GROUP BY DenniVysledekVypoctu.Den, DenniVysledekVypoctu.TypCasoveSlozkyMzdy' .
                ' ORDER BY DenniVysledekVypoctu.Den';
        } else {
            $sql =
                'SELECT DenniVysledekVypoctu.Den, DenniVysledekVypoctu.TypCasoveSlozkyMzdy, SUM(DenniVysledekVypoctu.Dny)' .
                ' FROM DenniVysledekVypoctu' .
                ' WHERE DenniVysledekVypoctu.Den >= ' . $params ->addParameter($this ->database ->dateToParam(RonPortalSession::obdobi())) .
                ' AND DenniVysledekVypoctu.Den <= ' . $params ->addParameter($this ->database ->dateToParam(RonPortalSession::konecObdobi())) .
                ' GROUP BY DenniVysledekVypoctu.Den, DenniVysledekVypoctu.TypCasoveSlozkyMzdy' .
                ' ORDER BY DenniVysledekVypoctu.Den';
        }

        return $this ->database ->select($sql, $params);
    }


    function vytvorRadek($datum, $valuesStr, $jeToSoucet)
    {
        $browserrow = [];

        if (!$jeToSoucet) {
            // V nulte polozce radku ulozi typ dne 0=pracovni, 1=vikend, 2=svatek
            $browserrow[] = Kalendar::vratTypDne($this ->database, $datum);

            // Vytvori hodnotu v prvnim sloupci - datum
            $browserrow[] = date('j.n.Y', $datum);
        }

        // Vytvori jednotlive promenne pro vypocty a doplni jejich hodnoty
        eval($valuesStr);

        $vysledek = null;

        // Provede jednotlive vypocty
        foreach ($this->expressions as $index => $expression) {
            if ($expression) eval('$vysledek=' . $expression . ';');
            else $vysledek = null;

            if (!$jeToSoucet && !$vysledek && !RonPortalSettings::getIntValue(self::SETTINGS_ZOBRAZOVAT_NULOVY_POCET)) $browserrow[] = '';
            else $browserrow[] = StringUtils::format('number2', $vysledek);

            if (!$jeToSoucet) $this->soucet[$index] += $vysledek;
        }

        return $browserrow;
    }


    /**
     * @param Cursor $cursor
     * @return array
     */
    function prevedDataDoPole($cursor)
    {
        $this->soucet = [];
        $datum = RonPortalSession::obdobi();
        $posledniDatum = RonPortalSession::konecObdobi();
        $result = [];

        $zaznam = $cursor ->fetch();
        // prevede data na zacatek, resp. konec dne
        $datum = DateUtils::startOfTheDay($datum);
        $posledniDatum = DateUtils::endOfTheDay($posledniDatum);

        // Pro kazdy den vytvori radek
        while ($datum <= $posledniDatum) {
            // Prida do vypoctu vsechny zaznamy z daneho dne
            $valuesStr = '';
            $dalsidatum = DateUtils::incDayStart($datum, 1);

            while ($zaznam && $this ->database ->stringToTimestamp($zaznam[0]) < $dalsidatum) {
                $valuesStr = $valuesStr . '$_' . $zaznam[1] . '+=' . $zaznam[2] . ';';
                $zaznam = $cursor ->fetch();
            }

            $result[] = $this->vytvorRadek($datum, $valuesStr, false);
            $datum = DateUtils::startOfTheDay($datum + 86400 + 3600); // takhle je to kvuli prechodu letni/zimni cas
        }

        $cursor ->close();

        return $result;
    }


    function load($sqlFilter = '')
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, 'pocty');
        $this ->filterForm ->setUseGet(true, 'pocty');
        $this ->filterForm ->nastavObdobi(true, 'ADS', false); // jenOtevrene = false
        $this ->filterForm ->nastavRozdeleni(true);
        $this->filterForm->showVcetnePodrizenychRozdeleni = false;
        $this ->filterForm ->loadData();

        Prava::nastavObdobiProPrava(RonPortalSession::obdobi(), RonPortalSession::konecObdobi());

        $cursor = $this ->nactiPoctyLidi();
        $this ->data = $this ->prevedDataDoPole($cursor);
    }


    function renderFilter()
    {
        $this ->filterForm ->render();
    }


    function vypisPoctyLidi()
    {
        foreach ($this ->data as $index => $row) {
            if ($row[0]) echo "<tr class='browser_row_high'>";
            else echo '<tr>';

            // Vypise obsah radku
            $prvnidatovysloupec = 1;
            for ($i = $prvnidatovysloupec; $i < count($row); $i++) {
                if ($i == $prvnidatovysloupec) echo '<td class="browser_rowheader">'; // datum
                else echo '<td class="align-right">';

                echo $row[$i];
                echo '</td>';
            }
            echo "</tr>\n";
        }
    }


    function vypisSoucet()
    {
        echo '<tr class="browserfooter">';
        echo '<td>' . \Ron\ronText('Celkem') . '</td>';
        foreach ($this->soucet as $index => $value) {
            echo '<td class="align-right"> ' . StringUtils::format('number2', $value) . '</td>';
        }
        echo "</tr>\n";
    }


    function render()
    {
        $this->renderHeader();
        $this->renderTableStart();
        echo "<tbody class='border-bottom'>\n";
        $this->vypisPoctyLidi();
        echo "</tbody>\n";
        echo "<tbody>\n";
        $this->vypisSoucet();
        echo "</tbody>\n";
        $this->renderTableEnd();
    }

}

