<?php
/**
 * UTF8
 * stanislav.elbl, 16.11.2014
 */

namespace Ron\RonPortal;

require_once('ron/controls.php');
require_once('ron/browser.php');
require_once('dochazka/schvalenidochazky.php');
require_once('adsapp/ronportalbrowser.php');
require_once('adsapp/ronportalfilter.php');

use Ron\BrowserField;
use Ron\Browser;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\HttpUtils;
use Ron\StringUtils;


class SchvalovaniFilterForm extends RonPortalFilterForm
{
    function __construct($database)
    {
        parent::__construct($database, 'schvalovani', true);

        $this ->nastavObdobi(true, 'ADS', false);  // jenotevrene = false
        $this ->nastavRozdeleni(true);

        $this ->f_pravarozdeleni = Prava::vratPrava('prohlizeni');

        if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP) ) {
            $this ->nastavRozdeleni2(true);
        }
    }


    function loadData()
    {
        parent::loadData();

        if (isset($_GET['urovenschvaleni'])) {
            $actualvalue = (int)$_GET['urovenschvaleni'];
            $_SESSION['filtr_urovenschvaleni'] = $actualvalue;
        }
    }


    function renderFilterStav()
    {
        echo '<div class="filterrow" id="filter_urovenschvaleni">';
        echo '<span class="filtercaption"></span>';

//        if (isset($_GET['urovenschvaleni'])) {
//            $actualvalue = (int)$_GET['urovenschvaleni'];
//            $_SESSION['filtr_urovenschvaleni'] = $actualvalue;
//        } else {
//            $actualvalue = (int)$_SESSION['filtr_urovenschvaleni'];
//        }


        global $STR_Neschvaleno1; // jen pro kompatibilitu starych prekladu
        global $STR_Neschvaleno2;
        global $STR_Neschvaleno3;

        LinksList::renderLinks([
                0 => \Ron\ronText('Vsechno'),
                1 => $STR_Neschvaleno1 ?: \Ron\ronText('Neschvaleno1'),
                2 => $STR_Neschvaleno2 ?: \Ron\ronText('Neschvaleno2'),
                3 => $STR_Neschvaleno3 ?: \Ron\ronText('Neschvaleno3')
            ],
            (int)$_SESSION['filtr_urovenschvaleni'], '?menuid=schvalovani&urovenschvaleni=', '', false);

        echo "</div>\n";
    }


    function renderFilters()
    {
        parent::renderFilters();

        // vsechno / nezpracovano / nezkontrolovano / neschvaleno
        $this->renderFilterStav();
    }
}




class Schvalovani_RozdeleniField extends BrowserField
{
    function __construct()
    {
        parent::__construct();

        $this ->code = 'col-rozdeleni';

        $this ->align = 'left';
        $this ->ordering = 1;
        $this ->header = RonPortalSettings::getValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_NADPIS);

        switch (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT)) {
            case 0 :
                $this->sqlFieldName = 'Rozdeleni.Popis';
                break;

            case 1 :
            case 2 :
                $this->sqlFieldName = 'Rozdeleni.Kod';
                break;
        }
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        switch (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT)) {
            case 0 :
                // popis
                return $rowData[18];

            case 1 :
                // kod
                return $rowData[17];

            case 2 :
                if (isset($rowData[17])  &&  isset($rowData[18])) return $rowData[17] . ' - ' . $rowData[18];
                return $rowData[17] . $rowData[18];
        }

        return '';
    }

}



class SchvaleniDochazkyField extends BrowserField
{
    var $indexData;
    var $indexUzivatel;
    var $indexCas;

    /**
     * @param Database $database
     * @param int $indexData
     * @param int $indexUzivatel
     * @param int $indexCas
     */
    function __construct($database, $indexData, $indexUzivatel, $indexCas)
    {
        parent::__construct($database);

        $this -> align = 'right';
        $this -> ordering = 1;

        $this -> indexData = $indexData;
        $this -> indexUzivatel = $indexUzivatel;
        $this -> indexCas = $indexCas;
    }



    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ( !$rowData[$this -> indexData] ) return '';

        $result = DBUtils::getFormatedValue($this ->database, 'date', $rowData[$this -> indexData]);

        if ( $rowData[$this -> indexUzivatel] )
            $result =
                '<SPAN title="'.$rowData[$this -> indexUzivatel].
                ' (' . DBUtils::getFormatedValue($this ->database, 'finedatetime', $rowData[$this -> indexCas]).')">' . $result . '</SPAN>';

        return $result;
    }
}


class SchvaleniDochazkyEditaceField extends BrowserField
{
    function __construct()
    {
        parent::__construct();

        $this ->code = 'col-edit';
        $this -> header = '';
        $this -> align = 'left';
        $this -> ordering = 0;

        //$this -> f_prava = VratPrava('editace');
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //if (!$this -> f_prava -> LzeZobrazitPP($rowdata[0], S_GetZacatekObdobi(), S_GetKonecObdobi())) return '';

        return
            '<A onclick="StopEventPropagation(event)" target="zpracovanadochazkawin" href="?menuid=zpracovanadochazka&osoba='.$rowData[0].'&obdobi=' . DateUtils::dateToUrlString(RonPortalSession::obdobi()).'">' .
            \Ron\ronText('dochazka') .
            '</A>';
    }
}


class SchvalovaniDochazkyBrowser extends Browser
{
    var $fieldJmeno;
    var $fieldPrijmeni;
    var $fieldOsobniCislo;

    var $filterForm;


    function __construct($database)
    {
        global $STR_ZpracovanoDo;   // kompat
        global $STR_ZkontrolovanoDo;// kompat
        global $STR_SchvalenoDo;    // kompat

        parent::__construct($database);

        $this ->title = \Ron\ronText('Schvalovani');
        $this ->code = 'browserschvaldoch';
        $this ->browserUrl = '?menuid=schvalovani&';

        $this ->sqlSelectPart =
            'PracovniPomer.ID, Osoba.RC, ' . // 0,1
            'SchvaleniDochazky.ZpracovanoDo, SchvaleniDochazky.Zpracoval, SchvaleniDochazky.DT_Zpracovani, ' . // 2..4
            'SchvaleniDochazky.ZkontrolovanoDo, SchvaleniDochazky.Zkontroloval, SchvaleniDochazky.DT_Zkontrolovani, ' . // 5..7
            'SchvaleniDochazky.SchvalenoDo, SchvaleniDochazky.Schvalil, SchvaleniDochazky.DT_Schvaleni, '. // 8..10
            'Osoba.Titul, Osoba.Prijmeni, Osoba.Jmeno, PracovniPomer.OsobniCislo, '. // 11..14
            (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1) ? RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1) : 'null') . ', ' . // 15
            (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2) ? RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2) : 'null'); // 16

        if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) ) {
            $this->sqlSelectPart .= ', Rozdeleni.Kod, Rozdeleni.Popis'; // 17,18
        }

        $this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'PracovniPomer.ID', BROWSER_FIELD_SQL_INDEX => 0] );
        $this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'Osoba.RC', BROWSER_FIELD_SQL_INDEX => 1] );

        $this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.ZpracovanoDo', BROWSER_FIELD_SQL_INDEX => 2] );
        $this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.Zpracoval', BROWSER_FIELD_SQL_INDEX => 3] );
        $this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.DT_Zpracovani', BROWSER_FIELD_SQL_INDEX => 4] );

        $this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.ZkontrolovanoDo', BROWSER_FIELD_SQL_INDEX => 5] );
        $this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.Zkontroloval', BROWSER_FIELD_SQL_INDEX => 6] );
        $this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.DT_Zkontrolovani', BROWSER_FIELD_SQL_INDEX => 7] );

        $this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.SchvalenoDo', BROWSER_FIELD_SQL_INDEX => 8] );
        $this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.Schvalil', BROWSER_FIELD_SQL_INDEX => 9] );
        $this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.DT_Schvaleni', BROWSER_FIELD_SQL_INDEX => 10] );

        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1)) {
            $this->addField([
                BROWSER_FIELD_SQL_NAME => RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1),
                BROWSER_FIELD_CODE => 'col-pole1',
                BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_SQL_INDEX => 15, 'htmlConvert' => false] );
        }

        $this->addField([
            BROWSER_FIELD_SQL_NAME => 'Osoba.Titul', BROWSER_FIELD_SQL_INDEX => 11,
            BROWSER_FIELD_CODE => 'col-titul', BROWSER_FIELD_HEADER_TEXT => 'Titul', BROWSER_FIELD_ORDERING => 1, ] );

        // zapamatuje si pole, aby mohl spravne nastavit link podle zvoleneho obdobi (nutno nastavit az po zobrazeni filtru)
        $this->fieldPrijmeni = $this->addField([
            BROWSER_FIELD_SQL_NAME => 'Osoba.Prijmeni', BROWSER_FIELD_HEADER_TEXT => 'Prijmeni', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_SQL_INDEX => 12,
            BROWSER_FIELD_CODE => 'col-prijmeni',
            'sortSQL' => 'Prijmeni, Jmeno', 'sortSQLDesc' => 'Prijmeni DESC, Jmeno DESC'] );

        $this->defaultSort = count($this->fields);

        $this->fieldJmeno = $this->addField([
            BROWSER_FIELD_SQL_NAME => 'Osoba.Jmeno', BROWSER_FIELD_SQL_INDEX => 13,
            BROWSER_FIELD_CODE => 'col-jmeno',
            BROWSER_FIELD_HEADER_TEXT => 'Jmeno', BROWSER_FIELD_ORDERING => 1] );
        $this->fieldOsobniCislo = $this->addField([
            BROWSER_FIELD_SQL_NAME => 'PracovniPomer.OsobniCislo', BROWSER_FIELD_SQL_INDEX => 14,
            BROWSER_FIELD_CODE => 'col-osobnicislo',
            BROWSER_FIELD_HEADER_TEXT => 'Osobni_cislo', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1] );

        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2)) {
            $this->addField([
                BROWSER_FIELD_SQL_NAME => RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2), BROWSER_FIELD_SQL_INDEX => 16,
                BROWSER_FIELD_CODE => 'col-pole2',
                BROWSER_FIELD_ORDERING => 1, 'htmlConvert' => false]);
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP)) {
            $this->addField(new Schvalovani_RozdeleniField());
        }


        $f = new SchvaleniDochazkyField($this ->database, 2, 3, 4);
        $f->sortSQL = 'SchvaleniDochazky.ZpracovanoDo';
        $f->code = 'col-schvaleni1';
        $f->header = $STR_ZpracovanoDo ?: \Ron\ronText('Dochazka_schvaleni_1_do');
        $f->className = 'textschvaleni1';
        $this->addField($f);

        $f = new SchvaleniDochazkyField($this ->database, 5, 6, 7);
        $f->sortSQL = 'SchvaleniDochazky.ZkontrolovanoDo';
        $f->code = 'col-schvaleni2';
        $f->header = $STR_ZkontrolovanoDo ?: \Ron\ronText('Dochazka_schvaleni_2_do');
        $f->className = 'textschvaleni2';
        $this->addField($f);

        $f = new SchvaleniDochazkyField($this ->database, 8, 9, 10);
        $f->sortSQL = 'SchvaleniDochazky.SchvalenoDo';
        $f->code = 'col-schvaleni3';
        $f->header = $STR_SchvalenoDo ?: \Ron\ronText('Dochazka_schvaleni_3_do');
        $f->className = 'textschvaleni3';
        $this->addField($f);

        $this->addField(new SchvaleniDochazkyEditaceField());

        $this->sqlWherePart =
            '   PracovniPomer.Zamestnanec > 0 ' .   // nezobrazuje externi osoby
            '   AND PracovniPomer.ZpracovatDochazku = 1 ';

        $filtrprava = Prava::sqlFiltrPodlePrav(array(Prava::vratPrava('zpracovani'), Prava::vratPrava('zkontrolovani'), Prava::vratPrava('schvaleni')), 'PracovniPomer', RonPortalSession::obdobi(), RonPortalSession::konecObdobi());
        if ($filtrprava) $this ->sqlWherePart .= ' AND (' . $filtrprava . ')';

        $this->sqlOrderPart = '';
        $this->primaryField = 1;

        $this -> filterForm = new SchvalovaniFilterForm($this ->database);
        $this ->loadConfigurationXml(RonPortalBrowser::loadBrowserConfiguration($this ->database, $this ->code));
    }


    function load($sqlFilter = '')
    {
        $this->filterForm ->loadData();

        // sql dotaz se nastavi az po vyberu obdobi
        $this->sqlFromPart =
            '( Osoba INNER JOIN PracovniPomer ' .
            '  ON Osoba.RC = PracovniPomer.RC ' .
            ') ' .
            'LEFT OUTER JOIN SchvaleniDochazky ' .
            '  ON SchvaleniDochazky.RC = PracovniPomer.RC ' .
            ' AND SchvaleniDochazky.OsobniCislo = PracovniPomer.OsobniCislo ' .
            ' AND SchvaleniDochazky.DatumNastupu = PracovniPomer.DatumNastupu ' .
            " AND SchvaleniDochazky.Aplikace = 'ADS' " .
            ' AND SchvaleniDochazky.PocatekObdobi = ' . $this ->database ->timestampToString(RonPortalSession::obdobi());

        if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) ) {
            $this->sqlFromPart =
                '( ( ' .
                $this->sqlFromPart .
                ') ' .
                'LEFT OUTER JOIN RozdeleniOsob ' .
                ' ON RozdeleniOsob.RC = PracovniPomer.RC ' .
                ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                ' AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) .
                ' AND RozdeleniOsob.PlatnostOd <= ' . $this ->database -> timestampToString(RonPortalSession::konecObdobi()) .
                ' AND (RozdeleniOsob.PlatnostDo >= ' . $this ->database -> timestampToString(RonPortalSession::konecObdobi()) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ') ' .
                ' LEFT OUTER JOIN Rozdeleni ' .
                ' ON Rozdeleni.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) .
                ' AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni ';
        }

        parent::load($sqlFilter);
    }


    function renderFilter()
    {
        $this ->filterForm ->render();

        $params = '&obdobi=' . DateUtils::dateToUrlString(RonPortalSession::obdobi());

        require_once 'ron/httputils.php';
        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) {
            $params .= '&rozdeleni=' . HttpUtils::urlEncode(RonPortalSession::rozdeleni());
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
            $params .= '&rozdeleni2=' . HttpUtils::urlEncode(RonPortalSession::rozdeleni2());
        }

        echo '<A href="?menuid=schvalovani&hromadneschvaleni=1' . $params . '">' . \Ron\ronText('Schvalit_hromadne') . '</A><BR>';
    }


    function getSQLFilter()
    {
        $result =
            ' PracovniPomer.DatumNastupu < ' . $this ->database -> timestampToString(RonPortalSession::konecObdobi()) . // jen platne PP
            ' AND ( PracovniPomer.DatumUkonceni >= ' . $this ->database -> timestampToString(RonPortalSession::obdobi()) .
            ' OR PracovniPomer.DatumUkonceni IS NULL )';

        if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) && RonPortalSession::rozdeleni() !== null ) {
            $filtr = $this->filterForm->vratSQLFiltrRozdeleni('PracovniPomer', RonPortalSession::rozdeleniVcetnePodrizenych());

            if ($filtr) $result .= ' AND (' . $filtr . ')';
        }


        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP) && RonPortalSession::rozdeleni2()) {
            require_once('adsapp/rozdeleni.php');
            $filtr = Rozdeleni::sqlFilterRozdeleni($this ->database, 'PracovniPomer',
                RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), RonPortalSession::rozdeleni2(),
                RonPortalSession::obdobi(), RonPortalSession::konecObdobi(), RonPortalSession::rozdeleni2VcetnePodrizenych()); //

            if ($filtr) $result .= ' AND (' . $filtr . ')';
        }

        switch ($_SESSION['filtr_urovenschvaleni']) {
            case 1 :
                $filtruroven = '(SchvaleniDochazky.ZpracovanoDo IS NULL  OR  SchvaleniDochazky.ZpracovanoDo < ' . $this ->database ->dateToString(RonPortalSession::konecObdobi()) . ')';
                break;
            case 2 :
                $filtruroven = '(SchvaleniDochazky.ZkontrolovanoDo IS NULL  OR  SchvaleniDochazky.ZkontrolovanoDo < ' . $this ->database ->dateToString(RonPortalSession::konecObdobi()) . ')';
                break;
            case 3 :
                $filtruroven = '(SchvaleniDochazky.SchvalenoDo IS NULL  OR  SchvaleniDochazky.SchvalenoDo < ' . $this ->database ->dateToString(RonPortalSession::konecObdobi()) . ')';
                break;

            default :
                $filtruroven = null;
                break;
        }

        if ($filtruroven) $result .= ' AND ' . $filtruroven;


        return $result;
    }


    function getRowAttributes($values, $className, $rowNumber)
    {
        $attrs = parent::getRowAttributes($values, $className, $rowNumber);
        $onclick = "ronPortal.dochazka.zobrazSchvaleniDochazky({$values[0]}, '". date('d.m.Y', RonPortalSession::obdobi()) . "'); return false;";
        $attrs .= ' onclick="' . $onclick . '"  style="cursor: pointer"';

        return $attrs;
    }


    function enableLine($values)
    {
        return SchvaleniDochazky::lzeSchvalovatOsobu($values[0], $this ->filterForm ->vybraneObdobiOd, $this ->filterForm ->vybraneObdobiDo);
    }
}


