<?php
/**
 * UTF8
 * stanislav.elbl, 07.09.2015
 */

namespace Ron\RonPortal;

use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Controller;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;


require_once('adsapp/ppomer.php');



class BlokovaniController implements Controller
{

    /** @var  Database */
    var $database;

    /**
     * BlokovaniController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }



    function vyberOsobyProBlokovani($action)
    {
        if (!Prava::vratHodnotuPrava($action)) {
            RonPortalUtils::neniPravoException();
        }

        Model::addData('action', $action);

        require_once('adsapp/ronportalcontrols.php');

        $combo = new ComboBoxZamestnanec('osoba'); // jmeno promenne
        $combo->loadDataPP($this ->database,
            time(), time(), Prava::vratPrava($action),
            false, RonPortalSession::vybranaOsoba(), true); // $emptyrow = false, $selectedvalue, $vcetneexternich = true
        $combo->nastavOsobuDoSession();

        Model::addData('comboOsoba', $combo);

        return new BaseViewObject('dochazka/blokovani_vyberosoby.php');
    }


    function vyberKartyProBlokovani($action)
    {
        $prava = Prava::vratPrava($action);

        if (!$prava->jsouNejakePrava()) {
            RonPortalUtils::neniPravoException();
        }

        $pp = new PracovniPomer();
        if (!$pp->nactiPPPodleId($this ->database, $_POST['osoba'])) {
            throw new \Exception('Chyba! Zadany pracovni pomer nebyl nalezen!');
        }

        if (!$pp->vyberNactenyPP($this ->database, $prava)) {
            RonPortalUtils::neniPravoException();
        }

        if (!RonPortalSession::vybranaOsoba()) {
            MessageManager::addErrorMessage( \Ron\ronText('Nutno_vybrat_osobu') );
            return $this ->vyberOsobyProBlokovani($action);
        }

        Model::addData('PP', $pp);
        Model::addData('action', $action);

        $datum = DateUtils::startOfTheDay(time());

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT PrirazeniKarty.kodKarty, Karta.OznaceniMedia ' .
            ' FROM PrirazeniKarty, PracovniPomer, Karta ' .
            ' WHERE PracovniPomer.RC = PrirazeniKarty.RC ' .
            ' AND PrirazeniKarty.osobniCislo = PracovniPomer.osobniCislo ' .
            ' AND PrirazeniKarty.datumNastupu = PracovniPomer.datumNastupu ' .
            ' AND Karta.Kod = PrirazeniKarty.kodKarty ' .
            ' AND PrirazeniKarty.RC = ' . $params ->addParameter(RonPortalSession::osobaRC()) .
            ' AND PracovniPomer.datumNastupu <= ' . $params ->addParameter($this ->database ->timestampToParam($datum)) .
            ' AND (PracovniPomer.datumUkonceni IS NULL OR PracovniPomer.datumUkonceni >= ' . $params ->addParameter($this ->database ->timestampToParam($datum)) . ')' .

            ' AND PrirazeniKarty.datumPrideleni <= ' . $params ->addParameter($this ->database ->timestampToParam($datum)) .
            ' AND (PrirazeniKarty.datumVraceni >= ' . $params ->addParameter($this ->database ->timestampToParam($datum)) . ' OR PrirazeniKarty.datumVraceni IS NULL)';

        if ($action == 'odblokovani') $sql .= ' AND Karta.blokovana = 1 ';
        else $sql .= ' AND (Karta.blokovana = 0   OR  Karta.blokovana IS NULL)';

        $sql .= ' ORDER BY PrirazeniKarty.kodKarty';

        require_once('ron/controls.php');

        $combo = new ComboBox('karta');
        $combo ->loadData($this ->database, $sql, '', $params);
        Model::addData('comboKarta', $combo);

        return new BaseViewObject('dochazka/blokovani_vyberkarty.php');
    }


    /**
     * Zobrazeni potvrzovaciho formulare
     * @param string $action
     * @return BaseViewObject
     * @throws \Exception
     */
    function potvrzeniBlokovani($action)
    {
        $prava = Prava::vratPrava($action);

        $pp = new PracovniPomer();
        if (!$pp->nactiPPPodleId($this ->database, $_POST['osoba'])) {
            throw new \Exception('Chyba! Zadany pracovni pomer nebyl nalezen!');
        }

        if (!$prava->lzeZobrazitPP($pp->idPP, time(), time())) {
            RonPortalUtils::neniPravoException();
        }

        if (!$_POST['karta']) {
            MessageManager::addErrorMessage(\Ron\ronText('Nutno_vybrat_kartu'));
            return $this ->vyberKartyProBlokovani($action);
        }

        Model::addData('action', $action);
        Model::addData('prava', $prava);
        Model::addData('PP', $pp);

        return new BaseViewObject('dochazka/blokovani_potvrzeni.php');
    }


    function nastavPriznakBlokovani($karta, $akce)
    {
        require_once('adsapp/ronportaldbrecord.php');
        $rec = new RonPortalDBRecord($this ->database, 'Karta');
        $rec->logDescription = StringUtils::fromUTF8('Blokování karet');

        $rec->addField('Kod', '', true); // readonly
        $rec->addField('Kod2', '', true);
        $rec->addField('DatumPorizeni', 'date', true);
        $rec->addField('DatumVyrazeni', 'date', true);
        $rec->addField('OznaceniMedia', '', true);

        $rec->addField('Blokovana', 'int', false, 0, ($akce == 'odblokovani') ? '0' : '1'); // readonly = false

        $params = $this ->database ->createParameters();
        $rec->setWhere(' Kod = ' . $params ->addParameter($karta), $params);

        if (!$rec->loadRecord()) {
            throw new \Exception(\Ron\ronText('Chyba_pri_blokovani') . ' Nelze nacist puvodni zaznam!');
        }

        if (!$rec->updateRecord()) {
            throw new \Exception(\Ron\ronText('Chyba_pri_blokovani'));
        }

        return new TextViewObject(
            \Ron\ronText( ($akce == 'odblokovani') ? 'Odblokovani_probehlo' : 'Blokovani_probehlo' ));
    }


    function provedBlokovani($action)
    {
        $prava = Prava::vratPrava($action);

        if (!$prava->prava) {
            RonPortalUtils::neniPravoException();
        }

        $pp = new PracovniPomer();
        if (!$pp->nactiPPPodleId($this ->database, $_POST['osoba'])) {
            throw new \Exception ('Chyba! Zadany pracovni pomer nebyl nalezen!');
        }

        // kontrola prav
        if (!$pp->vyberNactenyPP($this ->database, $prava)) {
            RonPortalUtils::neniPravoException();
        }

        if (!RonPortalSession::vybranaOsoba()) {
            MessageManager::addErrorMessage( \Ron\ronText('Nutno_vybrat_osobu') );
            return $this ->vyberOsobyProBlokovani($action);
        }

        if (!$_POST['karta']) {
            MessageManager::addErrorMessage(\Ron\ronText('Nutno_vybrat_kartu'));
            return $this ->vyberKartyProBlokovani($action);
        }

        if ($prava->priznak) {
            // je zapnuta kontrola hesla
            require_once('adsapp/security.php');

            if (!Security::checkUserPassword($this ->database, User::getRC(), $_POST['heslo'])) {
                MessageManager::addErrorMessage( \Ron\ronText('Bad_password') );

                return $this->potvrzeniBlokovani($action);
            }
        }

        return $this ->nastavPriznakBlokovani($_POST['karta'], $action);
    }


    /**
     * @param string $action
     * @return string|Redirect|ViewObject
     */
    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('users.ini');
        TextManager::getInstance() ->loadStrings('dochazka.ini');

        Prava::nastavObdobiProPrava(time(), time());

        if (isset($_POST['akceblokovani'])) {
            switch ($_POST['akceblokovani']) {
                case 'vyberosoby' :
                    // byla vybrana osoba => nasleduje vyber karty
                    return $this ->vyberKartyProBlokovani($action);

                case 'vyberkarty' :
                    // byla vybrana karta => nasleduje potvrzeni vyberu
                    return $this ->potvrzeniBlokovani($action);

                case 'potvrzeniblokovani' :
                    return $this ->provedBlokovani($action);
            }
        }

        return $this ->vyberOsobyProBlokovani($action);
    }

}