<?php
/**
 * UTF8
 * stanislav.elbl, 25.01.2017
 */

namespace Ron\RonPortal;

use Ron\DB\Database;
use Ron\StringUtils;

class MesicniVysledek
{
    public $idPP;

    public $rc;
    public $osobniCislo;
    public $datumNastupu;

    /** @var int */
    public $mesic;
    /** @var string */
    public $typCasoveSlozkyMzdy;

    /** @var float */
    public $hodiny;
    /** @var float */
    public $dny;
    /** @var float */
    public $kalendarniDny;

    public $poznamka;


    /**
     * @param Database $database
     * @param array $row
     */
    private function load($database, $row)
    {
        $this ->idPP = $row[0];
        $this ->rc = $row[1];
        $this ->osobniCislo = $row[2];
        $this ->datumNastupu = $database ->stringToTimestamp($row[3]);

        $this ->mesic = $database ->stringToTimestamp($row[4]);
        $this ->typCasoveSlozkyMzdy = $row[5];

        $this ->hodiny = isset($row[6]) ? (float)$row[6] : null;
        $this ->dny = isset($row[7]) ? (float)$row[7] : null;
        $this ->kalendarniDny = isset($row[8]) ? (float)$row[8] : null;

        $this ->poznamka = $row[9];
    }


    /**
     * Nacte mesicni vysledky. Pokud se nacitaji data jen pro jednu osobu a mesic, tak vraci asociativni pole indexovane typem.
     * @param Database   $database
     * @param int|int[]  $idPP
     * @param int        $mesic
     * @param string|string[] [$typCasoveSlozkyMzdy]
     * @return MesicniVysledek[]
     */
    public static function nactiMesicniVysledky($database, $idPP, $mesic, $typCasoveSlozkyMzdy = null)
    {
        $result = [];

        $params = $database ->createParameters();
        $sql =
            'SELECT PracovniPomer.Id, PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, ' . // 0..3
            ' MesicniVysledekVypoctu.Mesic, MesicniVysledekVypoctu.TypCasoveSlozkyMzdy, ' . //4,5
            ' MesicniVysledekVypoctu.Hodiny, MesicniVysledekVypoctu.Dny, MesicniVysledekVypoctu.KalendarniDny, MesicniVysledekVypoctu.Poznamka' . // 6..9
            ' FROM MesicniVysledekVypoctu, PracovniPomer' .
            ' WHERE PracovniPomer.RC = MesicniVysledekVypoctu.RC' .
            '   AND PracovniPomer.OsobniCislo = MesicniVysledekVypoctu.OsobniCislo' .
            '   AND PracovniPomer.DatumNastupu = MesicniVysledekVypoctu.DatumNastupu';

        $jedenPP = false;

        if ($idPP) {
            if (is_array($idPP)) {

                $sql .= ' AND (0=1';
                foreach ($idPP as $id) {
                    $sql .= ' OR PracovniPomer.Id = ' . $params ->addParameter($id);
                }
                $sql .= ')';

            } else {
                $jedenPP = true;
                $sql .= ' AND PracovniPomer.Id = ' . $params->addParameter($idPP);
            }
        }

        if ($mesic) {
            $sql .=
                ' AND MesicniVysledekVypoctu.Mesic = ' . $params ->addParameter($database ->dateToParam($mesic));
        }

        if ($typCasoveSlozkyMzdy) {
            $sql .= ' AND';

            if (is_array($typCasoveSlozkyMzdy)) {
                $sql .= ' (';

                $first = true;
                foreach ($typCasoveSlozkyMzdy as $slozka) {
                    if (!$first) {
                        $sql .= ' OR';
                    }
                    $first = false;

                    $sql .= ' TypCasoveSlozkyMzdy = ' . $params ->addParameter($slozka);
                }

                $sql .= ')';

            } else {
                $sql .= ' TypCasoveSlozkyMzdy = ' . $params ->addParameter($typCasoveSlozkyMzdy);
            }
        }

        $sql .= ' ORDER BY PracovniPomer.Id, MesicniVysledekVypoctu.Mesic, MesicniVysledekVypoctu.TypCasoveSlozkyMzdy';

        $cursor = $database ->getSqlResult($sql, $params, 'Pri nacitani mesicnich vysledku doslo k chybe!');
        while ($row = $cursor ->fetch()) {
            $vysledek = new static();
            $vysledek ->load($database, $row);

            if ($jedenPP  &&  $mesic) {
                // data jen pro jednu osobu/mesic - vraci asociativni pole
                $result[$vysledek ->typCasoveSlozkyMzdy] = $vysledek;
            } else {
                $result[] = $vysledek;
            }
        }
        $cursor ->close();

        return $result;
    }


    /**
     * Nacte mesicni vysledek pro zadanou osobu, obdobi a casovou slozku
     * @param Database $database
     * @param int  $idPP
     * @param int  $mesic
     * @param string $typCasoveSlozkyMzdy
     * @return MesicniVysledek
     */
    public static function vratMesicniVysledek($database, $idPP, $mesic, $typCasoveSlozkyMzdy)
    {
        $data = self::nactiMesicniVysledky($database, $idPP, $mesic, $typCasoveSlozkyMzdy);

        return count($data) ? $data[$typCasoveSlozkyMzdy] : null;
    }

    /**
     * Vraci ciselnou hodnotu pro zadanou jednotku
     * @param $jednotka
     * @return float|string
     */
    function vratHodnotu($jednotka)
    {
        switch (strtoupper($jednotka)) {
            case 'D' :
                return $this ->dny;

            case 'K' :
                return $this ->kalendarniDny;

            case 'H' :
            case '' :
                return $this ->hodiny;
        }

        return '';
    }


    /**
     * @param string $jednotka H/D/K/P
     * @param string $format
     * @return string
     */
    function vratText($jednotka, $format = null)
    {
        switch (strtoupper($jednotka)) {
            case 'H' :
                return ($this ->hodiny) ? StringUtils::format($format ?: 'minuty', round($this ->hodiny * 60)) : '';

            case 'D' :
                return $this ->dny ? StringUtils::format($format ?: 'number_2', $this ->dny) : '';

            case 'K' :
                return $this ->kalendarniDny ? StringUtils::format($format ?: 'number_2', $this ->kalendarniDny) : '';

            case 'P' :
                return $this ->poznamka;
        }

        return '';
    }
}
