<?php
/**
 * UTF8
 */

namespace Ron\RonPortal;

require_once('ron/browser.php');
require_once('adsapp/ronportalfilter.php');
require_once('adsapp/ronportalbrowser.php');
require_once('adsapp/dochazkautils.php');

use Exception;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\DateInput;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\GuiUtils;
use Ron\Http\HtmlHeaders;
use Ron\Http\HttpResponse;
use Ron\MessageManager;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;


class MonitorovaniFilterForm extends RonPortalFilterForm
{
    const SETTINGS_JEN_HLAVNI_ROZDELENI = 'monitorovani_jennejvyssirozdeleni';

    var $aktualniStav;
    var $datumCas;

    public function __construct($database, $action, $kontrolovatprava)
    {
        parent::__construct($database, $action);

        $this ->nastavAkciGet($action);

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) {
            $jenvlastni = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_JEN_VLASTNI);
            if (!User::getRC()) $jenvlastni = 0; // pro monitorovani bez prihlaseni

            $filtr = RonPortalSettings::getIntValue(self::SETTINGS_JEN_HLAVNI_ROZDELENI) ? 'Rodic IS NULL' : '';
            $this ->nastavRozdeleni(true, 0, $jenvlastni, $filtr); // typRozdeleni=0, jinak by se musel zadavat i v RonPortalSession::rozdeleni()!

            if ($kontrolovatprava) $this->f_pravarozdeleni = Prava::vratPrava('monitorovani');
        }

        if (isset($_GET['datum'])) {
            $this->aktualniStav = $_GET['aktualnistav'] ? 1 : 0;
        } else {
            $this->aktualniStav = 1;
        }

        // Pri prvnim zobrazeni stranky (ne po volbe "nastavit") nastavi vzdy aktualni stav na true
        // Nelze tady testovat $_GET["aktualnistav"] protoze pri vypnuti checkboxu je NULL,
        // proto testuji "datum"

        if (isset($_GET['datum'])) {
            try {
                $datum = DateUtils::dateStringToTimestamp($_GET['datum']);
                $timeStamp = DateUtils::timeStringToTimestamp($_GET['cas'], $datum);
            } catch(Exception $e) {
                MessageManager::addErrorText( $e ->getMessage() );
                $timeStamp = time();
            }
        } else {
            $timeStamp = time();
        }

        $this ->datumCas = $timeStamp;
        $_SESSION['monitorovatcas'] = $timeStamp;

        if (isset($_GET['operace']))
            $_SESSION['monitorovat_operaci'] = $_GET['operace'];

        if (isset($_GET['terminal']))
            $_SESSION['monitorovat_terminal'] = $_GET['terminal'];
    }


    function renderAktualniStav()
    {
        // Checkbox na zaskrtnuti "aktualni stav"
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">' . \Ron\ronText('Aktualni_stav') . '</span>';
        require_once('ron/guiutils.php');
        GuiUtils::renderCheckbox('aktualnistav', '', $this->aktualniStav);

        echo '&nbsp;&nbsp;';

        if (!$this->aktualniStav) echo "<a href='" . $this ->f_formaction . "'>" . \Ron\ronText('Aktualni_stav') . "</a>";
        echo "</div>\n";
    }


    function renderDatumCas()
    {
        $timeStamp = $this ->datumCas; // nastaveno v konstruktoru podle GET
        $cas = date('G:i', $timeStamp);

        echo '<div class="filterrow">';
        echo '<span class="filtercaption">' . \Ron\ronText('Datum_a_cas') . '</span>';
        DateInput::renderDateSelect('datum', date('j.n.Y', $timeStamp), true); // submitonenter = true
        echo '&nbsp;&nbsp;';
        echo '<input type="text" name="cas" value="' . $cas . '" maxlength="8" size="5">';
        echo "</div>\n";
    }

    function renderOperace()
    {
        $sql = "SELECT Kod, Popis FROM OperaceDochazky WHERE Systemova <> 1 AND PritomnostProMonitorovani <> 3";

        $comboOperacePopis = new ComboBox('operace_popis');
        $comboOperacePopis->className = 'adsfilterforminput_popis';
        $comboOperacePopis->htmlParams = 'data-ron-select2="operace" data-ron-autosubmit="1"';
        $comboOperacePopis->emptyRow = \Ron\ronText('_vse_');

        $comboOperace = new ComboBox('operace');
        $comboOperace->captionIndex = 0;
        $comboOperace->className = 'adsfilterforminput_kod';
        $comboOperace->htmlParams = 'data-ron-select2="operace_popis" data-ron-autosubmit="1"';
        $comboOperace->emptyRow = \Ron\ronText('_vse_');

        try {
            $comboOperacePopis->loadData($this->database, $sql . "ORDER BY Popis", $_SESSION['monitorovat_operaci']);
            $comboOperace->loadData($this->database, $sql . "ORDER BY Kod", $_SESSION['monitorovat_operaci']);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage("Chyba pri nacitani operaci. " . $exception->getMessage());
        }

        echo '<div class="filterrow">';
        echo '<span class="filtercaption">' . \Ron\ronText('Operace') . '</span>';

        $comboOperacePopis->render();
        $comboOperace->render();

        echo "</div>\n";
    }

    function renderTerminal()
    {
        $sql = "SELECT Kod, Popis FROM Terminal WHERE Aktivni = 1";

        $comboTerminalPopis = new ComboBox('terminal_popis');
        $comboTerminalPopis->className = 'adsfilterforminput_popis';
        $comboTerminalPopis->htmlParams = 'data-ron-select2="terminal" data-ron-autosubmit="1"';
        $comboTerminalPopis->emptyRow = \Ron\ronText('_vse_');

        $comboTerminal = new ComboBox('terminal');
        $comboTerminal->captionIndex = 0;
        $comboTerminal->className = 'adsfilterforminput_kod';
        $comboTerminal->htmlParams = 'data-ron-select2="terminal_popis" data-ron-autosubmit="1"';
        $comboTerminal->emptyRow = \Ron\ronText('_vse_');

        try {
            $comboTerminalPopis->loadData($this->database, $sql . "ORDER BY Popis", $_SESSION['monitorovat_terminal']);
            $comboTerminal->loadData($this->database, $sql . "ORDER BY Kod", $_SESSION['monitorovat_terminal']);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage("Chyba pri nacitani terminalu. " . $exception->getMessage());
        }

        echo '<div class="filterrow">';
        echo '<span class="filtercaption">' . \Ron\ronText('Terminal') . '</span>';

        $comboTerminalPopis->render();
        $comboTerminal->render();

        echo "</div>\n";
    }

    function renderFilters()
    {
        $this->renderAktualniStav();
        $this->renderDatumCas();
        $this->renderOperace();
        $this->renderTerminal();

        parent::renderFilters(); // rozdeleni
    }
}




class Monitorovani_FotoField extends BrowserField
{
    public $browserAction;

    /** @var  boolean true = zobrazi foto ve strance, false = zobrazi odkaz na zobrazeni fota */
    private $showFoto;

    /**
     * Monitorovani_FotoField constructor.
     * @param string $action menuid action - monitorovani muze byt zobrazeno s prihlasenim i bez
     * @param bool $showFoto
     */
    public function __construct($action, $showFoto = false)
    {
        parent::__construct();
        $this->align = 'center';

        $this->browserAction = $action;
        $this->showFoto = $showFoto;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($value) {
            if ($this->showFoto) {
                return '<div class="browserfoto ronFoto">
                            <img src="?menuid=' . $this->browserAction . '&fotopp=' . (int)$rowData[0] . '"/>
                        </div>';
            }
            else {
                return '<a target="ads_foto" class="link_foto" href="?menuid=' . $this->browserAction . '&fotopp=' . (int)$rowData[0] . '">' . \Ron\ronText('Foto') . '</a>';
            }
        }

        return '';
    }
}

class Email_BrowserField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($value) {
            return "<a href='mailto:$value'>$value</a>";
        }

        return '';
    }
}

class MonitorovaniButton_BrowserField extends BrowserField
{
    private $osobaIDnumber;
    private $nastaveniSloupce;

    public function __construct($osobaIDnumber, $nastaveniSloupce)
    {
        parent::__construct(null, null);
        $this->osobaIDnumber = $osobaIDnumber;
        $this->nastaveniSloupce = $nastaveniSloupce;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $osoba = $rowData[$this->osobaIDnumber];
        $buttonID = $this->nastaveniSloupce['buttonid'];
        $action = $this->nastaveniSloupce['typ'];
        $nadpis = $this->nastaveniSloupce['nadpis'];

        $prava = null;

        if ($this->nastaveniSloupce['typ'] == 'vlozeniabsence')
            $prava = Prava::vratPrava('absence', 'absence_' . $this->nastaveniSloupce['operace']);
        if ($this->nastaveniSloupce['typ'] == 'vlozenidochazky')
            $prava = Prava::vratPrava('vkladani');
        if ($this->nastaveniSloupce['typprava'])
            $prava = Prava::vratPrava($this->nastaveniSloupce['typprava']);

        Prava::nastavObdobiProPrava(DateUtils::today());

        if ($prava && !$prava ->lzeZobrazitPP($osoba))
            return '';

        $form = 
            "<form class='submitWorkingInfo sendPosition' method='POST' action='?menuid=monitorovani'>
                <input type='hidden' name='action' value='$action' />
                <input type='hidden' name='buttonid' value='$buttonID' />
                <input type='hidden' name='osoba' value='$osoba' />
                <input type='hidden' name='x' />
                <input type='hidden' name='y' />
                <input type='submit' class='submitbutton' value='$nadpis' />
            </form>";

        return $form;
    }
}

class RozdeleniIMG_BrowserField extends BrowserField
{
    public $typRozdeleni;

    public function __construct($database, $kodTypuRozdeleni)
    {
        parent::__construct($database);

        $this->typRozdeleni = $kodTypuRozdeleni;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $idPP = $rowData[0];

        require_once('adsapp/rozdeleniosob.php');
        $ikony = RozdeleniOsob::nactiIkonyRozdeleni($this->database, $idPP, $this->typRozdeleni, time(), time());

        return RozdeleniOsob::getIkonyHtml($ikony);
    }


}

class MonitorovaniSloupecRozdeleni
{
    /** @var  int kod typu rozdeleni */
    public $kodTypu;
    /** @var  string kod, popis, vedoucipopis, vedouci, zastupce rozdeleni */
    public $typ;
    /** @var  int index sloupce v dotazu */
    public $indexSloupce;

    /**
     * MonitorovaniSloupecRozdeleni constructor.
     * @param int $kodTypu kod typu rozdeleni
     * @param string $typ kod, popis, vedoucipopis, vedouci, zastupce rozdeleni
     */
    public function __construct($kodTypu, $typ)
    {
        $this->kodTypu = $kodTypu;
        $this->typ = $typ;
    }
}

class MonitorovaniPocetDleOperace
{
    /** @var string kod operace */
    public $kod;
    /** @var string popis operace */
    public $popis;
    /** @var int */
    public $pocetPritomno;
    /** @var int */
    public $pocetNepritomno;
    /** @var int */
    public $pocetCelkem;

    /**
     * @param MonitorovaniPocetDleOperace[] $pole
     */
    public static function seradOperace(&$pole)
    {
        uasort($pole, 'Ron\RonPortal\MonitorovaniPocetDleOperace::porovnejOperace');
    }

    /**
     * @param MonitorovaniPocetDleOperace $operace1
     * @param MonitorovaniPocetDleOperace $operace2
     * @return int
     */
    public static function porovnejOperace($operace1, $operace2)
    {
        $por = strnatcasecmp($operace1->popis, $operace2->popis);
        return $por;
    }
}

/**
 * Class MonitorovaniOsobBrowser
 * @package Ron\RonPortal
 */
class MonitorovaniOsobBrowser extends RonPortalBrowser
{
    const SETTINGS_SLOUPCE = 'sloupcemonitorovani';
    const SETTINGS_TRIDIT = 'triditmonitorovani';
    const SETTINGS_POUZIT_RYCHLE = 'pouzitrychlemonitorovani';

    var $zobrazovaneSloupce;
    var $editovat;
    //var $aktualniStav;

    var $kontrolovatPrava;
    /** @var  boolean nastavi se na true, pokud se ma zobrazovat sloupec s rozdelenim osoby */
    var $zobrazitRozdeleni;

    /** @var  int  Pocet sloupcu, ktere jsou v zakladnim sql dotazu (osoba, pracovni pomer) <br/>
    U rychleho monitorovani jsou za nima dalsi sloupce - jejich pozice se
    pocita relativne vzhledem k teto hodnote */
    var $pocetSloupcuSql;


    var $indexPoleProTrideni;

    /** @var MonitorovaniFilterForm filterForm */
    var $filterForm;

    /** @var MonitorovaniSloupecRozdeleni[] seznam pouzitych kodu typu rozdeleni */
    private $kodyTypuRozdeleni = [];

    public $nastaveniSloupcuXML;

    function __construct($database, $action)
    {
        parent::__construct($database, $action);

        $this->title = \Ron\ronText('Monitorovani');

        $this->kontrolovatPrava = $kontrolovatprava = $action != 'monitorovaninolog';
        $this ->cssClass .= ' monitorovani oddEvenRows';

        // Nastavi pole $this->f_sql_fields, $this->f_fieldsformat a $this->f_header
        // podle nastaveni ulozeneho v databazi
        $this->indexPoleProTrideni = -1;

        $this->nactiNastaveni();

        $this->sestavHlavicku();
    }

    function nactiNastaveni()
    {
        $text = RonPortalSettings::getBvarSettings($this->database, 'RonPortal_monitorovani_nastaveni_xml');

        if ($text == '') {
            $str = RonPortalSettings::getValue(self::SETTINGS_SLOUPCE);
            $header = [];
            StringUtils::parseStringToArrays($str, $header, $sloupce, true);

            foreach ($sloupce as $i => $sloupec) {
                $sloupecNastaveni = [];
                $sloupecNastaveni['typ'] = $sloupec;
                $sloupecNastaveni['nadpis'] = $header[$i];

                $rozdeleni = explode('_', $sloupec);

                if (($rozdeleni[0] == 'r') || ($rozdeleni[0] == 'rkod') || ($rozdeleni[0] == 'rvedoucipopis') || ($rozdeleni[0] == 'rvedouci') || ($rozdeleni[0] == 'rzastupce')) {
                    $sloupecNastaveni['typrozdeleni'] = $rozdeleni[1];

                    switch ($rozdeleni[0]) {
                        case 'r':
                            $sloupecNastaveni['typ'] = 'rozdp';
                            break;
                        case 'rkod':
                            $sloupecNastaveni['typ'] = 'rozd';
                            break;
                        case 'rvedoucipopis':
                            $sloupecNastaveni['typ'] = 'rozdvedoucipopis';
                            break;
                        case 'rvedouci':
                            $sloupecNastaveni['typ'] = 'rozdvedouci';
                            break;
                        case 'rzastupce':
                            $sloupecNastaveni['typ'] = 'rozdzastupce';
                            break;
                    }
                }

                if (strpos($sloupec, 'uzivatelska') !== false) {
                    $sloupecNastaveni['typ'] = 'uzivatelska';
                    $sloupecNastaveni['index'] = substr($sloupec, -1);
                }

                $this->zobrazovaneSloupce[] = $sloupecNastaveni;
            }

            return;
        }

        require_once('ron/xmlloader.php');

        $loader = new XmlLoader();
        $loader->includeAttributes = true;
        $loader->prefixToSubarray = true;
        $loader->ignorePrefixes = true;
        $loader->convertFromUtf8 = true;
        $this->nastaveniSloupcuXML = $loader->readXml($text);

        $this->zobrazovaneSloupce = $this->nastaveniSloupcuXML['sloupecmonitorovani'];
    }

    function setFilterForm()
    {
        $this->filterForm = new MonitorovaniFilterForm($this->database, $this ->action, $this->kontrolovatPrava);
    }


    function load($sqlFilter = '')
    {
        if (isset($_GET['monitorovatco'])) $_SESSION['comonitorovat'] = $_GET['monitorovatco'];

        parent::load();
        //$this->aktualniStav = $this ->filterForm ->aktualniStav;
    }


    function zobrazPosledniNacteni()
    {
        $sql = 'SELECT MAX(CasPoslednihoNacteni) FROM StavTerminalu';
        try {
            $cursor = $this->database->select($sql);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage('Chyba pri nacitani casu posledniho nacteni. ' . $exception->getMessage());
        }

        $row = isset($cursor) ? $cursor ->fetch() : null;

        if ($row) {
            echo '<div id="monitorovani_posledninacteni" class="align-right">' . \Ron\ronText('Cas_posledniho_nacteni') . ' <strong>' . DBUtils::getFormatedValue($this ->database, 'datetime', $row[0]) . "</strong></div>\n";
        }
    }


    function renderFilterPritomnost()
    {
        $data = [
            0 => \Ron\ronText('Pritomni'),
            1 => \Ron\ronText('Nepritomni'),
            2 => \Ron\ronText('Vsichni')
        ];

        LinksList::renderLinks($data, $_SESSION['comonitorovat'], '?menuid=' . $this ->action . '&monitorovatco=');
    }


    function renderFilter()
    {
        $this->zobrazPosledniNacteni();
        $this->renderFilterPritomnost();

        parent::renderFilter();
    }


    /**
     * Vybere zaznamy z tabulky PuvodniDochazka a HistoriePruchodu, ktere nejsou
     * starsi nez 1 den (vzhledem k zadanemu datu) pro zadaneho cloveka. Z nich vybere ten posledni.
     * @param int $datum
     * @param string $rc
     * @param string $osobnicislo
     * @param int $datumnastupu
     * @return array|bool|null
     * @throws DBException
     */
    function vratPosledniPruchodSQL($datum, $rc, $osobnicislo, $datumnastupu)
    {
        return DochazkaUtils::vratPosledniPruchodSQL($this->database, $datum, $rc, $osobnicislo, $datumnastupu);
    }


    /**
     *  Vybere zaznamy z tabulky PuvodniDochazka a HistoriePruchodu, ktere nejsou
     * starsi nez 1 den (vzhledem k zadanemu datu) pro zadaneho cloveka. Z nich vybere ten posledni.
     * @param int $datum
     * @param array $row
     * @return array|bool|null
     * @throws Exception
     */
    function vratPosledniPruchod($datum, $row)
    {
        if ( RonPortalSettings::getIntValue(self::SETTINGS_POUZIT_RYCHLE) && $this->filterForm ->aktualniStav) {

            $result = [];

            // datum - u absence vyplni i datum do
            if ($row[$this->pocetSloupcuSql + 8]) {
                // pokud je vyplneny datum do, je to absence
                $result[] = DBUtils::getFormatedValue($this ->database, 'date', $row[$this->pocetSloupcuSql]) . ' - ' . DBUtils::getFormatedValue($this ->database, 'date', $row[$this->pocetSloupcuSql + 8]);
            } else {
                $result[] = DBUtils::getFormatedValue($this ->database, 'datetime', $row[$this->pocetSloupcuSql]);
            }


            for ($i = 1; $i < 4; $i++)
                $result[] = $row[$this->pocetSloupcuSql + $i]; // terminal, poznamka, popis operace

            // pritomnost
            $result[] = ($this ->database ->stringToTimestamp($row[$this->pocetSloupcuSql]) < DateUtils::incDay($datum, -1))
                ? false
                : $row[$this->pocetSloupcuSql + 4];

            $result[] = $row[$this->pocetSloupcuSql + 5]; // umisteni

            $result[] = $row[$this->pocetSloupcuSql + 6]; // rozdeleni
            $result[] = $row[$this->pocetSloupcuSql + 7]; // popis rozdeleni
            $result[] = $row[$this->pocetSloupcuSql + 9]; //kod operace

            return $result;
        } else {
            return $this->vratPosledniPruchodSQL($datum, $row[0], $row[1], $this ->database ->stringToTimestamp($row[2]));
        }
    }


    /**
     * Sestavi hlavicku a seznam poli podle nastaveni
     */
    function sestavHlavicku()
    {
        $this ->addField( [BROWSER_FIELD_TYPE => 'line'] );

        $this ->addField( [BROWSER_FIELD_TYPE =>BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'PracovniPomer.Id'] );
        $hlpcnt = 1;

        $this ->zobrazitRozdeleni = false;

        foreach ($this->zobrazovaneSloupce as $i => $sloupec) {
            $newfield = null;

            $zobrazitSloupec = true;
            if (Prava::existujePravo('monitorovani_' . $sloupec['typ']) ) {
                $zobrazitSloupec = Prava::jsouNejakePrava('monitorovani_' . $sloupec['typ']);
            }
            if ($sloupec['typprava'])
                $zobrazitSloupec = $zobrazitSloupec && Prava::jsouNejakePrava($sloupec['typprava']);

            if ($zobrazitSloupec) {
                switch (strtolower($sloupec['typ'])) {
                    case 'rc' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => 'PracovniPomer.RC']);
                        break;
                    case 'osobni' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => 'PracovniPomer.OsobniCislo', BROWSER_FIELD_HEADER_TEXT => 'Osobni_cislo', BROWSER_FIELD_ALIGN => 'right']);
                        break;
                    case 'datumnastupu' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => 'PracovniPomer.DatumNastupu', BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_ALIGN => 'center']);
                        break;
                    case 'cislosmlouvy' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => 'PracovniPomer.CisloPracovniSmlouvy', BROWSER_FIELD_ALIGN => 'right']);
                        break;
                    case 'kodtypupomeru' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => 'PracovniPomer.TypPracovnihoPomeru']);
                        break;
                    case 'datumukonceni' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => 'PracovniPomer.DatumUkonceni', BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_ALIGN => 'center']);
                        break;
                    case 'jmeno' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => 'Osoba.Jmeno', BROWSER_FIELD_HEADER_TEXT => 'Jmeno']);
                        break;
                    case 'prijmeni' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => 'Osoba.Prijmeni', BROWSER_FIELD_HEADER_TEXT => 'Prijmeni']);
                        //$this->f_defaultsort=$i + 2; // cislovani sloupcu v browseru je od 1, navic preskoci line
                        break;
                    case 'celejmeno' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => 'Osoba.Prijmeni, Osoba.Jmeno', BROWSER_FIELD_HEADER_TEXT => 'Jmeno_a_prijmeni']);
                        //$this->f_defaultsort=$i + 2;
                        break;
                    case 'titul' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => 'Osoba.Titul']);
                        break;
                    case 'telefon' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => 'Osoba.Telefon']);
                        break;
                    case 'zamestnanec' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => 'PracovniPomer.Zamestnanec']);
                        break;
                    case 'casoperace' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => '1', BROWSER_FIELD_HEADER_TEXT => 'Cas_operace', BROWSER_FIELD_ALIGN => 'right']);
                        break;
                    case 'terminal' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => '2', BROWSER_FIELD_HEADER_TEXT => 'Terminal', BROWSER_FIELD_ALIGN => 'right']);
                        break;
                    case 'operace' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => '3']);
                        break;
                    case 'pritomnost' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => '4', BROWSER_FIELD_HEADER_TEXT => 'Pritomnost', BROWSER_FIELD_FORMAT => 'anone', BROWSER_FIELD_ALIGN => 'center']);
                        break;
                    case 'umisteni' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => '5']);
                        break;
                    case 'poznamka' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => '6', BROWSER_FIELD_HEADER_TEXT => 'Poznamka']);
                        break;
                    case 'rozdeleni' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => '7']);
                        break;
                    case 'kodrozdeleni' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => '8']);
                        break;

                    case 'uzivatelska' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => (10 + (int)$sloupec['index'] - 1)]);
                        break;

                    case 'uzivatelskapp' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => (30 + (int)$sloupec['index'] - 1)]);
                        break;

                    case 'kodrozdeleniosoby' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => '54']);
                        $this->zobrazitRozdeleni = true;
                        break;
                    case 'rozdeleniosoby' :
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => '55']);
                        $this->zobrazitRozdeleni = true;
                        break;

                    case 'foto' :
                        $newfield = $this->addField(new Monitorovani_FotoField($this->action));
                        $newfield->sqlFieldName = '51';
                        break;

                    case 'fotoimg' :
                        $newfield = $this->addField(new Monitorovani_FotoField($this->action, true));
                        $newfield->sqlFieldName = '51';
                        break;

                    case 'email' :
                        $newfield = $this->addField(new Email_BrowserField());
                        $newfield->sqlFieldName = '53';
                        break;

                    case 'rozdimg':
                        $newfield = $this->addField(new RozdeleniIMG_BrowserField($this->database, $sloupec['typrozdeleni']));
                        break;

                    case 'rozdp':
                    case 'rozd':
                    case 'rozdvedoucipopis':
                    case 'rozdvedouci':
                    case 'rozdzastupce':
                        $newfield = $this->addField([BROWSER_FIELD_SQL_NAME => '18']);
                        $this->kodyTypuRozdeleni[$i] = new MonitorovaniSloupecRozdeleni($sloupec['typrozdeleni'], $sloupec['typ']);
                        break;

                    case 'vlozeniabsence':
                        if (empty($sloupec['typprava'])) {
                            $prava = Prava::vratPrava('absence', 'absence_' . $sloupec['operace']);
                            $zobrazitSloupec = Prava::jsouNejakePrava($prava);
                        }
                        if ($zobrazitSloupec)
                            $newfield = $this->addField(new MonitorovaniButton_BrowserField(0, $sloupec));
                        break;
                    case 'vlozenidochazky':
                        if (empty($sloupec['typprava']))
                            $zobrazitSloupec = Prava::jsouNejakePrava('vkladani');
                        if ($zobrazitSloupec)
                            $newfield = $this->addField(new MonitorovaniButton_BrowserField(0, $sloupec));
                        break;

                    default :
                        $newfield = $this->addField([BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_TEXT, BROWSER_FIELD_TEXT => $sloupec['typ']]);
                }
            }

            if ($newfield) {
                if ($sloupec['nadpis'])
                    $newfield ->header = \Ron\ronText($sloupec['nadpis']);

                if ($sloupec['class'])
                    $newfield->className = $sloupec['class'];

                if ($newfield ->sqlFieldName) {
                    $newfield->sqlFieldIndex = $hlpcnt;
                    $hlpcnt++;
                }
            }

        }
    }


    /**
     * @param array $row
     * @param array $poslednipruchod
     * @return array
     * @throws Exception
     */
    function sestavRadek($row, $poslednipruchod)
    {
        $result = [ $row[50] ]; // id PP

        foreach ($this->zobrazovaneSloupce as $i => $sloupec) {

            $lzeZobrazit = true;
            if (Prava::existujePravo('monitorovani_' . $sloupec['typ'])) {
                $lzeZobrazit = Prava::lzeZobrazitPP('monitorovani_' . $sloupec['typ'], $row[50], time(), time());
            }
            if ($sloupec['typprava'])
                $lzeZobrazit = $lzeZobrazit && Prava::lzeZobrazitPP($sloupec['typprava'], $row[50], time(), time());

            if ($lzeZobrazit) {
                switch (strtolower($sloupec['typ'])) {
                    case 'rc' :
                        $result[] = $row[0];
                        break;
                    case "osobni" :
                        $result[] = $row[1];
                        break;
                    case "datumnastupu" :
                        $result[] = $row[2];
                        break;
                    case "cislosmlouvy" :
                        $result[] = $row[3];
                        break;
                    case "kodtypupomeru" :
                        $result[] = $row[4];
                        break;
                    case "datumukonceni" :
                        $result[] = $row[5];
                        break;
                    case "jmeno" :
                        $result[] = $row[6];
                        break;
                    case "prijmeni" :
                        $result[] = $row[7];
                        break;
                    case "celejmeno" :
                        $result[] = $row[6] . " " . $row[7];
                        break;
                    case "titul" :
                        $result[] = $row[8];
                        break;
                    case "telefon" :
                        $result[] = $row[52];
                        break;
                    case "zamestnanec" :
                        $result[] = $row[9];
                        break;

                    case "uzivatelska" :
                        $result[] = $row[10 + $sloupec['index'] - 1];
                        break;
                    case "uzivatelskapp" :
                        $result[] = $row[30 + $sloupec['index'] - 1];
                        break;

                        break;
                    case "fotoimg":
                    case "foto" :
                        $result[] = $row[51]; // id fotografie
                        break;

                    case 'email' :
                        $result[] = $row[53];
                        break;

                    case 'kodrozdeleniosoby' :
                        $result[] = $row[54];
                        break;
                    case 'rozdeleniosoby' :
                        $result[] = $row[55];
                        break;

                    case "casoperace" :
                        $result[] = $poslednipruchod[0];
                        break;
                    case "terminal" :
                        $result[] = $poslednipruchod[1];
                        break;
                    case "poznamka" :
                        $result[] = $poslednipruchod[2];
                        break;

                    case "operace" :
                        $result[] = $poslednipruchod[3];
                        break;
                    case "pritomnost" :
                        $result[] = $poslednipruchod[4];
                        break;
                    case "umisteni" :
                        $result[] = $poslednipruchod[5];
                        break;
                    case "kodrozdeleni" :
                        $result[] = $poslednipruchod[6];
                        break;
                    case "rozdeleni" :
                        $result[] = $poslednipruchod[7];
                        break;

                    default :
                        if ($this->kodyTypuRozdeleni[$i] instanceof MonitorovaniSloupecRozdeleni) {
                            $result[] = $row[$this->kodyTypuRozdeleni[$i]->indexSloupce];
                            break;
                        }

                } // switch
            } else {
                $result[] = null;
            }
        } // foreach

        return $result;
    }


    function vratPoleProTrideni()
    {
        $this->indexPoleProTrideni = -1;

        switch (RonPortalSettings::getValue(self::SETTINGS_TRIDIT)) {
            case "kodtypupomeru" :
                $this->indexPoleProTrideni = 4;
                break;
            case "titul"        :
                $this->indexPoleProTrideni = 8;
                break;
            case "zamestnanec"  :
                $this->indexPoleProTrideni = 9;
                break;
            case "uzivatelska1" :
                $this->indexPoleProTrideni = 10;
                break;
            case "uzivatelska2" :
                $this->indexPoleProTrideni = 11;
                break;
            case "uzivatelska3" :
                $this->indexPoleProTrideni = 12;
                break;
            case "uzivatelska4" :
                $this->indexPoleProTrideni = 13;
                break;
            case "uzivatelska5" :
                $this->indexPoleProTrideni = 14;
                break;
            case "uzivatelska6" :
                $this->indexPoleProTrideni = 15;
                break;
            case "uzivatelska7" :
                $this->indexPoleProTrideni = 16;
                break;
            case "uzivatelska8" :
                $this->indexPoleProTrideni = 17;
                break;
            case "uzivatelska9" :
                $this->indexPoleProTrideni = 18;
                break;
        }

        if (($this ->database ->getDbType() == 'interbase') || ($this ->database ->getDbType() == 'oracle')) {
            if ($this->indexPoleProTrideni >= 0) return '' . ($this->indexPoleProTrideni + 1);
            else return '';
        } else {
            switch (RonPortalSettings::getValue(self::SETTINGS_TRIDIT)) {
                case "kodtypupomeru" :
                    return "PracovniPomer.TypPracovnihoPomeru";
                case "titul"        :
                    return "osoba.titul";
                case "zamestnanec"  :
                    return "PracovniPomer.Zamestnanec";
                case "uzivatelska1" :
                    return "Osoba.Uzivatelska1";
                case "uzivatelska2" :
                    return "Osoba.Uzivatelska2";
                case "uzivatelska3" :
                    return "Osoba.Uzivatelska3";
                case "uzivatelska4" :
                    return "Osoba.Uzivatelska4";
                case "uzivatelska5" :
                    return "Osoba.Uzivatelska5";
                case "uzivatelska6" :
                    return "Osoba.Uzivatelska6";
                case "uzivatelska7" :
                    return "Osoba.Uzivatelska7";
                case "uzivatelska8" :
                    return "Osoba.Uzivatelska8";
                case "uzivatelska9" :
                    return "Osoba.Uzivatelska9";
            } // switch
        }

        return '';
    }

    /**
     * @param int $datum
     * @param bool $nastavitIndexSloupcu
     * @return string
     */
    function getSQLTextSloupceRozdeleni($datum, $nastavitIndexSloupcu = false)
    {
        $sql = '';

        foreach ($this->kodyTypuRozdeleni as $sloupecRozdeleni) {
            switch (strtoupper($sloupecRozdeleni->typ)) {
                case 'POPIS' :
                case 'ROZDP' :
                    $sql .=
                        ", (SELECT Popis FROM Rozdeleni
                            WHERE KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND Kod = 
                            (SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob
                            WHERE RozdeleniOsob.KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND RozdeleniOsob.RC = PracovniPomer.RC
                              AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo
                              AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu
                              AND RozdeleniOsob.PlatnostOd <= " . $this->database->dateToString($datum) .
                        "     AND (RozdeleniOsob.PlatnostDo >= " . $this->database->timestampToString(DateUtils::startOfTheDay($datum) - 1) . " OR RozdeleniOsob.PlatnostDo IS NULL)
                        " .
                        ")) AS PopisRozdeleni" . (($sloupecRozdeleni->kodTypu > 0) ? $sloupecRozdeleni->kodTypu : 878);

                    break;
                case 'KOD' :
                case 'ROZD' :
                    $sql .=
                        ", (SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob
                            WHERE RozdeleniOsob.KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND RozdeleniOsob.RC = PracovniPomer.RC
                              AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo
                              AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu
                              AND RozdeleniOsob.PlatnostOd <= " . $this->database->dateToString($datum) .
                        "     AND (RozdeleniOsob.PlatnostDo >= " . $this->database->timestampToString(DateUtils::startOfTheDay($datum) - 1) . " OR RozdeleniOsob.PlatnostDo IS NULL)
                        " .
                        ") AS KodRozdeleni" . (($sloupecRozdeleni->kodTypu > 0) ? $sloupecRozdeleni->kodTypu : 878);
                    break;
                case 'VEDOUCIPOPIS' :
                case 'ROZDVEDOUCIPOPIS' :
                    $sql .=
                        ", (SELECT Vedouci FROM Rozdeleni
                            WHERE KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND Kod = 
                            (SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob
                            WHERE RozdeleniOsob.KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND RozdeleniOsob.RC = PracovniPomer.RC
                              AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo
                              AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu
                              AND RozdeleniOsob.PlatnostOd <= " . $this->database->dateToString($datum) .
                        "     AND (RozdeleniOsob.PlatnostDo >= " . $this->database->timestampToString(DateUtils::startOfTheDay($datum) - 1) . " OR RozdeleniOsob.PlatnostDo IS NULL)
                        " .
                        ")) AS VedouciRozdeleniPopis" . (($sloupecRozdeleni->kodTypu > 0) ? $sloupecRozdeleni->kodTypu : 878);

                    break;
                case 'VEDOUCI' :
                case 'ROZDVEDOUCI' :
                    $sql .=
                        ", (SELECT Osoba.Prijmeni {fn_concat()} ' ' {fn_concat()} COALESCE(Osoba.Jmeno,'') FROM Rozdeleni JOIN Osoba ON Osoba.RC = Rozdeleni.VedouciRC
                            WHERE KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND Kod = 
                            (SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob
                            WHERE RozdeleniOsob.KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND RozdeleniOsob.RC = PracovniPomer.RC
                              AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo
                              AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu
                              AND RozdeleniOsob.PlatnostOd <= " . $this->database->dateToString($datum) .
                        "     AND (RozdeleniOsob.PlatnostDo >= " . $this->database->timestampToString(DateUtils::startOfTheDay($datum) - 1) . " OR RozdeleniOsob.PlatnostDo IS NULL)
                        " .
                        ")) AS VedouciRozdeleni" . (($sloupecRozdeleni->kodTypu > 0) ? $sloupecRozdeleni->kodTypu : 878);

                    break;
                case 'ZASTUPCE' :
                case 'ROZDZASTUPCE' :
                    $sql .=
                        ", (SELECT Osoba.Prijmeni {fn_concat()} ' ' {fn_concat()} COALESCE(Osoba.Jmeno,'') FROM Rozdeleni JOIN Osoba ON Osoba.RC = Rozdeleni.ZastupceRC
                            WHERE KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND Kod = 
                            (SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob
                            WHERE RozdeleniOsob.KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND RozdeleniOsob.RC = PracovniPomer.RC
                              AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo
                              AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu
                              AND RozdeleniOsob.PlatnostOd <= " . $this->database->dateToString($datum) .
                        "     AND (RozdeleniOsob.PlatnostDo >= " . $this->database->timestampToString(DateUtils::startOfTheDay($datum) - 1) . " OR RozdeleniOsob.PlatnostDo IS NULL)
                        " .
                        ")) AS ZastupceRozdeleni" . (($sloupecRozdeleni->kodTypu > 0) ? $sloupecRozdeleni->kodTypu : 878);

                    break;
            }

            if ($nastavitIndexSloupcu) {
                $sloupecRozdeleni->indexSloupce = $this->pocetSloupcuSql;
                $this->pocetSloupcuSql++;
            }
        }

        return $sql;
    }

    function getSQLTextRychle($datum)
    {
        $this->pocetSloupcuSql = 54;

        // Predelano 25.6.2007
        $sql =
            'SELECT PuvDochProMonitorovani.RC AS RC, PuvDochProMonitorovani.osobniCislo, PuvDochProMonitorovani.DatumNastupu,' .
            'PracovniPomer.CisloPracovniSmlouvy, PracovniPomer.TypPracovnihoPomeru,' . // 3..4
            'PracovniPomer.DatumUkonceni, osoba.jmeno, osoba.prijmeni, osoba.titul,' . // 5..8
            'PracovniPomer.Zamestnanec,' . // 9
            'Osoba.Uzivatelska1, Osoba.Uzivatelska2, Osoba.Uzivatelska3, Osoba.Uzivatelska4, Osoba.Uzivatelska5,  ' . // 10..
            'Osoba.Uzivatelska6, Osoba.Uzivatelska7, Osoba.Uzivatelska8, Osoba.Uzivatelska9, Osoba.Uzivatelska10, ' . // ..19
            'Osoba.Uzivatelska11, Osoba.Uzivatelska12, Osoba.Uzivatelska13, Osoba.Uzivatelska14, Osoba.Uzivatelska15, ' . // 20..
            'Osoba.Uzivatelska16, Osoba.Uzivatelska17, Osoba.Uzivatelska18, Osoba.Uzivatelska19, Osoba.Uzivatelska20, ' . //   ..29
            'PracovniPomer.Uzivatelska1, PracovniPomer.Uzivatelska2, PracovniPomer.Uzivatelska3, PracovniPomer.Uzivatelska4, PracovniPomer.Uzivatelska5,  ' . // 30..
            'PracovniPomer.Uzivatelska6, PracovniPomer.Uzivatelska7, PracovniPomer.Uzivatelska8, PracovniPomer.Uzivatelska9, PracovniPomer.Uzivatelska10, ' . // ..39
            'PracovniPomer.Uzivatelska11, PracovniPomer.Uzivatelska12, PracovniPomer.Uzivatelska13, PracovniPomer.Uzivatelska14, PracovniPomer.Uzivatelska15, ' . // 40..
            'PracovniPomer.Uzivatelska16, PracovniPomer.Uzivatelska17, PracovniPomer.Uzivatelska18, PracovniPomer.Uzivatelska19, PracovniPomer.Uzivatelska20, ' . //   ..49
            'PracovniPomer.Id, Osoba.idFotografie, Osoba.Telefon, Osoba.eMail '; // 50..53

        if ($this ->zobrazitRozdeleni) {
            $sql .= ', RozdeleniOsoby.Kod, RozdeleniOsoby.Popis ';  // 54,55
            $this->pocetSloupcuSql += 2;
        }

        //11.10.2016 RON-PR vybrane rozdeleni se pripoji k dotazu pomoci subselectu
        $sql .= $this->getSQLTextSloupceRozdeleni($datum, true);

        $sql .=
            // nasleduji sloupce jen pro rychle monitorovani (viz pocetSloupcuSql)
            ', PuvDochProMonitorovani.DatumCasOperace AS DatumCas,PuvDochProMonitorovani.Terminal, PuvDochProMonitorovani.Poznamka,' . // pocetSloupcuSql + 0..2
            'OperaceDochazky.popis as PopisOperace, PuvDochProMonitorovani.Pritomnost,' . // +3..4
            'Umisteni.popis, PuvDochProMonitorovani.Rozdeleni, Rozdeleni.Popis,  '; // +5..7

        if ($this ->database ->getDbType() == 'interbase') $sql .= 'cast(null as timestamp)';
        else $sql .= 'null'; // + 8

        $sql .= ' , OperaceDochazky.Kod'; // + 9

        $sql .=
            ' FROM '.
            ' ( ( ( ( (PuvDochProMonitorovani INNER JOIN PracovniPomer ' .
            '          ON (PracovniPomer.RC = PuvDochProMonitorovani.RC ' .
            '          AND PracovniPomer.OsobniCislo = PuvDochProMonitorovani.OsobniCislo ' .
            '          AND PracovniPomer.DatumNastupu = PuvDochProMonitorovani.DatumNastupu)' .
            '         ) INNER JOIN Osoba ' .
            '         ON Osoba.RC = PracovniPomer.RC ' .
            '       ) INNER JOIN OperaceDochazky ' .
            '       ON OperaceDochazky.kod = PuvDochProMonitorovani.KodOperace ' .
            '     ) LEFT OUTER JOIN Terminal ' .
            '     ON Terminal.kod = PuvDochProMonitorovani.terminal ' .
            '   ) LEFT OUTER JOIN Umisteni '.
            '   ON Umisteni.kod = Terminal.KodUmisteni ' .
            ' ) LEFT OUTER JOIN Rozdeleni ' .
            ' ON Rozdeleni.KodTypuRozdeleni = PuvDochProMonitorovani.KodTypuRozdeleni' .
            ' AND Rozdeleni.Kod = PuvDochProMonitorovani.Rozdeleni ';

        if ($this ->zobrazitRozdeleni) {
            $sql .=
                'LEFT OUTER JOIN RozdeleniOsob ' .
                ' ON RozdeleniOsob.RC = PracovniPomer.RC AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $this ->database ->dateToString($datum) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $this ->database ->dateToString($datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                ' LEFT OUTER JOIN Rozdeleni RozdeleniOsoby ' .
                '  ON RozdeleniOsoby.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND RozdeleniOsoby.Kod = RozdeleniOsob.Rozdeleni';
        }

        $sql .=
            ' WHERE PracovniPomer.DatumNastupu <= ' . $this ->database ->dateToString($datum) .
            ' AND ( (PracovniPomer.DatumUkonceni >= ' . $this ->database ->timestampToString(DateUtils::startOfTheDay($datum) - 1) . ') OR (PracovniPomer.DatumUkonceni IS NULL) ) ';

        if (RonPortalSettings::getIntValue('monitorovani_skryt_ext')) {
            $sql .= ' AND PracovniPomer.Zamestnanec <> 0 ';
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) && isset($this ->filterForm ->kodRozdeleni)) {
            // TODO: opravit pro novy filtr "vcetne podrizenych"
            $sql .=
                ' AND EXISTS ' .
                '(SELECT * FROM RozdeleniOsob ' .
                ' WHERE RozdeleniOsob.RC = PracovniPomer.RC' .
                ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                ' AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString($datum) .
                ' AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString(DateUtils::startOfTheDay($datum)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                " AND ( (RozdeleniOsob.KodTypuRozdeleni = '" . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) . "'" .
                " AND RozdeleniOsob.Rozdeleni = '" . $this->database->addSlashes($this ->filterForm ->kodRozdeleni) . "')";
            if (RonPortalSession::rozdeleniVcetnePodrizenych()) {
                $sql .=
                    " OR EXISTS (SELECT * FROM HierarchieRozdeleni WHERE" .
                    " HierarchieRozdeleni.typNadrazene = '" . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) . "'" .
                    " AND HierarchieRozdeleni.kodNadrazene = '" . $this->database->addSlashes($this ->filterForm ->kodRozdeleni) . "'" .
                    ' AND HierarchieRozdeleni.typPodrizene = RozdeleniOsob.KodTypuRozdeleni ' .
                    ' AND HierarchieRozdeleni.kodPodrizene = RozdeleniOsob.Rozdeleni' .
                    ')'; // exists
                    }
            $sql .=
                ')' . // and
                ')';

        }

        if ($_SESSION['monitorovat_operaci']) {
            $sql .=
                " AND OperaceDochazky.Kod = '" . $this->database->addSlashes($_SESSION['monitorovat_operaci']) . "'";
        }

        // Prida do dotazu absence - pridano 25.7.2007
        $sql .=
            ' UNION ' .

            ' SELECT Absence.RC AS RC, Absence.osobniCislo, Absence.DatumNastupu,' .
            'PracovniPomer.CisloPracovniSmlouvy, PracovniPomer.TypPracovnihoPomeru,' .
            'PracovniPomer.DatumUkonceni, Osoba.jmeno, Osoba.prijmeni, Osoba.titul,' .
            'PracovniPomer.Zamestnanec,' .
            'Osoba.Uzivatelska1, Osoba.Uzivatelska2, Osoba.Uzivatelska3, Osoba.Uzivatelska4, Osoba.Uzivatelska5,  ' . // 10..
            'Osoba.Uzivatelska6, Osoba.Uzivatelska7, Osoba.Uzivatelska8, Osoba.Uzivatelska9, Osoba.Uzivatelska10, ' . // ..19
            'Osoba.Uzivatelska11, Osoba.Uzivatelska12, Osoba.Uzivatelska13, Osoba.Uzivatelska14, Osoba.Uzivatelska15, ' . // 20..
            'Osoba.Uzivatelska16, Osoba.Uzivatelska17, Osoba.Uzivatelska18, Osoba.Uzivatelska19, Osoba.Uzivatelska20, ' . //   ..29
            'PracovniPomer.Uzivatelska1, PracovniPomer.Uzivatelska2, PracovniPomer.Uzivatelska3, PracovniPomer.Uzivatelska4, PracovniPomer.Uzivatelska5,  ' . // 30..
            'PracovniPomer.Uzivatelska6, PracovniPomer.Uzivatelska7, PracovniPomer.Uzivatelska8, PracovniPomer.Uzivatelska9, PracovniPomer.Uzivatelska10, ' . // ..39
            'PracovniPomer.Uzivatelska11, PracovniPomer.Uzivatelska12, PracovniPomer.Uzivatelska13, PracovniPomer.Uzivatelska14, PracovniPomer.Uzivatelska15, ' . // 40..
            'PracovniPomer.Uzivatelska16, PracovniPomer.Uzivatelska17, PracovniPomer.Uzivatelska18, PracovniPomer.Uzivatelska19, PracovniPomer.Uzivatelska20, ' . //   ..49
            'PracovniPomer.Id, Osoba.idFotografie, Osoba.Telefon, Osoba.eMail ';

        if ($this ->zobrazitRozdeleni) {
            $sql .= ', RozdeleniOsoby.Kod, RozdeleniOsoby.Popis ';  // 54,55
        }

        //11.10.2016 RON-PR vybrane rozdeleni se pripoji k dotazu pomoci subselectu
        $sql .= $this->getSQLTextSloupceRozdeleni($datum);

        // nasleduji sloupce pro rychle monitorovani ...
        if ($this ->database ->getDbType() == 'interbase') {
            $sql .=
                ', Absence.DatumOd as DatumCas,  cast(null as varchar(10)), cast(SUBSTRING(Absence.Poznamka FROM 1 FOR 250) as varchar(250)),' .
                'OperaceDochazky.popis as PopisOperace, OperaceDochazky.PritomnostProMonitorovani,' .
                'cast(null as varchar(40)), cast(null as varchar(10)), cast(null as varchar(40)), Absence.DatumDo, OperaceDochazky.Kod';
        } else {
            $sql .=
                ', Absence.DatumOd as DatumCas, null, CAST(Absence.Poznamka AS nvarchar(250)),' .
                'OperaceDochazky.popis as PopisOperace,OperaceDochazky.PritomnostProMonitorovani,' .
                'null, null, null, Absence.DatumDo, OperaceDochazky.Kod';
        }

        $sql .=
            ' FROM Absence INNER JOIN OperaceDochazky ' .
            '   ON (OperaceDochazky.kod = Absence.OperaceDochazky)' .
            ' INNER JOIN PracovniPomer ' .
            '   ON PracovniPomer.RC = Absence.RC' .
            '  AND PracovniPomer.OsobniCislo = Absence.OsobniCislo' .
            '  AND PracovniPomer.DatumNastupu = Absence.DatumNastupu' .
            ' INNER JOIN Osoba' .
            '   ON Osoba.RC = Absence.RC';

        if ($this ->zobrazitRozdeleni) {
            $sql .=
                ' LEFT OUTER JOIN RozdeleniOsob ' .
                ' ON RozdeleniOsob.RC = PracovniPomer.RC AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $this ->database ->dateToString($datum) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $this ->database ->dateToString($datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                ' LEFT OUTER JOIN Rozdeleni RozdeleniOsoby ' .
                '  ON RozdeleniOsoby.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND RozdeleniOsoby.Kod = RozdeleniOsob.Rozdeleni';
        }

        $sql .=
            ' WHERE  (Absence.datumOd <= ' . $this ->database ->timestampToString($datum) . ')' .
            ' AND (Absence.datumDo >= ' . $this ->database ->timestampToString(DateUtils::startOfTheDay($datum)) . ')' .
            ' AND Absence.Schvaleni <> 2  AND  Absence.Schvaleni <> 4 '; // nezobrazuje zamitnute a zrusene

        if (RonPortalSettings::getIntValue('monitorovani_skryt_ext')) {
            $sql .= ' AND PracovniPomer.Zamestnanec <> 0 ';
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) && isset($this ->filterForm ->kodRozdeleni)) {
            $sql .=
                ' AND EXISTS ' .
                '(SELECT * FROM RozdeleniOsob ' .
                ' WHERE RozdeleniOsob.RC = PracovniPomer.RC' .
                ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                ' AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString($datum) .
                ' AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString(DateUtils::startOfTheDay($datum)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                " AND ( (RozdeleniOsob.KodTypuRozdeleni = " . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                " AND RozdeleniOsob.Rozdeleni = '" . $this->database->addSlashes($this ->filterForm ->kodRozdeleni) . "')";
            if (RonPortalSession::rozdeleniVcetnePodrizenych()) {
                $sql .=
                    " OR EXISTS (SELECT * FROM HierarchieRozdeleni WHERE" .
                    " HierarchieRozdeleni.typNadrazene = " . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                    " AND HierarchieRozdeleni.kodNadrazene = '" . $this->database->addSlashes($this ->filterForm ->kodRozdeleni) . "'" .
                    " AND HierarchieRozdeleni.typPodrizene = RozdeleniOsob.KodTypuRozdeleni " .
                    " AND HierarchieRozdeleni.kodPodrizene = RozdeleniOsob.Rozdeleni" .
                    ')'; // exists
            }

            $sql .=
                ')' . // and
                ')';
        }

        if ($_SESSION['monitorovat_operaci']) {
            $sql .=
                " AND OperaceDochazky.Kod = '" . $this->database->addSlashes($_SESSION['monitorovat_operaci']) . "'";
        }

        // Razeni
        $trideni = $this->vratPoleProTrideni();
        $sql .= ' ORDER BY ';
        if ($trideni) $sql .= $trideni . ", ";

        if ($this ->database ->getDbType() == 'interbase' || $this ->database ->getDbType() == 'oracle') $sql .= ' 8,7,1,21 DESC';
        else $sql .= ' Osoba.Prijmeni, Osoba.Jmeno, RC, DatumCas DESC';

        return $sql;
    } // GetSQLTextRychle


    function getSQLTextPuvodni($datum)
    {
        $sql =
            'SELECT DISTINCT PracovniPomer.RC, PracovniPomer.osobniCislo, PracovniPomer.datumNastupu,' .
            'PracovniPomer.CisloPracovniSmlouvy, PracovniPomer.TypPracovnihoPomeru,' .
            'PracovniPomer.DatumUkonceni, Osoba.jmeno, Osoba.prijmeni, Osoba.titul,' .
            'PracovniPomer.Zamestnanec,' .
            'Osoba.Uzivatelska1, Osoba.Uzivatelska2, Osoba.Uzivatelska3, Osoba.Uzivatelska4, Osoba.Uzivatelska5,  ' . // 10..
            'Osoba.Uzivatelska6, Osoba.Uzivatelska7, Osoba.Uzivatelska8, Osoba.Uzivatelska9, Osoba.Uzivatelska10, ' . // ..19
            'Osoba.Uzivatelska11, Osoba.Uzivatelska12, Osoba.Uzivatelska13, Osoba.Uzivatelska14, Osoba.Uzivatelska15, ' . // 20..
            'Osoba.Uzivatelska16, Osoba.Uzivatelska17, Osoba.Uzivatelska18, Osoba.Uzivatelska19, Osoba.Uzivatelska20, ' . //   ..29
            'PracovniPomer.Uzivatelska1, PracovniPomer.Uzivatelska2, PracovniPomer.Uzivatelska3, PracovniPomer.Uzivatelska4, PracovniPomer.Uzivatelska5,  ' . // 30..
            'PracovniPomer.Uzivatelska6, PracovniPomer.Uzivatelska7, PracovniPomer.Uzivatelska8, PracovniPomer.Uzivatelska9, PracovniPomer.Uzivatelska10, ' . // ..39
            'PracovniPomer.Uzivatelska11, PracovniPomer.Uzivatelska12, PracovniPomer.Uzivatelska13, PracovniPomer.Uzivatelska14, PracovniPomer.Uzivatelska15, ' . // 40..
            'PracovniPomer.Uzivatelska16, PracovniPomer.Uzivatelska17, PracovniPomer.Uzivatelska18, PracovniPomer.Uzivatelska19, PracovniPomer.Uzivatelska20, ' . //   ..49
            'PracovniPomer.Id, Osoba.IdFotografie, Osoba.Telefon, Osoba.eMail '; // 50 - 53

        //11.10.2016 RON-PR pouziva se pro zjisteni indexu sloupce rozdeleni
        $this->pocetSloupcuSql = 54;

        if ($this ->zobrazitRozdeleni) {
            $sql .= ', RozdeleniOsoby.Kod, RozdeleniOsoby.Popis ';
            $this->pocetSloupcuSql += 2;
        }

        //11.10.2016 RON-PR vybrane rozdeleni se pripoji k dotazu pomoci subselectu
        $sql .= $this->getSQLTextSloupceRozdeleni($datum, true);

        $sql .=
            ' FROM PracovniPomer INNER JOIN Osoba ON Osoba.RC = PracovniPomer.RC ';

        if ($this ->zobrazitRozdeleni) {
            $sql .=
                'LEFT OUTER JOIN RozdeleniOsob ' .
                ' ON RozdeleniOsob.RC = PracovniPomer.RC AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $this ->database ->dateToString($datum) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $this ->database ->dateToString($datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                ' LEFT OUTER JOIN Rozdeleni RozdeleniOsoby ' .
                '  ON RozdeleniOsoby.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND RozdeleniOsoby.Kod = RozdeleniOsob.Rozdeleni';
        }

        $sql .=
            ' WHERE PracovniPomer.DatumNastupu <= ' . $this ->database ->dateToString($datum) .
            ' AND ( (PracovniPomer.DatumUkonceni >= ' . $this ->database ->timestampToString(DateUtils::startOfTheDay($datum) - 1) . ') OR (PracovniPomer.DatumUkonceni IS NULL) ) ';

        if (RonPortalSettings::getIntValue('monitorovani_skryt_ext')) {
            $sql .= ' AND PracovniPomer.Zamestnanec <> 0 ';
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) && isset($this ->filterForm ->kodRozdeleni)) {
            $sql .=
                ' AND EXISTS ' .
                '(SELECT * FROM RozdeleniOsob ' .
                ' WHERE RozdeleniOsob.RC = PracovniPomer.RC' .
                ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                ' AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString($datum) .
                ' AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString(DateUtils::startOfTheDay($datum)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                " AND ( (RozdeleniOsob.KodTypuRozdeleni = '" . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) . "'" .
                " AND RozdeleniOsob.Rozdeleni = '" . $this->database->addSlashes($this ->filterForm ->kodRozdeleni) . "')";
            if (RonPortalSession::rozdeleniVcetnePodrizenych()) {
                $sql .=
                    " OR EXISTS (SELECT * FROM HierarchieRozdeleni WHERE" .
                    " HierarchieRozdeleni.typNadrazene = '" . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) . "'" .
                    " AND HierarchieRozdeleni.kodNadrazene = '" . $this->database->addSlashes($this ->filterForm ->kodRozdeleni) . "'" .
                    ' AND HierarchieRozdeleni.typPodrizene = RozdeleniOsob.KodTypuRozdeleni ' .
                    ' AND HierarchieRozdeleni.kodPodrizene = RozdeleniOsob.Rozdeleni' .
                    ')'; // exists
            }
            $sql .=
                ')' . // and
                ')';
        }

        $trideni = $this->vratPoleProTrideni();
        $sql .= ' ORDER BY ';
        if ($trideni) $sql .= $trideni . ', ';

        if ($this ->database ->getDbType() == 'interbase' || $this ->database ->getDbType() == 'oracle') $sql .= ' 8, 7, 1';
        else $sql .= ' Osoba.Prijmeni, Osoba.Jmeno, PracovniPomer.RC';

        return $sql;
    } // function GetSQLTextPuvodni


    function getSQL($sqlFilter = '')
    {

        /** @noinspection PhpUndefinedFieldInspection */
        $datumCas = $this ->filterForm ->datumCas;

        /** @noinspection PhpUndefinedFieldInspection */
        if ( RonPortalSettings::getIntValue(self::SETTINGS_POUZIT_RYCHLE) && $this->filterForm ->aktualniStav) {
            return $this->getSQLTextRychle($datumCas);
        } else {
            return $this->getSQLTextPuvodni($datumCas);
        }
    }


    function zobrazPoctyOsob($pritomno, $nepritomno, $poctyDleOperace)
    {
        echo '<div id="monitorovani_pocty">';

        echo '<table class="browser">';

//        echo '<div>' . \Ron\ronText('Pritomno_osob') . "<strong>$pritomno</strong></div>";
//        echo '<div>' . \Ron\ronText('Nepritomno_osob') . "<strong>$nepritomno</strong></div>\n";

        MonitorovaniPocetDleOperace::seradOperace($poctyDleOperace);

        echo '<thead>';
        echo '<tr><th>' . \Ron\ronText('Operace') . '</th><th>' . \Ron\ronText('Pritomni') . '</th><th>' . \Ron\ronText('Nepritomni') . '</th><th>' . \Ron\ronText('Celkem') . '</th></tr>';
        echo '</thead>';

        echo '<tbody class="border-bottom">';
        /** @var MonitorovaniPocetDleOperace $prehledOperace */
        foreach ($poctyDleOperace as $index => $prehledOperace) {
            echo '<tr><td>' . StringUtils::htmlTextConvert($prehledOperace->popis) . " </td><td class='align-right'>" . $prehledOperace->pocetPritomno . "</td>
                        <td class='align-right'>" . $prehledOperace->pocetNepritomno . "</td><td class='align-right'>" . $prehledOperace->pocetCelkem . "</td></tr>";
        }
        echo '</tbody>';

        echo '<tbody>';
        echo '<tr class="browserfooter"><td>' . \Ron\ronText('Celkem') . "</td><td class='align-right'>$pritomno</td><td class='align-right'>$nepritomno</td><td class='align-right'>" . ($pritomno+$nepritomno) . "</td></tr>";
        echo '</tbody>';

        echo '</table>';

        echo "</div>\n";
    }


    function zobrazOddelovac($row)
    {
        $pocet = count($this->fields) - 1; // presah do dalsich bunek
        echo "<tr class='oddelovacmonitorovani border-bottom'><td></td><td colspan='$pocet'>";
        echo StringUtils::htmlTextConvert($row[$this->indexPoleProTrideni]);
        echo "</td></tr>";
    }


    /**
     * @throws Exception
     */
    function renderContent()
    {
        $datum = $_SESSION['monitorovatcas'];

        $pritomno = 0;
        $nepritomno = 0;

        $poctyDleOperaci = [];

        //$cursor = getSqlResult($this ->database, $sql);

        $this->renderTableStart();
        echo '<tbody>';

        $i = 1;

        // Promenne pro zapamatovani posledni osoby, aby se nezobrazovala vicekrat
        // (v pripade, ze ma nejakou operaci i planovanou absenci)
        $old_rc = '';
        $old_osobni = '';
        $old_datumnastupu = 0;

        $old_trideni = NULL;

        $prava = Prava::vratPrava('monitorovani');

        while ($row = $this ->cursor ->fetch()) {
            // Zkontroluje, jesli ma uzivatel pravo monitorovat osobu
            if ($this->kontrolovatPrava) {
                $lzezobrazit = $prava->lzeZobrazitPP($row[50], time(), time());
                // ? ? ? nastavit nejdrive obdobi pro prava ? A co prava v pripade divani na jiny cas nez aktualni ?
            } else {
                $lzezobrazit = true;
            }

            if ($lzezobrazit) {
                if ($old_rc != $row[0] || $old_osobni != $row[1] || $old_datumnastupu != $row[2]) {
                    // Osoba jeste nebyla zobrazena
                    $old_rc = $row[0];
                    $old_osobni = $row[1];
                    $old_datumnastupu = $row[2];

                    // Zjisti posledni pruchod a podle nej pritomnost osoby
                    $poslednipruchod = $this->vratPosledniPruchod($datum, $row);

                    if ((($_SESSION['monitorovat_operaci'] && ($_SESSION['monitorovat_operaci'] == $poslednipruchod[8])) || !($_SESSION['monitorovat_operaci'])) &&
                        ((isset($_SESSION['monitorovat_terminal']) && ($_SESSION['monitorovat_terminal'] == $poslednipruchod[1])) || !($_SESSION['monitorovat_terminal']))) {

                        if ($poslednipruchod[4]) $pritomno++;
                        else $nepritomno++;

                        if (isset($poctyDleOperaci[$poslednipruchod[8]]))
                            $pocetOperace = $poctyDleOperaci[$poslednipruchod[8]];
                        else
                            $pocetOperace = new MonitorovaniPocetDleOperace();

                        $pocetOperace->kod = $poslednipruchod[8];
                        $pocetOperace->popis = $poslednipruchod[3];
                        $pocetOperace->pocetCelkem++;
                        if ($poslednipruchod[4]) $pocetOperace->pocetPritomno++;
                        else $pocetOperace->pocetNepritomno++;

                        $poctyDleOperaci[$poslednipruchod[8]] = $pocetOperace;

                        if (($poslednipruchod[4] && $_SESSION['comonitorovat'] != 1) ||
                            (!$poslednipruchod[4] && $_SESSION['comonitorovat'])
                        ) {

                            // Trideni podle nektereho pole - nastaveno v aplikaci v nastaveni intrawebu
                            if ($this->indexPoleProTrideni >= 0) {
                                if ($old_trideni != $row[$this->indexPoleProTrideni]) {
                                    $old_trideni = $row[$this->indexPoleProTrideni];
                                    $this->zobrazOddelovac($row);
                                }
                            }

                            $radek = $this->sestavRadek($row, $poslednipruchod);
                            //$classname = ($i % 2) ? 'browsercolor1' : 'browsercolor2';
                            $classname = '';

                            if ($poslednipruchod[4]) {
                                $this->renderLine($radek, $classname . ' monitorovanipritomen', $i);
                            } else {
                                $this->renderLine($radek, $classname . ' monitorovaninepritomen', $i);
                            }

                            $i++;
                        }
                    }
                }
            }
        } // while

        echo "</tbody>\n";

        $this->renderTableEnd();

        $this->zobrazPoctyOsob($pritomno, $nepritomno, $poctyDleOperaci);
    }



    function render()
    {
        echo '<div id="monitorovani">';
        parent::render();

        echo '<script type="text/javascript">$(function() { ronPortal.monitorovani("' . $this ->action . '"); } ); </script>';

        echo "</div>\n";
    }


}



class MonitorovaniController extends HttpController
{
    const SETTINGS_POVOLIT_BEZ_PRIHLASENI = 'monitorovanibezprihlaseni';

    /** @var Database */
    var $database;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        $this ->database = $database;
        $this ->enableAccessWithoutLogin = RonPortalSettings::getIntValue(self::SETTINGS_POVOLIT_BEZ_PRIHLASENI);
    }


    /**
     * @param $idPP
     * @return FotoOutputObject
     * @throws Exception
     */
    function downloadFoto($idPP)
    {
        // pozor! kontrola prava se neprovadi, pokud je zapnuto monitorovani bez prihlaseni!
        if ( !RonPortalSettings::getIntValue(self::SETTINGS_POVOLIT_BEZ_PRIHLASENI) ) {
            Prava::nastavObdobiProPrava(time(), time());

            if (Prava::existujePravo('monitorovani_fotoimg')) {
                $prava = Prava::vratPrava('monitorovani_fotoimg');
            } else if (Prava::existujePravo('monitorovani_foto')) {
                $prava = Prava::vratPrava('monitorovani_foto');
            } else {
                $prava = Prava::vratPrava('monitorovani');
            }

            if ( !$prava->lzeZobrazitPP($idPP, time(), time()) ){
                RonPortalUtils::neniPravoException();
            }
        }

        require_once ('adsapp/fotoutils.php');
        $result = new FotoOutputObject($this ->database);
        $result ->nactiIdFotkyPracovnihoPomeru($idPP);

        return $result;
    }

    /**
     * @param $nastaveniOperace
     * @return Redirect
     * @throws Exception
     */
    private function handleVlozeniAbsence($nastaveniOperace)
    {
        if (!is_null($nastaveniOperace)) {
            $osoba = $_POST['osoba'];
            $kodOperace = $nastaveniOperace['operace'];
            $delka = $nastaveniOperace['delka'];

            if ($nastaveniOperace['delka'] > 2)
                return new Redirect(null, "?menuid=planovani&osoba=$osoba&datum=" . StringUtils::format('date',DateUtils::today()) . "&delka=$delka&backaction=monitorovani&operace=" . urlencode($kodOperace));

            require_once('adsapp/ppomer.php');

            $pPomer = new PracovniPomer();
            $pPomer->nactiPPPodleId($this->database, $osoba);

            require_once('dochazka/operacedochazky.php');
            $operaceDochazky = OperaceDochazky::nactiOperaci($this->database, $kodOperace);

            require_once('dochazka/absencerec.php');
            $absenceRec = new AbsenceDBRecord($this->database, $pPomer, 'monitorovani');
            $absenceRec->logDescription = StringUtils::fromUTF8('Monitorování');

            $absenceRec->fOperace->setNewValue($operaceDochazky->kod);
            $absenceRec->popisOperace = $operaceDochazky->popis;

            $absenceRec->fDatumOd->setNewValue(DateUtils::today());
            $absenceRec->fDatumDo->setNewValue(DateUtils::today());

            $absenceRec->fDelka->setNewValue($delka);

            if ($nastaveniOperace['typprava'])
                $prava = Prava::vratPrava($nastaveniOperace['typprava']);
            else
                $prava = Prava::vratPrava('absence', 'absence_' . $operaceDochazky->kod);

            Prava::nastavObdobiProPrava(DateUtils::today());

            if (!$prava ->lzeZobrazitPP($pPomer ->idPP)) {
                RonPortalUtils::neniPravoException();
            }

            $absenceRec ->fHistorie ->setNewValue(
                date('j.n.Y G:i', time()) . ', ' . RonPortalUtils::getLogUserName(false) . ': ' .
                \Ron\ronText('Log_naplanovani_absence') . ' ' .
                date('j.n.Y', $absenceRec ->fDatumOd ->value) . ' - ' . date('j.n.Y', $absenceRec->fDatumDo->value) . "\n");

            if ($operaceDochazky ->autoSchvalovat) {
                $absenceRec->fSchvaleni->setNewValue(1);
            } else {
                $prava = Prava::vratPrava('absence_schval', 'absence_schval_' . $absenceRec ->fOperace ->value);
                if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)  &&  $absenceRec->pp ->lzeZobrazitPP($prava)) {
                    // muze i schvalovat
                    // nastavi schvaleni (f_updatelog = true -> provede pouze v pripade zmeny neceho jineho)
                    $absenceRec ->fSchvaleni ->setNewValue($_POST['Schvaleni'] ? 1 : 0);
                    $absenceRec ->fSchvalil ->setNewValue($_POST['Schvaleni'] ? RonPortalUtils::getLogUserName() : null);
                    $absenceRec ->fDatumSchvaleni ->setNewValue($_POST['Schvaleni'] ? time() : null);
                } else {
                    $absenceRec ->fSchvaleni ->setNewValue(0);
                }
            }

            AbsenceDBRecord::zkontrolujPovoleneDelkyOperace($this->database, $operaceDochazky->kod, $delka);

            AbsenceDBRecord::zkontrolujObdobiAbsence($this ->database, $absenceRec->fDatumOd->value, $absenceRec->fDatumDo->value, $absenceRec->pp);

            AbsenceDBRecord::zkontrolujPrekryvaniAbsenci($this ->database, $absenceRec->pp, $absenceRec->fOperace->value,
                $absenceRec->fDatumOd->value, $absenceRec->fDatumDo->value, $absenceRec->fDelka->value, $absenceRec->fCasOd->value, $absenceRec->fCasDo->value);

            if (RonPortalSettings::getIntValue('zakazatabsencepoukonceni')) {
                if ($absenceRec->pp->datumUkonceni  &&  $absenceRec->fDatumDo->value > $absenceRec->pp->datumUkonceni) {
                    throw new Exception( sprintf(\Ron\ronText('Nelze_vkladat_absence_po_ukonceni_pp'), StringUtils::format('date', $absenceRec->pp ->datumUkonceni)) );
                }
            }

            require_once('adsapp/events.php');
            $absenceRec->absenceScript(RonPortalScriptEvents::VLOZENI_ABSENCE_BEFORE);

            $absenceRec ->provedVlozeni();
        }

        return new Redirect(null, '?menuid=monitorovani');
    }

    /**
     * @param $nastaveni
     * @return Redirect
     * @throws Exception
     */
    private function handleVlozeniDochazky($nastaveni)
    {
        $osoba = $_POST['osoba'];

        if ($nastaveni['typprava'])
            $prava = Prava::vratPrava($nastaveni['typprava']);
        else
            $prava = Prava::vratPrava('vkladani');

        Prava::nastavObdobiProPrava(DateUtils::today());

        if (!$prava ->lzeZobrazitPP($osoba)) {
            RonPortalUtils::neniPravoException();
        }

        return new Redirect(null, "?menuid=vlozeni&osoba=$osoba&backaction=monitorovani");
    }

    /**
     * @param $nastaveni
     * @return Redirect
     * @throws Exception
     */
    public function handleButtonMonitorovani($nastaveni)
    {
        foreach ($nastaveni['sloupecmonitorovani'] as $sloupec) {
            if (($sloupec['typ'] == $_POST['action']) && ($sloupec['buttonid'] == $_POST['buttonid'])) {
                if ($sloupec['typ'] == 'vlozeniabsence')
                    return $this->handleVlozeniAbsence($sloupec);
                if ($sloupec['typ'] == 'vlozenidochazky')
                    return $this->handleVlozeniDochazky($sloupec);
            }
        }

        throw new Exception(StringUtils::fromUTF8('Neznámá akce'));
    }

    /**
     * @param string $action
     * @return ViewObject|string
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (isset($_GET['fotopp'])) {
            return $this ->downloadFoto( (int)$_GET['fotopp'] );
        }

        TextManager::getInstance() ->loadStrings('dochazka.ini');
        HtmlHeaders::getInstance() ->addScript('script/dochazka.js');
        HtmlHeaders::getInstance() ->setTitle( \Ron\ronText('RON_Portal') . ' - ' . \Ron\ronText('Monitorovani'));

        if (!(RonPortalSettings::getValue(MonitorovaniOsobBrowser::SETTINGS_SLOUPCE) || RonPortalSettings::getValue('RonPortal_monitorovani_nastaveni_xml'))) {
            throw new Exception( \Ron\ronText('Neni_nastaveni_sloupcu_tabulky') );
        }

        if (isset($_POST['monitorovatco'])) HttpResponse::setCookie('ads_monitorovatco', $_POST['monitorovatco'], time() + 30000000);
        else if (isset($_GET['monitorovatco'])) HttpResponse::setCookie('ads_monitorovatco', $_GET['monitorovatco'], time() + 30000000);
        else $_SESSION['comonitorovat'] = $_COOKIE['ads_monitorovatco'];

        if (User::userAuthenticated()) {
            if (!Prava::jsouNejakePrava('monitorovani')) {
                throw new NeniPravoException();
            }
        }

        $b = new MonitorovaniOsobBrowser($this ->database, $action);

        if (isset($_POST['action']) && isset($_POST['buttonid']) && User::userAuthenticated())
            return $this->handleButtonMonitorovani($b->nastaveniSloupcuXML);

        $b ->load();

        if (User::userAuthenticated()) {
            // uzivatel je prihlasen => vraci ViewObject
            return $b;
        }

        Model::addViewObject($b);
        return 'view/appview_nouser.php';
    }

}
