<?php
/**
 * UTF8
 * stanislav.elbl, 26.01.2017
 */

namespace Ron\RonPortal;

use Ron\DB\Database;
use Ron\StringUtils;


class MzdovaPolozka
{
    public $rc;
    public $osobniCislo;
    public $datumNastupu;

    /** @var int */
    public $obdobi;

    /** @var string */
    public $mzdovaPolozka;

    /** @var int */
    public $druhZdrojeMzdovePolozky;
    /** @var int */
    public $serioveCislo;

    /** @var int */
    public $kodTypuRozdeleni;
    /** @var string */
    public $rozdeleni;

    /** @var float */
    public $pocetJednotek;
    /** @var float */
    public $cenaZaJednotku;
    /** @var float */
    public $castka;


    /**
     * @param Database $database
     * @param array $row
     */
    private function load($database, $row)
    {
        $this ->rc = $row[0];
        $this ->osobniCislo = $row[1];
        $this ->datumNastupu = $database ->stringToTimestamp($row[2]);

        $this ->obdobi = $database ->stringToTimestamp($row[3]);
        $this ->mzdovaPolozka = $row[4];

        $this ->druhZdrojeMzdovePolozky = $row[5];
        $this ->serioveCislo = $row[6];
        $this ->kodTypuRozdeleni = $row[7];
        $this ->rozdeleni = $row[8];

        $this ->pocetJednotek = $row[9];
        $this ->cenaZaJednotku = $row[10];
        $this ->castka = $row[11];
    }


    /**
     * @param Database $database
     * @param PracovniPomer $osoba
     * @param int    $pocatekObdobi
     * @param string|string[] $typMzdovePolozky
     * @return MzdovaPolozka[]
     */
    public static function nactiMzdovePolozky($database, $osoba, $pocatekObdobi, $typMzdovePolozky = null)
    {
        $result = [];

        $params = $database ->createParameters();
        $sql =
            'SELECT RC, OsobniCislo, DatumNastupu, Obdobi, MzdovaPolozka, ' . // 0..4
            ' DruhZdrojeMzdovePolozky, SerioveCislo, KodTypuRozdeleni, Rozdeleni, ' . // 5..8
            ' PocetJednotek, CenaZaJednotku, Castka' . // 9..11
            ' FROM MzdovaPolozkaVeMzde';

        $filtr = '';
        if ($osoba) {
            $filtr =
                ' RC = ' . $params ->addParameter($osoba ->rc) .
                ' AND OsobniCislo = ' . $params ->addParameter($osoba ->osobniCislo) .
                ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($osoba ->datumNastupu));
        }

        if ($pocatekObdobi) {
            if ($filtr) $filtr .= ' AND';
            $filtr .=
                ' Obdobi = ' . $params ->addParameter($database ->dateToParam($pocatekObdobi));
        }

        if ($typMzdovePolozky) {
            if ($filtr) $filtr .= ' AND';

            if (is_array($typMzdovePolozky)) {
                $filtr .= ' (0 = 1';

                foreach ($typMzdovePolozky as $polozka) {
                    if (isset($polozka)) {
                        $filtr .= ' OR MzdovaPolozka = ' . $params ->addParameter($polozka);
                    }
                }

                $filtr .= ')';

            } else {
                $filtr .= ' MzdovaPolozka = ' . $params ->addParameter($typMzdovePolozky);
            }
        }

        if ($filtr) $sql .= ' WHERE ' . $filtr;

        $sql .= ' ORDER BY RC, OsobniCislo, DatumNastupu, Obdobi, MzdovaPolozka';

        $cursor = $database ->getSqlResult($sql, $params, 'Pri nacitani mesicnich vysledku doslo k chybe!');
        while ($row = $cursor ->fetch()) {
            $vysledek = new static();
            $vysledek ->load($database, $row);

            $result[] = $vysledek;
        }
        $cursor ->close();

        return $result;
    }


    /**
     * @param string $format
     * @param bool $skrytNuly
     * @return string
     */
    function vratText($format = null, $skrytNuly = false)
    {
        if (!isset($this ->castka)) return '';
        if (!$this ->castka && $skrytNuly) return '';

        return StringUtils::format($format ?: 'number2', $this ->castka);
    }


    function vratHint($format = null) {
        if ($this ->pocetJednotek != 1) {
            return $this ->pocetJednotek . ' x ' . StringUtils::format($format ?: 'number2', $this ->cenaZaJednotku);
        }

        return null;
    }
}
