<?php
/**
 * UTF8
 * stanislav.elbl, 15.01.2016
 * Nacitani poznamek dochazky a poznamek ke dni
 */

namespace Ron\RonPortal;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\SQLUtils;
use Ron\GuiUtils;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\OutputObject;
use Ron\RonPortal\Obdobi;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\User;
use Ron\StringUtils;


class DenniPoznamkaDochazky
{
    public $idPP;
    public $den;
    public $poznamka;


    function getJson()
    {
        require_once('ron/stringutils.php');
        require_once('ron/guiutils.php');

        return [
            'idPP' => $this->idPP,
            'datumStr' => StringUtils::jsonDate($this->den),
            'text' => StringUtils::jsonText($this->poznamka)
        ];
    }


    /**
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int|int[] [$idPP]     Muze byt null, jeden id pracovniho pomeru, nebo pole id pracovnich pomeru
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @return DenniPoznamkaDochazky[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiPoznamkyDochazky($database, $datumOd, $datumDo, $idPP = null, $prava = null)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT PracovniPomer.Id, DenniPoznamkaDochazky.Den, ' . $database->castWideMemo('DenniPoznamkaDochazky.Poznamka') .
            ' FROM DenniPoznamkaDochazky, PracovniPomer ' .
            ' WHERE DenniPoznamkaDochazky.RC = PracovniPomer.RC ' .
            '   AND DenniPoznamkaDochazky.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND DenniPoznamkaDochazky.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND DenniPoznamkaDochazky.Den >= ' . $params->addParameter($database->dateToParam($datumOd)) .
            '   AND DenniPoznamkaDochazky.Den <= ' . $params->addParameter($database->dateToParam($datumDo));

        if ($idPP) {
            if (is_array($idPP)) {
                $sql .= ' AND (';
                $first = true;
                foreach ($idPP as $id) {
                    if (!$first) $sql .= ' OR ';
                    $first = false;

                    $sql .= 'PracovniPomer.Id = ' . $params->addParameter((int)$id);
                }
                $sql .= ')';
            } else {
                $sql .= ' AND PracovniPomer.Id = ' . $params->addParameter($idPP);
            }
        }

//        if ($filtr) {
//            $sql .= ' AND (' . $filtr . ')';
//        }

        $result = [];
        $cursor = $database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()) {
            if (!$prava || Prava::lzeZobrazitPP($prava, $row[0])) {
                $poznamka = new DenniPoznamkaDochazky();
                $poznamka->idPP = $row[0];
                $poznamka->den = $cursor->stringToTimestamp($row[1]);
                $poznamka->poznamka = $row[2];

                $result[] = $poznamka;
            }
        }

        $cursor->close();

        return $result;
    }


    /**
     * @param Database $database
     * @param int $datum
     * @param int $idPP
     * @return string
     */
    public static function vratPoznamkuDochazky($database, $datum, $idPP)
    {
        $poznamky = self::nactiPoznamkyDochazky($database, $datum, $datum, $idPP);
        if (!count($poznamky)) return null;

        return $poznamky[0] ->poznamka;
    }


    /**
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int|int[] [$idPP]     Muze byt null, jeden id pracovniho pomeru, nebo pole id pracovnich pomeru
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @return DenniPoznamkaDochazky[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiPoznamkyDochazkyJSON($database, $datumOd, $datumDo, $idPP = null, $prava = null)
    {
        $poznamky = self::nactiPoznamkyDochazky($database, $datumOd, $datumDo, $idPP, $prava);
        $jsonPoznamky = [];
        foreach ($poznamky as $poznamka) {
            $jsonPoznamky[] = $poznamka->getJson();
        }

        return $jsonPoznamky;
    }
}


class PoznamkaKeDni
{
    public $datumOd;
    public $datumDo;
    public $typ;
    public $barva;
    public $text;
    public $poznamka;

    function getJson()
    {
        require_once('ron/stringutils.php');
        require_once('ron/guiutils.php');

        return [
            'datumOd' => StringUtils::jsonDate($this ->datumOd),
            'datumDo' => StringUtils::jsonDate($this ->datumDo),
            'typ' => StringUtils::jsonText($this ->typ),
            'barva' => isset($this ->barva) ? GuiUtils::getHtmlColor($this ->barva) : null,
            'text' => StringUtils::jsonText($this ->text),
            'poznamka' => StringUtils::jsonText($this ->poznamka)
        ];
    }


    /**
     * @param Database $database
     * @param string $sql
     * @param Parameters $params
     * @return PoznamkaKeDni[]
     */
    private static function nactiPoznamkySQL($database, $sql, $params)
    {
        $cursor = $database ->getSqlResult($sql, $params);

//        $prava = Prava::vratPrava('kapacity');
//        $pravaEdit = Prava::vratPrava('kapacity_edit');

        $poznamky = [];

        while ($row = $cursor ->fetch()) {
            $poznamka = new PoznamkaKeDni();
            $poznamka ->datumOd = $cursor ->stringToTimestamp($row[0]);
            $poznamka ->datumDo = $cursor ->stringToTimestamp($row[1]);
            $poznamka ->typ = $row[2];
            $poznamka ->barva = $row[4];
            $poznamka ->text = $row[3];
            $poznamka ->poznamka = $row[5];

            $poznamky[] = $poznamka;
        }

        $cursor ->close();

        return $poznamky;
    }


    /**
     * Hromadne nacte poznamky ke dni
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int $typRozdeleni
     * @param string $rozdeleni
     * @return PoznamkaKeDni[]
     */
    public static function nactiPoznamkyKeDni($database, $datumOd, $datumDo, $typRozdeleni, $rozdeleni)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PoznamkaKeDni.DatumOd, PoznamkaKeDni.DatumDo, PoznamkaKeDni.Typ, PoznamkaKeDni.Popis,' .
            '  TypPoznamkyKeDni.Barva, ' . $database ->castWideMemo('PoznamkaKeDni.Poznamka') .
            ' FROM PoznamkaKeDni INNER JOIN TypPoznamkyKeDni' .
            '  ON TypPoznamkyKeDni.Kod = PoznamkaKeDni.Typ ' .
            (isset($rozdeleni)
                ? ' LEFT OUTER JOIN HierarchieRozdeleni' .
                  ' ON HierarchieRozdeleni.TypNadrazene = PoznamkaKeDni.KodTypuRozdeleni' .
                  ' AND HierarchieRozdeleni.KodNadrazene = PoznamkaKeDni.Rozdeleni'
                : ''
            ) .
            ' WHERE (TypPoznamkyKeDni.ZobrazitVHlavicce = 1  OR  TypPoznamkyKeDni.ZobrazitVPaticce = 1)' .
            '   AND PoznamkaKeDni.DatumDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND PoznamkaKeDni.DatumOd <= ' . $params ->addParameter($database ->dateToParam($datumDo));

        if (isset($rozdeleni)) {
            $sql .=
                ' AND (' .
                    '(HierarchieRozdeleni.TypPodrizene = ' . $params ->addParameter((int)$typRozdeleni) .
                    ' AND HierarchieRozdeleni.KodPodrizene = ' . $params ->addParameter($rozdeleni) .
                    ') OR PoznamkaKeDni.Rozdeleni IS NULL' .
                ' )';
        }

        $sql .= ' ORDER BY PoznamkaKeDni.DatumOd';  

        return self::nactiPoznamkySQL($database, $sql, $params);
    }


    /**
     * Nacte poznamky pro zadanou osobu (podle RozdeleniOsob) v zadanem obdobi.
     * @param Database $database
     * @param int $idpp
     * @param int $datumOd
     * @param int $datumDo
     * @param string[]|string $typy Nepovinny parametr - kod typu nebo seznam typu, ktere se maji nacitat. Pokud neni zadan, nacitaji se vsechny.
     * @return PoznamkaKeDni[]
     */
    public static function nactiPoznamkyProOsobu($database, $idpp, $datumOd, $datumDo, $typy = null)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PoznamkaKeDni.DatumOd, PoznamkaKeDni.DatumDo, PoznamkaKeDni.Typ, PoznamkaKeDni.Popis, ' .
            '  TypPoznamkyKeDni.Barva, ' . $database ->castWideMemo('PoznamkaKeDni.Poznamka') .
            ' FROM PoznamkaKeDni, TypPoznamkyKeDni, RozdeleniOsob, PracovniPomer ' .
            ' WHERE TypPoznamkyKeDni.Kod = PoznamkaKeDni.Typ ' .
            // poznamky v zadanem obdobi
            '  AND PoznamkaKeDni.DatumDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '  AND PoznamkaKeDni.DatumOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            // prirazene k rozdeleni zadane osoby
            '  AND PoznamkaKeDni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni' .
            '  AND PoznamkaKeDni.Rozdeleni = RozdeleniOsob.Rozdeleni' .
            '  AND PracovniPomer.RC = RozdeleniOsob.RC ' .
            '  AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo ' .
            '  AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu ' .
            '  AND PracovniPomer.Id = ' . $params ->addParameter((int)$idpp) .
            '  AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '  AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)';

        if ($typy) {
            $sql .=
                ' AND (' .
                SQLUtils::getINCondition($database, 'PoznamkaKeDni.Typ', $typy, $params) .
                ')';
        }

        return self::nactiPoznamkySQL($database, $sql, $params);
    }
}
