<?php
/**
 * UTF8
 * stanislav.elbl, 01.10.2015
 */

namespace Ron\RonPortal;

require_once('ron/mvc.php');
require_once('ron/database.php');
require_once('adsapp/obdobi.php');
require_once('adsapp/dochazkautils.php');
require_once('dochazka/schvalenidochazky.php');

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\Http\App;
use Ron\Http\HtmlHeaders;
use Ron\MVC\JsonController;
use Ron\MVC\Model;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Redirect;
use Ron\MVC\JsonOutputObject;
use Ron\RonPortal\XML\RonPortalXMLForm;
use Ron\StringUtils;
use Ron\TextManager;


class SchvaleniView extends BaseViewObject
{
    var $xmlEventsText = '';
    var $showDialog;

    var $redirectUrl = null;

    /**
     * SchvaleniView constructor.
     * @param Database $database
     * @param bool $showDialog
     */
    public function __construct($database, $showDialog = false)
    {
        parent::__construct('view/schvalenidochazky.php');

        $this ->showDialog = $showDialog;

        $obdobi = new Obdobi();
        $obdobi ->nactiObdobi($database, 'ADS', RonPortalSession::obdobi());

        Model::setData('obdobi', $obdobi);
        Model::setData('datumzpracovanidochazky', DochazkaUtils::datumCasZpracovaniDochazky($database));

        // nacte pripadne xml formulare
        require_once('xml/xmlformportal.php');
        //require_once('xml/xmlformelement.php');
        require_once('adsapp/events.php');
        $this ->xmlEventsText = RonPortalXMLForm::xmlEvents($database, RonPortalXMLEvents::SCHVALOVANI_SHOW, '', '', true); // returnContent = true
    }


    public function render()
    {
        parent::render();

        echo $this ->xmlEventsText;

        if ($this ->showDialog) {
            $obdobistr = date('j.n.Y', RonPortalSession::obdobi());
            $idpp = RonPortalSession::vybranaOsoba();
            echo
                "<script type='text/javascript'>" .
                ( App::getDebugMode() ? "console.log('Zobrazuji schvaleni pro idpp $idpp, obdobi $obdobistr');" : '') .
                "ronPortal.dochazka.zobrazSchvaleniDochazky($idpp, '$obdobistr', " . ($this ->redirectUrl ? "'{$this ->redirectUrl}'" : 'null') . ");" .
                "</script>";

        }
    }
}



class SchvalovaniController extends JsonController
{

    /** @var Database */
    var $database;

    /**
     * SchvalovaniController constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;
    }


    public static function lzeSchvalovatOsobu($idpp)
    {
        $prava = Prava::vratPrava('zpracovani');
        if ($prava -> lzeZobrazitPP($idpp, RonPortalSession::obdobi(), RonPortalSession::konecObdobi())) return true;

        $prava = Prava::vratPrava('zkontrolovani');
        if ($prava -> lzeZobrazitPP($idpp, RonPortalSession::obdobi(), RonPortalSession::konecObdobi())) return true;

        $prava = Prava::vratPrava('schvaleni');
        if ($prava -> lzeZobrazitPP($idpp, RonPortalSession::obdobi(), RonPortalSession::konecObdobi())) return true;

        return false; // nema prava pro zadnou uroven
    }


    /**
     * Vraci ID XML formulare s nastavenim seznamu pro schvalovani dochazky
     * @return bool|int
     */
    function getXMLSchvalovani()
    {
//        global $_allwaysloadsettings;
//        if ( !isset($_allwaysloadsettings)  ||  !$_allwaysloadsettings ) {
//            if ( isset($_SESSION['XML_FORMULAR_SCHVALENI_DOCHAZKY']) ) {
//                return $_SESSION['XML_FORMULAR_SCHVALENI_DOCHAZKY'];
//            }
//        }

        $xmlid = DBUtils::getSqlValue($this ->database, "SELECT Id FROM SablonaXMLFormulare WHERE Popis = 'SCHVALOVANI_DOCHAZKY'");
        $result = $xmlid ? $xmlid : false;

//        $_SESSION['XML_FORMULAR_SCHVALENI_DOCHAZKY'] = $result;
        return $result;
    }


    function schvalovaniBrowser()
    {
        $xmlId = $_GET['force_no_xml'] ? null : $this->getXMLSchvalovani();

        if ($xmlId) {

            // zobrazeni uzivatelsky nastaveneho seznamu
            require_once('dochazka/schvalovani_xml.php');

            $form = new SchvalovaniXML($this ->database);
            $form->load($xmlId);

            Model::addViewObject($form);

            Model::addViewObject( new BaseViewObject('view/schvaleni_progress.php'));
            //require('view/schvaleni_progress.php');

        } else {
            // standardni zobrazeni seznamu osob
            require_once('dochazka/b_schvalovani.php');
            $b = new SchvalovaniDochazkyBrowser($this->database);
            $b->load();

            Model::addViewObject($b);
        }


        Model::addViewObject(new SchvaleniView($this->database));
        //zobrazDialogSchvaleniOsoby($database, false); // zobrazitHned=false => dialog je hidden

        //require_once

        return 'view/appview.php';
    }


    function hromadneSchvalovani()
    {
        require_once('dochazka/f_hromadneschvaleni.php');

        $obdobi = isset($_GET['obdobi']) ? DateUtils::urlStringToDate($_GET['obdobi']) : RonPortalSession::obdobi();

        $form = new HromadneSchvaleniFormular($this ->database);
        $form->nastavObdobi($obdobi);
        $form->nactiData();

        return $form;
    }


    function provedHromadneSchvaleni()
    {
        if (!$_POST['obdobi']) throw new \Exception('Chyba - neni zadano obdobi!');

        require_once('dochazka/f_hromadneschvaleni.php');

        $form = new HromadneSchvaleniFormular($this ->database);
        $form->nastavObdobi( DateUtils::urlStringToDate($_POST['obdobi']) );

        if (!$form->zapisData()) {
            $form->nactiData();
            return $form;
        }

        return new Redirect(null, "?menuid=schvalovani&schvaleniOK"); // parametr nepodstatny
    }


    /**
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function loadData($output)
    {
        require_once('ron/sqlutils.php');
        require_once('adsapp/ppomer.php');

        $pp = new PracovniPomer();
        if (!$pp ->nactiPPPodleId($this ->database, (int)$_GET['idpp'])) {
            throw new \Exception('Nelze nacist pracovni pomer!');
        }

        $pocatekobdobi = DateUtils::dateStringToTimestamp($_GET['obdobi']);
        if (!$pocatekobdobi) {
            throw new \Exception('Chyba - spatne zadane obdobi!');
        }

        $obdobi = Obdobi::vratObdobi($this ->database, 'ADS', $pocatekobdobi);

        //nactiPravaUzivatele($this ->database, S_GetUserRC());
        Prava::nastavObdobiProPrava($obdobi ->zacatek, $obdobi ->konec);

        if ( !SchvaleniDochazky::lzeSchvalovatOsobu($pp ->idPP, $obdobi ->zacatek, $obdobi ->konec) ) {
            // zkontroluje pravo na zobrazeni
            $prava = Prava::vratPrava('prohlizeni');
            if ( !$prava ->lzeZobrazitPP($pp ->idPP) ) {
                // jeste zkontroluje pravo na editaci
                $prava = Prava::vratPrava('editace');
                if ( !$prava ->lzeZobrazitPP($pp ->idPP)) {
                    if (!User::getRC()) {
                        // doslo k odhlaseni uzivatele
                        throw new \Exception('Pravdepodobne doslo k automatickemu odhlaseni! Prihlaste se prosim znovu.');
                    } else {
                        throw new NeniPravoException();
                    }
                }
            }
        }

        $schvaleni = new SchvaleniDochazky();

        $schvaleni->load($this->database, $pp->rc, $pp->osobniCislo, $pp->datumNastupu, $obdobi->zacatek, $obdobi->zacatek, true);

        $output ->addData('idpp', $pp->idPP);
        $output ->addData('pocatekObdobi', date('j.n.Y', $obdobi->zacatek));
        $output ->addText('jmeno', $pp->jmenoPrijmeni, false); // htmlconvert = false
        $output ->addData('zpracovanoDo', StringUtils::jsonDate($schvaleni->schvaleni1));
        $output ->addText('zpracoval', $schvaleni->schvalil1 . ($schvaleni->datumSchvaleni1 ? ' (' . DateUtils::fineDateString($schvaleni->datumSchvaleni1, true) . ')' : ''));
        $output ->addData('pravoZpracovat', Prava::vratPrava('zpracovani') ->lzeZobrazitPP($pp->idPP) ? 1 : 0);
        $output ->addData('zkontrolovanoDo', StringUtils::jsonDate($schvaleni->schvaleni2));
        $output ->addText('zkontroloval', $schvaleni->schvalil2 . ($schvaleni->datumSchvaleni2 ? ' (' . DateUtils::fineDateString($schvaleni->datumSchvaleni2, true) . ')' : ''));
        $output ->addData('pravoZkontrolovat', Prava::vratPrava('zkontrolovani') ->lzeZobrazitPP($pp->idPP) ? 1 : 0);
        $output ->addData('schvalenoDo', StringUtils::jsonDate($schvaleni->schvaleni3));
        $output ->addText('schvalil', $schvaleni->schvalil3 . ($schvaleni->datumSchvaleni3 ? ' (' . DateUtils::fineDateString($schvaleni->datumSchvaleni3, true) . ')' : ''));
        $output ->addData('pravoSchvalit', Prava::vratPrava('schvaleni') ->lzeZobrazitPP($pp->idPP) ? 1 : 0);
        $output ->addData('zpracovanoVeMzdach', (int)$schvaleni->zpracovanoVeMzdach);
        $output ->addText('poznamka', $schvaleni->poznamka);
    }


    /**
     * Spusti skript SCHVALENIDOCHAZKY_IW_BEFORE, Schvali a pak jeste provede IW_AFTER
     */
    function provedSchvaleniOsoby()
    {
        /** @var SchvaleniDBRecord $rec */
        require_once('dochazka/schvalenirec.php');
        $rec = new SchvaleniDBRecord($this->database, $this);

        // nacte puvodni data
        $exists = $rec->loadDataJS();

        $rec->loadPost(); // nacte data odeslana uzivatelem

        $rec->zkontrolujAZapisSchvaleni(!$exists);
    }


    /**
     * Spusti skript SCHVALENIDOCHAZKY_IW_CHECK
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function checkSchvaleniOsoby($output)
    {
        require_once('dochazka/schvalenirec.php');
        $rec = new SchvaleniDBRecord($this->database, $this);

        $rec->loadDataJS();

        // nacte data odeslana uzivatelem
        $rec->loadPost();

        if ($rec->isChanged()) {
            $ok = true;

            require_once('adsapp/events.php'); // nazvy skriptu a udalosti
            try {
                $scriptNames = [RonPortalScriptEvents::SCHVALENI_DOCHAZKY_CHECK];
                if ($rec ->fZpracovano ->isChanged()) $scriptNames[] = RonPortalScriptEvents::SCHVALENI_DOCHAZKY_1_CHECK;
                if ($rec ->fZkontrolovano ->isChanged()) $scriptNames[] = RonPortalScriptEvents::SCHVALENI_DOCHAZKY_2_CHECK;
                if ($rec ->fSchvaleno ->isChanged()) $scriptNames[] = RonPortalScriptEvents::SCHVALENI_DOCHAZKY_3_CHECK;

                $rec->executeScripts($scriptNames);
            } catch (\Exception $e) {
                $output ->addError($e ->getMessage());
                $output->setResult('warning');
                $ok = false;
            }

            if ($ok) {
                // check probehl v poradku, provede rovnou i schvaleni
                $this->provedSchvaleniOsoby();
            }
        }
    }


    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'loaddata' :
                $this ->loadData($output);
                return;

            case 'schvalit' :
                // spusteni skriptu pro overeni - v pripade chyby se zobrazi dotaz, jestli chce pokracovat
                // a pokud neni chyba, nebo skript neexistuje, provede schvaleni
                $this ->checkSchvaleniOsoby($output);
                return;

            case 'schvalit_no_check' :
                // spusteni skriptu pro kontrolu a zapis schvaleni (bez skriptu pro overeni - uzivatel uz potvrdil)
                $this ->provedSchvaleniOsoby();
                return;

            default :
                throw new \Exception("chyba ve SchvalovaniController::handleJsonAction - neznama akce!");
        }
    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini');
        HtmlHeaders::getInstance() ->addScript('script/dochazka.js');
        HtmlHeaders::getInstance() ->addCss('css/dochazka.css');

        if (isset($_POST['schvaleniaction'])) {
            switch ($_POST['schvaleniaction']) {
//                case 'provedschvaleni' :
//                    throw new \Exception('NENI IMPLEMENTOVANO!');
//                    return provedSchvaleniDochazky($connection);

                case 'hromadneschvaleni' :
                    return $this ->provedHromadneSchvaleni();
            }
        }

        if (isset($_GET['hromadneschvaleni'])  &&  $_GET['hromadneschvaleni']) {
            return $this ->hromadneSchvalovani();
        }

        switch ($_GET['action']) {
            case 'loaddata' :
            case 'schvalit' :
            case 'schvalit_no_check' :
                return parent::handleAction($action);
        }

        return $this ->schvalovaniBrowser();
    }

}
