<?php
/**
 * UTF8
 * stanislav.elbl, 26.10.2015
 */

namespace Ron\RonPortal;

require_once('adsapp/ronportaldbrecord.php');

use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\DBRecordField;
use Ron\MessageManager;
use Ron\NumberUtils;
use Ron\StringUtils;


class DenniVysledekVypoctuRecord extends RonPortalDBRecord
{
    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $den
     * @param string $typCasoveSlozkyMzdy
     * @param string|object $logDescription
     */
    function __construct($database, $pp, $den, $typCasoveSlozkyMzdy, $logDescription = 'Zpracovana dochazka')
    {
        parent::__construct($database, 'DenniVysledekVypoctu', $logDescription);

        $this->addStringField('RC', true, 15, $pp ->rc);
        $this->addStringField('OsobniCislo', true, 15, $pp ->osobniCislo);
        $this->addDateField('DatumNastupu', true, $pp ->datumNastupu);
        $this->addField('', '', true, 0, '   ' . $pp ->jmenoPrijmeni);

        $this->addDateField('Den', true, $den);
        $this->addStringField('TypCasoveSlozkyMzdy', true, 10, $typCasoveSlozkyMzdy);

        $this->addStringField('Insert_UserName', true, 25, RonPortalUtils::getLogUserName());
        $this->addTimestampField('Insert_Timestamp', true, time());

        $f = $this->addStringField('Update_UserName', false, 25, RonPortalUtils::getLogUserName());
        $f ->setUpdateLog(true) ->setNoInsert(true);

        $f = $this->addTimestampField('Update_Timestamp', false, time());
        $f ->setUpdateLog(true) ->setNoInsert(true);

        $params = $database ->createParameters();
        $this->setWhere(
            'RC = ' . $params ->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            ' AND Den = ' . $params ->addParameter($database ->dateToParam($den)) .
            ' AND TypCasoveSlozkyMzdy = ' . $params ->addParameter($typCasoveSlozkyMzdy),
            $params);
    }


    /**
     * @param string $jednotka
     * @param mixed $hodnota
     * @param bool $vlozitNulovouHodnotu    Urcuje, jestli se ma vlozit novy zaznam, pokud je zadana hodnota nulova
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function zapisHodnotu($jednotka, $hodnota, $vlozitNulovouHodnotu = false)
    {
        // nastaveni pole s hodnotou
        $hodiny = 0;
        $dny = 0;
        $kalendarnidny = 0;
        $poznamka = null;

        switch ($jednotka) {
            case 'D' :
                $dny = $hodnota;
                break;
            case 'K' :
                $kalendarnidny = $hodnota;
                break;
            case 'P' :
                $poznamka = $hodnota;
                break;
            default :
                $hodiny = $hodnota;
                break;
        }

        $this ->addField('Hodiny', 'float', $jednotka == 'D' || $jednotka == 'K' || $jednotka == 'P', 0, $hodiny); // readonly pokud se nejedna o hodiny
        $this ->addField('Dny', 'float', $jednotka != 'D', 0, $dny);
        $this ->addField('KalendarniDny', 'float', $jednotka != 'K', 0, $kalendarnidny);
        $this ->addStringField('Poznamka', $jednotka != 'P', 250, $poznamka);

        if ($this ->loadRecord()) {
            $this ->updateRecord();
        } else if ($vlozitNulovouHodnotu  ||  $hodnota || ($jednotka == 'P' && $hodnota != '')) {
            $this ->insertRecord();
        }
    }
}



class MesicniVysledekVypoctuRecord extends RonPortalDBRecord
{
    var $f_hodiny;
    var $f_dny;
    var $f_kalendarniDny;
    var $f_poznamka;

    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $mesic
     * @param string $typCasoveSlozkyMzdy
     * @param string|object $logDescription
     */
    function __construct($database, $pp, $mesic, $typCasoveSlozkyMzdy, $logDescription = 'Zpracovana dochazka')
    {
        parent::__construct($database, 'MesicniVysledekVypoctu', $logDescription);

        $this->addField('RC', '', true, 0, $pp->rc); // readonly = true
        $this->addField('OsobniCislo', '', true, 0, $pp->osobniCislo);
        $this->addField('DatumNastupu', 'date', true, 0, $pp->datumNastupu);
        $this->addField('', '', true, 0, '   ' . $pp ->jmenoPrijmeni);

        $this->addField('Mesic', 'date', true, 0, $mesic); // readonly = true
        $this->addField('TypCasoveSlozkyMzdy', '', true, 0, $typCasoveSlozkyMzdy);

        $this ->f_hodiny = $this ->addField('Hodiny', 'float', true, 0, 0); // prozatim readonly
        $this ->f_dny = $this ->addField('Dny', 'float', true, 0, 0);
        $this ->f_kalendarniDny = $this ->addField('KalendarniDny', 'float', true, 0, 0);

        $this ->f_poznamka = $this ->addField('Poznamka', '', true, 250, null);

        $params = $database ->createParameters();
        $this->setWhere(
            'RC = ' . $params ->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            ' AND Mesic = ' . $params ->addParameter($database ->dateToParam($mesic)) .
            ' AND TypCasoveSlozkyMzdy = ' . $params ->addParameter($typCasoveSlozkyMzdy),
            $params);
    }


    function zapisHodnotu($hodiny, $dny, $kalendarniDny, $poznamka)
    {
        if (isset($hodiny)) $this ->f_hodiny ->setNewValue($hodiny);
        if (isset($dny)) $this ->f_dny ->setNewValue($dny);
        if (isset($kalendarniDny)) $this ->f_kalendarniDny ->setNewValue($kalendarniDny);

        if (isset($poznamka)) $this ->f_poznamka ->setNewValue($poznamka === '' ? null : $poznamka);

        $this ->updateOrInsert();
    }


}



class DenniPoznamkaDochazkyRecord extends RonPortalDBRecord
{
    /** @var DBRecordField  */
    var $fPoznamka;

    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int    $den
     * @param string|object $logDescription    Vychozi hodnota je 'Zpracovana dochazka'
     */
    function __construct($database, $pp, $den, $logDescription = 'Zpracovana dochazka')
    {
        parent::__construct($database, 'DenniPoznamkaDochazky', $logDescription);

        $this->addField('RC', '', true, 15, $pp ->rc);
        $this->addField('OsobniCislo', '', true, 15, $pp ->osobniCislo);
        $this->addField('DatumNastupu', 'date', true, 0, $pp ->datumNastupu);
        $this->addField('', '', true, 0, '   ' . $pp ->jmenoPrijmeni);

        $this->addField('Den', 'date', true, 0, $den);

        $this->fPoznamka = $this->addField('Poznamka', 'memo', false, 0, null);

        $pomfield = $this->addField('Update_UserName', '', false, 25, RonPortalUtils::getLogUserName());
        $pomfield->updateLog = true;
        $pomfield->noInsert = true;

        $pomfield = $this->addField('Update_Timestamp', 'datetime', false, 25, time());
        $pomfield->updateLog = true;
        $pomfield->noInsert = true;

        $this->addField('Insert_UserName', '', true, 25, RonPortalUtils::getLogUserName());
        $this->addField('Insert_Timestamp', 'datetime', true, 25, time());

        $params = $database ->createParameters();
        $this->setWhere(
            'RC = ' . $params ->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            ' AND Den = ' . $params ->addParameter($database ->dateToParam($den)),
            $params);
    }


    /**
     * @param string $text
     */
    function zapisPoznamku($text)
    {
        $this->fPoznamka->setNewValue($text);

        if ($this->loadRecord()) {
            if (trim($text) == '') {
                $this->deleteRecord();
            } else {
                $this->updateRecord();
            }

        } else {
            if (trim($text) == '') return; // neni co vkladat

            $this->insertRecord();
        }
    }
}



class RozliseniDochazkyRecord extends RonPortalDBRecord
{
    /** @var DBRecordField  */
    var $fTypRozdeleni;
    /** @var DBRecordField  */
    var $fKodRozdeleni;

    /**
     * @param Database      $database
     * @param PracovniPomer $pp
     * @param int           $datumCasPocatku
     * @param string|object $logDescription
     */
    function __construct($database, $pp, $datumCasPocatku, $logDescription = 'Zpracovana dochazka')
    {
        parent::__construct($database, 'RozliseniDochazky', $logDescription);

        $this->addField('RC', '', true, 15, $pp ->rc);
        $this->addField('OsobniCislo', '', true, 15, $pp ->osobniCislo);
        $this->addField('DatumNastupu', 'date', true, 0, $pp ->datumNastupu);

        $this->addField('DatumCasPocatku', 'datetime', true, 0, $datumCasPocatku);

        $this->fTypRozdeleni = $this->addField('KodTypuRozdeleni', 'int', true, 0, null);
        $this->fKodRozdeleni = $this->addField('KodRozdeleni', '', true, 30, null);

        $params = $database ->createParameters();
        $this->setWhere(
            'RC = ' . $params->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            ' AND DatumCasPocatku = ' . $params ->addParameter($database ->timestampToParam($datumCasPocatku)) .
            ' AND KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)),
            $params);
    }


    /**
     * @param int    $typ
     * @param string $kod
     * @param bool   $onlyInsert  pokud je true, tak nekontroluje, jestli zaznam existuje (pouzije se pri vlozeni zaznamu dochazky)
     */
    function zapisRozdeleni($typ, $kod, $onlyInsert = false)
    {
        $this ->fTypRozdeleni ->setNewValue($typ);
        $this ->fKodRozdeleni ->setNewValue($kod);

        if ($onlyInsert) {
            $exists = false;
        } else {
            $exists = $this->loadRecord();
        }

        if ($exists) {
            if (trim($kod) == '') {
                $this->doDelete(null);
            } else {
                $this->updateRecord();
            }
        } else {
            if (trim($kod) != '') {
                $this->insertRecord();
            }
        }
    }
}


class PoznamkaDochazkyRecord extends RonPortalDBRecord
{
    /** @var DBRecordField  */
    var $fPoznamka;


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int           $datumCasPocatku
     * @param string|object $logDescription
     */
    function __construct($database, $pp, $datumCasPocatku, $logDescription = 'Zpracovana dochazka')
    {
        parent::__construct($database, 'PoznamkaDochazky', $logDescription);

        $this->addField('RC', '', true, 15, $pp ->rc);
        $this->addField('OsobniCislo', '', true, 15, $pp ->osobniCislo);
        $this->addField('DatumNastupu', 'date', true, 0, $pp ->datumNastupu);

        $this->addField('DatumCasPocatku', 'datetime', true, 0, $datumCasPocatku);

        $this->fPoznamka = $this->addField('Poznamka', '', true, 160, null);

        $params = $database ->createParameters();
        $this->setWhere(
            'RC = ' . $params ->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            ' AND DatumCasPocatku = ' . $params ->addParameter($database ->timestampToParam($datumCasPocatku)),
            $params);
    }


    /**
     * @param string $text
     * @param bool   $onlyInsert  pokud je true, tak nekontroluje, jestli zaznam existuje (pouzije se pri vlozeni zaznamu dochazky)
     */
    function zapisPoznamku($text, $onlyInsert = false)
    {
        $this ->fPoznamka ->setNewValue( $text );

        if ($onlyInsert) {
            $exists = false;
        } else {
            $exists = $this ->loadRecord();
        }

        if ($exists) {
            if (trim($text) == '') {
                $this ->doDelete(null);
            } else {
                $this->updateRecord();
            }
        } else {
            if (trim($text) != '') {
                $this->insertRecord();
            }
        }
    }
}


class DochazkaRecord extends RonPortalDBRecord
{
    const SETTINGS_PREPISOVAT = 'editaceprepisovat';


    /** @var PracovniPomer */
    var $f_pp;

    /** @var DBRecordField */
    var $fRC;
    /** @var DBRecordField */
    var $fOsobniCislo;
    /** @var DBRecordField */
    var $fDatumNastupu;

    /** @var DBRecordField */
    var $fDatumCasPocatku;
    /** @var DBRecordField */
    var $fDatumCasUkonceni;
    /** @var DBRecordField */
    var $f_den;
    /** @var DBRecordField */
    var $f_operace;
    /** @var DBRecordField */
    var $fDeltaPred;
    /** @var DBRecordField */
    var $fDeltaZa;
    /** @var DBRecordField */
    var $fDatumCasOperacePuvDoch;
    /** @var DBRecordField */
    var $f_smena;
    /** @var DBRecordField */
    var $f_zpracoval;

    /**
     * @param Database      $database
     * @param PracovniPomer $pp
     * @param string|object $logDescription
     */
    function __construct($database, $pp, $logDescription = 'Zpracovana dochazka')
    {
        parent::__construct($database, 'Dochazka', $logDescription);

        $this ->f_pp = $pp;

        $this ->fRC = $this ->addField('RC', '', true, 15, $pp ->rc);
        $this ->fOsobniCislo = $this ->addField('OsobniCislo', '', true, 15, $pp ->osobniCislo);
        $this ->fDatumNastupu = $this ->addField('DatumNastupu', 'date', true, 0, $pp ->datumNastupu);
        if ($pp ->jmenoPrijmeni) $this ->addField(null, '', true, 0, '   ' . $pp ->jmenoPrijmeni); // jen pro log

        $this ->f_den = $this ->addField('Den', 'date', true);

        $this ->fDatumCasPocatku = $this ->addField('DatumCasPocatku', 'datetime', true);
        $this ->fDatumCasUkonceni = $this ->addField('DatumCasUkonceni', 'datetime', true);
        $this ->f_operace = $this ->addField('Operace', '', true);
        $this ->fDeltaPred = $this ->addField('DeltaPred', 'float', true);
        $this ->fDeltaZa = $this ->addField('DeltaZa', 'float', true);

        $this ->f_smena = $this ->addField('Smena', '', true);

        $this ->fDatumCasOperacePuvDoch = $this ->addField('DatumCasOperacePuvDoch', 'datetime', true); // NESMI BYT NULL!

        $this ->f_zpracoval = $this ->addField('Zpracoval', '', false, 25, RonPortalUtils::getLogUserName());
        $this ->f_zpracoval ->updateLog = true;

        $params = $database ->createParameters();
        $this->setWhere(
            'RC = ' . $params ->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)),
            $params);

        $this ->orderBy = 'DatumCasPocatku';
    }


    /**
     * Vraci zapocitany cas od (tj. cas pocatku upraveny o deltaPred)
     * @param bool $original    Pokud je true, tak vraci hodnotu nactenou z databaze
     * @return float|mixed
     */
    function getZapoctenyCasOd($original = false)
    {
        $result = $original ? $this ->fDatumCasPocatku ->originalValue : $this ->fDatumCasPocatku ->value;
        $result -= round(86400 * ($original ? $this ->fDeltaPred ->originalValue : $this ->fDeltaPred ->value) );

        return $result;
    }


    /**
     * Vraci zapocitany cas od (tj. cas pocatku upraveny o deltaPred)
     * @param bool $original    Pokud je true, tak vraci hodnotu nactenou z databaze
     * @return float|mixed
     */
    function getZapoctenyCasDo($original = false)
    {
        $result = $original ? $this ->fDatumCasUkonceni ->originalValue : $this ->fDatumCasUkonceni ->value;
        $result += round(86400 * ($original ? $this ->fDeltaZa ->originalValue : $this ->fDeltaZa ->value) );

        return $result;
    }


    /**
     * @param string $podminka
     * @param Parameters $params
     * @return int  Vraci pocet smazanych zaznamu
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function smazZaznamyDochazky($podminka, $params)
    {
        if (!$podminka) {
            throw new \Exception('Chyba ve funkci smazZaznamyDochazky - neni zadana podminka!');
        }

        return $this ->deleteRecords($podminka, $params);
    }


    /**
     * V aktualnim zaznamu zmeni cas prichodu
     * @param int $cas
     * @param bool $zapocitanyCas   Pokud je true, meni se jen zapocitany cas, zobrazeny zustane (ale v pripade, ze je po zapocitanem, tak ho taky posune)
     * @return bool|int
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function nastavCasPrichodu($cas, $zapocitanyCas = false)
    {
        $params = $this ->database ->createParameters();
        $filtr = ' DatumCasPocatku = ' . $params ->addParameter($this ->database ->timestampToParam($this ->fDatumCasPocatku ->originalValue));

        $this ->logInfoMessage = StringUtils::fromUTF8('Posunuti casu zacatku');

        if ($zapocitanyCas) {
            // Meni jen zapocitany cas. Zobrazeny (pokud je to mozne) nechava - posune jej jen v pripade, ze je potreba prodlouzit.
            if ($this->fDatumCasPocatku ->originalValue >= $cas) {
                // zaznam zacinal az po zadanem case - posune zacatek vcetne zobrazeneho casu
                $this->fDatumCasPocatku->setNewValue($cas);
                $this->fDeltaPred->setNewValue(null);
            } else {
                // meni jen zapocitany cas
                $this ->fDeltaPred ->setNewValue( ($this ->fDatumCasPocatku ->originalValue - $cas) / 86400 );
            }
        } else {
            // nastavi zobrazeny cas
            $this->fDatumCasPocatku->setNewValue($cas);

            // a pokud byla delta tak ji pripadne upravi
            if ($this ->fDeltaPred ->originalValue) {
                //$zapoctenyOd = $this ->fDatumCasPocatku ->originalValue - round(86400 * $this ->fDeltaPred ->originalValue);
                $zapoctenyOd = $this ->getZapoctenyCasOd(true);

                if ($zapoctenyOd > $cas) {
                    $newDeltaPred = ($cas - $zapoctenyOd) / 86400;
                    require_once('ron/numberutils.php');
                    if (NumberUtils::floatsEqual($newDeltaPred, 0)) $newDeltaPred = null;
                    $this->fDeltaPred->setNewValue($newDeltaPred);
                } else {
                    $this->fDeltaPred->setNewValue(null);
                }
            }
        }


        if ( $this ->isChanged() ) {
            require_once('adsapp/events.php');

            $this->executeXmlEvents(RonPortalXMLEvents::EDITACE_DOCHAZKY_BEFORE, 'update');
            $this->updateRecord($filtr, $params);
            $this->executeXmlEvents(RonPortalXMLEvents::EDITACE_DOCHAZKY_AFTER, 'update');
        }

        $this->fDatumCasPocatku->readOnly = true; // prepne zpatky na readonly
        $this ->fDeltaPred ->readOnly = true;

        return true;
    }


    /**
     * V aktualnim zaznamu zmeni cas odchodu (pouzito pri editaci prichodu/odchodu nebo pri automatickem prepisovani zaznamu)
     * @param int $cas
     * @param bool $zapocitanyCas
     * @return bool|int
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function nastavCasOdchodu($cas, $zapocitanyCas = false)
    {
        $params = $this ->database ->createParameters();
        $filtr = ' DatumCasPocatku = ' . $params ->addParameter($this ->database ->timestampToParam($this ->fDatumCasPocatku ->originalValue));

        $this ->logInfoMessage = StringUtils::fromUTF8('Posunuti casu ukonceni');

        if ($zapocitanyCas) {
//            // Meni jen zapocitany cas. Zobrazeny (pokud je to mozne) nechava - posune jej jen v pripade, ze je potreba prodlouzit.
//            if ($this->f_datumcaspocatku ->originalValue > $cas) {
//                // zaznam zacinal az po zadanem case - posune zacatek vcetne zobrazeneho casu
//                $this->f_datumcaspocatku->setNewValue($cas);
//                $this->f_deltapred->setNewValue(null);
//            } else {
//                // meni jen zapocitany cas
//                $this ->f_deltapred ->originalValue = ($this ->f_datumcaspocatku ->originalValue - $cas) / 86400;
//            }
            // Meni jen zapocitany cas. Zobrazeny  nechava (pokud je to mozne) - posune jej jen v pripade, ze je potreba prodlouzit.
            if ($this->fDatumCasUkonceni ->originalValue <= $cas) {
                // zaznam koncil az po zadanem case - posune konec vcetne zobrazeneho casu
                $this->fDatumCasUkonceni->setNewValue($cas);
                $this->fDeltaZa->setNewValue(null);
            } else {
                // meni jen zapocitany cas
                $this ->fDeltaZa ->setNewValue(-($this ->fDatumCasUkonceni ->originalValue - $cas) / 86400);
            }
        } else {
            // meni i zobrazeny cas
            $this->fDatumCasUkonceni->setNewValue($cas);

            if ($this->fDeltaZa->originalValue) {
                $zapoctenoDo = $this ->getZapoctenyCasDo(true);
                if ($zapoctenoDo < $cas) {
                    $this->fDeltaZa->setNewValue(-($cas - $zapoctenoDo) / 86400);
                } else {
                    $this->fDeltaZa->setNewValue(null);
                }
            }

            if ($this ->fDeltaPred ->originalValue) {
                // jeste muze byt potreba zkratit delta pred (napriklad kdyz se misto casti "zobacku" vklada nova operace
                $zapoctenyOd = $this ->getZapoctenyCasOd(true);
                if ($zapoctenyOd > $cas) {
                    $newDeltaPred = ($this ->fDatumCasPocatku ->value - $cas) / 86400;
                    require_once('ron/numberutils.php');
                    if (NumberUtils::floatsEqual($newDeltaPred, 0)) $newDeltaPred = null;
                    $this ->fDeltaPred ->setNewValue($newDeltaPred);
                }
            }
        }

        $result = true;

        if ( $this ->isChanged() ) {
            require_once('adsapp/events.php');

            $result
                = $this->executeXmlEvents(RonPortalXMLEvents::EDITACE_DOCHAZKY_BEFORE, 'update')
                && $this->updateRecord($filtr, $params)
                && $this->executeXmlEvents(RonPortalXMLEvents::EDITACE_DOCHAZKY_AFTER, 'update');
        }

        $this->fDatumCasUkonceni->readOnly = true; // prepne zpatky na readonly
        $this->fDeltaZa->readOnly = true;

        return $result;
    }


    /**
     * Zkrati zaznam tak, aby nezasahoval do zadaneho intervalu (casod-casdo),
     * nebo ho uplne odstrani (pokud je kompletne v zadanem intervalu)
     * @param int $casOd
     * @param int $casDo
     * @return bool
     */
    function odstranZIntervalu($casOd, $casDo)
    {
        if ($this ->fDatumCasPocatku ->value >= $casOd  &&  $this ->fDatumCasUkonceni ->value <= $casDo) {
            // operace je cela v novem intervalu - smaze zaznam
            $params = $this ->database ->createParameters();
            $this ->doDelete( 'DatumCasPocatku = ' . $params ->addParameter($this ->database ->timestampToParam($this ->fDatumCasPocatku ->value)), $params );

        } else if ($this ->fDatumCasPocatku ->value < $casOd  &&  $this ->fDatumCasUkonceni ->value <= $casDo) {
            // operace zacina pred novym intervalem - zkrati zaznam (usekne konec)
            if ( !$this ->nastavCasOdchodu($casOd) ) {
                return MessageManager::addErrorMessage('Chyba ve funkci DochazkaRecord.odstranZIntervalu - chyba pri zkraceni zaznamu!');
            }
        } else if ($this ->fDatumCasPocatku ->value >= $casOd  &&  $this ->fDatumCasUkonceni ->value > $casDo) {

            // operace konci za novym intervalem - zkrati zaznam (usekne zacatek)
            if ( !$this ->nastavCasPrichodu($casDo) ) {
                return MessageManager::addErrorMessage('Chyba ve funkci DochazkaRecord.odstranZIntervalu - chyba pri zkraceni zaznamu!');
            }
        } else {
            $puvodniKonec = $this ->fDatumCasUkonceni ->value;
            $puvodniZapoctenoOd = $this ->getZapoctenyCasOd(true);
            $puvodniZapoctenoDo = $this ->getZapoctenyCasDo(true);

            // operace zacina pred a konci za - je potreba zkratit a vlozit kopii za interval
            if ( !$this ->nastavCasOdchodu($casOd) ) {
                return MessageManager::addErrorMessage('Chyba ve funkci DochazkaRecord.odstranZIntervalu - chyba pri preruseni zaznamu!');
            }

            if ( !$this ->vlozKopii($casDo, $puvodniKonec, $this ->f_operace ->value, $puvodniZapoctenoOd, $puvodniZapoctenoDo) ) {
                return MessageManager::addErrorMessage('Chyba ve funkci DochazkaRecord.odstranZIntervalu - chyba pri kopirovani zaznamu!');
            }
        }

        return true;
    }


    /**
     * Vlozi kopii aktualniho zaznamu, ale se zadanym casem od a do
     * @param int $casOd
     * @param int $casDo
     * @param string $operace
     * @param int $zapoctenoOd
     * @param int $zapoctenoDo
     * @return bool
     * @throws \Ron\DB\DBException
     */
    function vlozKopii($casOd, $casDo, $operace, $zapoctenoOd = null, $zapoctenoDo = null)
    {
        $this ->fDatumCasPocatku ->setNewValue($casOd);
        $this ->fDatumCasUkonceni ->setNewValue($casDo);
        $this ->fDeltaPred ->setNewValue(null);
        $this ->fDeltaZa ->setNewValue(null);
        $this ->fDatumCasOperacePuvDoch ->setNewValue($casOd); // nesmi byt NULL!

        if (isset($zapoctenoOd)  &&  $zapoctenoOd > $casOd) {
            // nastavi zobacek na zacatku
            $newDeltaPred = ($casOd - $zapoctenoOd) / 86400;
            require_once('ron/numberutils.php');
            if (NumberUtils::floatsEqual($newDeltaPred, 0)) $newDeltaPred = null;
            $this ->fDeltaPred ->setNewValue($newDeltaPred);
        }

        if ($zapoctenoDo < $casOd) {
            $zapoctenoDo = $casOd;
        }

        if (isset($zapoctenoDo)  &&  $zapoctenoDo < $casDo) {
            // nastavi zobacek na konci
            $this->fDeltaZa->setNewValue($zapoctenoDo < $casDo
                    ? -($casDo - $zapoctenoDo) / 86400
                    : null);
        }
/*
            if ($this->fDeltaZa->originalValue) {
                $zapoctenoDo = $this ->getZapoctenyCasDo(true);
                if ($zapoctenoDo < $cas) {
                    $this->fDeltaZa->setNewValue(-($cas - $zapoctenoDo) / 86400);
                } else {
                    $this->fDeltaZa->setNewValue(null);
                }
            }

            if ($this ->fDeltaPred ->originalValue) {
                // jeste muze byt potreba zkratit delta pred (napriklad kdyz se misto casti "zobacku" vklada nova operace
                $zapoctenyOd = $this ->getZapoctenyCasOd(true);
                if ($zapoctenyOd > $cas) {
                    $newDeltaPred = ($cas - $zapoctenyOd) / 86400;
                    require_once('ron/numberutils.php');
                    if (NumberUtils::floatsEqual($newDeltaPred, 0)) $newDeltaPred = null;
                    $this ->fDeltaPred ->setNewValue($newDeltaPred);
                }
            }


 */

        $this ->f_operace ->setNewValue($operace);

        require_once('adsapp/events.php');

        $this ->executeXmlEvents(RonPortalXMLEvents::EDITACE_DOCHAZKY_BEFORE, 'insert');
        $this ->insertRecord();
        $this ->executeXmlEvents(RonPortalXMLEvents::EDITACE_DOCHAZKY_AFTER, 'insert');

        return true;
    }


    /**
     * @param int $den
     * @param int $casOd
     * @param int $casDo
     * @param string $operace
     * @param string|false  $smena  Smena - pokud je null, dohleda se automaticky. Pokud je false, nenastavi se vubec.
     * @return bool
     */
    function vlozZaznamDochazky($den, $casOd, $casDo, $operace, $smena = null)
    {
        $this ->fDatumCasPocatku ->setNewValue($casOd);
        $this ->fDatumCasUkonceni ->setNewValue($casDo);
        $this ->f_den ->setNewValue($den);

        $this ->fDeltaPred ->setNewValue(null);
        $this ->fDeltaZa ->setNewValue(null);

        $this ->f_operace ->setNewValue($operace);

        $this ->fDatumCasOperacePuvDoch ->setNewValue($casOd); // nesmi byt NULL!

        require_once('editace/editace_utils.php');

        if (!isset($smena)) {
            $smena = EditaceUtils::vychoziSmenaProOsobu($this->database, $this->f_pp, $den);
        }

        if ($smena !== false) {
            $this ->f_smena ->setNewValue($smena);
        }

        require_once('adsapp/events.php');
        $this ->executeXmlEvents(RonPortalXMLEvents::EDITACE_DOCHAZKY_BEFORE, 'insert');
        $this ->insertRecord();
        $this ->executeXmlEvents(RonPortalXMLEvents::EDITACE_DOCHAZKY_AFTER, 'insert');

        return true;
    }


    /**
     * @param bool $insert
     * @return bool
     * @throws \Exception
     */
    function kontrolaPrekrytiZaznamuDochazky($insert)
    {
        $prava = Prava::vratPrava('editace');
        if ( RonPortalSettings::getIntValue(self::SETTINGS_PREPISOVAT)  &&  $prava ->priznak3 == '') {
            return true;
        }

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Dochazka.DatumCasPocatku, Dochazka.DatumCasUkonceni, Dochazka.Operace, OperaceDochazky.Popis ' .
            ' FROM Dochazka, OperaceDochazky ' .
            ' WHERE Dochazka.Operace = OperaceDochazky.Kod ' .
            '   AND Dochazka.RC = ' . $params->addParameter($this ->f_pp ->rc) .
            '   AND Dochazka.OsobniCislo = ' . $params->addParameter($this ->f_pp ->osobniCislo) .
            '   AND Dochazka.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->f_pp ->datumNastupu)) .
            // zaznamy v editovanem obdobi
            '   AND Dochazka.DatumCasPocatku < ' . $params ->addParameter($this ->database ->timestampToParam($this ->fDatumCasUkonceni ->value)) .
            '   AND Dochazka.DatumCasUkonceni > ' . $params ->addParameter($this ->database ->timestampToParam($this ->fDatumCasPocatku ->value));

        if (!$insert) // pri update vynecha editovany zaznam
            $sql .=
                '   AND Dochazka.DatumCasPocatku <> ' . $params ->addParameter($this ->database ->timestampToParam($this ->fDatumCasPocatku ->originalValue));

        $sql .= '  ORDER BY Dochazka.DatumCasPocatku ';

        $cursor = $this ->database ->select($sql, $params);
        $isError = false;

        require_once 'editace/editace_utils.php';

        while ($row = $cursor ->fetch()) {
            // vypise zaznamy, s kterymi se novy zaznam prekryva a nelze je automaticky zkratit (bud to neni vubec povoleno, nebo nema uzivatel na operaci pravo)
            if (!EditaceUtils::jePravoEditovatOperaci($row[2])) {
                // zaznam nelze ani prepsat - nema na to pravo
                throw new \Exception(\Ron\ronText('Chyba_prekryvani_zaznamu'));
            }

            //if (!RonPortalSettings::getIntValue(self::SETTINGS_PREPISOVAT)  ||  !EditaceUtils::jePravoEditovatOperaci($row[2])) {
            if (!RonPortalSettings::getIntValue(self::SETTINGS_PREPISOVAT)) {
                if (!$isError) {
                    MessageManager::addErrorMessage(\Ron\ronText('Chyba_prekryvani_zaznamu'));
                }

                $isError = true;

                MessageManager::addInfoMessage(DBUtils::getFormatedValue($this->database, 'datetime', $row[0]) . ' - ' . DBUtils::getFormatedValue($this->database, 'datetime', $row[1]) . ' (' . $row[2] . ' - ' . $row[3] . ")");
            }
        }
        $cursor ->close();

        return !$isError;
    }


    /**
     * Smaze nebo zkrati prekryvane zaznamy
     * @param bool $insert  Pokud je true, kontroluje vsechny zaznamy, jinak vynecha aktualni (editovany) zaznam
     * @param int $casOd    Pocatek intervalu, ktery se ma odstranit - pokud neni zadan, pouzije se pocatek aktualniho zaznamu
     * @param int $casDo    Konec ...
     * @return bool
     * @throws \Exception
     */
    function zkratPrekryvaneZaznamy($insert, $casOd = null, $casDo = null)
    {
        if (!isset($casOd)) {
            $casOd = $this ->fDatumCasPocatku ->value;
        }

        if (!isset($casDo)) {
            $casDo = $this ->fDatumCasUkonceni ->value;
        }

        $rec = new DochazkaRecord($this ->database, $this ->f_pp, $this ->logDescription);

        $params = $this ->database ->createParameters();
        $rec ->loadRecords(
            'DatumCasUkonceni > ' . $params ->addParameter($this ->database ->timestampToParam($casOd)) .
            ' AND DatumCasPocatku < ' . $params ->addParameter($this ->database ->timestampToParam($casDo)) .
            ($insert ? '' : (' AND DatumCasPocatku <> ' . $params ->addParameter($this ->database ->timestampToParam($this ->fDatumCasPocatku ->originalValue)) )),
            $params);

        while ( $rec ->next() ) {
            if ( !$rec ->odstranZIntervalu($casOd, $casDo) ) return false;

        }

        return true;
    }


    /**
     * V pripade chyby vyhodi vyjimku
     * @param string $name
     * @param bool   $puvodniCas
     * @param bool   $pridatHodnoty
     * @param string $parametry
     * @return bool
     */
    function executeScripts($name, $puvodniCas, $pridatHodnoty, $parametry = '')
    {
        $parametrSkriptu =
            "IdPP={$this ->f_pp ->idPP}";

        if ($puvodniCas)
            $parametrSkriptu .=
                ';DatumCasPocatkuPuvodni=' . date('j.n.Y-G:i:s', $this ->fDatumCasPocatku ->originalValue);

        $parametrSkriptu .=
            ';DatumCasPocatku=' . date('j.n.Y-G:i:s', $this ->fDatumCasPocatku ->value);

        if ($pridatHodnoty) {
            $parametrSkriptu .=
                ';DatumCasUkonceni=' . date('j.n.Y-G:i:s', $this ->fDatumCasUkonceni ->value) .
                ";Operace={$this ->f_operace ->value};Smena={$this ->f_smena ->value}";
        }

        if ($parametry) $parametrSkriptu .= ';' . $parametry;

        require_once('adsapp/agentmanager.php');
        AgentManager::executeScriptEvents($this ->database, RonPortalScriptEvents::CATEGORY_EDITACE, $name, $parametrSkriptu);

        return true;
    }

}
