<?php
/**
 * UTF8
 * Formular pro vlozeni dochazky podle kalendare
 * Uzivatel muze nastavit datum od - do (pouze v aktualni obdobi)
 * Vkladani pak probehne volanim agenta
 */

namespace Ron\RonPortal;


use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SQLUtils;
use Ron\MVC\ViewObject;

require_once('adsapp/ppomer.php');
require_once('adsapp/obdobi.php');


class PodleKalendareFormular implements ViewObject
{
    /** @var  Database */
    var $database;

    /** @var PracovniPomer */
    var $pp;
    /** @var Obdobi */
    var $obdobi;

//    var $f_Schvaleni; // schvaleni dochazky 
    var $datumZpracovaniDochazky;

    var $datumOd;
    var $datumDo;
    var $datumOdStr;
    var $datumDoStr;

    var $operace;
    var $rozdeleni;
    var $delka;         // cela smena, 1./2. pulka

    var $pouzitPocatek; // pouzit pocatek a konec smeny
    var $pevneJadro;    // pouzit pevne jadro

    var $automactikyOpakovat;
    var $vlozitPrestavky;


    function __construct($database)
    {
        $this ->database = $database;
    }


    function nactiDataPodlePost()
    {
        $OK = true;

        $this ->datumOdStr = $_POST['DatumOd'];
        $this ->datumDoStr = $_POST['DatumDo'];
        $this ->operace = $_POST['operace'];
        $this ->delka = $_POST['delka'];

        // zaskrtavatka
        $this ->pouzitPocatek = $_POST['pocatekkonec']; // zaskrtavatko pouzit pocatek a konec smeny
        $this ->pevneJadro = $_POST['pevnejadro']; // zaskrtavatko pouzit pevne jadro
        $this ->automactikyOpakovat = $_POST['autoopakovat'];
        $this ->vlozitPrestavky = RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_VKLADAT_PRESTAVKY)  ||  $_POST['vlozitprestavky'];

        if (RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)) {
            $this ->rozdeleni = $_POST['rozdeleni_popis'];
        }

        $this ->obdobi ->nactiObdobi($this ->database, 'ADS', $_POST['obdobi']);

        if (!isset($_POST['DatumOd']) || $_POST['DatumOd'] === '') {
            throw new \Exception( \Ron\ronText('form.Missing_value') . ' ' . \Ron\ronText('Datum_od') . '!');
        }

        $this ->datumOd = DateUtils::dateStringToTimestamp($_POST['DatumOd']);

        if (!isset($_POST['DatumDo']) || $_POST['DatumDo'] === '') {
            throw new \Exception( \Ron\ronText('form.Missing_value') . ' ' . \Ron\ronText('Datum_do') . '!');
        }

        $this ->datumDo = DateUtils::dateStringToTimestamp($_POST['DatumDo']);

        // datumy jsou spravne zadane - jeste zkontroluje obdobi atd.
        if ($OK) if ($this ->datumDo < $this ->datumOd) {
            throw new \Exception( \Ron\ronText('Pocatecni_datum_vetsi_nez_koncove') );
        }

        $mindatum = $this ->obdobi ->zacatek;
        $maxdatum = $this ->obdobi ->konec;
        if ($maxdatum > $this ->datumZpracovaniDochazky) $maxdatum = $this ->datumZpracovaniDochazky;
        if ($maxdatum < $mindatum) throw new \Exception(\Ron\ronText('Dochazka_jeste_neni_zpracovana'));

        if ($OK) if ($this ->datumOd < $mindatum || $this ->datumDo > $maxdatum) {
            throw new \Exception( sprintf(\Ron\ronText('Datum_musi_byt_v_rozsahu'), date('j.n.Y', $mindatum), date('j.n.Y', $maxdatum)));
        }

        if (!isset($this ->operace) || $this ->operace == '') {
            throw new \Exception( \Ron\ronText('form.Missing_value') . ' ' . \Ron\ronText('Operace') );
        }

        if (!isset($this ->delka)) {
            throw new \Exception( \Ron\ronText('form.Missing_value') . ' ' . \Ron\ronText('Delka') );
        }

        require_once 'editace/editace_utils.php';
        if (!EditaceUtils::jePravoEditovatOperaci($this ->operace)) {
            throw new NeniPravoException();
        }
    }


    function nactiData($idpp, $den)
    {
        $this ->pp = new PracovniPomer();
        if (!$this ->pp ->nactiPPPodleId($this ->database, $idpp)) {
            throw new \Exception('Zadany pracovni pomer nebyl nalezen!');
        }

        $this ->obdobi = new Obdobi();

        require_once('adsapp/dochazkautils.php');
        $this ->datumZpracovaniDochazky = DochazkaUtils::datumCasZpracovaniDochazky($this ->database);

        // $this -> f_Schvaleni = new SchvaleniDochazky();

        if ($den) {
            // formular je volan pro konkretni datum (napr. z editace dne) - predvyplni zadany den
            $this ->datumOd = $den;
            $this ->datumDo = $den;

            $this ->obdobi ->nactiObdobiProDen($this ->database, 'ADS', $den);
        } else {
            $this ->obdobi ->nactiObdobi($this ->database, 'ADS', RonPortalSession::obdobi());
        }

        if ($this ->datumOd) $this ->datumOdStr = date('j.n.Y', $this ->datumOd);
        if ($this ->datumDo) $this ->datumDoStr = date('j.n.Y', $this ->datumDo);
    }



    function renderVyberOperace($popis)
    {
        require_once('ron/controls.php');
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Kod, Popis FROM OperaceDochazky ' .
            ' WHERE VkladatVEditaciDochazky = 1';

        $prava = Prava::vratPrava('editace');
        if (trim($prava ->priznak3) != '') {
            $sql .= ' AND ' . SQLUtils::getINCondition($this ->database, 'Kod', explode(';', $prava ->priznak3), $params);
        }

        $sql .=
            ' ORDER BY ' . ($popis ? 'Popis' : 'Kod');

        $combo = new ComboBox($popis ? 'operace' : 'operace_kod');
        $combo ->captionIndex = $popis ? 1 : 0;
        $combo ->emptyRow = ' ';
        $combo ->loadData($this ->database, $sql, $this ->operace, $params);
        $select2 = $popis ? 'operace_kod' : 'operace';
        $combo ->htmlParams = " style='width:100%' data-ron-select2='$select2'";
        $combo ->render();
    }


    function renderVyberRozdeleni($popis)
    {
        require_once('ron/controls.php');
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Kod, Popis FROM Rozdeleni ' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)) .
            '   AND (DatumZruseni >= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumDo)) . ' OR DatumZruseni IS NULL) ' .
            '   AND DatumVytvoreni <= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumOd));

        $sql .= ' ORDER BY '.($popis ? 'Popis' : 'Kod');

        $combo = new ComboBox($popis ? 'rozdeleni_popis' : 'rozdeleni_kod');
        $combo ->alwaysShowSelected = true;
        $combo ->captionIndex = $popis ? 1 : 0;
        $combo ->emptyRow = ' ';
        $combo ->loadData($this ->database, $sql, $this ->rozdeleni, $params);
        $select2 = $popis ? 'rozdeleni_kod' : 'rozdeleni_popis';
        $combo ->htmlParams = " style='width:100%' data-ron-select2='$select2'";
        $combo ->render();
    }


    public function render()
    {
        require('editace/view_podlekalendare.php');
    }



    function provedVlozeni()
    {
        if ( RonPortalSettings::getSpousteniADS() ) {
            $params = '9 ' .
                ' -interval:' . date('Y-n-j', $this ->datumOd) . ';' . date('Y-n-j', $this ->datumDo) .
                ' -IDPP:' . $this ->pp ->idPP .
                ' -operace:' . $this ->operace . ' -delka:' . $this ->delka .
                ' -alwaysLog ';

            if ( RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)  &&  $this ->rozdeleni != '' ) {
                $params .= ' -attSubDiv:"' . RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI) . '/' . $this ->rozdeleni . '"';
            }

            if ($this ->pouzitPocatek) $params .= ' -pouzitKonecSmeny';
            if ($this ->pevneJadro) $params .= ' -pevneJadro';
            if ($this ->automactikyOpakovat) $params .= ' -autoOpakovat';
            if ($this ->vlozitPrestavky) $params .= ' -vkladatPrestavky';

            require_once('adsapp/agentmanager.php');
            return AgentManager::executeADS('ADS_Agent_Processing.exe', $params);
        }

        return false;
    }

}
