<?php
/**
 * UTF8
 * stanislav.elbl, 10.11.2015
 */

namespace Ron\RonPortal;

require_once('adsapp/ronportaldbrecord.php');

use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\StringUtils;

/**
 * Pomocny zaznam pro zpracovani zadaneho rozdeleni vysledku
 */
class ZaznamRozdeleniVysledku
{
    /** @var int */
    var $den;

    var $idPP;
    var $jmeno;
    var $prijmeni;
    var $osobniCislo;
    var $titul;

    /** @var string */
    var $rozdeleni;
    /** @var string */
    var $casovaSlozka;

    /** @var string     pouzije se v hromadnem zadavani */
    var $popisRozdeleni;
    var $popisCasoveSlozky;
    var $uzivatelska1;
    var $uzivatelska2;
    var $uzivatelska3;
    var $uzivatelska4;
    var $uzivatelska5;
    var $uzivatelska6;
    var $uzivatelska7;
    var $uzivatelska8;
    var $uzivatelska9;
    var $uzivatelska10;

    var $error_rozdeleni;
    var $error_casovaslozka;

    var $hodiny;
    /** @var string     zadany text */
    var $hodinyStr;
    var $error_hodiny;

    /** @var bool   priznak, ze se ma radek smazat (uzivatel kliknul na ikonu x) */
    var $smazat;
    /** @var bool   priznak se pouzije pri zapisu */
    var $zapsano;
    /** @var bool   priznak, ze se jedna o novy radek */
    var $novy_radek;

    /** @var int    Priznak, ze uz je schvaleno rozdeleni vysledku pro tento den a kod rozdeleni (tabulka SchvaleniRozdeleniDoch) */
    var $rozdeleniSchvaleno;

    var $pravoSchvalovat;

    var $js_id; // pouzije se v javascriptu v hromadnem rozdeleni


    /**
     * @param Database $database
     * @param int $kodTypuRozdeleni
     * @param PracovniPomer $pp
     * @param string[]|null $casoveSlozky   Seznam casovych slozek, ktere se maji nacitat. Zaznamy s jinou casovou slozkou ignoruje. Pokud je null, nacita vse. Kody musi byt malymi pismeny (strtolower)!
     * @param int $datumOd
     * @param int $datumDo
     * @return ZaznamRozdeleniVysledku[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiZaznamyOsoby($database, $kodTypuRozdeleni, $pp, $casoveSlozky, $datumOd, $datumDo = null)
    {
        if (!isset($datumDo)) $datumDo = $datumOd;

        $params = $database ->createParameters();

        $sql =
            'SELECT DenniVysledek_R_E.Den, DenniVysledek_R_E.TypCasoveSlozkyMzdy, DenniVysledek_R_E.Rozdeleni, DenniVysledek_R_E.Hodiny, ' . // 0..3
            '   Rozdeleni.Popis, TypCasoveSlozkyMzdy.Popis, ' . // 4,5
            '   Rozdeleni.Uzivatelska1, Rozdeleni.Uzivatelska2, Rozdeleni.Uzivatelska3, Rozdeleni.Uzivatelska4, Rozdeleni.Uzivatelska5, ' . // 6..10
            '   Rozdeleni.Uzivatelska6, Rozdeleni.Uzivatelska7, Rozdeleni.Uzivatelska8, Rozdeleni.Uzivatelska9, Rozdeleni.Uzivatelska10, ' . // 11..15
            '   SchvaleniRozdeleniDoch.Schvaleno ' . // 16
            ' FROM DenniVysledek_R_E INNER JOIN Rozdeleni' .
            '  ON Rozdeleni.KodTypuRozdeleni = DenniVysledek_R_E.KodTypuRozdeleni ' .
            '  AND Rozdeleni.Kod = DenniVysledek_R_E.Rozdeleni ' .
            ' INNER JOIN TypCasoveSlozkyMzdy ' .
            '  ON TypCasoveSlozkyMzdy.Kod = DenniVysledek_R_E.TypCasoveSlozkyMzdy ' .
            ' LEFT OUTER JOIN SchvaleniRozdeleniDoch ' .
            '  ON SchvaleniRozdeleniDoch.Den = DenniVysledek_R_E.Den ' .
            '  AND SchvaleniRozdeleniDoch.KodTypuRozdeleni = DenniVysledek_R_E.KodTypuRozdeleni ' .
            '  AND SchvaleniRozdeleniDoch.Rozdeleni = DenniVysledek_R_E.Rozdeleni ' .
            ' WHERE DenniVysledek_R_E.KodTypuRozdeleni = ' . $params ->addParameter((int)$kodTypuRozdeleni) .
            ' AND DenniVysledek_R_E.RC = ' . $params->addParameter($pp ->rc) .
            ' AND DenniVysledek_R_E.OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
            ' AND DenniVysledek_R_E.DatumNastupu = ' . $params ->addParameter($database ->timestampToParam($pp ->datumNastupu)) .
            ' AND DenniVysledek_R_E.Den >= ' . $params ->addParameter($database ->timestampToParam($datumOd)) .
            ' AND DenniVysledek_R_E.Den <= ' . $params ->addParameter($database ->timestampToParam($datumDo)) .
            ' ORDER BY DenniVysledek_R_E.Den, DenniVysledek_R_E.Rozdeleni';

        $cursor = $database ->getSqlResult($sql, $params);

        $itemid = 1;
        $result = [];

        while ($row = $cursor ->fetch()) {
            if (!$casoveSlozky  ||  in_array(strtolower($row[1]), $casoveSlozky, true)) {
                $item = new static();

                $item->den = $database->stringToTimestamp($row[0]);
                $item->casovaSlozka = $row[1];
                $item->rozdeleni = $row[2];
                $item->hodiny = $row[3];
                $item->hodinyStr = StringUtils::format('minuty', round($item->hodiny * 60));
                $item->popisRozdeleni = $row[4];
                $item->popisCasoveSlozky = $row[5];

                $item->uzivatelska1 = $row[6];
                $item->uzivatelska2 = $row[7];
                $item->uzivatelska3 = $row[8];
                $item->uzivatelska4 = $row[9];
                $item->uzivatelska5 = $row[10];
                $item->uzivatelska6 = $row[11];
                $item->uzivatelska7 = $row[12];
                $item->uzivatelska8 = $row[13];
                $item->uzivatelska9 = $row[14];
                $item->uzivatelska10 = $row[15];

                $item->rozdeleniSchvaleno = (int)$row[16];

                $item->js_id = 'zaznamrozdeleniDV_' . $itemid;
                $itemid++;

                $result[] = $item;
            }
        }

        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @param int $kodTypuRozdeleni
     * @param string $rozdeleni
     * @param int $datumOd
     * @param int $datumDo
     * @return ZaznamRozdeleniVysledku[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiZaznamyRozdeleni($database, $kodTypuRozdeleni, $rozdeleni, $datumOd, $datumDo = null)
    {
        if (!isset($datumDo)) $datumDo = $datumOd;

        $params = $database ->createParameters();

        $sql =
            'SELECT DenniVysledek_R_E.Den, DenniVysledek_R_E.TypCasoveSlozkyMzdy, DenniVysledek_R_E.Hodiny, ' . // 0..2
            '   DenniVysledek_R_E.OsobniCislo, PracovniPomer.Id, Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul, ' . // 3..7
            '   TypCasoveSlozkyMzdy.Popis' . // 8
            ' FROM DenniVysledek_R_E INNER JOIN PracovniPomer' .
            '  ON PracovniPomer.RC = DenniVysledek_R_E.RC' .
            '  AND PracovniPomer.OsobniCislo = DenniVysledek_R_E.OsobniCislo' .
            '  AND PracovniPomer.DatumNastupu = DenniVysledek_R_E.DatumNastupu' .
            ' INNER JOIN Osoba' .
            '  ON Osoba.RC = DenniVysledek_R_E.RC' .
            ' INNER JOIN TypCasoveSlozkyMzdy ' .
            '  ON TypCasoveSlozkyMzdy.Kod = DenniVysledek_R_E.TypCasoveSlozkyMzdy ' .
            ' WHERE DenniVysledek_R_E.KodTypuRozdeleni = ' . $params ->addParameter((int)$kodTypuRozdeleni) .
            ' AND DenniVysledek_R_E.Rozdeleni = ' . $params ->addParameter($rozdeleni) .
            ' AND DenniVysledek_R_E.Den >= ' . $params ->addParameter($database ->timestampToParam($datumOd)) .
            ' AND DenniVysledek_R_E.Den <= ' . $params ->addParameter($database ->timestampToParam($datumDo)) .
            ' ORDER BY Den, Osoba.Prijmeni, Osoba.Jmeno, PracovniPomer.Id';

        $cursor = $database ->getSqlResult($sql, $params);

        $result = [];

        while ($row = $cursor ->fetch()) {
            $item = new static();

            $item ->den = $database ->stringToTimestamp($row[0]);
            $item ->casovaSlozka = $row[1];
            $item ->hodiny = $row[2];
            $item ->hodinyStr = StringUtils::format('minuty', round($item ->hodiny * 60));
            $item ->osobniCislo = $row[3];
            $item ->idPP = (int)$row[4];
            $item ->jmeno = $row[5];
            $item ->prijmeni = $row[6];
            $item ->titul = $row[7];
            $item ->popisCasoveSlozky = $row[8];
            $item ->rozdeleni = $rozdeleni;

            $result[] = $item;
        }

        $cursor ->close();

        return $result;
    }
}





class RozdeleniDennichVysledkuRecord extends RonPortalDBRecord
{
    /* @var int     kod typu rozdeleni */
    var $typRozdeleni;

    /* @var \Ron\DBRecordField */
    var $fRozdeleni;
    /* @var \Ron\DBRecordField */
    var $fSlozka;
    /* @var \Ron\DBRecordField */
    var $fHodnota;

    var $jeZmena;


    /**
     * @param Database $database
     * @param string $typRozdeleni
     * @param string $rc
     * @param $osobniCislo
     * @param $datumNastupu
     * @param $den
     */
    function __construct($database, $typRozdeleni, $rc, $osobniCislo, $datumNastupu, $den)
    {
        parent::__construct($database, 'DenniVysledek_R_E');

        $this ->logDescription = StringUtils::fromUTF8('Zpracovaná docházka');
        $this ->jeZmena = false;

        $this ->typRozdeleni = (int)$typRozdeleni;

        $this ->addField('RC', '', true, 15, $rc);
        $this ->addField('OsobniCislo', '', true, 15, $osobniCislo);
        $this ->addField('DatumNastupu', 'date', true, 0, $datumNastupu);

        $this ->addField('Den', 'date', true, 0, $den);

        $this ->addField('KodTypuRozdeleni', 'int', true, 0, $this ->typRozdeleni);

        $this ->fRozdeleni = $this ->addField('Rozdeleni', '', true, 30, null);

        $this ->fSlozka = $this ->addField('TypCasoveSlozkyMzdy', '', true, 10, null);

        $this ->fHodnota = $this ->addField('Hodiny', 'float', false, 0, null); // readonly=false

        $params = $this ->database ->createParameters();
        $this ->setWhere(
            'RC = ' . $params->addParameter($rc) .
            ' AND OsobniCislo = ' . $params->addParameter($osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($datumNastupu)) .
            ' AND Den = ' . $params->addParameter($database ->dateToParam($den)) .
            ' AND KodTypuRozdeleni = ' . $params ->addParameter((int)$this ->typRozdeleni),
            $params);
    }


//    /**
//     * @param $kodRozdeleni
//     * @param $casovaSlozka
//     * @return bool
//     * @throws \Exception
//     * @throws \Ron\DB\DBException
//     */
//    function loadRecordRozdeleni($kodRozdeleni, $casovaSlozka)
//    {
//        $params = $this ->database ->createParameters();
//        $filtr =
//            ' Rozdeleni = ' . $params->addParameter($kodRozdeleni) .
//            ' AND TypCasoveSlozkyMzdy = ' . $params->addParameter($casovaSlozka);
//
//        return $this ->loadRecord($filtr, $params);
//    }

}


class RozdeleniVysledkuUtils
{
    const SETTINGS_POUZE_VLASTNI_ROZDELENI = 'dv_pouzevlastnir';

    /**
     * @param Database $database
     * @param string $rozdeleni
     * @param int $den
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public static function overSchvaleni($database, $rozdeleni, $den)
    {
        $params = $database ->createParameters();
        $sql =
            ' SELECT Schvaleno FROM SchvaleniRozdeleniDoch ' .
            ' WHERE SchvaleniRozdeleniDoch.Den = ' . $params ->addParameter($database ->dateToParam($den)) .
            ' AND SchvaleniRozdeleniDoch.KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::DV_TYP_ROZDELENI)) .
            ' AND SchvaleniRozdeleniDoch.Rozdeleni = ' . $params ->addParameter($rozdeleni);

        $cursor = $database ->getSqlResult($sql, $params);

        $result = ($row = $cursor ->fetch())
            ? (int)$row[0]
            : 0;

        $cursor ->close();

        if ($result) throw new \Exception( \Ron\ronText('Rozdeleni_DV_uz_je_schvaleno') );
    }


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $typRozdeleni
     * @param string $hodnota
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $kod
     * @return SqlStatement
     */
    public static function getSQLVyberRozdeleniDV($database, $pp, $typRozdeleni, $hodnota, $datumOd, $datumDo, $kod = false)
    {
        $params = $database ->createParameters();

        if (RonPortalSettings::getIntValue(RonPortalSettings::DV_POVOLIT_PRIRAZENI_ROZDELENI)) {
            // uzivatel si muze sam vybirat rozdeleni - zobrazuji vzdy jen aktualne prirazene zaznamy
            $datumPrirazeniOd = time();
            $datumPrirazeniDo = time();
        } else {
            $datumPrirazeniOd = $datumOd;
            $datumPrirazeniDo = $datumDo;
        }

        if (RonPortalSettings::getIntValue(self::SETTINGS_POUZE_VLASTNI_ROZDELENI) == 3) { // podrizena
            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass ' .
                '  FROM RozdeleniOsob, Rozdeleni, HierarchieRozdeleni' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
                '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addParameter($database->dateToParam($datumOd)) . ' OR Rozdeleni.DatumZruseni IS NULL)' .
                '   AND Rozdeleni.DatumVytvoreni <= ' . $params ->addParameter($database->dateToParam($datumDo)) .

                '   AND Rozdeleni.KodTypuRozdeleni = HierarchieRozdeleni.TypPodrizene' .
                '   AND Rozdeleni.Kod = HierarchieRozdeleni.KodPodrizene' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.TypNadrazene ' .
                '   AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.KodNadrazene ' .
                '   AND HierarchieRozdeleni.KodPodrizene <> HierarchieRozdeleni.KodNadrazene ' . // vynecha vlastni

                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database->dateToParam($datumPrirazeniOd)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database->dateToParam($datumPrirazeniDo)) .
                '   AND RozdeleniOsob.RC = ' . $params->addParameter($pp->rc) .
                '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
                '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($database->dateToParam($pp->datumNastupu));
            // ... OR ( Rozdeleni.Kod = '".db_addslashes($hodnota)."'" ... - nepouzivat - je moc pomale !
        } else if (RonPortalSettings::getIntValue(self::SETTINGS_POUZE_VLASTNI_ROZDELENI) == 2) { // vlastni a podrizena
            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass ' .
                '  FROM RozdeleniOsob, Rozdeleni, HierarchieRozdeleni' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
                '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addParameter($database->dateToParam($datumOd)) . ' OR Rozdeleni.DatumZruseni IS NULL)' .
                '   AND Rozdeleni.DatumVytvoreni <= ' . $params ->addParameter($database->dateToParam($datumDo)) .

                '   AND Rozdeleni.KodTypuRozdeleni = HierarchieRozdeleni.TypPodrizene' .
                '   AND Rozdeleni.Kod = HierarchieRozdeleni.KodPodrizene' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.TypNadrazene ' .
                '   AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.KodNadrazene ' .

                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database->dateToParam($datumPrirazeniOd)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database->dateToParam($datumPrirazeniDo)) .
                '   AND RozdeleniOsob.RC = ' . $params->addParameter($pp->rc) .
                '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
                '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($database->dateToParam($pp->datumNastupu));
            // ... OR ( Rozdeleni.Kod = '".db_addslashes($hodnota)."'" ... - nepouzivat - je moc pomale !
        } else if (RonPortalSettings::getIntValue(self::SETTINGS_POUZE_VLASTNI_ROZDELENI)) { // pouze vlastni
            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass ' .
                '  FROM RozdeleniOsob, Rozdeleni' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
                '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addParameter($database->dateToParam($datumOd)) . ' OR Rozdeleni.DatumZruseni IS NULL)' .
                '   AND Rozdeleni.DatumVytvoreni <= ' . $params ->addParameter($database->dateToParam($datumDo)) .
                '   AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
                '   AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod ' .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database->dateToParam($datumPrirazeniOd)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database->dateToParam($datumPrirazeniDo)) .
                '   AND RozdeleniOsob.RC = ' . $params->addParameter($pp->rc) .
                '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
                '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($database->dateToParam($pp->datumNastupu));
            // ... OR ( Rozdeleni.Kod = '".db_addslashes($hodnota)."'" ... - nepouzivat - je moc pomale !
        } else {
            $sql =
                'SELECT Kod, Popis, cssClass FROM Rozdeleni' .
                ' WHERE KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
                ' AND (' .
                    '( (DatumZruseni >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR DatumZruseni IS NULL)' .
                    '  AND Rozdeleni.DatumVytvoreni <= ' . $params ->addParameter($database->dateToParam($datumDo)) .
                    ')';

            if ($hodnota) {
                // zvolenou hodnotu zobrazi vzdy - pozor pri volbe "pouze vlastni ..." je to moc pomale
                $sql .= ' OR (Rozdeleni.Kod = ' . $params ->addParameter($hodnota) . ')';
            }

            $sql .=
                ')';
        }

        $sql .= " AND (cssClass IS NULL  OR  cssClass <> 'hidden') ";

        $sql .= $kod ? ' ORDER BY Kod ' : ' ORDER BY Popis ';

        return new SqlStatement($sql, $params);
    }


    /**
     * @param Database $database
     * @return string
     */
    public static function getSQLVyberCasSlozkyDV($database)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Kod, Popis FROM TypCasoveSlozkyMzdy'.
            ' WHERE ';

        $found = false;

        $slozky = explode(';', RonPortalSettings::getValue(RonPortalSettings::DV_CASOVE_SLOZKY));
        foreach ( $slozky as $kodSlozky ) if ($kodSlozky) {
            $sql .= $found ? ', ' : ' Kod IN (';
            $found = true;

            $sql .= $params ->addParameter($kodSlozky);
        }

        $sql .=
            ($found ? ')' : '0 = 1') .
            ' ORDER BY Popis';

        return new SqlStatement($sql, $params);
    }

}
