<?php
/**
 * UTF8
 */

namespace Ron\ELearning;

require_once('ron/browser.php');

use Ron\Browser;
use Ron\DB\Database;
use Ron\RonPortal\Prava;


class KurzBrowser extends Browser
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this->title = \Ron\ronText('Kurzy');

        $this->sqlSelectPart =
            'Id, Nazev, PlatnostOd, PlatnostDo, Stav, ' . $database ->castWideMemo('Popis'); // 0..5

        $this->sqlFromPart = 'EKurz';

        $this->defaultSort = 2;
        $this->primaryField = 0;
        $this->rowClickLink = '?menuid=kurzy&action=show';

        $this->code = 'browser_ekurzy';

        $this ->browserUrl = '?menuid=kurzy&';

        // Pole
        $this ->addField( [
            BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN,
            BROWSER_FIELD_SQL_NAME => 'Id', BROWSER_FIELD_SQL_INDEX => 0] );

        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'EKurz.Nazev', BROWSER_FIELD_SQL_INDEX => 1,
            BROWSER_FIELD_HEADER => \Ron\ronText('Nazev'), BROWSER_FIELD_LINK => '?menuid=kurzy&action=show',
            BROWSER_FIELD_ORDERING => 1 ] );

        $this->addField( [
            BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_SELECT,
            BROWSER_FIELD_SQL_NAME => "EKurz.Stav", BROWSER_FIELD_SQL_INDEX => 4,
            BROWSER_FIELD_HEADER => \Ron\ronText('Stav_kurzu'),
            BROWSER_FIELD_OPTIONS => [0 => \Ron\ronText('Kurz_Stav_0'), 1 => \Ron\ronText('Kurz_Stav_1'), 2 => \Ron\ronText('Kurz_Stav_2')],
            BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_WRAP => 1 ] );

        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'EKurz.PlatnostOd', BROWSER_FIELD_SQL_INDEX => 2,
            BROWSER_FIELD_HEADER => \Ron\ronText('Platnost_od'), BROWSER_FIELD_FORMAT => 'finedate',
            BROWSER_FIELD_ORDERING => 1 ] );

        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'EKurz.PlatnostDo', BROWSER_FIELD_SQL_INDEX => 3,
            BROWSER_FIELD_HEADER => \Ron\ronText('Platnost_do'), BROWSER_FIELD_FORMAT => 'finedate',
            BROWSER_FIELD_ORDERING => 1 ] );
    }



    function renderFilter()
    {
        parent::renderFilter();

        // pokud muze editovat, tak zobrazi odkaz "novy zaznam"
        if ( Prava::vratHodnotuPrava('editacekurzu') ) {
            echo "<br/><div class='normaltext'><a href='?menuid=kurzy&formaction=new'>" . \Ron\ronText('New_record') . "</a></div>\n";
        }
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        // zobrazi podrobnosti
        echo "<tr class='$className'>";
        echo '<td colspan="' . ($this->columnCount) . '"><div class="browserpoznamka">';
        echo $values[5];
        echo '</div></td>';
        echo '</tr>';
    }

}

