<?php

/**
 * UTF8
 */

namespace Ron\Jidelna;

use Ron\DateUtils;
use Ron\RonPortal\Kalendar;
use Ron\RonPortal\RonPortalBrowser;
use Ron\StringUtils;

require_once('ron/sqlutils.php');
require_once('adsapp/ronportalbrowser.php');


class DenniPohybyBrowser extends RonPortalBrowser
{

    var $soucet;      // pole souctu jednotlivych polozek

    /** @var  array */
    var $data;

    function __construct($database)
    {
        parent::__construct($database, 'dennipohyby');

        $this->title = \Ron\ronText('Denni_pohyby');
        $this->cssClass = 'browser grid';

        $this->addField([ BROWSER_FIELD_HEADER => \Ron\ronText('Date'), BROWSER_FIELD_ALIGN => 'center' ]);
        $this->addField([ BROWSER_FIELD_HEADER => \Ron\ronText('Pohyby'), BROWSER_FIELD_ALIGN => 'right' ]);
        $this->addField([ BROWSER_FIELD_HEADER => \Ron\ronText('Dotace'), BROWSER_FIELD_ALIGN => 'right' ]);
        $this->addField([ BROWSER_FIELD_HEADER => '', BROWSER_FIELD_ALIGN => 'right' ]); // priznak "upraveno"

        $this ->filterFormAppCode = 'JIDELNA';
        $this ->filterFormRights = 'prohl_jidelna';
        $this ->filterFormVcetneExternich = true;
    }


    function setFilterForm()
    {
        parent::setFilterForm();
        $this ->filterForm ->nastavAkciGet('dennipohyby');
    }


    /**
     * @return \Ron\DB\Cursor
     * @throws \Ron\DB\DBException
     */
    function nactiPohyby()
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT DatumCasPohybu, Castka, TypPohybu, PopisPohybu FROM PohybNaUctu' .
            ' WHERE RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
            ' AND OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)) .
            ' AND DatumCasPohybu >= ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybraneObdobiOd)) .
            ' AND DatumCasPohybu <= ' . $params ->addParameter($this ->database ->timestampToParam(DateUtils::endOfTheDay($this ->filterForm ->vybraneObdobiDo))) .
            ' ORDER BY DatumCasPohybu';

        return $this ->database ->getSqlResult($sql, $params);
    }


//    function nactiDotace()
//    {
//        $sql =
//            'SELECT DatumCasPohybu, Castka FROM PohybNaUctu' .
//            " WHERE RC = '" . $this ->database ->addSlashes(RonPortalSession::osobaRC()) . "'" .
//            " AND OsobniCislo = '" . $this ->database ->addSlashes(RonPortalSession::osobaOsobniCislo()) . "'" .
//            ' AND DatumNastupu = ' . $this ->database ->dateToString(RonPortalSession::osobaNastup()) .
//            ' AND DatumCasPohybu >= ' . $this ->database ->timestampToString(RonPortalSession::obdobi()) .
//            ' AND DatumCasPohybu <= ' . $this ->database ->timestampToString(DateUtils::endOfTheDay(RonPortalSession::konecObdobi())) .
//            ' AND (PohybNaUctu.TypPohybu = 11)' .
//            ' ORDER BY DatumCasPohybu';
//
//        return $this ->database ->getSqlResult($sql);
//    }


    /**
     * @return \Ron\DB\Cursor
     * @throws \Ron\DB\DBException
     */
    function nactiNaroky()
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Den, narok_PocetDotaci, narok_PocetDotaciUpraveny FROM DenniDotace' .
            ' WHERE RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
            ' AND OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)) .
            ' AND Den >= ' . $params ->addParameter($this ->database ->timestampToParam($this ->filterForm ->vybraneObdobiOd)) .
            ' AND Den <= ' . $params ->addParameter($this ->database ->timestampToParam($this ->filterForm ->vybraneObdobiDo)) .
            ' ORDER BY Den';

        return $this ->database ->getSqlResult($sql, $params);
    }


    /**
     * @param string $sqlFilter
     * @throws \Exception
     */
    function load($sqlFilter = '')
    {
        $this ->setFilterForm();
        $this ->filterForm ->loadData();

        $pohyby = $this->nactiPohyby();
        $naroky = $this->nactiNaroky();

        $this->soucet = [];
        $datum = DateUtils::startOfTheDay($this ->filterForm ->vybraneObdobiOd);
        $posledniDatum = DateUtils::endOfTheDay($this ->filterForm ->vybraneObdobiDo);

        $this ->data = [];

        $pohyb = $pohyby ->fetch();
        $narok = $naroky ->fetch();

        // Pro kazdy den vytvori radek
        while ($datum <= $posledniDatum) {
            $dalsiDatum = DateUtils::startOfTheDay($datum + 86400 + 3600); // takhle je to kvuli prechodu letni/zimni cas

            // Secte vsechny pohyby k aktualnimu dni
            $soucetPohybu = 0;
            $popisPohybu = '';
            $soucetDotaci = 0;
            while ($pohyb  &&  $this ->database ->stringToTimestamp($pohyb[0]) < $dalsiDatum) {
                if ($pohyb[2] == 1  ||  $pohyb[2] == 2  ||  $pohyb[2] == 3) {
                    // odber zbozi, objednavka, odber bez objednavky
                    $soucetPohybu += $pohyb[1];
                    $popisPohybu .= ($popisPohybu ? "\n" : '') . $pohyb[3];
                } else if ($pohyb[2] == 11) {
                    // dotace
                    $soucetDotaci += $pohyb[1];
                }
                // TODO: doplnit vklady a vybery?

                $pohyb = $pohyby ->fetch();
            }

            $upravenyNarok = false;
            while ($narok  &&  $this ->database ->stringToTimestamp($narok[0]) < $dalsiDatum) {
                if ((int)$narok[1] != (int)$narok[2]) $upravenyNarok = true;
                $narok = $naroky ->fetch();
            }

            $this ->data[] = $this->vytvorRadek($datum, $soucetPohybu, $popisPohybu, $soucetDotaci, $upravenyNarok, false);

            $datum = DateUtils::startOfTheDay($datum + 86400 + 3600); // takhle je to kvuli prechodu letni/zimni cas
        }
    }


    function vytvorRadek($datum, $soucetPohybu, $popisPohybu, $soucetDotaci, $upravenyNarok, $jeToSoucet)
    {
        $browserRow = [];
        if (!$jeToSoucet) {
            // V nulte polozce radku ulozi cislo dne - 0 = nedele, 1 = pondeli, ...
            $browserRow[] = date('w', $datum);
            // Vytvori hodnotu v prvnim sloupci - datum
            $browserRow[] =
                '<span class="zkratkadne">' . \Ron\ronText('Day_short_' . date('w', $datum)) . '</span>' .
                date('j.n.Y', $datum);
        }

        if ($jeToSoucet || $soucetPohybu) $browserRow[] = '<span title="' . StringUtils::htmlTextConvert($popisPohybu) . '">' . StringUtils::format('curr', $soucetPohybu) . '</span>';
        else $browserRow[] = '';

        if ($jeToSoucet || $soucetDotaci) $browserRow[] = StringUtils::format('curr', $soucetDotaci);
        else $browserRow[] = '';

        if (!$jeToSoucet) {
            $this->soucet[0] += $soucetPohybu;
            $this->soucet[1] += $soucetDotaci;
        }

        if ($upravenyNarok) $browserRow[] = \Ron\ronText('Upraveny_narok');

        return $browserRow;
    }



    function renderDenniPohyby()
    {
        require_once('dochazka/kalendar.php');

        foreach ($this ->data as $index => $row) {
            // Podle toho jestli je to vikend nebo ne zvoli barvu
            $cssClass = Kalendar::getStyleForDay($row[0]);
            if ($index == count($this ->data) - 1) $cssClass .= ' border-bottom';
            if ($cssClass) echo "<tr class='$cssClass'>";
            else echo '<tr>';

            // Vypise obsah radku
            // datum
            echo "<td class='browser_rowheader'>{$row[1]}</td>";

            // hodnoty
            echo "<td class='align-right'>{$row[2]}</td>";
            echo "<td class='align-right'>{$row[3]}</td>";

            // oznaceni upravenych naroku
            echo "<td>{$row[4]}</td>";

            echo "</tr>";
        }
    }


    function renderFooter()
    {
        echo "<tr class='browserfooter'>";
        echo '<td>' . \Ron\ronText('Celkem') . '</td>';

        foreach ($this->soucet as $index => $value) {
            echo "<td class='align-right'>" . StringUtils::format('curr', $value) . '</td>';
        }

        echo '<td></td>'; // sloupec pro oznaceni upravenych naroku
        echo '</tr>';

        echo "<tr class='browserfooter'>";
        echo '<td>' . \Ron\ronText('Rozdil') . '</td>';
        echo "<td colspan='2' class='align-right'>" . StringUtils::format('curr', $this->soucet[0] + $this->soucet[1]) . '</td>';
        echo '<td></td>'; // sloupec pro oznaceni upravenych naroku
        echo '</tr>';
    }


    function render()
    {
        $this->renderHeader();

        $this->renderTableStart();

        echo "<tbody>\n";
        $this ->renderDenniPohyby();
        $this ->renderFooter();
        echo "</tbody>\n";

        $this ->renderTableEnd();
    }

}
