<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 13.09.2019
 */

namespace Ron\RonPortal\Jidelna;


use Exception;
use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\DateInput;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalBrowser;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\StringUtils;
use function Ron\ronText;


require_once 'adsapp/ronportalbrowser.php';


/**
 * Class SeznamObjednavekJidlaFilter
 * @package Ron\RonPortal\Jidelna
 */
class SeznamObjednavekJidlaFilter extends RonPortalFilterForm
{

    /**
     * SeznamObjednavekJidlaFilter constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravoPrevedeni
     * @param PravaNaOsoby $pravoZrusit
     */
    public function __construct($database, $pravoPrevedeni, $pravoZrusit)
    {
        parent::__construct($database, 'jidelnaobjednavky');

        $this->f_classname = 'filtercontainer leftcaptions';
        $this->f_formaction = '?menuid=jidelnaobjednavky';

        Prava::nastavObdobiProPrava(time(),time());
        $this->nastavDatumOdDo(true, false);
        $this->nastavOsoby(true, [$pravoPrevedeni, $pravoZrusit]);
        $this->povolitVsechyOsoby = true;
//        $this->nastavRozdeleni(true, RonPortalSettings::getIntValue(RonPortalSettings::JIDELNA_TYP_ROZDELENI), 0, '', true);
//        $this->nastavRozdeleni(true, 8);
//        $this->vcetnePodrizenychRozdeleni = true;
    }


    public function renderDobaZobrazenychObjednavek()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption font-weight-bold">';
        echo ronText('Jidelna_doba_zobrazeni_objednavek');
        echo '</span>';
        echo '<span class="filtercaption font-weight-bold">';
        echo ronText('Jidelna_dnesni_den');
        echo '</span>';
        echo '</div>';
    }
    
    /**
     * @throws \Ron\DB\DBException
     */
    public function renderPouzeTypJidla()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenTypJidla', \Ron\ronText('Jidelna_pouze_typ_jidla'). '&nbsp;&nbsp;', $_SESSION['jenTypJidla'],
            'onClick="submit()"');
        echo '</span>';

        $sql =
            'SELECT Distinct TypDennihoJidla.Kod, TypDennihoJidla.Popis' .
             ' FROM TypDennihoJidla' .
             ' WHERE TypDennihoJidla.Aktivni = 1' .
             ' ORDER BY TypDennihoJidla.Popis';

        $combo = new ComboBox('typJidla');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenTypJidla.checked) submit();"';
        $combo->dataTypeComparison = false;
        $combo->showCode = 0;
        $combo->loadData($this->database, $sql, $_SESSION['typJidla']);
        $combo->render();

        echo '</div>';
    }


    function renderDatumOdDo()
    {
        $nadpis = $this->nadpisDatumOdDo ?: \Ron\ronText('Jidelna_doba_zobrazeni_objednavek');

        echo '<DIV class="filterrow">';

            echo '<SPAN class="filtercaption">';
            if ($this->datumOdDoPovinne) {
                echo $nadpis; // .' &nbsp;&nbsp;';
            } else {
                require_once('ron/guiutils.php');
                GuiUtils::renderCheckbox('jendatumoddo', $nadpis, RonPortalSession::getFiltrovatDatum(), '', '', null, false);
            }
            echo '</SPAN>';

            echo '<SPAN style="white-space:nowrap;">';
            DateInput::renderDateSelect('datumod', date('j.n.Y', RonPortalSession::getDatumOd()), $this ->f_autosubmit); // submitonenter

            echo '&nbsp;&nbsp;&nbsp; ' . \Ron\ronText('Do') . ' &nbsp;';
            DateInput::renderDateSelect('datumdo', date('j.n.Y', RonPortalSession::getDatumDo()), $this ->f_autosubmit); // submitonenter
            echo '</SPAN>';

        echo "</DIV>\n";
    }


    function loadTypJidla()
    {
        if (isset($_POST['adsfilterform'])){
            $_SESSION['jenTypJidla'] = isset($_POST['jenTypJidla']);
            $_SESSION['typJidla'] = $_POST['typJidla'];
        }
    }


    function loadData()
    {
        $this->loadTypJidla();
        parent::loadData();
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderFilters()
    {
        if (!RonPortalSession::getFiltrovatDatum()){
            $this->renderDobaZobrazenychObjednavek();
        }

        parent::renderFilters();
        $this->renderPouzeTypJidla();
    }
}


/**
 * Class DetailPolozkyJidelnicku
 * @package Ron\RonPortal\Jidelna
 */
class DetailPolozkyJidelnicku extends BrowserField
{
    /** @var PravaNaOsoby $pravo_prevest */
    public $pravo_prevest;

    /** @var PravaNaOsoby $pravo_zrusit */
    public $pravo_zrusit;


    /**
     * DetailPolozkyJidelnicku constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravo_prevest
     * @param PravaNaOsoby $pravo_zrusit
     * @param null $options
     */
    public function __construct($database, $pravo_prevest, $pravo_zrusit, $options = null)
    {
        parent::__construct($database, $options);
        $this->pravo_prevest = $pravo_prevest;
        $this->pravo_zrusit = $pravo_zrusit;
        $this->header = \Ron\ronText('Detail');
        $this->code = 'col-popis';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = "";

        if ($this->pravo_prevest->lzeZobrazitPP($rowData[19]) || $this->pravo_zrusit->lzeZobrazitPP($rowData[19])) {
            $result .= "<a class='detail-objednavky' href='?menuid=jidelnaobjednavky&action=detailobjednavkyjidla&id=$rowData[0]'>";
            $result .= StringUtils::htmlTextConvert($rowData[16]);
            $result .= "</a>";
        } else {
            $result .= StringUtils::htmlTextConvert($rowData[16]);
        }

        return $result;
    }
}


/**
 * Class OdstranitPolozkuJidelnicku
 * @package Ron\RonPortal\Jidelna
 */
class OdstranitPolozkuJidelnicku extends BrowserField
{
    /** @var PravaNaOsoby $pravoZruseni */
    public $pravoZruseni;


    /**
     * DetailPolozkyJidelnicku constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravoZruseni
     */
    public function __construct($database, $pravoZruseni)
    {
        parent::__construct($database);
        $this->pravoZruseni = $pravoZruseni;
        $this->code = 'col-zrusit';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = "";

        //Pokud ma nastaveno datum vydani pak nezobrazit odkaz
        if (!$rowData[8]) {
            if ($this->pravoZruseni->lzeZobrazitRC($rowData[2])) {
                $confirmText = sprintf(ronText('Jidelna_zrusit_objednavku_potvrzeni'), $rowData[15]);
                $result .= "<span class='rectangle confirmQuestion js zrusit-polozku-jidelnicku'  
                        data-ron-confirm-text='$confirmText' 
                        data-ron-href='?menuid=jidelnaobjednavky&action=objednavkuodstranitodkazem&osoba=$rowData[21]&id=$rowData[1]' 
                        data-ron-working-info='" . ronText('Jidelna_odstranuji_objednavku') . "'>" . ronText('jidelna_objednavku_odstranit') . "</span>";

            }
        } else {
            $result .= ronText('Jidelna_objednavka_vydana');
        }
        return $result;
    }
}


/**
 * Class PrevestPolozkuJidelnicku
 * @package Ron\RonPortal\Jidelna
 */
class PrevestPolozkuJidelnicku extends BrowserField
{
    /** @var PravaNaOsoby $pravo */
    public $pravoPrevedeni;


    /**
     * PrevestPolozkuJidelnicku constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravoPrevedeni
     */
    public function __construct($database, $pravoPrevedeni)
    {
        parent::__construct($database);
        $this->pravoPrevedeni = $pravoPrevedeni;
        $this->code = 'col-prevest';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $ajaxSchvalit = "";

        //Pokud ma nastaveno datum vydani pak nezobrazit odkaz
        if (!$rowData[8]) {
            if ($this->pravoPrevedeni->lzeZobrazitRC($rowData[2])) {
                //Momentalne nelze pripojit poznamku prevedeni (neni vyreseno datove)
                $ajaxSchvalit =
                    "<span 
                        class='objednavku-prevest js' 
                        data-ron-akce='prevestobjednavku'
                        data-ron-idpp='$rowData[21]'
                        data-ron-id-polozky-jidelnicku='$rowData[1]'
                        data-ron-jidelna='" . StringUtils::htmlTextConvert($rowData[22]) . "'
                        data-ron-jidelna-popis='" . StringUtils::htmlTextConvert($rowData[18]) . "'
                        data-ron-jmeno='" . StringUtils::htmlTextConvert($rowData[15]) . "'
                        data-ron-popis-objednavky='" . StringUtils::htmlTextConvert($rowData[16]) . "'
                        data-ron-cislo-objednavky='$rowData[0]'>" . ronText('Jidelna_objednavku_prevest') .
                    "</span>";
            }
        } else {
            $ajaxSchvalit .= ronText('Jidelna_objednavka_vydana');
        }
        return $ajaxSchvalit;
    }
}


/**
 * Class SeznamObjednavekJidlaBrowser
 * @package Ron\RonPortal\Jidelna
 */
class SeznamObjednavekJidlaBrowser extends RonPortalBrowser
{

    /** @var PravaNaOsoby $pravoPrevedeni */
    public $pravoPrevedeni;

    /** @var PravaNaOsoby $pravoZruseni */
    public $pravoZruseni;

    /** @var SeznamObjednavekJidlaFilter */
    public $filterForm;


    /**
     * SeznamObjednavekJidlaBrowser constructor.
     * @param Database $database
     * @param $pravoPrevedeni
     * @param $pravoZruseni
     * @throws Exception
     */
    public function __construct($database, $pravoPrevedeni, $pravoZruseni)
    {
        parent::__construct($database, 'jidelnaobjednavky');

        $this->title = ronText('Jidelna_seznam_objednavek');
        $this->code = 'browserjidelnaobjednavky';
        $this->tableId = 'browserjidelnaobjednavky';
        $this->browserUrl = '?menuid=jidelnaobjednavky';

        $this->pravoPrevedeni = $pravoPrevedeni;
        $this->pravoZruseni = $pravoZruseni;

        $this->sqlSelectPart =
            "OJ.ID, OJ.IDPolozkyJidelnicku, OJ.RC, OJ.OsobniCislo, OJ.DatumNastupu, OJ.PoradiVydani, OJ.DatumCasObjednani, " .               //0..6
            "OJ.DatumCasZruseni, OJ.DatumCasVydani, OJ.IDPuvodnihoOdberu, OJ.uzivatel, OJ.DatumCasVlozeniNaBurzu, " .                  //7..11
            "OJ.StavVlozeniNaBurzu, OJ.IDobjednavkyZBurzy, OJ.StavPrevzetiZBurzy, " .                        //12..14
            "COALESCE (Osoba.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} COALESCE (Osoba.Jmeno, '') AS Stravnik, PJ.popisNabidky, PJ.ID, Rozdeleni.Popis, " .       //15..18
            "TypDennihoJidla.Popis, PJ.Den, PracovniPomer.Id, PJ.Jidelna";        //19..22

        $this->sqlFromPart =
            'ObjednaneJidlo AS OJ' .
            ' JOIN PracovniPomer ON (PracovniPomer.RC = OJ.RC) AND (PracovniPomer.OsobniCislo = OJ.OsobniCislo) AND (PracovniPomer.DatumNastupu = OJ.DatumNastupu)' .
            ' JOIN Osoba ON (Osoba.RC = PracovniPomer.RC)' .
            ' JOIN PolozkaJidelnicku PJ ON (PJ.ID = OJ.IDPolozkyJidelnicku)' .
            ' JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = 8 AND Rozdeleni.Kod = PJ.jidelna)' .
            ' JOIN TypDennihoJidla ON (TypDennihoJidla.Kod = PJ.typDennihoJidla)';

        $this->sqlWherePart =
            'OJ.DatumCasZruseni IS NULL';

        $this->filterForm = new SeznamObjednavekJidlaFilter($this->database, $this->pravoPrevedeni, $this->pravoZruseni);
        $this->filterForm->loadData();

        $this->addField()
            ->setSqlFieldName('Stravnik', 15) ->setOrdering(1) ->setHeaderText('Osoba') ->setClassName('jmeno-stravnika')
            ->setCode('col-osoba') ->setVisible($this ->filterForm ->vsechnyOsoby);

        $this->addField()
            ->setSqlFieldName('OJ.ID', 0) ->setOrdering(1) ->setHeaderText('Jidelna_ID_polozky')
            ->setCode('col-id');

        $this->addField()
            ->setSqlFieldName('PJ.Den', 20) ->setHeaderText('Den') ->setOrdering(1) ->setFormat('finedate') ->setAlign('center')
            ->setCode('col-datum');

        $this->defaultSort = -count($this->fields);

        $this->addField()
            ->setSqlFieldName('TypDennihoJidla.Popis', 19) ->setOrdering(1) ->setAlign('center') ->setHeaderText('Jidelna_typ_jidla')
            ->setCode('col-typjidla');

        $this->addField()
            ->setSqlFieldName('Rozdeleni.Popis', 18) ->setOrdering(1) ->setHeaderText('Jidelna')
            ->setCode('col-jidelna');

        $this->addField(new DetailPolozkyJidelnicku($database, $this->pravoPrevedeni, $this->pravoZruseni));

        $this->addField(new OdstranitPolozkuJidelnicku($database, $this->pravoZruseni));

        $this->addField(new PrevestPolozkuJidelnicku($database, $this->pravoPrevedeni));
    }


    function setFilterForm()
    {
        // nedela nic - filterform uz byl nastaven v konstruktoru
    }


    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result = "";

        if (RonPortalSession::getFiltrovatDatum()) {

            $result .= ' PJ.Den >= ' . $params->addParameter($this->database->timestampToParam(RonPortalSession::getDatumOd())) .
                ' AND PJ.Den <= ' . $params->addParameter($this->database->timestampToParam(RonPortalSession::getDatumDo()));
        } else {
            //Pokud se nebude filtrovat od-do, pak se zobrazuji polozky k dnesku
            $result .= ' PJ.Den >= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) .
                ' AND PJ.Den <= ' . $params->addParameter($this->database->timestampToParam(DateUtils::incDay(DateUtils::today())));
        }

        if (!RonPortalSession::vsechnyOsoby()) {
            if ($result) {
                $result .= ' AND ';
            }
            $result .= " OJ.RC = " . $params->addParameter(RonPortalSession::osobaRC());
        } else {
            if (RonPortalSession::rozdeleni() != '') {
                if (RonPortalSession::rozdeleniVcetnePodrizenych()) {
                    $result .= ' AND (EXISTS (SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
                        '   WHERE RozdeleniOsob.RC = Osoba.RC ' .
                        '   AND HierarchieRozdeleni.TypNadrazene = ' . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                        "   AND HierarchieRozdeleni.KodNadrazene = " . $params->addParameter(RonPortalSession::rozdeleni()) .
                        '   AND HierarchieRozdeleni.TypPodrizene = RozdeleniOsob.KodTypuRozdeleni ' .
                        '   AND HierarchieRozdeleni.KodPodrizene = RozdeleniOsob.Rozdeleni ' .
                        '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) .
                        '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                        '))';
                } else {
                    $result .= ' AND EXISTS(SELECT * FROM RozdeleniOsob ' .
                        '   WHERE RozdeleniOsob.KodTypuRozdeleni = ' . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                        "     AND RozdeleniOsob.Rozdeleni = " . $params->addParameter(RonPortalSession::rozdeleni()) .
                        '     AND RozdeleniOsob.RC = PracovniPomer.RC ' .
                        '     AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                        '     AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                        '     AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) .
                        '     AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                        ')';
                }
            }
        }

        if ($_SESSION['jenTypJidla']){
            if ($result != ''){
                $result .= ' AND ';
            }
            $result .= " PJ.typDennihoJidla = " . $params->addParameter($_SESSION['typJidla']);
        }

        return new SqlStatement($result, $params);
    }
}
