<?php

/**
 * UTF8
 */

namespace Ron\Jidelna;

use Ron\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\RonPortal\RonPortalBrowser;

require_once('adsapp/ronportalbrowser.php');



class FieldPopisNabidkyOdberu extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        // pokud neni zadan text primo u polozky, tak zobrazuje popis z jidelnicku
        return $rowData[4] ?: $rowData[5];
        //return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}



class PuvodniOdberBrowser extends RonPortalBrowser
{

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'puvodniodber');

        $this->code = 'browserpuvodniodber';
        $this->title = \Ron\ronText('Puvodni_odbery');

        $this ->sqlSelectPart =
            'PuvodniOdber.DatumCasOperace, Terminal.Popis, PuvodniOdber.Cena, PuvodniOdber.Mnozstvi,' . // 0..3
            ' PuvodniOdber.Popis, PolozkaJidelnicku.PopisNabidky,' . // 4,5
            ' PuvodniOdber.Karta, PuvodniOdber.Uzivatel, PuvodniOdber.KodZbozi, PuvodniOdber.OdberBezObjednavky,' . // 6..9
            ' PuvodniOdber.DatumCasVlozeniZaznamu, PuvodniOdber.Poznamka'; // 10,11

        // definice dotazu a hlavicky
        $this->sqlFromPart =
            'PuvodniOdber LEFT OUTER JOIN Terminal ON (Terminal.kod = PuvodniOdber.Terminal)' .
            ' LEFT OUTER JOIN PolozkaJidelnicku ON (PolozkaJidelnicku.Id = PuvodniOdber.IdPolozkyJidelnicku)';

        $this->addField([
            BROWSER_FIELD_SQL_NAME => 'PuvodniOdber.DatumCasOperace', BROWSER_FIELD_HEADER => \Ron\ronText('Datum_a_cas'), BROWSER_FIELD_SQL_INDEX => 0,
            BROWSER_FIELD_CODE => 'col-datumcas',
            BROWSER_FIELD_FORMAT => 'datetime', BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING =>1 ]);
        $this->defaultSort = 1;
        $this->primaryField = 0;

        $this->addField([
            BROWSER_FIELD_SQL_NAME =>'Terminal.Popis', BROWSER_FIELD_SQL_INDEX => 1,
            BROWSER_FIELD_CODE => 'col-terminal',
            BROWSER_FIELD_HEADER => \Ron\ronText('Misto_odberu'), BROWSER_FIELD_ORDERING => 1 ]);
        $this->addField([
            BROWSER_FIELD_SQL_NAME => 'PuvodniOdber.Cena', BROWSER_FIELD_HEADER => \Ron\ronText('Cena'), BROWSER_FIELD_SQL_INDEX => 2,
            BROWSER_FIELD_CODE => 'col-cena',
            BROWSER_FIELD_FORMAT => 'curr', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1 ]);
        $this->addField([
            BROWSER_FIELD_SQL_NAME => 'PuvodniOdber.Mnozstvi', BROWSER_FIELD_HEADER => \Ron\ronText('Mnozstvi'), BROWSER_FIELD_SQL_INDEX => 3,
            BROWSER_FIELD_CODE => 'col-mnozstvi',
            BROWSER_FIELD_FORMAT => 'number_2', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1 ]);

        $fieldPopis = new FieldPopisNabidkyOdberu($database, [
            BROWSER_FIELD_SQL_NAME => 'PuvodniOdber.Popis', BROWSER_FIELD_SQL_INDEX => 4,
            BROWSER_FIELD_CODE => 'col-popis',
            BROWSER_FIELD_HEADER => \Ron\ronText('Popis_odberu'), BROWSER_FIELD_ORDERING => 0 ]);
        $this ->addField($fieldPopis);

        $this->addField([
            BROWSER_FIELD_SQL_NAME =>'PuvodniOdber.Karta', BROWSER_FIELD_SQL_INDEX => 6,
            BROWSER_FIELD_CODE => 'col-karta', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_HEADER => \Ron\ronText('Cislo_karty'), BROWSER_FIELD_ORDERING => 1 ]);

        $this->addField([
            BROWSER_FIELD_SQL_NAME =>'PuvodniOdber.Uzivatel', BROWSER_FIELD_SQL_INDEX => 7,
            BROWSER_FIELD_CODE => 'col-uzivatel', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_HEADER => \Ron\ronText('Uzivatel'), BROWSER_FIELD_ORDERING => 1 ]);

        $this->addField([
            BROWSER_FIELD_SQL_NAME =>'PuvodniOdber.KodZbozi', BROWSER_FIELD_SQL_INDEX => 8,
            BROWSER_FIELD_CODE => 'col-kodzbozi', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_HEADER => \Ron\ronText('Kod_zbozi'), BROWSER_FIELD_ORDERING => 1 ]);

        $this->addField([
            BROWSER_FIELD_SQL_NAME =>'PuvodniOdber.OdberBezObjednavky', BROWSER_FIELD_SQL_INDEX => 9,
            BROWSER_FIELD_CODE => 'col-bezobjednavky', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_FORMAT => 'anone', BROWSER_FIELD_HEADER => \Ron\ronText('Kod_zbozi'), BROWSER_FIELD_ORDERING => 1 ]);

        $this->addField([
            BROWSER_FIELD_SQL_NAME => 'PuvodniOdber.DatumCasVlozeniZaznamu', BROWSER_FIELD_SQL_INDEX => 10,
            BROWSER_FIELD_CODE => 'col-casvlozeni', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_HEADER => \Ron\ronText('Cas_vlozeni'),
            BROWSER_FIELD_FORMAT => 'datetime', BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING =>1 ]);

        $this->addField([
            BROWSER_FIELD_SQL_NAME =>'PuvodniOdber.Poznamka', BROWSER_FIELD_SQL_INDEX => 11,
            BROWSER_FIELD_CODE => 'col-poznamka', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_HEADER => \Ron\ronText('Poznamka'), BROWSER_FIELD_ORDERING => 1 ]);

        $this ->filterFormAppCode = 'JIDELNA';
        $this ->filterFormRights = 'prohl_jidelna';
        $this ->filterFormVcetneExternich = true;
    }

    function setFilterForm()
    {
        parent::setFilterForm();
        $this ->filterForm ->nastavAkciGet('puvodniodber');
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        return new SqlStatement(
            'PuvodniOdber.DatumCasOperace >= ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybraneObdobiOd)) .
            ' AND PuvodniOdber.DatumCasOperace <= ' . $params ->addParameter($this ->database ->timestampToParam(DateUtils::endOfTheDay($this ->filterForm ->vybraneObdobiDo))) .
            ' AND PuvodniOdber.RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
            ' AND PuvodniOdber.OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
            ' AND PuvodniOdber.DatumNastupu = ' . $params ->addParameter($this ->database ->timestampToParam($this ->filterForm ->vybranaOsobaDatumNastupu)),
            $params);
    }
}
