<?php

/**
 * UTF8
 */

namespace Ron\Jidelna;

use Ron\DB\SqlStatement;
use Ron\RonPortal\RonPortalBrowser;
use Ron\RonPortal\RonPortalSession;

require_once('adsapp/ronportalbrowser.php');

class UcetStravnikaBrowser extends RonPortalBrowser
{
    /**
     * @param \Ron\DB\Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'ucetstravnika');

        $this->title = \Ron\ronText('Ucet_stravnika');
        $this->code = 'browserucetstravnika';

        $this->sqlSelectPart =
            'Obdobi.Popis, Obdobi.PocatekObdobi,' . // 0,1
            ' UcetStravnikaZaObdobi.Castka_Celkem, UcetStravnikaZaObdobi.Castka_Dotace, UcetStravnikaZaObdobi.Castka_SumaOperaciHotove,' . // 2..4
            ' UcetStravnikaZaObdobi.Castka_AktualniStav, UcetStravnikaZaObdobi.Pocet_NarokNaDotace,' . // 5,6
            ' UcetStravnikaZaObdobi.Pocet_Odebrano, UcetStravnikaZaObdobi.Pocet_Dotovano, UcetStravnikaZaObdobi.Pocet_Nedotovano'; // 7..9

        $this->sqlFromPart = 'UcetStravnikaZaObdobi, Obdobi';
        $this->sqlWherePart =
            'UcetStravnikaZaObdobi.Obdobi = Obdobi.PocatekObdobi' .
            " AND Obdobi.Aplikace = 'JIDELNA' ";

        $this->defaultSort = -2;
        $this->primaryField = 0;

        $this->addField([
            BROWSER_FIELD_SQL_NAME => 'Obdobi.Popis', BROWSER_FIELD_SQL_INDEX => 0,
            BROWSER_FIELD_CODE => 'col-obdobi',
            BROWSER_FIELD_HEADER => \Ron\ronText('Obdobi'), BROWSER_FIELD_ORDERING => 1 ]);

        $this->addField([
            BROWSER_FIELD_TYPE => 'hidden', BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_SQL_NAME => 'Obdobi.PocatekObdobi'] );

        $this->addField([
            BROWSER_FIELD_SQL_NAME => 'UcetStravnikaZaObdobi.Castka_Celkem', BROWSER_FIELD_SQL_INDEX => 2,
            BROWSER_FIELD_CODE => 'col-castkacelkem',
            BROWSER_FIELD_HEADER_TEXT => 'Celkem_za_stravu',
            BROWSER_FIELD_FORMAT => 'curr', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1 ]);

        $this->addField([
            BROWSER_FIELD_SQL_NAME => 'UcetStravnikaZaObdobi.Castka_Dotace', BROWSER_FIELD_SQL_INDEX => 3,
            BROWSER_FIELD_CODE => 'col-castkadotace',
            BROWSER_FIELD_HEADER_TEXT => 'Castka_dotace',
            BROWSER_FIELD_FORMAT => 'curr', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1 ]);

        $this->addField([
            BROWSER_FIELD_SQL_NAME => 'UcetStravnikaZaObdobi.Castka_SumaOperaciHotove', BROWSER_FIELD_SQL_INDEX => 4,
            BROWSER_FIELD_CODE => 'col-sumahotove',
            BROWSER_FIELD_HEADER_TEXT => 'Hotovost',
            BROWSER_FIELD_FORMAT => 'curr', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1 ]);

        $this->addField([
            BROWSER_FIELD_SQL_NAME => 'UcetStravnikaZaObdobi.Castka_AktualniStav', BROWSER_FIELD_SQL_INDEX => 5,
            BROWSER_FIELD_CODE => 'col-aktualnistav',
            BROWSER_FIELD_HEADER_TEXT => 'Aktualni_stav_uctu',
            BROWSER_FIELD_FORMAT => 'curr', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1 ]);

        $this->addField([
            BROWSER_FIELD_SQL_NAME => 'UcetStravnikaZaObdobi.Pocet_NarokNaDotace', BROWSER_FIELD_SQL_INDEX => 6,
            BROWSER_FIELD_CODE => 'col-naroknadotace',
            BROWSER_FIELD_HEADER_TEXT => 'Narok_na_dotace',
            BROWSER_FIELD_FORMAT => 'number_2', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1 ]);

        $this->addField([
            BROWSER_FIELD_SQL_NAME => 'UcetStravnikaZaObdobi.Pocet_Odebrano', BROWSER_FIELD_SQL_INDEX => 7,
            BROWSER_FIELD_CODE => 'col-pocetodebrano',
            BROWSER_FIELD_HEADER_TEXT => 'Pocet_odebrano', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_FORMAT => 'number_2', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1 ]);

        $this->addField([
            BROWSER_FIELD_SQL_NAME => 'UcetStravnikaZaObdobi.Pocet_Dotovano', BROWSER_FIELD_SQL_INDEX => 8,
            BROWSER_FIELD_CODE => 'col-pocetdotovano',
            BROWSER_FIELD_HEADER_TEXT => 'Pocet_dotovano', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_FORMAT => 'number_2', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1 ]);

        $this->addField([
            BROWSER_FIELD_SQL_NAME => 'UcetStravnikaZaObdobi.Pocet_Nedotovano', BROWSER_FIELD_SQL_INDEX => 9,
            BROWSER_FIELD_CODE => 'col-pocetnedotovano',
            BROWSER_FIELD_HEADER_TEXT => 'Pocet_nedotovano', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_FORMAT => 'number_2', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1 ]);

        $this ->filterFormAppCode = null; // vyber obdobi nezobrazuje
        $this ->filterFormRights = 'prohl_jidelna';
        $this ->filterFormVcetneExternich = true;
    }


    function setFilterForm()
    {
        parent::setFilterForm();
        $this ->filterForm ->nastavAkciGet('ucetstravnika');
    }


    function load($sqlFilter = '')
    {
        if (isset($_GET['rok'])) $_SESSION['rok'] = (int)$_GET['rok'];
        if (!isset($_SESSION['rok'])) $_SESSION['rok'] = date('Y');

        parent::load($sqlFilter);
    }


    function renderFilter()
    {
        parent::renderFilter();

        $rok = $_SESSION['rok'];

        echo
            '<DIV class="text_link">' .
            '<a href="?menuid=ucetstravnika&rok=' . ($rok - 1) . '"> << </a>' .
            "&nbsp;&nbsp;&nbsp;$rok&nbsp;&nbsp;&nbsp;" .
            '<a href="?menuid=ucetstravnika&rok=' . ($rok + 1) . '"> >> </a>' .
            '</DIV>';
    }


    function getSQLFilter()
    {
        $rok = $_SESSION['rok'];

        $params = $this ->database ->createParameters();

        return new SqlStatement(
            'UcetStravnikaZaObdobi.Obdobi >= ' . $params ->addParameter($this ->database ->dateToParam(mktime(0, 0, 0, 1, 1, $rok))) .
            ' AND UcetStravnikaZaObdobi.Obdobi <= ' . $params ->addParameter($this ->database ->dateToParam(mktime(0, 0, 0, 12, 31, $rok))) .
            ' AND UcetStravnikaZaObdobi.RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
            ' AND UcetStravnikaZaObdobi.OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
            ' AND UcetStravnikaZaObdobi.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)),
            $params);
    }
}

