<?php
/**
 * UTF8
 * stanislav.elbl, 25.11.2015
 * roman.szablatura, 15.2.2017
 */

namespace Ron\Jidelna;


use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\InternalRedirect;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\RonPortal\FotoOutputObject;
use Ron\RonPortal\Log;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\User;
use Ron\StringUtils;
use Ron\TextManager;
use function Ron\ronText;

class JidelnaController extends HttpController
{
    /** @var Database */
    var $database;


    public static function getDatumJidelnicku()
    {
        return $_SESSION['datumJidelnicku'];
    }

    public static function setDatumJidelnicku($value)
    {
        $_SESSION['datumJidelnicku'] = $value;
    }

    /**
     * JidelnaController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        $this ->enableAccessWithoutLogin = true;
    }


    /**
     * @param $typPrava
     * @param $idPP
     * @throws Exception
     */
    function zkontrolujPrava($typPrava, $idPP)
    {
        $prava = Prava::vratPrava($typPrava);
        if (!$prava->lzeZobrazitPP($idPP)) {
            RonPortalUtils::neniPravoException();
        }
    }


    /**
     * @return StravenkyBrowser|VyuctovaniStravenekBrowser
     * @throws Exception
     */
    function stravenky()
    {
        require_once('jidelna/b_stravenky.php');
        $b = new StravenkyBrowser($this->database);
        $b->load();

        $this->zkontrolujPrava('prohl_jidelna', RonPortalSession::vybranaOsoba());

        Model::addViewObject($b);

        require_once('jidelna/vyuctovanistravenek.php');
        $b = new VyuctovaniStravenekBrowser($this->database);
        $b->load();

        return $b;
    }


    /**
     * @param bool $zobrazitObjednavky Priznak, ze se maji zobrazit pocty jidel a objednavani
     * @return BaseViewObject|string
     * @throws Exception
     */
    function zobrazJidelnicek($zobrazitObjednavky = true)
    {
        // nastaveni data
        if (isset($_GET['datum'])) {
            try {
                $datum = DateUtils::firstDayOfWeek(DateUtils::dateStringToTimestamp($_GET['datum']));
                self::setDatumJidelnicku($datum);
            } catch (Exception $e) {
                // chybne zadane datum ignoruje
            }
        }

        if (!self::getDatumJidelnicku()) {
            self::setDatumJidelnicku(DateUtils::firstDayOfWeek(time()));
        }

        $datumOd = self::getDatumJidelnicku();
        $datumDo = DateUtils::incDay($datumOd, 7);

        require_once('jidelna/jidelnicek.php');
        $filterForm = new JidelnicekFilterForm($this ->database, $zobrazitObjednavky);
        $filterForm ->loadData();
        Model::addData('filterForm', $filterForm);

        require_once('jidelna/cenovakategorie.php');
        $cenovaKategorie = new CenovaKategorie($filterForm ->vybranaOsobaRC, $filterForm ->vybranaOsobaOsobniCislo, $filterForm ->vybranaOsobaDatumNastupu);

        require_once('jidelna/jidelnicek.php');
        $jidelnicek = new Jidelnicek($cenovaKategorie);
        $jidelnicek ->load($this ->database, RonPortalSession::vybranaJidelna(), $datumOd, $datumDo);
        Model::addData('jidelnicek', $jidelnicek);

        if ($zobrazitObjednavky) {
            return new BaseViewObject('jidelna/view_jidelnicek.php');
        } else {
            // frame
            Model::addViewObject(new BaseViewObject('jidelna/view_jidelnicek.php'));
            return 'view/appview_frame.php';
        }
    }


    private function redirect($idPolozky) {
        return new Redirect(null, "?menuid=jidelnicek&polozka={$idPolozky}" . ($_POST['x'] ? '&x=' . (int)$_POST['x'] : '') . ($_POST['y'] ? '&y=' . (int)$_POST['y'] : ''));
    }


    /**
     * @return BaseViewObject|Redirect|string
     * @throws Exception
     */
    function objednejJidlo()
    {

        $id = (int)$_POST['polozka'];
        if (!$id) {
            throw new Exception('Chyba: Nelze objednat jidlo, neni zadana polozka jidelnícku!');
        }

        require_once('jidelna/polozkajidelnicku.php');
        $polozkaJidelnicku = new PolozkaJidelnicku();

        $polozkaJidelnicku->nactiPolozkuACelkovyPocetObjednavek($this->database, $id);

        Prava::nastavObdobiProPrava($polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        require_once('adsapp/ppomer.php');
        $ppomer = new PracovniPomer();

        if ( !$ppomer ->nactiPPPodleId($this ->database, $_POST['osoba']) ) {
            throw new Exception('Chyba - osoba nenalezena!');
        }

        $prava = Prava::vratPrava('objednani');
        if (!$ppomer ->vyberNactenyPP($this ->database, $prava)) {
            RonPortalUtils::neniPravoException();
        }

        require_once('jidelna/objednavky.php');
        $manager = new ObjednavkyManager($this ->database);
        $manager ->nactiData($ppomer,
            //$ppomer ->rc, $ppomer ->osobniCislo, $ppomer ->datumNastupu, $ppomer->idPP,
            $polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        try {
            $manager->objednejJidlo($polozkaJidelnicku);

            // OK => redirect na jidelnicek
            //return new Redirect(null, "?menuid=jidelnicek&polozka={$polozkaJidelnicku ->f_id}#{$polozkaJidelnicku ->f_den}");
            return $this ->redirect($polozkaJidelnicku ->f_id);
        } catch (DBException $e) {
            MessageManager::addErrorMessage($e->getMessage(), null, null, $e->getErrorDescription());
        } catch (Exception $e) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba při objednávání jídla!'));
        }

        // pri chybe nedela redirect
        return $this->zobrazJidelnicek();
    }


    /**
     * @return BaseViewObject|Redirect|string
     * @throws Exception
     */
    function zrusObjednavku()
    {
        $id = (int)$_POST['polozka'];
        if (!$id) {
            throw new Exception('Chyba: Nelze zrusit objednavku, neni zadana polozka jidelnicku!');
        }

        require_once('jidelna/polozkajidelnicku.php');
        $polozkaJidelnicku = new PolozkaJidelnicku();

        $polozkaJidelnicku->nactiPolozkuACelkovyPocetObjednavek($this->database, $id);

        Prava::nastavObdobiProPrava($polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        require_once('adsapp/ppomer.php');
        $ppomer = new PracovniPomer();

        if ( !$ppomer ->nactiPPPodleId($this ->database, $_POST['osoba']) ) {
            throw new Exception('Chyba - osoba nenalezena!');
        }

        $prava = Prava::vratPrava('objednani');
        if (!$ppomer ->vyberNactenyPP($this ->database, $prava)) {
            RonPortalUtils::neniPravoException();
        }

        require_once('jidelna/objednavky.php');
        $manager = new ObjednavkyManager($this ->database);
        $manager ->nactiData($ppomer,
            //$ppomer ->rc, $ppomer ->osobniCislo, $ppomer ->datumNastupu, $ppomer->idPP,
            $polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        try {
            $manager ->zrusObjednavku($polozkaJidelnicku);
            //return new Redirect(null, "?menuid=jidelnicek&polozka={$polozkaJidelnicku ->f_id}#{$polozkaJidelnicku ->f_den}");
            return $this ->redirect($polozkaJidelnicku ->f_id);
        } catch (DBException $e) {
            require_once('adsapp/ronportallog.php');
            MessageManager::addErrorMessage($e->getMessage(), null, 'alertMessage', $e->getErrorDescription());
            Log::addRecordToDBLog($this->database, 'ObjednaneJidlo', 'ERROR', StringUtils::fromUTF8('Jídelníček'),
                StringUtils::fromUTF8('Chyba při rušení objednávky!') . "\n{$e ->getMessage()}\n{$e ->getErrorDescription()}");
            return $this->zobrazJidelnicek();
        } catch (Exception $e) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba při objednávání jídla! ') . $e->getMessage(), null, 'alertMessage');
            return $this->zobrazJidelnicek();
        }
    }


    /**
     * @return BaseViewObject|Redirect
     * @throws Exception
     */
    function vlozObjednavkuDoBurzy()
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::BURZA_POVOLENA)) {
            return $this ->zobrazJidelnicek();
        }

        $id = (int)$_POST['polozka'];
        if (!$id) {
            throw new Exception('Chyba: Nelze vlozit objednavku do burzy! Neni zadana polozka jidelnicku.');
        }

        require_once('jidelna/polozkajidelnicku.php');
        $polozkaJidelnicku = new PolozkaJidelnicku();
        $polozkaJidelnicku->nactiPolozkuACelkovyPocetObjednavek($this->database, $id);

        Prava::nastavObdobiProPrava($polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        require_once('adsapp/ppomer.php');
        $ppomer = PracovniPomer::nactiPP($this ->database, (int)$_POST['osoba']);

        $prava = Prava::vratPrava('objednani');
        if (!$ppomer ->vyberNactenyPP($this ->database, $prava)) {
            RonPortalUtils::neniPravoException();
        }

        require_once('jidelna/objednavky.php');
        $manager = new ObjednavkyManager($this ->database);
        $manager ->nactiData($ppomer,
            //$ppomer ->rc, $ppomer ->osobniCislo, $ppomer ->datumNastupu, $ppomer->idPP,
            $polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        $manager ->nactiStavObjednavani($polozkaJidelnicku);
        $polozkaJidelnicku ->f_pocetobjednavekosoby = $manager ->pocetObjednavekOsobyProPolozku($polozkaJidelnicku ->f_id);

        if (!$manager->lzePredatObjednavkuDoBurzy($polozkaJidelnicku)) {
            MessageManager::addErrorMessage(ronText('Objednavka_nebyla_nalezena'), null, 'alertMessage');
            return new Redirect(null, "?menuid=jidelnicek#{$polozkaJidelnicku ->f_den}");
        }

        require_once('jidelna/burza.php');
        try {
            Burza::vlozPolozkuDoBurzy($this->database, $ppomer, $polozkaJidelnicku);
        } catch (Exception $e) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba při vkládání jídla do burzy! ') . $e->getMessage(), null, 'alertMessage');
            return $this->zobrazJidelnicek();
        }

        //return new Redirect(null, "?menuid=jidelnicek&polozka={$polozkaJidelnicku ->f_id}#{$polozkaJidelnicku ->f_den}");
        return $this ->redirect($polozkaJidelnicku ->f_id);
    }


    /**
     * @return BaseViewObject|Redirect|string
     * @throws DBException
     * @throws Exception
     */
    function vyzvedniObjednavkuZBurzy()
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::BURZA_POVOLENA)) {
            return $this ->zobrazJidelnicek();
        }

        $id = (int)$_POST['polozka'];
        if (!$id) {
            throw new Exception('Chyba: Nelze vlozit objednavku do burzy! Neni zadana polozka jidelnicku.');
        }

        require_once('jidelna/polozkajidelnicku.php');
        $polozkaJidelnicku = new PolozkaJidelnicku();
        $polozkaJidelnicku->nactiPolozkuACelkovyPocetObjednavek($this->database, $id);

        Prava::nastavObdobiProPrava($polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        require_once('adsapp/ppomer.php');
        $ppomer = PracovniPomer::nactiPP($this ->database, (int)$_POST['osoba']);

        $prava = Prava::vratPrava('objednani');
        if (!$ppomer ->vyberNactenyPP($this ->database, $prava)) {
            RonPortalUtils::neniPravoException();
        }

        require_once('jidelna/objednavky.php');
        $manager = new ObjednavkyManager($this ->database);
        $manager ->nactiData($ppomer,
            //$ppomer ->rc, $ppomer ->osobniCislo, $ppomer ->datumNastupu, $ppomer->idPP,
            $polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        $manager ->nactiStavObjednavani($polozkaJidelnicku);
        $polozkaJidelnicku ->f_pocetobjednavekosoby = $manager ->pocetObjednavekOsobyProPolozku($polozkaJidelnicku ->f_id);

        if (!$manager->lzeVyzvednoutPolozkuZBurzy($polozkaJidelnicku)) {
            MessageManager::addErrorMessage(ronText('Objednavka_nebyla_nalezena'), null, 'alertMessage');
            return new Redirect(null, "?menuid=jidelnicek#{$polozkaJidelnicku ->f_den}");
        }

        require_once('jidelna/burza.php');
        try {
            Burza::vyzvedniZBurzy($this->database, $ppomer, $polozkaJidelnicku);
        } catch (Exception $e) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba při vyzvednutí jídla z burzy! ') . $e->getMessage(), null, 'alertMessage');
            return $this->zobrazJidelnicek();
        }

        //return new Redirect(null, "?menuid=jidelnicek&polozka={$polozkaJidelnicku ->f_id}#{$polozkaJidelnicku ->f_den}");
        return $this ->redirect($polozkaJidelnicku ->f_id);
    }


    /**
     * @param int $idpolozky
     * @return null|FotoOutputObject
     * @throws DBException
     * @throws Exception
     */
    private function fotoPolozkyJidelnicku($idpolozky)
    {
        $params = $this->database->createParameters();
        $sql =
            "SELECT KodSerialFoto FROM PolozkaJidelnicku WHERE ID = " . $params->addParameter($idpolozky);

        $cursor = $this->database->getSqlResult($sql, $params);

        if ($row = $cursor->fetch()) {
            require_once('adsapp/fotoutils.php');
            return new FotoOutputObject($this->database, $row[0]);
        }
        $cursor->close();

        return null;
    }

    /**
     * @return BaseViewObject|Redirect|FotoOutputObject|string|null
     * @throws DBException
     * @throws Exception
     */
    public function jidelnicek()
    {
        if (isset($_GET['action']) && $_GET['action'] == 'zobrazfoto') {
            return $this->fotoPolozkyJidelnicku((int)$_GET['idpolozky']);
        }

        HtmlHeaders::getInstance() ->setTitle( ronText('RON_Portal') . ' - ' . ronText('Jidelnicek'));

        if (!isset($_GET['frame'])) {

            if (isset($_POST['action'])) {
                switch ($_POST['action']) {
                    case 'objednatjidlo' :
                        return $this->objednejJidlo();

                    case 'zrusitobjednavku' :
                        return $this->zrusObjednavku();

                    case 'objednavkadoburzy' :
                        return $this->vlozObjednavkuDoBurzy();

                    case 'objednavkazburzy' :
                        return $this->vyzvedniObjednavkuZBurzy();

                    default :
                        MessageManager::addErrorMessage("Neznama akce! ({$_POST['action']})");
                }
            }

            return $this->zobrazJidelnicek();
        } else {
            return $this->zobrazJidelnicek(false);
        }
    }

    /**
     * @param string $action
     * @return StravenkyBrowser|VyuctovaniStravenekBrowser|BaseViewObject|InternalRedirect|OutputObject|Redirect|ViewObject|FotoOutputObject|string|null
     * @throws Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance()->loadStrings('jidelna.ini');

        HtmlHeaders::getInstance()->addScript('script/jidelnascripts.js');
        HtmlHeaders::getInstance()->setIcon(RonPortalSettings::getValue('shortcut_icon') ?: 'images/icon_jidelna.png');
        HtmlHeaders::getInstance()->addCss('css/jidelna.css');

        // akce povolene bez prihlaseni
        switch ($action) {
            case 'jidelnicek' :
                return $this->jidelnicek();
        }

        if (!User::userAuthenticated()) {
            return new Redirect(null, '?menuid=logout');
        }

        switch ($action) {
            case 'stravenky' :
                return $this->stravenky();
        }

        throw new Exception('Chyba v metode JidelnaController::handleAction - neznama akce!');
    }
}