<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 13.09.2019
 */

namespace Ron\RonPortal\Jidelna;


use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\Http\HtmlHeaders;
use Ron\Jidelna\DetailObjednavky;
use Ron\Jidelna\PolozkaJidelnicku;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\TranslationsOutputObject;
use Ron\MVC\ViewObject;
use Ron\RonPortal\AgentManager;
use Ron\RonPortal\FotoOutputObject;
use Ron\RonPortal\Log;
use Ron\RonPortal\NeniPravoException;
use Ron\RonPortal\Obdobi;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalUtils;
use Ron\StringUtils;
use Ron\TextManager;
use function Ron\ronText;


require_once('jidelna/polozkajidelnicku.php');
require_once ('adsapp/ronportalcontrols.php');


/**
 * Class JidelnaObjednavkyController
 * @package Ron\RonPortal\Jidelna
 */
class JidelnaObjednavkyController extends JsonController
{

    /** @var Database $database */
    public $database;

    /** @var PravaNaOsoby $pravo_prevedeni_objednavky */
    public $pravo_prevedeni_objednavky;

    /** @var PravaNaOsoby $pravo_zruseni_objednavky */
    public $pravo_zruseni_objednavky;



    /**
     * JidelnaObjednavkyController constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;
//        $this->pravo_zobrazeni_objednavek = Prava::vratPrava('prohl_jidelna');
        //TODO Doplnit prava na zruseni a prevedeni objednavky
        $this->pravo_prevedeni_objednavky = Prava::vratPrava('jidelna_prevedeni_objednavky');
        $this->pravo_zruseni_objednavky = Prava::vratPrava('jidelna_zruseni_objednavky');

        HtmlHeaders::getInstance()->addScript('script/jidelnascripts.js');
        HtmlHeaders::getInstance()->addScript('', '?menuid=jidelnaobjednavky&action=translations&ver=' . HtmlHeaders::getInstance() ->versionId);
        TextManager::getInstance()->loadStrings('jidelna.ini');
        TextManager::getInstance()->loadStrings('person.ini');
    }


    /**
     * @param $id
     * @return BaseViewObject
     * @throws Exception
     */
    public function detailObjednavky($id)
    {
        require_once 'jidelna/objednavky.php';
        $objednavka = DetailObjednavky::nactiDetail($this->database, $id);

        //Zpristupni dle prava tlacitka pro prevod / odstraneni objednavky
        Model::addData('pravo_prevedeni_objednavky', $this->pravo_prevedeni_objednavky);
        Model::addData('pravo_zruseni_objednavky', $this->pravo_zruseni_objednavky);
        Model::addData('objednavka', $objednavka);

        return new BaseViewObject("jidelna/view_detailobjednavky.php");
    }


    /**
     * @param $idpolozky
     * @return FotoOutputObject|null
     * @throws DBException
     */
    private function fotoPolozkyJidelnicku($idpolozky)
    {
        $params = $this->database->createParameters();
        $sql =
            "SELECT KodSerialFoto FROM PolozkaJidelnicku WHERE ID = " . $params->addParameter($idpolozky);

        $cursor = $this->database->getSqlResult($sql, $params);

        if ($row = $cursor->fetch()) {
            require_once('adsapp/fotoutils.php');
            return new FotoOutputObject($this->database, $row[0]);
        }
        $cursor->close();

        return null;
    }

    /**
     * @param int $osoba
     * @param $id
     * @throws DBException
     */
    public function zrusitObjednavkuPresDetail($osoba, $id)
    {
        $this->zrusitObjednavku($osoba, $id);
    }


    /**
     * @param $osoba
     * @param $id
     * @return Redirect
     * @throws DBException
     * @throws Exception
     */
    public function zrusitObjednavku($osoba, $id)
    {
        $polozkaJidelnicku = new PolozkaJidelnicku();
        $polozkaJidelnicku->nactiPolozkuACelkovyPocetObjednavek($this->database, $id);

        //Na pravo 'jidelna_prevedeni'
        Prava::nastavObdobiProPrava( $polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);
        require_once('adsapp/ppomer.php');
        $ppomer = new PracovniPomer();
        if ( !$ppomer ->nactiPPPodleId($this ->database, $osoba) ) {
            throw new Exception('Chyba - osoba nenalezena!');
        }

        if (!$ppomer ->vyberNactenyPP($this ->database, $this->pravo_zruseni_objednavky)) {
            RonPortalUtils::neniPravoException();
        }

        // Nacte seznam objednavek k zadane polozce
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT ObjednaneJidlo.Id, ObjednaneJidlo.PoradiVydani, ' .
            ' PolozkaJidelnicku.Jidelna, PolozkaJidelnicku.Den, PolozkaJidelnicku.TypDennihoJidla, ObjednaneJidlo.DatumCasVydani ' .

            ' FROM ObjednaneJidlo LEFT JOIN PolozkaJidelnicku ON (PolozkaJidelnicku.Id = ObjednaneJidlo.IDPolozkyJidelnicku) ' .
            ' LEFT JOIN PracovniPomer ON (PracovniPomer.RC = ObjednaneJidlo.RC AND PracovniPomer.OsobniCislo = ObjednaneJidlo.OsobniCislo
                AND PracovniPomer.DatumNastupu = ObjednaneJidlo.DatumNastupu) ' .

            ' WHERE IDPolozkyJidelnicku = ' . $params ->addParameter($polozkaJidelnicku ->f_id) .
            '  AND PracovniPomer.ID = ' . $params ->addParameter($ppomer->idPP) .
            '  AND DatumCasZruseni IS NULL ' .
            ' ORDER BY PoradiVydani DESC';

        $cursor = $this->database->getSqlResult($sql, $params);
        $radek_objednavky = $cursor->fetch();
        $cursor ->close();


        //Objednavka je zrusena
        if (!$radek_objednavky) {
            throw new Exception(ronText('Objednavka_nebyla_nalezena'));
        }

        //Objednavka je vydana nebo zrusena
        if ($radek_objednavky[5] != "") {
            throw new Exception(ronText('Jidelna_objednavka_vydana'));
        }

        require_once('adsapp/ronportallog.php');
        $params = $this ->database ->createParameters();
        $sql =
            'UPDATE ObjednaneJidlo SET DatumCasZruseni = ' . $params ->addParameter($this->database->timestampToParam(time())) .
            ' WHERE Id = ' . $params ->addParameter((int)$radek_objednavky[0]);

        if (!$this->database->sql($sql, $params)) {
            throw new DBException($this->database, $sql, 'Chyba pri ruseni objednavky');
        }

        // zaloguje zruseni objednavky
        Log::addRecordToDBLog($this->database, 'ObjednaneJidlo', 'UPDATE', StringUtils::fromUTF8('Jídelníček'),
            StringUtils::fromUTF8('Zrušení objednávky: RC=') . $ppomer ->rc . ', PolozkaJidelnicku=' . $polozkaJidelnicku ->f_id . "\n   " .
            $ppomer->jmenoPrijmeniKratke . "\n   " .
            date('j.n.Y',
                $polozkaJidelnicku->f_den) . ', ' .
                $polozkaJidelnicku->f_typdennihojidla . ' ' .
                $polozkaJidelnicku->f_cislonabidky . ' - ' .
                $polozkaJidelnicku->f_popisnabidky .
            "\n\n" .
            $sql);

        // Musi opravit poradi vydeje ostatnich polozek pro stejny typ jidla,
        // jinak by to delalo problem u vydeje pri pridelovani priloh
        $params = $this->database->createParameters();
        $sql =
            'UPDATE ObjednaneJidlo ' .
            ' SET PoradiVydani = (PoradiVydani - 1) ' .
            " WHERE (RC = " . $params ->addParameter($ppomer ->rc) . ") " .
            "   AND (OsobniCislo = " . $params ->addParameter($ppomer ->osobniCislo) . ") " .
            '   AND (DatumNastupu = ' . $params ->addParameter($this->database->timestampToParam($ppomer ->datumNastupu)) . ') ' .
            '   AND (DatumCasZruseni IS NULL) ' .
            '   AND (DatumCasVydani IS NULL) ' .
            '   AND (PoradiVydani >= ' . (int)$radek_objednavky[1] . ') ' .
            '   AND (EXISTS(SELECT ID ' .
            '        FROM PolozkaJidelnicku ' .
            '        WHERE (PolozkaJidelnicku.ID = ObjednaneJidlo.IDPolozkyJidelnicku) ' .
            '          AND (PolozkaJidelnicku.KodTypuRozdeleni = 8) ' . // typ rozdeleni jidelna
            "          AND (PolozkaJidelnicku.Jidelna = " . $params ->addParameter($radek_objednavky[2]) . ") " .
            '          AND (PolozkaJidelnicku.Den = ' . $params ->addParameter($this->database->timestampToParam($this->database->stringToTimestamp($radek_objednavky[3]))) . ') ' .
            "          AND (PolozkaJidelnicku.TypDennihoJidla = " . $params ->addParameter($radek_objednavky[4]) . ") " .
            '       ))';

        if (!$this->database->sql($sql, $params)) {
            // chyba pri update
            $errorText = $this->database->lastErrorText();

            MessageManager::addDbErrorMessage($this->database, $sql, $errorText);
            Log::addRecordToDBLog($this->database, 'ObjednaneJidlo', 'ERROR', $this,
                StringUtils::fromUTF8('Chyba pri oprave poradi vydeje objednanych jidel!') . "\n$errorText\n\n$sql");
        } else {
            MessageManager::addInfoMessage(ronText("Jidelna_zruseni_objednavky"));
        }

        require_once('adsapp/agentmanager.php');
        AgentManager::vyuctovaniJidelny($ppomer ->idPP);

        //redirect na browser
        return new Redirect(null, "?menuid=jidelnaobjednavky");
    }


    /**
     * @param JsonOutputObject $json
     * @param int   $cisloObjednavky
     * @param int   $vlastnik
     * @param int   $prijemce
     * @throws DBException
     * @throws Exception
     */
    public function prevestObjednavku($json, $cisloObjednavky, $vlastnik, $prijemce)
    {
        Prava::nastavObdobiProPrava(time(), time());

        // nacte detail objednavky
        require_once 'jidelna/objednavky.php';
        $detail = DetailObjednavky::nactiDetail($this ->database, $cisloObjednavky);

        // kontrola prav
        if (!$this->pravo_prevedeni_objednavky ->lzeZobrazitPP($detail ->stravnik)) {
            throw new NeniPravoException();
        }

        if (!$this->pravo_prevedeni_objednavky ->lzeZobrazitPP($prijemce)) {
            throw new NeniPravoException();
        }

        // zkontroluje, jestli je pro stejneho vlastnika - pokud ne, tak uz ji nekdo prevedl
        if ($vlastnik  &&  $detail ->stravnik != $vlastnik) {
            throw new Exception('Objednavka je jiz prevedena na jineho vlastnika!');
        }

        require_once 'adsapp/obdobi.php';
        if (Obdobi::jeObdobiUzavreno($this ->database, 'JIDELNA', $detail ->den)) {
            throw new Exception(ronText('Obdobi_uzavreno'));
        }

        if ($detail ->datumCasVydani) {
            throw new Exception(ronText('Objednavka_jiz_byla_vydana'));
        }

        if ($detail ->datumCasZruseni) {
            throw new Exception(ronText('Objednavka_jiz_byla_zrusena'));
        }

        require_once 'adsapp/ppomer.php';
        $ppomerPrijemce = PracovniPomer::nactiPP($this ->database, $prijemce);


//        $json->addData('vlastnik', $vlastnik);
//        $json->addData('cisloObjednavky',$cisloObjednavky);
//        $json->addData('prijemce',$prijemce);


//        //Na pravo 'objednani'
//        require_once('adsapp/ppomer.php');
//        /** @var PracovniPomer $ppomerVlastnika */
//        $ppomerVlastnika = new PracovniPomer();
//        if ( !$ppomerVlastnika ->nactiPPPodleId($this ->database, $vlastnik) ) {
//            $json->setResult('Jidelna_chyba_vlastnika');
//            return $json;
//        }

//        /** @var PracovniPomer $ppomerPrijemce */
//        $ppomerPrijemce = new PracovniPomer();
//        if ( !$ppomerPrijemce ->nactiPPPodleId($this ->database, $prijemce) ) {
//            $json->setResult('Jidelna_chyba_prijemce');
//            return $json;
//        }

//        if (!$ppomerVlastnika ->vyberNactenyPP($this ->database, $this->pravo_prevedeni_objednavky)) {
//            $json->setResult('Jidelna_neni_pravo_na_vlastnika');
//            return $json;
//        }
//
//        if (!$ppomerPrijemce ->vyberNactenyPP($this ->database, $this->pravo_prevedeni_objednavky)) {
//            $json->setResult('Jidelna_neni_pravo_na_prijemce');
//            return $json;
//        }

        $polozkaJidelnicku = PolozkaJidelnicku::nactiPolozuJidelnicku($this->database, $detail ->idPolozkyJidelnicku);

//        // Nacte seznam objednavek k zadane polozce
//        $params = $this ->database ->createParameters();
//        $sql =
//            'SELECT OJ.ID, OJ.IDPolozkyJidelnicku, OJ.DatumCasZruseni, OJ.DatumCasVydani, OJ.uzivatel, ' .
//            ' PJ.den, PJ.typDennihoJidla, PJ.cisloNabidky, PJ.popisNabidky ' .
//
//            ' FROM PolozkaJidelnicku PJ ' .
//            ' LEFT OUTER JOIN ObjednaneJidlo OJ ON (PJ.ID = OJ.IDPolozkyJidelnicku) ' .
//            " LEFT OUTER JOIN PracovniPomer PP ON (PP.RC = OJ.RC AND PP.OsobniCislo = OJ.OsobniCislo
//                AND PP.DatumNastupu = OJ.DatumNastupu AND PP.DatumNastupu <= " . $params->addParameter($this->database->timestampToParam(time())) . "
//                AND ( PP.DatumUkonceni >= " . $params->addParameter($this->database->timestampToParam(time())) . " OR PP.DatumUkonceni IS NULL)) " .
//
//            ' WHERE OJ.ID = ' . $params ->addParameter($cisloObjednavky) .
//            '  AND PP.ID = ' . $params ->addParameter($ppomerVlastnika->idPP) .
//            '  AND DatumCasZruseni IS NULL ' .
//            ' ORDER BY PoradiVydani DESC';
//
//        $cursor = $this->database->getSqlResult($sql, $params);
//
//        $radekObjednavky = $cursor->fetch();
//        $cursor ->close();
//
//        //Objednavka je zrusena
//        if (!$radekObjednavky) {
//            $json->setResult('Objednavka_nebyla_nalezena');
//            return $json;
//        }
//
//        //Objednavka je vydana nebo zrusena
//        if ($radekObjednavky[3] != "") {
//            $json->setResult('Jidelna_objednavka_vydana');
//            return $json;
//        }

        require_once 'adsapp/ronportaldbrecord.php';

        $this ->database ->beginTransaction();
        try {
            // zrusi puvodni objednavku
            $rec = new RonPortalDBRecord($this ->database, 'ObjednaneJidlo', $this);
            $rec ->addLogText('Prevedeni objednavky na jineho stravnika - zruseni puvodni objednavky');
            $rec ->addField('Id', 'int', true);
            $rec ->addField('IdPolozkyJidelnicku', 'int', true);
            $rec ->addField('RC', '', true);
            $rec ->addField('OsobniCislo', '', true);
            $rec ->addField('DatumNastupu', 'timestamp', true);
            $rec ->addField('DatumCasZruseni', 'timestamp', false, 0, time()); // zrusi objednavku

            $rec ->addField('StavVlozeniNaBurzu', 'int', true);
            $rec ->addField('DatumCasVlozeniNaBurzu', 'timestamp', true);

            $params = $this ->database ->createParameters();
            $rec ->setWhere('Id = ' . $params ->addParameter($detail ->idObjednavky), $params);

            if ($rec ->loadRecord()) {
                $rec ->updateRecord();
            } else {
                throw new Exception('Chyba pri zruseni objednavky puvodniho stravnika: Objednavka nenalezena!');
            }

            // ... a vytvori novou pro prijemce
            require_once('ron/sqlutils.php');
            $newId = SQLUtils::newSerialNumber($this->database, 'ObjednaneJidlo', 'ID');
            $newPoradi = $polozkaJidelnicku->novePoradiVydani($this->database,
                $ppomerPrijemce->rc, $ppomerPrijemce->osobniCislo, $ppomerPrijemce->datumNastupu);

            require_once 'adsapp/ronportaldbrecord.php';
            $rec = new RonPortalDBRecord($this->database, 'ObjednaneJidlo', $this);
            $rec ->addLogText(StringUtils::fromUTF8('Prevedeni objednavky na jineho stravnika - vlozeni nove objednavky)'));
            $rec ->addField('Id', 'int', true, 0, $newId);
            $rec ->addField('IdPolozkyJidelnicku', 'int', true, 0, $detail ->idPolozkyJidelnicku);
            $rec ->addField('RC', '', true, 15, $ppomerPrijemce->rc);
            $rec ->addField('OsobniCislo', '', true, 15, $ppomerPrijemce->osobniCislo);
            $rec ->addField('DatumNastupu', 'timestamp', true, 0, $ppomerPrijemce->datumNastupu);
            $rec ->addField('PoradiVydani', 'int', true, 0, $newPoradi);
            $rec ->addField('DatumCasObjednani', 'timestamp', true, 0, time());
            $rec ->addField('Uzivatel', '', true, 25, RonPortalUtils::getLogUserName());
            $rec->insertRecord();

            $this ->database ->commitTransaction();
        } catch (Exception $e) {
            $this ->database ->rollbackTransaction();
            throw $e;
        }

        require_once('adsapp/agentmanager.php');
        AgentManager::vyuctovaniJidelny($detail ->stravnik);
        AgentManager::vyuctovaniJidelny($prijemce);



        //Na vlastnikovi zrusit objednavku
//        require_once('adsapp/ronportallog.php');
//        $params = $this ->database ->createParameters();
//        $sql =
//            'UPDATE ObjednaneJidlo SET DatumCasZruseni = ' . $params ->addParameter($this->database->timestampToParam(time())) .
//            ' WHERE Id = ' . $params ->addParameter((int)$radekObjednavky[0]);
//
//        if (!$this->database->sql($sql, $params)) {
//            $json->setResult('Chyba pri ruseni objednavky');
//            return $json;
//        }

//        require_once('adsapp/agentmanager.php');
//        AgentManager::vyuctovaniJidelny($vlastnik);
//
//        Log::addRecordToDBLog($this->database, 'ObjednaneJidlo', 'UPDATE', StringUtils::fromUTF8('Jídelníček'),
//         StringUtils::fromUTF8('Zrušení objednávky: RC=') . $ppomerVlastnika->rc . ', PolozkaJidelnicku=' . $radekObjednavky[0] . "\n   " .
//            $ppomerVlastnika->jmenoPrijmeniKratke . "\n   " .
//            date('j.n.Y', ($this->database->stringToTimestamp($radekObjednavky[5]))) . ', ' .
//                $radekObjednavky[6] . ' ' .
//                $radekObjednavky[7] . ' - ' .
//                $radekObjednavky[8] .
//            "\n\n" . $sql);


//        //Vytvorit objednavku u prijemce
//        require_once('ron/sqlutils.php');
//        $newId = SQLUtils::newSerialNumber($this->database, 'ObjednaneJidlo', 'ID');
//        $newPoradi = $polozkaJidelnicku->novePoradiVydani($this->database,
//            $ppomerPrijemce->rc, $ppomerPrijemce->osobniCislo, $ppomerPrijemce->datumNastupu);
//
//        require_once 'adsapp/ronportaldbrecord.php';
//        $rec = new RonPortalDBRecord($this->database, 'ObjednaneJidlo');
//        $rec ->addLogText(StringUtils::fromUTF8('Převedení objednávky (vložení nové objednávky)'));
//        $rec ->addField('Id', 'int', true, 0, $newId);
//        $rec ->addField('IdPolozkyJidelnicku', 'int', true, 0, $radekObjednavky[1]);
//        $rec ->addField('RC', '', true, 15, $ppomerPrijemce->rc);
//        $rec ->addField('OsobniCislo', '', true, 15, $ppomerPrijemce->osobniCislo);
//        $rec ->addField('DatumNastupu', 'timestamp', true, 0, $ppomerPrijemce->datumNastupu);
//        $rec ->addField('PoradiVydani', 'int', true, 0, $newPoradi);
//        $rec ->addField('DatumCasObjednani', 'timestamp', true, 0, time());
//        $rec ->addField('Uzivatel', '', true, 25, RonPortalUtils::getLogUserName());
//
//        try {
//            $rec->insertRecord();
//        } catch (Exception $e) {
//            $json->setResult($e ->getMessage());
//            return $json;
//        }
//
//        AgentManager::vyuctovaniJidelny($prijemce);
//
//        $json = new JsonOutputObject();
//        $json->setResult('OK');
//        return $json;
    }


//    /**
//     * @param $osoba
//     * @param $cisloObjednavky
//     * @return JsonOutputObject
//     * @throws DBException
//     * @throws Exception
//     */
//    public function detailObjednavkyPrevoduNaOsobu($osoba, $cisloObjednavky)
//    {
//        Prava::nastavObdobiProPrava( time(),  time());
//        require_once('adsapp/ppomer.php');
//        $ppomer = new PracovniPomer();
//        if ( !$ppomer ->nactiPPPodleId($this ->database, $osoba) ) {
//            $json = new JsonOutputObject();
//            $json->setResult("Chyba - osoba nenalezena!");
//            return $json;
//        }
//
//        if (!$ppomer ->vyberNactenyPP($this ->database, $this->pravo_prevedeni_objednavky)) {
//            $json = new JsonOutputObject();
//            $json->setResult("Nemate_pravo");
//            return $json;
//        }
//
//        $params = $this->database->createParameters();
//        $sql = "SELECT OJ.ID, OJ.IDPolozkyJidelnicku, " .                       //0-1
//                   "OJ.DatumCasObjednani, OJ.DatumCasZruseni, " .               //2-3
//                   "OJ.DatumCasVydani, OJ.uzivatel, " .                       //4-5
//                   "Rozdeleni.Popis, PolozkaJidelnicku.den, " .                                 //6-7
//                   "PolozkaJidelnicku.typDennihoJidla, PolozkaJidelnicku.cisloNabidky, " .            //8-9
//                   "PolozkaJidelnicku.popisNabidky, PolozkaJidelnicku.popisNabidky_Jazyk2, " .       //10-11
//                   "{fn_CASTWIDEMEMO(PolozkaJidelnicku.rozsirujiciPopis)}, " .                       //12
//                   "PolozkaJidelnicku.kodSerialFoto, PolozkaJidelnicku.cenaJidla_A, " .                       //13-14
//                   "TypDennihoJidla.Popis, O.Jmeno, O.Prijmeni " .                       //15-17
//            "FROM  PolozkaJidelnicku " .
//                   "LEFT OUTER JOIN ObjednaneJidlo OJ ON (PolozkaJidelnicku.ID = OJ.IDPolozkyJidelnicku) " .
//                   "LEFT OUTER JOIN PracovniPomer PP ON (PP.RC = OJ.RC AND PP.OsobniCislo = OJ.OsobniCislo AND PP.DatumNastupu = OJ.DatumNastupu)" .
//                   "LEFT OUTER JOIN Rozdeleni ON (PolozkaJidelnicku.jidelna = Rozdeleni.Kod)" .
//                   "LEFT OUTER JOIN TypDennihoJidla ON (TypDennihoJidla.Kod = PolozkaJidelnicku.typDennihoJidla)" .
//                   "LEFT OUTER JOIN Osoba O ON (O.RC = OJ.RC)";
//
//        //Rozdeleni Jidelna
//        $sql .= " WHERE (OJ.ID = " . $params->addParameter($cisloObjednavky) . ") AND (Rozdeleni.KodTypuRozdeleni = '8')";
//
//        MessageManager::addDebugLog($sql);
//
//        $cursor = $this->database->getSqlResult($sql, $params);
//        $objednavka = [];
//
//        $row = $cursor->fetch();
//        $cursor->close();
//
//        if ($row){
//            $objednavka['DatumCasVydani'] = StringUtils::jsonDate($this->database->stringToTimestamp($row[4]));
//            $objednavka['JmenoOsobyObjednavky'] = StringUtils::jsonText($row[16]);
//            $objednavka['PrijmeniOsobyObjednavky'] = StringUtils::jsonText($row[17]);
//            $objednavka['IDObjednavky'] = StringUtils::jsonText($row[0]);
//            $objednavka['IDPolozkyJidelnicku'] = StringUtils::jsonText($row[1]);
//            $objednavka['PopisObjednavky'] = StringUtils::jsonText($row[10]);
//            $objednavka['TypObjednavky'] = StringUtils::jsonText($row[15]);
//            $objednavka['SQL'] = StringUtils::jsonText($sql);
//
//            $json = new JsonOutputObject($objednavka);
//            $json->setResult("OK");
//            return $json;
//        }
//
//        $json = new JsonOutputObject();
//        $json->setResult("Chyba - objednavka nenactena");
//        return $json;
//    }


    /**
     * @param JsonOutputObject $output
     * @param $jidelna
     * @throws DBException
     * @throws Exception
     */
    function seznamOsob($output, $jidelna)
    {
        $osoby = [];
        $osoba = [];

        if ($jidelna != "") {
            $params = $this->database->createParameters();
            $sql =
                'SELECT PP.ID, Osoba.Prijmeni, Osoba.Jmeno' .
                ' FROM PracovniPomer PP' .
                ' LEFT JOIN RozdeleniOsob RO ON (PP.RC = RO.RC AND PP.OsobniCislo = RO.OsobniCislo AND PP.DatumNastupu = RO.DatumNastupu)' .
                ' LEFT JOIN Osoba ON (PP.RC = Osoba.RC)' .

                ' WHERE (PP.DatumNastupu <= ' . $params->addParameter($this->database->timestampToParam(time())) .
                ' AND (PP.DatumUkonceni >= ' . $params->addParameter($this->database->timestampToParam(time())) . ' OR PP.DatumUkonceni IS NULL))' .
                ' AND RO.KodTypuRozdeleni = 8' .
                ' AND RO.Rozdeleni = ' . $params->addParameter($jidelna) .
                ' ORDER BY Osoba.Prijmeni';

            $cursor = $this->database->getSqlResult($sql, $params);

            while ($row = $cursor->fetch()){
                if ($this->pravo_prevedeni_objednavky->lzeZobrazitPP($row[0])) {
                    $osoba['idpp'] = StringUtils::jsonText($row[0]);
                    $osoba['prijmeni'] = StringUtils::jsonText($row[1]);
                    $osoba['jmeno'] = StringUtils::jsonText($row[2]);
                    $osoba['jmenoPrijmeni'] = StringUtils::jsonText( ($row[1] ?: '') . ' ' . ($row[2] ?: ''));

                    $osoby[] = $osoba;
                }
            }

            $cursor ->close();
        }

        $output ->addData('items', $osoby);
        $output->compress = 'gzip';
    }


    /**
     * @return TranslationsOutputObject
     */
    function translations()
    {
        require_once('ron/utils.php');
        return new TranslationsOutputObject();
    }



    /**
     * @param $action
     * @param JsonOutputObject $output
     * @throws Exception
     */
    function handleJsonAction($action, $output)
    {
        //Nacte osoby dle prava osoby pro danou jidelnu
        if (isset($_GET['nactiosoby'])) {
            $this ->seznamOsob($output, $_GET['jidelna']);
            return;
        }

        switch ($_GET['action']){
            case 'prevodobjednavkynaosobu':
                $this->prevestObjednavku($output, $_POST['cisloObjednavky'], $_POST['osoba'], $_POST['novaOsobaIDPP']);
                break;

            default :
                throw new Exception('Neznama akce!');
        }

    }



    /**
     * @param string $action
     * @return InternalRedirect|OutputObject|Redirect|ViewObject|string|void
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!($this->pravo_prevedeni_objednavky->jsouNejakePrava() || $this->pravo_zruseni_objednavky->jsouNejakePrava())){
            RonPortalUtils::neniPravoException();
        }

        if (isset($_GET['action']) && $_GET['action'] == 'zobrazfoto') {
            return $this->fotoPolozkyJidelnicku((int)$_GET['idpolozky']);
        }

        Prava::nastavObdobiProPrava(time(), time());

        if (isset($_POST['jidelnazrusitobjednavku']) && $_POST['jidelnazrusitobjednavku'] == 1){
            return $this->zrusitObjednavku($_POST['osoba'], $_POST['id']);
        }


        switch ($_GET['action']){
            case 'seznamosob':
                return parent::handleAction($action);

            case 'detailobjednavkyjidla':
                return $this->detailObjednavky($_GET['id']);

            case 'objednavkuodstranitodkazem':
                return $this->zrusitObjednavku($_GET['osoba'], $_GET['id']);

//            case 'detailobjednavkyprevodunaosobu':
//                return $this->detailObjednavkyPrevoduNaOsobu($_GET['idpp'], $_GET['cisloObjednavky']);

            case 'translations':
                return $this->translations();

            case 'prevodobjednavkynaosobu':
                return parent::handleAction($action);

            default:
                require_once 'jidelna/b_jidelnaobjednavky.php';
                $v = new BaseViewObject('jidelna/view_prevedeniobjednavky.php');
                $b = new SeznamObjednavekJidlaBrowser($this->database, $this->pravo_prevedeni_objednavky, $this->pravo_zruseni_objednavky);
                $b->load();

                Model::addViewObject($b);
                Model::addViewObject($v);
                return 'view/appview.php';
        }
    }
}