<?php
/**
 * UTF8
 * stanislav.elbl, 26.12.2015
 */

namespace Ron\Jidelna;

use Exception;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\GuiUtils;
use Ron\Http\HttpResponse;
use Ron\RonPortal\NastaveniUzivatele;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\DB\Database;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\StringUtils;

require_once('adsapp/ronportalfilter.php');
require_once('jidelna/polozkajidelnicku.php');
require_once('adsapp/usersettings.php');

class JidelnicekFilterForm extends RonPortalFilterForm
{
    public $jidelny = [];

    /** @var array Seznam typu jidel: key = kod typu jidla, value = popis typu jidla */
    private $typyJidel = [];
    /** @var array vybrane typy, jen kody */
    private $vybraneTypy = [];

    function __construct($database, $zobrazitObjednavky)
    {
        parent::__construct($database, 'jidelnicek', true);

        $this ->getAction = $_SERVER['SCRIPT_NAME']; // kvuli odstraneni #..., jinak by nebylo potreba

        if ($zobrazitObjednavky) {
            $prava = Prava::vratPrava('objednani');

            if ($prava->jsouNejakePrava()) {
                $this->nastavOsoby(true, $prava, time(), time(), true); // vcetne externich = true
            }
        } else {
            $this ->skrytTlacitkoNastavit = true;
        }
    }

    /**
     * @throws DBException
     */
    private function loadTypyJidel()
    {
        $sql =
            "SELECT Kod, Popis 
            FROM TypDennihoJidla
            WHERE Aktivni = 1
              AND Kod <> 'ZB'
            ORDER BY Poradi"; //Typ Zbozi se nezobrazuje natvrdo - stejne jako u objednani v jidelne

        $cursor = $this->database->getSqlResult($sql);

        //nutne pro porovnani pri ukladani uz. nastaveni, v poli musi byt kody
        $vsechnyTypy = [];

        while ($row = $cursor->fetch()) {
            if (isset($_GET['jidelnicek_typ_' . $row[0]])) {
                $this->vybraneTypy[] = $row[0];
            }
            $vsechnyTypy[] = $row[0];

            $this->typyJidel[$row[0]] = $row[1];
        }

        //nastaveni se ulozi jen pri odeslani filtru (je nastavena osoba)
        if (isset($_GET['osoba'])) {
            NastaveniUzivatele::setUserSettingsItem($this->database, 'jidelnicek_vybrane_typy', implode(';', $this->vybraneTypy), implode(';', $vsechnyTypy));
        }
        //jinak se vycte z uzivatelskeho nastaveni
        else {
            $nastaveniTypu = NastaveniUzivatele::getUserSettingsItem($this->database, 'jidelnicek_vybrane_typy');
            if ($nastaveniTypu != '') {
                $this->vybraneTypy = explode(';', $nastaveniTypu);
            }
        }
    }

    /**
     * Zjisti, zda ma zamestnanec nastavenou jednu hlavni jidelnu. Pokud ano, nastavi ji jako vybranou do session.
     * Pokud nema zadnou hlavni nebo ma vice hlavnich, nenastavi nic.
     * POZOR! Uz musi byt nastaveni zamestnanec!
     * @throws Exception
     */
    private function nastavHlavniJidelnuZamestnance()
    {
        $datumOd = JidelnaController::getDatumJidelnicku();
        $datumDo = DateUtils::incDay($datumOd, 7);

        $params = $this->database->createParameters();

        $sql =
            'SELECT Rozdeleni.kod, Rozdeleni.popis, RozdeleniOsob.Hlavni, RozdeleniOsob.PlatnostOd ' .
            ' FROM Rozdeleni, RozdeleniOsob ' .
            ' WHERE Rozdeleni.KodTypuRozdeleni = 8 ' .
            '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
            '   AND Rozdeleni.kod = RozdeleniOsob.Rozdeleni ' .
            "   AND RozdeleniOsob.RC = " . $params->addParameter($this ->vybranaOsobaRC) .
            "   AND RozdeleniOsob.OsobniCislo = " . $params->addParameter($this ->vybranaOsobaOsobniCislo) .
            '   AND RozdeleniOsob.DatumNastupu = ' . $params->addParameter($this->database->dateToParam( $this ->vybranaOsobaDatumNastupu )) .
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam( $datumDo )) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam( $datumOd )) . ' OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
            '   AND (Rozdeleni.DatumZruseni >= ' . $params->addParameter($this->database->timestampToParam( $datumOd )) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
            ' ORDER BY RozdeleniOsob.Hlavni DESC, RozdeleniOsob.PlatnostOd DESC';

        $cursor = $this ->database ->getSqlResult($sql, $params);

        $pouzitHlavni = false;
        $hlavniJidelna = null;

        while ($row = $cursor->fetch()) {
            if ((int)$row[2] == 1) {
                if (!$pouzitHlavni) {
                    $pouzitHlavni = true;
                    $hlavniJidelna = $row[0];
                }
                else {
                    $pouzitHlavni = false;
                }
            }
        }

        if ($pouzitHlavni) {
            RonPortalSession::setVybranaJidelna($hlavniJidelna);
        }
    }

    function loadData()
    {
        $this ->jidelny = [];

        parent::loadData();

        //10.7.2017 Po prihlaseni se zjisti, jestli ma zamestnanec nastavenou hlavni jidelnu. Pokud ano, vybere se jako vychozi. Pokud ma vice, nevybere se zadna.
        if (RonPortalSession::vybranaJidelna() == '') {
            $this->nastavHlavniJidelnuZamestnance();
        }

        HttpResponse::setCookieAndSession('jidelna_jidelna', 'vyberjidelny', false, true); // GET

        $datumOd = JidelnaController::getDatumJidelnicku();
        $datumDo = DateUtils::incDay($datumOd, 7);

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Rozdeleni.kod, Rozdeleni.popis ' .
            ' FROM Rozdeleni, RozdeleniOsob ' .
            ' WHERE Rozdeleni.KodTypuRozdeleni = 8 ' .
            '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
            '   AND Rozdeleni.kod = RozdeleniOsob.Rozdeleni ' .
            '   AND RozdeleniOsob.RC = ' . $params->addParameter($this ->vybranaOsobaRC) .
            '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($this ->vybranaOsobaOsobniCislo) .
            '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->vybranaOsobaDatumNastupu) ) .
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($this ->database ->dateToParam($datumDo)) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($this ->database ->dateToParam($datumOd)) . ' OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
            '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addParameter($this ->database ->dateToParam($datumOd)) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
            ' ORDER BY Popis';

        $cursor = $this ->database ->getSqlResult($sql, $params);

        $prvniJidelna = null;
        $jeVybranaJidelna = false;

        while ($row = $cursor ->fetch()) {
            if (!isset($prvniJidelna)) {
                $prvniJidelna = $row[0];
            }

            $this ->jidelny[$row[0]] = $row[1];
            if ($row[0] == RonPortalSession::vybranaJidelna()) {
                $jeVybranaJidelna = true;
            }
        }

        if (!$jeVybranaJidelna  &&  isset($prvniJidelna) ) {
            RonPortalSession::setVybranaJidelna($prvniJidelna);
        }

        $this->loadTypyJidel();
    }


    function renderJidelny()
    {
        echo '<DIV class="filterrow">';
        echo '<SPAN class="filtercaption"></SPAN>';
        echo '<SPAN class="filterformtext">';
        LinksList::renderLinks($this ->jidelny, RonPortalSession::vybranaJidelna(), '?menuid=jidelnicek&vyberjidelny=', '', false);
        echo '</SPAN>';
        echo "</DIV>\n";

    }

    private function renderTypyJidel()
    {
        echo '<div class="filterrow">';
        echo '<SPAN class="filtercaption"></SPAN>';
        echo '<SPAN class="filterformtext">';

        require_once('ron/guiutils.php');

        foreach ($this->typyJidel as $kodTypu => $popisTypu) {
            //pokud je uz. nastaveni prazdne jsou vybrane vsechny typy
            GuiUtils::renderCheckbox('jidelnicek_typ_' . $kodTypu, $popisTypu, empty($this->vybraneTypy) ? true : in_array($kodTypu, $this->vybraneTypy), true);
        }

        echo '</SPAN>';
        echo "</DIV>\n";
    }

    public function renderFilters()
    {
        parent::renderFilters();
        $this ->renderJidelny();

        if (RonPortalSettings::getIntValue('jidelnicek_filtrovat_typy', 0)) {
            $this->renderTypyJidel();
        }
    }
}


class DenJidelnicku
{
    var $datum;

    /** @var PolozkaJidelnicku[] */
    var $polozky = [];

    /**
     * DenJidelnicku constructor.
     * @param $datum
     */
    public function __construct($datum)
    {
        $this->datum = $datum;
    }

}


class Jidelnicek
{
    /** @var  CenovaKategorie */
    var $cenovaKategorie;

    var $datumOd;
    var $datumDo;

    /** @var  DenJidelnicku[] */
    var $data;

    var $jeFotografie;


    /**
     * Jidelnicek constructor.
     * @param CenovaKategorie $cenovaKategorie
     */
    public function __construct($cenovaKategorie)
    {
        $this ->cenovaKategorie = $cenovaKategorie;
        $this ->data = [];
    }


    /**
     * @param Database $database
     * @param PolozkaJidelnicku $polozka
     * @throws DBException
     */
    function nastavZobrazenouCenu($database, $polozka)
    {
        $polozka ->zobrazenaCena = '';

        if ($polozka->f_cenajidlaA) {
            $zobrazenaCena = $this ->cenovaKategorie
                ? $this ->cenovaKategorie->typZobrazeneCenyProTypJidla($polozka->f_typdennihojidla)
                : 1; // neni cenova kategorie (frame) => jen plna cena

            switch ($zobrazenaCena) {
                case 0 :
                    break; // nezobrazovat cenu

                case 1 :
                    $polozka ->zobrazenaCena = StringUtils::format('curr', $polozka->f_cenajidlaA);
                    break; // plna cena

                /** @noinspection PhpMissingBreakStatementInspection */
                case 3 :
                    $polozka ->zobrazenaCena = StringUtils::format('curr', $polozka->f_cenajidlaA) . ' / '; // NE BREAK! plna cena i dotovana
                case 2 :
                    $polozka ->zobrazenaCena .= StringUtils::format('curr', $this ->cenovaKategorie ->vratDotovanouCenuProTypJidla($polozka ->f_typdennihojidla, $polozka-> f_cenajidlaA));
            }
        }
    }

    /**
     * @param Database $database
     * @param string $jidelna
     * @param int $datum
     * @return DenJidelnicku
     * @throws DBException
     */
    private function nactiDenJidelnicku($database, $jidelna, $datum)
    {
        if ($this ->cenovaKategorie) {
            $this->cenovaKategorie->overNacteniVlastnostiKDatu($database, $datum);
        }

        $den = new DenJidelnicku($datum);

        $vybraneTypy = '';
        if (RonPortalSettings::getIntValue('jidelnicek_filtrovat_typy', 0)) {
            $vybraneTypy = NastaveniUzivatele::getUserSettingsItem($database, 'jidelnicek_vybrane_typy');
        }

        $sql = PolozkaJidelnicku::sqlProPolozkyJidelnicku($database, 0, $datum, $jidelna, $vybraneTypy);

        $cursor = $database ->getSqlResult($sql);

        while ($row = $cursor ->fetch()) {
            $polozka = new PolozkaJidelnicku();
            $polozka->nactiPolozkuZRadku($database, $row);

            if ($polozka ->kodFotografie) $this ->jeFotografie = true;

            $polozka->nactiPocetObjednavekCelkem($database);  // celkovy pocet objednavek - nacita se, jen kdyz je zapnuto omezeni

            if ( RonPortalSettings::getIntValue(RonPortalSettings::BURZA_POVOLENA) ) {
                $polozka->nactiInfoZBurzy($database); // toto by se dalo optimalizovat (nacitat hromadne)!
            }

            //$polozka->nactiAlergeny($database);     - alergeny
            //$polozka->nactiHodnoceni($database);
//            if ($this ->cenovaKategorie) {
//                $polozka->nactiStavObjednavani($database, $this->cenovaKategorie);
//            }

            $this ->nastavZobrazenouCenu($database, $polozka);

            $den ->polozky[] = $polozka;
        }


        return $den;
    }


//    /**
//     * Nacte pocty objednavek osoby pro vsechny polozky jidelnicku v obdobi
//     * @param $database
//     */
//    function doplnPoctyObjednavekOsoby($database)
//    {
//        /** @var array $poctyObjednavekOsoby Pocty objednavek jednotlivych polozek jidelnicku - nacitaji se hromadne kvuli rychlosti */
//        $poctyObjednavekOsoby = JidelnaUtils::nactiPoctyObjednavekOsoby($database, $this ->datumOd, $this ->datumDo);
//
//        foreach ($this ->data as $denJidenicku) {
//            foreach ($denJidenicku ->polozky as $polozkaJidelnicku) {
//                $polozkaJidelnicku ->f_pocetobjednavekosoby = isset($poctyObjednavekOsoby[$polozkaJidelnicku ->f_id])
//                    ? $poctyObjednavekOsoby[$polozkaJidelnicku ->f_id]
//                    : 0;
//            }
//        }
//    }

    /**
     * @param $database
     * @throws DBException
     * @throws Exception
     */
    function nastavObjednavani($database)
    {
        if (!$this ->cenovaKategorie) {
            return;
        }

        require_once 'adsapp/ppomer.php';
        $pp = PracovniPomer::nactiPP($database, RonPortalSession::vybranaOsoba());

        require_once 'jidelna/objednavky.php';
        $manager = new ObjednavkyManager($database);
        $manager ->nactiData($pp,
            //RonPortalSession::osobaRC(), RonPortalSession::osobaOsobniCislo(), RonPortalSession::osobaNastup(), RonPortalSession::vybranaOsoba(),
            $this ->datumOd, $this ->datumDo);

        // nejdrive doplni pocty objednavek ...
        foreach ($this ->data as $denJidenicku) {
            foreach ($denJidenicku ->polozky as $polozkaJidelnicku) {
                $polozkaJidelnicku ->f_pocetobjednavekosoby = $manager ->pocetObjednavekOsobyProPolozku($polozkaJidelnicku ->f_id);
            }
        }

        // ... teprve pak nastavi objednavani
        foreach ($this ->data as $denJidenicku) {
            foreach ($denJidenicku ->polozky as $polozkaJidelnicku) {
                $manager ->nactiStavObjednavani($polozkaJidelnicku);
            }
        }
    }


    /**
     * Nacte hodnoceni polozek pro vsechny polozky jidelnicku v obdobi
     * @param Database $database
     * @throws Exception
     */
    private function doplnHodnoceni($database)
    {
        $hodnoceni = PolozkaJidelnicku::nactiHodnoceniPolozek($database, $this ->datumOd, $this ->datumDo);

        foreach ($this ->data as $denJidenicku) {
            foreach ($denJidenicku ->polozky as $polozkaJidelnicku) {
                $hodnoceniPolozky = isset($hodnoceni[$polozkaJidelnicku ->f_id])
                    ? $hodnoceni[$polozkaJidelnicku ->f_id]
                    : null;

                if ($hodnoceniPolozky) {
                    $polozkaJidelnicku ->hodnoceniExistuje = true;
                    $polozkaJidelnicku ->hodnoceni = $hodnoceniPolozky ->hodnoceni;
                    $polozkaJidelnicku ->hodnoceniPoznamka = $hodnoceniPolozky ->poznamka;
                }
            }
        }
    }


    /**
     * @param $database
     * @throws Exception
     */
    private function doplnAlergeny($database)
    {
        $seznamAlergenu = PolozkaJidelnicku::nactiAlergenyPolozek($database, $this ->datumOd, $this ->datumDo);

        foreach ($this ->data as $denJidenicku) {
            foreach ($denJidenicku ->polozky as $polozkaJidelnicku) {
                $polozkaJidelnicku ->alergeny = isset($seznamAlergenu[$polozkaJidelnicku ->f_id])
                    ? $seznamAlergenu[$polozkaJidelnicku ->f_id]
                    : [];
            }
        }
    }

    /**
     * @param Database $database
     * @param string $jidelna
     * @param int $datumOd
     * @param int $datumDo
     * @throws DBException
     * @throws Exception
     */
    function load($database, $jidelna, $datumOd, $datumDo)
    {
        $this ->datumOd = $datumOd;
        $this ->datumDo = $datumDo;

        $this ->data = [];
        $this ->jeFotografie = false;

        $params = $database ->createParameters();
        $sql =
            'SELECT Den FROM Jidelnicek WHERE Potvrzeno = 1 ' .
            ' AND Den >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            ' AND Den < ' . $params ->addParameter($database ->dateToParam($datumDo));

        if ($jidelna) {
            $sql .=
                ' AND KodTypuRozdeleni = 8 AND jidelna =' . $params->addParameter($jidelna);
        }

        $sql .=
            ' ORDER BY Den';

        $cursor = $database ->select($sql, $params);
        while ($row = $cursor ->fetch()) {
            $this ->data[] = $this ->nactiDenJidelnicku($database, $jidelna, $database ->stringToTimestamp($row[0]));
        }

        if (Prava::vratPrava('objednani') ->jsouNejakePrava()) {
            //$this->doplnPoctyObjednavekOsoby($database);
            $this->nastavObjednavani($database);  // az po doplnPoctyObjednavekOsoby!
        }

        $this ->doplnHodnoceni($database);
        $this ->doplnAlergeny($database);
    }
}
