<?php

/**
 * UTF8
 */

namespace Ron\Jidelna;

use Ron\Browser;
use Ron\DB\DBException;
use Ron\RonPortal\RonPortalSession;
use function Ron\ronText;

require_once('ron/browser.php');


class VyuctovaniStravenekBrowser extends Browser
{
    var $data;

    function __construct($database)
    {
        parent::__construct($database);

        $this->title = ronText('Vyuctovani_stravenek');
        $this ->cssClass = 'browser oddEvenRows';

        $this->addField( [BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_SQL_NAME => '1', BROWSER_FIELD_HEADER => ronText('Mesic'), BROWSER_FIELD_ALIGN => 'center'] );
        $this->addField( [BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_SQL_NAME => '1', BROWSER_FIELD_HEADER => ronText('Narok_na_dotace'), BROWSER_FIELD_ALIGN => 'right'] );
        $this->addField( [BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_SQL_NAME => '1', BROWSER_FIELD_HEADER => ronText('Odebrane_stravenky'), BROWSER_FIELD_ALIGN => 'right'] );
        $this->addField( [BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_SQL_NAME => '1', BROWSER_FIELD_HEADER => ronText('Zbyva_odebrat'), BROWSER_FIELD_ALIGN => 'right'] );

        $this ->showFooter = true;
    }


    /**
     * @param int $rok
     * @throws DBException
     */
    function loadVyuctovani($rok)
    {
        $this ->data = [];

        for ($i = 1; $i <= 12; $i++) $this ->data[$i] = [$i, 0, 0, 0];

        $datumdo = mktime(0, 0, 0, 12, 31, $rok);

        // Nacte naroky podle mesicniho vyuctovani
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Obdobi, Pocet_NarokNaDotace FROM UcetStravnikaZaObdobi' .
            ' WHERE Obdobi >= ' . $params ->addParameter($this ->database ->dateToParam(mktime(0, 0, 0, 1, 1, $rok))) .
            '   AND Obdobi <= ' . $params ->addParameter($this ->database ->dateToParam($datumdo)) .
            '   AND UcetStravnikaZaObdobi.RC = ' . $params->addParameter(RonPortalSession::osobaRC()) .
            '   AND UcetStravnikaZaObdobi.OsobniCislo = ' . $params->addParameter(RonPortalSession::osobaOsobniCislo()) .
            '   AND UcetStravnikaZaObdobi.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam(RonPortalSession::osobaNastup())) .
            ' ORDER BY Obdobi';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $index = (int)date('n', $this ->database ->stringToTimestamp($row[0])); // index mesice
            $this ->data[$index][1] = $row[1]; // pocet naroku na dotace
        }
        $cursor ->close();

        // Nacte Odbery
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Obdobi, Pocet FROM PohybStravenek ' .
            ' WHERE Obdobi >= ' . $params ->addParameter($this ->database ->dateToParam(mktime(0, 0, 0, 1, 1, $rok))) .
            '   AND Obdobi < ' . $params ->addParameter($this ->database ->dateToParam(mktime(0, 0, 0, 1, 1, $rok + 1))) .
            '   AND RC = ' . $params->addParameter(RonPortalSession::osobaRC()) .
            '   AND OsobniCislo = ' . $params->addParameter(RonPortalSession::osobaOsobniCislo()) .
            '   AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam(RonPortalSession::osobaNastup())) .
            ' ORDER BY Obdobi';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $index = (int)date('n', $this ->database ->stringToTimestamp($row[0])); // index mesice
            $this ->data[$index][2] -= $row[1]; // pricte pocet odebranych stravenek (pohyb v DB je minusovy)
        }
        $cursor ->close();

        // spocita rozdily
        for ($i = 1; $i <= 12; $i++)
            $this ->data[$i][3] = $this ->data[$i][1] - $this ->data[$i][2];
    }


    function load($sqlFilter = '')
    {
        if (isset($_GET['rok'])) $_SESSION['rok'] = (int)$_GET['rok'];
        if (!isset($_SESSION['rok'])) $_SESSION['rok'] = date('Y');

        $this ->loadVyuctovani((int)$_SESSION['rok']);
    }


    function renderData()
    {
        foreach ($this ->data as $index => $row) {
            $cssClass = ($index == count($this ->data)) ? 'border-bottom' : '';
            $this->renderLine($row, $cssClass, $index);
        }
    }

    function renderTableFooter()
    {
        $sum = [ronText('Celkem'), 0, 0, 0]; // narok, odbery, rozdil

        foreach ($this ->data as $row) {
            $sum[1] += $row[1];
            $sum[2] += $row[2];
            $sum[3] += $row[3];
        }

        $this->renderLine($sum, "browserfooter", 13);
    }


    function renderFilter()
    {
        $rok = (int)$_SESSION['rok'];

        echo
            '<div>' .
            '<a href="?menuid=stravenky&rok=' . ($rok - 1) . '"> <<  </a>' .
            "<span class='margin-left-2 margin-right-2'>$rok</span>" .
            '<a href="?menuid=stravenky&rok=' . ($rok + 1) . '"> >> </a>' .
            '</div>';
    }

}


