<?php
/**
 * UTF8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 29.4.2016
 * Time: 11:10
 */

namespace Ron\RonPortal\Zastupy;

use Exception;
use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\GuiUtils;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalBrowser;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use function Ron\ronText;

require_once 'adsapp/ronportalbrowser.php';
require_once 'adsapp/ronportalfilter.php';

class ZastupyFilterForm extends RonPortalFilterForm
{
    public function __construct($database, $action)
    {
        parent::__construct($database, $action);

        $this->nastavObdobi(true, 'ADS', false, false);
        $this->nastavDatumOdDo(true, false);

        $this->nastavRozdeleni(true, RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_TYP_ROZDELENI), 0, '', true);
        $this->nastavRozdeleni2(true, RonPortalSettings::getIntValue(ZastupyController::TYP_ROZDELENI_POZICE), false, '', true);
    }

    protected function renderUrovenSchvaleni()
    {
        echo '<div class="filterrow">';

        echo '<span class="filtercaption">' . ronText('Zastup_uroven_schvaleni') . '</span>';

        $data = array(
            0 => ronText('Vse'),
            -1 => ronText('Zastup_ke_schvaleni'),
            -2 => ronText('Zastup_bez_schvaleni'),
            1 => ronText('Zastup_schvaleni'),
            2 => ronText('Zastup_zamitnuti'),
            3 => ronText('Zastup_zruseni'),
            100 => ronText('Zastup_k_prirazeni')
        );

        LinksList::renderLinks($data, $_SESSION['zastupyUrovenSchvaleni'], $this ->f_formaction .  '&urovenschvaleni=', '', false);

        echo '</div>';
    }

    /**
     * @throws Exception
     */
    protected function renderDuvodZadosti()
    {
        echo '<div class="filterrow">';

        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenDuvodZadosti', ronText('Zastup_jen_duvod'), $_SESSION['jenDuvodZadosti'], 'onClick="submit()"');
        echo '</span>';

        $sql = 'SELECT Kod, Popis FROM DuvodZastupu ORDER BY Popis';

        $combo = new ComboBox('duvodZadosti');
        $combo->loadData($this->database, $sql, $_SESSION['duvodZadosti']);
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'data-ron-autosubmit="1"';
        $combo->render();

        echo '</div>';
    }

    protected function loadUrovenSchvaleni()
    {
        if (isset($_GET['urovenschvaleni'])) {
            $_SESSION['zastupyUrovenSchvaleni'] = $_GET['urovenschvaleni'];
        }
    }

    protected function loadDuvodZadosti()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenDuvodZadosti'] = isset($_POST['jenDuvodZadosti']);
            $_SESSION['duvodZadosti'] = $_POST['duvodZadosti'];
        }
    }

    function loadData()
    {
        parent::loadData();

        $this->loadUrovenSchvaleni();
        $this->loadDuvodZadosti();
    }

    /**
     * @throws Exception
     */
    public function renderFilters()
    {
        parent::renderFilters();
        $this->renderDuvodZadosti();
        $this->renderUrovenSchvaleni();
    }


}

class ZastupujiciZamestnanecField extends BrowserField
{
    private $zamestnanecRowId;
    private $schvaleniRowId;
    private $typRozdeleniRowId;
    private $rozdeleniRowId;
    private $datumRowId;

    /**
     * ZastupujiciZamestnanecField constructor.
     * @param null|Database $database
     * @param int $zamestnanecRowId
     * @param int $schvaleniRowId
     * @param int $typRozdeleniRowId
     * @param int $rozdeleniRowId
     * @param int $datumRowId
     */
    public function __construct($database, $zamestnanecRowId, $schvaleniRowId, $typRozdeleniRowId, $rozdeleniRowId, $datumRowId)
    {
        parent::__construct($database);

        $this->zamestnanecRowId = $zamestnanecRowId;
        $this->schvaleniRowId = $schvaleniRowId;
        $this->typRozdeleniRowId = $typRozdeleniRowId;
        $this->rozdeleniRowId = $rozdeleniRowId;
        $this->datumRowId = $datumRowId;

        $this->header = ronText('Prijmeni');

        $this->ordering = true;
        $this->sortSQL = 'Osoba.Prijmeni';
        $this->sortSQLDesc = 'Osoba.Prijmeni DESC';
        $this->code = 'col-prijmeni';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        //pokud je osoba prirazena, zobrazi prijmeni
        if ($rowData[$this->zamestnanecRowId]) {
            $result = $rowData[$this->zamestnanecRowId];
        }
        //pokud neni prirazena osoba a zaznam je schvalen, zobrazi odkaz na prirazeni osoby
        elseif ($rowData[$this->schvaleniRowId] == 1) {
            $prava = Prava::vratPrava('zastupy_prideleni');

            if ($prava->jePravoNaRozdeleni($rowData[$this->typRozdeleniRowId], $rowData[$this->rozdeleniRowId],
                $this ->database ->stringToTimestamp($rowData[$this->datumRowId]), $this ->database ->stringToTimestamp($rowData[$this->datumRowId]))) {
                $result = "<a href='?menuid=zastup&action=priradit&formaction=edit&record=" . $rowData[0] . "'>" . ronText('Zastup_priradit') . "</a>";
            }
        }

        return $result;
    }

}

// Odkaz na schvaleni - podle prav
class SchvaleniZastupuField extends BrowserField
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this->code = 'col-schvaleni';
        $this->align = 'left';
        $this->header = ronText('Zastup_schvaleni');
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[15] == 1) { //schvaleno
            $date = $rowData[4];
            $name = $rowData[5];

            $class = 'zastup_textschvaleni';
            $hint = ronText('Zastup_schvaleni');
        } else if ($rowData[15] == 2) { //zamitnuto
            $date = $rowData[4];
            $name = $rowData[5];

            $class = 'zastup_textzamitnuti';
            $hint = ronText('Zastup_zamitnuti');
        }
        else if ($rowData[15] == 3) { //zrusena zadost (smazano planovani kapacit prirazeneho zamestnance)
            $date = $rowData[4];
            $name = $rowData[5];

            $class = 'zastup_textzruseni';
            $hint = ronText('Zastup_zruseni');
        } else {
            return '';
        }

        $result = "<SPAN class='$class' title='$hint'>";
        if ($date) $result .= DBUtils::getFormatedValue($this ->database, 'date', $date);
        if ($name) $result .= " ($name)";
        $result .= "</SPAN>\n";

        return $result;
    }
}


class SchvaleniZastupuEditField extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $prava;
    var $indexSchvaleno;
    var $indexDatum;
    var $indexUzivatel;

    /** @var ZastupyBrowser */
    var $browser;

    var $existsEdit;


    /**
     * @param Browser $browser
     * @param PravaNaOsoby $prava
     * @param int $indexSchvaleno
     * @param int $indexDatum
     * @param int $indexUzivatel     
     * @param string $caption
     */
    function __construct($browser, $prava, $indexSchvaleno, $indexDatum, $indexUzivatel, $caption)
    {
        parent::__construct($browser ->database);

        $this ->code = 'col-schvaleniedit';
        $this ->align = "center";
        $this ->header =
            "<span id='schvaleniZastupuMenu_Header''>$caption</span>";

        $this ->existsEdit = false;

        $this ->browser = $browser;
        
        $this ->prava = $prava;
        $this ->indexSchvaleno = $indexSchvaleno;
        $this ->indexDatum = $indexDatum;
        $this ->indexUzivatel = $indexUzivatel;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $schvaleno = isset($this ->indexSchvaleno) ? $rowData[$this ->indexSchvaleno] == 1 : $rowData[$this ->indexDatum];

        $hint = '';
        if ($schvaleno) {
            $hint = $rowData[$this ->indexUzivatel];
            if ($this ->indexDatum  &&  $rowData[$this ->indexDatum]) $hint .= ' (' . DBUtils::getFormatedValue($this ->database, 'finedatetime', $rowData[$this ->indexDatum]) . ')';
        }

        if ($this ->prava ->jePravoNaRozdeleni($rowData[14], $rowData[6], $rowData[1], $rowData[1]) && $rowData[$this->indexSchvaleno] != 3) {
            // muze schvalovat
            $this ->existsEdit = true;

            return
                "<input type='hidden' name='NastavitSchvaleni_{$rowData[0]}' value='1' />" .
                "<input type='checkbox' data-ronschvalenizastupu_index='{$this ->browser ->jsonRowIndex}' name='Schvaleni_{$rowData[0]}' " .
                "id='Schvaleni_{$rowData[0]}' " .
                ($schvaleno ? " checked='checked'" : '') . ( $hint ? " title='$hint'" : '' ) . " />\n";
        } else {
            return '<span' . ( $hint ? " title='$hint'" : '' ) . '>' . ronText($schvaleno ? 'Str_Yes' : 'Str_No') . '</span>';
        }
    }


    /**
     * @param int $fieldpos
     * @param string $url
     * @return string
     */
    function getFooterText($fieldpos, $url = null)
    {
        if ($this ->existsEdit) {
            return "<input type='checkbox' name='schvaleni_ALL' id='schvaleni_ALL' title='' />\n";
        }
        else return '';
    }

}

class ZruseniZastupuField extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $prava;

    /**
     * @param Database $database
     * @param PravaNaOsoby $prava
     */
    function __construct($database, $prava)
    {
        parent::__construct($database);
        $this->prava = $prava;
        $this->code = 'col-delete';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[12]) {
            // je prirazena osoba, nejde smazat
            return '';
        }

        if ($rowData[15]) return ''; // jiz je schvaleno/zamitnuto, nejde smazat

        // zkontroluje prava na editaci zastupu
        if ($this->prava->jePravoNaRozdeleni($rowData[14], $rowData[6], $this ->database ->stringToTimestamp($rowData[1]), $this ->database ->stringToTimestamp($rowData[1]))) {
            return GuiUtils::changeLocationQuest('<i class="inline-glyph glyph-button fa glyph-delete"></i>', ronText('Zastup_opravdu_zrusit'),
                '?menuid=zastup&action=zrusitzastup&record=' . $rowData[0]);
        }

        return '';
    }
}

class ZastupyBrowser extends RonPortalBrowser
{
    private $schvalovani;

    private $pravaVlozit;
    private $pravaSchvalit;
    private $pravaPriradit;
    private $pravaZobrazit;
    private $pravaZrusit;

    public $jsonRowIndex;
    public $jsonObject;
    public $hiddenFields;

    /**
     * ZastupyBrowser constructor.
     * @param Database $database
     * @param bool $schvalovani
     * @throws Exception
     */
    public function __construct($database, $schvalovani = false)
    {
        parent::__construct($database, 'zastup');

        $this->title = ronText('Zastup_zadosti');
        
        //$this->browserUrl = '?menuid=zastup';
        $this->code = 'browserzastupy';
        $this->tableId = 'seznamzastupu';

        $this->pravaVlozit = Prava::vratPrava('zastupy_vlozeni');
        $this->pravaSchvalit = Prava::vratPrava('zastupy_schvaleni');
        $this->pravaPriradit = Prava::vratPrava('zastupy_prideleni');
        $this->pravaZobrazit = Prava::vratPrava('zastupy_prohl');
        $this->pravaZrusit = Prava::vratPrava('zastupy_zruseni');

        $this->schvalovani = $schvalovani;
        $this->showFooter = $schvalovani;

        $this->sqlSelectPart =
            'ZadostOZastup.ID, ZadostOZastup.Datum, ZadostOZastup.CasOd, ZadostOZastup.CasDo, ' . // 0 - 3
            'ZadostOZastup.Schvaleni, ZadostOZastup.Schvalil, ' . // 4, 5
            'ZadostOZastup.Rozdeleni, ZadostOZastup.Rozdeleni2, Rozdeleni.Popis, COALESCE(Pozice.Popis, PoziceKategorie.Popis) AS PopisPozice, ' . //6 - 9
            'Osoba.Jmeno, Osoba.Prijmeni, PracovniPomer.ID, ZadostOZastup.Poznamka, ' . // 10 - 13
            'ZadostOZastup.KodTypuRozdeleni, ZadostOZastup.TypSchvaleni, DuvodZastupu.Popis, ZadostOZastup.Specifikace '; //14 - 17

        $this->sqlFromPart =
            'ZadostOZastup LEFT JOIN Osoba ON (ZadostOZastup.RC = Osoba.RC) ' .
            ' LEFT JOIN PracovniPomer ON (PracovniPomer.RC = ZadostOZastup.RC AND PracovniPomer.OsobniCislo = ZadostOZastup.OsobniCislo AND PracovniPomer.DatumNastupu = ZadostOZastup.DatumNastupu) ' .
            ' LEFT JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = ZadostOZastup.KodTypuRozdeleni AND Rozdeleni.Kod = ZadostOZastup.Rozdeleni) ' .
            ' LEFT JOIN Rozdeleni AS Pozice ON (Pozice.KodTypuRozdeleni = ZadostOZastup.KodTypuRozdeleni2 AND Pozice.Kod = ZadostOZastup.Rozdeleni2) ' .
            ' LEFT JOIN DuvodZastupu ON (DuvodZastupu.Kod = ZadostOZastup.DuvodZadosti)' .
            ' LEFT JOIN Kategorie AS PoziceKategorie ON (PoziceKategorie.Kod = ZadostOZastup.KodKategorie AND PoziceKategorie.TypKategorie = ZadostOZastup.TypKategorie)';

        $this->primaryField = 0;

        $this->addFields([
            [BROWSER_FIELD_HEADER_TEXT => 'Zastup_datum', BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_SQL_NAME => 'ZadostOZastup.Datum',
                BROWSER_FIELD_CODE => 'col-datum',
                BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_LINK => '?menuid=zastup&action=detail'],

            [BROWSER_FIELD_HEADER_TEXT => 'Zastup_cas_od', BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_SQL_NAME => 'ZadostOZastup.CasOd',
                BROWSER_FIELD_CODE => 'col-casod',
                BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_FORMAT => 'time'],

            [BROWSER_FIELD_HEADER_TEXT => 'Zastup_cas_do', BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_SQL_NAME => 'ZadostOZastup.CasDo',
                BROWSER_FIELD_CODE => 'col-casdo',
                BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_FORMAT => 'time'],

            [BROWSER_FIELD_HEADER_TEXT => 'Kod', BROWSER_FIELD_SQL_INDEX => 6, BROWSER_FIELD_SQL_NAME => 'ZadostOZastup.Rozdeleni',
                BROWSER_FIELD_CODE => 'col-kodrozdeleni',
                BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1],

            [BROWSER_FIELD_HEADER_TEXT => 'Zastup_rozdeleni', BROWSER_FIELD_SQL_INDEX => 8, BROWSER_FIELD_SQL_NAME => 'Rozdeleni.Popis',
                BROWSER_FIELD_CODE => 'col-popisrozdeleni',
                BROWSER_FIELD_ALIGN => 'left', BROWSER_FIELD_ORDERING => 1],

            [BROWSER_FIELD_HEADER_TEXT => 'Zastup_pozice', BROWSER_FIELD_SQL_INDEX => 9, BROWSER_FIELD_SQL_NAME => 'PopisPozice',
                BROWSER_FIELD_CODE => 'col-pozice',
                BROWSER_FIELD_ALIGN => 'left', BROWSER_FIELD_ORDERING => 1],
        ]);

        if ($this->schvalovani) {
            $this->addField(new SchvaleniZastupuEditField($this, $this->pravaSchvalit, 15, 4, 5, ronText('Zastup_schvaleni')));
        } else {
            $this->addField(new SchvaleniZastupuField($this ->database));
        }

        $this->addField(new ZastupujiciZamestnanecField($this->database, 11, 15, 14, 6, 1));

        $this->addFields([
            [BROWSER_FIELD_HEADER_TEXT => 'Jmeno', BROWSER_FIELD_SQL_INDEX => 10, BROWSER_FIELD_SQL_NAME => 'Osoba.Jmeno',
                BROWSER_FIELD_CODE => 'col-jmeno',
                BROWSER_FIELD_ALIGN => 'left', BROWSER_FIELD_ORDERING => 1],

            [BROWSER_FIELD_HEADER_TEXT => 'Zastup_duvod', BROWSER_FIELD_SQL_INDEX => 16, BROWSER_FIELD_SQL_NAME => 'ZadostOZastup.DuvodZadosti',
                BROWSER_FIELD_CODE => 'col-duvod',
                BROWSER_FIELD_ALIGN => 'left', BROWSER_FIELD_ORDERING => 1],

            [BROWSER_FIELD_HEADER_TEXT => 'Zastup_poznamka', BROWSER_FIELD_SQL_INDEX => 13, BROWSER_FIELD_SQL_NAME => 'ZadostOZastup.Poznamka',
                BROWSER_FIELD_CODE => 'col-poznamka',
                BROWSER_FIELD_ALIGN => 'left', BROWSER_FIELD_ORDERING => 1]
        ]);

        $this->addField(new ZruseniZastupuField($database, $this->pravaVlozit));

        $this->defaultSort = 1;
    }


    function setFilterForm()
    {
        $this->filterForm = new ZastupyFilterForm($this->database,'zastup');
    }


    public function renderFilter()
    {
        $this->filterForm->render();

        if (!$this->schvalovani) {
            if ($this->pravaVlozit->jsouNejakePrava()) {
                echo "<div class='normaltext' id='zastup_links'><a href='?menuid=zastup&formaction=new'>" . ronText('New_record') . "</a></div>\n";
            }

            if ($this->pravaSchvalit->jsouNejakePrava()) {
                echo "<div id='poklk_hromadneschvalovani'><input type='button' class='button' name='zobrazithromadne' value='" . ronText('Zastup_schvalit_hromadne') . "'" .
                    " onClick='ron.zastup.hromadneSchvaleni.zobrazHromadne()'/></div>";
            }
        }
    }

    function getSQLFilter()
    {
        $result = '';

        // uroven schvaleni (vse, ke schvaleni, ...)
        switch ($_SESSION['zastupyUrovenSchvaleni']) {
            case -2 : // bez schvaleni
                $result .= 'ZadostOZastup.TypSchvaleni = 0 ';
                break;

            case 1 : // schvalene
                $result .= 'ZadostOZastup.TypSchvaleni = 1 ';
                break;

            case 2 : // zamitnute
                $result .= 'ZadostOZastup.TypSchvaleni = 2 ';
                break;

            case 3 : // zrusene
                $result .= 'ZadostOZastup.TypSchvaleni = 3 ';
                break;

            case -1 : // ke schvaleni (dale se filtruje v enable line podle konkretnich prav)
                $result .= 'ZadostOZastup.TypSchvaleni = 0 ';
                break;

            case 100: //zaznamy, kde neni osoba, dale se filtruje v enable line podle prav
                $result .= 'ZadostOZastup.RC IS NULL ';
                break;

            default : // vse
                $result .= ' 1 = 1 ';
        }

        if ($this ->filterForm ->jenObdobi) {
            if ($result) {
                $result .= ' AND ';
            }
            $result .=
                '  ZadostOZastup.Datum >= ' . $this->database->timestampToString($this ->filterForm ->vybraneObdobiOd) .
                ' AND ZadostOZastup.Datum <= ' . $this->database->timestampToString($this ->filterForm ->vybraneObdobiDo);
        } elseif ($this ->filterForm ->filtrovatDatum) {
            if ($result) {
                $result .= ' AND ';
            }
            $result .=
                '  ZadostOZastup.Datum >= ' . $this->database->timestampToString($this ->filterForm ->datumOd) .
                ' AND ZadostOZastup.Datum <= ' . $this->database->timestampToString($this ->filterForm ->datumDo);
        }

        if (RonPortalSession::rozdeleni($this->filterForm->f_typrozdeleni) !== null  &&  RonPortalSession::rozdeleni($this->filterForm->f_typrozdeleni) !== '') {
            if ($result) {
                $result .= ' AND ';
            }
            if (RonPortalSession::rozdeleniVcetnePodrizenych($this->filterForm->f_typrozdeleni)) {
                $result .=
                    " EXISTS(SELECT * FROM HierarchieRozdeleni " .
                    " WHERE HierarchieRozdeleni.typNadrazene = " . (int)$this->filterForm->f_typrozdeleni .
                    "   AND HierarchieRozdeleni.kodNadrazene = '" . $this->database->addSlashes(RonPortalSession::rozdeleni($this->filterForm->f_typrozdeleni)) . "'" .
                    "   AND HierarchieRozdeleni.typPodrizene = ZadostOZastup.KodTypuRozdeleni " .
                    "   AND HierarchieRozdeleni.kodPodrizene = ZadostOZastup.Rozdeleni" .
                    ")";
            } else {
                $result .= " Rozdeleni = '" . $this->database->addSlashes(RonPortalSession::rozdeleni($this->filterForm->f_typrozdeleni)) . "'";
            }
        }

        if (RonPortalSession::rozdeleni2($this->filterForm->f_typrozdeleni2) !== null  &&  RonPortalSession::rozdeleni2($this->filterForm->f_typrozdeleni2) !== '') {

            if ($result) {
                $result .= ' AND ';
            }
            if (RonPortalSession::rozdeleni2VcetnePodrizenych($this->filterForm->f_typrozdeleni2)) {
                $result .=
                    " EXISTS(SELECT * FROM HierarchieRozdeleni " .
                    " WHERE HierarchieRozdeleni.typNadrazene = " . (int)$this->filterForm->f_typrozdeleni2 .
                    "   AND HierarchieRozdeleni.kodNadrazene = '" . $this->database->addSlashes(RonPortalSession::rozdeleni2($this->filterForm->f_typrozdeleni2)) . "'" .
                    "   AND HierarchieRozdeleni.typPodrizene = ZadostOZastup.KodTypuRozdeleni2 " .
                    "   AND HierarchieRozdeleni.kodPodrizene = ZadostOZastup.Rozdeleni2" .
                    ")";
            } else {
                $result .= " Rozdeleni2 = '" . $this->database->addSlashes(RonPortalSession::rozdeleni2($this->filterForm->f_typrozdeleni2)) . "'";
            }
        }

        if ($_SESSION['jenDuvodZadosti']) {
            if ($result) {
                $result .= ' AND ';
            }
            $result .= " DuvodZadosti = '" . $this->database->addSlashes($_SESSION['duvodZadosti']) . "'";
        }

        return $result;
    }

    /**
     * @param array $values
     * @param int $datumod
     * @param int $datumdo
     * @return bool
     * @throws Exception
     */
    function zastupEnableLine($values, $datumod, $datumdo)
    {
        $this ->jsonRowIndex = null;

        if ($_SESSION['zastupyUrovenSchvaleni'] == -1) {
            // filtr Ke schvaleni - zkontroluje, jesti to ceka na me

            if ($values[15] == 1) return false;
            // jiz je schvaleno nejvyssi urovni = neceka na nikoho

            return $this->pravaSchvalit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo);
            // neni schvaleno zadnou urovni => pokud mam pravo, ceka na me
        }

        if ($_SESSION['zastupyUrovenSchvaleni'] == 100) {
            // filtr K prirazeni - zkontroluje, jesti to ceka na me

            if ($values[15] != 1) return false;

            return $this->pravaPriradit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo);
            // neni schvaleno zadnou urovni => pokud mam pravo, ceka na me
        }

        // Zobrazi zaznam, pokud mam kterekoliv pravo
        if (!$this->schvalovani) {
            if ($this->pravaVlozit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo)) return true;
            if ($this->pravaPriradit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo)) return true;
            //pokud ma uzivatel pravo zobrazit prirazenou osobu, nebo rozdeleni zobrazi se mu zaznam
            if (($this->pravaZobrazit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo)) ||
                ($this->pravaZobrazit->lzeZobrazitPP($values[12], $datumod, $datumdo))) return true;
        }

        if ($this->pravaSchvalit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo)) return true;
        //pravo zrusit musi mit zaroven na rozdeleni, kam je podana zadost, i na prirazeneho zamestnance
        if (($this->pravaZrusit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo)) &&
            ($this->pravaZrusit->lzeZobrazitPP($values[12], $datumod, $datumdo))) return true;

        return false;
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        $datumod = $this ->database ->stringToTimestamp($values[1]);
        $datumdo = $datumod;

        $result = $this ->zastupEnableLine($values, $datumod, $datumdo);
        if (!$result) return false;

        if ($this ->schvalovani) {

            $this ->jsonRowIndex = count($this ->jsonObject['items']);

            $this ->hiddenFields .= "<input type='hidden' name='idZastup_{$this ->jsonRowIndex}' value='{$values[0]}'/>";

            $this ->jsonObject['items'][] = [
                'id' => $values[0],
                'pravo' => $this ->pravaSchvalit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo) ? 1 : 0,
                'schvaleno' => ($values[15] == 1) ? 1 : 0,
            ];

        } else {
            $this ->jsonRowIndex = null;
        }

        return true;
    }

    /**
     * @param array  $values
     * @param string $className
     * @param int    $rowNumber
     * @return string   Vraci atributy pro <tr>
     */
    function getRowAttributes($values, $className, $rowNumber)
    {
        if ($values[15] == 1) $className .= ' zastup_schvalen';
        else if ($values[15] == 2) $className .= ' zastup_zamitnut';

        $result = parent::getRowAttributes($values, $className, $rowNumber);

        $result .= ' id="zastup_row_' . $values[0] . '"';

        return $result;
    }

    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($values[17]) {
            // zobrazi specifikaci
            echo "<tr class='$className'>";
            echo    '<td></td><td colspan="' . ($this->columnCount - 1) . '"><div class="browserpoznamka">' . $values[17] . '</div></td>';
            echo '</tr>';
        }
    }

    function renderContent()
    {
        $this ->jsonObject = [];
        $this ->jsonObject['items'] = [];

        $this ->hiddenFields = '';

        if ($this ->schvalovani) {
            echo
                '<form method="post" name="hromadneSchvalovaniZastupu" action="?menuid=zastup">' .
                '<input type="hidden" name="hromadneSchvaleni" value="1"/>';
        }

        echo
            '<ul class="popupmenu"  id="schvaleniZastupuMenu" style="display:none">' .
            "<li data-ron-hodnotaschvaleni='1'>" . ronText('Oznacit_vse') . "</li>".
            "<li data-ron-hodnotaschvaleni='0'>" . ronText('Zrusit_oznaceni') . "</li>".
            "<li data-ron-hodnotaschvaleni='-1'>" . ronText('form.Original') . "</li>".
            '</ul>';


        parent::renderContent();

        if ($this ->hiddenFields) echo $this ->hiddenFields;
        echo '<input type="hidden" name="pocet" value="' . count($this ->jsonObject['items']) . '" />';

        if ($this->schvalovani) {
            echo
                "<div class='buttons'>" .
                "<input type='button' class='submitbutton' value='" . ronText('Provest') . "' id='provestHromadneSchvaleniZastupu' />" .
                '<input type="button" class="button" value="' . ronText('Cancel') . '" id="stornoHromadneSchvaleniZastupu" />' .
                '</div>'.
                "</form>\n";

            echo
                '<script type="text/javascript">' .
                'ron.zastup.hromadneSchvaleni.init(' . json_encode($this ->jsonObject) . ');' .
                '</script>';
        } else {
            if ($this->pravaVlozit->jsouNejakePrava()) echo "<div class='normaltext'><a href='?menuid=zastup&formaction=new'>" . ronText('New_record') . "</a></div>\n";
        }
    }
}