<?php
/**
 * UTF8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 29.4.2016
 * Time: 15:43
 */

namespace Ron\RonPortal\Zastupy;

use Exception;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SQLUtils;
use Ron\DBRecord;
use Ron\DBRecordField;
use Ron\FormField;
use Ron\MessageManager;
use Ron\RonPortal\OsobaField;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalForm;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\RonPortalXMLEvents;
use Ron\RonPortal\Rozdeleni;
use Ron\RonPortal\UserLoginField;
use Ron\StringUtils;
use function Ron\ronText;

require_once('adsapp/ronportalform.php');
require_once('adsapp/rozdeleni.php');
require_once('ron/controls.php');
require_once('adsapp/events.php');

class ZastupyVyberRozdeleniField extends FormField
{
    /** @var  PravaNaOsoby */
    var $prava;
    var $typrozdeleni;

    private $vycetRozdeleni;
    private $indexRozdeleni;

    /**
     * ZastupyVyberRozdeleniField constructor.
     * @param int $typrozdeleni
     * @param PravaNaOsoby|null $prava
     * @param string $fieldname Nazev pole pro html a v databazi
     * @param string $captionText Kod retezce v ini
     * @param bool $edit zda je pole editovatelne
     * @param bool $required povinnost pole
     * @param int $indexRozdeleni
     * @param null|string $vycet vycet kodu rozdeleni (pokud je zadan, nepouziji se prava)
     */
    function __construct($typrozdeleni, $prava, $fieldname, $captionText, $edit, $required, $indexRozdeleni, $vycet = null)
    {
        parent::__construct();
        $this->prava = $prava;
        $this->typrozdeleni = $typrozdeleni;
        $this->indexRozdeleni = $indexRozdeleni;
        $this->vycetRozdeleni = $vycet;

        $this->fieldType = "rozdeleni";
        $this->fieldName = $fieldname;
        $this->sqlFieldName = $fieldname;
        $this->required = $required;
        $this->editable = $edit;
        $this->maxLength = 30;

        $this->caption = ronText($captionText);
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canedit
     * @param bool $insert
     * @param array $rowdata
     * @throws DBException
     */
    function showFieldContent($database, $value, $canedit, $insert, $rowdata)
    {
        if ($canedit) {

            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass '.
                ' FROM Rozdeleni'.
                ' WHERE Rozdeleni.KodTypuRozdeleni = '.(int)$this->typrozdeleni.
                ' AND (Rozdeleni.DatumZruseni >= '.$database->timestampToString(time()).' OR Rozdeleni.DatumZruseni IS NULL)' ;

            if ($this->vycetRozdeleni) {
                $sql .= ' AND (' . SQLUtils::getINCondition($database, 'Rozdeleni.Kod', explode(';', $this->vycetRozdeleni)) . ')';
            } elseif ($this->prava) {
                $sql .= '  AND (' . $this->prava->sqlfiltrRozdeleni(time(), time()) . ')';
            }

            $sql .= " AND (cssClass IS NULL  OR  cssClass <> 'hidden') ";

            MessageManager::addDebugLog('Combo vyber rozdeleni: ' . $sql);

            $combo = new ComboBox($this->fieldName);
            $combo ->alwaysShowSelected = isset($value) && $value != '';
            $combo ->cssClassIndex = 2;
            $combo ->captionIndex = 0;
            $combo ->keyIndex = 0;
            $combo ->loadData($database, $sql.' ORDER BY Kod', $rowdata[$this->indexRozdeleni]);
            $combo ->emptyRow = (isset($value) && $value != ''  &&  $this->required) ? null : ' ';
            $combo ->className .= ' col1';
            $combo ->htmlParams = "id='{$this->fieldName}_kod' data-ron-select2='" . $this->fieldName . "_nazev'";
            $combo ->render();

            $combo = new ComboBox($this->fieldName.'_nazev');
            $combo ->alwaysShowSelected = isset($value) && $value != '';
            $combo ->cssClassIndex = 2;
            $combo ->captionIndex = 1;
            $combo ->keyIndex = 0;
            $combo ->loadData($database, $sql.' ORDER BY Popis', $rowdata[$this->indexRozdeleni]);
            $combo ->emptyRow = (isset($value) && $value != ''  &&  $this->required) ? null : ' ';
            $combo ->className .= ' col23';
            $combo ->htmlParams = "id='{$this->fieldName}_popis' data-ron-select2='$this->fieldName'";
            $combo ->render();

        } else {
            $rozdeleni =  new Rozdeleni();
            $rozdeleni->load($database, $this->typrozdeleni, $rowdata[$this->indexRozdeleni]);
            echo '<INPUT type="hidden" name="' . $this->fieldName . '" value="' . $rozdeleni->kod . '" />';
            echo StringUtils::htmlTextConvert($rozdeleni->popis) . "\n";
        }
    }

    function createDBRecordField($database, $tableName, $insert)
    {
        $field = new DBRecordField($this->fieldName, $this->fieldType, !$this->editable, $this->maxLength);
        $field->value = $_POST[$this->fieldName];

        return $field;
    }
}

class ZastupyFiltrRozdeleniField extends FormField
{
    var $typrozdeleni;

    private $vybraneRozdeleni;
    private $url;

    /**
     * ZastupyVyberRozdeleniField constructor.
     * @param int $typrozdeleni
     * @param string $fieldname Nazev pole pro html a v databazi
     * @param string $captionText Kod retezce v ini
     * @param string $vybraneRozdeleni
     * @param string $url
     */
    function __construct($typrozdeleni, $fieldname, $captionText, $vybraneRozdeleni, $url)
    {
        parent::__construct();
        $this->typrozdeleni = $typrozdeleni;
        $this->vybraneRozdeleni = $vybraneRozdeleni;
        $this->url = $url;

        $this->fieldType = "rozdeleni";
        $this->fieldName = $fieldname;
        $this->sqlFieldName = $fieldname;
        $this->maxLength = 30;
        $this->editable = true;

        $this->caption = ronText($captionText);
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canedit
     * @param bool $insert
     * @param array $rowdata
     * @throws DBException
     */
    function showFieldContent($database, $value, $canedit, $insert, $rowdata)
    {
        if ($canedit) {

            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass '.
                ' FROM Rozdeleni'.
                ' WHERE Rozdeleni.KodTypuRozdeleni = '.(int)$this->typrozdeleni.
                ' AND (Rozdeleni.DatumZruseni >= '.$database->timestampToString(time()).' OR Rozdeleni.DatumZruseni IS NULL)' ;

            $sql .= " AND (cssClass IS NULL  OR  cssClass <> 'hidden') ";

            MessageManager::addDebugLog('Combo filtr rozdeleni: ' . $sql);

            $combo = new ComboBox($this->fieldName);
            $combo ->alwaysShowSelected = isset($value) && $value != '';
            $combo ->cssClassIndex = 2;
            $combo ->captionIndex = 0;
            $combo ->keyIndex = 0;
            $combo ->loadData($database, $sql.' ORDER BY Kod', $this->vybraneRozdeleni);
            $combo ->emptyRow = (isset($value) && $value != ''  &&  $this->required) ? null : ' ';
            $combo ->className .= ' col1';
            $combo ->htmlParams = "id='{$this->fieldName}_kod' data-ron-select2='" . $this->fieldName . "_nazev' OnChange='window.location=&quot;" . $this->url . "&quot;+this.value;'";
            $combo ->render();

            $combo = new ComboBox($this->fieldName.'_nazev');
            $combo ->alwaysShowSelected = isset($value) && $value != '';
            $combo ->cssClassIndex = 2;
            $combo ->captionIndex = 1;
            $combo ->keyIndex = 0;
            $combo ->loadData($database, $sql.' ORDER BY Popis', $this->vybraneRozdeleni);
            $combo ->emptyRow = (isset($value) && $value != ''  &&  $this->required) ? null : ' ';
            $combo ->className .= ' col23';
            $combo ->htmlParams = "id='{$this->fieldName}_popis' data-ron-select2='$this->fieldName' OnChange='window.location=&quot;" . $this->url . "&quot;+this.value;'";
            $combo ->render();

        }
    }
}

class ZastupyVyberKategorieFiled extends FormField
{
    private $typKategorie;
    private $indexKategorie;

    /**
     * ZastupyVyberKategorieFiled constructor.
     * @param string $typKategorie
     * @param string $fieldname
     * @param string $captionText kod ini textu
     * @param boolean $edit
     * @param boolean $required
     * @param int $indexKategorie
     */
    public function __construct($typKategorie, $fieldname, $captionText, $edit, $required, $indexKategorie)
    {
        parent::__construct();

        $this->typKategorie = $typKategorie;
        $this->indexKategorie = $indexKategorie;

        $this->fieldType = FORM_FIELD_TYPE_SELECT;
        $this->fieldName = $fieldname;
        $this->sqlFieldName = $fieldname;
        $this->required = $required;
        $this->editable = $edit;
        $this->maxLength = 30;

        $this->caption = ronText($captionText);
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canedit
     * @param bool $insert
     * @param array $rowdata
     * @throws DBException
     */
    function showFieldContent($database, $value, $canedit, $insert, $rowdata)
    {
        if ($canedit) {

            $sql =
                "SELECT DISTINCT Kategorie.Kod, Kategorie.Popis ".
                " FROM Kategorie".
                " WHERE Kategorie.TypKategorie = '" . $database->addSlashes($this->typKategorie) . "'";


            MessageManager::addDebugLog('Combo vyber kategorie: ' . $sql);

            $combo = new ComboBox($this->fieldName);
            $combo ->alwaysShowSelected = isset($value) && $value != '';
            $combo ->captionIndex = 0;
            $combo ->keyIndex = 0;
            $combo ->loadData($database, $sql.' ORDER BY Kod', '');
            $combo ->emptyRow = (isset($value) && $value != ''  &&  $this->required) ? null : ' ';
            $combo ->className .= ' col1';
            $combo ->htmlParams = "id='{$this->fieldName}_kod' data-ron-select2='" . $this->fieldName . "_nazev'";
            $combo ->render();

            $combo = new ComboBox($this->fieldName.'_nazev');
            $combo ->alwaysShowSelected = isset($value) && $value != '';
            $combo ->captionIndex = 1;
            $combo ->keyIndex = 0;
            $combo ->loadData($database, $sql.' ORDER BY Popis', '');
            $combo ->emptyRow = (isset($value) && $value != ''  &&  $this->required) ? null : ' ';
            $combo ->className .= ' col23';
            $combo ->htmlParams = "id='{$this->fieldName}_popis' data-ron-select2='$this->fieldName'";
            $combo ->render();

        } else {
            $popisKategorie = DBUtils::getSqlValue($database, "SELECT Popis FROM Kategorie WHERE Kod = '" . $database->addSlashes($rowdata[$this->indexKategorie]) . "'" .
                " AND TypKategorie = '" . $database->addSlashes($this->typKategorie) . "'");

            echo '<INPUT type="hidden" name="' . $this->fieldName . '" value="' . $rowdata[$this->indexKategorie] . '" />';
            echo StringUtils::htmlTextConvert($popisKategorie) . "\n";
        }
    }
}

class ZastupyDatumDoField extends FormField
{
    public function __construct($caption, $fieldName, $maxLength, $defaultValue)
    {
        parent::__construct();

        $this->fieldType = FORM_FIELD_TYPE_DATE;
        $this->caption = $caption;
        $this->fieldName = $fieldName;
        $this->maxLength = $maxLength;
        $this->defaultValue = $defaultValue;

        $this ->otherHtmlParams = 'data-roncalendar-prev="Datum"'; // pro popup kalendar
    }

}

class ZastupyCasOdDoField extends FormField
{
    function __construct($captionText, $edit)
    {
        parent::__construct();

        $this ->caption = ronText($captionText);

        $this ->fieldType = 'casoddo';

        $this ->editable = $edit;
    }


    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($canEdit && isset($_POST['caszastupuod'])) $valueOd = $_POST['caszastupuod'];
        else if ($rowData[2]) $valueOd = date('H:i', $database ->stringToTimestamp($rowData[2]));
        else $valueOd = '';

        if (isset($_POST['caszastupudo'])) $valueDo = $_POST['caszastupudo'];
        else if ($rowData[3]) $valueDo = date('H:i', $database ->stringToTimestamp($rowData[3]));
        else $valueDo = '';

        if ($canEdit) {
            echo '<div id="zastup_od_do">';

            echo '<input class="app_input inputTime" type="text" name="caszastupuod" maxlength="5" size="5" value="' . StringUtils::htmlTextConvert($valueOd) . '" />';
            echo '&nbsp;&nbsp;' . ronText('Do') . '&nbsp;&nbsp;';
            echo '<input class="app_input inputTime" type="text" name="caszastupudo" maxlength="5" size="5" value="' . StringUtils::htmlTextConvert($valueDo) . '" />';

            echo '</div>';
        } else {
            echo $valueOd . ' ' . ronText('Do') . ' ' . $valueDo;
        }

    }

}


class ZastupyForm extends RonPortalForm
{
    /** @var PravaNaOsoby */
    private $prava;
    /** @var  PravaNaOsoby */
    private $pravaSchvalit;

    private $prirazeniZastupu = false;

    private $zastup;

    public $id;

    /**
     * ZastupyForm constructor.
     * @param Database $database
     * @param string $formAction
     * @param PravaNaOsoby $prava
     * @param PravaNaOsoby $pravaSchvalit
     * @param bool $prirazeniZastupu
     * @param Zastup $zastup
     * @param string $rozdeleniFiltr
     * @throws Exception
     */
    public function __construct($database, $formAction, $prava, $pravaSchvalit, $prirazeniZastupu = false, $zastup = null, $rozdeleniFiltr = '')
    {
        parent::__construct($database, $formAction);

        $this->prava = $prava;

        $this->pravaSchvalit = $pravaSchvalit;

        $this->prirazeniZastupu = $prirazeniZastupu;
        $this->zastup = $zastup;

        $this ->htmlAction = '?menuid=zastup' . ($prirazeniZastupu ? '&action=priradit' /*.
                (isset($_GET{'record'}) ? '&formaction=' . $this->formAction . '&record=' . $_GET['record'] : '')*/
                : '');
        $this ->stornoAction = '?menuid=zastup';

        $this->formCode = 'ZadostOZastup';

        $this ->title = $this->formAction == 'new' ? ronText('Zastup_novy') : ($prirazeniZastupu ? ronText('Zastup_prirazeni') : ronText('Zastup_editace'));

        $this ->sqlTableName = 'ZadostOZastup';
        $this ->dbLogSrcDescription = StringUtils::fromUTF8('Žádost o zástup');

        $this->sqlSelectPart =
            'ID, Datum, CasOd, CasDo, KodTypuRozdeleni, Rozdeleni, KodTypuRozdeleni2, Rozdeleni2, ' . // 0 - 7
            'RC, OsobniCislo, DatumNastupu, Schvaleni, Schvalil, Poznamka, Insert_UserName, Insert_Timestamp, Update_UserName, Update_Timestamp, ' .  // 8 - 17
            'DuvodZadosti' . (ZastupyController::$jeNovaVerzeADS ? ', KodKategorie' : ''); //18, 19

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_ID, FORM_FIELD_NAME => 'ID', FORM_FIELD_PRIMARY => true, FORM_FIELD_SQL_INDEX => 0 ]);


        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_CAPTION_TEXT => 'Zastup_datum_od', FORM_FIELD_REQUIRED => true, FORM_FIELD_SQL_INDEX => 1,
            FORM_FIELD_SQL_NAME => 'Datum',  FORM_FIELD_NAME => 'Datum', FORM_FIELD_EDIT => false, FORM_FIELD_MAXLENGTH => 10,
            FORM_FIELD_DEFAULT_VALUE => StringUtils::format('date',DateUtils::startOfTheDay(time())),
            'otherHtmlParams' => 'data-roncalendar-next="Datum_do"'
        ]);

        if ($this->isInsert()) {
            $this->addField(new ZastupyDatumDoField(ronText('Zastup_datum_do'), 'Datum_do', 10, StringUtils::format('date', DateUtils::startOfTheDay(time()))));
        }

        $this->addField(new ZastupyCasOdDoField('Zastup_cas_od', !$prirazeniZastupu));

        //rozdeleni na ktere se zada zastup - dle nastaveni planovani kapacit
        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_HIDDEN, FORM_FIELD_DEFAULT_VALUE => RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_TYP_ROZDELENI),
            FORM_FIELD_SQL_NAME => 'KodTypuRozdeleni', FORM_FIELD_NAME => 'KodTypuRozdeleni',
        ]);

        $rozdeleni = new ZastupyVyberRozdeleniField(RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_TYP_ROZDELENI), $this->prava, 'Rozdeleni', 'Zastup_rozdeleni',
            !$prirazeniZastupu, true, 5);
        $this->addField($rozdeleni);

        //pokud neni zadana kategorie rozdeleni, pro vyber pozice se pouzije nastaveny typ rozdeleni
        if ((RonPortalSettings::getValue(ZastupyController::TYP_KATEGORIE_POZICE) == '') ||
            //nebo se jedna o prirazeni zastupu a zastup byl pozadovan na pozici dle rozdeleni (v nastaveni jiz je ale pozadovana kategorie)
            ($prirazeniZastupu && $zastup->pozice)) {
            //pozice na kterou se zada zastup - dle nastaveni pro zastupy
            $this->addField([
                FORM_FIELD_TYPE => FORM_FIELD_TYPE_HIDDEN, FORM_FIELD_DEFAULT_VALUE => RonPortalSettings::getIntValue(ZastupyController::TYP_ROZDELENI_POZICE,-1),
                FORM_FIELD_SQL_NAME => 'KodTypuRozdeleni2', FORM_FIELD_NAME => 'KodTypuRozdeleni2',
            ]);

            $rozdeleni2 = new ZastupyVyberRozdeleniField(RonPortalSettings::getIntValue(ZastupyController::TYP_ROZDELENI_POZICE, -1), null, 'Rozdeleni2', 'Zastup_pozice',
                !$prirazeniZastupu, false, 7, RonPortalSettings::getValue(ZastupyController::POZICE_VYCET));
            $this->addField($rozdeleni2);
        }
        //v nastaveni byl zvolen typ kategorie pro vyber pozice, zobrazi se vyber kategorii
        else {
            $this->addField([
                FORM_FIELD_TYPE => FORM_FIELD_TYPE_HIDDEN, FORM_FIELD_DEFAULT_VALUE => RonPortalSettings::getValue(ZastupyController::TYP_KATEGORIE_POZICE),
                FORM_FIELD_SQL_NAME => 'TypKategorie', FORM_FIELD_NAME => 'TypKategorie',
            ]);

            $kategorie = new ZastupyVyberKategorieFiled(RonPortalSettings::getValue(ZastupyController::TYP_KATEGORIE_POZICE), 'KodKategorie', 'Zastup_kategorie_pozice',
                !$prirazeniZastupu, false, 19);
            $this->addField($kategorie);
        }

        if ($prirazeniZastupu) {
            //pokud je zadan typ rozdeleni pro filtrovani osob, zobrazi vyber rozdeleni
            if (RonPortalSettings::getIntValue(ZastupyController::TYP_ROZDELENI_FILTR) != 0) {
                $comboRozdeleni = new ZastupyFiltrRozdeleniField(RonPortalSettings::getIntValue(ZastupyController::TYP_ROZDELENI_FILTR), 'RozdeleniFiltr', 'Zastup_rozdeleni_filtr',
                    $rozdeleniFiltr, '?menuid=zastup&action=priradit&formaction=edit&record=' . $zastup->id . '&rozdeleni=');
                $this->addField($comboRozdeleni);
            }

            $jePreferovanaOsoba = false;

            //pokud neni zadana kategorie rozdeleni, pro vyber pozice se pouzije nastaveny typ rozdeleni
            if ((RonPortalSettings::getValue(ZastupyController::TYP_KATEGORIE_POZICE) == '') ||
                //nebo se jedna o prirazeni zastupu a zastup byl pozadovan na pozici dle rozdeleni (v nastaveni jiz je ale pozadovana kategorie)
                ($prirazeniZastupu && $zastup->pozice)) {
                $filtrPozice =
                    ' EXISTS(SELECT * FROM RozdeleniOsob ' .
                    '        WHERE RozdeleniOsob.RC = PracovniPomer.RC ' .
                    '          AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                    '          AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                    '          AND RozdeleniOsob.KodTypuRozdeleni = ' . (int)RonPortalSettings::getIntValue(ZastupyController::TYP_ROZDELENI_POZICE, -1) .
                    "          AND RozdeleniOsob.Rozdeleni = '" . $database->addSlashes($zastup->pozice) . "'" .
                    '          AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString($zastup->datum) .
                    '          AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString($zastup->datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';
            }
            //v nastaveni byl zvolen typ kategorie pro vyber pozice, zobrazi se vyber kategorii
            else {
                $filtrPozice =
                    ' EXISTS(SELECT * FROM RozdeleniOsob ' .
                    '        JOIN KategorieRozdeleni ON (KategorieRozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND KategorieRozdeleni.Rozdeleni = RozdeleniOsob.Rozdeleni)' .
                    '        WHERE RozdeleniOsob.RC = PracovniPomer.RC ' .
                    '          AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                    '          AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                    '          AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString($zastup->datum) .
                    '          AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString($zastup->datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .

                    '          AND KategorieRozdeleni.TypKategorie = ' . RonPortalSettings::getValue(ZastupyController::TYP_KATEGORIE_POZICE) .
                    "          AND KategorieRozdeleni.KodKategorie = '" . $database->addSlashes($zastup->kategoriePozice) . "'" .
                    ')';
            }

            $filtrRozdeleni = '';
            //filtrovani podle vybraneho rozdeleni
            if (RonPortalSettings::getIntValue(ZastupyController::TYP_ROZDELENI_FILTR) != 0) {
                $filtrRozdeleni =
                    ' EXISTS(SELECT * FROM RozdeleniOsob ' .
                    '        WHERE RozdeleniOsob.RC = PracovniPomer.RC ' .
                    '          AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                    '          AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                    '          AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(ZastupyController::TYP_ROZDELENI_FILTR) .
                    "          AND RozdeleniOsob.Rozdeleni = '" . $database->addSlashes($rozdeleniFiltr) . "'" .
                    '          AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString($zastup->datum) .
                    '          AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString($zastup->datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';
            }

            if (RonPortalSettings::getIntValue(ZastupyController::PREFEROVANY_TYP_ROZDELENI) != 0) {
                $osoba1 = new OsobaField(ronText('Zastup_preferovana_osoba'), 'zastupy_prideleni', time(), true);
                $osoba1->fieldName = 'PreferovanaOsoba';
                $osoba1->required = false;
                $osoba1->caption2 = ronText('Zastup_zapsat_preferovanou_osobu');
                $osoba1->rowId = 'zastup_osoba1';

                $filtr =
                    //21.7.2016 je mozne priradit i zamestnance, ktery jiz ma naplanovanou smenu, jine rozdeleni, pri vkladani se pak zobrazi hlaska ze je zaznam prepsan
                    /*'NOT EXISTS(SELECT * FROM PlanovaniKapacit ' .
                    '           WHERE PlanovaniKapacit.RC = PracovniPomer.RC ' .
                    '             AND PlanovaniKapacit.OsobniCislo = PracovniPomer.OsobniCislo ' .
                    '             AND PlanovaniKapacit.DatumNastupu = PracovniPomer.DatumNastupu ' .
                    '             AND PlanovaniKapacit.Datum = ' . $this->database->timestampToString($zastup->datum) .
                    ' )'.
                    'AND ' .*/
                    '    EXISTS(SELECT * FROM RozdeleniOsob ' .
                    '           WHERE RozdeleniOsob.RC = PracovniPomer.RC ' .
                    '             AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                    '             AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                    '             AND RozdeleniOsob.KodTypuRozdeleni = ' . (int)RonPortalSettings::getIntValue(ZastupyController::PREFEROVANY_TYP_ROZDELENI) .
                    "             AND RozdeleniOsob.Rozdeleni = '" . $this->database->addSlashes(RonPortalSettings::getValue(ZastupyController::PREFEROVANE_ROZDELENI)) . "'" .
                    '             AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString($zastup->datum) .
                    '             AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString($zastup->datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')' ;



                $osoba1->filtr = $filtr . ' AND ' . $filtrPozice . ($filtrRozdeleni ? ' AND ' . $filtrRozdeleni : '');

                $jePreferovanaOsoba = DBUtils::getSqlValue($this->database, 'SELECT Osoba.Prijmeni FROM Osoba JOIN PracovniPomer ON (Osoba.RC = PracovniPomer.RC) WHERE ' . $osoba1->filtr);

                if ($jePreferovanaOsoba) { //prida pole jen pokud nejaka osoba existuje
                    $this->addField($osoba1);
                }
            }

            $osoba = new OsobaField(ronText('Zastup_osoba'), 'zastupy_prideleni', time(), true);
            $osoba->required = false;
            if ($jePreferovanaOsoba) {//neni vybran preferovany typ, zobrazi se vzdy
                $osoba->rowId = 'zastup_osoba2'; //defaultne skryto, pokud se v predchozim vybere prazdny zaznam, je mozno vybirat zde
            }
            


                $filtr =

                //21.7.2016 je mozne priradit i zamestnance, ktery jiz ma naplanovanou smenu, jine rozdeleni, pri vkladani se pak zobrazi hlaska ze je zaznam prepsan
                /*'NOT EXISTS(SELECT * FROM PlanovaniKapacit ' .
                '           WHERE PlanovaniKapacit.RC = PracovniPomer.RC ' .
                '             AND PlanovaniKapacit.OsobniCislo = PracovniPomer.OsobniCislo ' .
                '             AND PlanovaniKapacit.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '             AND PlanovaniKapacit.Datum = ' . $this->database->timestampToString($zastup->datum) .
                ' )' .
                'AND ' .*/
                ''
            ;

            $osoba->filtr = $filtr . $filtrPozice. ($filtrRozdeleni ? ' AND ' . $filtrRozdeleni : '');
            MessageManager::addDebugLog($osoba->filtr);
            $this->addField($osoba);
            
            $this->addField([
                FORM_FIELD_TYPE => FORM_FIELD_TYPE_TEXT, 
                FORM_FIELD_TEXT => '<a id="zastup_kalendar_link" href="#" target="_blank" data-ronlink="?menuid=prehledsmen&obdobi=' . DateUtils::firstDayOfMonth($zastup->datum) .
                    '&osoba=">'. ronText('Zastup_zobraz_kalendar').'</a>',
            ]);
        }

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_CAPTION_TEXT => 'Zastup_duvod', FORM_FIELD_REQUIRED => false,
            FORM_FIELD_SQL_NAME => 'DuvodZadosti', FORM_FIELD_NAME => 'DuvodZadosti', FORM_FIELD_SQL_INDEX => 18, FORM_FIELD_EDIT => !$prirazeniZastupu,
            FORM_FIELD_SQL => 'SELECT Kod, Popis FROM DuvodZastupu ORDER BY Popis'
        ]);

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_EDIT, FORM_FIELD_CAPTION_TEXT => 'Zastup_poznamka', FORM_FIELD_REQUIRED => false,
            FORM_FIELD_SQL_NAME => 'Poznamka',  FORM_FIELD_NAME => 'Poznamka', FORM_FIELD_EDIT => true, FORM_FIELD_SQL_INDEX => 13,
        ]);


        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_ACTUAL_TIME, FORM_FIELD_SQL_NAME => 'Update_Timestamp',  FORM_FIELD_NAME => 'Update_Timestamp', FORM_FIELD_EDIT => true ]);
        $this->addField(new UserLoginField('Update_UserName', true));

        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_ACTUAL_TIME, FORM_FIELD_SQL_NAME => 'Insert_Timestamp',  FORM_FIELD_NAME => 'Insert_Timestamp', FORM_FIELD_EDIT => false ]);
        $this->addField(new UserLoginField('Insert_UserName', false));
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     */
    public function afterInsertRecord($rec)
    {
        $result = true;
        $viceDnu = false;

        if (isset($_POST['Datum_do']) && ($_POST['Datum_do'] != '')) {
            $datumDo = DateUtils::dateStringToTimestamp($_POST['Datum_do']);

            if (($datumDo != $rec['Datum']) && ($datumDo > $rec['Datum'])) {
                $viceDnu = true;
                
                $den = DateUtils::incDayStart($rec['Datum'], 1);

                $lzeSchvalit = $this->pravaSchvalit->jePravoNaRozdeleni($rec['KodTypuRozdeleni'], $rec['Rozdeleni'], $rec['Datum'], $datumDo);

                while ($den <= $datumDo) {
                    $dbRecord = new RonPortalDBRecord($this ->database, 'ZadostOZastup');
                    $dbRecord->addField('ID', 'int', true, 0, SQLUtils::newSerialNumber($this->database,'ZadostOZastup','ID'));
                    $dbRecord->addField('Datum', 'timestamp', true, 0, $den);
                    $dbRecord->addField('CasOd', 'time', true, 0, $rec['CasOd']);
                    $dbRecord->addField('CasDo', 'time', true, 0, $rec['CasDo']);
                    $dbRecord->addField('KodTypuRozdeleni', 'int', true, 0, $rec['KodTypuRozdeleni']);
                    $dbRecord->addField('Rozdeleni', '', true, 30, $rec['Rozdeleni']);
                    $dbRecord->addField('KodTypuRozdeleni2', 'int', true, 0, $rec['KodTypuRozdeleni2']);
                    $dbRecord->addField('Rozdeleni2', '', true, 30, $rec['Rozdeleni2']);
                    $dbRecord->addField('Poznamka', '', true, 250, $rec['Poznamka']);

                    $dbRecord->addField('TypKategorie', '', true, 30, $rec['TypKategorie']);
                    $dbRecord->addField('KodKategorie', '', true, 30, $rec['KodKategorie']);

                    //pokud lze schvalit v celem vkladanem obdobi, schvali kazdy zaznam
                    if ($lzeSchvalit) {
                        $dbRecord->addField('TypSchvaleni', 'int', true, 0, 1);
                        $dbRecord->addField('Schvaleni', 'timestamp', true, 0, time());
                        $dbRecord->addField('Schvalil', '', false, 0, RonPortalUtils::getLogUserName());
                    }

                    $dbRecord->addField('DuvodZadosti', '', true, '10', $rec['DuvodZadosti']);

                    $dbRecord->addField('Insert_UserName', '', true, 25, $rec['Insert_UserName']);
                    $dbRecord->addField('Insert_Timestamp', 'timestamp', true, 0, $rec['Insert_Timestamp']);
                    $dbRecord->addField('Update_UserName', '', true, 25, $rec['Update_UserName']);
                    $dbRecord->addField('Update_Timestamp', 'timestamp', true, 0, $rec['Update_Timestamp']);

                    $result = $result && $dbRecord->insertRecord();

                    $den = DateUtils::incDayStart($den, 1);
                }
            }
        }

        if (!$viceDnu) {
            $this->id = $rec['ID'];
        }
        
        $result = $result && $rec->executeXmlEvents(RonPortalXMLEvents::ZASTUP_AFTER_POST, 'insert', null, false);
        
        return $result;
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     */
    function afterUpdateRecord($rec)
    {
        $this->id = $rec['ID'];

        //ve formulari se prirazuje zastup, zapise se do planovani kapacit
        if ($this->prirazeniZastupu && $rec['RC']) {
            try {
                if (!Zastup::vlozPlanovaniKapacit($this->database, $rec)) {
                    return false;
                }
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e);
            }
        }
        
        return $rec->executeXmlEvents(RonPortalXMLEvents::ZASTUP_AFTER_POST, 'update', null, false); 
    }
    
    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     */
    function beforeInsertRecord($rec)
    {
        $result = $rec->executeXmlEvents(RonPortalXMLEvents::ZASTUP_BEFORE_POST, 'insert', null, false);

        if ($this->pravaSchvalit->jePravoNaRozdeleni($rec['KodTypuRozdeleni'],$rec['Rozdeleni'],$rec['Datum'],$rec['Datum'])) {
            $rec->addField('TypSchvaleni', 'int', true, 0, 1);
            $rec->addField('Schvaleni', 'datetime', true, 0, time());
            $rec->addField('Schvalil', '', true, 25, RonPortalUtils::getLogUserName());
        }

        return $result;
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     */
    function beforeUpdateRecord($rec)
    {
        if (isset($_POST['PreferovanaOsoba']) || isset($_POST['osoba'])) {
            $osobaID = $_POST['PreferovanaOsoba'] ?: $_POST['osoba'];

            if ($osobaID == '') {
                //není vybrána žádná osoba a formulář je v režimu přiřazení osoby, vypíše se chyba
                MessageManager::addErrorMessage(ronText('Zastup_nevybrana_osoba'));
                return false;
            }

            if (!$this->prava->lzeZobrazitPP($osobaID)) {
                RonPortalUtils::neniPravoException();
            }

            require_once('adsapp/ppomer.php');

            $pracovniPomer = new PracovniPomer();
            $pracovniPomer->nactiPPPodleId($this->database,$osobaID);

            $rec->addField('RC', '', false, 15, $pracovniPomer->rc);
            $rec->addField('OsobniCislo', '', false, 15, $pracovniPomer->osobniCislo);
            $rec->addField('DatumNastupu', 'timestamp', false, 0, $pracovniPomer->datumNastupu);

        }

        return $rec->executeXmlEvents(RonPortalXMLEvents::ZASTUP_BEFORE_POST, 'update', null, false);
    }

    /**
     * @return DBRecord|RonPortalDBRecord
     * @throws Exception
     */
    function createDBRecord()
    {
        $rec = parent::createDBRecord();

        $basedate = $rec['Datum'];

        if (($_POST['caszastupuod'] != '') || ($_POST['CasOd'] != '')) {
            $rec->addField('CasOd', 'time', false, 0, $_POST['caszastupuod'] ? DateUtils::timeStringToTimestamp($_POST['caszastupuod'], $basedate) : $_POST['CasOd']);
        }
        if (($_POST['caszastupudo'] != '') || ($_POST['CasDo'] != '')) {
            $rec->addField('CasDo', 'time', false, 0, $_POST['caszastupudo'] ? DateUtils::timeStringToTimestamp($_POST['caszastupudo'], $basedate) : $_POST['CasDo']);
        }

        return $rec;
    }

    function showCommonFields()
    {
        //datum pro pouziti v javascriptu
        if ($this->prirazeniZastupu) {
            echo "<input type='hidden' name='DatumZastupu' value='" . $this->zastup->datum . "'>";
            echo "<input type='hidden' name='CasOd' value='" . $this->zastup->casOd . "'>";
            echo "<input type='hidden' name='CasDo' value='" . $this->zastup->casDo . "'>";
        }
        parent::showCommonFields();
    }


}