<?php
/**
 * stanislav.elbl, 07.11.2019
 */

namespace Ron\Kapacity;

use Ron\Controls\LinksList;
use Ron\RonPortal\NastaveniUzivatele;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\User;

require_once 'adsapp/ronportalfilter.php';
require_once 'kapacity/kapacityutils.php';
require_once 'adsapp/usersettings.php';


class KapacityFilterForm extends RonPortalFilterForm
{
    const REZIM_UPRAV = 'kapacity_rezimuprav';

    var $zobrazitLinkSkutecneAbsence;

    /**
     * KapacityFilterFrom constructor.
     * @param \Ron\DB\Database $database
     * @param string $action
     */
    public function __construct($database, $action = 'kapacity')
    {
        parent::__construct($database, $action, true);  // useGet = true

        // nastaveni rozdeleni je pres if ve dvou vetvich - jinak si neprenasi vybrane rozdeleni do jinych seznamu (napr. do zpracovane dochazky)
        $typRozdeleni = KapacityUtils::typRozdeleniProVyber();
        if ($typRozdeleni != RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) {
            $this->f_pravarozdeleni = Prava::vratPrava('kapacity');
            $this->nastavRozdeleni(true, $typRozdeleni, 0, '', true);
        } else {
            $this->nastavRozdeleni(true);
            $this->f_pravarozdeleni = Prava::vratPrava('kapacity'); // aby se filtroval seznam rozdeleni
        }
    }


    function nastavProPlanovaniObdobi()
    {
        $this ->nastavObdobi(true, 'ADS', false);       // jenOtevrene = false

        $this ->zobrazitLinkSkutecneAbsence =
            $this ->getAction == 'kapacity'  &&
            NastaveniUzivatele::getUserSettingsItem($this ->database, self::REZIM_UPRAV, RonPortalSettings::getIntValue('kapacity_rezim')) < 3;

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
            $this ->nastavRozdeleni2(true);
        }
    }

    function renderRezimZobrazeni()
    {
        require_once('ron/controls.php');

        if (RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_POVOLIT_REZIM_UPRAV)) {
            $data = [
                0 => \Ron\ronText('Puvodni_plan'),
                1 => \Ron\ronText('Upraveny_plan'),
                2 => \Ron\ronText('Kombinovany_plan'),
                3 => \Ron\ronText('Kapacity_dle_smen')
            ];
        } else {
            $data = [
                0 => \Ron\ronText('Puvodni_plan'),
                3 => \Ron\ronText('Kapacity_dle_smen')
            ];
        }

        if (RonPortalSettings::getIntValue('kapacity_povolitsloty')) {
            $data[4] = \Ron\ronText('Prehled_podle_slotu');
        }

        if (RonPortalSettings::getIntValue('planovanipodminek')) {
            $data[5] = \Ron\ronText('Planovani_podminek');
        }

        echo '<div class="filterrow" id="filter_urovenschvaleni">';
        echo '<span class="filtercaption"></span>';
        LinksList::renderLinks($data,
            NastaveniUzivatele::getUserSettingsItem($this ->database, self::REZIM_UPRAV, RonPortalSettings::getIntValue('kapacity_rezim')),
            '?menuid=kapacity&rezim=', '', false);
        echo "</div>\n";
    }


    function renderSkutecneAbsence()
    {
        echo '<div class="filterrow" id="filter_urovenschvaleni">';
        echo '<span class="filtercaption"></span>';

        if (KapacityController::getZobrazitSkuteneAbsence()) {
            echo "<a href='?menuid=kapacity&skutecneabsence=0'>" . \Ron\ronText('Skryt_skutecne_absence') . "</a>";
        } else {
            echo "<a href='?menuid=kapacity&skutecneabsence=1'>" . \Ron\ronText('Zobrazit_skutecne_absence') . "</a>";
        }

        echo "</div>\n";
    }


    function renderFilters()
    {
        parent::renderFilters();

        if (User::isAuthenticated()) {
            $this->renderRezimZobrazeni(); // prepinani zobrazeni
        }

        if ($this ->zobrazitLinkSkutecneAbsence) {
            $this ->renderSkutecneAbsence();
        }
    }

}
