<?php
/**
 * UTF8
 * stanislav.elbl, 16.07.2015
 * Planovani kapacit
 */


namespace Ron\Kapacity;

use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\Http\HtmlHeaders;
use Ron\Http\HttpResponse;
use Ron\MessageManager;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\RonPortal\NastaveniUzivatele;
use Ron\RonPortal\NeniPravoException;
use Ron\RonPortal\Obdobi;
use Ron\RonPortal\Prava;
use Ron\RonPortal\ReportOutputObject;
use Ron\RonPortal\RonPortalReport;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\Rozdeleni;
use Ron\RonPortal\TypRozdeleni;
use Ron\RonPortal\User;
use Ron\StringUtils;
use Ron\TextManager;

class KapacityController extends HttpController
{

    var $database;

    function __construct($database)
    {
        $this->database = $database;
    }


    public static function getZobrazitSkuteneAbsence()
    {
        return $_SESSION['skutecneabsence']; // nacteno z cookie
    }


    function report($utf8)
    {
        require_once('kapacity/kapacityutils.php');
        require_once('adsapp/reportserver.php');

        $result = new ReportOutputObject();
        $result->loadReportFromGet($this ->database, ['PrehledKapacit', 'PlanovaniKapacit'], $utf8, 'PlanovaniKapacit'); // treti parametr z duvodu kompatibility

        $kodRozdeleni = $utf8 ? StringUtils::fromUTF8($_GET['rozdeleni']) : $_GET['rozdeleni'];
        $kodRozdeleni2 = $utf8 ? StringUtils::fromUTF8($_GET['rozdeleni2']) : $_GET['rozdeleni2'];

        if ($kodRozdeleni != '') {
            require_once 'adsapp/rozdeleni.php';
            $rozdeleni = Rozdeleni::loadRozdeleni($this ->database, KapacityUtils::typRozdeleniProVyber(), $kodRozdeleni);
            $result ->nastavRozdeleni(true, $rozdeleni, false); // vyresit vcetne podrizenych
        }

        if ($kodRozdeleni2 != '') {
            require_once 'adsapp/rozdeleni.php';
            $rozdeleni = Rozdeleni::loadRozdeleni($this ->database, RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), $kodRozdeleni2);
            $result ->nastavRozdeleni2(true, $rozdeleni, false); // vyresit vcetne podrizenych
        }

        if (isset($_GET['idpp'])) {
            $result->stringParams['IdPP'] = $_GET['idpp'];
        }

        require_once 'adsapp/obdobi.php';
        $obdobi = Obdobi::vratObdobi($this ->database, 'ADS', DateUtils::urlStringToDate($_GET['obdobi']));
        $result ->nastavObdobi(0, $obdobi ->zacatek, $obdobi ->konec, $obdobi ->popis);

        if (isset($_GET['den'])) {
            $result->dateParams['den'] = $_GET['den'];
        }

        return $result;
    }


    /**
     * @param int $pocatekObdobi
     * @return SchvaleniPodminekPlanovani
     */
    private function nactiSchvaleniPoctu($pocatekObdobi)
    {
        // overi, jestli uz jsou schvalene podminky planovani nastavenym levelem
        $rozdeleni = $_GET['rozdeleni'] ?: RonPortalSession::rozdeleni();

        require_once 'kapacity/kapacityutils.php';
        require_once 'kapacity/schvaleni_podminek.php';

        return SchvaleniPodminekPlanovani::nactiSchvaleni($this->database, $pocatekObdobi, KapacityUtils::typRozdeleniProVyber(), $rozdeleni);
    }

    /**
     * @param int $pocatekObdobi
     * @return SchvaleniKapacitRozdeleni
     */
    private function nactiSchvaleniRozdeleni($pocatekObdobi)
    {
        $rozdeleni = $_GET['rozdeleni'] ?: RonPortalSession::rozdeleni();

        require_once 'kapacity/kapacityutils.php';
        require_once 'kapacity/schvaleni_kapacit.php';

        return SchvaleniKapacitRozdeleni::nactiSchvaleni($this ->database, $pocatekObdobi, KapacityUtils::typRozdeleniProVyber(), $rozdeleni);
    }


    /**
     * Zobrazi aktivni planovani kapacit
     * @param bool $planovaniPoSlotech
     * @param bool $rezimUprav
     * @return string
     * @throws \Exception
     */
    function aktivniPlanovani($planovaniPoSlotech, $rezimUprav)
    {
        $vyzadovanoSchvaleniPoctu = Prava::vratPrava('kapacity_edit')->priznak2;

        $pocatekObdobi = $_GET['obdobi'] ? DateUtils::urlStringToDate($_GET['obdobi']) : RonPortalSession::obdobi();

        if ($vyzadovanoSchvaleniPoctu) {
            // overi, jestli uz jsou schvalene podminky planovani nastavenym levelem
            $schvaleniPoctu = $this ->nactiSchvaleniPoctu($pocatekObdobi);

            if (!$schvaleniPoctu->lzePlanovatDen(DateUtils::lastDayOfMonth($schvaleniPoctu ->pocatekObdobi))) {
                throw new \Exception(\Ron\ronText('Pocty_zamestnancu_jeste_nejsou_schvaleny'));
            }
        }

        if (!isset($rezimUprav)  &&  RonPortalSettings::getIntValue('kapacity_rezimuprav')  &&  RonPortalSettings::getIntValue('kapacity_minschvaleniproupravy')) {
            // nastavi rezim automaticky podle toho, jestli je uz schvaleno
            $schvaleniKapacit = $this ->nactiSchvaleniRozdeleni($pocatekObdobi);

            if ($schvaleniKapacit ->getSchvaleniPuvodnihoPlanu() >= DateUtils::lastDayOfMonth($pocatekObdobi)) {
                $rezimUprav = true;
            }
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_TYP_ROZDELENI2)) {
            require_once('adsapp/rozdeleni.php');
            $typRozdeleni2 = TypRozdeleni::getTypRozdeleni($this->database, RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_TYP_ROZDELENI2));
            Model::addData('typRozdeleni2', $typRozdeleni2);
        }

        // sestavy -
        require_once('adsapp/reportutils.php');
        $reportList = RonPortalReport::loadFormReports($this->database, ['PlanovaniKapacit', 'PrehledKapacit'], 'PlanovaniKapacit'); // treti parametr z duvodu kompatibility
        Model::addData('reportList', $reportList);

        require_once('adsapp/usersettings.php');
        $vychoziTypDokumentu = NastaveniUzivatele::getUserSettingsItem($this->database, 'pk_typdokumentu', 0);

        require_once('ron/controls.php');
        $combo = new ComboBox('typ');
        $combo->dataTypeComparison = false;
        $combo->className = 'form-input planovani_detailkod';
        $combo->htmlParams = ['id' => 'planovani_detail_typ_dokumentu'];
        $combo->loadData($this->database, 'SELECT Kod, Popis FROM TypDokumentu WHERE ZobrazitNaIntrawebu = 1 ORDER BY Kod', $vychoziTypDokumentu);
        Model::addData('typDokumentuComboBox', $combo);

        require_once('kapacity/kapacityutils.php');
        Model::addData('sirkaBunky', KapacityUtils::sirkaBunky($this->database));
        Model::addData('vyskaBunky', KapacityUtils::vyskaBunky($this->database));
        Model::addData('provadetKontroly', KapacityUtils::pouzivatKontrolyPlanovani());
        Model::addData('database', $this->database);

        Model::addData('planovaniPoSlotech', $planovaniPoSlotech);
        Model::addData('rezimUprav', $rezimUprav);

        // aktivni rezim
        return 'kapacity/view_planovani.php';
    }


    /**
     * @return null|Redirect
     */
    public function ulozitSchvaleni($menuId)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_EDITACE)) {
            return null;
        }

        require_once('kapacity/f_schvaleni.php');
        try {
            SchvaleniKapacitForm::provedSchvaleni($this->database);
            //$this->data->provedSchvaleni();

            return new Redirect(null, 'menuid=' . $menuId);
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        return null;
    }


    public function planovaniPodminek()
    {
        return new Redirect(null, 'menuid=planovanipodminek');
    }

    /**
     * @param string $action
     * @return \Ron\MVC\InternalRedirect|\Ron\MVC\Redirect|string
     * @throws NeniPravoException
     */
    public function handleAction($action)
    {
        require_once('adsapp/usersettings.php');
        //require_once('kapacity/kapacity_filter.php');

        if (isset($_GET['rezim'])) {
            NastaveniUzivatele::setUserSettingsItem($this->database, 'kapacity_rezimuprav', (int)$_GET['rezim'],
                RonPortalSettings::getIntValue('kapacity_rezim'));
        }

        $rezim = NastaveniUzivatele::getUserSettingsItem($this ->database, 'kapacity_rezimuprav', RonPortalSettings::getIntValue('kapacity_rezim'));

        if ($rezim == 5) {
            return $this ->planovaniPodminek();
        }


        TextManager::getInstance() ->loadStrings('kapacity.ini');

        if (!User::isAuthenticated()) {
            if (!RonPortalSettings::getIntValue('kapacitydlesmen_bezprihlaseni')  ||   $action != 'kapacitydlesmen') {
                throw new NeniPravoException();
            }
        }

        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'translations' :
                    return $this ->translations();

                case 'report' :
                    return $this ->report(false);
                case 'reporta' :    // reporty v aktivnim rezimu - url generuje javascript, takze nektere hodnoty jsou v utf8
                    return $this ->report(true);
            }
        }

        if (isset($_POST['ulozitschvaleni'])) {
            $result = $this ->ulozitSchvaleni($action);
            if ($result) {
                return $result;
            }
        }

        HtmlHeaders::getInstance() ->addScript('', '?menuid=kapacity&action=translations&ver=' . HtmlHeaders::getInstance() ->versionId, 'translations');
        HtmlHeaders::getInstance() ->addCss('css/s_kapacity.css', '?ver=' . HtmlHeaders::getInstance() ->versionId);

        if (isset($_GET['skutecneabsence'])) {
            HttpResponse::setCookie('ads_skutecneabsence', (int)$_GET['skutecneabsence'], time() + 30000000);
            $_SESSION['skutecneabsence'] = (int)$_GET['skutecneabsence'];
        } else {
            $_SESSION['skutecneabsence'] = (int)$_COOKIE['ads_skutecneabsence'];
        }

        HtmlHeaders::getInstance() ->setTitle(\Ron\ronText('RON_Portal') . ' - ' . \Ron\ronText('Planovani_kapacit'));

        try {
            $rezimUprav = null;

            if (isset($_GET['rezimUprav'])) {
                $rezimUprav = (int)$_GET['rezimUprav'];
            }

            switch ($action) {
                case 'planovanikapacit':
                    return $this->aktivniPlanovani(false, $rezimUprav);

                case 'planovanislotu':
                    return $this->aktivniPlanovani(true, $rezimUprav);
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        HtmlHeaders::getInstance() ->addScript('script/kapacity_html.js');

        if ($action == 'kapacitydlesmen'  ||  $rezim == 3) {
            // prehled podle smen
            require_once('kapacity/prehled_podlesmen.php');
            $viewObject = new KapacityDleSmenView($this ->database);
            $viewObject ->load();

            if (User::isAuthenticated()) {
                return $viewObject;
            } else {
                Model::addViewObject($viewObject);
                return 'view/appview_nouser.php';
            }
        }

        // pasivni zobrazeni
        HttpResponse::setCookieAndSession('kapacity_sort', 'kapacity_sort', false, true); // GET

        require_once('kapacity/prehled_kapacit.php');
        $viewObject = new PlanovaniKapacitView($this ->database);
        $viewObject ->load();

        return  $viewObject;
    }

}
