<?php
/*
 * UTF8
 * Funkce pro planovani kapacit - pouzivaji se ve standardnim rezimu zobrazeni
 * i v rozhrani pro javascript pro edici
 */

namespace Ron\Kapacity;


use Ron\DateUtils;
use Ron\RonPortal\MesicniVysledek;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RozdeleniOsob;
use Ron\RonPortal\Smena;
use Ron\RonPortal\SmenaProDen;
use Ron\RonPortal\User;
use Ron\DB\Database;
use Ron\StringUtils;

class KapacityUtils
{
    const SETTINGS_SIRKA = 'kapacity_sirka';
    const SETTINGS_VYSKA = 'kapacity_vyska';
    const SETTINGS_EDITACE_SCHVALENYCH = 'editaceschvalenychkapacit';
    const SETTINGS_ZAKAZAT_SNIZIT_SCHVALENI = 'kapacity_zakazatsnizitschvaleni'; // jestli muze snizit datum schvaleni pro svou nejvyssi povolenou uroven
    const SETTINGS_ZPUSOB_SCHVALOVANI = 'kapacity_zpusobschvalovani';   // jestli se schvaluje cele rozdeleni najednou
    const SETTINGS_MIN_SCHVALENI_PRO_UPRAVY = 'kapacity_minschvaleniproupravy';
    const SETTINGS_TLACITKO_PREPOCET = 'kapacity_tlacitkoprepocet';
    const SETTINGS_POVOLIT_REZIM_UPRAV = 'kapacity_rezimuprav';
    //const SETTINGS_POVOLIT_PODRIZENA_ROZDELENI = 'kapacity_povolitpodrizenarozdeleni'; // z nastaveni aplikace

    /**
     * Vraci typ rozdeleni, ktery bude ve vyberu
     * @return int
     */
    public static function typRozdeleniProVyber()
    {
        $result = RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_TYP_ROZDELENI);
        if ($result != 0) {
            // typ rozdeleni pro predavani na strediscich je nastaven
            return $result;
        }

        return RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP);
    }


    public static function sirkaBunky($database)
    {
        // zatim vypnuto - neni dodelano - defaultne by melo byt prazdne - otestovat
//        require_once('adsapp/usersettings.php');
//        $userSirka = NastaveniUzivatele::getUserSettingsItem($database, 'pk_sirkabunky');
//        return ($userSirka && (int)$userSirka >= 10)
//            ? (int)$userSirka
//            : RonPortalSettings::getIntValue(self::SETTINGS_SIRKA);
        return RonPortalSettings::getIntValue(self::SETTINGS_SIRKA) ?: 35;
    }


    public static function vyskaBunky($database)
    {
//        require_once('adsapp/usersettings.php');
//        $userVyska = NastaveniUzivatele::getUserSettingsItem($database, 'pk_vyskabunky');
//        return ($userVyska && (int)$userVyska >= 10)
//            ? (int)$userVyska
//            : RonPortalSettings::getIntValue(self::SETTINGS_VYSKA);
        return RonPortalSettings::getIntValue(self::SETTINGS_VYSKA) ?: 35;
    }


    public static function pouzivatKontrolyPlanovani()
    {
        return (int)RonPortalSettings::getInstance()->getIntValue('kapacity_kontrolovat_podminky');
    }

    /**
     * @param Database $database
     * @param int $typRozdeleni
     * @return string  Vraci filtr pro zadany typ rozdeleni podle prav uzivatele. Pokud jsou prava nastavena podle jineho typu rozdeleni, tak se nefiltruje!
     */
    public static function filtrRozdeleniProPlanovani($database, $typRozdeleni)
    {
        $filtr = '';

        // prida filtr podle prav
        $pravazobrazeni = Prava::vratPrava('kapacity');
        $pravaeditace = Prava::vratPrava('kapacity_edit');

        // VSE - vraci prazdny filtr
        if ($pravazobrazeni->prava == 3 || $pravaeditace->prava == 3) return '';


        // VLASTNI ROZDELENI VCETNE PODRIZENYCH
        $typr = null;

        if ($pravazobrazeni->prava >= 12 && $pravazobrazeni->prava <= 14 && $typRozdeleni == $pravazobrazeni->typRozdeleni) $typr = $pravazobrazeni->typRozdeleni;
        else if ($pravaeditace->prava >= 12 && $pravaeditace->prava <= 14 && $typRozdeleni == $pravaeditace->typRozdeleni) $typr = $pravaeditace->typRozdeleni;

        if ($typr) {
            // zobrazeni nebo editace nastaveno na "vlastni a podrizena" nebo podobne
            return
                'EXISTS (' .
                'SELECT * FROM RozdeleniOsob RU, HierarchieRozdeleni H ' .
                ' WHERE RU.KodTypuRozdeleni = ' . (int)$typr .
                "   AND RU.RC = '" . $database->addSlashes(User::getRC()) . "' " .
                '   AND (RU.PlatnostOd <= ' . $database->timestampToString(RonPortalSession::konecObdobi()) . ')' .
                '   AND (RU.PlatnostDo >= ' . $database->timestampToString(RonPortalSession::obdobi()) . ' OR RU.PlatnostDo IS NULL) ' .
                '   AND H.TypNadrazene = RU.KodTypuRozdeleni ' .
                '   AND H.KodNadrazene = RU.Rozdeleni ' .
                '   AND H.KodPodrizene = Rozdeleni.Kod ' .
                ')';
        }


        // VLASTNI ROZDELENI (BEZ PODRIZENYCH)
        if ($pravazobrazeni->prava == 10 || $pravazobrazeni->prava == 11 && $typRozdeleni == $pravazobrazeni->typRozdeleni) $typr = $pravazobrazeni->typRozdeleni;
        else if ($pravaeditace->prava == 10 || $pravaeditace->prava == 11 && $typRozdeleni == $pravaeditace->typRozdeleni) $typr = $pravaeditace->typRozdeleni;

        if ($typr) {
            // jen vlastni (muze jich byt i vice)
            return
                'EXISTS (' .
                'SELECT * FROM RozdeleniOsob RU ' .
                ' WHERE RU.KodTypuRozdeleni = ' . (int)$typr .
                "   AND RU.RC = '" . $database->addSlashes(User::getRC()) . "' " .
                '   AND (RU.PlatnostOd <= ' . $database->timestampToString(RonPortalSession::konecObdobi()) . ')' .
                '   AND (RU.PlatnostDo >= ' . $database->timestampToString(RonPortalSession::obdobi()) . ' OR RU.PlatnostDo IS NULL) ' .
                '   AND RU.Rozdeleni  = Rozdeleni.Kod ' .
                ')';
        }

        /*
        - nastaveni podle pristupovych skupin - skupiny nefiltruje, protoze vubec nemusi byt stejne jako rozdeleni
        - navic pak nejdou planovat "vybrane osoby", protoze nemusi byt v prislusnem rozdeleni

        // prida skupiny pro zobrazovani
        if ($pravazobrazeni -> f_prava == 2  ||  $pravazobrazeni -> f_prava == 4) // vybrane osoby a skupiny
        {
        $poma = explode(';', $pravazobrazeni -> f_skupiny);

        if ($poma)
          foreach ($poma as $skupina)
          {
            if ($filtr) $filtr .= ' OR ';
            $filtr .= "HierarchieRozdeleni.KodNadrazene = '$skupina'";
          }
        }

        // prida skupiny pro editaci
        if ($pravaeditace -> f_prava == 2  ||  $pravaeditace -> f_prava == 4) // vybrane osoby a skupiny
        {
        $poma = explode(';', $pravaeditace -> f_skupiny);
        foreach ($poma as $skupina)
        {
          if ($skupina)
          {
            if ($filtr) $filtr .= ' OR ';
            $filtr .= "HierarchieRozdeleni.KodNadrazene = '$skupina'";
          }
        }
        }
        */
        if ($filtr) {
            $filtr =
                'EXISTS (SELECT * FROM HierarchieRozdeleni WHERE' .
                ' HierarchieRozdeleni.TypNadrazene = Rozdeleni.KodTypuRozdeleni' .
                " AND ($filtr)" . // kod nadrazene - podle seznamu
                ' AND HierarchieRozdeleni.TypPodrizene = Rozdeleni.KodTypuRozdeleni ' .
                ' AND HierarchieRozdeleni.KodPodrizene = Rozdeleni.Kod' .
                ')';
        } else {
            // - $filtr = '(0 = 1)';
            // jinak vidi ve filtru vsechna rozdeleni a nejak se s tim musi poprat
        }

        return $filtr;
    }


    /**
     * Zkontroluje, jestli nebude po naplanovani smeny v zadane delce v intervalu $denOd-$denDo prekrocen maximalni povoleny pocet hodin.
     * (ten zjisti z mesicniho vysledku)
     * Interval muze presahovat do dalsiho mesice - pak kontroluje kazde obdobi zvlast
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $denOd Datum, od ktereho se planuje nova smena
     * @param int $denDo Datum, do ktereho se planuje nova smena
     * @param $delkaNoveSmeny
     * @param $rezimUprav
     * @throws \Exception
     */
    public static function zkontrolujMaximalniPocetHodin($database, $pp, $denOd, $denDo, $delkaNoveSmeny, $rezimUprav)
    {
        if (!RonPortalSettings::getValue('kapacity_cs_soucet')) return;

        require_once('dochazka/mesicnivysledek.php');
        require_once('kapacity/planovanikapacitdata.php');

        $obdobiOd = DateUtils::firstDayOfMonth($denOd);
        $obdobiDo = DateUtils::lastDayOfMonth($denDo);

        $smeny = PlanovaniKapacit_PlanovanaSmena::nactiSmeny($database, $pp->idPP, $obdobiOd, $obdobiDo, $rezimUprav);
        //throw new \Exception(count($smeny));
        $index = 0;
        while ($obdobiOd < $denDo) {
            $aktualniObdobiDo = DateUtils::lastDayOfMonth($obdobiOd);

            // spocita soucet smen v aktualnim obdobi ($obdobiOd)
            $soucet = 0;
            while ($index < count($smeny) && $smeny[$index]->datum <= $aktualniObdobiDo) {
                if ($smeny[$index]->datum < $denOd || $smeny[$index]->datum > $denDo) { // nepocita smeny, ktere spadaji do planovaneho intervalu
                    $soucet += $smeny[$index]->delka / 3600;
                }

                $index++;
            }

            $mesicniVysledek = MesicniVysledek::vratMesicniVysledek($database, $pp->idPP, $obdobiOd, RonPortalSettings::getValue('kapacity_cs_soucet'));

            if ($mesicniVysledek && $mesicniVysledek->hodiny) {
                $datumOd = ($obdobiOd < $denOd) ? $denOd : $obdobiOd;
                $datumDo = ($aktualniObdobiDo > $denDo) ? $denDo : $aktualniObdobiDo;

                // pricte nove naplanovane hodiny v aktualnim obdobi
                $soucet += $delkaNoveSmeny / 3600 * (DateUtils::daysBetween($datumOd, $datumDo) + 1);

                // a overi, jestli neni prekrocen limit
                if ($soucet > $mesicniVysledek->hodiny) {
                    throw new \Exception(\Ron\ronText('Prekrocen_povoleny_limit_planovanych_hodin'));
                }
            }

            $obdobiOd = DateUtils::incMonth($obdobiOd);
        }
    }


    /**
     * Zkontroluje, jestli zadany den jeste neni schvaleny a je mozne jej editovat.
     * V rezimu uprav navic zkontroluje, jestli je puvodni plan schvalen pozadovanou urovni
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $den
     * @param string $kodRozdeleni
     * @return SchvaleniKapacit
     * @throws \Exception
     */
    public static function vratSchvaleni($database, $pp, $den, $kodRozdeleni = null)
    {
        require_once('kapacity/schvaleni_kapacit.php');
        if (RonPortalSettings::getIntValue(self::SETTINGS_ZPUSOB_SCHVALOVANI)) {
            require_once 'adsapp/rozdeleniosob.php';
            $typRozdeleni = self::typRozdeleniProVyber();

            if (!$kodRozdeleni) {
                $rozdeleniOsoby = RozdeleniOsob::nactiRozdeleniOsoby($database, $pp, $typRozdeleni, $den, $den);

                if (!$rozdeleniOsoby) {
                    // nemelo by nastat - zamestnanec neni v zadnem rozdeleni
                    return null;
                }

                $kodRozdeleni = $rozdeleniOsoby[0] ->kod;
            }

            // TODO: zkontrolovat vsechna rozdeleni a vracet max? Asi neni treba - zamestnanec by mel mit jen jedno.
            return SchvaleniKapacitRozdeleni::nactiSchvaleni($database, DateUtils::firstDayOfMonth($den), $typRozdeleni, $kodRozdeleni);
        } else {
            return SchvaleniKapacitPP::nactiSchvaleni($database, DateUtils::firstDayOfMonth($den), $pp->idPP);
        }
    }


    /**
     * Zkontroluje, jestli zadany den jeste neni schvaleny a je mozne jej editovat.
     * V rezimu uprav navic zkontroluje, jestli je puvodni plan schvalen pozadovanou urovni
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $den
     * @param bool $rezimUprav
     * @param string $kodRozdeleni
     * @throws \Exception
     */
    public static function zkontrolujSchvaleni($database, $pp, $den, $rezimUprav, $kodRozdeleni = null)
    {
        // nacte schvaleni - podle nastaveni bud pro PP nebo rozdeleni
        $schvaleni = self::vratSchvaleni($database, $pp, $den, $kodRozdeleni);

        if ($schvaleni ->lzeEditovatKapacity($den, $rezimUprav)) return;

        if ($rezimUprav && RonPortalSettings::getIntValue(self::SETTINGS_MIN_SCHVALENI_PRO_UPRAVY)) {
            if ($schvaleni->getSchvaleniPuvodnihoPlanu() < $den) {
                // nelze editovat v rezimu uprav - puvodni plan jeste neni schvalen
                throw new \Exception(\Ron\ronText('Puvodni_plan_jeste_neni_schvalen'));
            }
        }

        throw new \Exception( \Ron\ronText('Plan_jiz_je_schvalen') );
    }


    /**
     * Zkontroluje, jestli zadany den jeste neni schvaleny a je mozne jej editovat.
     * V rezimu uprav navic zkontroluje, jestli je puvodni plan schvalen pozadovanou urovni
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $rezimUprav
     * @return SchvaleniKapacitRozdeleni|void
     * @throws \Exception
     */
    public static function zkontrolujSchvaleniProObdobi($database, $pp, $datumOd, $datumDo, $rezimUprav)   {
        require_once('kapacity/schvaleni_kapacit.php');
        if (RonPortalSettings::getIntValue(self::SETTINGS_ZPUSOB_SCHVALOVANI)) {
            // schvaluje se po rozdelenich - musim pro kazdy den zkontrolovat schvaleni rozdeleni, na kterem je zamestnanec
            $params = $database ->createParameters();
            $sql =
                'SELECT Rozdeleni, Datum' .
                ' FROM PlanovaniKapacit' .
                ' WHERE RC = ' . $params ->addParameter($pp ->rc) .
                '   AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
                '   AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
                '   AND Datum >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
                '   AND Datum <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
                ' ORDER BY Datum';

            $cursor = $database ->getSqlResult($sql, $params);
            try {
                $zkontrolovanaRozdeleni = []; // pamatuje si rozdeleni, ktera uz kontroloval

                $row = $cursor ->fetch();
                $datumZaznamu = $row ? $database ->stringToTimestamp($row[1]) : null;

                for ($datum = $datumOd;  $datum <= $datumDo;  $datum = DateUtils::incDayStart($datum)) {
                    $rozdeleni = null;
                    if ($row  &&  $datumZaznamu == $datum) {
                        $rozdeleni = $row[0];
                        $row = $cursor ->fetch();
                        $datumZaznamu = $row ? $database ->stringToTimestamp($row[1]) : null;
                    }

                    // vlastni rozdeleni kontroluje vzdy (muze se menit)
                    if (!$rozdeleni  ||  !isset($zkontrolovanaRozdeleni[$rozdeleni])) {
                        self::zkontrolujSchvaleni($database, $pp, $datum, $rezimUprav, $rozdeleni);

                        if ($rozdeleni) {
                            $zkontrolovanaRozdeleni[$rozdeleni] = 1;
                        }
                    }
                }

                $cursor ->close();
            } catch (\Exception $e) {
                $cursor ->close();
                throw $e;
            }
        } else {
            // schvaluje se po osobach - staci zkontrolovat schvaleni prvniho dne
            self::zkontrolujSchvaleni($database, $pp, $datumOd, $rezimUprav);
        }
    }


    /**
     * Zapise naplanovane smeny
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param string $kodSmeny
     * @param bool $rezimUprav
     * @param string $logDescription
     * @throws \Exception
     * @throws \Ron\RonPortal\NeniPravoException
     */
    public static function zapisPlanovanouSmenu($database, $pp, $datumOd, $datumDo, $kodSmeny, $rezimUprav, $logDescription = null)
    {
        if (!$datumOd || !$datumDo) {
            throw new \Exception('Chyba - spatne zadane datum!');
        }

        Prava::nactiPravaUzivatele($database, User::getRC());
        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        require_once('kapacity/planovanikapacitrec.php'); // PlanovaniKapacitRecord

        if (!$kodSmeny) {
            $rec = new PlanovaniKapacitRecord($database, $pp, $logDescription);
            $rec->smazNaplanovaneSmeny($datumOd, $datumDo, $rezimUprav);
            return;
        }

        require_once('dochazka/smena.php');
        $smena = Smena::nactiSmenu($database, $kodSmeny);

        // zjisti, jestli je smena variabilni a pripadne jeji preddefinovany pocatek, konec a delku
        if (!$smena) {
            throw new \Exception('Chyba - nelze nacist udaje o smene - zadana smena (' . $kodSmeny . ') neexistuje!');
        }

        $pocatek = null;
        $konec = null;
        $delka = null;
        $preruseniOd = null;
        $preruseniDo = null;

        if ($smena->delena) {
            // delena smena - nastavi prestavku od-do
            if (isset($_GET['preruseniOd'])) {
                $preruseniOd = DateUtils::timeStringToTimestamp($_GET['preruseniOd'], 0);
                $preruseniDo = DateUtils::timeStringToTimestamp($_GET['preruseniDo'], 0);
            } else {
                $preruseniOd = $smena->preruseniOd;
                $preruseniDo = $smena->preruseniDo;
            }
        }

        if ($smena->variabilni) {
            // variabilni smena
            if (isset($_GET['pocatek'])) {
                $pocatek = DateUtils::timeStringToTimestamp($_GET['pocatek'], 0);
                $konec = DateUtils::timeStringToTimestamp($_GET['konec'], 0, true); // $enable_24_00 = true
                if ($konec == 86400) {
                    $konec = 86399;  // 24:00 prevede na 23:59:59
                }
                if ($konec < $pocatek) $konec = DateUtils::incDay($konec, 1);

                $delka = $smena ->vypocitejDelkuSmeny($database, $pocatek, $konec, $preruseniOd, $preruseniDo);
            } else {
                $pocatek = $smena->pocatekSmeny; // isset($row[2]) ? $this ->database ->stringToTimestamp($row[2]) : null;
                $konec = $smena->konecSmeny; // isset($row[3]) ? $this ->database ->stringToTimestamp($row[3]) : null;
                $delka = $smena ->delkaSmeny;
            }
        } else {
            $delka = $smena->delkaSmeny;
        }

        $rezimUprav &= RonPortalSettings::getIntValue(self::SETTINGS_POVOLIT_REZIM_UPRAV);

        // zkontrolovat maximalni pocet hodin
        require_once('kapacity/kapacityutils.php');
        self::zkontrolujMaximalniPocetHodin($database, $pp, $datumOd, $datumDo, $delka, $rezimUprav);

        $den = $datumOd;
        while ($den <= $datumDo) {
            $rec = new PlanovaniKapacitRecord($database, $pp, $logDescription);
            $rec->fHarmonogram->setNewValue(null);
            $rec->zapisSmenu($den, $kodSmeny, $rezimUprav, $pocatek, $konec, $delka, $preruseniOd, $preruseniDo);
            $den = DateUtils::incDayStart($den, 1);
        }
    }


    /**
     * @param Database $database
     * @param SmenaProDen[] $smeny
     * @param PracovniPomer $pp
     * @param int   $datumOd
     * @param int   $datumDo
     * @param bool  $rezimUprav
     * @param string|object $logDescription
     * @throws \Exception
     * @throws \Ron\RonPortal\NeniPravoException
     */
    private static function naplanujSmeny($database, $smeny, $pp, $datumOd, $datumDo, $rezimUprav, $logDescription)
    {
        require_once('kapacity/planovanikapacitrec.php');
        require_once('kapacity/kapacityutils.php');

        $datum = $datumOd;
        $indexSmeny = 0;
        while ($datum <= $datumDo) {
            // preskoci zaznamy pro predchozi dny - ty tam mohou byt, protoze v kalendari muze byt vice smen pro jeden den
            while ($indexSmeny < count($smeny)  &&  $smeny[$indexSmeny] ->datum < $datum) {
                $indexSmeny ++;
            }

            if ($indexSmeny < count($smeny)  &&  $smeny[$indexSmeny] ->datum == $datum) {
                // nasel smenu pro aktualni den => naplanuje ji
                $pocatek = null;
                $konec = null;
                $delkaSmeny = isset($smeny[$indexSmeny] ->delkaSmeny) ? $smeny[$indexSmeny] ->delkaSmeny * 60 : null;
                $preruseniOd = null;
                $preruseniDo = null;

                if ($smeny[$indexSmeny]->variabilni) { // variabilni
                    $pocatek = isset($smeny[$indexSmeny] ->pocatekSmeny) ? $smeny[$indexSmeny] ->pocatekSmeny * 60 : null;
                    $konec = isset($smeny[$indexSmeny] ->konecSmeny) ? $smeny[$indexSmeny] ->konecSmeny * 60 : null;
                }

                if ($smeny[$indexSmeny] ->delena) { // delena smena - doplni prestavku
                    $preruseniOd = isset($smeny[$indexSmeny] ->preruseniOd) ? $smeny[$indexSmeny] ->preruseniOd * 60 : null;
                    $preruseniDo = isset($smeny[$indexSmeny] ->preruseniDo) ? $smeny[$indexSmeny] ->preruseniDo * 60 : null;
                }

                if ($delkaSmeny) {
                    KapacityUtils::zkontrolujMaximalniPocetHodin($database, $pp, $datum, $datum, $delkaSmeny, $rezimUprav);
                }

                $rec = new PlanovaniKapacitRecord($database, $pp, $logDescription);
                $rec->logInfoMessage = 'Naplanovani smen podle kalendare ' . date('j.n.Y', $datumOd) . ' - ' . date('j.n.Y', $datumDo);
                $rec->zapisSmenu($datum, $smeny[$indexSmeny]->kod, $rezimUprav, $pocatek, $konec, $delkaSmeny, $preruseniOd, $preruseniDo);
            } else {
                // pro aktualni den neni smena v kalendari => smaze naplanovanou smenu
                $rec = new PlanovaniKapacitRecord($database, $pp, $logDescription);
                $rec->smazNaplanovaneSmeny($datum, $datum, $rezimUprav);
            }

            $datum = DateUtils::incDayStart($datum);
        }
    }


    /**
     * @param Database $database
     * @param int $idPP
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $rezimUprav
     * @param string|object $logDescription
     * @throws \Exception
     */
    public static function naplanujPodleKalendare($database, $idPP, $datumOd, $datumDo, $rezimUprav, $logDescription)
    {
        require_once('adsapp/ppomer.php');
        $pp = PracovniPomer::nactiPP($database, $idPP);

        Prava::nactiPravaUzivatele($database, User::getRC());
        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        require_once('kapacity/kapacityutils.php');
        self::zkontrolujSchvaleni($database, $pp, $datumOd, $rezimUprav);

        require_once('dochazka/smenaproden.php');
        $smeny = SmenaProDen::nactiSmenyPodleKalendare($database, $pp, $datumOd, $datumDo);

        self::naplanujSmeny($database, $smeny, $pp, $datumOd, $datumDo, $rezimUprav, $logDescription);
    }


    /**
     * @param Database $database
     * @param int   $idPP
     * @param int   $datumOd
     * @param int   $datumDo
     * @param bool  $rezimUprav
     * @param string|object $logDescription
     * @throws \Exception
     */
    public static function naplanujPodleKalendareRozdeleni($database, $idPP, $datumOd, $datumDo, $rezimUprav, $logDescription)
    {
        $typRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_TYP_ROZDELENI);
        if (!$typRozdeleni) {
            throw new \Exception('Chyba - nelze naplanovat podle kalendare rozdeleni - prerazovani zamestnancu neni povoleno!');
        }

        require_once('adsapp/ppomer.php');
        $pp = PracovniPomer::nactiPP($database, $idPP);

        Prava::nactiPravaUzivatele($database, User::getRC());
        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        require_once('kapacity/kapacityutils.php');
        self::zkontrolujSchvaleni($database, $pp, $datumOd, $rezimUprav);

        // nacte smeny podle kalendare rozdeleni prirazeneho osobe ...
        require_once('dochazka/smenaproden.php');
        $smeny1 = SmenaProDen::nactiSmenyPodleKalendareRozdeleni($database, $typRozdeleni, $pp, $datumOd, $datumDo);
        // ... a podle kalandare rozdeleni zmeneneho pro jednotlive dny
        $smeny2 = SmenaProDen::nactiSmenyPodleZmenenehoRozdeleni($database, $pp, $datumOd, $datumDo, RonPortalSettings::getIntValue(self::SETTINGS_POVOLIT_REZIM_UPRAV) && !$rezimUprav);

        // slouci zaznamy z $smeny2 a $smeny1 do jednoho pole. Pokud je pro nejaky den zaznam v $smeny2 i $smeny1,
        // tak ma prednost $smeny2 (tj. rozdeleni do ktereho je zamestnanec prerazen)
        $smeny = [];
        $index1 = 0;
        $index2 = 0;

        while ($index1 < count($smeny1)  ||  $index2 < count($smeny2)) {
            if ($index2 < count($smeny2)) {
                // jeste jsou nejake zaznamy pro jednotlive dny

                // doplni zaznamy pro predchozi dny
                while ($index1 < count($smeny1)  &&  $smeny1[$index1] ->datum < $smeny2[$index2] ->datum) {
                    $smeny[] = $smeny1[$index1];
                    $index1 ++;
                }

                // prida aktualni zaznam z $smeny2
                $smeny[] = $smeny2[$index2];

                // preskoci zaznamy z $smeny1 pro aktualni den (protoze nasel zaznam v $smeny2)
                while ($index1 < count($smeny1)  &&  $smeny1[$index1] ->datum = $smeny2[$index2] ->datum) {
                    $index1 ++;
                }

                $index2++;
            } else {
                // uz nejsou zaznamy pro jednotlive dny - doplni zaznamy podle rozdeleni osoby
                while ($index1 < count($smeny1)) {
                    $smeny[] = $smeny1[$index1];
                    $index1 ++;
                }
            }
        }


        self::naplanujSmeny($database, $smeny, $pp, $datumOd, $datumDo, $rezimUprav, $logDescription);
    }
}
