<?php

/**
 * UTF8
 * Nacitani dat pro planovani kapacit
 */

namespace Ron\Kapacity;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\RonPortal\DokumentDochazky;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RozdeleniOsob;
use Ron\StringUtils;


require_once 'kapacity/planovanikapacit_den.php';
require_once 'kapacity/schvaleni_kapacit.php';
require_once 'kapacity/kapacityutils.php';
require_once 'kapacity/planovana_smena.php';
require_once 'kapacity/planovana_absence.php';

class PlanovaniKapacit_PrirazeniKalendare
{
    var $idPP;

    var $datumOd;
    var $datumDo; // muze byt null !
    var $kod;


    function getJson() {
        $result = [
            'kod' => StringUtils::jsonText($this ->kod),
            'datumOd' => date('j.n.Y', $this ->datumOd)
        ];

        if ($this ->datumDo) {
            $result['datumDo'] = date('j.n.Y', $this ->datumDo);
        }

        return $result;
    }

    /**
     * Nacte prirazene kalendare zadanym osobam pro jednotlive dny v zadanem obdobi
     * - podle prirazenych kalendaru
     * - podle prirazeneho rozdeleni (pokud je zadan typ rozdeleni pro planovani)
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int|int[] $idPP
     * @return PlanovaniKapacit_PrirazeniKalendare[]
     * @throws DBException
     */
    public static function nactiKalendare($database, $idPP, $datumOd, $datumDo)
    {
        $result = [];

        $params = $database ->createParameters();

        if (RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_TYP_ROZDELENI) > 0) {
            // je zadan typ rozdeleni pro planovani - kalendar se urci podle rozdeleni osoby
            $sql =
                'SELECT PracovniPomer.Id, RozdeleniOsob.PlatnostOd, RozdeleniOsob.PlatnostDo, Rozdeleni.Kalendar' .
                ' FROM RozdeleniOsob, Rozdeleni, PracovniPomer' .
                ' WHERE Rozdeleni.Kod = RozdeleniOsob.Rozdeleni' .
                '   AND Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_TYP_ROZDELENI)) .
                '   AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                '   AND Rozdeleni.Kalendar IS NOT NULL' .
                '   AND RozdeleniOsob.RC = PracovniPomer.RC' .
                '   AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                '   AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)';

            if (is_array($idPP)) {
                $sql .= ' AND (0 = 1';
                foreach ($idPP as $id) {
                    $sql .= ' OR PracovniPomer.Id = ' . $params ->addParameter($id);
                }
                $sql .= ')';
            } else {
                $sql .= ' AND PracovniPomer.Id = ' . $params ->addParameter($idPP);
            }

            $sql .=
                ' ORDER BY RozdeleniOsob.PlatnostOd';

        } else {
            // neni zadan typ rozdeleni pro planovani - kalendar se urci z karty zamestnance
            $sql =
                'SELECT PracovniPomer.Id, PlatnostOd, PlatnostDo, Kalendar' .
                ' FROM PrirazeniKalendare, PracovniPomer' .
                ' WHERE PrirazeniKalendare.RC = PracovniPomer.RC' .
                '   AND PrirazeniKalendare.OsobniCislo = PracovniPomer.OsobniCislo' .
                '   AND PrirazeniKalendare.DatumNastupu = PracovniPomer.DatumNastupu' .
                '   AND PrirazeniKalendare.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
                '   AND (PrirazeniKalendare.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR PrirazeniKalendare.PlatnostDo IS NULL)';

            if (is_array($idPP)) {
                $sql .= ' AND (0 = 1';
                foreach ($idPP as $id) {
                    $sql .= ' OR PracovniPomer.Id = ' . $params ->addParameter($id);
                }
                $sql .= ')';
            } else {
                $sql .= ' AND PracovniPomer.Id = ' . $params ->addParameter($idPP);
            }

            $sql .=
                ' ORDER BY PlatnostOd';
        }

        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $kalendar = new PlanovaniKapacit_PrirazeniKalendare();

            $kalendar->idPP = (int)$row[0];
            $kalendar->datumOd = $database ->stringToTimestamp($row[1]);
            $kalendar->datumDo = $database ->stringToTimestamp($row[2]);
            $kalendar->kod = $row[3];

            if ($kalendar ->datumOd < $datumOd) $kalendar ->datumOd = $datumOd;
            if (!$kalendar ->datumDo || $kalendar ->datumDo > $datumDo) $kalendar ->datumDo = $datumDo;
            $result[] = $kalendar;
        }

        $cursor ->close();

        return $result;
    }
}



class PlanovaniKapacit_Suma
{
    var $kodCasoveSlozky;
    var $jednotka;  // H, D, K
    var $nadpis;

    /** @var string[] */
    var $data;

    function __construct()
    {
        $this->data = [];
    }


    /**
     * @param Database $database
     * @param int    $typRozdeleni
     * @param string $rozdeleni
     * @param int    $datumOd
     * @param int    $datumDo
     */
    function nactiDenniCasovaSlozkaR($database, $typRozdeleni, $rozdeleni, $datumOd, $datumDo)
    {
        $datum = $datumOd;
        $this->data = [];

        if (isset($rozdeleni)) {
            $params = $database->createParameters();
            $sql =
                'SELECT Den, Hodiny, Dny, KalendarniDny' .
                ' FROM DenniCasovaSlozka_R' .
                ' WHERE Den >= ' . $params->addParameter($database->dateToParam($datumOd)) .
                '   AND Den <= ' . $params->addParameter($database->dateToParam($datumDo)) .
                '   AND KodTypuRozdeleni = ' . $params->addParameter((int)$typRozdeleni) .
                '   AND Rozdeleni = ' . $params->addParameter($rozdeleni) .
                '   AND TypCasoveSlozkyMzdy = ' . $params->addParameter($this->kodCasoveSlozky) .
                ' ORDER BY Den';

            $cursor = $database->select($sql, $params);

            while ($row = $cursor->fetch()) {
                $pomd = $database->stringToTimestamp($row[0]);

                // pro chybejici datumy doplni prazdny text
                for (; $datum < $pomd; $datum = DateUtils::incDayStart($datum, 1)) $this->data[] = '';

                switch ($this->jednotka) {
                    case 'D' :
                        $text = StringUtils::format('number_2', $row[2]);
                        break;
                    case 'K' :
                        $text = StringUtils::format('number_2', $row[3]);
                        break;
                    default :
                        $text = StringUtils::format('hodiny', $row[1]);
                }

                $this->data[] = $text;

                $datum = DateUtils::incDayStart($datum, 1);
            }

            $cursor ->close();
        }

        // nakonec doplni prazdne texty pro dny, ve kterych neni zaznam
        for (; $datum <= $datumDo; $datum = DateUtils::incDayStart($datum, 1)) $this->data[] = '';
    }
}


class PlanovaniKapacit_Osoba
{
    var $jmenoPrijmeni;
    var $jmeno;
    var $prijmeni;
    var $rc;
    var $osobniCislo;
    var $datumNastupu;
    var $datumUkonceni;
    var $idPP;

    /** @var bool  true, pokud je osoba pouze pujcena z jineho rozdeleni */
    var $cizi;

    var $rozdeleni;
    /** @var string[]  hodnoty uzivatelskych poli osoby */
    var $uzivatelske;
    /** @var string[] */
    var $uzivatelskePP;

    var $lzeEditovat;

    var $hodnotyHlavicky; // hodnoty dodatecnych sloupcu (casovych slozek) v hlavicce
    var $classHlavicky;   // atribut class pro dany sloupec hlavicky

    /** @var RozdeleniOsob[]   Seznam ikon rozdeleni, ktere se maji zobrazit */
    var $ikony;

    // prava na schvalovani
    var $lzeSchvalit1;
    var $lzeSchvalit2;
    var $lzeSchvalit3;

    /** @var int  Datum schvaleno do 1 */
    var $schvaleni1;
    /** @var int  Datum schvaleno do 2 */
    var $schvaleni2;
    /** @var int  Datum schvaleno do 3 */
    var $schvaleni3;

    /** @var int  Datum schvaleno do (puvodni plan) 1 */
    var $schvaleniPuvodnihoPlanu1;
    /** @var int  Datum schvaleno do (puvodni plan) 2 */
    var $schvaleniPuvodnihoPlanu2;
    /** @var int  Datum schvaleno do (puvodni plan) 3 */
    var $schvaleniPuvodnihoPlanu3;

    var $schvaleniPuvodnihoPlanu;

    // data pro pouziti v HTML
    /** @var PlanovaniKapacit_PlanovanaSmena[] */
    var $seznamSmen;
    /** @var PlanovaniKapacit_PlanovanaAbsence[] */
    var $seznamAbsenci;
    /** @var PlanovaniKapacit_PrirazeniKalendare[] */
    var $seznamKalendaru;
    /** @var DokumentDochazky[] */
    var $dokumentyDochazky;
    /** @var PlanovaniKapacit_SluzebniCesta[] */
    var $sluzebniCesty;
    /** @var VysledekKontrolyPlanovani[] */
    var $kontroly;

    /** @var RozdeleniOsob[]    Nacita se pro vypocet souctu podle podminek ve slotovem planovani*/
    var $rozdeleniOsoby = [];

    // data pro pouziti v HTML - primo jednotlive bunky
    /** @var PlanovaniKapacit_Bunka[]  */
    //var $data;

    /** @var float Limit - nacte se z (nepovinne) nastavene casove slozky */
    var $maximalniPocetHodin;

    /** @var string     CSS tridy pro oznaceni radku */
    var $rowClasses;

    //var $f_error;

    // -----     Osoba     ----------------------------------------------------
    /**
     * @param Database $database
     * @param int  $datumOd
     * @param int  $datumDo
     * @param bool $rezimUprav
     * @throws DBException
     */
    function nactiSmeny($database, $datumOd, $datumDo, $rezimUprav)
    {
        $this ->seznamSmen = PlanovaniKapacit_PlanovanaSmena::nactiSmeny($database,
            $this ->idPP, $datumOd, $datumDo, $rezimUprav);
    }


    // -----     Osoba     ----------------------------------------------------
    /**
     * @param Database $database
     * @param int  $datumOd
     * @param int  $datumDo
     * @param bool $rezimUprav
     */
    function nactiAbsence($database, $datumOd, $datumDo, $rezimUprav)
    {
        // vraci seznam absenci pro zadanou osobu a rozsah dnu
        $this ->seznamAbsenci = PlanovaniKapacit_PlanovanaAbsence::nactiAbsence($database,
            $this ->idPP, $datumOd, $datumDo, false, $rezimUprav);
    }


    // -----     Osoba     ----------------------------------------------------
    /**
     * Nacte data osoby.
     * @param Database $database
     * @param array $row
     * @throws DBException
     */
    function nactiData($database, $row)
    {
        $this ->rc = $row[0];
        $this ->osobniCislo = $row[1];
        $this ->datumNastupu = $database ->stringToTimestamp($row[2]);

        if ($row[7]) $this ->datumUkonceni = $database ->stringToTimestamp($row[7]);
        else $this ->datumUkonceni = null;

        $this ->jmeno = $row[3];
        $this ->prijmeni = $row[4];
        $this ->jmenoPrijmeni = $row[4] . " " . $row[3];
        if (trim($row[5])) $this ->jmenoPrijmeni = $this ->jmenoPrijmeni . ', ' . $row[5];

        $this ->idPP = $row[6];

        if (!RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_ZPUSOB_SCHVALOVANI)) {
            $this->schvaleni1 = $database->stringToTimestamp($row[8]);
            $this->schvaleni2 = $database->stringToTimestamp($row[9]);
            $this->schvaleni3 = $database->stringToTimestamp($row[10]);

            $this->schvaleniPuvodnihoPlanu1 = $database->stringToTimestamp($row[51]);
            $this->schvaleniPuvodnihoPlanu2 = $database->stringToTimestamp($row[52]);
            $this->schvaleniPuvodnihoPlanu3 = $database->stringToTimestamp($row[53]);


            if (RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_POVOLIT_REZIM_UPRAV) && RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_MIN_SCHVALENI_PRO_UPRAVY)) {
                $this->schvaleniPuvodnihoPlanu = null;

                if ($this->schvaleniPuvodnihoPlanu1 && RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_MIN_SCHVALENI_PRO_UPRAVY) == 1) {
                    $this->schvaleniPuvodnihoPlanu = $this->schvaleniPuvodnihoPlanu1;
                }

                if ($this->schvaleniPuvodnihoPlanu2 && RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_MIN_SCHVALENI_PRO_UPRAVY) <= 2) {
                    if (!$this->schvaleniPuvodnihoPlanu || $this->schvaleniPuvodnihoPlanu < $this->schvaleniPuvodnihoPlanu2) {
                        $this->schvaleniPuvodnihoPlanu = $this->schvaleniPuvodnihoPlanu2;
                    }
                }

                if ($this->schvaleniPuvodnihoPlanu3) {
                    if (!$this->schvaleniPuvodnihoPlanu || $this->schvaleniPuvodnihoPlanu < $this->schvaleniPuvodnihoPlanu3) {
                        $this->schvaleniPuvodnihoPlanu = $this->schvaleniPuvodnihoPlanu3;
                    }
                }
            }
        }

        // uzivatelske polozky
        $this ->uzivatelske = [];
        for ($i = 0; $i < 20; $i++) $this ->uzivatelske[] = $row[11 + $i];

        $this ->uzivatelskePP = [];
        for ($i = 0; $i < 20; $i++) $this ->uzivatelskePP[] = $row[31 + $i];

        // kod rozdeleni
        if (isset($row[54])) $this ->rozdeleni = $row[54];

        $this ->seznamSmen = [];
        $this ->seznamAbsenci = [];
        $this ->seznamKalendaru = [];
    }



    // -----     Osoba     ----------------------------------------------------
    /**
     * @param Database $database
     * @param int  $idpp
     * @param int  $datumOd
     * @param int  $datumDo
     * @param bool $rezimUprav
     */
    function nactiDataProRefresh($database, $idpp, $datumOd, $datumDo, $rezimUprav)
    {
        require_once('adsapp/ppomer.php');
        $pp = new PracovniPomer();
        $pp ->nactiPPPodleId($database, $idpp);

        $this ->rc = $pp ->rc;
        $this ->osobniCislo = $pp ->osobniCislo;
        $this ->datumNastupu = $pp ->datumNastupu;
        $this ->datumUkonceni = $pp ->datumUkonceni;
        $this ->jmeno = $pp ->jmeno;
        $this ->prijmeni = $pp ->prijmeni;
        $this ->jmenoPrijmeni = $pp ->jmenoPrijmeni;
        $this ->idPP = $idpp;

        // podrobnosti nenacita, ale promenne inicializuje
        $this ->uzivatelske = [];
        $this ->uzivatelskePP = [];
        $this ->hodnotyHlavicky = [];
        $this ->seznamKalendaru = [];

        if (!RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_ZPUSOB_SCHVALOVANI)) {
            $schvaleni = SchvaleniKapacitPP::nactiSchvaleni($database, $datumOd, $this->idPP);
            if ($rezimUprav) {
                $this->schvaleni1 = $schvaleni->schvaleniUprav1;
                $this->schvaleni2 = $schvaleni->schvaleniUprav2;
                $this->schvaleni3 = $schvaleni->schvaleniUprav3;
            } else {
                $this->schvaleni1 = $schvaleni->schvaleni1;
                $this->schvaleni2 = $schvaleni->schvaleni2;
                $this->schvaleni3 = $schvaleni->schvaleni3;
            }

            $this->schvaleniPuvodnihoPlanu = $schvaleni->getSchvaleniPuvodnihoPlanu();
        }

        $this ->nactiSmeny($database, $datumOd, $datumDo, $rezimUprav);
        $this ->nactiAbsence($database, $datumOd, $datumDo, $rezimUprav);
    }



    function getJson() {
        $result = [
            'idPP' => $this ->idPP,
            'jmenoPrijmeni' => StringUtils::jsonText($this ->jmenoPrijmeni),
            'jmeno' => StringUtils::jsonText($this ->jmeno),
            'prijmeni' => StringUtils::jsonText($this ->prijmeni),
            'osobniCislo' => StringUtils::jsonText($this ->osobniCislo),
            'datumNastupuStr' => date('j.n.Y', $this ->datumNastupu)
        ];

        if ($this ->datumUkonceni) $result['datumUkonceniStr'] = date('j.n.Y', $this ->datumUkonceni);

        if ($this ->cizi) $result['cizi'] = 1;

        if ($this ->schvaleni1) $result['schvaleni1Str'] = date('j.n.Y', $this ->schvaleni1);
        if ($this ->schvaleni2) $result['schvaleni2Str'] = date('j.n.Y', $this ->schvaleni2);
        if ($this ->schvaleni3) $result['schvaleni3Str'] = date('j.n.Y', $this ->schvaleni3);

        if ($this ->schvaleniPuvodnihoPlanu) $result['schvaleniPuvodnihoPlanuStr'] = date('j.n.Y', $this ->schvaleniPuvodnihoPlanu);

        if ((int)$this ->lzeEditovat) $result['lzeEditovat'] = (int)$this ->lzeEditovat;

        if ($this ->lzeSchvalit1) $result['lzeSchvalit1'] = (int)$this ->lzeSchvalit1;
        if ($this ->lzeSchvalit2) $result['lzeSchvalit2'] = (int)$this ->lzeSchvalit2;
        if ($this ->lzeSchvalit3) $result['lzeSchvalit3'] = (int)$this ->lzeSchvalit3;

        $result['maxUrovenSchvaleni'] = $this ->lzeSchvalit3
            ? 3
            : ($this ->lzeSchvalit2 ? 2 : ($this ->lzeSchvalit1 ? 1 : 0));

        //if ($this ->f_error) $result['error'] = StringUtils::jsonText($this ->f_error);

        if ($this ->maximalniPocetHodin) $result['maximalniPocetHodin'] = $this ->maximalniPocetHodin;

        $smeny = [];
        foreach ($this ->seznamSmen as $smena) $smeny[] = $smena ->getJSON();
        $result['smeny'] = $smeny;

        $seznamAbsenci = [];
        foreach ($this ->seznamAbsenci as $absence) $seznamAbsenci[] = $absence ->getJSON();
        $result['absence'] = $seznamAbsenci;

        $kalendare = [];
        foreach ($this ->seznamKalendaru as $kalendar) $kalendare[] = $kalendar ->getJson();
        $result['kalendare'] = $kalendare;

        // hodnoty v hlavicce (mesicni vysledky atd.)
        $hlavicky = [];
        foreach ($this ->hodnotyHlavicky as $i => $hodnota) {
            $obj = [
                //'sloupec' => $i,
                'text' => StringUtils::jsonText($hodnota)
            ];

            if (isset($this ->classHlavicky[$i])) $obj['className'] = $this ->classHlavicky[$i];

            $hlavicky[$i] = $obj;
        }

        $result['dataHlavicky'] = $hlavicky;

        if ($this ->ikony) {
            $result['ikony'] = [];
            foreach ($this ->ikony as $ikona) {
                $result['ikony'][] = ['css' => $ikona ->cssClassObrazku, 'popis' => StringUtils::jsonText($ikona ->title)];
            }
        }

        if ($this ->rozdeleniOsoby) {
            $result['rozdeleniOsoby'] = [];
            foreach ($this ->rozdeleniOsoby as $rozdeleniOsoby) {
                $result['rozdeleniOsoby'][] = $rozdeleniOsoby ->getJson();
            }
        }

        if ($this ->rowClasses) {
            $result['rowClasses'] = $this ->rowClasses;
        }

        return $result;
    }


    function jeVRozdeleniOsoby($kodTypuRozdeleni, $kodRozdeleni, $datum)
    {
        if (!$this ->rozdeleniOsoby) {
            return false;
        }

        foreach ($this ->rozdeleniOsoby as $rozdeleniOsoby) {
            if ($rozdeleniOsoby ->kodTypuRozdeleni == $kodTypuRozdeleni  &&  $rozdeleniOsoby ->kod == $kodRozdeleni) {
                if ($datum) {
                    if ($rozdeleniOsoby ->platnostOd <= $datum  &&  (!$rozdeleniOsoby ->platnostDo  ||  $rozdeleniOsoby ->platnostDo >= $datum)) {
                        return true;
                    }
                } else {
                    return true;
                }
            }
        }

        return false;
    }
}


/**
 * Class PlanovaniKapacit_Absence
 * @package Ron\Kapacity
 */
class PlanovaniKapacit_Absence
{
    var $kod;
    var $popis;
    var $zkratka;
    var $barva;

    /** @var int[]  Seznam povolenych delek */
    var $povoleneDelky = [];

    private function nactiData($row)
    {
        // SELECT Kod, Popis, ZkratkaProPlanovani, BarvaProVykresleni
        $this ->kod = $row[0];
        $this ->popis = $row[1];
        $this ->zkratka = $row[2];
        $this ->barva = $row[3];

        if (isset($row[4])) {
            $this ->povoleneDelky = [$row[4]];
        }
    }


    /**
     * @param Database $database
     * @param bool $nacistPovoleneDelky     Urcuje, jestli se maji nacist i seznamy povolenych delek absenci
     * @return PlanovaniKapacit_Absence[]
     * @throws \Exception
     */
    public static function nactiSeznamOperaci($database, $nacistPovoleneDelky = false)
    {
        $result = [];

        // nacte seznam absenci
        $sql =
            'SELECT OperaceDochazky.Kod, OperaceDochazky.Popis, OperaceDochazky.ZkratkaProPlanovani, OperaceDochazky.BarvaProVykresleni' .
            ($nacistPovoleneDelky ? ',  PovolenaDelkaAbsence.Delka ' : '') .
            ' FROM OperaceDochazky' .
            ($nacistPovoleneDelky ? ' LEFT OUTER JOIN PovolenaDelkaAbsence ON PovolenaDelkaAbsence.OperaceDochazky = OperaceDochazky.Kod' : '') .
            ' WHERE OperaceDochazky.Absence = 1 ' .
            ' ORDER BY OperaceDochazky.Kod';


        /** @var PlanovaniKapacit_Absence $posledniAbsence */
        $posledniAbsence = null;

        if ($cursor = $database ->select($sql)) {
            while ($row = $cursor ->fetch()) {
                if ($nacistPovoleneDelky  &&  $posledniAbsence  &&  $posledniAbsence ->kod == $row[0]) {
                    $posledniAbsence ->povoleneDelky[] = $row[4];
                } else {
                    $posledniAbsence = new PlanovaniKapacit_Absence();
                    $posledniAbsence->nactiData($row);
                    $result[] = $posledniAbsence;
                }
            }
        }

        return $result;
    }


    function getJson() {
        $result = [
            'kod' => StringUtils::jsonText($this ->kod),
            'popis' => StringUtils::jsonText($this ->popis),
            'zkratka' => StringUtils::jsonText($this ->zkratka),
            'barva' => GuiUtils::getHtmlColor($this ->barva)
        ];

        if ($this ->povoleneDelky) {
            $result['povoleneDelky'] = $this ->povoleneDelky;
        }

        return $result;
    }
}



class PlanovaniKapacit_SluzebniCesta
{
    public $idPP;
    public $id;
    public $datumOd;
    public $datumDo;
    public $popis;


    /**
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int[] $seznamPP
     * @return PlanovaniKapacit_SluzebniCesta[]
     */
    public static function nactiSluzebniCestyZDB($database, $datumOd, $datumDo, $seznamPP)
    {
        $prava = Prava::vratPrava('kapacity');
        $pravaEdit = Prava::vratPrava('kapacity_edit');

        $params = $database ->createParameters();
        $sql =
            'SELECT PracovniPomer.Id, SluzebniCesta.Id, SluzebniCesta.DatumCasOd, SluzebniCesta.DatumCasDo, SluzebniCesta.Popis' .
            ' FROM SluzebniCesta, PracovniPomer' .
            ' WHERE SluzebniCesta.RC = PracovniPomer.RC' .
            ' AND SluzebniCesta.OsobniCislo = PracovniPomer.OsobniCislo' .
            ' AND SluzebniCesta.DatumNastupu = PracovniPomer.DatumNastupu' .
            ' AND SluzebniCesta.DatumCasDo >= ' . $params ->addParameter($database ->timestampToParam($datumOd)) .
            ' AND SluzebniCesta.DatumCasOd <= ' . $params ->addParameter($database ->timestampToParam($datumDo));

        $sql .= ' AND (0 = 1';
        foreach ($seznamPP as $idPP) {
            $sql .= ' OR PracovniPomer.Id = ' . $params ->addParameter((int)$idPP);
        }
        $sql .= ')';

        $result = [];

        try {
            $cursor = $database->getSqlResult($sql, $params, 'Chyba pri nacitani sluzebnich cest');
            while ($row = $cursor->fetch()) {
                if (Prava::lzeZobrazitPP([$prava, $pravaEdit], $row[0], $datumOd, $datumDo)) {
                    $sc = new PlanovaniKapacit_SluzebniCesta();
                    $sc->idPP = $row[0];
                    $sc->id = $row[1];
                    $sc->datumOd = $cursor->stringToTimestamp($row[2]);
                    $sc->datumDo = $cursor->stringToTimestamp($row[3]);
                    $sc->popis = $row[4];

                    $result[] = $sc;
                }
            }
            $cursor->close();
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        return $result;
    }


    function getJson() {
        return [
            'idPP' => $this ->idPP,
            'id' => $this ->id,
            'datumOdStr' => StringUtils::jsonDate($this ->datumOd),
            'datumDoStr' => StringUtils::jsonDate($this ->datumDo),
            'popis' => StringUtils::jsonText($this ->popis)
        ];
    }
}


/**
 * Class PlanovaniKapacit_Kalendar
 * @package Ron\Kapacity
 */
class PlanovaniKapacit_Kalendar
{
    var $kod;
//    var $f_popis;

    /** @var PlanovaniKapacit_PlanovanaSmena[] */
    var $seznamSmen; // seznam povolenych smen

    function __construct()
    {
        $this->seznamSmen = [];
    }


    // -----     PlanovaniKapacit_Kalendar     --------------------------------
    /**
     * @param Database $database
     * @param string $kod
     * @param bool $podleSmeny2
     * @param int  $datumOd
     * @param int  $datumDo
     * @throws \Exception
     */
    function nactiData($database, $kod, $podleSmeny2, $datumOd, $datumDo)
    {
        $this->kod = $kod;
        //$this -> f_popis = $row[1];

        $this->seznamSmen = [];

        $params = $database ->createParameters();
        if ($podleSmeny2) {
            $sql =
                'SELECT Smena, Den ' .
                ' FROM SmenaProDenKalendare ' .
                ' WHERE Kalendar = ' . $params ->addParameter($this->kod) .
                ' AND Den >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
                ' AND Den <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
                ' ORDER BY Den';
        } else {
            $sql =
                'SELECT KodSmeny, 0 ' .  // datum se nenacita
                ' FROM PouzivaneSmeny ' .
                ' WHERE KodKalendare = ' . $params ->addParameter($this->kod) .
                ' ORDER BY Poradi';
        }

        $cursor = $database ->select($sql, $params);

        while ($row = $cursor ->fetch()) {
            $smena = new PlanovaniKapacit_PlanovanaSmena();
            $smena->kod = $row[0];

            if ($podleSmeny2) $smena->datum = $database ->stringToTimestamp($row[1]);
            else $smena->datum = 0;

            $this->seznamSmen[] = $smena;
        }

        $cursor ->close();
    }


    function getJsonSmeny()
    {
        $result = [];

        foreach ($this->seznamSmen as $smena) {
            $obj = [
                'kod' => StringUtils::jsonText($smena ->kod)
            ];

            if ($smena ->datum) $obj['datumStr'] = StringUtils::jsonDate($smena->datum);

            $result[] = $obj;
        }

        return $result;
    }


    function getJson()
    {
        return [
            'kod' => StringUtils::jsonText($this ->kod),
            // 'Popis' => ...
            'povoleneSmeny' => $this ->getJsonSmeny()
        ];
    }
}


/**
 * Nastaveni sloupcu v hlavicce
 * Class PlanovaniKapacit_SloupecHlavicky
 * @package Ron\Kapacity
 */
class PlanovaniKapacit_SloupecHlavicky
{
    var $nadpis;

    var $kodCasoveSlozky; // R = kod rozdeleni, U<n> = uzivatelska, P<n> = uzivatelska pracovniho pomeru
    var $jednotka; // H/D/K
    var $className;

    /** @var bool   Priznak, ze se ma polozka ve slotovem rezimu zobrazit na dalsim radku */
    var $dalsiRadek = false;

    function nactiNastaveni($nastavenistr)
    {
        // kod:nadpis:jednotka
        $poma = explode(':', $nastavenistr);

        $this->kodCasoveSlozky = $poma[0];

        if ($this ->kodCasoveSlozky[0] == '/') {
            $this ->dalsiRadek = true;
            $this ->kodCasoveSlozky = substr($this->kodCasoveSlozky, 1);
        }

        if (isset($poma[1])) $this->nadpis = \Ron\ronText($poma[1]);
        if (isset($poma[2])) $this->jednotka = $poma[2];

        switch ($this ->kodCasoveSlozky) {
            case 'R' :
            case 'IMG' :
            case 'U1' : case 'U2' : case 'U3' : case 'U4' : case 'U5' :
            case 'U6' : case 'U7' : case 'U8' : case 'U9' : case 'U10' :
            case 'U11' : case 'U12' : case 'U13' : case 'U14' : case 'U15' :
            case 'U16' : case 'U17' : case 'U18' : case 'U19' : case 'U20' :
            case 'P1' : case 'P2' : case 'P3' : case 'P4' : case 'P5' :
            case 'P6' : case 'P7' : case 'P8' : case 'P9' : case 'P10' :
            case 'P11' : case 'P12' : case 'P13' : case 'P14' : case 'P15' :
            case 'P16' : case 'P17' : case 'P18' : case 'P19' : case 'P20' :
                $this ->className = 'textcol textcol_' . $this ->kodCasoveSlozky;
                break;

            default :
                $this ->className = 'align-right mvcol mvcol_' . str_replace(' ', '_', $this ->kodCasoveSlozky);
        }
    }


    function getJson() {
        return [
            'kodCasoveSlozky' => StringUtils::jsonText($this->kodCasoveSlozky),
            'nadpis' => StringUtils::jsonText($this ->nadpis),
            'className' => StringUtils::jsonText($this ->className),
            'dalsiRadek' => $this ->dalsiRadek ? 1 : 0
        ];
    }
}


