<?php
/**
 * UTF8
 * stanislav.elbl, 14.02.2018
 */

namespace Ron\Kapacity;

use Ron\DateUtils;
use Ron\StringUtils;
use Ron\DB\Database;

class PlanovaniKapacit_Podminka
{
    const TYP_SMENA = 0;
    const TYP_CAS_OD_DO = 1;
    const TYP_OPERACE = 2;
    const TYP_PRACOVNI_DOBA = 3;
    const TYP_ZACATEK = 4;
    const TYP_KONEC = 5;

    public $id;

    /** @var int    0=smena, 1=cas od do, 2=operacedochazky, 3=pracovni doba, 4=zacatek pracovni doby, 5=konec pracovni doby */
    public $typPodminky;

    /** @var string */
    public $typKontroly;

    /** @var int    Priznak, ze se jedna o slot pro planovani kapacit */
    public $slotProPlanovani;

    /** @var int    Priznak, ze se muze editovat - 0=ne, 1=ano, 2=pouze zobrazit */
    public $lzeEditovat;

    /** @var string Kod smeny */
    public $smena;

    /** @var float  Pocet hodin*/
    public $casOd;
    /** @var float  Pocet hodin */
    public $casDo;

    /** @var int */
    public $pocetMin;
    /** @var int */
    public $pocetMax;

    public $kodTypuRozdeleni;
    public $rozdeleni;

    public $kodDovednosti;
    public $kodUrovne;

    /** @var string */
    public $updateUserName;
    /** @var int */
    public $updateTimestamp;

    /**
     * @return array
     */
    public function getJson()
    {
        $result = [
            'typPodminky' => $this ->typPodminky,
            'smena' => StringUtils::jsonText($this ->smena),
            'casOdStr' => $this ->casOd ? DateUtils::minutesToString($this ->casOd * 60) : null,
            'casDoStr' => $this ->casDo ? DateUtils::minutesToString($this ->casDo * 60) : null,
            'pocetMin' => $this ->pocetMin,
            'pocetMax' => $this ->pocetMax
        ];

        if ($this ->kodTypuRozdeleni  &&  $this ->rozdeleni) {
            // pouzije se pro vypocet souctu ve slotech
            $result['kodTypuRozdeleni'] = $this ->kodTypuRozdeleni;
            $result['rozdeleni'] = StringUtils::jsonText($this ->rozdeleni);
            // pouzije se pro zobrazeni
            $result['specifikace'] = StringUtils::jsonText($this ->rozdeleni);
        } else if ($this ->kodDovednosti  &&  $this ->kodUrovne) {
            $result['specifikace'] = StringUtils::jsonText($this ->kodDovednosti) . '/' . $this ->kodUrovne;
        }

        return $result;
    }


    public function getPopis()
    {
        // 0=smena, 1=cas od do, 2=operacedochazky, 3=pracovni doba, 4=zacatek pracovni doby, 5=konec pracovni doby
        switch ($this ->typPodminky) {
            case 0 :
                return $this ->smena;

            case 1 :
                return
                    StringUtils::format("timeofday", $this ->casOd) .
                    ' - ' .
                    StringUtils::format("timeofday", $this ->casDo);

            case 2 :
                return '(operace)';

            case 3 :
                return \Ron\ronText('Pracovni_doba');

            case 4 :
                return \Ron\ronText('Zacatek_pracovni_doby');

            case 5 :
                return \Ron\ronText('Konec_pracovni_doby');

            default :
                return 'Neznamy typ podminky!';
        }
    }


    /**
     * @param PlanovaniKapacit_Podminka $podminka
     * @return bool
     */
    public function isSameAs($podminka)
    {
        if ($this == $podminka) {
            // primo stejny objekt podminky ... (nebo porovnat id?)
            return true;
        }

        return
            $this ->casOd == $podminka ->casOd  &&
            $this ->casDo == $podminka ->casDo  &&
            $this ->smena == $podminka ->smena  &&
            $this ->kodTypuRozdeleni == $podminka ->kodTypuRozdeleni &&
            $this ->rozdeleni == $podminka ->rozdeleni &&
            $this ->typPodminky == $podminka ->typPodminky &&
            $this ->kodDovednosti == $podminka ->kodDovednosti &&
            $this ->kodUrovne == $podminka ->kodUrovne;
    }


    public static function compare($podminka1, $podminka2)
    {
        if (!$podminka1) {
            return $podminka2 ? -1 : 0;
        }
        if (!$podminka2) {
            return 1;
        }

        // oba radky maji prirazenou podminku
        if ($podminka2 ->typPodminky < $podminka1 ->typPodminky) {
            return 1; // seradit podle typu podminky (0=smeny, 1=cas od-do, ...)
        }
        if ($podminka2 ->typPodminky > $podminka1 ->typPodminky) {
            return -1;
        }

        // stejny typ podminky
        if (!$podminka1 ->typPodminky) {
            // smena
            if ($podminka1 ->smena > $podminka2 ->smena) {
                return 1;
            }
            if ($podminka1 ->smena < $podminka2 ->smena) {
                return -1;
            }
            return 0;
        }

        if ($podminka1 ->typPodminky == 1) {
            // cas od - do
            if ($podminka1 ->casOd > $podminka2 ->casOd) {
                return 1;
            }
            if ($podminka1 ->casOd < $podminka2 ->casOd) {
                return -1;
            }
            // stejny cas od - jeste zkontroluju cas do
            if ($podminka1 ->casDo > $podminka2 ->casDo) {
                return 1;
            }
            if ($podminka1 ->casDo < $podminka2 ->casDo) {
                return -1;
            }
            return 0;
        }

        return 0;

    }
}


class PlanovaniKapacit_SadaPodminek
{
    public $id;

    public $datumOd;
    public $datumDo;

    /** @var int    0=vychozi, 1=prioritni, 2=trvala  */
    public $prioritni;

    /** @var int    0=bez omezeni, 1=typ dne, 2=dny v tydnu */
    public $typOmezeni;
    /** @var int    typ dne podle kalendare u daneho strediska */
    public $typDne;
    /** @var int    Bitova maska - 1=Po,2=Ut,...,64=Ne */
    public $denVTydnu;

    /** @var PlanovaniKapacit_Podminka[]  Seznam podminek */
    public $podminky = [];

    /**
     * Nacte sady podminek platne pro zadane rozdeleni
     * @param Database $database
     * @param int $typRozdeleni
     * @param string $rozdeleni
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @param string $filter
     * @return PlanovaniKapacit_SadaPodminek[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiPodminky($database, $typRozdeleni, $rozdeleni, $obdobiOd, $obdobiDo, $filter = null)
    {
        $result = [];

        $params = $database ->createParameters();

        $sql =
            'SELECT SadaPodminekRozdeleni.IdSady, SadaPodminekRozdeleni.DatumOd, SadaPodminekRozdeleni.DatumDo, SadaPodminekRozdeleni.Prioritni,' . // 0..3
            ' SadaPodminekRozdeleni.TypOmezeni, SadaPodminekRozdeleni.TypDne, SadaPodminekRozdeleni.DenVTydnu,' . // 4..6
            ' PodminkaPlanovaniKapacit.Id,' .    // 7
            ' PodminkaPlanovaniKapacit.TypPodminky, PodminkaPlanovaniKapacit.Smena, PodminkaPlanovaniKapacit.CasOd, PodminkaPlanovaniKapacit.CasDo,' . // 8..11
            ' PodminkaPlanovaniKapacit.KodTypuRozdeleni, PodminkaPlanovaniKapacit.Rozdeleni,' . // 12,13
            ' PodminkaPlanovaniKapacit.KodDovednosti, PodminkaPlanovaniKapacit.KodUrovne,' . // 14,15
            ' PodminkaPlanovaniKapacit.PocetMin, PodminkaPlanovaniKapacit.PocetMax, PodminkaPlanovaniKapacit.TypKontroly,' . // 16..18
            ' PodminkaPlanovaniKapacit.SlotProPlanovani, PodminkaPlanovaniKapacit.LzeEditovat,' . // 19,20
            ' PodminkaPlanovaniKapacit.Update_UserName, PodminkaPlanovaniKapacit.Update_Timestamp' . // 21,22
            "\n" .
            'FROM SadaPodminekRozdeleni LEFT OUTER JOIN PodminkaPlanovaniKapacit' .
            '  ON PodminkaPlanovaniKapacit.IdSady = SadaPodminekRozdeleni.IdSady' .
            "\n" .
            'WHERE SadaPodminekRozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($typRozdeleni) .
            '  AND SadaPodminekRozdeleni.Rozdeleni = ' . $params ->addParameter($rozdeleni) .
            '  AND SadaPodminekRozdeleni.DatumOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '  AND (SadaPodminekRozdeleni.DatumDo IS NULL OR SadaPodminekRozdeleni.DatumDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . ')';

        if ($filter) {
            $sql .= ' AND ' . $filter;
        }

        $sql .=
            ' ORDER BY SadaPodminekRozdeleni.IdSady, SadaPodminekRozdeleni.DatumOd, PodminkaPlanovaniKapacit.CasOd';

        $cursor = $database ->getSqlResult($sql, $params);

        /** @var PlanovaniKapacit_SadaPodminek $sada */
        $sada = null;

        while ($row = $cursor ->fetch()) {
            $datumOd = $database ->stringToTimestamp($row[1]);
            if (!$sada  ||  $row[0] != $sada ->id  ||  $datumOd != $sada ->datumOd) {
                $sada = new PlanovaniKapacit_SadaPodminek();
                $sada ->id = $row[0];
                $sada ->datumOd = $datumOd;
                $sada ->datumDo = $database ->stringToTimestamp($row[2]);
                $sada ->prioritni = $row[3];
                $sada ->typOmezeni = (int)$row[4];
                $sada ->typDne = (int)$row[5];
                $sada ->denVTydnu = $row[6];
                $result[] = $sada;
            }

            if (isset($row[7])) {
                $podminka = new PlanovaniKapacit_Podminka();
                $podminka->id = (int)$row[7];
                $podminka->typPodminky = (int)$row[8];
                if ($podminka->typPodminky == 0) {
                    // smena
                    $podminka->smena = $row[9];
                }
                $podminka->casOd = $database->stringToTimeOfDay($row[10]);
                $podminka->casDo = $database->stringToTimeOfDay($row[11]);

                $podminka->pocetMin = (int)$row[16];
                $podminka->pocetMax = (int)$row[17];

                $podminka->kodTypuRozdeleni = ($row[12] && $row[13]) ? (int)$row[12] : null;
                $podminka->rozdeleni = ($row[12] && $row[13]) ? $row[13] : null;

                $podminka->kodDovednosti = $row[14];
                $podminka->kodUrovne = isset($row[15]) ? (int)$row[15] : null;

                $podminka->typKontroly = $row[18];

                $podminka->slotProPlanovani = $row[19];
                $podminka->lzeEditovat = $row[20];

                $podminka->updateUserName = $row[21];
                $podminka->updateTimestamp = $database ->stringToTimestamp($row[22]);

                $sada->podminky[] = $podminka;
            }
        }

        $cursor ->close();

        // TODO: pro podmiky typu "Operace dochazky" nacist prirazene operace

        return $result;
    }


    /**
     * Nacte sady podminek platne pro zadane rozdeleni a den. Vraci jen sady, ktere se maji skutecne pouzit,
     * takze pokud existuje prioritni sada, tak nevraci vychozi.
     * @param Database $database
     * @param int $typRozdeleni
     * @param string $rozdeleni
     * @param int $datum
     * @param string $filter
     * @return PlanovaniKapacit_SadaPodminek[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiSadyPodminekProDen($database, $typRozdeleni, $rozdeleni, $datum, $filter = null)
    {
        require_once 'kapacity/podminky_planovani.php';
        $sadyPodminek = self::nactiPodminky($database, $typRozdeleni, $rozdeleni, $datum, $datum, $filter);

        // profiltruje jen platne sady - vynecha vychozi, pokud existuje prioritni
        $pouziteSady = [];
        $vychoziSada = null;
        $jePrioritni = false;
        foreach ($sadyPodminek as $sada) {
            if ($sada ->jePlatnaProDen($datum)) {
                if ($sada ->prioritni == 1) {
                    // prioritni
                    $pouziteSady[] = $sada;
                    $jePrioritni = true;
                } else if ($sada ->prioritni == 2) {
                    // trvala
                    $pouziteSady[] = $sada;
                } else {
                    // vychozi - pouzije se jen v pripade, ze neexistuje prioritni
                    $vychoziSada = $sada;
                }
            }
        }

        if (!$jePrioritni  &&  $vychoziSada) {
            $pouziteSady[] = $vychoziSada;
        }

        return $pouziteSady;
    }


    /**
     * @return array
     */
    public function getJson()
    {
        $result = [
            'id' => $this ->id,
            'datumOdStr' => StringUtils::jsonDate($this ->datumOd),
            'datumDoStr' => StringUtils::jsonDate($this ->datumDo),
            'prioritni' => $this ->prioritni ? 1 : 0,
            'typOmezeni' => $this ->typOmezeni,
            'typDne' => $this ->typDne,
            'denVTydnu' => $this ->denVTydnu,
            'podminky' => []
        ];

        foreach ($this ->podminky as $podminka) {
            $result['podminky'][] = $podminka ->getJson();
        }

        return $result;
    }


    /**
     * Vraci true, pokud je sada platna pro zadane datum (podle datumu platnosti a omezeni na typ dne nebo den v tydnu)
     * @param $datum
     * @param int $typDne
     * @return bool|int
     */
    public function jePlatnaProDen($datum, $typDne = 0)
    {
        // kontrola platnosti od do
        if ($datum < $this ->datumOd) return false;
        if ($this ->datumDo  &&  $datum > $this ->datumDo) return false;

        // kontrola podle typu dne
        if ($this ->typOmezeni == 1  &&  $this ->typDne != $typDne) return false;

        // kontrola podle dnu v tydnu
        if ($this ->typOmezeni == 2) {
            // jen vybrane dny v tydnu. Bit 1=Po atd.
            return (0x01 << (date('N', $datum) - 1)) & $this ->denVTydnu;
        }

        // zadne omezeni
        return true;
    }


    public function najdiPodminku($hledanaPodminka)
    {
        foreach ($this ->podminky as $podminka) {
            if ($podminka ->isSameAs($hledanaPodminka)) {
                return $podminka;
            }
        }

        return null;
    }
}

