<?php
/**
 * stanislav.elbl, 22.02.2019
 */

namespace Ron\Kapacity;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\RonPortal\NeniPravoException;
use Ron\RonPortal\Obdobi;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\RonPortalXMLEvents;
use Ron\RonPortal\Rozdeleni;
use Ron\RonPortal\TypRozdeleni;
use Ron\RonPortal\User;
use Ron\RonPortal\XML\RonPortalXMLForm;
use Ron\StringUtils;
use Ron\TextManager;


class PodminkyPlanovaniController extends HttpController
{
    /** @var Database */
    var $database;

    /**
     * PodminkyPlanovaniController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * @param PodminkyPlanovaniForm $form
     * @return mixed
     */
    public function showForm($form) {
        if ($form ->editace) {
            require_once 'adsapp/rozdeleni.php';
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($this ->database, $form ->rozdeleni ->typ);
            Model::addData('typRozdeleni', $typRozdeleni);
        }

        return $form;
    }


    /**
     * @param int $pocatekObdobi
     * @param string $kodRozdeleni
     * @return Redirect
     */
    public function redirect($pocatekObdobi, $kodRozdeleni)
    {
        return new Redirect('?menuid=podminkyplanovani&obdobi=' . DateUtils::dateToUrlString($pocatekObdobi) . '&rozdeleni=' . StringUtils::htmlTextConvert($kodRozdeleni));
    }


    /**
     * @param string $eventName
     * @param string $context
     * @param Rozdeleni $rozdeleni
     * @param Obdobi $obdobi
     * @param RonPortalDBRecord $record
     */
    public function executeEvents($eventName, $context, $rozdeleni, $obdobi, $record = null)
    {
        require_once 'xml/xmlformportal.php';
        RonPortalXMLForm::xmlEvents($this ->database, $eventName,
            [
                'kodTypuRozdeleni' => $rozdeleni ->typ,
                'rozdeleni' => $rozdeleni ->kod,
                'obdobi' => $obdobi ->zacatek,
                'record' => $record
            ],
            $context);
    }


    public function zpracujPost()
    {
        $typRozdeleni = KapacityUtils::typRozdeleniProVyber();
        $rozdeleni = Rozdeleni::loadRozdeleni($this ->database, $typRozdeleni, $_POST['rozdeleni']);
        $obdobi = Obdobi::vratObdobi($this->database, 'ADS', DateUtils::urlStringToDate($_POST['obdobi']));

        if ($_POST['cancel']) {
            return $this ->redirect($obdobi ->zacatek, $rozdeleni ->kod);
        }

        $form = new PodminkyPlanovaniForm($this->database);
        // nacte originalni data
        $form->load($obdobi, $rozdeleni, true);

        require_once 'adsapp/events.php';
        $this ->executeEvents(RonPortalXMLEvents::KAPACITY_PODMINKY_BEFORE, '', $rozdeleni, $obdobi);
        $form->zpracujPost();
        $this ->executeEvents(RonPortalXMLEvents::KAPACITY_PODMINKY_AFTER, '', $rozdeleni, $obdobi);

        if (MessageManager::getInstance() ->isError()) {
            return $this ->showForm($form);
        }

        return $this ->redirect($obdobi ->zacatek, $rozdeleni ->kod);
    }


    public function editace()
    {
        $kodTypuRozdeleni = KapacityUtils::typRozdeleniProVyber();

        $rozdeleni = Rozdeleni::loadRozdeleni($this->database, $kodTypuRozdeleni, $_GET['rozdeleni']);
        $obdobi = Obdobi::vratObdobi($this->database, 'ADS', DateUtils::urlStringToDate($_GET['obdobi']));

        $form = new PodminkyPlanovaniForm($this->database);
        $form->load($obdobi, $rozdeleni, true);

        return $this ->showForm($form);
    }


    /**
     * @param int $uroven
     * @param bool $schvalit  Urcuje, jestli se ma schvalit (true), nebo odschvalit (false)
     * @return Redirect
     * @throws \Exception
     */
    public function nastavSchvaleni($uroven, $schvalit)
    {
        $kodTypuRozdeleni = KapacityUtils::typRozdeleniProVyber();

        $rozdeleni = Rozdeleni::loadRozdeleni($this->database, $kodTypuRozdeleni, $_GET['rozdeleni']);
        $obdobi = Obdobi::vratObdobi($this->database, 'ADS', DateUtils::urlStringToDate($_GET['obdobi']));

        try {
            require_once 'kapacity/schvaleni_podminek.php';
            SchvaleniPodminekPlanovani::nastavSchvaleniPodminek($this ->database,
                $rozdeleni, $obdobi, $uroven, $schvalit ? $obdobi->konec : null);
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
        }

        return $this->redirect($obdobi->zacatek, $rozdeleni->kod);
    }


    public function handleAction($action)
    {
        require_once 'kapacity/podminky_rozdeleni.php';
        require_once 'adsapp/ronportalfilter.php';
        require_once 'kapacity/kapacityutils.php';
        require_once 'adsapp/obdobi.php';
        require_once 'adsapp/rozdeleni.php';

        HtmlHeaders::getInstance() ->addScript('', '?menuid=kapacity&action=translations&ver=' . HtmlHeaders::getInstance() ->versionId, 'translations');
        HtmlHeaders::getInstance() ->addCss('css/s_kapacity.css', '?ver=' . HtmlHeaders::getInstance() ->versionId);

        TextManager::getInstance() ->loadStrings('kapacity.ini');

        if (isset($_POST['editacepodminek']) && (int)$_POST['editacepodminek']) {
            return $this ->zpracujPost();
        }

        if ($_GET['action']) {
            switch ($_GET['action']) {
                case 'editace' :
                    return $this->editace();

                    // schvaleni
                case 'schvalit1' :
                    return $this ->nastavSchvaleni(1, true);
                case 'schvalit2' :
                    return $this ->nastavSchvaleni(2, true);
                case 'schvalit3' :
                    return $this ->nastavSchvaleni(3, true);

                case 'zrusitschvaleni1' :
                    return $this ->nastavSchvaleni(1, false);
                case 'zrusitschvaleni2' :
                    return $this ->nastavSchvaleni(2, false);
                case 'zrusitschvaleni3' :
                    return $this ->nastavSchvaleni(3, false);
            }
        }

        $filter = new RonPortalFilterForm($this ->database, $action, true);
        $filter ->nastavObdobi(true, 'ADS', false); // jenOtevrene = false
        $filter ->header = \Ron\ronText('Minimalni_pocty_osob');

        $typRozdeleni = KapacityUtils::typRozdeleniProVyber();
        $filter ->f_pravarozdeleni = Prava::vratPrava('kapacity');
        $filter ->nastavRozdeleni(true, $typRozdeleni, 0, '', true);
        $filter ->loadData();

        Model::addViewObject($filter);

        $form = null;
        if ($filter ->kodRozdeleni) {
            $obdobi = Obdobi::vratObdobi($this->database, 'ADS', $filter->vybraneObdobiOd);
            $rozdeleni = Rozdeleni::loadRozdeleni($this->database, $typRozdeleni, $filter->kodRozdeleni);

            $form = new PodminkyPlanovaniForm($this->database);
            $form->load($obdobi, $rozdeleni);
        }


        if ($form) {
            return $this ->showForm($form);
        } else {
            return new TextViewObject('Nejdrive_vyberte_rozdeleni');
        }
    }

}
