<?php
/**
 * stanislav.elbl, 16.08.2019
 */

namespace Ron\Kapacity;

use Ron\DB\Database;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\Rozdeleni;
use Ron\StringUtils;


class SadaPodminekPlanovani
{

    /**
     * Vytvori novou sadu podminek a priradi ji aktualnimu rozdeleni. Vraci Id nove vytvoreneho zaznamu.
     * @param Database $database
     * @param Rozdeleni $rozdeleni
     * @param int $datum
     * @return int
     */
    public static function vytvorSaduPodminekProDen($database, $rozdeleni, $datum)
    {
        require_once 'adsapp/ronportaldbrecord.php';

        // vytvori sadu podminek
        $rec = new RonPortalDBRecord($database, 'SadaPodminekPlanovani', 'PodminkyPlanovaniForm');
        $rec ->addField('Id', 'serial', true);
        $rec ->addField('Popis', '', false, 160, $rozdeleni ->popis . ' (' . $rozdeleni ->kod . ') - ' . StringUtils::format('date', $datum));
        $rec ->insertRecord();

        $serial = $rec ->findFieldByName('Id') ->value;

        // priradi ji rozdeleni pro aktualni den
        $rec = new RonPortalDBRecord($database, 'SadaPodminekRozdeleni', 'PodminkyPlanovaniForm');
        $rec ->addField('KodTypuRozdeleni', 'integer', true, 0, $rozdeleni ->typ);
        $rec ->addField('Rozdeleni', '', true, 30, $rozdeleni ->kod);
        $rec ->addField('IdSady', 'integer', true, 0, $serial);
        $rec ->addField('DatumOd', 'date', true, 0, $datum);
        $rec ->addField('DatumDo', 'date', true, 0, $datum);
        $rec ->addField('Prioritni', 'integer', true, 0, 1);
        $rec ->addField('TypOmezeni', 'integer', true, 0, 0);
        $rec ->insertRecord();

        return $serial;
    }


    /**
     * Vraci Id prioritni sady podminek prirazenou zadanemu rozdeleni.
     * @param Database $database
     * @param Rozdeleni $rozdeleni
     * @param int $datum
     * @param bool $vytvoritNovou   Pokud je true a prioritni sada neexistuje, tak ji vytvori
     * @return int
     */
    public static function vratIdSadyPodminekProDen($database, $rozdeleni, $datum, $vytvoritNovou = false)
    {
        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult(
            'SELECT IdSady FROM SadaPodminekRozdeleni' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter($rozdeleni ->typ) .
            '  AND Rozdeleni = ' . $params ->addParameter($rozdeleni ->kod) .
            '  AND DatumOd = ' . $params ->addParameter($database ->dateToParam($datum)) .
            '  AND Prioritni = 1  AND  TypOmezeni = 0',
            $params
        );

        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            return $row[0];
        };

        // neexistuje
        if ($vytvoritNovou) {
            return self::vytvorSaduPodminekProDen($database, $rozdeleni, $datum);
        }

        return null;
    }
}
