<?php
/**
 * stanislav.elbl, 27.05.2019
 */

namespace Ron\Kapacity;

use Ron\DB\Database;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\Schvaleni;
use Ron\StringUtils;

require_once 'adsapp/schvaleni.php';


abstract class SchvaleniKapacit extends Schvaleni
{
    /** @var int */
    public $schvaleniUprav1;
    /** @var int */
    public $schvaleniUprav2;
    /** @var int */
    public $schvaleniUprav3;


    /**
     * @param int $uroven
     * @return \Ron\RonPortal\PravaNaOsoby
     */
    public function vratPravoProUroven($uroven)
    {
        return Prava::vratPrava('kapac_schval' . $uroven);
    }


    public function jeSchvalenoPlanovani($datum, $rezimUprav)
    {
        if (!$rezimUprav) return $this->jeSchvalenoDatum($datum);

        return $this ->schvaleniUprav1 >= $datum  ||  $this ->schvaleniUprav2 >= $datum  ||  $this ->schvaleniUprav3 >= $datum;
    }


    public function lzeEditovatKapacity($datum, $rezimUprav)
    {
        require_once 'kapacity/kapacityutils.php';
        $lzeEditovatSchvalene = RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_EDITACE_SCHVALENYCH);
        if (!$lzeEditovatSchvalene) {
            return !$this->jeSchvalenoPlanovani($datum, $rezimUprav);
        }

        $schvalenaUroven = $this ->vratUrovenSchvaleniKapacit($datum, $rezimUprav);

        if (!$schvalenaUroven) return true;
        if ($schvalenaUroven > 2) return false; // schvaleno nejvyssi urovni => nelze

        if ($this ->lzeSchvalovatDen(3, $datum)) {
            return true; // uzivatel ma pravo schvalovat uroven 3 => muze editovat
        }

        if ($schvalenaUroven > 1) return false;

        if ($this ->lzeSchvalovatDen(2, $datum)) {
            return true; // uzivatel ma pravo schvalovat uroven 2 => muze editovat
        }

        return false; // schvaleno urovni 1 a uzivatel nema pravo schvalovat vyssi uroven
    }

    /**
     * @param int $datum
     * @param bool $rezimUprav
     * @return int
     */
    function vratUrovenSchvaleniKapacit($datum, $rezimUprav)
    {
        if (!$rezimUprav) {
            return $this->vratUrovenSchvaleni($datum);
        }

        if ($this ->schvaleniUprav3 >= $datum) {
            return 3;
        }
        if ($this ->schvaleniUprav2 >= $datum) {
            return 2;
        }
        if ($this ->schvaleniUprav1 >= $datum) {
            return 1;
        }

        return 0;
    }


    /**
     * Vraci schvaleni zadane urovne pro zadany rezim ($rezimUprav)
     * @param int  $level
     * @param bool $rezimUprav
     * @return int
     */
    function vratSchvaleni($level, $rezimUprav)
    {
        switch ($level) {
            case 3 :
                return $rezimUprav ? $this ->schvaleniUprav3 : $this ->schvaleni3;
            case 2 :
                return $rezimUprav ? $this ->schvaleniUprav2 : $this ->schvaleni2;
            default :
                return $rezimUprav ? $this ->schvaleniUprav1 : $this ->schvaleni1;
        }
    }


    function posledniSchvaleni($rezimUprav)
    {
        if ($rezimUprav) {
            return max($this->schvaleniUprav1, $this->schvaleniUprav2, $this->schvaleniUprav3);
        } else {
            return max($this->schvaleni1, $this->schvaleni2, $this->schvaleni3);
        }
    }


    /**
     * Vraci datum, do ktereho je schvalen puvodni plan. Musi byt schvalen minimalne nastavenou urovni
     * @return int
     */
    public function getSchvaleniPuvodnihoPlanu()
    {
        /** @var int $result */
        $result = null;

        require_once 'kapacity/kapacityutils.php';
        if (RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_MIN_SCHVALENI_PRO_UPRAVY)) {
            if ($this ->schvaleni1  &&  RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_MIN_SCHVALENI_PRO_UPRAVY) == 1) {
                $result = $this ->schvaleni1;
            }

            if ($this ->schvaleni2  &&  RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_MIN_SCHVALENI_PRO_UPRAVY) <= 2) {
                if (!$result  ||  $result < $this ->schvaleni2) {
                    $result = $this ->schvaleni2;
                }
            }

            if ($this ->schvaleni3) {
                if (!$result  ||  $result < $this ->schvaleni3) {
                    $result = $this ->schvaleni3;
                }
            }
        }

        return $result;
    }


    /**
     * Vraci HTML class pro zobrazeni priznaku schvaleni (podle nejvyssiho schvaleni v danem dni)
     * @param int  $datum
     * @param bool $rezimUprav
     * @return string
     */
    public function getHtmlClassKapacity($datum, $rezimUprav)
    {
        $uroven = $this ->vratUrovenSchvaleniKapacit($datum, $rezimUprav);
        if (!$uroven) return '';

        return 'schvaleni' . $uroven;
    }


    public function getJsonData($rezimUprav) {
        if ($rezimUprav) {
            return [
                'schvaleni1Str' => StringUtils::jsonDate($this ->schvaleniUprav1),
                'schvaleni2Str' => StringUtils::jsonDate($this ->schvaleniUprav2),
                'schvaleni3Str' => StringUtils::jsonDate($this ->schvaleniUprav3),
                'schvaleniPuvodnihoPlanuStr' =>StringUtils::jsonDate($this ->getSchvaleniPuvodnihoPlanu())
            ];
        } else {
            return [
                'schvaleni1Str' => StringUtils::jsonDate($this ->schvaleni1),
                'schvaleni2Str' => StringUtils::jsonDate($this ->schvaleni2),
                'schvaleni3Str' => StringUtils::jsonDate($this ->schvaleni3)
            ];
        }
    }
}



class SchvaleniKapacitPP extends SchvaleniKapacit
{
    /** @var int */
    var $idPP;

    public function lzeSchvalovatDen($uroven, $datum)
    {
        $pravo = $this ->vratPravoProUroven($uroven);
        return $pravo ->lzeZobrazitPP($this ->idPP, $datum, $datum);
    }

    /**
     * Nacte datumy schvaleni kapacit osoby v zadanem obdobi
     * @param Database $database
     * @param int $obdobi
     * @param int $idPP
     * @return SchvaleniKapacitPP
     */
    public static function nactiSchvaleni($database, $obdobi, $idPP)
    {
        $result = new static();

        $result ->schvaleni1 = $result ->schvaleni2 = $result ->schvaleni3 = 0;
        $result ->schvaleniUprav1 = $result ->schvaleniUprav2 = $result ->schvaleniUprav3 = 0;

        $params = $database ->createParameters();

        $sql =
            'SELECT ZpracovanoDo, ZkontrolovanoDo, SchvalenoDo, UpravyZpracovanyDo, UpravyZkontrolovanyDo, UpravySchvalenyDo' .
            ' FROM SchvaleniKapacit, PracovniPomer' .
            ' WHERE PracovniPomer.Id = ' . $params ->addParameter($idPP) .
            ' AND SchvaleniKapacit.RC = PracovniPomer.RC' .
            ' AND SchvaleniKapacit.OsobniCislo = PracovniPomer.OsobniCislo' .
            ' AND SchvaleniKapacit.DatumNastupu = PracovniPomer.DatumNastupu' .
            " AND SchvaleniKapacit.Aplikace = 'ADS'" .
            ' AND SchvaleniKapacit.PocatekObdobi = ' . $params ->addParameter($database ->dateToParam($obdobi));

        $cursor = $database ->getSqlResult($sql, $params);

        $row = $cursor ->fetch();
        if ($row) {
            $result ->schvaleni1 = $row[0] ? $database ->stringToTimestamp($row[0]) : 0;
            $result ->schvaleni2 = $row[1] ? $database ->stringToTimestamp($row[1]) : 0;
            $result ->schvaleni3 = $row[2] ? $database ->stringToTimestamp($row[2]) : 0;

            $result ->schvaleniUprav1 = $row[3] ? $database ->stringToTimestamp($row[3]) : 0;
            $result ->schvaleniUprav2 = $row[4] ? $database ->stringToTimestamp($row[4]) : 0;
            $result ->schvaleniUprav3 = $row[5] ? $database ->stringToTimestamp($row[5]) : 0;
        }

        return $result;
    }
}


class SchvaleniKapacitRozdeleni extends SchvaleniKapacit
{
    /** @var int */
    var $kodTypuRozdeleni;
    /** @var string */
    var $rozdeleni;


    public function lzeSchvalovatDen($uroven, $datum)
    {
        $pravo = $this ->vratPravoProUroven($uroven);
        return $pravo ->jePravoNaRozdeleni($this ->kodTypuRozdeleni, $this ->rozdeleni, $datum, $datum);
    }


    /**
     * @param Database $database
     * @param int    $pocatekObdobi
     * @param int    $typRozdeleni
     * @param string $kodRozdeleni
     * @return SchvaleniKapacitRozdeleni
     * @throws \Ron\DB\DBException
     */
    public static function nactiSchvaleni($database, $pocatekObdobi, $typRozdeleni, $kodRozdeleni)
    {
        $result = new static();

        $result ->schvaleni1 = $result ->schvaleni2 = $result ->schvaleni3 = 0;
        $result ->schvaleniUprav1 = $result ->schvaleniUprav2 = $result ->schvaleniUprav3 = 0;

        $params = $database ->createParameters();
        $sql =
            'SELECT Schvaleni1, Schvaleni2, Schvaleni3, SchvaleniUprav1, SchvaleniUprav2, SchvaleniUprav3' .
            ' FROM SchvaleniPlanuRozdeleni' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter($typRozdeleni) .
            ' AND Rozdeleni = ' . $params ->addParameter($kodRozdeleni) .
            " AND Aplikace = 'ADS'" .
            ' AND PocatekObdobi = ' . $params ->addParameter($database ->dateToParam($pocatekObdobi));

        $cursor = $database ->getSqlResult($sql, $params);

        $row = $cursor ->fetch();
        if ($row) {
            $result ->schvaleni1 = $row[0] ? $database ->stringToTimestamp($row[0]) : 0;
            $result ->schvaleni2 = $row[1] ? $database ->stringToTimestamp($row[1]) : 0;
            $result ->schvaleni3 = $row[2] ? $database ->stringToTimestamp($row[2]) : 0;

            $result ->schvaleniUprav1 = $row[3] ? $database ->stringToTimestamp($row[3]) : 0;
            $result ->schvaleniUprav2 = $row[4] ? $database ->stringToTimestamp($row[4]) : 0;
            $result ->schvaleniUprav3 = $row[5] ? $database ->stringToTimestamp($row[5]) : 0;
        }

        return $result;
    }

}
