<?php
/**
 * UTF8
 * stanislav.elbl, 22.10.2015
 */

namespace Ron\Kapacity;

use Ron\DateUtils;
use Ron\DBRecordField;
use Ron\RonPortal\AgentManager;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\DB\Database;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\RonPortalScriptEvents;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\RonPortalXMLEvents;
use Ron\RonPortal\Rozdeleni;
use Ron\StringUtils;

require_once('adsapp/ronportaldbrecord.php');
require_once('adsapp/ronportalutils.php');

abstract class SchvaleniKapacitAbstractRecord extends RonPortalDBRecord
{
    /**
     * @var int
     */
    var $obdobi;


    /** @var DBRecordField */
    var $fSchvaleni1;
    /** @var DBRecordField */
    var $fSchvalil1;
    /** @var DBRecordField */
    var $fCas_Schvaleni1;

    /** @var DBRecordField */
    var $fSchvaleni2;
    /** @var DBRecordField */
    var $fSchvalil2;
    /** @var DBRecordField */
    var $fCas_Schvaleni2;

    /** @var DBRecordField */
    var $fSchvaleni3;
    /** @var DBRecordField */
    var $fSchvalil3;
    /** @var DBRecordField */
    var $fCas_Schvaleni3;

    /** @var DBRecordField */
    var $fSchvaleniUprav1;
    /** @var DBRecordField */
    var $UpravySchvalil1;
    /** @var DBRecordField */
    var $Cas_SchvaleniUprav1;

    /** @var DBRecordField */
    var $fSchvaleniUprav2;
    /** @var DBRecordField */
    var $UpravySchvalil2;
    /** @var DBRecordField */
    var $Cas_SchvaleniUprav2;

    /** @var DBRecordField */
    var $fSchvaleniUprav3;
    /** @var DBRecordField */
    var $UpravySchvalil3;
    /** @var DBRecordField */
    var $Cas_SchvaleniUprav3;

    /** @var DBRecordField */
    var $fPoznamka;

    /**
     * @param Database $database
     * @param string $tableName
     * @param int $obdobi
     */
    function __construct($database, $tableName, $obdobi)
    {
        parent::__construct($database, $tableName, StringUtils::fromUTF8('Plánování kapacit'));

        $this ->obdobi = $obdobi;

        $this ->addField('Aplikace', '', true, 0, 'ADS');
        $this ->addField('PocatekObdobi', 'date', true, 0, $obdobi);
    }

    /**
     * @param PravaNaOsoby $pravo
     * @return bool
     */
    abstract function zkontrolujPravo($pravo);

    /**
     * @param int $uroven
     * @param int $datumDo
     */
    abstract function overKontroly($uroven, $datumDo);

    /**
     * Zkontroluje, jestli ma uzivatel pravo na schvaleni. Zaznam uz musi byt nacten!
     * @param int  $uroven
     * @param int  $hodnota
     * @param int  $hodnotaOriginal   Puvodni hodnota - pro kontrolu prava na snizeni
     * @throws \Exception
     */
    function zkontrolujPravoNaSchvaleni($uroven, $hodnota, $hodnotaOriginal)
    {
        $prava = Prava::vratPrava('kapac_schval' . $uroven);
        if (!$this->zkontrolujPravo($prava)) {
            // vubec nema pravo
            RonPortalUtils::neniPravoException();
        }


        if ((int)$hodnota < (int)$hodnotaOriginal) {
            // jedna se o snizeni - overi jestli muze snizit datum
            if ($prava ->priznak) {
                // uzivatel nesmi snizit tuto uroven
                throw new \Exception(\Ron\ronText('Nelze_snizit_schvaleni_PK'));
            }

            if ($uroven < 3 && RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_ZAKAZAT_SNIZIT_SCHVALENI)) {
                // uzivatel snizuje - provede se kontrola, jestli se nejedna o nejvyssi uroven, na kterou ma pravo
                $prava = Prava::vratPrava('kapac_schval3');
                if ($this->zkontrolujPravo($prava)) {
                    // ma pravo na uroven 3 => muze snizovat
                    return;
                }

                if ($uroven == 1) {
                    $prava = Prava::vratPrava('kapac_schval2');
                    if ($this->zkontrolujPravo($prava)) {
                        // ma pravo na uroven 2 => muze snizovat uroven 1
                        return;
                    }
                }

                throw new \Exception(\Ron\ronText('Nelze_snizit_schvaleni_PK'));
            }
        }
    }


    /**
     * @param int $uroven
     * @param bool $rezimUprav
     * @param int $datumDo
     * @return string
     */
    public function getParametrSkriptu($uroven, $rezimUprav, $datumDo)
    {
        $datumDoStr = $datumDo ? date('j.n.Y', $datumDo) : '0';
        return
            "uroven=$uroven;obdobi=" . date('j.n.Y', $this ->obdobi) .
            ';datum=' . $datumDoStr .
            ';rezimuprav=' . ($rezimUprav ? '1' : '0');
    }


    public function getAgentParameters($uroven, $rezimUprav, $hodnota)
    {
        //$hodnotaStr = date('Y-n-j', $hodnota ?: DateUtils::incDay($this ->obdobi, -1));
        $hodnotaStr = date('Y-n-j', $hodnota ?: $this ->obdobi);
        return
            ' -period:' . date('Y-n-j', $this ->obdobi) .
            ' -apprLevel:' . $uroven .
            ($rezimUprav ? ' -modificationMode' : '') .
            ' -dateTo:' . $hodnotaStr;
    }

    /**
     * @param Database $database
     * @param int $uroven
     * @param bool $rezimUprav
     * @param int $hodnota  Datum, do kdy se ma schvalit
     * @throws \Exception
     */
    public function schvalKapacityAgentem($database, $uroven, $rezimUprav, $hodnota)
    {
        if (!RonPortalSettings::getSpousteniADS()) {
            throw new \Exception('Chyba ve funkci SchvaleniKapacitDBRecord::schvalKapacityAgentem - neni povoleno spousteni agentu!');
        }

        $parametrskriptu = $this ->getParametrSkriptu($uroven, $rezimUprav, $hodnota);

        require_once('adsapp/agentmanager.php');
        require_once('adsapp/events.php');
        AgentManager::executeScriptEvents($database, RonPortalScriptEvents::CATEGORY_KAPACITY, RonPortalScriptEvents::KAPACITY_SCHVALENI_BEFORE, $parametrskriptu);

        // prepocet kontrol planovani
        AgentManager::executeADS(
            'ADS_Agent_CapacityPlanning.exe', $this ->getAgentParameters($uroven, $rezimUprav, $hodnota));
    }


    /**
     * @param int  $uroven
     * @param bool $rezimUprav
     * @param int  $hodnota
     * @return bool
     * @throws \Exception
     */
    function zapisSchvaleni($uroven, $rezimUprav, $hodnota)
    {
        // nacte existujici zaznam
        $exists = $this ->loadRecord();
        $hodnotaOriginal = 0;
        $xmlContext = null;

        switch ($rezimUprav ? $uroven + 10 : $uroven) {
            case 1 :
                $this ->fSchvaleni1 ->setNewValue($hodnota);
                $hodnotaOriginal = $this ->fSchvaleni1 ->originalValue;
                $this ->fSchvalil1 ->setNewValue( RonPortalUtils::getLogUserName() );
                $this ->fCas_Schvaleni1 ->setNewValue( time() );
                $xmlContext = 'schvaleni1';
                break;

            case 2 :
                $this ->fSchvaleni2 ->setNewValue($hodnota);
                $hodnotaOriginal = $this ->fSchvaleni2 ->originalValue;
                $this ->fSchvalil2 ->setNewValue( RonPortalUtils::getLogUserName() );
                $this ->fCas_Schvaleni2 ->setNewValue( time() );
                $xmlContext = 'schvaleni2';
                break;

            case 3 :
                $this ->fSchvaleni3 ->setNewValue($hodnota);
                $hodnotaOriginal = $this ->fSchvaleni3 ->originalValue;
                $this ->fSchvalil3 ->setNewValue( RonPortalUtils::getLogUserName() );
                $this ->fCas_Schvaleni3 ->setNewValue( time() );
                $xmlContext = 'schvaleni3';
                break;

            case 11 :
                $this ->fSchvaleniUprav1 ->setNewValue($hodnota);
                $hodnotaOriginal = $this ->fSchvaleniUprav1 ->originalValue;
                $this ->UpravySchvalil1 ->setNewValue( RonPortalUtils::getLogUserName() );
                $this ->Cas_SchvaleniUprav1 ->setNewValue( time() );
                $xmlContext = 'schvaleniuprav1';
                break;

            case 12 :
                $this ->fSchvaleniUprav2 ->setNewValue($hodnota);
                $hodnotaOriginal = $this ->fSchvaleniUprav2 ->originalValue;
                $this ->UpravySchvalil2 ->setNewValue( RonPortalUtils::getLogUserName() );
                $this ->Cas_SchvaleniUprav2 ->setNewValue( time() );
                $xmlContext = 'schvaleniuprav2';
                break;

            case 13 :
                $this ->fSchvaleniUprav3 ->setNewValue($hodnota);
                $hodnotaOriginal = $this ->fSchvaleniUprav3 ->originalValue;
                $this ->UpravySchvalil3 ->setNewValue( RonPortalUtils::getLogUserName() );
                $this ->Cas_SchvaleniUprav3 ->setNewValue( time() );
                $xmlContext = 'schvaleniuprav3';
                break;
        }

        // zkontroluje prava
        $this ->zkontrolujPravoNaSchvaleni($uroven, $hodnota, $hodnotaOriginal);

        if (!$this ->isChanged()) {
            // nedoslo ke zmene
            return;
        }

        $parametrskriptu = $this ->getParametrSkriptu($uroven, $rezimUprav, $hodnota);

        require_once 'adsapp/agentmanager.php';
        require_once 'adsapp/events.php';

        $this ->executeXmlEvents(RonPortalXMLEvents::KAPACITY_SCHVALENI_BEFORE, $xmlContext);
        AgentManager::executeScriptEvents($this ->database, RonPortalScriptEvents::CATEGORY_KAPACITY, RonPortalScriptEvents::KAPACITY_SCHVALENI_BEFORE, $parametrskriptu);

        $pouzitAgenta = (KapacityUtils::pouzivatKontrolyPlanovani()  &&  $hodnota); // agent (zatim) neumi "odschvalit" //  X nepodporuje rezim uprav - uz neplati X
//        if (!$pouzitAgenta) {
//            // schvaleni celeho rozdeleni se vzdy provede agentem
//            $pouzitAgenta = !!RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_ZPUSOB_SCHVALOVANI);
//        }

        if ($pouzitAgenta) {
            $this ->schvalKapacityAgentem($this ->database, $uroven, $rezimUprav, $hodnota);
        } else {
            if ($hodnota) {
                $this->overKontroly($uroven, $hodnota);
            }

            $result = $exists
                ? $this ->updateRecord()
                : $this ->insertRecord();

            if (!$result) {
                throw new \Exception('Chyba pri zapisu zaznamu schvaleni!', null, $this->lastError);
            }
        }


        $this ->executeXmlEvents(RonPortalXMLEvents::KAPACITY_SCHVALENI_AFTER, $xmlContext);
        AgentManager::executeScriptEvents($this ->database, RonPortalScriptEvents::CATEGORY_KAPACITY, RonPortalScriptEvents::KAPACITY_SCHVALENI_AFTER, $parametrskriptu);

        return;
    }
}


class SchvaleniKapacitDBRecord extends SchvaleniKapacitAbstractRecord
{
    /** @var PracovniPomer */
    var $pp;

    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $obdobi
     */
    function __construct($database, $pp, $obdobi)
    {
        parent::__construct($database, 'SchvaleniKapacit', $obdobi);

        $this ->pp = $pp;

        $this ->addField('RC', '', true, 15, $pp ->rc); // readonly = true
        $this ->addField('OsobniCislo', '', true, 15, $pp ->osobniCislo); // readonly = true
        $this ->addField('DatumNastupu', 'date', true, 0, $pp ->datumNastupu);

        $this ->fSchvaleni1 = $this ->addField('ZpracovanoDo', 'date', true);
        $this ->fSchvalil1 = $this ->addField('Zpracoval', '', true, 25);
        $this ->fSchvalil1 ->updateLog = true;
        $this ->fCas_Schvaleni1 = $this ->addField('Cas_Zpracovano', 'timestamp', true);
        $this ->fCas_Schvaleni1 ->updateLog = true;

        $this ->fSchvaleni2 = $this ->addField('ZkontrolovanoDo', 'date', true);
        $this ->fSchvalil2 = $this ->addField('Zkontroloval', '', true, 25);
        $this ->fSchvalil2 ->updateLog = true;
        $this ->fCas_Schvaleni2 = $this ->addField('Cas_Zkontrolovano', 'timestamp', true);
        $this ->fCas_Schvaleni2 ->updateLog = true;

        $this ->fSchvaleni3 = $this ->addField('SchvalenoDo', 'date', true);
        $this ->fSchvalil3 = $this ->addField('Schvalil', '', true, 25);
        $this ->fSchvalil3 ->updateLog = true;
        $this ->fCas_Schvaleni3 = $this ->addField('Cas_Schvaleno', 'timestamp', true);
        $this ->fCas_Schvaleni3 ->updateLog = true;

        $this ->fSchvaleniUprav1 = $this ->addField('UpravyZpracovanyDo', 'date', true);
        $this ->UpravySchvalil1 = $this ->addField('UpravyZpracoval', '', true, 25);
        $this ->UpravySchvalil1 ->updateLog = true;
        $this ->Cas_SchvaleniUprav1 = $this ->addField('Cas_UpravyZpracovany', 'timestamp', true);
        $this ->Cas_SchvaleniUprav1 ->updateLog = true;

        $this ->fSchvaleniUprav2 = $this ->addField('UpravyZkontrolovanyDo', 'date', true);
        $this ->UpravySchvalil2 = $this ->addField('UpravyZkontroloval', '', true, 25);
        $this ->UpravySchvalil2 ->updateLog = true;
        $this ->Cas_SchvaleniUprav2 = $this ->addField('Cas_UpravyZkontrolovany', 'timestamp', true);
        $this ->Cas_SchvaleniUprav2 ->updateLog = true;

        $this ->fSchvaleniUprav3 = $this ->addField('UpravySchvalenyDo', 'date', true);
        $this ->UpravySchvalil3 = $this ->addField('UpravySchvalil', '', true, 25);
        $this ->UpravySchvalil3 ->updateLog = true;
        $this ->Cas_SchvaleniUprav3 = $this ->addField('Cas_UpravySchvaleny', 'timestamp', true);
        $this ->Cas_SchvaleniUprav3 ->updateLog = true;

        $params = $database ->createParameters();
        $this ->setWhere(
            ' RC = ' . $params ->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            " AND Aplikace = 'ADS'" .
            ' AND PocatekObdobi = ' . $params ->addParameter($database ->dateToParam($obdobi)),
            $params);
    }


    /**
     * @param PravaNaOsoby $pravo
     * @return bool
     */
    function zkontrolujPravo($pravo)
    {
        return $pravo ->lzeZobrazitPP($this ->pp ->idPP);
    }


    /**
     * @param int $uroven
     * @param int $datumDo
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function overKontroly($uroven, $datumDo)
    {
        // overi kontroly osoby
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT VysledekKontrolyPlanovani.Datum, TypKontrolyPlanovani.Popis' .
            ' FROM VysledekKontrolyPlanovani, TypKontrolyPlanovani' .
            ' WHERE VysledekKontrolyPlanovani.TypKontroly = TypKontrolyPlanovani.Kod' .
            ' AND VysledekKontrolyPlanovani.Ignorovat = 0' .
            ' AND TypKontrolyPlanovani.BlokovatSchvaleni' . (int)$uroven . ' = 1' .
            ' AND VysledekKontrolyPlanovani.Datum >= ' . $params ->addParameter($this ->database ->dateToParam($this ->obdobi)) .
            ' AND VysledekKontrolyPlanovani.Datum <= ' . $params ->addParameter($this ->database ->dateToParam($datumDo)) .

            ' AND VysledekKontrolyPlanovani.RC = ' . $params ->addParameter($this ->pp ->rc) .
            ' AND VysledekKontrolyPlanovani.OsobniCislo = ' . $params ->addParameter($this ->pp ->osobniCislo) .
            ' AND VysledekKontrolyPlanovani.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu)) .

            ' ORDER BY VysledekKontrolyPlanovani.Datum';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            throw new \Exception( \Ron\ronText('Nelze_schvalit_plan') . "\n" . StringUtils::format('date', $this ->database ->stringToTimestamp($row[0])) . ': ' . $row[1]);
        }

        // TODO: zkontrolovat i rozdeleni, na ktere je prerazen!

        // overi kontroly rozdeleni
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT VysledekKontrolyPlanovani.Datum, TypKontrolyPlanovani.Popis' .
            ' FROM VysledekKontrolyPlanovani, TypKontrolyPlanovani, RozdeleniOsob' .
            ' WHERE VysledekKontrolyPlanovani.TypKontroly = TypKontrolyPlanovani.Kod' .
            ' AND VysledekKontrolyPlanovani.Ignorovat = 0' .
            ' AND VysledekKontrolyPlanovani.Datum >= ' . $params ->addParameter($this ->database ->dateToParam($this ->obdobi)) .
            ' AND VysledekKontrolyPlanovani.Datum <= ' . $params ->addParameter($this ->database ->dateToParam($datumDo)) .
            ' AND TypKontrolyPlanovani.BlokovatSchvaleni' . (int)$uroven . ' = 1' .

            ' AND VysledekKontrolyPlanovani.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni' .
            ' AND VysledekKontrolyPlanovani.Rozdeleni = RozdeleniOsob.Rozdeleni' .

            ' AND RozdeleniOsob.RC = ' . $params ->addParameter($this ->pp ->rc) .
            ' AND RozdeleniOsob.OsobniCislo = ' . $params ->addParameter($this ->pp ->osobniCislo) .
            ' AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu)) .

            ' AND RozdeleniOsob.PlatnostOd <= VysledekKontrolyPlanovani.Datum' .
            ' AND (RozdeleniOsob.PlatnostDo >= VysledekKontrolyPlanovani.Datum  OR  RozdeleniOsob.PlatnostDo IS NULL)' .

            ' ORDER BY VysledekKontrolyPlanovani.Datum';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            throw new \Exception( \Ron\ronText('Nelze_schvalit_plan') . "\n" . StringUtils::format('date', $this ->database ->stringToTimestamp($row[0])) . ': ' . $row[1]);
        }
    }


    /**
     * @param int $uroven
     * @param bool $rezimUprav
     * @param int $datumDo
     * @return string
     */
    public function getParametrSkriptu($uroven, $rezimUprav, $datumDo)
    {
        return
            "RC={$this ->pp ->rc};OsobniCislo={$this ->pp ->osobniCislo};DatumNastupu=" . date('j.n.Y', $this ->pp ->datumNastupu) . ';' .
            parent::getParametrSkriptu($uroven, $rezimUprav, $datumDo) .
            ";IdPP={$this ->pp ->idPP}";
    }


    public function getAgentParameters($uroven, $rezimUprav, $hodnota)
    {
        return '1 -IDPP:' . $this ->pp ->idPP  . parent::getAgentParameters($uroven, $rezimUprav, $hodnota);
    }


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $uroven
     * @param int $obdobi
     * @param bool $rezimUprav
     * @param int $hodnota  Datum, do kdy se ma schvalit
     * @throws \Exception
     */
    public static function schvalKapacity($database, $pp, $uroven, $obdobi, $rezimUprav, $hodnota)
    {
        require_once('kapacity/kapacityutils.php');
        $rec = new static($database, $pp, $obdobi);
        $rec->zapisSchvaleni($uroven, $rezimUprav, $hodnota);
    }
}




class SchvaleniPlanuRozdeleniDBRecord extends SchvaleniKapacitAbstractRecord
{
    /** @var Rozdeleni */
    var $rozdeleni;

    /**
     * @param Database $database
     * @param Rozdeleni $rozdeleni
     * @param int $obdobi
     */
    function __construct($database, $rozdeleni, $obdobi)
    {
        parent::__construct($database, 'SchvaleniPlanuRozdeleni', $obdobi);

        $this ->rozdeleni = $rozdeleni;

        $this ->addField('KodTypuRozdeleni', 'int', true, 0, $rozdeleni ->typ); // readonly = true
        $this ->addField('Rozdeleni', '', true, 30, $rozdeleni ->kod); // readonly = true

        $this ->fSchvaleni1 = $this ->addField('Schvaleni1', 'date', true);
        $this ->fSchvalil1 = $this ->addField('Schvalil1', '', true, 25);
        $this ->fSchvalil1 ->updateLog = true;
        $this ->fCas_Schvaleni1 = $this ->addField('Cas_Schvaleni1', 'timestamp', true);
        $this ->fCas_Schvaleni1 ->updateLog = true;

        $this ->fSchvaleni2 = $this ->addField('Schvaleni2', 'date', true);
        $this ->fSchvalil2 = $this ->addField('Schvalil2', '', true, 25);
        $this ->fSchvalil2 ->updateLog = true;
        $this ->fCas_Schvaleni2 = $this ->addField('Cas_Schvaleni2', 'timestamp', true);
        $this ->fCas_Schvaleni2 ->updateLog = true;

        $this ->fSchvaleni3 = $this ->addField('Schvaleni3', 'date', true);
        $this ->fSchvalil3 = $this ->addField('Schvalil3', '', true, 25);
        $this ->fSchvalil3 ->updateLog = true;
        $this ->fCas_Schvaleni3 = $this ->addField('Cas_Schvaleni3', 'timestamp', true);
        $this ->fCas_Schvaleni3 ->updateLog = true;

        $this ->fSchvaleniUprav1 = $this ->addField('SchvaleniUprav1', 'date', true);
        $this ->UpravySchvalil1 = $this ->addField('UpravySchvalil1', '', true, 25);
        $this ->UpravySchvalil1 ->updateLog = true;
        $this ->Cas_SchvaleniUprav1 = $this ->addField('Cas_SchvaleniUprav1', 'timestamp', true);
        $this ->Cas_SchvaleniUprav1 ->updateLog = true;

        $this ->fSchvaleniUprav2 = $this ->addField('SchvaleniUprav2', 'date', true);
        $this ->UpravySchvalil2 = $this ->addField('UpravySchvalil2', '', true, 25);
        $this ->UpravySchvalil2 ->updateLog = true;
        $this ->Cas_SchvaleniUprav2 = $this ->addField('Cas_SchvaleniUprav2', 'timestamp', true);
        $this ->Cas_SchvaleniUprav2 ->updateLog = true;

        $this ->fSchvaleniUprav3 = $this ->addField('SchvaleniUprav3', 'date', true);
        $this ->UpravySchvalil3 = $this ->addField('UpravySchvalil3', '', true, 25);
        $this ->UpravySchvalil3 ->updateLog = true;
        $this ->Cas_SchvaleniUprav3 = $this ->addField('Cas_SchvaleniUprav3', 'timestamp', true);
        $this ->Cas_SchvaleniUprav3 ->updateLog = true;

        $params = $database ->createParameters();
        $this ->setWhere(
            ' KodTypuRozdeleni = ' . $params ->addParameter($rozdeleni ->typ) .
            ' AND Rozdeleni = ' . $params->addParameter($rozdeleni ->kod) .
            " AND Aplikace = 'ADS'" .
            ' AND PocatekObdobi = ' . $params ->addParameter($database ->dateToParam($obdobi)),
            $params);
    }


    /**
     * @param PravaNaOsoby $pravo
     * @return bool
     */
    function zkontrolujPravo($pravo)
    {
        return $pravo ->jePravoNaRozdeleni($this ->rozdeleni ->typ, $this ->rozdeleni ->kod);
    }


    /**
     * @param int $uroven
     * @param int $datumDo
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function overKontroly($uroven, $datumDo)
    {
        throw new \Exception('Schvaleni rozdeleni se musi provest agentem!');
    }


    /**
     * @param int $uroven
     * @param bool $rezimUprav
     * @param int $datumDo
     * @return string
     */
    public function getParametrSkriptu($uroven, $rezimUprav, $datumDo)
    {
        return
            "KodTypuRozdeleni={$this ->rozdeleni ->typ};Rozdeleni={$this ->rozdeleni ->kod};" .
            parent::getParametrSkriptu($uroven, $rezimUprav, $datumDo);
    }


    public function getAgentParameters($uroven, $rezimUprav, $hodnota)
    {
        return '2 -subDiv:' . $this ->rozdeleni ->typ . '/' . $this ->rozdeleni ->kod  . parent::getAgentParameters($uroven, $rezimUprav, $hodnota);
    }


    /**
     * @param Database $database
     * @param Rozdeleni $rozdeleni
     * @param int $uroven
     * @param int $obdobi
     * @param bool $rezimUprav
     * @param int $hodnota  Datum, do kdy se ma schvalit
     * @throws \Exception
     */
    public static function schvalKapacity($database, $rozdeleni, $uroven, $obdobi, $rezimUprav, $hodnota)
    {
        require_once('kapacity/kapacityutils.php');
        $rec = new static($database, $rozdeleni, $obdobi);
        $rec->zapisSchvaleni($uroven, $rezimUprav, $hodnota);
    }
}
