<?php
/**
 * UTF8
 * User: pavel.rudol
 * Date: 13.6.2016
 * Time: 13:09
 */

namespace Ron\RonPortal\Zastupy;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DBRecordField;
use Ron\MessageManager;
use Ron\RonPortal\Log;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalReport;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\Smena;
use Ron\StringUtils;
use function Ron\ronText;

require_once('adsapp/reportutils.php');
require_once('adsapp/ronportaldbrecord.php');

class Zastup
{
    public $id;
    public $datum;
    public $casOd;
    public $casDo;

    public $jmeno;
    public $prijmeni;
    public $titul;
    public $osobniCislo;
    /** @var  int id PP přiřazeného zaměstnance */
    public $idPP;

    public $kodTypuRozdeleni;
    public $rozdeleni;
    public $rozdeleniPopis;
    public $pozice;
    public $pozicePopis;
    public $kategoriePozice;
    public $kategoriePozicePopis;
    public $duvodZadosti;
    public $poznamka;
    public $specifikace;

    /** @var int Typ schválení: 0 = nerozhodnuto, 1 = schvaleno, 2 = zamitnuto, 3 = zrusena zadost (jiz je prirazen zamestnanec) */
    public $typSchvaleni;
    public $datumSchvaleni;
    public $schvalil;

    /** @var  RonPortalReport[] seznam reportu zacinajicich kodem 'Zastup' */
    public $reportList;

    public $lzeEditovat = false;
    public $lzePriraditOsobu = false;
    public $lzeSchvalit = false;
    public $lzeZobrazit = false;
    public $lzeZrusitPlan = false;

    /** Vytvoří záznam v plánování kapacit podle žádosti o zástup. Pokud záznam pro daný den již existuje vypíše upozornění.
     * @param Database $database
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    public static function vlozPlanovaniKapacit($database, $rec)
    {
        require_once('dochazka/smena.php');

        $planKapacitRec = new RonPortalDBRecord($database,'PlanovaniKapacit');
        $planKapacitRec->logDescription = $rec->logDescription;
        $planKapacitRec->logText = StringUtils::fromUTF8("Přepsáno plánování kapacit pro zaměstnance: %s\n");

        $planKapacitRec->addField('RC', 'string', true, 15, $rec['RC']);
        $planKapacitRec->addField('OsobniCislo', 'string', true, 15, $rec['OsobniCislo']);
        $planKapacitRec->addField('DatumNastupu', 'timestamp', true, 0, $rec['DatumNastupu']);
        $planKapacitRec->addField('Datum', 'timestamp', true, 0, $rec['Datum']);

        $planKapacitRec->addField('Smena', 'string', false, 10, RonPortalSettings::getValue(ZastupyController::SMENA_PRO_PLANOVANI));
        $planKapacitRec->addField('Smena_Pocatek', 'time', false, 0, $rec['CasOd']);
        $planKapacitRec->addField('Smena_Konec', 'time', false, 0, $rec['CasDo']);
        //7.10.2016 zjistovani delky smeny podle nastaveni smeny
        $planKapacitRec->addField('Smena_Delka', 'time', false, 0, Smena::vratDelkuSmeny($database, $rec['CasOd'], $rec['CasDo'], RonPortalSettings::getValue(ZastupyController::SMENA_PRO_PLANOVANI)));
        $planKapacitRec->addField('KodTypuRozdeleni', 'int', false, 0, $rec['KodTypuRozdeleni']);
        $planKapacitRec->addField('Rozdeleni', 'string', false, 0, $rec['Rozdeleni']);

        $userLogin = new DBRecordField('Update_UserName', 'string', false, 25);
        $userLogin->updateLog = false;
        $userLogin->setNewValue(RonPortalUtils::getLogUserName());
        $planKapacitRec->addDBRecordField($userLogin);

        $updateTimestamp = new DBRecordField('Update_TimeStamp', 'timestamp');
        $updateTimestamp->updateLog = false;
        $updateTimestamp->setNewValue(time());
        $planKapacitRec->addDBRecordField($updateTimestamp);

        $params = $database ->createParameters();
        $planKapacitRec->setWhere(
            ' RC = ' . $params->addParameter($rec['RC']) .
            ' AND OsobniCislo = ' . $params->addParameter($rec['OsobniCislo']) .
            ' AND DatumNastupu = ' . $params ->addParameter($database->dateToParam($rec['DatumNastupu'])) .
            ' AND Datum = ' . $params ->addParameter($database->timestampToParam($rec['Datum'])),
            $params
        );

        $loaded = $planKapacitRec->loadRecord();

        if ($loaded === false) {
            throw new Exception('Chyba pri nacitani zaznamu planovani kapacit.');
        }
        elseif ($loaded === true) {
            //21.7.2016 je mozne priradit i zamestnance, ktery jiz ma naplanovanou smenu, jine rozdeleni, pri vkladani se pak zobrazi hlaska ze je zaznam prepsan

            $pracPomer = new PracovniPomer();
            $pracPomer->nactiPPPodleRC($database, $rec['RC'], $rec['OsobniCislo'], $rec['DatumNastupu']);

            MessageManager::addInfoMessage(sprintf(ronText('Zastup_prepsani_planu'), $pracPomer->jmenoPrijmeniKratke, StringUtils::format('date', $rec['Datum'])));

            if ($planKapacitRec->updateRecord() > 0) {
                Log::addRecordToLog($database, StringUtils::fromUTF8('Přepsání plánu kapacit zaměstnance: ') . $pracPomer->jmenoPrijmeniKratke, $rec->logDescription,
                    sprintf(StringUtils::fromUTF8(
                        "Přepsáno plánování kapacit pro zaměstnance: %s\n" .
                        "RČ: %s\n" .
                        "Osobní číslo: %s\n" .
                        "Datum nástupu: %s\n" .
                        "Datum: %s\n" .
                        "Původní směna: %s\n" .
                        "Původní začátek směny: %s\n" .
                        "Původní konec směny: %s\n" .
                        "Původní délka směny: %s\n" .
                        "Původní kód typu rozdělení: %s\n" .
                        "Původní rozdělení: %s"),
                        $pracPomer->jmenoPrijmeni, $pracPomer->rc, $pracPomer->osobniCislo, StringUtils::format('date', $pracPomer->datumNastupu),
                        StringUtils::format('date', $planKapacitRec['Datum']), $planKapacitRec->findFieldByName('Smena')->originalValue,
                        StringUtils::format('time', $planKapacitRec->findFieldByName('Smena_Pocatek')->originalValue),
                        StringUtils::format('time', $planKapacitRec->findFieldByName('Smena_Konec')->originalValue),
                        StringUtils::format('time', $planKapacitRec->findFieldByName('Smena_Delka')->originalValue),
                        $planKapacitRec->findFieldByName('KodTypuRozdeleni')->originalValue, $planKapacitRec->findFieldByName('Rozdeleni')->originalValue
                    )
                );
                return true;
            }

            //throw new \Exception('Pro vybraný den má již zaměstnanec přiřazen směnu!');
        }
        elseif ($loaded === 0) {
            return $planKapacitRec->insertRecord();
        }

        return false;
    }

    /** Smaže záznam v plánování kapacit pro zvolenou osobu a den.
     * Vraci true, pokud byl smazan, false pokud nebyl nalezen a v pripade chyby vyhodi vyjimku
     * @param Database $database
     * @param int $idPP
     * @param int $datum
     * @param string $logDescription Ve výchozím stavu informace o zrušení žádosti
     * @return bool
     * @throws DBException
     */
    public static function zrusPlanovaniKapacit($database, $idPP, $datum, $logDescription = 'Zrušení žádosti o zástup')
    {
        require_once('adsapp/ppomer.php');

        $pracPomer = new PracovniPomer();
        $pracPomer->nactiPPPodleId($database, $idPP);

        $planKapacitRec = new RonPortalDBRecord($database,'PlanovaniKapacit');
        $planKapacitRec->logDescription = $logDescription;

        $planKapacitRec->addField('RC', '', true);
        $planKapacitRec->addField('OsobniCislo', '', true);
        $planKapacitRec->addField('DatumNastupu', 'timestamp', true);
        $planKapacitRec->addField('Datum', 'timestamp', true);
        $planKapacitRec->addField('Smena', '', true);
        $planKapacitRec->addField('Smena_Pocatek', 'time', true);
        $planKapacitRec->addField('Smena_Konec', 'time', true);
        $planKapacitRec->addField('Smena_Delka', 'time', true);
        $planKapacitRec->addField('KodTypuRozdeleni', '', true);
        $planKapacitRec->addField('Rozdeleni', '', true);

        $params = $database ->createParameters();
        $planKapacitRec->setWhere(
            ' RC = ' . $params->addParameter($pracPomer->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pracPomer->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database->dateToParam($pracPomer->datumNastupu)) .
            ' AND Datum = ' . $params ->addParameter($database->timestampToParam($datum)),
            $params
        );

        return $planKapacitRec->deleteRecord();
    }

    /**
     * @param Database $database
     * @param array $row
     */
    private function nactiRadek($database, $row)
    {
        $this->id = $row[0];
        $this->datum = $database->stringToTimestamp($row[1]);
        $this->casOd = $database->stringToTimestamp($row[2]);
        $this->casDo = $database->stringToTimestamp($row[3]);

        $this->jmeno = $row[11];
        $this->prijmeni = $row[10];
        $this->titul = $row[15];
        $this->osobniCislo = $row[16];
        $this->idPP = $row[12];
        $this->poznamka = $row[13];
        $this->duvodZadosti = $row[18];
        $this->specifikace = $row[19];

        $this->kodTypuRozdeleni = $row[14];
        $this->rozdeleni = $row[6];
        $this->rozdeleniPopis = $row[8];
        $this->pozice = $row[7];
        $this->pozicePopis = $row[9];
        $this->kategoriePozice = $row[20];
        $this->kategoriePozicePopis = $row[21];

        $this->typSchvaleni = $row[17];
        $this->datumSchvaleni = $database->stringToTimestamp($row[4]);
        $this->schvalil = $row[5];

    }

    /**
     * @return bool
     * @throws Exception
     */
    public function lzeZobrazitZaznam()
    {
        $pravaEditovat = Prava::vratPrava('zastupy_vlozeni');
        $pravaSchvalit = Prava::vratPrava('zastupy_schvaleni');
        $pravaPriradit = Prava::vratPrava('zastupy_prideleni');
        $pravaZobrazit = Prava::vratPrava('zastupy_prohl');
        $pravaZrusitPlan = Prava::vratPrava('zastupy_zruseni');

        $this->lzeZobrazit = $pravaZobrazit->jePravoNaRozdeleni($this->kodTypuRozdeleni, $this->rozdeleni, $this->datum, $this->datum) ||
            $pravaZobrazit->lzeZobrazitPP($this->idPP, $this->datum, $this->datum);

        //editovat lze jen neschvalene zaznamy
        $this->lzeEditovat = ($this->typSchvaleni == 0) && $pravaEditovat->jePravoNaRozdeleni($this->kodTypuRozdeleni, $this->rozdeleni, $this->datum, $this->datum);
        $this->lzeSchvalit = ($this->typSchvaleni != 3) && $pravaSchvalit->jePravoNaRozdeleni($this->kodTypuRozdeleni, $this->rozdeleni, $this->datum, $this->datum);
        $this->lzePriraditOsobu = ($this->typSchvaleni == 1) &&
            $pravaPriradit->jePravoNaRozdeleni($this->kodTypuRozdeleni, $this->rozdeleni, $this->datum, $this->datum);
        $this->lzeZrusitPlan =
            ($this->typSchvaleni == 1) && $this->idPP && //zrusit lze schvalene zadosti s prirazenym zamestnancem
            $pravaZrusitPlan->jePravoNaRozdeleni($this->kodTypuRozdeleni, $this->rozdeleni, $this->datum, $this->datum) && //musi byt pravo na rozdeleni
            $pravaZrusitPlan->lzeZobrazitPP($this->idPP, $this->datum, $this->datum); //i pravo na prirazeneho zamestnance

        $lzeZobrazit = $pravaEditovat->jePravoNaRozdeleni($this->kodTypuRozdeleni, $this->rozdeleni, $this->datum, $this->datum);
        $lzeZobrazit = $lzeZobrazit || $this->lzeSchvalit || $pravaPriradit->jePravoNaRozdeleni($this->kodTypuRozdeleni, $this->rozdeleni, $this->datum, $this->datum);
        $lzeZobrazit = $lzeZobrazit || $this->lzeZobrazit || $this->lzeZrusitPlan;

        return $lzeZobrazit;
    }

    /** Načte zástup z databáze.
     * @param Database $database
     * @param int $id
     * @return null|Zastup
     * @throws DBException
     * @throws Exception
     */
    public static function nactiZastup($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT ZadostOZastup.ID, ZadostOZastup.Datum, ZadostOZastup.CasOd, ZadostOZastup.CasDo, " . // 0 - 3
            "ZadostOZastup.Schvaleni, ZadostOZastup.Schvalil, " . // 4, 5
            "ZadostOZastup.Rozdeleni, ZadostOZastup.Rozdeleni2, Rozdeleni.Popis, Pozice.Popis, Osoba.Jmeno, Osoba.Prijmeni, PracovniPomer.ID, ZadostOZastup.Poznamka, " . // 6 - 13
            "ZadostOZastup.KodTypuRozdeleni, Osoba.Titul, PracovniPomer.OsobniCislo, ZadostOZastup.TypSchvaleni, DuvodZastupu.Popis, ZadostOZastup.Specifikace " . //14 - 19
            (ZastupyController::$jeNovaVerzeADS ? ", ZadostOZastup.KodKategorie, Kategorie.Popis " : '' ). // 20 - 21
            "FROM ZadostOZastup LEFT JOIN Osoba ON (ZadostOZastup.RC = Osoba.RC) " .
            " LEFT JOIN PracovniPomer ON (PracovniPomer.RC = ZadostOZastup.RC AND PracovniPomer.OsobniCislo = ZadostOZastup.OsobniCislo AND PracovniPomer.DatumNastupu = ZadostOZastup.DatumNastupu) " .
            " LEFT JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = ZadostOZastup.KodTypuRozdeleni AND Rozdeleni.Kod = ZadostOZastup.Rozdeleni) " .
            " LEFT JOIN Rozdeleni AS Pozice ON (Pozice.KodTypuRozdeleni = ZadostOZastup.KodTypuRozdeleni2 AND Pozice.Kod = ZadostOZastup.Rozdeleni2)" .
            " LEFT JOIN DuvodZastupu ON (DuvodZastupu.Kod = ZadostOZastup.DuvodZadosti)" .
            (ZastupyController::$jeNovaVerzeADS ? " LEFT JOIN Kategorie ON (Kategorie.TypKategorie = ZadostOZastup.TypKategorie AND Kategorie.Kod = ZadostOZastup.KodKategorie)" : '') .
            "WHERE ZadostOZastup.ID = " . $params->addParameter($id);

        $cursor = $database->getSqlResult($sql,$params);
        MessageManager::addDebugLog($sql);

        $row = $cursor->fetch();

        if ($row) {
            $zastup = new Zastup();
            $zastup->nactiRadek($database, $row);

            $zastup->lzeZobrazitZaznam();

            $zastup->reportList = RonPortalReport::loadFormReports($database, 'Zastup', 'Zastup');

            return $zastup;
        }

        throw new Exception('Zaznam zadosti o zastup nenalezen.');
    }

    /**
     * @return string Příjmení Jméno, Titul
     */
    public function vratJmenoPrijmeni()
    {
        return $this->prijmeni . ' ' . $this->jmeno . ($this->titul ? ', ' . $this->titul : '');
    }
}