<?php
/**
 * UTF8
 * User: pavel.rudol
 * Date: 13.6.2016
 * Time: 13:10
 */

namespace Ron\RonPortal\Zastupy;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DBRecordField;
use Ron\MessageManager;
use Ron\RonPortal\NeniPravoException;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\RonPortalXMLEvents;
use Ron\StringUtils;
use function Ron\ronText;

require_once('adsapp/ronportaldbrecord.php');
require_once('adsapp/events.php');

class ZastupRecord extends RonPortalDBRecord
{
    /** @var int */
    public $id;

    /** @var int  */
    public $typRozdeleni;

    /** @var  string */
    public $rozdeleni;

    /** @var DBRecordField */
    public $datumpZastupu;
    /** @var DBRecordField */
    public $rcfield;
    /** @var DBRecordField */
    public $osobniCislofield;
    /** @var DBRecordField */
    public $datumNastupufield;

    /** @var DBRecordField */
    public $schvaleni;

    /** @var  DBRecordField */
    public $typSchvaleni;

    public function __construct($database, $id)
    {
        parent::__construct($database, 'ZadostOZastup');
        $this->id = (int)$id;

        $this->logDescription = StringUtils::fromUTF8('Žádost o zástup');
        $this->where = 'ZadostOZastup.Id = ' . (int)$id;

        $this->addField('ZadostOZastup.Id', 'int', true); // readonly

        $this->typRozdeleni = $this->addField('ZadostOZastup.KodTypuRozdeleni', '', true); //readonly
        $this->rozdeleni = $this->addField('ZadostOZastup.Rozdeleni', '', true);

        $this->rcfield = $this->addField('ZadostOZastup.RC', '', true); // readonly
        $this->osobniCislofield = $this->addField('ZadostOZastup.OsobniCislo', '', true);
        $this->datumNastupufield = $this->addField('ZadostOZastup.DatumNastupu', 'date', true);

        $this->datumpZastupu = $this->addField('ZadostOZastup.Datum', 'date', true);

        $this->addField('ZadostOZastup.Poznamka', '', true);

        $this->schvaleni = $this->addField('ZadostOZastup.Schvaleni', 'datetime', true);
        $this->typSchvaleni = $this->addField('ZadostOZastup.TypSchvaleni', 'int', true);
    }

    /**
     * @param array $row
     * @return bool
     * @throws Exception
     */
    public function loadRecordValues($row)
    {
        if (!$this ->id) {
            throw new Exception('Chyba - neni zadano ID zaznamu zadosti o zastup!');
        }

        parent::loadRecordValues($row);

        return true;
    }

    /**
     * @param PravaNaOsoby $prava
     * @throws Exception
     */
    private function zkontrolujPrava($prava)
    {

        Prava::nastavObdobiProPrava($this ->datumpZastupu ->value, $this ->datumpZastupu ->value);

        if (!$prava ->jePravoNaRozdeleni($this->typRozdeleni->value,$this->rozdeleni->value)) {
            throw new NeniPravoException();
        }
    }


    /**
     * @param string $eventName
     * @param string $xmlContext
     * @return bool
     */
    private function xmlEventsZastup($eventName, $xmlContext)
    {
        return $this ->executeXmlEvents($eventName, $xmlContext, ['zadostozastup' => $this ->id]);
    }

    /**
     * @param PravaNaOsoby $prava
     * @param int $hodnota 0 = nerozhodnuto, 1 = schvaleno, 2 = zamitnuto, 3 = zrusena zadost (jiz je prirazen zamestnanec)
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    public function schval($prava, $hodnota)
    {
        $this->typSchvaleni->setNewValue($hodnota);

        $pomf = $this ->addField('Schvalil', '', false, 0, RonPortalUtils::getLogUserName());
        $pomf ->updateLog = true;

        $this->schvaleni->setNewValue(time());
        $this->schvaleni->updateLog = true;

        switch ($hodnota) {
            case 0 :
                $logstr = ronText("Zastup_zruseno_schvaleni_log");
                $scriptContext = "zrusitschvaleni_";
                break;

            case 1 :
                $logstr = ronText("Zastup_schvaleni_log");
                $scriptContext = "schvaleni_";
                break;

            case 2 :
                $logstr = ronText("Zastup_zamitnuti_log");
                $scriptContext = "zamitnuti_";
                break;

            case 3 :
                $logstr = ronText("Zastup_zruseni_log");
                $scriptContext = "zruseniplanu_";
                break;

            default :
                $scriptContext = null;
                $logstr = '';
        }

        if (!$this ->loadRecord()) {
            throw new Exception('Chyba - zadana zadost o zastup nebyla nalezena!');
        }

        if ($this->typSchvaleni->originalValue == 3) {
            throw new Exception( StringUtils::fromUTF8('Chyba - nelze měnit schválení u zamítnuté žádosti!') );
        }
        
        if (!$this ->isChanged()) return true; // neni co menit

        $this ->zkontrolujPrava($prava);

        if ($hodnota != 3) {
            if ($this->rcfield->value != '') {
                throw new Exception( StringUtils::fromUTF8('Chyba - nelze měnit schválení, je přiřazen zaměstnanec!') );
            }
        }
        else if ($this->rcfield->value == '') {
            throw new Exception( StringUtils::fromUTF8('Chyba - nelze zrušit žádost, která nemá přiřazeného zaměstnance!') );
        }

        $this ->logText = $logstr;

        if (!$this ->xmlEventsZastup(RonPortalXMLEvents::ZASTUP_SCHVALENI, $scriptContext.'before')) return false;

        if (!$this ->updateRecord()) return false;

        return $this ->xmlEventsZastup(RonPortalXMLEvents::ZASTUP_SCHVALENI, $scriptContext.'after');
    }

    /**
     * @return bool
     * @throws Exception
     */
    function zrusSchvaleni()
    {

        $this ->schvaleni ->setNewValue(null);

        $f = $this ->addField('Schvalil', '', false, 0, null);
        $f ->updateLog = true;

        if (!$this ->loadRecord()) {
            throw new Exception('Chyba - zadana zadost o zastup nebyla nalezena!');
        }

        if (!$this ->isChanged()) return true; // neni co menit

        $this ->zkontrolujPrava(Prava::vratPrava('zastupy_schvaleni'));

        $this ->logText = StringUtils::fromUTF8('Zrušeno schválení zástupu');

        if (!$this ->xmlEventsZastup(RonPortalXMLEvents::ZASTUP_SCHVALENI, 'zrusitschvaleni_before')) return false;

        if (!$this ->updateRecord()) return false;

        return $this ->xmlEventsZastup(RonPortalXMLEvents::ZASTUP_SCHVALENI, 'zrusitschvaleni_after');
    }

    /** Zrusi prirazeneho zamestnance
     * @return bool
     * @throws Exception
     */
    function zrusZamestnance()
    {
        $this->rcfield ->setNewValue(null);
        $this->osobniCislofield->setNewValue(null);
        $this->datumNastupufield->setNewValue(null);

        if (!$this ->loadRecord()) {
            throw new Exception('Chyba - zadana zadost o zastup nebyla nalezena!');
        }

        if (!$this ->isChanged()) return true; // neni co menit

        $this ->zkontrolujPrava(Prava::vratPrava('zastupy_prideleni'));

        $this ->logText = StringUtils::fromUTF8('Zrušen přiřazený zaměstnanec');

        return $this ->updateRecord();
    }

    /**
     * @param Database $database
     * @return bool
     */
    public static function hromadneSchvaleniZastupu($database)
    {
        // uzivatel odeslal hromadne schvaleni Zastupu
        $pocet = (int)$_POST['pocet'];

        $result = true;

        for ($i = 0;  $i < $pocet;  $i++) {
            $id = $_POST["idZastup_$i"];
            $rec = new ZastupRecord($database, $id);

            try {
                if (isset($_POST["NastavitSchvaleni_{$id}"]) && $_POST["NastavitSchvaleni_{$id}"]) {
                    if ($_POST["Schvaleni_{$id}"]) {
                        if (!$rec->schval(Prava::vratPrava('zastupy_schvaleni'),1)) $result = false;
                    } else {
                        if (!$rec->schval(Prava::vratPrava('zastupy_schvaleni'),0)) $result = false;
                    }
                }

            } catch (Exception $e) {
                $result = false;
                MessageManager::addErrorMessage($e ->getMessage());
            }
        }

        if (!$result) {
            MessageManager::addErrorMessage( StringUtils::fromUTF8('Při hromadném schvalování zástupů došlo k chybě!') );
        }

        return $result;
    }
}