<?php
/**
 * UTF8
 * User: pavel.rudol
 * Date: 29.4.2016
 * Time: 11:03
 */

namespace Ron\RonPortal\Zastupy;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\MVC\TranslationsOutputObject;
use Ron\MVC\ViewObject;
use Ron\RonPortal\Prava;
use Ron\RonPortal\ReportOutputObject;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\User;
use Ron\StringUtils;
use Ron\TextManager;
use function Ron\ronText;

require_once('kapacity/zastup.php');

class ZastupyController extends HttpController
{
    const TYP_ROZDELENI_POZICE = 'zastup_typ_rozdeleni_pozice';
    const POZICE_VYCET = 'zastup_pozice_vycet';
    const SMENA_PRO_PLANOVANI = 'zastup_smena_plan';
    const PREFEROVANY_TYP_ROZDELENI = 'zastup_preferovat_typ_rozdeleni';
    const PREFEROVANE_ROZDELENI = 'zastup_preferovat_typ_rozdeleni';
    const TYP_KATEGORIE_POZICE = 'zastup_typ_kategorie_pozice';
    const TYP_ROZDELENI_FILTR = 'zastup_filtrovat_typ_rozdeleni';

    /** @var bool priznak, zda jsou v databazi nova pole pro kategorii pozice */
    public static $jeNovaVerzeADS;

    private $database;

    private $pravaVlozit;
    private $pravaSchvalit;
    private $pravaPriraditPP;
    private $pravaZrusit;
    private $pravaZobrazit;

    /**
     * ZastupyController constructor.
     * @param Database $database
     * @throws Exception
     */
    public function __construct($database)
    {
        $this->database = $database;

        TextManager::getInstance()->loadStrings('zastupy.ini');

        HtmlHeaders::getInstance()->addScript('script/zastupy.js');
        HtmlHeaders::getInstance()->addScript('', '?menuid=zastup&action=translations&ver=' . HtmlHeaders::getInstance() ->versionId);

        Prava::nastavObdobiProPrava(time(),time());
        $this->pravaVlozit = Prava::nactiPravaUzivatele($this->database,User::getRC(),'zastupy_vlozeni');
        $this->pravaSchvalit = Prava::nactiPravaUzivatele($this->database,User::getRC(),'zastupy_schvaleni');
        $this->pravaPriraditPP = Prava::nactiPravaUzivatele($this->database, User::getRC(), 'zastupy_prideleni');
        $this->pravaZrusit = Prava::nactiPravaUzivatele($this->database, User::getRC(), 'zastupy_zruseni');
        $this->pravaZobrazit = Prava::nactiPravaUzivatele($this->database, User::getRC(), 'zastupy_prohl');

        self::$jeNovaVerzeADS = true;
    }

    /** Seznam žádostí o zástup.
     * @return ZastupyBrowser
     * @throws Exception
     */
    private function seznamZadosti()
    {
        require_once('kapacity/b_zastupy.php');
        $b = new ZastupyBrowser($this->database, isset($_GET['schvalovani'])  &&  $_GET['schvalovani']);
        $b->load();
        
        return $b;
    }

    /** Přesměrování na detail zástupu.
     * @param int $id
     * @return Redirect
     */
    private function redirectToView($id)
    {
        return new Redirect(null, '?menuid=zastup&action=detail&record='.$id);
    }

    /** Přesměrování na seznam žádostí.
     * @return Redirect
     */
    private function redirectToBrowser()
    {
        return new Redirect(null, '?menuid=zastup');
    }

    /** Zobrazení a zpracování formuláře pro vložení/přiřazení zástupu.
     * @param string $formAction
     * @return Redirect|ZastupyForm
     * @throws Exception
     */
    private function zastupyFormular($formAction)
    {
        if (!($this->pravaVlozit->jsouNejakePrava() || $this->pravaPriraditPP->jsouNejakePrava())) {
            RonPortalUtils::neniPravoException();
        }

        $prirazeniPP = false;
        $zastup = null;
        
        require_once('kapacity/f_zastupy.php');

        if (($formAction == 'edit') || ($formAction == 'save')) {
            $idZaznamu = isset($_POST['record']) ? $_POST['record'] : $_GET['record'];
            $zastup = Zastup::nactiZastup($this->database, $idZaznamu);

            if (is_null($zastup)) {
                throw new Exception('Zastup nebyl nalezen.');
            }

            if (isset($_GET['action']) && ($_GET['action'] == 'priradit')) {
                $prirazeniPP = true;
            }

            if ($prirazeniPP) {
                if (!($zastup->lzePriraditOsobu)) {
                    RonPortalUtils::neniPravoException();
                }

                //pokud uzivatel meni prirazenou osobu, je nutne puvodnimu zamestnanci zrusit plan
                if ($zastup->idPP) {
                    try {
                        $this->database->beginTransaction();
                        if (Zastup::zrusPlanovaniKapacit($this->database, $zastup->idPP, $zastup->datum, StringUtils::fromUTF8('Změna zaměstnance v žádosti o zástup'))) {
                            require_once('kapacity/zastup_record.php');
                            $zastupRec = new ZastupRecord($this->database, $zastup->id);
                            $zastupRec->zrusZamestnance();
                            $this->database->commitTransaction();
                        }
                    }
                    catch (Exception $e) {
                        $this->database->rollbackTransaction();
                        MessageManager::addErrorMessage($e);
                    }
                }
            }
            elseif (!$zastup->lzeEditovat) {
                RonPortalUtils::neniPravoException();
            }

        }

        $form = new ZastupyForm($this->database, $formAction, $this->pravaVlozit, $this->pravaSchvalit, $prirazeniPP, $zastup, $_GET['rozdeleni']);

        //pokud má uživatel právo na schválení při vložení, záznam se rovnou schválí
        if ($form->handleAction())
        {
            if ($form->id) {
                return $this->redirectToView($form->id);
            } else {
                return $this->redirectToBrowser();
            }
        }

        return $form;
    }

    /** Zobrazení detailu zástupu s možností schválení.
     * @param int $id
     * @return BaseViewObject
     * @throws Exception
     */
    private function zobrazDetailZaznamu($id)
    {
        $zastup = Zastup::nactiZastup($this->database,$id);

        if (!$zastup->lzeZobrazitZaznam()) {
            RonPortalUtils::neniPravoException();
        }

        Model::addData('zastup',$zastup);

        return new BaseViewObject('kapacity/zastup_detail.php');
    }

    /** Provede schválení, při úspěchu přesměruje na view, jinak zobrazí view.
     * @param int $id
     * @param int $hodnota 0 = nerozhodnuto, 1 = schvaleno, 2 = zamitnuto, 3 = zrusena zadost (jiz je prirazen zamestnanec)
     * @return Redirect|ViewObject
     * @throws Exception
     */
    private function schvaleni($id, $hodnota)
    {
        require_once('kapacity/zastup_record.php');
        $rec = new ZastupRecord($this ->database, $id);
        $result = $rec ->schval($hodnota == 3 ? $this->pravaZrusit : $this->pravaSchvalit, $hodnota);

        if ($result) {
            return $this ->redirectToView($id);
        } else {
            return $this ->zobrazDetailZaznamu($id);
        }
    }

    /**
     * @param int $id id zadosti
     * @return Redirect
     * @throws DBException
     */
    private function smazZastup($id)
    {
        $zastup = Zastup::nactiZastup($this->database, $id);

        try {
            if (!$zastup->lzeEditovat) {
                RonPortalUtils::neniPravoException();
            }

            require_once('kapacity/zastup_record.php');
            $rec = new ZastupRecord($this->database, $id);
            if ($rec->deleteRecord()) {
                MessageManager::addInfoMessage(ronText('Zastup_zruseni_ok'));
            } else {
                MessageManager::addDbErrorMessage($this->database, null, StringUtils::fromUTF8('Při rušení žádosti o zástup došlo k chybě!'));
            }

        } catch (Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
        }

        return $this->redirectToBrowser();
    }

    /** Provede smazání plánování kapacit pro stejný den a prac. poměr jako je zástup.
     * @param int $idZaznamu
     * @return Redirect|ViewObject
     * @throws Exception
     */
    private function zrusPlanovaniKapacit($idZaznamu)
    {
        $zastup = Zastup::nactiZastup($this->database, $idZaznamu);

        if (!$zastup->lzeZrusitPlan) {
            RonPortalUtils::neniPravoException();
        }

        $result = false;

        try {
            $this->database->beginTransaction();
            Zastup::zrusPlanovaniKapacit($this->database, $zastup->idPP, $zastup->datum);
            $result = $this->schvaleni($idZaznamu, 3);
        }
        catch (Exception $e) {
            $this->database->rollbackTransaction();
            MessageManager::addErrorMessage($e);
        }

        if ($result instanceof Redirect) {
            $this->database->commitTransaction();
            return $result;
        }
        else {
            $this->database->rollbackTransaction();
            return $this->zobrazDetailZaznamu($idZaznamu);
        }
    }

    /** Zobrazí zvolenou sestavu začínající v kódu 'Zastup'.
     * @param int $idZaznamu
     * @return ReportOutputObject
     * @throws Exception
     */
    private function report($idZaznamu)
    {
        $zastup = Zastup::nactiZastup($this->database, $idZaznamu);

        Prava::nastavObdobiProPrava($zastup->datum, $zastup->datum);

        if (!$zastup->lzeZobrazit) {
            RonPortalUtils::neniPravoException();
        }

        require_once('adsapp/reportserver.php');

        $result = new ReportOutputObject();
        $result->loadReportFromGet($this ->database, 'Zastup', false, 'Zastup');
        $result ->nastavObdobiOdDo($zastup->datum, $zastup->datum);
        $result->integerParams = ['IdZastupu' => $zastup->id];

        return $result;
    }

    /** Vrátí JsonObject se směnou a rozdělením z plánování kapacit, pokud existuje.
     * @param int $idPP
     * @param int $datum
     * @return JsonOutputObject
     * @throws Exception
     */
    private function kontrolaPlanovaniKapacit($idPP, $datum)
    {
        if (!$this->pravaPriraditPP->lzeZobrazitPP($idPP, $datum, $datum)) {
            RonPortalUtils::neniPravoException();
        }

        $params = $this->database->createParameters();

        $sql = "SELECT Smena, Rozdeleni 
                FROM PlanovaniKapacit 
                JOIN PracovniPomer ON (PracovniPomer.RC = PlanovaniKapacit.RC AND PracovniPomer.OsobniCislo = PlanovaniKapacit.OsobniCislo AND PracovniPomer.DatumNastupu = PlanovaniKapacit.DatumNastupu) 
                WHERE PracovniPomer.ID = " . $params->addParameter($idPP) .
                " AND PlanovaniKapacit.Datum = " . $params->addParameter($this->database->dateToParam($datum));

        MessageManager::addDebugLog($sql);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if ($row) {
            return new JsonOutputObject([
                'existujeplan' => true,
                'smena' => StringUtils::jsonText($row[0]),
                'rozdeleni' => StringUtils::jsonText($row[1])
            ]);
        }

        return new JsonOutputObject(['existujeplan' => false]);
    }


    /**
     * @param string $action
     * @return BaseViewObject|JsonOutputObject|Redirect|TranslationsOutputObject|ReportOutputObject|ZastupyBrowser|ZastupyForm
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_KAPACITY)) {
            throw new Exception('Tato funkce není povolena.');
        }
        
        if (isset($_POST['formaction'])) $formaction = $_POST['formaction'];
        else if (isset($_GET['formaction'])) $formaction = $_GET['formaction'];
        else $formaction = '';

        if ($formaction) {
            // formular (editace, insert)
            return $this ->zastupyFormular($formaction);
        }

        if (isset($_POST['hromadneSchvaleni'])  &&  $_POST['hromadneSchvaleni']) {
            require_once('kapacity/zastup_record.php');
            ZastupRecord::hromadneSchvaleniZastupu($this ->database);
        }

        switch ($_GET['action']) {
            case 'report' :
                return $this->report((int)$_GET['id']);

            case 'detail' :
                return $this->zobrazDetailZaznamu((int)$_GET['record']);

            case 'schval' :
                return $this->schvaleni((int)$_GET['record'], 1);

            case 'zamitni' :
                return $this->schvaleni((int)$_GET['record'], 2);

            case 'zrusplan' :
                return $this->zrusPlanovaniKapacit((int)$_GET['record']);

            case 'zrusitzastup' :
                return $this->smazZastup((int)$_GET['record']);

            case 'kontrolaplanu' :
                return $this->kontrolaPlanovaniKapacit((int)$_GET['idpp'], (int)$_GET['datum']);

            case 'translations' :   // texty pro javascripty
                return $this ->translations();

            default :
                return $this->seznamZadosti();
        }
    }
}
