<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 14.02.2019
 */

namespace Ron\RonPortal;


use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\StringUtils;
use function Ron\ronText;


/**
 * Class ProhlaseniPoplatnikaDane
 * @package Ron\RonPortal
 */
class ProhlaseniPoplatnikaDane
{
    public $rc;
    public $osobniCislo;
    public $datumNastupu;
    public $elektronickyPodpis;
    public $elektronickyPodpisDodatecne;
    public $elektronickyPodpisUcetni;
    public $prohlaseniPortalRC;
    public $prohlaseniPortalOsc;
    public $prohlaseniPortalDatNastup;
    public $prohlaseniPPID;

    public $elektronickyPodpisUcetniDodatecne;
    public $prohlaseniElektronicky;
    public $poznamkaProZamestnance;

    //RocniVyuctovaniDane
    public $rok;
    public $datumProhlaseni;
    public $datumProhlaseniPortal;
    public $prohlaseniPortalStav;
    public $provestRocniVyuctovani;
    /** @var int pole ProhlaseniPortal <br>
     * Kolonka na prohlášení: Prohlášení k dani činím 0 - prohlášení k dani ne 1 - prohlášení k dani ano */
    public $prohlaseniKDaniCinim;
    public $prohlaseniPortalZmenaStav;

    //Odpocet
    public $datumOdOdpocet;
    public $datumDoOdpocet;
    public $datumVlozeniNavrhu;
    public $zakladniOdpocet;
    public $odpocetNaStudenta;
    public $predpokladUrokuZUveru;
    public $naZdravotniStav;
    public $ztpp;

    //Jiny poplatnik
    public $vyzivujeJinyPoplatnik;
    public $jinyPoplatnikJmeno;
    public $jinyPoplatnikPrijmeni;
    public $jinyPoplatnikRC;
    public $jinyPoplatnikUplatnujeZv;
    public $jinyPoplatnikZamestnani;
    public $jinyPoplatnikUlice;
    public $jinyPoplatnikObec;
    public $jinyPoplatnikPSC;
    public $jinyPoplatnikNazevPlatce;
    public $jinyPoplatnikUlicePlatce;
    public $jinyPoplatnikObecPlatce;
    public $jinyPoplatnikPSCPlatce;

    //Odpocet navrh
    public $datumOdOdpocetNavrh;
    public $datumOdOdpocetNavrhSchvaleno;
    public $zakladniOdpocetNavrh;
    public $odpocetNaStudentaNavrh;
    public $naZdravotniStavNavrh;

    //Udaje nerezidenta
    public $danovyNerezident;
    public $cisloDanoveIdentifikace;
    public $typDanovehoIdentifikatoru;
    public $typDanovehoIdentifikatoru_popis;
    public $statDanoveRezidence;
    public $statDanoveRezidence_popis;


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param $obdobi
     * @param bool $testProhlaseni
     * @return ProhlaseniPoplatnikaDane
     * @throws \Ron\DB\DBException
     * @throws Exception
     */
    public static function nactiProhlaseni($database, $pp, $obdobi, $testProhlaseni=false)
    {
        $params = $database->createParameters();
//        $aktualniRok = DateUtils::firstDayOfYear(DateUtils::today());
        $rok = DateUtils::firstDayOfYear($obdobi);
//        $datumOd = DateUtils::firstDayOfMonth($pp->datumNastupu);

        $datum1 = (mktime(0,0,0,1,1,date("Y", $obdobi)));
        $datum2 = (mktime(0,0,0,12,31,date("Y", $obdobi)));

        $sql = "SELECT";
        $sql .= " RVD.Rok, RVD.RC, RVD.DatumProhlaseni, RVD.ProhlaseniPortalStav, RVD.DatumProhlaseniPortal, RVD.ProvestRocniVyuctovani, ";      //0..5
        $sql .= " RVD.CisloDanoveIdentifikace, RVD.TypDanovehoIdentifikatoru, RVD.DanovyNerezident, RVD.StatDanoveRezidence, ";     //6..9
        $sql .= " Odpocet.DatumOd, Odpocet.DatumDo, Odpocet.ZakladniOdpocet, Odpocet.OdpocetNaStudenta, Odpocet.PredpokladUrokuZUveru, "; //10..14
        $sql .= " Odpocet.NaZdravotniStav, Odpocet.ZTPP, Odpocet.PoradiPrvnihoDitete, TypPrukazu.Popis, Stat.Popis, ";     //15..19
        $sql .= " OdpocetNavrh.DatumOd, OdpocetNavrh.ZakladniOdpocet, OdpocetNavrh.OdpocetNaStudenta, OdpocetNavrh.NaZdravotniStav, OdpocetNavrh.Stav, ";     //20..24
        $sql .= "(SELECT DatumOd From OdpocetNavrh Where DatumOd >= " . $params->addParameter($database->timestampToParam($datum1)) .
                " AND DatumOd <= " . $params->addParameter($database->timestampToParam($datum2)) . " AND Stav = 1 AND RC = " . $params->addParameter($pp->rc)
                . " AND OsobniCislo = " . $params->addParameter($pp->osobniCislo)
                . " AND DatumNastupu = " . $params->addParameter($database->timestampToParam($pp->datumNastupu)) . "), ";  //25
        $sql .= " {fn_CASTWIDEMEMO(RVD.ElektronickyPodpis)}, {fn_CASTWIDEMEMO(RVD.ElektronickyPodpisUcetni)}, 
                    RVD.ProhlaseniElektronicky, {fn_CASTWIDEMEMO(RVD.PoznamkaProZamestnance)}, ";     //26..29

        $sql .= "VyzivujeJinyPoplatnik, JinyPoplatnikJmeno, JinyPoplatnikPrijmeni, JinyPoplatnikRC, JinyPoplatnikUplatnujeZv, " .   //30..34
                "JinyPoplatnikZamestnani, JinyPoplatnikUlice, JinyPoplatnikObec, JinyPoplatnikPSC, " .                          //35..38
                "JinyPoplatnikNazevPlatce, JinyPoplatnikUlicePlatce, JinyPoplatnikObecPlatce, JinyPoplatnikPSCPlatce, RVD.ProhlaseniPortal, ";           //39..43

        $sql .= "RVD.ProhlaseniPortalZmenaStav, OdpocetNavrh.DatumVlozeniNavrhu, RVD.ProhlaseniPortalRC, RVD.ProhlaseniPortalOsc, RVD.ProhlaseniPortalDatNastup,  ";        //44 - 48

        $sql .= "(SELECT ID From PracovniPomer PP Where PP.RC = RVD.ProhlaseniPortalRC AND PP.OsobniCislo = RVD.ProhlaseniPortalOsc AND PP.DatumNastupu = RVD.ProhlaseniPortalDatNastup)";  //49

        $sql .= " FROM RocniVyuctovaniDane RVD LEFT JOIN Odpocet ON (Odpocet.RC = " . $params->addParameter($pp->rc)
                    . " AND Odpocet.OsobniCislo = " . $params->addParameter($pp->osobniCislo)
                    . " AND Odpocet.DatumNastupu = " . $params->addParameter($database->timestampToParam($pp->datumNastupu))
                    . " AND (Odpocet.DatumDo IS NULL OR DatumDo < RVD.DatumProhlaseni) )" .

                    
                    
                " LEFT JOIN TypPrukazu ON (RVD.TypDanovehoIdentifikatoru = TypPrukazu.Kod)";
        $sql .= " LEFT JOIN Stat ON (Stat.Kod = RVD.StatDanoveRezidence) ";
        $sql .= " LEFT JOIN OdpocetNavrh ON ( (OdpocetNavrh.RC = RVD.RC) AND ( OdpocetNavrh.DatumOd >= " . $params->addParameter($database->timestampToParam($datum1))
                    . " AND OdpocetNavrh.DatumOd <= " . $params->addParameter($database->timestampToParam($datum2))
                    . " AND OdpocetNavrh.RC =" . $params->addParameter($pp->rc)
                    . " AND OdpocetNavrh.OsobniCislo = " . $params->addParameter($pp->osobniCislo)
                    . " AND OdpocetNavrh.DatumNastupu = " . $params->addParameter($database->timestampToParam($pp->datumNastupu)) . ")) "
                    . " AND (NOT EXISTS (SELECT Stav From OdpocetNavrh OdN Where OdN.DatumOd >= " . $params->addParameter($database->timestampToParam($datum1))
                        . " AND OdN.DatumOd <= " . $params->addParameter($database->timestampToParam($datum2))
                        . " AND OdN.Stav > 0 AND OdN.RC = " . $params->addParameter($pp->rc)
                        . " AND OdN.OsobniCislo = " . $params->addParameter($pp->osobniCislo)
                        . " AND OdN.DatumNastupu = " . $params->addParameter($database->timestampToParam($pp->datumNastupu)) . "))";

        $sql .= " WHERE RVD.RC = " . $params->addParameter($pp->rc);
        $sql .= " AND RVD.Rok = " . $params->addParameter($database->timestampToParam($rok));
        $sql .= " AND Odpocet.DatumDo IS NULL";

        MessageManager::addDebugLog($sql);
        $cursor = $database->getSqlResult($sql, $params);

        if (!$row = $cursor->fetch()) {
            //Pro potreby overeni, zda je Prohlaseni vubec zalozeno - pokud nastane novy rok a neni
            //vytvorene obdobi, pak je potreba vytvorit rucne zaznam v tab. RocniVyuctovaniDane
            if ($testProhlaseni == false) {
                throw new Exception(ronText('prohlaseni_nenalezeno'));
            } else {
                return null;
            }
        }


        $prohlaseni = new ProhlaseniPoplatnikaDane();
        $prohlaseni->nactiHodnotyZRadku($database, $row);
        $cursor->close();

        return $prohlaseni;
    }

    /**
     * @param Database $database
     * @param $row
     */
    public function nactiHodnotyZRadku($database, $row)
    {
        $this->rok = $database->stringToTimestamp($row[0]);
        $this->rc = $row[1];
        $this->prohlaseniPPID = $row[49];


        $this->datumProhlaseni = $database->stringToTimestamp($row[2]);
        $this->prohlaseniPortalStav = (int)$row[3];
        $this->datumProhlaseniPortal = $database->stringToTimestamp($row[4]);
        $this->provestRocniVyuctovani = (int)$row[5];
        $this->elektronickyPodpis = $row[26];
        $this->elektronickyPodpisUcetni = $row[27];
        $this->prohlaseniElektronicky = (int)$row[28];
        $this->poznamkaProZamestnance = $row[29];
        $this->prohlaseniPortalZmenaStav = $row[44];
        $this->prohlaseniPortalRC = $row[46];
        $this->prohlaseniPortalOsc = $row[47];
        $this->prohlaseniPortalDatNastup = $database->stringToTimestamp($row[48]);

        $this->danovyNerezident = (int)$row[8];
        $this->cisloDanoveIdentifikace = $row[6];
        $this->typDanovehoIdentifikatoru = $row[7];
        $this->statDanoveRezidence = $row[9];
        $this->typDanovehoIdentifikatoru_popis = $row[18];
        $this->statDanoveRezidence_popis = $row[19];
        $this->prohlaseniKDaniCinim = $row[43];

        //Odpocet
        $this->datumOdOdpocet = $database->stringToTimestamp($row[10]);
        $this->datumDoOdpocet = $database->stringToTimestamp($row[11]);
        $this->zakladniOdpocet = (int)$row[12];
        $this->odpocetNaStudenta = (int)$row[13];
        $this->predpokladUrokuZUveru = $row[14];
        $this->naZdravotniStav = (int)$row[15];
        $this->ztpp = (int)$row[16];

        $this->vyzivujeJinyPoplatnik = (int)$row[30];
        $this->jinyPoplatnikUplatnujeZv = (int)$row[34];
        $this->jinyPoplatnikZamestnani = (int)$row[35];
        $this->jinyPoplatnikJmeno = $row[31];
        $this->jinyPoplatnikPrijmeni = $row[32];
        $this->jinyPoplatnikRC = $row[33];
        $this->jinyPoplatnikUlice = StringUtils::htmlTextConvert($row[36]);
        $this->jinyPoplatnikObec = StringUtils::htmlTextConvert($row[37]);
        $this->jinyPoplatnikPSC = StringUtils::htmlTextConvert($row[38]);
        $this->jinyPoplatnikNazevPlatce = StringUtils::htmlTextConvert($row[39]);
        $this->jinyPoplatnikUlicePlatce = StringUtils::htmlTextConvert($row[40]);
        $this->jinyPoplatnikObecPlatce = StringUtils::htmlTextConvert($row[41]);
        $this->jinyPoplatnikPSCPlatce = StringUtils::htmlTextConvert($row[42]);

         //Odpocet návrh
        $this->datumOdOdpocetNavrh = $database->stringToTimestamp($row[20]);
        $this->zakladniOdpocetNavrh = $row[21];
        $this->odpocetNaStudentaNavrh = $row[22];
        $this->naZdravotniStavNavrh = $row[23];
        $this->datumOdOdpocetNavrhSchvaleno = $database->stringToTimestamp($row[25]);
        $this->datumVlozeniNavrhu = $database->stringToTimestamp($row[45]);
    }
}

