<?php
/**
 * UTF8
 * Roman.Szablatura, 18.3.2016
 */

namespace Ron\Personalistika;

use Exception;
use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\HttpUtils;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalBrowser;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\StringUtils;
use function Ron\ronText;

require_once 'ron/controls.php';
require_once 'ron/httputils.php';

require_once 'adsapp/ronportalbrowser.php';
require_once 'adsapp/ronportalfilter.php';
require_once 'adsapp/prava.php';
require_once 'ron/guiutils.php';


/**
 * Class LekarskeProhlidkyFilterForm
 * @package Ron\Personalistika
 */
class LekarskeProhlidkyFilterForm extends RonPortalFilterForm
{
    /**
     * LekarskeProhlidkyFilterForm constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravaProhlizet
     * @param PravaNaOsoby $pravaAbsolvovat
     * @param PravaNaOsoby $pravaZapis
     */
    function __construct($database, $pravaProhlizet, $pravaAbsolvovat, $pravaZapis)
    {
        parent::__construct($database, 'lekarskeprohlidky');

        $this->f_formaction = '?menuid=lekarskeprohlidky';
        $this->f_classname = 'filtercontainer leftcaptions';
        $this->nastavRozdeleni(true);
        $this->nastavRozdeleni2(true);

        $this->nastavOsoby(true,[$pravaProhlizet, $pravaAbsolvovat, $pravaZapis],time(),time());
        $this->povolitVsechyOsoby = 1;
    }

//    function renderVcetnePodrizenych2()
//    {
//        echo '<DIV class="filterrow">';
//        echo '<SPAN class="filtercaption"></SPAN>';
//        GuiUtils::renderCheckbox('vcetnePodrizenych2',
//            '&nbsp;' . ronText('Vcetne_podrizenych'), $_SESSION['vcetnePodrizenych2'],
//            'onClick="submit()"');
//        echo "</DIV>\n";
//    }

    /**
     * @throws DBException
     */
    function renderTypProhlidky()
    {
        echo '<DIV class="filterrow">';
        {
            echo '<SPAN class="filtercaption">';
            GuiUtils::renderCheckbox('jenTypProhlidky', ronText('typ_prohlidky') . '&nbsp;', $_SESSION['jenTypProhlidky'],
                'onClick="submit()"');
            echo '</SPAN>';

            $sql = 'SELECT S_TypLekarskeProhlidky.Kod, S_TypLekarskeProhlidky.Popis FROM S_TypLekarskeProhlidky ';

            $combo = new ComboBox('typProhlidky');
            $combo->className = 'app_combo adsfilterforminput_full';
            $combo->dataTypeComparison = false;
            $combo->htmlParams = 'onChange="if (jenTypProhlidky.checked)submit();"';
            $combo->loadData($this->database, $sql, $_SESSION['typProhlidky']);
            $combo->render();
        }
        echo "</DIV>";
    }

    /**
     * @throws DBException
     */
    function renderProhlidka()
    {
        echo '<DIV class="filterrow">';
        {
            echo '<SPAN class="filtercaption">';
            GuiUtils::renderCheckbox('jenVyberProhlidka', ronText('prohlidka') . '&nbsp;', $_SESSION['jenVyberProhlidka'],
                'onClick="submit()"');
            echo '</SPAN>';

            $sql = 'SELECT LekarskaProhlidka.Kod, LekarskaProhlidka.Popis FROM LekarskaProhlidka ';

            $combo = new ComboBox('vyberProhlidka');
            $combo->className = 'app_combo adsfilterforminput_full';
            $combo->dataTypeComparison = false;
            $combo->htmlParams = 'onChange="if (jenVyberProhlidka.checked)submit();"';
            $combo->loadData($this->database, $sql, $_SESSION['vyberProhlidka']);
            $combo->render();
        }
        echo "</DIV>";
    }

    /**
     * @throws DBException
     */
    function renderVysledekProhlidky()
    {
        echo '<DIV class="filterrow">';
        {
            echo '<SPAN class="filtercaption">';
            GuiUtils::renderCheckbox('jenVysledekProhlidky', ronText('vysledek') . '&nbsp;', $_SESSION['jenVysledekProhlidky'],
                'onClick="submit()"');
            echo '</SPAN>';

            $sql = 'SELECT S_VyhodnoceniProhlidky.Kod, S_VyhodnoceniProhlidky.Popis FROM S_VyhodnoceniProhlidky ';

            $combo = new ComboBox('vysledekProhlidky');
            $combo->className = 'app_combo adsfilterforminput_full';
            $combo->dataTypeComparison = false;
            $combo->htmlParams = 'onChange="if (jenVysledekProhlidky.checked)submit();"';
            $combo->loadData($this->database, $sql, $_SESSION['vysledekProhlidky']);
            $combo->render();
        }
        echo "</DIV>";
    }

    function renderPovinna()
    {
        echo '<DIV class="filterrow">';
        echo '<span class="filtercaption">' . ronText('zobraz_zdravotnicke_prohlidky') . '</span>';

        $data = array(
            0 => ronText('Vse'),
            1 => ronText('absolvovana'),
            2 => ronText('neabsolvovana')
        );

        LinksList::renderLinks($data, $_SESSION['vseAbsolvovanaNeabsolvovana'], $this ->f_formaction .  '&van=', '', false);

        echo '</DIV>';
    }

    function renderZadatLekarskouProhlidku()
    {
        echo '<DIV class="filterrow">';
        echo '<SPAN class="filtercaption">';
        echo '<a href="?menuid=lekarskeprohlidky&action=lekarskaprohlidkaform&formaction=new">';
        echo ronText('zadat_zdravotni_prohlidku');
        echo '</a>';
        echo '</SPAN>';
        echo "</DIV>";
    }


    /**
     * @throws DBException
     * @throws Exception
     */
    function renderFilters()
    {
        if ($this->f_showrozdeleni) $this->renderRozdeleni();
        if ($this->f_showrozdeleni2) {
            $this->renderRozdeleni2();
        }

        if ($this->f_showosoba) $this->renderOsoby();
        if ($this->f_link) $this->renderLink();
        $this->renderTypProhlidky();
        $this->renderProhlidka();
        $this->renderVysledekProhlidky();
        $this->renderPovinna();
        echo "<br />";
        if(Prava::vratPrava('lekarske_edit')->lzeZobrazitRC(RonPortalSession::osobaRC(),time(),time())){
            $this->renderZadatLekarskouProhlidku();
        }

        echo "<br />";
    }

    private function loadTypProhlidky()
    {
        if (isset($_GET['typProhlidky'])){
            $_SESSION['jenTypProhlidky'] = isset($_GET['typprohlidky']);
            $_SESSION['typProhlidky'] = $_GET['typprohlidky'];
        }

        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenTypProhlidky'] = isset($_POST['jenTypProhlidky']);
            $_SESSION['typProhlidky'] = $_POST['typProhlidky'];
        }
    }

    private function loadProhlidka()
    {
        if (isset($_GET['vyberProhlidka'])){
            $_SESSION['jenVyberProhlidka'] = isset($_GET['vyberProhlidka']);
            $_SESSION['vyberProhlidka'] = $_GET['vyberProhlidka'];
        }

        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenVyberProhlidka'] = isset($_POST['jenVyberProhlidka']);
            $_SESSION['vyberProhlidka'] = $_POST['vyberProhlidka'];
        }
    }
    private function loadVysledekProhlidky()
    {
        if (isset($_GET['vysledekProhlidky'])){
            $_SESSION['jenVysledekProhlidky'] = isset($_GET['vysledekprohlidky']);
            $_SESSION['vysledekProhlidky'] = $_GET['vysledekprohlidky'];
        }

        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenVysledekProhlidky'] = isset($_POST['jenVysledekProhlidky']);
            $_SESSION['vysledekProhlidky'] = $_POST['vysledekProhlidky'];
        }
    }

    private function loadUrovenProhlidky()
    {
        if(isset($_GET['van'])){
            $_SESSION['vseAbsolvovanaNeabsolvovana'] = $_GET['van'];
        }
    }

    private function loadVcetnePodrizenych2()
    {
        if(isset($_POST['adsfilterform'])){
            $_SESSION['vcetnePodrizenych2'] = isset($_POST['vcetnePodrizenych2']);
        }
    }

    function loadData()
    {
        parent::loadData();
        $this->loadUrovenProhlidky();
        $this->loadTypProhlidky();
        $this->loadProhlidka();
        $this->loadVysledekProhlidky();
        $this->loadVcetnePodrizenych2();
    }
}

/**
 * Class DatumAbsolvovaniProhlidkyField
 * @package Ron\Personalistika
 */
class DatumAbsolvovaniProhlidkyField extends BrowserField //pridani novEho sloupce
{

    private $prava;

    /**
     * DatumAbsolvovaniProhlidkyField constructor.
     * @param null|Database $database
     * @param PravaNaOsoby $prava
     * @internal param array|null $options
     */
    public function __construct($database, $prava)
    {
        parent::__construct($database);
        $this->header = ronText('datum_absolvovani');
        $this->align = "center";
        $this->ordering = 1;
        $this->code = 'col-datum-absolvovani';

        $this->sortSQL = "GenerovaneLekProhlidky.DatumAbsolvovani";
        $this->sortSQLDesc = "GenerovaneLekProhlidky.DatumAbsolvovani DESC";

        $this->prava = $prava;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $idosoby = $rowData[15];

        //Pokud je prohlidka absolvovana pak nezobrazit odkaz ale pouze "absolvovano"

        if($rowData[22]!= null || $rowData[22] != ""){
            if ($rowData[1] != '') {
                $result .= StringUtils::format('date', $this->database->stringToTimestamp($rowData[1]));
            } else {
                $result .= "<span title='" . ronText('lekarska_prohlidka_absolvovano_info') . "'>" . ronText('lekarska_prohlidka_absolvovano') . "</span>";
            }

        } else {

            if ($rowData[1] == '' && $this->prava->lzeZobrazitPP($idosoby)) {
                $result .= '<a href="?menuid=lekarskeprohlidky&action=absolvovat&formaction=edit&osoba=' . $idosoby .
                    '&kod=' . HttpUtils::urlEncode($rowData[2]) . '&absolvovat=1' . '&vznik=' . $this->database->stringToTimestamp($rowData[0]) . '">';
                $result .= ronText('absolvovat');
                $result .= '</a>';
            }
        }
        return  $result;
    }
}

/**
 * Class DetailProhlidkyField
 * @package Ron\Personalistika
 */
class DetailProhlidkyField extends BrowserField
{
    private $jePravoEditovatZaznam;

    /**
     * DetailProhlidkyField constructor.
     * @param null|Database $database
     * @param PravaNaOsoby $prava
     * @internal param array|null $options
     */
    public function __construct($database, $prava)
    {
        parent::__construct($database);
        $this->header = ronText('zdravotni_prohlidka_detail');
        $this->align = "center";
        $this ->code = "col-detail";

        $this->jePravoEditovatZaznam = $prava;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $idosoby = $rowData[15];

        if(!$this->jePravoEditovatZaznam->lzeZobrazitPP($idosoby)) {
            $result .= '<a  href="?menuid=lekarskeprohlidky&action=detail&osoba=' . $idosoby .
                '&kod=' . HttpUtils::urlEncode($rowData[2]) . '&vznik=' . $this->database->stringToTimestamp($rowData[0]) . '"' . (RonPortalSettings::getIntValue('otevreni_detailu') ? ' target="_blank">' : '">');
            $result .= ronText('detail');
            $result .= '</a>';
            $result .= '<br />';

        } else {
            $result .= '<a href="?menuid=lekarskeprohlidky&action=lekarskaprohlidkaform&formaction=edit&osoba=' . $idosoby .
                '&kod=' . HttpUtils::urlEncode($rowData[2]) . '&vznik=' . $this->database->stringToTimestamp($rowData[0]) . (RonPortalSettings::getIntValue('otevreni_detailu') ? ' target="_blank">' : '">');
            $result .= ronText('detail');
            $result .= '</a>';

        }
        return  $result;
    }
}

/**
 * Class LekarskeProhlidkyBrowser
 * @package Ron\Personalistika
 */
class LekarskeProhlidkyBrowser extends RonPortalBrowser
{
    /** @var PravaNaOsoby  */
    var $pravaProhlizet;
    /** @var PravaNaOsoby  */
    var $pravaZapis;
    /** @var PravaNaOsoby  */
    var $pravaAbsolvovat;

    /** @var  LekarskeProhlidkyFilterForm */
    //var $filterForm;

    /**
     * LekarskeProhlidkyBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    function __construct($database){
        parent::__construct($database, 'lekarskeprohlidky');

        $this->pravaProhlizet = Prava::vratPrava('lekarske_prohl');
        $this->pravaZapis = Prava::vratPrava('lekarske_edit');
        $this->pravaAbsolvovat = Prava::vratPrava('lekarske_absolv');

        $this->title = ronText('zdravotni_prohlidky');
        $this->code = 'browserlekarskeprohlidky';
        $this->tableId = 'browserlekarskeprohlidky';

        $this->sqlSelectPart =
            'GenerovaneLekProhlidky.DatumVznikuPovinnosti, GenerovaneLekProhlidky.DatumAbsolvovani, GenerovaneLekProhlidky.KodLekarskeProhlidky, ' .    //0 - 2
            'LekarskaProhlidka.Popis, S_DuvodProhlidky.Popis, GenerovaneLekProhlidky.Generovana, {fn_CASTWIDEMEMO(GenerovaneLekProhlidky.VysledekProhlidky)}, ' . //3 - 6
            'GenerovaneLekProhlidky.OsetrujiciLekar, {fn_CASTWIDEMEMO(GenerovaneLekProhlidky.Poznamka)},'                                                                    .    //7 - 8
            "S_DuvodProhlidky.Popis, GenerovaneLekProhlidky.Generovana, COALESCE(OsetrujiciLekar.JmenoLekare,'') {fn_concat()} ' ' {fn_concat()} COALESCE(OsetrujiciLekar.Prijmeni,''), " .    //9 - 11
            'Osoba.Jmeno, Osoba.Prijmeni, S_AnoNe.Popis, (SELECT MAX(ID) FROM PracovniPomer WHERE RC = Osoba.RC AND DatumNastupu <= ' . $database->timestampToString(time()) . ' AND (DatumUkonceni >= ' . $database->timestampToString(time()) . ' OR DatumUkonceni IS NULL)), S_VyhodnoceniProhlidky.Popis, ' .  //12 - 16
             $this->database->castWideMemo('GenerovaneLekProhlidky.DoplnujiciTextPozvanky') . ", GenerovaneLekProhlidky.CasProhlidkyRezervovany, DatumProhlidkyRezervovany, OsobniCislo, Osoba.RC, GenerovaneLekProhlidky.Vysledek " //17 - 22
            /*'(SELECT count(OsobniCislo) FROM PracovniPomer WHERE  RC = ' .  . ' DatumNastupu <=' . $database ->dateToString(date(time())) . ' AND (DatumUkonceni >= ' . $database ->dateToString(date(time())) . ' OR DatumUkonceni is null)) '*/ ;

        $this->sqlFromPart =
            'Osoba LEFT OUTER JOIN PracovniPomer pp ON pp.RC=Osoba.RC AND
               ((pp.Zamestnanec = 1) OR (pp.Zamestnanec = 2))
            and (pp.DatumNastupu<=' . $database ->dateToString(date(time())) . ')
            and ((pp.DatumUkonceni >=' . $database ->dateToString(date(time())) . ') OR (pp.DatumUkonceni IS NULL) OR ((pp.DatumUkonceni IS NOT NULL) AND (pp.DuvodUkonceniPP IS NULL)))
            and pp.DatumNastupu = (SELECT MAX(pp2.DatumNastupu) FROM PracovniPomer pp2
                                  WHERE pp2.RC=pp.RC AND ((pp2.Zamestnanec = 1) OR (pp2.Zamestnanec = 2))
                                  and (pp2.DatumNastupu<=' . $database ->dateToString(date(time())) . ')
                                  and ((pp2.DatumUkonceni >=' . $database ->dateToString(date(time())) . ') OR (pp2.DatumUkonceni IS NULL) OR ((pp2.DatumUkonceni IS NOT NULL) AND (pp2.DuvodUkonceniPP IS NULL))))
            and NOT EXISTS(SELECT pp3.RC FROM PracovniPomer pp3
                          WHERE pp3.RC=pp.RC and pp3.DatumNastupu=pp.DatumNastupu and pp3.OsobniCislo>pp.OsobniCislo
                          and ((pp3.Zamestnanec = 1) or (pp3.Zamestnanec = 2))
                          and ((pp3.DatumUkonceni >=' . $database ->dateToString(date(time())) . ') OR (pp3.DatumUkonceni IS NULL) OR ((pp3.DatumUkonceni IS NOT NULL) AND (pp3.DuvodUkonceniPP IS NULL)))),
            GenerovaneLekProhlidky LEFT OUTER JOIN OsetrujiciLekar ON (GenerovaneLekProhlidky.OsetrujiciLekar = OsetrujiciLekar.Kod)
            LEFT OUTER JOIN  LekarskaProhlidka ON (GenerovaneLekProhlidky.KodLekarskeProhlidky = LekarskaProhlidka.Kod)
            LEFT OUTER JOIN  S_DuvodProhlidky ON (GenerovaneLekProhlidky.DuvodProhlidky = S_DuvodProhlidky.Kod )
            LEFT OUTER JOIN  S_AnoNe ON  (GenerovaneLekProhlidky.Generovana = S_AnoNe.Kod)
            LEFT OUTER JOIN  S_TypLekarskeProhlidky ON  (LekarskaProhlidka.TypProhlidky = S_TypLekarskeProhlidky.Kod )
            LEFT OUTER JOIN  S_VyhodnoceniProhlidky ON  (GenerovaneLekProhlidky.Vysledek = S_VyhodnoceniProhlidky.Kod )';


        $this->sqlWherePart =
            'GenerovaneLekProhlidky.RC = Osoba.RC ' ;

        $this->browserUrl = '?menuid=lekarskeprohlidky&';

        $this ->filterForm = new LekarskeProhlidkyFilterForm($this ->database, $this->pravaProhlizet, $this->pravaAbsolvovat, $this->pravaZapis);
        $this ->filterForm ->loadData();

        $this->addField([
            BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_LINE, BROWSER_FIELD_CODE => 'col-line']);

        $this->addField([
            BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_SQL_NAME => 'GenerovaneLekProhlidky.DatumVznikuPovinnosti',
            BROWSER_FIELD_CODE => 'col-datum-vzniku',
            BROWSER_FIELD_HEADER_TEXT => 'datum_povinnosti', BROWSER_FIELD_FORMAT => 'date' , BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1]);

        //Urci podle ktereho fieldu se bude defaultne radit
        $this->defaultSort = count($this->fields);

        if(RonPortalSettings::getIntValue('lekprohl_rezervovany')) {
            $this->addField([
                BROWSER_FIELD_SQL_INDEX => 18, BROWSER_FIELD_SQL_NAME => 'GenerovaneLekProhlidky.CasProhlidkyRezervovany',
                BROWSER_FIELD_CODE => 'col-cas-rezervace',
                BROWSER_FIELD_HEADER_TEXT => 'rezervovany_cas', BROWSER_FIELD_FORMAT => 'time', BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1]);

            $this->addField([
                BROWSER_FIELD_SQL_INDEX => 19, BROWSER_FIELD_SQL_NAME => 'GenerovaneLekProhlidky.DatumProhlidkyRezervovany',
                BROWSER_FIELD_CODE => 'col-datum-rezervace',
                BROWSER_FIELD_HEADER_TEXT => 'rezervovany_datum', BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1]);

        }
        //Zobrazeni odkazu Absolvovani nebo pouze datumu
        $this->addField(new DatumAbsolvovaniProhlidkyField($this->database, $this->pravaAbsolvovat));

        //filtr na Vse/Absolvovana/Povinne
        if($_GET['van'] == 0 || $_GET['van'] == 1){
            $this->addField([
                BROWSER_FIELD_SQL_INDEX => 16, 'sqlFieldName' => 'S_VyhodnoceniProhlidky.Popis',
                BROWSER_FIELD_CODE => 'col-vysledek',
                BROWSER_FIELD_HEADER_TEXT => 'vysledek', 'align' => 'center', 'ordering' => 1]);
        }

        $this->addField(new DetailProhlidkyField($this->database, $this->pravaZapis));

        $this->addField([
            BROWSER_FIELD_SQL_INDEX => 2, 'sqlFieldName' => 'GenerovaneLekProhlidky.KodLekarskeProhlidky',
            BROWSER_FIELD_CODE => 'col-kod',
            BROWSER_FIELD_HEADER_TEXT => 'kod_zdravotni_prohlidky', 'align' => 'center', 'ordering' => 1]);

        $this->addField([
            BROWSER_FIELD_SQL_INDEX => 3, 'sqlFieldName' => 'LekarskaProhlidka.Popis',
            BROWSER_FIELD_CODE => 'col-popis',
            BROWSER_FIELD_HEADER_TEXT => 'popis_zdravotni_prohlidky', 'ordering' => 1]);

        $this->addField([
            BROWSER_FIELD_SQL_INDEX => 4, 'sqlFieldName' => 'S_DuvodProhlidky.Popis',
            BROWSER_FIELD_CODE => 'col-duvod',
            BROWSER_FIELD_HEADER_TEXT => 'duvod_zdravotni_prohlidky', 'ordering' => 1]);

        $this->addField([
            BROWSER_FIELD_SQL_INDEX => 14, 'sqlFieldName' => 'S_AnoNe.Popis',
            BROWSER_FIELD_CODE => 'col-generovana',
            BROWSER_FIELD_HEADER_TEXT => 'generovana_zdravotni_prohlidka', 'align' => 'center', 'ordering' => 1]);

        $this->addField([
            BROWSER_FIELD_SQL_INDEX => 11, 'sqlFieldName' => "OsetrujiciLekar.JmenoLekare + ' ' + OsetrujiciLekar.Prijmeni",
            BROWSER_FIELD_CODE => 'col-lekar',
            BROWSER_FIELD_HEADER_TEXT => 'osetrujici_lekar', 'ordering' => 1]);

        $this->addField([
            BROWSER_FIELD_SQL_INDEX => 13, 'sqlFieldName' => 'Osoba.Prijmeni',
            BROWSER_FIELD_CODE => 'col-prijmeni', BROWSER_FIELD_VISIBLE => RonPortalSession::vsechnyOsoby(),
            BROWSER_FIELD_HEADER_TEXT => 'Prijmeni', 'ordering' => 1]);

        $this->addField([
            BROWSER_FIELD_SQL_INDEX => 12, 'sqlFieldName' => "Osoba.Jmeno",
            BROWSER_FIELD_CODE => 'col-jmeno', BROWSER_FIELD_VISIBLE => RonPortalSession::vsechnyOsoby(),
            BROWSER_FIELD_HEADER_TEXT => 'Jmeno', 'ordering' => 1]);

        require_once('person/person_browserfield.php');
        $this->addField(new  OsobniCisloOsobyField($this->database, 21, [
            BROWSER_FIELD_CODE => 'col-osobni-cislo',
            BROWSER_FIELD_VISIBLE => RonPortalSession::vsechnyOsoby(),
            BROWSER_FIELD_ALIGN => 'center']));
    }


    function setFilterForm()
    {
        // parent::setFilterForm(); - nevolat
    }


//    function load($sqlFilter = '')
//    {
//        parent::load($sqlFilter);
//    }

//    function renderFilter()
//    {
//        $this->filterForm->render();
//        parent::renderFilter();
//    }

    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        //Doplnujici poznamka
        if(RonPortalSettings::getIntValue('lekprohl_doplnujici_text')) {
            $hodnotaSloupce = 17;
            if ($values[$hodnotaSloupce]) {
                echo "<tr class='$className'>";
                echo '<td></td><td colspan="' . ($this->columnCount - 1) . '">';
                echo '<span class="poznamka doplnujiciText">' . ronText('doplnujici_text') .  ': ' .StringUtils::htmlTextConvert($values[$hodnotaSloupce]) . '</span>';
                //echo '<td colspan="2"></td></tr>';
            }
        }

        //poznamka
        if(RonPortalSettings::getIntValue('lekprohl_poznamka')) {
            $hodnotaSloupce = 8;
            if ($values[$hodnotaSloupce]) {
                echo "<tr class='$className'>";
                echo '<td></td><td colspan="' . ($this->columnCount - 1) . '">';
                echo '<span class="poznamka poznamkaLekarskeProhlidky">' . ronText('Poznamka') . ': ' . StringUtils::htmlTextConvert($values[$hodnotaSloupce]) . '</span>';
                //echo '<td colspan="2"></td></tr>';
            }
        }

        //vysledek prohlidky
        if(RonPortalSettings::getIntValue('lekprohl_vysledek_prohlidky')) {
            $hodnotaSloupce = 6;
            if ($values[$hodnotaSloupce]) {
                echo "<tr class='$className'>";
                echo '<td></td><td colspan="' . ($this->columnCount - 1) . '">';
                echo '<span class="poznamka vysledekProhlidky">' . ronText('vysledek_prohlidky') . ': ' . StringUtils::htmlTextConvert($values[$hodnotaSloupce]) . '</span>';
                //echo '<td colspan="2"></td></tr>';
            }
        }
    }


    public function getSQLFilter()
    {
        $params = $this->database->createParameters();

        //Overeni prav
        $result =
            '(' .
            $this->pravaProhlizet->sqlFiltr('pp',time(),time()) .
            ' OR ' .
            $this->pravaZapis->sqlFiltr('pp',time(),time()) .
            ' OR ' .
            $this->pravaAbsolvovat->sqlFiltr('pp',time(),time()) .
            ')';

        //if (!RonPortalSession::vsechnyOsoby()) {
        if (!$this ->filterForm ->vsechnyOsoby) {
            $result .= ")  AND GenerovaneLekProhlidky.RC = " . $params->addParameter(RonPortalSession::osobaRC());
        } else {
            if ($this ->filterForm ->kodRozdeleni != '') {
                if(RonPortalSession::rozdeleniVcetnePodrizenych()){
                    $result .= ' AND (EXISTS (SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
                        '   WHERE RozdeleniOsob.RC = Osoba.RC ' .
                        '   AND HierarchieRozdeleni.TypNadrazene = ' . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                        "   AND HierarchieRozdeleni.KodNadrazene = " . $params->addParameter(RonPortalSession::rozdeleni()) .
                        '   AND HierarchieRozdeleni.TypPodrizene = RozdeleniOsob.KodTypuRozdeleni ' .
                        '   AND HierarchieRozdeleni.KodPodrizene = RozdeleniOsob.Rozdeleni ' .
                        '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(time())) .
                        '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam(time())). ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                        '))';
                }
                else {
                    $result .= ' AND EXISTS(SELECT * FROM RozdeleniOsob ' .
                        '   WHERE RozdeleniOsob.KodTypuRozdeleni = ' . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                        "     AND RozdeleniOsob.Rozdeleni = " . $params->addParameter(RonPortalSession::rozdeleni()) .
                        '     AND RozdeleniOsob.RC = pp.RC ' .
                        '     AND RozdeleniOsob.OsobniCislo = pp.OsobniCislo ' .
                        '     AND RozdeleniOsob.DatumNastupu = pp.DatumNastupu ' .
                        '     AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(time())).
                        '     AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam(time())) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                        ')';
                }
            }

            if ($this ->filterForm ->f_showrozdeleni2  &&  $this ->filterForm ->f_typrozdeleni2  &&  RonPortalSession::rozdeleni2() != '') {

                if(RonPortalSession::rozdeleni2VcetnePodrizenych()){
                    $result .= ' AND (EXISTS (SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
                        '   WHERE RozdeleniOsob.RC = Osoba.RC ' .
                        '   AND HierarchieRozdeleni.TypNadrazene = ' . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP) .
                        "   AND HierarchieRozdeleni.KodNadrazene = " . $params->addParameter(RonPortalSession::rozdeleni2()) .
                        '   AND HierarchieRozdeleni.TypPodrizene = RozdeleniOsob.KodTypuRozdeleni ' .
                        '   AND HierarchieRozdeleni.KodPodrizene = RozdeleniOsob.Rozdeleni ' .
                        '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(time())) .
                        '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam(time())) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                        '))';
                }
                else {
                    $result .= ' AND EXISTS(SELECT * FROM RozdeleniOsob ' .
                        '   WHERE RozdeleniOsob.KodTypuRozdeleni = ' . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP) .
                        "     AND RozdeleniOsob.Rozdeleni = " . $params->addParameter(RonPortalSession::rozdeleni2()) .
                        '     AND RozdeleniOsob.RC = pp.RC ' .
                        '     AND RozdeleniOsob.OsobniCislo = pp.OsobniCislo ' .
                        '     AND RozdeleniOsob.DatumNastupu = pp.DatumNastupu ' .
                        '     AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(time())) .
                        '     AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam(time())) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                        ')';
                }
            }
        }

        if ($_SESSION['jenTypProhlidky']) {
            $result .= " AND S_TypLekarskeProhlidky.Kod = " . $params->addParameter($_SESSION['typProhlidky']);
        }

        if ($_SESSION['jenVyberProhlidka']) {
            $result .= " AND LekarskaProhlidka.Kod = " . $params->addParameter($_SESSION['vyberProhlidka']);
        }

        if ($_SESSION['jenVysledekProhlidky']) {
            $result .= " AND S_VyhodnoceniProhlidky.Kod = " . $params->addParameter($_SESSION['vysledekProhlidky']);
        }



        if ($this ->filterForm ->vsechnyOsoby) {
            $result .= ')';
        }

        /* 0 => ('Vse'), 1 => ('absolvovana'), 2 => ('neabsolvovana')*/
        if($_SESSION['vseAbsolvovanaNeabsolvovana'] == 0){
            $result .= ' AND (GenerovaneLekProhlidky.DatumAbsolvovani IS NOT NULL ';
            $result .= ' OR GenerovaneLekProhlidky.DatumAbsolvovani IS NULL ';
        }

        if($_SESSION['vseAbsolvovanaNeabsolvovana'] == 1){
            $result .= ' AND (GenerovaneLekProhlidky.DatumAbsolvovani IS NOT NULL ';
        }

        if($_SESSION['vseAbsolvovanaNeabsolvovana'] == 2){
            $result .= ' AND (GenerovaneLekProhlidky.DatumAbsolvovani IS NULL ';
        }


        return new SqlStatement($result, $params);
    }
}