<?php
/**
 * UTF-8
 * Roman.Szablatura, 18.2.2016
 */

namespace Ron\Personalistika;

use Exception;
use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\HttpUtils;
use Ron\RonPortal\FormReportsListView;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalReport;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\Rozdeleni;
use Ron\RonPortal\User;
use Ron\StringUtils;
use function Ron\ronText;

//<editor-fold defaultstate="collapsed" desc="require_once">
require_once('ron/browser.php');
require_once('adsapp/ronportalfilter.php');
require_once('ron/guiutils.php');
require_once('ron/controls.php');
require_once('ron/httputils.php');
//</editor-fold>

/**
 * Class PracovniPomuckyFilterForm
 * @package Ron\Personalistika
 */
class PracovniPomuckyFilterForm extends RonPortalFilterForm //Přidání rozbalovacích seznamů
{
    /** @var PravaNaOsoby */
    var $pravaVydatVratit;

    public $timestamp;

    /**
     * @param Database $database
     * @param PravaNaOsoby $pravaProhlizet
     * @param PravaNaOsoby $pravaPotvrdit
     * @param PravaNaOsoby $pravaVydatVratit
     */
    function __construct($database, $pravaProhlizet, $pravaPotvrdit, $pravaVydatVratit)
    {
        parent::__construct($database, 'pracovnipomucky');

        $this->skrytTlacitkoNastavit = false;
        $this->f_formaction = '?menuid=pracovnipomucky';
        $this->f_classname = 'filtercontainer leftcaptions';
        $this->nastavRozdeleni(true);
        $this->nastavRozdeleni2(true);

        $this->nastavOsoby(true,[$pravaProhlizet, $pravaPotvrdit, $pravaVydatVratit],time(),time());
        $this->povolitVsechyOsoby = 1;
        $this->pravaVydatVratit = $pravaVydatVratit;
        $this->nastavDatumOdDo(true, true, ronText('PracovniPomucky_obdobi') . ' ' . ronText('Od'));


        if (!isset($_SESSION['skrytFormular']) && $_SESSION['skrytFormular'] == ""){
            $_SESSION['skrytFormular'] = 1;
        }
    }


    /**
     * @throws Exception
     */
    function renderPomucka()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        GuiUtils::renderCheckbox('jenPomucka', ronText('pomucka'). '&nbsp;&nbsp;', $_SESSION['jenPomucka'],
            'onClick="submit()"');
        echo '</SPAN>';

        $sql = "SELECT Kod, Popis, TypPomucky FROM PracovniPomucka ";

        if ($_SESSION['jenTypPomucky']) {
            $sql .= "WHERE PracovniPomucka.TypPomucky = '" . $_SESSION['typPomucky'] . "'";
        }
        $sql .= "ORDER BY Popis";

        $combo = new ComboBox('pomucka');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenPomucka.checked) submit();"';
        $combo->showCode = 1;
        $combo->loadData($this->database, $sql, $_SESSION['pomucka']);
        $combo->render();

        echo "</DIV>";
    }

    /**
     * @throws Exception
     */
    function renderTypPomucky()
    {
        echo '<DIV class="filterrow">';
            echo '<SPAN class="filtercaption">';
                GuiUtils::renderCheckbox('jenTypPomucky', ronText('typ_pomucky'). '&nbsp;&nbsp;', $_SESSION['jenTypPomucky'],
                'onClick="submit()"');
            echo '</SPAN>';

            $sql = 'SELECT Kod,TypPomucky.Popis FROM TypPomucky ORDER BY Popis';

            $combo = new ComboBox('typPomucky');
            $combo->className = 'app_combo adsfilterforminput_full';
            $combo->htmlParams = 'onChange="if (jenTypPomucky.checked) submit();"';
            $combo->loadData($this->database, $sql, $_SESSION['typPomucky']);
            $combo->render();
        echo "</DIV>";
    }

    function renderVydatPracPomucku()
    {
        echo '<DIV class="filterrow">';
            echo '<SPAN class="filtercaption">';
                echo '<a href="?menuid=pracovnipomucky&action=pomuckyform&formaction=new&vydanipomucky=2&individualni=1">';
                echo ronText('vydej_pomucky_zamestnanci');
                echo '</a>';
            echo '</SPAN>';
        echo "</DIV>";
    }

    function renderFiltrNevydanePracovniPomucky()
    {
        echo '<DIV>';
        echo '<span class="filtercaption">';
            GuiUtils::renderCheckbox('jenNevydane', ronText('nevydane_pomucky') . '&nbsp;&nbsp;', $_SESSION['jenNevydane'],
                'onClick="submit()"');

        //TODO doplnit filtr jenPotvrzene
            GuiUtils::renderCheckbox('jenPotvrzene', ronText('vydane_potvrzene_pomucky') . '&nbsp;&nbsp;', $_SESSION['jenPotvrzene'],
                'onClick="submit()"');


            GuiUtils::renderCheckbox('jenNepotvrzene', ronText('vydane_nepotvrzene_pomucky') . '&nbsp;&nbsp;', $_SESSION['jenNepotvrzene'],
                'onClick="submit()"');


            GuiUtils::renderCheckbox('jenExpirovane', ronText('vydane_expirovane_pomucky') . '&nbsp;&nbsp;', $_SESSION['jenExpirovane'],
                'onClick="submit()"');


            GuiUtils::renderCheckbox('jenVracene', ronText('vracene_pomucky') . '&nbsp;&nbsp;', $_SESSION['jenVracene'],
                'onClick="submit()"');
        echo "</span>";
        echo '</DIV>';
    }

    private function renderReportList()
    {
        require_once('adsapp/reportutils.php');
        $reportList = RonPortalReport::loadFormReports($this->database, 'PracovniPomucky');
        FormReportsListView::renderReportList($reportList,
            '?menuid=pracovnipomucky&action=report&osoba=' . RonPortalSession::vybranaOsoba() .
            '&datumod=' . StringUtils::format('date', RonPortalSession::getDatumOd()) .
            '&datumdo=' . StringUtils::format('date', RonPortalSession::getDatumDo()) .
            '&rozdeleni=' . RonPortalSession::rozdeleni() .
            '&rozdelenipodrizene=' . RonPortalSession::rozdeleniVcetnePodrizenych() .
            '&rozdeleni2=' . RonPortalSession::rozdeleni2() .
            '&rozdelenipodrizene2=' . RonPortalSession::rozdeleni2VcetnePodrizenych() .
            '&vsechnyosoby=' . RonPortalSession::vsechnyOsoby() .
            '&jentyp=' . $_SESSION['jenTypPomucky'] .
            '&typpomucky=' . $_SESSION['typPomucky'] .
            '&jenpomucka=' . $_SESSION['jenPomucka'] .
            '&pomucka=' . $_SESSION['pomucka'] .
            '&nevydane=' . $_SESSION['jenNevydane'] .
            '&nevracene=' . $_SESSION['jenNevracene'] .
            '&vracene=' . $_SESSION['jenVracene'] .
            '&potvrzene=' . $_SESSION['jenPotvrzene'] .
            '&expirovane=' . $_SESSION['jenExpirovane'] .
            '&nepotvrzene=' . $_SESSION['jenNepotvrzene']
        );
    }

    private function loadJenNevydane()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenNevydane'] = isset($_POST['jenNevydane']);
        }
    }

    private function loadJenNevracene()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenNevracene'] = isset($_POST['jenNevracene']);
        }
    }

    private function loadJenVracene()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenVracene'] = isset($_POST['jenVracene']);
        }
    }

    private function loadJenNepotvrzene(){
        if(isset($_POST['adsfilterform'])){
            $_SESSION['jenNepotvrzene'] = isset($_POST['jenNepotvrzene']);
        }
    }

    private function loadJenPotvrzene(){
        if(isset($_POST['adsfilterform'])){
            $_SESSION['jenPotvrzene'] = isset($_POST['jenPotvrzene']);
        }
    }

    private function loadJenExpirovane(){
        if(isset($_POST['adsfilterform'])){
            $_SESSION['jenExpirovane'] = isset($_POST['jenExpirovane']);
        }
    }

    private function loadTypPomucky()
    {
        if (isset($_GET['typpomucky']) || isset($_GET['record'])) {
            $_SESSION['jenTypPomucky'] = isset($_GET['typpomucky']);
            $_SESSION['typPomucky'] = $_GET['typpomucky'];

            $this->setVsechnyOsoby();
        }
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenTypPomucky'] = isset($_POST['jenTypPomucky']);
            $_SESSION['typPomucky'] = $_POST['typPomucky'];
        }
    }


    /**
     * @throws Exception
     */
    private function loadPomucka()
    {
        if (isset($_GET['typpomucky']) || isset($_GET['record'])) {
            $_SESSION['jenPomucka'] = isset($_GET['record']);
            $_SESSION['pomucka'] = ($_GET['record']);

            $this->setVsechnyOsoby();
        }
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenPomucka'] = isset($_POST['jenPomucka']);
            $_SESSION['pomucka'] = $_POST['pomucka'];
        }

        if ($_SESSION['pomucka'] != '' && $_SESSION['jenPomucka']) {
            $params = $this->database->createParameters();
            $_SESSION['typPomucky'] = DBUtils::getSqlValue($this->database,"SELECT TypPomucky FROM PracovniPomucka WHERE Kod = " .
                $params->addParameter($_SESSION['pomucka']), $params);
        }
    }

    private function setVsechnyOsoby()
    {
        RonPortalSession::setVsechnyOsoby(true);
    }

    function loadData()
    {
        //finta, po prihlaseni se nastavuje na aktu.den
        $datumOd = RonPortalSession::getDatumOd();
        if (!isset($datumOd)){
            $this ->datumOd = DateUtils::today();
            $this ->datumDo = DateUtils::today();

            RonPortalSession::setFiltrovatDatum($this ->filtrovatDatum,
                $this ->datumOd,
                $this ->datumDo );
        }

        $this->loadDatumOdDo();
        $this->loadJenExpirovane();
        $this->loadJenNevracene();
        $this->loadJenVracene();
        $this->loadJenNevydane();
        $this->loadJenNepotvrzene();
        $this->loadJenPotvrzene();
        $this->loadTypPomucky();
        $this->loadPomucka();
        parent::loadData();
    }

    /**
     * @throws Exception
     */
    function renderFilters()
    {
        $this->renderDatumOdDo();
        if ($this->f_showrozdeleni) $this->renderRozdeleni();
        if ($this->f_showrozdeleni2) $this->renderRozdeleni2();
        if ($this->f_showosoba) $this->renderOsoby();
        $this->renderTypPomucky();
        $this->renderPomucka();
        if ($this->f_link) $this->renderLink();

        $this->renderFiltrNevydanePracovniPomucky();

        echo "<br />";
        if($this->pravaVydatVratit->jsouNejakePrava()) {
            $this->renderVydatPracPomucku();
        }
        $this->renderReportList();
    }
}

/**
 * Class PotvrzeniPrijetiField
 * @package Ron\Personalistika
 */
class PotvrzeniPrevzetiField extends BrowserField //přidání nového sloupce
{
    /**
     * PotvrzeniPrijetiField constructor.
     * @param null|Database $database
     * @param array|null $options
     */
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = ronText('potvrzeni_prevzeti');
        $this->align = "center";
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $idosoby = $rowData[16];
        $params = $this->database->createParameters();
        $prihlasenaOsoba = DBUtils::getSqlValue($this->database, "SELECT MAX(ID) FROM PracovniPomer WHERE RC = " . $params->addParameter(User::getRC()), $params);
        $result = '';

        if ($rowData[11] != '') {
            $result = '<strong>' . StringUtils::format('datetime', $this->database->stringToTimestamp($rowData[11])) . '</strong><br />';
            $result .= $rowData[12];
        } elseif (Prava::vratPrava('pomucky_potvrzeni')->lzeZobrazitRC($rowData[6])) {
            if($rowData[1] != '' && !$rowData[2] && $rowData[17]){
                $result = '<a class="pomucky_potvrzeni" data-uzivatel="' . $idosoby . '" data-osoba="' . $prihlasenaOsoba . '" href="?menuid=pracovnipomucky&action=potvrzeniprevzeti&osoba=' . $idosoby . '&pomucka=' . HttpUtils::urlEncode($rowData[0]) .
                    '&datumnaroku=' . $this->database->stringToTimestamp($rowData[10]) . '&poradi=' . $rowData[3] . '"
                    data-ron-confirm-text="' . ronText('dotaz_potvrdit_prevzeti_za_jinou_osobu') . '">';
                $result .= ronText('potvrdit');
                $result .= '</a>';
            }
        }
        return $result;
    }
}

/**
 * Class DatumVraceniField
 * @package Ron\Personalistika
 */
class DatumVraceniField extends BrowserField //přidání nového sloupce
{
    var $pravaZapisPrevzeti;
    var $pomuckyPotvrzeni;
    /**
     * DatumVraceniField constructor.
     * @param null|Database $database
     * @param array|null $options
     */
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header= ronText('datum_vraceni');
        $this->align="center";
        $this->ordering = 1;

        $this->sortSQL = "PrirazenaPracPomucka.DatumVraceni";
        $this->sortSQLDesc = "PrirazenaPracPomucka.DatumVraceni DESC";
        $this->pomuckyPotvrzeni = Prava::vratPrava('pomucky_vydej');
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $idosoby = $rowData[16];
        if ($rowData[2] != '') {
                $result = '<DIV title="'. $this->zjistiPocet(1,2,1,$rowData) .'">';
                $result .= StringUtils::format('date', $this->database->stringToTimestamp($rowData[2]));
                $result .= '</DIV>';
        } else {
            if ($this->pomuckyPotvrzeni->lzeZobrazitRC($rowData[6])) {
                if ($rowData[1] != '') {
                    $result = '<a href="?menuid=pracovnipomucky&action=pomuckyform&formaction=edit&osoba=' . $idosoby . '&pomucka=' . HttpUtils::urlEncode($rowData[0]) .
                        '&datumnaroku=' . $this->database->stringToTimestamp($rowData[10]) . '&poradi=' . $rowData[3] . '&vydanipomucky=0">';
                    $result .= ronText('vratit');
                    $result .= '</a>';
                } else {
                    $result = "";
                }
            }
        }
        return $result;
    }

    function zjistiPocet ($jedna, $dva, $priznak, $rowData)
    {
        if ($priznak == 1) {
            $pocet = DateUtils::daysBetween($this->database->stringToTimestamp($rowData[$jedna]),$this->database->stringToTimestamp($rowData[$dva]));
            if ($pocet == 1) {
                return ronText('zapujceno') . StringUtils::format('number0',$pocet) . ronText('den');
            } elseif ($pocet > 1 && $pocet < 5) {
                return ronText('zapujceno') . StringUtils::format('number0',$pocet) . ronText('dny');
            } else {
                return ronText('zapujceno') . StringUtils::format('number0',$pocet) . ronText('dni');
            }
        }
        return null;
    }
}

/**
 * Class DatumVydaniField
 * @package Ron\Personalistika
 */
class DatumVydaniField extends BrowserField
{
    var $pravaZapisPrevzeti;
    /**
     * DatumVydaniField constructor.
     * @param null|Database $database
     * @param array|null $options
     */
    function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = ronText('datum_vydani');
        $this->align = "center";
        $this->ordering = 1;
        $this->sortSQL = "PrirazenaPracPomucka.DatumVydani";
        $this->sortSQLDesc = "PrirazenaPracPomucka.DatumVydani DESC";
        $this->pravaZapisPrevzeti = Prava::vratPrava('pomucky_vydej');
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $idosoby = $rowData[16];
        if($rowData[1] != ''){
            $result .= StringUtils::format('date', $this->database->stringToTimestamp($rowData[1]));
        }else{
            if ($this->pravaZapisPrevzeti->lzeZobrazitRC($rowData[6])) {
                $result = '<a href="?menuid=pracovnipomucky&action=pomuckyform&formaction=edit&osoba=' . $idosoby . '&pomucka=' . HttpUtils::urlEncode($rowData[0]) .
                    '&datumnaroku=' . $this->database->stringToTimestamp($rowData[10]) . '&poradi=' . $rowData[3] . '&vydanipomucky=1">';
                $result .= ronText('vydat');
                $result.='</a>';
            }
        }
        return $result;
    }
}

/**
 * Class EditaceExpiraceField
 * @package Ron\Personalistika
 */
class EditaceExpiraceField extends BrowserField //Přidání odkazu na editace data expirace
{
    /** @var  PravaNaOsoby */
    var $pravaZapisPrevzeti;
    function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = ronText('datum_expirace');
        $this->pravaZapisPrevzeti = Prava::vratPrava('pomucky_vydej');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sortSQL = "PrirazenaPracPomucka.DatumExpirace";
        $this->sortSQLDesc = "PrirazenaPracPomucka.DatumExpirace DESC";
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $idosoby = $rowData[16];

        if($rowData[2] != ''){
            $result .= StringUtils::format('date', $this->database->stringToTimestamp($rowData[4]));
        } else {

            if ($this->pravaZapisPrevzeti->lzeZobrazitRC($rowData[6]) && $rowData[1] != '') {
                $result = '<a href="?menuid=pracovnipomucky&action=editaceexpirace&formaction=edit&osoba=' . $idosoby . '&pomucka=' . HttpUtils::urlEncode($rowData[0]) .
                    '&datumnaroku=' . $this->database->stringToTimestamp($rowData[10]) . '&poradi=' . $rowData[3] . '&editexpirace=1">';
                if ($rowData[4] == '') {
                    $result .= "<DIV title='" . ronText('uprava_expirace_pomucky') . "'>" . ronText('expirace_neuvedena');
                    $result .= '</a>';
                } else {
                    $result .= "<DIV title='" . ronText('info_editace_expirace_pomucky') . "'>" . StringUtils::format('date', $this->database->stringToTimestamp($rowData[4]));
                    $result .= '</a>';
                }
            }
        }
        return $result;
    }
}

class ZobrazDatumExpiraceVypocteneField extends BrowserField
{

    function __construct($database, $options)
    {
        parent::__construct($database, $options);

        $this->header = ronText('datum_expirace_vypoctene');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sortSQL = "PrirazenaPracPomucka.DatumExpiraceVypoctene";
        $this->sortSQLDesc = "PrirazenaPracPomucka.DatumExpiraceVypoctene DESC";
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        if( $rowData[1] != ''){
            if ($rowData[18] == '') {
                $result .= "<DIV title='" . ronText('uprava_expirace_pomucky') . "'>" . ronText('expirace_neuvedena');
                $result .= '</DIV>';
            } else {
                $result .= StringUtils::format('date', $this->database->stringToTimestamp($rowData[18]));
                $result .= '</DIV>';
            }
        }
        return $result;
    }
}

/**
 * Class TypPomuckyField
 * @package Ron\Personalistika
 */
class TypPomuckyField extends BrowserField
{
    function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = ronText('typ_pomucky');
        $this->ordering = 1;
        $this->sortSQL = "PracovniPomucka.TypPomucky";
        $this->sortSQLDesc = "PracovniPomucka.TypPomucky DESC";
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $result .= '<a href="?menuid=pracovnipomucky&typpomucky=' . HttpUtils::urlEncode($rowData[13]) . '">';
        $result .= StringUtils::htmlTextConvert($rowData[7]);
        $result .= '</a>';
        return $result;
    }
}

class PopisPomickyField extends BrowserField
{
    function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = ronText('Popis');
        $this->ordering = 1;
        $this->sortSQL = "PracovniPomucka.Popis";
        $this->sortSQLDesc = "PracovniPomucka.Popis DESC";
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
//        $result .= '<a href="?menuid=pracovnipomucky&record=' . ($rowData[0]) . '">';
        $result .= '<a href="?menuid=pracovnipomucky&record=' . HttpUtils::urlEncode($rowData[0]) . '">';
        $result .= StringUtils::htmlTextConvert($rowData[8]);
        $result .= '</a>';
        return $result;
    }
}

/**
 * Class DetailPomuckyField
 * @package Ron\Personalistika
 */
class DetailPomuckyField extends BrowserField
{
    function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = ronText('detail');
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $result .= '<a href="?menuid=pracovnipomucky&action=detail&record=' . HttpUtils::urlEncode($rowData[0]). '">';
        $result .= ronText('detail');
        $result .= '</a>';
        return $result;
    }
}

/**
 * Class PracovniPomuckyBrowser
 * @package Ron\Personalistika
 */
class PracovniPomuckyBrowser extends Browser
{
    /** @var PravaNaOsoby  */
    var $pravaProhlizet;
    /** @var PravaNaOsoby  */
    var $pravaZapisPrevzetiVydej;
    /** @var PravaNaOsoby  */
    var $pomuckyPotvrzeni;

    /** @var PracovniPomuckyFilterForm */
    var $filterForm;
    var $typRozdeleni;
    var $rozdeleni;

    /**
     * PracovniPomuckyBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    function __construct($database) {
        parent::__construct($database);

        $this->pravaProhlizet = Prava::vratPrava('pomucky_prohl');
        $this->pravaZapisPrevzetiVydej = Prava::vratPrava('pomucky_vydej');
        $this->pomuckyPotvrzeni = Prava::vratPrava('pomucky_potvrzeni');
        Prava::nastavObdobiProPrava(time(),time());

        $this->title = ronText('pracovni_pomucky');
        $this->code = 'browserpracovnipomucky';
        $this->tableId = 'browserpracovnipomucky';

        $this->sqlSelectPart =
            'PracovniPomucka.Kod, PrirazenaPracPomucka.DatumVydani, PrirazenaPracPomucka.DatumVraceni, PrirazenaPracPomucka.Poradi,'. // 0 - 3
            'PrirazenaPracPomucka.DatumExpirace,  {fn_CASTWIDEMEMO(PrirazenaPracPomucka.Poznamka)}, PrirazenaPracPomucka.RC,'. // 4 - 6
            'TypPomucky.Popis, PracovniPomucka.Popis, PracovniPomucka.Hodnota, PrirazenaPracPomucka.DatumNaroku, '.   //7 - 10
            'PotvrzeniPrevzetiDatum, PotvrzeniPrevzetiPopis, TypPomucky.Kod, Osoba.Jmeno, Osoba.Prijmeni, '. //11 - 15
            '(SELECT MAX(ID) FROM PracovniPomer WHERE RC = Osoba.RC), VyzadovatPotvrzeni, PrirazenaPracPomucka.DatumExpiraceVypoctene, ' .  //16 - 18
            '(SELECT OsobniCislo FROM PracovniPomer WHERE ID = (SELECT MAX(ID) FROM PracovniPomer WHERE RC = Osoba.RC)) AS OsobniCislo '; //19

        $this->sqlFromPart =
            'PrirazenaPracPomucka, PracovniPomucka, StavPomucky, TypPomucky, Osoba';

        $this->sqlWherePart =
            'PrirazenaPracPomucka.PracovniPomucka=PracovniPomucka.Kod
             AND PrirazenaPracPomucka.RC = Osoba.RC
             AND PracovniPomucka.Stav = StavPomucky.Kod
             AND PracovniPomucka.TypPomucky = TypPomucky.Kod';

        $this->defaultSort = 5;
        $this->primaryField = 0;
        $this->browserUrl = '?menuid=pracovnipomucky&';

        $this->maxPageRows = 100;

        $this ->filterForm = new PracovniPomuckyFilterForm($this ->database, $this->pravaProhlizet, $this->pomuckyPotvrzeni, $this->pravaZapisPrevzetiVydej);
        $this ->filterForm ->loadData();

        $this->addField(new TypPomuckyField($database, null));

        $this->addField(new PopisPomickyField($database, null));

        /*$this ->addField([BROWSER_FIELD_SQL_INDEX => 8, BROWSER_FIELD_SQL_NAME => 'PracovniPomucka.Popis',
                BROWSER_FIELD_HEADER_TEXT => 'Popis', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_LINK => '?menuid=pracovnipomucky']);*/

        $this->addField(new DetailPomuckyField($database,null));

        $this->addField(new DatumVydaniField($database, null));

        $this->addField(new DatumVraceniField($database, null));

        $this->addField(new EditaceExpiraceField($database, null));

        $this->addField(new ZobrazDatumExpiraceVypocteneField($database, null));


/*        $this->addField([BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_SQL_NAME => 'PrirazenaPracPomucka.DatumExpirace',
            BROWSER_FIELD_HEADER_TEXT => 'datum_expirace', 'format' => 'date' , 'align' => 'center', 'ordering' => 1]);*/

        $this->addField(new PotvrzeniPrevzetiField($database, null));

        if (RonPortalSession::vsechnyOsoby()) {
            $this->addField([BROWSER_FIELD_SQL_INDEX => 19, BROWSER_FIELD_SQL_NAME => 'OsobniCislo',
                BROWSER_FIELD_HEADER_TEXT => 'osobni_cislo', BROWSER_FIELD_ORDERING => 1]);

           $this->addField([BROWSER_FIELD_SQL_INDEX => 15, BROWSER_FIELD_SQL_NAME => 'Osoba.Prijmeni',
                BROWSER_FIELD_HEADER_TEXT => 'Prijmeni', BROWSER_FIELD_ORDERING => 1]);

            $this->addField([BROWSER_FIELD_SQL_INDEX => 14, BROWSER_FIELD_SQL_NAME => 'Osoba.Jmeno',
                BROWSER_FIELD_HEADER_TEXT => 'Jmeno', BROWSER_FIELD_ORDERING => 1]);
        }
    }

    /**
     * @param string $sqlFilter
     * @throws Exception
     */
    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);
    }

    function renderFilter()
    {
        parent::renderFilter();
        $this->filterForm->render();
    }

    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        $hodnotaSloupce = 5;
        $colspan = ($this->columnCount - 1);
        if($values[$hodnotaSloupce]){
            echo "<tr class='$className'>";
            echo "<td></td><td colspan=" . $colspan . ">";
            echo '<span id="poznamka"><strong>' . ronText('Poznamka') . '</strong>: ' . StringUtils::htmlTextConvert($values[$hodnotaSloupce] ). '</span>';
            echo '</td>';
            echo '</tr>';
        }

        $this->renderSeparator();
    }

    /**
     * @return SqlStatement|string
     */
    public function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $result =   "(" .
                    $this->pravaProhlizet->sqlFiltr('PrirazenaPracPomucka',time(),time(), $params) .
                    " OR " .
                    $this->pravaZapisPrevzetiVydej->sqlFiltr('PrirazenaPracPomucka',time(),time(), $params) .
                    " OR " .
                    $this->pomuckyPotvrzeni->sqlFiltr('PrirazenaPracPomucka',time(),time(), $params) .
                    ")";
        if (!RonPortalSession::vsechnyOsoby()) {
            $result .= " AND PrirazenaPracPomucka.RC = " . $params->addParameter(RonPortalSession::osobaRC());
        }
        else {
            require_once('adsapp/rozdeleni.php');
            if (RonPortalSession::rozdeleni() != '') {
                $result .=  'AND ' . Rozdeleni::sqlFilterRozdeleni($this->database, 'PrirazenaPracPomucka', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), RonPortalSession::rozdeleni(),
                        DateUtils::today(), DateUtils::today(), RonPortalSession::rozdeleniVcetnePodrizenych());
            }
            if (RonPortalSession::rozdeleni2() != '') {
                $result .=  'AND ' . Rozdeleni::sqlFilterRozdeleni($this->database, 'PrirazenaPracPomucka', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), RonPortalSession::rozdeleni2(),
                        DateUtils::today(), DateUtils::today(), RonPortalSession::rozdeleni2VcetnePodrizenych());
            }
        }

        $result .= " AND ((PrirazenaPracPomucka.DatumVydani >= " . $params->addParameter($this->database->timestampToParam(RonPortalSession::getDatumOd())) .
                    " AND PrirazenaPracPomucka.DatumVydani <= " . $params->addParameter($this->database->timestampToParam(RonPortalSession::getDatumDo()));
        $result .= " ) OR PrirazenaPracPomucka.DatumVydani IS NULL )";


        $resultVydani = '';

        if ($_SESSION['jenNevydane']) {
            $resultVydani .= " AND (PrirazenaPracPomucka.DatumVydani IS NULL";
        }

        if ($_SESSION['jenNevracene']) {
            if ($resultVydani != '') {
                $resultVydani .= ' OR ';
            } else {
                $resultVydani .= ' AND ( ';
            }
            $resultVydani .= " PrirazenaPracPomucka.DatumVraceni IS NULL";
        }

        if ($_SESSION['jenVracene']) {
            if ($resultVydani != '') {
                $resultVydani .= ' OR ';
            } else {
                $resultVydani .= ' AND ( ';
            }
            $resultVydani .= " PrirazenaPracPomucka.DatumVraceni IS NOT NULL";
        }

        if($_SESSION['jenNepotvrzene'] ){
            if($resultVydani != ''){
                $resultVydani .=' OR';
            }else{
                $resultVydani .= ' AND ( ';
            }
            $resultVydani .= " ( PrirazenaPracPomucka.DatumVydani IS NOT NULL AND  PrirazenaPracPomucka.PotvrzeniPrevzetiDatum IS NULL AND PrirazenaPracPomucka.DatumVraceni IS NULL )";
        }

        if($_SESSION['jenPotvrzene'] ){
            if($resultVydani != ''){
                $resultVydani .=' OR';
            }else{
                $resultVydani .= ' AND ( ';
            }
            $resultVydani .= " (PrirazenaPracPomucka.DatumVydani IS NOT NULL AND PrirazenaPracPomucka.PotvrzeniPrevzetiDatum IS NOT NULL AND PrirazenaPracPomucka.DatumVraceni IS NULL  )";
        }

        if($_SESSION['jenExpirovane'] ){
            if($resultVydani != ''){
                $resultVydani .=' OR';
            }else{
                $resultVydani .= ' AND ( ';
            }
            $resultVydani .= " ( PrirazenaPracPomucka.DatumVraceni IS NULL AND PrirazenaPracPomucka.DatumExpirace IS NOT NULL AND PrirazenaPracPomucka.DatumExpirace <= " .
                $params->addParameter($this->database->timestampToParam(DateUtils::today())) . ") ";
        }


        if ($resultVydani != '') {
            $resultVydani .= ')';
        }

        $result .= $resultVydani;

        if ($_SESSION['jenTypPomucky']) {
            $result .= " AND PracovniPomucka.TypPomucky = " . $params->addParameter($_SESSION['typPomucky']);
        }

        if($_SESSION['jenPomucka']){
            $result .= " AND PracovniPomucka.Kod = " . $params->addParameter($_SESSION['pomucka']);
        }
        return new SqlStatement($result, $params);
    }
}
