<?php

/* 
 * Pavel Rudol, RON Software
 *  10.12.2015
 */

namespace Ron\RonPortal;

require_once('adsapp/ronportalbrowser.php');

use Exception;
use Ron;
use Ron\Controls\ComboBox;
use Ron\BrowserField;
use Ron\Controls\ICreateItem;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use Ron\StringUtils;
use function Ron\ronText;


class UmisteniSkoleniCombo extends ComboBox implements ICreateItem
{
    function createItem($row, $keyIndex = 0, $captionIndex = 1, $titleIndex = null, $cssClassIndex = null)
    {
        if (!$row) {
            return new Option('', ' ');
        }

        $key = $row[$keyIndex];
        $caption = '';
        if (trim($row[1]) != '')
            $caption = $row[1] . ' - ';

        $caption .= $row[2];

        return new Option($key, $caption);
    }
}


/**
 * Class PrihlaseniSkoleniFilterForm
 * @package Ron\RonPortal
 */
class PrihlaseniSkoleniFilterForm extends RonPortalFilterForm
{
    public $hromadnePrihlaseni;

    /**
     * PrihlaseniSkoleniFilterForm constructor.
     * @param Database $database
     * @param $action
     * @param $hromadnePrihlaseni
     */
    public function __construct($database, $action, $hromadnePrihlaseni) {
        parent::__construct($database, $action);
        
        $prava = Prava::vratPrava('skoleni_prihlasit');
        $this ->nastavOsoby(true, $prava, time(), time()); // prava podle aktualniho casu
        $this->povolitVsechyOsoby = true;
        $this->hromadnePrihlaseni = $hromadnePrihlaseni;

        $this->nastavDatumOdDo(true, 0);
        $this->nastavRozdeleni(true);
        $this->f_classname = 'filtercontainer leftcaptions';
    }


    /**
     * @param bool $filtrPopis
     * @throws Ron\DB\DBException
     */
    public function renderTypPopisSkoleni($filtrPopis = false)
    {
        $varName = 'typSkoleni';
        if ($filtrPopis) {
            $varName = 'popisSkoleni';
        }

        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        if ($filtrPopis)
            echo ronText('Skoleni_jenPopis');
        else
            echo  ronText('Skoleni_jenTyp');
        echo '</SPAN>';

        // combobox pro vyber
        $params = $this->database->createParameters();

        //Puvodne byl filtr pro vybranou osobu, momentalne upraven na vyber vsech
        if ($filtrPopis)
            $sql = "SELECT DISTINCT Skoleni.Popis, Skoleni.Popis ";
        else
            $sql = "SELECT DISTINCT Skoleni.DruhSkoleni, DruhSkoleni.Popis ";

        $sql .=
            "FROM Skoleni JOIN DruhSkoleni ON (DruhSkoleni.Kod = Skoleni.DruhSkoleni)
             WHERE ( Skoleni.StavSkoleni IN (0, 1) AND ( Skoleni.DatumPocatkuSkoleni >= " . $params->addParameter($this->database->dateToParam(time())) .
            " AND (Skoleni.DatumPotvrzeniUcasti >= " . $params->addParameter($this->database->dateToParam(time())) .
            " OR Skoleni.DatumPotvrzeniUcasti IS NULL) OR DruhSkoleni.Elearning = 1 ))" .
            ' AND (Skoleni.DatumUkonceniSkoleni >= ' .$params->addParameter($this->database->timestampToParam(time())) . ' OR Skoleni.DatumUkonceniSkoleni IS NULL)';
        $sql .=
            " AND ((";
        $sql .=
            "EXISTS(SELECT * FROM PozadovanaSkoleni JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PozadovanaSkoleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PozadovanaSkoleni.KodRozdeleni)
				WHERE Skoleni.DruhSkoleni=PozadovanaSkoleni.DruhSkoleni   
				  AND PozadovanaSkoleni.Povinne = 0 " .
                (RonPortalSession::vsechnyOsoby() ? "" : "  AND RozdeleniOsob.RC =  " . $params->addParameter(RonPortalSession::osobaRC())) .
                " AND RozdeleniOsob.PlatnostOd <= Skoleni.DatumPocatkuSkoleni
				  AND (RozdeleniOsob.PlatnostDo >= Skoleni.DatumPocatkuSkoleni OR RozdeleniOsob.PlatnostDo IS NULL)) ";
        $sql .=
            " OR ";
        $sql .=
            "EXISTS(SELECT * FROM GenerovaneSkoleni
				WHERE Skoleni.DruhSkoleni=GenerovaneSkoleni.DruhSkoleni 
				  AND GenerovaneSkoleni.Potvrzeni = 1 " .
                (RonPortalSession::vsechnyOsoby() ? "" : " AND GenerovaneSkoleni.RC =  " . $params->addParameter(RonPortalSession::osobaRC())) .
				" AND ((GenerovaneSkoleni.CisloSkoleni = '' OR GenerovaneSkoleni.CisloSkoleni IS NULL) OR GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni)) ))";

        if ($filtrPopis)
            $sql .= 'ORDER BY Skoleni.Popis';
        else
            $sql .= ' ORDER BY DruhSkoleni.Popis, Skoleni.DruhSkoleni ';

        // vyber typu skoleni podle popisu
        $combo = new ComboBox($varName);
        $combo->emptyRow = ronText('_vse_');
        $combo->className = 'form-input adsfilterforminput_full ronselect';
        $combo->htmlParams = "data-ronselect-class='ronselect adsfilterforminput_full' data-ron-autosubmit='1' data-columns='text'";
        $combo->loadData($this ->database, $sql, $_SESSION[$varName], $params);
        $combo->render(); 

        echo "</DIV>\n";
    }
    

    /**
     * @throws Exception
     */
    public function renderUmisteni()
    {        
        echo '<DIV class="filterrow">';

        // checkbox "Jen umisteni"
        echo '<SPAN class="filtercaption">';
        echo ronText('Skoleni_jenUmisteni');
        echo '</SPAN>';

        // combobox pro vyber
        $sql =
            "SELECT Kod, Mesto, Popis FROM Umisteni ";

        $sql .=
            ' ORDER BY Popis';

        // vyber umisteni podle popisu
        $combo = new UmisteniSkoleniCombo('umisteni');
        $combo->className = 'form-input adsfilterforminput_full ronselect';
        $combo->htmlParams = "data-ronselect-class='ronselect adsfilterforminput_full' data-ron-autosubmit='1' data-columns='text'";
        $combo->emptyRow = ronText('_vse_');
        $combo->loadData($this ->database, $sql, $_SESSION['umisteni']); 
        $combo->render();         
        
        echo "</DIV>\n";
        
    }
    
    public function renderPovinnostSkoleni()
    {
        echo '<DIV class="filterrow">';

        // checkbox "Jen typ skoleni"
        echo '<SPAN class="filtercaption">';
        echo ronText('Skoleni_jenPovinne');
        echo '</SPAN>';

        // vyber povinne/doporucene 
        $povinneVse = new Option(0, ronText('_vse_'));
        $povinne = new Option(1, ronText('Skoleni_povinne'));
        $doporucene = new Option(2, ronText('Skoleni_doporucene'));
        
        $combo = new ComboBox('typPovinnosti',[$povinneVse,$povinne,$doporucene]);
        $combo->className = 'form-input adsfilterforminput_full ronselect';
        $combo->htmlParams = "data-ronselect-class='ronselect adsfilterforminput_full' data-ron-autosubmit='1' data-columns='text'";
        $combo->selectedOption = $combo->findOption((int)$_SESSION['typPovinnosti']);
        $combo->render();
        
        echo "</DIV>\n";
    }


    /**
     * @throws Ron\DB\DBException
     */
    public function renderSpecifikaceKurzu()
    {
        echo '<div class="filterrow">';

        echo '<span class="filtercaption">';
        echo ronText('Skoleni_specifikace_filtr');

        echo '</span>';

        $params = $this->database->createParameters();
        $sql =
            "SELECT Kod, Popis FROM SpecifikaceKurzu ORDER BY Popis";

        // vyber specifikace skoleni
        $combo = new ComboBox('specifikaceSkoleni');
        $combo->emptyRow = ronText('_vse_');

        $combo->className = 'form-input adsfilterforminput_full ronselect';
        $combo->htmlParams = "data-ronselect-class='ronselect adsfilterforminput_full' data-ron-autosubmit='1' data-columns='text'";
        $combo->loadData($this ->database, $sql, $_SESSION['specifikaceSkoleni'], $params);
        $combo->render();

        echo "</div>";
    }




    /**
     * @throws Ron\DB\DBException
     */
    public function renderKategorieKurzu()
    {
        echo '<div class="filterrow">';

        echo '<span class="filtercaption">';
        echo ronText('Skoleni_kategorie_filtr');

        echo '</span>';

        $params = $this->database->createParameters();
        $sql =
            "SELECT Kod, Popis FROM KategorieSkoleni ORDER BY Popis";

        // vyber specifikace skoleni
        $combo = new ComboBox('kategorieSkoleni');
        $combo->emptyRow = ronText('_vse_');

        $combo->className = 'form-input adsfilterforminput_full ronselect';
        $combo->htmlParams = "data-ronselect-class='ronselect adsfilterforminput_full' data-ron-autosubmit='1' data-columns='text'";
        $combo->loadData($this ->database, $sql, $_SESSION['kategorieSkoleni'], $params);
        $combo->render();

        echo "</div>";
    }


    public function renderHromadnePrihlaseni()
    {
        echo "<br>";
        echo '<div class="filterrow">';
        $popis = ronText('Skoleni_prihlasit_hromadne');
        echo "<button class='button clickHref' type='button' name='hromadne_prihlaseni' data-ron-href='?menuid=skoleni&action=zobrazprihlaseni&hromadneprihlaseni=1' >$popis</button>";
        echo "</div>";
    }


    private function loadSpecifikaceSkoleni()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['specifikaceSkoleni'] = $_POST['specifikaceSkoleni'];
        }
    }

    private function loadKategorieSkoleni()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['kategorieSkoleni'] = $_POST['kategorieSkoleni'];
        }
    }

    private function loadTypSkoleni()
    {
        if (isset($_POST['adsfilterform'])) {            
            $_SESSION['jenTypSkoleni'] = ($_POST['typSkoleni'] !== '') ? true : false;
            $_SESSION['typSkoleni'] = $_POST['typSkoleni'];
        }
        elseif (isset($_GET['typskoleni'])) {
            $_SESSION['jenTypSkoleni'] = TRUE;
            $_SESSION['typSkoleni'] = $_GET['typskoleni'];
        }
    }
    
    private function loadUmisteni()
    {
        if (isset($_POST['adsfilterform'])) {            
            $_SESSION['jenUmisteni'] = ($_POST['umisteni'] !== '') ? true : false;
            $_SESSION['umisteni'] = $_POST['umisteni'];            
        }           
    }
    
    private function loadPovinnost()
    {
        if (isset($_POST['adsfilterform'])) {            
            $_SESSION['jenPovinne'] = ($_POST['typPovinnosti'] !== '') ? true : false;
            $_SESSION['typPovinnosti'] = $_POST['typPovinnosti'];             
        }           
    }

    private function loadPopisSkoleni()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenPopisSkoleni'] = ($_POST['popisSkoleni'] !== '') ? true : false;
            $_SESSION['popisSkoleni'] = $_POST['popisSkoleni'];
        }
    }


    public function loadData()
    {
        $this->loadTypSkoleni();
        
        $this->loadUmisteni();
        
        $this->loadPovinnost();

        $this->loadSpecifikaceSkoleni();

        $this->loadKategorieSkoleni();

        $this->loadPopisSkoleni();
        
        parent::loadData();        
    }

    /**
     * @throws Exception
     */
    public function renderFilters()
    {
        $this->comboOsoba->className .= ' ronselect';
        $this->comboOsoba->htmlParams['data-ronselect-class'] = 'ronselect adsfilterforminput_full';
        $this->comboOsoba->htmlParams['data-columns'] = 'text';

        parent::renderFilters(); // osoba      

        $this->renderSpecifikaceKurzu();

        $this->renderKategorieKurzu();

        $this->renderTypPopisSkoleni();

        $this->renderUmisteni();

        $this->renderPovinnostSkoleni();

        $this->renderTypPopisSkoleni(true);

        if (!$this->hromadnePrihlaseni){ $this->renderHromadnePrihlaseni();}
    }
}


/**
 * Class PrihlaseniSkoleniBrowserField
 * @package Ron\RonPortal
 */
class PrihlaseniSkoleniBrowserField extends BrowserField
{
    private $kapacitaIndex;
    private $prihlasenychIndex;
    private $cisloSkoleniIndex;
    private $elearningIndex;
    private $osobaIDIndex;
    private $osobaIDPPIndex;
    private $pocetAbsolvovanychElearninguIndex;
    private $datumPovinnostiIndex;
    private $vyzadovatSchvaleniIndex;
    private $schvaleniPrihlaskyIndex;
    private $pocetPrihlasekOsobyIndex;
    private $prihlasenIndex;
    public $hromadnePrihlaseni;


    /**
     * @param Database $database
     * @param int $kapacitaIndex
     * @param int $prihlasenychIndex
     * @param int $cisloSkoleniIndex
     * @param int $elearningIndex
     * @param int $osobaIDIndex
     * @param int $osobaIDPPIndex
     * @param int $pocetAbsolvovanychElearninguIndex
     * @param int $datumPovinnostiIndex
     * @param int $vyzadovatSchvaleniIndex
     * @param int $schvaleniPrihlaskyIndex
     * @param int $pocetPrihlasekOsobyIndex
     * @param $prihlasenIndex
     * @param int | null $hromadnePrihlaseni
     */
    function __construct($database, $kapacitaIndex, $prihlasenychIndex, $cisloSkoleniIndex, $elearningIndex, $osobaIDIndex, $osobaIDPPIndex,
                         $pocetAbsolvovanychElearninguIndex, $datumPovinnostiIndex, $vyzadovatSchvaleniIndex, $schvaleniPrihlaskyIndex, $pocetPrihlasekOsobyIndex,
                         $prihlasenIndex, $hromadnePrihlaseni=null)
    {
        parent::__construct($database);

        $this->align = 'center';

        if ($hromadnePrihlaseni){
            $this->header = ronText('Skoleni_prihlasit_odhlasit');
        }

        $this->kapacitaIndex = $kapacitaIndex;
        $this->prihlasenychIndex = $prihlasenychIndex;
        $this->cisloSkoleniIndex = $cisloSkoleniIndex;
        $this->elearningIndex = $elearningIndex;
        $this->osobaIDIndex = $osobaIDIndex;
        $this->osobaIDPPIndex = $osobaIDPPIndex;
        $this->pocetAbsolvovanychElearninguIndex = $pocetAbsolvovanychElearninguIndex;
        $this->datumPovinnostiIndex = $datumPovinnostiIndex;
        $this->vyzadovatSchvaleniIndex = $vyzadovatSchvaleniIndex;
        $this->schvaleniPrihlaskyIndex = $schvaleniPrihlaskyIndex;
        $this->pocetPrihlasekOsobyIndex = $pocetPrihlasekOsobyIndex;
        $this->prihlasenIndex = $prihlasenIndex;
        $this->hromadnePrihlaseni = $hromadnePrihlaseni;
    }

    /**
     * @param string $skoleni kod skoleni
     * @param string $rc RC osoby
     * @return boolean true pokud ma osoba ucast na nekterem z terminu skoleni
     * @throws Exception
     */
    private function jeOsobaNaTerminuSkoleni($skoleni, $rc)
    {
        $params = $this->database->createParameters();

        $sql =  "SELECT RC FROM UcastNaTerminuSkoleni " .
                "WHERE CisloSkoleni = " . $params->addParameter($skoleni) .
                "  AND RC = " . $params->addParameter($rc);

        $cursor = $this->database->select($sql, $params);

        $row = $cursor->fetch();

        if ($row)
            return TRUE;
        else
            return FALSE;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $prihlasen = false;
        if ($rowData[$this->prihlasenIndex])
            $prihlasen = true;

        $datumPovinnostiStr = '';
        if ($rowData[$this->datumPovinnostiIndex]) {
            $datumPovinnosti = $this->database->stringToTimestamp($rowData[$this->datumPovinnostiIndex]);
            $datumPovinnostiStr = '&datumpovinnosti=' . $datumPovinnosti;
        }

        $pravaPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        $pravaPotvrditPrihlaseni = Prava::vratPrava('skoleni_schvalit_prihlaseni');

        $prihlasit = false;
        $odhlasit = false;
        $schvalitPrihlasku = false;

        $prihlasitStr = null;

        if ($pravaPrihlasit->lzeZobrazitRC($rowData[$this->osobaIDIndex]))
            $prihlasitStr = ronText('Skoleni_prihlasit');

        $result = '';

        if ($rowData[$this->vyzadovatSchvaleniIndex] == 1) {
            //pokud je vyzadovano potvrzeni schvaleni prihlasky a uzivatel nema pravo ke schvaleni prihlasky, zobrazi se text registrovat a prihlaska bude cekat na schvaleni
            if (($pravaPrihlasit->lzeZobrazitRC($rowData[$this->osobaIDIndex])) && (!($pravaPotvrditPrihlaseni->lzeZobrazitRC($rowData[$this->osobaIDIndex]))))
                $prihlasitStr = ronText('Skoleni_registrovat_zajem');

            //existuje vygenerovany pozadavek, nebo jiz probehlo prihlaseni
            if (isset($datumPovinnosti)) {
                if ($pravaPotvrditPrihlaseni->lzeZobrazitRC($rowData[$this->osobaIDIndex])) {
                    //pokud zatim nebylo schvaleno a uzivatel ma pravo schvalit zobrazi odkaz na schvaleni/zamitnuti
                    if (($rowData[$this->schvaleniPrihlaskyIndex] == 0) && $prihlasen) {
                        $schvalitPrihlasku = true;
                    }
                    //u jiz schvalene registrace, umozni uzivateli s pravem schvalovani prihlasky odhlaseni
                    elseif ($prihlasen && ($rowData[$this->schvaleniPrihlaskyIndex] == 1))
                        $odhlasit = true;
                }
                elseif ($pravaPrihlasit->lzeZobrazitRC($rowData[$this->osobaIDIndex])) {
                    if ($prihlasen && ($rowData[$this->schvaleniPrihlaskyIndex] == 0))
                        $odhlasit = true;
                }
            }
            if (!$prihlasen)
                $prihlasit = true;
        }


        if ($rowData[$this->elearningIndex] == 1) {
            if ((((int)$rowData[$this->vyzadovatSchvaleniIndex] == 0) || ((int)$rowData[$this->schvaleniPrihlaskyIndex] != 0)) && ($rowData[$this->pocetPrihlasekOsobyIndex] <> $rowData[$this->pocetAbsolvovanychElearninguIndex]))
                return '';

            if ($pravaPrihlasit->lzeZobrazitRC($rowData[$this->osobaIDIndex])) {
                if (!$prihlasen)
                    $prihlasit = true;
            }
        }
        //ne elearningove kurzy
        else {
            if ($pravaPrihlasit->lzeZobrazitRC($rowData[$this->osobaIDIndex])) {
                if (!($rowData[$this->vyzadovatSchvaleniIndex] == 1)) {
                    if ($prihlasen)
                        $odhlasit = true;
                    elseif ($rowData[$this->prihlasenychIndex] < $rowData[$this->kapacitaIndex])
                        $prihlasit = true;
                }
            }
        }

        if ($prihlasit){

            if (!$this->hromadnePrihlaseni) {
                $result =
                    '<a href="?menuid=skoleni&action=prihlaseni&osoba=' . $rowData[$this->osobaIDPPIndex] . $datumPovinnostiStr . '&cisloskoleni=' . $rowData[$this->cisloSkoleniIndex] . '">' .
                    $prihlasitStr .
                    '</a>';

                /*$jmeno = StringUtils::htmlTextConvert($rowData[19]) . " " .  StringUtils::htmlTextConvert($rowData[18]);
                $datum = date("j.n.Y", $this->database->stringToTimestamp($rowData[4]));
                $datumKurzu = $this->database->stringToTimestamp($rowData[4]);
                $ajaxPrihlasitNaSkoleni =
                    "<span 
                        class='prihlasit-kurz js' 
                        data-ron-akce='prevestobjednavku'
                        data-ron-idpp='$rowData[20]'
                        data-ron-jmeno='$jmeno'
                        data-ron-datum='$datum'
                        data-ron-misto='$rowData[13]'
                        data-ron-mistopopis='$rowData[28]'
                        data-ron-terminkurzu='$datumKurzu'
                        data-ron-cisloobjednavky='$rowData[0]'>" . ronText('Skoleni_prihlasit_kurz') .
                    "</span>";
                $result .= " / " . $ajaxPrihlasitNaSkoleni;*/

            } else {
                $index = $rowData[$this->osobaIDPPIndex] . "_" .$rowData[$this->cisloSkoleniIndex] . "_" . $datumPovinnosti;
                $result .= "<input type='hidden'   name='prihlasitNaSkoleni[$index][ID]' value='{$rowData[$this->osobaIDPPIndex]}'  />";
                $result .= "<input type='hidden'   name='prihlasitNaSkoleni[$index][cislo]' value='{$rowData[$this->cisloSkoleniIndex]}'  />";
                $result .= "<input type='hidden'   name='prihlasitNaSkoleni[$index][datumpovinnosti]' value='{$datumPovinnosti}'  />";
                $result .= "<input type='hidden'   name='prihlasitNaSkoleni[$index][jizPrihlaseno]' value='0'  />";
                $result .= "<input type='checkbox' class='hromadne-prihlasit' name='prihlasitNaSkoleni[$index][prihlasit]' value='1'  />";
            }
        }

        if ($odhlasit) {
            if (!$this->hromadnePrihlaseni) {
                $result = '<a data-prihlaseny-terminy="' . ($this->jeOsobaNaTerminuSkoleni($rowData[$this->cisloSkoleniIndex], $rowData[$this->osobaIDIndex]) ? 1 : 0) . '"' .
                    ' data-skoleni-kod="' . $rowData[$this->cisloSkoleniIndex] . '" data-ponechat-navrh="' . RonPortalSettings::getIntValue(SkoleniController::NASTAVENI_PONECHAT_NAVRH) . '"' .
                    ' href="?menuid=skoleni&action=odhlaseni&osoba=' . $rowData[$this->osobaIDPPIndex] . $datumPovinnostiStr . '&cisloskoleni=' . $rowData[$this->cisloSkoleniIndex] . '">' . ronText('Skoleni_odhlasit') . '</a>';

/*                $jmeno = StringUtils::htmlTextConvert($rowData[19]) . " " .  StringUtils::htmlTextConvert($rowData[18]);
                $datum = date("j.n.Y", $this->database->stringToTimestamp($rowData[4]));
                $datumKurzu = $this->database->stringToTimestamp($rowData[4]);
                $ajaxPrihlasitNaSkoleni =
                    "<span 
                        class='prihlasit-kurz js' 
                        data-ron-akce='prevestobjednavku'
                        data-ron-idpp='$rowData[17]'
                        data-ron-jmeno='$jmeno'
                        data-ron-datum='$datum'
                        data-ron-misto='$rowData[13]'
                        data-ron-mistopopis='$rowData[28]'
                        data-ron-terminkurzu='$datumKurzu'
                        data-ron-cisloobjednavky='$rowData[0]'>" . ronText('Skoleni_prihlasit_kurz') .
                    "</span>";
                $result .= " / " . $ajaxPrihlasitNaSkoleni;*/

            } else {
                $index = $rowData[$this->osobaIDPPIndex] . "_" .$rowData[$this->cisloSkoleniIndex] . "_" . $datumPovinnosti;
                $result .= "<input type='hidden'   name='prihlasitNaSkoleni[$index][ID]' value='{$rowData[$this->osobaIDPPIndex]}'  />";
                $result .= "<input type='hidden'   name='prihlasitNaSkoleni[$index][cislo]' value='{$rowData[$this->cisloSkoleniIndex]}'  />";
                $result .= "<input type='hidden'   name='prihlasitNaSkoleni[$index][datumpovinnosti]' value='{$datumPovinnosti}'  />";
                $result .= "<input type='hidden'   name='prihlasitNaSkoleni[$index][jizPrihlaseno]' value='1'  />";
                $result .= "<input type='checkbox' class='hromadne-prihlasit' name='prihlasitNaSkoleni[$index][prihlasit]' value='1' checked='checked' />";
            }
        }

        if ($schvalitPrihlasku) {
            $result =
                '<a href="?menuid=skoleni&action=schvaleniprihlaseni&osoba=' . $rowData[$this->osobaIDPPIndex] . $datumPovinnostiStr . '&cisloskoleni=' . $rowData[$this->cisloSkoleniIndex] . '">' .
                    ronText('Skoleni_schvaleni_registrace') .
                '</a> &nbsp ' .
                '<a href="?menuid=skoleni&action=zamitnutiprihlaseni&osoba=' . $rowData[$this->osobaIDPPIndex] . $datumPovinnostiStr . '&cisloskoleni=' . $rowData[$this->cisloSkoleniIndex] . '">' .
                    ronText('Skoleni_zamitnuti_registrace') .
                '</a>';
        }

        return $result;
    }

    function getFooterText($fieldpos, $url = null)
    {
        //TODO doplnit jmeno

        //TODO js na hromadne prihlaseni

        //TODO odchyceni v controlleru
        return "<input type='checkbox' class='align-center' name='prihlaseni_ALL' id='prihlaseni_ALL' value='1' />\n";
    }
}


/**
 * Class PovinneSkoleniBrowserField
 * @package Ron\RonPortal
 */
class PovinneSkoleniBrowserField extends BrowserField
{
    private $skoleniIndex;
    private $druhSkoleniIndex;
    private $pocatekSkoleniIndex;
    private $osobaIDIndex;

    /**
     * @param Database $database
     * @param string $header
     * @param int $index
     * @param int $indexDruh
     * @param int $indexPocatek
     * @param int $osobaIDIndex
     */
    function __construct($database, $header, $index, $indexDruh, $indexPocatek, $osobaIDIndex)
    {
        parent::__construct($database);

        $this->header = $header;
        $this->align = 'center';
     
        $this->skoleniIndex = $index;
        $this->druhSkoleniIndex = $indexDruh;
        $this->pocatekSkoleniIndex = $indexPocatek;
        $this->osobaIDIndex = $osobaIDIndex;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $params = $this->database->createParameters();

        $sql = 
            "SELECT RC, GenerovaneSkoleni.Povinne FROM GenerovaneSkoleni " .
            "WHERE GenerovaneSkoleni.RC = " . $params->addParameter($rowData[$this->osobaIDIndex]) .
            "  AND GenerovaneSkoleni.DruhSkoleni = " . $params->addParameter($rowData[$this->druhSkoleniIndex]) .
            //"  AND GenerovaneSkoleni.DuvodSkoleni = 0 " .
            "  AND ((GenerovaneSkoleni.CisloSkoleni = '' OR GenerovaneSkoleni.CisloSkoleni IS NULL) " . 
            "      OR GenerovaneSkoleni.CisloSkoleni = " . $params->addParameter($rowData[$this->skoleniIndex]) . ")";
    
        $cursor = $this->database->select($sql, $params);
        
        $row = $cursor->fetch();

        $cursor->close();

        if ($row[1]) {

            $params = $this->database->createParameters();

            $sql = 
                "SELECT RC FROM PozadovanaSkoleni JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PozadovanaSkoleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PozadovanaSkoleni.KodRozdeleni) " .
                "WHERE PozadovanaSkoleni.DruhSkoleni = " . $params->addParameter($rowData[$this->druhSkoleniIndex]) .
                "  AND PozadovanaSkoleni.Povinne = 0 ".
                "  AND RozdeleniOsob.RC = " . $params->addParameter($rowData[$this->osobaIDIndex]) .
                "  AND RozdeleniOsob.PlatnostOd <= " . $params->addParameter($this->database->timestampToParam($this->database->stringToTimestamp($rowData[$this->pocatekSkoleniIndex]))) .
                "  AND (RozdeleniOsob.PlatnostDo >= " . $params->addParameter($this->database->timestampToParam($this->database->stringToTimestamp($rowData[$this->pocatekSkoleniIndex]))) . " OR RozdeleniOsob.PlatnostDo IS NULL)";
            
            $cursor = $this->database->select($sql, $params);
        
            $row = $cursor->fetch();

            $cursor->close();

            if ($row) {
               $result = ronText('Skoleni_povinne') . ', ' . ronText('Skoleni_doporucene');
            }
            else {
                $result = ronText('Skoleni_povinne');
            }
        } 
        else {
            $result = ronText('Skoleni_doporucene');
        }
            
        return $result;
    }
}

/**
 * Class AnoNeBrowserField
 * @package Ron\RonPortal
 */
class AnoNeBrowserField extends BrowserField
{
    private $rowIndex;

    /**
     * AnoNeBrowserField constructor.
     * @param string $header
     * @param int $rowIndex
     */
    public function __construct($header, $rowIndex) {
        parent::__construct();

        $this->align = 'center';
        $this->header = $header;
        
        $this->rowIndex = $rowIndex;
    }
    
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return $rowData[$this->rowIndex] == 1 ? ronText('Str_Yes') : ronText('Str_No');
    }
}


/**
 * Class SkoleniElearningField
 * @package Ron\RonPortal
 */
class SkoleniElearningField extends BrowserField
{
    private $elearningIndex;
    private $schvaleniPrihlaskyIndex;

    /**
     * AnoNeBrowserField constructor.
     * @param Database $database
     * @param array $options
     * @param int $elearningIndex
     * @param int|null $schvaleniPrihlaskyIndex
     */
    public function __construct($database, $options, $elearningIndex, $schvaleniPrihlaskyIndex = null) {
        parent::__construct($database, $options);

        $this->elearningIndex = $elearningIndex;
        $this->schvaleniPrihlaskyIndex = $schvaleniPrihlaskyIndex;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!is_null($this->schvaleniPrihlaskyIndex) && ($this->schvaleniPrihlaskyIndex == 2))
            $value = false;

        //pro bezne kurzy zobrazi kapacit a pocet prihlasenych osob, u e-learningoveho se nezobrazuje
        return $rowData[$this->elearningIndex] == 0 ?
            ($this ->getLink(($this ->htmlConvert ||  $this ->format) ? DBUtils::getFormatedValue($this ->database, $this ->format, $value) : $value, $recordId))
            : '';
    }
}


/**
 * Class SkoleniPrihlasenBrowserField
 * @package Ron\RonPortal
 */
class SkoleniPrihlasenBrowserField extends BrowserField
{
    private $rowIndex;
    private $stavPrihlaskyIndex;

    /**
     * AnoNeBrowserField constructor.
     * @param string $header
     * @param int $rowIndex
     * @param int $stavPrihlaskyIndex
     */
    public function __construct($header, $rowIndex, $stavPrihlaskyIndex) {
        parent::__construct();

        $this->align = 'center';
        $this->header = $header;

        $this->rowIndex = $rowIndex;
        $this->stavPrihlaskyIndex = $stavPrihlaskyIndex;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $stav = '';
        if ($rowData[$this->stavPrihlaskyIndex] != '')
            $stav = " ({$rowData[$this->stavPrihlaskyIndex]})";

        return $rowData[$this->rowIndex] >= 1 ? (ronText('Str_Yes') . $stav) : ronText('Str_No');
    }
}



/**
 * Class SkoleniPrihlaseniBrowser
 * @package Ron\RonPortal
 */
class SkoleniPrihlaseniBrowser extends RonPortalBrowser
{
    public $hromadnePrihlaseni;


    /**
     * SkoleniPrihlaseniBrowser constructor.
     * @param $database
     * @param null | int $hromadnePrihlaseni
     * @throws Exception
     */
    public function __construct($database, $hromadnePrihlaseni)
    {
        parent::__construct($database, 'skoleni&action=zobrazprihlaseni');

        $this->code = 'browserprihlaseniskoleni';
        $this ->maxPageRows = 100;
        $this->hromadnePrihlaseni = $hromadnePrihlaseni;

        $this->sqlSelectPart =
            "Skoleni.Kredity, Skoleni.CisloSkoleni, Skoleni.Popis, Skoleni.DruhSkoleni, Skoleni.DatumPocatkuSkoleni, Skoleni.CasPocatkuSkoleni, Skoleni.CasUkonceniSkoleni, " . // 0 - 6
            "Skoleni.DatumUkonceniSkoleni, Skoleni.CasUkonceniSkoleni, Skoleni.CenaZaSkoleni,  " . // 7 - 9
            "Skoleni.Kapacita, (SELECT count(rc) FROM GenerovaneSkoleni WHERE CisloSkoleni = Skoleni.CisloSkoleni) as PocetPrihlasenych, " . // 10, 11
            "(SELECT COUNT(RC) FROM GenerovaneSkoleni WHERE GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni AND GenerovaneSkoleni.RC = SkoleniOsoby.RC AND GenerovaneSkoleni.DatumVznikuPovinnosti = SkoleniOsoby.DatumPovinnosti), ". //12
            "Skoleni.MistoSkoleni, SkoleniOsoby.Elearning, {fn_CASTWIDEMEMO( Skoleni.ObsahSkoleni)}, {fn_CASTWIDEMEMO(Skoleni.hodnoceniSkoleni)}, " . //13, 16
            'SkoleniOsoby.RC, SkoleniOsoby.Prijmeni, SkoleniOsoby.Jmeno, ' . //17 - 19
            '(SELECT MAX(ID) FROM PracovniPomer WHERE PracovniPomer.RC = SkoleniOsoby.RC AND PracovniPomer.DatumNastupu <= ' . $this->database->timestampToString(time()) .
                ' AND (PracovniPomer.DatumUkonceni >= ' . $this->database->timestampToString(time()) . ' OR PracovniPomer.DatumUkonceni IS NULL)), ' . //20
            "(SELECT COUNT(ID) FROM GenerovaneSkoleni JOIN EKurzOsoby ON ((GenerovaneSkoleni.RC = EKurzOsoby.RC_Skoleni) AND (GenerovaneSkoleni.DruhSkoleni = EKurzOsoby.DruhSkoleni) 
                    AND (GenerovaneSkoleni.DatumVznikuPovinnosti = EKurzOsoby.DatumVznikuPovinnosti)) WHERE GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni AND GenerovaneSkoleni.RC = SkoleniOsoby.RC AND EKurzOsoby.Stav IN (3, 4)), " . //21
            "(SELECT MIN(Potvrzeni) FROM GenerovaneSkoleni WHERE GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni AND GenerovaneSkoleni.RC = SkoleniOsoby.RC AND GenerovaneSkoleni.DatumVznikuPovinnosti = SkoleniOsoby.DatumPovinnosti), " .  //22
            "SkoleniOsoby.DatumPovinnosti, Skoleni.PozadovatSchvaleniPrihl, SkoleniOsoby.SchvaleniPrihlasky, S_StavSchvalPrihlNaKurz.Popis, " . //23 - 26
            "(SELECT COUNT(RC) FROM GenerovaneSkoleni WHERE GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni AND GenerovaneSkoleni.RC = SkoleniOsoby.RC AND GenerovaneSkoleni.SchvaleniPrihlasky < 2), " . //27
            "(SELECT Umisteni.Popis FROM Umisteni WHERE Umisteni.Kod = Skoleni.MistoSkoleni)";  //28
          //cast(hodnoceniSkoleni as char(300))+(select ' *** Adresa : '+coalesce(umisteni.popis,'') + ', '+coalesce(umisteni.ulice,'') + ', '+coalesce(umisteni.mesto,'')+''+ coalesce(', patro : '+CAST(umisteni.patro as char(2)),'') from umisteni where umisteni.kod=Skoleni.MistoSkoleni)

        /*$this->sqlFromPart =
            'Skoleni JOIN DruhSkoleni ON (DruhSkoleni.Kod = Skoleni.DruhSkoleni) ' .
            'LEFT OUTER JOIN GenerovaneSkoleni ON (GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni) ' .
            'LEFT OUTER JOIN Osoba ON (GenerovaneSkoleni.RC = Osoba.RC) ';
        */

        $this->sqlFromPart =
            '(SELECT DruhSkoleni.Elearning, Osoba.RC, Osoba.Prijmeni, Osoba.Jmeno, DruhSkoleni.Kod AS DruhSkoleni, DruhSkoleni.Kategorie AS KategorieSkoleni, GenerovaneSkoleni.DatumVznikuPovinnosti AS DatumPovinnosti, ' .
            ' GenerovaneSkoleni.CisloSkoleni, GenerovaneSkoleni.SchvaleniPrihlasky ' .
            'FROM DruhSkoleni ' .
            'JOIN GenerovaneSkoleni ON (GenerovaneSkoleni.DruhSkoleni = DruhSkoleni.Kod) ' .
            'JOIN Osoba ON (GenerovaneSkoleni.RC = Osoba.RC) ' .
//            "WHERE ((GenerovaneSkoleni.Potvrzeni = 1 " .
//	        "   AND (DruhSkoleni.Elearning = 1 OR GenerovaneSkoleni.DatumSkoleni IS NULL)) OR GenerovaneSkoleni.SchvaleniPrihlasky = 0)" .
            "WHERE (GenerovaneSkoleni.Potvrzeni = 1 " .
	        "    OR  GenerovaneSkoleni.SchvaleniPrihlasky IN (0,2))" .
            'UNION  ' .
            'SELECT DruhSkoleni.Elearning, Osoba.RC, Osoba.Prijmeni, Osoba.Jmeno, DruhSkoleni.Kod AS DruhSkoleni, DruhSkoleni.Kategorie AS KategorieSkoleni, NULL AS DatumPovinnosti, NULL AS CisloSkoleni, NULL AS SchvaleniPrihlasky ' .
            'FROM DruhSkoleni ' .
            "JOIN PozadovanaSkoleni ON PozadovanaSkoleni.DruhSkoleni = DruhSkoleni.Kod " .
            "JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PozadovanaSkoleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PozadovanaSkoleni.KodRozdeleni) " .
            'JOIN Osoba ON (RozdeleniOsob.RC = Osoba.RC) ' .
            "       WHERE PozadovanaSkoleni.Povinne = 0 ".
            "         AND ((RozdeleniOsob.PlatnostOd <= " . $this->database->timestampToString(time()) .
            "               AND (RozdeleniOsob.PlatnostDo >= " . $this->database->timestampToString(time()) . " OR RozdeleniOsob.PlatnostDo IS NULL))  " .
            "           OR (DruhSkoleni.Elearning = 1)) " .
            "         AND NOT EXISTS(SELECT RC FROM GenerovaneSkoleni GS WHERE GS.RC = Osoba.RC AND GS.DruhSkoleni = DruhSkoleni.Kod AND GS.Potvrzeni = 1 AND GS.DatumSkoleni IS NULL)" .
            ") AS SkoleniOsoby ".
            "JOIN Skoleni ON Skoleni.DruhSkoleni = SkoleniOsoby.DruhSkoleni " .
            "LEFT OUTER JOIN S_StavSchvalPrihlNaKurz ON (S_StavSchvalPrihlNaKurz.Kod = SkoleniOsoby.SchvaleniPrihlasky)";

        $this->sqlWherePart =
            "  ((SkoleniOsoby.CisloSkoleni = '' OR SkoleniOsoby.CisloSkoleni IS NULL) OR SkoleniOsoby.CisloSkoleni = Skoleni.CisloSkoleni) " .

            //u doporucenych nezobrazi, pokud jiz existuje prihlaska, ktera nebyla schvalena, nebo existuje navrh bez prirazeneho skoleni
            // (29.8.2019 zrusena podminka na prihlasene skoleni, pri odhlaseni se skoleni rusi: AND GS.CisloSkoleni = Skoleni.CisloSkoleni)
            'AND ((DatumPovinnosti IS NULL AND NOT EXISTS(SELECT RC FROM GenerovaneSkoleni GS WHERE GS.RC = SkoleniOsoby.RC AND GS.DruhSkoleni = SkoleniOsoby.DruhSkoleni AND (GS.CisloSkoleni = Skoleni.CisloSkoleni OR GS.CisloSkoleni IS NULL)
                AND (GS.Potvrzeni = 1 OR (GS.Potvrzeni = 0 AND GS.SchvaleniPrihlasky <> 2))) ) ' .
//	        '   OR DatumPovinnosti IS NOT NULL)' .
            '   OR (DatumPovinnosti IS NOT NULL AND (SkoleniOsoby.SchvaleniPrihlasky = 2 OR NOT EXISTS(SELECT RC FROM GenerovaneSkoleni GS WHERE GS.RC = SkoleniOsoby.RC AND GS.DruhSkoleni = SkoleniOsoby.DruhSkoleni AND GS.CisloSkoleni = Skoleni.CisloSkoleni
                AND GS.SchvaleniPrihlasky < 2 AND GS.DatumVznikuPovinnosti <> SkoleniOsoby.DatumPovinnosti)))) ';

        //2.10.2019 prozatim bez filtru na konkretni skoleni, neni osestrene prihlasovani/odhlasovani
//        if (isset($_GET['cisloskoleni']))
//            $this->sqlWherePart .= " AND (Skoleni.CisloSkoleni = '" . $this->database->addSlashes($_GET['cisloskoleni']) . "')";
//        else {
            $this->sqlWherePart .=
                ' AND Skoleni.StavSkoleni IN (0, 1) ' . //skoleni ve stavu probiha, pripravuje se - stejne jako v aplikaci

                //8.8.2019 doplnena podminka na ukonceni skoleni, lze se prihlasit jen na neukoncene (plati i pro e-learning)
                ' AND (Skoleni.DatumUkonceniSkoleni >= ' . $this->database->timestampToString(time()) . ' OR Skoleni.DatumUkonceniSkoleni IS NULL)' .

                //19.7.2016 RON-PR - narozdil od aplikace je mozne se prihlasit jen na kurz, ktery zacina v budoucnosti a ma datum potvrzeni v budoucnu
                ' AND ( Skoleni.DatumPocatkuSkoleni >= ' . $this->database->timestampToString(time()) .
                ' AND   (Skoleni.DatumPotvrzeniUcasti >= ' . $this->database->timestampToString(time()) . ' OR Skoleni.DatumPotvrzeniUcasti IS NULL)' .

                //19.7.2016 RON-PR na e-learningovy je mozne prihlasit vzdy
                '      OR SkoleniOsoby.Elearning = 1' .
                '      )';
//        }

        //v get SQL filter se jeste filtruje na skoleni, ktera jsou pro osobu povinna, nebo doporucena a pokud bylo skoleni vytvorene z planovaneho a prihlaseni je jen pro zajemce o plan, zobrazi se jen potvrzenym zajemcum

        $this->filterForm = new PrihlaseniSkoleniFilterForm($this ->database, $this ->action, $hromadnePrihlaseni);
        $this->filterForm->loadData();

        $this->defaultSort = 1;
        $this->primaryField = 0;

        $this->cssClass .= ' prihlaseniSkoleniBrowser';

        if (RonPortalSession::vsechnyOsoby()) {
            $this->addFields([
                [BROWSER_FIELD_SQL_NAME => 'SkoleniOsoby.Prijmeni', BROWSER_FIELD_HEADER => ronText('Prijmeni'),
                BROWSER_FIELD_FORMAT => '', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_WRAP => 1, BROWSER_FIELD_SQL_INDEX => 18, BROWSER_FIELD_CLASS_NAME => 'prihlaseniskoleni_prijmeni'],
                [BROWSER_FIELD_SQL_NAME => 'SkoleniOsoby.Jmeno', BROWSER_FIELD_HEADER => ronText('Jmeno'),
                    BROWSER_FIELD_FORMAT => '', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_WRAP => 1, BROWSER_FIELD_SQL_INDEX => 19, BROWSER_FIELD_CLASS_NAME => 'prihlaseniskoleni_jmeno']
            ]);
        }



        $this->showFooter = $this->hromadnePrihlaseni;

        $this->addField([BROWSER_FIELD_SQL_NAME => 'Skoleni.CisloSkoleni', BROWSER_FIELD_HEADER => ronText('Skoleni_CisloSkoleni'), BROWSER_FIELD_ALIGN => 'right',
            BROWSER_FIELD_FORMAT => '', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_WRAP => 1, BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_CLASS_NAME => 'prihlaseniskoleni_cisloskoleni']);

        $f = $this->addField([BROWSER_FIELD_SQL_NAME => 'Skoleni.Popis', BROWSER_FIELD_HEADER => ronText('Skoleni_Popis'), BROWSER_FIELD_ALIGN => '',
            BROWSER_FIELD_FORMAT => '', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_WRAP => 1, BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_CLASS_NAME => 'prihlaseniskoleni_popis font-weight-bold']);

        $f ->footer = ronText('Skoleni_prihlasit_hromadne');

        $this->addFields([new SkoleniElearningField($this->database, [BROWSER_FIELD_SQL_NAME => 'Skoleni.DatumPocatkuSkoleni', BROWSER_FIELD_HEADER => ronText('Skoleni_DatumZacatku'),
            BROWSER_FIELD_FORMAT => 'finedate', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_WRAP => 1, BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_CLASS_NAME => 'prihlaseniskoleni_datumzacatku'], 14),
            [BROWSER_FIELD_SQL_NAME => 'Skoleni.DatumUkonceniSkoleni', BROWSER_FIELD_HEADER => ronText('Skoleni_DatumUkonceni'),
            BROWSER_FIELD_FORMAT => 'finedate', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_WRAP => 1, BROWSER_FIELD_SQL_INDEX => 7, BROWSER_FIELD_CLASS_NAME => 'prihlaseniskoleni_datumukonceni'],
        ]);

        if (!$this->hromadnePrihlaseni){
            $this->addFields([
                [BROWSER_FIELD_SQL_NAME => 'Skoleni.CasPocatkuSkoleni', BROWSER_FIELD_HEADER => ronText('Skoleni_CasZacatku'),
                BROWSER_FIELD_FORMAT => 'time', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_WRAP => 1, BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_CLASS_NAME => 'prihlaseniskoleni_caspocatku'],

                [BROWSER_FIELD_SQL_NAME => 'Skoleni.CasUkonceniSkoleni', BROWSER_FIELD_HEADER => ronText('Skoleni_CasUkonceni'),
                BROWSER_FIELD_FORMAT => 'time', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_WRAP => 1, BROWSER_FIELD_SQL_INDEX => 6, BROWSER_FIELD_CLASS_NAME => 'prihlaseniskoleni_casukonceni'],

                [BROWSER_FIELD_SQL_NAME => 'Skoleni.Kredity', BROWSER_FIELD_HEADER => ronText('Skoleni_Kredity'), BROWSER_FIELD_CLASS_NAME => 'prihlaseniskoleni_kredity',
                BROWSER_FIELD_FORMAT => '', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_WRAP => 1, BROWSER_FIELD_SQL_INDEX => 0]]);
        }

        $this->addField(new PovinneSkoleniBrowserField($this->database, ronText('Skoleni_Povinnost'),1,3,4, 17));

        $this->addFields([
            new SkoleniElearningField($this->database, [BROWSER_FIELD_SQL_NAME => 'Skoleni.Kapacita', BROWSER_FIELD_HEADER => ronText('Skoleni_kapacita'),
            BROWSER_FIELD_FORMAT => '', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_WRAP => 1, BROWSER_FIELD_SQL_INDEX => 10, BROWSER_FIELD_CLASS_NAME => 'prihlaseniskoleni_kapacita center'], 14),
            new SkoleniElearningField($this->database, [BROWSER_FIELD_SQL_NAME => 'PocetPrihlasenych', BROWSER_FIELD_HEADER => ronText('Skoleni_PocetPrihlasenych'),
            BROWSER_FIELD_FORMAT => '', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_WRAP => 1, BROWSER_FIELD_SQL_INDEX => 11, BROWSER_FIELD_CLASS_NAME => 'prihlaseniskoleni_pocetprihlasenych center'], 14, 25)
        ]);

        if (!$this->hromadnePrihlaseni) {
            $this->addField(new SkoleniPrihlasenBrowserField(ronText('Skoleni_Prihlasen_stav'), 12, 26));
            $this->addField(new AnoNeBrowserField(ronText('Skoleni_elearning'), 14));
            $this->addField(new AnoNeBrowserField(ronText('Skoleni_vyzadovat_schvaleni'), 24));
            $this->addField(new PrihlaseniSkoleniBrowserField($this->database, 10, 11, 1, 14, 17, 20, 21,
                23, 24, 25, 27, 12));
        } else {
            $this->addField(new PrihlaseniSkoleniBrowserField($this->database, 10, 11, 1, 14, 17, 20, 21,
                23, 24, 25, 27, 12, $this->hromadnePrihlaseni));
        }
    }
    
    function load($sqlFilter = '')
    {
        $this ->orderFieldIndex = $this ->getOrderFieldIndex();
        $sqlText = $this ->getSQL($sqlFilter);

        $this ->loadSQL($sqlText);
    }

    public function getSQLFilter()
    {
        $pravaPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        $pravaPotvrditPrihlaseni = Prava::vratPrava('skoleni_schvalit_prihlaseni');

        $filtr =
            ' EXISTS(SELECT RC FROM PracovniPomer WHERE PracovniPomer.RC = SkoleniOsoby.RC AND PracovniPomer.DatumNastupu <= ' . $this->database->dateToString(DateUtils::today()) .
            '       AND (PracovniPomer.DatumUkonceni >= ' . $this->database->dateToString(DateUtils::today()) . ' OR PracovniPomer.DatumUkonceni IS NULL) ' .

            ' AND (' . $pravaPrihlasit->sqlFiltr('PracovniPomer', DateUtils::today(), DateUtils::today()) .
            '       OR ' . $pravaPotvrditPrihlaseni->sqlFiltr('PracovniPomer', DateUtils::today(), DateUtils::today()) .
            '   )' .
            ')';



        /*$filtr =
            "(" . //doporucene skoleni
            "EXISTS(SELECT * FROM PozadovanaSkoleni JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PozadovanaSkoleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PozadovanaSkoleni.KodRozdeleni) " .
            "       WHERE Skoleni.DruhSkoleni=PozadovanaSkoleni.DruhSkoleni ".
            "         AND PozadovanaSkoleni.Povinne = 0 ".
            "         AND RozdeleniOsob.RC = Osoba.RC " .
            "         AND ((RozdeleniOsob.PlatnostOd <= Skoleni.DatumPocatkuSkoleni " .
            "               AND (RozdeleniOsob.PlatnostDo >= Skoleni.DatumPocatkuSkoleni OR RozdeleniOsob.PlatnostDo IS NULL))  " .
            "           OR (DruhSkoleni.Elearning = 1))" .
            " )" .
            " OR " . //vygenerovane skoleni ve stavu potrvrzeny navrh
            "EXISTS(SELECT * FROM GenerovaneSkoleni " .
            "       WHERE GenerovaneSkoleni.RC =  Osoba.RC " .
            "         AND Skoleni.DruhSkoleni=GenerovaneSkoleni.DruhSkoleni " .
            //"         AND GenerovaneSkoleni.DuvodSkoleni = 0 " .
            "         AND GenerovaneSkoleni.Potvrzeni = 1" .
            "         AND ((GenerovaneSkoleni.CisloSkoleni = '' OR GenerovaneSkoleni.CisloSkoleni IS NULL) OR GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni)) " .
            ")";*/

        //pokud bylo skoleni vytvorene z planovaneho a prihlaseni je jen pro zajemce o plan, zobrazi se jen potvrzenym zajemcum
        $filtr .=
            " AND (Skoleni.UcastniciDlePlanSkoleni = 0 " .
            "        OR EXISTS(SELECT RC FROM PlanovaneSkoleniOsob " .
            "                  WHERE PlanovaneSkoleniOsob.CisloSkoleni = Skoleni.CisloPlanovanehoSkoleni " .
            "                    AND PlanovaneSkoleniOsob.Potvrzeni = 1 " .
            "                    AND PlanovaneSkoleniOsob.RC = SkoleniOsoby.RC ))";

        if (!RonPortalSession::vsechnyOsoby()) {
            $filtr .= " AND SkoleniOsoby.RC = '" . $this->database->addSlashes(RonPortalSession::osobaRC()) . "'";
        }

        if (RonPortalSession::rozdeleni() != '') {
            if (RonPortalSession::rozdeleniVcetnePodrizenych()) {
                $filtr .= ' AND (EXISTS (SELECT * FROM RozdeleniOsob, HierarchieRozdeleni, PracovniPomer ' .
                    '   WHERE RozdeleniOsob.RC = SkoleniOsoby.RC ' .
                    '   AND PracovniPomer.RC = RozdeleniOsob.RC ' .
                    '   AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo ' .
                    '   AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu ' .
                    '   AND PracovniPomer.DatumNastupu  <= ' . $this->database->dateToString(DateUtils::today()) .
                    '   AND (PracovniPomer.DatumUkonceni >= ' . $this->database->dateToString(DateUtils::today()) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
                    '   AND HierarchieRozdeleni.TypNadrazene = ' . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                    "   AND HierarchieRozdeleni.KodNadrazene = '" . $this->database->addSlashes(RonPortalSession::rozdeleni()) . "'" .
                    '   AND HierarchieRozdeleni.TypPodrizene = RozdeleniOsob.KodTypuRozdeleni ' .
                    '   AND HierarchieRozdeleni.KodPodrizene = RozdeleniOsob.Rozdeleni ' .
                    '   AND RozdeleniOsob.PlatnostOd <= ' . $this->database->dateToString(DateUtils::today()) .
                    '   AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->dateToString(DateUtils::today()) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                    '))';
            } else {
                $filtr .= ' AND EXISTS(SELECT * FROM RozdeleniOsob JOIN PracovniPomer ON (PracovniPomer.RC = RozdeleniOsob.RC AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu)' .
                    '   WHERE RozdeleniOsob.KodTypuRozdeleni = ' . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                    "     AND RozdeleniOsob.Rozdeleni = '" . $this->database->addSlashes(RonPortalSession::rozdeleni()) . "'" .
                    '     AND RozdeleniOsob.RC = SkoleniOsoby.RC ' .
                    '     AND PracovniPomer.DatumNastupu  <= ' . $this->database->dateToString(DateUtils::today()) .
                    '     AND (PracovniPomer.DatumUkonceni >= ' . $this->database->dateToString(DateUtils::today()) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
                    '     AND RozdeleniOsob.PlatnostOd <= ' . $this->database->dateToString(DateUtils::today()) .
                    '     AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->dateToString(DateUtils::today()) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';
            }
        }

        if ($_SESSION['jenTypSkoleni']) {
            $filtr .= " AND Skoleni.DruhSkoleni =  '" . $this->database->addSlashes($_SESSION['typSkoleni']) . "'";
        }


        if (RonPortalSession::getFiltrovatDatum()) {
            $filtr .= ' AND Skoleni.DatumPocatkuSkoleni >= ' . $this->database->dateToString(RonPortalSession::getDatumOd()) .
                ' AND Skoleni.DatumPocatkuSkoleni <= ' . $this->database->dateToString(RonPortalSession::getDatumDo());
        }


        
        if ($_SESSION['jenUmisteni']) {
            $filtr .= " AND Skoleni.MistoSkoleni =  '" . $this->database->addSlashes($_SESSION['umisteni']) . "'";
        }
        
        if ($_SESSION['jenPovinne']) {
            switch ($_SESSION['typPovinnosti']) {
                case 1: //povinne
                    $filtr .= 
                        " AND EXISTS(SELECT * FROM GenerovaneSkoleni " .
                        "        WHERE GenerovaneSkoleni.RC =  SkoleniOsoby.RC" .
                        "          AND Skoleni.DruhSkoleni=GenerovaneSkoleni.DruhSkoleni " .
                        //"          AND GenerovaneSkoleni.DuvodSkoleni = 0 " .
                        "          AND GenerovaneSkoleni.Povinne = 1" .
                        "          AND GenerovaneSkoleni.Potvrzeni = 1" .
                        "          AND ((GenerovaneSkoleni.CisloSkoleni = '' OR GenerovaneSkoleni.CisloSkoleni IS NULL) OR GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni)) ";
                    break;
                case 2: //doporucene
                    $filtr .= 
                        " AND ( " .
                        "   EXISTS(SELECT * FROM PozadovanaSkoleni JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PozadovanaSkoleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PozadovanaSkoleni.KodRozdeleni) " .
                        "       WHERE Skoleni.DruhSkoleni=PozadovanaSkoleni.DruhSkoleni ".
                        "         AND PozadovanaSkoleni.Povinne = 0 ".
                        "         AND RozdeleniOsob.RC =  SkoleniOsoby.RC " .
                        "         AND RozdeleniOsob.PlatnostOd <= Skoleni.DatumPocatkuSkoleni " .
                        "         AND (RozdeleniOsob.PlatnostDo >= Skoleni.DatumPocatkuSkoleni OR RozdeleniOsob.PlatnostDo IS NULL))  " .
                        "   OR EXISTS(SELECT * FROM GenerovaneSkoleni " .
                        "        WHERE GenerovaneSkoleni.RC =  SkoleniOsoby.RC" .
                        "          AND Skoleni.DruhSkoleni=GenerovaneSkoleni.DruhSkoleni " .
                        "          AND (GenerovaneSkoleni.Povinne = 0 OR GenerovaneSkoleni.Povinne IS NULL)" .
                        "          AND (GenerovaneSkoleni.Potvrzeni = 1)" .
                        "          AND ((GenerovaneSkoleni.CisloSkoleni = '' OR GenerovaneSkoleni.CisloSkoleni IS NULL) OR GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni)) ".
                        ")";
                    break;
            }
        }


        if ($_SESSION['specifikaceSkoleni']) {
            $filtr .= " AND Skoleni.Specifikace = '" . $this->database->addSlashes($_SESSION['specifikaceSkoleni']) . "'";
        }


        if ($_SESSION['kategorieSkoleni']) {
            $filtr .= " AND SkoleniOsoby.KategorieSkoleni = '" . $this->database->addSlashes($_SESSION['kategorieSkoleni']) . "'";
        }

        if ($_SESSION['jenPopisSkoleni']) {
            $filtr .= " AND Skoleni.Popis = '" . $this->database->addSlashes($_SESSION['popisSkoleni']) . "'";
        }

        return $filtr;
    }

    /**
     * @param $values
     * @param $className
     * @param bool $verejne
     * @throws Exception
     */
    private function zobrazDokumentySkoleni($values, $className, $verejne = false)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT Dokument.ID, Dokument.popis, Dokument.jmenoSouboru " . //0..2
            "FROM DokumentSkoleni JOIN Dokument ON (DokumentSkoleni.id = Dokument.id) " .
            " JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ) " .
            "WHERE DokumentSkoleni.cisloSkoleni = " . $params->addParameter($values[1]) .
            "  AND (Dokument.zobrazitNaIntrawebu = 1 OR TypDokumentu.zobrazitNaIntrawebu = 1)";

        if ($verejne)
            $sql .= " AND TypDokumentu.Verejny = 1";
        
        $cursor = $this->database->select($sql, $params);
        
        while ($row = $cursor->fetch()) {
            echo "<tr class='$className'><td></td><td colspan='" . (int)(count($this->fields) - 1) . "'>";

            $linkStr = "?menuid=skoleni&action=download&cisloskoleni=$values[1]&dokument=$row[0]";
            
            if ($linkStr) echo "<a target='_blank' href='$linkStr'>";
            echo Ron\StringUtils::htmlTextConvert($row[1]) . ' - ' . Ron\StringUtils::htmlTextConvert($row[2]);
            if ($linkStr) echo "</a>";

            echo "</td></tr>\n";
        }
    }
    
    /**
     * @param $values
     * @param $className
     * @throws Exception
     */
    private function zobrazUmisteni($values, $className)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT Umisteni.Popis, Umisteni.Ulice, Umisteni.Mesto, Umisteni.Patro " .
            "FROM Umisteni WHERE Umisteni.Kod = " . $params->addParameter($values[13]);
        
        $cursor = $this->database->select($sql, $params);
        
        if ($row = $cursor->fetch()) {
            echo "<tr class='$className'><td></td><td colspan='" . (int)(count($this->fields) - 1) . "'>";
            
            echo ronText('Skoleni_Adresa') . ': ' . $row[0] .
                ($row[1] ? ', ' . $row[1] : '') .
                ($row[2] ? ', ' . $row[2] : '') .
                ($row[3] ? ', ' . ronText('Skoleni_Patro') . ': ' . $row[3] : '');
            
            echo "</td></tr>\n";
        }
    }

    /**
     * @param $values
     * @param $className
     * @throws Exception
     */
    private function zobrazElearning($values, $className)
    {
        /** @noinspection PhpDeprecationInspection ponecháno pro kompatibilitu s ODBC driverem (generuje chybu pro parametry v LEFT JOIN nebo v EXISTS) */
        $sql =
            'SELECT EKurz.ID, EKurz.Nazev, EKurz.PlatnostOd, EKurz.PlatnostDo, ' . // 0..3
            '  EKurzOsoby.Id, EKurzOsoby.Stav, EKurzOsoby.DatumCasZahajeni, EKurzOsoby.PosledniPristup, ' . // 4..7
            '  RevizeEKurzu.SestavaProCertifikat, RevizeEKurzu.UlozitCertifikatDoDok, SestavyProIntraweb.Nazev, ' . // 8..10
            '  GenerovaneSkoleni.DatumVznikuPovinnosti' . //11
            ' FROM (EKurzDruhuSkoleni INNER JOIN EKurz  ON EKurzDruhuSkoleni.IDKurzu = EKurz.ID ' .
            '   INNER JOIN Skoleni ON Skoleni.DruhSkoleni = EKurzDruhuSkoleni.KodDruhuSkoleni' .
            '   INNER JOIN GenerovaneSkoleni ON GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni )' .
            ' LEFT OUTER JOIN (EKurzOsoby INNER JOIN RevizeEKurzu ON EKurzOsoby.IdRevize = RevizeEKurzu.Id)' .
            '   ON EKurzOsoby.RC_Skoleni = GenerovaneSkoleni.RC AND EKurzOsoby.DruhSkoleni = GenerovaneSkoleni.DruhSkoleni AND EKurzOsoby.DatumVznikuPovinnosti = GenerovaneSkoleni.DatumVznikuPovinnosti AND RevizeEKurzu.IDKurzu = EKurz.ID ' .
            ' LEFT OUTER JOIN SestavyProIntraweb ON RevizeEKurzu.SestavaProCertifikat = SestavyProIntraweb.Kod' .
            " WHERE EKurzDruhuSkoleni.KodDruhuSkoleni = '" . $this ->database ->addSlashes($values[3]) . "'" .
            '   AND EKurz.Stav = 1' . // jen aktivni
            "   AND Skoleni.CisloSkoleni = '" . $this->database->addSlashes($values[1]) . "'" .
            "   AND GenerovaneSkoleni.RC = '" . $this->database->addSlashes($values[17]) . "'" .
            "   AND GenerovaneSkoleni.SchvaleniPrihlasky <> 2" .
            ' ORDER BY EKurzDruhuSkoleni.Poradi, EKurzOsoby.DatumCasZahajeni DESC';

        try {
            $cursor = $this->database->getSqlResult($sql);
            while ($row = $cursor->fetch()) {
                //pokud je u kurzu vyzadovano schvaleni a neni aktualne schvalena prihlaska, zobrazi jen absolvovane kurzy
                if (($values[24] == 0 ) || (($values[24] == 1) && (((($values[25] == -1) || ($values[25] == 1))) || (($row[5] == 3) || ($row[5] == 4))))) {
                    echo "<tr class='$className'><td></td><td colspan='" . (int)(count($this->fields) - 1) . "'>";

                    if ($row[3] && $this->database->stringToTimestamp($row[3]) < DateUtils::startOfTheDay(time())) {
                        echo ronText('Platnost_kurzu_skoncila');
                    } else {
                        if ($row[4]) {
                            // existuje zaznam EKurzOsoby
                            $linkStr = '?menuid=prezentace&kurzosoby=' . $row[4];
                        } else {
                            $linkStr =
                                '?menuid=prezentace&zahajitkurz=' . $row[0] .
                                '&druhskoleni=' . StringUtils::htmlTextConvert($values[3]) .
                                '&datumvznikupovinnosti=' . $this->database->stringToTimestamp($row[11]); // date('j.n.Y', $this ->database ->stringToTimestamp($values[5]) ); - je tam i cas!
                        }

                        if ($linkStr) echo "<a target='_blank' href='$linkStr'>";
                        echo StringUtils::htmlTextConvert($row[1]);
                        if ($linkStr) echo "</a>";

                        switch ($row[5]) {
                            case 1 :
                                echo ' - <span class="ekurz_probiha">' . ronText('EKurz_probiha') . '</span>';
                                break;

                            case 3 :
                                echo ' - <span class="ekurz_absolvovan_uspesne">' . ronText('EKurz_absolvovan_uspesne') . '</span>';
                                if ($row[8]) {
                                    // download certifikatu
                                    // TODO: presunout do kontroleru skoleni?
                                    echo ", <a target='_blank' href='?menuid=prezentace&action=certifikat&id={$row[4]}'>" . StringUtils::htmlTextConvert($row[10]) . '</a>';
                                }
                                break;

                            case 4 :
                                echo ' - <span class="ekurz_absolvovan_neuspesne">' . ronText('EKurz_absolvovan_neuspesne') . '</span>';
                                break;
                        }
                    }

                    echo "</td></tr>\n";
                }
            }
        }
        catch (Exception $exception) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8("Chyba při načítání ekurzů ") . $exception->getMessage());
        }
    }

    /**
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @param bool $edit
     * @throws Exception
     */
    public function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if (!$this->hromadnePrihlaseni) {
            if ($values[13])
                $this->zobrazUmisteni($values, $className);

            if ($values[12])
                $this->zobrazDokumentySkoleni($values, $className);
            else
                $this->zobrazDokumentySkoleni($values, $className, true);

            //druh skoleni je e-learning
            if ($values[14]) {
                //u kurzu se vyzaduje schvaleni
                if (($values[24] == 1) && (($values[25] == -1) || ($values[25] == 1)))
                    $this->zobrazElearning($values, $className);
                else
                    $this->zobrazElearning($values, $className);
            }

            // zobrazi obsah kurzu pokud je povoleno v nastaveni
            if (trim($values[15]) && RonPortalSettings::getIntValue('skoleni_zobrazit_obsah')) {
                echo "<tr class='$className'>";
                echo '<td></td><td></td><td colspan="' . ($this->columnCount - 1) . '"><div class="browserpoznamka">' . ronText('Skoleni_obsah_kurzu') . ": &nbsp;" . $values[15] . '</div></td>';
                echo '</tr>';
            }

            // zobrazi poznamku kurzu pokud je povoleno v nastaveni
            if (trim($values[16]) && RonPortalSettings::getIntValue('skoleni_zobrazit_poznamku')) {
                echo "<tr class='$className'>";
                echo '<td></td><td></td><td colspan="' . ($this->columnCount - 1) . '"><div class="browserpoznamka">' . ronText('Skoleni_poznamka') . ": &nbsp;" . $values[16] . '</div></td>';
                echo '</tr>';
            }

        }

        $this ->renderSeparator();
    }


    public function renderTableFooter()
    {
        if(true){
            parent::renderTableFooter();
        }

        echo "<tr>";
        echo "<script>personalistika.odhlaseniSkoleni();</script>";

        //dialog Existuji terminy, smazat?
        echo "<div id='dialogSkoleniTerminy' style='display: none;' title='' >";

        echo "<div id='skoleniTerminy' class='dialogcontent'>" . ronText('Skoleni_SmazatTerminy') . "</div>";

        echo "<DIV class='buttons align-center'>";
        echo "<input type='submit' id='btnTerminySmazat' class='submitbutton' value='" . ronText('Str_Yes') . "' />";
        echo "<input type='button' id='btnTerminyNechat' class='button' value='" . ronText('Str_No') . "' />";
        echo "<input type='button' id='btnTerminyZavrit' class='button' style='display: none;' value='" . ronText('Zavrit') . "' />";
        echo "</DIV>";
        echo "</div>";

        //dialog Ponechat potvrzeny navrh
        echo "<div id='dialogSkoleniNavrh' style='display: none;' title='' >";

        echo "<div id='skoleniNavrh' class='dialogcontent'>" . ronText('Skoleni_NechatNavrh') . "</div>";

        echo "<DIV class='buttons align-center'>";
        echo "<input type='submit' id='btnPonechatNavrh' class='submitbutton' value='" . ronText('Str_Yes') . "' />";
        echo "<input type='button' id='btnSmazatNavrh' class='button' value='" . ronText('Str_No') . "' />";
        echo "<input type='button' id='btnNavrhZavrit' class='button' style='display: none;' value='" . ronText('Zavrit') . "' />";
        echo "</DIV>";
        echo "</div>";

        echo "</tr>";

    }

    function renderContent()
    {
        if ($this->hromadnePrihlaseni){
            echo "<form method='POST' name='hromadneSchvaleniSK' action='?menuid=skoleni&action=zobrazprihlaseni'>";
            echo "<input type='hidden' id='hromadnePrihlasitSkoleni' name='hromadnePrihlasitSkoleni' value='1'>";
        }

        parent::renderContent();

        if ($this->hromadnePrihlaseni){
            echo "<button type='button' class='button clickHref' data-ron-href='?menuid=skoleni&action=zobrazprihlaseni' name='zpet' >" . ronText('zpet') . "</button>";
            echo "<button type='submit' class='button' >" . ronText('Provest') . "</button>";

            echo "</form>";
        }
    }
}