<?php
/** Roman.Szablatura, 18.4.2016 */

namespace Ron\Personalistika;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DBRecord;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalForm;
use Ron\RonPortal\User;
use Ron\StringUtils;
use function Ron\ronText;

require_once ('adsapp/ronportalform.php');


class PotvrzeniPrecteniForm extends RonPortalForm
{

    protected $idDokumentu;
    protected $platnostDokumentuOd;
    protected $idVzkazu;

    protected $predmetVzkazu;
    protected $popisTypuDokumentu;

    /**
     * PotvrzeniPrecteniForm constructor.
     * @param Database $database
     * @param string $idDokumentu
     * @param int $platnostOd
     * @param int $idVzkazu
     * @throws DBException
     */
    function __construct($database, $idDokumentu, $platnostOd, $idVzkazu)
    {
        parent::__construct($database);

        $this->idDokumentu = $idDokumentu;
        $this->platnostDokumentuOd = $platnostOd;
        $this->idVzkazu = $idVzkazu;

        $this->nactiDetailyVzkazu();

        $this->title = ronText('potvrzeni_precteni');

        $this->htmlAction = '?menuid=dokumenty&potvrzeniprecteniform=1&';
        $this->stornoAction = '?menuid=dokumenty';

        $this->sqlTableName = 'DokumentVzkazu';
        $this->dbLogSrcDescription = StringUtils::fromUTF8('Potvrzeni přečtení dokumentu');
        $this->sqlSelectPart =
            'IdVzkazu, idDokumentu, PlatnostOd, Precteno, ' . $this->database->castWideMemo('IdentifikacePodpisu') . ', ' . $this->database->castWideMemo('Vyjadreni');   // 0 - 5

        //Možné hodnoty: VzkazId, VzkazPredmet, VzkazPopis, DokumentVzkazuidDokumentu, DokumentVzkazuPlatnostOd, TypDokumentuPopis, VzkazDatumCas

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_TEXT, FORM_FIELD_CAPTION_TEXT => 'Nazev',
            FORM_FIELD_TEXT => StringUtils::htmlTextConvert($this->predmetVzkazu)
        ]);

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_TEXT, FORM_FIELD_CAPTION_TEXT => 'popis_typu_dokumentu',
            FORM_FIELD_TEXT => StringUtils::htmlTextConvert($this->popisTypuDokumentu)
        ]);

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_TEXT, FORM_FIELD_CAPTION_TEXT => 'platnost_dokumentu_od',
            FORM_FIELD_TEXT => StringUtils::format('date', $this->platnostDokumentuOd)
        ]);

        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_SQL_NAME => 'Precteno',
            FORM_FIELD_NAME => 'Precteno', FORM_FIELD_SQL_INDEX => 3, FORM_FIELD_CAPTION_TEXT => 'Precteno', FORM_FIELD_SQL => 'SELECT Kod, Popis FROM S_AnoNe',
            FORM_FIELD_EDIT => 0, FORM_FIELD_REQUIRED => 0]);


        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_MEMO, FORM_FIELD_SQL_NAME => 'Vyjadreni',
            FORM_FIELD_NAME => 'Vyjadreni', FORM_FIELD_SQL_INDEX => 5, FORM_FIELD_CAPTION_TEXT => 'vyjadreni',
            FORM_FIELD_EDIT => 1, FORM_FIELD_REQUIRED => 0]);

        /*$this->addFields([
            [FORM_FIELD_TYPE => FORM_FIELD_TYPE_HIDDEN, FORM_FIELD_NAME => 'dokumentid', FORM_FIELD_SQL_NAME => 'idDokumentu', FORM_FIELD_DEFAULT_VALUE => $this->idDokumentu],
            [FORM_FIELD_TYPE => FORM_FIELD_TYPE_HIDDEN, FORM_FIELD_NAME => 'vzkazid', FORM_FIELD_SQL_NAME => 'idVzkazu', FORM_FIELD_DEFAULT_VALUE => $this->idVzkazu],
            [FORM_FIELD_TYPE => FORM_FIELD_TYPE_HIDDEN, FORM_FIELD_NAME => 'platnostod', FORM_FIELD_SQL_NAME => 'PlatnostOd', FORM_FIELD_DEFAULT_VALUE => StringUtils::format('date',$this->platnostDokumentuOd)]
        ]);*/
    }

    /**
     * @return string
     */
    function getWhereCondition()
    {
        return
            ' IdVzkazu = ' . (int)$this->idVzkazu .
            ' AND idDokumentu = ' . (int)$this->idDokumentu .
            ' AND PlatnostOd = ' . $this->database->timestampToString($this->platnostDokumentuOd);
    }

    /**
     * @return DBRecord|RonPortalDBRecord
     */
    function createDBRecord()
    {
        //readonly:false - neboť se edituje stavající, při vytváření nového se zadává true
        $result =  parent::createDBRecord();
        $result->addField('Precteno', 'int', false, 0, 1);
        $result->addField('IdentifikacePodpisu', 'memo', false, 0, sprintf(ronText('dokumenty_identifikace_podpisu'),StringUtils::format('datetime',time()), User::getLoginName()));

        return $result;
    }

    /**
     * @param DBRecord $rec
     */
    function setDBRecord($rec)
    {
        parent::setDBRecord($rec);
        $rec->where = $this->getWhereCondition();
    }


    function showCommonFields()
    {
        parent::showCommonFields();

        echo '<input type="hidden" name="dokumentid" value="' . $this->idDokumentu . '" />';
        echo '<input type="hidden" name="vzkazid" value="' . $this->idVzkazu . '" />';
        echo '<input type="hidden" name="platnostod" value="' . StringUtils::format('date', $this->platnostDokumentuOd) . '" />';
    }

    /**
     * @throws DBException
     */
    private function nactiDetailyVzkazu()
    {
        $params = $this->database ->createParameters();
        $sql =
            "SELECT Vzkaz.Id, Vzkaz.Predmet, " . $this->database->castWideMemo('Vzkaz.Popis') .  // 0 - 2
            ", DokumentVzkazu.idDokumentu, DokumentVzkazu.PlatnostOd, TypDokumentu.Popis, Vzkaz.DatumCas " .  //3 - 6
            "FROM Vzkaz JOIN DokumentVzkazu ON (Vzkaz.Id = DokumentVzkazu.IdVzkazu)
                    JOIN Dokument ON (DokumentVzkazu.idDokumentu = Dokument.id)
                    JOIN TypDokumentu ON (Dokument.typ = TypDokumentu.kod) " .
            "WHERE (Vzkaz.ZobrazitNaIntrawebu = 1)


                    AND (TypDokumentu.Distribuovany = 1)
                    AND (DokumentVzkazu.idVzkazu = " .  $params->addParameter($this->idVzkazu) . ")
                  ORDER BY TypDokumentu.Kod ";

        //       AND (Vzkaz.RCPrijemce = " . $params->addParameter(RonPortalSession::osobaRC()) . ")
        //       AND (DokumentVzkazu.Precteno = 0)

        $cursor = $this->database ->getSqlResult($sql, $params);
        $row = $cursor->fetch();

        $this->predmetVzkazu = $row[1];
        $this->popisTypuDokumentu = $row[5];
    }
}