<?php

/**
 * UTF-8
 * Pavel Rudol, RON Software
 * 8.7.2015
 */
namespace Ron\Personalistika;

require_once('adsapp/ronportalform.php');
require_once('adsapp/ppomer.php');

use Exception;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB;
use Ron\DB\Database;
use Ron\DBRecord;
use Ron\DBRecordField;
use Ron\FormField;
use Ron\RonPortal\OsobaField;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalForm;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\UserLoginField;
use Ron\StringUtils;
use function Ron\ronText;


class OsobaDovednostField extends OsobaField
{
    public $idPP;

    public function __construct($caption, $prava, $datumPlatnostiPP, $edit, $idPP = 0)
    {
        parent::__construct($caption, $prava, $datumPlatnostiPP, $edit);
        $this->idPP = $idPP;
    }


    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($this->idPP !== 0)
            $value = $this->idPP;

        $canEdit = $this->editable;
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
    }

}

/**
 * Zobrazi pole pro vyber dovednosti (pokud je editovatelne)
 */
class DovednostField extends FormField
{
    private $idPP;

    /**
     *
     * @param string $caption Nadpis pole
     * @param boolean $edit priznak, zda je pole editovatelne, nebo se zobrazi jen jeho hodnota
     * @param int $idPP id vybraneho pracovniho pomeru
     */
    function __construct($caption, $edit, $idPP)
    {
        parent::__construct();

        $this->fieldType = 'dovednost';
        $this->fieldName = 'dovednost';
        $this->caption = $caption;
        $this->required = true;

        $this->idPP = $idPP;

        $this->editable = $edit;
        $this->otherHtmlParams = 'onChange="personalistika.zobrazUrovneDovednosti(this.value);"';
    }

    /**
     * @param DB\Database $database
     * @param mixed $value
     * @param bool $canedit
     * @param bool $insert
     * @param array $rowdata
     * @throws DB\DBException
     * @throws Exception
     */
    function showFieldContent($database, $value, $canedit, $insert, $rowdata)
    {
        $edit = (isset($_POST[$this->fieldName . "_edit"]) ? $_POST[$this->fieldName . "_edit"] : $this->editable);
        
        if ($edit) {
            //if ( !isset($value) ) $value = RonPortalSession::vybranaOsoba();

            $combo = new ComboBox($this->fieldName); // jmeno promenne

            $filtrTPM = '';

            $typRozdeleniTPM = RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_tpm');

            //pokud je v nastaveni zvolen typ rozdeleni pro typova pracovni mista, bude mozne zamestnanci zadat jen dovednosti
            // pozadovane pro rozdeleni (typove pracovni misto) zvoleneho zamestnance
            if (($typRozdeleniTPM != -1) && ($typRozdeleniTPM != 0)) {
                $filtrTPM =
                    ' AND EXISTS(SELECT Rozdeleni FROM RozdeleniOsob 
                                   JOIN PracovniPomer ON (PracovniPomer.RC = RozdeleniOsob.RC AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo 
                                        AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu)
                                   JOIN PozadavekNaDovednost ON (PozadavekNaDovednost.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni 
                                        AND PozadavekNaDovednost.KodRozdeleni = RozdeleniOsob.Rozdeleni)     
                                 WHERE PracovniPomer.ID = ' . (int)$this->idPP .
                    '              AND RozdeleniOsob.KodTypuRozdeleni = ' . (int)$typRozdeleniTPM .
                    '              AND RozdeleniOsob.PlatnostOd <= ' . $database->timestampToString(DateUtils::today()) .
                    '              AND (RozdeleniOsob.PlatnostDo >= ' . $database->timestampToString(DateUtils::today()) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                    '              AND PozadavekNaDovednost.KodDovednosti = Dovednost.Kod ' .
                    ')';
            }

            $sql = "SELECT Kod, Popis FROM Dovednost " . 
                    "WHERE Aktivni = 1" .
                    "  AND EXISTS(SELECT KodUrovne FROM UrovenDovednosti WHERE KodDovednosti = Dovednost.Kod)" .
                    //17.3.2017 RON-PR dovednosti, ktere jsou slozene nebo definovane hodnotou nelze rucne zadat
                    "  AND Slozena = 0 " .
                    "  AND DefinovanaHodnotou = 0  
                    $filtrTPM                    
                    ORDER BY Popis";

            $combo->emptyRow = ' ';
            $combo->loadData($database, $sql, $this->defaultValue);
            $combo->htmlParams = $this->otherHtmlParams;
            $combo->render();
        } else {
            echo '<input type="hidden" name="' . $this->fieldName . '" value="' . $this->defaultValue . '" />';
            echo '<input type="hidden" name="' . $this->fieldName . "_edit" . '" value="' . $this->editable . '" />';
            echo $this->loadName($database, $this->defaultValue) . "\n";
        }
    }
    
    function createDBRecordField($database, $tablename, $insert)
    {
        $field = new DBRecordField($this->sqlFieldName, '', !$this->editable, $this->maxLength);
        $field->value = trim($_POST[$this->fieldName]);
        
        return $field;
    }

    /**
     *
     * @param DB\Database $database
     * @param $value
     * @return string
     * @throws Exception
     */
    function loadName($database, $value)
    {
        $params = $database->createParameters();
        $sql = "SELECT Popis FROM Dovednost WHERE Kod = " . $params->addParameter($value);
        
        $cursor = $database ->getSqlResult($sql, $params);

        if (!$row = $cursor->fetch()) {
            throw new Exception('Chyba - dovednost: ' . $this->defaultValue . ' nebyla nalezena!' . $sql);
        }

        return $row[0];
    }

}

class DovednostForm extends RonPortalForm
{
    const XML_EVENT_BEFORE = 'DOVEDNOST_BEFORE';
    const XML_EVENT_AFTER = 'DOVEDNOST_AFTER';
    const XML_CONTEXT_INS = 'insert';
    const XML_CONTEXT_UPD = 'update';

    private $prava;
    private $pracPomer;
    private $dovednost;
    private $editDovednost = true;

    /**
     * DovednostForm constructor.
     * @param Database $database
     * @param int $idPP
     * @throws Exception
     */
    function __construct($database, $idPP = 0)
    {
        parent::__construct($database);

        Prava::nastavObdobiProPrava(time(),time());
        $this ->prava = Prava::vratPrava('doved_edit');

        if ($idPP) {
            $this->pracPomer = new PracovniPomer();
            $this->pracPomer->nactiPPPodleId($this->database, $idPP);
        }
        $this->nactiParametr();

        if (isset($_POST['dovednost'])) { $this->dovednost = filter_input(INPUT_POST, 'dovednost'); }

        $this ->htmlAction = '?menuid=maticedovednosti';
        if (Prava::vratHodnotuPrava('doved_ziskane') != 0) {
            $stornoURL = '?menuid=maticedovednosti&action='.$_SESSION['dovednostibrowser'];
        } else {
            $stornoURL = '?menuid=maticedovednosti&action=novadovednost';
        }

        $this -> stornoAction = $stornoURL;

        $this -> sqlTableName = 'DovednostOsoby';

        $this -> dbLogSrcDescription = 'Formular nove dovednosti osoby';

        $this->sqlSelectPart =
            'DovednostOsoby.RC, DovednostOsoby.KodDovednosti, DovednostOsoby.KodUrovne, DovednostOsoby.DatumZiskani, DovednostOsoby.PlatnostDo, '. //0 - 4
            $this->database->castWideMemo('DovednostOsoby.Poznamka') . ', ' . // 5
            'DovednostOsoby.Insert_UserName, DovednostOsoby.Insert_Timestamp, DovednostOsoby.Update_UserName, DovednostOsoby.Update_Timestamp, '. //6 - 9
            'DovednostOsoby.SpecifikaceUrovne'; //10

        $osoba = new OsobaDovednostField(ronText('Osoba'), 'doved_edit', time(), $this->isInsert(), $idPP);
        $osoba->otherHtmlParams = 'onChange="window.location = &quot;?menuid=maticedovednosti&formaction=' . $this->formAction . '&idosoby=&quot;+this.value;"';

        $this->addField($osoba);

        $dovednost = new DovednostField(ronText('Dovednosti_dovednost_form'),  $this->editDovednost, $this->pracPomer->idPP);
        $dovednost->defaultValue = $this->dovednost;
        $dovednost->sqlFieldName = 'DovednostOsoby.KodDovednosti';

        $this->addField($dovednost);

        $this ->addFields([
            [FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_NAME => 'urovenDovednosti', FORM_FIELD_SQL_NAME => 'DovednostOsoby.KodUrovne', FORM_FIELD_SQL_INDEX => 2,
                FORM_FIELD_ROW_ID => 'urovneDovednosti', FORM_FIELD_PRIMARY => true, FORM_FIELD_CAPTION_TEXT => 'Dovednosti_uroven_dovednosti',
                FORM_FIELD_SQL => "SELECT KodUrovne, Popis FROM UrovenDovednosti " .
                    "WHERE KodDovednosti = '" . $database->addSlashes($this->dovednost) . "'" .
                    "ORDER BY KodUrovne, Popis",
                FORM_FIELD_DATA_TYPE => 'int', FORM_FIELD_EDIT => true, FORM_FIELD_READONLY => 0, FORM_FIELD_REQUIRED => true],
            [FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_NAME => 'specifikaceUrovneDovednosti', FORM_FIELD_SQL_NAME => 'DovednostOsoby.SpecifikaceUrovne',
                FORM_FIELD_SQL_INDEX => 10, FORM_FIELD_ROW_ID => 'specifikaceUrovneDovednosti', FORM_FIELD_PRIMARY => FALSE,
                FORM_FIELD_CAPTION_TEXT => 'Dovednosti_specifikace_urovne',
                FORM_FIELD_SQL => "SELECT Kod, Popis FROM S_SpecUrovneDovednosti " .
                    //"WHERE KodDovednosti = '" . $database->addSlashes($this->f_dovednost) . "'" .
                    "ORDER BY Kod, Popis",
                FORM_FIELD_DATA_TYPE => 'int', FORM_FIELD_EDIT => true, FORM_FIELD_READONLY => 0, FORM_FIELD_REQUIRED => true, FORM_FIELD_DEFAULT_VALUE => 0],
            [FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_NAME => 'DatumZiskani', FORM_FIELD_SQL_NAME => 'DovednostOsoby.DatumZiskani',
                FORM_FIELD_SQL_INDEX => 3 , FORM_FIELD_CAPTION_TEXT => 'Dovednosti_datum_ziskani', FORM_FIELD_PRIMARY => true,
                FORM_FIELD_EDIT => true, FORM_FIELD_REQUIRED => true, FORM_FIELD_MAXLENGTH => 10,
                FORM_FIELD_DEFAULT_VALUE => StringUtils::format('date', DateUtils::today())],
            //[FORM_FIELD_TYPE => 'date', FORM_FIELD_NAME => 'PlatnostDo', FORM_FIELD_SQL_NAME => 'DovednostOsoby.PlatnostDo', FORM_FIELD_SQL_INDEX => 4, FORM_FIELD_CAPTION => $STR_PlatnostDo, FORM_FIELD_MAXLENGTH => 10], nastavuje agent
            [FORM_FIELD_TYPE => FORM_FIELD_TYPE_MEMO, FORM_FIELD_NAME => 'Poznamka' ,FORM_FIELD_SQL_NAME => 'DovednostOsoby.Poznamka', FORM_FIELD_SQL_INDEX => 5,
                FORM_FIELD_CAPTION_TEXT => 'Poznamka'],
            [FORM_FIELD_TYPE => 'file', FORM_FIELD_NAME => 'Dokument', ''],
            [FORM_FIELD_TYPE => FORM_FIELD_TYPE_ACTUAL_TIME, FORM_FIELD_SQL_NAME => 'DovednostOsoby.Insert_Timestamp', FORM_FIELD_SQL_INDEX => 7, FORM_FIELD_EDIT => false],
            [FORM_FIELD_TYPE => FORM_FIELD_TYPE_ACTUAL_TIME, FORM_FIELD_SQL_NAME => 'DovednostOsoby.Update_Timestamp', FORM_FIELD_SQL_INDEX => 9]
        ]);

        $updateUser = new UserLoginField('Update_UserName', true);
        $updateUser->sqlFieldIndex = 8;

        $this -> addField($updateUser);

        $insertUser = new UserLoginField('Insert_UserName', false);
        $insertUser->sqlFieldIndex = 6;

        $this -> addField($insertUser);


    }

    /**
     *
     * Nacte z URL adresy osobu, (dovednost)
     * @return void Nacte z URL adresy osobu, (dovednost)
     */
    function nactiParametr()
    {
        if (isset($_GET['dovednost'])) {
            $this ->dovednost = $_GET['dovednost'];

            //pokud lze zobrazit matici ziskanych dovednosti, nebude pro prokliku z matice (pri zvolene dovednosti) aktivni vyber dovednosti
            if (Prava::vratHodnotuPrava('doved_ziskane') != 0)
                $this->editDovednost = false;
        }
    }

    function getWhereCondition()
    {
        $params = $this->database->createParameters();
        return new DB\SqlStatement("RC = " . $params->addParameter($this->pracPomer->rc) . " AND KodDovednosti = " . $params->addParameter($this->dovednost), $params);
    }

    /**
     * @return DBRecord|RonPortalDBRecord
     */
    function createDBRecord()
    {
        $rec = parent::createDBRecord();

        $rec->addField('RC', '', true, 15, $this->pracPomer->rc);

        return $rec;
    }


    /**
     *
     * @param RonPortalDBRecord $rec zaznam, ktery se bude vkladat
     * @return bool vraci true, pokud muze byt zaznam vlozen
     * @throws Exception
     */
    function beforeInsertRecord($rec)
    {
        if (!$this->prava->lzeZobrazitRC($rec['RC'])) {
            RonPortalUtils::neniPravoException();
        }

        if ($this->checkUrovenDovednosti($rec['RC'], $rec['KodDovednosti'], $rec['KodUrovne'], $rec['DatumZiskani'])) {
            throw new Exception(ronText('Dovednosti_uroven_existuje'));
        }

        if (!$rec->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_INS)) {
            return false;
        }

        return true;
    }

    /**
     *
     * @param RonPortalDBRecord $rec zaznam, ktery se bude editovat
     * @return boolean vraci true, pokud muze byt zaznam editovan
     */
    function beforeUpdateRecord($rec) {
        if (!$rec->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_UPD)) {
            return false;
        }

        return true;
    }

    /**
     *
     * @param RonPortalDBRecord $rec zaznam, ktery byl vlozen
     * @return boolean vraci true, pokud XML probehlo uspesne
     */
    function afterInsertRecord($rec) {
        if (!$rec->executeXmlEvents(self::XML_EVENT_AFTER, self::XML_CONTEXT_INS)) {
            return false;
        }

        return true;
    }

    /**
     *
     * @param RonPortalDBRecord $rec zaznam, ktery byl editovan
     * @return boolean vraci true, pokud XML probehlo uspesne
     */
    function afterUpdateRecord($rec) {
        if (!$rec->executeXmlEvents(self::XML_EVENT_AFTER, self::XML_CONTEXT_UPD)) {
            return false;
        }

        return true;
    }


    /**
     *
     * @param string $rc
     * @param string $dovednost
     * @param string $uroven
     * @param int $datumZiskani
     * @return bool true pokud uz uroven ma
     * @throws DB\DBException
     */
    function checkUrovenDovednosti($rc, $dovednost, $uroven, $datumZiskani)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT RC FROM DovednostOsoby 
             WHERE RC = " . $params->addParameter($rc) . " AND KodDovednosti = " . $params->addParameter($dovednost) .
            " AND KodUrovne = " . $params->addParameter($uroven) .
            " AND DatumZiskani = " . $params->addParameter($this->database->dateToParam($datumZiskani));

        $cursor = $this->database ->getSqlResult($sql, $params);

        if (!$row = $cursor->fetch())
            return false;

        if ($row[0])
            return true;

        return false;
    }
}
