<?php
/** UTF-8
 * Roman.Szablatura, 8.6.2016
 */

namespace Ron\Personalistika;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\DBRecord;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalForm;
use Ron\StringUtils;
use function Ron\ronText;

require_once ('adsapp/ronportalform.php');


class EditaceExpiracePomuckyForm extends RonPortalForm
{
    private $osoba;
    private $pomucka;
    private $datumnaroku;
    private $poradi;
    private $vydanipomucky;
    private $editexpirace;

    /**
     * EditaceExpiracePomuckyForm constructor.
     * @param Database $database
     * @param $formAction
     */
    function __construct($database, $formAction)
    {
        parent::__construct($database, $formAction);

        $this->title = ronText('editace_expirace_pomucky');
        $this->htmlAction = '?menuid=pracovnipomucky&action=editaceexpirace';
        $this->stornoAction = '?menuid=pracovnipomucky';

        $this->sqlTableName = 'PrirazenaPracPomucka';
        $this->dbLogSrcDescription = StringUtils::fromUTF8('Editace expirace pomůcky');
        $this->sqlSelectPart =
            'PrirazenaPracPomucka.DatumVydani, PrirazenaPracPomucka.DatumVraceni, ' .   // 0 - 1
            $database->castWideMemo('PrirazenaPracPomucka.Poznamka') . ' as Poznamka, PrirazenaPracPomucka.DatumExpirace, '. // 2 - 3
            'PrirazenaPracPomucka.PracovniPomucka, DatumExpiraceVypoctene';   //4 - 5

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_TEXT, FORM_FIELD_TEXT => $_GET['pomucka'], FORM_FIELD_CAPTION_TEXT => ronText('kod_pomucky')]);

        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_SQL_NAME => 'DatumVydani', FORM_FIELD_SQL_INDEX => 0,
            FORM_FIELD_NAME => 'DatumVydani', FORM_FIELD_CAPTION_TEXT => 'datum_vydani',
            FORM_FIELD_EDIT => 0]);

        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_SQL_NAME => 'DatumExpiraceVypoctene',
            FORM_FIELD_NAME => 'DatumExpiraceVypoctene', FORM_FIELD_SQL_INDEX => 5, FORM_FIELD_CAPTION_TEXT => 'datum_expirace_vypoctene',
            FORM_FIELD_EDIT => 0]);

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_SQL_NAME => 'DatumExpirace',
            FORM_FIELD_NAME => 'DatumExpirace', FORM_FIELD_SQL_INDEX => 3, FORM_FIELD_CAPTION_TEXT => 'datum_expirace', FORM_FIELD_EDIT => 1]);


        $this->addField( [ FORM_FIELD_TYPE => FORM_FIELD_TYPE_MEMO, FORM_FIELD_NAME => 'Poznamka', FORM_FIELD_CAPTION_TEXT => 'text_cinnosti',
            FORM_FIELD_EDIT => 1, FORM_FIELD_SQL_INDEX => 2, FORM_FIELD_SQL_NAME => 'Poznamka']);

    }

    private function getPomuckaDetail()
    {
        if (isset($_POST['osoba'])) $osoba = $_POST['osoba'];
        else if (isset($_GET['osoba'])) $osoba = $_GET['osoba'];
        else $osoba = '';
        $this->osoba = $osoba;

        if(isset($_POST['pomucka'])) $pomucka = $_POST['pomucka'];
        else if (isset($_GET['pomucka'])) $pomucka = $_GET['pomucka'];
        else $pomucka = '';
        $this->pomucka = $pomucka;

        if (isset($_POST['datumnaroku'])) $datumnaroku = $_POST['datumnaroku'];
        else if(isset($_GET['datumnaroku'])) $datumnaroku = $_GET['datumnaroku'];
        else $datumnaroku = '';
        $this->datumnaroku = $datumnaroku;

        if (isset($_POST['poradi'])) $poradi = $_POST['poradi'];
        else if(isset($_GET['poradi'])) $poradi = $_GET['poradi'];
        else $poradi = '';
        $this->poradi = $poradi;

        if (isset($_POST['vydanipomucky'])) $vydanipomucky = $_POST['vydanipomucky'];
        else if(isset($_GET['vydanipomucky'])) $vydanipomucky = $_GET['vydanipomucky'];
        else $vydanipomucky = '';
        $this->vydanipomucky = $vydanipomucky;

        if (isset($_POST['editexpirace'])) $editexpirace = $_POST['editexpirace'];
        else if(isset($_GET['editexpirace'])) $editexpirace = $_GET['editexpirace'];
        else $editexpirace = '';
        $this->editexpirace = $editexpirace;
    }

    /**
     * @return string
     * @throws DBException
     */
    function getWhereCondition()
    {
        $this->getPomuckaDetail();

        $params = $this->database->createParameters();
        $osobaRC = DBUtils::getSqlValue($this->database,"SELECT RC FROM PracovniPomer WHERE ID = " . $params->addParameter($this->osoba), $params);

        if ($this->osoba != '' && $this->pomucka != '' && $this->datumnaroku != '' && $this->poradi != '') {
            $params = $this->database->createParameters();
            return new SqlStatement(" RC = " . $params->addParameter($osobaRC) . " AND PracovniPomucka = " . $params->addParameter($this->pomucka) .
            ' AND DatumNaroku = ' .  $params->addParameter($this->database->timestampToParam($this->datumnaroku)) .  ' AND Poradi = ' .  $params->addParameter($this->poradi), $params);
        }
        return '';
    }

    function showCommonFields()
    {
        parent::showCommonFields();

        $this->getPomuckaDetail();

        echo '<input type="hidden" name="osoba" value="' . $this->osoba . '" />';
        echo '<input type="hidden" name="pomucka" value="' . $this->pomucka . '" />';
        echo '<input type="hidden" name="datumnaroku" value="' . $this->datumnaroku . '" />';
        echo '<input type="hidden" name="poradi" value="' . $this->poradi . '" />';
        echo '<input type="hidden" name="vydanipomucky" value="' . $this->vydanipomucky . '" />';
        echo '<input type="hidden" name="editexpirace" value="' . $this->editexpirace . '" />';
        echo '';
    }

    /**
     * @return DBRecord|RonPortalDBRecord
     * @throws DBException
     */
    function createDBRecord()
    {
        $result =  parent::createDBRecord();

        require_once('adsapp/ppomer.php');
        $pracPomer = new PracovniPomer();
        $pracPomer->nactiPPPodleId($this->database, (int)$_POST['osoba']);

        $result->addField('RC', '', true, 15, $pracPomer->rc);
        $result->addField('OsobniCislo', '', true, 15, $pracPomer->osobniCislo);
        $result->addField('DatumNastupu', '', true, 15, $pracPomer->datumNastupu);
        $result->addField('PracovniPomucka', '', true, 10,  $_POST['pomucka']);
        $result->addField('DatumNaroku', 'date', true, 0, time());
        $result->addField('Poradi', 'int', true, 0);

        return $result;
    }


    /**
     * @param DBRecord $rec
     * @throws DBException
     */
    function setDBRecord($rec)
    {
        parent::setDBRecord($rec);

        $where = $this->getWhereCondition();

        if ($where instanceof SqlStatement)
            $rec->setWhere($where->sqlText, $where->parameters);
        else
            $rec->where = $where;
    }


    /**
     * @param DBRecord $rec
     * @return bool
     * @throws Exception
     */
    function beforeUpdateRecord($rec)
    {
        if($rec['DatumExpirace'] == '') {
            throw new Exception(ronText('form.Missing_value') . ' ' . ronText('datum_expirace') . '!');
        }
        if($rec['DatumExpirace'] < $rec['DatumVydani']){
            throw new Exception(ronText('spatne_datum_expirace'));
        }

        return parent::beforeUpdateRecord($rec);
    }

    /**
     * @param DBRecord $rec
     * @return bool
     */
    function afterUpdateRecord($rec)
    {
        return parent::afterUpdateRecord($rec);
    }
}