<?php
/**
 * UTF-8
 * Roman.Szablatura 22.3.2016
 */

namespace Ron\Personalistika;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\DBRecord;
use Ron\FormField;
use Ron\MessageManager;
use Ron\RonPortal\DokumentyUtils;
use Ron\RonPortal\FormReportsListView;
use Ron\RonPortal\OsobaField;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalForm;
use Ron\RonPortal\RonPortalReport;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\UserLoginField;
use Ron\StringUtils;
use function Ron\ronText;

require_once ('adsapp/ronportalform.php');
require_once('ron/controls.php');
require_once('adsapp/ronportalcontrols.php');
require_once('person/lekarskaprohlidka.php');


/**
 * Class OsobaProhlidkyField
 * @package Ron\Personalistika
 */
class OsobaProhlidkyField extends OsobaField
{
    /** @var PracovniPomer */
    private $pracovniPomer;

    public function __construct($caption, $prava, $datumPlatnostiPP, $pp)
    {
        $this->pracovniPomer = $pp;
        parent::__construct($caption, $prava, $datumPlatnostiPP, false);
    }

    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($insert) {
            parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
        }
        elseif ($this->pracovniPomer) {
            echo '<input type="hidden" name="' . $this->fieldName . '" value="' . $this->pracovniPomer->idPP . '" />';
            echo StringUtils::htmlTextConvert($this->pracovniPomer->jmenoPrijmeniKratke) . "\n";
        }
    }
}


/**
 * Class DuvodLekarskeProhlidkyForm
 * @package Ron\Personalistika
 */
class DuvodLekarskeProhlidkyForm extends FormField
{
    /** @var PravaNaOsoby */
    var $prava;
    var $zdravotniprohlidka;


    function __construct($prava, $edit, $zdravotniprohlidka)
    {
        parent::__construct();

        $this->prava = $prava;
        $this->editable = $edit;
        $this->caption = ronText('duvod_zdravotni_prohlidky');
        $this->zdravotniprohlidka = $zdravotniprohlidka;
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws Exception
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        echo StringUtils::htmlTextConvert($this->zdravotniprohlidka->duvodProhlidky);
    }
}

/**
 * Class RezervovanyCasLekarskeProhlidkyForm
 * @package Ron\Personalistika
 */
class RezervovanyCasLekarskeProhlidkyForm extends FormField
{
    /** @var PravaNaOsoby */
    var $prava;
    var $zdravotniprohlidka;


    /**
     * RezervovanyCasLekarskeProhlidkyForm constructor.
     * @param null $prava
     * @param $edit
     * @param LekarskaProhlidka $zdravotniprohlidka
     * @param $fieldName
     */
    function __construct($prava, $edit, $zdravotniprohlidka, $fieldName)
    {
        parent::__construct();

        $this->fieldType = FORM_FIELD_TYPE_DATEANDTIME;
        $this->fieldName = $fieldName;

        $this->prava = $prava;
        $this->editable = $edit;
        $this->caption = ronText('rezervovany_cas_prohlidky');
        $this->zdravotniprohlidka = $zdravotniprohlidka;
    }

    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($canEdit) {
            echo '<input type="text" size="3" class="inputTime" value="' . StringUtils::format('time', $this->zdravotniprohlidka->casProhlidkyRezervovany) . '"
                    name="' . $this->fieldName . '_time">';
            echo '&nbsp;';
            echo '<input type="text" class="app_input inputDate" size="7" name="' . $this->fieldName . '"
                    value="' . StringUtils::format('date', $this->zdravotniprohlidka->datumProhlidkyRezervovany) . '">';
            $this->showCalendarButton();
        } else {
            echo StringUtils::format('time', $this->zdravotniprohlidka->casProhlidkyRezervovany) . '&nbsp;&nbsp;' .
                StringUtils::format('date', $this->zdravotniprohlidka->datumProhlidkyRezervovany);
        }
    }
}

/**
 * Class TiskoveSestavyLekarskeProhlidkyForm
 * @package Ron\Personalistika
 */
class TiskoveSestavyLekarskeProhlidkyForm extends FormField
{
    /** @var PravaNaOsoby */
    var $prava;
    var $zdravotniprohlidka;


    function __construct($prava, $edit, $zdravotniprohlidka)
    {
        parent::__construct();

        $this->prava = $prava;
        $this->editable = $edit;
        $this->zdravotniprohlidka = $zdravotniprohlidka;
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws Exception
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        echo '<div id="prohlidky_reports">';

        require_once('adsapp/reportutils.php');
        $reportList = RonPortalReport::loadFormReports($database, 'LekarskeProhlidky', 'LekarskeProhlidky');
        FormReportsListView::renderReportList($reportList,
            '?menuid=lekarskeprohlidky&action=report&osoba=' . RonPortalSession::vybranaOsoba() .
            '&kod=' . ($_GET['kod'] ?: '') .
            '&vznik=' . (StringUtils::format('date', $_GET['vznik']) ?: '')
        );

        echo '</div>';
    }
}


/**
 * Class DokumentyLekarskeProhlidkyForm
 * @package Ron\Personalistika
 */
class DokumentyLekarskeProhlidkyForm extends FormField
{
    /** @var PravaNaOsoby */
    var $prava;
    var $zdravotniprohlidka;


    function __construct($prava, $edit, $zdravotniprohlidka)
    {
        parent::__construct();

        $this->prava = $prava;
        $this->editable = $edit;
        $this->zdravotniprohlidka = $zdravotniprohlidka;
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws Exception
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if (is_array($this->zdravotniprohlidka->dokumenty)) {
            foreach ($this->zdravotniprohlidka->dokumenty as $prohlidkaDokumenty) {

                echo "<tr><td class='formtablecaption'>" . StringUtils::htmlTextConvert($prohlidkaDokumenty->popisDokumentu) . "</td><td class='inputtablesection'>";

                echo StringUtils::format('date', $prohlidkaDokumenty->datumVlozeniDokumentu);
                echo '&nbsp;';
                echo '<br />';
                echo '<a href="?menuid=lekarskeprohlidky&action=download&osoba=' . $this->zdravotniprohlidka->idOsoby
                    . '&dokument=' . $prohlidkaDokumenty->idDokumentu . '" target="_blank">' . StringUtils::htmlTextConvert($prohlidkaDokumenty->nazevDokumentu) . '</a>';
                echo '&nbsp;';
                echo '</td></tr>';
            }
        }
    }
}


/**
 * Class LekarskaProhlidkaEditaceAbsolvovaniForm
 * @package Ron\Personalistika
 */
class LekarskaProhlidkaEditaceAbsolvovaniForm extends RonPortalForm
{
    /** @var PravaNaOsoby $pravaZapis */
    var $pravaZapis;
    var $lzeEditovatZaznam;

    /** @var PravaNaOsoby $pravaAbsolvovat */
    var $pravaAbsolvovat;
    var $lzeAbsolvovatProhlidku;

    private $osobaRC;
    private $muzeAbsolvovat;

    /**
     * LekarskaProhlidkaEditaceAbsolvovaniForm constructor.
     * @param Database $database
     * @param string $formAction
     * @param PravaNaOsoby $pravoNaZapis
     * @param PravaNaOsoby $pravoAbsolvovat
     * @param bool $absolvovat
     * @param PracovniPomer $pracovniPomer
     * @throws Exception
     * @throws DBException
     */
    function __construct($database, $formAction, $pravoNaZapis, $pravoAbsolvovat, $absolvovat, $pracovniPomer)
    {
        parent::__construct($database, $formAction);

        $zdravotniprohlidka = null;
        $this->muzeAbsolvovat = $absolvovat;
        $vlozeni = ($formAction == 'new' || $formAction == 'insert');

        $this->osobaRC = $pracovniPomer->rc;
        $this->pravaZapis = $pravoNaZapis;
        $this->lzeEditovatZaznam = $this->pravaZapis->lzeZobrazitRC($this->osobaRC);
        $this->pravaAbsolvovat = $pravoAbsolvovat;
        $this->lzeAbsolvovatProhlidku = $this->pravaAbsolvovat->lzeZobrazitRC($this->osobaRC);


        $this->title = ronText('zdravotni_prohlidka_detail');
        $this->htmlAction = '?menuid=lekarskeprohlidky&action=' . ( $this->muzeAbsolvovat ? 'absolvovat' : 'lekarskaprohlidkaform');
        $this->stornoAction = '?menuid=lekarskeprohlidky';
        $this->sqlTableName = 'GenerovaneLekProhlidky';
        $this->dbLogSrcDescription = 'Lekarske Prohlidky'; //'Editace datumu absolvovane lekarske prohlidky';
        $this->sqlSelectPart =
            'GenerovaneLekProhlidky.DatumVznikuPovinnosti, GenerovaneLekProhlidky.DatumAbsolvovani, GenerovaneLekProhlidky.KodLekarskeProhlidky, ' .   // 0 - 2
            $database->castWideMemo('GenerovaneLekProhlidky.Poznamka') . ' as Poznamka, Vysledek, OsetrujiciLekar, ' . // 3 - 5
            $database->castWideMemo('GenerovaneLekProhlidky.DoplnujiciTextPozvanky') . ' as DoplnujiciTextPozvanky ' ; // 6

        if ($formAction == 'edit') {
//        if ($formAction == 'new' || $formAction == 'edit') {
            $zdravotniprohlidka = LekarskaProhlidka::nactiProhlidku($database, $pracovniPomer, $_GET['kod'], $_GET['vznik']);
        }

        //Osoba
        if (isset($_GET['osoba'])) {
            $pracovniPomer->vyberNactenyPP($database,Prava::vratPrava('lekarske_prohl'));
        }

//        $this->addField(new OsobaField(\Ron\ronText('Osoba'),'lekarske_edit',time(),$vlozeni));
        $this->addField(new OsobaProhlidkyField(ronText('Osoba'), 'lekarske_edit', DateUtils::now(), $pracovniPomer));

        //Skryte pole: Duvod zdravotni prohlidky - hodnota napevno nastavena na Individualni
        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_HIDDEN, FORM_FIELD_DEFAULT_VALUE => 1, FORM_FIELD_DATA_TYPE => 'int',
                        FORM_FIELD_SQL_NAME => 'DuvodProhlidky', FORM_FIELD_NAME => 'DuvodProhlidky']);

        //Zobrazeni sekce Detaily
        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_SECTION, FORM_FIELD_CAPTION_TEXT => 'Detail']);

        //Prohlidka
        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_CAPTION_TEXT => 'prohlidka', FORM_FIELD_REQUIRED => $vlozeni,
            FORM_FIELD_SQL_NAME => 'GenerovaneLekProhlidky.KodLekarskeProhlidky',  FORM_FIELD_NAME => 'KodLekarskeProhlidky', FORM_FIELD_SQL_INDEX => 2,
            FORM_FIELD_SQL => 'SELECT LekarskaProhlidka.Kod, LekarskaProhlidka.Popis  FROM LekarskaProhlidka', FORM_FIELD_EDIT => false]);

        //Datum vzniku povinnosti
        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_SQL_NAME => 'DatumVznikuPovinnosti',
            FORM_FIELD_NAME => 'DatumVznikuPovinnosti', FORM_FIELD_SQL_INDEX => 0, FORM_FIELD_CAPTION_TEXT => 'datum_vzniku_povinnosti',
            FORM_FIELD_REQUIRED => 1, FORM_FIELD_EDIT => 0, 'primary' => true, FORM_FIELD_DEFAULT_VALUE => StringUtils::format('date',DateUtils::startOfTheDay(time()))]);

        //Duvod lekarske prohlidky
        $this->addField(new DuvodLekarskeProhlidkyForm(Prava::vratPrava('lekarske_prohl'), true, $zdravotniprohlidka));

        //Rezervovane cas
        $this->addField(new RezervovanyCasLekarskeProhlidkyForm(Prava::vratPrava('lekarske_prohl'), $this->muzeAbsolvovat ? 0 : $this->lzeEditovatZaznam , $zdravotniprohlidka, 'DatumProhlidkyRezervovany'));

        //Osetrujici lekar
        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_CAPTION_TEXT => 'osetrujici_lekar', FORM_FIELD_REQUIRED => 0,
            FORM_FIELD_SQL_NAME => 'OsetrujiciLekar',  FORM_FIELD_NAME => 'OsetrujiciLekar', FORM_FIELD_EDIT => $this->muzeAbsolvovat ? 0 : $this->lzeEditovatZaznam , FORM_FIELD_SQL_INDEX => 5,
            FORM_FIELD_SQL => "SELECT OsetrujiciLekar.kod, COALESCE(OsetrujiciLekar.Prijmeni,'') {fn_concat()} ' ' {fn_concat()} COALESCE(OsetrujiciLekar.JmenoLekare,'')  FROM OsetrujiciLekar"]);

        //Doplnujici poznamka
        $this->addField( [ FORM_FIELD_TYPE => FORM_FIELD_TYPE_MEMO, FORM_FIELD_NAME => 'DoplnujiciTextPozvanky', FORM_FIELD_CAPTION_TEXT => 'doplnujici_text',
            FORM_FIELD_EDIT => $this->muzeAbsolvovat ? 0 : $this->lzeEditovatZaznam , FORM_FIELD_SQL_INDEX => 6, FORM_FIELD_SQL_NAME => 'DoplnujiciTextPozvanky']);

        //Poznamka
        $this->addField( [ FORM_FIELD_TYPE => FORM_FIELD_TYPE_MEMO, FORM_FIELD_NAME => 'Poznamka', FORM_FIELD_CAPTION_TEXT => 'Poznamka',
            FORM_FIELD_EDIT => $this->muzeAbsolvovat ? 0 : $this->lzeEditovatZaznam , FORM_FIELD_SQL_INDEX => 3, FORM_FIELD_SQL_NAME => 'Poznamka']);

        //Vysledek
        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_CAPTION_TEXT => 'vysledek', FORM_FIELD_REQUIRED => $this->muzeAbsolvovat/*!$vlozeni*/,
            FORM_FIELD_SQL_NAME => 'Vysledek',  FORM_FIELD_NAME => 'Vysledek', FORM_FIELD_EDIT => ($this->muzeAbsolvovat && $this->lzeAbsolvovatProhlidku) || $this->lzeEditovatZaznam, FORM_FIELD_SQL_INDEX => 4,
            FORM_FIELD_SQL => 'SELECT S_VyhodnoceniProhlidky.Kod, S_VyhodnoceniProhlidky.Popis FROM S_VyhodnoceniProhlidky']);

        //31.1.2018 po porada s RaUr datum absolvovani skryji (31.1.2018), doplni se automaticky po spusteni personalniho agenta
        //Datum absolvovani
        //$this->addField([FORM_FIELD_TYPE => FORM_FIELD_TEXT, FORM_FIELD_CAPTION_TEXT => 'datum_absolvovani', FORM_FIELD_TEXT => \Ron\ronText('datum_absolvovani_agentem')]);

//        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_SQL_NAME => 'DatumAbsolvovani',
//            FORM_FIELD_NAME => 'DatumAbsolvovani', FORM_FIELD_SQL_INDEX => 1, FORM_FIELD_CAPTION_TEXT => 'datum_absolvovani',
//            FORM_FIELD_REQUIRED => $this->muzeAbsolvovat, FORM_FIELD_EDIT => ($this->muzeAbsolvovat && $this->lzeAbsolvovatProhlidku) || $this->lzeEditovatZaznam , 'useDefaultValueToEditNull' => 1,
//            FORM_FIELD_DEFAULT_VALUE => $this->muzeAbsolvovat ? ($formAction == 'edit' ? StringUtils::format('date',DateUtils::startOfTheDay(time())) : '') : '']);

        //Zobrazeni sekce Dokumenty
        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_SECTION, FORM_FIELD_CAPTION_TEXT => 'tiskove_sestavy']);

        //Zobrazeni tiskove sestavy
        $this->addField(new TiskoveSestavyLekarskeProhlidkyForm(Prava::vratPrava('lekarske_prohl'), true, $zdravotniprohlidka));

        //Zobrazeni sekce Dokumenty
        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_SECTION, FORM_FIELD_CAPTION_TEXT => 'Dokumenty']);

        ///Dokumenty
        $this->addField(new DokumentyLekarskeProhlidkyForm(Prava::vratPrava('lekarske_prohl'), $this->lzeEditovatZaznam , $zdravotniprohlidka));

        $this->addField(new UserLoginField('Insert_UserName', false));
        $this->addField(new UserLoginField('Update_UserName', true));
        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_ACTUAL_TIME, FORM_FIELD_SQL_NAME => 'Insert_Timestamp', FORM_FIELD_NAME => 'Insert_Timestamp', FORM_FIELD_EDIT => false]);
        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_ACTUAL_TIME, FORM_FIELD_SQL_NAME => 'Update_Timestamp', FORM_FIELD_NAME => 'Update_Timestamp', FORM_FIELD_EDIT => true]);

        if($this->lzeEditovatZaznam) {
            $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_FILE, FORM_FIELD_CAPTION_TEXT => '', FORM_FIELD_REQUIRED => false, FORM_FIELD_NAME => 'dokument' ]);
        }
    }


    /**
     * @return SqlStatement
     */
    function getWhereCondition()
    {
        if(isset($_POST['kod'])) $kod = $_POST['kod'];
        else if (isset($_GET['kod'])) $kod = $_GET['kod'];
        else $kod = '';

        if (isset($_POST['vznik'])) $vznik = $_POST['vznik'];
        else if(isset($_GET['vznik'])) $vznik = $_GET['vznik'];
        else $vznik = '';

        $params = $this->database->createParameters();
        $sql = "";
        if ($this->osobaRC != '' && $kod != '' && $vznik != '') {
            $sql .= " RC = " . $params->addParameter($this->osobaRC) . " AND KodLekarskeProhlidky = " .
                $params->addParameter($kod) . ' AND DatumVznikuPovinnosti = ' .
                $params->addParameter($this->database->timestampToParam($vznik));

        }
        return new SqlStatement($sql, $params);
    }

    function showCommonFields()
    {
        parent::showCommonFields();
        /*echo '<input type="hidden" name="absolvovat" value="' . $_GET['absolvovat'] . '" />';*/
        echo '<input type="hidden" name="kod" value="' . $_GET['kod'] . '" />';
        echo '<input type="hidden" name="vznik" value="' . $_GET['vznik'] . '" />';
    }

    /**
     * @param DBRecord $rec
     */
    function setDBRecord($rec)
    {
        parent::setDBRecord($rec);

        $where = $this->getWhereCondition();
        if ($where instanceof SqlStatement)
            $rec->setWhere($where->sqlText, $where->parameters);
        else
            $rec->where = $where;
    }


    /**
     * @return DBRecord|RonPortalDBRecord
     * @throws Exception
     */
    function createDBRecord()
    {
        $rec = parent::createDBRecord();
        $rec->addField('RC','',true,15, $this->osobaRC /*RonPortalSession::osobaRC()*/);

        //Datum
        if (isset($_POST['DatumProhlidkyRezervovany']) && ($_POST['DatumProhlidkyRezervovany'] != '')) {
            $rec->addField('DatumProhlidkyRezervovany','date',false,0, DateUtils::dateStringToTimestamp($_POST['DatumProhlidkyRezervovany']));
        }

        //cas
        if (isset($_POST['DatumProhlidkyRezervovany_time'])  && ($_POST['DatumProhlidkyRezervovany_time'] != '')) {
            $rec->addField('CasProhlidkyRezervovany', 'time', false, 0, DateUtils::timeStringToTimestamp($_POST['DatumProhlidkyRezervovany_time']));
        }
        return $rec;
    }

    /**
     * @param DBRecord $rec
     * @return bool
     * @throws DBException
     */
    function beforeInsertRecord($rec)
    {
        return $this->testovaniVlozeniProhlidky($rec);
    }

    /**
     * @param DBRecord $rec
     * @return bool
     * @throws DBException
     */
    function beforeUpdateRecord($rec)
    {
        return $this->testovaniVlozeniProhlidky($rec);
    }

    /**
     * @param $rec
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    function testovaniVlozeniProhlidky($rec)
    {
        $vlozeniDokumentu = true;

        if(DBUtils::getSqlValue($this->database, 'SELECT KategorieDokumentu FROM TypDokumentu WHERE kod = ' . (int)RonPortalSettings::getIntValue('lekprohl_typdokumentu')) != 3){
            throw new Exception(ronText('nekompatibilni_typ_dokumentu'));
        }

        if (is_uploaded_file($_FILES['dokument']['tmp_name'])) {
            require_once('adsapp/dokumentyutils.php');
            $idDokumentu = DokumentyUtils::vlozDokument($this->database, 'dokument', RonPortalSettings::getIntValue('lekprohl_typdokumentu'),
                sprintf(ronText('lekprohl_dokument'), $rec['KodLekarskeProhlidky']), $this->dbLogSrcDescription);

            $recDokumentyOsoby = new RonPortalDBRecord($this->database, 'DokumentyOsoby');
            $recDokumentyOsoby ->addField('id', 'int', false, 0, $idDokumentu);
            $recDokumentyOsoby ->addField('RC', '', false, 15, $this->osobaRC);
            $recDokumentyOsoby ->addField('GLP_RC', '', false, 15, $this->osobaRC);
            $recDokumentyOsoby ->addField('GLP_KodLP', '', false, 10, $rec['KodLekarskeProhlidky']);
            $recDokumentyOsoby ->addField('GLP_Datum', 'datetime', false, 0, $rec['DatumVznikuPovinnosti']);

            $vlozeniDokumentu = $recDokumentyOsoby ->insertRecord();
        }

        if ($this->muzeAbsolvovat) {
            $prava = $this->pravaAbsolvovat;
        }
        else {
            $prava = $this->pravaZapis;
        }

        if (!$prava->lzeZobrazitRC($rec['RC'],time(),time())) {
            MessageManager::addErrorMessage(ronText('Nemate_pravo'),'osoba');
            return false;
        }

        if (!$vlozeniDokumentu) {
            MessageManager::addErrorMessage(ronText('chyba_pri_vlozeni_dokumentu'),'dokument');
            return false;
        }
        return true;
    }
}