<?php
/**
 * UTF-8
 * Roman.Szablatura 24.2.2016
 */

namespace Ron\Personalistika;

use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\DBUtils;
use Ron\DB\SQLUtils;
use Ron\DBRecord;
use Ron\FormField;
use Ron\MVC\Model;
use Ron\RonPortal\ComboBoxZamestnanec;
use Ron\RonPortal\OsobaField;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalForm;
use Ron\RonPortal\RonPortalSession;
use Ron\StringUtils;

require_once ('adsapp/ronportalform.php');
require_once('ron/controls.php');
require_once('adsapp/ronportalcontrols.php');


/**
 * Class OsobaPracovnichPomucekField
 * @package Ron\Personalistika
 */
class OsobaPracovnichPomucekField extends OsobaField
{
    /** @var PracovniPomer */
    private $pracovniPomer;

    public function __construct($caption, $prava, $datumPlatnostiPP, $pp)
    {
        $this->pracovniPomer = $pp;
        parent::__construct($caption, $prava, $datumPlatnostiPP, false);
    }

    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($insert) {
            parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
        }
        elseif ($this->pracovniPomer) {
            echo '<input type="hidden" name="' . $this->fieldName . '" value="' . $this->pracovniPomer->idPP . '" />';
            echo StringUtils::htmlTextConvert($this->pracovniPomer->jmenoPrijmeniKratke) . "\n";
        }
    }
}


/**
 * Class VybratOsobuField
 * @package Ron\Personalistika
 */
class VybratOsobuField extends FormField
{
    /** @var PravaNaOsoby */
    var $prava;

    /**
     * VybratPracPomuckuField constructor.
     * @param null $prava
     * @param bool $edit
     */
    function __construct($prava, $edit)
    {
        parent::__construct();

        $this->prava = $prava;
        $this->editable = $edit;
        $this->required = true;
        $this->caption = \Ron\ronText('Osoba');
    }

    /**
     * @param \Ron\DB\Database $database
     * @param mixed $value
     * @param bool $canedit
     * @param bool $insert
     * @param array $rowData
     * @throws \Ron\DB\DBException
     */
    function showFieldContent($database, $value, $canedit, $insert, $rowData)
    {
        if($canedit){
            $comboOsoba = new ComboBoxZamestnanec('osoba'); // jmeno promenne
            $comboOsoba ->className = 'app_combo adsfilterforminput_full';

            $comboOsoba->loadDataPP($database,
                time(), time(), $this->prava,
                false, RonPortalSession::vybranaOsoba()); // $emptyrow = false, $selectedvalue, vcetneexternich
            $comboOsoba->render();
        }else{
            $sql =
                "SELECT COALESCE(Osoba.Prijmeni,'') + ' ' + COALESCE(Osoba.Jmeno,'') + COALESCE(', ' + Osoba.Titul,''), PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, " . //0 - 2
                "(SELECT COUNT(*) FROM PracovniPomer WHERE PracovniPomer.RC = Osoba.RC AND PracovniPomer.DatumNastupu <= " . $database->timestampToString(time()) .
                "  AND (PracovniPomer.DatumUkonceni >= " . $database->timestampToString(time()) . " OR PracovniPomer.DatumUkonceni IS NULL))" . //3
                "FROM Osoba JOIN PracovniPomer ON (Osoba.RC = PracovniPomer.RC) WHERE PracovniPomer.ID = " . (int)$_GET['osoba'];

            $cursor = $database->getSqlResult($sql);

            $row = $cursor->fetch();

            $osoba = $row[0];

            if ($row[3] > 1) {
                $osoba .= ' - ' . \Ron\ronText('Osobni_cislo'). ' ' . $row[1] . ', ' . \Ron\ronText('Datum_nastupu') . ' ' . StringUtils::format('date',$database->stringToTimestamp($row[2]));
            }
            echo  $osoba;
        }
        /* $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_TEXT, FORM_FIELD_NAME => 'uzivatel',
            FORM_FIELD_SQL_INDEX => 4, FORM_FIELD_CAPTION_TEXT =>  \Ron\ronText('Jmeno_a_prijmeni'), FORM_FIELD_TEXT => S_GetOsobaJmenoPrijmeni() ,FORM_FIELD_EDIT => 0 ]);*/
    }
}

/**
 * Class VybratPracPomuckuField
 * @package Ron\Personalistika
 */
class VybratPracPomuckuField extends FormField
{
    /** @var PravaNaOsoby */
    var $prava;

    private $url;

    /**
     * VybratPracPomuckuField constructor.
     * @param null $prava
     * @param $edit
     * @param $url
     */
    function __construct($prava, $edit, $url)
    {
        parent::__construct();

        $this->prava = $prava;
        $this->editable = $edit;
        $this->url = $url;
        $this->caption = \Ron\ronText('pracovni_pomucka');
    }

    /**
     * @param \Ron\DB\Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws \Ron\DB\DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($canEdit) {
//            $sql =
//                "SELECT PP.Kod,  +'" . \Ron\ronText('Typ') . ": ' + PP.TypPomucky + ' - " . \Ron\ronText('Popis') . ": ' + PP.Popis " .  //0 - 2
//                "FROM PracovniPomucka PP " .
//                "WHERE NOT EXISTS (SELECT * FROM PrirazenaPracPomucka WHERE PrirazenaPracPomucka.PracovniPomucka = PP.Kod " .
//                " AND PrirazenaPracPomucka.DatumVydani <= " . $database->timestampToString(DateUtils::startOfTheDay(time())) .
//                " AND (PrirazenaPracPomucka.DatumVraceni >= " . $database->timestampToString(DateUtils::startOfTheDay(time())) .
//                "   OR PrirazenaPracPomucka.DatumVraceni is null)) " .
//                " AND PP.Stav = 1 " .
//                " ORDER BY PP.TypPomucky, PP.Popis";

            $sql =
                "SELECT PracovniPomucka.Kod,  +'" . \Ron\ronText('Typ') . ": ' + PracovniPomucka.TypPomucky + ' - " . \Ron\ronText('Popis') . ": ' + PracovniPomucka.Popis " .  //0 - 2
                "FROM PracovniPomucka, TypPomucky " .
                " WHERE (PracovniPomucka.TypPomucky = TypPomucky.Kod) " .
                "AND ((PracovniPomucka.kod = '') " .
                "OR ((PracovniPomucka.Stav=1 OR PracovniPomucka.Stav=2 OR (PracovniPomucka.Stav=3 AND PracovniPomucka.VyrazenoDoData IS NOT NULL AND PracovniPomucka.VyrazenoDoData < " . $database->timestampToString(DateUtils::startOfTheDay(time())) . ")) " .
                "AND ((TypPomucky.Sledovany=0) OR ((TypPomucky.Sledovany=1) " .
                "and not exists(select * from PrirazenaPracPomucka where PracovniPomucka=PracovniPomucka.Kod and ((DatumVraceni IS NULL) OR (DatumVraceni > " . $database->timestampToString(DateUtils::startOfTheDay(time())) . "))) )) ))" .
                " ORDER BY PracovniPomucka.Kod";


            $combo = new ComboBox('pomucka');
            $combo->emptyRow = '';

            $combo->alwaysShowSelected = isset($value) && $value != '';
            $combo->className .= 'app_combo adsfilterforminput_popis';
            $combo->loadData($database, $sql, isset($_GET['pomucka']) ? ($_GET['pomucka']) : 0);
            $combo->showCode = 1;
            $combo->htmlParams = 'data-ron-select2="pomucka_kod" onChange="window.location.href = &quot;' . $this->url . '&amp;pomucka=&quot; + encodeURIComponent(this.value);"';
            $combo->render();

            $combo = new ComboBox('pomucka_kod');
            //$combo->alwaysShowSelected = isset($value) && $value != '';
            $combo->className .= 'app_combo adsfilterforminput_kod';
            $combo->keyIndex = 0;
            $combo->captionIndex = 0;
            $combo->loadData($database, $sql, isset($_GET['pomucka']) ? ($_GET['pomucka']) : 0);
            $combo->htmlParams = 'data-ron-select2="pomucka"  onChange="window.location.href = &quot;' . $this->url . '&amp;pomucka=&quot; + encodeURIComponent(this.value);"';
            $combo->render();

            if (isset($_GET['pomucka'])) {
                $this->showDetailPomucky($database, $_GET['pomucka']);
            }
//            else {
//                $this->showDetailPomucky($database, $combo->selectedOption->key);
//            }

        } else {
            $popisPomucky = DBUtils::getSqlValue($database, "SELECT PracovniPomucka.Popis " .
                "FROM PrirazenaPracPomucka, PracovniPomucka WHERE PrirazenaPracPomucka.PracovniPomucka = PracovniPomucka.Kod " .
                "AND PracovniPomucka.Kod = '" . (isset($_POST['pomucka']) ? $_POST['pomucka'] : $_GET['pomucka']) . "'");

            echo StringUtils::htmlTextConvert($popisPomucky);
        }
    }

    /**
     * @param $database
     * @param $pomucka
     * @throws \Ron\DB\DBException
     */
    private function showDetailPomucky($database, $pomucka)
    {
        //Vložení view do formu
        $pracovnipomucka = PracovniPomucka::nactiPomucku($database,$pomucka);

        Model::addData('pracovnipomucka', $pracovnipomucka);
        Model::addData('detailform',true);

        echo "</td></tr><tr><td colspan='2'> ";

        include('person/view_pracovnipomuckydetail.php');
    }
}

/**
 * Class DoporucenaVelikostPomuckyField
 * @package Ron\Personalistika
 */
class DoporucenaVelikostPomuckyField extends FormField
{
    function __construct($options)
    {
        parent::__construct($options);
        $this->caption = \Ron\ronText('doporucena_velikost');
    }


    /**
     * @param \Ron\DB\Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws \Ron\DB\DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
        $sql =
            "SELECT   VelikostPPomucky.TypVelikosti + '~' + VelikostPPomucky.Velikost AS typVelikosti, VelikostPPomucky.Velikost " .
            'FROM PrirazenaPracPomucka LEFT JOIN PracovniPomucka ON (PrirazenaPracPomucka.PracovniPomucka = PracovniPomucka.Kod) LEFT JOIN ' .
            'TypPomucky ON (PracovniPomucka.TypPomucky = TypPomucky.Kod) LEFT JOIN ' .
            'TypVelikosti ON (TypPomucky.TypVelikosti = TypVelikosti.Kod) LEFT JOIN ' .
            'VelikostPPomucky ON (TypVelikosti.Kod = VelikostPPomucky.TypVelikosti) RIGHT JOIN ' .
            'VelikostPPOsoby ON (VelikostPPOsoby.TypVelikosti = VelikostPPomucky.TypVelikosti AND  VelikostPPOsoby. Velikost = VelikostPPomucky.Velikost) ' .

            "WHERE OsobniCislo = '" . RonPortalSession::osobaOsobniCislo() . "'" .
            " AND PracovniPomucka = '" . (isset($_POST['pomucka']) ? $_POST['pomucka'] : $_GET['pomucka']) . "'" .
            " AND RCOsoby = '" . RonPortalSession::osobaRC() . "'";

        $cursor = $database->getSqlResult($sql);
        $row = $cursor->fetch();
        $_POST['doporucenaVelikostPomucky'] = $row[0];
        echo $row[1];
        echo '<br />';
    }
}

/**
 * Class VydanaVelikostPomuckyField
 * @package Ron\Personalistika
 */
class VydanaVelikostPomuckyField extends FormField
{
    function __construct($options)
    {
        parent::__construct($options);
        $this->caption = \Ron\ronText('vydana_velikost');
    }

    /**
     * @param \Ron\DB\Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws \Ron\DB\DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);

        $sql =
            "SELECT VelikostPPomucky.TypVelikosti+'~'+VelikostPPomucky.Velikost, VelikostPPomucky.Velikost " .
            'FROM PracovniPomucka LEFT JOIN TypPomucky ON (PracovniPomucka.TypPomucky = TypPomucky.Kod) LEFT JOIN ' .
            'TypVelikosti ON (TypPomucky.TypVelikosti = TypVelikosti.Kod) LEFT JOIN VelikostPPomucky ON (TypVelikosti.Kod = VelikostPPomucky.TypVelikosti) ' .
            "WHERE PracovniPomucka.Kod = '" . (isset($_POST['pomucka']) ? $_POST['pomucka'] : $_GET['pomucka']) . "'";

        $cursor = $database->getSqlResult($sql);
        $row = $cursor->fetch();

        if($row[0] != '') {
            $combo = new ComboBox('velikost');
            $combo->alwaysShowSelected = isset($value) && $value != '';
            $combo->className .= 'app_combo adsfilterforminput_kod';
            $combo->loadData($database, $sql, isset($_POST['doporucenaVelikostPomucky']) ? $_POST['doporucenaVelikostPomucky'] : 0);
//        $combo->showCode = 1;
            $combo->render();
        }
    }
}


//Momentálně nepoužito - doporučení Radka U.
class DoporucenaExpiraceForm extends FormField
{
    /**
     * DoporucenaExpiraceForm constructor.
     * @param $options
     */
    function __construct($options)
    {
        parent::__construct($options);
        $this->caption = "<DIV title='" . \Ron\ronText('doporucena_expirace_pomucky') . "'>" . \Ron\ronText('expirace_pomucky');
    }

    /**
     * @param \Ron\DB\Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws \Ron\DB\DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);

        $sql =
            'SELECT ExpiraceRok *12 + ExpiraceMesic ' .

            'FROM Osoba LEFT JOIN PracovniPomer ON (Osoba.RC = PracovniPomer.RC) LEFT JOIN ' .
            'RozdeleniOsob ON (PracovniPomer.RC = RozdeleniOsob.RC and PracovniPomer.datumNastupu = RozdeleniOsob.datumNastupu ' .
            'AND PracovniPomer.osobniCislo = RozdeleniOsob.osobniCislo) LEFT JOIN ' .
            'Rozdeleni ON (Rozdeleni.Kod = RozdeleniOsob.Rozdeleni AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni) ' .
            'LEFT JOIN PomuckyNaPracMiste ON (PomuckyNaPracMiste.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni AND PomuckyNaPracMiste.KodRozdeleni = Rozdeleni.Kod)' .

            'WHERE  RozdeleniOsob.DatumNastupu <= ' . $database->timestampToString(time()) . ' ' .
            'AND (PracovniPomer.DatumUkonceni >= ' . $database->timestampToString(time()). ' ' .
            'OR PracovniPomer.DatumUkonceni IS NULL) AND RozdeleniOsob.PlatnostOd <= ' . $database->timestampToString(time()) . ' ' .
            'AND (RozdeleniOsob.PlatnostDo >= ' . $database->timestampToString(time()) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' . ' ' .
            "AND PracovniPomer.ID = " . $_GET['osoba'] . " AND PracovniPomucka = '" . $_GET['pomucka'] . "'";

        $cursor = $database->getSqlResult($sql);

        $row = $cursor->fetch();

        if($row[0] == 0){
            echo \Ron\ronText('neuvedeno');
        } else {
            echo $row[0];
        }
    }
}

/**
 * Class PracovniPomuckyEditaceDatumuVraceniForm
 * @package Ron\Personalistika
 */
class PracovniPomuckyEditaceDatumuVraceniForm extends RonPortalForm
{

    /**
     * EditaceDatumuVraceniPomuckyForm constructor.
     * @param \Ron\DB\Database $database
     * @param string $osoba
     * @param string $kodPomucky
     * @param \Date $datumnaroku
     * @param int $poradi
     * @param int $typPohybu typ akce s pomuckou : 0 = vraceni prirazene pomucky, 1 = vydani prirazene pomucky, 2 = vydani nove
     * @param null $individualni
     * @throws \Ron\DB\DBException
     * @throws \Exception
     */
    function __construct($database, $osoba, $kodPomucky, $datumnaroku, $poradi, $typPohybu, $individualni = null)
    {
        parent::__construct($database);

        $this->htmlAction = '?menuid=pracovnipomucky&action=pomuckyform';
        $this->stornoAction = '?menuid=pracovnipomucky';

        $this->cssClass .= ' pracovnipomuckyform';

        if ($typPohybu == 2){
            $this->title = \Ron\ronText('vydej_pomucky_zamestnanci');
        } else{
            $typPohybu ? $this->title = \Ron\ronText('formular_vydani_pomucky') : $this->title = \Ron\ronText('formular_vraceni_pomucky');
        }

        $this->sqlTableName = 'PrirazenaPracPomucka';
        $this->dbLogSrcDescription = 'Editace datumu vrácení pomůcky';
        $this->sqlSelectPart =
            'PrirazenaPracPomucka.DatumVydani, PrirazenaPracPomucka.DatumVraceni, ' .   // 0 - 1
             $database->castWideMemo('PrirazenaPracPomucka.Poznamka') . ' as Poznamka, PrirazenaPracPomucka.DatumExpirace, '. // 2 - 3
            'PrirazenaPracPomucka.PracovniPomucka, Individualni, OpakovanyNarok';   //4 - 6

        if (isset($_GET['osoba'])) {
            require_once('adsapp/ppomer.php');
            $pracovniPomer = new PracovniPomer();
            $pracovniPomer->nactiPPPodleId($database,$_GET['osoba']);
            $pracovniPomer->vyberNactenyPP($database,Prava::vratPrava('pomucky_vydej'));
        }
//        $osobaField = new VybratOsobuField('pomucky_vydej',false);
        $osobaField = new OsobaPracovnichPomucekField(\Ron\ronText('Osoba'), 'pomucky_vydej', DateUtils::now(), $pracovniPomer);
        $osobaField->otherHtmlParams =
            'onchange="window.location=&quot;?menuid=pracovnipomucky&action=pomuckyform&formaction=new&individualni=1&vydanipomucky=' . $typPohybu .
            (isset($_GET['pomucka']) ? '&pomucka=' . $_GET['pomucka'] : '') . '&osoba=&quot;+this.value;"';
        $osobaField->otherHtmlParams .= "class='adsfilterforminput_full'";

        $this->addField($osobaField);

        $this->addField(new VybratPracPomuckuField(Prava::vratPrava('pomucky_vydej'), false, $this->htmlAction . '&formaction=new&individualni=1&vydanipomucky=' . $typPohybu));

        if($individualni) {
            $this->addField(new DoporucenaVelikostPomuckyField(null));
            $this->addField(new VydanaVelikostPomuckyField(null));
        }

        //Zobrazení polí s daty a memo
        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_TEXT, FORM_FIELD_CAPTION_TEXT => '&nbsp;']);

        if($individualni) {
            $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_CHECK, FORM_FIELD_SQL_INDEX => 6, FORM_FIELD_SQL_NAME => 'OpakovanyNarok',
                FORM_FIELD_NAME => 'OpakovanyNarok', FORM_FIELD_CAPTION_TEXT => 'opakovany', FORM_FIELD_EDIT => 1]);
        }

        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_SQL_NAME => 'DatumVydani',
            FORM_FIELD_NAME => 'DatumVydani', FORM_FIELD_SQL_INDEX => 0, FORM_FIELD_CAPTION_TEXT => 'datum_vydani',
            FORM_FIELD_EDIT => $typPohybu, 'useDefaultValueToEditNull' => $typPohybu!= 2 ? $typPohybu : $typPohybu,
            FORM_FIELD_DEFAULT_VALUE => StringUtils::format('date', DateUtils::startOfTheDay(time()))]);

        //Editace po domluvě Radka U. s Martinem přesunuta do noveho formu přes odkaz v Browseru

        if($_GET['editexpirace'] == 1) {
            $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_SQL_NAME => 'DatumExpirace',
                FORM_FIELD_NAME => 'DatumExpirace', FORM_FIELD_SQL_INDEX => 3, FORM_FIELD_CAPTION_TEXT => 'datum_expirace', FORM_FIELD_EDIT => $_GET['editexpirace'] == 1]);
        }
        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_SQL_NAME => 'DatumVraceni',
            FORM_FIELD_NAME => 'DatumVraceni', FORM_FIELD_SQL_INDEX => 1, FORM_FIELD_CAPTION_TEXT => 'datum_vraceni', FORM_FIELD_EDIT => !$typPohybu,
            'useDefaultValueToEditNull' => $typPohybu == 0, FORM_FIELD_DEFAULT_VALUE => $typPohybu == 0 ? StringUtils::format('date', DateUtils::startOfTheDay(time())) : null]);

        $this->addField( [ FORM_FIELD_TYPE => FORM_FIELD_TYPE_MEMO, FORM_FIELD_NAME => 'Poznamka', FORM_FIELD_CAPTION_TEXT => 'text_cinnosti',
            FORM_FIELD_EDIT => 1, FORM_FIELD_SQL_INDEX => 2, FORM_FIELD_SQL_NAME => 'Poznamka']);
    }

    /**
     * @return string
     * @throws \Ron\DB\DBException
     */
    function getWhereCondition()
    {
        if (isset($_POST['osoba'])) $osoba = $_POST['osoba'];
        else if (isset($_GET['osoba'])) $osoba = $_GET['osoba'];
        else $osoba = '';

        if(isset($_POST['pomucka'])) $pomucka = $_POST['pomucka'];
        else if (isset($_GET['pomucka'])) $pomucka = $_GET['pomucka'];
        else $pomucka = '';

        if (isset($_POST['datumnaroku'])) $datumnaroku = $_POST['datumnaroku'];
        else if(isset($_GET['datumnaroku'])) $datumnaroku = $_GET['datumnaroku'];
        else $datumnaroku = '';

        if (isset($_POST['poradi'])) $poradi = $_POST['poradi'];
        else if(isset($_GET['poradi'])) $poradi = $_GET['poradi'];
        else $poradi = '';

        $osobaRC = DBUtils::getSqlValue($this->database,"SELECT RC FROM PracovniPomer WHERE ID = '" . $this->database->addSlashes($osoba) . "'");
        if ($osoba != '' && $pomucka != '' && $datumnaroku != '' && $poradi != '') {
            return  " RC = '" . $this->database->addSlashes($osobaRC) . "' AND PracovniPomucka = '" . $pomucka . "'" .
            ' AND DatumNaroku = ' .  $this->database->timestampToString($datumnaroku) .  ' AND Poradi = ' .  (int)$poradi;
        }
        return '';
    }

    function showCommonFields()
    {
        parent::showCommonFields();

        if (isset($_POST['osoba'])) $osoba = $_POST['osoba'];
        else if (isset($_GET['osoba'])) $osoba = $_GET['osoba'];
        else $osoba = '';

        if(isset($_POST['pomucka'])) $pomucka = $_POST['pomucka'];
        else if (isset($_GET['pomucka'])) $pomucka = $_GET['pomucka'];
        else $pomucka = '';

        if (isset($_POST['datumnaroku'])) $datumnaroku = $_POST['datumnaroku'];
        else if(isset($_GET['datumnaroku'])) $datumnaroku = $_GET['datumnaroku'];
        else $datumnaroku = '';

        if (isset($_POST['poradi'])) $poradi = $_POST['poradi'];
        else if(isset($_GET['poradi'])) $poradi = $_GET['poradi'];
        else $poradi = '';

        if (isset($_POST['vydanipomucky'])) $vydanipomucky = $_POST['vydanipomucky'];
        else if(isset($_GET['vydanipomucky'])) $vydanipomucky = $_GET['vydanipomucky'];
        else $vydanipomucky = '';

        if (isset($_POST['editexpirace'])) $editexpirace = $_POST['editexpirace'];
        else if(isset($_GET['editexpirace'])) $editexpirace = $_GET['editexpirace'];
        else $editexpirace = '';

        if (isset($_POST['individualni'])) $individualni = $_POST['individualni'];
        else if(isset($_GET['individualni'])) $individualni = $_GET['individualni'];
        else $individualni = '';

        echo '<input type="hidden" name="osoba" value="' . $osoba . '" />';
        echo '<input type="hidden" name="pomucka" value="' . $pomucka . '" />';
        echo '<input type="hidden" name="datumnaroku" value="' . $datumnaroku . '" />';
        echo '<input type="hidden" name="poradi" value="' . $poradi . '" />';
        echo '<input type="hidden" name="vydanipomucky" value="' . $vydanipomucky . '" />';
        echo '<input type="hidden" name="typpohybu" value="' . $vydanipomucky . '" />';
        echo '<input type="hidden" name="editexpirace" value="' . $editexpirace . '" />';
        echo '<input type="hidden" name="individualni" value="' . $individualni . '" />';
        echo '';
    }


    /*/**
     * @return bool
     * @throws \Ron\DB\DBException
     * @throws \Exception

    function editRecord()
    {
        if (!$this->checkRequiredFields(true)) {return false;}

        /** @var DBRecord $rec
        $rec = $this->createDBRecord();

        foreach ($this->fields as $i => $field) {
            $rec_field = $field->createDBRecordField($this ->database, $this->sqlTableName, false); // $insert = false
            if ($rec_field) {
                $rec->addDBRecordField($rec_field);
            }
        }

        $rec->where = $this->getWhereCondition();

        if (!$rec->loadRecord()) {throw new \Exception('Chyba pri editaci zaznamu - puvodni zaznam nebyl nalezen!');}

        if (!$this->beforeUpdateRecord($rec)) {return false;}

        if (!$rec->updateRecord()) {return false;}

        return $this->afterUpdateRecord($rec);
    }*/

    /**
     * @return DBRecord|\Ron\RonPortal\RonPortalDBRecord
     * @throws \Ron\DB\DBException
     */
    function createDBRecord()
    {
        //readonly:false - neboť se edituje stavající, při vytváření nového se zadává true
        $result =  parent::createDBRecord();

        $velikostPomucky = explode('~', $_POST['velikost']);
        $velikostPomucky[0]; //typ velikosti
        $velikostPomucky[1]; //velikost

        require_once('adsapp/ppomer.php');
        $pracPomer = new PracovniPomer();
        $pracPomer->nactiPPPodleId($this->database, (int)$_POST['osoba']);

        $result->addField('RC', '', true, 15, $pracPomer->rc);
        $result->addField('OsobniCislo', '', true, 15, $pracPomer->osobniCislo);

        $result->addField('DatumNastupu', 'date', true, 15,  $pracPomer->datumNastupu);
        $result->addField('PracovniPomucka', '', true, 10,  $_POST['pomucka']);
        $result->addField('DatumNaroku', 'date', true, 0, time());
        $result->addField('Poradi', 'int', true, 0);
        $result->addField('Individualni', 'int', true, 0, $_POST['individualni']);
        $result->addField('TypVelikostiPP', '', true, 10, $velikostPomucky[0]);
        $result->addField('VelikostPP', '', true, 20, $velikostPomucky[1]);

        return $result;
    }


    /**
     * @param DBRecord $rec
     * @throws \Ron\DB\DBException
     */
    function setDBRecord($rec)
    {
        parent::setDBRecord($rec);
        $rec->where = $this->getWhereCondition();
    }


    /**
     * @param DBRecord $rec
     * @return bool
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function beforeInsertRecord($rec)
    {
        if (isset($_POST['osoba'])) $osoba = $_POST['osoba'];
        else $osoba = '';

        if(isset($_POST['pomucka'])) $pomucka = $_POST['pomucka'];
        else $pomucka = '';

        $zadaneDatumVydani = $rec['DatumVydani'];
        $jizVydaneVDatu = $this->database->stringToTimestamp(DBUtils::getSqlValue($this->database,
                "SELECT max (DatumVydani)
                  FROM PrirazenaPracPomucka ppp LEFT JOIN PracovniPomucka pp ON (ppp.PracovniPomucka = pp.Kod) LEFT JOIN TypPomucky tp ON (pp.TypPomucky = tp.Kod) " .
                 "WHERE Sledovany = 1 AND ppp.PracovniPomucka = '". $pomucka . "'"));
        if($zadaneDatumVydani <  $jizVydaneVDatu){
            throw new \Exception(\Ron\ronText('datum_obsazeno'));
        }

        $datumnaroku = DateUtils::startOfTheDay(time());
        $osobaRC = DBUtils::getSqlValue($this->database,"SELECT RC FROM PracovniPomer WHERE ID = '" . $this->database->addSlashes($osoba) . "'");

        if (($osobaRC == '') || ($pomucka == '')) {
            return false;
        }

        //umožni přiřazení stejné pomucky jedné osobe ve stejný aktuální den vložením noveho čísla poradi
        $rec['Poradi'] = SQLUtils::newSerialNumber($this->database,'PrirazenaPracPomucka','Poradi',
            "RC = '" . $this->database->addSlashes($osobaRC) . "' AND PracovniPomucka = '" . $pomucka . "' AND DatumNaroku = " . $this->database->timestampToString($datumnaroku));

        return true;
    }

    /**
     * @param DBRecord $rec
     * @return bool
     * @throws \Exception
     */
    function beforeUpdateRecord($rec)
    {
        if($_POST['vydanipomucky'] == 0) {
            if ($rec['DatumVraceni'] == '') {
                throw new \Exception(\Ron\ronText('form.Missing_value') . ' ' . \Ron\ronText('datum_vraceni') . '!');
            }

            if($rec['DatumVraceni'] < $rec['DatumVydani']){
                throw new \Exception(\Ron\ronText('spatne_datum_vraceni'));
            }
        }
        return parent::beforeUpdateRecord($rec);
    }

    /**
     * @param DBRecord $rec
     * @return bool
     */
    function afterUpdateRecord($rec)
    {
//        var_dump($rec, $rec->isChanged());
        return parent::afterUpdateRecord($rec);
    }
}