<?php
/** UTF8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 14.6.2016
 * Time: 14:28
 */

namespace Ron\RonPortal;

use Exception;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\DBRecordField;
use Ron\FormField;
use Ron\MessageManager;
use Ron\StringUtils;
use function Ron\ronText;

require_once('adsapp/ronportalform.php');
require_once('adsapp/events.php');

class OsobaSkoleniField extends OsobaField
{
    /** @var PracovniPomer */
    private $pracovniPomer;

    public function __construct($caption, $prava, $datumPlatnostiPP, $pp)
    {
        $this->pracovniPomer = $pp;
        parent::__construct($caption, $prava, $datumPlatnostiPP, false);
    }

    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($insert) {
            parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
        }
        elseif ($this->pracovniPomer) {
            echo '<input type="hidden" name="' . $this->fieldName . '" value="' . $this->pracovniPomer->idPP . '" />';
            echo StringUtils::htmlTextConvert($this->pracovniPomer->jmenoPrijmeniKratke) . "\n";
        }
    }
}

/**
 * Class KurzField
 * @package Ron\RonPortal
 */
class KurzField extends FormField
{

    /** @var Database $database */
    public $database;

    /**
     * KurzField constructor.
     * @param Database $database
     * @param $fieldname
     * @param $edit
     * @param null $options
     */
    public function __construct($database, $fieldname, $edit, $options = null)
    {
        parent::__construct($options);

        $this->database = $database;
        $this->fieldName = $fieldname;
        $this->sqlFieldName = $fieldname;
        $this->editable = $edit;
        $this->maxLength = 10;
        $this->caption = ronText('Skoleni_druh_kurzu');
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    public function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);

        $params = $database->createParameters();

        $sql = "Select CisloSkoleni, Popis From Skoleni WHERE DruhSkoleni = " . $params->addParameter($rowData[1]);
        $sql .= " AND Skoleni.StavSkoleni IN (0,1) ";
//        $sql .= " AND Skoleni.DatumPocatkuSkoleni <= " . $params->addParameter($this->database->timestampToParam($rowData[9]));
//        $sql .= " AND (Skoleni.DatumUkonceniSkoleni  >= " . $params->addParameter($this->database->timestampToParam($rowData[9]));
//        $sql .= " OR Skoleni.DatumUkonceniSkoleni IS NULL )";

        if (true) {
            $combo = new ComboBox('Kurz');
            $combo->emptyRow = '';
            $combo->loadData($database, $sql, "", $params);
            $combo->render();
        } else {
            echo '<input class="app_input" type="text" disabled="disabled" name="Kurz" value="' . $rowData[2] .  '">';
            echo '<input class="app_input" type="hidden" name="Kurz" value="' . $rowData[2] . '">';
        }
    }


    function createDBRecordField($database, $tableName, $insert)
    {
        $field = new DBRecordField($this->sqlFieldName, '', !$this->editable, $this->maxLength);
        if ($this->editable) {
            if (isset($_POST['Kurz']) && $_POST['Kurz'] != '') {
//                $osobaRC = DBUtils::getSqlValue($database, "SELECT RC FROM PracovniPomer WHERE ID = '" . $database ->addSlashes($_POST['Pilot']) . "'");
                $field->setValue($_POST['Kurz']);
            }
        } else {
            $field->setValue("");
        }
        return $field;
    }
}

class SkoleniForm extends RonPortalForm
{
    private $osobaRC;
    private $druhSkoleni;
    private $datumPovinnosti;

    /**
     * SkoleniForm constructor.
     * @param Database $database
     * @param PracovniPomer $pp
     * @param string $druhSkoleni
     * @param int $datumPovinnosti
     */
    public function __construct($database, $pp, $druhSkoleni, $datumPovinnosti)
    {
        parent::__construct($database);

        $this->osobaRC = $pp->rc;
        $this->druhSkoleni = $druhSkoleni;
        $this->datumPovinnosti = $datumPovinnosti;

        $this->htmlAction = '?menuid=skoleni&action=zapisabsolvovani';
        $this->stornoAction = '?menuid=skoleni';

        $this->sqlTableName = 'GenerovaneSkoleni';
        $this->dbLogSrcDescription = StringUtils::fromUTF8('Absolvování školení');

        $this->sqlSelectPart =
            'RC, DruhSkoleni, DatumVznikuPovinnosti, DuvodSkoleni, DatumZapisu, DatumSkoleni, PozadovaneDatumSkoleni, ' . // 0 - 6
            'PozadovanaPlatnostDo, Vlozil, CisloSkoleni, DatumPrihlasky, DatumPoslednihoSkoleni, Prihlasil, Potvrzeni, ' . // 7 - 13
            'Povinne, {fn_CASTWIDEMEMO(Poznamka)}, Kredity, Generovane, PopisSkoleni, CenaZKurzu, CenaIndividualni, CenaCelkova, HodinyIndividualni, PrihlasenoZAplikace'; // 14 - 23

        $this->addField(new OsobaSkoleniField(ronText('Osoba'), 'skoleni_absolvovani', DateUtils::now(), $pp));

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_CAPTION_TEXT => 'Druh_skoleni', FORM_FIELD_EDIT => false, FORM_FIELD_REQUIRED => true,
            FORM_FIELD_NAME => 'DruhSkoleni', FORM_FIELD_SQL_NAME => 'DruhSkoleni', FORM_FIELD_SQL_INDEX => 1, FORM_FIELD_PRIMARY => 1,
            FORM_FIELD_SQL => 'SELECT Kod, Popis FROM DruhSkoleni WHERE Aktivni = 1 AND Elearning = 0 ORDER BY Popis'
        ]);

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_CAPTION_TEXT => 'Skoleni_datum', FORM_FIELD_DEFAULT_VALUE => StringUtils::format('date',DateUtils::today()),
            'useDefaultValueToEditNull' => true, FORM_FIELD_REQUIRED => true, FORM_FIELD_NAME => 'DatumSkoleni', FORM_FIELD_SQL_NAME => 'DatumSkoleni', FORM_FIELD_SQL_INDEX => 5
        ]);

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_CAPTION_TEXT => 'Skoleni_duvod', FORM_FIELD_EDIT => false, FORM_FIELD_DEFAULT_VALUE => '1',
            FORM_FIELD_NAME => 'DuvodSkoleni', FORM_FIELD_SQL_NAME => 'DuvodSkoleni', FORM_FIELD_SQL_INDEX => 3, FORM_FIELD_REQUIRED => true,
            FORM_FIELD_SQL => 'SELECT Kod, Popis FROM S_DuvodSkoleni ORDER BY Popis',
        ]);

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_CAPTION_TEXT => 'Skoleni_potvrzeni', FORM_FIELD_EDIT => true, FORM_FIELD_DEFAULT_VALUE => '3',
            FORM_FIELD_NAME => 'Potvrzeni', FORM_FIELD_SQL_NAME => 'Potvrzeni', FORM_FIELD_REQUIRED => true, //FORM_FIELD_SQL_INDEX => 13, - u vygenerovanych naroku je jine potvrzeni
            FORM_FIELD_SQL => 'SELECT Kod, Popis FROM S_StavPotvrzeni WHERE KOD IN (3,4,7,8) ORDER BY Popis', //Pro JiSz pridano 7,8
        ]);

        $this->addField(new KurzField($database, 'CisloSkoleni', true, null));


        $this->addField(new UserLoginField('Vlozil', false));

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_EDIT, FORM_FIELD_CAPTION_TEXT => 'Popis_skoleni', FORM_FIELD_REQUIRED => false,
            FORM_FIELD_NAME => 'PopisSkoleni', FORM_FIELD_SQL_NAME => 'PopisSkoleni', FORM_FIELD_SQL_INDEX => 18, FORM_FIELD_EDIT => true
        ]);

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_FILE, FORM_FIELD_CAPTION_TEXT => 'Skoleni_dokument', FORM_FIELD_REQUIRED => false,
            FORM_FIELD_NAME => 'dokument', FORM_FIELD_EDIT => true,
        ]);

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_FILENAME, FORM_FIELD_CAPTION_TEXT => 'Skoleni_dokument_popis', FORM_FIELD_REQUIRED => false,
            FORM_FIELD_NAME => 'dokument_popis', FORM_FIELD_EDIT => true,
        ]);

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_MEMO, FORM_FIELD_CAPTION_TEXT => 'Poznamka', FORM_FIELD_REQUIRED => false,
            FORM_FIELD_NAME => 'Poznamka', FORM_FIELD_SQL_NAME => 'Poznamka', FORM_FIELD_SQL_INDEX => 15, FORM_FIELD_EDIT => true
        ]);
        /* Informace o tom, že požadovaná platnost do a požadované datum dalšího školení budou přepočteny agentem. Zatím skryto. Tyto údaje se stejně nezobrazují.
        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_TEXT, FORM_FIELD_TEXT => \Ron\ronText('Skoleni_bude_generovano')
        ]);*/
    }

    function showCommonFields()
    {
        parent::showCommonFields();

        echo '<input type="hidden" name="DatumPovinnosti" value="' . ($this->datumPovinnosti ?: DateUtils::now()) . '"/>';
    }

    function checkRequiredFields($edit)
    {
        if ((is_uploaded_file($_FILES['dokument']['tmp_name'])) && ($_POST['dokument_popis'] == '')) {
            MessageManager::addErrorMessage(ronText('Skoleni_dokument_chybi_popis'), 'dokument_popis');
            return false;
        }
        return parent::checkRequiredFields($edit);
    }


    function createDBRecord()
    {
        $rec = parent::createDBRecord();

        $rec->addField('RC', '', true, 15, $this->osobaRC);
        $rec->addField('DatumVznikuPovinnosti', 'timestamp', true, 0, $this->datumPovinnosti);
        $rec->addField('DatumZapisu', 'timestamp', true, 0, DateUtils::today());
        $rec->addField('Generovane', 'int', true, 0, 0); //vlozeno rucne
        $rec->addField('PrihlasenoZAplikace', 'int', true, 0, 1); // 1 = portal

        return $rec;
    }

    /**
     * @return SqlStatement
     */
    function getWhereCondition()
    {
        $params = $this->database->createParameters();

        return new SqlStatement(
            " RC = " . $params->addParameter($this->osobaRC) .
            " AND DruhSkoleni = " . $params->addParameter($this->druhSkoleni) .
            " AND DatumVznikuPovinnosti = " . $params->addParameter($this->database->timestampToParam($this->datumPovinnosti)), $params);
    }

    function setDBRecord($rec)
    {
        parent::setDBRecord($rec);

        $where = $this->getWhereCondition();
        $rec->setWhere($where->sqlText, $where->parameters);
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return mixed
     * @throws Exception
     */
    function beforeInsertRecord($rec)
    {
        //vkladat jen pro nevygenerovane naroky pro druh skoleni
        if (SkoleniController::existujeNavrhTypuSkoleni($this->database, $rec['RC'], $rec['DruhSkoleni'])) {
            throw new Exception(ronText('Skoleni_existuje_navrh'));
        }        

        return $rec->executeXmlEvents(RonPortalXMLEvents::SKOLENI_ABSOLVOVANI_BEFORE, 'insert');
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return mixed
     */
    function beforeUpdateRecord($rec)
    {
        return $rec->executeXmlEvents(RonPortalXMLEvents::SKOLENI_ABSOLVOVANI_BEFORE, 'update');
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return mixed
     */
    function afterInsertRecord($rec)
    {
        $this->uploadDokument($rec);

        return $rec->executeXmlEvents(RonPortalXMLEvents::SKOLENI_ABSOLVOVANI_AFTER, 'insert');
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return mixed
     */
    function afterUpdateRecord($rec)
    {
        $this->uploadDokument($rec);

        return $rec->executeXmlEvents(RonPortalXMLEvents::SKOLENI_ABSOLVOVANI_AFTER, 'update');
    }

    private function uploadDokument($rec)
    {
        if (is_uploaded_file($_FILES['dokument']['tmp_name'])) {
            require_once('adsapp/dokumentyutils.php');
            try {
                $idDokumentu = DokumentyUtils::vlozDokument($this->database, 'dokument', RonPortalSettings::getIntValue('skoleni_typ_dokumentu'),
                    $_POST['dokument_popis'], $this->dbLogSrcDescription);

                $recDokumentOsoby = new RonPortalDBRecord($this->database, 'DokumentyOsoby', $this->dbLogSrcDescription);
                $recDokumentOsoby->addField('RC', '', true, 15, $rec['RC']);
                $recDokumentOsoby->addField('id', 'int', true, 0, $idDokumentu);
                $recDokumentOsoby->addField('GSk_RC', '', false, 15, $rec['RC']);
                $recDokumentOsoby->addField('GSk_DruhSkoleni', '', false, 10, $rec['DruhSkoleni']);
                $recDokumentOsoby->addField('GSk_Datum', 'timestamp', true, 0, $rec['DatumVznikuPovinnosti']);

                $recDokumentOsoby->insertRecord();
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e);
            }
        }
    }
}