<?php
/** Roman.Szablatura 15.3.2016 */

namespace Ron\Personalistika;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;
use Ron\RonPortal\PracovniPomer;

/**
 * Class ZdravotniProhlidka
 * @package Ron\Personalistika
 */
class LekarskaProhlidka
{
    public $rc;
    public $jmeno;
    public $prijmeni;
    public $kodLekarskeProhlidky;
    public $popisLekarskeProhlidky;
    public $datumVznikuPovinnosti;
    public $duvodProhlidky;
    public $datumAbsolvovani;
    public $nazevDokumentu;
    public $vysledekProhlidky;
    public $platnostDo;
    public $datumPosledniProhlidky;
    public $povinna;
    public $vysledek;
    public $datumProhlidkyRezervovany;
    public $casProhlidkyRezervovany;
    public $doplnujiciTextPozvanky;
    public $historieKorespondence;
    public $osetrujiciLekar;
    public $poznamka;
    public $idOsoby;

    /* @var ZdravotniProhlidkaDokumenty[] $dokumenty */
    public $dokumenty;

    /**
     * @param Database $database
     * @param PracovniPomer $pracovniPomer
     * @param string $kodLP
     * @param int $datumVzniku
     * @return LekarskaProhlidka
     * @throws Exception
     * @throws DBException
     */
    public static function nactiProhlidku($database, $pracovniPomer, $kodLP, $datumVzniku)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT KodLekarskeProhlidky, DatumVznikuPovinnosti, S_DuvodProhlidky.Popis, DatumAbsolvovani, S_VyhodnoceniProhlidky.Popis, PlatnostDo, ' . //0 - 5
            'DatumPosledniProhlidky, Povinna, Vysledek, DatumProhlidkyRezervovany, CasProhlidkyRezervovany, ' . $database->castWideMemo('DoplnujiciTextPozvanky') . ', ' . //6 - 11
            $database->castWideMemo('HistorieKorespondence') . ",  COALESCE(OsetrujiciLekar.JmenoLekare,'') {fn_concat()} ' ' {fn_concat()} COALESCE(OsetrujiciLekar.Prijmeni,''), " . // 12,13
            $database->castWideMemo('GenerovaneLekProhlidky.Poznamka') . ", " . //14
            'Osoba.Jmeno, Osoba.Prijmeni, LekarskaProhlidka.Popis ' . //15 - 17
            'FROM LekarskaProhlidka ' .
            '  JOIN GenerovaneLekProhlidky ON (GenerovaneLekProhlidky.KodLekarskeProhlidky = LekarskaProhlidka.Kod) ' .
            '  JOIN Osoba ON (GenerovaneLekProhlidky.RC = Osoba.RC) ' .
            'LEFT OUTER JOIN OsetrujiciLekar ON (GenerovaneLekProhlidky.OsetrujiciLekar = OsetrujiciLekar.Kod)
            LEFT OUTER JOIN S_DuvodProhlidky ON (GenerovaneLekProhlidky.DuvodProhlidky = S_DuvodProhlidky.Kod)
            LEFT OUTER JOIN S_VyhodnoceniProhlidky ON (GenerovaneLekProhlidky.Vysledek = S_VyhodnoceniProhlidky.Kod) ' .
            "WHERE GenerovaneLekProhlidky.RC = ". $params->addParameter($pracovniPomer->rc)  .
            " AND LekarskaProhlidka.Kod = " . $params->addParameter($kodLP)  .
            ' AND GenerovaneLekProhlidky.DatumVznikuPovinnosti = ' . $params->addParameter($database->timestampToParam($datumVzniku)) ;

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql, $params);

        if(!$row = $cursor->fetch()){
            throw new Exception('prohlidka_neexistuje');
        }

        $prohlidka = new LekarskaProhlidka();
        $prohlidka->rc = $pracovniPomer->rc;
        $prohlidka->idOsoby = $pracovniPomer->idPP;
        $prohlidka->nactiProhlidkuZRadku($database, $row);
        $prohlidka->nactiDokumentyProhlidky($database, $pracovniPomer->rc, $kodLP, $datumVzniku);
        return $prohlidka;
    }

    /**
     * @param Database $database
     * @param $row
     */
    public function nactiProhlidkuZRadku($database ,$row)
    {
        $this->kodLekarskeProhlidky = $row[0];
        $this->datumVznikuPovinnosti = $database->stringToTimestamp($row[1]);
        $this->duvodProhlidky = $row[2];
        $this->datumAbsolvovani = $database->stringToTimestamp($row[3]);
        $this->vysledekProhlidky = $row[4];
        $this->platnostDo = $database->stringToTimestamp($row[5]);
        $this->datumPosledniProhlidky = $database->stringToTimestamp($row[6]);
        $this->povinna = $row[7];
        $this->vysledek = $row[8];
        $this->datumProhlidkyRezervovany = $database->stringToTimestamp($row[9]);
        $this->casProhlidkyRezervovany = $database->stringToTimestamp($row[10]);
        $this->doplnujiciTextPozvanky = $row[11];
        $this->historieKorespondence = $row[12];
        $this->osetrujiciLekar = $row[13];
        $this->poznamka = $row[14];
        $this->jmeno = $row[15];
        $this->prijmeni = $row[16];
        $this->popisLekarskeProhlidky = $row[17];
    }

    /**
     * @param $database
     * @param string $rc
     * @param string $kodLP
     * @param int $datumVzniku
     * @throws DBException
     */
    public function nactiDokumentyProhlidky($database, $rc, $kodLP, $datumVzniku)
    {
        $this->dokumenty = ZdravotniProhlidkaDokumenty::nactiZdravotniDokumenty($database,$rc, $kodLP, $datumVzniku);
    }
}

/**
 * Class ZdravotniProhlidkaDokumenty
 * @package Ron\Personalistika
 */
class ZdravotniProhlidkaDokumenty
{
    public $idDokumentu;
    public $popisDokumentu;
    public $typDokumentu;
    public $datumVlozeniDokumentu;
    public $nazevDokumentu;
    public $poznamkaDokumentu;


    /**
     * @param Database $database
     * @param string $rc
     * @param string $kodLP
     * @param int $datumVzniku
     * @return ZdravotniProhlidkaDokumenty[]
     * @throws DBException
     */
    public static function nactiZdravotniDokumenty($database, $rc, $kodLP, $datumVzniku)
    {
        $params = $database->createParameters();

        $sql = 'SELECT DokumentyOsoby.id, popis, typ, datum, jmenoSouboru, ' . $database->castWideMemo('poznamka') .   //0 - 5
            ' FROM Dokument JOIN DokumentyOsoby ON (Dokument.id=DokumentyOsoby.id) '.
            " WHERE DokumentyOsoby.GLP_RC = " . $params->addParameter($rc) .
            "  AND DokumentyOsoby.GLP_KodLP= " . $params->addParameter($kodLP) .
            '  AND DokumentyOsoby.GLP_Datum = ' . $params->addParameter($database->timestampToParam($datumVzniku)) .
            "  AND EXISTS (select * from TypDokumentu WHERE TypDokumentu.Kod = Dokument.typ AND TypDokumentu.KategorieDokumentu= 3) " .
            ' ORDER BY Dokument.id DESC, Dokument.datum  DESC';


        MessageManager::addDebugLog($sql);

        $cursor = $database ->getSqlResult($sql,$params);

        $seznamDokumentu = [];
        while($row = $cursor->fetch())
        {
            $zdravotniDokumenty = new ZdravotniProhlidkaDokumenty();
            $zdravotniDokumenty->nactiZdravotniDokumentyZRadku($database, $row);
            $seznamDokumentu[] = $zdravotniDokumenty;
        }
        return $seznamDokumentu;
    }

    /**
     * @param Database $database
     * @param $row
     */
    private function nactiZdravotniDokumentyZRadku($database, $row)
    {
        $this->idDokumentu = $row[0];
        $this->popisDokumentu = $row[1];
        $this->typDokumentu = $row[2];
        $this->datumVlozeniDokumentu = $database->stringToTimestamp($row[3]);
        $this->nazevDokumentu = $row[4];
        $this->poznamkaDokumentu = $row[5];
    }
}




