<?php
/**
 * UTF-8
 * roman.szablatura, 11.03.2016
 */

namespace Ron\Personalistika;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Http\HtmlHeaders;
use Ron\HttpUtils;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\RonPortal\DokumentOutputObject;
use Ron\RonPortal\DokumentUploadForm;
use Ron\RonPortal\DokumentUploadOutput;
use Ron\RonPortal\DokumentyUtils;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\ReportOutputObject;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\User;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\RonPortal\RonPortalSettings;
use function Ron\ronText;

require_once('person/lekarskaprohlidka.php');
require_once('adsapp/dokumentyutils.php');


/**
 * Class LekarskeProhlidkyController
 * @package Ron\Personalistika
 */
class LekarskeProhlidkyController extends JsonController
{
    const POVOLIT_LEKARSKE_PROHLIDKY = 'povolitprohlidky';

    /** @var Database  */
    private $database;

    /** @var  PravaNaOsoby */
    private $pravaProhlizet;

    /** @var  PravaNaOsoby */
    private $pravaAbsolvovat;

    /** @var  PravaNaOsoby */
    private $pravaZapis;

    /**
     * LekarskeProhlidkyController constructor.
     * @param Database $database
     * @throws Exception
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        Prava::nastavObdobiProPrava(time(),time());
        Prava::nactiPravaUzivatele($this->database,User::getRC(), 'lekarske_prohl');

        $this->pravaProhlizet = Prava::vratPrava('lekarske_prohl');
        $this->pravaAbsolvovat = Prava::vratPrava('lekarske_absolv');
        $this->pravaZapis = Prava::vratPrava('lekarske_edit');

    }

    /**
     * @param $absolvovat
     * @return Redirect|LekarskaProhlidkaEditaceAbsolvovaniForm
     * @throws Exception
     * @throws DBException
     */
    private function lekarskaProhlidkaForm($absolvovat)
    {
        require_once('adsapp/ppomer.php');
        $pracPomer = PracovniPomer::nactiPP($this->database);

        if ((!($this->pravaProhlizet->lzeZobrazitRC($pracPomer->rc)
                || $this->pravaAbsolvovat->lzeZobrazitRC($pracPomer->rc)
                || $this->pravaZapis->lzeZobrazitRC($pracPomer->rc)))) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        $action = (isset($_POST['formaction']) ? $_POST['formaction'] : (isset($_GET['formaction']) ? $_GET['formaction'] : ''));

        require_once ('person/f_lekarskeprohlidky.php');
        $f = new LekarskaProhlidkaEditaceAbsolvovaniForm($this->database, $action, $this->pravaZapis, $this->pravaAbsolvovat, $absolvovat, $pracPomer);
        if($f->handleAction()){
            return new Redirect(null, '?menuid=lekarskeprohlidky');
        }
        return $f;
    }

    /**
     * @return BaseViewObject
     * @throws Exception
     */
    public function  zdravotniProhlidkaDetailView()
    {
        if (isset($_GET['osoba'])) $osoba = $_GET['osoba'];
        else $osoba = '';

        require_once('adsapp/ppomer.php');
        $pracPomer = new PracovniPomer();
        $pracPomer->nactiPPPodleId($this->database, $osoba);
        $osobaRC = $pracPomer->rc;

        if (!$this->pravaProhlizet->lzeZobrazitRC($osobaRC,time(),time())) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        $zdravotniprohlidka = LekarskaProhlidka::nactiProhlidku($this->database, $pracPomer, $_GET['kod'], $_GET['vznik']);

        Model::addData('zdravotniprohlidka', $zdravotniprohlidka);
        Model::addData('pravaProhlizet', $this->pravaProhlizet);
        Model::addData('database', $this->database);

        return new BaseViewObject('person/view_lekarskaprohlidkadetail.php');
    }

    /**
     * @param int $idOsoby
     * @param int $id ID dokumentu
     * @return DokumentOutputObject
     * @throws Exception
     * @throws DBException
     */
    private function downloadDocument($idOsoby, $id)
    {
        $params = $this->database->createParameters();
        $osobaRC = DBUtils::getSqlValue($this->database,"SELECT RC FROM PracovniPomer WHERE ID = " . $params->addParameter($idOsoby), $params);

        if (!$this->pravaProhlizet->lzeZobrazitRC($osobaRC, time(), time())) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        $params = $this->database->createParameters();
        $sql = 'SELECT Dokument.ID ' .
            'FROM GenerovaneLekProhlidky ' .
            'JOIN DokumentyOsoby ON  (GenerovaneLekProhlidky.RC = DokumentyOsoby.GLP_RC
                AND  GenerovaneLekProhlidky.KodLekarskeProhlidky = DokumentyOsoby.GLP_KodLP
                AND  GenerovaneLekProhlidky.DatumVznikuPovinnosti = DokumentyOsoby.GLP_Datum)
            JOIN Dokument ON (DokumentyOsoby.id = Dokument.id)
            JOIN TypDokumentu ON (Dokument.typ = TypDokumentu.kod) ' .
            'WHERE DokumentyOsoby.ID = ' . $params->addParameter((int)$id) .
            ' AND GenerovaneLekProhlidky.RC = ' . $params->addParameter($osobaRC) .
            ' AND ( Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1 ))';

        MessageManager::addDebugLog($sql);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if (!$row){
            throw new Exception(ronText('Nemate_pravo'));
        }

        require_once('adsapp/dokumentyutils.php');
        return new DokumentOutputObject($this->database,$id);
    }

    /**
     * @param int $osoba
     * @param string $kod kod lekarske prohlidky
     * @param int $vznik datum vzniku povinnosti absolvovat lek. prohlidku
     * @return DokumentUploadOutput
     */
    private function uploadDokument($osoba, $kod, $vznik)
    {
        $result = new DokumentUploadOutput();

        try {
            $params = $this->database->createParameters();
            $osobaRC = DBUtils::getSqlValue($this->database,"SELECT RC FROM PracovniPomer WHERE ID = " . $params->addParameter($osoba), $params);

            if (!$this->pravaProhlizet->lzeZobrazitRC($osobaRC)) {
                RonPortalUtils::neniPravoException();
            }

            $id = DokumentyUtils::vlozDokument($this ->database, 'soubor', RonPortalSettings::getIntValue('lekprohl_typdokumentu'), $_POST['popis'], 'Lékařský dokument');

            // Vloží zaznam do vazební tabulky
            require_once('adsapp/ronportaldbrecord.php');
            $rec = new RonPortalDBRecord($this ->database, 'DokumentyOsoby');
            $rec->logDescription = 'Lekarske Prohlidky';
            $rec ->addField('id', 'int', false, 0, $id);
            $rec ->addField('rc', '', false, 15, $osobaRC);
            $rec ->addField('GLP_RC', '', false, 15, $osobaRC);
            $rec ->addField('GLP_KodLP', '', false, 10, $kod);
            $rec ->addField('GLP_Datum', 'datetime', false, 0, $vznik);
            $rec ->insertRecord();
        } catch (Exception $e) {
            $result ->setError($e ->getMessage());
        }
        return $result;
    }


    /**
     * @param int $idOsoby
     * @param string $kodLP kod lekarske prohlidky
     * @param int $datumPovinnosti datum vzniku povinnosti absolvovat lek. prohlidku
     * @return ReportOutputObject
     * @throws Exception
     * @throws DBException
     */
    private function report($idOsoby, $kodLP, $datumPovinnosti)
    {
        require_once('adsapp/reportserver.php');
        require_once('adsapp/ppomer.php');

        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $idOsoby);

        if ((!$this->pravaProhlizet->lzeZobrazitRC($pp->rc))) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        $result = new ReportOutputObject();
        $result ->loadReportFromGet($this ->database, 'LekarskeProhlidky',false, 'LekarskeProhlidky');
        $result ->nastavPP($pp);
        $result ->stringParams = ['kodLP' => $kodLP];
        $result ->dateParams = ['datumPovinnosti' => $datumPovinnosti];

        return $result;
    }

    /**
     * @param JsonOutputObject $output
     * @throws DBException
     */
    private function mojeProhlidky($output)
    {
        require_once('ron/httputils.php');

        $html =
            '<table class="dashboardTable" data-ron-href="?menuid=lekarskeprohlidky">' .
            '<tr><th>' . ronText('datum_vzniku_povinnosti') . '</th><th>' . ronText('popis_zdravotni_prohlidky') . '</th><th></th></tr>';

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT (SELECT MAX(ID) FROM PracovniPomer WHERE PracovniPomer.RC = GenerovaneLekProhlidky.RC ), ' .  // 0
            '  GenerovaneLekProhlidky.KodLekarskeProhlidky, GenerovaneLekProhlidky.DatumVznikuPovinnosti, LekarskaProhlidka.Popis ' . // 1..3
            ' FROM GenerovaneLekProhlidky JOIN LekarskaProhlidka ON (LekarskaProhlidka.Kod = GenerovaneLekProhlidky.KodLekarskeProhlidky)' .
            ' WHERE GenerovaneLekProhlidky.RC = ' . $params ->addParameter(User::getRC()) .
            '  AND GenerovaneLekProhlidky.DatumAbsolvovani IS NULL' .
            ' ORDER BY DatumVznikuPovinnosti';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $html .=
                '<tr>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[2])) . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($row[3]) . '</td>' .
                "<td><a href='?menuid=lekarskeprohlidky&action=detail&osoba={$row[0]}&kod=" . HttpUtils::urlEncode($row[1]) . "&vznik=" . $this->database->stringToTimestamp($row[2]) . "'>" . ronText('Podrobnosti').'</a></td>' .
                '</tr>';
        }

        $html .= '</table>';

        $output ->addText('html', $html);
    }

    /**
     * @param JsonOutputObject $output
     * @throws DBException
     */
    private function prohlidkyPodrizenych($output)
    {
        require_once('ron/httputils.php');

        $html =
            '<table class="dashboardTable" data-ron-href="?menuid=lekarskeprohlidky">' .
            '<tr><th>' . ronText('Zamestnanec') . '</th><th>' . ronText('datum_vzniku_povinnosti') . '</th><th>' . ronText('popis_zdravotni_prohlidky') . '</th><th></th></tr>';

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT (SELECT MAX(ID) FROM PracovniPomer WHERE PracovniPomer.RC = GenerovaneLekProhlidky.RC ), ' .  // 0
            '  GenerovaneLekProhlidky.KodLekarskeProhlidky, GenerovaneLekProhlidky.DatumVznikuPovinnosti, LekarskaProhlidka.Popis, ' . // 1..3
            ' Osoba.Prijmeni, Osoba.Jmeno ' . // 4..5
            ' FROM GenerovaneLekProhlidky JOIN LekarskaProhlidka ON (LekarskaProhlidka.Kod = GenerovaneLekProhlidky.KodLekarskeProhlidky)' .
            '   JOIN Osoba ON (Osoba.RC = GenerovaneLekProhlidky.RC)' .
            ' WHERE GenerovaneLekProhlidky.DatumAbsolvovani IS NULL' .


            '  AND EXISTS(SELECT PracovniPomer.RC FROM PracovniPomer ' .
            '             WHERE PracovniPomer.RC = GenerovaneLekProhlidky.RC ' .
            '               AND PracovniPomer.DatumNastupu <= ' . $params->addParameter($this->database->dateToParam(DateUtils::today())) .
            '               AND (PracovniPomer.DatumUkonceni >= ' . $params->addParameter($this->database->dateToParam(DateUtils::today())) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
            '               AND ' .  $this->pravaProhlizet->sqlFiltr('PracovniPomer', DateUtils::today(), DateUtils::today()) .
            '            )' .

            ' ORDER BY DatumVznikuPovinnosti, Osoba.Prijmeni, Osoba.Jmeno';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $html .=
                '<tr>' .
                '<td>' . StringUtils::htmlTextConvert($row[4]) . ' ' . StringUtils::htmlTextConvert($row[5]) . '</td>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[2])) . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($row[3]) . '</td>' .
                "<td><a href='?menuid=lekarskeprohlidky&action=detail&osoba={$row[0]}&kod=" . HttpUtils::urlEncode($row[1]) . "&vznik=" . $this->database->stringToTimestamp($row[2]) . "'>" . ronText('Podrobnosti').'</a></td>' .
                '</tr>';
        }

        $html .= '</table>';

        $output ->addText('html', $html);
    }

    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws DBException
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'mojeprohlidky' :
                $this ->mojeProhlidky($output);
                break;
            case 'prohlidkypodrizenych' :
                $this ->prohlidkyPodrizenych($output);
                break;
        }
    }


    /**
     * @param string $action
     * @return BaseViewObject|\Ron\MVC\InternalRedirect|\Ron\MVC\OutputObject|Redirect|\Ron\MVC\ViewObject|LekarskaProhlidkaEditaceAbsolvovaniForm|LekarskeProhlidkyBrowser|DokumentOutputObject|DokumentUploadOutput|ReportOutputObject|string
     * @throws Exception
     * @throws DBException
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(self::POVOLIT_LEKARSKE_PROHLIDKY)){
            throw new Exception(ronText('Nemate_pravo'));
        }

        if(!($this->pravaProhlizet->jsouNejakePrava() || ($this->pravaAbsolvovat->jsouNejakePrava()) || ($this->pravaZapis->jsouNejakePrava()))){
            throw new Exception(ronText('Nemate_pravo'));
        }

        HtmlHeaders::getInstance()->addCss('./css/person.css');
        HtmlHeaders::getInstance()->addScript('script/personscripts.js');
        TextManager::getInstance()->loadStrings('person.ini');

        switch($_GET['action']){
            // json akce
            case 'mojeprohlidky' :
            case 'prohlidkypodrizenych' :
                return parent::handleAction($action);

            case 'lekarskaprohlidkaform':
                return $this->lekarskaProhlidkaForm(false);
            case 'absolvovat':
                return $this->lekarskaProhlidkaForm(true);
            case 'detail':
                return $this->zdravotniProhlidkaDetailView();
            case 'download':
                return $this ->downloadDocument($_GET['osoba'],$_GET['dokument']);
            case 'dokumentupload' :
                return $this ->uploadDokument($_POST['idosoby'],$_POST['kod'],$_POST['vznik']);
            case 'report' :
                return $this ->report($_GET['osoba'], $_GET['kod'], $_GET['vznik']);

            default:
                require_once('person/b_lekarskeprohlidky.php');
                $b = new LekarskeProhlidkyBrowser($this->database);
                $b->load();
                return $b;
        }
    }
}

/**
 * Class NovyDokumentLekarskaProhlidkaForm
 * @package Ron\Personalistika
 */
class NovyDokumentLekarskaProhlidkaForm extends DokumentUploadForm
{
    function __construct($database, $osoba, $kod, $vznik )
    {
        parent::__construct($database, '?menuid=lekarskeprohlidky&action=dokumentupload');

        $this->typDokumentu = RonPortalSettings::getIntValue('lekprohl_typdokumentu');

        $this->hiddenFields = [
            'dokumentlekarskeprohlidky' => '1',
            'idosoby' => $osoba,
            'kod' => $kod,
            'vznik' => $vznik,
        ];
    }
}
