<?php

namespace Ron\Personalistika;

use Ron\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use function Ron\ronText;

/**
 * Class OsobniCisloOsobyField
 * @package Ron\Personalistika
 */
class OsobniCisloOsobyField extends BrowserField
{

    private $rcFieldIndex;

    /** Pro personalni agendy doplni sloupec s osobnimi cisly platnych pracovnich pomeru osoby.
     * OsobniCisloField constructor.
     * @param Database $database
     * @param int $rcFieldIndex index RC pole v dotazu
     * @param null|array $options
     */
    public function __construct($database, $rcFieldIndex, $options = null)
    {
        parent::__construct($database, $options);
        $this->header = ronText('Osobni_cislo');
        $this->ordering = 1;

        $this->rcFieldIndex = $rcFieldIndex;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws DBException
     */
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $params = $this->database->createParameters();

        $sql =
            'SELECT OsobniCislo ' .
            ' FROM PracovniPomer ' .
            ' WHERE  RC = ' . $params->addParameter($rowData[$this->rcFieldIndex]) . ' '.
            ' AND DatumNastupu <=' . $params->addParameter($this->database ->dateToParam(DateUtils::today())) .
            ' AND (DatumUkonceni >= ' . $params->addParameter($this->database ->dateToParam(DateUtils::today())) . ' OR DatumUkonceni IS NULL) ' .
            'ORDER BY DatumNastupu';

        $cursor = $this->database->getSqlResult($sql, $params);

        $result = '';

        /** @noinspection PhpAssignmentInConditionInspection */
        while ($row = $cursor->fetch()) {
            if ($result) {
                $result .= ', ';
            }
            $result .=  $row[0]  ;
        }
        return  $result;
    }
}
