<?php
/**
 * UTF-8
 * Roman.Szablatura, 18.2.2016
 */

namespace Ron\Personalistika;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\Http\HtmlHeaders;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\InternalRedirect;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\RonPortal\NeniPravoException;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\ReportOutputObject;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\Rozdeleni;
use Ron\RonPortal\User;
use Ron\StringUtils;
use Ron\TextManager;
use function Ron\ronText;

require_once ('person/pracovnipomucka.php');
require_once ('ron/httputils.php');


/**
 * Class PracovniPomuckyController
 * @package Ron\Personalistika
 */
class PracovniPomuckyController extends HttpController
{
    /** @var Database */
    private $database;

    /** @var PravaNaOsoby */
    private $pravaProhlizet;

    /** @var PravaNaOsoby */
    private $pravaZapisVydaniVraceni;

    private $pravaPotvrzeniPrevzeti;

    /**
     * PracovniPomuckyController constructor.
     * @param Database $database
     * @throws Exception
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        Prava::nastavObdobiProPrava(time(),time());
        $this->pravaProhlizet = Prava::nactiPravaUzivatele($this->database, User::getRC(), 'pomucky_prohl');
        $this->pravaZapisVydaniVraceni = Prava::nactiPravaUzivatele($this->database, User::getRC(), 'pomucky_vydej');
        $this->pravaPotvrzeniPrevzeti = Prava::nactiPravaUzivatele($this->database, User::getRC(), 'pomucky_potvrzeni');
    }

    /**
     * @param $record
     * @return BaseViewObject BaseViewObject
     * @throws Exception
     */
    private function pracovniPomuckyDetailView($record)
    {
        $pracovnipomucka = PracovniPomucka::nactiPomucku($this->database,$record);

        Model::addData('pracovnipomucka', $pracovnipomucka);

        return new BaseViewObject('person/view_pracovnipomuckydetail.php');
    }

    /**
     * @param int $typPohybu typ akce s pomuckou : 0 = vraceni prirazene pomucky, 1 = vydani prirazene pomucky, 2 = vydani nove
     * @param $individualni
     * @return Redirect|PracovniPomuckyEditaceDatumuVraceniForm
     * @throws Exception
     */
    private function pracovniPomuckyForm($typPohybu, $individualni)
    {
        $pracPomer = null;

        $osoba = isset($_POST['osoba']) ? $_POST['osoba'] : (isset($_GET['osoba']) ? $_GET['osoba'] : '');
        if ($osoba != '') {
            require_once('adsapp/ppomer.php');
            $pracPomer = new PracovniPomer();
            $pracPomer->nactiPPPodleId($this->database, $osoba);
        }

        if ($pracPomer instanceof PracovniPomer && !$this->pravaZapisVydaniVraceni->lzeZobrazitPP($pracPomer->rc)) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        require_once ('person/f_pracovnipomucky.php');

        $f = new PracovniPomuckyEditaceDatumuVraceniForm($this->database, $_GET['osoba'], $_GET['pomucka'], $_GET['datumnaroku'], $_GET['poradi'], $typPohybu, $individualni);

        if($f->handleAction()){
            return new Redirect(null, '?menuid=pracovnipomucky');
        }

        return $f;
    }

    /**
     * @param $osoba
     * @param $pomucka
     * @param $datumnaroku
     * @return Redirect|EditaceExpiracePomuckyForm
     * @throws Exception
     */
    private function editaceExpirace($osoba, $pomucka, $datumnaroku)
    {
        if (isset($_GET['osoba']) && (!$this->pravaZapisVydaniVraceni->lzeZobrazitPP($_GET['osoba']))) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        require_once ('person/f_editaceexpiracepomucky.php');

        $f = new EditaceExpiracePomuckyForm($this->database);

        if($f->handleAction()){
            return new Redirect(null, '?menuid=pracovnipomucky');
        }
        return $f;
    }

    /**
     * @param $osoba
     * @param $pomucka
     * @param $datumnaroku
     * @param $poradi
     * @return Redirect
     * @throws Exception
     */
    private function potvrzeniPrevzeti($osoba, $pomucka, $datumnaroku, $poradi)
    {
        $osobaRC = DBUtils::getSqlValue($this->database, 'SELECT RC FROM PracovniPomer WHERE ID = '. (int)$osoba);


        if (!$this->pravaPotvrzeniPrevzeti->lzeZobrazitRC($osobaRC)) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        require_once('adsapp/ronportaldbrecord.php');

        $recPotvrzeniPrevzeti = new RonPortalDBRecord($this->database, 'PrirazenaPracPomucka');
        $recPotvrzeniPrevzeti->logDescription = StringUtils::fromUTF8('Potvrzeni převzetí pomůcky pracovníkem');

        $recPotvrzeniPrevzeti->addField('Update_UserName', 'string', false, 25, User::getLoginName());
        $recPotvrzeniPrevzeti->addField('Update_TimeStamp', 'timestamp', false, 0,time());
        $recPotvrzeniPrevzeti->addField('PotvrzeniPrevzetiPopis', 'string', false, 100, StringUtils::fromUTF8('Portál: ') .  User::getLoginName() . ' (' . User::getUserName() . ')');
        $recPotvrzeniPrevzeti->addField('PotvrzeniPrevzetiDatum', 'timestamp', false, 0,time());

        $params = $this->database->createParameters();
        $recPotvrzeniPrevzeti->setWhere("  RC = " . $params->addParameter($osobaRC) .
            "  AND PracovniPomucka = " . $params->addParameter($pomucka) .
            "  AND DatumNaroku = " . $params->addParameter($this->database->timestampToParam($datumnaroku))  .
            "  AND Poradi = " . $params->addParameter($poradi), $params) ;

        if(!$recPotvrzeniPrevzeti->loadAndUpdateRecord()){
            throw new Exception(StringUtils::fromUTF8('Chyba při zápisu potvrzení.'));
        }

        return new Redirect(null,'?menuid=pracovnipomucky');
    }

    /**
     * @return ReportOutputObject
     * @throws Exception
     */
    private function report()
    {
        $osoba = $_GET['osoba'];
        $vsechnyosoby = (int)$_GET['vsechnyosoby'] == 1;
        $datumod = $_GET['datumod'];
        $datumdo = $_GET['datumdo'];

        $rozdeleniPodrizene = $_GET['rozdelenipodrizene'] == 1;
        $rozdeleni2Podrizene = $_GET['rozdelenipodrizene2'] == 1;
        $jenTyp = $_GET['jentyp'] == 1;
        $jenPomucka = $_GET['jenpomucka'] == 1;
        $nevydane = $_GET['nevydane'] == 1;
        $nevracene = $_GET['nevracene'] == 1;
        $vracene = $_GET['vracene'] == 1;
        $potvrzene = $_GET['potvrzene'] == 1;
        $expirovane = $_GET['expirovane'] == 1;
        $nepotvrzene = $_GET['nepotvrzene'] == 1;

        $pp = null;
        if ($vsechnyosoby) {
            if ((!$this->pravaProhlizet->jsouNejakePrava())) {
                throw new NeniPravoException();
            }
        } else {
            require_once('adsapp/ppomer.php');
            $pp = new PracovniPomer();
            $pp->nactiPPPodleId($this->database, $osoba);

            if ((!$this->pravaProhlizet->lzeZobrazitRC($pp ->rc))) {
                throw new NeniPravoException();
            }
        }

        require_once('adsapp/reportserver.php');

        $result = new ReportOutputObject();
        $result ->loadReportFromGet($this ->database, 'PracovniPomucky');

        if (!$vsechnyosoby) {
            $result->nastavPP($pp);
        }

        $result ->integerParams = [
            'TypRozdeleni' => RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), // jen kvuli kompatibilite starych sestav
            'RozdeleniPodrizene' => ($rozdeleniPodrizene ? 1 : 0),      // jen kvuli kompatibilite starych sestav
            'TypRozdeleni2' => RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), // jen kvuli kompatibilite starych sestav
            'RozdeleniPodrizene2' => ($rozdeleni2Podrizene ? 1 : 0),    // jen kvuli kompatibilite starych sestav
            'JenTyp' => ($jenTyp ? 1 : 0),
            'JenPomucka' => ($jenPomucka ? 1 : 0),
            'Nevydane' => ($nevydane ? 1 : 0),
            'Nevracene' => ($nevracene ? 1 : 0),
            'Vracene' => ($vracene ? 1 : 0),
            'Nepotvrzene' => ($nepotvrzene ? 1 : 0),
            'Potvrzene' => ($potvrzene ? 1 : 0),
            'Expirovane' => ($expirovane ? 1 : 0),
            'VsechnyOsoby' => ($vsechnyosoby ? 1 : 0)  // jen kvuli kompatibilite starych sestav
        ];
        $result ->stringParams = [
            'Rozdeleni' => $_GET['rozdeleni'],   // jen kvuli kompatibilite starych sestav
            'Rozdeleni2' => $_GET['rozdeleni2'], // jen kvuli kompatibilite starych sestav
            'TypPomucky' => $_GET['typpomucky'],
            'Pomucka' => $_GET['pomucka']
        ];
        $result ->dateParams = [
            'PocatekZvolenehoObdobi' => $datumod,
            'KonecZvolenehoObdobi' => $datumdo
        ];

        if (isset($_GET['rozdeleni'])  &&  $_GET['rozdeleni'] != '') {
            require_once 'adsapp/rozdeleni.php';
            $rozdeleni = Rozdeleni::loadRozdeleni($this ->database, RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $_GET['rozdeleni']);
            $result ->nastavRozdeleni(true, $rozdeleni, $rozdeleniPodrizene);
        }

        if (isset($_GET['rozdeleni2'])  &&  $_GET['rozdeleni2'] != '') {
            require_once 'adsapp/rozdeleni.php';
            $rozdeleni = Rozdeleni::loadRozdeleni($this ->database, RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), $_GET['rozdeleni2']);
            $result ->nastavRozdeleni2(true, $rozdeleni, $rozdeleni2Podrizene);
        }

        return $result;
    }

    /**
     * @param string $action
     * @return BaseViewObject|InternalRedirect|OutputObject|Redirect|ViewObject|EditaceExpiracePomuckyForm|PracovniPomuckyBrowser|PracovniPomuckyEditaceDatumuVraceniForm|ReportOutputObject|string
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_POMUCKY)) {
            throw new Exception(ronText('Nemate_pravo'));
        }
        if (!(($this->pravaProhlizet->jsouNejakePrava()) || ($this->pravaZapisVydaniVraceni->jsouNejakePrava()) || ($this->pravaPotvrzeniPrevzeti->jsouNejakePrava()) )) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        HtmlHeaders::getInstance()->addCss('./css/person.css');
        HtmlHeaders::getInstance()->addScript('script/personscripts.js');
        TextManager::getInstance() ->loadStrings('person.ini');

        switch($_GET['action']){
            case 'pomuckyform':
                return $this->pracovniPomuckyForm(isset($_POST['typpohybu']) ? $_POST['typpohybu'] : $_GET['vydanipomucky'], isset($_POST['individualni']) ? $_POST['individualni'] : $_GET['individualni']);

            case 'potvrzeniprevzeti':
                return $this->potvrzeniPrevzeti($_GET['osoba'],$_GET['pomucka'],$_GET['datumnaroku'],$_GET['poradi']);

            case 'editaceexpirace':
                return $this->editaceExpirace($_GET['osoba'],$_GET['pomucka'],$_GET['datumnaroku']);

            case 'detail':
                return $this ->pracovniPomuckyDetailView($_GET['record']);

            case 'report':
                return $this->report();

            default:
                //return new TextViewObject('Pomucky');
                require_once('person/b_pracovnipomucky.php');
                $b = new PracovniPomuckyBrowser($this ->database);
                $b ->load();
                return $b;
        }
    }
}