<?php

/**
 * UTF8
 * Pavel Rudol, Roman Szablatura RON Software
 *  29.11.2015
 */

namespace Ron\RonPortal;

use Exception;
use Ron\Controls\ComboBox;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\Http\HtmlHeaders;
use Ron\IcalOutputObject;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Controller;

use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\MessageManagerViewObject;
use Ron\MVC\Model;
use Ron\DateUtils;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\TranslationsOutputObject;
use Ron\MVC\ViewObject;
use Ron\Personalistika\DotaznikViewObject;
use Ron\Personalistika\HodnoceniSkoleni;
use Ron\RonException;
use function Ron\ronText;
use Ron\StringUtils;
use Ron\TextManager;
use stdClass;


require_once('adsapp/dokumentyutils.php');
require_once('adsapp/b_dokumenty.php');
require_once('ron/browsercontroller.php');
require_once('person/b_skoleni.php');
require_once('person/b_skoleniprihlaseni.php');
require_once('adsapp/ronportaldbrecord.php');


/**
 * Class NovyDokumentKurzuForm
 * @package Ron\RonPortal
 */
class NovyDokumentKurzuForm extends DokumentUploadForm
{

    function __construct($database, $kodKurzu)
    {
        parent::__construct($database, '?menuid=skoleni&action=dokumentupload');

        //Typ dokumentu false zobrazí combo v dialogu pro výběr typu dokumentu
        //$this->typDokumentu = true;

        $this->hiddenFields = [
            'KodKurzu' => $kodKurzu,
        ];
    }
}

/**
 * Class DokumentyKurzuBrowser
 * @package Ron\RonPortal
 */
class DokumentyKurzuBrowser extends DokumentyBrowser
{

    /**
     * DokumentyKurzuBrowser constructor.
     * @param $database
     * @param $kodKurzu
     */
    function __construct($database, $kodKurzu)
    {
        parent::__construct($database, "?menuid=skoleni&action=dokumentdownload&kodKurzu=$kodKurzu", 0, "");

        $this->sqlFromPart =
            'Dokument, TypDokumentu, DokumentSkoleni';

        $params = $this->database->createParameters();

        $this->sqlWherePart = new SqlStatement(
            'Dokument.Id = DokumentSkoleni.Id ' .
            ' AND TypDokumentu.Kod = Dokument.Typ ' .
            " AND DokumentSkoleni.CisloSkoleni = " . $params ->addParameter($kodKurzu), $params);
    }
}




class SkoleniController extends JsonController implements Controller
{   
    const XML_EVENT_BEFORE = 'SKOLENI_PRIHLASENI_BEFORE';
    const XML_EVENT_AFTER = 'SKOLENI_PRIHLASENI_AFTER';
    const XML_CONTEXT_LOGIN = 'login';
    const XML_CONTEXT_LOGOUT = 'logout';
    const NASTAVENI_PONECHAT_NAVRH = 'skoleni_ponechat_navrh';
    
    /* @var $database Database */
    private $database;

    /* @var $skoleni [] */
    private $skoleni;

    public function __construct($database) {
        $this->database = $database;

        TextManager::getInstance() ->loadStrings('person.ini');
        TextManager::getInstance() ->loadStrings('dotaznik.ini');
        HtmlHeaders::getInstance() ->addCss('css/person.css');
        HtmlHeaders::getInstance() ->addCss('css/skoleni.css');
        HtmlHeaders::getInstance() ->addCss('person/dotaznik.css');
        HtmlHeaders::getInstance() ->addScript('script/hodnoceni.js');
        HtmlHeaders::getInstance() ->addScript('script/skolenihodnoceni.js');
        HtmlHeaders::getInstance() ->addScript('?menuid=hodnoceni&translations=1');
        HtmlHeaders::getInstance()->addScript('', '?menuid=skoleni&action=translations&ver=' . HtmlHeaders::getInstance() ->versionId);
        HtmlHeaders::getInstance() ->setIcon(RonPortalSettings::getValue('shortcut_icon') ?: 'images/icon_person.png');
    }

    /** Vraci true, pokud pro zadanou osobu a druh skoleni existuje vygenerovany navrh bez zadaneho absolvovani.
     * @param Database $database
     * @param string $osobaRC
     * @param string $druhSkoleni
     * @return bool
     * @throws DBException
     */
    public static function existujeNavrhTypuSkoleni($database, $osobaRC, $druhSkoleni)
    {
        $params = $database->createParameters();
        $sql = 
            "SELECT COUNT(RC) FROM GenerovaneSkoleni LEFT JOIN Skoleni ON (Skoleni.CisloSkoleni = GenerovaneSkoleni.CisloSkoleni)
            WHERE GenerovaneSkoleni.RC = " . $params->addParameter($osobaRC) . "
              AND GenerovaneSkoleni.DruhSkoleni = " . $params->addParameter($druhSkoleni) . "
              AND (GenerovaneSkoleni.Potvrzeni IN (0,5,6)
                    OR (GenerovaneSkoleni.Potvrzeni = 1 AND (Skoleni.StavSkoleni IS NULL OR Skoleni.StavSkoleni <> 2)))";
        
        MessageManager::addDebugMessage($sql);
        
        $cursor = $database->getSqlResult($sql, $params);
        
        if ($cursor->fetch()[0] > 0) {
            return true;
        }
        
        return false;
    }

    /**
     *
     * @param string $skoleni kod skoleni
     * @param string $rc RC osoby
     * @return boolean vraci true, pokud je osoba prihlasena na skoleni
     * @throws Exception
     */
    private function jeOsobaNaSkoleni($skoleni, $rc)
    {
        $params = $this->database->createParameters();

        $sql =  "SELECT RC FROM GenerovaneSkoleni " .
                "WHERE CisloSkoleni = " . $params->addParameter($skoleni) .
                "  AND RC = " . $params->addParameter($rc) .
                "  AND SchvaleniPrihlasky <> 2";
        
        $cursor = $this->database->select($sql, $params);
        
        $row = $cursor->fetch();
                
        if ($row)
            return TRUE;

        return FALSE;
    }
    /**
     *
     * @param string $skoleni kod skoleni
     * @param string $rc RC osoby
     * @return boolean vraci true, pokud je osoba prihlasena na planu skoleni
     * @throws Exception
     */
    private function jeOsobaNaPlanovanemSkoleni($skoleni, $rc)
    {
        $params = $this->database->createParameters();

        $sql =  "SELECT RC FROM PlanovaneSkoleniOsob " .
                "WHERE CisloSkoleni = " . $params->addParameter($skoleni) .
                "  AND RC = " . $params->addParameter($rc) .
                "  AND Potvrzeni = 1";

        $cursor = $this->database->select($sql, $params);

        $row = $cursor->fetch();

        if ($row)
            return TRUE;

        return FALSE;
    }

    /**
     * @param string $skoleni kod skoleni
     * @param string $rc RC osoby
     * @return mixed
     * @throws DBException
     */
    private function pocetNeabsolvovanychEKurzu($skoleni, $rc)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT COUNT(ID)  
            FROM GenerovaneSkoleni 
            JOIN EKurzOsoby ON ((GenerovaneSkoleni.RC = EKurzOsoby.RC_Skoleni) AND (GenerovaneSkoleni.DruhSkoleni = EKurzOsoby.DruhSkoleni) AND (GenerovaneSkoleni.DatumVznikuPovinnosti = EKurzOsoby.DatumVznikuPovinnosti)) " .
            "WHERE GenerovaneSkoleni.CisloSkoleni = " . $params->addParameter($skoleni) .
            "  AND GenerovaneSkoleni.RC = " . $params->addParameter($rc) .
            "  AND EKurzOsoby.Stav NOT IN (3, 4) ";

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        return $row[0];
    }

    /**
     * @param string $skoleni kod skoleni
     * @param int $dokument id dokumentu
     * @return bool
     * @throws Exception
     */
    private function jePravoNaDokumentSkoleni($skoleni, $dokument)
    {
        if (!($this->jeOsobaNaSkoleni($skoleni,User::getRC()) || $this->jeDokumentSkoleniVerejny($dokument))) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        $params = $this->database->createParameters();

        $sql =
            "SELECT Dokument.id FROM DokumentSkoleni JOIN Dokument ON (Dokument.id = DokumentSkoleni.id) ".
            " JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ) " .
            "WHERE CisloSkoleni = " . $params->addParameter($skoleni) .
            "  AND Dokument.id = " . $params->addParameter($dokument) .
            "  AND (Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1))";

        MessageManager::addDebugLog($sql);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if ($row) {
            return true;
        }
        else {
            throw new Exception(ronText('Nemate_pravo'));
        }
    }

    /**
     * @param string $osobaRC
     * @param string $druhSkoleni
     * @param int $datumPovinnosti
     * @param int $dokument
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    private function jePravoNaDokumentSkoleniOsoby($osobaRC, $druhSkoleni, $datumPovinnosti, $dokument)
    {
        $pravaOsobniUdaje = Prava::nactiPravaUzivatele($this->database,User::getRC(),'prohl_osobni');
        $pravaSkoleni = Prava::vratPrava('skoleni_prohlizeni');
        Prava::nastavObdobiProPrava($datumPovinnosti, $datumPovinnosti);

        if (!($pravaOsobniUdaje->lzeZobrazitRC($osobaRC, $datumPovinnosti, $datumPovinnosti) || $pravaSkoleni->lzeZobrazitRC($osobaRC, $datumPovinnosti, $datumPovinnosti))) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        $params = $this->database->createParameters();

        $sql =
            "SELECT Dokument.id FROM DokumentyOsoby JOIN Dokument ON (DokumentyOsoby.id = Dokument.id) " .
            " JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ) " .
            "WHERE DokumentyOsoby.RC = " . $params->addParameter($osobaRC) .  //RC osoby skoleni
            "  AND DokumentyOsoby.GSk_RC = " . $params->addParameter($osobaRC) .
            "  AND DokumentyOsoby.GSk_DruhSkoleni = " . $params->addParameter($druhSkoleni) .  //druh skoleni
            "  AND DokumentyOsoby.GSk_Datum = " . $params->addParameter($this->database->timestampToParam($datumPovinnosti)) .  //datum vzniku povinnosti
            "  AND (Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1))" .
            "  AND Dokument.id = " . $params->addParameter($dokument) ;

        MessageManager::addDebugLog($sql);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if ($row) {
            return true;
        }
        else {
            throw new Exception(ronText('Nemate_pravo'));
        }
    }

    /**
     * @param $skoleni
     * @return IcalOutputObject
     * @throws Exception
     */
    private function downloadIcal($skoleni)
    {
        require_once('person/skoleni_ical.php');

        $cal = SkoleniCalendarItem::loadKurz($this->database, $skoleni);
        if (!$cal) throw new Exception('Kurz nebyl nalezen!');

        return new IcalOutputObject($cal);
    }

    /**
     * 
     * @param string $skoleni kod skoleni
     * @param integer $dokument id dokumentu
     * @return DokumentOutputObject
     * @throws Exception
     */
    private function downloadDokumentSkoleni($skoleni, $dokument)
    {
        //v url neni zadano konkretni skoleni, ani osoba ktera absolvovala skoleni
        if ($skoleni == '' && !(isset($_GET['osoba']) && isset($_GET['druhskoleni']) && isset($_GET['datum']))) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        //pokud se stahuje dokument skoleni, overi se pravo na skoleni
        if ($skoleni && !($this->jePravoNaDokumentSkoleni($skoleni, $dokument))) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        //muze se stahovat i dokument osoby, pak se overuje, ze je dokument pripojen ke skoleni osoby
        $osobaRC = DBUtils::getSqlValue($this->database, "SELECT RC FROM PracovniPomer WHERE ID = " . (int)$_GET['osoba']);

        if (isset($_GET['datum']) && !($this->jePravoNaDokumentSkoleniOsoby($osobaRC, $_GET['druhskoleni'], $_GET['datum'], $dokument))) {
            throw new Exception(ronText('Nemate_pravo'));
        }


        require_once('adsapp/dokumentyutils.php');
        return new DokumentOutputObject($this->database,$dokument);
    }

    /**
     * Kontroluje jestli je typ dokumentu nastaven jako verejny
     * @param integer $dokument id dokumentu
     * @return boolean true pokud je typ dokumentu verejny
     * @throws Exception
     */
    private function jeDokumentSkoleniVerejny($dokument)
    {        
        $params = $this->database->createParameters();

        $sql =  'SELECT Verejny FROM Dokument JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ)' .
                'WHERE Dokument.id = ' . $params->addParameter($dokument);
        
        $cursor  = $this->database->select($sql, $params);
        
        $row = $cursor->fetch();
        
        if ($row[0] == 1) {
            return TRUE;
        }
        else {
            return FALSE;
        }
    }
    
    /**
     * nacte detaily skoleni
     * @param string $cisloSkoleni kod skoleni
     * @throws Exception
     */
    private function nactiDetailySkoleni($cisloSkoleni)
    {
        $params = $this->database->createParameters();

        $sql = "SELECT DruhSkoleni, DatumPocatkuSkoleni, Kapacita, UcastniciDlePlanSkoleni, CisloPlanovanehoSkoleni, StavSkoleni, DS.Elearning, PozadovatSchvaleniPrihl 
                FROM Skoleni JOIN DruhSkoleni DS on Skoleni.DruhSkoleni = DS.Kod WHERE CisloSkoleni = " . $params->addParameter($cisloSkoleni);
        
        $cursor = $this->database->select($sql, $params);
        
        $row = $cursor->fetch();
        
        if (!$row)
        {
            throw new Exception(ronText('SkoleniNenalezeno'));
        }
        else {
            $skoleni = [];
            $skoleni['druhSkoleni'] = $row[0];
            $skoleni['datumPocatkuSkoleni'] = $this->database->stringToTimestamp($row[1]);
            $skoleni['kapacita'] = $row[2];
            $skoleni['stav'] = $row[5];
            if ($row[3])
                $skoleni['cisloplanovanehoskoleni'] = $row[4];
            $skoleni['elearning'] = (int)$row[6];
            $skoleni['pozadovatSchvaleni'] = (int)$row[7];
            
            $this->skoleni = $skoleni;
        }
    }

    /**
     * Zkontroluje jestli ma skoleni volna mista
     * @param string $cisloSkoleni kod skoleni
     * @return boolean true pokud je pocet prihlasenych < kapacita skoleni
     * @throws Exception
     */
    private function jeVolnaKapacitaSkoleni($cisloSkoleni)
    {
        $params = $this->database->createParameters();

        $sql =  "SELECT COUNT(RC) FROM GenerovaneSkoleni " .
                "WHERE CisloSkoleni = " . $params->addParameter($cisloSkoleni);
        
        $cursor = $this->database->select($sql, $params);
        
        $row = $cursor->fetch();
        
        $prihlasenych = $row[0] ? $row[0] : 0;
        
        $kapacita = $this->skoleni['kapacita'];
        
        if ($prihlasenych < $kapacita) {
            return TRUE;
        }
        else {
            return FALSE;
        }
    }

    /**
     * Zkontroluje jestli ma mistnost, ve stejnem case volna mista. Pokud neni kapacita zadana, bere se ze neni omezeno.
     * @param string $cisloSkoleni kod skoleni
     * @return boolean true pokud je pocet prihlasenych < kapacita mistnosti nebo kapacita neni zadana
     * @throws Exception
     */
    private function jeVolnaKapacitaMistnosti($cisloSkoleni)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT COUNT(RC) 
            FROM GenerovaneSkoleni JOIN Skoleni ON (Skoleni.CisloSkoleni = GenerovaneSkoleni.CisloSkoleni)
            WHERE EXISTS(SELECT RC FROM Skoleni S WHERE S.CisloSkoleni = " . $params->addParameter($cisloSkoleni) .
            " AND S.MistoSkoleni = Skoleni.MistoSkoleni AND S.DatumPocatkuSkoleni = Skoleni.DatumPocatkuSkoleni AND S.CasPocatkuSkoleni = Skoleni.CasPocatkuSkoleni)";

        $cursor = $this->database->select($sql, $params);

        $row = $cursor->fetch();

        $prihlasenych = $row[0] ?: 0;

        $params = $this->database->createParameters();

        $sql =
            "SELECT Umisteni.KapacitaMistnosti
            FROM Skoleni JOIN Umisteni ON Skoleni.MistoSkoleni = Umisteni.Kod
            WHERE CisloSkoleni = " . $params->addParameter($cisloSkoleni);

        $cursor = $this->database->select($sql, $params);

        $row = $cursor->fetch();

        $kapacita = $row[0] ?: null;

        //kapacita neni zadana
        if (is_null($kapacita) || ($prihlasenych < $kapacita)) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * zobrazi seznam skoleni, na ktere je mozne se prihlasit
     * @param null | int $hromadnePrihlaseni
     * @return string
     * @throws DBException
     * @throws Exception
     */
    private function zobrazPrihlaseniSkoleni($hromadnePrihlaseni)
    {
        if (Prava::vratHodnotuPrava('skoleni_prihlasit') == 0) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        $this->hlavickaVyberAkce();

        HtmlHeaders::getInstance()->addScript('script/personscripts.js');
        HtmlHeaders::getInstance()->addCss('css/skoleni.css');

        $dialog = new BaseViewObject('person/view_hromadneprihlasenikurzu.php');
        //$dialogHromadnehoVyhledani = new BaseViewObject('person/view_hromadnevyhledanikurzuproosoby.php');

        $browser = new SkoleniPrihlaseniBrowser($this->database, $hromadnePrihlaseni);
        $browser->load();

        Model::addViewObject($dialog);
//        Model::addViewObject($dialogHromadnehoVyhledani);
        Model::addViewObject($browser);

        return "view/appview.php";
    }

    /**
     * Prihlasi vybranou osobu na skoleni, pokud je dostupna kapacita, uzivatel ma na prihlaseni pravo
     * @param string $skoleni kod skoleni
     * @param string $osobaIDPP osoba, ktera se prihlasuje na skoleni
     * @param int $datumPovinnosti
     * @return Redirect pri uspechu presmeruje na seznam skoleni k prihlaseni
     * @throws Exception
     */
    private function prihlaseniSkoleni($skoleni, $osobaIDPP, $datumPovinnosti)
    {        
        $this->nactiDetailySkoleni($skoleni);

        require_once('adsapp/ppomer.php');
        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osobaIDPP);

        //skoleni musi byt ve stavu pripravuje se = 0 nebo probiha = 1
        if (!(($this->skoleni['stav'] == 0) || ($this->skoleni['stav'] == 1)))
            throw new Exception(ronText('SkoleniChybnyStav'));

        if ($this->skoleni['cisloplanovanehoskoleni']) {
            if (!$this->jeOsobaNaPlanovanemSkoleni($this->skoleni['cisloplanovanehoskoleni'], $pp->rc))
                throw new Exception(ronText('SkoleniNeniPrihlasenPlan'));
        }

        //kontrola kapacity a jednoho prihlaseni osoby se provadi pro bezne kurzy, u e-learningovych se ignoruje
        if ($this->skoleni['elearning'] == 0) {
            if (!$this->jeVolnaKapacitaSkoleni($skoleni)) {
                throw new Exception(ronText('SkoleniNaplnenaKapacita'));
            }

            if (!$this->jeVolnaKapacitaMistnosti($skoleni)) {
                throw new Exception(ronText('SkoleniNaplnenaKapacitaMistnosti'));
            }

            if ($this->jeOsobaNaSkoleni($skoleni, $pp->rc)) {
                throw new Exception(ronText('SkoleniJizPrihlasene'));
            }
        } else {
            if ($this->pocetNeabsolvovanychEKurzu($skoleni, $pp->rc) > 0) {
                throw new Exception(ronText('SkoleniExistujiNeabsolvaneEkurzy'));
            }
        }
        
        $prava = Prava::vratPrava('skoleni_prihlasit');
        $pravaPotvrditPrihlaseni = Prava::vratPrava('skoleni_schvalit_prihlaseni');
        Prava::nastavObdobiProPrava($this->skoleni['datumPocatkuSkoleni']);
        
        if (!$prava->lzeZobrazitRC($pp->rc)) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        if (!RonPortalSession::vsechnyOsoby()) {
            //pro pouziti v XML k identifikace vybraneho PP, napr. pro zapis do PK
            $osoba = ['osobaRC' => RonPortalSession::osobaRC(),
                'osobaOsobniCislo' => RonPortalSession::osobaOsobniCislo(),
                'osobaDatumNastupu' => RonPortalSession::osobaNastup()];
        } else {
            //libovolny PP osoby s nejvyssim platnym ID
            $osoba = ['osobaRC' => $pp->rc,
                'osobaOsobniCislo' => $pp->osobniCislo,
                'osobaDatumNastupu' => $pp->datumNastupu];
        }

        $uspesnaRegistrace = false;

        $record = new RonPortalDBRecord($this->database, 'GenerovaneSkoleni');
        
        $record->logDescription = 'Prihlaseni skoleni';
        
        $record->addField('DatumSkoleni', 'date', FALSE, 0, $this->skoleni['datumPocatkuSkoleni']);
        $record->addField('CisloSkoleni', '', FALSE, 10, $skoleni);
        $record->addField('DatumPrihlasky', 'date', FALSE, 0, time());
        $record->addField('PrihlasenoZAplikace', 'int', FALSE, 0, 1);
        $record->addField('Prihlasil', '', FALSE, 25, User::getLoginName());
        //$record->addField('Poznamka', 'memo', FALSE, 0, '');

        $record->addField('Potvrzeni', 'int', true);

        $record->addField('SchvaleniPrihlasky', 'int', true, 0);

        if ($this->skoleni['pozadovatSchvaleni'] == 1) {
            //pokud ma uzivatel pravo na potvrzeni prihlaseni, potvzeni zapise, jinak se provadi samostatnou akci
            if ($pravaPotvrditPrihlaseni->lzeZobrazitRC($pp->rc)) {
                $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(1);
                $record->addField('Vyhodnotil', '', false, 25, User::getLoginName());
                $record->addField('DatumVyhodnoceni', 'date', false, 0, DateUtils::today());
                $record->addField('PoznamkaSchvalovani', 'memo', false, 0);
            }
            else {
                $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(0);
                $uspesnaRegistrace = true;
            }
        } else {
            $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(-1);
        }

        $record->addField('Generovane', 'int', true, 0, 0);

        $params = $this->database->createParameters();

        //prihlasovani na jiz vygenerovany/rucne vlozeny zaznam
        if ($datumPovinnosti > 0) {
            $record->setWhere("RC = " . $params->addParameter($pp->rc) .
                " AND DruhSkoleni = " . $params->addParameter($this->skoleni['druhSkoleni']) .
                " AND DatumVznikuPovinnosti = " . $params->addParameter($this->database->timestampToParam($datumPovinnosti)), $params);
        } else {
            $record->setWhere("RC = " . $params->addParameter($pp->rc) .
                " AND DruhSkoleni = " . $params->addParameter($this->skoleni['druhSkoleni']) .
                " AND DatumSkoleni IS NULL " .
                " AND NOT EXISTS(SELECT * FROM GenerovaneSkoleni GS WHERE GS.RC = GenerovaneSkoleni.RC " .
                "  AND GS.DruhSkoleni = GenerovaneSkoleni.DruhSkoleni " .
                "  AND GS.DatumVznikuPovinnosti > GenerovaneSkoleni.DatumVznikuPovinnosti " .
                "  AND GS.CisloSkoleni IS NOT NULL)", $params);
        }
        
        //osoba jiz ma vygenerovanou povinnost skoleni absolvovat, zapisou se informace o prihlasenem skoleni
        if ($record->loadRecord()) {
            //pro povinnost = 0 (doporucene) se pri schvaleni nastavi potvrzeni na 1
            if ($record->findFieldByName('Potvrzeni')->value == 0)
                if (($record->findFieldByName('SchvaleniPrihlasky')->value == -1) || ($record->findFieldByName('SchvaleniPrihlasky')->value == 1))
                    $record->findFieldByName('Potvrzeni')->setNewValue(1);

            if (!$record->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_LOGIN, $osoba)) {
                throw new Exception(ronText('SkoleniChybaXMLudalosti'));
            }
            
            if (!$record->updateRecord()) {
                throw new Exception(ronText('SkoleniChybaPrihlaseni'));
            }
        }
        //pro osobu neexistuje povinnost absolvovat skoleni (skoleni je doporucene) vlozi se novy zaznam
        else {
            $record->addField('RC', '', FALSE, 15, $pp->rc);
            $record->addField('DruhSkoleni', '', FALSE, 10, $this->skoleni['druhSkoleni']);
            $record->addField('DatumVznikuPovinnosti', 'timestamp', FALSE, 0, DateUtils::now());
            $record->addField('DuvodSkoleni', 'int', FALSE, 0, 1); //individualni duvod skoleni
            $record->addField('DatumZapisu', 'timestamp', FALSE, 0, DateUtils::today());
            $record->addField('Vlozil', '', FALSE, 25, User::getLoginName());
            $record->addField('PozadovaneDatumSkoleni', 'date', FALSE, 0, $this->skoleni['datumPocatkuSkoleni']);
            $record->findFieldByName('Potvrzeni')->setNewValue(0);

            //pro schvalene nebo pokud se nevyzaduje schvaleni nastavi se potvrzeni na 1
            if (($record->findFieldByName('SchvaleniPrihlasky')->value == -1) || ($record->findFieldByName('SchvaleniPrihlasky')->value == 1))
                $record->findFieldByName('Potvrzeni')->setNewValue(1);

            if (!$record->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_LOGIN, $osoba)) {
                throw new Exception(ronText('SkoleniChybaXMLudalosti'));
            }

            if (!$record->insertRecord()) {
                throw new Exception(ronText('SkoleniChybaPrihlaseni'));
            }
        }
        
        if (!$record->executeXmlEvents(self::XML_EVENT_AFTER, self::XML_CONTEXT_LOGIN, $osoba)) {
            throw new Exception(ronText('SkoleniChybaXMLudalosti'));
        }

        if ($uspesnaRegistrace)
            MessageManager::addInfoMessage(ronText('Skoleni_uspesna_registrace'));
        else
            MessageManager::addInfoMessage(ronText('SkoleniUspesnePrihlaseni'));
     
        return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');
    }

    /**
     * provede odhlaseni osoby ze skoleni (smaze zaznam, nebo zrusi prirazeni skoleni)
     * @param string $skoleni kod skoleni
     * @param string $osobaIDPP rc osoby
     * @param int $datumPovinnosti
     * @return JsonOutputObject|Redirect pri uspechu presmeruje na seznam skoleni k prihlaseni
     * @throws Exception
     */
    private function odhlaseniSkoleni($skoleni, $osobaIDPP, $datumPovinnosti)
    {
        require_once('adsapp/ppomer.php');
        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osobaIDPP);

        $smazat = isset($_GET['navrh']) && ($_GET['navrh'] == 'smazat');
        //pokud je v nastaveni zvoleno ponechat navrh, nikdy se nebude mazat
        if (RonPortalSettings::getIntValue(self::NASTAVENI_PONECHAT_NAVRH))
            $smazat = false;
        $json = isset($_GET['json']);

        if (!$this->jeOsobaNaSkoleni($skoleni, $pp->rc)) {
            if ($json) {
                return new JsonOutputObject(['text' => StringUtils::jsonText(ronText('SkoleniNeniPrihlasene'))]);
            }
            throw new Exception(ronText('SkoleniNeniPrihlasene'));
        }
        
        $prava = Prava::vratPrava('skoleni_prihlasit');
        $pravaPotvrditPrihlaseni = Prava::vratPrava('skoleni_schvalit_prihlaseni');
        
        if (!$prava->lzeZobrazitRC($pp->rc)) {
            if ($json) {
                return new JsonOutputObject(['text' => StringUtils::jsonText(ronText('Nemate_pravo'))]);
            }
            throw new Exception(ronText('Nemate_pravo'));
        }

        if (!RonPortalSession::vsechnyOsoby()) {
            //pro pouziti v XML k identifikace vybraneho PP, napr. pro zapis do PK
            $osoba = ['osobaRC' => RonPortalSession::osobaRC(),
                'osobaOsobniCislo' => RonPortalSession::osobaOsobniCislo(),
                'osobaDatumNastupu' => RonPortalSession::osobaNastup()];
        } else {
            //libovolny PP osoby s nejvyssim platnym ID
            $osoba = ['osobaRC' => $pp->rc,
                'osobaOsobniCislo' => $pp->osobniCislo,
                'osobaDatumNastupu' => $pp->datumNastupu];
        }
        
        $record = new RonPortalDBRecord($this->database, 'GenerovaneSkoleni');
        
        $record->logDescription = 'Odhlaseni skoleni';
        
        $record->addField('CisloSkoleni', '', FALSE, 10);
        $record->addField('DatumSkoleni', 'date', FALSE, 0);
        $record->addField('DatumPrihlasky', 'date', FALSE, 0);
        $record->addField('Prihlasil', '', FALSE, 25);

        $record->addField('DuvodSkoleni', 'int', TRUE);

        $record->addField('Potvrzeni', 'int', true);

        $record->addField('SchvaleniPrihlasky', 'int', true, 0);

        $params = $this->database->createParameters();

        $record->setWhere("RC = " . $params->addParameter($pp->rc) .
                            " AND CisloSkoleni = " . $params->addParameter($skoleni) .
                            ($datumPovinnosti > 0 ? " AND DatumVznikuPovinnosti = " . $params->addParameter($this->database->timestampToParam($datumPovinnosti)) : ''), $params);
        
        if (!$record->loadRecord()) {
            if ($json) {
                return new JsonOutputObject(['text' => StringUtils::jsonText(ronText('SkoleniChybaNacteniZaznamu'))]);
            }
            throw new Exception(ronText('SkoleniChybaNacteniZaznamu'));
        }
        
        if (!$record->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_LOGOUT, $osoba)) {
            if ($json) {
                return new JsonOutputObject(['text' => StringUtils::jsonText(ronText('SkoleniChybaXMLudalosti'))]);
            }
            throw new Exception(ronText('SkoleniChybaXMLudalosti'));
        }

        //skoleni se nemaze, jen se zrusi prirazene skoleni
        if (!$smazat) {

            //prihlaska jiz byla schvalena, odhlaseni muze provest jen uzivatel s pravy pro schvaleni prihlasky, pri odhlaseni se zrusi zaroven schvaleni prihlasky
            if ($record->findFieldByName('SchvaleniPrihlasky')->value == 1) {
                if (!$pravaPotvrditPrihlaseni->lzeZobrazitRC($pp->rc))
                    throw new Exception(ronText('Nemate_pravo'));

                    $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(0);

                //pro doporucene (individualni duvod skoleni - teoreticky i rucne vlozeny zaznam) se nastavi potvrzeni na 0
                if ($record->findFieldByName('DuvodSkoleni')->value == 1)
                    $record->findFieldByName('Potvrzeni')->setNewValue(0);
            }

            if (!$record->updateRecord()) {
                if ($json) {
                    return new JsonOutputObject(['text' => StringUtils::jsonText(ronText('SkoleniChybaOdhlaseni'))]);
                }
                throw new Exception(ronText('SkoleniChybaOdhlaseni'));
            }
        }
        //smaze se zaznam generovaneho skoleni
        else {
            if (!$record->deleteRecord()) {
                if ($json) {
                    return new JsonOutputObject(['text' => StringUtils::jsonText(ronText('SkoleniChybaOdhlaseni'))]);
                }
                throw new Exception(ronText('SkoleniChybaOdhlaseni'));
            }
        }
        
        if (!$record->executeXmlEvents(self::XML_EVENT_AFTER, self::XML_CONTEXT_LOGOUT, $osoba)) {
            if ($json) {
                return new JsonOutputObject(['text' => StringUtils::jsonText(ronText('SkoleniChybaXMLudalosti'))]);
            }
            throw new Exception(ronText('SkoleniChybaXMLudalosti'));
        }

        if ($json) {
            return new JsonOutputObject(['text' => StringUtils::jsonText(ronText('SkoleniUspesneOdhlaseni'))]);
        }

        MessageManager::addInfoMessage(ronText('SkoleniUspesneOdhlaseni'));
        
        return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');
    }

    /**
     * provede potvrzeni(zamitnuti) prihlaseni osoby na skoleni (nastavi schvaleniprihlasky na hodnotu)
     * @param string $skoleni kod skoleni
     * @param string $osobaIDPP rc osoby
     * @param int $datumPovinnosti
     * @param int $hodnota - hodnota schvaleni: 1 = schvaleno, 2 = vraceno
     * @return JsonOutputObject|Redirect pri uspechu presmeruje na seznam skoleni k prihlaseni
     * @throws Exception
     */
    private function nastavSchvaleniPrihlaseni($skoleni, $osobaIDPP, $datumPovinnosti, $hodnota = 1)
    {
        require_once('adsapp/ppomer.php');
        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osobaIDPP);

        if (!$this->jeOsobaNaSkoleni($skoleni, $pp->rc)) {
            throw new Exception(ronText('SkoleniNeniPrihlasene'));
        }

        $prava = Prava::vratPrava('skoleni_schvalit_prihlaseni');

        if (!$prava->lzeZobrazitRC($pp->rc)) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        $record = new RonPortalDBRecord($this->database, 'GenerovaneSkoleni');

        $record->logDescription = 'Schvaleni prihlasky skoleni';

        $record->addField('CisloSkoleni', '', true, 10);
        $record->addField('DatumSkoleni', 'date', true, 0);
        $record->addField('DatumPrihlasky', 'date', true, 0);
        $record->addField('Prihlasil', '', true, 25);

        $record->addField('Vyhodnotil', '', true, 25);
        $record->addField('DatumVyhodnoceni', 'date', true, 0);
        $record->addField('Potvrzeni', 'int', true, 0);
        $record->addField('SchvaleniPrihlasky', 'int', true, 0);

        $record->addField('DuvodSkoleni', 'int', true);

        $params = $this->database->createParameters();

        $record->setWhere("RC = " . $params->addParameter($pp->rc) .
                            " AND CisloSkoleni = " . $params->addParameter($skoleni) .
                            " AND DatumVznikuPovinnosti = " . $params->addParameter($this->database->timestampToParam($datumPovinnosti)), $params);

        if (!$record->loadRecord()) {
            throw new Exception(ronText('SkoleniChybaNacteniZaznamu'));
        }

        $record->findFieldByName('Vyhodnotil')->setNewValue(User::getLoginName());
        $record->findFieldByName('DatumVyhodnoceni')->setNewValue(DateUtils::today());
        $record->findFieldByName('SchvaleniPrihlasky')->setNewValue($hodnota);

        //pro doporucene nastavi pri schvaleni potvrzeni na 1, pri zamitnuti necha 0
        if (($record->findFieldByName('Potvrzeni')->value == 0)) {
            if ($hodnota == 1) {
                $record->findFieldByName('Potvrzeni')->setNewValue(1);
                MessageManager::addInfoMessage(ronText('Skoleni_uspesne_schvaleni'));
            }
            else {
                $record->findFieldByName('Potvrzeni')->setNewValue(0);
                MessageManager::addInfoMessage(ronText('Skoleni_uspesne_zamitnuti'));
            }
        } elseif ($hodnota == 2) { //zamitnuti schvaleni zrusi u povinneho skoleni vazbu na kurz (cislo skoleni)
            $record->findFieldByName('CisloSkoleni')->setNewValue('');
            $record->findFieldByName('DatumSkoleni')->setNewValue('');
            $record->findFieldByName('DatumPrihlasky')->setNewValue('');
            $record->findFieldByName('Prihlasil')->setNewValue('');
        }

        if (!$record->updateRecord()) {
            throw new Exception(ronText('Skoleni_chyba_schvaleni'));
        }

        return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');
    }

    /**
     * Zobrazi prehled skoleni s filtrem na osobu
     * @return string
     * @throws DBException
     * @throws Exception
     */
    private function prehledSkoleni()
    {
        $this->hlavickaVyberAkce();


        $browser = new SkoleniBrowser($this->database);
        $browser->load();

        Model::addViewObject($browser);

        
//        if (Prava::vratHodnotuPrava('skoleni_prihlasit') != 0) {
//            Model::addViewObject(new TextViewObject("<br/><A href='?menuid=skoleni&action=zobrazprihlaseni'>" . ronText('Skoleni_prihlaseni') . "</A>",'textlink'));
//        }
//        if (Prava::vratHodnotuPrava('skoleni_absolvovani') != 0) {
//            Model::addViewObject(new TextViewObject("<br/><A href='?menuid=skoleni&action=zapisabsolvovani&formaction=new'>" . ronText('Skoleni_zapsat_absolvovani') . "</A>",'textlink'));
//        }

        return 'view/appview.php';
    }

    /** Zobrazi/zpracuje formular absolvovani skoleni.
     * @param string $formaction
     * @return Redirect|SkoleniForm
     * @throws DBException
     * @throws Exception
     */
    private function skoleniFormular($formaction)
    {
        $this->hlavickaVyberAkce();

        $osoba = isset($_POST['osoba']) ? $_POST['osoba'] : $_GET['osoba'];
        $druhSkoleni = isset($_POST['DruhSkoleni']) ? $_POST['DruhSkoleni'] : $_GET['druhskoleni'];
        $datumPovinnosti = isset($_POST['DatumPovinnosti']) ? $_POST['DatumPovinnosti'] : $_GET['datumpovinnosti'];

        $pp = null;
        require_once('adsapp/ppomer.php');
        if ($osoba)
            $pp = PracovniPomer::nactiPP($this->database, $osoba);

        if ($formaction == 'new') {
            //pri zobrazeni formulare pro vlozeni musi mit nejake prava
            if (!Prava::vratPrava('skoleni_absolvovani')->jsouNejakePrava()) {
                RonPortalUtils::neniPravoException();
            }
        }
        else {
            //pri vkladani vybrane osoby, editace zaznamu s osobou musi mit pravo na konkretni osobu
            if (!Prava::vratPrava('skoleni_absolvovani')->lzeZobrazitRC($pp->rc, DateUtils::now(), DateUtils::now())) {
                RonPortalUtils::neniPravoException();
            }
        }

        require_once('person/f_skoleni.php');

        $form = new SkoleniForm($this->database, $pp, $druhSkoleni, $datumPovinnosti);

        if ($form->handleAction()) {
            return new Redirect(null, '?menuid=skoleni');
        }
        
        return $form;
    }

    /**
     * Menu školení
     */
    private function hlavickaVyberAkce()
    {

        $sekce = [];
        if (Prava::vratHodnotuPrava('skoleni_prohlizeni') != 0) {
            $sekce['prehled'] = ronText('Skoleni_prehled');
        }

        if (Prava::vratHodnotuPrava('skoleni_vyslat') != 0) {
            $sekce['vyslatnaskoleni'] = ronText('Skoleni_vyslat_na_skoleni');
        }

        if (Prava::vratHodnotuPrava('skoleni_absolvovani') != 0) {
            $sekce['zapisabsolvovani'] = ronText('Skoleni_zapsat_absolvovani');
        }

        if (Prava::vratHodnotuPrava('skoleni_prihlasit') != 0) {
            $sekce['zobrazprihlaseni'] = ronText('MI_Prihlaseni_na_skoleni');
        }

        if (Prava::vratHodnotuPrava('skoleni_prihlasit') != 0) {
            $sekce['skolenikurzy'] = ronText('MI_Skoleni_a_kurzy');
            $sekce['prihlaseninaplanovaneskoleni'] = ronText('MI_prihlasovani_na_planovane_skoleni');
        }

        if (Prava::vratHodnotuPrava('skoleni_plan') != 0) {
            $sekce['planovaneskolenikurzy'] = ronText('MI_Planovane_Skoleni_Kurzy');
        }

        if (Prava::vratHodnotuPrava('skoleni_plan') != 0) {
            $sekce['vytvoreniskolenikurzu'] = ronText('MI_Zadat_novy_play_skoleni');
        }

        $aktivni_sekce = ($_GET['action'] ?: (isset($_GET['formaction']) ? 'zapisabsolvovani' : ''));

        Model::addData('header',  ronText('Skoleni'));
        Model::addData('aktivni_sekce', $aktivni_sekce);
        Model::addData('sekce', $sekce);
        Model::addData('url', '?menuid=skoleni&action=');
        Model::addData('wrap_class', 'skoleni');
        Model::addViewObject(new BaseViewObject('view/appsections.php'));
    }

    /** Zobrazi/zpracuje hodnoceni skoleni.
     * @return Redirect|ViewObject
     * @throws Exception
     */
    private function hodnoceniSkoleni()
    {
        $id = (int)$_GET['record'];

        if (isset($_POST['hodnoceniaction'])  &&  $_POST['hodnoceniaction']) {
            // odeslani formulare - ulozeni hodnoceni nebo uzavreni
            switch ($_POST['hodnoceniaction']) {
                case 'ulozdotaznik' :
                    return $this ->ulozHodnoceni();

                case'uzavridotaznik' :
                    return $this ->uzavriHodnoceni($id);

                default :
                    throw new Exception('Chyba v metode SkoleniController::handleAction - neznamy typ akce!');
            }
        }

        $hodnoceniAction = isset($_GET['hodnoceniaction']) ? $_GET['hodnoceniaction'] : '';
        return $this ->pripravenyDotaznik($id, $hodnoceniAction == 'editujhodnoceni', false);
    }

    /**
     * Zkontroluje, jestli ma uzivatel pravo na tento dotaznik
     * - jestli je pro zamestnance prihlaseneho na skoleni
     * - jestli je pro jeho vedouciho RCHodnoticiOsoby nebo je uzivatel na primo nadrizenem rozdeleni
     * - vraci 0 nebo false - nema pravo,  1 - muze prohlizet,  2 - muze editovat
     * @param Database $database
     * @param int  $id
     * @param bool $editace  Pokud je $editace = true a zjisti, ze nemuze editovat, tak rovnou vraci 0 a nekontroluje, jestli lze i cist
     * @return int
     * @throws Exception
     */
    public static function vratPravoNaHodnoceni($database, $id, $editace)
    {
        // zjisti stav, uroven predavani a rozdeleni osoby
        $sql =
            'SELECT PripraveneTemaHodnoceni.Stav, PripraveneTemaHodnoceni.HodnotilRC, ' . // 0, 1
            '    PripraveneTemaHodnoceni.HodnocenyRC, HodnoceniSkoleni.TypGenerovani, ' . // 2, 3
            '    HodnoceniSkoleni.RCHodnoticiOsoby, HodnoceniSkoleni.TypRozdeleni ' . // 4, 5
            ' FROM PripraveneTemaHodnoceni JOIN HodnoceniSkoleni ON ((PripraveneTemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni) ' .
            '           AND (PripraveneTemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo))' .
            ' WHERE PripraveneTemaHodnoceni.Id = ' . (int)$id ;

        MessageManager::addDebugLog($sql);

        $cursor = $database ->getSqlResult($sql);

        $row = $cursor ->fetch();
        if (!$row) {
            throw new Exception(StringUtils::fromUTF8("Chyba - zadaný dotazník (číslo $id) nebyl nalezen nebo není určen pro hodnocení školení!"));
        }

        $result = 2; // lze editovat

        // pokud se ma editovat, zkontroluje, jestli neni uzavren
        if ($row[0] == 2 || $row[0] == 3) {
            // dotaznik je uzavren
            if ($editace) {
                return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - dotazník nelze editovat, protože je již uzavřen!'));
            }

            $result = 1; // nelze editovat - jeste bude kontrolovat jesli lze cist
        }

        if ($row[1] == User::getRC()) return $result;
        // dotaznik vyplnil aktualni uzivatel, takze ma na jeho zobrazeni pravo
        // uz neni potreba kontrolovat stromovou strukturu

        switch ($row[3]) {
            case 1 : //dle typu rozdeleni
                $kodtypurozdeleni = $row[5];
                $obdobiod = DateUtils::today();
                $obdobido = DateUtils::today();

                $params = $database->createParameters();

                $sql =
                    'SELECT * ' . // kontrola, ze jsem (byl) v hodnocenem obdobi v primo nadrazenem rozdeleni
                    '  FROM RozdeleniOsob R_JA JOIN Rozdeleni R_Nad ON (R_Nad.RozdeleniRodice = R_Ja.KodTypuRozdeleni AND R_Nad.Rodic = R_Ja.Rozdeleni) ' .
                    '       JOIN RozdeleniOsob R_Hodnoceny ON (R_Hodnoceny.KodTypuRozdeleni = R_Nad.KodTypuRozdeleni AND R_Hodnoceny.Rozdeleni = R_Nad.Kod) ' .
                    "  WHERE R_Ja.RC = " . $params->addParameter(User::getRC()) .
                    '    AND R_JA.KodTypuRozdeleni = ' . $params->addParameter($kodtypurozdeleni) .
                    "    AND R_Hodnoceny.RC = " . $params->addParameter($row[2]) .
                    '    AND R_Ja.PlatnostOd <= ' . $params->addParameter($database->timestampToParam($obdobido)) .
                    '    AND (R_Ja.PlatnostDo >= ' . $params->addParameter($database->timestampToParam($obdobiod)) . '  OR  R_Ja.PlatnostDo IS NULL)' .
                    '    AND R_Hodnoceny.PlatnostOd <= ' . $params->addParameter($database->timestampToParam($obdobido)) .
                    '    AND (R_Hodnoceny.PlatnostDo >= ' . $params->addParameter($database->timestampToParam($obdobiod)) . '  OR  R_Hodnoceny.PlatnostDo IS NULL)';

                $cursor = $database->getSqlResult($sql, $params);

                $row = $cursor->fetch();
                if (!$row) {
                    return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - na zobrazení zvoleného dotazníku nemáte právo! Hodnocená osoba patří do jiného rozdělení.'));
                }
                break;
            case 2 : //dle osoby
                if ($row[4] != User::getRC()) {
                    return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - na zvolené hodnocení nemáte právo, nejste osobou určenou k hodnocení!'));
                }
                break;
            case 3 : //generovat - skoleni je vygenerovano pro ucastnika
                if ($row[2] != User::getRC()) {
                    return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - na zvolené hodnocení nemáte právo!'));
                }
                break;
        }

        // vse je OK
        return $result; // nastaveno drive na 1 (jen cist) nebo 2 (i editace)
    }

    /**
     * @param int  $id
     * @param bool $editace
     * @return HodnoceniSkoleni
     * @throws Exception
     */
    function nactiHodnoceni($id, $editace)
    {
        require_once('person/hodnoceniskoleni.php');

        /** @var $hodnoceni HodnoceniSkoleni */
        $hodnoceni = new HodnoceniSkoleni($this ->database, $id);

        $pravo = self::vratPravoNaHodnoceni($this ->database, $id, $editace);
        $hodnoceni ->fPravo = $pravo;

        if (!$pravo  ||  ($editace  &&  $pravo < 2)) {
            RonPortalUtils::neniPravoException();
        }

        return $hodnoceni;
    }

    /**
     * @param $kodKurzu
     * @return SkoleniUcastBrowser
     * @throws Exception
     */
    private function zobrazUcastSkoleni($kodKurzu)
    {
        HtmlHeaders::getInstance()->addCss('css/skoleni.css');
        HtmlHeaders::getInstance()->addScript('script/skoleniucast.js');

        require_once ('person/b_skoleniucast.php');
        $b = new SkoleniUcastBrowser($this->database, $kodKurzu);
        $b->load();
        return $b;
    }


    /**
     * @param $kodKurzu
     * @return HodnoceniSkoleniBrowser
     * @throws Exception
     */
     private function zobrazHodnoceniSkoleni($kodKurzu)
    {
        HtmlHeaders::getInstance()->addCss('css/skoleni.css');
        HtmlHeaders::getInstance()->addScript('script/skolenihodnoceni.js');

        require_once ('person/b_skolenihodnoceni.php');
        $b = new HodnoceniSkoleniBrowser($this->database, $kodKurzu);
        $b->load();
        return $b;
    }


    /**
     * @return SkoleniAKurzy
     * @throws Exception
     */
    private function zobrazSkolenikurzy()
    {
        $this->hlavickaVyberAkce();

        HtmlHeaders::getInstance()->addCss('css/skoleni.css');

        require_once ('person/b_skoleni_kurzy.php');
        $b = new SkoleniAKurzy($this->database);
        $b->load();
        return $b;
    }

    /**
     * @param int  $id
     * @param bool $editace
     * @param bool $postValues
     * @return ViewObject
     * @throws Exception
     * @throws DBException
     */
    function pripravenyDotaznik($id, $editace, $postValues)
    {
        if (!$id) throw new Exception(StringUtils::fromUTF8('Chyba! Není zadán Id dotazníku!'));

        $dotaznik = $this ->nactiHodnoceni($id, $editace);

        // kontrola, jestli se needituje uzavreny dotaznik
        if ($editace  &&  !$dotaznik ->jeEditovatelnyStav()) { // ukonceno, ukonceno-vraceno
            MessageManager::addErrorMessage( ronText('Nelze_Editovat_uzavrene_hodnoceni') );
            $editace = false;
        }

        // Nastavi hodnotitele dotazniku.
        if ($editace  &&  !$postValues) {
            if (!$dotaznik->nastavHodnotitele()) {
                return new MessageManagerViewObject(); // zobrazi chybove hlasky
            }
        }

        return new DotaznikViewObject($dotaznik, $editace, $postValues);
    }

    /** Provede ulozeni hodnoceni
     * @return Redirect|ViewObject
     * @throws Exception
     */
    function ulozHodnoceni()
    {
        $id = (int)$_GET['record'];
        if (!$id) throw new Exception(StringUtils::fromUTF8('Chyba! Není zadán Id dotazníku!'));

        $dotaznik = $this ->nactiHodnoceni($id, true); // editace = true

        $dotaznik->fUlozenoVXML = null;
        $dotaznik->fPrepoctenoVXML = null;

        if ($dotaznik->fXMLSablona) {
            $dotaznik->vyplnDotaznikXML();
        }

        // pokud nebyl dotaznik ulozen uz v XML, nebo nema XML, tak jej ulozi nyni
        if (isset($dotaznik->fUlozenoVXML)) {
            MessageManager::addDebugMessage(StringUtils::fromUTF8('Uložení odpovědí proběhlo přímo v XML formuláři'));
            $ulozeno_result = $dotaznik->fUlozenoVXML;
        } else {
            if ($dotaznik->fXMLSablona) {
                MessageManager::addDebugMessage(StringUtils::fromUTF8('Uložení odpovědí neproběhlo v XML -> proběhne nyní'));
            }

            try {
                $ulozeno_result = $dotaznik->ulozVyplnenyDotaznik();
            } catch (RonException $e) {
                MessageManager::addErrorMessage($e ->getMessage(), $e ->fieldName, null, $e ->getErrorDescription());
                $ulozeno_result = false;
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e ->getMessage());
                $ulozeno_result = false;
            }
        }

        if ($ulozeno_result) {
            MessageManager::addInfoMessage(ronText('Hodnoceni_ulozeno'));
        } else {
            return new DotaznikViewObject($dotaznik, true, true);
        }

        // test, jestli se ma zaroven s ulozenim vratit nebo uzavrit (stisknuto tlacitko "uzavrit")
        if (isset($_POST['uzavrit']) && $_POST['uzavrit']) {
            if ($dotaznik->fVyplnenoVse) {
                return $this ->uzavriHodnoceni($id);
            }

            if (!$dotaznik->fVyplnenoVse) {
                MessageManager::addErrorMessage( ronText('Nelze_uzavrit_nevyplnene_hodnoceni') );
            }

            // pouzit redirect ? ? ?
            return $this ->pripravenyDotaznik($id, true, false); // editace = true, postvalues = false (uz je ulozeno)

        } else {
            // jen ulozeni - zobrazi v readonly
            return new Redirect(null, '?menuid=skoleni&action=hodnoceni&record=' . $id);
        }
    }


    /**
     * @param $timestampTerminuHodnoceni
     * @return Redirect
     * @throws Exception
     */
    private function zapisUcastiNaSkoleni($timestampTerminuHodnoceni)
    {
        $this->pravoNaZapisUcastAHodnoceni();

        try {
            $this->ulozZmenenyZaznam($this->database, $timestampTerminuHodnoceni);
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
        }

        return new Redirect(null, '?menuid=skoleni&action=skoleniucast');
    }

    /**
     * @return Redirect
     * @throws Exception
     */
    private function zapisHodnoceniSkoleni()
    {
        $this->pravoNaZapisUcastAHodnoceni();

        try {
            $this->ulozZmenyHodnoceni($this->database);
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
        }

//        return new Redirect(null, '?menuid=skoleni&action=skolenihodnoceni&record=' . $cisloSkoleni);
        return new Redirect(null, '?menuid=skoleni&action=skolenikurzy');
    }

    /**
     * Uložení Účasti na školení
     * @param Database $database
     * @param $timestampTerminuSkoleni
     * @throws Exception
     */
    function ulozZmenenyZaznam($database, $timestampTerminuSkoleni)
    {
        $RcUcastnika = '';
        $poleRCaOsC = $this->nactiRCaOsobniCislaUGenerovanehoSkoleni($database, $_SESSION['kodDlouhodobehoKurzu']);
        $this->pravoNaZapisUcastAHodnoceni();

        $druhSkoleni = $_SESSION['kodDlouhodobehoKurzu'];

        for ($i = 1; $i <= $_POST['pocetRadku']; $i++) {

            $idUcastnika_post =  $i . "_idUcastnika";
            $idUcastnika = $_POST[$idUcastnika_post];

            $poznamkaKUchazeci_post =  $i . "_poznamkaKUchazeci";
            $poznamkaKUchazeci = $_POST[$poznamkaKUchazeci_post];

            //Procházím pole účastníků, které se jednou načte z SQL
            foreach ($poleRCaOsC as $osoba => $oc){
                if($oc[0] == $idUcastnika) {
                    $RcUcastnika = $oc[1];
                    //Pokud je RC nalezeno ukončím foreach.
                    break 1;
                }
            }

            $pritomenNaTerminu_post =  $i . "_pritomenNaTerminu";
            $pritomenNaTerminu = ($_POST[$pritomenNaTerminu_post] != "") ? $_POST[$pritomenNaTerminu_post] : "0";

            $omluvenaUcast_post =  $i . "_omluvenaUcast";
            $omluvenaUcast = ($_POST[$omluvenaUcast_post] != "") ? $_POST[$omluvenaUcast_post] : "0";


            $puvodniHodnoty_post =  $i . "_idUcastnikaPuvodniHodnoty";
            $puvodniHodnoty = explode(";;", $_POST[$puvodniHodnoty_post]);


            $zmenaUcasti = false;
            if (($puvodniHodnoty[0] != $pritomenNaTerminu) || ($puvodniHodnoty[1] != $omluvenaUcast) || ($puvodniHodnoty[2] != $poznamkaKUchazeci)){
                $zmenaUcasti = true;
            }

            if ($zmenaUcasti)
            {
                require_once('adsapp/ronportaldbrecord.php');
                $record = new RonPortalDBRecord($database, 'UcastNaTerminuSkoleni');
                $record->logDescription = 'Editace ucasti na skoleni - portal';

                $record->addField('CisloSkoleni', '', FALSE, 10, $druhSkoleni);
                $record->addField('Datum', 'timestamp', FALSE, 0, $timestampTerminuSkoleni);
                $record->addField('RC', '', FALSE, 15, $RcUcastnika);
                $record->addField('Pritomen', 'int', FALSE, 0, $pritomenNaTerminu);
                $record->addField('DuvodNepritomnosti', 'int', FALSE, 0, ($pritomenNaTerminu == 1) ? 0 : $omluvenaUcast);
                $record->addField('Poznamka', 'memo', FALSE, 0, $poznamkaKUchazeci);

                $params = $database->createParameters();
                $record->setWhere( "RC = " . $params->addParameter($RcUcastnika) .
                    " AND CisloSkoleni = " . $params->addParameter($druhSkoleni) .
                    " AND Datum = " . $params->addParameter($this->database->dateToParam($timestampTerminuSkoleni)), $params);

                if (!$record->updateRecord()) {
                    throw new Exception(ronText('zmena_ucasti_na_terminu_neprobehla'));
                }
            }
        }
    }


    /**
     * @throws Exception
     */
    private function pravoNaZapisUcastAHodnoceni()
    {
        $prava = Prava::vratPrava('skoleni_absolvovani');
        Prava::nastavObdobiProPrava(time(), time());

        if (!$prava->jsouNejakePrava()){
            throw new Exception(ronText('neni_pravo'));
        }
    }

    /**
     * @param Database $database
     * @param $kurz
     * @return array
     * @throws Exception
     */
    private function nactiRCaOsobniCislaUGenerovanehoSkoleni($database, $kurz)
    {
        $result = [];

        $params = $database->createParameters();

        $sql = "SELECT PracovniPomer.ID, PracovniPomer.RC, Osoba.Prijmeni " .
               " FROM GenerovaneSkoleni INNER JOIN Osoba ON (Osoba.RC = GenerovaneSkoleni.RC) INNER JOIN PracovniPomer ON (PracovniPomer.RC = GenerovaneSkoleni.RC) " .
               " WHERE GenerovaneSkoleni.CisloSkoleni = " . $params->addParameter($kurz) .
               " ORDER BY PracovniPomer.ID ";
        $cursor = $database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()){
            $result[] = $row;
        }
        $cursor->close();

        return $result;
    }

    /**
     * Uložení Hodnocení
     * @param Database $database
     * @throws Exception
     */
    function ulozZmenyHodnoceni($database)
    {
        $RcUcastnika = '';
        $druhSkoleni = $_POST['druhSkoleni'];
        $cisloSkoleni = $_POST['cisloSkoleni'];

        $poleRCaOsC = $this->nactiRCaOsobniCislaUGenerovanehoSkoleni($database, $cisloSkoleni);

        for ($i = 1; $i <= $_POST['pocetRadku']; $i++)
        {
            //Datum vzniku povinnosti pro setWhere
            $timestampDatumVznikuPovinnosti_post =  $i . "_datumVznikuPovinnosti";
            $timestampDatumVznikuPovinnosti = (isset($_POST[$timestampDatumVznikuPovinnosti_post]) && $_POST[$timestampDatumVznikuPovinnosti_post] != "") ? $_POST[$timestampDatumVznikuPovinnosti_post] : "";

            //ID pro vyhledání Rč
            $idUcastnika_post =  $i . "_idUcastnika";
            $idUcastnika = (isset($_POST[$idUcastnika_post]) && $_POST[$idUcastnika_post] != "") ? $_POST[$idUcastnika_post] : "";

            //Hodnocení daného účastníka
            $hodnoceniUcastnika_post =  $i . "_hodnoceniCombo";
            $hodnoceniUcastnika = $_POST[$hodnoceniUcastnika_post];

            //Komentář hodnocení účastníka
            $slovniHodnoceniUcastnika_post =  $i . "_slovniHodnoceni";
            $slovniHodnoceniUcastnika = $_POST[$slovniHodnoceniUcastnika_post];

            //Procházím pole účastníků, které se jednou načte z SQL
            foreach ($poleRCaOsC as $osoba => $oc){
                if($oc[0] == $idUcastnika) {
                    $RcUcastnika = $oc[1];
                    //Pokud je RC nalezeno skončit foreach.
                    break 1;
                }
            }

            $puvodniHodnoty_post =  $i . "_idUcastnikaPuvodniHodnotyHodnoceni";
            $puvodniHodnoty = explode(";;", filter_input(INPUT_POST, $puvodniHodnoty_post, FILTER_SANITIZE_SPECIAL_CHARS));

            $zmenaUcasti = false;
            if (($puvodniHodnoty[0] != $hodnoceniUcastnika) || ($puvodniHodnoty[1] != $slovniHodnoceniUcastnika)){
                $zmenaUcasti = true;
            }

            if ($zmenaUcasti) {
                require_once('adsapp/ronportaldbrecord.php');
                $record = new RonPortalDBRecord($database, 'GenerovaneSkoleni');
                $record->logDescription = 'Hodnoceni ucasti na skoleni - portal';

                $record->addField('RC', '', FALSE, 15, $RcUcastnika);
                $record->addField('DruhSkoleni', '', FALSE, 10, $druhSkoleni);
                $record->addField('DatumVznikuPovinnosti', 'timestamp', FALSE, 0, $timestampDatumVznikuPovinnosti);
                $record->addField('Potvrzeni', 'int', FALSE, 0, $hodnoceniUcastnika);
                $record->addField('Poznamka', 'memo', FALSE, 0, $slovniHodnoceniUcastnika);

                $params = $this->database->createParameters();
                $record->setWhere('RC = ' . $params->addParameter($RcUcastnika) .
                    ' AND DruhSkoleni = ' . $params->addParameter($druhSkoleni) .
                    ' AND DatumVznikuPovinnosti = ' . $params->addParameter($this->database->timestampToParam($timestampDatumVznikuPovinnosti)), $params);


                if (!$record->updateRecord()) {
                    throw new Exception(ronText('zmena_hodnoceni_neprobehla'));
                }
            }
        }
    }


    /**
     * @param $kodKurzu
     * @param $datumKurzu
     * @param $casKurzu
     * @param $poznamkaKurzu
     * @return Redirect
     * @throws Exception
     */
    private function vytvoreniTerminuSkoleni($kodKurzu, $datumKurzu, $casKurzu, $poznamkaKurzu)
    {
        $prava = Prava::vratPrava('skoleni_absolvovani');
        Prava::nastavObdobiProPrava(time(), time());

        if (!$prava->jsouNejakePrava()){
            throw new Exception(ronText('Nemate_pravo'));
        }

        //Převod z textu do timestamp
        $datum = DateUtils::dateStringToTimestamp($datumKurzu);
        $cas = DateUtils::timeStringToTimestamp($casKurzu);

        $params = $this->database->createParameters();
        //Test zdali je termín volný
        $sql = "SELECT Datum FROM TerminSkoleni WHERE CisloSkoleni = " . $params->addParameter($kodKurzu) . " AND Datum = " . $params->addParameter($this->database->timestampToParam($datum));

        $terminJeVolny = DBUtils::getSqlValue($this->database, $sql, $params);

        if ($terminJeVolny == ""){
            $_SESSION['terminJeObsazen'] = 0;
            $_SESSION['poznamkaNovehoTerminu'] = "";

            require_once('adsapp/ronportaldbrecord.php');
            $record = new RonPortalDBRecord($this->database, 'TerminSkoleni');

            $record->logDescription = 'Vytvoreni terminu ucasti na skoleni';

            $record->addField('CisloSkoleni', '', FALSE, 10, $kodKurzu);
            $record->addField('Datum', 'timestamp', FALSE, 0, $datum);
            $record->addField('Cas', 'time', FALSE, 0, $cas);
            $record->addField('Poznamka', 'memo', FALSE, 0, $poznamkaKurzu);

            if (!$record->insertRecord()) {
                MessageManager::addInfoMessage(ronText('vytvoreni_terminu_selhalo'));
            } else {
                //Pokud zapis úspešný - připojit účastníky zájezdu
                $params = $this->database->createParameters();
                $sql = "SELECT GS.RC " .           //0
                    "FROM GenerovaneSkoleni GS " .
                    "JOIN Osoba ON (Osoba.RC = GS.RC) " .
                    "LEFT JOIN TerminSkoleni ON (TerminSkoleni.CisloSkoleni=GS.CisloSkoleni) LEFT JOIN PracovniPomer On (Osoba.RC = PracovniPomer.RC)" .
                    "LEFT JOIN UcastNaTerminuSkoleni UTS ON (UTS.CisloSkoleni=TerminSkoleni.CisloSkoleni AND UTS.RC=GS.RC AND UTS.Datum=TerminSkoleni.Datum) " .
                    "WHERE (GS.CisloSkoleni = " . $params->addParameter($kodKurzu) . ") AND (GS.Potvrzeni<5)  AND TerminSkoleni.Datum = " . $params ->addParameter($this ->database ->timestampToParam($datum)) . " " .
                    " AND PracovniPomer.ID = (SELECT Max(PP.ID) FROM PracovniPomer PP Where PP.RC = GS.RC) " .
                    "ORDER BY RC";

                $cursor = $this->database->getSqlResult($sql, $params);

                while ($row = $cursor->fetch()) {

                    $record = new RonPortalDBRecord($this->database, 'UcastNaTerminuSkoleni');
                    $record->logDescription = StringUtils::fromUTF8('Připojení účastníka k termínu');

                    $record->addField('CisloSkoleni', '', FALSE, 10, $kodKurzu);
                    $record->addField('Datum', 'timestamp', FALSE, 0, $datum);
                    $record->addField('RC', '', FALSE, 15, $row[0]);
                    $record->addField('Pritomen', 'int', FALSE, 0, 0);
                    $record->addField('DuvodNepritomnosti', 'int', FALSE, 0, 0);
                    $record->addField('Poznamka', 'memo', FALSE, 0, "");

                    if (!$record->insertRecord()) {
                        throw new Exception(ronText('pripojeni_na_termin_selhalo'));
                    }
                }
            }
        } else {
            $_SESSION['terminJeObsazen'] = 2;
            $_SESSION['poznamkaNovehoTerminu'] = $poznamkaKurzu;
        }
        return new Redirect(null, '?menuid=skoleni&action=skoleniucast');
    }


    /**
     * Uzavreni dotazniku (kliknuti na tlacitko Odeslat nebo Vratit k doplneni)
     * @param int $id
     * @return Redirect
     * @throws DBException
     * @throws Exception
     */
    public function uzavriHodnoceni($id)
    {
        if (!$id) throw new Exception(StringUtils::fromUTF8('Chyba! Není zadán Id dotazníku!'));

        MessageManager::addDebugMessage("HodnoceniController::uzavriDotaznik id=$id ");

        $params = $this->database->createParameters();

        // zapise jmeno hodnoticiho a cas
        $sql =
            'UPDATE PripraveneTemaHodnoceni ' .
            "   SET HodnotilRC = " . $params ->addParameter(User::getRC()) . ", " .
            "       Hodnotici = " . $params ->addParameter(RonPortalUtils::getLogUserName()) . ", " .
            '       DatumHodnoceni = ' . $params->addParameter($this ->database ->timestampToParam(time())) . ', ' .
            '       Stav = 2 '  . //2 - hodnocení ukončeno
            ' WHERE Id = ' . $params->addParameter($id);

        $result = $this ->database ->sql($sql, $params);
        if (!$result) {
            throw new DBException($this ->database, $sql, StringUtils::fromUTF8('Chyba při nastavení hodnotitele!'));
        }

        return new Redirect(null, "?menuid=skoleni");
    }

    /**
     * @param $kodKurzu     //Kod
     * @param $datumKurzu   //datum
     * @param $casKurzu        //Čas
     * @param $poznamkaKurzu    //Poznamka
     * @return Redirect
     * @throws Exception
     */
    private function editaceTerminuSkoleni($kodKurzu, $datumKurzu, $casKurzu, $poznamkaKurzu)
    {
        $prava = Prava::vratPrava('skoleni_absolvovani');
        Prava::nastavObdobiProPrava(time(), time());

        if (!$prava->jsouNejakePrava()){
            throw new Exception(ronText('Nemate_pravo'));
        }

        //Převod z textu do timestamp
//        $datum = DateUtils::dateStringToTimestamp($datumKurzu);
        $cas = DateUtils::timeStringToTimestamp($casKurzu);

        $_SESSION['terminJeObsazen'] = 0;
        $_SESSION['poznamkaNovehoTerminu'] = "";

        require_once('adsapp/ronportaldbrecord.php');
        $record = new RonPortalDBRecord($this->database, 'TerminSkoleni');

        $record->logDescription = 'Editace terminu na skoleni';

        $record->addField('Cas', 'time', FALSE, 0, $cas);
        $record->addField('Poznamka', 'memo', FALSE, 0, $poznamkaKurzu);

        $params = $this->database->createParameters();

        $record->setWhere(" CisloSkoleni = " . $params->addParameter($kodKurzu) .
                         " AND Datum = " . $params->addParameter($this->database->timestampToParam($datumKurzu)), $params);

        if (!$record->updateRecord()) {
            MessageManager::addInfoMessage(ronText('editace_terminu_selhala'));
        }

        return new Redirect(null, '?menuid=skoleni&action=skoleniucast');
    }


    /**
     * @return PlanovaneSkoleniKurzy
     * @throws Exception
     */
    private function zobrazPlanovaneSkoleni()
    {
        $prava = Prava::vratPrava('skoleni_plan');
        Prava::nastavObdobiProPrava(time(), time());

        if (!$prava->jsouNejakePrava()){
            throw new Exception(ronText('Nemate_pravo'));
        }

        HtmlHeaders::getInstance()->addCss('css/skoleni.css');
        HtmlHeaders::getInstance()->addScript('script/skolenihodnoceni.js');

        $this->hlavickaVyberAkce();

        require_once ('person/b_planovanekurzyskoleni.php');
        $b = new PlanovaneSkoleniKurzy($this->database);
        $b->load();
        return $b;
    }


    /**
     * @param Database $database
     * @param $selected
     * @return mixed
     * @throws DBException
     */
    private function comboTypuSkoleni($database, $selected = null)
    {
        $sql = "SELECT Kod, Popis FROM DruhSkoleni WHERE Aktivni = 1 ORDER BY Popis";

        $combo = new ComboBox('druhSkoleni');
        $combo->emptyRow = '';
        $combo->alwaysShowSelected = isset($value) && $value != '';
        $combo->htmlParams = "title='Typ skoleni'";
        $combo->className .= 'adsfilterforminput_full nonempty';
        $combo->emptyRow = ronText('nevybrano');
        $combo->showCode = true;
        $combo->loadData($database, $sql, $selected);

        return $combo;
    }


    /**
     * @param Database $database
     * @param $selected
     * @param PlanovaneSkoleniObjekt |null $hodnotyPlanu
     * @return mixed
     * @throws DBException
     * @throws Exception
     */
    private function comboStavuSkoleni($database, $selected = null, $hodnotyPlanu = null)
    {
        $sqlWhere = '';
        $prava = Prava::vratPrava('skoleni_schvaleni_planu');
        Prava::nastavObdobiProPrava(time(), time());
        if (Prava::vratHodnotuPrava('skoleni_schvaleni_planu') != 3)
        {
            //Pokud nemá právo na vše, pak testovat zda může upravovat zvolené
            if ($selected === null)
            {
                //Při Insert testovat zda mám právo na přihlášenou osobu na portále
                if ($prava->lzeZobrazitRC(User::getRC(), time(), time()))
                {
                    //Má právo na Schválení k přihlašování pro sebe (nemůže schvalovat jiné - ale sebe ano) - přístupny všechny možnosti
                    $selected = 0;
                } else {
                    //Nemá právo na Schválení k přihlašování (combo má tedy jedinou možnost - Požadavek)
                    $selected = -1;
                    $sqlWhere = "Where Kod IN (-1)";
                }

            } else {
                //Edit - existuje alespoň jedna přihlášená osoba - autor, testovat zda mám právo na autora
                //Autor 1. osoba na kurz přihlášena - mela by být pouze 1, tedy MIN(DatumVlozeni)
                $params = $database->createParameters();

                $sql = "SELECT RC FROM PlanovaneSkoleniOsob WHERE (CisloSkoleni = " . $params->addParameter($hodnotyPlanu->cisloSkoleni) . ") 
                     AND DatumVlozeni = (Select MIN(DatumVlozeni) FROM PlanovaneSkoleniOsob WHERE (CisloSkoleni = " . $params->addParameter($hodnotyPlanu->cisloSkoleni) . ") )";

                $autorPlanu = $database->getSqlResult($sql, $params);
                $row = $autorPlanu->fetch();
                if ($prava->lzeZobrazitRC($row[0], time(), time()))
                {
                    //Má právo na Schválení k přihlašování - a může tedy zmenit stav
                    $sqlWhere = '';

                } else {

                    if ($selected > -1) {
                        //Nemá právo na Schválení k přihlašování ale Schválení k přihlašování je nastaveno - nemenit
                        $sqlWhere = "Where Kod IN ($selected)";
                    } else {
                        //Nemá právo na Schválení k přihlašování a proto zůstane Požadavek
                        $sqlWhere = "Where Kod IN (-1)";
                    }
                }
            }
        }

        $sql = "SELECT Kod, Popis FROM S_StavPlanovanehoSkoleni {$sqlWhere}";

        $combo = new ComboBox('stav');
        $combo->alwaysShowSelected = isset($value) && $value != '';
        $combo->htmlParams = "title='Stav skoleni'";
        $combo->className .= 'adsfilterforminput_full';
        $combo->loadData($database, $sql, $selected);

        return $combo;
    }

    /**
     * @param Database $database
     * @param $firma
     * @return ComboBox
     * @throws DBException
     */
    public function comboFirmaSkoleni($database, $firma=null)
    {
        $params = $this->database->createParameters();
        $sqlWhere = $firma != "" ? ("Where kodFirmy = " . $params->addParameter($firma)) : "";

        $sql = "SELECT kodFirmy AS Kod, COALESCE(nazev1  + ' (' + nazev2 + ')', nazev1) AS Popis FROM Firma {$sqlWhere} ORDER BY Popis";

        $combo = new ComboBox('skoliciFirma');
        $combo->emptyRow = ronText('nevybrano');
        $combo->alwaysShowSelected = isset($value) && $value != '';
        $combo->htmlParams = "title='Skolici firma'";
        $combo->className .= 'adsfilterforminput_full';
        $combo->loadData($database, $sql, $firma, $params);

        return $combo;
    }


    /**
     * @param Database $database
     * @param $cisloSkoleni
     * @return array
     * @throws Exception
     */
    public function comboNakladuSkoleni($database, $cisloSkoleni)
    {
        $comboNakladovePolozky = [];
        $params = $database->createParameters();

        $sql = "SELECT TP.Kod, TP.Popis, TP.SazbaDPH
                FROM TypPolozky TP
                
                WHERE (NakladNaTermin = 0) 
                AND NOT (EXISTS (SELECT TypPolozky FROM PlanovanaCenaSkolCelkova WHERE CisloSkoleni = " . $params->addParameter($cisloSkoleni) . " 
                AND TP.kod = PlanovanaCenaSkolCelkova.TypPolozky))
                ORDER BY Popis";

        $cursor = $this->database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()){
            $nakladovaPolozka = [];
            $nakladovaPolozka['kod'] = $row[0];
            $nakladovaPolozka['popis'] = $row[1];
            $nakladovaPolozka['sazbaDPH'] = $row[2];

            $comboNakladovePolozky[] = $nakladovaPolozka;
        }

        return $comboNakladovePolozky;
    }

    /**
     * @param $cisloSkoleni
     * @return array
     * @throws Exception
     */
    public function nakladovePolozkyPlanovanehoSkoleni($cisloSkoleni)
    {
        $params = $this->database->createParameters();

        $nakladovePolozky = [];
        $sql =
            "SELECT PC.CisloSkoleni, PC.TypPolozky, PC.SazbaDPH, PC.DPH, PC.CenaSDPH, TypPolozky.Popis
             FROM PlanovanaCenaSkolCelkova PC LEFT JOIN TypPolozky ON (TypPolozky.Kod = PC.TypPolozky)
             WHERE PC.CisloSkoleni = " . $params->addParameter($cisloSkoleni) .
            " ORDER BY Popis"    ;

        $cursor = $this->database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()){
            $nakladovaPolozka = [];
            $nakladovaPolozka['cisloSkoleni'] = $row[0];
            $nakladovaPolozka['typPolozky'] = $row[1];
            $nakladovaPolozka['sazbaDPH'] = $row[2];
            $nakladovaPolozka['DPH'] = $row[3];
            $nakladovaPolozka['cenaSDPH'] = $row[4];
            $nakladovaPolozka['popis'] = $row[5];

            $cenaBezDPH = 0;
            if($row[2] == 0){
                //Pokud neni DPH pak je cena bez DPH jako s DPH
                $cenaBezDPH = $row[4];
            }

            if ($row[2] > 0  && $row[4] > 0){
                $koeficient = $row[2] / ($row[2]+100);
                $cenaBezDPH = $row[4]  - ($koeficient * $row[4]);
            }

            $nakladovaPolozka['cenaBezDPH'] = $cenaBezDPH;

            $nakladovePolozky[] = $nakladovaPolozka;
        }

        return $nakladovePolozky;
    }

    /**
     * @param $cisloSkoleni
     * @param PlanovaneSkoleniObjekt | null $nactenyPost
     * @return BaseViewObject
     * @throws Exception
     */
    public function detailPlanovaneSkoleni($cisloSkoleni, $nactenyPost = null)
    {
        $prava = Prava::vratPrava('skoleni_plan');
        Prava::nastavObdobiProPrava(time(), time());

        $params = $this->database->createParameters();
        require_once ("adsapp/ppomer.php");
        $prihlasenaOsoba = DBUtils::getSqlValue($this->database, "SELECT ID FROM PracovniPomer WHERE RC = " . $params->addParameter(User::getRC()), $params);

        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $prihlasenaOsoba);


        $params = $this->database->createParameters();
        $sql = "Select RC From PlanovaneSkoleniOsob 
                            Where DatumVlozeni = (Select MIN(DatumVlozeni) From PlanovaneSkoleniOsob Where PlanovaneSkoleniOsob.CisloSkoleni = " . $params->addParameter($cisloSkoleni) .")";
        $autorPlanu = DBUtils::getSqlValue($this->database, $sql, $params);


        if (!$prava->lzeZobrazitRC($autorPlanu)){
            if ($prava->prava != 3) {
                throw new Exception(ronText('Nemate_pravo'));
            }
        }

        HtmlHeaders::getInstance()->addCss('css/skoleni.css');
        HtmlHeaders::getInstance()->addScript('script/skoleniucast.js');


        $nakladovePolozky = $this->nakladovePolozkyPlanovanehoSkoleni($cisloSkoleni);

        require_once ('person/planovaneskoleni.php');
        /** @var PlanovaneSkoleniObjekt $detailPlanovanehoSkoleni */
        $detailPlanovanehoSkoleni = PlanovaneSkoleniObjekt::nactiPlanovaneSkoleni($this->database, $cisloSkoleni);

        Model::addData("pp", $pp);
        Model::addData("nactenyPost", $nactenyPost);
        Model::addData("nakladovePolozky", $nakladovePolozky);

        Model::addData('comboNakladuSkoleni', $this->comboNakladuSkoleni($this->database, $cisloSkoleni));
        Model::addData('comboFirmaSkoleni', $this->comboFirmaSkoleni($this->database, $detailPlanovanehoSkoleni->firma));
        Model::addData("detailPlanovanehoSkoleni", $detailPlanovanehoSkoleni);

        Model::addData('comboTypuSkoleni', $this->comboTypuSkoleni($this->database, $nactenyPost != "" ? $nactenyPost->druhSkoleni : $detailPlanovanehoSkoleni->druhSkoleni));
        Model::addData('comboStavuSkoleni', $this->comboStavuSkoleni($this->database, $detailPlanovanehoSkoleni->stav, $detailPlanovanehoSkoleni));
        return new  BaseViewObject("person/view_vytvoreniskoleni.php");
    }

    /**
     * @param PlanovaneSkoleniObjekt|null $nactenyPost
     * @return BaseViewObject
     * @throws Exception
     */
    public function vytvoreniPlanovanehoSkoleni($nactenyPost = null)
    {
        $prava = Prava::vratPrava('skoleni_plan');
        Prava::nastavObdobiProPrava(time(), time());

        if (!$prava->jsouNejakePrava()){
            RonPortalUtils::neniPravoException();
        }

        $this->hlavickaVyberAkce();

        HtmlHeaders::getInstance()->addCss('css/skoleni.css');
        HtmlHeaders::getInstance()->addScript('script/skoleniucast.js');

        $params = $this->database->createParameters();
        require_once ("adsapp/ppomer.php");
        $id = DBUtils::getSqlValue($this->database, "SELECT MAX(ID) FROM PracovniPomer WHERE RC = " . $params->addParameter(User::getRC()), $params);

        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $id);

        Model::addData("pp", $pp);
        Model::addData("nactenyPost", $nactenyPost);

        Model::addData('comboFirmaSkoleni', $this->comboFirmaSkoleni($this->database ));
        Model::addData('comboTypuSkoleni', $this->comboTypuSkoleni($this->database, $nactenyPost->druhSkoleni));
        Model::addData('comboStavuSkoleni', $this->comboStavuSkoleni($this->database));
        return new  BaseViewObject("person/view_vytvoreniskoleni.php");
    }

    public function nactiPOSTProDialog()
    {
        $hodnoty = new stdClass();
        foreach ($_POST as $key => $value) {

            switch ($key){
                case "datumPocatkuSkoleni":
                case "datumUkonceniSkoleni":
                    try{
                        if ($value != "") {
                            $hodnoty->$key = DateUtils::dateStringToTimestamp($value);
                        }
                    } catch (Exception $e) {
                        MessageManager::addDebugLog($e, $key);
                        $hodnoty->$key = null;
                    }
                    break;

                case "casPocatkuSkoleni":
                case "casUkonceniSkoleni":
                    try{
                        if ($value != "") {
                            $hodnoty->$key = DateUtils::timeStringToTimestamp($value, $hodnoty->datumPocatkuSkoleni);
                        }
                    } catch (Exception $e) {
                        MessageManager::addDebugLog($e, $key);
                        $hodnoty->$key = null;
                    }
                    break;


                default:
                    $hodnoty->$key = $value;
            }
        }

        return $hodnoty;
    }


    /**
     * @param $cisloSkoleni
     * @return bool
     * @throws Exception
     */
    private function shodaCislaSkoleni($cisloSkoleni)
    {
        $params = $this->database->createParameters();
        $sql = "SELECT CisloSkoleni FROM PlanovaneSkoleni WHERE CisloSkoleni = " . $params->addParameter($cisloSkoleni);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if ($row[0])
            return true;

        return false;
    }

    /**
     * @throws Exception
     */
    public function upravaNakladuPlanovanehoSkoleni()
    {
        $cnt = 0;
        $polePolozek = [];
        $pocet_polozek = $_POST['pocet_polozek'];
        for ($i = 0; $i <= $pocet_polozek; $i++)
        {
            $polozka = "polozka". $cnt;

            $polePolozek[$cnt]['nakladovaPolozka'] = $_POST[$polozka . '_nakladovaPolozka'];
            $polePolozek[$cnt]['cenaBezDPH'] = $_POST[$polozka . '_cenaBezDPH'];
            $polePolozek[$cnt]['sazbaDPH'] = $_POST[$polozka . '_sazbaDPH'];
            $polePolozek[$cnt]['cenaSDPH'] = $_POST[$polozka . '_cenaSDPH'];
            ++$cnt;
        }

        foreach ($polePolozek as $polozka=>$hodnota)
        {
            $cisloSkoleni = StringUtils::htmlTextConvert($_POST['cisloSkoleni']);
            $sazbaDPH = $hodnota['sazbaDPH'];
            $vypocteneDPH = ($hodnota['cenaBezDPH'] / 100) * $sazbaDPH;
            $cenaSDPH = $hodnota['cenaBezDPH'] + $vypocteneDPH;

            require_once('adsapp/ronportaldbrecord.php');
            $rec = new RonPortalDBRecord($this->database, 'PlanovanaCenaSkolCelkova');
            $rec->addField('CisloSkoleni', '', false, 10, $cisloSkoleni);
            $rec->addField('TypPolozky', '', false, 10, $hodnota['nakladovaPolozka']);
            $rec->addField('SazbaDPH', 'float', false, 0, $sazbaDPH);
            $rec->addField('DPH', 'float', false, 0, $vypocteneDPH);
            $rec->addField('CenaSDPH', 'float', false, 0, $cenaSDPH);

            $params = $this->database->createParameters();
            $sqlWhere =
                ' CisloSkoleni = ' . $params->addParameter($cisloSkoleni) .
                ' AND TypPolozky = ' . $params->addParameter($hodnota['nakladovaPolozka']);
            $rec->setWhere($sqlWhere, $params);

            if ($rec->loadRecord() == 0) {
                if (!$rec->insertRecord()) {
                    throw new Exception(StringUtils::fromUTF8('Chyba při vložení nákladů plánu školení!'));
                }
            } else {
                if (!$rec->loadAndUpdateRecord()) {
                    throw new Exception(StringUtils::fromUTF8('Chyba při aktualizaci nákladů plánu školení!'));
                }
            }
        }
    }


    /**
     * @throws Exception
     */
    public function noveNakladyPlanovanehoSkoleni()
    {
        $hodnotyPolozky = explode(";;", $_POST['pridatTypPolozky']);

        require_once('adsapp/ronportaldbrecord.php');
        $rec = new RonPortalDBRecord($this ->database, 'PlanovanaCenaSkolCelkova');
        $rec ->addField('CisloSkoleni', '', false, 10, $_POST['cisloSkoleni']);
        $rec ->addField('TypPolozky', '', false, 10, $hodnotyPolozky[0]);
        $rec ->addField('SazbaDPH', 'float', false, 0, $hodnotyPolozky[1]);
        $rec ->addField('DPH', 'float', false, 0, 0);
        $rec ->addField('CenaSDPH', 'float', false, 0, 0);

        $params = $this->database->createParameters();
        $sqlWhere =
            ' CisloSkoleni = ' . $params->addParameter($_POST['cisloSkoleni']) .
            ' AND TypPolozky = ' . $params->addParameter($hodnotyPolozky[0]);
        $rec->setWhere($sqlWhere, $params);

        if ($rec->loadRecord() == 0) {
            if (!$rec->insertRecord()) {
                throw new Exception(StringUtils::fromUTF8('Chyba při vložení nákladů plánu školení!'));
            }
        }
        else {
            if (!$rec->loadAndUpdateRecord()) {
                throw new Exception(StringUtils::fromUTF8('Chyba při aktualizaci nákladů plánu školení!'));
            }
        }
    }


    /**
     * @param $cisloSkoleni
     * @return mixed
     * @throws DBException
     */
    private function pocetOsobNaKurzu($cisloSkoleni)
    {
        $params = $this->database->createParameters();
        $sql = "SELECT COUNT(RC) FROM PlanovaneSkoleniOsob PSO Where PSO.Potvrzeni = 1 AND PSO.CisloSkoleni = " . $params->addParameter($cisloSkoleni);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        return $row[0];
    }


    /**
     * @param $cislo
     * @return BaseViewObject|Redirect
     * @throws Exception
     */
    public function zapisPlanovanehoSkoleni($cislo)
    {
        /** @var PlanovaneSkoleniObjekt $hodnoty */
        $hodnoty = $this->nactiPOSTProDialog();
        if ($_POST['cisloSkoleni'] == "") {
            if ($this->shodaCislaSkoleni($hodnoty->cisloSkoleni)) {
                MessageManager::addInfoMessage(ronText('cislo_skoleni_existuje'));
                return $this->vytvoreniPlanovanehoSkoleni($hodnoty);
            }
        }

        if (isset($_POST['nakladyplanovanehoskoleni']) && isset($_POST['nakladyplanovanehoskoleni']) != "" && $_POST['cisloSkoleni'] != "") {
            $this->upravaNakladuPlanovanehoSkoleni();
            return $this->detailPlanovaneSkoleni($_POST['cisloSkoleni']);
        }


        if (isset($_POST['novenakladyplanovanehoskoleni']) && isset($_POST['novenakladyplanovanehoskoleni']) != "" && $_POST['cisloSkoleni'] != "") {
            $this->noveNakladyPlanovanehoSkoleni();
            return $this->detailPlanovaneSkoleni($_POST['cisloSkoleni']);
        }


        $pocetPrihlasenychNaKurz = $this->pocetOsobNaKurzu($cislo);
        if ($hodnoty->pocetOsob >= $pocetPrihlasenychNaKurz) {
            //Nelze snížit počet osob, pod přihlášený počet


            require_once('adsapp/ronportaldbrecord.php');
            $rec = new RonPortalDBRecord($this->database, 'PlanovaneSkoleni');
            $rec->addField('CisloSkoleni', '', false, 10, $hodnoty->cisloSkoleni);
            $rec->addField('Popis', '', false, 160, $hodnoty->popis);
            $rec->addField('DruhSkoleni', '', false, 10, $hodnoty->druhSkoleni);
            $rec->addField('Stav', 'int', false, 0, $hodnoty->stav != "" ? $hodnoty->stav : -1);


            $rec->addField('PocetOsob', 'int', false, 0, $hodnoty->pocetOsob);
            $rec->addField('KonkretniUcastnici', 'int', false, 0, $hodnoty->konkretniUcastnici != 1 ? 0 : 1);
            $rec->addField('DelkaSkoleniDnu', 'float', false, 0, $hodnoty->delkaSkoleniDnu);
            $rec->addField('DelkaSkoleniHodin', 'float', false, 0, $hodnoty->delkaSkoleniHodin);
            $rec->addField('DatumPocatkuSkoleni', 'timestamp', false, 0, $hodnoty->datumPocatkuSkoleni != "" ? $hodnoty->datumPocatkuSkoleni : null);
            $rec->addField('CasPocatkuSkoleni', 'timestamp', false, 0, $hodnoty->casPocatkuSkoleni != "" ? $hodnoty->casPocatkuSkoleni : null);
            $rec->addField('DatumUkonceniSkoleni', 'timestamp', false, 0, $hodnoty->datumUkonceniSkoleni != "" ? $hodnoty->datumUkonceniSkoleni : null);
            $rec->addField('CasUkonceniSkoleni', 'timestamp', false, 0, $hodnoty->casUkonceniSkoleni != "" ? $hodnoty->casUkonceniSkoleni : null);
            $rec->addField('Poznamka', 'memo', false, 0, $hodnoty->poznamka);
            $rec->addField('SkoliciFirma', '', false, 10, $hodnoty->firma);

            $rec->addField('Insert_UserName', '', true, 25, RonPortalUtils::getLogUserName());
            $rec->addField('Insert_Timestamp', 'timestamp', true, 0, time());
            $rec->addField('Update_UserName', '', false, 25, RonPortalUtils::getLogUserName())
                ->updateLog = true;
            $rec->addField('Update_Timestamp', 'timestamp', false, 0, time())
                ->updateLog = true;

            $params = $this->database->createParameters();
            $sqlWhere =
                ' CisloSkoleni = ' . $params->addParameter($hodnoty->cisloSkoleni);
            $rec->setWhere($sqlWhere, $params);


            if ($rec->loadRecord() == 0) {
                if (!$rec->insertRecord()) {
                    throw new Exception(StringUtils::fromUTF8('Chyba při vložení plánu školení!'));
                } else {
                    $this->pripojeniAutoraPlanovanehoKurzu($hodnoty);
                }
            } else {
                if (!$rec->loadAndUpdateRecord()) {
                    throw new Exception(StringUtils::fromUTF8('Chyba při aktualizaci plánu školení!'));
                }
            }

        } else {
            MessageManager::addErrorMessage(sprintf(ronText('nelze_snizit_pocet_osob_pod_prihlasene_mnozstvi'), $pocetPrihlasenychNaKurz));
        }



        //Pokud ok redirect na detail
        return new Redirect(null, '?menuid=skoleni&action=detailplanovanehoskolenikurzu&record=' . StringUtils::htmlTextConvert($cislo) );
    }


    /**
     * @param $hodnoty
     * @throws DBException
     * @throws Exception
     */
    public function pripojeniAutoraPlanovanehoKurzu($hodnoty)
    {
        //připojit User na školení - jako autora a pokud má právo na přihlášení dané osoby tak nastavit potvrzení na 1
        $pravoPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        Prava::nastavObdobiProPrava(time(), time());

        $potvrzeni = 0;
        if ($pravoPrihlasit->lzeZobrazitRC(User::getRC())){
            $potvrzeni = 1;
        }

        $recZapis = new RonPortalDBRecord($this->database, 'PlanovaneSkoleniOsob');
        $recZapis->logDescription = StringUtils::fromUTF8('Plánované školení - přihlašování na plánované školení');
        $recZapis->addField('CisloSkoleni', '', false, 10, $hodnoty->cisloSkoleni);
        $recZapis->addField('RC', '', false, 15, User::getRC());
        $recZapis->addField('Potvrzeni', 'int', false, 0, $potvrzeni);
        $recZapis->addField('Vlozil', '', false, 25, User::getRC());
        $recZapis->addField('DatumVlozeni', 'timestamp', false, 0, time());
        $recZapis->addField('Vyhodnotil', '', false, 25, '');
        $recZapis->addField('DatumVyhodnoceni', 'timestamp', false, 0, '');
        $recZapis->addField('PrihlasenoZAplikace', 'int', false, 0, 1);

        $params = $this->database->createParameters();
        $recZapis->setWhere( 'RC  = ' . $params->addParameter(User::getRC()) .
            ' AND CisloSkoleni = ' . $params->addParameter($hodnoty->cisloSkoleni), $params);

        if ($recZapis->loadRecord() == 0){
            if (!$recZapis->insertRecord()){
                throw new Exception(StringUtils::fromUTF8('Chyba při přihlašování na plánované školení.'));
            } else {
                MessageManager::addInfoMessage(StringUtils::fromUTF8('Byl/a jste na plánované školení přihlášen/a.'));
            }
        } else {
            MessageManager::addInfoMessage(StringUtils::fromUTF8('Již jste na plánované školení přihlášen/a.'));
        }
    }

    /**
     * @param JsonOutputObject $output
     * @throws DBException
     */
    private function mojeSkoleni($output)
    {
        require_once('ron/httputils.php');

        $params = $this ->database ->createParameters();
        $sql =
            "SELECT DruhSkoleni.Kod, COALESCE(Skoleni.DatumPocatkuSkoleni, GS.PozadovaneDatumSkoleni), 
                    DruhSkoleni.Popis, Skoleni.DatumPocatkuSkoleni, GS.PozadovaneDatumSkoleni
                FROM GenerovaneSkoleni GS 
                    LEFT JOIN DruhSkoleni ON (DruhSkoleni.Kod = GS.DruhSkoleni) 
                    LEFT JOIN Skoleni ON  (Skoleni.CisloSkoleni = GS.CisloSkoleni) 
                WHERE  (GS.RC= " . $params->addParameter(User::getRC()) .
            "  AND (GS.Potvrzeni=0 OR  (GS.Potvrzeni = 1  AND (Skoleni.StavSkoleni IS NULL  OR  Skoleni.StavSkoleni <> 2)) )
                OR GS.Potvrzeni=5 OR GS.Potvrzeni=6) " .
            ' ORDER BY COALESCE(Skoleni.DatumPocatkuSkoleni, GS.PozadovaneDatumSkoleni)';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $html =
            '<table class="dashboardTable" data-ron-href="?menuid=skoleni">';

        $html .=
            '<tr><th>' . ronText('Skoleni_termin_platnost') . '&nbsp;&nbsp;&nbsp;' .
            '</th><th>' . ronText('Skoleni_nazev_kurzu_skoleni') .
            '</th><th>' . ronText('Skoleni_termin_povinnosti') . '</th><th></th></tr>';


        while ($row = $cursor ->fetch()) {
            MessageManager::addDebugMessage($row[3]);
            $html .=
                '<tr>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[1])) . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($row[2]) . '</td>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[4])) . '</td>' .
                '</tr>';
        }

        $html .= '</table>';

        $output ->addText('html', $html);
        $output->addDebugMessage($sql);
    }


    /**
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function skoleniPodrizenych($output)
    {
        require_once('ron/httputils.php');
        $pravo = Prava::nactiPravaUzivatele($this->database,null,'skoleni_prohlizeni');

        $params = $this->database->createParameters();
        $html =
            '<table class="dashboardTable" data-ron-href="?menuid=skoleni">' .
            '<tr><th>' . ronText('Zamestnanec') .
            '</th><th>' . ronText('Skoleni_termin_platnost') . '&nbsp;&nbsp;' .
            '</th><th>' . ronText('Skoleni_nazev_kurzu_skoleni') .
            '</th><th>' . ronText('Skoleni_termin_povinnosti') .
            '</th><th></th></tr>';

        $sql =
            "SELECT DruhSkoleni.Kod, COALESCE(Skoleni.DatumPocatkuSkoleni, GenerovaneSkoleni.PozadovaneDatumSkoleni), 
                    DruhSkoleni.Popis, Osoba.Prijmeni, Osoba.Jmeno, Skoleni.DatumPocatkuSkoleni, GenerovaneSkoleni.PozadovaneDatumSkoleni 
                FROM (  (  ( GenerovaneSkoleni INNER JOIN Osoba ON (Osoba.RC = GenerovaneSkoleni.RC) INNER JOIN DruhSkoleni ON GenerovaneSkoleni.DruhSkoleni=DruhSkoleni.Kod) 
                INNER JOIN S_DuvodSkoleni ON GenerovaneSkoleni.DuvodSkoleni=S_DuvodSkoleni.Kod) 
                INNER JOIN S_StavPotvrzeni ON GenerovaneSkoleni.Potvrzeni=S_StavPotvrzeni.Kod) 
                LEFT OUTER JOIN Skoleni ON Skoleni.CisloSkoleni = GenerovaneSkoleni.CisloSkoleni 
                LEFT OUTER JOIN KategorieSkoleni ON DruhSkoleni.Kategorie=KategorieSkoleni.Kod 
                WHERE  (GenerovaneSkoleni.RC = Osoba.RC  AND (GenerovaneSkoleni.Potvrzeni=0 OR  (GenerovaneSkoleni.Potvrzeni = 1  AND (Skoleni.StavSkoleni IS NULL  OR  Skoleni.StavSkoleni <> 2)) )
                OR GenerovaneSkoleni.Potvrzeni=5 OR GenerovaneSkoleni.Potvrzeni=6) " .


        '  AND EXISTS(SELECT PracovniPomer.RC FROM PracovniPomer ' .
            '             WHERE PracovniPomer.RC = GenerovaneSkoleni.RC ' .
            '               AND PracovniPomer.DatumNastupu <= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) .
            '               AND (PracovniPomer.DatumUkonceni >= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
            '               AND ' .  $pravo->sqlFiltr('PracovniPomer', DateUtils::today(), DateUtils::today()) .
            '            )' .

            '   AND GenerovaneSkoleni.RC <> ' . $params->addParameter(User::getRC()) .

            ' ORDER BY COALESCE(Skoleni.DatumPocatkuSkoleni, GenerovaneSkoleni.PozadovaneDatumSkoleni)';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $html .=
                '<tr>' .
                '<td>' . StringUtils::htmlTextConvert($row[3]) . ' ' . StringUtils::htmlTextConvert($row[4]) . '</td>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[1])) . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($row[2]) . '</td>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[6])) . '</td>' .
                '</tr>';
        }

        $html .= '</table>';

        $output ->addText('html', $html);
    }


    /**
     * @return Redirect|ComboBoxZamestnanec
     */
    public function osobaVyslanaNaSkoleni()
    {
        $url = "?menuid=skoleni&action=vyslatnaskoleni";

        $prava = Prava::vratPrava('skoleni_vyslat');
        Prava::nastavObdobiProPrava(DateUtils::today(), DateUtils::today());

        $comboOsoby = new ComboBoxZamestnanec('osoba');
        $comboOsoby->className .= 'form-input nonempty';
        $comboOsoby->alwaysShowSelected = false;
        $comboOsoby->htmlParams = "title='" . ronText('skoleni_osoba_na_skoleni') . "'";
        $comboOsoby->htmlParams .= 'onChange="window.location.href = &quot;' . $url . '&amp;osoba=&quot; + encodeURIComponent(this.value)"';
        $comboOsoby->showCode = 1;
        $comboOsoby->loadDataPP($this->database, time(), time(), $prava, ronText('nevybrano'), '', false);

        return $comboOsoby;
    }


    /**
     * @param Database $database
     * @param int $osoba
     * @param $cisloSkoleni
     * @return ComboBox
     * @throws DBException
     */
    private function comboTypSkoleni($database, $osoba, $cisloSkoleni)
    {
        $sql = "SELECT Kod, Popis FROM DruhSkoleni WHERE Aktivni = 1 ORDER BY Popis";

        $url = "?menuid=skoleni&action=vyslatnaskoleni&osoba=" . $osoba;

        $combo = new ComboBox('druhSkoleni');
        $combo->alwaysShowSelected = isset($value) && $value != '';
        $combo->className .= 'form-input nonempty';
        $combo->htmlParams = "title='" . ronText('Skoleni_typ_skoleni') . "'";
        $combo->htmlParams .= 'onChange="window.location.href=&quot;' . $url . '&amp;druhskoleni=&quot; + encodeURIComponent(this.value)"';
        $combo->emptyRow = ronText('nevybrano');
        $combo->showCode = true;
        $combo->loadData($database, $sql, $cisloSkoleni);

        return $combo;
    }

    /**
     * @param Database $database
     * @param int $osoba
     * @param $druhSkoleni
     * @param null $selected
     * @return ComboBox
     * @throws DBException
     */
    private function comboKurz($database, $osoba, $druhSkoleni, $selected = null)
    {
        $params = $database->createParameters();
        $sql = "SELECT CisloSkoleni, Popis FROM Skoleni WHERE ((StavSkoleni = 0) OR (StavSkoleni = 1)) AND DruhSkoleni = " .
            $params->addParameter($druhSkoleni) . "  ORDER BY Popis";

        $url = "?menuid=skoleni&action=vyslatnaskoleni&osoba=" . $osoba . "&druhskoleni=" . $druhSkoleni;

        $combo = new ComboBox('kurz');
        $combo->readOnly = (bool)!$druhSkoleni;
        $combo->alwaysShowSelected = isset($value) && $value != '';
        $combo->className .= 'form-input';
        $combo->htmlParams = 'onChange="window.location.href=&quot;' . $url . '&amp;kurz=&quot; + encodeURIComponent(this.value)"';
        $combo->emptyRow = ronText('nevybrano');
        $combo->showCode = true;
        $combo->loadData($database, $sql, $selected, $params);

        return $combo;
    }


    /**
     * @param $kurz
     * @return array|bool
     * @throws DBException
     */
    public function naplnenostKurzu($kurz)
    {
        $params = $this->database->createParameters();

        $sql = "SELECT Skoleni.Kapacita,  Count(*) AS 'Prihlasenych' 
                FROM GenerovaneSkoleni GS LEFT JOIN Skoleni ON (Skoleni.CisloSkoleni = GS.CisloSkoleni)
                Where GS.CisloSkoleni = " . $params->addParameter($kurz) . " AND GS.Potvrzeni < 5
                
                GROUP BY GS.DruhSkoleni, GS.CisloSkoleni, Skoleni.Kapacita";

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();

        $naplnenostKurzu = [];
        if (!empty($row) ){
            $naplnenostKurzu[0] = $row[0];
            $naplnenostKurzu[1] = $row[1];
            $naplnenostKurzu[2] = $row[0] - $row[1];
        } else {
            $naplnenostKurzu[2] = -1;
        }

        return $naplnenostKurzu;
    }



    /**
     * @param int $osoba
     * @param $druhSkoleni
     * @param $kurz
     * @return BaseViewObject|Redirect
     * @throws DBException
     * @throws Exception
     */
    public function vyslatNaSkoleni($osoba, $druhSkoleni, $kurz)
    {
        if (Prava::vratHodnotuPrava('skoleni_vyslat') == 0) {
            RonPortalUtils::neniPravoException();
        }

        $this->hlavickaVyberAkce();

        $comboOsoby = $this->osobaVyslanaNaSkoleni();
        $combo_typ_skoleni = $this->comboTypSkoleni($this->database, $osoba, $druhSkoleni);
        $combo_kurz = $this->comboKurz($this->database, $osoba, $druhSkoleni, $kurz);

        $naplnenostKurzu[2] = -1;
        if ($kurz != '') $naplnenostKurzu = $this->naplnenostKurzu($kurz);

        Model::addData('combo_typ_skoleni', $combo_typ_skoleni);
        Model::addData('combo_kurz', $combo_kurz);
        Model::addData('comboOsoby', $comboOsoby);
        Model::addData('kurz', $kurz);
        Model::addData('naplnenostKurzu', $naplnenostKurzu);

        return new BaseViewObject('person/view_zadaniskolenizamestnanci.php');
    }


    /**
     * @throws Exception
     */
    public function zapisVyslaniZamestnanceNaSkoleni()
    {
        $hodnoty = new stdClass();
        foreach ($_POST as $key => $value) {
            switch ($key){
                case "pozadovanedatumskoleni":
                    try{
                        if ($value != "") {
                            $hodnoty->$key = DateUtils::dateStringToTimestamp($value);
                        }
                    } catch (Exception $e) {
                        MessageManager::addErrorMessage($e, $key);
                        $hodnoty->$key = null;
                    }
                    break;

                default:
                    $hodnoty->$key = filter_input(INPUT_POST, $key, FILTER_SANITIZE_SPECIAL_CHARS);
            }
        }

        if ($hodnoty->osoba == ''){
            throw new Exception(StringUtils::fromUTF8('Není zadaná osoba!'));
        }
        if ($hodnoty->druhSkoleni == ''){
            throw new Exception(StringUtils::fromUTF8('Není zadán typ školení!'));
        }
        if ($hodnoty->pozadovanedatumskoleni == ''){
            throw new Exception(StringUtils::fromUTF8('Není zadáno požadované datum školení!'));
        }

        if ($hodnoty->pozadovanedatumskoleni < DateUtils::today()){
            throw new Exception(StringUtils::fromUTF8('Na Portále není možno vyslat na školení do minulosti!'));
        }

        if ($hodnoty->volnych_mist == 0 && $_GET['kurz'] != ''){
            throw new Exception(StringUtils::fromUTF8('Kurz vyčerpán!'));
        }

        Prava::nastavObdobiProPrava(time(), time());
        if (!(Prava::lzeZobrazitPP('skoleni_vyslat', $hodnoty->osoba, time(), time()))){
            RonPortalUtils::neniPravoException();
        }

        require_once('adsapp/ppomer.php');
        $pracovniPomer = new PracovniPomer();
        $pracovniPomer->nactiPPPodleId($this->database, $hodnoty->osoba);
        $pracovniPomer->vyberNactenyPP($this->database, Prava::vratPrava('skoleni_vyslat'));

        $record = new RonPortalDBRecord($this->database, 'GenerovaneSkoleni');
        $record->logDescription = 'Vyslani zamestnance na skoleni';

        $record->addField('Prihlasil', '', FALSE, 25, User::getLoginName());
        $record->addField('RC', '', FALSE, 15, $pracovniPomer->rc);
        $record->addField('DruhSkoleni', '', FALSE, 10, $hodnoty->druhSkoleni);
        $record->addField('CisloSkoleni', '', FALSE, 10, $hodnoty->kurz);
        $record->addField('PopisSkoleni', '', FALSE, 80, $hodnoty->popis_skoleni);
        $record->addField('DatumVznikuPovinnosti', 'timestamp', FALSE, 0, DateUtils::today());
        $record->addField('DuvodSkoleni', 'int', FALSE, 0, 1);
        $record->addField('Potvrzeni', 'int', FALSE, 0, 1);
        $record->addField('Generovane', 'int', FALSE, 0, 0);
        $record->addField('PrihlasenoZAplikace', 'int', FALSE, 0, 1);
        $record->addField('Poznamka', 'memo', FALSE, 0, $hodnoty->poznamka_skoleni);
        $record->addField('PozadovaneDatumSkoleni', 'date', FALSE, 0, $hodnoty->pozadovanedatumskoleni);

        $params = $this->database->createParameters();
        $sqlWhere =
            ' RC = ' . $params->addParameter($pracovniPomer->rc) .
            ' AND DruhSkoleni = ' . $params->addParameter($hodnoty->druhSkoleni) .
            ' AND DatumVznikuPovinnosti = ' . $params->addParameter($this->database->timestampToParam(DateUtils::today()));
        $record->setWhere($sqlWhere, $params);

        if ($record->loadRecord() == 0) {
            if (!$record->insertRecord()) {
                throw new Exception(StringUtils::fromUTF8('Chyba při zápisu vyslání zaměstnance na školení!'));
            }
        } else {
            throw new Exception(StringUtils::fromUTF8('Chyba! Zápis vyslání zaměstnance na dané školení s dnešním datem povinnosti již existuje!'));
        }



        if (is_uploaded_file($_FILES['dokument']['tmp_name'])) {
            try {
                $idDokumentu = DokumentyUtils::vlozDokument($this->database, 'dokument', RonPortalSettings::getIntValue('skoleni_typ_dokumentu'),
                    $_POST['dokument_popis'], 'Dokument vyslani na skoleni');

                $recDokumentOsoby = new RonPortalDBRecord($this->database, 'DokumentyOsoby', 'Portal - dokument vyslani na skoleni');
                $recDokumentOsoby->addField('RC', '', true, 15, $pracovniPomer->rc);
                $recDokumentOsoby->addField('id', 'int', true, 0, $idDokumentu);
                $recDokumentOsoby->addField('GSk_RC', '', false, 15, $pracovniPomer->rc);
                $recDokumentOsoby->addField('GSk_DruhSkoleni', '', false, 10, $hodnoty->druhSkoleni);
                $recDokumentOsoby->addField('GSk_Datum', 'timestamp', true, 0, DateUtils::today());

                $recDokumentOsoby->insertRecord();
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e);
            }
        }

        MessageManager::addInfoMessage( ronText('Skoleni_vyslani_ulozeno') );
        return new Redirect(NULL, '?menuid=skoleni&action=vyslatnaskoleni');
    }


    /**
     * @param JsonOutputObject $output
     * @throws DBException
     */
    function downloadFirmy($output)
    {
        $firmy = [];
        $firma = [];

        $sql = "SELECT Firma.kodFirmy AS Kod, COALESCE(Firma.nazev1,'') + ' (' + COALESCE(Firma.mesto, '') + ')' AS Popis                
                FROM Firma                
                ORDER BY Firma.nazev1";

        $cursor = $this ->database->getSqlResult($sql);
        while ($row = $cursor->fetch()) {
            $firma['kod'] = StringUtils::jsonText($row[0]);
            $firma['popis'] = StringUtils::jsonText($row[1]);

            $firmy[] = $firma;
        }
        $cursor ->close();

        $output ->addData('items', $firmy);
        $output->compress = 'gzip';
    }


    /**
     * @param $cisloSkoleni
     * @return string
     */
    public function dokumentSkoleni($cisloSkoleni)
    {
        $dokument = '';
        $params = $this->database->createParameters();
        $sql =  "SELECT Dokument.ID, Dokument.popis, Dokument.jmenoSouboru " . //0..2
                "FROM DokumentSkoleni JOIN Dokument ON (DokumentSkoleni.id = Dokument.id) " .
                " JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ) " .
                "WHERE DokumentSkoleni.cisloSkoleni = " . $params->addParameter($cisloSkoleni) .
            "  AND (Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1))";

        try {
            $cursor = $this->database->select($sql, $params);

            while ($row = $cursor->fetch()) {
                $linkStr = "?menuid=skoleni&action=download&cisloskoleni=" . urlencode($cisloSkoleni) . "&dokument={$row[0]}";

                if ($linkStr) $dokument .= "<a target='_blank' href='$linkStr'>";
                $dokument .= StringUtils::htmlTextConvert($row[1]) . ' - ' . StringUtils::htmlTextConvert($row[2]);
                if ($linkStr) $dokument .= "</a>";
            }
        } catch (Exception $exception) {
            MessageManager::addDebugMessage(StringUtils::fromUTF8("Chyba při načítání dokumentů školení ") . StringUtils::htmlTextConvert($cisloSkoleni) . "\n" . $exception->getMessage());
        }

        return $dokument;
    }

    /**
     * @param $idPP
     * @param $druh
     * @param $datum
     * @return JsonOutputObject
     * @throws DBException
     * @throws Exception
     */
    private function detailniInformaceSkoleniKurzu($idPP, $druh, $datum)
    {
        $params = $this->database->createParameters();

        $sql = "SELECT D.Kod, D.Aktivni, D.DelkaPlatnosti, D.URLexternihoSkoleni, D.Elearning, {fn_CASTWIDEMEMO(D.Poznamka)}, D.uzivatelska1, " .      //0..6
                " D.uzivatelska2, D.uzivatelska3, D.uzivatelska4, D.uzivatelska5, Skoleni.Popis, " .               //7..11
                " Skoleni.DatumPocatkuSkoleni, Skoleni.DatumUkonceniSkoleni, Skoleni.MistoSkoleni, Skoleni.DelkaSkoleniDnu, " .     //12..15
                " Skoleni.DelkaSkoleniHodin, Skoleni.Kapacita, Skoleni.Skolitel, Skoleni.Hodnotici, {fn_CASTWIDEMEMO(Skoleni.HodnoceniSkoleni)}, " .    //16..20
                " Skoleni.DlouhodobyKurz, {fn_CASTWIDEMEMO(Skoleni.ObsahSkoleni)}, Skoleni.CisloPlanovanehoSkoleni, GS.DatumVznikuPovinnosti, S_DuvodSkoleni.Popis, " .  //21..25
                " GS.DatumSkoleni, GS.PozadovaneDatumSkoleni, GS.CisloSkoleni, Skoleni.Kredity, " .          //26..29
                " {fn_CASTWIDEMEMO(GS.PoznamkaSchvalovani)}, GS.DatumPoslednihoSkoleni, GS.Prihlasil, S_StavPotvrzeni.Popis, GS.Povinne, " .    //30..34
                " {fn_CASTWIDEMEMO(GS.Poznamka)}, GS.PopisSkoleni, KS.Popis, SpecifikaceKurzu.Popis, " .                             //35..38
                " S_StavSchvalPrihlNaKurz.Popis, Firma.nazev1, Firma.nazev2, S_AnoNe.Popis, Skoleni.CasPocatkuSkoleni, Skoleni.CasUkonceniSkoleni,  " .        //39..44
                " S_StavSkoleni.Popis, Umisteni.Popis, GS.PozadovanaPlatnostDo, Skoleni.CisloSkoleni, D.Popis, GS.Potvrzeni " .   //45..50

                " FROM  DruhSkoleni AS D 
                           LEFT JOIN Skoleni ON D.Kod = Skoleni.DruhSkoleni 
                            LEFT JOIN GenerovaneSkoleni GS ON D.Kod = GS.DruhSkoleni 
                             LEFT JOIN KategorieSkoleni KS ON KS.Kod = D.Kategorie
                              LEFT JOIN Firma ON (Skoleni.SkoliciFirma = Firma.kodFirmy)
                               LEFT JOIN PracovniPomer PP ON (GS.RC = PP.RC)
                                LEFT JOIN SpecifikaceKurzu ON (SpecifikaceKurzu.Kod = Skoleni.Specifikace)
                                 LEFT JOIN S_StavSchvalPrihlNaKurz ON (S_StavSchvalPrihlNaKurz.Kod = GS.SchvaleniPrihlasky)
                                  LEFT JOIN S_DuvodSkoleni ON (S_DuvodSkoleni.kod = GS.DuvodSkoleni) 
                                   LEFT JOIN S_AnoNe ON (D.Elearning = S_AnoNe.Kod)
                                    LEFT JOIN S_StavSkoleni ON (S_StavSkoleni.Kod = Skoleni.StavSkoleni)
                                     LEFT JOIN Umisteni ON (Skoleni.MistoSkoleni = Umisteni.Kod)
                                      LEFT JOIN S_StavPotvrzeni ON  (S_StavPotvrzeni.Kod = GS.Potvrzeni)

                WHERE (GS.CisloSkoleni = Skoleni.CisloSkoleni OR GS.CisloSkoleni IS NULL ) AND GS.DruhSkoleni = " . $params->addParameter($druh) .
                " AND PP.ID = " . $params->addParameter($idPP) .
                " AND GS.DatumVznikuPovinnosti = " . $params->addParameter($this->database->timestampToParam($datum));

        MessageManager::addDebugLog($sql);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        $hodnoty = [];
        if ($row) {
            $platnost = $this->database->stringToTimestamp($row[47]);
            $platnostText = ($platnost == '2147483647' ? ronText('Skoleni_platnost_neomezene') : "");

            //Skoleni
            $hodnoty['Skoleni_absolvovane_caption'] = StringUtils::jsonText("Skoleni_absolvovane_caption");

            $hodnoty['Skoleni_typ_skoleni'] = StringUtils::jsonText($row[28]);
            $hodnoty['Skoleni_popis_typu_skoleni'] = StringUtils::jsonText($row[49]);
            //absolvovano uspesne   absolvovano neuspesne neucast, prominuto
            if (($row[50] == 3) || ($row[50] == 4) || ($row[50] >= 7))
                $hodnoty['Skoleni_datum'] = StringUtils::jsonDate($this->database->stringToTimestamp($row[26]));
            else
                $hodnoty['Skoleni_datum'] = '';
            $hodnoty['Skoleni_duvod'] = StringUtils::jsonText($row[25]);
            $hodnoty['Skoleni_delka_platnosti'] = StringUtils::jsonText($platnostText);
            $hodnoty['skoleni_vysledek'] = StringUtils::jsonText($row[33]);
            $hodnoty['Skoleni_Kredity'] = StringUtils::jsonText($row[29]);
            $hodnoty['Skoleni_poznamka_skoleni'] = StringUtils::jsonText($row[35]);
            $hodnoty['Skoleni_elearning'] = StringUtils::jsonText($row[42]);
            $hodnoty['Skoleni_kategorie'] = StringUtils::jsonText($row[37]);
            $hodnoty['Skoleni_poznamka_typu_skoleni'] = StringUtils::jsonText($row[5]);


//            if ($row[28]){
//                //Pokud existuje dokument skoleni, pak pripojit
//                $dokument = $this->dokumentSkoleni($row[28]);
//                if ($dokument) $hodnoty['Skoleni_dokument'] = $dokument;
//            }

            if ($row[28]) {
                //Kurz pokud existuje kod pak nacist informace kurzu
                $hodnoty['Skoleni_kurz_caption'] = StringUtils::jsonText("Skoleni_kurz_caption");

                $hodnoty['Skoleni_ICAL'] = StringUtils::jsonText("?menuid=skoleni&action=ical&cisloskoleni=$row[48]");
                $hodnoty['Skoleni_stav_kurzu'] = StringUtils::jsonText(ucfirst($row[45]));
                $hodnoty['Skoleni_CisloSkoleni'] = StringUtils::jsonText($row[48]);
                $hodnoty['Skoleni_popis_kurzu'] = StringUtils::jsonText($row[11]);
                $hodnoty['Skoleni_poznamka'] = StringUtils::jsonText($row[20]);
                $hodnoty['Skoleni_obsah_kurzu'] = StringUtils::jsonText($row[22]);
                $hodnoty['Skoleni_schvaleni_prihlasky'] = StringUtils::jsonText($row[39]);
                $hodnoty['datum_pocatku_skoleni'] = StringUtils::jsonDate($this->database->stringToTimestamp($row[12]));
                $hodnoty['Skoleni_cas_skoleni'] = StringUtils::jsonText(date('G:i', $this->database->stringToTimestamp($row[43]))) . " - " . StringUtils::jsonText(date('G:i', $this->database->stringToTimestamp($row[44])));
                $hodnoty['datum_ukonceni_skoleni'] = StringUtils::jsonDate($this->database->stringToTimestamp($row[13]));
                $hodnoty['misto_skoleni'] = StringUtils::jsonText($row[46]);
                $hodnoty['delka_skoleni_dnu'] = StringUtils::jsonText($row[15]);
                $hodnoty['delka_skoleni_hodin'] = StringUtils::jsonText($row[16]);
                $hodnoty['Skoleni_kapacita'] = StringUtils::jsonText($row[17]);
                $hodnoty['Skoleni_poznamka_schvalovani'] = StringUtils::jsonText($row[30]);
                $hodnoty['Skoleni_specifikace'] = StringUtils::jsonText($row[38]);
                $hodnoty['skolitel'] = StringUtils::jsonText($row[18]);
                $hodnoty['hodnotici'] = StringUtils::jsonText($row[19]);
                $hodnoty['Skoleni_skolici_firma'] = StringUtils::jsonText($row[40]) . ($row[41] ? " (" . StringUtils::jsonText($row[41]) . ")" : "");
            }

            $cursor->close();
            return new JsonOutputObject($hodnoty);
        }

        $cursor->close();

        return new JsonOutputObject($hodnoty['existujedetail'] = false);
    }


    /**
     * @param $skoleni
     * @param $osobaIDPP
     * @param $datumPovinnosti
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    public function odhlasitSkoleniVeHromadnemVyberu($skoleni, $osobaIDPP, $datumPovinnosti)
    {

        require_once('adsapp/ppomer.php');
        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osobaIDPP);

        $smazat = false;
        //pokud je v nastaveni zvoleno ponechat navrh, nikdy se nebude mazat
        if (RonPortalSettings::getIntValue(self::NASTAVENI_PONECHAT_NAVRH))
            $smazat = false;

        if (!$this->jeOsobaNaSkoleni($skoleni, $pp->rc)) {
//            throw new Exception(ronText('SkoleniNeniPrihlasene'));
             MessageManager::addErrorMessage(ronText('SkoleniNeniPrihlasene'));
             return true;
        }

        $prava = Prava::vratPrava('skoleni_prihlasit');
        $pravaPotvrditPrihlaseni = Prava::vratPrava('skoleni_schvalit_prihlaseni');

        if (!$prava->lzeZobrazitRC($pp->rc)) {
//            throw new Exception(ronText('Nemate_pravo'));
            MessageManager::addErrorMessage(ronText('Nemate_pravo'));
            return true;
        }


        $record = new RonPortalDBRecord($this->database, 'GenerovaneSkoleni');

        $record->logDescription = 'Odhlaseni skoleni';

        $record->addField('CisloSkoleni', '', FALSE, 10);
        $record->addField('DatumSkoleni', 'date', FALSE, 0);
        $record->addField('DatumPrihlasky', 'date', FALSE, 0);
        $record->addField('Prihlasil', '', FALSE, 25);
        $record->addField('DuvodSkoleni', 'int', TRUE);
        $record->addField('Potvrzeni', 'int', true);
        $record->addField('SchvaleniPrihlasky', 'int', true, 0);

        $params = $this->database->createParameters();

        $record->setWhere("RC = " . $params->addParameter($pp->rc) .
                            " AND CisloSkoleni = " . $params->addParameter($skoleni) .
                            ($datumPovinnosti > 0 ? " AND DatumVznikuPovinnosti = " . $params->addParameter($this->database->timestampToParam($datumPovinnosti)) : ''), $params);

        if (!$record->loadRecord()) {
//            throw new Exception(ronText('SkoleniChybaNacteniZaznamu'));
            MessageManager::addErrorMessage(ronText('SkoleniChybaNacteniZaznamu'));
            return true;
        }

        //skoleni se nemaze, jen se zrusi prirazene skoleni
        if (!$smazat) {

            //prihlaska jiz byla schvalena, odhlaseni muze provest jen uzivatel s pravy pro schvaleni prihlasky, pri odhlaseni se zrusi zaroven schvaleni prihlasky
            if ($record->findFieldByName('SchvaleniPrihlasky')->value == 1) {
                if (!$pravaPotvrditPrihlaseni->lzeZobrazitRC($pp->rc)){
//                    throw new Exception(ronText('Nemate_pravo'));
                    MessageManager::addErrorMessage(ronText('Nemate_pravo'));
                    return true;
                }

                $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(0);

                //pro doporucene (individualni duvod skoleni - teoreticky i rucne vlozeny zaznam) se nastavi potvrzeni na 0
                if ($record->findFieldByName('DuvodSkoleni')->value == 1) {
                    $record->findFieldByName('Potvrzeni')->setNewValue(0);
                }
            }

            if (!$record->updateRecord()) {
//                throw new Exception(ronText('SkoleniChybaOdhlaseni'));
                MessageManager::addErrorMessage(ronText('SkoleniChybaOdhlaseni'));
                return true;
            }
        }
        //smaze se zaznam generovaneho skoleni
        else {
            if (!$record->deleteRecord()) {
//                throw new Exception(ronText('SkoleniChybaOdhlaseni'));
                MessageManager::addErrorMessage(ronText('SkoleniChybaOdhlaseni'));
                return true;
            }
        }

//        MessageManager::addInfoMessage(ronText('SkoleniUspesneOdhlaseni'));
        return true;
    }


    /**
     * @param int $osoba
     * @param $cislo
     * @return JsonOutputObject
     * @throws DBException
     * @throws Exception
     */
    public function nactiKurzyKPrihlaseni($osoba, $cislo)
    {
        $json = new JsonOutputObject();

        $seznamKurzu = [];
        //pravo na osobu jinak ven
        require_once('adsapp/ppomer.php');
        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osoba);
        
        $prava = Prava::vratPrava('skoleni_prihlasit');
        if (!$prava->lzeZobrazitRC($pp->rc)) {
            $json->setResult(ronText('Nemate_pravo'));
            return $json;
        }

        $params = $this->database->createParameters();
        //Nabidnout kurzy ktere jsou ve stejnem dni a miste jako zvoleny kurz

        $sql = "SELECT S.CisloSkoleni, S.Popis, S.DruhSkoleni, S.DatumPocatkuSkoleni, S.CasPocatkuSkoleni, S.MistoSkoleni, S.Kapacita,
					(SELECT COUNT (GenerovaneSkoleni.RC) From GenerovaneSkoleni Where RC = " . $params->addParameter($pp->rc) . " 
					        AND GenerovaneSkoleni.CisloSkoleni = S.CisloSkoleni), GS.DatumVznikuPovinnosti, GS.Potvrzeni
                FROM SKOLENI S
                    INNER JOIN SKOLENI SK ON (( S.MistoSkoleni = SK.MistoSkoleni )
                        AND (S.DatumPocatkuSkoleni = SK.DatumPocatkuSkoleni) AND (S.CasPocatkuSkoleni = SK.CasPocatkuSkoleni) 
                        AND (SK.CISLOSKOLENI = " . $params->addParameter($cislo) . "))
                INNER JOIN Umisteni U ON (SK.MistoSkoleni = U.Kod)
                INNER JOIN DruhSkoleni DS ON (DS.Kod = S.DruhSkoleni)
                    LEFT OUTER JOIN GenerovaneSkoleni GS ON (GS.RC = " . $params->addParameter($pp->rc) . " AND DS.Kod = GS.DruhSkoleni and GS.Potvrzeni in (0,1))
                ORDER BY 1,10,9 DESC";

        MessageManager::addDebugLog($sql);
        $cursor = $this->database->getSqlResult($sql, $params);

        //TODO nacist vse do pole a pak porovnavat sousedni (po sobe jdouci) zaznamy, a pokud budou stejneho


        $result .= "<div class='form-row form-section'>";
        $result .= ronText('Skoleni_kurzy_k_prihlaseni');
        $result .= "</div>";

        $kurzyKOsobe = [];
        while ($row = $cursor->fetch()){

            $kurz = $row[0];

            //Po sobe jdouci kurzy
            if (array_key_exists($kurz, $kurzyKOsobe)){
                //pokud jiz existuje porovnej potvrzeni

                $aktualniKurz = $row;
                $jizVlozenyKurz = $kurzyKOsobe[$kurz];

                $potvrzeniZpracovavaneho = $aktualniKurz[9];
                $potvrzeniJizVlozeneho = $jizVlozenyKurz[9];
                $datumPovinnostiZpracovavaneho = $this->database->stringToTimestamp($aktualniKurz[8]);
                $datumPovinnostiJizVlozeneho = $this->database->stringToTimestamp($jizVlozenyKurz[8]);

                if (($potvrzeniJizVlozeneho < $potvrzeniZpracovavaneho) || ($potvrzeniJizVlozeneho == $potvrzeniZpracovavaneho && $datumPovinnostiJizVlozeneho > $datumPovinnostiZpracovavaneho)) {
                    //v pripade vice kurzu ve stejnem dni a case, je potreba
                    //pouzit (nahradit stary novym) prioritne potvrzene (potvrzeni = 1), nebo potvrzene a s nizsim datem vzniku povinnosti (25.5. 2020 / 24.5.2020 => 24.5.2020)
                    unset($kurzyKOsobe[$kurz]);
                    $kurzyKOsobe[$kurz] = $aktualniKurz;
                }

            } else {
                $kurzyKOsobe[$row[0]] = $row;
            }
        }


        $dalsiKurzNeoznacit = false;
        foreach ($kurzyKOsobe as $kurzy => $row){
            $hodnoty = [];
            if ($row[8] != ""){
                $datumPovinnosti = $this->database->stringToTimestamp($row[8]);
            } else {
                $datumPovinnosti = 0;
            }

            if (!$dalsiKurzNeoznacit){
                $dalsiKurzNeoznacit = $row[7];
            }

            //pokud neni jiz prihlasen pak oznaci zvoleny kurz v dialogu, aby jej nemusel opakovane vybirat - vyjma pripadu, kdy je jiz nektery oznaceny
            $prihlasen = $row[7] > 0 ? ronText("Skoleni_Prihlasen") : ronText("Skoleni_Neprihlasen");
            $oznacitKurz = ($row[7] == 0 && $row[0] == $cislo && !$dalsiKurzNeoznacit) ? 1 : 0;

            $hodnoty['Kurz_idpp'] = StringUtils::jsonText($osoba);
            $hodnoty['Kurz_cislo'] = StringUtils::jsonText($row[0]);
            $hodnoty['Kurz_druh'] = StringUtils::jsonText($row[2]);
            $hodnoty['Kurz_oznacit'] = StringUtils::jsonText($oznacitKurz);
            $hodnoty['Kurz_vznik_povinnosti'] = StringUtils::jsonText($datumPovinnosti);
            $hodnoty['Kurz_prihlasen_text'] = StringUtils::jsonText($prihlasen);
            $hodnoty['Kurz_prihlasen'] = StringUtils::jsonText($row[7]);
            $hodnoty['Kurz_Popis'] = "(" . StringUtils::jsonText($row[0]) . ") " . StringUtils::jsonText($row[1]);
            $hodnoty['Kurz_datum'] = StringUtils::jsonDate($this->database->stringToTimestamp($row[3]));

            $seznamKurzu[] = $hodnoty;
        }
        $cursor->close();


        $json->addData("kurzy", $seznamKurzu);
        $json->setResult('OK');

        return $json;
    }


    /**
     * @param PracovniPomer $pp
     * @param PravaNaOsoby $pravaPotvrditPrihlaseni
     * @param int $terminKurzu
     * @param $cislo
     * @param $druh
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    public function hromadnaRegistraceNaKurzy($pp, $pravaPotvrditPrihlaseni, $terminKurzu, $cislo, $druh)
    {
        $record = new RonPortalDBRecord($this->database, 'GenerovaneSkoleni');
        $record->logDescription = 'Prihlaseni skoleni';

        $record->addField('DatumSkoleni', 'date', FALSE, 0, $terminKurzu);
        $record->addField('CisloSkoleni', '', FALSE, 10, $cislo);
        $record->addField('DatumPrihlasky', 'date', FALSE, 0, time());
        $record->addField('PrihlasenoZAplikace', 'int', FALSE, 0, 1);
        $record->addField('Prihlasil', '', FALSE, 25, User::getLoginName());
        $record->addField('Potvrzeni', 'int', true);
        $record->addField('SchvaleniPrihlasky', 'int', true, 0);
        $record->addField('Generovane', 'int', true, 0, 0);

        //pokud ma uzivatel pravo na potvrzeni prihlaseni, potvzeni zapise, jinak se provadi samostatnou akci
        if ($pravaPotvrditPrihlaseni->lzeZobrazitRC($pp->rc)) {
            $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(1);
            $record->addField('Vyhodnotil', '', false, 25, User::getLoginName());
            $record->addField('DatumVyhodnoceni', 'date', false, 0, DateUtils::today());
            $record->addField('PoznamkaSchvalovani', 'memo', false, 0);
        }
        else {
            $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(-1);
        }

        $params = $this->database->createParameters();
        $record->setWhere("RC = " . $params->addParameter($pp->rc) .
                " AND DruhSkoleni = " . $params->addParameter($druh) .
                " AND DatumSkoleni IS NULL " .
                " AND NOT EXISTS(SELECT * FROM GenerovaneSkoleni GS WHERE GS.RC = GenerovaneSkoleni.RC " .
                "  AND GS.DruhSkoleni = GenerovaneSkoleni.DruhSkoleni " .
                "  AND GS.DatumVznikuPovinnosti > GenerovaneSkoleni.DatumVznikuPovinnosti " .
                "  AND GS.CisloSkoleni IS NOT NULL)", $params);


        //osoba jiz ma vygenerovanou povinnost skoleni absolvovat, zapisou se informace o prihlasenem skoleni
        if ($record->loadRecord()) {
            //pro povinnost = 0 (doporucene) se pri schvaleni nastavi potvrzeni na 1
            if ($record->findFieldByName('Potvrzeni')->value == 0) {
                if (($record->findFieldByName('SchvaleniPrihlasky')->value == -1) || ($record->findFieldByName('SchvaleniPrihlasky')->value == 1)) {
                    $record->findFieldByName('Potvrzeni')->setNewValue(1);
                }
            }

            if (!$record->updateRecord()) {
//                throw new Exception(ronText('SkoleniChybaPrihlaseni'));
                MessageManager::addErrorMessage(ronText('SkoleniChybaPrihlaseni'));
                return true;
            }
        }
        //pro osobu neexistuje povinnost absolvovat skoleni (skoleni je doporucene) vlozi se novy zaznam
        else {
            $record->addField('RC', '', FALSE, 15, $pp->rc);
            $record->addField('DruhSkoleni', '', FALSE, 10, $druh);
            $record->addField('DatumVznikuPovinnosti', 'timestamp', FALSE, 0, DateUtils::now());
            $record->addField('DuvodSkoleni', 'int', FALSE, 0, 1); //individualni duvod skoleni
            $record->addField('DatumZapisu', 'timestamp', FALSE, 0, DateUtils::today());
            $record->addField('Vlozil', '', FALSE, 25, User::getLoginName());

            //Nastaveno na datum++
            $record->addField('PozadovaneDatumSkoleni', 'date', FALSE, 0, DateUtils::incDay($terminKurzu));
            $record->findFieldByName('Potvrzeni')->setNewValue(0);

            //pro schvalene nebo pokud se nevyzaduje schvaleni nastavi se potvrzeni na 1
            if (($record->findFieldByName('SchvaleniPrihlasky')->value == -1) || ($record->findFieldByName('SchvaleniPrihlasky')->value == 1)){
                $record->findFieldByName('Potvrzeni')->setNewValue(1);
            }

            if (!$record->insertRecord()) {
//                throw new Exception(ronText('SkoleniChybaPrihlaseni'));
                MessageManager::addErrorMessage(ronText('SkoleniChybaPrihlaseni'));
                return true;
            }
        }

        return true;
    }


    /**
     * @throws DBException
     * @throws Exception
     */
    public function hromadnaRegistraceKurzuNaTerminu()
    {
        $osoba = (int)$_POST['osoba'];
        $terminKurzu = $_POST['terminkurzu'];

        require_once('adsapp/ppomer.php');
        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osoba);

        $prava = Prava::vratPrava('skoleni_prihlasit');
        $pravaPotvrditPrihlaseni = Prava::vratPrava('skoleni_schvalit_prihlaseni');

        if (!$prava->lzeZobrazitRC($pp->rc)) {
            MessageManager::addErrorMessage(ronText('Nemate_pravo'));
            return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');
        }

        if(!empty($_POST['registrovatKurzy']))
        {
            foreach ($_POST['registrovatKurzy'] as $item => $value) {
                if ($value['prihlasit'] == 1) {
                    //Prihlasit/Registrovat skoleni
                    $this->hromadnaRegistraceNaKurzy($pp, $pravaPotvrditPrihlaseni, (int)$terminKurzu, $value['cislo'], $value['druh']);
                }
            }
        }

        return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');
    }


    /**
     * @throws Exception
     */
    private function hromadnePrihlasitOsobyNaSkoleni()
    {
        foreach ($_POST['prihlasitNaSkoleni'] as $item => $value){

            if((isset($value['jizPrihlaseno']) && $value['jizPrihlaseno']) && (!isset($value['prihlasit'])) )
            {
                //Zrusit skoleni
                $this->odhlasitSkoleniVeHromadnemVyberu($value['cislo'], $value['ID'], (int)$value['datumpovinnosti']);
            }
            elseif ((isset($value['jizPrihlaseno']) && $value['jizPrihlaseno'] == 0) && (isset($value['prihlasit']) && $value['prihlasit'] == 1) )
            {
                //Prihlasit skoleni
                $this->prihlasitSkoleniVeHromadnemVyberu($value['cislo'], $value['ID'], (int)$value['datumpovinnosti']);
            }
        }
    }


    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws Exception
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'seznamfirem' :
                $this ->downloadFirmy($output);
                break;

            case 'mojeskoleni' :
                $this ->mojeSkoleni($output);
                break;

            case 'skolenipodrizenych' :
                $this ->skoleniPodrizenych($output);
                break;
        }
    }


    /**
     * @param string $action
     * @return BaseViewObject|InternalRedirect|JsonOutputObject|OutputObject|Redirect|TranslationsOutputObject|ViewObject|DokumentOutputObject|HodnoceniSkoleniBrowser|PlanovaneSkoleniKurzy|PrihlaseniNaPlanovaneSkoleniBrowser|SkoleniAKurzy|SkoleniForm|SkoleniPrihlaseniBrowser|SkoleniUcastBrowser|string
     * @throws DBException
     * @throws Exception
     */
    public function handleAction($action) {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_SKOLENI)) {
            RonPortalUtils::neniPravoException();
        }

        if (isset($_POST['formaction'])) $formaction = $_POST['formaction'];
        else if (isset($_GET['formaction'])) $formaction = $_GET['formaction'];
        else $formaction = '';

        if (isset($_POST['timestampTerminuSkoleni'])  &&  $_POST['timestampTerminuSkoleni']) {
            return $this ->zapisUcastiNaSkoleni($_POST['timestampTerminuSkoleni']);
        }

        if (isset($_POST['hodnoceniSkoleni'])  &&  $_POST['hodnoceniSkoleni']) {
            return $this ->zapisHodnoceniSkoleni();
        }

        if (isset($_POST['vytvoritNovyTerminBtn'])  &&  $_POST['vytvoritNovyTerminBtn'] != "" && ($_POST['pridaniTerminu'] != "")) {
            return $this ->vytvoreniTerminuSkoleni($_POST['kodKurzu'], $_POST['pridaniTerminu'],
                $_POST['pridaniTerminu_time'], $_POST['poznamkaKNovemuTerminu']);
        }

        if (isset($_POST['timestampEditovanehoTerminuSkoleni'])  &&  $_POST['timestampEditovanehoTerminuSkoleni']) {
            return $this ->editaceTerminuSkoleni($_POST['kodKurzu'],$_POST['timestampEditovanehoTerminuSkoleni'],
                $_POST['time'], $_POST['poznamkaKEditovanemuTerminu']);
        }

        if (isset($_POST['zaznamenatvyslaninaskoleni']) && $_POST['zaznamenatvyslaninaskoleni']){
            return $this->zapisVyslaniZamestnanceNaSkoleni();
        }

        //Vytvoření plánu školení a připojení nákladů
        if (isset($_POST['upravaplanovanehoskoleni'])  &&  $_POST['upravaplanovanehoskoleni'] == 1) {
            return $this->zapisPlanovanehoSkoleni($_POST['cisloSkoleni']);
        }
        if (isset($_POST['nakladyplanovanehoskoleni'])  &&  $_POST['nakladyplanovanehoskoleni']) {
            return $this->zapisPlanovanehoSkoleni($_POST['cisloSkoleni']);
        }
        if (isset($_POST['novenakladyplanovanehoskoleni'])  &&  $_POST['novenakladyplanovanehoskoleni']) {
            return $this->zapisPlanovanehoSkoleni($_POST['cisloSkoleni']);
        }

        //Hromadne prihlaseni / odhlaseni osob na skoleni
        if (isset($_POST['hromadnePrihlasitSkoleni']) && $_POST['hromadnePrihlasitSkoleni'] == 1){
            $this->hromadnePrihlasitOsobyNaSkoleni();
        }

        //Pro jednu osobu prihlasit / registrovat jeden nebo vice kurzu
        if (isset($_POST['hromadnaregistracekurzunaterminu']) && $_POST['hromadnaregistracekurzunaterminu'] == 1){
            $this->hromadnaRegistraceKurzuNaTerminu();
        }

        if ($formaction) {
            // formular (editace, insert)
            return $this ->skoleniFormular($formaction);
        }

        $skoleniaction = isset($_GET['action']) ? $_GET['action'] : '';
        $skoleni = isset($_GET['cisloskoleni']) ? $_GET['cisloskoleni'] : '';


        switch ($skoleniaction) {
            // json akce
            case 'mojeskoleni' :
            case 'skolenipodrizenych' :
            case 'seznamfirem' :
                return parent::handleAction($skoleniaction);

            case 'hodnoceni':
                return $this->hodnoceniSkoleni();

            case 'download':
                return $this->downloadDokumentSkoleni($skoleni, $_GET['dokument']);

            case 'ical' :
                return $this ->downloadIcal($skoleni);

            case 'zobrazprihlaseni':
                return $this->zobrazPrihlaseniSkoleni($_GET['hromadneprihlaseni']);

            case 'prihlaseninaplanovaneskoleni':
                $this->hlavickaVyberAkce();
                require_once('person/prihlaseninaplanovaneskolenicontroller.php');
                $prihlaseniPlanController = new PrihlaseniNaPlanovaneSkoleniController($this->database);
                return $prihlaseniPlanController->handleAction('');

            case 'prihlaseni':
                return $this->prihlaseniSkoleni($skoleni, $_GET['osoba'], (int)$_GET['datumpovinnosti']);

            case 'nactikurzykprihlaseni':
                return $this->nactiKurzyKPrihlaseni($_GET['osoba'], (int)$_GET['terminkurzu'], $_GET['misto']);

            case 'schvaleniprihlaseni':
                return $this->nastavSchvaleniPrihlaseni($skoleni, $_GET['osoba'], (int)$_GET['datumpovinnosti']);

            case 'zamitnutiprihlaseni':
                return $this->nastavSchvaleniPrihlaseni($skoleni, $_GET['osoba'], (int)$_GET['datumpovinnosti'], 2);

            case 'odhlaseni':
                return $this->odhlaseniSkoleni($skoleni, $_GET['osoba'], (int)$_GET['datumpovinnosti']);

            case 'skolenikurzy':
                return $this->zobrazSkolenikurzy();

            case 'skoleniucast':
                return $this->zobrazUcastSkoleni($_GET['record']);

            case 'skolenihodnoceni':
                return $this->zobrazHodnoceniSkoleni($_GET['record']);

            case 'vyslatnaskoleni':
                //View
                return $this->vyslatNaSkoleni($_GET['osoba'], StringUtils::fromUTF8($_GET['druhskoleni']), StringUtils::fromUTF8($_GET['kurz']));

            case 'planovaneskolenikurzy':
                //Browser
                return $this->zobrazPlanovaneSkoleni();

            case 'detailplanovanehoskolenikurzu':
                $this->hlavickaVyberAkce();
                //View
                return $this->detailPlanovaneSkoleni($_GET['record']);

            case 'vytvoreniskolenikurzu':
                //View
                return $this->vytvoreniPlanovanehoSkoleni($_GET['record']);

            case 'zapisabsolvovani':
                return $this->skoleniFormular('new');

            case 'translations':
                return $this->translations();

            case 'detailnipopisskolenikurzu':
                return $this->detailniInformaceSkoleniKurzu($_GET['idpp'], StringUtils::fromUTF8($_GET['druhskoleni']), $_GET['datum']);


            case 'prehled':
            default:
                return $this->prehledSkoleni();
        }
    }
}