<?php

/**
 * UTF8
 */

namespace Ron;


class EditField
{
    var $name; // jmeno pole v HTML formulari

    var $format;
    var $readOnly;     // priznak jestli je jen pro cteni

    var $styl;         // zakladni css styl
    var $skrytNuly;    // priznak, jestli se maji skryt nulove hodnoty
    var $sirkaPole;

    var $hodnota;      // datova hodnota (cislo)
    var $text;         // datova hodnota (text)
    var $puvodniText;  // puvodni datova hodnota (napr. nactena z databaze)
    var $chyba;        // priznak chyby (napr. spatny format cisla)

    //var $f_Limit;       // limity hodnot pro jednotlive sloupce

    function __construct()
    {
        $this->skrytNuly = true;
    }


    function isTimeFormat()
    {
        return $this->format == 'hodiny';
    }

    function isFloatFormat()
    {
        switch ($this->format) {
            case 'float' :
            case 'number0' :
            case 'number1' :
            case 'number_1' :
            case 'number2' :
            case 'number_2' :
            case 'curr' :
            return true;
        }

        return false;
    }


    function vratTextProHodnotu($hodnota)
    {
        if (!isset($hodnota)) return ''; // hodnota vubec neni nastavena

        if (!$hodnota && $this->skrytNuly) return ''; // nulova hodnota se nema zobrazovat => prazdny text

        return StringUtils::format($this->format, $hodnota);
    }


    // Prevede text zadany uzivatelem na hodnotu
    function vratHodnotuProText($text)
    {
        if ($text == '') {
            if ($this ->isTimeFormat()  ||  $this ->isFloatFormat()) return 0;
            return '';
        }

        if ($this->isTimeFormat()) {
            return DateUtils::timeStringToFloat($text, true); // povolitZkraceny = true
        }

        if ($this ->isFloatFormat()) {
            if (StringUtils::isFloat($text)) {
                return StringUtils::stringToFloat($text);
            }
            throw new \Exception( ronText('Bad_number_format'));
        }

        return false;
    }


    function setValue($hodnota)
    {
        $this->hodnota = $hodnota;

        $this->text = $this->vratTextProHodnotu($hodnota);
        $this->puvodniText = $this->text;

        $this->chyba = false;
    }


    function getFieldHTML()
    {
        $text = $this->text;

        if ($this->readOnly) return StringUtils::htmlTextConvert($text); // jen zobrazit

        // editace
        $stylestr = 'text-align:right;';
        if ($this->sirkaPole) $stylestr .= " width:{$this -> sirkaPole}px;";

        $className = $this->isTimeFormat() ? 'inputTime' : 'inputFloat';
        if ($this->chyba) $className .= ' error';

        // pokud je hodnota editovana (puvodni hodnota z DB je jina nez hodnota zadana uzivatelem), zobrazi ji tucne
        if ($this->puvodniText != $this->text) $stylestr .= 'font-weight:bold;';

        $readonlystr = $this->readOnly ? "readonly='readonly'" : '';
        $result = "<input type='text' name='{$this -> name}' value='" . StringUtils::htmlTextConvert($text) . "' $readonlystr class='$className' style='$stylestr' maxlength='10' />";

        return $result;
    }


    /**
     * Nacte text zadany uzivatelem. Zaroven se jej pokusi prevest na hodnotu.
     * @param bool $throwException
     * @return bool
     * @throws \Exception
     */
    function nactiPost($throwException = false)
    {
        if ($this->readOnly) return true;

        if (isset($this->name) && isset($_POST[$this->name])) {
            $this->text = trim($_POST[$this->name]);
            try {
                $this ->hodnota = $this->vratHodnotuProText($this->text);
            } catch (\Exception $e) {
                $this ->chyba = true;
                if ($throwException) throw $e;
                return false;
            }
        } else {
            $this->text = '';
        }

        return true;
    }
}
