<?php

/**
 * UTF8
 * Spravce souboru - moznost uploadu souboru, downloadu, mazani
 */

namespace Ron;

use Ron\MVC\ViewObject;

require_once('ron/fileutils.php');

class FileManager implements ViewObject
{
    const FIELD_NAME = 'name';
    const FIELD_SIZE = 'size';
    const FIELD_DATE = 'date';
    const FIELD_TIME = 'time';

    public $dir;

    public $url;
    public $deleteUrl;
    public $downloadUrl;
    public $uploadUrl;

    public $columns = [self::FIELD_NAME, self::FIELD_SIZE, self::FIELD_DATE, self::FIELD_TIME];
    public $showDirs = false;
    public $header;

    public $files = [];


    /**
     * FileManager constructor.
     * @param string $dir
     * @param string $url
     * @param string $downloadUrl
     * @param string $deleteUrl
     */
    public function __construct($dir, $url = null, $downloadUrl = null, $deleteUrl = null)
    {
        $this->url = $url;
        $this->dir = $dir;
        $this->deleteUrl = $deleteUrl;
        $this->downloadUrl = $downloadUrl;
    }


    public function load($loadDetails = true, $sort = true)
    {
        $this ->files = file_exists($this ->dir)
            ? FileUtils::getFileList($this ->dir, $this ->showDirs, $loadDetails)
            : [];

        if ($sort) {
            if ($sort === true) {
                $sort = $_SESSION['FileManager.SORT'];
                if (!$sort) {
                    $sort = "name";
                }
            } else if ($sort) {
                $_SESSION['FileManager.SORT'] = $sort;
            }

            // a seradi panely
            usort($this->files,
                function ($item1, $item2) use ($sort) {
                    /**
                     * @var FileInfo $item1
                     * @var FileInfo $item2
                     */
                    // nejdrive jsou adresare
                    if ($item1 ->isDir  &&  !$item2 ->isDir) {
                        return -1;
                    }
                    if (!$item1 ->isDir  &&  $item2 ->isDir) {
                        return 1;
                    }

                    if ($sort == "size") {
                        if (!$item1 ->isDir) {
                            if ($item1 ->size < $item2 ->size) {
                                return -1;
                            }
                            if ($item1 ->size > $item2 ->size) {
                                return 1;
                            }
                            // dale podle jmena
                        }
                    }

                    if ($sort == "time") {
                        if ($item1 ->modificationTime < $item2 ->modificationTime) {
                            return -1;
                        }
                        if ($item1 ->modificationTime > $item2 ->modificationTime) {
                            return 1;
                        }
                        // dale podle jmena
                    }

                    if ($item1 ->fileName < $item2 ->fileName) {
                        return -1;
                    }

                    if ($item1 ->fileName > $item2 ->fileName) {
                        return 1;
                    }

                    return 0;
                }
            );
        }
    }


    /**
     * @param FileInfo $fileInfo
     */
    public function renderLine($fileInfo)
    {
        $urlFileName = FileUtils::translateFileNameToLink($fileInfo ->fileName);
        echo "<tr>";

        foreach ($this ->columns as $column) {
            switch ($column) {
                case self::FIELD_NAME :
                    echo '<td class="txt"><a href="' . $this ->downloadUrl . '&filename=' . $urlFileName . '" target="_blank">' . $fileInfo ->fileName . "</a></td>";
                    break;

                case self::FIELD_SIZE :
                    $title = StringUtils::format('number0', $fileInfo ->size) . ' B';
                    echo '<td class="align-right" title="' . $title . '"> ' . FileUtils::getFileSizeText($fileInfo ->size) . '</td>';
                    break;

                case self::FIELD_DATE :
                    echo "<td class='align-center'> ".date("j.n.Y" ,$fileInfo ->modificationTime) . "</td>";
                    break;

                case self::FIELD_TIME :
                    echo "<td> ".date("H:i" ,$fileInfo ->modificationTime) . '</td>';
                    break;
            }
        }

        // Odkaz pro smazani souboru - javascript
        if ($this ->deleteUrl) {
            require_once('ron/guiutils.php');
            $question = sprintf( ronText('Really_delete_file'), $fileInfo ->fileName );
            $fileName = $urlFileName;
            $href = "{$this ->deleteUrl}&filename=$fileName";
            echo '<td>' . GuiUtils::changeLocationQuest( ronText('delete'), $question, $href ) . '</td>';
        }

        echo "</tr>\n";
    }


    public function renderUpload()
    {
        if ( !$this ->uploadUrl ) {
            return;
        }

        echo
            '<div>' .
            '<form enctype="multipart/form-data" method="post" action="' . $this ->uploadUrl . '">' .
            ronText('Upload_file') .
            '<input type="file" name="uploadfilename">' .
            "&nbsp;&nbsp;" .
            '<input type="submit" class="submitbutton" value="' . ronText('Upload') . '">' .
            "</form>\n" .
            "</div>\n";
    }


    private function rendeTableHeader()
    {
        echo '<thead>';
        foreach ($this ->columns as $column) {
            switch ($column) {
                case self::FIELD_NAME :
                    echo
                        '<th class="txt">' .
                        ($this->url ? '<a href="' . $this ->url . '&sort=name">' : '') .
                        ronText('Jmeno_souboru') .
                        ($this->url ? '</a>' : '') .
                        '</th>';
                    break;

                case self::FIELD_SIZE :
                    echo
                        '<th class="txt align-right">' .
                        ($this->url ? '<a href="' . $this ->url . '&sort=size">' : '') .
                        ronText('Velikost') .
                        ($this->url ? '</a>' : '') .
                        '</th>';
                    break;

                case self::FIELD_DATE :
                    echo
                        '<th class="txt align-center">' .
                        ($this->url ? '<a href="' . $this ->url . '&sort=time">' : '') .
                        ronText('Date') .
                        ($this->url ? '</a>' : '') .
                        '</th>';
                    break;

                case self::FIELD_TIME :
                    echo
                        '<th class="txt">' .
                        ($this->url ? '<a href="' . $this ->url . '&sort=time">' : '') .
                        ronText('Cas') .
                        ($this->url ? '</a>' : '') .
                        '</th>';
                    break;
            }
        }
        echo "</thead>\n";
    }

    public function render()
    {
        if ($this ->header) {
            echo "<h1>{$this ->header}</h1>\n";
        }

        MessageManager::getInstance() ->showMessages();

        echo '<table class="filetable rowOver">';

        $this ->rendeTableHeader();

        echo '<tbody class="border-top">';
        foreach ($this ->files as $fileInfo)
            $this ->renderLine($fileInfo); // $dir, $filename, $columns, $downloadlink, $deletelink);
        echo '</tbody>';

        echo "</table>\n";

        $this ->renderUpload();
    }

}
