<?php
/**
 * UTF8
 * User: stanislav.elbl
 * Date: 14.7.15
 * Time: 0:19
 */


namespace Ron;

require_once('ron/sqlutils.php');
require_once('ron/formfield.php');
require_once('ron/dbrecord.php');

use Ron\DB;
use Ron\MVC\ViewObject;


class InputForm implements ViewObject
{
    const NEW_RECORD = 'new';
    const COPY_RECORD = 'copy';
    const INSERT_RECORD = 'insert';
    const EDIT_RECORD = 'edit';
    const SAVE_RECORD = 'save';

    /** @var  DB\Database */
    var $database;

    /** @var string         nadpis formulare */
    var $title;
    var $f_errormessage;

    /** @var string         kod formulare pro nacteni konfigurace */
    var $code;

    /** @var string         kod formulare - slouzi pro rozliseni vice formularu na jedne strance */
    var $formCode;

    /** @var string         Jmeno SQL tabulky */
    var $sqlTableName;
    var $sqlSelectPart;

    /** @var FormField[]    Pole zobrazovanych poli - viz formfield.php */
    var $fields;

    /** @var bool           nastavit na true, pokud se nema kazde pole zobrazovat v novem radku tabulky */
    var $noTableRows;
    /** @var bool           nastavit na true, pokud se nema zobrazovat tlacitko "puvodni" */
    var $noOldValuesButton;
    /** @var string         jmeno css tridy pro formular */
    var $cssClass;

    /** @var array          Html parametry formulare (napr. id, style, data atd.) - asociativni pole nazev=>hodnota */
    var $htmlParams = [];

    /** @var string         edit,save / new,copy,insert */
    var $formAction;
    /** @var string         akce pro formular v HTML */
    var $htmlAction;
    /** @var string         pokud je nastavena, zobrazi se tlacitko Storno */
    var $stornoAction;

    var $readOnly;

    var $data;      // nactena data

    /** @var bool           Vyrenderuje formular jako dialog (skryty atd.) */
    var $dialogMode = false;


    /**
     * @param DB\Database $database
     * @param string $formAction
     * @param bool $dialogMode
     */
    function __construct($database, $formAction, $dialogMode = false)
    {
        $this->database = $database;

        $this ->dialogMode = $dialogMode;

        //$this->cssClass = 'inputtable';
        $this->cssClass = 'form large-form form-30-70' . ($dialogMode ? ' ronform' : '');
        $this->fields = [];

        if ($this ->dialogMode) {
            $this ->htmlParams['data-ronform-field-selector'] = 'input, select, textarea';
        }

        $this->formCode = '';

        $this->readOnly = false;

        $post_ok = true;
        if (isset($_POST['formcode'])) {
            $post_ok = $_POST['formcode'] == $this->formCode;
        } else if ($this->formCode) {
            // jedna se o data z jineho formulare - ? ? ? muzu vzit formaction z GET?
            $post_ok = false;
        }

        if ($post_ok && isset($_POST['formaction'])) $this->formAction = StringUtils::secureString($_POST['formaction']);
        else if (isset($_GET['formaction'])) $this->formAction = StringUtils::secureString($_GET['formaction']);
        else $this->formAction = $formAction;
    }


    /**
     * @param  string|FormField|array $field
     * @return FormField
     */
    function addField($field = FORM_FIELD_TYPE_EDIT)
    {
        if ($field instanceof FormField) {

        } else {
            // array nebo string
            $field = new FormField($field);
        }

        if ($field) $this->fields[] = $field;

        return $field;
    }


    /**
     * Prida BrowserFields ze zadaneho pole $fields - to muhou byt primo objekty BrowserField nebo pole pri init()
     * @param array $fields
     */
    function addFields($fields)
    {
        foreach ($fields as $field) $this->addField($field);
    }


    /**
     * Vraci pole se zadanym kodem
     * @param $code
     * @return null|int
     */
    function findFieldIndexByCode($code)
    {
        foreach ($this ->fields as $index => $field) {
            if ($field ->code == $code) return $index;
        }

        return null;
    }


    /**
     * Presune zadane pole ($fieldIndex) pred pole s kodem $before, nebo za pole s kodem $after
     * @param int $fieldIndex
     * @param string $before
     * @param string $after
     */
    function moveField($fieldIndex, $before, $after)
    {
        if ($before) {
            $field2Index = $this ->findFieldIndexByCode($before);

            if (isset($field2Index)) {
                if ($field2Index == $fieldIndex - 1) {
                    // uz je na pozadovane pozici
                    return;
                }

                $removed = array_splice($this ->fields, $fieldIndex, 1);

                if ($field2Index > $fieldIndex) {
                    // po odstraneni pole se index zmenil
                    $field2Index --;
                }

                array_splice($this ->fields, $field2Index, 0, $removed);
                return;
            }
        }

        if ($after) {
            $field2Index = $this ->findFieldIndexByCode($after);

            if (isset($field2Index)) {
                if ($field2Index == $fieldIndex + 1) {
                    // uz je na pozadovane pozici
                    return;
                }

                if ($field2Index > $fieldIndex) {
                    // po odstraneni pole se index zmenil
                    $field2Index --;
                }

                $removed = array_splice($this ->fields, $fieldIndex, 1);
                array_splice($this ->fields, $field2Index + 1, 0, $removed);
                return;
            }
        }
    }


    /**
     * Nacte konfiguraci ze zadaneho xmlTextu
     * @param string $xmlText
     */
    function loadConfigurationXml($xmlText)
    {
        if (!$xmlText) {
            return;
        }

        require_once 'ron/xmlloader.php';
        $a = XmlLoader::loadXmlToArray($xmlText,
            [XmlLoader::PARAM_CONVERT_FROM_UTF8 => true, XmlLoader::PARAM_INCLUDE_ATTRIBUTES => true]
        );

        if (isset($a['field'])) {
            foreach ($a['field'] as $fieldSettings) {
                $fieldIndex = $this ->findFieldIndexByCode($fieldSettings['code']);
                if (isset($fieldIndex)) {
                    $this ->fields[$fieldIndex] ->loadConfigurationArray($fieldSettings);

                    if ($fieldSettings['before']  ||  $fieldSettings['after']) {
                        $this ->moveField($fieldIndex, $fieldSettings['before'], $fieldSettings['after']);
                    }
                }
            }
        }

        foreach ($a as $key => $value) {
            if (is_array($value)) {
                // hodnoty definovane pomoci vnorenych tagu spoji do jednoho textu
                $newVal = '';
                foreach ($value as $valA) {
                    if (isset($valA['#text']))
                        $newVal .= ($newVal ? "\n" : '') . $valA['#text'];
                }

                $value = $newVal;
            }

            switch ($key) {
//                case 'style' :
//                    $this ->cssStyle = ($this ->cssStyle ?: ''). $value;
//                    break;

                case 'noTableRows' :
                case 'noOldValuesButton' :
                case 'readOnly' :
                    $this ->{$key} = (int)$value;
                    break;

                case 'cssClass' :
                case 'stornoAction' :
                    $this ->{$key} = $value;
                    break;
//
//                case 'sqlFilter' :
//                    $sqlFilter = $value;
//
//                case 'title' :
//                    $this ->{$key} = ronText($value);
//                    break;
            }
        }
    }


        /**
     * Zobrazi formular pro vkladani. $action je akce tlacitka pro vlozeni zaznamu.
     * @param bool $postvalues Urcuje, jestli se maji zobrazit hodnoty z $_POST (true), nebo defaultni hodnoty
     * @return bool
     */
    function showFieldsInsert($postvalues)
    {
        echo '<div class="form-content">';
        foreach ($this->fields as $i => $field) {
            if ($field ->visible) {
                $field->showFieldInsert($this->database, $postvalues, !$this->noTableRows);
            }
        }
        echo '</div>';


        echo '<div class="buttons">';
        //if (!$this->noTableRows) echo '<tr class="formbuttons">';
        //echo '<td></td><td>';
        echo '<input type="submit" class="submitbutton' . ($this ->dialogMode ? ' ronform-submit' : '') . '" value="' . ronText('form.Insert') . '"/>';

        if ($this->stornoAction) {
            $classStr = $this ->stornoAction == '#'
                ? 'cancel'
                : 'clickHref';

            echo "<input type='button' class='button $classStr' value='" . ronText('Cancel') . "' data-ron-href='{$this->stornoAction}'/>";
        }

        //echo '</td>';
        //if (!$this->noTableRows) echo "</tr>\n";
        echo '</div>';

        return true;
    }


    /**
     * volano pri zobrazeni formulare pro editaci - pokud funkce vrati false
     * zobrazi se jen readonly verze formulare
     * @param array $dbrow pole nactenych dat z databaze
     * @return bool
     */
    function canEditRecord($dbrow)
    {
        if ($this->readOnly) return false;

        return true;
    }


    // --------------------------------------------------------------------
    function showFieldsEdit($values, $postvalues, $caneditrecord)
    {
        echo '<div class="form-content">';
        foreach ($this->fields as $i => $field) {
            if ($field ->visible) {
                if ($postvalues && $caneditrecord && $field->editable) {
                    $value = $field->getPostValueText();
                } else {
                    // Standardni zobrazeni hodnot z databaze
                    $value = (isset($field->sqlFieldIndex) && $field->sqlFieldIndex >= 0)
                        ? $field->getValueFromDBValue($this->database, $values[$field->sqlFieldIndex])
                        : $field->defaultValue;
                }


                if ($caneditrecord) {
                    $field->showFieldEdit($this->database, $value, !$this->noTableRows, $values);
                } else {
                    $field->showFieldReadOnly($this->database, $value, !$this->noTableRows, $values);
                }
            }
        }
        echo '</div>';


        if ($caneditrecord) {
            echo '<div class="buttons">';
            //if (!$this->noTableRows) echo '<TR>';
            //echo '<TD></TD><TD>';
            echo '<input type="submit" class="submitbutton" value="' . ronText('form.Store') . '">';

            if (!$this->noOldValuesButton) {
                echo '&nbsp;&nbsp;<input type="reset" class="button" value="' . ronText('form.Original') . '">';
            } // if

            if ($this->stornoAction) {
                echo '<input type="button" class="button clickHref" value="' . ronText('Cancel') . '" data-ron-href="' . $this->stornoAction . '"/>';
            }

            //echo '</TD>';
            //if (!$this->noTableRows) echo '</TR>';
            echo '</div>';
        }

        return true;
    }


    /**
     * Zkontroluje, jestli jsou zadana vsechna povinna pole. Chybove hlasky vlozi do MessageManageru.
     * Vraci true, pokud je vse v poradku
     *
     * @param bool $edit
     * @return bool
     */
    function checkRequiredFields($edit)
    {
        $result = true;

        foreach ($this->fields as $i => $field) {
            if (!$field->checkRequired($edit)) {
                MessageManager::getInstance()->addError(ronText('form.Missing_value') . $field->caption, $field->fieldName);
                $result = false;
            }
        }

        return $result;
    }


    /**
     * @return string|DB\SqlStatement
     */
    function getWhereCondition()
    {
        if (isset($_POST['record'])) $record = $_POST['record'];
        else if (isset($_GET['record'])) $record = $_GET['record'];
        else $record = 0;

        // najde primarni pole a urci podle nej podminku
        foreach ($this->fields as $field)
            if ($field->primary) {
                return $field->sqlFieldName . ' = ' . FormField::fieldValueToSQLConst($this->database, $field->fieldType, $record);
            }

        return '';
    }


    /**
     * Sestavi dotaz pro nacteni zaznamu z databaze
     * @return string|DB\SqlStatement
     */
    function buildSelectSQL()
    {
        if ($this->sqlSelectPart) {
            $fieldsStr = $this->sqlSelectPart;
        } else {

            $fieldsStr = '';

            $dbFieldIndex = 0;

            foreach ($this->fields as $i => $field) {
                $field->sqlFieldIndex = null;

                if ($field->sqlFieldName) {
                    //if (db_type() == 'mssql'  &&  $field -> fieldType == 'memo') $fieldname = 'CAST ('.$field -> f_sql_fieldname.' AS TEXT)';
                    $fieldName = ($field->fieldType == 'memo')
                        ? $this->database->castWideMemo($field->sqlFieldName)
                        : $field->sqlFieldName;

                    $field->sqlFieldIndex = $dbFieldIndex;
                    //$this -> f_fields[$i] -> sqlFieldIndex = $dbfieldindex; // verze pro PHP4 !
                    $dbFieldIndex++;

                    if ($fieldsStr) $fieldsStr .= ', ';
                    $fieldsStr .= $fieldName;
                }
            }
        }

        $where = $this->getWhereCondition();
        if ($where instanceof DB\SqlStatement) {
            return new DB\SqlStatement(
                "SELECT $fieldsStr FROM {$this->sqlTableName} WHERE " . $where ->sqlText,
                $where ->parameters);
        }

        return
            "SELECT $fieldsStr FROM " . $this->sqlTableName .
            ' WHERE ' . $where;
    }


    /**
     * @return array|bool|null
     * @throws \Exception
     */
    function loadRecord()
    {
        // Nacte aktualni hodnoty poli z databaze
        $sql = $this->buildSelectSQL();

        if (!$sql) {
            throw new \Exception('Neni zadan SQL vyraz pro nacteni dat z databaze!');
        }

        //MessageManager::addDebugLog('InputForm::loadRecord: ' . $sql);
        $cursor = $this->database->select($sql);
        $result = $cursor->fetch();
        $cursor ->close();

        return $result;
    }


    /**
     * vraci true, pokud se jedna o vkladani noveho zaznamu
     * @return bool
     */
    function isInsert()
    {
        return $this->formAction == 'new'
        || $this->formAction == 'copy'
        || $this->formAction == 'insert';
    }


    /**
     * Vlozi zakladni pole (formaction). Potomek muze pridat svoje vlastni dodatecna pole.
     */
    function showCommonFields()
    {
        $formAction = $this->isInsert() ? 'insert' : 'save';

        echo '<INPUT type="hidden" name="formaction" value="' . $formAction . '">';
        if ($this->formCode != '') {
            echo '<INPUT type="hidden" name="formcode" value="' . $this->formCode . '">';
        }

        if (isset($_POST['record'])) $record = $_POST['record'];
        else if (isset($_GET['record'])) $record = $_GET['record'];
        else $record = '';

        if ($record) {
            // id editovaneho zaznamu
            echo '<INPUT type="hidden" name="record" value="' . $record . '">';
        }
    }

    /** Zobrazi formularova pole
     * @return bool
     * @throws \Exception
     */
    function showFormFields()
    {
        if ($this->formAction == 'new') {
            return $this->showFieldsInsert(false); // postvalues = false
        }

        if ($this->formAction == 'insert') {
            return $this->showFieldsInsert(true); // postvalues = true
        }

        if (!isset($this->data)) {
            throw new \Exception('Chyba! Pozadovany zaznam nebyl nalezen!');
        } else if (!$this->data) {
            // chyba v SQL dotazu
            throw new \Exception('Chyba ve formulari (funkce showFormFields) - nelze nacist hodnoty z databaze!');
        }

        $canEditRecord = $this->canEditRecord($this->data);

        if ($this->formAction == 'save') {
            return $this->showFieldsEdit($this->data, true, $canEditRecord); // postvalues=true
        }

        // edit
        return $this->showFieldsEdit($this->data, false, $canEditRecord); // postvalues=false
    }


    function showHeader()
    {
        if ($this->title) echo "<h1>{$this->title}</h1>\n";
    }


    function getHtmlFormParams()
    {
        return
            ($this ->code
                ? " data-code='{$this ->code}'"
                : '') .
            ($this ->htmlAction
                ? " action='{$this->htmlAction}'"
                : '');
    }

    /** Zobrazi formular
     * @param string $formAction
     * @throws \Exception
     */
    function showForm($formAction = '')
    {
        if ($formAction) {
            $this->formAction = $formAction;
        }

        $this->showHeader(); // header pred <FORM> - muze obsahovat jiny formular!

        MessageManager::getInstance()->showMessages();


        echo '<div';
        if ($this ->cssClass) {
            echo ' class="' . $this->cssClass . '"';
        }
        if ($this ->htmlParams) {
            foreach ($this ->htmlParams as $paramName => $paramValue) {
                echo " $paramName='$paramValue'";
            }
        }
        echo '>';

        echo
            "\n<form enctype='multipart/form-data' method='post' " . $this ->getHtmlFormParams() . ">\n";

        $this->showCommonFields();
        $this->showFormFields();

        echo "\n</form>\n";

        echo '</div>';
    }


    /**
     * @return DBRecord
     */
    function createDBRecord()
    {
        return new DBRecord($this->database, $this->sqlTableName);
    }


    /**
     * @param DBRecord $rec
     * @return bool
     */
    function beforeInsertRecord($rec)
    {
        return true;
    }

    /**
     * @param DBRecord $rec
     * @return bool
     */
    function afterInsertRecord($rec)
    {
        return true;
    }


    /**
     * @param DBRecord $rec
     * @param bool $insert
     */
    function createDBRecordFields($rec, $insert)
    {
        foreach ($this->fields as $i => $field) {
            if ($field ->visible) {
                $rec_field = $field->createDBRecordField($this->database, $this->sqlTableName, $insert); // $insert=true

                if ($rec_field) {
                    $rec->addDBRecordField($rec_field);
                }
            }
        }
    }


    /**
     * @return bool
     * @throws DB\DBException
     */
    function insertRecord()
    {
        if (!$this->checkRequiredFields(false)) {
            return false;
        }

        $rec = $this->createDBRecord();
        $this ->createDBRecordFields($rec, true);

        if (!$this->beforeInsertRecord($rec)) {
            return false;
        }

        $rec->insertRecord();

        return $this->afterInsertRecord($rec);
    }


//    /**
//     * @return bool
//     */
//    function insert()
//    {
//        if (!$this->checkRequiredFields(false)) {
//            return false;
//        }
//
//        $rec = $this->createDBRecord();
//        $this ->createDBRecordFields($rec, true);
//
//        if (!$this->beforeInsertRecord($rec)) {
//            return false;
//        }
//
//        if (!$rec->insertRecord()) {
//            return false;
//        }
//
//        return $this->afterInsertRecord($rec);
//    }


    /**
     * @param DBRecord $rec
     * @return bool
     */
    function beforeUpdateRecord($rec)
    {
        return true;
    }

    /**
     * @param DBRecord $rec
     * @return bool
     */
    function afterUpdateRecord($rec)
    {
        return true;
    }


    /**
     * @param DBRecord $rec
     */
    function setDBRecord($rec)
    {
        $params = $this ->database ->createParameters();
        $whereStr = '';

        $this ->createDBRecordFields($rec, false);
        foreach ($this->fields as $i => $field) {
            if ($field->primary  &&  $field ->sqlFieldName) {
                $rec_field = $rec ->findFieldByName($field ->sqlFieldName);
                if ($whereStr) $whereStr .= ' AND ';
                $whereStr .= $field->sqlFieldName . ' = ' . $params ->addParameter($rec_field->value);
            }
        }

        $rec->setWhere($whereStr, $params);
    }

    /**
     * @return bool|int
     * @throws \Exception
     */
    function editRecord()
    {
        if (!$this->checkRequiredFields(true)) {
            return false;
        }

        /** @var DBRecord $rec */
        $rec = $this->createDBRecord();
        $this ->setDBRecord($rec);

        if (!$rec->loadRecord()) {
            throw new \Exception('Chyba pri editaci zaznamu - puvodni zaznam nebyl nalezen!');
        }

        if (!$this->beforeUpdateRecord($rec)) {
            return false;
        }

        if (!$rec->updateRecord()) {
            return false;
        }

        return $this->afterUpdateRecord($rec);
    }


    /**
     * @return bool|int
     * @throws \Exception
     */
    function handleAction() {
        $loadRecord = false;

        try {
            switch ($this->formAction) {
                /** @noinspection PhpMissingBreakStatementInspection */
                case 'edit' :
                    $loadRecord = true;
                    // pokracuje dale!
                case 'new' :
                    // je potreba zobrazit formular
                    $result = false;
                    break;

                case 'copy' :
                    // je potreba zobrazit formular
                    $loadRecord = true;
                    $result = false;
                    break;

                case 'insert' :
                    if ($this->insertRecord()) {
                        return true;
                    }
                    $result = false;
                    break;

                case 'save' :
                    $loadRecord = true;
                    $result = $this->editRecord();
                    break;

                default :
                    throw new \Exception('Chyba v metode Form::handleAction! Neznama akce!');
            }
        } catch (RonException $e) {
            MessageManager::addErrorMessage($e ->getMessage(), $e ->getFieldName(), null, $e ->getErrorDescription());
            $result = false;
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            $result = false;
        }

        if (!$result  &&  $loadRecord) {
            $this ->data = $this->loadRecord();
        }

        return $result;
    }


    /**
     * @throws \Exception
     */
    function render()
    {
        $this ->showForm();
    }

}

