<?php
/**
 * UTF8
 * User: stanislav.elbl
 * Date: 14.7.15 0:21
 */

namespace Ron;

use Ron\Controls\ComboBox;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;

define('FORM_FIELD_TYPE', 'type');
define('FORM_FIELD_NAME', 'name');
define('FORM_FIELD_SQL_NAME', 'sqlFieldname');
define('FORM_FIELD_SQL_INDEX', 'sqlIndex');
define('FORM_FIELD_PRIMARY', 'primary');
define('FORM_FIELD_READONLY', 'readOnly');
define('FORM_FIELD_EDIT', 'edit');
define('FORM_FIELD_REQUIRED', 'required');
define('FORM_FIELD_CAPTION', 'caption');
define('FORM_FIELD_CAPTION_TEXT', 'captionText');
define('FORM_FIELD_CAPTION2', 'caption2');
define('FORM_FIELD_DEFAULT_VALUE', 'defaultValue');
define('FORM_FIELD_DATA_TYPE', 'dataType');
define('FORM_FIELD_MAXLENGTH', 'maxLength');
define('FORM_FIELD_ROWS', 'rows');
define('FORM_FIELD_COLS', 'cols');
define('FORM_FIELD_WRAP', 'wrap');
define('FORM_FIELD_TEXT', 'text');
define('FORM_FIELD_SQL', 'sql');
define('FORM_FIELD_OPTIONS', 'options');
define('FORM_FIELD_ID', 'fieldId');
define('FORM_FIELD_ROW_ID', 'rowId');
define('FORM_FIELD_ROW_CLASS', 'rowClass');

define('FORM_FIELD_TYPE_TEXT', 'text'); // jen text
define('FORM_FIELD_TYPE_SECTION', 'section'); // nadpis sekce
define('FORM_FIELD_TYPE_EDIT', 'edit'); // normalni editovatelne pole
define('FORM_FIELD_TYPE_MEMO', 'memo'); // viceradkove editovatelne pole
define('FORM_FIELD_TYPE_DATE', 'date'); // datum
define('FORM_FIELD_TYPE_DATETIME', 'datetime'); // datum cas
define('FORM_FIELD_TYPE_DATEANDTIME', 'dateandtime'); // datum + cas (2 pole)
define('FORM_FIELD_TYPE_INT', 'int');
define('FORM_FIELD_TYPE_FLOAT', 'float');
//define('FORM_FIELD_TYPE_PASSWD', 'passwd');
define('FORM_FIELD_TYPE_PASSWORD', 'password');
define('FORM_FIELD_TYPE_CHECK', 'check');
define('FORM_FIELD_TYPE_SELECT', 'select');
define('FORM_FIELD_TYPE_COMBO', 'combo');
define('FORM_FIELD_TYPE_COMBOSTR', 'combostr');
define('FORM_FIELD_TYPE_RADIO', 'radio');
define('FORM_FIELD_TYPE_HIDDEN', 'hidden');
define('FORM_FIELD_TYPE_ACTUAL_TIME', 'actualtime');
define('FORM_FIELD_TYPE_ACTUAL_DATE', 'actualdate');
define('FORM_FIELD_TYPE_ID', 'id');
define('FORM_FIELD_TYPE_FILE', 'file');
define('FORM_FIELD_TYPE_FILENAME', 'filename');


class FormField
{
    /**
     * @var string
     * Typ polozky
     *   text   - text, ktery se primo zobrazi
     *   edit   - normalni text (string)
     *   check  - zaskrtavatko - hodnoty 0/1
     *   memo   - viceradkovy text
     *   select - combo box - viz $selectSql
     *   combo  - combobox s predvyplnenymi moznostmi, datovy typ INT - viz $options
     *   combostr  - combobox s predvyplnenymi moznostmi - viz $options
     *   hidden - ve formulari se nezobrazi, pri insertu se nastavi defaultni hodnota, pri editaci nelze zmenit
     *   id     - automaticky cislovane pole typu int
     *   password - heslo (pri editaci se nenacita z databaze)
     *   file   - pole pro upload souboru
     *   filename - pole pro upload souboru, jmeno souboru se zapise do databaze
     *   actualtime - aktualni cas
     *   serial - seriove cislo - pri insert se hodnota urci automaticky
     *   _custom - uzivatelske pole - typ muze byt cokoliv, ale obsluhu musi zajistit formular
     */
    var $fieldType;
    /**
     * @var string
     * jmeno pole
     * pokud jmeno zacina znakem *, tak se nejedna o databazove pole
     * jinak je to databazove pole a nastavi se podle nej take sqlFieldName
     */
    var $fieldName;
    /** @var string     jmeno pole v databazi */
    var $sqlFieldName;
    /** @var integer    index pole v radku nactenem z databaze */
    var $sqlFieldIndex;

    /** @var string     Kod pro konfiguraci pomoci XML */
    var $code;

    /** @var bool       priznak, jestli se jedna o primarni klic */
    var $primary;
    /** @var bool       jestli se muze pole editovat (plati pouze pro editaci, ne pro vkladani) */
    var $editable = true;
    /** @var bool       jestli je pole povinne */
    var $required;
    /** @var bool       umoznuje vypnout zobrazeni pole */
    var $visible = true;
    /** @var bool       Urcuje, jestli je mozne zmenit nenullovou hodnotu */
    var $changeable = true;

    /** @var string     nadpis polozky */
    var $caption;
    /** @var string     2. nadpis, ktery se zobrazi za controlem */
    var $caption2;

    /** @var mixed      defaultni hodnota pole - pro insert */
    var $defaultValue;
    /** @var bool       priznak, ze se ma pouzit defaultValue pri editaci, pokud je v datech null */
    var $useDefaultValueToEditNull;

    /** @var string|SqlStatement    sql dotaz pro vyber ( napr. select kod, popis from ... ) */
    var $selectSql;
    /** @var bool       jestli se ma pri porovnavani porovnavat datovy typ (defaultne ano) */
    var $selectDataTypeTest = true;
    /** @var bool       priznak, jestli se ma vzdy (i kdyz je required) zobrazit prazdna hodnota - null/true/false */
    var $showEmptyValue = null;

    /** @var bool       priznak, jestli se ma zobrazit tlacitko "vlozit aktualni cas". Funguje jen kdyz je nastaveno fieldId. */
    var $currentTimeButton = false;

    /** @var string[]|string    dalsi parametry pro prvek formulare (napr. styl, javascript a podobne) */
    var $otherHtmlParams;

    /** @var array      pole voleb pro combo (napr. 1 => volba 1, 2 => volba 2, ...) */
    var $options;
    /** @var string     zobrazeny text - pro typ text, pro typ password je to nazev pole */
    var $text;
    /** @var int        maximalni pocet znaku - pro edit */
    var $maxLength;
    /** @var string     jmeno funkce pro zjisteni hodnoty daneho pole */
    var $functionName;
    /** @var int        pocet radku pro memo */
    var $rows;
    /** @var int        pocet sloupcu pro memo */
    var $cols;
    /** @var bool       zalamovani slov pro memo */
    var $wrap;
    /** @var string     datovy typ pro hidden pole ('', 'date', 'datetime', 'int') */
    var $dataType;

    /** @var string     html id editacniho prvku */
    var $fieldId;
    /** @var string     css class editacniho prvku. Pokud neni nastavena, doplni se automaticky podle typu */
    var $fieldCssClass;
    /** @var string     html id radku formulare. Standardne nastaveno na row_{$fieldName} */
    var $rowId;
    /** @var string */
    var $rowClass;

    /** @var bool   Priznak, ze se jedna jen o logovaci pole - viz DBRecordField.updateLog */
    var $updateLog;

    /** @var array              Asociativni pole indexu, ktere se pouziji pro datove atributy (nazev atributu => index) */
    var $attrIndexes;

    /**
     * @param string|mixed[] $options   Typ pole (string) nebo pole options
     */
    function __construct($options = FORM_FIELD_TYPE_EDIT)
    {
        if (is_array($options)) {
            $this->rowClass = 'form-row';
            $this->init($options);
        } else {
            $this ->setFieldType($options);
            if (!$this ->rowClass) {
                $this->rowClass = 'form-row';
            }
        }
    }

    /**
     * @param string $fieldType
     * @return FormField
     */
    public function setFieldType($fieldType)
    {
        $this->fieldType = $fieldType;

        if ($this ->fieldType == FORM_FIELD_TYPE_SECTION  &&  !$this ->rowClass) {
            $this ->rowClass = 'form-section'; // vychozi class pro section
        }

        return $this;
    }


    /**
     * @param string $code
     * @return FormField
     */
    public function setCode($code)
    {
        $this->code = $code;
        return $this;
    }

    /**
     * @param boolean $visible
     * @return FormField
     */
    public function setVisible($visible)
    {
        $this->visible = $visible;
        return $this;
    }


    /**
     * @param bool $value
     * @return FormField
     */
    public function setEditable($value) {
        $this ->editable = $value;
        return $this;
    }

    /**
     * @param boolean $changeable
     * @return FormField
     */
    public function setChangeable($changeable)
    {
        $this->changeable = $changeable;
        return $this;
    }



    /**
     * @param bool $value
     * @return FormField
     */
    public function setRequired($value) {
        $this ->required = $value;
        return $this;
    }

    /**
     * Nastavi jmeno html pole a take jmeno SQL pole (pokud jiz neni nastaveno)
     * @param $value
     * @return FormField
     */
    public function setFieldName($value)
    {
        // POZOR! Nastavuje i sql_fieldname (to se da dodatecne zmenit)
        if ($value[0] == '*') {
            $this->fieldName = substr($value, 1); //  str_replace('*', '', $value[1]);
            $this->sqlFieldName = '';
        } else {
            $this->fieldName = $value;
            if (!$this ->sqlFieldName) {
                $this->sqlFieldName = $this->fieldName;
            }
        }

        return $this;
    }

    public function setUpdateLog($value)
    {
        $this ->updateLog = $value;
        return $this;
    }

    /**
     * @param $value
     * @return FormField
     */
    public function setSqlFieldIndex($value)
    {
        $this ->sqlFieldIndex = $value;
        return $this;
    }

    /**
     * @param string $sqlFieldName
     * @return FormField
     */
    public function setSqlFieldName($sqlFieldName)
    {
        $this->sqlFieldName = $sqlFieldName;
        return $this;
    }

    /**
     * @param boolean $primary
     * @return FormField
     */
    public function setPrimary($primary)
    {
        $this->primary = $primary;
        return $this;
    }

    /**
     * @param boolean $useDefaultValueToEditNull
     * @return FormField
     */
    public function setUseDefaultValueToEditNull($useDefaultValueToEditNull)
    {
        $this->useDefaultValueToEditNull = $useDefaultValueToEditNull;
        return $this;
    }

    /**
     * @param string $otherHtmlParams
     * @return FormField
     */
    public function setOtherHtmlParams($otherHtmlParams)
    {
        $this->otherHtmlParams = $otherHtmlParams;
        return $this;
    }

    /**
     * @param string $text
     * @return FormField
     */
    public function setText($text)
    {
        $this->text = $text;
        return $this;
    }

    /**
     * @param string $functionName
     * @return FormField
     */
    public function setFunctionName($functionName)
    {
        $this->functionName = $functionName;
        return $this;
    }

    /**
     * @param string $dataType
     * @return FormField
     */
    public function setDataType($dataType)
    {
        $this->dataType = $dataType;
        return $this;
    }



    /**
     * @param mixed $defaultValue
     * @return FormField
     */
    public function setDefaultValue($defaultValue)
    {
        $this->defaultValue = $defaultValue;
        return $this;
    }

    /**
     * @param string $caption
     * @param bool $translate
     * @return FormField
     */
    public function setCaption($caption, $translate = true)
    {
        $this->caption = $translate ? ronText($caption) : $caption;
        return $this;
    }

    /**
     * @param string $caption2
     * @param bool $translate
     * @return FormField
     */
    public function setCaption2($caption2, $translate = true)
    {
        $this->caption2 = $translate ? ronText($caption2) : $caption2;
        return $this;
    }

    /**
     * @param string|SqlStatement $selectSql
     * @return FormField
     */
    public function setSelectSql($selectSql)
    {
        $this->selectSql = $selectSql;
        return $this;
    }

    /**
     * @param boolean $selectDataTypeTest
     * @return FormField
     */
    public function setSelectDataTypeTest($selectDataTypeTest)
    {
        $this->selectDataTypeTest = $selectDataTypeTest;
        return $this;
    }

    /**
     * Nastavi jestli se ma zobrazovat prazdny radek. Hodnota null znamena, ze se bude zobrazovat automaticky (tj. pro required pole pri edit ne, jinak ano)
     * @param boolean|null $showEmptyValue
     * @return FormField
     */
    public function setShowEmptyValue($showEmptyValue)
    {
        $this->showEmptyValue = $showEmptyValue;
        return $this;
    }


    /**
     * @param boolean $currentTimeButton
     * @return FormField
     */
    public function setCurrentTimeButton($currentTimeButton)
    {
        $this->currentTimeButton = $currentTimeButton;
        return $this;
    }


    /**
     * @param array $attrIndexes
     * @return FormField
     */
    public function setAttrIndexes($attrIndexes)
    {
        $this->attrIndexes = $attrIndexes;
        return $this;
    }




    /**
     * @param string $fieldId
     * @return FormField
     */
    public function setFieldId($fieldId)
    {
        $this->fieldId = $fieldId;
        return $this;
    }

    /**
     * @param string $fieldCssClass
     * @return FormField
     */
    public function setFieldCssClass($fieldCssClass)
    {
        $this->fieldCssClass = $fieldCssClass;
        return $this;
    }

    /**
     * @param string $rowId
     * @return FormField
     */
    public function setRowId($rowId)
    {
        $this->rowId = $rowId;
        return $this;
    }

    /**
     * @param string $rowClass
     * @return FormField
     */
    public function setRowClass($rowClass)
    {
        $this->rowClass = $rowClass;
        return $this;
    }

    /**
     * Nastavi options pro ComboBox
     * @param array $options
     * @param bool $dataTypeTest
     * @return FormField
     */
    public function setOptions($options, $dataTypeTest = true)
    {
        $this->options = $options;
        if (isset($dataTypeTest)) {
            $this->selectDataTypeTest = true;
        }
        return $this;
    }

    /**
     * @param int $maxLength
     * @return FormField
     */
    public function setMaxLength($maxLength)
    {
        $this->maxLength = $maxLength;
        return $this;
    }

    /**
     * @param int $rows
     * @param int|null $cols
     * @return FormField
     */
    public function setRows($rows, $cols = null)
    {
        $this->rows = $rows;
        if (isset($cols)) {
            $this ->cols = $cols;
        }
        return $this;
    }

    /**
     * @param int $cols
     * @return FormField
     */
    public function setCols($cols)
    {
        $this->cols = $cols;
        return $this;
    }

    /**
     * @param boolean $wrap
     * @return FormField
     */
    public function setWrap($wrap)
    {
        $this->wrap = $wrap;
        return $this;
    }






    /**
     * Inicializuje pole podle asociativniho pole $options
     * @param mixed[] $options
     */
    function init($options) {
        if (!$options) return;

        $this ->editable = true;
        $this ->selectDataTypeTest = true;

        foreach ($options as $key => $value) {
            switch ($key) {
                case FORM_FIELD_TYPE :
                    $this ->setFieldType($value);
                    break;

                case FORM_FIELD_NAME :
                    $this ->setFieldName($value);
//                    // POZOR! Nastavuje i sql_fieldname (to se da dodatecne zmenit)
//                    if ($value[0] == '*') {
//                        $this->fieldName = substr($value, 1); //  str_replace('*', '', $value[1]);
//                        $this->sqlFieldName = '';
//                    } else {
//                        $this->fieldName = $value;
//                        $this->sqlFieldName = $this->fieldName;
//                    }
                    break;

                case FORM_FIELD_SQL_NAME :
                    $this ->sqlFieldName = $value;
                    break;

                case FORM_FIELD_SQL_INDEX :
                    $this ->sqlFieldIndex = $value;
                    break;

                case FORM_FIELD_READONLY :
                    $this ->editable = !$value;
                    break;

                case FORM_FIELD_SQL :
                    $this->selectSql = $value;
                    break;

                case FORM_FIELD_EDIT :
                    $this ->editable = $value;
                    break;

                case FORM_FIELD_CAPTION_TEXT :
                    $this ->caption = ronText($value);
                    break;

                case FORM_FIELD_PRIMARY :
                case FORM_FIELD_REQUIRED :
                case FORM_FIELD_CAPTION :
                case FORM_FIELD_CAPTION2 :
                case FORM_FIELD_DEFAULT_VALUE :
                case FORM_FIELD_DATA_TYPE :
                case FORM_FIELD_MAXLENGTH :
                case FORM_FIELD_ROWS :
                case FORM_FIELD_COLS :
                case FORM_FIELD_WRAP :
                case FORM_FIELD_TEXT :
                case FORM_FIELD_OPTIONS :
                case FORM_FIELD_ROW_ID :
                case FORM_FIELD_ROW_CLASS :
                default :
                    $this ->{$key} = $value;
                    break;
            }
        }
    }


    /**
     * Nacte nastaveni jedne vlastnosti (atributu) z konfigurace
     * @param $key
     * @param $value
     * @return BrowserField
     */
    function loadConfigurationItem($key, $value)
    {
        if (substr($key, 0, 5) === 'html:') {
            if (!$this ->otherHtmlParams) {
                $this ->otherHtmlParams = [];
            }

            $this ->otherHtmlParams[substr($key, 5)] = $value;
        } else {
            switch ($key) {
                case 'fieldName' :
                case 'sqlFieldName' :
                case 'fieldType' :
                case 'defaultValue' :
                case 'selectSql' :
                case 'dataType' :
                    $this->{$key} = $value;
                    break;

//                case 'class' :
//                    $this ->className = $value;
//                    break;
//
//                case 'addClass' :
//                    $this ->className .= ($this ->className ? ' ' : '') . $value;
//                    break;

                case 'caption' :
                case 'caption2' :
                case 'text' :
                    $this->{$key} = ronText($value);
                    break;

                case 'visible' :
                case 'required' :
                case 'changeable' :
                case 'sqlFieldIndex' :
                case 'useDefaultValueToEditNull' :
                case 'selectDataTypeTest' :
                case 'maxLength' :
                case 'rows' :
                case 'cols' :
                case 'wrap' :
                case 'currentTimeButton' :
                    $this->{$key} = (int)$value;
                    break;

                case 'readOnly' :
                    $this->editable = !(int)$value;
                    break;
            }
        }
    }


    /**
     * Zpracuje konfiguraci sloupce nactenou z XML pomoci @XmlLoader
     * @param $confArray mixed[]
     */
    function loadConfigurationArray($confArray)
    {
        if (!is_array($confArray)) {
            return;
        }

        foreach ($confArray as $key => $value) {
            $this ->loadConfigurationItem($key, $value);
        }

        // TODO: options
    }


    /**
     * @param Database $database
     * @param string $fieldType Muze byt:
     *                          edit (normalni text),
     *                          actualtime (aktualni cas)
     *                          date (text reprezentujici datum)
     *                          check (booleovska hodnota),
     *                          password (heslo - zakoduje se)
     *                          memo
     *                          - pro jiny typ vraci prazdny retezec
     * @param mixed $value
     * @return string Vraci SQL konstantu pro hodnotu zadaneho typu
     */
    public static function fieldValueToSQLConst($database, $fieldType, $value)
    {
        switch ($fieldType) {
            case FORM_FIELD_TYPE_EDIT :
            case FORM_FIELD_TYPE_MEMO :
            case FORM_FIELD_TYPE_SELECT :
            case FORM_FIELD_TYPE_PASSWORD :
            case 'passwd' : // pro kompatibilitu, nepouzivat!
                return $value ? "'" . $database ->addSlashes($value) . "'" : 'NULL';

            case FORM_FIELD_TYPE_RADIO :
            case FORM_FIELD_TYPE_COMBO :
                return (isset($value) && $value != '') ? (int)$value : 'NULL';

            case FORM_FIELD_TYPE_COMBOSTR :
                return (isset($value) && $value != '') ? "'" . $database ->addSlashes($value) . "'" : 'NULL';

            case FORM_FIELD_TYPE_ACTUAL_TIME :
                return $database ->timestampToString(time());

            case FORM_FIELD_TYPE_ACTUAL_DATE :
                return $database ->timestampToString(DateUtils::startOfTheDay(time()));

            case FORM_FIELD_TYPE_DATE :
                $datum = DateUtils::dateStringToTimestamp($value);
                if ($datum) return $database ->timeToString($datum);
                else return 'NULL';

            case FORM_FIELD_TYPE_DATETIME :
                $datum = DateUtils::datetimeStringToTimestamp($value);
                if ($datum) return $database ->timeToString($datum);
                else return 'NULL';

            case FORM_FIELD_TYPE_INT :
                return (isset($value) && $value != '') ? (int)$value : 'NULL';

            case FORM_FIELD_TYPE_FLOAT :
                return (isset($value) && $value != '') ? (float)$value : 'NULL';

            case FORM_FIELD_TYPE_HIDDEN :
            case FORM_FIELD_TYPE_ID :       // Tohle bylo zapoznamkovane. Proc? Nefungovala kvuli tomu editace dotazu a aktualit
                return stripslashes($value);


            case FORM_FIELD_TYPE_CHECK :
                return $value ? '1' : '0';
        }

        return null;
    }


    function getPostValueText()
    {
        if ($this->fieldType == FORM_FIELD_TYPE_DATEANDTIME) {
            return array($_POST[$this->fieldName], $_POST[$this->fieldName . '_time']);
        } else {
            return $_POST[$this->fieldName];
        }
    }


    function getPostValue()
    {
        $value = $this->getPostValueText();

        switch ($this->fieldType) {
            case FORM_FIELD_TYPE_EDIT :
            case FORM_FIELD_TYPE_MEMO :
            case FORM_FIELD_TYPE_SELECT :
                $value = trim($value);
                // pokracuje dale
            case 'passwd' :
            case FORM_FIELD_TYPE_PASSWORD :
                return $value != '' ? $value : null;

            case FORM_FIELD_TYPE_COMBOSTR :
                return $value != '' ? $value : null;

            case FORM_FIELD_TYPE_COMBO :
            case FORM_FIELD_TYPE_RADIO :
                return (isset($value) && trim($value) != '') ? $value : null;

            case FORM_FIELD_TYPE_ACTUAL_TIME :
                return time();

            case FORM_FIELD_TYPE_ACTUAL_DATE :
                return DateUtils::startOfTheDay(time());

            case FORM_FIELD_TYPE_DATE :
                if (trim($value) == '') return null;

                return DateUtils::dateStringToTimestamp($value);

            case FORM_FIELD_TYPE_DATETIME :
                if (trim($value) == '') return null;

                return DateUtils::datetimeStringToTimestamp($value);

            case FORM_FIELD_TYPE_DATEANDTIME :
                if ((trim($value[0]) == '') || (trim($value[1]) == '')) return null;

                return DateUtils::datetimeStringToTimestamp($value[0]) + DateUtils::timeStringToTimestamp($value[1], 0, true);

            case FORM_FIELD_TYPE_INT :
                if (trim($value) == '') return null;

                return (int)$value;

            case FORM_FIELD_TYPE_FLOAT :
                if (trim($value) == '') return null;

                return StringUtils::stringToFloat($value);

            case FORM_FIELD_TYPE_HIDDEN :
            case FORM_FIELD_TYPE_ID : // Tohle bylo zapoznamkovane. Proc? Nefungovala kvuli tomu editace dotazu a aktualit
                return stripslashes($value);

            case FORM_FIELD_TYPE_CHECK :
                return $value ? 1 : 0;
        }

        return null;
    }


    function showCalendarButton()
    {
        /** @noinspection HtmlUnknownTarget */
        echo "<i class='showcalendar inline-glyph glyph-button fa glyph-calendar' data-roncalendar-inputname='$this->fieldName'></i>";
    }


    function showCurrentTimeButton()
    {
        if (!$this ->fieldId) {
            return;
        }

        echo
            "<button class='button actual-time-button' type='button' data-date-input='{$this ->fieldId}' data-time-input='{$this ->fieldId}_time'>" .
            ronText('Insert_current_time') .
            '</button>';
    }


    function getHtmlParams()
    {
        if (!$this ->otherHtmlParams) {
            return '';
        }

        if (is_array($this ->otherHtmlParams)) {
            $result = '';
            foreach ($this ->otherHtmlParams as $key => $value) {
                $result .= " $key='" . StringUtils::htmlTextConvert($value) . "'";
            }

            return $result;
        } else {
            return $this ->otherHtmlParams;
        }
    }

    function getFieldCssClass()
    {
        switch ($this->fieldType) {
            case FORM_FIELD_TYPE_DATE :
                $className = 'inputDate';
                break;
            case FORM_FIELD_TYPE_DATETIME :
                $className = '';
                break;

            case FORM_FIELD_TYPE_INT :
                $className = "form-input inputInt";
                break;

            case FORM_FIELD_TYPE_FLOAT :
                $className = "form-input inputFloat";
                break;

            case FORM_FIELD_TYPE_CHECK :
                $className = "app-check";
                break;

            case FORM_FIELD_TYPE_EDIT :
            default :
                $className = "form-input";
                break;
        }

        $className .=
            ($this ->fieldCssClass ? ' ' . $this ->fieldCssClass : '') .
            ($this ->fieldName
                ? ' ' . MessageManager::getClass($this ->fieldName)
                : ''
            );

        return $className;
    }

    /**
     * Zobrazi obsah pole.
     * @param Database $database
     * @param mixed $value
     * @param bool  $canEdit    Pokud je false, nebude pole editovatelne
     * @param bool  $insert     Urcuje, jestli se jedna o vkladani noveho zaznamu
     * @param array $rowData    Data nactena z databaze
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $className = $this ->getFieldCssClass();

        $maxLength = $this ->maxLength;
        if (is_array($this ->otherHtmlParams)  &&  isset($this ->otherHtmlParams['maxlength'])) {
            $maxLength = $this ->otherHtmlParams['maxlength'];
        }

        switch ($this->fieldType) {
            case FORM_FIELD_TYPE_DATE :
            case FORM_FIELD_TYPE_EDIT :
            case FORM_FIELD_TYPE_INT :
            case FORM_FIELD_TYPE_FLOAT :
            case FORM_FIELD_TYPE_DATETIME :
                if ($canEdit) // $readonly = ""; else $readonly = 'readonly="readonly"';
                {
                    echo
                        '<INPUT class="' . $className . '" type="text" name="' . $this->fieldName . '"' .
                        ($this ->fieldId ? " id='{$this ->fieldId}'" : '');

                    if ($maxLength) {
                        echo
                            ' maxlength="' . $maxLength . '"' .
                            ' size="' . (($maxLength <= 60) ? $maxLength : '60') . '"';
                    }

                    echo
                        $this ->getHtmlParams() .
                        ' ' . MessageManager::getTitleHtml($this ->fieldName) .
                        ' value="' . StringUtils::htmlTextConvert($value) . '">';

                    if ($this->fieldType == FORM_FIELD_TYPE_DATE) {
                        $this->showCalendarButton();
                    }
                } else {
                    echo StringUtils::htmlTextConvert($value);
                }
                break;

            case FORM_FIELD_TYPE_DATEANDTIME : // 2 samostatne pole pro zadani casu a data
                if ($canEdit) {
                    $valuestr = $value ? $value[0] : '';

                    echo
                        '<input class="inputDate" type="text" name="' . $this->fieldName . '" maxlength="10" size="10"' .
                        ($this ->fieldId ? " id='{$this ->fieldId}'" : '') .
                        $this ->getHtmlParams() .
                        ' value="' . StringUtils::htmlTextConvert($valuestr) . '">';

                    $this->showCalendarButton();

                    $valuestr = $value ? $value[1] : '';

                    echo
                        ' <input class="inputTime" type="text" name="' . $this->fieldName . '_time" maxlength="8" size="8"' .
                        ($this ->fieldId ? " id='{$this ->fieldId}_time'" : '') .
                        $this ->getHtmlParams() .
                        ' value="' . StringUtils::htmlTextConvert($valuestr) . '">';

                    if ($this ->currentTimeButton) {
                        echo ' ';
                        $this->showCurrentTimeButton();
                    }
                } else {
                    echo $value
                        ? $value[0] . ' ' . $value[1]
                        : '';
                }
                break;

            case 'passwd' :
            case FORM_FIELD_TYPE_PASSWORD :
                echo
                    "<INPUT class='$className' type='password' name='{$this->fieldName}'" .
                    ($maxLength ? ' maxlength="' . $maxLength . '"' : '') .
                    ' value="">';
                break;

            case FORM_FIELD_TYPE_MEMO :
                $cols = $this->cols ? $this->cols : 60;
                $rows = $this->rows ? $this->rows : 8;
                $wrap = $this->wrap ? $this->wrap : 'virtual';
                if ($canEdit) {
                    echo
                        '<TEXTAREA class="form-input" cols="' . $cols . '" rows="' . $rows . '" name="' . $this->fieldName . '"' . ($this ->fieldId ? ' id="' . $this ->fieldId . '"' : '') .
                        ' wrap="' . $wrap . '">' . StringUtils::htmlTextConvert($value) . '</TEXTAREA>';
                } else {
                    echo str_replace("\n", '<br/>', StringUtils::htmlTextConvert($value));
                }
                break;

            case FORM_FIELD_TYPE_CHECK :
                if ($canEdit) {
                    echo
                        "<INPUT class='$className' type='checkbox' name='{$this->fieldName}' " .
                        ($value ? 'checked="checked" ' : '') .
                        $this ->getHtmlParams() .
                        '>';
                } else {
                    echo ronText($value ? 'Str_Yes' : 'Str_No');
                }
                break;

            case FORM_FIELD_TYPE_SELECT :
                require_once('ron/controls.php');

                //createcombo(
                //    $connection, $this -> selectSql, $this -> fieldName,
                //    0, 1, ' ', 0, $value, '', $canedit, $this -> otherHtmlParams);
                $combo = new ComboBox($this->fieldName);
                $combo ->dataTypeComparison = $this ->selectDataTypeTest;
                if ($this ->showEmptyValue !== false) {
                    if ($this->showEmptyValue === true || !($this->required && $this->defaultValue)) {
                        $combo->emptyRow = $this ->showEmptyValue;
                    }
                }
                $combo ->alwaysShowSelected = true;
                $combo ->attrIndexes = $this ->attrIndexes;
                $combo ->htmlParams = $this ->getHtmlParams() . ($this->fieldId ? " id='" . $this->fieldId . "'" : '');;
                $combo ->className = $className;
                $combo ->readOnly = !$canEdit;
                $combo ->loadData($database, $this->selectSql, $value);
                $combo->render(); // width = 0
                break;

            case FORM_FIELD_TYPE_COMBO :
            case FORM_FIELD_TYPE_COMBOSTR :
                if ($canEdit) {
                    require_once('ron/controls.php');
                    ComboBox::renderComboFromArray($this->options,
                        $this->fieldName,
                        $value, $this->getHtmlParams() . ($this ->fieldId ? ' id="' . $this ->fieldId . '"' : ''), $this->selectDataTypeTest);
                } else {
                    echo $this->options[$value];
                }
                break;

            case FORM_FIELD_TYPE_RADIO :
                if ($canEdit) {
                    foreach ($this ->options as $key => $text) {
                        echo
                            '<label class="margin-right-2">' .
                            "<input type='radio' name='{$this ->fieldName}' value='{$key}' " . (($value == $key) ? 'checked="checked"' : '') .
                            $this ->getHtmlParams() .
                            '/>' .
                            $text .
                            '</label>';
                    }
                } else {
                    echo $this->options[$value];
                }
                break;



            case FORM_FIELD_TYPE_TEXT :
                $params =
                    $this ->getHtmlParams() .
                    ($this ->fieldId ? " id='{$this ->fieldId}'" : '');

                echo
                    ($params ? "<div $params>" : '') .
                    $this->text .
                    ($params ? '</div>' : '');
                break;

            /** @noinspection PhpMissingBreakStatementInspection */
            case FORM_FIELD_TYPE_FILENAME :
                if (!$insert && $canEdit) {
                    echo '<INPUT type=text name="' . $this->fieldName . '" value="' . StringUtils::htmlTextConvert($value) . '">';
                    break;
                }
            // POKRACUJE SE DALE!
            case FORM_FIELD_TYPE_FILE :
                if ($canEdit) echo '<INPUT type="file" name="' . $this->fieldName . '">';
                else echo StringUtils::htmlTextConvert($value);
                break;
        }
    }


    /**
     * Zobrazi formularove pole.
     * @param $connection
     * @param $value
     * @param bool $canEdit         Pomoci $canedit lze zakazat editaci.
     * @param bool $tableRow        Urcuje jestli se ma zobrazit v radku tabulky
     * @param bool $insert          Urcuje, jestli se jedna o vkladani noveho zaznamu
     * @param mixed[] $rowData      Data nactena z databaze
     */
    function showField($connection, $value, $canEdit, $tableRow, $insert, $rowData)
    {
        $codeHtml = $this ->code
            ? " data-code='{$this ->code}'"
            : '';

        $idHtml = $this ->fieldId
            ? " id='{$this ->fieldId}'"
            : '';

        switch ($this->fieldType) {
            /** @noinspection PhpMissingBreakStatementInspection */
            case FORM_FIELD_TYPE_ACTUAL_TIME :
            /** @noinspection PhpMissingBreakStatementInspection */
            case FORM_FIELD_TYPE_ACTUAL_DATE :
                if (!$canEdit) return;
            // jinak pokracuje dale
            case FORM_FIELD_TYPE_ID :
                echo "<input type='hidden' name='{$this->fieldName}' $codeHtml $idHtml value='" . StringUtils::htmlTextConvert($value) . "' />";
                return;

            case FORM_FIELD_TYPE_HIDDEN :
                echo "<input type='hidden' name='{$this->fieldName}' $codeHtml $idHtml value='" . StringUtils::htmlTextConvert($value) . "' />";
                return;
        }

        $rowid = isset($this ->rowId)
            ? $this ->rowId
            : ( $this ->fieldName ? 'row_' . $this ->fieldName : '');

        if ($tableRow) {
            echo '<div' . ($rowid ? " id='$rowid'" : '') . ($this ->rowClass ? " class='{$this ->rowClass}'" : '') . '>';
        }

        if ($this ->fieldType == FORM_FIELD_TYPE_SECTION) {
            echo
                ($codeHtml ? "<span $codeHtml>" : '') .
                $this ->caption .
                ($codeHtml ? '</span>' : '');
        } else {
            // Nadpis pole
            echo '<div class="form-col form-col1 caption' . ($this->required ? ' required' : '') . '">' . $this->caption . '</div>';

            // Hodnota pole
            echo "<div class='form-col form-col2' $codeHtml>";
            $this->showFieldContent($connection, $value, $canEdit, $insert, $rowData);
            if ($this->caption2) {
                echo $this->caption2;
            }
            echo '</div>';
        }

        if ($tableRow) {
            echo '</div>';
        }

        echo "\n";
    }


    /**
     * Zobrazi zadane pole pro vkladani - naplni ho defaultni hodnotou
     * nebo hodnotou, kterou uz uzivatel zadal (pokud je $usepostvalues true) - napr.
     * v pripade, ze zapomnel vyplnit nejakou povinnou hodnotu
     * pri kopirovani existujiciho zaznamu muze byt zadano i $rowdata
     * @param Database $database
     * @param bool    $usePostValues
     * @param bool    $tableRow
     * @param mixed[] $rowData
     */
    function showFieldInsert($database, $usePostValues, $tableRow, $rowData = null)
    {
        switch ($this->fieldType) {
            case FORM_FIELD_TYPE_HIDDEN :
            case FORM_FIELD_TYPE_ID :
            case FORM_FIELD_TYPE_PASSWORD :
            case 'passwd' :
                $value = $this->defaultValue;
                break;

            case FORM_FIELD_TYPE_ACTUAL_TIME :
                $value = null; // $database ->timestampToString(time());
                break;

            case FORM_FIELD_TYPE_ACTUAL_DATE :
                $value = null; // $database ->timestampToString(DateUtils::startOfTheDay(time()));
                break;

            case FORM_FIELD_TYPE_DATEANDTIME :
                // datum a cas rozdeleny do dvou poli - jako hodnota se predava pole se dvema prvky!
                if ($usePostValues) $value = $this->getPostValueText(); // array($_POST[$this -> fieldName], $_POST[$this -> fieldName.'_time']);
                else if ($this->defaultValue) $value = array(date('j.n.Y', $this->defaultValue), date('H:i', $this->defaultValue));
                else $value = array('', '');
                break;

            default :
                $defvalue = isset($_GET[$this->fieldName]) ? $_GET[$this->fieldName] : $this->defaultValue;
                //$value = $usepostvalues ? $_POST[$this -> fieldName] : $defvalue;
                $value = $usePostValues ? $this->getPostValueText() : $defvalue;
        }

        $this->showField($database, $value, true, $tableRow, true, $rowData); // canedit = true, insert = true, rowdata = null (insert)
    }



    /**
     * Zobrazi zadane pole pro editaci.
     * @param Database  $database
     * @param mixed     $value
     * @param bool      $tableRow
     * @param mixed[]   $rowData
     */
    function showFieldEdit($database, $value, $tableRow, $rowData)
    {
        if ($this ->useDefaultValueToEditNull  &&  !isset($value)) {
            $value = $this ->defaultValue;
        }

        if ($this->primary && $this->fieldType != FORM_FIELD_TYPE_HIDDEN && $this->fieldType != FORM_FIELD_TYPE_ID) {
            // hidden pole a "id" pole se vlozi automaticky
            echo '<INPUT type="hidden" name="' . $this->fieldName . '" value="' . StringUtils::htmlTextConvert($value) . '">';
        }

        $canEdit = $this->editable && !$this->primary;
        if (!$this ->changeable  &&  $value) {
            $canEdit = false;
        }

        $this->showField($database, $value, $canEdit, $tableRow, false, $rowData); // insert = false
    }



    /**
     * Zobrazi zadane pole bez moznosti editace
     * @param Database  $database
     * @param mixed     $value
     * @param bool      $tableRow
     * @param mixed[]   $rowData
     */
    function showFieldReadOnly($database, $value, $tableRow, $rowData)
    {
        $this->showField($database, $value, false, $tableRow, false, $rowData); // canedit = false, insert = false
    }


    function createDBRecordField($database, $tableName, $insert)
    {
        if (!$this->sqlFieldName) return NULL;

        switch ($this->fieldType) {
            case FORM_FIELD_TYPE_EDIT :
            case FORM_FIELD_TYPE_SELECT :
            case FORM_FIELD_TYPE_PASSWORD :
            case 'passwd' :
            case FORM_FIELD_TYPE_COMBOSTR :
                $field = new DBRecordField($this->sqlFieldName, '', !$this->editable, $this->maxLength); // format = ''
                $field ->setValue($this->getPostValue());
                break;

            case FORM_FIELD_TYPE_MEMO :
                $field = new DBRecordField($this->sqlFieldName, 'memo', !$this->editable); // format = ''
                $field->setValue($this->getPostValue());
                break;

            case FORM_FIELD_TYPE_ID : // Tohle bylo zapoznamkovane. Proc? Nefungovala kvuli tomu editace dotazu a aktualit (poznamka tu byla uz pred prepracovanim na DBRecord)
                $field = new DBRecordField($this->sqlFieldName, 'int', !$this->editable, $this->maxLength);
                if ($insert) {
                    require_once('ron/sqlutils.php');
                    $field ->setValue(SQLUtils::newSerialNumber($database, $tableName, $this->sqlFieldName));
                } else {
                    $field->setValue($this->getPostValue());
                }
                break;

            case FORM_FIELD_TYPE_HIDDEN : // ? ? ?   TODO: Opravit - doplnit moznost zadat datovy typ?
                $field = new DBRecordField($this->sqlFieldName, $this ->dataType, !$this->editable, $this->maxLength);
                $field->setValue($this->getPostValue());
                break;

            case FORM_FIELD_TYPE_COMBO :
            case FORM_FIELD_TYPE_RADIO :
            case FORM_FIELD_TYPE_CHECK :
            case FORM_FIELD_TYPE_INT :
                $field = new DBRecordField($this->sqlFieldName, 'int', !$this->editable, $this->maxLength);
                $field->setValue($this->getPostValue());
                break;

            case FORM_FIELD_TYPE_FLOAT :
                $field = new DBRecordField($this->sqlFieldName, 'float', !$this->editable, $this->maxLength);
                $field->setValue($this->getPostValue());

                break;

            case FORM_FIELD_TYPE_ACTUAL_TIME :
            case FORM_FIELD_TYPE_ACTUAL_DATE :
            case FORM_FIELD_TYPE_DATE :
            case FORM_FIELD_TYPE_DATETIME :
            case FORM_FIELD_TYPE_DATEANDTIME :
                $field = new DBRecordField($this->sqlFieldName, 'datetime', !$this->editable, 0); // format = ''
                $field->setValue($this->getPostValue());

                break;

            default :
                return null;
        }

        $field ->setUpdateLog($this ->updateLog);
        $field ->setChangeable($this ->changeable);

        return $field;
    }


    /**
     * @param Database $database
     * @param mixed $dbValue
     * @return string  Vraci text, ktery se zobrazi uzivateli pro zadanou hodnotu z databaze
     */
    function getValueFromDBValue($database, $dbValue)
    {
        switch ($this->fieldType) {
            case FORM_FIELD_TYPE_DATE :
                return $dbValue ? date('j.n.Y', $database ->stringToTimestamp($dbValue)) : null;

            case FORM_FIELD_TYPE_DATETIME :
                return $dbValue ? date('j.n.Y H:i', $database ->stringToTimestamp($dbValue)) : null;

            case FORM_FIELD_TYPE_DATEANDTIME :
                $timestamp = $database ->stringToTimestamp($dbValue);

                return $timestamp ? array(date('j.n.Y', $timestamp), date('H:i', $timestamp)) : null;

            case FORM_FIELD_TYPE_HIDDEN :
                switch ($this->dataType) {
                    case 'date' :
                    case 'datetime' :
                        return $dbValue ? $database ->stringToTimestamp($dbValue) : null;

                    case 'int' :
                    default :
                        return $dbValue;
                }
        }

        return $dbValue;
    }


    /**
     * Vraci false pokud je pole povinne a jeho hodnota neni zadana, jinak vraci true
     *
     * @param bool $edit Pokud je $edit true a pole neni editovatelne, tak vraci true
     * @return bool
     */
    function checkRequired($edit)
    {
        if ($edit && !$this->editable) return true; // pole nelze editovat
        if (!$this->required) return true;
        if (!$this->fieldName) return true;

        $value = $_POST[$this->fieldName];

        if ($this->fieldType == FORM_FIELD_TYPE_DATE && $value != NULL && $value != '') {
            try {
                $value = DateUtils::dateStringToTimestamp($_POST[$this->fieldName]);
                if (!$value) return false;
            } catch (\Exception $e) {
                return false;
            }
        }

        if ($this->fieldType == FORM_FIELD_TYPE_DATETIME && $value != NULL && $value != '') {
            try {
                $value = DateUtils::datetimeStringToTimestamp($_POST[$this->fieldName]);
                if (!$value) return false;
            } catch (\Exception $e) {
                return false;
            }
        }

        if ($this->fieldType == FORM_FIELD_TYPE_CHECK || $this->fieldType == FORM_FIELD_TYPE_ID) return true;

        return ($value != NULL && $value != '');
    }

}
