<?php
/**
 * UTF8
 */

namespace Ron;

class GuiUtils
{
    /**
     * Zobrazi checkbox. Pokud je zadan text, tak renderuje i <label>
     * @param string $varName jmeno html inputu
     * @param string $text zobrazeny nadpis
     * @param bool $checked
     * @param string[]|string $params   Asociativni pole parametru nebo string s parametry
     * @param string $className         css class - pokud je zadana, prebije pripadnou hodnotu v $params
     * @param null $value
     * @param bool $textFirst
     */
    static function renderCheckbox($varName, $text, $checked, $params = '', $className = null, $value = null, $textFirst = false )
    {
        if ($text) echo '<label>';

        //$className = $className ?: 'app_check';

        $valueAttr = isset($value) ? "value='$value'" : '';

        if ($text  &&  $textFirst) {
            echo $checked
                ? "<span class='checkbox_text_checked'>$text</span>"
                : "<span class='checkbox_text_unchecked'>$text</span>";
        }

        if (is_array($params)) {
            if (isset($className)) {
                $params['class'] = $className;
            }

            $paramsStr = '';
            foreach ($params as $pName => $pValue) {
                $paramsStr .=
                    ($paramsStr ? ' ' : '') .
                    $pName . '="' . StringUtils::htmlTextConvert($pValue) . '"';
            }

        } else {
            $paramsStr =
                ($params ?: '') .
                ($className ? " class='$className'" : '');
        }

        echo
            "<input type='checkbox' name='$varName' $valueAttr" .
            ($checked ? ' checked="checked"' : '') .
            $paramsStr .
            '>';

        if ($text  &&  !$textFirst) {
            echo $checked
                ? "<span class='checkbox_text_checked'>$text</span>"
                : "<span class='checkbox_text_unchecked'>$text</span>";
        }

        if ($text) echo '</label>';
        echo "\n";
    }


    /**
     * Vraci url parametry pro nastaveni polohy (&x=...&y=...)
     * @return string
     */
    static function scrollPageUrlParams()
    {
        $result = '';

        $polohaY = null;
        if (isset($_POST['y'])) $polohaY = (int)$_POST['y'];
        else if (isset($_GET['y'])) $polohaY = (int)$_GET['y'];

        $result .= $polohaY ? '&y=' . $polohaY : '';

        $polohaX = null;
        if (isset($_POST['x'])) $polohaX = (int)$_POST['x'];
        else if (isset($_GET['x'])) $polohaX = (int)$_GET['x'];

        $result .= $polohaX ? '&x=' . $polohaX : '';

        return $result;
    }

    /**
     * Vygeneruje skript pro zaskrolovani stranky ihned po jejim nacteni. Polohu urci z POST/GET hodnot x a y.
     * Pokud nejsou promenne v parametrech zadany, negeneruje nic.
     */
    static function scrollPageScript()      // pridat nepovinny parametr $element pro moznost zaskrolovani k zadanemu elementu?
    {
        $polohaX = null;
        if (isset($_POST['x'])) $polohaX = (int)$_POST['x'];
        else if (isset($_GET['x'])) $polohaX = (int)$_GET['x'];

        $polohaY = null;
        if (isset($_POST['y'])) $polohaY = (int)$_POST['y'];
        else if (isset($_GET['y'])) $polohaY = (int)$_GET['y'];

        if ($polohaX  ||  $polohaY) {
            $jsOptions = [];
            if ($polohaX) $jsOptions['x'] = $polohaX;
            if ($polohaY) $jsOptions['y'] = $polohaY;

            echo '<script' . '>ron.utils.initialScrollPage(' . json_encode($jsOptions) . ')</script>';
        }
    }


    /**
     * Vraci html text pro zobrazeni potvrzovaci hlasky
     * @param string $text
     * @param string $question
     * @param string $url
     * @param string $htmlParams
     * @return string
     */
    static function changeLocationQuest($text, $question, $url, $htmlParams = null)
    {
        return
            '<span class="js confirmQuestion" data-ron-href="'. StringUtils::htmlTextConvert($url) . '" data-ron-confirm-text="' . StringUtils::htmlTextConvert($question) . '" ' . ($htmlParams ?: '') . '>' .
            $text .
            "</span>\n";
    }


    /**
     * Vraci HTML reprezentaci zadane barvy (#rrggbb)
     * @param int $color
     * @return string
     */
    static function getHtmlColor($color)
    {
        $result = dechex($color % 256 * 0x10000 + (($color / 0x100) % 0x100) * 0x100 + $color / 0x10000);

        while ( strlen($result) < 6 ) $result = '0'.$result;
        return '#'.$result;
    }

}



if (file_exists('ron/guiutils_old.php')) {
    require_once('ron/guiutils_old.php');
}
