<?php
/**
 * UTF8
 * stanislav.elbl, 27.10.2015
 */

namespace Ron;

use Ron\DB\Database;


class Preference
{
    const DELETE = -1; // slouzi pro odstraneni starych nepouzivanych nastaveni
    const CATEGORY = 1;
    const CHECKBOX = 2;
    const TEXT = 3;
    const MEMO = 4;
    const COMBOBOX = 5;
    const SQL_COMBOBOX = 6;
    const COLOR = 7;

    const INFO_TEXT = 10;
    /**  jen readonly nastaveni - meni se jinym zpusobem, napr. v aplikaci */
    const READONLY = 11;

    //const CUSTOM = 100;  ?

    /** @var string     jmeno, ktere se uklada do textu ({$name}=...) */
    public $name;

    /** @var int    typ - viz konstanty vyse */
    public $type;

    /** @var string     zobrazeny titulek (preklada se pomoci ronText!) */
    public $title;
    /** @var string     dodatecny text zobrazeny za editacnim prvkem */
    public $infoText;
    public $readOnly;
    /** @var bool       Priznak, ze se nema zobrazovat nadpis a polozka bude pres oba sloupce (napr. pro memo) */
    public $noTitle;

    /** @var string     Class pro color-indicator - pouziva se kvuli zobrazeni vychozi barvy */
    public $indicatorClass;

    /** @var array  pro combo box - asociativni pole "hodnota" => "nadpis" */
    public $options = [];
    /** @var        string pro SQL_COMBOBOX  */
    public $sql;
    /** @var bool   priznak, jestli muze byt zadana prazdna hodnota v SQL_COMBOBOX */
    public $enableEmpty = true;

    // parametry pole v HTML
    public $id;
    public $htmlParams = '';

    public $defaultValue;
    public $storeDefaultValue = false;

    /** @var string  Slouzi pro presun stareho nastaveni - resi se v aplikaci */
    public $oldPrefName;
    /** @var string  Muze byt v jinem umisteni nez zbytek preferenci */
    public $storage = null;

    /**
     * @var mixed $value   Skutecna hodnota
     */
    public $value;


    public function init($initArray)
    {
        foreach ($initArray as $key => $value) {
            $this ->{$key} = $value;
        }

        $this ->value = $this ->defaultValue;
    }


    /**
     * Vraci html parametry - pokud nejsou explicitne zadany, vraci defaultni podle typu
     * @return string
     */
    public function getHtmlParams()
    {
        $result = $this ->htmlParams;

        if (!$result  &&  $this ->readOnly) {
            switch ($this ->type) {
                case self::SQL_COMBOBOX :
                case self::COMBOBOX :
                    $result = 'disabled="disabled"';
                    break;

                default : $result = 'readonly="readonly"';
            }

        }

        if ($this ->id) {
            $result .= ' id="' . $this ->id . '"';
        }

        return $result;
    }
}


class Preferences
{
    const FORMAT_INI = 'ini';
    const FORMAT_TEXT = 'text';

    var $caption;
    /** @var string     slouzi pro rozliseni jednotlivych stranek nastaveni */
    var $name;

    /** @var string     kategorie */
    var $category;

    /** @var string     nazev souboru s texty, ktery se ma nacist */
    var $strings;

    var $index;

    /** @var string     kde se maji ukladat - resi si to aplikace */
    var $storage;
    /** @var string     ini (default) / text */
    var $format;

    /**
     * @var Preference[]
     */
    var $items = [];

    public function __construct()
    {
        $this ->format = 'ini';
    }

    /**
     * Nacte polozky ze zadaneho pole.
     * @param $initArray
     */
    public function initItems($initArray)
    {
        foreach ($initArray as $itemArray) {
            if (!$itemArray['disabled']) {
                $item = new Preference();
                $item->init($itemArray);

                $this->items[] = $item;
            }
        }
    }


    /**
     * Nacte nastaveni ze zadaneho pole
     * @param $initArray
     */
    public function init($initArray)
    {
        foreach ($initArray as $key => $value) {
            switch ($key) {
                case 'items' :
                    $this ->initItems($value);
                    break;

                default :
                    $this ->{$key} = $value;
            }
        }
    }


    /**
     * Vraci polozku zadaneho jmena
     * @param string $name
     * @return null|Preference
     */
    public function findItem($name)
    {
        foreach ($this ->items as $item) {
            if ($item ->name == $name) {
                return $item;
            }
        }

        return null;
    }


    public function setItemValue($itemName, $value, $convertMemoLines = true, $storage = null)
    {
        $item = $this ->findItem($itemName);

        if ($item  &&  (!isset($storage)  ||  ($storage == $this ->storage  &&  !$item->storage)  || $storage == $item ->storage) ) {
            if ($convertMemoLines  &&  $item ->type == Preference::MEMO) {
                $item->value = str_replace('\n', "\n", $value);
            } else {
                $item->value = $value;
            }
        }
    }


    /**
     * Nacte hodnoty polozek z retezce s radky ve tvaru item_name=hodnota
     * @param string|string[] $preferencesText
     * @param string $storage
     */
    public function loadItemValues($preferencesText, $storage)
    {
        if (is_array($preferencesText)) {
            foreach ($preferencesText as $key => $value) {
                $this->setItemValue($key, $value, false, $storage);
            }
        } else {
            $str_settings = explode("\n", $preferencesText);

            foreach ($str_settings as $setStr) {
                $pom = sscanf($setStr, '%[^=]=%[^$]'); // , &$key, &$value
                $key = strtolower(trim($pom[0]));
                $value = trim($pom[1]);
                $this->setItemValue($key, $value);
            }
        }
    }


    /**
     * Nacte potrebna data z databaze (volby pro SQL_COMBOBOX)
     * @param Database $database
     */
    public function loadData($database)
    {
        foreach ($this ->items as $item) {
            if ($item ->type == Preference::SQL_COMBOBOX  &&  $item ->sql) {
                $item ->options = [];
                if ($item ->enableEmpty) {
                    $item ->options[''] = ' ';
                }

                $cursor = $database ->getSqlResult($item ->sql);
                while ($row = $cursor ->fetch()) {
                    $item ->options[$row[0]] = $row[1];
                }
                $cursor ->close();
            }
        }
    }


    public function loadPost()
    {
        foreach ($this ->items as $item) {
            if ($item ->name  &&  !$item ->readOnly) {
                switch ($item->type) {
                    case Preference::CHECKBOX :
                        $item->value = isset($_POST[$item->name]) ? 1 : 0;
                        break;

                    default :
                        $item ->value = $_POST[$item ->name];
                        break;
                }
            }
        }
    }


    /**
     * Nacte text nastaveni do pole
     * @param string $preferencesText
     * @return array
     */
    private static function loadItemsToArray($preferencesText)
    {
        $str_settings = explode("\n", $preferencesText);

        $resultArray = [];

        foreach ($str_settings as $setStr) {
            $pom = sscanf($setStr, '%[^=]=%[^$]'); // , &$key, &$value
            $key = strtolower(trim($pom[0]));
            $value = trim($pom[1]);

            if ($key != '') {   // &&  $key[0] != "\\"    - toto slouzi pro opravu
                $resultArray[$key] = $value;
            }
        }

        return $resultArray;
    }


    private static function arrayToText($preferencesArray)
    {
        $result = '';

        foreach($preferencesArray as $key => $value) {
            $result .= $key . '=' . $value . "\n";
        }

        return $result;
    }


    public function storeItemValues($preferencesText, &$logText, $storage = null)
    {
        $storage = $storage ?: $this ->storage;

        $logText = '';

        // nacte puvodni hodnoty
        $resultArray = self::loadItemsToArray($preferencesText);

        // zapise zmenene hodnoty
        foreach ($this ->items as $item) {
            if ($item  &&  $item ->name  &&  !$item ->readOnly  &&  ($item ->storage == $storage  ||  (!$item ->storage  &&  $storage == $this ->storage)) ) {
                if ($item ->type == Preference::DELETE) {
                    unset($resultArray[$item->name]);
                } else {
                    $newValue = ($item->type == Preference::MEMO)
                        ? str_replace("\n", '\n', str_replace("\r\n", '\n', $item->value))
                        : $item->value;

                    $isDefault = isset($item->defaultValue) && $newValue === $item->defaultValue;

                    if ($isDefault  &&  !isset($resultArray[$item->name])) {
                        // defaultni hodnota se nezmenila
                    } else if ( !isset($resultArray[$item->name]) || $item->value != $resultArray[$item->name]) {
                        // doslo ke zmene
                        $logText .=
                            $item->name . ":\n" .
                            '   puvodni: ' . $resultArray[$item->name] . "\n" .
                            '      nova: ' . $newValue . ($isDefault ? ' (default)' : '') . "\n";

                        if ($isDefault) {
                            unset($resultArray[$item->name]);
                        } else {
                            $resultArray[$item->name] = $newValue;
                        }
                    } else if ($isDefault) {
                        // nedoslo ke zmene, ale jedna se o defaultni hodnotu
                        unset($resultArray[$item->name]);
                    }
                }
            }
        }

        return self::arrayToText($resultArray);
//        $result = '';
//
//        foreach($resultArray as $key => $value) {
//            $result .= $key . '=' . $value . "\n";
//        }
//
//        return $result;
    }


    public static function removeItem($preferencesText, $itemName)
    {
        $resultArray = self::loadItemsToArray($preferencesText);
        unset($resultArray[$itemName]);
        return self::arrayToText($resultArray);
    }
}
