<?php
/**
 * UTF8
 * stanislav.elbl, 21.11.2015
 * Formular pro presouvani zaznamu mezi dvema seznamy pomoci ajax
 */

namespace Ron;

use Ron\DB\Cursor;
use Ron\DB\Database;


interface ShiftFormItem
{
    /**
     * @param string $columns
     * @return array
     */
    function getJson($columns);
}

class BaseShiftFormItem implements ShiftFormItem
{
    function getJson($columns)
    {
        $cols = is_array($columns) ? $columns : explode(';', $columns);

        $result = [];
        foreach ($cols as $columnName) {
            $result[$columnName] = StringUtils::jsonText($this ->{$columnName});
        }

        return $result;
    }
}



class ShiftForm
{
    /** @var  Database */
    var $database;

    /** @var ShiftFormItem[]   Pole polozek pro levy seznam */
    var $leftItems;
    /** @var ShiftFormItem[]   Pole polozek pro pravy seznam */
    var $rightItems;

    /** @var string     Seznam sloupcu pro levy seznam */
    var $leftColumns;
    /** @var string     Seznam sloupcu pro pravy seznam */
    var $rightColumns;

    /** @var string  Url pro javascripty  */
    var $url;

    /**
     * ShiftForm constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * Nacte pole polozek ze zadaneho kurzoru
     * @param Cursor $cursor
     * @param string $columns
     * @return array
     */
    public static function loadItems($cursor, $columns)
    {
        $cols = is_array($columns) ? $columns : explode(';', $columns);

        $result = [];

        while ($row = $cursor ->fetch()) {
            $item = new BaseShiftFormItem();

            foreach ($cols as $i => $columnName) {
                if ($columnName) {
                    $item->{$columnName} = $row[$i];
                }
            }

            $result[] = $item;
        }

        return $result;
    }


    /**
     * Prevede pole polozek na json pole
     * @param ShiftFormItem[] $items
     * @param string|string[] $cols
     * @return array|null
     */
    public static function getJsonItems($items, $cols)
    {
        if (!$items) return null;

        $cols = is_array($cols) ? $cols : explode(';', $cols);
        $result = [];
        foreach ($items as $item) $result[] = $item ->getJson($cols);

        return $result;
    }


    public function getJsonData()
    {
        $cols = is_array($this ->leftColumns) ? $this ->leftColumns : explode(';', $this ->leftColumns);
        $leftItems = [];
        foreach ($this ->leftItems as $item) $leftItems[] = $item ->getJson($cols);

        $cols = is_array($this ->rightColumns) ? $this ->rightColumns : explode(';', $this ->rightColumns);
        $rightItems = [];
        foreach ($this ->rightItems as $item) $rightItems[] = $item ->getJson($cols);

        $result = [
            'leftColumns' => $this ->leftColumns,
            'leftItems' => $leftItems,

            'rightColumns' => $this ->rightColumns,
            'rightItems' => $rightItems,

            'url' => $this ->url
        ];

        return $result;
    }


    function loadData()
    {
        return $this ->getJsonData();
    }


    function moveLeft($item)
    {
    }


    function moveRight($item)
    {
        return null;
    }


    function handleMoveLeft()
    {
        // vratit: leftaction=add  rightaction=remove
        // mozne hodnoty: add, remove, reload (cely seznam), none
        return [];
    }

    function handleMoveRight()
    {
        return [];
    }


    function handleAction()
    {
        switch ($_GET['shiftformaction']) {
            case 'loaddata' :
                return $this ->loadData();

            case 'moveleft' :
                return $this ->handleMoveLeft();
                break;

            case 'moveright' :
                return $this ->handleMoveRight();
                break;
        }

        return null;
    }
}

