<?php
/** pavel.rudol 9.4.2020 */

namespace Ron\RonPortal\Sklad;

use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\RonPortal\FormReportsListView;
use Ron\RonPortal\RonPortalBrowser;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalReport;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\TypRozdeleni;
use Ron\StringUtils;
use function Ron\ronText;

require_once('adsapp/ronportalbrowser.php');
require_once('ron/guiutils.php');

class FirmaFilterForm extends RonPortalFilterForm
{
    private $indexUzivatelskehoFiltru;
    private $typRozdeleniKategorie;
    private $typRozdeleniKategoriePopis;

    public $stav;

    /**
     * FirmaFilterForm constructor.
     * @param $database
     * @param $action
     * @param $useGet
     * @param $indexUzivatelskehoFiltru
     * @param TypRozdeleni $typRozdeleni
     */
    public function __construct($database, $action, $useGet, $indexUzivatelskehoFiltru, $typRozdeleni)
    {
        parent::__construct($database, $action, $useGet);

        $this->typRozdeleniKategorie = $typRozdeleni->kod;
        $this->typRozdeleniKategoriePopis = $typRozdeleni->popis;
        $this->indexUzivatelskehoFiltru = $indexUzivatelskehoFiltru;
    }

    private function loadMesto()
    {
        if (isset($_GET['mesto']))
            $_SESSION['firma_mesto'] = $_GET['mesto'];
    }

    private function loadUzivatelskyFiltr()
    {
        if (isset($_GET['uzivatelskyfiltr']))
            $_SESSION['firma_filtr'] = $_GET['uzivatelskyfiltr'];
    }

    private function loadStavFirmy()
    {
        if (isset($_GET['stav'])) {
            $_SESSION['firma_stav'] = $_GET['stav'];
        }
        if (isset($_GET['resetfilter'])) {
            $_SESSION['firma_stav'] = -1;
        }
        $this->stav = $_SESSION['firma_stav'];
    }

    private function loadTextovyFiltr()
    {
        if (isset($_GET['klicova_slova']))
            $_SESSION['firma_klicova_slova'] = $_GET['klicova_slova'];
        if (isset($_GET['nazev']))
            $_SESSION['firma_nazev'] = $_GET['nazev'];
    }

    private function loadFiltrRozdeleni()
    {
        if (isset($_GET['rozdeleni_filtr']))
            $_SESSION['rozdeleni_filtr'] = $_GET['rozdeleni_filtr'];
    }

    private function resetFilter()
    {
        if (isset($_GET['resetfilter'])) {
            $_SESSION['firma_mesto'] = "";
            $_SESSION['firma_filtr'] = "";
            $_SESSION['firma_klicova_slova'] = "";
            $_SESSION['firma_nazev'] = "";
            $_SESSION['rozdeleni_filtr'] = "";
        }
    }

    private function renderMesto()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo ronText('Mesto');
        echo '</SPAN>';

        // combobox pro vyber
        $sql =
            "SELECT DISTINCT Mesto FROM Firma ";

        $sql .=
            "ORDER BY Mesto";

        require_once('ron/controls.php');
        $combo = new ComboBox('mesto');
        $combo->emptyRow = ronText('_vse_');
        $combo->keyIndex = 0;
        $combo->captionIndex = 0;
        $combo->className = 'app_combo adsfilterforminput_full ';
        $combo->htmlParams['data-ron-autosubmit'] =  '1';
        $combo->loadData($this->database, $sql, $_SESSION['firma_mesto']);
        $combo->render();

        echo "</DIV>";
    }

    private function renderUzivatelskyFiltr($index)
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo RonPortalSettings::getValue('firma_uzivatelska'. $index);
        echo '</SPAN>';

        // combobox pro vyber
        $sql =
            "SELECT DISTINCT userData{$index} FROM Firma WHERE userData{$index} IS NOT NULL ";

        $sql .=
            "ORDER BY 1";

        require_once('ron/controls.php');
        $combo = new ComboBox('uzivatelskyfiltr');
        $combo->emptyRow = ronText('_vse_');
        $combo->keyIndex = 0;
        $combo->captionIndex = 0;
        $combo->className = 'app_combo adsfilterforminput_full ';
        $combo->htmlParams['data-ron-autosubmit'] =  '1';
        $combo->loadData($this->database, $sql, $_SESSION['firma_filtr']);
        $combo->render();

        echo "</DIV>";
    }

    private function renderStavFirmy()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo '</SPAN>';

        if (!isset($this->stav)) {
            $this->stav = 1;
        }

        $data = [
            -1 => ronText('Vse'),
            1 => ronText('Firma_aktivni'),
            0 => ronText('Firma_neaktivni'),
        ];

        $url = '?menuid=firma&stav=';

        LinksList::renderLinks($data, $this->stav, $url);

        echo "</DIV>\n";
    }

    private function renderTextovyFiltr($varName, $header_text)
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo ronText($header_text);
        echo '</SPAN>';

        $sessionVarName = 'firma_' . $varName;

        echo "<input type='text' class='form-input' data-ron-autosubmit='1' name='{$varName}' value='{$_SESSION[$sessionVarName]}' />";

        echo "</DIV>";
    }

    private function renderFiltrRozdeleni()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';

        echo StringUtils::htmlTextConvert($this->typRozdeleniKategoriePopis);

        echo '</SPAN>';

        $params = $this->database->createParameters();
        // combobox pro vyber
        $sql =
            'SELECT Kod, Popis, cssClass FROM Rozdeleni ' .
            ' WHERE KodTypuRozdeleni = ' . $params->addParameter($this->typRozdeleniKategorie) .
            ' AND (DatumZruseni >= ' . $params->addParameter($this->database->dateToParam(DateUtils::today())) . ' OR DatumZruseni IS NULL) ';

        $sql .=
            "ORDER BY Popis";

        require_once('ron/controls.php');
        $combo = new ComboBox('rozdeleni_filtr');
        $combo->emptyRow = ronText('_vse_');
        $combo->keyIndex = 0;
        $combo->captionIndex = 1;
        $combo->cssClassIndex = 2;
        $combo->className = 'app_combo adsfilterforminput_full ronselect';
        $combo->htmlParams = ['data-ron-autosubmit' =>  '1',
                            'data-ronselect-class' => 'adsfilterforminput_full ronselect'];
        $combo->loadData($this->database, $sql, $_SESSION['rozdeleni_filtr'], $params);
        $combo->render();

        echo "</DIV>";
    }

    private function renderReports()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';

        echo '</SPAN>';

        require_once('adsapp/reportutils.php');
        $reportList = RonPortalReport::loadFormReports($this->database, 'Firma');

        $params =
            ['rozdeleni' => $_SESSION['rozdeleni_filtr'],
                'mesto' => $_SESSION['firma_mesto'],
                'klicova-slova' => $_SESSION['firma_klicova_slova'],
                'nazev' => $_SESSION['firma_nazev'],
                'aktivni' => $this->stav];

        FormReportsListView::renderReportList($reportList, '?menuid=firma&action=report&' . http_build_query($params));

        echo "</DIV>";
    }

    private function renderXML()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';

        echo '</SPAN>';

        $params = $this->database->createParameters();
        $sql =
            "SELECT ID, Popis FROM SablonaXMLFormulare WHERE Popis LIKE " . $params->addParameter('NOVA_FIRMA%');

        $cursor = $this->database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()) {
            $popis = substr($row[1], strpos($row[1], '_', 5)+1);
            echo "<a href=?menuid=xml&record={$row[0]}>{$popis}</a><br />";
        }

        echo "</DIV>";
    }

    function renderFoto()
    {
        echo "<a class='submitbutton margin-left-2' href='?menuid=firma&resetfilter=1'>" . ronText('Objednavka_resetovat_filtr') . "</a>";
    }


    function loadData()
    {
        $this->loadFiltrRozdeleni();
        $this->loadMesto();
        if ($this->indexUzivatelskehoFiltru)
            $this->loadUzivatelskyFiltr();
        $this->loadTextovyFiltr();
        $this->loadStavFirmy();
        $this->resetFilter();
    }


    function renderFilters()
    {
        $this->renderFiltrRozdeleni();
        if ($this->indexUzivatelskehoFiltru)
            $this->renderUzivatelskyFiltr($this->indexUzivatelskehoFiltru);
        $this->renderMesto();
        $this->renderTextovyFiltr('klicova_slova', 'Klicova_slova');
        $this->renderTextovyFiltr('nazev', 'Nazev1');
        $this->renderStavFirmy();
        $this->renderReports();
        $this->renderXML();
    }


}

class RozdeleniFirmyBrowserField extends BrowserField
{
    private $typRozdeleni;
    private $indexKodFirmy;

    /**
     * RozdeleniFirmyBrowserField constructor.
     * @param Database $database
     * @param TypRozdeleni $typRozdeleni
     * @param $indexKodFirmy
     */
    public function __construct($database, $typRozdeleni, $indexKodFirmy)
    {
        parent::__construct($database);

        $this->typRozdeleni = $typRozdeleni->kod;
        $this->indexKodFirmy = $indexKodFirmy;
        $this->header = ronText($typRozdeleni->popis);
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $params = $this->database->createParameters();

        $sql =
            "SELECT Kod, Popis " .
            "FROM Rozdeleni JOIN RozdeleniFirmy RF ON (Rozdeleni.KodTypuRozdeleni = RF.KodTypuRozdeleni AND Rozdeleni.Kod = RF.Rozdeleni) " .
            "WHERE kodFirmy = " . $params->addParameter($rowData[$this->indexKodFirmy]) . " AND RF.KodTypuRozdeleni = " . $params->addParameter($this->typRozdeleni) .
            "  AND RF.PlatnostOd <= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) .
            "  AND (RF.PlatnostDo >= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " OR RF.PlatnostDo IS NULL)";

        $cursor = $this->database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()) {
            if ($result != '')
                $result .= ', ';

            $result .= "<span title='{$row[1]}'>{$row[0]}</span>";
        }

        return $result;
    }

}

class FirmaBrowser extends RonPortalBrowser
{
    private $indexUzivatelskehoFiltru;
    /** @var TypRozdeleni  */
    private $typRozdeleniKategorie;

    /** @var FirmaFilterForm */
    public $filterForm;

    /**
     * FirmaBrowser constructor.
     * @param $database
     * @param $action
     * @param $indexFiltr
     * @param TypRozdeleni $typRozdeleniKategorie
     */
    public function __construct($database, $action, $indexFiltr, $typRozdeleniKategorie)
    {
        parent::__construct($database, $action);

        $this->indexUzivatelskehoFiltru = $indexFiltr;
        $this->typRozdeleniKategorie = $typRozdeleniKategorie;

        $this->title = ronText('Seznam_firem');
        $this->code = "browserseznamfirem";

        $this->sqlSelectPart =
            'kodFirmy, nazev1, nazev2, mesto, ulice, Stat.Popis , {fn_CASTWIDEMEMO(KlicovaSlova)}, ' . // 0 - 6
            'userData1, userData2, userData3, userData4, userData5, userData6, userData7, userData8, userData9, ' . // 7 - 15
            '{fn_CASTWIDEMEMO(poznamka)}'; //16

        $this->sqlFromPart = 'Firma LEFT OUTER JOIN Stat ON (Stat.kod = Firma.stat)';

        $this->primaryField = 0;

        $this->addFields([
            [BROWSER_FIELD_HEADER_TEXT => 'Kod_firmy', BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_CODE => 'col-kod',
                BROWSER_FIELD_LINK => '?menuid=firma&action=detailfirmy', BROWSER_FIELD_SQL_NAME => 'kodFirmy', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_HEADER_TEXT => 'Nazev1', BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_CODE => 'col-nazev1',
                BROWSER_FIELD_LINK => '?menuid=firma&action=detailfirmy', BROWSER_FIELD_SQL_NAME => 'Nazev1', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_HEADER_TEXT => 'Nazev2', BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_CODE => 'col-nazev2',
                BROWSER_FIELD_SQL_NAME => 'Nazev2', BROWSER_FIELD_ORDERING => 1],
        ]);

        if ($this->indexUzivatelskehoFiltru) {
            $this->addField([BROWSER_FIELD_HEADER => RonPortalSettings::getValue('firma_uzivatelska' . $this->indexUzivatelskehoFiltru), BROWSER_FIELD_CODE => 'col-uzivatelsky-filtr',
                BROWSER_FIELD_SQL_INDEX => (6 + $this->indexUzivatelskehoFiltru), BROWSER_FIELD_SQL_NAME => 'userData' . $this->indexUzivatelskehoFiltru, BROWSER_FIELD_ORDERING => 1]);
        }

        $this->addFields([
            [BROWSER_FIELD_HEADER_TEXT => 'Mesto', BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_CODE => 'col-mesto',
                BROWSER_FIELD_SQL_NAME => 'mesto', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_HEADER_TEXT => 'Ulice', BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_CODE => 'col-ulice',
                BROWSER_FIELD_SQL_NAME => 'ulice', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_HEADER_TEXT => 'Stat', BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_CODE => 'col-stat',
                BROWSER_FIELD_SQL_NAME => 'Stat.Popis', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_HEADER_TEXT => 'Klicova_slova', BROWSER_FIELD_SQL_INDEX => 6, BROWSER_FIELD_CODE => 'col-klicova-slova',
                BROWSER_FIELD_SQL_NAME => 'KlicovaSlova'],
            [BROWSER_FIELD_HEADER_TEXT => 'Poznamka', BROWSER_FIELD_SQL_INDEX => 16, BROWSER_FIELD_CODE => 'col-poznamka',
                BROWSER_FIELD_SQL_NAME => 'poznamka'],
        ]);

        $this->addField(new RozdeleniFirmyBrowserField($database, $typRozdeleniKategorie, 0));

        $this->defaultSort = 2;
    }

    function setFilterForm()
    {
        $this->filterForm = new FirmaFilterForm($this->database, $this->action, true, $this->indexUzivatelskehoFiltru, $this->typRozdeleniKategorie);
    }

    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $sql = '';

        if ($_SESSION['firma_mesto']) {
            $sql = 'Firma.Mesto = ' . $params->addParameter($_SESSION['firma_mesto']);
        }

        if (isset($this->filterForm->stav)) {
            if ($this->filterForm->stav >= 0) {
                if ($sql != '')
                    $sql .= ' AND ';
                $sql .= ' Firma.Aktivni = ' . $params->addParameter($this->filterForm->stav);
            }
        }

        if ($_SESSION['firma_klicova_slova']) {
            $sqlKlicovaSlova = '';

            $klicovaSlovaArr = explode(' ', $_SESSION['firma_klicova_slova']);

            foreach ($klicovaSlovaArr as $klicoveSlovo) {
                if (!empty($klicoveSlovo)) {
                    if ($sqlKlicovaSlova != '') {
                        $sqlKlicovaSlova .= ' OR ';
                    }

                    $sqlKlicovaSlova .= " Firma.KlicovaSlova LIKE " . $params->addParameter('%' . $klicoveSlovo . '%');
                }
            }

            if ($sql != '')
                $sql .= ' AND ';
            $sql .= "(" . $sqlKlicovaSlova . ")";
        }

        if ($_SESSION['firma_nazev']) {
            if ($sql != '')
                $sql .= ' AND ';

            $sql .= "(Firma.nazev1 LIKE " . $params->addParameter('%' . $_SESSION['firma_nazev'] . '%') . ")";
        }

        if ($this->indexUzivatelskehoFiltru && ($_SESSION['firma_filtr'])) {
            if ($sql != '')
                $sql .= ' AND ';

            $sql .= "Firma.UserData{$this->indexUzivatelskehoFiltru} = " . $params->addParameter($_SESSION['firma_filtr']);
        }

        if ($_SESSION['rozdeleni_filtr']) {
            if ($sql != '')
                $sql .= ' AND ';

            $sql .=
                " EXISTS(SELECT Rozdeleni FROM RozdeleniFirmy WHERE RozdeleniFirmy.kodFirmy = Firma.kodFirmy AND RozdeleniFirmy.KodTypuRozdeleni = " . $params->addParameter($this->typRozdeleniKategorie->kod) .
                "       AND RozdeleniFirmy.Rozdeleni = " . $params->addParameter($_SESSION['rozdeleni_filtr']) . " AND RozdeleniFirmy.PlatnostOd <= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) .
                "       AND (RozdeleniFirmy.PlatnostDo >= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " OR RozdeleniFirmy.PlatnostDo IS NULL)" .
                ")";
        }

        return new SqlStatement($sql, $params);
    }


}