<?php
/**
 * UTF8
 * stanislav.elbl, 29.07.2015
 */

namespace Ron\RonPortal;

require_once('ron/controls.php');
require_once('ron/guiutils.php');
require_once('adsapp/ronportalbrowser.php');
require_once('adsapp/ronportalfilter.php');

use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;


class SluzebniCestyFilterForm extends RonPortalFilterForm
{
    /**
     * @param Database $database
     * @param bool $schvalovani
     */
    function __construct($database, $schvalovani)
    {
        parent::__construct($database, 'sluzebnicesty');

        $this->nastavObdobi(true, 'ADS', false, false);
        $this->nastavDatumOdDo(true, false, \Ron\ronText('Datum_od'));
        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) $this->nastavRozdeleni(true);

        $this->nastavAkci( '?menuid=sluzebnicesty' . ($schvalovani ? '&schvalovani=1' : '') );

        if (isset($_GET['urovenschvaleni'])) self::setUroveniSchvaleni($_GET['urovenschvaleni']);
        if (isset($_GET['jenzahranicnisc'])) $_SESSION['jenzahranicnisc'] = ($_GET['jenzahranicnisc']);

        $this ->nastavOsoby(true, [Prava::vratPrava('sluzebnicesty'), Prava::vratPrava('slcesty_prohl')]); // time(), time()
        $this->povolitVsechyOsoby = true; // zobrazi volbu "vsichni"

        $prava = Prava::vratPrava('slcesty_prohl');
        if ($prava ->jsouNejakePrava()) {
            // uzivatel ma nastaveno pravo pro prohlizeni => rozdeleni se bude filtrovat podle tohoto prava
            // (predpoklad je, ze ma uzivatel nastaveno pravo pro prohlizeni vsech SC, ktere muze editovat  nebo schvalovat)
            $this ->f_pravarozdeleni = $prava;
        }
    }


    public static function setUroveniSchvaleni($value)
    {
        $_SESSION['urovenschvaleni_sc'] = $value;
    }

    public static function getUrovenSchvaleni()
    {
        return $_SESSION['urovenschvaleni_sc'];
    }


    function renderFilterStav()
    {
        echo '<div class="filterrow" id="filter_urovenschvaleni">';
        echo '<span class="filtercaption">' . \Ron\ronText('Uroven_schvaleni') . '</span>';

        global $STR_SchvaleniSC1; // jen pro kompatibilitu
        global $STR_SchvaleniSC2;
        global $STR_SchvaleniSC3;

        $data = array(
            0 => \Ron\ronText('Vse'),
            -1 => \Ron\ronText('SC_ke_schvaleni'),
            -2 => \Ron\ronText('SC_bez_schvaleni'),
        );

        //filtr na schvaleni nadrizenym 2 musi byt povolen v nastaveni
        $pocetUrovni = RonPortalSettings::getValue('slcesty_filtry_nadrizeny');
        if (!isset($pocetUrovni)) $pocetUrovni = RonPortalSettings::getIntValue('slcesty_filtr_nadrizeny2') ? 2 : 1; // pro kompatibilitu se starym nastavenim

        if ($pocetUrovni) {
            $data[99] = \Ron\ronText('Povoleno_nadrizenym');
        }

        if ($pocetUrovni >= 2) {
            $data[100] = \Ron\ronText('Povoleno_nadrizenym_2');
        }

        if ($pocetUrovni >= 3) {
            $data[101] = \Ron\ronText('Povoleno_nadrizenym_3');
        }

        if ($pocetUrovni >= 4) {
            $data[102] = \Ron\ronText('Povoleno_nadrizenym_4');
        }

        $data[1] = $STR_SchvaleniSC1 ?: \Ron\ronText('Schvaleni_sc_plne_1');
        $data[2] = $STR_SchvaleniSC2 ?: \Ron\ronText('Schvaleni_sc_plne_2');
        $data[3] = $STR_SchvaleniSC3 ?: \Ron\ronText('Schvaleni_sc_plne_3');

        //filtr na schvaleni urovni 4 musi byt povolen v nastaveni
        if (RonPortalSettings::getIntValue('slcesty_filtr_uroven4')) {
            $data[4] = \Ron\ronText('Schvaleni_sc_plne_4');
        }

        //filtr na zauctovano se musi povolit v nastaveni
        if (RonPortalSettings::getIntValue('slcesty_filtr_zauctovano')) {
            $data[80] = \Ron\ronText('SC_zauctovana');
        }

        LinksList::renderLinks($data, self::getUrovenSchvaleni(), $this ->f_formaction .  '&urovenschvaleni=', '', false);

        echo "</div>\n";
    }


    function renderFilterZahranicni()
    {
        echo '<div class="filterrow" id="filter_zahranicni">';
        echo '<span class="filtercaption"></span>';

        $data = array(
            0 => \Ron\ronText('Vse'),
            1 => \Ron\ronText('Jen_zahranicni'),
            2 => \Ron\ronText('Jen_tuzemske'),
        );

        LinksList::renderLinks($data, $_SESSION['jenzahranicnisc'], $this ->f_formaction .  '&jenzahranicnisc=', '', false);

        echo "</div>\n";
    }


    function renderFilters()
    {
        parent::renderFilters();

        $this ->renderFilterZahranicni();   // vse / zahranicni / tuzemske
        $this ->renderFilterStav();         // vsechno / ke schvaleni / ...
    }
}



class DokumentySCField extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this->align = 'center';
        $this->code = 'col-dokumenty';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[SluzebniCestaBrowser::INDEX_POCET_DOKUMENTU]) {
            $text = '<i class="inline-glyph glyph-button fa glyph-attachment" title="' . \Ron\ronText('Pocet_priloh') . ': ' . $rowData[SluzebniCestaBrowser::INDEX_POCET_DOKUMENTU] . '"></i>';

            // pokud je jen jeden dokument, tak rovnou zobrazi proklik na stahnuti souboru
            if ($rowData[SluzebniCestaBrowser::INDEX_POCET_DOKUMENTU] == 1) {
                return "<a target='_blank' href='?menuid=sluzebnicesty&action=download&idslcesty={$rowData[0]}&record={$rowData[SluzebniCestaBrowser::INDEX_ID_DOKUMENTU]}'>$text</a>";
            }

            return $text;
        }

        return '';
    }
}




// Odkaz na schvaleni - podle prav
class SchvaleniSlCestyField extends BrowserField
{
    /**
     * @param Database $database
     * @param array $options
     */
    function __construct($database, $options = null)
    {
        parent::__construct($database, $options);

        $this->code = 'col-schvaleni';
        $this->align = 'left';

        $this->header = \Ron\ronText('Schvaleni_sc');
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $hint = '';

        if ($rowData[30]) {
            $date = $rowData[30];
            $name = $rowData[31];
            $class = 'textschvaleni4';
        } else if ($rowData[6]) {
            $date = $rowData[6];
            $name = $rowData[7];
            $class = 'textschvaleni3';
        } else if ($rowData[4]) {
            $date = $rowData[4];
            $name = $rowData[5];
            $class = 'textschvaleni2';
        } else if ($rowData[2]) {
            $date = $rowData[2];
            $name = $rowData[3];
            $class = 'textschvaleni1';
        } else if ($rowData[43]) {
            // povoleno/zamitnuto nadrizenym 3
            $date = $rowData[45];
            $name = $rowData[44]; // SN3_Uzivatel

            if ($rowData[43] == 1) {
                $class = 'textschvaleninadrizenym4';
                $hint = \Ron\ronText('Povoleno_nadrizenym_4');
            } else {
                $class = 'textzamitnutinadrizenym4';
                $hint = \Ron\ronText('Zamitnuto_nadrizenym_4');
            }
        } else if ($rowData[40]) {
            // povoleno/zamitnuto nadrizenym 3
            $date = $rowData[42];
            $name = $rowData[41]; // SN3_Uzivatel

            if ($rowData[40] == 1) {
                $class = 'textschvaleninadrizenym3';
                $hint = \Ron\ronText('Povoleno_nadrizenym_3');
            } else {
                $class = 'textzamitnutinadrizenym3';
                $hint = \Ron\ronText('Zamitnuto_nadrizenym_3');
            }
        } else if ($rowData[26]) {
            // povoleno/zamitnuto nadrizenym 2
            $date = $rowData[28];
            $name = $rowData[27]; // SN2_Uzivatel

            if ($rowData[26] == 1) {
                $class = 'textschvaleninadrizenym2';
                $hint = \Ron\ronText('Povoleno_nadrizenym_2');
            } else {
                $class = 'textzamitnutinadrizenym2';
                $hint = \Ron\ronText('Zamitnuto_nadrizenym_2');
            }
        } else if ($rowData[23] == 1) {
            // schvaleno nadrizenym

            $date = $rowData[25];
            $name = $rowData[24]; // SN_Uzivatel

            if ($rowData[23] == 1) {
                $class = 'textschvaleninadrizenym';
                $hint = \Ron\ronText('Povoleno_nadrizenym');
            } else {
                $class = 'textzamitnutinadrizenym';
                $hint = \Ron\ronText('Zamitnuto_nadrizenym');
            }
        } else if ($rowData[46]) {
            return \Ron\ronText('Schvaleno_zamestnancem');
        } else {
            return \Ron\ronText('Schvaleni_sc_neschvaleno'); // implicitne prazdny text, lze nastavit v prekladech
        }

        $result = "<SPAN class='$class' title='$hint'>";
        if ($date) $result .= DBUtils::getFormatedValue($this ->database, 'date', $date);
        if ($name) $result .= " ($name)";
        $result .= "</SPAN>\n";

        return $result;
    }
}


class SchvaleniSlCestyEditField extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $prava;
    var $indexSchvaleno;
    var $indexDatum;
    var $indexUzivatel;
    var $htmlIndex;

    /** @var  SluzebniCestaBrowser */
    var $browser;

    var $existsEdit;


    /**
     * @param Browser $browser
     * @param PravaNaOsoby $prava
     * @param int $indexSchvaleno
     * @param int $indexDatum
     * @param int $indexUzivatel
     * @param int $htmlIndex
     * @param string $caption
     */
    function __construct($browser, $prava, $indexSchvaleno, $indexDatum, $indexUzivatel, $htmlIndex, $caption)
    {
        parent::__construct($browser ->database);

        $this ->code = 'col-schvaleni' . $htmlIndex;

        $this ->align = "center";
        $this ->header =
            "<span id='schvaleniSCMenu_{$htmlIndex}''>$caption</span>";

        $this ->existsEdit = false;

        $this ->browser = $browser;

        $this ->htmlIndex = $htmlIndex;
        $this ->prava = $prava;
        $this ->indexSchvaleno = $indexSchvaleno;
        $this ->indexDatum = $indexDatum;
        $this ->indexUzivatel = $indexUzivatel;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $schvaleno = isset($this ->indexSchvaleno) ? $rowData[$this ->indexSchvaleno] : $rowData[$this ->indexDatum];

        $hint = '';
        if ($schvaleno) {
            $hint = $rowData[$this ->indexUzivatel];
            if ($this ->indexDatum  &&  $rowData[$this ->indexDatum]) $hint .= ' (' . DBUtils::getFormatedValue($this ->database, 'finedatetime', $rowData[$this ->indexDatum]) . ')';
        }

        if ($this ->prava ->lzeZobrazitPP($rowData[22])) {
            // muze schvalovat
            $this ->existsEdit = true;

            return
                "<input type='hidden' name='NastavitSchvaleni_{$rowData[0]}_{$this ->htmlIndex}' value='1' />" .
                "<input type='checkbox' data-ronschvalenisc_index='{$this ->browser ->f_jsonRowIndex}' name='Schvaleni_{$rowData[0]}_{$this ->htmlIndex}' id='Schvaleni_{$rowData[0]}_{$this ->htmlIndex}' " .
                ($schvaleno ? " checked='checked'" : '') . ( $hint ? " title='$hint'" : '' ) . " />\n";
        } else {
            return '<span' . ( $hint ? " title='$hint'" : '' ) . '>' . \Ron\ronText($schvaleno ? 'Str_Yes' : 'Str_No') . '</span>';
        }
    }


    /**
     * @param int $fieldpos
     * @param string $url
     * @return string
     */
    function getFooterText($fieldpos, $url = null)
    {
        if ($this ->existsEdit) {
            return "<input type='checkbox' name='schvaleni_ALL_{$this ->htmlIndex}' id='schvaleni_ALL_{$this ->htmlIndex}' title='' />\n";
        }
        else return '';
    }

}



class KopieSCField extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this->align = 'left';
        $this->code = 'col-kopie';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        /** @noinspection HtmlUnknownTarget */
        return
            '<a href="?menuid=sluzebnicesty&formaction=newfromcopy&record=' . $rowData[0] .
            '" onclick="StopEventPropagation(event)"><i class="inline-glyph glyph-button fa glyph-copy" title="' . \Ron\ronText('Kopirovat_zaznam') . '"></i></a>';
    }
}




class ZruseniSCField extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $f_prava;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->f_prava = Prava::vratPrava('sluzebnicesty');
        $this->code = 'col-zrusit';

        $this->ordering = true;
        $this->sqlFieldName = 'SluzebniCesta.Zauctovano';
        $this->header = '<span title="' . \Ron\ronText('SC_zauctovana') . '/' . \Ron\ronText('Zrusit') . '">' . \Ron\ronText('Sloupec_zrusit_sc') . '</span>';
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[14]) {
            // zauctovano - nejde smazat, zobrazuje fajfku
            /** @noinspection HtmlUnknownTarget */
            return '<i class="inline-glyph fa glyph-check" title="' . \Ron\ronText('SC_zauctovana') . '">';
        }

        if ($rowData[2] || $rowData[4] || $rowData[6]) return ''; // jiz je schvaleno, nejde smazat

        // zkontroluje prava na editaci SC
        if ($this->f_prava->lzeZobrazitPP($rowData[22], $this ->database ->stringToTimestamp($rowData[17]), $this ->database ->stringToTimestamp($rowData[18]))) {
            /** @noinspection HtmlUnknownTarget */
            return GuiUtils::changeLocationQuest('<i class="inline-glyph glyph-button fa glyph-delete" title="' . \Ron\ronText('Zrusit_sluzebku') . '"></i>', \Ron\ronText('Opravdu_zrusit_sluzebku'),
                '?menuid=sluzebnicesty&action=zrusitsluzebku&record=' . $rowData[0]);
        }

        return '';
    }
}




class SluzebniCestaBrowser extends RonPortalBrowser
{
    //nastaveno podle indexu pole v SQL dotazu
    const INDEX_POCET_DOKUMENTU = 50;
    const INDEX_ID_DOKUMENTU = 51;
    const INDEX_ROZDELENI_KOD = 52;
    const INDEX_ROZDELENI_POPIS = 53;

    var $f_pravaeditace;
    //var $f_pravaprohlizeni;
    var $f_pravaSC; // prohlizeni SC

    /** @var  bool */
    var $schvalovani; // rezim schvalovani

    var $f_pravaNadrizeny;
    var $f_pravaNadrizeny2;
    var $f_pravaschvaleni1;
    var $f_pravaschvaleni2;
    var $f_pravaschvaleni3;
    var $f_pravaschvaleni4;
    var $f_pravaZauctovano;

    var $f_jsonObject;
    var $f_jsonRowIndex;
    var $f_hiddenFields;

    /** @var SluzebniCestyFilterForm */
    var $filterForm;

    /**
     * @param Database $database
     * @param bool $schvalovani - zapina rezim hromadneho schvalovani
     */
    function __construct($database, $schvalovani)
    {
        parent::__construct($database, 'sluzebnicesty');

        $this->title = \Ron\ronText('Prehled_sluzebnich_cest');
        $this->tableId = 'seznamsluzebnichcest';
        $this->code = 'browsersluzebnicesta';

        $this->schvalovani = $schvalovani;
        $this->showFooter = $schvalovani;

        $this->sqlSelectPart =
            'SluzebniCesta.Id, Osoba.RC, ' .
            'SluzebniCesta.Schvaleni1, SluzebniCesta.Schvalil1, ' . // 2,3
            'SluzebniCesta.Schvaleni2, SluzebniCesta.Schvalil2, ' . // 4,5
            'SluzebniCesta.Schvaleni3, SluzebniCesta.Schvalil3, ' . // 6,7
            'SluzebniCesta.PoznamkaKVraceni, ' . // 8
            'SluzebniCesta.MistoZacatku, SluzebniCesta.PopisMistaZacatku, SluzebniCesta.Cil, SluzebniCesta.PopisCile, ' . // 9..12
            'SluzebniCesta.Pres, SluzebniCesta.Zauctovano, ' . // 13,14
            'Osoba.Jmeno, Osoba.Prijmeni, ' . // 15, 16
            'SluzebniCesta.DatumCasOd, SluzebniCesta.DatumCasDo, ' . // 17,18
            'UcelSluzebniCesty.Popis, SluzebniCesta.Popis, RadekUctovaniSCesty.Castka, ' . // 19..21
            'PracovniPomer.Id, SluzebniCesta.SchvalenoNadrizenym, SluzebniCesta.SN_Uzivatel, SluzebniCesta.SN_DatumCas, ' . // 22..25
            'SluzebniCesta.SchvalenoNadrizenym2, SluzebniCesta.SN2_Uzivatel, SluzebniCesta.SN2_DatumCas, ' . // 26..28
            'SluzebniCesta.Zahranicni, ' . //29
            'SluzebniCesta.Schvaleni4, SluzebniCesta.Schvalil4, PracovniPomer.DatumUkonceni, SluzebniCesta.CilovyStat, CilovyStat.Popis, ' . // 30..34
            'SluzebniCesta.Uzivatelska1, SluzebniCesta.Uzivatelska2, SluzebniCesta.Uzivatelska3, SluzebniCesta.Uzivatelska4, SluzebniCesta.Uzivatelska5, ' . //35..39
            'SluzebniCesta.SchvalenoNadrizenym3, SluzebniCesta.SN3_Uzivatel, SluzebniCesta.SN3_DatumCas, ' . // 40..42
            'SluzebniCesta.SchvalenoNadrizenym4, SluzebniCesta.SN4_Uzivatel, SluzebniCesta.SN4_DatumCas, ' . // 43..45
            'SluzebniCesta.SchvalenoZamestnancem, SluzebniCesta.OsobniCislo, ' . // 46,47
            'SluzebniCesta.DopravniProstredek, SluzebniCesta.RegistracniZnacka'; // 48,49

        $this->sqlGroupByPart = $this->sqlSelectPart;

        $this->sqlSelectPart .=
            ', COUNT(DokumentSluzebniCesty.Id) AS PocetDokumentu ' . //je potreba nastavit konstanty s indexem pole !
            ', MIN(DokumentSluzebniCesty.Id) AS IdDokumentu ';


        $this->sqlFromPart =
            '( ( ( ( SluzebniCesta INNER JOIN Osoba' .
            '  ON Osoba.RC = SluzebniCesta.RC' .
            ') ' .
            'INNER JOIN PracovniPomer ' .
            ' ON PracovniPomer.RC = SluzebniCesta.RC ' .
            '  AND PracovniPomer.OsobniCislo = SluzebniCesta.OsobniCislo ' .
            '  AND PracovniPomer.DatumNastupu = SluzebniCesta.DatumNastupu ' .
            ') ' .
            'LEFT OUTER JOIN UcelSluzebniCesty ' .
            '  ON UcelSluzebniCesty.Kod = SluzebniCesta.Ucel ' .
            ') ' .
            ' LEFT OUTER JOIN RadekUctovaniSCesty ' .
            ' ON RadekUctovaniSCesty.SluzebniCesta_Id = SluzebniCesta.Id ' .
            '  AND RadekUctovaniSCesty.Typ = 3 ' . // soucet vyuctovani v domaci mene
            ') ' .
            'LEFT OUTER JOIN DokumentSluzebniCesty ' .
            ' ON DokumentSluzebniCesty.SluzebniCesta_ID = SluzebniCesta.Id ' .
            'LEFT OUTER JOIN Stat AS CilovyStat ON (CilovyStat.Kod = SluzebniCesta.CilovyStat) ';

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP)) {
            $this->sqlSelectPart .= ', Rozdeleni.Kod, Rozdeleni.Popis';
            $this->sqlGroupByPart .= ', Rozdeleni.Kod, Rozdeleni.Popis';

            $this->sqlFromPart =
                '( ( ' .
                $this->sqlFromPart .
                ' ) ' .
                'LEFT OUTER JOIN RozdeleniOsob ' .
                ' ON RozdeleniOsob.RC = SluzebniCesta.RC ' .
                ' AND RozdeleniOsob.OsobniCislo = SluzebniCesta.OsobniCislo ' .
                ' AND RozdeleniOsob.DatumNastupu = SluzebniCesta.DatumNastupu ' .
                ' AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) .
                ' AND RozdeleniOsob.PlatnostOd <= SluzebniCesta.DatumCasOd ' .
                ' AND (RozdeleniOsob.PlatnostDo >= SluzebniCesta.DatumCasOd  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ') ' .
                ' LEFT OUTER JOIN Rozdeleni ' .
                ' ON Rozdeleni.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) .
                ' AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni ';
        }

        $this->sqlWherePart = '';

        $this->primaryField = 0;

        if (!$schvalovani) $this->rowClickLink = '?menuid=sluzebnicesty&action=zobrazslcestu';

        $this->browserUrl = '?menuid=sluzebnicesty&' . ($this ->schvalovani ? 'schvalovani=1&' : '');

        $this->code = 'browsersluzebnicesty';

        //$this -> f_pravaprohlizeni = VratPrava('prohlizeni');
        $this->f_pravaeditace = Prava::vratPrava('sluzebnicesty');
        $this->f_pravaSC = Prava::vratPrava('slcesty_prohl');
        $this->f_pravaNadrizeny = Prava::vratPrava('slcesty_nadriz');
        $this->f_pravaNadrizeny2 = Prava::vratPrava('slcesty_nadriz2');
        $this->f_pravaschvaleni1 = Prava::vratPrava('slcesty_schval1');
        $this->f_pravaschvaleni2 = Prava::vratPrava('slcesty_schval2');
        $this->f_pravaschvaleni3 = Prava::vratPrava('slcesty_schval3');
        $this->f_pravaschvaleni4 = Prava::vratPrava('slcesty_schval4');
        $this->f_pravaZauctovano = Prava::vratPrava('slcesty_zauct');

        $this->addField( ['sqlFieldName' =>'SluzebniCesta.Id', 'sqlIndex' => 0,
            BROWSER_FIELD_CODE => 'col-id',
            BROWSER_FIELD_HEADER_TEXT => 'Cislo_sc', 'align' => 'center', 'link' => '?menuid=sluzebnicesty&action=zobrazslcestu', 'ordering' => 1] );

        $f = $this->addField( new UkonceniPracPomeruInfoField($this->database, [
            'sqlFieldName' => 'Osoba.Jmeno', 'sqlIndex' =>15,
            BROWSER_FIELD_CODE => 'col-jmeno',
            'header' => \Ron\ronText('Jmeno'), 'link' => '?menuid=sluzebnicesty&action=zobrazslcestu', 'ordering' => 1], 32) );

        if ($this ->schvalovani) {
            $f ->footer = \Ron\ronText('Schvalit_hromadne');
            $f ->footerColSpan = 4;
        }

        $this->addField( new UkonceniPracPomeruInfoField($this->database, [
            'sqlFieldName' => 'Osoba.Prijmeni', 'sqlIndex' => 16,
            BROWSER_FIELD_CODE => 'col-prijmeni',
            BROWSER_FIELD_HEADER_TEXT => 'Prijmeni', 'link' => '?menuid=sluzebnicesty&action=zobrazslcestu', 'ordering' => 1], 32) );

        $this->addField([BROWSER_FIELD_SQL_NAME => 'SluzebniCesta.OsobniCislo', BROWSER_FIELD_SQL_INDEX => 47,
            BROWSER_FIELD_CODE => 'col-osobnicislo', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_HEADER_TEXT => 'Osobni_cislo', BROWSER_FIELD_ORDERING => 1]);

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP)) {
            // col-rozdeleni
            $this->addField(new RozdeleniField($this ->database, SluzebniCestaBrowser::INDEX_ROZDELENI_KOD, SluzebniCestaBrowser::INDEX_ROZDELENI_POPIS));
        }

        $this->addField( [ 'sqlFieldName' => 'SluzebniCesta.DatumCasOd', 'sqlIndex' => 17,
            BROWSER_FIELD_CODE => 'col-datumcasod',
            BROWSER_FIELD_HEADER_TEXT => 'Zacatek', 'format' => 'date', 'link' => '?menuid=sluzebnicesty&action=zobrazslcestu', 'ordering' => 1] );
        $this->defaultSort = count($this->fields);

        $this->addField( ['sqlFieldName' => 'SluzebniCesta.DatumCasDo', 'sqlIndex' => 18,
            BROWSER_FIELD_CODE => 'col-datumcasdo',
            BROWSER_FIELD_HEADER_TEXT => 'Konec', 'format' => 'date', 'ordering' => 1] );

        $this->addField([BROWSER_FIELD_SQL_NAME => 'SluzebniCesta.CilovyStat', BROWSER_FIELD_SQL_INDEX => 33,
            BROWSER_FIELD_CODE => 'col-kodcilovehostatu', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_HEADER_TEXT => 'Cilovy_stat', BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_NAME => 'CilovyStat.Popis', BROWSER_FIELD_SQL_INDEX => 34,
            BROWSER_FIELD_CODE => 'col-cilovystat', BROWSER_FIELD_VISIBLE => RonPortalSettings::getIntValue('cilovystatcsbrowser'),
            BROWSER_FIELD_HEADER_TEXT => 'Cilovy_stat', BROWSER_FIELD_ORDERING => 1]);

        $this->addField( ['sqlFieldName' => 'UcelSluzebniCesty.Popis', 'sqlIndex' => 19,
            BROWSER_FIELD_CODE => 'col-ucel',
            'header' => \Ron\ronText('Ucel_sc'), 'ordering' => 1,
            BROWSER_FIELD_CLASS_NAME => 'col_ucel'] );

        $this->addField( [
            'sqlFieldName' => 'SluzebniCesta.Popis', 'sqlIndex' => 20,
            BROWSER_FIELD_CODE => 'col-popis', BROWSER_FIELD_VISIBLE => !$this->schvalovani, // pri schvalovani se nezobrazi
            'header' => \Ron\ronText('Popis_sc'), 'ordering' => 1, 'wrap' => true] );

        $this ->addField( ['type' => 'select', 'sqlFieldName' => 'Zahranicni', 'sqlIndex' => 29,
            BROWSER_FIELD_CODE => 'col-zahranicni',
            BROWSER_FIELD_HEADER_TEXT => 'Typ_sc', BROWSER_FIELD_OPTIONS => [\Ron\ronText('Tuzemska'), \Ron\ronText('Zahranicni')],
            BROWSER_FIELD_CLASS_NAME => 'seznamsc_typ' ]);


        $this->addField([BROWSER_FIELD_SQL_NAME => 'SluzebniCesta.MistoZacatku', BROWSER_FIELD_SQL_INDEX => 9,
            BROWSER_FIELD_CODE => 'col-mistozacatku', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_HEADER_TEXT => 'Misto_zacatku', BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_NAME => 'SluzebniCesta.PopisMistaZacatku', BROWSER_FIELD_SQL_INDEX => 10,
            BROWSER_FIELD_CODE => 'col-popismistazacatku', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_HEADER_TEXT => 'Popis_mista_zacatku', BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_NAME => 'SluzebniCesta.Cil', BROWSER_FIELD_SQL_INDEX => 11,
            BROWSER_FIELD_CODE => 'col-cil', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_HEADER_TEXT => 'Cil_sc', BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_NAME => 'SluzebniCesta.PopisCile', BROWSER_FIELD_SQL_INDEX => 12,
            BROWSER_FIELD_CODE => 'col-popiscile', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_HEADER_TEXT => 'Popis_cile', BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_NAME => 'SluzebniCesta.Pres', BROWSER_FIELD_SQL_INDEX => 13,
            BROWSER_FIELD_CODE => 'col-pres', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_HEADER_TEXT => 'Pres', BROWSER_FIELD_ORDERING => 1]);

        $this->addField( ['sqlFieldName' => 'RadekUctovaniSCesty.Castka', 'sqlIndex' => 21,
            BROWSER_FIELD_CODE => 'col-vyuctovani',
            BROWSER_FIELD_HEADER_TEXT => 'Sluzebni_cesta_celkem', 'format' => 'number2',
            'align' => 'right', 'ordering' => 1, BROWSER_FIELD_CLASS_NAME => 'col_vyuctovani'] );

        if (RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA1)) {
            $this->addField([ BROWSER_FIELD_SQL_NAME => 'SluzebniCesta.Uzivatelska1', BROWSER_FIELD_SQL_INDEX => 35,
                BROWSER_FIELD_CODE => 'col-uzivatelska1',
                BROWSER_FIELD_HEADER => RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA1), BROWSER_FIELD_ORDERING => 1,
                'className' => 'uzivatelska1', 'wrap' => true
            ]);
        }
        if (RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA2)) {
            $this->addField([ BROWSER_FIELD_SQL_NAME => 'SluzebniCesta.Uzivatelska2', BROWSER_FIELD_SQL_INDEX => 36,
                BROWSER_FIELD_CODE => 'col-uzivatelska2',
                BROWSER_FIELD_HEADER => RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA2), BROWSER_FIELD_ORDERING => 1,
                'className' => 'uzivatelska2', 'wrap' => true
            ]);
        }
        if (RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA3)) {
            $this->addField([ BROWSER_FIELD_SQL_NAME => 'SluzebniCesta.Uzivatelska3', BROWSER_FIELD_SQL_INDEX => 37,
                BROWSER_FIELD_CODE => 'col-uzivatelska3',
                BROWSER_FIELD_HEADER => RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA3), BROWSER_FIELD_ORDERING => 1,
                'className' => 'uzivatelska3', 'wrap' => true
            ]);
        }
        if (RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA4)) {
            $this->addField([ BROWSER_FIELD_SQL_NAME => 'SluzebniCesta.Uzivatelska4', BROWSER_FIELD_SQL_INDEX => 38,
                BROWSER_FIELD_CODE => 'col-uzivatelska4',
                BROWSER_FIELD_HEADER => RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA4), BROWSER_FIELD_ORDERING => 1,
                'className' => 'uzivatelska4', 'wrap' => true
            ]);
        }
        if (RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA5)) {
            $this->addField([ BROWSER_FIELD_SQL_NAME => 'SluzebniCesta.Uzivatelska5', BROWSER_FIELD_SQL_INDEX => 39,
                BROWSER_FIELD_CODE => 'col-uzivatelska5',
                BROWSER_FIELD_HEADER => RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA5), BROWSER_FIELD_ORDERING => 1,
                'className' => 'uzivatelska5', 'wrap' => true
            ]);
        }

        if ($this->schvalovani) {
            $this->addField(new SchvaleniSlCestyEditField($this, $this->f_pravaNadrizeny, 23, 25, 24, 'N', \Ron\ronText('Povoleno_nadrizenym')));
            if (RonPortalSettings::getIntValue('slcesty_filtry_nadrizeny') > 1) {
                $this->addField(new SchvaleniSlCestyEditField($this, $this->f_pravaNadrizeny2, 26, 28, 27, 'N2', \Ron\ronText('Povoleno_nadrizenym_2')));
            }
            global $STR_SchvaleniSC1; // jen pro kompat.
            $this->addField(new SchvaleniSlCestyEditField($this, $this->f_pravaschvaleni1, null, 2, 3, '1', $STR_SchvaleniSC1 ?: \Ron\ronText('Schvaleni_sc_plne_1')));
            global $STR_SchvaleniSC2; // jen pro kompat.
            $this->addField(new SchvaleniSlCestyEditField($this, $this->f_pravaschvaleni2, null, 4, 5, '2', $STR_SchvaleniSC2 ?: \Ron\ronText('Schvaleni_sc_plne_2')));
            global $STR_SchvaleniSC3; // jen pro kompat.
            $this->addField(new SchvaleniSlCestyEditField($this, $this->f_pravaschvaleni3, null, 6, 7, '3', $STR_SchvaleniSC3 ?: \Ron\ronText('Schvaleni_sc_plne_3')));
            if (RonPortalSettings::getIntValue('slcesty_filtr_uroven4')) {
                $this->addField(new SchvaleniSlCestyEditField($this, $this->f_pravaschvaleni4, null, 30, 31, '4', \Ron\ronText('Schvaleni_sc_plne_4')));
            }
        } else {
            $this->addField( ['sqlFieldName' => 'SluzebniCesta.SchvalenoZamestnancem', 'sqlIndex' => 46,
                BROWSER_FIELD_CODE => 'col-schvalenozamestnancem', BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_VISIBLE => 0,
                BROWSER_FIELD_HEADER_TEXT => 'Schvaleno_zamestnancem', BROWSER_FIELD_FORMAT => 'anone', 'ordering' => 1] );

            $this->addField(new SchvaleniSlCestyField($this ->database));
        }

        $this->addField([BROWSER_FIELD_SQL_NAME => 'SluzebniCesta.DopravniProstredek', BROWSER_FIELD_SQL_INDEX => 48,
            BROWSER_FIELD_CODE => 'col-dopravniprostredek', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_HEADER_TEXT => 'Dopravni_prostredek', BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_NAME => 'SluzebniCesta.RegistracniZnacka', BROWSER_FIELD_SQL_INDEX => 49,
            BROWSER_FIELD_CODE => 'col-registracniznacka', BROWSER_FIELD_VISIBLE => 0,
            BROWSER_FIELD_HEADER_TEXT => 'Vozidlo', BROWSER_FIELD_ORDERING => 1]);


        // dokumenty
        $this->addField(new DokumentySCField());

        if (!$this ->schvalovani  &&  $this->f_pravaeditace->jsouNejakePrava()) {
            $this->addField(new ZruseniSCField($this ->database));
            $this->addField(new KopieSCField());
        }
    }


    function setFilterForm()
    {
        $this ->filterForm = new SluzebniCestyFilterForm($this ->database, $this ->schvalovani);
        $this ->filterForm ->htmlId = 'sc_filter_form';
        $this ->filterForm ->loadData();
    }


    function renderFilter()
    {
        $this ->filterForm ->render();

        // pokud muze editovat, tak zobrazi odkaz "novy zaznam"
        if (!$this->schvalovani) {
            $pravo = Prava::vratHodnotuPrava('sluzebnicesty');
            if ($pravo) echo "<div class='normaltext' id='sc_links'><a href='?menuid=sluzebnicesty&formaction=new'>" . \Ron\ronText('New_record') . "</a></div>\n";

            echo "<div id='sc_hromadneschvalovani'><input type='button' class='button' name='zobrazithromadne' value='" . \Ron\ronText('Schvalit_hromadne') . "' onclick='zobrazHromadneSchvaleniSC()'/></div>";
        }
    }


    function getSQLFilter()
    {
        $result = '';

        // uroven schvaleni (vse, ke schvaleni, ...)
        switch (SluzebniCestyFilterForm::getUrovenSchvaleni()) {
            case -2 : // bez schvaleni
                $result = 'SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL  AND  SluzebniCesta.SchvalenoNadrizenym = 0 AND SluzebniCesta.SchvalenoNadrizenym2 = 0 ';
                break;

            case 80 : // zauctovano
                $result = 'SluzebniCesta.Zauctovano = 1 ';
                break;

            case 99 : // povolene nadrizenym a zaroven nepovolene nadrizenym 2, urovni 1,2,3 ani 4
                $result =
                    'SluzebniCesta.SchvalenoNadrizenym = 1 AND SluzebniCesta.SchvalenoNadrizenym2 = 0 AND SluzebniCesta.SchvalenoNadrizenym3 = 0  AND SluzebniCesta.SchvalenoNadrizenym4 = 0' .
                    ' AND  SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL';
                break;

            case 100 : //povolene nadrizenym 2 a zaroven neschvalene urovni 1,2,3 ani 4
                $result =
                    'SluzebniCesta.SchvalenoNadrizenym2 = 1 AND SluzebniCesta.SchvalenoNadrizenym3 = 0  AND SluzebniCesta.SchvalenoNadrizenym4 = 0' .
                    ' AND  SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL';
                break;

            case 101 : //povolene nadrizenym 3 a zaroven neschvalene urovni 1,2,3 ani 4
                $result =
                    'SluzebniCesta.SchvalenoNadrizenym3 = 1 AND SluzebniCesta.SchvalenoNadrizenym4 = 0' .
                    ' AND  SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL';
                break;

            case 102 : //povolene nadrizenym 4 a zaroven neschvalene urovni 1,2,3 ani 4
                $result =
                    'SluzebniCesta.SchvalenoNadrizenym4 = 1' .
                    ' AND  SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL';
                break;

            case 1 :
                $result = 'SluzebniCesta.Schvaleni1 IS NOT NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL ';
                break;

            case 2 :
                $result = 'SluzebniCesta.Schvaleni2 IS NOT NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL ';
                break;

            case 3 :
                $result = 'SluzebniCesta.Schvaleni3 IS NOT NULL AND SluzebniCesta.Schvaleni4 IS NULL ';
                break;

            case 4 :
                $result = 'SluzebniCesta.Schvaleni4 IS NOT NULL ';
                break;

            case -1 : // ke schvaleni (dale se filtruje v enable line podle konkretnich prav)
                $result = 'SluzebniCesta.Schvaleni4 IS NULL';
                // ty co jsou schvaleny posledni urovni se nezobrazuji (nejsou "ke schvaleni")

                if (!$this->f_pravaschvaleni4->jsouNejakePrava()) {
                    $result .= ' AND SluzebniCesta.Schvaleni3 IS NULL ';
                    // pokud uz je schvalena urovni 3 a nemam pravo na 4, tak se nezobrazuje

                    if (!$this->f_pravaschvaleni3->jsouNejakePrava()) {
                        $result .= ' AND SluzebniCesta.Schvaleni2 IS NULL ';
                        // pokud uz je schvalena urovni 2 a nemam pravo na 3, tak se nezobrazuje

                        if (!$this->f_pravaschvaleni2->jsouNejakePrava()) {
                            $result .= ' AND SluzebniCesta.Schvaleni1 IS NULL ';
                            // pokud uz je schvalena urovni 1 a nemam pravo na 2 ani 3, tak se nezobrazuje
                        }
                    }
                }

                break;

            default : // vse
                ;
        }

        switch ($_SESSION['jenzahranicnisc']) {
            case 1 :
                if ($result) $result .= ' AND ';
                $result .= 'SluzebniCesta.Zahranicni = 1';
                break;

            case 2 :
                if ($result) $result .= ' AND ';
                $result .= 'SluzebniCesta.Zahranicni = 0';
                break;
        }

        // obdobi
        if ($this ->filterForm ->jenObdobi) {
            if ($result) $result .= ' AND ';
            $result .=
                'SluzebniCesta.DatumCasDo >= ' . $this ->database ->timestampToString($this ->filterForm ->vybraneObdobiOd) .
                ' AND SluzebniCesta.DatumCasOd < ' . $this ->database ->timestampToString(DateUtils::incDay($this ->filterForm ->vybraneObdobiDo, 1));
        } else if ($this ->filterForm ->filtrovatDatum) {
            if ($result) $result .= ' AND ';
            $result .=
                'SluzebniCesta.DatumCasDo >= ' . $this ->database ->timestampToString($this ->filterForm ->datumOd) .
                ' AND SluzebniCesta.DatumCasOd < ' . $this ->database ->timestampToString(DateUtils::incDay($this ->filterForm ->datumDo, 1));
        }

        // filtr podle rozdeleni
        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) && $this ->filterForm ->kodRozdeleni !== null) {
            if ($result) $result .= ' AND ';

//var_dump($this ->filterForm ->vcetnePodrizenychRozdeleni);
            if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) == RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP)) {
                switch ($this ->filterForm ->vcetnePodrizenychRozdeleni) {
                    case 0 : // jen vybrane
                        $result .=
                            ' Rozdeleni.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                            " AND Rozdeleni.Kod = '" . $this->database->addSlashes($this->filterForm->kodRozdeleni) . "'";
                        break;
                    case 3 : // jen primo podrizene
                        $result .=
                            ' Rozdeleni.RozdeleniRodice = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                            " AND Rozdeleni.Rodic = '" . $this->database->addSlashes($this->filterForm->kodRozdeleni) . "'";
                        break;

                    default :
                        // rozdeleniosob je primo soucasti SQL dotazu, hledam jen zaznam hierarchie
                        $result .=
                            ' EXISTS (' .
                            '   SELECT * FROM HierarchieRozdeleni ' .
                            "   WHERE HierarchieRozdeleni.KodNadrazene = '" . $this->database->addSlashes($this->filterForm->kodRozdeleni) . "'" .
                            '     AND HierarchieRozdeleni.TypNadrazene = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                            '     AND HierarchieRozdeleni.KodPodrizene = RozdeleniOsob.Rozdeleni' .
                            '     AND HierarchieRozdeleni.TypPodrizene = RozdeleniOsob.KodTypuRozdeleni' .
                            (($this->filterForm->vcetnePodrizenychRozdeleni == 2) ? ' AND (HierarchieRozdeleni.kodNadrazene <> HierarchieRozdeleni.kodPodrizene) ' : '') . // jen podrizene
                            ' )';
                }


            } else {
                // jiny typ rozdeleni nez v dotazu, nebo v dotazu rozdeleni vubec neni
                // nutno pripojit i tabulku rozdeleniosob
                switch ($this ->filterForm ->vcetnePodrizenychRozdeleni) {
                    case 0 : // jen vybrane
                    case 3 : // jen primo podrizene
                        $result .=
                            ' EXISTS (' .
                            '   SELECT * FROM Rozdeleni R, RozdeleniOsob RO ' .
                            '   WHERE RO.RC = SluzebniCesta.RC ' .
                            '     AND RO.OsobniCislo = SluzebniCesta.OsobniCislo ' .
                            '     AND RO.DatumNastupu = SluzebniCesta.DatumNastupu ' .
                            '     AND RO.PlatnostOd <= SluzebniCesta.DatumCasOd ' .
                            '     AND (RO.PlatnostDo >= SluzebniCesta.DatumCasOd  OR  RO.PlatnostDo IS NULL)' . // Pozor! Nezobrazi se osoby, ktere maji ukonceno prirazeni v den sluzebni cesty!
                            '     AND RO.Rozdeleni = R.Kod' .
                            '     AND RO.KodTypuRozdeleni = R.KodTypuRozdeleni';
//                            '     AND H.KodPodrizene = RO.Rozdeleni' .
//                            '     AND H.TypPodrizene = RO.KodTypuRozdeleni' .

                        if ($this ->filterForm ->vcetnePodrizenychRozdeleni == 3) {
                            // jen primo podrizene
                            $result .=
                                " AND R.Rodic = '" . $this ->database ->addSlashes($this ->filterForm ->kodRozdeleni) . "'" .
                                ' AND R.RozdeleniRodice = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP);
                        } else {
                            // jen vybrane
                            $result .=
                                " AND R.Kod = '" . $this ->database ->addSlashes($this ->filterForm ->kodRozdeleni) . "'" .
                                ' AND R.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP);
                        }

                        $result .=
                            ' )';
                        break;

                    default :
                        // vcetne podrizenych (1) nebo jen podrizene (2)
                        $result .=
                            ' EXISTS (' .
                            '   SELECT * FROM HierarchieRozdeleni H, RozdeleniOsob RO ' .
                            '   WHERE RO.RC = SluzebniCesta.RC ' .
                            '     AND RO.OsobniCislo = SluzebniCesta.OsobniCislo ' .
                            '     AND RO.DatumNastupu = SluzebniCesta.DatumNastupu ' .
                            '     AND RO.PlatnostOd <= SluzebniCesta.DatumCasOd ' .
                            '     AND (RO.PlatnostDo >= SluzebniCesta.DatumCasOd  OR  RO.PlatnostDo IS NULL)' . // Pozor! Nezobrazi se osoby, ktere maji ukonceno prirazeni v den sluzebni cesty!
                            "     AND H.KodNadrazene = '" . $this ->database ->addSlashes($this ->filterForm ->kodRozdeleni) . "'" .
                            '     AND H.TypNadrazene = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                            '     AND H.KodPodrizene = RO.Rozdeleni' .
                            '     AND H.TypPodrizene = RO.KodTypuRozdeleni' .
                            (($this ->filterForm ->vcetnePodrizenychRozdeleni == 2) ? ' AND (H.kodNadrazene <> H.kodPodrizene) ' : '') . // jen podrizene
                            ' )';
                }
            }
        }

        // jen vybrana osoba
        if (!$this ->filterForm ->vsechnyOsoby) {
            if ($result) {
                $result .= ' AND ';
            }
            $result .=
                "     SluzebniCesta.RC = '" . $this ->database ->addSlashes($this ->filterForm ->vybranaOsobaRC) . "'" .
                " AND SluzebniCesta.OsobniCislo = '" . $this ->database ->addSlashes($this ->filterForm ->vybranaOsobaOsobniCislo) . "'" .
                ' AND SluzebniCesta.DatumNastupu = ' . $this ->database ->timestampToString($this ->filterForm ->vybranaOsobaDatumNastupu);
        }

        return $result;
    }


    /**
     * @param array $values
     * @param int $datumod
     * @param int $datumdo
     * @return bool
     * @throws \Exception
     */
    function scEnableLine($values, $datumod, $datumdo)
    {
        $this ->f_jsonRowIndex = null;

        if (SluzebniCestyFilterForm::getUrovenSchvaleni() == -1) {
            // filtr Ke schvaleni - zkontroluje, jesti to ceka na me

            if ($values[30]) return false;
            // jiz je schvaleno nejvyssi urovni = neceka na nikoho

            if ($values[6]) return $this->f_pravaschvaleni4->lzeZobrazitPP($values[22], $datumod, $datumdo);
            //je schvaleno 3, ceka na 4 => pokud mamm pravo, ceka na me

            if ($values[4]) return $this->f_pravaschvaleni3->lzeZobrazitPP($values[22], $datumod, $datumdo);
            // je schvaleno 2, ceka na 3 => pokud mam pravo, ceka na me

            if ($values[2]) return $this->f_pravaschvaleni2->lzeZobrazitPP($values[22], $datumod, $datumdo);

            // je schvaleno 1, ceka na 2 => pokud mam pravo, ceka na me

            return $this->f_pravaschvaleni1->lzeZobrazitPP($values[22], $datumod, $datumdo);
            // neni schvaleno zadnou urovni => pokud mam pravo na 1, ceka na me
        }

        // Zobrazi zaznam, pokud mam kterekoliv pravo
        if (!$this->schvalovani) {
            if ($this->f_pravaeditace->lzeZobrazitPP($values[22], $datumod, $datumdo)) return true;
            if ($this->f_pravaSC->lzeZobrazitPP($values[22], $datumod, $datumdo)) return true;
            if ($this->f_pravaZauctovano->lzeZobrazitPP($values[22], $datumod, $datumdo)) return true;
        }

        if ($this->f_pravaschvaleni1->lzeZobrazitPP($values[22], $datumod, $datumdo)) return true;
        if ($this->f_pravaschvaleni2->lzeZobrazitPP($values[22], $datumod, $datumdo)) return true;
        if ($this->f_pravaschvaleni3->lzeZobrazitPP($values[22], $datumod, $datumdo)) return true;
        if ($this->f_pravaschvaleni4->lzeZobrazitPP($values[22], $datumod, $datumdo)) return true;

        return false;
    }


    /**
     * @param array $values
     * @return bool
     * @throws \Exception
     */
    function enableLine($values)
    {
        $datumod = $this ->database ->stringToTimestamp($values[17]);
        $datumdo = $this ->database ->stringToTimestamp($values[18]);

        $result = $this ->scEnableLine($values, $datumod, $datumdo);
        if (!$result) return false;

        if ($this ->schvalovani) {

            $this ->f_jsonRowIndex = count($this ->f_jsonObject['items']);

            $this ->f_hiddenFields .= "<input type='hidden' name='idSC_{$this ->f_jsonRowIndex}' value='{$values[0]}'/>";

            $this ->f_jsonObject['items'][] = [
                'id' => $values[0],
                'pravo_n' => $this ->f_pravaNadrizeny ->lzeZobrazitPP($values[22], $datumod, $datumdo) ? 1 : 0,
                'pravo_n2' => $this ->f_pravaNadrizeny2 ->lzeZobrazitPP($values[22], $datumod, $datumdo) ? 1 : 0,
                'pravo_1' => $this ->f_pravaschvaleni1 ->lzeZobrazitPP($values[22], $datumod, $datumdo) ? 1 : 0,
                'pravo_2' => $this ->f_pravaschvaleni2 ->lzeZobrazitPP($values[22], $datumod, $datumdo) ? 1 : 0,
                'pravo_3' => $this ->f_pravaschvaleni3 ->lzeZobrazitPP($values[22], $datumod, $datumdo) ? 1 : 0,
                'pravo_4' => $this ->f_pravaschvaleni4 ->lzeZobrazitPP($values[22], $datumod, $datumdo) ? 1 : 0,
                'schvaleno_n' => $values[23] ? 1 : 0,
                'schvaleno_n2' => $values[26] ? 1 : 0,
                'schvaleno_1' => $values[2] ? 1 : 0,
                'schvaleno_2' => $values[4] ? 1 : 0,
                'schvaleno_3' => $values[6] ? 1 : 0,
                'schvaleno_4' => $values[30] ? 1 : 0
            ];

        } else {
            $this ->f_jsonRowIndex = null;
        }

        return true;
    }


    /**
     * @param array  $values
     * @param string $className
     * @param int    $rowNumber
     * @return string   Vraci atributy pro <tr>
     */
    function getRowAttributes($values, $className, $rowNumber)
    {
        if ($values[26]) $className .= ' zahranicni';

        if ($values[30]) $className .= ' sc_schvalena4';
        else if ($values[6]) $className .= ' sc_schvalena3';
        else if ($values[4]) $className .= ' sc_schvalena2';
        else if ($values[2]) $className .= ' sc_schvalena1';
        else if ($values[26] == 1) $className .= ' sc_schvalenanadrizenym2';
        else if ($values[26] == 2) $className .= ' sc_zamitnutanadrizenym2';
        else if ($values[23] == 1) $className .= ' sc_schvalenanadrizenym';
        else if ($values[23] == 2) $className .= ' sc_zamitnutanadrizenym';

        $result = parent::getRowAttributes($values, $className, $rowNumber);

        $result .= ' id="sc_row_' . $values[0] . '"';

        return $result;
    }


    /**
     * @param array    $values
     * @param string   $className
     * @param int      $rowNumber
     * @param bool     $edit
     */
    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber);

        if ($this->schvalovani && $values[20]) {
            // v rezimu schvalovani je popis na dalsim radku
            echo "<tr class='$className'>";
            echo '<td></td><td></td><td colspan="' . ($this->columnCount - 2) . '"><div class="poznamka browserpoznamka">' . StringUtils::htmlTextConvert($values[20]) . '</div></td>';
            echo "</tr>\n";
        }


        $zacatek = '';
        if ($values[9] && (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_AUTOMATICKY) || $values[9] != RonPortalSettings::getValue(SluzebniCesta::SETTINGS_ZACATEK))) {
            $zacatek = $values[9];
        }
        if ($values[10]) {
            if ($zacatek) $zacatek .= ', ';
            $zacatek .= $values[10];
        }

        $konec = '';
        if ($values[11] && (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_AUTOMATICKY) || $values[11] != RonPortalSettings::getValue(SluzebniCesta::SETTINGS_ZACATEK))) {
            $konec = $values[11];
        }
        if ($values[12]) {
            if ($konec) $konec .= ', ';
            $konec .= $values[12];
        }

        if ($zacatek || $konec || $values[13]) {
            // zobrazi podrobnosti
            echo "<tr class='$className'>";
            echo '<td></td><td></td><td colspan="' . ($this->columnCount - 2) . '"><div class="poznamka browserpoznamka">';

            echo $zacatek;
            if ($zacatek && $konec) echo ' - ';
            echo $konec;

            if ($values[13]) echo ', ' . \Ron\ronText('Pres') . ': ' . $values[13]; // pres

            echo '</div></td>';
            echo '</tr>';
        }

        if ($values[8]) {
            // zobrazi poznamku k vraceni
            echo "<tr class='$className'>";
            echo '<td></td><td><div class="poznamka browserpoznamka" style="text-align:right"><strong>' . \Ron\ronText('SC_Vracena') . '</strong>: </div></td><td colspan="' . ($this->columnCount - 2) . '"><div class="poznamka browserpoznamka">' . $values[8] . '</div></td>';
            echo '</tr>';
        }
    }


    function renderContent()
    {
        $this ->f_jsonObject = [];
        $this ->f_jsonObject['items'] = [];

        $this ->f_hiddenFields = '';

        if ($this ->schvalovani) {
            echo
                '<form method="post" name="hromadneSchvalovaniSC" action="?menuid=sluzebnicesty">' .
                '<input type="hidden" name="hromadneSchvaleni" value="1"/>';
        }

        echo
            '<ul class="popupmenu"  id="schvaleniSCMenu" style="display:none">' .
            "<li data-ron-hodnotaschvaleni='1'>" . \Ron\ronText('Oznacit_vse') . "</li>".
            "<li data-ron-hodnotaschvaleni='0'>" . \Ron\ronText('Zrusit_oznaceni') . "</li>".
            "<li data-ron-hodnotaschvaleni='-1'>" . \Ron\ronText('form.Original') . "</li>".
            '</ul>';


        parent::renderContent();

        if ($this ->f_hiddenFields) echo $this ->f_hiddenFields;
        echo '<input type="hidden" name="pocet" value="' . count($this ->f_jsonObject['items']) . '" />';

        if ($this->schvalovani) {
            echo
                "<div class='buttons'>" .
                "<input type='button' class='submitbutton' value='" . \Ron\ronText('Provest') . "' id='provestHromadneSchvaleniSC' />" .
                '<input type="button" class="button" value="' . \Ron\ronText('Cancel') . '" id="stornoHromadneSchvaleniSC" />' .
                '</div>'.
                "</form>\n";

            echo '<script type="text/javascript">';
            echo 'initSchvalovaniSC(' . json_encode($this ->f_jsonObject) . ');';
            echo '</script>';
        } else {
            $pravo = Prava::vratHodnotuPrava('sluzebnicesty');
            if ($pravo) echo "<div class='normaltext'><a href='?menuid=sluzebnicesty&formaction=new'>" . \Ron\ronText('New_record') . "</a></div>\n";
        }
    }
}
