<?php
/**
 * UTF8
 */

namespace Ron\RonPortal;

use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SQLUtils;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\StringUtils;

require_once('ron/browser.php');
require_once('sluzebnicesty/radekuctovanisc.php');



class EditaceUSCField extends BrowserField
{
    /** @var SluzebniCesta */
    public $sc;
    public $action;
    public $title;


    /**
     * @param SluzebniCesta $sc
     * @param string $action
     */
    function __construct($sc, $action, $code)
    {
        parent::__construct();
        $this ->sc = $sc;
        $this ->action = $action;
        $this ->code = $code;
        $this ->align = 'left';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[2] >= 1 && $rowData[2] <= 3) return '';

        $pravo = Prava::vratPrava('slcesty_ucet', 'slcesty_ucet_' . $rowData[2]);
        if (!$pravo ->lzeZobrazitPP($this ->sc ->f_PP ->idPP, $this ->sc ->datumCasOd, $this ->sc ->datumCasDo)) {
            return '';
        }

        switch ($this ->action) {
            case 'editovatuctovanisluzebky' :
                $glyph = 'glyph-edit';
                $title = \Ron\ronText('Editovat_zaznam');
                break;

            case 'kopirovatuctovanisluzebky' :
                $glyph = 'glyph-copy';
                $title = \Ron\ronText('Kopirovat_zaznam');
                break;

            default :
                $glyph = '';
                $title = '';
        }

        return
            "<a class='sendPosition' href='?menuid=sluzebnicesty&action={$this ->action}&record={$rowData[0]}&radek={$rowData[1]}'><i class='inline-glyph glyph-button fa $glyph' title='$title'></i></a>";
    }
}


class ZruseniUSCField extends BrowserField
{
    /** @var SluzebniCesta */
    public $sc;

    /**
     * @param SluzebniCesta $sc
     */
    function __construct($sc)
    {
        parent::__construct();
        $this ->sc = $sc;
        $this ->code = 'col-delete';
        $this ->align = 'left';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[2] >= 1 && $rowData[2] <= 3) return '';

        $pravo = Prava::vratPrava('slcesty_ucet', 'slcesty_ucet_' . $rowData[2]);
        if (!$pravo ->lzeZobrazitPP($this ->sc ->f_PP ->idPP, $this ->sc ->datumCasOd, $this ->sc ->datumCasDo)) {
            return '';
        }

        require_once('ron/guiutils.php');
        return GuiUtils::changeLocationQuest(
            '<i class="inline-glyph glyph-button fa glyph-delete" title="' . \Ron\ronText('Zrusit_zaznam') . '"></i>', \Ron\ronText('SC_Opravdu_zrusit_radek'),
            "?menuid=sluzebnicesty&action=zrusituctovanisluzebky&record=" . $rowData[0] . "&radek=" . $rowData[1],
            'data-ron-send-position="1"');
    }
}


class CastkaUSCField extends BrowserField
{
    function __construct()
    {
        parent::__construct();

        $this ->code = 'col-castka';
        $this ->header = \Ron\ronText('Castka');
        $this ->align = 'right';
        $this ->sqlFieldName = 'RadekUctovaniSCesty.Castka';
        $this ->sqlFieldIndex = 12;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ( !isset($value) ) return '';

        switch ($rowData[17]) {
            // druh pohybu minus nebo plus
            case 1 :
            case 2 :
                // pokud se ma znamenko zobrazovat kladne (nastaveni 0) prevede na ABS
                if (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZNAMENKO_UCTOVANI)) $value = abs($value);
                break;
        }

        return StringUtils::format( 'number2', $value );
    }
}


class DokumentyUSCField extends BrowserField
{
    var $lzeEditovat;

    /**
     * @param Database $database
     * @param bool $lzeEditovat
     */
    function __construct($database, $lzeEditovat)
    {
        parent::__construct($database);
        $this->code = 'col-dokumenty';
        $this->align = 'center';

        $this ->lzeEditovat = $lzeEditovat;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $sql =
            'SELECT DokumentSluzebniCesty.Id, Dokument.Popis' .
            '  FROM DokumentSluzebniCesty, Dokument ' .
            ' WHERE SluzebniCesta_ID = ' . (int)$rowData[0] .
            '   AND Dokument.Id = DokumentSluzebniCesty.Id ' .
            '   AND CisloRadku = ' . (int)$rowData[1];

        $result = '';

        $cursor = $this->database->getSqlResult($sql);
        while ($row = $cursor->fetch()) {
            $result .=
                '<a target="_blank" href="?menuid=sluzebnicesty&action=download&idslcesty=' . (int)$rowData[0] . '&record=' . $row[0] . '">' .
                '<i class="inline-glyph glyph-button fa glyph-attachment" title="' . StringUtils::htmlTextConvert($row[1]) . '"></i>' .
                '</a>';

        }
        $cursor ->close();

        if ($this ->lzeEditovat  &&  $rowData[2] > 3  ||  $rowData[2] < 1) {
            $result .=
                '<a href="#" title="' . \Ron\ronText('Vlozit_novy_dokument') . '">' .
                '<i class="inline-glyph fa glyph-upload show-dokument-dialog" data-fields="CisloRadku" data-field-cisloradku="' . $rowData[1] . '"></i>' .
                '</a>';
        }

        return $result;
    }
}

class CastkaDMField extends BrowserField
{
    private $indexCastka;
    private $indexMena;

    /**
     * CastkaDMField constructor.
     * @param null|Database $database
     * @param int $indexCastka
     * @param int $indexMena
     */
    public function __construct($database, $indexCastka, $indexMena)
    {
        parent::__construct($database);

        $this->indexCastka = $indexCastka;
        $this->indexMena = $indexMena;

        $this->code = 'col-castkadm';
        $this->header = \Ron\ronText('Castka_DM');
        $this->ordering = true;
        $this->sortSQL = 'RadekUctovaniSCesty.DM_Castka';
        $this->sortSQLDesc = 'RadekUctovaniSCesty.DM_Castka DESC';

        $this->format = 'number2';
        $this->align = 'right';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        if ($rowData[$this->indexMena]) {
            $result = StringUtils::format($this->format, $rowData[$this->indexCastka]);
        }

        return $result;
    }
}

class MenaDMField extends BrowserField
{
    private $indexMena;

    /**
     * CastkaDMField constructor.
     * @param null|Database $database
     * @param int $indexMena
     */
    public function __construct($database, $indexMena)
    {
        parent::__construct($database);

        $this->indexMena = $indexMena;

        $this->code = 'col-menadm';
        $this->header = \Ron\ronText('Mena_DM');
        $this->ordering = true;
        $this->sortSQL = 'RadekUctovaniSCesty.DM_KodMeny';
        $this->sortSQLDesc = 'RadekUctovaniSCesty.DM_KodMeny DESC';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        if ($rowData[$this->indexMena]) {
            $result = $rowData[$this->indexMena];
        }

        return $result;
    }
}


class KurzDMField extends BrowserField
{

    /**
     * KurzDMField constructor.
     */
    public function __construct()
    {
        parent::__construct();

        $this ->code = 'col-kurz';
        $this ->header = \Ron\ronText('Kurz');
        $this ->sqlFieldName = 'RadekUctovaniSCesty.DM_Kurz'; // pouze pro funkci renderFieldEdit
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        if (!$rowData[20]) return '';

        // je zaskrtnuto "vlastni kurz"
        return StringUtils::format('number', $rowData[21]) . ' ' . $rowData[13] . ' = ' . StringUtils::format('number', $rowData[22]) . ' ' . $rowData[19]; // RonPortalSettings::getValue(RonPortalSettings::VYCHOZI_MENA);
    }
}


class RadekUctovaniSlCestyBrowser extends Browser
{
    var $sluzebniCesta;

    var $editovatRadek;
    var $kopirovatRadek; // cislo radku, ktery se ma kopirovat
    var $idSluzebniCesty;
    var $zobrazitNovyRadek;
    var $lzeEditovat;

    var $f_datum; // datum pocatku sluzebni cesty pro pridani noveho zaznamu

    var $odkazyEditace; // priznak, jestli jsou zobrazeny odkazy pro editaci

    var $f_vzdalenost_start;
    var $f_vzdalenost_cil;
    var $f_vzdalenost_pres;


    // datum urcuje hodnotu pro datumu pro novy zaznam - mel by to byt zacatek SC
    /**
     * @param Database $database
     * @param SluzebniCesta $sluzebniCesta
     * @param $akce
     */
    function __construct($database, $sluzebniCesta, $akce)
    {
        parent::__construct($database);

        $this ->sluzebniCesta = $sluzebniCesta;

        $this ->code = 'browseructovanisc';
        $this ->title = \Ron\ronText('Uctovani');
        $this ->tableId = 'tblRadekUctovaniSlCesty';

        $this -> sqlSelectPart =
            'RadekUctovaniSCesty.SluzebniCesta_ID, RadekUctovaniSCesty.CisloRadku, RadekUctovaniSCesty.Typ, ' . // 0..2
            'RadekUctovaniSCesty.NVV_SpotrebaVozidla, RadekUctovaniSCesty.NVV_PocetUjetychKM, RadekUctovaniSCesty.NVV_NepocitatZaklSazbu, ' . // 3..5
            'RadekUctovaniSCesty.NVV_NeprokazanaCenaPHM, RadekUctovaniSCesty.NVV_CenaPHM, RadekUctovaniSCesty.NVV_DruhPHM, RadekUctovaniSCesty.ZpusobPlatby, ' . //6..9

            'RadekUctovaniSCesty.Den, TypRUctovaniSlCesty.Popis, RadekUctovaniSCesty.Castka, RadekUctovaniSCesty.KodMeny, RadekUctovaniSCesty.CastkaDPH, '. // 10..14

            'ZpusobPlatbySC.Popis, RadekUctovaniSCesty.Popis, TypRUctovaniSlCesty.DruhPohybu, ' . // 15..17
            'RadekUctovaniSCesty.DM_Castka, RadekUctovaniSCesty.DM_KodMeny, ' . //18, 19
            'RadekUctovaniSCesty.DM_VlastniKurz, RadekUctovaniSCesty.DM_Kurz_Mnozstvi, RadekUctovaniSCesty.DM_Kurz, ' . // 20..22
            'RadekUctovaniSCesty.Rozdeleni, R1.Popis, TR1.Popis, RadekUctovaniSCesty.Rozdeleni2, R2.Popis, TR2.Popis';  // 23..28

        $this ->sqlFromPart =
            '(RadekUctovaniSCesty INNER JOIN TypRUctovaniSlCesty ' .
            '  ON RadekUctovaniSCesty.Typ = TypRUctovaniSlCesty.Kod ' .
            ' ) ' .
            ' LEFT OUTER JOIN ZpusobPlatbySC ' .
            ' ON ZpusobPlatbySC.Kod = RadekUctovaniSCesty.ZpusobPlatby ' .
            ' LEFT OUTER JOIN Rozdeleni R1' .
            ' ON R1.KodTypuRozdeleni = RadekUctovaniSCesty.KodTypuRozdeleni' .
            ' AND R1.Kod = RadekUctovaniSCesty.Rozdeleni' .
            ' LEFT OUTER JOIN TypRozdeleni TR1' .
            ' ON TR1.Kod = R1.KodTypuRozdeleni' .
            ' LEFT OUTER JOIN Rozdeleni R2' .
            ' ON R2.KodTypuRozdeleni = RadekUctovaniSCesty.KodTypuRozdeleni2' .
            ' AND R2.Kod = RadekUctovaniSCesty.Rozdeleni2' .
            ' LEFT OUTER JOIN TypRozdeleni TR2' .
            ' ON TR2.Kod = R2.KodTypuRozdeleni';

        $this ->sqlWherePart = '';

        $this ->primaryField = 1;

        if ($sluzebniCesta ->lzeEditovat) {
            $prava = Prava::vratPrava('slcesty_ucet');
            $this ->lzeEditovat = $prava ->lzeZobrazitPP($sluzebniCesta ->f_PP ->idPP, $sluzebniCesta ->datumCasOd, $sluzebniCesta ->datumCasDo);
        } else {
            $this->lzeEditovat = false;
        }

        $this->showMessages = isset($_POST['EditaceUctovaniSC']);

        $this ->cssClass = 'browser minwidth';

        $this ->defaultSort = 1;
        $this ->addField()
            ->setSqlFieldName('RadekUctovaniSCesty.Den', 10) ->setCode('col-datum') ->setHeaderText('Datum') ->setFormat('date');

        $this ->addField()
            ->setSqlFieldName('TypRUctovaniSlCesty.Popis', 11) ->setCode('col-popis') ->setHeaderText('Typ');

        $this ->addField( new CastkaUSCField() );

        $this ->addField()
            ->setSqlFieldName('RadekUctovaniSCesty.KodMeny', 13) ->setCode('col-mena') ->setHeaderText('Mena');

        if (RonPortalSettings::getIntValue('vlastnikurzradkusc')) {
            $this->addField(new KurzDMField());
        }

        if ( RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DPH) ) {
            $this ->addField()
                ->setSqlFieldName('RadekUctovaniSCesty.CastkaDPH', 14) ->setCode('col-dph') ->setHeaderText('SC_DPH')
                ->setFormat('number2') ->setAlign('right');
        }

        $this ->addField()
            ->setSqlFieldName('ZpusobPlatbySC.Popis', 15) ->setCode('col-zpusobplatby') ->setHeaderText('SC_Zpusob_platby');

        $this ->addField()
            ->setSqlFieldName('RadekUctovaniSCesty.Popis', 16) ->setCode('col-popis') ->setHeaderText('Popis');

        $this->addField(new CastkaDMField($database, 18, 19));
        $this->addField(new MenaDMField($database, 19));

        if ($this ->lzeEditovat &&
            $akce != 'editovatuctovanisluzebky' &&
            $akce != 'kopirovatuctovanisluzebky' &&
            $akce != 'pridatuctovanisluzebky' &&
            $akce != 'ulozuctovanisluzebky'
        ) {
            $this ->addField(new EditaceUSCField($this ->sluzebniCesta, 'editovatuctovanisluzebky', 'col-edit'));
            $this ->addField(new EditaceUSCField($this ->sluzebniCesta, 'kopirovatuctovanisluzebky', 'col-copy'));
            $this ->addField(new ZruseniUSCField($this ->sluzebniCesta));
            $this ->odkazyEditace = true;
        } else {
            $this ->odkazyEditace = false;
        }

        $this ->addField(new DokumentyUSCField($database, $this ->lzeEditovat));

        $this ->loadConfigurationXml(RonPortalBrowser::loadBrowserConfiguration($this ->database, $this ->code));
    }


    function renderFieldEdit($index, $field, $value, $rowNumber, $recordId, $values)
    {
        require_once('ron/controls.php');

        switch ($field ->sqlFieldName) {
            case 'RadekUctovaniSCesty.Den' :
                if (isset($_POST['Den'])) $value = StringUtils::htmlTextConvert($_POST['Den']);
                else if (!$values && $this ->f_datum) $value = date('j.n.Y', $this ->f_datum);
                else $value = DBUtils::getFormatedValue($this ->database, $field ->format, $value);

                echo
                    '<td style="white-space:nowrap">' .

                    '<div class="input-group">' .
                    '<input type="text" name="Den" class="form-input input-date appinput inputDate ' . MessageManager::getClass('Den') . '" value="' . $value . '" maxlength="10" size="10" />' .
                    '<i class="input-group-text fa glyph-calendar showcalendar"  data-roncalendar-inputname="Den"></i>' .
                    '</div>' .

                    '<input type="hidden" name="SluzebniCesta_ID" value="' . $this ->idSluzebniCesty . '" />' .
                    '<input type="hidden" name="CisloRadku" value="' . ($values ? $values[1] : '') . '" />' .
                    '<input type="hidden" name="EditaceUctovaniSC" value="1">' .

                    "</td>\n";

                break;

            case 'RadekUctovaniSCesty.Castka' :
                if (isset($_POST['Castka'])) $value = $_POST['Castka'];
                else {
                    $value = (float)$value;

                    if (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZNAMENKO_UCTOVANI)  &&  $values[17] >= 1) $value = abs($value);
                }

                $typ = isset($_POST['Typ']) ? $_POST['Typ'] : $values[2];

                $jenahrada = TypRadkuUctovaniSluzebniCesty::typUctovaniJeNVV($this ->database, $typ);

                echo '<td><input type="text" name="Castka" class="form-input inputFloat" value="' . StringUtils::htmlTextConvert($value) . '" size="8" maxlength="8" ';
                if ($jenahrada) echo ' disabled="disabled" ';
                echo ' /></td>';
                break;

            case 'RadekUctovaniSCesty.KodMeny' :
                echo '<TD>';
                if (isset($_POST['KodMeny'])) $value = $_POST['KodMeny'];
                else if (!$values) $value = RadekUctovaniSluzebniCesty::vratVychoziMenu($this ->database, $this ->idSluzebniCesty);

                $combo = new ComboBox('KodMeny');
                $combo ->captionIndex = 0;
                $combo ->emptyRow = ' ';
                $combo ->alwaysShowSelected = 1;
                $combo ->htmlParams = ['id' => 'KodMeny', 'class' => 'form-input'];
                $combo ->loadData($this ->database, 'SELECT KodMeny_ISO FROM Mena WHERE Aktivni = 1 ORDER BY KodMeny_ISO', $value);
                $combo ->render();

                echo "</TD>\n";
                break;

            case 'RadekUctovaniSCesty.DM_Kurz' :
                echo '<TD>';
                $checked = isset($_POST['DM_VlastniKurz']) ? $_POST['DM_VlastniKurz'] : (int)$values[20];
                $checkedStr = $checked ? 'checked="checked"' : '';
                $mnozstvi = isset($_POST['DM_Kurz_Mnozstvi']) ? $_POST['DM_Kurz_Mnozstvi'] : StringUtils::format('number', $values[21]);
                $kurz = isset($_POST['DM_Kurz']) ? $_POST['DM_Kurz'] : StringUtils::format('number', $values[22]);
                $styleStr = $checked ? '' : 'style="display:none"';
                echo
                    '<div>' .
                    "<input type='checkbox' id='DM_VlastniKurz' name='DM_VlastniKurz' $checkedStr/> " .
                    \Ron\ronText('Vlastni_kurz') .
                    '</div>' .

                    "<div id='editace_uctovani_kurz' $styleStr>" .
                    '<input type="text" name="DM_Kurz_Mnozstvi" class="appinput inputFloat" value="' . $mnozstvi . '" size="4" /> <span id="editace_uctovani_mena">' . $values[13] . '</span> = ' .
                    '<input type="text" name="DM_Kurz" class="appinput inputFloat" value="' . $kurz . '" size="4" /> ' . $values[19] .
                    '</div>';

                echo "</TD>\n";
                break;

            case 'RadekUctovaniSCesty.CastkaDPH' :
                if (isset($_POST['CastkaDPH'])) $value = $_POST['CastkaDPH'];
                else $value = (float)$value;

                if (isset($_POST['Typ'])) $typ = $_POST['Typ'];
                else $typ = $values[2];

                $jenahrada = TypRadkuUctovaniSluzebniCesty::typUctovaniJeNVV($this ->database, $typ);

                echo '<td><input type="text" name="CastkaDPH" class="form-input inputFloat" value="' . StringUtils::htmlTextConvert($value) . '" size="8" maxlength="8" ';
                if ($jenahrada) echo ' disabled="disabled" ';
                echo ' /></td>';
                break;

            case 'RadekUctovaniSCesty.Popis' :
                if (isset($_POST['Popis'])) $value = $_POST['Popis'];
                echo '<TD><INPUT type="text" class="form-input" name="Popis" value="' . StringUtils::htmlTextConvert($value) . '" size="40" maxlength="160"></TD>';
                break;

            case 'TypRUctovaniSlCesty.Popis' :
                echo '<TD>';
                if (isset($_POST['Typ'])) {
                    $value = $_POST['Typ'];
                } else if ($values) {
                    $value = $values[2];
                } else if (RonPortalSettings::getValue('typradkuuctovanisc') !== ''  &&  RonPortalSettings::getValue('typradkuuctovanisc') !== null) {
                    $value = RonPortalSettings::getIntValue('typradkuuctovanisc');
                } else {
                    $value = null;
                }

                $options = TypRadkuUctovaniSluzebniCesty::loadOptions($this ->database, $this ->sluzebniCesta ->f_PP ->idPP);
                ComboBox::renderComboBox('Typ', $options, $value,
                        [//'emptyRow' => ' ',
                        'htmlParams' => ['id' => 'typRadkuUctovaniSC', 'class' => 'form-input'],
                        'dataTypeComparison' => false]);

                echo "</TD>\n";
                break;

            case 'ZpusobPlatbySC.Popis' :
                echo '<TD>';
                $value =  (isset($_POST['ZpusobPlatby'])) ? $_POST['ZpusobPlatby'] : $values[9];
                $combo = new ComboBox('ZpusobPlatby');
                $combo ->dataTypeComparison = false;
                $combo ->loadData($this ->database, 'SELECT Kod, Popis FROM ZpusobPlatbySC ORDER BY Popis', $value);
                $combo ->render();

                echo "</TD>\n";
                break;

            default :
                echo "<td></td>\n";
        }
    }


    /**
     * Editace nahrad za sluzebni vozidlo
     * @param array $values
     * @param string $className
     */
    function showEditaceNahrad($values, $className)
    {
        $typ = isset($_POST['Typ']) ? $_POST['Typ'] : $values[2];

        $jenahrada = TypRadkuUctovaniSluzebniCesty::typUctovaniJeNVV($this ->database, $typ);
        $styleStr = $jenahrada ? '' : 'style="display:none"';

        echo "<tr class='$className'>";
        echo '<td></td>';
        $colspan = count($this ->fields);
        echo "<td colspan='$colspan'>";
        echo '<div id="nahradyzavozidlo" ' . $styleStr . '>';
        echo '<table class="middle">';

        // spotreba - litru
        if (isset($_POST['NVV_SpotrebaVozidla'])) $value = $_POST['NVV_SpotrebaVozidla'];
        else if ($values) $value = StringUtils::format('number2', (float)$values[3]);
        else $value = $this ->sluzebniCesta ->spotrebaVozidla ? StringUtils::format('number_2', $this ->sluzebniCesta ->spotrebaVozidla) : ''; // novy radek

        echo '<tr><td>' . \Ron\ronText('Spotreba_TP') . '</td>';
        echo '<td><input type="text" name="NVV_SpotrebaVozidla" class="appinput inputFloat" value="' . StringUtils::htmlTextConvert($value) . '" size="8" maxlength="8" /> ';
        echo \Ron\ronText('Litru_100') . "</td></tr>\n";


        if (isset($_POST['NVV_PocetUjetychKM'])) $value = $_POST['NVV_PocetUjetychKM'];
        else $value = (float)$values[4];

        echo '<tr><td>' . \Ron\ronText('Pocet_km') . '</td>';
        echo '<td><input type="text" name="NVV_PocetUjetychKM" class="appinput inputInt" value="' . StringUtils::htmlTextConvert($value) . '" size="8" maxlength="8" /> ';

        echo \Ron\ronText('Nepocitat_zakladni_sazbu'). ' <input name="NVV_NepocitatZaklSazbu" type="checkbox"';

        if (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_NEPOCITAT_ZS_NEMENIT) && isset($_POST['NVV_NepocitatZaklSazbu'])) $checked = $_POST['NVV_NepocitatZaklSazbu'];
        else if ($values) $checked = $values[5];
        else $checked = RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_NEPOCITAT_ZAKLADNI_SAZBU); // novy radek

        if ($checked) echo ' checked="checked"';
        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_NEPOCITAT_ZS_NEMENIT)) echo ' disabled="disabled"';

        echo '/>';

        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_VYPOCET_VZDALENOSTI) && $this ->f_vzdalenost_start && $this ->f_vzdalenost_cil) {
            // proklik na zobrazeni trasy:
            echo "<span class='margin-left-2 trasa_link'>";
            echo '<a target="_blank" href="https://www.google.com/maps/dir/' .
                StringUtils::htmlTextConvert($this ->f_vzdalenost_start) . '/' .
                ($this ->f_vzdalenost_pres ? StringUtils::htmlTextConvert($this ->f_vzdalenost_pres) . '/' : '') .
                StringUtils::htmlTextConvert($this ->f_vzdalenost_cil) . '/' .
                '">' . \Ron\ronText('Trasa') . '</a>';

            echo '</span>';

            // vypocet vzdalenosti:
            // http://maps.google.com/maps/api/distancematrix/xml?origins=Brno&destinations=Karvin%C3%A1&sensor=true
            //echo '<script src="https://maps.googleapis.com/maps/api/js?sensor=false"></script>';
            //echo '<span style="display:inline-block; width:50px;"> </span>';
            //$vzdalenostscript = "GetDistance(this, '" . $this ->f_vzdalenost_start . "', '" . $this ->f_vzdalenost_cil . "', '" . $this ->f_vzdalenost_pres . "')";
            //echo '<input type="button" class="button" name="vypocetvzdalenosti" value="' . \Ron\ronText('Vzdalenost') . '" onclick="' . $vzdalenostscript . '" />';

            //echo '<input type="checkbox" name="NVV_DvojnasobekVzdalenosti" />' . \Ron\ronText('Zpatecni_cesta');
            //if ($this ->f_vzdalenost_pres) echo '<input type="checkbox" checked="checked" name="NVV_PocitatPres" />' . \Ron\ronText('Pres');
        }

        echo "</td></tr>\n";

        // Neprokaze cenu PHM => vyber druhu PHM
        if (isset($_POST['NVV_NeprokazanaCenaPHM'])) $neprokazana = $_POST['NVV_NeprokazanaCenaPHM'];
        else $neprokazana = $values[6];
        echo "<tr><td>";
        echo '<input name="NVV_NeprokazanaCenaPHM" type="radio" value="1" ';
        if ($neprokazana) echo " checked";
        echo " />";
        echo \Ron\ronText('Neprokaze_cenu_PHM') . '</td>';

        echo "<td>";
        if (isset($_POST['NVV_DruhPHM'])) $value = $_POST['NVV_DruhPHM'];
        else if ($values) $value = $values[8];
        else $value = $this ->sluzebniCesta ->phmVozidla ?: null;

        $styleStr = $neprokazana ? '' : ' style="display:none"';

        $combo = new ComboBox('NVV_DruhPHM');
        $combo ->emptyRow = ' ';
        $combo ->htmlParams = 'id="NVV_DruhPHM"' . $styleStr;
        $combo ->loadData($this ->database, 'SELECT Kod, Popis FROM DruhPHM ORDER BY Popis', $value);
        $combo ->render();

        echo "</td></tr>";


        // Prokaze cenu PHM => zadani ceny
        echo '<tr><td>';
        echo '<input name="NVV_NeprokazanaCenaPHM" type="radio" value="0" ';
        if (!$neprokazana) echo ' checked';
        echo ' />';
        echo \Ron\ronText('Prokaze_cenu_PHM') . '</td>';

        if (isset($_POST['NVV_CenaPHM'])) $value = $_POST['NVV_CenaPHM'];
        else $value = (float)$values[7];

        if ($neprokazana) $styleStr = 'style="display:none"';
        else $styleStr = '';

        echo '<td><input type="text" id="NVV_CenaPHM" name="NVV_CenaPHM" class="appInput inputFloat" value="' . StringUtils::htmlTextConvert($value) . '" ' . $styleStr . ' size="8" maxlength="8" /> ';
        echo \Ron\ronText('Kc_litr') . "</td></tr>\n";

        echo '</table>';
        echo '</div>';
        echo "</td></tr>\n";
    }


    /**
     * Zobrazeni detailu radku NVV
     * @param $values
     * @param $className
     */
    function showNVV($values, $className)
    {
        echo "<tr class='$className'>";
        echo '<td></td>';

        $colspan = count($this ->fields) - 1; // bez datumu
        echo "<td colspan='$colspan' class='poznamka browserpoznamka sc_nvv'>";
        echo \Ron\ronText('Spotreba_TP') . ': ' . StringUtils::format('number_2', $values[3]) . ' ' . \Ron\ronText('Litru_100') . ', ';
        echo "<strong>{$values[4]} " . \Ron\ronText('km') . '</strong>';

        if (!$values[6]) echo ", " . (float)$values[7] . ' ' . \Ron\ronText('Kc_litr'); // bez (float) to zobrazuje napr. 38.20000003

        echo ', ' . \Ron\ronText('Zakladni_sazba') . ': ';
        echo \Ron\ronText($values[5] ? 'Str_No' : 'Str_Yes');

        echo '</td>';

//        if ($this ->lzeEditovat && $this ->odkazyEditace) {
//            echo '<td></td><td></td><td></td>'; // doplni prazdne sloupce misto odkazu pro editaci
//        }

//        echo '<td></td>'; // dokumenty

        echo "</tr>\n";
    }


    function showRozdeleni($values, $className, $edit)
    {
        if ($edit) {
            if (!RonPortalSettings::getIntValue('typrozdeleniusc')  &&  !RonPortalSettings::getIntValue('typrozdeleniusc2')) {
                return;
            }
        } else {
            if (!$values) return;
            if (!($values[23] || $values[26])) return;
        }

        // radek pro rozdeleni (poznamka nebo editace)
        $colspan = count($this ->fields) - 1; // bez datumu

        echo "<tr class='$className'>";
        echo '<td></td>';
        echo "<td colspan='$colspan' class='poznamka browserpoznamka sc_rozdeleni'>";

        if ($edit) {
            require_once('adsapp/ronportalcontrols.php');

            // editace rozdeleni 1
            if (RonPortalSettings::getIntValue('typrozdeleniusc')) {
                $comboBoxesRozdeleni1 = new ComboBoxesRozdeleni($this->database, 'Rozdeleni', 'rozdeleniusc');
                $comboBoxesRozdeleni1 ->loadRozdeleni(RonPortalSettings::getIntValue('rozdeleniusc_pouzevlastni'),
                    $this ->sluzebniCesta ->f_PP,
                    RonPortalSettings::getIntValue('typrozdeleniusc'),
                    isset($_POST['Rozdeleni']) ? $_POST['Rozdeleni'] : ($values ? $values[23] : null), false);

                echo '<div class="row" id="rozdeleni_usc_1">';

                echo '<div class="col col-2">' . StringUtils::htmlTextConvert($comboBoxesRozdeleni1 ->typRozdeleni ->popis) . '</div> ';

                echo '<div class="col col-4">';
                $comboBoxesRozdeleni1 ->combo1 ->render();
                echo '</div>';

                echo '<div class="col col-2">';
                $comboBoxesRozdeleni1 ->combo2 ->render();
                echo '</div>';

                echo '</div>';
            }

            // editace rozdeleni 2
            if (RonPortalSettings::getIntValue('typrozdeleniusc2')) {
                $comboBoxesRozdeleni2 = new ComboBoxesRozdeleni($this->database, 'Rozdeleni2', 'rozdeleniusc2');
                $comboBoxesRozdeleni2 ->loadRozdeleni(RonPortalSettings::getIntValue('rozdeleniusc2_pouzevlastni'),
                    $this ->sluzebniCesta ->f_PP,
                    RonPortalSettings::getIntValue('typrozdeleniusc2'),
                    isset($_POST['Rozdeleni2']) ? $_POST['Rozdeleni2'] : ($values ? $values[26] : null), false);

                echo '<div class="row" id="rozdeleni_usc_2">';

                echo '<div class="col col-2">' . StringUtils::htmlTextConvert($comboBoxesRozdeleni2 ->typRozdeleni ->popis) . '</div> ';

                echo '<div class="col col-4">';
                $comboBoxesRozdeleni2 ->combo1 ->render();
                echo '</div>';

                echo '<div class="col col-2">';
                $comboBoxesRozdeleni2 ->combo2 ->render();
                echo '</div>';

                echo '</div>';
            }
        } else {
            // rozdeleni 1
            if ($values[23]) {
                echo '<div>' . StringUtils::htmlTextConvert("{$values[25]}: {$values[24]} ({$values[23]})") . '</div>';
            }
            // rozdeleni 2
            if ($values[26]) {
                echo '<div>' . StringUtils::htmlTextConvert("{$values[28]}: {$values[27]} ({$values[26]})") . '</div>';
            }
        }

        echo '</td>';

        echo "</tr>\n";
    }


    function getRowAttributes($values, $className, $rowNumber)
    {
        $className .= ' ' . 'typradkuuctovani_' . $values[2];
        return parent::getRowAttributes($values, $className, $rowNumber);
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        if ($values) {
            $edit = $this ->editovatRadek == $values[1]
                && ($values[2] <= 0 || $values[2] >= 3) // nelze editovat zaznamy s vyuctovanim
                && (TypRadkuUctovaniSluzebniCesty::lzeEditovatRadekUctovani($this ->sluzebniCesta ->f_PP ->idPP, $values[2])); // prava
        } else {
            // novy radek
            $edit = true;
        }

        if ($edit) $className .= ' editing';
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($edit) {
            $this ->showEditaceNahrad($values, $className);
        } else {
            // zobrazeni detailu nahrad v needitovatelnem rezimu
            //if (TypUctovaniSC_Je_NVV($connection, $values[2])) {
            if (TypRadkuUctovaniSluzebniCesty::typUctovaniJeNVV($this ->database, $values[2])) {
                $this ->showNVV($values, $className);
            }
        }

        $this ->showRozdeleni($values, $className, $edit);

        // kopie aktualniho radku => zobrazi jej jeste jednou s nullovym cislem
        if ($values && $this ->kopirovatRadek == $values[1]) {
            $values[1] = null;
            $this ->renderLine($values, '', 0, true);
        }
    }


    function renderCursor($cursor)
    {
        parent::renderCursor($cursor);

        // prida navic jeden novy radek pro editaci
        if ($this ->zobrazitNovyRadek) {
            $this ->renderLine(null, '', 0, true); // edit = true
        }
    }


    /**
     * @param Database $database
     * @param SluzebniCesta $sc
     * @param int $cisloRadku
     * @param object|string $logDescription
     * @throws DBException
     * @throws NeniPravoException
     * @throws \Exception
     */
    static function zrusUctovaniSluzebniCesty($database, $sc, $cisloRadku, $logDescription)
    {
        $rec = new RadekUctovaniSCDBRecord($database, $sc ->id, (int)$cisloRadku, $logDescription);
        if ($rec->loadRecord()) {
            // kontrola prav
            if (!TypRadkuUctovaniSluzebniCesty::lzeEditovatRadekUctovani($sc ->f_PP ->idPP, $rec ->fTyp -> originalValue)) {
                throw new NeniPravoException();
            }

            $rec ->doDelete('');

            MessageManager::addInfoMessage(\Ron\ronText('SC_Zruseni_radku_OK'));
            return;
        }

        throw new DBException($database, null, 'Pri mazani radku uctovani sluzebni cesty doslo k chybe!');
    }


    /**
     * Zkontroluje zadana data. V pripade chyby zapise chybove hlaseni a vraci false
     * Pokud je vse spravne, tak vraci zadany cas (timestamp)
     * @param bool $jeNahrada
     * @param SluzebniCesta $sc
     * @return bool|int
     * @throws \Exception
     */
    static function zkontrolujUctovaniSluzebky($jeNahrada, $sc)
    {
        try {
            $datum = DateUtils::dateStringToTimestamp($_POST['Den']);
        } catch (\Exception $e) {
            return MessageManager::addErrorMessage( $e ->getMessage(), 'Den' );
        }

        if (!RonPortalSettings::getIntValue('datumuctovani_mimosc')) {
            // Kontrola, jestli je zadane datum v rozsahu datumu sluzebni cesty
            if ($datum < DateUtils::startOfTheDay($sc->datumCasOd) || $datum > $sc->datumCasDo) {
                return MessageManager::addErrorMessage(
                    sprintf(\Ron\ronText('Datum_musi_byt_v_rozsahu'), StringUtils::format('date', $sc->datumCasOd), StringUtils::format('date', $sc->datumCasDo)));
            }
        }

        if (!isset($_POST['Typ']) || trim($_POST['Typ']) == '') {
            return MessageManager::addErrorMessage(\Ron\ronText('Neni_zadan_typ'));
        }

        if ($jeNahrada) {
            $OK = true;

            // nahrada za sluzebni vozidlo - musi byt vyplnene dalsi hodnoty
            if (!$_POST['NVV_SpotrebaVozidla']) {
                $OK = MessageManager::addErrorMessage(\Ron\ronText('form.Missing_value') . \Ron\ronText('Spotreba_TP'));
            }

            if (!$_POST['NVV_PocetUjetychKM']) {
                $OK = MessageManager::addErrorMessage(\Ron\ronText('form.Missing_value') . \Ron\ronText('Pocet_km'));
            }

            if ($_POST['NVV_NeprokazanaCenaPHM']) {
                // neprokaze cenu PHM => musi vybrat druh PHM
                if (!$_POST['NVV_DruhPHM']) {
                    $OK = MessageManager::addErrorMessage(\Ron\ronText('form.Missing_value') . \Ron\ronText('Druh_PHM'));
                }
            } else {
                // prokaze cenu PHM => musi zadat cenu
                if (!$_POST['NVV_CenaPHM']) {
                    $OK = MessageManager::addErrorMessage(\Ron\ronText('form.Missing_value') . \Ron\ronText('Cena_PHM'));
                }
            }

            if (!$OK) return false;
        } else {
            // neni to nahrada za soukrome vozidlo - musi byt zadana castka
            if (!$_POST['Castka']) {
                return MessageManager::addErrorMessage(\Ron\ronText('Neni_zadana_castka'));
            } else {
                if (!StringUtils::isFloat($_POST['Castka'])) {
                    return MessageManager::addErrorMessage(\Ron\ronText('Spatna_hodnota_pro') . \Ron\ronText('Castka'));
                }
            }
        }

        if (!$_POST['KodMeny']) {
            return MessageManager::addErrorMessage(\Ron\ronText('Neni_zadana_mena'));
        }

        if (isset($_POST['DM_VlastniKurz']) && $_POST['DM_VlastniKurz']) {
            // zkontroluje zadane kurzy
            if (!StringUtils::isFloat($_POST['DM_Kurz_Mnozstvi'])) {
                return MessageManager::addErrorMessage(\Ron\ronText('Neni_zadano_mnozstvi_pro_kurz'));
            }

            if (!StringUtils::isFloat($_POST['DM_Kurz'])) {
                return MessageManager::addErrorMessage(\Ron\ronText('Neni_zadan_kurz'));
            }
        }

        return $datum;
    }


    /**
     * @param Database $database
     * @param RadekUctovaniSCDBRecord $rec
     * @param SluzebniCesta $sc
     * @return bool
     * @throws DBException
     * @throws \Exception
     */
    private static function loadPostToRecord($database, $rec, $sc)
    {
        $typUctovani = TypRadkuUctovaniSluzebniCesty::vratTypUctovani($database, $_POST['Typ']);
        $jenahrada = $typUctovani ? $typUctovani ->nahrada : false;

        $datum = self::zkontrolujUctovaniSluzebky($jenahrada, $sc);
        if (!$datum) return false;

        $castka = StringUtils::stringToFloat($_POST['Castka']);
        if ($typUctovani && $typUctovani ->druhPohybu == 1) $castka = abs($castka); // prijem
        else if ($typUctovani && $typUctovani ->druhPohybu == 2) $castka = -abs($castka); // vydej

        $rec ->fPopis ->setNewValue($_POST['Popis']);
        $rec ->fDen ->setNewValue($datum);
        $rec ->fKodMeny ->setNewValue($_POST['KodMeny']);
        $rec ->fTyp ->setNewValue($_POST['Typ']);
        $rec ->fZpusobPlatby ->setNewValue((int)$_POST['ZpusobPlatby']);

        if ($jenahrada) {
            $rec ->fCastka ->setNewValue(0); // nastavi na 0, musi se preuctovat!
            $rec ->fNVV_SpotrebaVozidla ->setNewValue(StringUtils::stringToFloat($_POST['NVV_SpotrebaVozidla']));
            $rec ->fNVV_PocetUjetychKM ->setNewValue((int)$_POST['NVV_PocetUjetychKM']);
            if (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_NEPOCITAT_ZS_NEMENIT)) {
                $rec ->fNVV_NepocitatZaklSazbu ->setNewValue($_POST['NVV_NepocitatZaklSazbu'] ? 1 : 0);
            }
            $rec ->fNVV_NeprokazanaCenaPHM ->setNewValue($_POST['NVV_NeprokazanaCenaPHM'] ? 1 : 0);

            if ($_POST['NVV_NeprokazanaCenaPHM']) {
                $rec->fNVV_DruhPHM ->setNewValue($_POST['NVV_DruhPHM']);
            } else {
                $rec->fNVV_CenaPHM ->setNewValue(StringUtils::stringToFloat($_POST['NVV_CenaPHM']));
            }

        } else {
            $rec ->fCastka ->setNewValue($castka);
            if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DPH)) {
                $rec ->fCastkaDPH ->setNewValue(StringUtils::stringToFloat($_POST['CastkaDPH']));
            }
        }

        if (RonPortalSettings::getIntValue('vlastnikurzradkusc')) {
            $vlastniKurz = isset($_POST['DM_VlastniKurz']) ? ($_POST['DM_VlastniKurz'] ? 1 : 0) : 0;
            $rec ->fDM_VlastniKurz ->setNewValue($vlastniKurz);

            if ($vlastniKurz) {
                $rec ->fDM_Kurz_Mnozstvi ->setNewValue( $_POST['DM_Kurz_Mnozstvi'] == '' ? null : StringUtils::stringToFloat($_POST['DM_Kurz_Mnozstvi']) );
                $rec ->fDM_Kurz ->setNewValue( $_POST['DM_Kurz'] == '' ? null : StringUtils::stringToFloat($_POST['DM_Kurz']) );
            }
        }

        if (RonPortalSettings::getIntValue('typrozdeleniusc')) {
            $rozdeleni = (isset($_POST['Rozdeleni'])  &&  $_POST['Rozdeleni'] != '') ?  $_POST['Rozdeleni'] : null;
            $rec ->fKodTypuRozdeleni ->setNewValue($rozdeleni ? RonPortalSettings::getIntValue('typrozdeleniusc') : null);
            $rec ->fRozdeleni ->setNewValue($rozdeleni ?: null);
        }

        if (RonPortalSettings::getIntValue('typrozdeleniusc2')) {
            $rozdeleni2 = (isset($_POST['Rozdeleni2'])  &&  $_POST['Rozdeleni2'] != '') ?  $_POST['Rozdeleni2'] : null;
            $rec ->fKodTypuRozdeleni2 ->setNewValue($rozdeleni2 ? RonPortalSettings::getIntValue('typrozdeleniusc2') : null);
            $rec ->fRozdeleni2 ->setNewValue($rozdeleni2 ?: null);
        }


        return true;
    }


    /**
     * @param Database $database
     * @param SluzebniCesta $sc
     * @param object|string $logDescription
     * @return bool
     * @throws DBException
     * @throws NeniPravoException
     * @throws \Exception
     */
    static function ulozUctovaniSluzebniCesty($database, $sc, $logDescription)
    {
        // TODO: zkontrolovat, jestli se nezmenil typ z nepovoleneho na povoleny?
        $typ = (int)$_POST['Typ'];
        if (!TypRadkuUctovaniSluzebniCesty::lzeEditovatRadekUctovani($sc ->f_PP ->idPP, $typ)) {
            RonPortalUtils::neniPravoException();
        }

        $rec = new RadekUctovaniSCDBRecord($database, (int)$_POST['SluzebniCesta_ID'], (int)$_POST['CisloRadku'], $logDescription);

        if ( !self::loadPostToRecord($database, $rec, $sc) ) {
            return false;
        }

        if (!$rec ->loadRecord()) {
            throw new \Exception('Chyba pri aktualizaci radku uctovani sluzebni cesty - radek nebyl nalezen!');
        }

        return $rec ->updateRecord();
    }


    /**
     * @param Database $database
     * @param SluzebniCesta $sc
     * @param object|string $logDescription
     * @return bool
     * @throws NeniPravoException
     */
    static function vlozNoveUctovaniSlCesty($database, $sc, $logDescription)
    {
        $typ = (int)$_POST['Typ'];
        if (!TypRadkuUctovaniSluzebniCesty::lzeEditovatRadekUctovani($sc ->f_PP ->idPP, $typ)) {
            RonPortalUtils::neniPravoException();
        }

        $rec = new RadekUctovaniSCDBRecord($database, (int)$_POST['SluzebniCesta_ID'], null, $logDescription);

        if ( !self::loadPostToRecord($database, $rec, $sc) ) {
            return false;
        }

        $rec ->fCisloRadku ->setNewValue( SQLUtils::newSerialNumber($database, 'RadekUctovaniSCesty', 'CisloRadku', 'SluzebniCesta_ID = ' . $_POST['SluzebniCesta_ID']) );

        $rec ->insertRecord();
        return true;
    }


    /**
     * vypise javascript - promenna se seznamem radku, ktere jsou NVV
     * @param Database $database
     * @throws DBException
     */
    private function typUctovaniSC_NVV_Script($database)
    {
        $sql = 'SELECT Kod, DruhPohybu, NahradaZaVlastniVozidlo FROM TypRUctovaniSlCesty WHERE Aktivni = 1';

        $nahrady = [];
        $prijmy = '';
        $vydaje = '';

        $cursor = $database ->select($sql);

        while ($row = $cursor ->fetch()) {
            if ($row[2]) {
                // je nahrada
                $nahrady[] .= $row[0];
            }

            if ($row[1] == 1) {
                // prijem
                if ($prijmy) $prijmy .= ', ';
                $prijmy .= $row[0];
            } else if ($row[1] == 2) {
                // vydej
                if ($vydaje) $vydaje .= ', ';
                $vydaje .= $row[0];
            }
        }

        $cursor ->close();

        echo '<script language="Javascript">';

        //echo '   TypUctovaniSC_NVV = [' . $nahrady . '];';
        echo '   TypUctovaniSC_NVV = ' . json_encode($nahrady) . ';';
        echo '   TypUctovaniSC_Prijmy = [' . $prijmy . '];';
        echo '   TypUctovaniSC_Vydaje = [' . $vydaje . '];';

        $data = [
            'idSC' => $this ->idSluzebniCesty,
            'typUctovaniSC_NVV' => $nahrady
        ];
        echo 'ron.sc.seznamUctovaniSC(' . json_encode($data) . ')';

        echo "</script>\n";
    }


    /**
     * @param Database $database
     * @param SluzebniCesta $sc
     * @param string $akce
     */
    public static function zobrazUctovani($database, $sc, $akce)
    {
        $startStr = '';
        $cilStr = '';

        switch (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_VYPOCET_VZDALENOSTI)) {
            case 1 : // prednostne podle lookupu
                $startStr = $sc ->mistoZacatkuPopis; // lookup
                if (!$startStr) $startStr = $sc ->popisZacatku;

                $cilStr = $sc ->cilPopis; // lookup
                if (!$cilStr) $cilStr = $sc ->popisCile;
                break;

            case 2 : // prednostne podle popisu
                $startStr = $sc ->popisZacatku;
                if (!$startStr) $startStr = $sc ->mistoZacatkuPopis; // lookup

                $cilStr = $sc ->popisCile;
                if (!$cilStr) $cilStr = $sc ->cilPopis; // lookup
                break;
        }

        $b = new RadekUctovaniSlCestyBrowser($database, $sc, $akce);
        $b ->idSluzebniCesty = $sc ->id;
        $b ->f_vzdalenost_start = $startStr;
        $b ->f_vzdalenost_cil = $cilStr;
        $b ->f_vzdalenost_pres = $sc ->pres;
        $b ->f_datum = DateUtils::startOfTheDay($sc ->datumCasOd);

        $formaction = '';

        if ($b ->lzeEditovat) {
            switch ($akce) {
                case 'editovatuctovanisluzebky' :
                    $formaction = 'ulozuctovanisluzebky';
                    if (isset($_POST['CisloRadku'])) $b ->editovatRadek = $_POST['CisloRadku'];
                    else $b ->editovatRadek = $_GET['radek'];
                    break;

                case 'kopirovatuctovanisluzebky' :
                    $formaction = 'vlozuctovanisluzebky';
                    //if ( isset($_POST['KopirovatRadek']) ) $b -> f_kopirovatradek = $_POST['KopirovatRadek'];
                    //else $b -> f_kopirovatradek = $_GET['radek'];
                    $b ->kopirovatRadek = $_GET['radek'];
                    break;


                case 'pridatuctovanisluzebky' :
                    $formaction = 'vlozuctovanisluzebky';
                    $b ->zobrazitNovyRadek = true;
                    break;
            }
        }

        $b ->load("RadekUctovaniSCesty.SluzebniCesta_ID = {$sc ->id}");

        if ($formaction && $b ->lzeEditovat) {
            echo '<FORM method="post" class="sendPosition" name="radky_uctovanisc_form" action="?menuid=sluzebnicesty&action=' . $formaction . '">' . "\n";
            echo '<INPUT type="hidden" name="x"><INPUT type="hidden" name="y">'; // pro zapamatovani scrollovani v prohlizeci
            echo "\n";
        }

        $b ->render();

        if ($formaction && $b ->lzeEditovat) {
            echo '<INPUT type="submit" class="submitbutton" value="' . \Ron\ronText('form.Store') . '">';

            $href = '?menuid=sluzebnicesty&action=zobrazslcestu&record=' . $sc ->id;
            echo '<INPUT type="button" class="button clickHref" value="' . \Ron\ronText('Cancel') . '" data-ron-href="' . $href . '" data-ron-send-position="1">';

            echo "</FORM>\n";
        }

        $b ->typUctovaniSC_NVV_Script($database);

        if (!$formaction && $b ->lzeEditovat) {
            /** @noinspection HtmlUnknownAnchorTarget */
            echo "<br/><div class='text-link'><a class='sendPosition' href='?menuid=sluzebnicesty&action=pridatuctovanisluzebky&record={$sc ->id}'>" . \Ron\ronText('Novy_radek') . "</a></div>\n";

            if (RonPortalSettings::getSpousteniADS() ) {
                echo "<div id='link_vyuctovani_sc'><a class='sendPosition' href='?menuid=sluzebnicesty&action=vyuctovatsluzebky&record={$sc ->id}'>" . \Ron\ronText('Vyuctovani_sc') . "</a></div>\n";
            }
        }
    }

}

