<?php
/**
 * UTF8
 * stanislav.elbl, 20.08.2015
 */

namespace Ron\RonPortal;

require_once('adsapp/ronportaldbrecord.php');

use Ron\Controls\Option;
use Ron\DB\Database;
use Ron\DBRecordField;
use Ron\StringUtils;


class TypRadkuUctovaniSluzebniCesty
{
    var $kod;
    var $druhPohybu;
    var $nahrada;

    /**
     * @param Database $database
     * @param string $kod
     * @return null|TypRadkuUctovaniSluzebniCesty
     */
    public static function vratTypUctovani($database, $kod)
    {
        if (!$kod) return null;

        $sql =
            'SELECT Kod, DruhPohybu, NahradaZaVlastniVozidlo FROM TypRUctovaniSlCesty WHERE Kod = ' . (int)$kod;

        $cursor = $database ->getSqlResult($sql);
        $row = $cursor ->fetch();
        if (!$row) return null;

        $result = new TypRadkuUctovaniSluzebniCesty();
        $result ->kod = $row[0];
        $result ->druhPohybu = $row[1];
        $result ->nahrada = $row[2];

        return $result;
    }


    public static function typUctovaniJeNVV($database, $kod)
    {
        $typRadku = self::vratTypUctovani($database, $kod);

        if (!$typRadku) return null;

        return $typRadku ->nahrada == 1;
    }


    static function lzeEditovatRadekUctovani($idPP, $typRadku)
    {
        $prava = Prava::vratPrava('slcesty_ucet', 'slcesty_ucet_' . $typRadku);
        return $prava ->lzeZobrazitPP($idPP);
    }


    /**
     * Nacte seznam typu uctovani, ktere muze uzivatel editovat pro zadany idPP
     * @param Database $database
     * @param int $idPP
     * @return Option[]
     */
    public static function loadOptions($database, $idPP)
    {
        $result = [new Option('', '')];

        $cursor = $database ->getSqlResult('SELECT Kod, Popis FROM TypRUctovaniSlCesty WHERE (Aktivni=1) AND (NOT (Kod IN (1,2,3,4))) ORDER BY Popis');
        while ($row = $cursor ->fetch()) {
            $pravo = Prava::vratPrava('slcesty_ucet', 'slcesty_ucet_' . $row[0]);
            if ($pravo ->lzeZobrazitPP($idPP)) {
                $result[] = new Option($row[0], $row[1]);
            }
        }

        return $result;
    }
}



class RadekUctovaniSluzebniCesty
{
    var $idSluzebniCesty;

    var $cisloRadku;
    var $popis;
    var $den;

    var $typ; // => TypRUctovaniSlCesty
    // nacteno z typu
    var $popisTypu;
    var $druhPohybu; // +/-

    var $castka;
    var $castkaStr;
    var $kodMeny;
    var $zpusobPlatby;
    var $castkaDPH;
    var $castkaDPHStr;

    var $nahrada;   // priznak, ze se jedna o nahradu
    var $NVVSpotrebaVozidla;
    var $NVVPocetUjetychKm;
    var $NVVNepocitatZakladniSazbu;
    var $NVVNeprokazanaCenaPHM;
    var $NVVCenaPHM;
    var $NVVDruhPHM;
    var $NVVVozidlo; // => VozidloOsoby, Vozidlo

    var $error;     // priznak chyby

    var $novyRadek; // priznak, ze se jedna o novy radek


    function __construct($idSluzebniCesty)
    {
        $this ->idSluzebniCesty = $idSluzebniCesty;
        $this ->error = false;

        $this ->castkaDPH = 0;
    }


    function getCastkaView() {
        if ( !$this->castka ) return '';

        switch ($this ->druhPohybu) {
            case 1 :
            case 2 :
                if (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZNAMENKO_UCTOVANI)) {
                    // je zadan druh pohybu (+/-) - vzdy zobrazi kladne
                    return abs($this->castka);
                }
        }

        return $this ->castka;
    }


    /**
     * @param Database $database
     * @param int $idSluzebniCesty
     * @return null|string
     */
    public static function vratVychoziMenu($database, $idSluzebniCesty)
    {
        $params = $database ->createParameters();

        // podle ciloveho statu
        $sql =
            'SELECT Stat.MenaStatu ' .
            ' FROM SluzebniCesta, Stat ' .
            ' WHERE SluzebniCesta.ID = ' . $params ->addParameter((int)$idSluzebniCesty) .
            '   AND Stat.Kod = SluzebniCesta.CilovyStat ';

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();
        if ($row) return $row[0];

        // podle cile
        $params = $database ->createParameters();
        $sql =
            'SELECT Stat.MenaStatu ' .
            ' FROM SluzebniCesta, MistoSluzebniCesty, Stat ' .
            ' WHERE SluzebniCesta.ID = ' . $params ->addParameter((int)$idSluzebniCesty) .
            '   AND MistoSluzebniCesty.Kod = SluzebniCesta.Cil ' .
            '   AND Stat.Kod = MistoSluzebniCesty.Stat ';

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();
        if ($row) return $row[0];

        // prednastavena vychozi mena
        if ( RonPortalSettings::getValue(RonPortalSettings::VYCHOZI_MENA) ) {
            return RonPortalSettings::getValue(RonPortalSettings::VYCHOZI_MENA);
        }

        return '';
    }

}


class RadekUctovaniSCDBRecord extends RonPortalDBRecord
{
    /** @var  DBRecordField */
    public $fCisloRadku;
    /** @var  DBRecordField */
    public $fDen;
    /** @var  DBRecordField */
    public $fPopis;
    /** @var  DBRecordField */
    public $fKodMeny;
    /** @var  DBRecordField */
    public $fTyp;
    /** @var  DBRecordField */
    public $fZpusobPlatby;
    /** @var  DBRecordField */
    public $fCastka;

    /** @var  DBRecordField */
    public $fCastkaDPH;
    /** @var  DBRecordField */
    public $fNVV_SpotrebaVozidla;
    /** @var  DBRecordField */
    public $fNVV_PocetUjetychKM;
    /** @var  DBRecordField */
    public $fNVV_NepocitatZaklSazbu;
    /** @var  DBRecordField */
    public $fNVV_NeprokazanaCenaPHM;
    /** @var  DBRecordField */
    public $fNVV_DruhPHM;
    /** @var  DBRecordField */
    public $fNVV_CenaPHM;
    /** @var  DBRecordField */
    public $fNVV_Vozidlo;

    /** @var  DBRecordField */
    public $fDM_VlastniKurz;
    /** @var  DBRecordField */
    public $fDM_Kurz_Mnozstvi;
    /** @var  DBRecordField */
    public $fDM_Kurz;

    /** @var DBRecordField */
    public $fKodTypuRozdeleni;
    /** @var DBRecordField */
    public $fRozdeleni;
    /** @var DBRecordField */
    public $fKodTypuRozdeleni2;
    /** @var DBRecordField */
    public $fRozdeleni2;

    /**
     * @param Database $database
     * @param int $idSC
     * @param int $cisloRadku
     * @param object|string $logDescription
     */
    function __construct($database, $idSC, $cisloRadku = null, $logDescription = 'Sluzebni cesty')
    {
        parent::__construct($database, 'RadekUctovaniSCesty', $logDescription);

        require_once 'adsapp/events.php';
        $this ->beforePostEvent = RonPortalXMLEvents::SC_UCTOVANI_BEFORE_POST;
        $this ->afterPostEvent = RonPortalXMLEvents::SC_UCTOVANI_AFTER_POST;

        $this ->addField('SluzebniCesta_ID', 'int', true, 0, $idSC); // readonly = true
        $this ->fCisloRadku = $this ->addField('CisloRadku', 'int', true, 0, $cisloRadku);

        $this ->fDen = $this ->addField('Den', 'date', true);
        $this ->fPopis = $this ->addField('Popis', '', true, 160);
        $this ->fKodMeny = $this ->addField('KodMeny', '', true, 10);
        $this ->fTyp = $this ->addField('Typ', 'int', true);
        $this ->fZpusobPlatby = $this ->addField('ZpusobPlatby', 'int', true, 0, 0);

        $this ->fNVV_Vozidlo = $this ->addField('NVV_Vozidlo', '', true, 30);

        $this ->fCastka = $this ->addField('Castka', 'float', true, 0, 0);
        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DPH)) {
            $this ->fCastkaDPH = $this ->addField('CastkaDPH', 'float', true, 0, 0);
        }

        // nahrada
        $this ->fNVV_SpotrebaVozidla = $this ->addField('NVV_SpotrebaVozidla', 'float', true);
        $this ->fNVV_PocetUjetychKM = $this ->addField('NVV_PocetUjetychKM', 'int', true);
        $this ->fNVV_NepocitatZaklSazbu = $this ->addField('NVV_NepocitatZaklSazbu', 'int', true, 0, RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_NEPOCITAT_ZAKLADNI_SAZBU));

        $this ->fNVV_NeprokazanaCenaPHM = $this ->addField('NVV_NeprokazanaCenaPHM', 'int', true, 0, 0);
        $this ->fNVV_DruhPHM = $this ->addField('NVV_DruhPHM', '', true, 10);
        $this ->fNVV_CenaPHM = $this ->addField('NVV_CenaPHM', 'float', true);

        if (RonPortalSettings::getIntValue('vlastnikurzradkusc')) {
            $this ->fDM_VlastniKurz = $this ->addField('DM_VlastniKurz', 'int', true, 0, 0);
            $this ->fDM_Kurz_Mnozstvi = $this ->addField('DM_Kurz_Mnozstvi', 'float', true);
            $this ->fDM_Kurz = $this ->addField('DM_Kurz', 'float', true);
        }

        if (RonPortalSettings::getIntValue('typrozdeleniusc')) {
            $this->fKodTypuRozdeleni = $this ->addField('KodTypuRozdeleni', 'int', true);
            $this->fRozdeleni = $this->addField('Rozdeleni', '', true, 30);
        }
        if (RonPortalSettings::getIntValue('typrozdeleniusc2')) {
            $this->fKodTypuRozdeleni2 = $this ->addField('KodTypuRozdeleni2', 'int', true);
            $this->fRozdeleni2 = $this->addField('Rozdeleni2', '', true, 30);
        }

        $params = $database ->createParameters();
        $where =
            ' SluzebniCesta_ID = ' . $params ->addParameter((int)$idSC);

        if ($cisloRadku) {
            $where .= ' AND CisloRadku = ' . $params ->addParameter($cisloRadku);
        }

        $this ->setWhere($where, $params);
    }

}

