<?php
/**
 * UTF8
 * stanislav.elbl, 30.11.2016
 */

namespace Ron\RonPortal;

use Ron\DateUtils;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\StringUtils;
use Ron\TextManager;


class SluzebniCestyJsController extends JsonController
{

    /** @var  \Ron\DB\Database */
    protected $database;


    public function __construct($database)
    {
        $this->database = $database;
    }


    /**
     * @param JsonOutputObject $output
     */
    function vyberSpolucestujich($output)
    {
        require_once('adsapp/ppomer.php');

        Prava::nastavObdobiProPrava(time(), time());
        Prava::nactiPravaUzivatele($this->database, User::getRC(), 'slcesty_spolu');

        $filter = '(PracovniPomer.DatumUkonceni IS NULL  OR  PracovniPomer.DatumUkonceni >= ' . $this ->database ->dateToString(time()) . ')';
        $seznamPP = PracovniPomer::nactiSeznamPP($this->database, $filter, Prava::vratPrava('slcesty_spolu'), time(), time());
        $seznamJs = [];
        foreach ($seznamPP as $pp) {
            $seznamJs[] = $pp->getJsonData();
        }

        $output->addData('pp', $seznamJs);
    }


    /**
     * Pridani spolucestujiciho ke sluzebni ceste action=pridejspolucestujiciho&idsc=<id_sluzebni_cesty>&idpp=<id_PP>
     * @param JsonOutputObject $output
     */
    function pridejSpolucestujiciho($output)
    {
        require_once 'adsapp/ppomer.php';
        require_once 'adsapp/ronportalutils.php';

        Prava::nastavObdobiProPrava(time(), time());
        Prava::nactiPravaUzivatele($this->database, User::getRC(), 'slcesty_spolu');
        $prava = Prava::vratPrava('slcesty_spolu');

        // zkontrolovat pravo na vybrany PP
        $idPP = (int)$_GET['idpp'];
        if (!$prava->lzeZobrazitPP($idPP, time(), time())) {
            RonPortalUtils::neniPravoException();
        }

        // zkontrolovat pravo na sluzebni cestu, schvaleni, zauctovani
        $idSC = (int)$_GET['idsc'];
        require_once 'sluzebnicesty/sluzebnicesty.php';
        $sc = SluzebniCesta::nactiSluzebniCestu($this->database, $idSC);
        $sc->lzeEditovat($this->database, true);

        $pp = PracovniPomer::nactiPP($this->database, $idPP);

        require_once 'adsapp/ronportaldbrecord.php';
        $rec = new RonPortalDBRecord($this->database, 'Spolucestujici');
        $rec->addField('SluzebniCesta_ID', 'int', true, 0, $idSC);
        $rec->addField('RC', '', true, 15, $pp->rc);
        $rec->addField('OsobniCislo', '', true, 15, $pp->osobniCislo);
        $rec->addField('DatumNastupu', 'date', true, 0, $pp->datumNastupu);
        $rec->addField('', '', true, 0, '   ' . $pp->jmenoPrijmeniKratke);

        $params = $this ->database ->createParameters();
        $rec->setWhere(
            'SluzebniCesta_ID = ' . $params ->addParameter($idSC) .
            ' AND RC = ' . $params->addParameter($pp->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this->database->timestampToParam($pp->datumNastupu)),
            $params);

        if (!$rec->loadRecord()) {
            $rec->insertRecord();
        }
    }


    /**
     * Zkontroluje, jestli zadany PP (spolucestujici) nema sluzebni cestu prekryvajici se se zadanou SC
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function overSpolucestujiciho($output)
    {
        Prava::nastavObdobiProPrava(time(), time());
        Prava::nactiPravaUzivatele($this->database, User::getRC(), 'slcesty_spolu');
        $prava = Prava::vratPrava('slcesty_spolu');

        $idPP = (int)$_GET['idpp'];
        if (!$prava->lzeZobrazitPP($idPP, time(), time())) {
            RonPortalUtils::neniPravoException();
        }

        $idSC = (int)$_GET['idsc'];
        require_once('sluzebnicesty/sluzebnicesty.php');
        $sc = SluzebniCesta::nactiSluzebniCestu($this->database, $idSC);

        $pp = PracovniPomer::nactiPP($this->database, $idPP);

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT DatumCasOd, DatumCasDo FROM SluzebniCesta' .
            ' WHERE RC = ' . $params ->addParameter($pp ->rc) . // kontroluje jen RC - predpokladam, ze jedna osoba nemuze na ruzne SC
            ' AND DatumCasOd < ' . $params ->addParameter($this ->database ->timestampToParam($sc ->datumCasDo)) .
            ' AND DatumCasDo > ' . $params ->addParameter($this ->database ->timestampToParam($sc ->datumCasOd)) .
            ' ORDER BY DatumCasOd';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        if ($row = $cursor ->fetch()) {
            $textSc =
                StringUtils::format('datetime', $cursor ->stringToTimestamp($row[0])) .
                ' - ' .
                StringUtils::format('datetime', $cursor ->stringToTimestamp($row[1]));

            $output ->addText('infoText', sprintf(\Ron\ronText('Spolucestujici_jiz_ma_cestu'), $textSc) );
            $output ->setResult('warning');
        }
        $cursor ->close();
    }


    /**
     * Vraci max. stravne pro zadany stat
     * @param JsonOutputObject $output
     */
    function maximalniDenniStravneProStat($output)
    {
        // TODO: nefunguje, pokud se urcuje vychozi stat podle rozdeleni
        $cilovyStat = $_GET['cilovystat'];
        $vychoziStat = $_GET['vychozistat'];

        require_once 'sluzebnicesty/sluzebnicesty.php';
        if (RonPortalSettings::getValue(SluzebniCesta::SETTINGS_STAT)) {
            $vychoziStat = RonPortalSettings::getValue(SluzebniCesta::SETTINGS_STAT);
        }

        if (!$cilovyStat) {
            $output->addData('text', '');
            return;
        }

        try {
            $datum = DateUtils::dateStringToTimestamp($_GET['datum']);
        } catch (\Exception $e) {
            $output->addError($e->getMessage(), 'DatumCasOd');
            return;
        }

        require_once('sluzebnicesty/sc_utils.php');
        $poma = SluzebniCestyUtils::vratMaximalniDenniStravneProStat($this->database, $datum, $cilovyStat, $vychoziStat);
        $output->addText('mena', $poma['mena']);
        $output->addText('text', StringUtils::format('number_2', $poma['sazba']) . ' ' . $poma['mena']);
        $output->addData('sazba', $poma['sazba']);
    }


    /**
     * @param JsonOutputObject $output
     */
    function statProMistoSC($output)
    {
        $misto = StringUtils::fromUTF8($_GET['kod']);
        $params = $this ->database ->createParameters();
        $sql = 'SELECT Stat FROM MistoSluzebniCesty WHERE Kod = ' . $params ->addParameter($misto);
        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        if ($row) $output ->addData('text', StringUtils::jsonText($row[0]));
    }


    /**
     * @param JsonOutputObject $output
     */
    function kapesneProStat($output)
    {
        $stat = StringUtils::fromUTF8($_GET['stat']);
        $params = $this ->database ->createParameters();
        $sql = "SELECT ImplicitniSazbaKapesneho FROM Stat WHERE Kod = " . $params ->addParameter($stat);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();
        if ($row) $output ->addData('text', StringUtils::format('float', $row[0]));
    }


    /**
     * @param JsonOutputObject $output
     */
    function adresaRozdeleni($output)
    {
        $kod = StringUtils::fromUTF8($_GET['kod']);
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Stat, Ulice, Mesto, PSC FROM Rozdeleni' .
            ' WHERE Kod = ' . $params ->addParameter($kod) .
            ' AND KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue('typrozdelenisc'));
        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        if ($row) {
            $output ->addData('stat', StringUtils::jsonText($row[0]));
            $output ->addData('ulice', StringUtils::jsonText($row[1]));
            $output ->addData('mesto', StringUtils::jsonText($row[2]));
            $output ->addData('psc', StringUtils::jsonText($row[3]));
        }
    }


    /**
     * Obslouzi vyber rozdeleni osoby (shiftform)
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    public function rozdeleniOsoby($output)
    {
        if ( !RonPortalSettings::getIntValue('rozdelenisc_povolitprirazeni')  ||  !RonPortalSettings::getIntValue('typrozdelenisc')) {
            throw new \Exception('Chyba! Tato funkce neni povolena!');
        }

        Prava::nastavObdobiProPrava(time(), time());

        require_once('adsapp/rozdelenishiftform.php');

        $shiftForm = new RozdeleniShiftForm($this ->database, RonPortalSettings::getIntValue('typrozdelenisc'), Prava::vratPrava('sluzebnicesty'));
        $data = $shiftForm ->handleAction();

        $output ->addData('data', $data);
    }


    function dokumentUpload()
    {
        require_once 'adsapp/dokumentyutils.php';
        require_once 'sluzebnicesty/sluzebnicesty.php';


        //$result = new DokumentUploadOutput();

        //try {
            //nactiPravaUzivatele($this ->database, S_GetUserRC());
            $idSluzebniCesty = (int)$_GET['idslcesty'];

            $cesta = SluzebniCesta::nactiSluzebniCestu($this->database, $idSluzebniCesty);

            Prava::nastavObdobiProPrava($cesta->datumCasDo, $cesta->datumCasDo);

            if (!$cesta->lzeEditovatSluzebniCestu($this ->database, true)) {
                RonPortalUtils::neniPravoException();
            }

            if (!is_uploaded_file($_FILES['soubor']['tmp_name']) && $_POST['pripojitdokument']) {
                // jen pripojeni existujiciho dokumentu
                $id = $_POST['pripojitdokument'];
            } else {
                // vlozi do databaze obsah dokumentu
                $id = DokumentyUtils::vlozDokument($this ->database, 'soubor', (int)$_POST['typ'], StringUtils::fromUTF8($_POST['popis']), $this);
            }

            $cisloRadku = isset($_POST['CisloRadku']) ? (int)$_POST['CisloRadku'] : null;

            require_once 'adsapp/ronportaldbrecord.php';
            $rec = new RonPortalDBRecord($this ->database, 'DokumentSluzebniCesty', $this);
            $rec ->addField('Id', 'int', false, 0, (int)$id);
            $rec ->addField('SluzebniCesta_ID', 'int', false, 0, (int)$idSluzebniCesty);
            if ($cisloRadku) {
                $rec->addField('CisloRadku', 'int', false, 0, (int)$cisloRadku);
            }
            $rec ->insertRecord();
//        } catch (\Exception $e) {
//            $result ->setError($e ->getMessage());
//        }
//
//        return $result;
    }


    function handleJsonAction($action, $output)
    {
        TextManager::getInstance() ->loadStrings('sluzebnicesty.ini');

        switch($_GET['action']) {
            case 'vyberspolucestujicich' :
                $this ->vyberSpolucestujich($output);
                break;

            case 'pridejspolucestujiciho' :
                $this ->pridejSpolucestujiciho($output);
                break;

            case 'overspolucestujiciho' :
                $this ->overSpolucestujiciho($output);
                break;

            case 'maxstravneprostat' :
                $this ->maximalniDenniStravneProStat($output);
                break;

            case 'statpromistosc' :
                $this ->statProMistoSC($output);
                break;

            case 'kapesneprostat' :
                $this ->kapesneProStat($output);
                break;

            case 'adresarozdeleni' :
                $this ->adresaRozdeleni($output);
                break;

            case 'rozdeleniosoby' :
                $this ->rozdeleniOsoby($output);
                break;

            case 'dokumentupload' :
                $this ->dokumentUpload();
                break;

            default :
                throw new \Exception('Chyba v SluzebniCestyJsController::handleJsonAction - neznama akce!');
        }
    }

}