<?php

/**
 * UTF8
 */

namespace Ron\RonPortal\XML;

use Ron\Browser;
use Ron\DB\SqlStatement;
use Ron\Http\App;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PredavaciProcesyUtils;
use Ron\StringUtils;

require_once('ron/browser.php');
require_once('adsapp/predavaciproces.php');


class SablonaXMLFormulareBrowser extends Browser
{
    var $procesyUzivatele; // seznam urovni predavani aktualniho uzivatele

    function __construct($database, $xmlMenuId)
    {
        parent::__construct($database);

        global $STR_SablonyXMLFormularu; // kompat.
        global $STR_NazevSablony;       // kompat.
        global $STR_VyplnitDotaznik;    // kompat.

        $this->title = $STR_SablonyXMLFormularu ?: \Ron\ronText('Sablony_xml_formularu');

        $this->browserUrl = '?menuid=seznamxmlformularu';

        $this->sqlFromPart = 'SablonaXMLFormulare';
        $params = $database ->createParameters();
        $where =
            '(PlatnostOd IS NULL  OR  PlatnostOd <= ' . $params ->addParameter($database->dateToParam(time())) . ')' .
            ' AND (PlatnostDo IS NULL  OR  PlatnostDo >= ' . $params ->addParameter($database->dateToParam(time())) . ')';

        $this->browserUrl = '?menuid=seznamxmlformularu&';
        if ($xmlMenuId) {
            $this->browserUrl .= 'xmlmenuid=' . $xmlMenuId . '&';
            $where .=
                ' AND MenuId = ' . $params->addParameter($xmlMenuId);
        }
        $this ->sqlWherePart = new SqlStatement($where, $params);

        $this->sqlSelectPart = 'Id, PredavaciProces, TypPrava, Popis, PlatnostDo';

        $this->addField([BROWSER_FIELD_SQL_NAME => 'Id', BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_ORDERING => 1]);
        if (App::getDebugMode()) {
            $this->addField([BROWSER_FIELD_SQL_NAME => 'PredavaciProces', BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_HEADER => StringUtils::fromUTF8('Předávací proces'), BROWSER_FIELD_ORDERING => 1]);
        }

        $this->addField([BROWSER_FIELD_SQL_NAME => 'Popis', BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_HEADER => $STR_NazevSablony ?: \Ron\ronText('Nazev_xml_sablony'), BROWSER_FIELD_ORDERING => 1]);
        $this->addField([BROWSER_FIELD_SQL_NAME => 'PlatnostDo', BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_HEADER => \Ron\ronText('Platnost_do'), BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_ORDERING => 1]);
        $this->addField([BROWSER_FIELD_TYPE => 'text', BROWSER_FIELD_TEXT => $STR_VyplnitDotaznik ?: \Ron\ronText('Vyplnit_xml_formular'), BROWSER_FIELD_LINK => '?menuid=xml', BROWSER_FIELD_ORDERING => 1]); // .($xmlmenuid ? '&xmlmenuid='.$xmlmenuid : '')

        $this->defaultSort = 1;
        $this->primaryField = 0;

        // nacte urovne predavani aktualniho uzivatele
        //$this -> f_urovneuzivatele = SeznamUrovniPredavaniUzivatele($connection);
        $this->procesyUzivatele = PredavaciProcesyUtils::seznamProcesuUzivatele($database, true); // jen prvni uroven = true
    }


    function enableLine($values)
    {
        if ($values[2]) {
            // kontrola prav (pole TypPrava)
            $prava = Prava::vratPrava($values[2]);
            if (!$prava->jsouNejakePrava()) return false;
        }

        // odfiltruje jen formulare, ktere muze aktualni uzivatel vyplnit
        // (= je na prvni urovni v predavacim procesu)
        return in_array($values[1], $this->procesyUzivatele);
    }

}

