<?php
/**
 * UTF8
 * stanislav.elbl, 26.05.2017
 */

namespace Ron\RonPortal\XML;


use Ron\MessageManager;
use Ron\MVC\Redirect;
use Ron\RonPortal\Log;
use Ron\RonPortal\NastaveniUzivatele;
use Ron\RonPortal\Prava;
use Ron\RonPortal\User;
use Ron\StringUtils;


/**
 * Obecny predek pro prikazy (obvykle elementy, ktere negeneruji zadny obsah, ale provadi nejakou cinnost)
 * Class XMLFormElementCommand
 * @package Ron\RonPortal\XML
 */
abstract class XMLFormElementCommand extends XMLFormElement
{
    abstract function doCommand();


    function getHTMLOutput() {
        $this->doCommand();
        $this->getElementsHTMLOutput(); // provede i vnorene tagy (dulezite napr. pro dbrecord)
        return '';
    }


    function writeXMLOutput($xmlWriter) {
        $this ->doCommand();
        parent::writeXMLOutput($xmlWriter);
    }


    function event()
    {
        $this ->doCommand();
        parent::event();
    }


    function process()
    {
        $this ->doCommand();
        parent::process();
    }
}




class XMLFormElementSimpleCommand extends XMLFormElementCommand
{
    function doCommand()
    {
        switch ($this->elementName) {
            case 'exit' :
                $this->xmlForm->exit = true;
                break;

            case 'mapovanimenu' :
                $this->xmlForm->f_mapovanimenu_pokracovat = self::getBoolValue($this->getAttributeValue('pokracovat'));
                $this->xmlForm->f_mapovanimenu_newmenuid = $this->getAttributeValue('menuid');
                break;
        }
    }
}


class XMLFormElementVariable extends XMLFormElementCommand
{
    var $value;
    var $name;
    var $index;

    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        $this ->name = $this ->getAttributeValue('name');
        $this ->index = StringUtils::fromUTF8($this ->getAttributeValue('index'));

        $val = $this ->getAttributeValue('value');
        if (isset($val)) {
            $this ->value = StringUtils::fromUTF8($val);
        } else {
            if (!$this ->isEmpty  ||  $this ->elementName != 'datavariable') {
                $this->value = $this->getTextContent(false);
            }
        }
    }


    function doCommand()
    {
        $name = $this ->nahradPromenne($this ->name);

        if ($this ->elementName == 'datavariable') {
            $this ->xmlForm ->addDataVariable($name);
            // prazdny tag <datavariable name=".." /> nenastavuje hodnotu, jen zajisti ulozeni promenne do dat
            if (!isset($this ->value)) return '';
        }

        $value = self::getBoolValue($this ->getAttributeValue('preserveparams')) ?
            $this ->value :
            $this ->nahradPromenne($this ->value);


        $index = (isset($this ->index)  &&  $this ->index !== '') ? $this ->nahradPromenne($this ->index) : null;

        if ($delimiter = $this ->getAttributeValue('delimiter')) {
            $value = explode($delimiter, $value, $this ->getAttributeValue('explodelimit', PHP_INT_MAX));
        }

        $this ->setVariableValue($name, $value, $index);

        return '';
    }


    function loadData()
    {
        // nacteni dat z datoveho XML
        $this ->doCommand();
        // prida do datovych promennych, aby se znovu ulozila
        $this ->xmlForm ->addDataVariable($this ->name);
    }
}


class XMLFormElementExpression extends XMLFormElementCommand
{
    var $expression;
    var $name;
    var $copy;


    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        $this->name = $this->getAttributeValue('name');
        $this->expression = $this->getTextContent(false);
        $this->copy = self::getBoolValue($this ->getAttributeValue('copy', '1')); // defaultne se kopiruje cely tag
    }


    function evalError($expression)
    {
        $errorInfo = error_get_last();
        $errorMsg = ($errorInfo && isset($errorInfo['message'])) ? $errorInfo['message'] : '';
        $this->xmlForm->addErrorMsg("Chyba v tagu <expression>!\nVyhodnocovany vyraz: $expression\n" . $errorMsg);
    }


    function doCommand()
    {
        $name = $this->nahradPromenne($this->name);

        if (!$this ->copy  &&  $name) {
            $value = $this->variableValue($name);
            if (isset($value)) {
                // copy=false a promenna uz je nastavena => nic nedela
                return;
            }
        }

        $expression = $this->nahradPromenne($this->expression);

        MessageManager::addDebugLog("XMLDebug&gt; Tag expression - vyhodnocuji vyraz ($name): $expression");

        /** @noinspection PhpUnusedLocalVariableInspection */
        $xmlform = $this->xmlForm; // pouziva se ve formularich (napr. $xmlform ->database)!

        $oldReporting = error_reporting(E_PARSE | E_ERROR); // upraveno SE 28.3.2017 - kdyz tu byla 0, tak se nezobrazovaly chyby a neslo zjistit, kde je problem

        if ($name) {
            $value = null;
            $ok = eval('$value = ' . $expression . ';');
            if ($ok === false) {
                $this->evalError($expression);
            } else {
                $this->setVariableValue($name, $value);
            }

            MessageManager::addDebugLog("Tag expression - vysledek: $value");
        } else {
            $ok = eval($expression);
            if ($ok === false) {
                $this->evalError($expression);
            }
        }

        error_reporting($oldReporting);
    }


    // TODO: function writeXMLOutput($xmlWriter) - osetrit parametr "copy"

    function process()
    {
        if (!$this ->copy) {
            // tag se ma provadet jen jednou
            $name = $this->nahradPromenne($this->name);
            if ($name) {
                parent::process();
                $this ->xmlForm ->addDataVariable($name);
            }
        } else {
            parent::process();
        }
    }
}


class XMLFormElementUsePost extends XMLFormElementCommand
{
    function doCommand()
    {
        $this ->xmlForm ->usePostValues = (int)$this ->getAttributeValue('value', 1);
    }
}


class XMLFormElementPostVariable extends XMLFormElementCommand
{
    var $name;
    var $htmlName;
    var $format;
    var $charset;

    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this->name = $this->getAttributeValue('name');
        $this->htmlName = $this->getAttributeValue('htmlname');
        $this->format = $this->getAttributeValue('format');
        $this->charset = $this->getAttributeValue('charset');
    }


    function doCommand()
    {
        $name = $this->nahradPromenne($this->name);

        $htmlname = $this->htmlName ? $this->nahradPromenne($this->htmlName) : $name;

        $value = $_POST[$htmlname];
        if ($this->charset && $this->charset != 'windows-1250') {
            $value = iconv('utf-8', 'windows-1250', $value);
        }

        $this->setVariableValueWithFormat($name, $this->format, $value);
    }
}


class XMLFormElementRedirect extends XMLFormElementCommand
{
    /*
     * <redirect url="..."> - redirect na zadanou url
     * <redirect urlparams="..."> - obecny redirect na jinou stranku v portalu, napr. urlparams="menuid=dochazkazamestnance"
     * <redirect sablona="..." params="...">  - na zadanou sablonu
     * <redirect xmlid="..." params="...">  - na zadane vyplnene xml
     * <redirect> - sam na sebe
     * <redirect params="....">  - sam na sebe (plati i pro sablonu i pro vyplneny dotaznik) + zadane parametry
     */
    function getRedirect()
    {
        if ($value = $this ->getAttributeValue('value') != null) {
            if (!$value) return null;
        }

        $urlParams = $this ->getAttributeValue('urlparams');
        if ($urlParams) $urlParams = $this ->nahradPromenne($urlParams, 'html'); // TODO: udelat format 'url' ?

        if ($url = $this ->getAttributeValue('url')) {
            // obecny redirect kamkoliv
            $url = $this->nahradPromenne($url, 'html');
            return new Redirect($url, $urlParams);
        }

        if ($urlParams) {
            // redirect na jakoukoliv stranku portalu
            return new Redirect(null, $urlParams);
        }

        $idSablony = $this ->xmlForm ->idSablony;

        if ($xmlId = $this ->getAttributeValue('xmlid')) {
            // redirect na zadane xml
        } else if ($hlpId = $this ->getAttributeValue('sablona')) {
            // redirect na zadanou sablonu
            $idSablony = $hlpId;
        } else if ($xmlId = $this ->xmlForm -> id) {
            // vyplneny dotaznik - redirect na sebe
        }

        $params = $this ->nahradPromenne($this ->getAttributeValue('params'), 'html');
        $params = $params ? '&' . $params : '';

        if ($params) {
            if ($xmlId) {
                // uz vyplneny formular
                return new Redirect(null, '?menuid=editovatdotaznik&record=' . $xmlId . $params);
            }

            return new Redirect(null, '?menuid=xml&record=' . $idSablony . $params);
        }

        // pri prvnim vyplnovani predavaci sablony jeste neni prirazen Id vyplneneho formulare - nelze jeste vytvorit Redirect
        return null;
    }


    function doCommand()
    {
        $redirect = $this ->getRedirect();

        if ($redirect) {
            $this->xmlForm->redirect = $redirect;
        } else {
            $this->xmlForm->pokracovat = true;
        }
//        switch ($this ->elementName) {
//            case 'pokracovat' :
//                $value = $this->getAttributeValue('value');
//                if (isset($value)) $value = (int)$this->nahradPromenne($value);
//                else $value = 1;
//                $this ->xmlForm ->pokracovat = $value;
//                break;
//
//            case 'redirect' :
//                $this ->xmlForm ->redirect = $this;
//                break;
//
//            default :
//                throw new \Exception("Element {$this ->elementName} neni implementovan!");
//        }
    }
}


class XMLFormElementSQLAction extends XMLFormElementCommand
{

    var $sqlText;


    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);
        $this ->sqlText = $this ->getTextContent(false);
    }


    function doCommand()
    {
        $sqlvar = $this ->getAttributeValue('sqltextvariable');

        $sql = $sqlvar ? $this ->variableValue($sqlvar) : $this ->sqlText; // convert = false

        $sql = $this ->nahradPromenneSQL($sql);

        try {
            $ok = $this->xmlForm->database->sql($sql); // nevraci zadne radky
        } catch (\Exception $e) {
            $ok = false;
        }

        if ($ok) {
            require_once('adsapp/ronportallog.php');
            Log::logSQL($this ->xmlForm ->database, $sql, 'XMLAkce (' . $this ->xmlForm -> popis . ')');
        } else {
            $this ->xmlForm ->addErrorMsg("Chyba v SQL dotazu! \n" . $sql, $this ->xmlForm ->database ->lastErrorText());
            $this ->xmlForm ->addErrorMsg($e);
        }
    }
}


class XMLFormElementPodrobnosti extends XMLFormElementCommand
{
    var $fieldName;
    var $value;


    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader); // TODO: Change the autogenerated stub

        if ($this ->elementName == 'nastavpodrobnostiformulare') {
            $this->fieldName = $this->getAttributeValue('fieldname');
            $this->value = $this->getTextContent(false);
        }
    }


    function doCommand()
    {
        switch ($this ->elementName) {
            case 'stav' :
                $this ->xmlForm ->stav = (int)$this ->getAttributeValue('novystav');
                break;

            case 'uzavrit' :
                $this ->xmlForm ->stav = 128;
                break;

            case 'nastavosobuformulare' :
                $rc = $this ->nahradPromenne( StringUtils::fromUTF8($this ->getAttributeValue('rc')) );
                $this ->xmlForm ->nastavOsobu($this ->getAttributeValue('name'), $rc);
                break;

            case 'nastavpracovnipomer' :
                $idpp = $this ->nahradPromenne( StringUtils::fromUTF8($this ->getAttributeValue('idpp')) );
                $this ->xmlForm ->nastavPracovniPomer( $this ->getAttributeValue('name'), $idpp );
                break;

            case 'nastavobdobiformulare' :
                $obdobi = $this ->nahradPromenne($this ->getAttributeValue('obdobi'));
                $this ->xmlForm ->nastavObdobiAplikace($this ->getAttributeValue('name'),
                    $this ->getAttributeValue('aplikace'), $obdobi,
                    self::getBoolValue($this ->getAttributeValue('variabilni')));
                break;

            case 'nastavrozdeleniformulare' :
                $typ = $this ->nahradPromenne($this ->getAttributeValue('typ'));
                $kod = $this ->nahradPromenne($this ->getAttributeValue('kod'));
                $this ->xmlForm ->nastavRozdeleni($this ->getAttributeValue('name'), $typ, $kod);
                break;

            case 'nastavpodrobnostiformulare' :
                $value = $this->nahradPromenne($this->value);

                switch (strtolower($this->fieldName)) {
                    case 'context' :
                        $this->xmlForm->context = $value;
                        break;

                    case 'rozliseni1' :
                        $this->xmlForm->rozliseni1 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni2' :
                        $this->xmlForm->rozliseni2 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni3' :
                        $this->xmlForm->rozliseni3 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni4' :
                        $this->xmlForm->rozliseni4 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni5' :
                        $this->xmlForm->rozliseni5 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni6' :
                        $this->xmlForm->rozliseni6 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni7' :
                        $this->xmlForm->rozliseni7 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni8' :
                        $this->xmlForm->rozliseni8 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni9' :
                        $this->xmlForm->rozliseni9 = substr(trim($value), 0, 40);
                        break;
                    case 'rozliseni10' :
                        $this->xmlForm->rozliseni10 = substr(trim($value), 0, 40);
                        break;

                    case 'dodatecnypopis' :
                        $this->xmlForm->dodatecnyPopis = substr(trim($value), 0, 120);
                        break;

                    case 'poznamka' :
                        $this->xmlForm->poznamka = $value;
                        break;

                    case 'resultcode' :
                        $this->xmlForm->xmlResultCode = $value;
                        break;

                    case 'resultmessage' :
                        $this->xmlForm->xmlResultMessage = $value;
                        break;
                }

                break;

            default :
                throw new \Exception('Chyba v metode XMLFormElementPodrobnosti::doCommand - neznamy tag!');
        }
    }
}



/**
 * Class XMLFormElementFiltrPodlePrav
 * Do promenne se zadanym jmenem (name) vlozi sql filtr pro odfiltrovani zaznamu podle prav uzivatele
 */
class XMLFormElementFiltrPodlePrav extends XMLFormElementCommand
{
    /** @var string */
    var $name;
    /** @var string */
    var $typPrava;
    var $tabulka;
    var $obdobiOd;
    var $obdobiDo;

    function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this ->name = $this ->getAttributeValue('name');
        $this ->typPrava = $this ->getAttributeValue('typprava');
        $this ->tabulka = $this ->getAttributeValue('tabulka');
        $this ->obdobiOd = $this ->getAttributeValue('obdobiod');
        $this ->obdobiDo = $this ->getAttributeValue('obdobido');
    }


    function doCommand()
    {
        $tabulka = ($this ->tabulka) ? $this ->tabulka : 'PracovniPomer';
        $typPrava = $this ->nahradPromenne($this ->typPrava);

        $obdobiOd = $this ->obdobiOd ? (int)$this -> nahradPromenne($this ->obdobiOd) : null;

        if (!$obdobiOd) {
            $obdobiOd = $this ->xmlForm ->pocatekObdobi ?: time();
        }

        $obdobiDo = $this ->obdobiDo ? (int)$this -> nahradPromenne($this ->obdobiDo) : null;

        if (!$obdobiDo) {
            $obdobiDo = $this ->xmlForm ->konecObdobi ?: time();
        }

        $prava = Prava::vratPrava($typPrava);
        $filtr = $prava ->sqlFiltr($tabulka, $obdobiOd, $obdobiDo);
        $this ->setVariableValue($this ->name, $filtr);
    }
}


/**
 * Class XMLFormElementPrava
 * Zkontroluje prava uzivatele a vysledek vlozi do promenne se zadanym jmenem
 * atributy name, typprava, typprava2, idpp, rc, errormessage, exitonerror, resultcode
 */
class XMLFormElementPrava extends XMLFormElementCommand
{
    /** @var string */
    var $typPrava;
    /** @var string */
    var $typPrava2;


    function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this ->typPrava = $this ->getAttributeValue('typprava');
        $this ->typPrava2 = $this ->getAttributeValue('typprava2');
    }


    function doCommand()
    {
        Prava::nactiPravaUzivatele( $this ->xmlForm ->database, User::getRC() );
        Prava::nastavObdobiProPrava(time(), time());

        $prava = null;

        $typprava = $this ->nahradPromenne($this ->typPrava);

        if ( $typprava  &&  Prava::existujePravo($typprava) ) {
            $prava = Prava::vratPrava($typprava);
        } else {
            // typprava2 jen v pripade, ze vubec neexistuje typprava
            $typprava = $this ->nahradPromenne($this ->typPrava2);
            if ( $typprava  &&  Prava::existujePravo($typprava) ) {
                $prava = Prava::vratPrava($typprava);
            }
        }

        if ($prava) {
            if ( $idpp = $this ->nahradPromenne($this ->getAttributeValue('idpp')) ) {
                // kontrola prava na konkretni PP
                $result = $prava ->lzeZobrazitPP($idpp, time(), time());
            } else if ( $rc = $this ->nahradPromenne($this ->getAttributeValue('rc')) ) {
                // kontrola prava na osobu
                $result = $prava ->lzeZobrazitRC($rc, time(), time());
            } else {
                // jen nastavi hodnotu prava
                $result = $prava ->prava;
            }
        }
        else {
            $result = 0;
        }

        if ($name = $this ->getAttributeValue('name')) {
            $this ->setVariableValue($name, $result);
            if ($prava) {
                $this ->setVariableValue($name.'_priznak', $prava ->priznak);
                $this ->setVariableValue($name.'_priznak2', $prava ->priznak2);
            }
        }

        if (!$result) {
            if ($message = $this ->getAttributeValue('errormessage')) {
                $message = $this ->nahradPromenne(StringUtils::fromUTF8($message));
                $this ->xmlForm ->addErrorMsg($message);
            }

            if ( self::getBoolValue($this ->getAttributeValue('exitonerror')) ) {
                $this ->xmlForm ->exit = true;

                $resultcode = $this ->getAttributeValue('errorresultcode');
                if ($resultcode) $this ->xmlForm ->f_XmlResultCode = $resultcode;
            }
        }
    }
}



class XMLFormElementMessage extends XMLFormElementCommand
{
    var $fieldName;
    var $text;


    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        $this ->fieldName = $this ->getAttributeValue('fieldname');
        $this ->text = $this ->getAttributeValue('text');

        if (!isset($this ->text)) {
            $this ->text = $this ->getTextContent(true);
        } else {
            $this ->text = StringUtils::fromUTF8($this ->text);
        }
    }


    function doCommand()
    {
        $text = $this ->nahradPromenne($this ->text);

        switch ($this ->elementName) {
            case 'error' :
            case 'errormsg' :
            case 'errormessage' :
                $fieldName = $this ->nahradPromenne($this ->fieldName);
                $this ->xmlForm ->addErrorMsg( $text, $fieldName );
                break;

            case 'infomsg' :
            case 'infomessage' :
                MessageManager::addInfoMessage($text);
                break;

            case 'warning' :
                MessageManager::addWarning($text);
                break;

            case 'debugmsg' :
            case 'debugmessage' :
                $this ->xmlForm ->addDebugMessage($text);
                break;

            case 'debuglog' :
                $this ->xmlForm ->addDebugMessage($text, true);
                break;
        }
    }
}


class XMLFormElementDelete extends XMLFormElementCommand
{
    function doCommand()
    {
        if ($this ->xmlForm ->readOnly) {
            return;
        }

        $okinfomsg = StringUtils::fromUTF8($this ->getAttributeValue('okinfomsg'));
        $this ->xmlForm ->delete($okinfomsg);
        $this ->xmlForm ->exit = true;
    }
}


