<?php
/**
 * UTF8
 * stanislav.elbl, 28.06.2017
 */

namespace Ron\RonPortal\XML;


use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\Personalistika\OtazkaDotazniku;
use Ron\StringUtils;

class XMLFormElementHlavickaHodnoceni extends XMLFormElement
{
    function getHTMLOutput()
    {
        if (!$this ->xmlForm ->dotaznik) {
            throw new \Exception('Chyba v tagu hlavickahodnoceni! Neni nacten dotaznik!');
        }

        // TODO: Tady se primo zobrazuje - predelat na View?
        $this ->xmlForm ->dotaznik ->zobrazHlavicku();
        return '';
    }
}


class XMLFormElementOtazkyHodnoceni extends XMLFormElement
{
    private function getCursor()
    {
        $id = (int)$this->nahradPromenne($this ->getAttributeValue('id'));
        $druhHodnoceni = (int)$this->nahradPromenne($this ->getAttributeValue('druhhodnoceni'));
        $temaCislo = (int)$this->nahradPromenne($this ->getAttributeValue('temacislo'));

        $params = $this ->xmlForm ->database ->createParameters();

        if ($id) {
            $sql =
                'SELECT HodnoticiKriterium.KodKriteria ' .
                '  FROM PripraveneTemaHodnoceni, HodnoticiKriterium ' .
                ' WHERE PripraveneTemaHodnoceni.Id = ' . $params ->addParameter($id) .
                '   AND HodnoticiKriterium.DruhHodnoceni = PripraveneTemaHodnoceni.DruhHodnoceni ' .
                '   AND HodnoticiKriterium.TemaCislo = PripraveneTemaHodnoceni.TemaCislo ' .
                '   AND HodnoticiKriterium.KodKriteriaHlavni IS NULL' . // 15.4.2015 RON-PR nacitaji se jen otazky, ktere nejsou dalsim typem odpovedi
                ' ORDER BY HodnoticiKriterium.Poradi ';
        } else {
            $sql =
                'SELECT HodnoticiKriterium.KodKriteria ' .
                '  FROM HodnoticiKriterium ' .
                ' WHERE HodnoticiKriterium.DruhHodnoceni = ' . $params ->addParameter($druhHodnoceni) .
                '   AND HodnoticiKriterium.TemaCislo = ' . $params ->addParameter($temaCislo) .
                '   AND HodnoticiKriterium.KodKriteriaHlavni IS NULL' . // 15.4.2015 RON-PR nacitaji se jen otazky, ktere nejsou dalsim typem odpovedi
                ' ORDER BY HodnoticiKriterium.Poradi ';
        }

        MessageManager::addDebugLog('XMLDebug> otazkyhodnoceni: ' . $sql);

        return $this ->xmlForm ->database ->getSqlResult($sql, $params);
    }


    function getHTMLOutput()
    {
        $result = '';

        $cursor = $this ->getCursor();
        while ($row = $cursor ->fetch()) {
            $this ->setVariableValue('HODNOCENI_KodKriteria', StringUtils::xmlTextConvert($row[0]));
            $result .= $this->getElementsHTMLOutput();
        }

        $cursor ->close();

        return $result;
    }


    function process()
    {
        $cursor = $this ->getCursor();

        while ($row = $cursor ->fetch()) {
            $this ->setVariableValue('HODNOCENI_KodKriteria', StringUtils::xmlTextConvert($row[0]));
            $this->processElements();
        }

        $cursor ->close();
    }
}


class XMLFormElementOtazkaHodnoceni extends XMLFormElement
{
    function getHTMLOutput()
    {
        $id = (int)$this->nahradPromenne($this ->getAttributeValue('id'));
        $druhHodnoceni = (int)$this->nahradPromenne($this ->getAttributeValue('druhhodnoceni'));
        $temaCislo = (int)$this->nahradPromenne($this ->getAttributeValue('temacislo'));
        $kodKriteria = $this->nahradPromenne($this ->getAttributeValue('kodkriteria'));

        $otazka = null;

        if ($druhHodnoceni == $this ->xmlForm ->dotaznik ->druhHodnoceni  &&  $temaCislo == $this ->xmlForm ->dotaznik ->temaCislo) {
            $otazka = $this ->xmlForm ->dotaznik ->vratOtazku($kodKriteria);
        }

        if (!$otazka) {
            $otazka = new OtazkaDotazniku($id, $druhHodnoceni, $temaCislo);
            $otazka->nactiOtazku($this ->xmlForm ->database, $kodKriteria);
        }

        $zpusobZobrazeni = $this ->getAttributeValue('zpusobzobrazeni');
        if (!is_null($zpusobZobrazeni) && ($zpusobZobrazeni !== '')) {
            $otazka -> zpusobZobrazeni = $zpusobZobrazeni;
        }

        $stylestr = ($otazka->barvaPozadi) ? 'style="background-color:' . GuiUtils::getHtmlColor($otazka->barvaPozadi) . '"' : '';

        $result =
            "<DIV $stylestr>".
            $otazka -> vratHTML_Otazka().
            $otazka -> vratHTML_Odpoved(!$this ->xmlForm -> readOnly).
            '</DIV>';

        if ( !$otazka ->odpovedVyplnena  &&  $otazka ->povinnaOdpoved ) $this ->xmlForm -> dotaznik -> fVyplnenoVse = false;

        return $result;
    }
}


class XMLFormElementPoznamkaHodnoceni extends XMLFormElement
{
    function getHTMLOutput()
    {
        return $this ->xmlForm -> dotaznik -> vratHTMLPoznamka(!$this ->xmlForm -> readOnly);
    }
}


class XMLFormElementTlacitkaHodnoceni extends XMLFormElement
{
    function getHTMLOutput()
    {
        $ulozit = StringUtils::fromUTF8($this ->getAttributeValue('ulozit'));
        $odeslat = StringUtils::fromUTF8($this ->getAttributeValue('odeslat'));
        $vratit = StringUtils::fromUTF8($this ->getAttributeValue('vratit'));
        $uzavrit = StringUtils::fromUTF8($this ->getAttributeValue('uzavrit'));

        return $this ->xmlForm -> dotaznik ->vratHTMLTlacitka(!$this ->xmlForm -> readOnly, $ulozit, $odeslat, $vratit, $uzavrit);
    }
}


class XMLFormElementUlozHodnoceni extends XMLFormElement
{
    function getHTMLOutput()
    {
        return '';
    }


    function process()
    {
        $this ->xmlForm ->dotaznik ->fUlozenoVXML = $this ->xmlForm ->dotaznik ->ulozVyplnenyDotaznik();
        $this ->xmlForm ->variables['HODNOCENI_Ulozeno'] = $this ->xmlForm ->dotaznik ->fUlozenoVXML;
    }
}


class XMLFormElementPrepocitejHodnoceni extends XMLFormElement
{
    function getHTMLOutput()
    {
        return '';
    }


    function process()
    {
        $neulozene = self::getBoolValue($this ->getAttributeValue('neulozene'));

        if ( $neulozene  || ($this ->xmlForm ->dotaznik ->fUlozenoVXML  &&  $this ->xmlForm ->dotaznik ->fVyplnenoVse) ) {
            $this ->xmlForm ->dotaznik ->fPrepoctenoVXML = $this ->xmlForm ->dotaznik ->prepocitejDotaznik();
            $this ->xmlForm ->variables['HODNOCENI_Prepocteno'] = $this ->xmlForm ->dotaznik ->fPrepoctenoVXML;
        }
    }
}
