<?php
/**
 * UTF8
 * stanislav.elbl, 26.05.2017
 */

namespace Ron\RonPortal\XML;


use Ron\MessageManager;
use Ron\RonPortal\User;
use Ron\StringUtils;


/**
 * Obecny predek pro vstupni elementy (edit, checkbox, select, ...)
 * Class XMLFormElementInput
 * @package Ron\RonPortal\XML
 */
class XMLFormElementInput extends XMLFormElement
{
    var $required;
    var $name;
    var $htmlParams; // DEPRECATED!

    /** @var bool Urcuje, jestli se ma v datovem formulari vytvaret datova promenna (default je true).
     *            Musi to byt implementovano v potomkovi! */
    var $dataVariable;

    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this->required = self::getBoolValue($this->getAttributeValue('required'));
        $this->dataVariable = self::getBoolValue($this ->getAttributeValue('datavariable', 1));

        $this->name = $this->getAttributeValue('name');
        if (!$this ->xmlForm ->mode) {
            // deprecated parametr htmlparams uz neni u novych formularu podporovan
            MessageManager::addDebugMessage('Parametr "htmlparams" jiz neni podporovan! Pouzivejte misto nej prefix html:!');
            $this->htmlParams = $this->getAttributeValue('htmlparams');
        }
    }


    /**
     * @param \XMLWriter $xmlWriter
     * @param string $value
     * @param bool $inline Pokud je true, zabali se do <span>, jinak do <div>
     * @return void  Vlozi html text s posledni odpovedi (tag <posledniodpoved>)
     */
    protected function posledniOdpovedXML($xmlWriter, $value, $inline = true)
    {
        $xmlWriter->startElement('posledniodpoved');

        $xmlWriter->startElement($inline ? 'span' : 'div');
        $xmlWriter->writeAttribute('class', 'dotaznik_jmeno');
        $xmlWriter->text(date('j.n.Y G:i', time()) . ' - ' . StringUtils::toUTF8(StringUtils::xmlTextConvert(User::getUserName())) . ': ');
        $xmlWriter->endElement(); // span/div

        $xmlWriter->text( StringUtils::toUTF8($value) );

        $xmlWriter->endElement(); // posledniodpoved
    }
}



class XMLFormElementEdit extends XMLFormElementInput
{
    var $format;
    var $defaultValue;
    var $size;
    /** @var string  yes/no/append/nolog */
    var $editing;


    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader); // TODO: Change the autogenerated stub

        $this->format = $this->getAttributeValue('format');
        $this->size = $this->getAttributeValue('size');
        $this->editing = $this->getAttributeValue('editing'); // yes/no/append/nolog

        //$this ->f_defaultvalue = $xmlReader ->hasValue ? iconv('utf-8', 'windows-1250', $xmlReader ->value) : '';
        //$this ->f_defaultvalue = iconv('utf-8', 'windows-1250', $xmlReader ->value);
        $this->defaultValue = $this->getTextContent(false);
    }


    function getHTMLOutput()
    {
        $name = $this->nahradPromenne($this->name);

        if ($this ->xmlForm ->dataMode()  &&  ($this ->editing == 'no' ||  $this ->editing == '0')  &&  $this ->variableValue($this ->name) !== null) {
            // datove XML, needitovatelne pole, uz vyplnene
            return '<span class="dotaznik_odpoved">' . StringUtils::htmlTextConvert($this ->variableValue($this ->name)) . '</span>';
        }

        $attrs = [
            'type' => 'text',
            'name' => $name
        ];

        $htmlparams = $this->nahradPromenne($this->htmlParams);
        $htmlConvert = true;

        if ($this->xmlForm->usePostValues && isset($_POST[$name])) {
            $value = $_POST[$name];

            if ( $this->format == 'datetime'  &&  $_POST[$name . '_time'] != '') {
                $value .= ' ' . $_POST[$name . '_time'];
            }
        } else {
            $value = null;

            if ($this ->xmlForm ->dataMode()) {
                $value = $this ->variableValue($name);
            }

            if (!isset($value)) {
                $value = $this->defaultValue;
                // nahradi promenne - aby bylo mozno napr. prednastavit aktualni cas
                $value = $this->nahradPromenne($value, 'html'); // bez toho se neformatuji promenne (napr. number_2)
                $htmlConvert = false;
            }
        }

        $attrs['value'] = $htmlConvert ? StringUtils::htmlTextConvert($value) : $value;

        //questform_nastavpromennou($name, $this -> f_format, $value);
        $this->setVariableValueWithFormat($name, $this->format, $value);

        $attrs['class'] = $this->xmlForm->isError($name) ? 'dotaznik_errinput' : 'app_input';

        if ($this->xmlForm->readOnly && strpos($htmlparams, 'readonly') === false) $attrs['readonly'] = 'readonly';

        switch ($this->format) {
            case 'float' :
                $attrs['class'] .= ' inputFloat';
                break;

            case 'int' :
            case 'integer' :
            case 'positive' :
                $attrs['class'] .= ' inputInt';
                break;

            case 'time' :
                $attrs['size'] = '5';
                $attrs['class'] .= ' inputTime';
                break;

            case 'date' :
                $attrs['size'] = '10';
                $attrs['maxlength'] = '10';
                $attrs['class'] .= ' inputDate';
                break;

            case 'datetime' :
                break;
        }

        if ($this->size && (int)$this->size > 0) $attrs['size'] = (int)$this ->nahradPromenne($this->size);

        // doplni atributy oznacene prefixem html:
        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        switch ($this->format) {
            case 'date' :
                return
                    '<span>' .
                    "<input $attrs_str $htmlparams/>" .
                    '<i class="showcalendar inline-glyph glyph-button fa glyph-calendar" data-roncalendar-inputname="' . $name . '"></i>' .
                    '</span>';

            case 'datetime' :
                $poma = explode(' ', $attrs['value']);
                $readonlystr = isset($attrs['readonly']) ? 'readonly="' . $attrs['readonly'] . '"' : '';

                return
                    '<span>' .
                    '<input type="text" name="' . $attrs['name'] . '" class="' . $attrs['class'] . ' inputDate" maxlength="10" size="10" value="' . $poma[0] . '" ' . $htmlparams . '/>' .
                    '<i class="showcalendar inline-glyph glyph-button fa glyph-calendar" data-roncalendar-inputname="' . $name . '"></i>' .
                    '<input type="text" name="' . $attrs['name'] . '_time" class="' . $attrs['class'] . ' inputTime" ' . $readonlystr . ' size="5" value="' . $poma[1] . '" ' . $htmlparams . '/>' .
                    '</span>';

            default :
                return "<input $attrs_str $htmlparams/>";
        }
    }


    function getPostValue($name)
    {
        if ($this->format == 'datetime') {
            $value = '';

            if ($_POST[$name] != '' && $_POST[$name . '_time'] == '') $value = $_POST[$name] . ' 00:00'; // neni zadan cas - doplni 0
            else if ($_POST[$name] == '' && $_POST[$name . '_time'] != '') $this->xmlForm->addErrorMsg(\Ron\ronText('Bad_date_format'), $name); // neni zadan datum
            else if ($_POST[$name] != '') $value = $_POST[$name] . ' ' . $_POST[$name . '_time'];
        } else {
            $value = $_POST[$name];
        }

        if ($value == '') {
            if ($this->required) {
                $this->xmlForm->addErrorMsg( \Ron\ronText('Chybi_povinna_hodnota'), $name);
            }
        }

        return $value;
    }


    function writeXMLOutput($xmlWriter)
    {
        $name = $this->nahradPromenne($this->name);
        $value = $this ->getPostValue($name);

        $newdefaultvalue = $this->defaultValue;
        $showedit = true;
        $logvalue = true;

        // nastavi hodnotu promenne
        //questform_nastavpromennou($name, $this -> f_format, $value);
        $this->setVariableValueWithFormat($name, $this->format, $value);

        if ($this->editing == 'append') {
            // pridava se dalsi text, vychozi hodnota je porad stejna jako v sablone
            if ($value == '') $logvalue = false;
            else if (!strcmp($value, $this->defaultValue)) $logvalue = false;
            // neni co logovat (byla zadana prazdna nebo vychozi hodnota)

            $newdefaultvalue = $this->defaultValue;
        } else if ($this->editing == 'yes'  ||  $this->editing == '1') {
            $logvalue = (strcmp($value, $this->defaultValue)); // loguje v pripade zmeny
            $newdefaultvalue = $value;
        } else if ($this->editing == 'nolog') {
            // editace je povolena a zadana hodnota se neloguje
            $logvalue = false;
            $newdefaultvalue = $value;
        } else {
            $showedit = false;
            $logvalue = false;
            // needitovatelne pole netreba logovat
        }

        if ($logvalue) {
            $this->posledniOdpovedXML($xmlWriter, StringUtils::xmlTextConvert($value));
        }

        if ($showedit) {
            $xmlWriter->startElement(($this->prefix ? $this->prefix . ':' : '') . $this->elementName);
            $this->writeXMLAttributes($xmlWriter);
            $xmlWriter->text( StringUtils::toUTF8($newdefaultvalue) );
            $xmlWriter->endElement();
        } else {
            if ($this->format == 'date' || $this->format == 'datetime' || $this->format == 'time') {
                $this ->writeXmlVariable($xmlWriter, $name . '_TIMESTAMP', $this->variableValue($name . '_TIMESTAMP'));
            }

            $this ->writeXmlVariable($xmlWriter, $name, $value);

            $xmlWriter->startElement('div');
            $xmlWriter->writeAttribute('class', 'dotaznik_odpoved');
            $xmlWriter->text( StringUtils::toUTF8($value) );
            $xmlWriter->endElement();
        }
    }


    function process()
    {
        $name = $this->nahradPromenne($this->name);

        if ($this->editing != 'append'  &&  $this->editing != 'yes'  &&  $this->editing != '1'  &&  $this->editing != 'nolog') {
            if ($this ->variableValue($name) != null) {
                // neni editovatelny a uz ma vyplnenou hodnotu
                return;
            }
        }

        $value = $this ->getPostValue($name);
        $oldValue = $this ->variableValue($name);

        $this->setVariableValueWithFormat($name, $this->format, $value);
        if ($this ->dataVariable) {
            $this->xmlForm->addDataVariable($name);
        }

        if ($value != $oldValue) {
            // zaloguje zmenu
            $this ->xmlForm ->addLogData($name, $value);
        }
    }
}


class XMLFormElementMemo extends XMLFormElementInput
{
    var $defaultValue;
    var $editing;


    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this->editing = $this->getAttributeValue('editing'); // yes/no/append/nolog
        //$this ->f_defaultvalue = iconv('utf-8', 'windows-1250', $xmlReader ->value);
        $this->defaultValue = StringUtils::fromUTF8($this->loadXMLValue($xmlReader));
    }


    function getHTMLOutput()
    {
        $name = $this->nahradPromenne($this->name);

        if ($this ->xmlForm ->dataMode()  &&  $this ->editing == 'no' ||  $this ->editing == '0'  &&  $this ->variableValue($this ->name) !== null) {
            // datove XML, needitovatelne pole, uz vyplnene
            return '<div class="dotaznik_odpoved">' . StringUtils::longTextToHtml($this ->variableValue($this ->name), true) . '</div>';
        }

        $attrs = ['name' => $name];

        $rows = $this->getAttributeValue('rows');
        $attrs['rows'] = $rows ? $rows : 8;

        $cols = $this->getAttributeValue('cols');
        $attrs['cols'] = $cols ? $cols : 60;

        $attrs['wrap'] = 'virtual';

        if ($this->xmlForm->readOnly)
            $attrs['readonly'] = 'readonly';

        if ($this->xmlForm->usePostValues && isset($_POST[$name])) {
            $value = $_POST[$name];
        } else {
            $value = null;

            if ($this ->xmlForm ->dataMode()) {
                if ($this->editing == 'append') {
                    $value = $this ->defaultValue;
                } else {
                    $value = $this->variableValue($name);
                }
            }

            if (!isset($value)) {
                $value = $this->nahradPromenne($this->defaultValue);
            }
        }

        $this->setVariableValue($name, $value);

        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        return "<textarea $attrs_str>" . StringUtils::xmlTextConvert($value) . '</textarea>';
    }


    function getPostValue($name)
    {
        // z formulare prijde jiny konec radku nez z xml v $defaultvalue!
        // musi se to sjednotit, aby potom fungovalo porovnavani
        return str_replace("\r\n", "\n", $_POST[$name]);
    }


    function writeXMLOutput($xmlWriter)
    {
        $name = $this->nahradPromenne($this->name);

        $value = $this ->getPostValue($name);
        $defaultvalue = $this->defaultValue; // iconv('utf-8', 'windows-1250', $this->defaultValue);

        $newdefaultvalue = $defaultvalue;
        $showedit = true;
        $logvalue = true;

        if ($this->editing == 'append') {
            // pridava se dalsi text, vychozi hodnota je porad stejna jako v sablone
            if ($value == '') $logvalue = false;
            else if (!strcmp($value, $defaultvalue)) $logvalue = false;
            // neni co logovat (byla zadana prazdna nebo vychozi hodnota)

            $newdefaultvalue = $defaultvalue; // zustava puvodni vychozi hodnota
        } else if ($this->editing == 'yes') {
            $logvalue = (strcmp($value, $defaultvalue)); // loguje pouze v pripade zmeny
            $newdefaultvalue = $value; // nova vychozi hodnota
        } else if ($this->editing == "nolog") {
            // editace je povolena a zadana hodnota se neloguje
            $logvalue = false;
            $newdefaultvalue = $value;
        } else {
            $showedit = false;
            $logvalue = false;
            // needitovatelne pole netreba logovat
        }

        // nastavi hodnotu promenne
        $this->setVariableValue($name, $value);

        if ($logvalue) {
            //$xmlWriter ->text(questform_getlogname(false));
            //$xmlWriter ->writeElement('posledniodpoved', str_replace("\n", "<BR/>", $value));
            $this->posledniOdpovedXML($xmlWriter, str_replace("\n", '<br/>', StringUtils::xmlTextConvert($value)), false);
        }

        if ($showedit) {
            $xmlWriter->startElement(($this->prefix ? $this->prefix . ':' : '') . $this->elementName);
            $this->writeXMLAttributes($xmlWriter);
            $xmlWriter->text( StringUtils::toUTF8($newdefaultvalue) );
            $xmlWriter->endElement();
        } else {
            $xmlWriter->startElement('longvariable');
            $xmlWriter->writeAttribute('name', $name);
            $xmlWriter->text($value);
            $xmlWriter->endElement();

            $xmlWriter->startElement('div');
            $xmlWriter->writeAttribute('class', 'dotaznik_odpoved');
            $xmlWriter->text(str_replace("\n", "<br/>", StringUtils::toUTF8($value) ));
            $xmlWriter->endElement();
        }
    }


    function process()
    {
        $name = $this->nahradPromenne($this->name);

        if ($this->editing != 'append'  &&  $this->editing != 'yes'  &&  $this->editing != '1'  &&  $this->editing != 'nolog') {
            if ($this ->variableValue($name) != null) {
                // neni editovatelny a uz ma vyplnenou hodnotu
                return;
            }
        }

        $value = $this ->getPostValue($name);

        $oldValue = ($this ->editing == 'append')
            ? $this ->defaultValue
            : $this ->variableValue($name);

        $this->setVariableValue($name, $value);
        if ($this ->dataVariable) {
            $this->xmlForm->addDataVariable($name);
        }

        if ($value != $oldValue) {
            // zaloguje zmenu
            $this ->xmlForm ->addLogData($name, $value);
        }
    }
}


class XMLFormElementCheckbox extends XMLFormElementInput
{
    var $editing;


    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);
        $this->editing = self::getBoolValue($this->getAttributeValue('editing'), true); // yes/no
    }

    function getHTMLOutput()
    {
        $name = $this ->nahradPromenne($this ->name);

        if ($this ->xmlForm ->dataMode()  &&  !$this ->editing  &&  $this ->variableValue($this ->name) !== null) {
            // datove XML, needitovatelne pole, uz vyplnene
            return '<span class="dotaznik_odpoved">' . \Ron\ronText($this ->variableValue($this ->name) ? 'Str_Yes' : 'Str_No' )  . '</span>';
        }

        $attrs = [
            'type' => 'checkbox',
            'name' => $name
        ];

        if ($this ->xmlForm ->usePostValues) {
            $checked = $_POST[$name] ? $_POST[$name] : 0;
        } else {
            $checked = null;

            if ($this ->xmlForm ->dataMode()) {
                $checked = $this ->variableValue($name);
            }

            if (!isset($checked)) {
                $checked = $this ->nahradPromenne($this ->getAttributeValue('checked'));
            }
        }

        if ($checked) {
            $attrs['checked'] = 'checked';
            $this -> setVariableValue($name, 1);
        } else {
            $this -> setVariableValue($name, 0);
        }

        $attrs = $this ->setHTMLAttributes($attrs);
        $attrs_str = $this ->getHTMLAttributes($attrs);

        $htmlParams = $this->nahradPromenne($this->htmlParams); // u formularu s nastavenym mode se vubec nenacita, takze nelze pouzit!

        return "<input $attrs_str $htmlParams/>";
    }


    function writeXMLOutput($xmlWriter)
    {
        $name = $this ->nahradPromenne($this ->name);

        $checked = !!$_POST[$name];
        if ($checked) {
            $this ->setVariableValue($name, 1);
        } else {
            $this ->setVariableValue($name, 0);
        }

        if ($this ->editing) {
            $xmlWriter->startElement(($this->prefix ? $this->prefix . ':' : '') . $this->elementName);
            $this->writeXMLAttributes($xmlWriter, ['checked' => ($checked ? '1' : '0')]);
            $xmlWriter->endElement();
        } else {
            $this ->writeXmlVariable($xmlWriter, $name, $checked ? '1' : '0');

            $xmlWriter->startElement('span');
            $xmlWriter->writeAttribute('class', 'dotaznik_odpoved');
            $xmlWriter->text( StringUtils::toUTF8(\Ron\ronText($checked ? 'Str_Yes' : 'Str_No')) );
            $xmlWriter->endElement();
        }
    }


    function process()
    {
        $name = $this->nahradPromenne($this->name);
        $value = $_POST[$name] ? 1 : 0;
        $oldValue = $this ->variableValue($name);

        $this ->setVariableValue($name, $value);
        if ($this ->dataVariable) {
            $this->xmlForm->addDataVariable($name);
        }

        if ($value != $oldValue) {
            // zaloguje zmenu
            $this ->xmlForm ->addLogData($name, $value);
        }
    }

}


class XMLFormElementRadio extends XMLFormElementInput
{

    var $value;
    var $checked;

    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this->value = $this->getAttributeValue('value');
        $this->checked = $this->getAttributeValue('checked');
    }


    function getHTMLOutput()
    {
        $name = $this->nahradPromenne($this->name);

        $attrs = [
            'type' => 'radio',
            'name' => $name
        ];

        $value = $this->nahradPromenne(StringUtils::fromUTF8($this->value));
        $checked = $this->nahradPromenne($this->checked);

        $attrs['value'] = $value;

        if ($this->xmlForm->readOnly)
            $attrs['disabled'] = true;

        if ($this->xmlForm->usePostValues && isset($_POST[$name])) {
            $checked = ($_POST[$name] == $value) ? 1 : 0;
        } else if ($this ->xmlForm ->dataMode()) {
            // datove xml
            $selectedValue = $this ->variableValue($name);
            if (isset($selectedValue)) {
                $checked = $selectedValue == $value;
            }
        }

        if ($checked) {
            $this->setVariableValue($name, $value);
            $attrs['checked'] = 'checked';
        }

        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        $htmlParams = $this->nahradPromenne($this->htmlParams); // u formular s nastavenym mode neni podporovan (vubec se nenacita)

        $result = "<input $attrs_str $htmlParams/>";
        $result .= $this ->getElementsHTMLOutput(); // text k radiobuttonku muze byt vnoreny (umoznuje to zadat editing="remove")

        return $result;
    }


    function writeXMLOutput($xmlWriter)
    {
        // editing="yes", "no", "remove" - nahradi se jen textem
        $name = $this->nahradPromenne($this->name);
        $value = $this->nahradPromenne(StringUtils::fromUTF8($this->value));
        $checked = ($_POST[$name] == $value) ? 1 : 0;
        if ($checked) {
            $this->setVariableValue($name, $value);
        }

        $checked = ($_POST[$name] == $value) ? 1 : 0;
        $editing = $this ->getAttributeValue('editing');

        if ($editing == 'remove') {
            // checkboxy se maji odstranit
            if ($checked) {
                $this ->writeXmlVariable($xmlWriter, $name, $value);

                $xmlWriter->startElement('span');
                $xmlWriter->writeAttribute('class', 'dotaznik_odpoved');
                $this->writeXMLElements($xmlWriter);
                $xmlWriter->endElement();
            }

        } else {
            $attrs = ['checked' => ($checked ? '1' : '0')];

            if ($editing == 'no' || $editing == '0' || $editing == 'false') {
                if (!$checked) {
                    $attrs['html:disabled'] = 'disabled';
                }
            }

            $xmlWriter->startElement(($this->prefix ? $this->prefix . ':' : '') . $this->elementName);
            $this->writeXMLAttributes($xmlWriter, $attrs);
            $this->writeXMLElements($xmlWriter);
            $xmlWriter->endElement();
        }
    }


    function process()
    {
        $name = $this->nahradPromenne($this->name);
        $value = $_POST[$name];
        if ($value == $this ->value) {
            // zaskrtnuta tato volba
            $oldValue = $this->variableValue($name);

            $this->setVariableValue($name, $value);
            if ($this ->dataVariable) {
                $this->xmlForm->addDataVariable($name);
                if ($value != $oldValue) {
                    // zaloguje zmenu
                    $this->xmlForm->addLogData($name, $value);
                }
            }
        }
    }
}



class XMLFormElementSQLSelect extends XMLFormElementInput
{
    var $sql;
    var $sqlTextVariable;
    var $format;
    var $value;
    var $showCode;

    public function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        $this -> value = $this ->getAttributeValue('value');
        $this -> sqlTextVariable = $this ->getAttributeValue('sqltextvariable');
        $this -> format = $this ->getAttributeValue('format');
        $this -> showCode = self::getBoolValue($this ->getAttributeValue('showcode'));

        $this -> sql = $this ->getTextContent(false);
    }


    private function loadSQLData($name)
    {
        $sql = $this -> sqlTextVariable ?
            $this -> variableValue($this -> sqlTextVariable) : // convert = false
            $this -> sql;

        $sql = $this -> nahradPromenneSQL($sql);
        $this ->xmlForm ->addDebugMessage("tag sqlselect, name: $name, sql: $sql", true);

        return $this ->xmlForm ->database ->getSqlResult($sql, null, 'TAG sqlselect: chyba v dotazu!');
    }


    function getHTMLOutput()
    {
        // TODO: poresit priznak editing v datovych XML?
        $name = $this ->nahradPromenne($this ->name);
        $attrs = ['name' => $name];

        if ($this->xmlForm->readOnly)
            $attrs['disabled'] = true;

        if ( $this -> xmlForm -> usePostValues  &&  isset($_POST[$name]) ) {
            $value = $_POST[$name];
        } else if ($this ->xmlForm ->dataMode()  &&  $this ->variableValue($name) !== null) {
            // datove xml - hodnota uz je ulozena v datech
            $value = $this ->variableValue($name);

            if (!self::getBoolValue($this ->getAttributeValue('editing' ))) {
                // needitovatelny, uz vyplneny select v datovem formulari
                return '<span class="dotaznik_odpoved">' . $this ->variableValue($name . '_text') . '</span>';
            }
        } else {
            // defaultni hodnota zadana ve formulari nebo drive vyplnena hodnota
            $value = $this -> nahradPromenne(StringUtils::fromUTF8($this -> value));
        }

        if ( $this ->xmlForm ->isError($name)) $attrs['class'] = 'dotaznik_errinput';

        $cursor = $this ->loadSQLData($name);

        $htmlParams = $this -> nahradPromenne($this -> htmlParams);

        $attrs = $this ->setHTMLAttributes($attrs);
        $attrs_str = $this ->getHTMLAttributes($attrs);

        $result = "<select $attrs_str $htmlParams>";

        if ( !isset($value)  ||  !$this -> required ) { // pokud jeste nebyla zadana hodnota nebo pole neni povinne, prida prazdnou hodnotu
            $selectedstr = $value ? '' : 'selected="selected"';
            $result .= "<option value='' $selectedstr>&nbsp;</option> ";
        }

        while ($row = $cursor ->fetch()) {
            $rowval = $this ->formatSqlValue($this -> format, $row[0]);
            // formatovatni (format timestamp na cislo, ostatni standardne na retezec)

            if ( $rowval == $value ) {
                $selectedstr = 'selected="selected"';
                $this -> setVariableValue($name, $value);
                $this -> setVariableValue($name.'_text', $row[1]);
            } else {
                $selectedstr = '';
            }

            $text = StringUtils::htmlTextConvert($row[1]);
            if ($this -> showCode) $text .= ' ('.$rowval.')';

            $result .= "<option value='$rowval' $selectedstr>$text</option> ";
        }

        $cursor ->close();

        $result .= "</select>\n";

        return $result;
    }


    function writeXMLOutput($xmlWriter)
    {
        $name = $this ->nahradPromenne($this ->name);

        if ($this ->required && $_POST[$name] == '') {
            $this ->xmlForm->addErrorMsg(\Ron\ronText('Chybi_povinna_hodnota'), $name);
        }

        $cursor = $this ->loadSQLData($name);

        $found = false;

        $kod = null;
        $popis = null;
        while (!$found && $row = $cursor ->fetch()) {
            $rowval = $this ->formatSqlValue($this -> format, $row[0]);

            if ($rowval == StringUtils::htmlTextConvert($_POST[$name])) {
                $kod = $rowval;
                $popis = $row[1];
                $found = true;
            }
        }
        $cursor ->close();

        $this -> setVariableValue($name, $kod);
        $this -> setVariableValue($name.'_text', $popis);

        if (self::getBoolValue($this ->getAttributeValue('editing'))) {
            $xmlWriter->startElement(
                ($this->prefix ? $this->prefix . ':' : '') .
                $this->elementName
            );

            $this->writeXMLAttributes($xmlWriter, ['value' => $kod]);

            $this->writeXMLElements($xmlWriter);

            $xmlWriter->endElement();
        } else {
            $this ->writeXmlVariable($xmlWriter, $name, $kod);
            $this ->writeXmlVariable($xmlWriter, $name . '_text', $popis);
            $xmlWriter ->startElement('span');
            $xmlWriter ->writeAttribute('class', 'dotaznik_odpoved');
            $xmlWriter ->text( StringUtils::toUTF8($popis) );
            $xmlWriter ->endElement();
        }
    }


    function process()
    {
        $name = $this ->nahradPromenne($this ->name);

        $editing = self::getBoolValue($this ->getAttributeValue('editing'));
        if (!$editing  &&  $this ->variableValue($name) != null) {
            // neni editovatelny a uz ma vyplnenou hodnotu
            return;
        }

        $oldValue = $this ->variableValue($name);

        if ($this ->required  &&  $_POST[$name] == '') {
            $this ->xmlForm ->addErrorMsg(\Ron\ronText('Chybi_povinna_hodnota'), $name);
        }

        $cursor = $this ->loadSQLData($name);

        $found = false;
        $value = null;
        $valueText = null;

        while (!$found  &&  $row = $cursor ->fetch()) {
            $rowval = $this ->formatSqlValue($this ->format, $row[0]);

            if ($rowval == $_POST[$name]) {
                $value = $rowval;
                $valueText = $row[1];

                $found = true;
            }
        }

        $cursor ->close();

        $this ->setVariableValue($name, $value);
        $this ->setVariableValue($name . '_text', $valueText);

        if ($this ->dataVariable) {
            $this->xmlForm->addDataVariable($name);
            $this->xmlForm->addDataVariable($name . '_text');

            if ($value != $oldValue) {
                $this->xmlForm->addLogData($name, $value . ' (' . $valueText . ')');
            }
        }
    }
}



class XMLFormElementOption extends XMLFormElement
{
    // pouze pro pouziti v select
    var $value;
    var $text;
    var $selected;

    public function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        $this->value = StringUtils::fromUTF8($this->getAttributeValue('value'));
        $this->selected = self::getBoolValue($this ->getAttributeValue('default'));

        $this->text = $this->getTextContent(false);
    }

    function getHTMLOutput()
    {
        $attrs = ['value' => $this->value];
        if ($this->selected) {
            $attrs['selected'] = 'selected';
        }
        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);
        return "<option $attrs_str>{$this ->text}</option>";
    }
}


class XMLFormElementSelect extends XMLFormElementInput
{
    var $value;

    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader); // nenacita podrizene polozky!

        $value = $this->getAttributeValue('value');
        $this->value = isset($value) ? StringUtils::fromUTF8($this->getAttributeValue('value')) : null;
    }


    function getHTMLOutput()
    {
        $name = $this->nahradPromenne($this->name);

        $attrs = ['name' => $name];

        if ($this->xmlForm->usePostValues && isset($_POST[$name])) {
            $value = $_POST[$name];
        } else if ($this->xmlForm->dataMode() && $this->variableValue($name) !== null) {
            // datove xml - hodnota uz je ulozena v datech
            $value = $this->variableValue($name);
        } else {
            $value = isset($this ->value) ? $this->nahradPromenne($this->value) : null;
        }

        if ($this->xmlForm->isError($name)) $attrs['class'] = 'dotaznik_errinput';

        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        $result = "<select $attrs_str>";

        foreach ($this->elements as $option) {
            if ($option instanceof XMLFormElementOption) {
                if (isset($value)) {
                    $option->selected = $value == $option->value;
                }

                if ($option->selected) {
                    $this->setVariableValue($name, $option->value);
                    $this->setVariableValue($name . '_text', $option->text);
                }

                $result .= $option->getHTMLOutput();
            }
        }

        $result .= "</select>\n";

        return $result;
    }


    function writeXMLOutput($xmlWriter)
    {
        $name = $this->nahradPromenne($this->name);

        $selectedValue = null;
        $selectedText = null;

        foreach ($this->elements as $option) {
            if ($option instanceof XMLFormElementOption) {
                if ($_POST[$name] == $option->value) {
                    $selectedValue = $option ->value;
                    $selectedText = $option ->text;
                }
            }
        }

        if (!isset($selectedValue)  &&  $this->required) {
            $this->xmlForm->addErrorMsg(\Ron\ronText('Chybi_povinna_hodnota'), $name);
        }

        $this->setVariableValue($name, $selectedValue);
        $this->setVariableValue($name . '_text', $selectedText);

        if (self::getBoolValue($this ->getAttributeValue('editing'))) {
            $xmlWriter->startElement('select');
            $this->writeXMLAttributes($xmlWriter, ['value' => $selectedValue]);
            $this->writeXMLElements($xmlWriter);
            $xmlWriter->endElement();
        } else {
            $this ->writeXmlVariable($xmlWriter, $name, $selectedValue);
            $this ->writeXmlVariable($xmlWriter, $name . '_text', $selectedText);
            $xmlWriter ->startElement('span');
            $xmlWriter ->writeAttribute('class', 'dotaznik_odpoved');
            $xmlWriter ->text( StringUtils::toUTF8($selectedText) );
            $xmlWriter ->endElement();
        }
    }


    function process()
    {
        $name = $this->nahradPromenne($this->name);
        $oldValue = $this ->variableValue($name);

        if ($this ->required  &&  $_POST[$name] == '') {
            $this ->xmlForm ->addErrorMsg(\Ron\ronText('Chybi_povinna_hodnota'), $name);
        }

        $selectedValue = null;
        $selectedText = null;

        foreach ($this->elements as $option) {
            if ($option instanceof XMLFormElementOption) {
                if ($_POST[$name] == $option->value) {
                    $selectedValue = $option ->value;
                    $selectedText = $option ->text;
                }
            }
        }

        $this->setVariableValue($name, $selectedValue);
        $this->setVariableValue($name . '_text', $selectedText);
        if ($selectedValue != $oldValue) {
            $this ->xmlForm ->addLogData($name, $selectedValue . '(' . $selectedText . ')');
        }

        if ($this ->dataVariable) {
            $this->xmlForm->addDataVariable($name);
            $this->xmlForm->addDataVariable($name . '_text');
        }
    }
}


class XMLFormElementHidden extends XMLFormElementInput
{

    var $format;
    var $value;

    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this ->format = $this ->getAttributeValue('format');
        //$this ->f_value = iconv('utf-8', 'windows-1250', $xmlReader ->value);
        //$this ->f_defaultvalue = iconv('utf-8', 'windows-1250', $this ->loadXMLValue($xmlReader) );
    }


    function getHTMLOutput()
    {
        $attrs = [];
        $attrs['type'] = 'hidden';

        $name = $this ->nahradPromenne($this ->name);
        $attrs['name'] = $name;

        if ($this ->xmlForm ->usePostValues && isset($_POST[$name])) {
            $value = $_POST[$name];
        } else {
            $value = null;

            if ($this ->xmlForm ->dataMode()) {
                $value = $this ->variableValue($name);
            }

            if (!isset($value)) {
                $value = StringUtils::fromUTF8($this ->getAttributeValue('value'));
                // nahradi promenne - aby bylo mozno napr. prednastavit aktualni cas
                $value = $this->nahradPromenne($value);
            }
        }

        $attrs['value'] = $value;

        //questform_nastavpromennou($name, $this ->f_format, $value);
        $this ->setVariableValueWithFormat($name, $this -> format, $value);

        $attrs = $this ->setHTMLAttributes($attrs);
        $attrs_str = $this ->getHTMLAttributes($attrs);

        return "<input $attrs_str/>";
    }


    function writeXMLOutput($xmlWriter)
    {
        $name = $this ->nahradPromenne($this ->name);
        $this->setVariableValueWithFormat($name, $this ->format, $_POST[$name]);

        parent::writeXMLOutput($xmlWriter);
    }


    function process()
    {
        $name = $this ->nahradPromenne($this ->name);
        $this->setVariableValueWithFormat($name, $this ->format, $_POST[$name]);
        if ($this ->dataVariable) {
            $this->xmlForm->addDataVariable($name);
        }
    }

}

