<?php

/**
 * UTF8
 */

namespace Ron\RonPortal\XML;

use Ron\BrowserField;
use Ron\DB\Database;
use Ron\DB\Cursor;
use Ron\MessageManager;
use Ron\RonPortal\XMLDochazkaBrowser;
use Ron\StringUtils;

require_once('ron/browserfield.php');


class XMLBrowserField extends BrowserField
{
    /** @var RonPortalXMLForm */
    var $xmlForm;
    /** @var string */
    var $xmlText;
    /** @var  string */
    var $xmlHTML;

    /**
     * @param RonPortalXMLForm $xmlForm
     */
    function __construct($xmlForm)
    {
        parent::__construct($xmlForm ->database);
        $this->xmlForm = $xmlForm;

        $this->xmlText = null;
        $this->xmlHTML = '';
    }


    /**
     * @param string $text
     * @param string $recordId
     * @return string
     */
    function getLink($text, $recordId)
    {
        $result = '';

        if ($this->link) {
            $link = $this->xmlForm->nahradPromenne($this->link, false);

            $link = str_replace('&', '&#38;', $link);

            $result .= '<A ' . $this->xmlHTML . ' href="' . $link . '">';
        }

        $result .= $text;

        if ($this->link) $result .= '</A>';

        return $result;
    }


    function getFieldCellText($value, $rowNumber, $recordId, $rowData)
    {
        if ($this->xmlText) {
            $value = $this->xmlForm->nahradPromenne($this->xmlText, false);
        }
        return parent::getFieldCellText($value, $rowNumber, $recordId, $rowData);
    }
}



class XMLBrowser
{
    /** @var Database */
    var $database;

    /** @var RonPortalXMLForm   odkaz na xml formular */
    var $xmlForm;
    /** @var string         jmeno pro vytvarene promenne */
    var $name;

    var $fixedCols;

    var $sql;
    /** @var XMLBrowserField[] */
    var $fields;

    /** @var string         css trida pro zobrazeni, defaultne "browser" */
    var $cssClass;
    var $columnCount;

    /** @var string         dodatecny text, ktery se zobrazi u kazdeho zaznamu */
    var $infoLineText;
    /** @var int            sloupec tabulky, na kterem zacina dodatecny text (odsazeni) */
    var $infoLineCol;

    /** @var string         odkaz pro kliknuti na radek (onclick) */
    var $rowClickLink;


    /**
     * @param Database $database
     * @param RonPortalXmlForm $xmlForm
     */
    function __construct($database, $xmlForm)
    {
        $this ->database = $database;
        $this ->xmlForm = $xmlForm;
        $this ->infoLineCol = 1;
    }



    function addField($field)
    {
        $this ->fields[] = $field;
    }


    function getTableHeader($sortFieldNumber)
    {
        if (!$this ->fields) return '';

        $result = '<tr>';

        $this ->columnCount = 0;

        foreach ($this ->fields as $i => $field) {
            if ($poms = $field ->getHeader($i, $sortFieldNumber)) {
                $result .= $poms;
                $this ->columnCount++;
            }
        }
        $result .= "</tr>\n";

        return $result;
    }



    function getTableStart()
    {
        if ($this ->cssClass) $cssclass = $this ->cssClass;
        else $cssclass = 'browser rowOver';

        $fixedCols = null;
        if ($this->fixedCols) {
            $fixedCols = "data-ron-fixed-cols='{$this->fixedCols}'";
            $cssclass .= " fixedCols";
        }

        return "<table class='$cssclass' $fixedCols>";
    }



    function createXMLVariables($values)
    {
        if ($this ->xmlForm ->verze > 1) {
            // od 02-2015 - promenne typu pole uz jsou podporovany primo ve vyrazech (plati pro verzi XML formulare 2)
            $this->xmlForm->variables[$this->name] = $values;
        } else {
            if ($this->xmlForm && $this->name) {
                foreach ($values as $i => $sqlvalue) {
                    $this->xmlForm->variables[$this->name . "[$i]"] = $values[$i];
                }
            }
        }
    }


    /**
     * Odkaz na proklik
     * @return string
     */
    function getClickLink()
    {
        if (!$this ->rowClickLink) return '';

        $link = $this ->xmlForm ->nahradPromenne($this ->rowClickLink, false);

        return $link;
    }


    /**
     * Zobrazeni jednoho radku
     * @param $values
     * @param $className
     * @param $rowNumber
     * @return string
     */
    function getLine($values, $className, $rowNumber)
    {
        if (!$this ->fields) return '';

        $skript = '';

        $link = $this ->getClickLink();
        $link = str_replace('&', '&#38;', $link);

        if ($link) $skript .= ' onclick="window.location=' . "'$link'" . '"';

        $result = '<tr class="' . $className . '" ' . $skript . '>';

        //$primaryindex = $this -> GetPrimaryFieldSQLIndex();
        //if ($primaryindex >= 0) $recordid = $values[$primaryindex];
        //else $recordid = '';
        $recordid = '';

        foreach ($this ->fields as $i => $field) {
            $value = (isset($field ->sqlFieldIndex)  &&  $field ->sqlFieldIndex) >= 0 ? (isset($values[$field ->sqlFieldIndex]) ? $values[$field ->sqlFieldIndex] : null) : null;

            $result .= $field ->getFieldCellText($value, $rowNumber, $recordid, $values);
        }

        $result .= "</tr>\n";

        return $result;
    }


    /**
     * @param string $className
     * @return string
     */
    function getInfoLine($className)
    {
        if (!$this ->infoLineText) return '';

        $detailtext = $this ->xmlForm ->nahradPromenne($this ->infoLineText, false);
        if (!$detailtext) return '';

        $result = '<tr class="' . $className . '">';

        // prazdne sloupce na zacatku (odsazeni)
        for ($i = 0; $i < $this ->infoLineCol; $i++) $result .= '<td></td>';

        // spojeny sloupec
        $result .= '<td colspan="' . (count($this ->fields) - $this ->infoLineCol) . '">';
        $result .= '<div class="poznamka browserpoznamka">' . str_replace("\n", '<BR/>', questform_convert($detailtext)) . '</div>';
        $result .= '</td>';

        $result .= "</tr>\n";

        return $result;
    }


    /**
     * Vraci data pro zobrazeni ze zadane query.
     * @param Cursor $cursor
     * @return string
     */
    function getDBData($cursor)
    {
        $i = 1;

        $result = '';

        while ($row = $cursor ->fetch()) {
            $this ->createXMLVariables($row);

            $class = ($i % 2) ? 'browsercolor1' : 'browsercolor2';

            $result .= $this ->getLine($row, $class, $i);
            $result .= $this ->getInfoLine($class);

            $i++;
        }

        return $result;
    }


    /**
     * @return string
     */
    function getDBDataFromSQL()
    {
        MessageManager::addDebugLog('XMLBrowser::GetDBDataFromSQL: ' . $this ->sql);

        $cursor = $this ->database ->select($this ->sql);

        if ($cursor) {
            $result =
                $this ->getTableStart() .
                '<thead>' .
                $this ->getTableHeader(0) . // sortfieldnumber = 0 (prozatim ?)
                "</thead>\n" .
                "<tbody>\n" .
                $this ->getDBData($cursor) .
                "</tbody></table>\n";

            return $result;

        } else {
            MessageManager::addDbErrorMessage($this ->database, $this ->sql, 'Chyba v XMLBrowseru!');
            return '';
        }
    }


    /**
     * @return string
     */
    function getHTML()
    {
        return $this ->getDBDataFromSQL();
    }
}


class XMLFormElementBrowser extends XMLFormElement
{
    var $f_name;

    function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this ->f_name = $this ->getAttributeValue('name');
    }

    function getHTMLOutput() {
        $browser = new XMLBrowser($this ->xmlForm ->database, $this ->xmlForm);
        $browser ->name = $this -> f_name;

        $className = $this ->getAttributeValue('class');
        if ($className) $browser ->cssClass = $className;

        $fixedCols = $this ->getAttributeValue('fixedcols');
        if ($fixedCols) $browser->fixedCols = $fixedCols;

        /** @var XMLFormElementBrowserPart $sqlElement */
        $sqlElement = $this ->findChildByName('browsersql');
        if (!$sqlElement) {
            return '';
        }
        $browser ->sql = $this ->nahradPromenneSQL($sqlElement ->sqlText);

        /** @var XMLFormElementBrowserPart $fieldElements */
        $fieldElements = $this ->findChildrenByName('browserfield');
        foreach ($fieldElements as $fieldElement) {
            $field = new XMLBrowserField($this ->xmlForm);
            $field ->type = 'normal';
            $field ->format = $fieldElement ->format;
            $field ->style = $fieldElement ->style;
            $field ->sqlFieldIndex = $fieldElement ->sqlIndex;

            $field ->header = $this ->nahradPromenne($fieldElement ->caption);

            // v linku a textu se promenne nahradi az pri pouziti (=na kazdem radku)!
            $field ->link = $fieldElement ->link;
            $field ->xmlText = $fieldElement ->text;

            $browser ->addField($field);
        }

        /** @var XMLFormElementBrowserPart $detailElement */
        $detailElement = $this ->findChildByName('browserdetail');
        if ($detailElement) {
            if ($detailElement ->col != '') $browser ->infoLineCol = $detailElement ->col;
            $browser ->infoLineText = $detailElement ->detailText;
        }

        /** @var XMLFormElementBrowserPart $linkElement */
        $linkElement = $this ->findChildByName('browserlink');
        if ($linkElement) {
            $browser ->rowClickLink = $linkElement ->link;
        }

        return $browser->getHTML();
    }
}



class XMLFormElementBrowserPart extends XMLFormElement
{
    // browsersql
    var $sqlText;

    // browserfield, browserlink
    var $link;
    // browserfield
    var $sqlIndex;
    var $caption;
    var $format;
    var $style;
    var $text;

    // browserdetail
    var $col;
    var $detailText;

    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        switch ($this ->elementName) {
            case 'browsersql' :
                $this ->sqlText = StringUtils::fromUTF8($this ->loadXMLValue($xmlReader));
                break;

            case 'browserfield' :
                $this ->sqlIndex = $this ->getAttributeValue('sqlindex');
                $this ->caption = StringUtils::fromUTF8($this ->getAttributeValue('caption'));
                $this ->format = $this ->getAttributeValue('format');
                $this ->style = $this ->getAttributeValue('style');
                $this ->link = $this ->getAttributeValue('link');
                $this ->text = StringUtils::fromUTF8($this ->getAttributeValue('text'));
                break;

            case 'browserlink' :
                $this ->link = StringUtils::fromUTF8($this ->loadXMLValue($xmlReader));
                break;

            case 'browserdetail' :
                $this ->col = (int)$this ->getAttributeValue('col');
                $this ->detailText = StringUtils::fromUTF8($this ->loadXMLValue($xmlReader) );
                break;
        }
    }
}


/**
 * Zobrazuje pro kazdy den v obdobi jeden radek a na nem nactene hodnoty
 * Musi byt potomek XMLFormElementBrowser, protoze tag
 * Class XMLFormElementDochazkaBrowser
 * @package Ron\RonPortal\XML
 */
class XMLFormElementDochazkaBrowser extends XMLFormElement
{
    function getHTMLOutput() {
        require_once('xml/xmldochazkabrowser.php');
        $browser = new XMLDochazkaBrowser($this ->xmlForm ->database);

        $browser ->filterForm = $this ->xmlForm ->ronPortalFilterForm;

        $browser ->datumOd = $this ->nahradPromenne($this ->getAttributeValue('datumod'));
        $browser ->datumDo = $this ->nahradPromenne($this ->getAttributeValue('datumdo'));

        /** @var XMLFormElementBrowserPart $sqlElement */
        $sqlElement = $this ->findChildByName('browsersql');
        if (!$sqlElement) {
            return '';
        }
        $browser ->sql = $this ->nahradPromenneSQL($sqlElement ->sqlText);

        return $browser->getHTML();
    }

}
