<?php
/**
 * UTF8
 * stanislav.elbl, 03.01.2018
 */

namespace Ron\RonPortal\XML;

use Ron\DateUtils;
use Ron\DB\SQLUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Redirect;
use Ron\Personalistika\Dotaznik;
use Ron\Profiler;
use Ron\RonPortal\NastaveniUzivatele;
use Ron\RonPortal\NeniPravoException;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PredavaciProcesyUtils;
use Ron\RonPortal\RonPortalApp;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\Rozdeleni;
use Ron\RonPortal\UrovenPredavani;
use Ron\RonPortal\User;
use Ron\StringUtils;

require_once('xml/xmlform.php');
require_once('xml/xmlfunctions.php');
require_once('adsapp/rozdeleni.php');
require_once('ron/messagemanager.php');
require_once('ron/profiler.php');


class RonPortalXMLForm extends XMLForm
{
    /** @var int    Id vyplneneho XML formulare v databazi */
    var $id;
    /** @var int */
    var $idSablony;

    /** @var int    0 - rozpracovano atd. - viz tabulka S_StavXMLFormulare */
    var $stav;

    /** @var bool   Priznak ochrany heslem - nastavuje se atributem formulare passwordprotected="1" */
    var $passwordProtected = false;

    //var $f_typ; // 0 = sablona,  1 = jiz vyplneny dotaznik - ODSTRANENO, staci testovat, jestli je nastaveno $id

    /**
     * @var bool Priznak, ze se nema formular ukladat do vyplnenych formularu - pouze pro stare formulare bez "mode"
     * @deprecated
     */
    var $neukladat;


    /** @var int Platnost sablony od */
    var $platnostOd;
    /** @var int Platnost sablony do */
    var $platnostDo;

    /** @var string     Kod predavaciho procesu */
    var $predavaciProces;
    /** @var string     Vstupni uroven predavaciho procesu */
    var $pocatecniUroven;
    /** @var int        Aktualni uroven predavaciho procesu (viz tabulka PoradiPredavani) */
    var $poradiPredavani;
    // pokud se xml ridi predavacim procesem, tak je zde aktualni uroven
    /** @var string     Kod aktualni urovne predavani */
    var $kodUrovne;
    /** @var bool       Priznak, ze se na aktualni urovni predavani zobrazuji vlastni formulare */
    var $poradiPredavaniVlastni;
    /** @var bool       Priznak, ze se na aktualni urovni predavani nekontroluje hierarchie */
    var $poradiPredavaniNeHeirarchie;
    /** @var int        Typ rozdeleni pro predavaci proces */
    var $typRozdeleniProcesu;

    /** @var string     Typ prava, kterym se ridi zobrazeni XML formulare */
    var $typPrava;

    /** @var string     Popis sablony/formulare */
    var $popis;
    var $dodatecnyPopis;
    var $poznamka;
    var $rozliseni1;
    var $rozliseni2;
    var $rozliseni3;
    var $rozliseni4;

    var $rozliseni5;
    var $rozliseni6;
    var $rozliseni7;
    var $rozliseni8;
    var $rozliseni9;
    var $rozliseni10;

    var $rc; // vybrana osoba, ktere se formular tyka

    var $pp_RC; // vybrany pracovni pomer
    var $osobniCislo;
    var $datumNastupu;

    /* @var int         Pocatek prirazeneho obdobi */
    var $pocatekObdobi;
    /* @var int         Konec prirazeneho obdobi */
    var $konecObdobi;

    /** @var int */
    var $kodTypuRozdeleni;
    /** @var string     Prirazene rozdeleni */
    var $rozdeleni;

    /** @var string[]   Seznam id novych dokumentu pro dodatecne vlozeni do DokumentXMLFormulare */
    var $noveDokumenty = [];
    /** @var bool       Priznak, ze je formular smazan (pokud v XML dojde ke zpracovani prikazu <delete> */
    var $deleted = 0;

    /** @var string     uzivatel, ktery formular vlozil do databaze */
    var $rc_Vlozil;


    /** @var JsonOutputObject   pouziva se v XML formularich pro ajax */
    var $jsonOutputObject;

    /** @var \Ron\RonPortal\RonPortalFilterForm */
    var $ronPortalFilterForm;
    /** @var  Dotaznik  instance tridy Dotaznik (dotaznik.php) - sablona slouzi pro zobrazeni daneho hodnoceni/testu */
    var $dotaznik;


    // --- PRO XSL -----
    /**
     * Pouziva se ve starych XML formularich zpracovavanych pomoci XSL
     * @var \Ron\RonPortal\Dochazka\EditaceMesicnichHromadna
     * @deprecated
     */
    var $f_editacemesicnich;

    var $f_xmlsablona_after; // id xml sablony, ktera se ma zobrazit po zavreni tohoto XML

    /** @var  \Ron\RonPortal\XML\XMLBrowser */
    var $f_browser;
    /** @var  \Ron\RonPortal\RonPortalDBRecord */
    var $f_dbzaznam; // editace zaznamu s logovanim

    /** @var \Ron\RonPortal\ReportServer */
    var $f_reportServer;
    //var $f_reportsocket; // socket pro komunikaci s report serverem

    var $f_mapovanimenu_pokracovat; // slouzi pro mapovani xml formular v menu - priznak, ze se ma provest defaultni akce
    var $f_mapovanimenu_newmenuid; // je mozne zmenit akci, ktera se pak provede
    // --- PRO XSL - KONEC -----

    /** @var Redirect   nastavi se pri volani udalosti, pokud v ni byl pouzit <redirect> */
    static $eventRedirect;



    function __construct($database)
    {
        if (!$database) throw new \Exception("Chyba v konstruktoru RonPortalXMLForm! Neni zadan parametr database!");

        parent::__construct($database);

        $this ->id = 0;
        $this ->idSablony = 0;

        $this ->showHeader = true;

        global $xmlform;
        $xmlform = $this;
    }

    /**
     * Implementace tagu specifickych pro Portal
     * @param XMLFormElement $parentElement
     * @param string $tag
     * @return XMLFormElement
     */
    public function createElementForTag($parentElement, $tag)
    {
        switch ($tag) {
            case 'filtrpodleprav' :
                require_once('xml/xml_commands.php');
                return new XMLFormElementFiltrPodlePrav($this, $parentElement);

            case 'prava' :
                require_once('xml/xml_commands.php');
                return new XMLFormElementPrava($this, $parentElement);

            case 'delete' :
                require_once('xml/xml_commands.php');
                return new XMLFormElementDelete($this, $parentElement);

            case 'stav' :
            case 'uzavrit' :
            case 'nastavpodrobnostiformulare' :
            case 'nastavosobuformulare' :
            case 'nastavpracovnipomer' :
            case 'nastavobdobiformulare' :
            case 'nastavrozdeleniformulare' :
                require_once('xml/xml_commands.php');
                return new XMLFormElementPodrobnosti($this, $parentElement);

            case 'dochazkabrowser' :
                require_once('xml/xmlbrowser.php');
                return new XMLFormElementDochazkaBrowser($this, $parentElement);

            case 'hlavickahodnoceni' :
                require_once('xml/xml_hodnoceni.php');
                return new XMLFormElementHlavickaHodnoceni($this, $parentElement);
            case 'otazkyhodnoceni' :
                require_once('xml/xml_hodnoceni.php');
                return new XMLFormElementOtazkyHodnoceni($this, $parentElement);
            case 'otazkahodnoceni' :
                require_once('xml/xml_hodnoceni.php');
                return new XMLFormElementOtazkaHodnoceni($this, $parentElement);
            case 'poznamkahodnoceni' :
                require_once('xml/xml_hodnoceni.php');
                return new XMLFormElementPoznamkaHodnoceni($this, $parentElement);
            case 'tlacitkahodnoceni' :
                require_once('xml/xml_hodnoceni.php');
                return new XMLFormElementTlacitkaHodnoceni($this, $parentElement);
            case 'ulozodpovedihodnoceni' :
                require_once('xml/xml_hodnoceni.php');
                return new XMLFormElementUlozHodnoceni($this, $parentElement);
            case 'prepocitejhodnoceni' :
                require_once('xml/xml_hodnoceni.php');
                return new XMLFormElementPrepocitejHodnoceni($this, $parentElement);
        }

        if (file_exists('xmltag/' . $tag . '.php')) {
            return require 'xmltag/' . $tag . '.php';
        }

        return parent::createElementForTag($parentElement, $tag);
    }

    /**
     * Nacte dotaznik se zadanym id - bud z tabulky SablonaDotazniku, nebo z tabulky Dotaznik
     * @param $id
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function nactiXMLSablonu($id)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT SablonaXMLFormulare.Popis, ' . $this ->database ->castWideMemo('SablonaXMLFormulare.ObsahXML') . ', ' . // 0,1
            ' SablonaXMLFormulare.SouborXML, SablonaXMLFormulare.PredavaciProces, ' . // 2,3
            ' PredavaciProces.KodTypuRozdeleni, SablonaXMLFormulare.TypPrava, SablonaXMLFormulare.PlatnostOd, SablonaXMLFormulare.PlatnostDo ' . // 4-7
            ' FROM SablonaXMLFormulare LEFT OUTER JOIN PredavaciProces ' .
            '   ON PredavaciProces.Kod = SablonaXMLFormulare.PredavaciProces ' .
            ' WHERE Id = ' . $params ->addParameter($id);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) throw new \Exception("Chyba - formular s ID = $id nenalezen!");

        $this ->popis = $row[0];
        $this ->xml = $row[1];

        // nacteni sablony - ze souboru nebo z databaze
        $this ->id = 0;
        $this ->idSablony = $id;
        //$this ->f_typ = 0;

        $this ->platnostOd = $this ->database ->stringToTimestamp($row[6]);
        $this ->platnostDo = $this ->database ->stringToTimestamp($row[7]);

        $this ->stav = 0; // rozpracovano

        if (!$this ->xml && $row[2]) {
            // neni zadan obsah a je zadan soubor
            $this ->xml = file_get_contents($row[2]);
        }

        $this ->predavaciProces = $row[3];
        $this ->pocatecniUroven = null; // urci se az podle vybrane osoby
        $this ->poradiPredavani = null;
        $this ->poradiPredavaniVlastni = false;
        $this ->poradiPredavaniNeHeirarchie = false;

        $this ->kodUrovne = null;
        $this ->typRozdeleniProcesu = $row[4];

        $this ->typPrava = $row[5];

        $this ->rc = null;
        $this ->pp_RC = null;
        $this ->osobniCislo = null;
        $this ->datumNastupu = null;

        $this ->pocatekObdobi = null;
        $this ->konecObdobi = null;
        $this ->kodTypuRozdeleni = null;
        $this ->rozdeleni = null;
    }



    /**
     * @param $id
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function nactiVyplnenyFormular($id)
    {
        $id = (int)$id;

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT VyplnenyXMLFormular.Popis, ' . $this ->database ->castWideMemo('VyplnenyXMLFormular.ObsahXML') . ', RC, ' . // 0, 1, 2
            ' PP_RC, PP_OsobniCislo, PP_DatumNastupu, ' . // 3, 4, 5
            ' VyplnenyXMLFormular.Stav, ' . // 6
            ' VyplnenyXMLFormular.KodProcesu, VyplnenyXMLFormular.PocatecniUroven, ' . // 7, 8
            ' VyplnenyXMLFormular.PoradiPredavani, PoradiPredavani.KodUrovne, ' . // 9, 10
            ' VyplnenyXMLFormular.Obdobi, VyplnenyXMLFormular.KodTypuRozdeleni, VyplnenyXMLFormular.Rozdeleni, ' . // 11, 12, 13

            ' VyplnenyXMLFormular.DodatecnyPopis, ' . $this ->database ->castWideMemo('VyplnenyXMLFormular.Poznamka') . ', ' . // 14,15
            ' VyplnenyXMLFormular.Rozliseni1, VyplnenyXMLFormular.Rozliseni2, VyplnenyXMLFormular.Rozliseni3, VyplnenyXMLFormular.Rozliseni4, ' . // 16..19

            ' PredavaciProces.KodTypuRozdeleni, VyplnenyXMLFormular.RC_Vlozil, ' . // 20, 21
            ' PoradiPredavani.Vlastni, PoradiPredavani.NekontrolovatHierarchii, ' . // 22, 23
            ' VyplnenyXMLFormular.IdSablony, ' . $this ->database ->castWideMemo('SablonaXMLFormulare.ObsahXML') . ', SablonaXMLFormulare.SouborXML, ' . // 24..26
            ' VyplnenyXMLFormular.Rozliseni5, VyplnenyXMLFormular.Rozliseni6, VyplnenyXMLFormular.Rozliseni7, VyplnenyXMLFormular.Rozliseni8, VyplnenyXMLFormular.Rozliseni9, VyplnenyXMLFormular.Rozliseni10 ' . // 27..32
            ' FROM (VyplnenyXMLFormular LEFT OUTER JOIN PoradiPredavani ' .
            '   ON VyplnenyXMLFormular.KodProcesu = PoradiPredavani.KodProcesu ' .
            '   AND VyplnenyXMLFormular.PocatecniUroven = PoradiPredavani.PocatecniUroven ' .
            '   AND VyplnenyXMLFormular.PoradiPredavani = PoradiPredavani.Poradi ' .
            ' ) ' .
            ' LEFT OUTER JOIN PredavaciProces ' .
            ' ON PredavaciProces.Kod = VyplnenyXMLFormular.KodProcesu ' .
            // sablona
            ' LEFT OUTER JOIN SablonaXMLFormulare ' .
            ' ON SablonaXMLFormulare.Id = VyplnenyXMLFormular.IdSablony' .

            ' WHERE VyplnenyXMLFormular.Id = ' . $params ->addParameter($id);

        $cursor = $this ->database ->getSqlResult($sql, $params);

        $row = $cursor ->fetch();
        $cursor ->close();
        if (!$row) {
            throw new \Exception("Chyba - formular s ID = $id nenalezen!");
        }

        $this ->popis = $row[0];
        $this ->xml = $row[1];

        //$this ->f_typ = 1;

        // nacteni jiz vyplneneho dotazniku
        $this ->id = $id;

        // obsah sablony
        $this ->idSablony = (int)$row[24];
        $this ->xmlSablony = $row[25];
        if (!$this ->xmlSablony  &&  $row[26]) {
            // neni zadan obsah a je zadan soubor
            $this ->xmlSablony = file_get_contents($row[26]);
        }

        $this ->stav = $row[6];

        $this ->predavaciProces = $row[7];
        $this ->pocatecniUroven = $row[8];
        $this ->poradiPredavani = $row[9];
        $this ->kodUrovne = $row[10]; // kod aktualni urovne predavani
        $this ->typRozdeleniProcesu = $row[20];
        $this ->poradiPredavaniVlastni = $row[22];
        $this ->poradiPredavaniNeHeirarchie = $row[23];

        $this ->rc = $row[2];
        $this ->pp_RC = $row[3];
        $this ->osobniCislo = $row[4];
        $this ->datumNastupu = $this ->database ->stringToTimestamp($row[5]);

        $this ->pocatekObdobi = $this ->database ->stringToTimestamp($row[11]);
        if ($this ->pocatekObdobi) $this ->konecObdobi = DateUtils::lastDayOfMonth($this ->pocatekObdobi);
        $this ->kodTypuRozdeleni = $row[12];
        $this ->rozdeleni = $row[13];

        $this ->dodatecnyPopis = $row[14];
        $this ->poznamka = $row[15];
        $this ->rozliseni1 = $row[16];
        $this ->rozliseni2 = $row[17];
        $this ->rozliseni3 = $row[18];
        $this ->rozliseni4 = $row[19];
        $this ->rozliseni5 = $row[27];
        $this ->rozliseni6 = $row[28];
        $this ->rozliseni7 = $row[29];
        $this ->rozliseni8 = $row[30];
        $this ->rozliseni9 = $row[31];
        $this ->rozliseni10 = $row[32];

        $this ->rc_Vlozil = $row[21];

        // Nastavi priznak readonly
        $this ->readOnly = false;

        if ($this ->stav >= 128) {
            // jiz je uzavreno
            $this ->readOnly = true;
        } else if ($this ->kodUrovne) {
            // kontrola na prava v predavacim procesu
            if ($this ->poradiPredavaniVlastni) {
                // uroven nastavena na "pouze vlastni"
                $this->readOnly = strcasecmp($this ->rc, User::getRC())  &&  strcasecmp($this ->pp_RC, User::getRC());
            } else {
                if (!PredavaciProcesyUtils::overUrovenPredavani($this ->database, $this->kodUrovne)) $this->readOnly = true;
            }
        }
    }


    /**
     * Nacte dotaznik se zadanym id - bud z tabulky SablonaXMLFormulare, nebo z tabulky VyplnenyXMLFormular
     * @param int $id
     * @param bool $zeSablony
     * @return bool
     * @throws \Exception
     * @throws \Ron\RonPortal\NeniPravoException
     */
    function nactiXMLFormular($id, $zeSablony)
    {
        MessageManager::addDebugMessage("nactiXMLFormular $id " . ($zeSablony ? '(ze sablony)' : ''));
        if (!$id) {
            throw new \Exception('Chyba ve funkci NactiXMLFormular - neni zadan id formulare');
        }

        if (!User::userAuthenticated()) {
            if (!$zeSablony) throw new \Exception('Uzivatel neni prihlasen!');
        }

        $this ->usePostValues = false;

        if ($zeSablony) {
            $this ->nactiXMLSablonu($id);

            if ($this ->platnostOd  &&  $this ->platnostOd > time()) {
                throw new \Exception('XML formular jeste neni povolen (nastaveni Platnost od)!');
            }

            if ($this ->platnostDo  &&  $this ->platnostDo < DateUtils::today()) {
                throw new \Exception('XML formular jeste neni povolen (nastaveni Platnost do)!');
            }

            if (!User::userAuthenticated()) {
                // uzivatel neni prihlasen - kontrola nazvu formulare
                if (substr(strtolower($this ->popis), 0, 10) != 'login_page') {
                    throw new \Exception('formular nelze zobrazit bez prihlaseni (spatny nazev formulare)!');
                }

                if ($this ->typPrava) {
                    throw new \Exception('formular nelze zobrazit bez prihlaseni (ma prirazene pravo)!');
                }

                if ($this ->predavaciProces) {
                    throw new \Exception('formular nelze zobrazit bez prihlaseni (ma prirazeny predavaci proces)!');
                }
            } else if ($this ->typPrava) {
                $prava = Prava::vratPrava($this ->typPrava);
                if (!$prava ->jsouNejakePrava()) {
                    throw new NeniPravoException();
                }
            }
        } else {
            $this ->nactiVyplnenyFormular($id);

            if (!$this ->jePravoNaFormular()) {
                throw new NeniPravoException();
            }
        }

        if (!$this ->xml) {
            throw new \Exception("Chyba - formular s ID = $id je prazdny!");
        }

        $this ->nastavXmlAction();
    }


    /**
     * nacte puvodni sablonu nebo vyplneny formular, ktery uzivatel odeslal
     * @throws \Exception
     * @throws \Ron\RonPortal\NeniPravoException
     */
    function nactiOdeslanyFormular()
    {
        $id = $_POST['id'];
        $idsablony = $_POST['idsablony'];

        $zeSablony = !(int)$id;

        $this ->nactiXMLFormular($zeSablony ? $idsablony : $id, $zeSablony);
        $this ->usePostValues = true;
    }


    /**
     * Vraci true, pokud ma uzivatel na tento formular pravo
     * @return bool
     */
    function jePravoNaFormular()
    {
        global $_XML_jenPodrizene; //pro zachovani funkcnosti v appdbset, pokud budou chtit zmenit, musi se zrusit v appdbset
        $XML_jenPodrizene = $_XML_jenPodrizene == 1 ? true : RonPortalSettings::getIntValue('jenvlastnirozdelenihodnoceni') == 2;

        //10.1.2020 uprava funkcnosti, true vraci az na konci, aby bylo mozne provest kontrolu a nastaveni readonly stavu
        //pro stavy, kdy uzivatel formular zalozil, vyplnil nebo ma pravo na vsechny formulare - umozni do formulare pristup v readonly modu,
        // editaci povoli jen pokud ma uzivatel pravo na osobu, pracovni pomer nebo rozdeleni pro ktere je formular vytvoren, nebo je nastaveno nekontrolovat hierarchii
        $readOnlyDlePrava = false;
        $jePravo = false;

        MessageManager::addDebugMessage('Kontrola prava na formular');

        if ($this ->rc_Vlozil == User::getRC()) {
            $readOnlyDlePrava = true;
            $jePravo = true;
        }

        if (Prava::vratHodnotuPrava('xml') == 3) {
            // ma pravo na vsechny XML formulare
            MessageManager::addDebugMessage('Uzivatel ma pravo na vsechny formulare');
            $readOnlyDlePrava = true;
            $jePravo = true;
        }

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT RC FROM XMLFormularVyplnil ' .
            ' WHERE RC = ' . $params ->addParameter(User::getRC()) .
            ' AND IdFormulare = ' . $params ->addParameter($this ->id);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        if ($row) {
            // aktualni uzivatel vyplnil tento formular => ma na nej pravo
            $readOnlyDlePrava = true;
            $jePravo = true;
        }

        if (!$this ->predavaciProces) {
            // neridi se predavacim procesem
            MessageManager::addDebugMessage(' - neni nastaven predavaci proces formulare = neni pravo');
            return false;
        }

        if (!PredavaciProcesyUtils::jeVPredavacimProcesu($this ->database, $this ->predavaciProces, $this ->pocatecniUroven)) {
            // uzivatel neni v predavacim procesu => nema pravo na editaci formulare (jen cteni)
            MessageManager::addDebugMessage(' - uzivatel neni v predavacim procesu formulare = jen cteni');
            $readOnlyDlePrava = true;
            $jePravo = true;
        }

        if ($this ->poradiPredavaniNeHeirarchie) {
            MessageManager::addDebugMessage('Na aktualni urovni predavani se nekontroluje hierarchie = uzivatel ma pravo na formular');
            $readOnlyDlePrava = false;
            $jePravo = true;
        }

        // zkontroluje jestli je pro vlastni nebo podrizene rozdeleni
        $typrozdeleni = $this ->typRozdeleniProcesu;
        if (!$typrozdeleni) $typrozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI);

        if (!$typrozdeleni) {
            MessageManager::addDebugMessage(' - neni nastaven typ rozdeleni pro kontrolu prav = neni pravo');
            return false; // neni nastaven typ rozdeleni
        }

        if ($this ->rc) {
            MessageManager::addDebugMessage(' - kontroluji pravo podle osoby');
            // kontrola, jestli je uzivatel ve stejnem nebo nadrazenem rozdeleni jako vybrana osoba
            if (Rozdeleni::jePodrizeny($this ->database, User::getRC(), $this ->rc, null, null, $typrozdeleni, $XML_jenPodrizene ? false : true, time(), time())) {
                $readOnlyDlePrava = false;
                $jePravo = true;
            }
        }

        if ($this ->pp_RC) {
            MessageManager::addDebugMessage(' - kontroluji pravo podle pracovniho pomeru');
            // kontrola, jestli je uzivatel ve stejnem nebo nadrazenem rozdeleni jako vybrany prac. pomer
            if (Rozdeleni::jePodrizeny($this ->database, User::getRC(), $this ->pp_RC, $this ->osobniCislo, $this ->datumNastupu, $typrozdeleni, $XML_jenPodrizene ? false : true, time(), time())) {
                $readOnlyDlePrava = false;
                $jePravo = true;
            }
        }

        if ($this ->kodTypuRozdeleni && $this ->rozdeleni) {
            MessageManager::addDebugMessage(" - kontroluji pravo podle rozdeleni (typ {$this ->kodTypuRozdeleni} kod {$this ->rozdeleni})");
            if (Rozdeleni::jeVRozdeleni($this ->database, User::getRC(), null, $this ->kodTypuRozdeleni, $this ->rozdeleni, $XML_jenPodrizene ? 2 : true, time(), time())) {
                $readOnlyDlePrava = false;
                $jePravo = true;
            }
        }

        if ($jePravo) {
            if (!$this->readOnly)
                $this->readOnly = $readOnlyDlePrava;
            return true;
        }

        $this ->addDebugMessage(' - neni pravo na formular!');

        return false;
    }



    /**
     * Zapise aktualniho uzivatele do seznamu osob, ktere vyplnily formular
     */
    function zapisUzivatele()
    {
        $params = $this->database->createParameters();

        $sql =
            'INSERT INTO XMLFormularVyplnil (RC, IdFormulare, DatumCasZmeny, NovePoradiPredavani, NovyStavFormulare)
                VALUES('. $params->addParameter(User::getRC()) . ', ' . $params->addParameter($this->id) . ', ' .
            $params->addParameter($this->database->timestampToParam(time())) . ', ' .
            $params->addParameter($this->poradiPredavani) . ', ' . $params->addParameter($this->stav) . ')';

        $this->database->sql($sql, $params);
    }


    // ----------------------------------------------------------------------------
    // Ulozi dotaznik do databaze - bud vytvori novy dotaznik (pokud je f_typ = 0 - nacteno ze sablony)
    // nebo edituje dotaznik s prislusnym id
    function save()
    {
        if (!$this ->xml) return MessageManager::addErrorMessage('Chyba pri ukladani XML formulare - obsah je prazdny!');

        if (!$this ->id) { // nacteno ze sablony
            $this ->nastavPredavaciProcesPodleUzivatele();
        }

        require_once('adsapp/ronportaldbrecord.php');
        $rec = new RonPortalDBRecord($this ->database, 'VyplnenyXMLFormular');

        $fId = $rec ->addField('Id', 'int', true);
        $rec ->addField('IdSablony', 'int', true, 0, $this ->idSablony);

        $rec ->addField('Stav', 'int', false, 0, $this ->stav);

        $rec ->addField('Popis', '', true, 60, $this ->popis);

        $rec ->addField('RC', '', false, 15, $this ->rc);

        $rec ->addField('PP_RC', '', false, 15, $this ->pp_RC);
        $rec ->addField('PP_OsobniCislo', '', false, 15, $this ->osobniCislo);
        $rec ->addField('PP_DatumNastupu', 'timestamp', false, 0, $this ->datumNastupu);

        $rec ->addField('Obdobi', 'timestamp', false, 0, $this ->pocatekObdobi);

        //if ($this ->kodTypuRozdeleni && $this ->rozdeleni) {  // TODO: odstranit podminku? takto to nejde vynulovat!
            $rec ->addField('KodTypuRozdeleni', 'int', false, 0, ($this ->kodTypuRozdeleni  &&  $this ->rozdeleni) ? $this ->kodTypuRozdeleni : null);
            $rec ->addField('Rozdeleni', '', false, 30, ($this ->kodTypuRozdeleni  &&  $this ->rozdeleni) ? $this ->rozdeleni : null);
        //}

        if ($this ->predavaciProces && $this ->pocatecniUroven && $this ->poradiPredavani) {
            $rec ->addField('KodProcesu', '', false, 0, $this ->predavaciProces);
            $rec ->addField('PocatecniUroven', '', false, 0, $this ->pocatecniUroven);
            $rec ->addField('PoradiPredavani', 'int', false, 0, $this ->poradiPredavani);
        }

        $rec ->addField('DodatecnyPopis', '', false, 120, $this ->dodatecnyPopis);
        $rec ->addField('Poznamka', 'memo', false, 0, $this ->poznamka);

        $rec ->addField('Rozliseni1', '', false, 40, $this ->rozliseni1);
        $rec ->addField('Rozliseni2', '', false, 40, $this ->rozliseni2);
        $rec ->addField('Rozliseni3', '', false, 40, $this ->rozliseni3);
        $rec ->addField('Rozliseni4', '', false, 40, $this ->rozliseni4);
        $rec ->addField('Rozliseni5', '', false, 40, $this ->rozliseni5);
        $rec ->addField('Rozliseni6', '', false, 40, $this ->rozliseni6);
        $rec ->addField('Rozliseni7', '', false, 40, $this ->rozliseni7);
        $rec ->addField('Rozliseni8', '', false, 40, $this ->rozliseni8);
        $rec ->addField('Rozliseni9', '', false, 40, $this ->rozliseni9);
        $rec ->addField('Rozliseni10', '', false, 40, $this ->rozliseni10);

        $rec ->addField('ObsahXML', 'memo', false, 0, $this ->xml);

        $rec ->addField('DatumVytvoreni', 'timestamp', true, 0, time());
        $rec ->addField('RC_Vlozil', '', true, 15, User::getRC());
        $rec ->addField('PosledniZmena', 'timestamp', false, 0, time());

        if (!$this ->id) { // nacteno ze sablony
            $newId = SQLUtils::newSerialNumber($this ->database, 'VyplnenyXMLFormular', 'Id', '');
            $fId ->setNewValue( $newId );

            $this ->id = $newId;
            //$this ->f_typ = 1; // uz to neni sablona

            $rec ->insertRecord();
        } else {
            if (!$this ->id) return MessageManager::addErrorMessage('Chyba pri ukladani dotazniku - neni zadan Id dotazniku!');

            $params = $this ->database ->createParameters();
            $filter = 'Id = ' . $params ->addParameter($this ->id);
            $rec ->loadRecord($filter, $params);
            $rec ->updateRecord($filter, $params);
        }

        $this ->zapisUzivatele();
        $this ->zapisPrilozeneDokumenty();

        return $this ->id;
    }





    function getVariableValue($varName)
    {
        switch ($varName) {
            case 'XMLID' :
            case 'XML_ID' :
                return $this ->id;
            case 'XML_IDSABLONY' :
                return $this ->idSablony;

            case 'XML_STAV' :
                return $this ->stav;

            case 'USER_LOGIN' :
                return User::getLoginName();
            case 'USER_RC' :
                return User::getRC();
            case 'USER_NAME' :
                return User::getUserName();
            case 'USER_NAME_LOG' :
                require_once('adsapp/ronportalutils.php');
                return RonPortalUtils::getLogUserName();
            case 'USER_PROFIL' :
                return User::getSkupinaUzivatele();

            case 'USER_SETTINGS' :
                require_once('adsapp/usersettings.php');
                return NastaveniUzivatele::loadUserSettings($this ->database);

            case 'XML_PORADIPREDAVANI' :
                return $this ->poradiPredavani;
            case 'XML_PREDAVACIPROCES' :
                return $this ->predavaciProces;
            case 'XML_POCATECNIUROVEN' :
                return $this ->pocatecniUroven;
            case 'XML_KODUROVNE' :
                return $this ->kodUrovne;

            case 'XML_OBDOBI' :
                return $this ->pocatekObdobi;
            case 'XML_KONECOBDOBI' :
                return $this ->konecObdobi;
            case 'XML_POCATEKOBDOBISTR' :
                return ($this ->pocatekObdobi) ? date('j.n.Y', $this ->pocatekObdobi) : '';

            case 'XML_KODTYPUROZDELENI' :
                return $this ->kodTypuRozdeleni;
            case 'XML_ROZDELENI' :
                return $this ->rozdeleni;

            case 'XML_RC' :
                return $this ->rc; // vybrana osoba, ktere se formular tyka

            case 'XML_PP_RC' :
                return $this ->pp_RC; // vybrany pracovni pomer
            case 'XML_PP_OsobniCislo' :
                return $this ->osobniCislo;
            case 'XML_PP_DatumNastupu' :
                return $this ->datumNastupu;

            case 'XML_ROZLISENI1' :
                return $this ->rozliseni1;
            case 'XML_ROZLISENI2' :
                return $this ->rozliseni2;
            case 'XML_ROZLISENI3' :
                return $this ->rozliseni3;
            case 'XML_ROZLISENI4' :
                return $this ->rozliseni4;
            case 'XML_ROZLISENI5' :
                return $this ->rozliseni5;
            case 'XML_ROZLISENI6' :
                return $this ->rozliseni6;
            case 'XML_ROZLISENI7' :
                return $this ->rozliseni7;
            case 'XML_ROZLISENI8' :
                return $this ->rozliseni8;
            case 'XML_ROZLISENI9' :
                return $this ->rozliseni9;
            case 'XML_ROZLISENI10' :
                return $this ->rozliseni10;
            case 'XML_DODATECNYPOPIS' :
                return $this ->dodatecnyPopis;
            case 'XML_POZNAMKA' :
                return $this ->poznamka;
            case 'XML_POPIS' :
                return $this ->popis;

            case 'XML_LINK' :
                return '?menuid=xml&record=' . $this ->idSablony;

            default :
                return parent::getVariableValue($varName);
        }
    }


    /**
     * POZOR! Pouziva se i v eval() pri vyhodnocovani vyrazu!
     * @param string $jmenopromenne
     * @param bool $charsetconvert
     * @return mixed
     * @deprecated
     */
    function VariableValue($jmenopromenne, $charsetconvert)
    {
        $result = $this ->getVariableValue($jmenopromenne);

        if ($charsetconvert) {
            // konverze z windows-1250 do utf-8
            return StringUtils::toUTF8($result);
        } else {
            return $result;
        }
    }


    /**
     * Nastavi hodnotu promenne
     * @param $jmenoPromenne
     * @param $value
     */
    function setVariable($jmenoPromenne, $value)
    {
        switch ($jmenoPromenne) {
            case 'XML_ROZLISENI1' :
                $this ->rozliseni1 = $value;
                break;
            case 'XML_ROZLISENI2' :
                $this ->rozliseni2 = $value;
                break;
            case 'XML_ROZLISENI3' :
                $this ->rozliseni3 = $value;
                break;
            case 'XML_ROZLISENI4' :
                $this ->rozliseni4 = $value;
                break;
            case 'XML_ROZLISENI5' :
                $this ->rozliseni5 = $value;
                break;
            case 'XML_ROZLISENI6' :
                $this ->rozliseni6 = $value;
                break;
            case 'XML_ROZLISENI7' :
                $this ->rozliseni7 = $value;
                break;
            case 'XML_ROZLISENI8' :
                $this ->rozliseni8 = $value;
                break;
            case 'XML_ROZLISENI9' :
                $this ->rozliseni9 = $value;
                break;
            case 'XML_ROZLISENI10' :
                $this ->rozliseni10 = $value;
                break;
            case 'XML_DODATECNYPOPIS' :
                $this ->dodatecnyPopis = $value;
                break;
            case 'XML_POZNAMKA' :
                $this ->poznamka = $value;
                break;
            case 'XML_POPIS' :
                $this ->popis = $value;
                break;

            case 'XML_ROZDELENI' :
                $this ->rozdeleni = $value;
                break;

            default :
                parent::setVariable($jmenoPromenne, $value);
        }
    }



    /**
     * Nastavi zadane obdobi a navic vytvori dalsi promenne (nacte z databaze), ktere pojmenuje
     * <$variableName>_popis, <$variableName>_pocatek, <$variableName>_konec, <$variableName>_pocatekstr, <$variableName>_konecstr
     * @param $variableName
     * @param $pocatek
     * @param $konec
     * @param $popis
     * @param null $uzavreno
     */
    function nastavObdobi($variableName, $pocatek, $konec, $popis, $uzavreno = null)
    {
        // nastavi vybrane obdobi ve formulari
        $this ->pocatekObdobi = $pocatek;
        $this ->konecObdobi = $konec;

        // nastavi promenne
        if ($variableName) {
            if (isset($pocatek)) {
                $this ->variables[$variableName] = $pocatek;

                $this ->variables[$variableName . '_pocatek'] = $pocatek;
                $this ->variables[$variableName . '_pocatekstr'] = date('d.m.Y', $pocatek);

                $this ->variables[$variableName . '_konec'] = $konec;
                $this ->variables[$variableName . '_konecstr'] = date('d.m.Y', $konec);

                $this ->variables[$variableName . '_popis'] = $popis;

                if (isset($uzavreno)) $this ->variables[$variableName . '_uzavreno'] = $uzavreno;
                // u variabilniho se nepouziva
            } else {
                // neni zadano zadne obdobi
                $this ->variables[$variableName . '_pocatek'] = null;
                $this ->variables[$variableName . '_pocatekstr'] = '';
                $this ->variables[$variableName . '_konec'] = null;
                $this ->variables[$variableName . '_konecstr'] = '';
                $this ->variables[$variableName . '_popis'] = '';
                $this ->variables[$variableName . '_uzavreno'] = 0;
            }
        }
    }



    /**
     * Nastavi zadane obdobi a navic vytvori dalsi promenne (nacte z databaze), ktere pojmenuje
     * <varname>_popis, <varname>_pocatek, <varname>_konec, <varname>_pocatekstr, <varname>_konecstr
     * @param string $variableName
     * @param string $aplikace
     * @param int $obdobi
     * @param bool $variabilni
     * @throws \Ron\DB\DBException
     */
    function nastavObdobiAplikace($variableName, $aplikace, $obdobi, $variabilni = false)
    {
        if ($obdobi) {
            $this->addDebugMessage('Nastavuji obdobi: ' . $aplikace . ' ' . date('j.n.Y', $obdobi));
        }

        $this ->pocatekObdobi = $obdobi; // nastavi vybrane obdobi ve formulari

        if ($variableName) $this ->variables[$variableName] = $obdobi;

        $params = $this ->database ->createParameters();
        if ($obdobi && ($aplikace || $variabilni) && $variableName) {
            if ($variabilni && $variabilni !== 'no') {
                $sql =
                    'SELECT KonecObdobi, PopisObdobi FROM VariantniObdobi ' .
                    ' WHERE PocatekObdobi = ' . $params ->addParameter($this ->database ->dateToParam($obdobi));
            } else {
                $sql =
                    'SELECT KonecObdobi, Popis, Uzavreno FROM Obdobi ' .
                    ' WHERE Aplikace = ' . $params->addParameter($aplikace) .
                    ' AND PocatekObdobi = ' . $params ->addParameter($this ->database ->dateToParam($obdobi));
            }

            $cursor = $this ->database ->getSqlResult($sql, $params);

            $row = $cursor ->fetch();

            if ($row) {
                $uzavreno = $variabilni ? null : (int)$row[2];
                $this ->nastavObdobi($variableName, $obdobi, $cursor ->stringToTimestamp($row[0]), $row[1], $uzavreno);
            } else {
                $this ->addErrorMsg('Chyba - vybrane obdobi nebylo nalezeno!');
            }

            $cursor ->close();
        } else if ($variableName) {
            $this ->nastavObdobi($variableName, $obdobi ?: null, null, null, null);
        }
    }



    /**
     * Nastavi zadane rozdeleni a pokud je zadano $variableName, tak navic vytvori dalsi promenne
     * (nacte je z databaze), ktere pojmenuje <$variableName>_popis, <$variableName>_kod (stejna hodnota jako $variableName), <$variableName>_typ
     * @param string $variableName
     * @param int    $typ
     * @param string $rozdeleni
     * @throws \Ron\DB\DBException
     */
    function nastavRozdeleni($variableName, $typ, $rozdeleni)
    {
        $this ->kodTypuRozdeleni = $typ; // nastavi vybrane rozdeleni ve formulari
        $this ->rozdeleni = $rozdeleni; // nastavi vybrane rozdeleni ve formulari

        if ($variableName) {
            if ($rozdeleni) {
                $typ = (int)$typ;

                $this->variables[$variableName] = $rozdeleni;
                $this->variables[$variableName . '_typ'] = $typ;
                $this->variables[$variableName . '_kod'] = $rozdeleni;

                $params = $this ->database ->createParameters();
                $sql =
                    'SELECT Popis FROM Rozdeleni ' .
                    ' WHERE KodTypuRozdeleni = ' . $params ->addParameter((int)$typ) .
                    '   AND Kod = ' . $params ->addParameter($rozdeleni);

                $cursor = $this ->database ->getSqlResult($sql, $params);
                $row = $cursor ->fetch();
                $cursor ->close();

                if ($row) {
                    $this->variables[$variableName . '_popis'] = $row[0];
                } else {
                    $this ->addErrorMsg('Chyba - vybrane rozdeleni nebylo nalezeno!');
                }
            } else {
                // neni vybrano zadne rozdeleni
                $this->variables[$variableName . '_typ'] = 0;
                $this->variables[$variableName . '_kod'] = '';
                $this->variables[$variableName . '_popis'] = '';
                $this->variables[$variableName] = '';
            }
        }
    }


    /**
     * @param string $variableName
     * @param int    $typ
     * @param string $rozdeleni
     */
    function nastavRozdeleni2($variableName, $typ, $rozdeleni)
    {
        $this ->variables[$variableName] = $rozdeleni;
        $this ->variables[$variableName . '_typ'] = $typ;
        $this ->variables[$variableName . '_kod'] = $rozdeleni;
    }


    /**
     * Nastavi zadanou osobu a navic vytvori dalsi promenne (nacte z databaze), ktere pojmenuje
     * <varname>_jmeno, <varname>_prijmeni, <varname>_jmenoprijmeni, <varname>_titul
     * @param $variableName
     * @param $RC
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function nastavOsobu($variableName, $RC)
    {
        $this ->rc = $RC; // nastavi vybranou osobu ve formulari

        if ($variableName) {
            $this ->variables[$variableName] = $RC;
            $this ->variables[$variableName . "_RC"] = $RC;
        }

        if ($RC) {
            $params = $this ->database ->createParameters();
            $sql = 'SELECT RC, Prijmeni, Jmeno, Titul FROM Osoba WHERE RC = ' . $params ->addParameter($RC);
            $cursor = $this ->database ->getSqlResult($sql, $params);
            $row = $cursor ->fetch();

            if ($row) {
                if ($variableName) {
                    $this ->variables[$variableName . "_jmeno"] = $row[2];
                    $this ->variables[$variableName . "_prijmeni"] = $row[1];
                    $this ->variables[$variableName . "_titul"] = $row[3];

                    $jmenoprijmeni = $row[1];
                    if ($row[2]) $jmenoprijmeni = $row[2] . " " . $jmenoprijmeni;
                    if ($row[3]) $jmenoprijmeni = $row[3] . " " . $jmenoprijmeni;
                    $this ->variables[$variableName . "_jmenoprijmeni"] = $jmenoprijmeni;
                }

                $this ->nastavVstupniUrovenProcesu($RC, null, null);
            } else {
                $this ->addErrorMsg('Chyba - vybrana osoba nebyla nalezena!');
            }
        } else {
            if ($variableName) {
                // neni vybrana zadna osoba
                $this ->variables[$variableName . "_jmeno"] = "";
                $this ->variables[$variableName . "_prijmeni"] = "";
                $this ->variables[$variableName . "_jmenoprijmeni"] = "";
                $this ->variables[$variableName . "_titul"] = "";
            }
        }
    }


    /**
     * @param string $variableName
     * @param int $id
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function nastavPracovniPomer($variableName, $id)
    {
        $this ->variables[$variableName] = $id;
        $this ->variables[$variableName . "_id"] = $id;

        if ($id) {
            $params = $this ->database ->createParameters();
            $sql =
                'SELECT Osoba.RC, Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, PracovniPomer.ID' .
                ' FROM Osoba, PracovniPomer' .
                ' WHERE Osoba.RC = PracovniPomer.RC' .
                '   AND PracovniPomer.ID = ' . $params ->addParameter($id) .
                ' ORDER BY Osoba.Prijmeni, Osoba.Jmeno, PracovniPomer.OsobniCislo';

            $cursor = $this ->database ->getSqlResult($sql, $params);
            $row = $cursor ->fetch();
            $cursor ->close();

            if ($row) {
                $datumNastupu = $cursor ->stringToTimestamp($row[5]);

                $this ->pp_RC = $row[0]; // nastavi vybrany PP ve formulari
                $this ->osobniCislo = $row[4];
                $this ->datumNastupu = $datumNastupu;

                $this ->variables[$variableName . '_RC'] = $row[0];
                $this ->variables[$variableName . '_jmeno'] = $row[2];
                $this ->variables[$variableName . '_prijmeni'] = $row[1];
                $this ->variables[$variableName . '_titul'] = $row[3];

                $jmenoprijmeni = $row[1];
                if ($row[2]) $jmenoprijmeni = $row[2] . ' ' . $jmenoprijmeni;
                if ($row[3]) $jmenoprijmeni = $row[3] . ' ' . $jmenoprijmeni;
                $this ->variables[$variableName . '_jmenoprijmeni'] = $jmenoprijmeni;

                $this ->variables[$variableName . '_osobnicislo'] = $row[4];
                $this ->variables[$variableName . '_datumnastupu'] = $datumNastupu;
                $this ->variables[$variableName . '_datumnastupu_str'] = StringUtils::format('date', $datumNastupu);

                // popis pracovniho pomeru - jmeno, osobni cislo, datum nastupu
                $this ->variables[$variableName . '_popis'] = $jmenoprijmeni . ' - ' .
                    \Ron\ronText('Osobni_cislo') . ' ' . $row[4] . ', ' .
                    \Ron\ronText('Datum_nastupu') . ' ' . StringUtils::format('date', $datumNastupu);

                $this ->nastavVstupniUrovenProcesu($this ->pp_RC, $this ->osobniCislo, $datumNastupu);
            } else {
                throw new \Exception('Chyba - vybrana osoba nebyla nalezena!');
            }
        } else {
            // neni vybrana zadna osoba
            $this ->pp_RC = null; // nastavi vybrany PP ve formulari
            $this ->osobniCislo = null;
            $this ->datumNastupu = null;
            $this ->variables[$variableName . '_jmeno'] = '';
            $this ->variables[$variableName . '_prijmeni'] = '';
            $this ->variables[$variableName . '_jmenoprijmeni'] = '';
            $this ->variables[$variableName . '_titul'] = '';
            $this ->variables[$variableName . '_osobnicislo'] = '';
            $this ->variables[$variableName . '_datumnastupu'] = '';
            $this ->variables[$variableName . '_datumnastupu_str'] = '';
            $this ->variables[$variableName . '_popis'] = '';
        }
    }


    /**
     * Pokud se XML ridi predavacim procesem, tak nastavi aktualni uroven podle zvolene osoby
     * (tj. podle zadane Osoby nebo Pracovniho pomeru urci vstupni uroven a podle toho vybere prvni uroven predavani v procesu)
     * Protoze proces muze mit ruzne vstupni urovne, je potreba vybrat takovou "vetev",
     * aby aktualni uzivatel byl na prvni pozoci v procesu a aby vybrana osoba byla na vstupni urovni
     * @param string $rc
     * @param string $osobniCislo
     * @param int $datumNastupu
     * @throws \Exception
     */
    function nastavVstupniUrovenProcesu($rc, $osobniCislo, $datumNastupu)
    {
        if (!$this ->predavaciProces) return; // XML se neridi predavacim procesem

        require_once('adsapp/predavaciproces.php');
        $uroven = UrovenPredavani::vstupniUrovenProcesuPodleOsoby($this ->database, $this ->predavaciProces, $rc, $osobniCislo, $datumNastupu);

        if (!$uroven) {
            throw new \Exception('Chyba - nenalezena vstupni uroven predavaciho procesu podle vybrane osoby!');
        }

        $this ->pocatecniUroven = $uroven ->pocatecniUroven;
        $this ->poradiPredavani = $uroven ->poradi;
        $this ->kodUrovne = $uroven ->kodUrovne;

        $osobainfo = ($osobniCislo && $datumNastupu) ? "os.c. $osobniCislo" : "RC $rc, vsechny pracovni pomery";
        $this ->addDebugMessage("Nastavuji predavaci proces podle vybrane osoby ($osobainfo) a uzivatele: ");
        $this ->addDebugMessage("proces {$this ->predavaciProces}, pocatecni uroven: {$this ->pocatecniUroven}, poradi: {$this ->poradiPredavani}, kod urovne: {$this ->kodUrovne}");
    }


    /**
     * Pokud jeste neni nastavena vstupni uroven tak ji doplni automaticky podle aktualne prihlaseneho uzivatele
     * Pouzije se v pripade, ze formular neobsahuje tag vyberosoby nebo vyberpracovnihopomeru (tyto tagy nastavuji predavaci proces podle vybrane osoby)
     * Pozor! Pokud je v predavacim procesu vice vstupnich urovni na kterych je uzivatel, vybere se (nahodne) jedna z nich
     * @return bool
     */
    function nastavPredavaciProcesPodleUzivatele()
    {
        if (!$this ->predavaciProces) return false; // neridi se procesem - nedela nic

        if ($this ->pocatecniUroven && $this ->poradiPredavani) return true; // jiz je nastaveno - nedela nic

        $this ->addDebugMessage('XMLDebug> Nastavuji predavaci proces podle uzivatele: ');

        /** @var \Ron\RonPortal\UrovenPredavani[] $urovne */
        $urovne = UrovenPredavani::nactiVstupniUrovnePredavani($this ->database, $this ->predavaciProces);
        if ($urovne  &&  count($urovne)) {
            $this ->pocatecniUroven = $urovne[0] ->pocatecniUroven;
            $this ->poradiPredavani = $urovne[0] ->poradi;
            $this ->poradiPredavaniVlastni = $urovne[0] ->vlastni;
            $this ->poradiPredavaniNeHeirarchie = $urovne[0] ->nekontrolovatHierarchii;
            $this ->kodUrovne = $urovne[0] ->kodUrovne;

            $this ->addDebugMessage("proces {$this ->predavaciProces}, pocatecni uroven: {$this ->pocatecniUroven}, poradi: {$this ->poradiPredavani}, kod urovne: {$this ->kodUrovne}");
        } else {
            $this ->addDebugMessage('nenalezena vstupni uroven predavaciho procesu');
        }

        return true;
    }


    /**
     * @param int $smerPredani        1=dale, -1=zpet, -999=na zacatek
     * @return bool
     * @throws \Ron\DB\DBException
     */
    function predej($smerPredani)
    {
        if (!$smerPredani) return true; // nedela nic - nema se predavat

        if (!$this ->predavaciProces) return false;     // XML se neridi predavacim procesem

        $this ->nastavPredavaciProcesPodleUzivatele();  // pokud jeste neni nastaven predavaci proces

        if (!$this ->pocatecniUroven) return false;     // Neni nastavena uroven predavaciho procesu
        if (!$this ->poradiPredavani) return false;

        if ($this ->readOnly) return false;

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Poradi, KodUrovne FROM PoradiPredavani ' .
            ' WHERE KodProcesu = ' . $params ->addParameter($this ->predavaciProces) .
            ' AND PocatecniUroven = ' . $params ->addParameter($this ->pocatecniUroven);

        if ($smerPredani > 0) {
            $sql .=
                ' AND Poradi > ' . $params ->addParameter($this ->poradiPredavani) . // predat dale
                ' ORDER BY Poradi ';
        } else if ($smerPredani == -999) {
            $sql .=
                ' ORDER BY Poradi '; // predat na zacatek
        } else {
            $sql .=
                '   AND Poradi < ' . $params ->addParameter($this ->poradiPredavani) . // vratit zpet
                ' ORDER BY Poradi DESC';
        }

        $this ->addDebugMessage('XMLDebug> Predavam na ' . (($smerPredani > 0) ? 'dalsi' : 'predchozi') . ' uroven predavani: ');

        $cursor = $this ->database ->getSqlResult($sql, $params);

        $row = $cursor ->fetch();
        $pocet = ($smerPredani == -999) ? 0 : (abs($smerPredani) - 1);
        while ($pocet  &&  $row) {
            $pocet --;
            $row = $cursor ->fetch();
        }

        if ($row) {
            $this ->poradiPredavani = $row[0];
            $this ->kodUrovne = $row[1];

            if ($smerPredani > 0) $this ->stav = 0; // rozpracovano
            else $this ->stav = 1; // vraceno

            $this ->addDebugMessage("proces {$this ->predavaciProces}, pocatecni uroven: {$this ->pocatecniUroven}, poradi: {$this ->poradiPredavani}, kod urovne: {$this ->kodUrovne}");

            return true;
        } else {
            $this ->addDebugMessage('nenalezena dalsi/prechozi uroven predavaciho procesu');
            // chyba ?
            // aktualni uzivatel nema pravo ?
        }

        // dalsi / predchozi uroven neexistuje
        return false;
    }


    function  zapisPrilozeneDokumenty()
    {
        foreach ($this ->noveDokumenty as $idDokumentu) {
            $params = $this ->database ->createParameters();
            $sql =
                'SELECT * FROM DokumentXMLFormulare ' .
                ' WHERE IdDokumentu = ' . $params ->addParameter((int)$idDokumentu) .
                ' AND IdFormulare = ' . $params ->addParameter((int)$this ->id);

            $cursor = $this ->database ->getSqlResult($sql, $params);
            $row = $cursor ->fetch();
            $cursor ->close();

            if (!$row) {
                // zaznam jeste neexistuje - vlozi novy
                $params = $this ->database ->createParameters();
                $sql =
                    'INSERT INTO DokumentXMLFormulare (IdDokumentu, IdFormulare) ' .
                    ' VALUES (' . $params ->addParameter((int)$idDokumentu) . ', ' .
                    $params ->addParameter((int)$this ->id) . ')';

                $this ->database ->sql($sql, $params);
            }
        }

        // vycisti seznam pripojenych dokumentu
        $this ->noveDokumenty = [];
    }


    /**
     * Smaze dotaznik z databaze a zaloguje smazani
     * @param string $OKInfoMsg
     */
    function delete($OKInfoMsg)
    {
        if (!$this ->id) return; // nelze smazat (dotaznik jeste neni zapsan do databaze)

        require_once('adsapp/ronportaldbrecord.php');

        $rec = new RonPortalDBRecord($this ->database, 'VyplnenyXMLFormular');
        $rec ->addField('Id', 'int');
        $rec ->addField('Stav', 'int');
        $rec ->addField('IdSablony', 'int');
        $rec ->addField('DatumVytvoreni', 'timestamp');
        $rec ->addField('PosledniZmena', 'timestamp');
        $rec ->addField('Popis', '');
        $rec ->addField('DodatecnyPopis', '');
        $rec ->addField('KodProcesu', '');
        $rec ->addField('PocatecniUroven', '');
        $rec ->addField('PoradiPredavani', 'int');
        $rec ->addField('KodTypuRozdeleni', 'int');
        $rec ->addField('Rozdeleni', '');
        $rec ->addField('Obdobi', 'timestamp');
        $rec ->addField('RC', '');
        $rec ->addField('PP_RC', '');
        $rec ->addField('PP_OsobniCislo', '');
        $rec ->addField('PP_DatumNastupu', 'timestamp');

        $params = $this ->database ->createParameters();
        $filter = 'id = ' . $params ->addParameter((int)$this ->id);
        $rec ->deleteRecord($filter, $params);

        $this ->deleted = 1;

        if ($OKInfoMsg) {
            MessageManager::addInfoMessage($OKInfoMsg);
        }
    }


    function process()
    {
        if (RonPortalApp::getDebugMode()) {
            $this ->addDebugMessage('XMLDebug> Process()');
            MessageManager::debugPost();
        }

        parent::process();
    }


    /**
     * Provede kod XML formulare a vraci HTML obsah
     * @return string
     * @throws \Exception
     */
    function getHtml()
    {
        $profilerItem = Profiler::addItem('RonPortalXMLForm::getHtml()', 'XML');
        try {
            //$this ->f_formaction = 'show';

            $this->addDebugMessage(
                'Generovani XML formulare s ID = ' . $this->id . "\n" .
                'Aktualni uroven: ' . $this->kodUrovne . ', ' .
                'Aktualni poradi: ' . $this->poradiPredavani . ($this->poradiPredavaniVlastni ? ' - pouze vlastni' : '') . ($this->poradiPredavaniNeHeirarchie ? ' - nekontrolovat hierarchii' : '') . "\n" .
                'XML akce: ' . $this->xmlAction,
                true
            );

            $this->prvniSekce = true;
            $this->viceSekci = false;

            if (!$this->rootElement) {
                require_once('xml/xmlformelement.php'); // SE: upraveno 10/2018 - aby si tag mohl zapamatovat stav z process pri chybe (napr. <vyrobaosoby>)
                $this->loadXML();
            }

            if ($this->passwordProtected && !RonPortalSession::enablePasswordProtected()) return null;

            if (!(($this->rootElement->getAttributeValue('mode')) || ($this->rootElement->elementName == 'ronxmldata'))) {
                // dokument nema atribut "mode" - pouzije XSL
                $xsl = new \DOMDocument('1.0', 'utf-8');

                $xsl->load('xml/form_show.xsl');

                $xslt = new \XSLTProcessor();
                $xslt->registerPHPFunctions();
                $xslt->importStylesheet($xsl);

                $domdocument = new \DOMDocument('1.0', 'utf-8');
                $domdocument->loadXML($this->xml);

                $result = $xslt->transformToXML($domdocument);

                if ($profilerItem) $profilerItem ->texts[] = '... pouzito XSLT!';
                Profiler::endItem($profilerItem);
                return $result;
            }

            if ($this->rootElement->elementName == 'ronxmldata') {
                $this->rootElement->loadData();

                $this->loadXML($this->xmlSablony);
            }

            if ($this->forbiddenMode()) {
                throw new \Exception('Tento formular neni mozne samostatne zobrazit (mod ' . $this->mode . ')');
            }

            if ($this->mode == self::XML_MODE_HODNOCENI && !$this->dotaznik) {
                throw new \Exception("Chyba v metode XMLFormular::fill - formular s modem hodnoceni nema prirazen dotaznik!");
            }

            $result = $this->rootElement->getHTMLOutput();

            Profiler::endItem($profilerItem);

            return $result;
        } catch (\Exception $e) {
            $profilerItem ->texts[] = 'Exception: ' . $e ->getMessage();
            Profiler::endItem($profilerItem);
            throw $e;
        }
    }



    /**
     * Vyplni dotaznik podle uzivatelem zadanych hodnot - tj. provede transformaci xml
     * @throws \Exception
     */
    function fill()
    {
        $profilerItem = Profiler::addItem('RonPortalXMLForm::fill()', 'XML');
        try {
            if (RonPortalApp::getDebugMode()) {
                $this->addDebugMessage('XMLDebug> Fill()');
                MessageManager::debugPost();
            }

            //$this ->f_formaction = 'fill';
            //$this ->f_connection = $connection;

            $this->prvniSekce = true;
            $this->viceSekci = false;

            $newxml = null;

            require_once('xml/xmlformelement.php');
            $this->loadXML();

            if (!(($this->rootElement->getAttributeValue('mode')) || ($this->rootElement->elementName == 'ronxmldata'))) {
                // dokument nema atribut "mode" - pouzije XSL
                $this->addDebugMessage('Vyplnovani XML, pouzivam stare XSL ...');

                $xsl = new \DOMDocument('1.0', 'utf-8');
                if (!$xsl->load('xml/form_fill.xsl')) {
                    throw new \Exception('Chyba pri nacitani form_fill.xsl');
                }

                $xslt = new \XSLTProcessor();
                $xslt->registerPHPFunctions();
                $xslt->importStylesheet($xsl);

                $domdocument = new \DOMDocument('1.0', 'windows-1250');
                if (!$domdocument->loadXML($this->xml)) {
                    throw new \Exception('Chyba pri nacitani obsahu xml formulare!');
                }

                $newxml = $xslt->transformToXML($domdocument);
                if ($profilerItem) $profilerItem ->texts[] = '... pouzito XSLT!';
            } else {
                $this->addDebugMessage('Vyplnovani XML, nova metoda ...');

                if ($this->rootElement->elementName == 'ronxmldata') {   // &&  $this ->f_id
                    // nacte data z vyplneneho xml
                    $this->rootElement->loadData();
                    $this->loadXML($this->xmlSablony);
                }

                if ($this->forbiddenMode()) {
                    // include, event - nelze zobrazit
                    throw new \Exception('Tento formular neni mozne samostatne zobrazit (mod ' . $this->mode . ')');
                }

                if ($this->mode == self::XML_MODE_PROCESS || $this->mode == self::XML_MODE_SHOW) {
                    throw new \Exception("Chyba v metode XMLFormular::fill - spatny mod formulare ({$this ->mode})");
                }

                if ($this->mode == self::XML_MODE_HODNOCENI && !$this->dotaznik) {
                    throw new \Exception("Chyba v metode XMLFormular::fill - formular s modem hodnoceni nema prirazen dotaznik!");
                }

                $xmlWriter = new \XMLWriter();
                // Output directly to the user
                $xmlWriter->openMemory();
                $xmlWriter->startDocument('1.0', 'windows-1250');

                if ($this->dataMode()) {
                    // ukladaji se jen data
                    $xmlWriter->setIndent(true);

                    $this->rootElement->process(); // nevolat $this ->process (jinak by byla chyba "spatny mod formulare")

                    $xmlWriter->startElement('ronxmldata');
                    $this->writeData($xmlWriter);
                    $xmlWriter->endElement();
                } else {
                    $xmlWriter->setIndent(false);
                    $this->rootElement->writeXMLOutput($xmlWriter);
                }
                $xmlWriter->endDocument();
                $newxml = $xmlWriter->outputMemory();
            }

            if (!$newxml) {
                throw new \Exception('Chyba pri prevodu xml formulare!');
            }

            if ($newxml && !$this->isError()) $this->xml = $newxml;

            Profiler::endItem($profilerItem);
        } catch (\Exception $e) {
            Profiler::endItem($profilerItem);
            if ($profilerItem) $profilerItem ->texts[] = 'Exception: ' . $e ->getMessage();
            throw $e;
        }
    }


    /**
     * Provede zadany XML formular a vraci bud true/false (jestli ne/doslo k chybe) nebo vygenerovany htmlObsah (pokud je returnContent true)
     * @param Database $database
     * @param int    $id
     * @param array  $variables
     * @param string $xmlContext
     * @param bool   $returnContent     pokud je true, tak vraci obsah xml formulare
     * @param bool   $exceptionOnError  Pokud je true, tak v pripade chyby ve formulari vyhodi vyjimku. Jinak jen vraci false.
     * @return bool|string
     * @throws \Exception
     */
    static function xmlEvent($database, $id, $variables, $xmlContext, $returnContent = false, $exceptionOnError = true)
    {
        MessageManager::addDebugMessage("Spoustim XML formular $id, context=$xmlContext");

        global $xmlform;
        $oldXmlForm = $xmlform;
            // zapamatuje si puvodni xml formular - pokud je totiz volana udalost uvnitr formulare (napr. EDITACE_MESICNICH_AFTER),
            // tak je po ukonceni potreba nastavit puvodni hodnotu

        /** @var RonPortalXMLForm $form */
        $form = new static($database);
        $form ->context = $xmlContext;

        $form -> nactiXMLFormular($id, true); // zeSablony = true
        //$rootElement = XMLFormElement::loadXMLForm($form, null, $form -> xml);
        $form ->loadXML();

        $form ->setVariables($variables);

        //$form ->f_formaction = 'event';
        if ($returnContent) {
            $htmlContent = $form->rootElement->getHTMLOutput();
            $xmlform = $oldXmlForm;
            $result = $htmlContent;
        } else {
            $form->event();
            $xmlform = $oldXmlForm;
            $result = !$form ->isError(null, $exceptionOnError); // pokud je returnContent=false, tak v pripade chyby vyhodi vyjimku
        }

        if ($form ->redirect) self::$eventRedirect = $form ->redirect;

        return $result;
    }


    static $xmlEventsCache = [];


    /**
     * @param Database $database
     * @param string $eventName
     * @param array  $variables
     * @param string $xmlContext
     * @param bool   $returnContent     Pokud je true, vraci funkce string - obsah vygenerovanych formularu. Jinak vraci jen true/false
     * @param bool   $exceptionOnError  Pokud je true, tak v pripade chyby ve formulari vyhodi vyjimku. Jinak jen vraci false.
     * @return bool|string
     * @throws \Exception
     */
    public static function xmlEvents($database, $eventName, $variables, $xmlContext, $returnContent = false, $exceptionOnError = true)
    {
        self::$eventRedirect = null;

        $formIds = [];

        if (isset(self::$xmlEventsCache[$eventName])) {
            $formIds = self::$xmlEventsCache[$eventName];
        } else {
            $params = $database->createParameters();
            $sql =
                'SELECT Id, Popis FROM SablonaXMLFormulare ' .
                ' WHERE Popis LIKE ' . $params->addParameter($eventName . '%') .
                ' ORDER BY Popis';

            $cursor = $database->select($sql, $params);

            while ($row = $cursor->fetch()) {
                $formIds[] = $row[0];
            }
            $cursor->close();

            self::$xmlEventsCache[$eventName] = $formIds;
        }

        $result = '';
        foreach ($formIds as $formId) {
            // pokud pri provedeni skriptu dojde k chybe, tak vraci false
            $pom = self::xmlEvent($database, $formId, $variables, $xmlContext, $returnContent, $exceptionOnError);

            if ($returnContent) {
                $result .= $pom;
            } else {
                if (!$pom) {
                    return false;
                }
            }

        }

        return $returnContent ? $result : true;
    }

}
