<?php
/**
 * UTF8
 * stanislav.elbl, 18.09.2018
 */

namespace Ron\RonPortal\XML;

use Ron\RonPortal\AgentManager;
use Ron\RonPortal\Log;
use Ron\RonPortal\TaskManager;
use Ron\StringUtils;

require_once 'xml/xml_commands.php';

class XMLFormElementAgent extends XMLFormElementCommand
{
    function doCommand()
    {
        $app = $this ->getAttributeValue('app', 'ads'); // person/jidelna/cokoliv jineho=ads
        $name = $this ->getAttributeValue('name');
        $background = $this ->getBoolAttribute('background');

        $params = StringUtils::fromUTF8($this ->getAttributeValue('params'));
        $params = $this ->nahradPromenne($params, 'cmd'); // formatovani pro command line (datum ve formatu yyyy-mm-dd

        $defaultParams = $this ->getBoolAttribute('defaultparams', true);

        require_once 'adsapp/agentmanager.php';

        $result = null;
        switch (strtolower($app)) {
            case 'jidelna' :
                AgentManager::executeJidelna($name, $params, $defaultParams);
                break;

            case 'person' :
                $result = AgentManager::executePersonAgent($params, $defaultParams, $background);
                break;

            case 'ads' :
            default :
                $result = AgentManager::executeADS($name, $params, $defaultParams, $background);
                break;
        }

        if ($result) {
            $variablename = $this ->nahradPromenne($this ->getAttributeValue('variablename'));
            if ($variablename) {
                $this ->setVariableValue($variablename, $result ->resultCode, 'resultCode');
                $this ->setVariableValue($variablename, $result ->finalMessage, 'finalMessage');
                $this ->setVariableValue($variablename, $result ->errorMessage, 'errorMessage');
                $this ->setVariableValue($variablename, $result ->logText, 'logText');
                $this ->setVariableValue($variablename, $result ->progressMessage, 'progressMessage');
                $this ->setVariableValue($variablename, $result ->isRunning ? '1' : '0', 'isRunning');
                $this ->setVariableValue($variablename, $result ->taskId, 'taskId');
            }
            require_once('adsapp/ronportallog.php');
            Log::addRecordToLog($this ->xmlForm ->database, 'Start agent task', 'XML', 'ProgressMessage: ' . $result ->progressMessage);
        }
    }
}


class XMLFormElementAgentStatus extends XMLFormElementCommand
{
    function doCommand()
    {
        require_once 'adsapp/taskmanager.php';
        require_once 'adsapp/agentmanager.php';

        $result = null;
        $task = TaskManager::getInststance()->findTask($this->nahradPromenne($this->getAttributeValue('id')));
        TaskManager::getInststance()->updateTask($task);

//        if ($task  &&  $task ->isRunning) {
//            $manager = new AgentManager();
//            $result = $manager->getTaskStatus($task);
//        }

        $variablename = $this->nahradPromenne($this->getAttributeValue('variablename'));

        if ($variablename) {
            if ($task) {
                $this->setVariableValue($variablename, $task->progressMessage, 'progressMessage');
                $this->setVariableValue($variablename, $task->isRunning ? '1' : '0', 'isRunning');
                $this->setVariableValue($variablename, $task->errorMessage, 'errorMessage');
                $this->setVariableValue($variablename, $task->id, 'taskId');

//                if ($result) {
//                    $this->setVariableValue($variablename, $result->resultCode, 'resultCode');
//                    $this->setVariableValue($variablename, $result->finalMessage, 'finalMessage');
//                    $this->setVariableValue($variablename, $result->logText, 'logText');
//                }
            } else {
                $this->setVariableValue($variablename, '0', 'isRunning');
            }
        }
    }
}


class XMLFormElementScript extends XMLFormElementCommand
{
    function doCommand()
    {
        $scriptCategory = StringUtils::fromUTF8($this ->getAttributeValue('scriptcategory'));
        $scriptName = StringUtils::fromUTF8($this ->getAttributeValue('scriptname'));

        $parameter = StringUtils::fromUTF8($this ->getAttributeValue('scriptparameter'));
        $parameter = $this ->nahradPromenne($parameter);

        $app = $this ->getAttributeValue('app', 'ads'); // person/jidelna/cokoliv jineho=ads
        $background = $this ->getBoolAttribute('background');
        // TODO: doplnit parametr background

        require_once('adsapp/agentmanager.php');
        try {
            AgentManager::executeScript($scriptCategory, $scriptName, $parameter, $app);
            $hodnota = 1;
        } catch (\Exception $e) {
            $hodnota = 0;
            $this ->xmlForm ->addErrorMsg($e ->getMessage());
        }

        $variablename = $this ->getAttributeValue('variablename');
        if ($variablename) $this ->setVariableValue($variablename, $hodnota);
    }
}
