<?php
/**
 * stanislav.elbl, 28.01.2019
 */

namespace Ron\RonPortal\XML;

use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalSession;
use Ron\StringUtils;


/**
 * Class XMLFormADSFilterForm
 * @package Ron\RonPortal\XML
 */
class XMLFormADSFilterForm extends XMLFormElement
{
    /** @var \Ron\RonPortal\RonPortalFilterForm */
    var $filterForm;


    private function createFilterForm()
    {
        require_once('adsapp/ronportalfilter.php');

        $this->filterForm = new RonPortalFilterForm($this->xmlForm->database, 'xml&record=' . $this->xmlForm->idSablony);
        if ($this ->getBoolAttribute('useget')) {
            $this ->filterForm ->setUseGet(true, ['menuid' => 'xml', 'record' => $this->xmlForm->idSablony]);
        }

        $autoSubmit = $this->getAttributeValue('autosubmit');
        if (!isset($autoSubmit) || $autoSubmit === '') $this->filterForm->f_autosubmit = 1;
        else $this->filterForm->f_autosubmit = self::getBoolValue($autoSubmit);

        // vnorene tagy adsfilterform_obdobi atd. vyhledaji tento element a nastavi filterForm pro zobrazeni
        $this->getElementsHTMLOutput();

        $this->filterForm->loadData();

        // doplni promenne do XML formulare
        if ($this->filterForm->showObdobi) {
            $this->xmlForm->nastavObdobiAplikace('OBDOBI', $this->filterForm->obdobiAplikace, RonPortalSession::obdobi());
        }

        if ($this->filterForm->f_showrozdeleni) {
            //echo 'jsme tu ' . $this->filterForm->f_typrozdeleni . ', ' . RonPortalSession::rozdeleni($this->filterForm->f_typrozdeleni);
            $this->xmlForm->nastavRozdeleni('ROZDELENI', $this->filterForm->f_typrozdeleni, $this->filterForm->kodRozdeleni);

            if ($this ->filterForm ->showVcetnePodrizenychRozdeleni) {
                $this->setVariableValue('ROZDELENI_vcetnepodrizenych', $this->filterForm->vcetnePodrizenychRozdeleni);
            }
        }

        if ($this->filterForm->f_showrozdeleni2) {
            $this->xmlForm->nastavRozdeleni2('ROZDELENI2', $this->filterForm->f_typrozdeleni2, $this->filterForm->kodRozdeleni2);

            if ($this ->filterForm ->showVcetnePodrizenychRozdeleni2) {
                $this->setVariableValue('ROZDELENI2_vcetnepodrizenych', $this->filterForm->vcetnePodrizenychRozdeleni2);
            }
        }

        if ($this->filterForm->f_showosoba) {
            $this->xmlForm->nastavPracovniPomer('OSOBA', RonPortalSession::vybranaOsoba());
        }

        $this->setVariableValue('adsfilterformsubmit', isset($_POST['adsfilterform']) ? (int)$_POST['adsfilterform'] : 0);
    }


    function getHTMLOutput()
    {
        $this ->createFilterForm();

        // zobrazeni filtru zajisti formular - vyrenderuje az zaroven s formularem
        $this ->xmlForm ->ronPortalFilterForm = $this ->filterForm;

        return '';
    }


    /**
     * Provadi se i v rezimu process, aby se spravne nastavily promenne
     */
    function process()
    {
        $this ->createFilterForm();
    }
}


class XMLFormADSFilterFormPart extends XMLFormElement
{
    /** @var string Filtr pro seznam rozdeleni */
    var $sqlFiltr;

    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        switch ($this ->elementName) {
            case 'adsfilterform_rozdeleni' :
            case 'adsfilterform_rozdeleni2' :
            case 'adsfilterform_osoby' :
                $this ->sqlFiltr = StringUtils::fromUTF8($this ->loadXMLValue($xmlReader) );
                break;



        }
    }


    function process()
    {
        // nastavi nejblizsi nadrazeny adsfilterform
        /* @var  XMLFormADSFilterForm $filterformelement*/
        $filterformelement = $this ->findParentByName('adsfilterform');
        if (!$filterformelement) return;

        switch ($this ->elementName) {
            case 'adsfilterform_obdobi' :
                $filterformelement ->filterForm ->nastavObdobi(true, $this ->getAttributeValue('aplikace'), self::getBoolValue($this ->getAttributeValue('jenotevrene')));
                break;

            case 'adsfilterform_rozdeleni' :
                $typrozdeleni = (int)$this ->nahradPromenne($this ->getAttributeValue('typrozdeleni'));
                $filterformelement ->filterForm ->nastavRozdeleni(true, $typrozdeleni, (int)$this ->getAttributeValue('pouzevlastni'), $this ->sqlFiltr, $this ->getBoolAttribute('pouzitpopistypurozdeleni'));
                $typPrava = $this ->getAttributeValue('prava');
                if ($typPrava) $filterformelement ->filterForm ->f_pravarozdeleni = Prava::vratPrava($typPrava);
                $filterformelement->filterForm->showVcetnePodrizenychRozdeleni = self::getBoolValue($this ->getAttributeValue('showvcetnepodrizenych'));
                break;

            case 'adsfilterform_rozdeleni2' :
                $typrozdeleni = (int)$this ->nahradPromenne($this ->getAttributeValue('typrozdeleni'));
                $filterformelement ->filterForm ->nastavRozdeleni2(true, $typrozdeleni, self::getBoolValue($this ->getAttributeValue('pouzevlastni')), $this ->sqlFiltr, $this ->getBoolAttribute('pouzitpopistypurozdeleni'));
                $filterformelement->filterForm->showVcetnePodrizenychRozdeleni2 = self::getBoolValue($this ->getAttributeValue('showvcetnepodrizenych'));;
                break;

            case 'adsfilterform_osoby' :
                $prava = $this ->getAttributeValue('prava');

                if ($prava) {
                    $prava = Prava::vratPrava($prava);
                    $filterformelement ->filterForm ->nastavOsoby(true, $prava, 0, 0);
                } else {
                    $predavaciproces = $this ->getAttributeValue('predavaciproces');
                    if ( !$predavaciproces ) $predavaciproces = $this ->xmlForm -> predavaciProces;
                    $filterformelement ->filterForm ->nastavOsobyDleProcesu(true, $predavaciproces, $this ->xmlForm -> typRozdeleniProcesu);
                }

                $filterformelement->filterForm->osobyFiltr = $this->sqlFiltr;

                if (self::getBoolValue($this ->getAttributeValue('vsechnyosoby'))) {
                    $filterformelement ->filterForm ->povolitVsechyOsoby = true;
                }
        }
    }


    function getHTMLOutput()
    {
        $this ->process();

        return '';
    }
}
