<?php
/**
 * UTF8
 * stanislav.elbl, 21.06.2017
 * Presunuto z xml/xml_report.php
 */

namespace Ron\RonPortal\XML;


use Ron\RonPortal\Obdobi;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Report;
use Ron\RonPortal\ReportOutputObject;
use Ron\RonPortal\ReportServer;
use Ron\RonPortal\User;
use Ron\StringUtils;
use Ron\XmlLoader;

class XMLFormElementReport extends XMLFormElement
{
    /** @var ReportServer */
    //var $reportServer;

    /** @var string */
    var $variablesXml = '';

    /** @var Report */
    var $report;

    function getXML()
    {
        $reportName = StringUtils::fromUTF8($this ->getAttributeValue('reportname'));
        $reportName = $this ->nahradPromenne($reportName);

        if (!$this ->report  ||  $this ->report ->kod != $reportName) {
            require_once 'adsapp/report.php';
            $this ->report = Report::loadReport($this ->xmlForm ->database, $reportName);
        }

        // TODO: vyresit promenne pro panely ze sestav v aplikaci?
        $globalVariablesXml = '';

        if (!$this ->report ->appReportName) {
            // u starych reportu posle jeste puvodni parametry (pro kompatibilitu)
            $globalVariablesXml .=
//                "    <reportparam name='RC' value='" . StringUtils::xmlTextConvert($this->pp ? $this->pp->rc : '') . "'/>\n" .
//                "    <reportparam name='osobniCislo' value='" . StringUtils::xmlTextConvert($this->pp ? $this->pp->osobniCislo : '') . "'/>\n" .
//                "    <reportparam name='datumNastupu' type='DATE' value='" . ($this->pp ? date('j.n.Y', $this->pp->datumNastupu) : '1.1.1970') . "'/>\n" . // TODO: jake datum?
                // u starych sestav se pouzivala promenna LoginName
                "    <reportparam name='LoginName' value='" . StringUtils::htmlTextConvert(User::getLoginName()) . "'/>\n" .
                "    <reportparam name='RealLoginName' value='" . StringUtils::htmlTextConvert(User::getRealLoginName()) . "'/>\n";
        }

        return
            "  <preparereport kod='" . StringUtils::xmlTextConvert($reportName) ."' sessionId='" . ReportServer::getReportSessionId() . "'>\n" .
            $globalVariablesXml .
            $this ->variablesXml .
            "  </preparereport>";
    }


    private function getReportHtml($reportKey)
    {
        $text = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('text')));

        $format = $this ->getAttributeValue('format');
        $formatArray = $format ? explode(';', $format) : null;

        $attrs = [
            'class' => 'text_link report',
            'target' => '_blank'
        ];

        // doplni atributy oznacene prefixem html:
        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        if ($formatArray  &&  sizeof($formatArray) > 1) {
            // jsou zadane formaty, napr. "pdf;xls"  nebo ";xls" (prvni se pouzije defaultni)
            $resultStr = '';
            $textZobrazen = false;

            foreach ($formatArray as $format_item) {
                if ($format_item) {
                    $resultStr .= " <a $attrs_str href='?menuid=xmlform&amp;action=report&amp;format={$format_item}&amp;reportkey={$reportKey}'>$format_item</a> ";
                } else {
                    // defaultni format
                    $resultStr .= " <a $attrs_str href='?menuid=xmlform&amp;action=report&amp;reportkey={$reportKey}'>$text</a> ";
                    $textZobrazen = true; // text uz se dale nebude zobrazovat
                }
            }

            $result =
                '<span class="reportlinks">' .
                ( $textZobrazen ? '' : $text) .
                $resultStr .
                "</span>\n";
        } else {
            // format neni zadan, nebo je zadan jenom jeden
            $formatstr = $format ? '&amp;format=' . $format : '';
            $result =
                "<a $attrs_str href='?menuid=xmlform&amp;action=report{$formatstr}&amp;reportkey={$reportKey}'>$text</a>";
        }

        return $result;
    }


    function getHTMLOutput()
    {
        require_once('adsapp/reportserver.php');

        // zpracuje podrizene tagy - ty doplni $variablesXml
        $this ->variablesXml = '';
        $this ->getElementsHTMLOutput();

        try {
            $reportServer = ReportServer::createInstance();

            $xmlResponse = $reportServer->sendXmlRequest($this->getXML());
            require_once 'ron/xmlloader.php';
            $a = XmlLoader::loadXmlToArray($xmlResponse);

            //var_dump($a);
            $reportKey = $a['#attr']['key'];
            //$reportKey = trim($this ->reportServer ->sendRequest('PREPAREREPORT ' . ReportServer::getReportSessionId() . "\n", true)); // jednoradkova odpoved = true

            $reportServer ->sendMessage("EXIT\n");

            $reportServer ->close();

            $result = $this ->getReportHtml($reportKey);
        } catch (\Exception $e) {
            $this ->xmlForm ->addErrorMsg($e ->getMessage());
            return '';
        }

        /*
        try {
            $this ->reportServer = ReportServer::createInstance();

            $reportKey = trim($this ->reportServer ->sendRequest('PREPAREREPORT ' . ReportServer::getReportSessionId() . "\n", true)); // jednoradkova odpoved = true
            if ( !$reportKey ) {
                throw new \Exception('Chyba pri komunikaci s report serverem! Report server nevratil klic sestavy.');
            }

            $reportName = StringUtils::fromUTF8($this ->getAttributeValue('reportname'));
            $reportName = $this ->nahradPromenne($reportName);

            $this ->reportServer ->sendMessage("REPORTNAME $reportName");
            $this ->reportServer ->sendMessage('VAR LoginName=' . User::getLoginName());

            $result = $this ->getReportHtml($reportKey);

            $this ->getElementsHTMLOutput();

            $this ->reportServer ->sendMessage("EXIT\n");

            $this ->reportServer ->close();
            $this ->reportServer = null;

        } catch (\Exception $e) {
            $this ->xmlForm ->addErrorMsg($e ->getMessage());
            return '';
        }
        */

        return $result;
    }
}


class XMLFormElementReportVariable extends XMLFormElement
{
    function getHTMLOutput()
    {
        /** @var XMLFormElementReport $reportElement */
        $reportElement = $this ->findParentByName('report');

        if (!$reportElement) {
            throw new \Exception('Chyba ve formulari v tagu reportvariable - nenalezen nadrazeny tag report!');
        }

        $name = StringUtils::fromUTF8($this ->getAttributeValue('name'));
        $value = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('value')));

        switch ($this ->getAttributeValue('format')) {
            case 'int' :
            case 'integer' :
                $reportElement ->variablesXml .=
                    "    <reportparam name='$name' type='INT' value='$value'/>\n";
                break;

            case 'timestamp' :
                $reportElement ->variablesXml .=
                    "    <reportparam name='$name' type='DATE' value='" . date('j.n.Y H:i:s', $value) . "'/>\n";
                break;

            case 'obdobi' :
                require_once 'adsapp/obdobi.php';
                $obdobi = Obdobi::vratObdobi($this ->xmlForm ->database, $this ->nahradPromenne($this ->getAttributeValue('aplikace')) ?: 'ADS', (int)$value);
                $reportElement ->variablesXml .=
                    ReportOutputObject::getXMLParamsObdobi(0, $obdobi ->zacatek, $obdobi ->konec, $obdobi ->popis);
                break;

            case 'pp' :
                require_once 'adsapp/ppomer.php';

                $ppomer = ($value != ''  &&  (int)$value)
                    ? PracovniPomer::nactiPP($this ->xmlForm ->database, (int)$value)
                    : null;
                $reportElement ->variablesXml .=
                    ReportOutputObject::getXMLParamsOsoba($ppomer);

                break;

            default :
                $reportElement ->variablesXml .=
                    "    <reportparam name='$name' value='" . StringUtils::xmlTextConvert($value) . "'/>\n";
        }

        /*
        if (!$reportElement ->reportServer) {
            throw new \Exception('Chyba ve funkci questform_reportvariable - neni vytvoreno spojeni na report server!');
        }

        $pomstr = ($this ->getAttributeValue('format') == 'timestamp')
            ? 'DATEVAR ' . $name . '=' . date('j.n.Y H:i:s', $value)
            : 'VAR ' . $name . '=' . $value;

        $reportElement ->reportServer ->sendMessage($pomstr);
        */
    }
}
