<?php
/**
 * UTF8
 * stanislav.elbl, 09.10.2018
 * XML elementy <switch>, <case> a <default>
 */

namespace Ron\RonPortal\XML;

/**
 * XML element switch - muze mit parametr variable nebo expression
 * Class XMLFormElementSwitch
 * @package Ron\RonPortal\XML
 */
class XMLFormElementSwitch extends XMLFormElement
{
    /**
     * @var XMLFormElementCase[]
     */
    public $cases = [];

    /**
     * @var XMLFormElementDefault[]
     */
    public $default = null;

    /**
     * @return null|XMLFormElementCase
     */
    private function findCase()
    {
        $variableName = $this ->getAttributeValue('variable');
        if ($variableName) {
            $value = $this ->variableValue($variableName);
        } else {
            $value = $this ->nahradPromenne($this ->getAttributeValue('expression'));
        }

        foreach ($this ->cases as $case) {
            $caseValue = $case ->getAttributeValue('value');
            if (isset($caseValue)) {
                $caseValue = $this ->nahradPromenne($caseValue);
                if ($value === $caseValue) return $case;
            }
        }

        return $this ->default;
    }


    function getHTMLOutput()
    {
        $case = $this ->findCase();
        if (!$case) return '';
        return $case ->getElementsHTMLOutput();
    }


    function event()
    {
        $case = $this ->findCase();
        if ($case) $case ->elementsEvent();
    }


    function writeXMLOutput($xmlWriter)
    {
        throw new \Exception("Chyba: mod transform neni v tagu switch podporovan!");
    }


    function process()
    {
        $case = $this ->findCase();
        if ($case) $case ->processElements();
    }
}

/**
 * XML element case - musi mit parametr value
 * Class XMLFormElementCase
 * @package Ron\RonPortal\XML
 */
class XMLFormElementCase extends  XMLFormElement
{
    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        /** @var XMLFormElementSwitch $switch */
        $switch = $this ->findParentByName('switch');

        if ($switch) $switch ->cases[] = $this;
    }
}

/**
 * XML element default
 * Class XMLFormElementDefault
 * @package Ron\RonPortal\XML
 */
class XMLFormElementDefault extends  XMLFormElement
{
    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        /** @var XMLFormElementSwitch $switch */
        $switch = $this ->findParentByName('switch');

        if ($switch) $switch ->default = $this;
    }
}
