<?php
/**
 * UTF8
 * stanislav.elbl, 16.05.2017
 */

namespace Ron\RonPortal\XML;

/** @var RonPortalXMLForm $this  */
/** @var XMLFormElement $parentElement */

use Exception;
use Ron\DB\DBUtils;
use Ron\EMail;
use Ron\MailManager;
use Ron\RonPortal\Log;
use Ron\RonPortal\User;
use Ron\StringUtils;

if (!class_exists('\Ron\RonPortal\XML\XMLFormElementEmail')) {

    require_once('xml/xml_commands.php');

    class XMLFormElementEmail extends XMLFormElementCommand
    {
        var $to;
        /** @var string email zjisti z osoby */
        var $toRC;
        /** @var int email zjisti z osoby pro dany PP */
        var $toIdPP;
        var $toTypRozdeleni;
        /** @var string email zjisti z rozdeleni ( bud je zadano typ/kod nebo zjisti typ z toTypRozdeleni ) */
        var $toRozdeleni;
        /** @var string email zjisti z osob na dalsi urovni predavaciho procesu (v parametru bude zadano osoba/PP/rozdeleni pro rozliseni zda vyhledavat dle osoby, PP nebo rozdeleni formulare) */
        var $toDalsiUroven;

        var $from;
        var $fromRC;
        var $fromIdPP;

        var $copyTo;
        var $copyToRC;
        var $copyToIdPP;

        var $subject;
        var $text;

        /** @var  number   hodnota 0/1/2/3  1=jen chyby, 2=vzdy, 3 = vzdy+SQL */
        var $log;
        /** @var  string   promenna pro odeslani vysledku - nastavi se na 1 v pripade OK, 0 pri chybe */
        var $variableName;
        var $logSql;


        function loadXML($xmlReader)
        {
            parent::loadXML($xmlReader);

            $this->to = StringUtils::fromUTF8($this->getAttributeValue('to'));
            $this->toRC = StringUtils::fromUTF8($this->getAttributeValue('torc'));
            $this->toIdPP = $this->getAttributeValue('toidpp');
            $this->toTypRozdeleni = $this->getAttributeValue('totyprozdeleni');
            $this->toRozdeleni = StringUtils::fromUTF8($this->getAttributeValue('torozdeleni'));
            $this->toDalsiUroven = StringUtils::fromUTF8($this->getAttributeValue('todalsiuroven'));

            $this->from = StringUtils::fromUTF8($this->getAttributeValue('from'));
            $this->fromRC = StringUtils::fromUTF8($this->getAttributeValue('fromrc'));
            $this->fromIdPP = $this->getAttributeValue('fromidpp');

            $this->copyTo = StringUtils::fromUTF8($this->getAttributeValue('copyto'));
            $this->copyToRC = StringUtils::fromUTF8($this->getAttributeValue('copytorc'));
            $this->copyToIdPP = $this->getAttributeValue('copytoidpp');

            $this->subject = StringUtils::fromUTF8($this->getAttributeValue('subject'));

            $this->variableName = $this->getAttributeValue('variablename');
            $this->log = $this->getAttributeValue('log'); // 0/1/2
            if (!isset($this->log) || $this->log == '') $this->log = 1; // defaultne loguje v pripade chyby

            $this->text = $this->getTextContent(false);
        }


        function getEmailProRC($rc)
        {
            if (!$rc) {
                return null;
            }

            $params = $this->xmlForm->database->createParameters();
            $sql = 'SELECT eMail FROM Osoba WHERE RC = ' . $params->addParameter($rc);
            $address = DBUtils::getSqlValue($this->xmlForm->database, $sql, $params);

            if ($address) {
                if ($this->logSql != '') $this->logSql .= "\n";
                $this->logSql .= $sql . "\n" . $params->getLogText();
                return $address;
            }

            return null;
        }


        function getEmailProIdPP($idpp)
        {
            if (!$idpp) {
                return null;
            }

            $params = $this->xmlForm->database->createParameters();
            $sql =
                'SELECT Osoba.eMail FROM Osoba, PracovniPomer ' .
                ' WHERE Osoba.RC = PracovniPomer.RC AND PracovniPomer.Id = ' . $params->addParameter((int)$idpp);

            $address = DBUtils::getSqlValue($this->xmlForm->database, $sql, $params);

            if ($address) {
                if ($this->logSql != '') $this->logSql .= "\n";
                $this->logSql .= $sql . "\n" . $params->getLogText();
                return $address;
            }

            return null;
        }


        function getEmailAddres($address, $rc, $idpp)
        {
            $address = $this->nahradPromenne($address);
            if ($address == '#USER') {
                $address = $this ->getEmailProRC(User::getRC());
            }

            if ($address) {
                return $address;
            }

            // neni zadana natvrdo adresa - zkusi RC
            if ($rc) {
                $address = $this ->getEmailProRC($this->nahradPromenne($rc));
                if ($address) {
                    return $address;
                }
            }

            // ... zkusi PP
            if ($idpp) {
                $address = $this ->getEmailProIdPP($this->nahradPromenne($idpp));
                if ($address) {
                    return $address;
                }
            }

            return null;
        }


        function getEmailRozdeleni($rozdeleni, $typRozdeleni = null)
        {
            $rozdeleni = $this->nahradPromenne($rozdeleni);

            if ($typRozdeleni) {
                $kodRozdeleni = $rozdeleni;
            } else {
                $poma = explode('/', $rozdeleni, 2);
                $typRozdeleni = (int)$poma[0];
                $kodRozdeleni = $poma[1];
            }

            if ($typRozdeleni) {
                $params = $this->xmlForm->database->createParameters();
                $sql =
                    'SELECT eMail FROM Rozdeleni ' .
                    ' WHERE KodTypuRozdeleni = ' . $params->addParameter($typRozdeleni) .
                    '   AND Kod = ' . $params->addParameter($kodRozdeleni);

                $address = DBUtils::getSqlValue($this->xmlForm->database, $sql, $params);

                if ($address) {
                    if ($this->logSql != '') $this->logSql .= "\n";
                    $this->logSql .= $sql . "\n" . $params->getLogText();
                    return $address;
                }
            }

            return null;
        }

        /**
         * Vraci emailovou adresu vedouciho zadaneho rozdeleni
         * @param string $kodRozdeleni
         * @param int $typRozdeleni
         * @return string
         */
        function getEmailVedoucihoRozdeleni($kodRozdeleni, $typRozdeleni)
        {
            $kodRozdeleni = $this->nahradPromenne($kodRozdeleni);

            $params = $this->xmlForm->database->createParameters();
            $sql =
                'SELECT Osoba.eMail FROM Rozdeleni, Osoba ' .
                ' WHERE Rozdeleni.VedouciRC = Osoba.RC' .
                ' AND Rozdeleni.KodTypuRozdeleni = ' . $params->addParameter($typRozdeleni) .
                ' AND Rozdeleni.Kod = ' . $params->addParameter($kodRozdeleni);

            $address = DBUtils::getSqlValue($this->xmlForm->database, $sql, $params);

            if ($address) {
                if ($this->logSql != '') $this->logSql .= "\n";
                $this->logSql .= $sql . "\n" . $params->getLogText();
                return $address;
            }

            return null;
        }


        function getEmailRozdeleniOsoby($typRozdeleni, $idPP)
        {
            $params = $this->xmlForm->database->createParameters();
            $sql =
                'SELECT Rozdeleni.eMail FROM Rozdeleni, RozdeleniOsob, PracovniPomer ' .
                ' WHERE PracovniPomer.Id = ' . $params->addParameter($idPP) .
                '   AND RozdeleniOsob.RC = PracovniPomer.RC' .
                '   AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                '   AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) .
                '   AND (RozdeleniOsob.PlatnostDo IS NULL  OR  RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) . ')' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = ' . $params->addParameter((int)$typRozdeleni) .
                '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni' .
                '   AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni';

            $address = DBUtils::getSqlValue($this->xmlForm->database, $sql, $params);
            if ($address) {
                if ($this->logSql != '') $this->logSql .= "\n";
                $this->logSql .= $sql . "\n" . $params->getLogText();
                return $address;
            }

            return null;
        }


        function getEmailAddresPodleRozdeleni()
        {
            $typRozdeleni = $this->toTypRozdeleni ? (int)$this->nahradPromenne($this->toTypRozdeleni) : null;

            if ($this->toRozdeleni) {
                $address = $this ->getEmailRozdeleni($this->toRozdeleni, $typRozdeleni);
                if ($address) {
                    return $address;
                }
            }

            $toRozdelniOsoby = $this->getAttributeValue('torozdeleniosoby');
            if ($toRozdelniOsoby && $typRozdeleni) {
                $address = $this ->getEmailRozdeleniOsoby($typRozdeleni, (int)$this->nahradPromenne($toRozdelniOsoby));
                if ($address) {
                    return $address;
                }
            }

            $toVedouciRozdeleni = $this ->getAttributeValue('tovedoucirozdeleni');
            if ($toVedouciRozdeleni && $typRozdeleni) {
                $address = $this ->getEmailVedoucihoRozdeleni($this ->nahradPromenne($toVedouciRozdeleni), $typRozdeleni);

                if ($address) {
                    return $address;
                }
            }

            return null;
        }

        function getEmailAddresPodleProcesu()
        {
            $params = $this->xmlForm->database->createParameters();

            $sql =
                'SELECT O.eMail FROM Osoba O ' .
                '   JOIN PracovniPomer PPOsoby ON (PPOsoby.RC = O.RC) ' .
                '   JOIN RozdeleniOsob PMOsoby ON (PMOsoby.RC = PPOsoby.RC AND PMOsoby.OsobniCislo = PPOsoby.OsobniCislo AND PMOsoby.DatumNastupu = PPOsoby.DatumNastupu) ' .
                '   JOIN PMProUrovne PMU ON (PMU.KodTypuRozdeleni = PMOsoby.KodTypuRozdeleni AND PMU.KodRozdeleni = PMOsoby.Rozdeleni) ' .
                '   JOIN PoradiPredavani ON (PoradiPredavani.KodUrovne = PMU.KodUrovne) ' .
                '   JOIN PredavaciProces ON (PredavaciProces.Kod = PoradiPredavani.KodProcesu) ' .
                'WHERE PMOsoby.PlatnostOd <= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) .
                '   AND (PMOsoby.PlatnostDo >= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) . '  OR   PMOsoby.PlatnostDo IS NULL) ' .

                '   AND PPOsoby.DatumNastupu <= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) .
                '   AND (PPOsoby.DatumUkonceni >= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) . ' OR PPOsoby.DatumUkonceni IS NULL) ' .

                '   AND PMU.KodUrovne = ' . $params->addParameter($this->xmlForm->kodUrovne) .
                '   AND PoradiPredavani.KodProcesu = ' . $params->addParameter($this->xmlForm->predavaciProces) .
                '   AND PoradiPredavani.PocatecniUroven = ' . $params->addParameter($this->xmlForm->pocatecniUroven) .
                '   AND PoradiPredavani.Poradi = ' . $params->addParameter($this->xmlForm->poradiPredavani) .

                '   AND ((PoradiPredavani.NekontrolovatHierarchii = 1) OR ' .
                '        (EXISTS(SELECT RZamestnance.RC ' .
                '               FROM RozdeleniOsob RZamestnance ' .
                '               JOIN HierarchieRozdeleni ON (HierarchieRozdeleni.typPodrizene = RZamestnance.KodTypuRozdeleni AND HierarchieRozdeleni.kodPodrizene = RZamestnance.Rozdeleni) ' .
                '               JOIN RozdeleniOsob ROsoby ON (ROsoby.KodTypuRozdeleni = HierarchieRozdeleni.typNadrazene AND ROsoby.Rozdeleni = HierarchieRozdeleni.kodNadrazene) ' .
                '               JOIN PracovniPomer PPZamestnance ON (PPZamestnance.RC = RZamestnance.RC AND PPZamestnance.OsobniCislo = RZamestnance.OsobniCislo AND PPZamestnance.DatumNastupu = RZamestnance.DatumNastupu)' .
                '               WHERE HierarchieRozdeleni.typNadrazene = PredavaciProces.KodTypuRozdeleni ';


            switch ($this->toDalsiUroven) {
                case 'osoba' :
                    $sql .=
                        '       AND RZamestnance.RC = ' . $params->addParameter($this->xmlForm->rc) .
                        '       AND PPZamestnance.DatumNastupu <= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())).
                        '       AND (PPZamestnance.DatumUkonceni >= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) . ' OR PPZamestnance.DatumUkonceni IS NULL)';
                    break;
                case 'PP' :
                    $sql .=
                        '       AND RZamestnance.RC = ' . $params->addParameter($this->xmlForm->pp_RC) .
                        '       AND RZamestnance.OsobniCislo = ' . $params->addParameter($this->xmlForm->osobniCislo) .
                        '       AND RZamestnance.DatumNastupu = ' . $params->addParameter($this->xmlForm->database->dateToParam($this->xmlForm->datumNastupu));
                    break;
                case 'rozdeleni' :
                    $sql .=
                        '       AND RZamestnance.KodTypuRozdeleni = ' . $params->addParameter($this->xmlForm->kodTypuRozdeleni) .
                        '       AND RZamestnance.Rozdeleni = ' . $params->addParameter($this->xmlForm->rozdeleni);
                    break;
                default :
                    return null;
            }

            $sql .=
                '                   AND RZamestnance.PlatnostOd <= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) .
                '                   AND (RZamestnance.PlatnostDo >= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) . ' OR RZamestnance.PlatnostDo IS NULL) ' .

                '                   AND ROsoby.RC = PPOsoby.RC ' .
                '                   AND ROsoby.OsobniCislo = PPOsoby.OsobniCislo ' .
                '                   AND ROsoby.DatumNastupu = PPOsoby.DatumNastupu ' .
                '                   AND ROsoby.PlatnostOd <= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) .
                '                   AND (ROsoby.PlatnostDo >= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) . ' OR ROsoby.PlatnostDo IS NULL) ' .
                '               ) ' .
                '        )' .
                '       ) ' .

                '   AND O.eMail IS NOT NULL ' .
                'GROUP BY O.Prijmeni, O.eMail';

            $this->logSql = $sql . "\n" . $params->getLogText();

            $cursor = $this->xmlForm->database->getSqlResult($sql, $params);
            $address = '';

            while ($row = $cursor->fetch()) {
                if ($address != '')
                    $address .= ';';
                $address .= $row[0];
            }

            if ($address) {
                return $address;
            }

            return null;
        }


        function sendEmail()
        {
            $to = $this->getEmailAddres($this->to, $this->toRC, $this->toIdPP);
            if (!$to) $to = $this ->getEmailAddresPodleRozdeleni();
            if (!$to) $to = $this->getEmailAddresPodleProcesu();

            $from = $this->getEmailAddres($this->from, $this->fromRC, $this->fromIdPP);
            $copyTo = $this->getEmailAddres($this->copyTo, $this->copyToRC, $this->copyToIdPP);

            $text = $this->nahradPromenne($this->text);
            $subject = $this->nahradPromenne($this->subject);

            $logtext = sprintf(StringUtils::fromUTF8(
                "Příjemce: %s\n" .
                "Předmět: %s\n" .
                "Odesílatel: %s\n" .
                "Kopie: %s\n" .
                "Text:\n%s"),
                $to, $subject, $from, $copyTo, $text);

            $hodnota = 0;

            if ($to) {
                require_once('ron/mailer.php');
                $eMail = new EMail();
                $eMail->to = $to;
                $eMail->subject = $subject;
                $eMail->text = $text;
                $eMail->from = $from;
                $eMail->copyTo = $copyTo;

                $OK = true;
                $errorMessage = null;

                try {
                    MailManager::sendMail($eMail);
                } catch (Exception $e) {
                    if (!$this->variableName || $this->getBoolAttribute("generateerror")) {
                        // chybu generuje jen v pripade, ze neni zadano variableName, nebo je to explicitne povoleno atributem generateerror
                        $this ->xmlForm ->addErrorMsg($e);
                    }

                    $errorMessage = $e->getMessage();
                    $OK = false;
                }

                if ($OK) $hodnota = 1;

                if ($this->log == 2 || $this->log == 3 || ($this->log && !$OK)) {
                    require_once('adsapp/ronportallog.php');

                    if ($this->log == 3) {
                        $logtext .= "\n" . $this->logSql;
                    }

                    Log::addRecordToLog($this->xmlForm->database,
                        StringUtils::fromUTF8($OK ? 'Odeslání emailu proběhlo úspěšně' : 'Chyba při odesílání emailu'),
                        StringUtils::fromUTF8('XML formuláře'),
                        ($OK ? '' : "Chyba: $errorMessage\n\n") .
                        $logtext
                    );

                    $this->xmlForm->addLogData(null, 'Email => ' . $to . ': ' . $subject . ($OK ? ' - OK' : ' - chyba: ' . $errorMessage));
                }
            } else if ($this->log) {
                require_once('adsapp/ronportallog.php');
                Log::addRecordToLog($this->xmlForm->database, StringUtils::fromUTF8('Chyba při odesílání emailu'),
                    StringUtils::fromUTF8('XML formuláře'),
                    StringUtils::fromUTF8("Není zadána adresa příjemce\n\n") . $logtext);
                $this->xmlForm->addLogData(null, StringUtils::fromUTF8('Chyba při odesílání emailu - není zadána adresa příjemce'));
            }

            if ($this->variableName) $this->setVariableValue($this->variableName, $hodnota);
        }


        function doCommand()
        {
            $this->sendEmail();
        }

    }
}


return new XMLFormElementEmail($this, $parentElement);
