<?php
/**
 * stanislav.elbl, 28.01.2019
 */

namespace Ron\RonPortal\XML;

/** @var RonPortalXMLForm $this  */
/** @var XMLFormElement $parentElement */

use Ron\MessageManager;
use Ron\StringUtils;

if (!class_exists('\Ron\RonPortal\XML\XMLFormElementForSql')) {

    class XMLFormElementForSql extends XMLFormElement
    {
        var $name;

        var $expand;
        var $sqlText;
        var $sqlTextVariable;


        function loadXMLElement($xmlReader)
        {
            parent::loadXMLElement($xmlReader);

            $this->name = $this->getAttributeValue('name');
            $this->expand = self::getBoolValue($this->getAttributeValue('expand'));

            $this->sqlText = StringUtils::fromUTF8($this->getAttributeValue('sqltext'));
            $this->sqlTextVariable = $this->getAttributeValue('sqltextvariable');
        }


        private function getCursor()
        {
            $sql = $this->sqlTextVariable
                ? $this->variableValue($this->sqlTextVariable)
                : $this->sqlText;

            $sql = $this->nahradPromenneSQL($sql);

            MessageManager::addDebugLog("XMLDebug> forsql: $sql");

            return $this->xmlForm->database->getSqlResult($sql);
        }


        function getHTMLOutput()
        {
            $cursor = $this->getCursor();
            $result = '';

            while ($row = $cursor->fetch()) {
                // nastavi promenne
                foreach ($row as $fieldindex => $value) {
                    if ($this->xmlForm->verze > 1) {
                        $this->setVariableValue($this->name, $value, $fieldindex);
                    } else {
                        $this->setVariableValue("{$this->name}[$fieldindex]", $value);
                    }
                }

                $result .= $this->getElementsHTMLOutput();
            }

            $cursor ->close();

            return $result;
        }


        function event()
        {
            $cursor = $this->getCursor();

            while ($row = $cursor->fetch()) {
                // nastavi promenne
                foreach ($row as $fieldindex => $value) {
                    if ($this->xmlForm->verze > 1) {
                        $this->setVariableValue($this->name, $value, $fieldindex);
                    } else {
                        $this->setVariableValue("{$this->name}[$fieldindex]", $value);
                    }
                }

                $this->elementsEvent();
            }

            $cursor ->close();
        }


        function writeXMLOutput($xmlWriter)
        {
            if (!$this ->expand) {
                $this->copyXML($xmlWriter);

                // neni expand - provede se jen kdyz je nastaveno processonfill
                if (!self::getBoolValue($this ->getAttributeValue('processonfill'))) return;
            }

            $cursor = $this->getCursor();
            while ($row = $cursor->fetch()) {
                // nastavi promenne
                foreach ($row as $fieldindex => $value) {
                    if ($this->xmlForm->verze > 1) {
                        $this->setVariableValue($this->name, $value, $fieldindex);
                        if ($this ->expand) {
                            $xmlWriter ->startElement('variable');
                            $xmlWriter ->writeAttribute('name', $this ->name);
                            $xmlWriter ->writeAttribute('index', $fieldindex);
                            $xmlWriter ->writeAttribute('value', StringUtils::toUTF8($value));
                            $xmlWriter ->endElement();
                        }
                    } else {
                        $this->setVariableValue("{$this->name}[$fieldindex]", $value);
                        if ($this ->expand) {
                            $xmlWriter->startElement('variable');
                            $xmlWriter->writeAttribute('name', "{$this->name}[$fieldindex]");
                            $xmlWriter->writeAttribute('value', StringUtils::toUTF8($value));
                            $xmlWriter->endElement();
                        }
                    }
                }

                // provede vnorene prikazy
                if ($this ->expand) {
                    $this ->writeXMLElements($xmlWriter);
                } else {
                    parent::process();
                }
            }

            $cursor ->close();
        }


        function process()
        {
            if ($this ->expand) {
                throw new \Exception('XMLFormElementForSQL::process - parametr expand neni implementovan!');
            }
            $cursor = $this->getCursor();
            while ($row = $cursor->fetch()) {
                // nastavi promenne
                foreach ($row as $fieldindex => $value) {
                    if ($this->xmlForm->verze > 1) {
                        $this->setVariableValue($this->name, $value, $fieldindex);
                    } else {
                        $this->setVariableValue("{$this->name}[$fieldindex]", $value);
                    }
                }

                $this ->processElements();
                //parent::process();
            }

            $cursor ->close();
        }
    }

}

return new XMLFormElementForSql($this, $parentElement);
