<?php
/**
 * stanislav.elbl, 05.02.2020
 * XML tag pro nacteni informaci o planovanych poctech zamestnancu do promenne
 */

namespace Ron\RonPortal\XML;

/** @var RonPortalXMLForm $this  */
/** @var XMLFormElement $parentElement */

use Ron\Kapacity\PodminkyPlanovaniForm;
use Ron\RonPortal\Obdobi;
use Ron\RonPortal\Rozdeleni;
use Ron\StringUtils;

if (!class_exists('\Ron\RonPortal\XML\PodminkyPlanovani')) {

    require_once('xml/xml_commands.php');

    class XMLFormElementPodminkyPlanovani extends XMLFormElementCommand
    {
        function doCommand()
        {
            require_once 'kapacity/podminky_rozdeleni.php';
            require_once 'adsapp/obdobi.php';
            require_once 'adsapp/rozdeleni.php';

            $variableName = $this->getAttributeValue('variablename');

            $pocatekObdobi = $this ->nahradPromenne($this->getAttributeValue('obdobi'));

            $obdobi = Obdobi::vratObdobi($this ->xmlForm ->database, 'ADS', $pocatekObdobi);

            $typRozdeleni = (int)$this ->nahradPromenne($this->getAttributeValue('typrozdeleni'));
            $kodRozdelni = $this ->nahradPromenne(StringUtils::fromUTF8($this->getAttributeValue('rozdeleni')));

            $rozdeleni = Rozdeleni::loadRozdeleni($this ->xmlForm ->database, $typRozdeleni, $kodRozdelni);

            $form = new PodminkyPlanovaniForm($this ->xmlForm ->database);
            $form ->load($obdobi, $rozdeleni, false);

            $html = $this ->getBoolAttribute('html');
            $format = StringUtils::fromUTF8($this ->getAttributeValue('format', '%DAYOFWEEK% %DATE% %PODMINKA%: %PUVODNI_HODNOTA% => %NOVA_HODNOTA% (%USER%)'));

            $resultStr = '';

            foreach ($form ->radky as $radek) {
                //var_dump($radek);
                foreach ($form ->dny as $index => $den) {
                    if ($radek ->data[$index] ->hodnota != $radek ->data[$index] ->vychoziHodnota) {
                        // doslo ke zmene hodnoty
                        // sestavi text radku podle zadaneho formatu
                        $text = $format;
                        $text = str_replace('%DAYOFWEEK%', \Ron\ronText('Day_short_' . date('w', $den->datum)), $text);
                        $text = str_replace('%DATE%', date('j.n.Y', $den->datum), $text);
                        $text = str_replace('%PODMINKA%', $radek ->getHtmlPopis(), $text);
                        $text = str_replace('%PUVODNI_HODNOTA%', $radek ->data[$index] ->vychoziHodnota, $text);
                        $text = str_replace('%NOVA_HODNOTA%', $radek ->data[$index] ->hodnota, $text);
                        $text = str_replace('%USER%', $radek ->data[$index] ->updateUserName, $text);
                        $text = str_replace('%UPDATE_TIME%', StringUtils::format('datetime', $radek ->data[$index] ->updateTimestamp), $text);
                        $text = str_replace('%CSS_CLASS%', ($radek ->data[$index] ->hodnota > $radek ->data[$index] ->vychoziHodnota) ? 'raising' : 'missing', $text);
                        $text = str_replace('%CSS_STYLE%', ($radek ->data[$index] ->hodnota > $radek ->data[$index] ->vychoziHodnota) ? 'background-color: #fcd0c0' : 'background-color: deepskyblue', $text);

                        $resultStr .=
                            ($html ? '<div>' : '') .
                            $text .
                            ($html ? '</div>' : '') .
                            "\n";
                    }
                }
            }

            $this ->setVariableValue($variableName, $resultStr);
        }
    }
}


return new XMLFormElementPodminkyPlanovani($this, $parentElement);
