<?php
/**
 * stanislav.elbl, 19.02.2019
 */

namespace Ron\RonPortal\XML;

/** @var RonPortalXMLForm $this  */
/** @var XMLFormElement $parentElement */

use Ron\StringUtils;

if (!class_exists('\Ron\RonPortal\XML\XMLElementVyberObdobi')) {

    class XMLElementVyberObdobi extends XMLFormElement
    {
        /** @var string */
        var $name;
        /** @var bool */
        var $required;
        /** @var string */
        var $aplikace;
        /** @var bool */
        var $jenNeuzavrene;
        /** @var bool */
        var $jenUkoncene;
        /** @var bool */
        var $variabilni;
        /** @var string */
        var $sqlFilter;
        /** @var bool */
        var $editing;
        /** @var int */
        var $value;


        protected function loadXMLElement($xmlReader)
        {
            parent::loadXMLElement($xmlReader);

            $this ->name = $this ->getAttributeValue('name');
            $this ->required = self::getBoolValue($this ->getAttributeValue('required'), true); // defaultne je true
            $this ->aplikace = $this ->getAttributeValue('aplikace');
            $this ->jenNeuzavrene = self::getBoolValue($this ->getAttributeValue('jenneuzavrene'));
            $this ->jenUkoncene = self::getBoolValue($this ->getAttributeValue('jenukoncene'));
            $this ->variabilni = self::getBoolValue($this ->getAttributeValue('variabilni'));
            $this ->editing = self::getBoolValue($this ->getAttributeValue('editing'));
            $this ->value = $this ->getAttributeValue('value');

            // TODO: pro implementaci writeXmlOutput bude asi potreba predelat! (jako u <variable>
            $this ->sqlFilter = StringUtils::fromUTF8($this ->loadXMLValue($xmlReader) );
        }


        function getHTMLOutput()
        {
            $params = $this ->xmlForm ->database ->createParameters();

            $this->sqlFilter = $this->nahradPromenneSQL($this->sqlFilter);

            if ($this ->variabilni) {
                $filter = '';
                if ($this ->jenUkoncene) $filter = 'KonecObdobi < ' . $params ->addParameter($this ->xmlForm ->database ->dateToParam(time()));
                if ($this ->sqlFilter) {
                    $filter .= ($filter ? ' AND ' : '');
                    $filter .= " ({$this ->sqlFilter}) ";
                }

                $sql =
                    'SELECT PocatekObdobi, PopisObdobi, KonecObdobi FROM VariantniObdobi ' .
                    ($filter ? ' WHERE ' . $filter : '') .
                    ' ORDER BY PocatekObdobi DESC ';
            } else {
                $sql =
                    'SELECT PocatekObdobi, Popis, KonecObdobi, Uzavreno FROM Obdobi ' .
                    ' WHERE Aplikace = ' . $params ->addParameter($this ->aplikace);

                if ($this ->jenNeuzavrene) {
                    $sql .= ' AND Uzavreno = 0 ';
                }

                if ($this ->jenUkoncene) {
                    $sql .= ' AND KonecObdobi < ' . $params ->addParameter($this ->xmlForm ->database ->dateToParam(time()));
                }

                if ($this ->sqlFilter) $sql .= " AND  ({$this ->sqlFilter}) ";

                $sql .= ' ORDER BY PocatekObdobi DESC ';
            }

            if ($this ->xmlForm ->usePostValues) {
                // hodnota z POST
                $value = $_POST[$this->name];
            } else {
                // defaultni hodnota
                $value = null;

                if ($this ->xmlForm ->dataMode()) {
                    // ... nebo hodnota zadana uz drive
                    $value = $this ->variableValue($this ->name);
                }

                if (!isset($value)) {
                    $value = $this ->nahradPromenne(StringUtils::fromUTF8($this->value)); // umoznuje zadat defaultni hodnotu
                }
            }

            $this ->xmlForm ->addDebugMessage('Vyberobdobi, sql: ' . $sql, true);

            $cursor = $this ->xmlForm ->database ->getSqlResult($sql, $params);

            $attrs = ['name' => $this ->name];
            $attrs = $this->setHTMLAttributes($attrs);
            if ($this ->xmlForm ->isError($this ->name)) {
                if (isset($attrs['class'])) $attrs['class'] .= ' dotaznik_errinput';
                else $attrs['class'] = 'dotaznik_errinput';
            }

            $attrs_str = $this->getHTMLAttributes($attrs);

            $result = '<SELECT ' . $attrs_str . '>';

            if (!$value  ||  !$this ->required) { // pokud jeste nebyla zadana hodnota, prida prazdnou hodnotu  -  pridat prazdny radek, kdyz pole neni povinne? ? ?
                $selectedStr = $value ? '' : 'selected="selected"';
                $result .= "<OPTION value='' $selectedStr></OPTION> ";

                $this ->xmlForm ->nastavObdobi($this ->name, null, null, null, null);
            }

            while ($row = $cursor ->fetch()) {
                $hlpVal = $cursor ->stringToTimestamp($row[0]);

                if ($hlpVal == $value) {
                    $selectedStr = 'selected="selected"';
                    // nastavi prislusne promenne
                    if ($this ->variabilni) {
                        $this ->xmlForm ->nastavObdobi($this ->name, $hlpVal, $cursor->stringToTimestamp($row[2]), $row[1]);
                    } else {
                        $this ->xmlForm ->nastavObdobi($this ->name, $hlpVal, $cursor->stringToTimestamp($row[2]), $row[1], (int)$row[3]);
                    }
                } else {
                    $selectedStr = '';
                }

                $result .= "<OPTION value='{$hlpVal}' $selectedStr>" . StringUtils::htmlTextConvert($row[1]) . "</OPTION>";
            }

            $cursor ->close();

            $result .= '</SELECT>';

            return $result;
        }


        function writeXMLOutput($xmlWriter)
        {
            throw new \Exception('XMLElementVyberObdobi::writeXMLOutput neni implementovano!');
            //parent::writeXMLOutput($xmlWriter);
        }


        function process()
        {
            if ( $this ->required  &&  !$_POST[$this ->name] ) {
                $this ->xmlForm -> addErrorMsg( \Ron\ronText('Chybi_povinna_hodnota'), $this ->name );
            }

            $oldValue = $this ->xmlForm ->getVariableValue($this ->name);
            if (!isset($oldValue)) $oldValue = $this ->value;

            $this ->xmlForm -> nastavObdobiAplikace($this ->name, $this ->aplikace, $_POST[$this ->name], $this ->variabilni);

            if ($oldValue != $_POST[$this ->name]) {
                $this ->xmlForm ->addLogData($this ->name, $_POST[$this ->name] . ' (' . $this ->variableValue($this ->name . '_popis') . ')');
            }

            $this ->xmlForm ->addDataVariable($this ->name);
            $this ->xmlForm ->addDataVariable($this ->name . '_pocatek');
            $this ->xmlForm ->addDataVariable($this ->name . '_pocatekstr');
            $this ->xmlForm ->addDataVariable($this ->name . '_konec');
            $this ->xmlForm ->addDataVariable($this ->name . '_konecstr');
            $this ->xmlForm ->addDataVariable($this ->name . '_popis');
        }
    }

}

return new XMLElementVyberObdobi($this, $parentElement);
